@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalsievefilter.cpp 30152 2015-09-06 12:29:19Z rouault $
+ * $Id: gdalsievefilter.cpp 33715 2016-03-13 08:52:06Z goatbar $
  *
  * Project:  GDAL
  * Purpose:  Raster to Polygon Converter
@@ -31,29 +31,29 @@
 #include "gdal_alg_priv.h"
 #include "cpl_conv.h"
 #include <vector>
+#include <set>
 
-CPL_CVSID("$Id: gdalsievefilter.cpp 30152 2015-09-06 12:29:19Z rouault $");
+CPL_CVSID("$Id: gdalsievefilter.cpp 33715 2016-03-13 08:52:06Z goatbar $");
 
 #define MY_MAX_INT 2147483647
 
 /*
  * General Plan
  *
- * 1) make a pass with the polygon enumerator to build up the 
+ * 1) make a pass with the polygon enumerator to build up the
  *    polygon map array.  Also accumulate polygon size information.
  *
  * 2) Identify the polygons that need to be merged.
- * 
- * 3) Make a pass with the polygon enumerator.  For each "to be merged" 
- *    polygon keep track of it's largest neighbour. 
- * 
+ *
+ * 3) Make a pass with the polygon enumerator.  For each "to be merged"
+ *    polygon keep track of its largest neighbour.
+ *
  * 4) Fix up remappings that would go to polygons smaller than the seive
- *    size.  Ensure these in term map to the largest neighbour of the 
- *    "to be seieved" polygons. 
- * 
+ *    size.  Ensure these in term map to the largest neighbour of the
+ *    "to be sieved" polygons.
+ *
  * 5) Make another pass with the polygon enumerator. This time we remap
  *    the actual pixel values of all polygons to be merged.
- * 
  */
 
 /************************************************************************/
@@ -63,14 +63,14 @@ CPL_CVSID("$Id: gdalsievefilter.cpp 30152 2015-09-06 12:29:19Z rouault $");
 /*      band is zero.                                                   */
 /************************************************************************/
 
-static CPLErr 
-GPMaskImageData( GDALRasterBandH hMaskBand, GByte *pabyMaskLine, int iY, int nXSize, 
+static CPLErr
+GPMaskImageData( GDALRasterBandH hMaskBand, GByte *pabyMaskLine, int iY, int nXSize,
                  GInt32 *panImageLine )
 
 {
     CPLErr eErr;
 
-    eErr = GDALRasterIO( hMaskBand, GF_Read, 0, iY, nXSize, 1, 
+    eErr = GDALRasterIO( hMaskBand, GF_Read, 0, iY, nXSize, 1,
                          pabyMaskLine, nXSize, 1, GDT_Byte, 0, 0 );
     if( eErr == CE_None )
     {
@@ -85,6 +85,8 @@ GPMaskImageData( GDALRasterBandH hMaskBand, GByte *pabyMaskLine, int iY, int nXS
     return eErr;
 }
 
+// TODO: What is "eaches" supposed to be?
+
 /************************************************************************/
 /*                          CompareNeighbour()                          */
 /*                                                                      */
@@ -99,19 +101,19 @@ GPMaskImageData( GDALRasterBandH hMaskBand, GByte *pabyMaskLine, int iY, int nXS
 /*      smaller than our sieve threshold.                               */
 /************************************************************************/
 
-static inline void CompareNeighbour( int nPolyId1, int nPolyId2, 
-                                     int *panPolyIdMap, 
-                                     int *panPolyValue,
+static inline void CompareNeighbour( int nPolyId1, int nPolyId2,
+                                     int *panPolyIdMap,
+                                     int * /* panPolyValue */,
                                      std::vector<int> &anPolySizes,
                                      std::vector<int> &anBigNeighbour )
 
 {
     // nodata polygon do not need neighbours, and cannot be neighbours
-    // to valid polygons. 
+    // to valid polygons.
     if( nPolyId1 < 0 || nPolyId2 < 0 )
         return;
 
-    // make sure we are working with the final merged polygon ids. 
+    // make sure we are working with the final merged polygon ids.
     nPolyId1 = panPolyIdMap[nPolyId1];
     nPolyId2 = panPolyIdMap[nPolyId2];
 
@@ -119,8 +121,8 @@ static inline void CompareNeighbour( int nPolyId1, int nPolyId2,
         return;
 
     // nodata polygon do not need neighbours, and cannot be neighbours
-    // to valid polygons. 
-    // should no longer happen with r28826 optim
+    // to valid polygons.
+    // should no longer happen with r28826 optimization
     //if( panPolyValue[nPolyId1] == GP_NODATA_MARKER
     //    || panPolyValue[nPolyId2] == GP_NODATA_MARKER )
     //    return;
@@ -138,43 +140,43 @@ static inline void CompareNeighbour( int nPolyId1, int nPolyId2,
 /*                          GDALSieveFilter()                           */
 /************************************************************************/
 
-/** 
- * Removes small raster polygons. 
+/**
+ * Removes small raster polygons.
  *
  * The function removes raster polygons smaller than a provided
- * threshold size (in pixels) and replaces replaces them with the pixel value 
- * of the largest neighbour polygon.  
+ * threshold size (in pixels) and replaces replaces them with the pixel value
+ * of the largest neighbour polygon.
  *
  * Polygon are determined (per GDALRasterPolygonEnumerator) as regions of
  * the raster where the pixels all have the same value, and that are contiguous
- * (connected).  
+ * (connected).
  *
  * Pixels determined to be "nodata" per hMaskBand will not be treated as part
  * of a polygon regardless of their pixel values.  Nodata areas will never be
- * changed nor affect polygon sizes. 
+ * changed nor affect polygon sizes.
  *
  * Polygons smaller than the threshold with no neighbours that are as large
  * as the threshold will not be altered.  Polygons surrounded by nodata areas
- * will therefore not be altered.  
+ * will therefore not be altered.
  *
  * The algorithm makes three passes over the input file to enumerate the
- * polygons and collect limited information about them.  Memory use is 
+ * polygons and collect limited information about them.  Memory use is
  * proportional to the number of polygons (roughly 24 bytes per polygon), but
  * is not directly related to the size of the raster.  So very large raster
  * files can be processed effectively if there aren't too many polygons.  But
- * extremely noisy rasters with many one pixel polygons will end up being 
+ * extremely noisy rasters with many one pixel polygons will end up being
  * expensive (in memory) to process.
- * 
+ *
  * @param hSrcBand the source raster band to be processed.
- * @param hMaskBand an optional mask band.  All pixels in the mask band with a 
+ * @param hMaskBand an optional mask band.  All pixels in the mask band with a
  * value other than zero will be considered suitable for inclusion in polygons.
  * @param hDstBand the output raster band.  It may be the same as hSrcBand
- * to update the source in place. 
+ * to update the source in place.
  * @param nSizeThreshold raster polygons with sizes smaller than this will
  * be merged into their largest neighbour.
  * @param nConnectedness either 4 indicating that diagonal pixels are not
  * considered directly adjacent for polygon membership purposes or 8
- * indicating they are. 
+ * indicating they are.
  * @param papszOptions algorithm options in name=value list form.  None currently
  * supported.
  * @param pfnProgress callback for reporting algorithm progress matching the
@@ -204,19 +206,17 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
     CPLErr eErr = CE_None;
     int nXSize = GDALGetRasterBandXSize( hSrcBand );
     int nYSize = GDALGetRasterBandYSize( hSrcBand );
-    GInt32 *panLastLineVal = (GInt32 *) VSIMalloc2(sizeof(GInt32), nXSize);
-    GInt32 *panThisLineVal = (GInt32 *) VSIMalloc2(sizeof(GInt32), nXSize);
-    GInt32 *panLastLineId =  (GInt32 *) VSIMalloc2(sizeof(GInt32), nXSize);
-    GInt32 *panThisLineId =  (GInt32 *) VSIMalloc2(sizeof(GInt32), nXSize);
-    GInt32 *panThisLineWriteVal = (GInt32 *) VSIMalloc2(sizeof(GInt32), nXSize);
-    GByte *pabyMaskLine = (hMaskBand != NULL) ? (GByte *) VSIMalloc(nXSize) : NULL;
+    GInt32 *panLastLineVal = (GInt32 *) VSI_MALLOC2_VERBOSE(sizeof(GInt32), nXSize);
+    GInt32 *panThisLineVal = (GInt32 *) VSI_MALLOC2_VERBOSE(sizeof(GInt32), nXSize);
+    GInt32 *panLastLineId =  (GInt32 *) VSI_MALLOC2_VERBOSE(sizeof(GInt32), nXSize);
+    GInt32 *panThisLineId =  (GInt32 *) VSI_MALLOC2_VERBOSE(sizeof(GInt32), nXSize);
+    GInt32 *panThisLineWriteVal = (GInt32 *) VSI_MALLOC2_VERBOSE(sizeof(GInt32), nXSize);
+    GByte *pabyMaskLine = (hMaskBand != NULL) ? (GByte *) VSI_MALLOC_VERBOSE(nXSize) : NULL;
     if (panLastLineVal == NULL || panThisLineVal == NULL ||
         panLastLineId == NULL || panThisLineId == NULL ||
         panThisLineWriteVal == NULL ||
         (hMaskBand != NULL && pabyMaskLine == NULL))
     {
-        CPLError(CE_Failure, CPLE_OutOfMemory,
-                 "Could not allocate enough memory for temporary buffers");
         CPLFree( panThisLineId );
         CPLFree( panLastLineId );
         CPLFree( panThisLineVal );
@@ -237,21 +237,21 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
 
     for( iY = 0; eErr == CE_None && iY < nYSize; iY++ )
     {
-        eErr = GDALRasterIO( 
+        eErr = GDALRasterIO(
             hSrcBand,
-            GF_Read, 0, iY, nXSize, 1, 
+            GF_Read, 0, iY, nXSize, 1,
             panThisLineVal, nXSize, 1, GDT_Int32, 0, 0 );
-        
+
         if( eErr == CE_None && hMaskBand != NULL )
             eErr = GPMaskImageData( hMaskBand, pabyMaskLine, iY, nXSize, panThisLineVal );
 
         if( iY == 0 )
-            oFirstEnum.ProcessLine( 
+            oFirstEnum.ProcessLine(
                 NULL, panThisLineVal, NULL, panThisLineId, nXSize );
         else
             oFirstEnum.ProcessLine(
-                panLastLineVal, panThisLineVal, 
-                panLastLineId,  panThisLineId, 
+                panLastLineVal, panThisLineVal,
+                panLastLineId,  panThisLineId,
                 nXSize );
 
 /* -------------------------------------------------------------------- */
@@ -262,7 +262,7 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
 
         for( iX = 0; iX < nXSize; iX++ )
         {
-            iPoly = panThisLineId[iX]; 
+            iPoly = panThisLineId[iX];
 
             if( iPoly >= 0 && anPolySizes[iPoly] < MY_MAX_INT )
                 anPolySizes[iPoly] += 1;
@@ -282,8 +282,8 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
 /* -------------------------------------------------------------------- */
 /*      Report progress, and support interrupts.                        */
 /* -------------------------------------------------------------------- */
-        if( eErr == CE_None 
-            && !pfnProgress( 0.25 * ((iY+1) / (double) nYSize), 
+        if( eErr == CE_None
+            && !pfnProgress( 0.25 * ((iY+1) / (double) nYSize),
                              "", pProgressArg ) )
         {
             CPLError( CE_Failure, CPLE_UserInterrupt, "User terminated" );
@@ -299,7 +299,7 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
     oFirstEnum.CompleteMerges();
 
 /* -------------------------------------------------------------------- */
-/*      Push the sizes of merged polygon fragments into the the         */
+/*      Push the sizes of merged polygon fragments into the             */
 /*      merged polygon id's count.                                      */
 /* -------------------------------------------------------------------- */
     for( iPoly = 0; iPoly < oFirstEnum.nNextPolygonId; iPoly++ )
@@ -309,7 +309,7 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
             GIntBig nSize = anPolySizes[oFirstEnum.panPolyIdMap[iPoly]];
 
             nSize += anPolySizes[iPoly];
-            
+
             if( nSize > MY_MAX_INT )
                 nSize = MY_MAX_INT;
 
@@ -319,7 +319,7 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
     }
 
 /* -------------------------------------------------------------------- */
-/*      We will use a new enumerator for the second pass primariliy     */
+/*      We will use a new enumerator for the second pass primarily      */
 /*      so we can preserve the first pass map.                          */
 /* -------------------------------------------------------------------- */
     GDALRasterPolygonEnumerator oSecondEnum( nConnectedness );
@@ -339,7 +339,7 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
 /* -------------------------------------------------------------------- */
 /*      Read the image data.                                            */
 /* -------------------------------------------------------------------- */
-        eErr = GDALRasterIO( hSrcBand, GF_Read, 0, iY, nXSize, 1, 
+        eErr = GDALRasterIO( hSrcBand, GF_Read, 0, iY, nXSize, 1,
                              panThisLineVal, nXSize, 1, GDT_Int32, 0, 0 );
 
         if( eErr == CE_None && hMaskBand != NULL )
@@ -353,12 +353,12 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
 /*      the same thing done in the first pass above).                   */
 /* -------------------------------------------------------------------- */
         if( iY == 0 )
-            oSecondEnum.ProcessLine( 
+            oSecondEnum.ProcessLine(
                 NULL, panThisLineVal, NULL, panThisLineId, nXSize );
         else
             oSecondEnum.ProcessLine(
-                panLastLineVal, panThisLineVal, 
-                panLastLineId,  panThisLineId, 
+                panLastLineVal, panThisLineVal,
+                panLastLineId,  panThisLineId,
                 nXSize );
 
 /* -------------------------------------------------------------------- */
@@ -369,30 +369,29 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
         {
             if( iY > 0 )
             {
-                CompareNeighbour( panThisLineId[iX], 
+                CompareNeighbour( panThisLineId[iX],
                                   panLastLineId[iX],
                                   oFirstEnum.panPolyIdMap,
                                   oFirstEnum.panPolyValue,
                                   anPolySizes, anBigNeighbour );
 
                 if( iX > 0 && nConnectedness == 8 )
-                    CompareNeighbour( panThisLineId[iX], 
+                    CompareNeighbour( panThisLineId[iX],
                                       panLastLineId[iX-1],
                                       oFirstEnum.panPolyIdMap,
                                       oFirstEnum.panPolyValue,
                                       anPolySizes, anBigNeighbour );
-                    
+
                 if( iX < nXSize-1 && nConnectedness == 8 )
-                    CompareNeighbour( panThisLineId[iX], 
+                    CompareNeighbour( panThisLineId[iX],
                                       panLastLineId[iX+1],
                                       oFirstEnum.panPolyIdMap,
                                       oFirstEnum.panPolyValue,
                                       anPolySizes, anBigNeighbour );
-                    
             }
-            
+
             if( iX > 0 )
-                CompareNeighbour( panThisLineId[iX], 
+                CompareNeighbour( panThisLineId[iX],
                                   panThisLineId[iX-1],
                                   oFirstEnum.panPolyIdMap,
                                   oFirstEnum.panPolyValue,
@@ -400,7 +399,7 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
 
             // We don't need to compare to next pixel or next line
             // since they will be compared to us.
-        }                     
+        }
 
 /* -------------------------------------------------------------------- */
 /*      Swap pixel value, and polygon id lines to be ready for the      */
@@ -417,8 +416,8 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
 /* -------------------------------------------------------------------- */
 /*      Report progress, and support interrupts.                        */
 /* -------------------------------------------------------------------- */
-        if( eErr == CE_None 
-            && !pfnProgress( 0.25 + 0.25 * ((iY+1) / (double) nYSize), 
+        if( eErr == CE_None
+            && !pfnProgress( 0.25 + 0.25 * ((iY+1) / (double) nYSize),
                              "", pProgressArg ) )
         {
             CPLError( CE_Failure, CPLE_UserInterrupt, "User terminated" );
@@ -440,7 +439,7 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
         if( oFirstEnum.panPolyIdMap[iPoly] != iPoly )
             continue;
 
-        // Ignore nodata polygons. 
+        // Ignore nodata polygons.
         if( oFirstEnum.panPolyValue[iPoly] == GP_NODATA_MARKER )
             continue;
 
@@ -460,28 +459,50 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
             continue;
         }
 
-        // If our biggest neighbour is larger than the threshold
-        // then we are golden. 
-        if( anPolySizes[anBigNeighbour[iPoly]] >= nSizeThreshold )
-            continue;
+        std::set<int> oSetVisitedPoly;
+        oSetVisitedPoly.insert(iPoly);
 
-#ifdef notdef
-        // Will our neighbours biggest neighbour do?  
-        // Eventually we need something sort of recursive here with
-        // loop detection.
-        if( anPolySizes[anBigNeighbour[anBigNeighbour[iPoly]]] 
-            >= nSizeThreshold )
+        // Walk through our neighbours until we find a polygon large enough
+        int iFinalId = iPoly;
+        bool bFoundBigEnoughPoly = false;
+        while(true)
         {
-            anBigNeighbour[iPoly] = anBigNeighbour[anBigNeighbour[iPoly]];
+            iFinalId = anBigNeighbour[iFinalId];
+            if( iFinalId < 0 )
+            {
+                break;
+            }
+            // If the biggest neighbour is larger than the threshold
+            // then we are golden.
+            if( anPolySizes[iFinalId] >= nSizeThreshold )
+            {
+                bFoundBigEnoughPoly = true;
+                break;
+            }
+            // Check that we don't cycle on an already visited polygon
+            if( oSetVisitedPoly.find(iFinalId) != oSetVisitedPoly.end() )
+                break;
+            oSetVisitedPoly.insert(iFinalId);
+        }
+
+        if( !bFoundBigEnoughPoly )
+        {
+            nFailedMerges++;
+            anBigNeighbour[iPoly] = -1;
             continue;
         }
-#endif
 
-        nFailedMerges++;
-        anBigNeighbour[iPoly] = -1;
-    }									
+        // Map the whole intermediate chain to it
+        int iPolyCur = iPoly;
+        while( anBigNeighbour[iPolyCur] != iFinalId )
+        {
+            int iNextPoly = anBigNeighbour[iPolyCur];
+            anBigNeighbour[iPolyCur] = iFinalId;
+            iPolyCur = iNextPoly;
+        }
+    }
 
-    CPLDebug( "GDALSieveFilter", 
+    CPLDebug( "GDALSieveFilter",
               "Small Polygons: %d, Isolated: %d, Unmergable: %d",
               nSieveTargets, nIsolatedSmall, nFailedMerges );
 
@@ -491,14 +512,13 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
 /*      "final maps" from the first.                                    */
 /* ==================================================================== */
     oSecondEnum.Clear();
-    
 
     for( iY = 0; eErr == CE_None && iY < nYSize; iY++ )
     {
 /* -------------------------------------------------------------------- */
 /*      Read the image data.                                            */
 /* -------------------------------------------------------------------- */
-        eErr = GDALRasterIO( hSrcBand, GF_Read, 0, iY, nXSize, 1, 
+        eErr = GDALRasterIO( hSrcBand, GF_Read, 0, iY, nXSize, 1,
                              panThisLineVal, nXSize, 1, GDT_Int32, 0, 0 );
 
         memcpy( panThisLineWriteVal, panThisLineVal, 4 * nXSize );
@@ -514,12 +534,12 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
 /*      the same thing done in the first pass above).                   */
 /* -------------------------------------------------------------------- */
         if( iY == 0 )
-            oSecondEnum.ProcessLine( 
+            oSecondEnum.ProcessLine(
                 NULL, panThisLineVal, NULL, panThisLineId, nXSize );
         else
             oSecondEnum.ProcessLine(
-                panLastLineVal, panThisLineVal, 
-                panLastLineId,  panThisLineId, 
+                panLastLineVal, panThisLineVal,
+                panLastLineId,  panThisLineId,
                 nXSize );
 
 /* -------------------------------------------------------------------- */
@@ -535,7 +555,7 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
 
                 if( anBigNeighbour[iThisPoly] != -1 )
                 {
-                    panThisLineWriteVal[iX] = 
+                    panThisLineWriteVal[iX] =
                         oFirstEnum.panPolyValue[
                             anBigNeighbour[iThisPoly]];
                 }
@@ -545,7 +565,7 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
 /* -------------------------------------------------------------------- */
 /*      Write the update data out.                                      */
 /* -------------------------------------------------------------------- */
-        eErr = GDALRasterIO( hDstBand, GF_Write, 0, iY, nXSize, 1, 
+        eErr = GDALRasterIO( hDstBand, GF_Write, 0, iY, nXSize, 1,
                              panThisLineWriteVal, nXSize, 1, GDT_Int32, 0, 0 );
 
 /* -------------------------------------------------------------------- */
@@ -563,8 +583,8 @@ GDALSieveFilter( GDALRasterBandH hSrcBand, GDALRasterBandH hMaskBand,
 /* -------------------------------------------------------------------- */
 /*      Report progress, and support interrupts.                        */
 /* -------------------------------------------------------------------- */
-        if( eErr == CE_None 
-            && !pfnProgress( 0.5 + 0.5 * ((iY+1) / (double) nYSize), 
+        if( eErr == CE_None
+            && !pfnProgress( 0.5 + 0.5 * ((iY+1) / (double) nYSize),
                              "", pProgressArg ) )
         {
             CPLError( CE_Failure, CPLE_UserInterrupt, "User terminated" );
