@@ -2,10 +2,10 @@
  * $Id$
  *
  * Project:  GDAL Utilities
- * Purpose:  Commandline application to list info about a given CRS.
+ * Purpose:  Command line application to list info about a given CRS.
  *           Outputs a number of formats (WKT, PROJ.4, etc.).
  * Author:   Frank Warmerdam, warmerdam@pobox.com
- *           Etienne Tourigny, etourigny.dev-at-gmail-dot-com       
+ *           Etienne Tourigny, etourigny.dev-at-gmail-dot-com
  *
  * ****************************************************************************
  * Copyright (c) 1998, Frank Warmerdam
@@ -39,11 +39,11 @@
 
 CPL_CVSID("$Id$");
 
-int FindSRS( const char *pszInput, OGRSpatialReference &oSRS );
-CPLErr PrintSRS( const OGRSpatialReference &oSRS, 
-                 const char * pszOutputType, 
-                 int bPretty, int bPrintSep );
-void PrintSRSOutputTypes( const OGRSpatialReference &oSRS, 
+bool FindSRS( const char *pszInput, OGRSpatialReference &oSRS );
+CPLErr PrintSRS( const OGRSpatialReference &oSRS,
+                 const char * pszOutputType,
+                 bool bPretty, bool bPrintSep );
+void PrintSRSOutputTypes( const OGRSpatialReference &oSRS,
                           const char ** papszOutputTypes );
 int FindEPSG( const OGRSpatialReference &oSRS );
 int SearchCSVForWKT( const char *pszFileCSV, const char *pszTarget );
@@ -52,7 +52,7 @@ int SearchCSVForWKT( const char *pszFileCSV, const char *pszTarget );
 /*                               Usage()                                */
 /************************************************************************/
 
-void Usage(const char* pszErrorMsg = NULL)
+static void Usage(const char* pszErrorMsg = NULL)
 
 {
     printf( "\nUsage: gdalsrsinfo [options] srs_def\n"
@@ -61,7 +61,7 @@ void Usage(const char* pszErrorMsg = NULL)
             "from which to extract SRS information\n"
             "OR any of the usual GDAL/OGR forms "
             "(complete WKT, PROJ.4, EPSG:n or a file containing the SRS)\n"
-            "\n"          
+            "\n"
             "Options: \n"
             "   [--help-general] [-h]  Show help and exit\n"
             "   [-p]                   Pretty-print where applicable (e.g. WKT)\n"
@@ -70,7 +70,7 @@ void Usage(const char* pszErrorMsg = NULL)
             "   [-o out_type]          Output type { default, all, wkt_all,\n"
             "                                        proj4, epsg,\n"
             "                                        wkt, wkt_simple, wkt_noct, wkt_esri,\n"
-            "                                        mapinfo, xml }\n\n" ); 
+            "                                        mapinfo, xml }\n\n" );
 
     if( pszErrorMsg != NULL )
         fprintf(stderr, "\nFAILURE: %s\n", pszErrorMsg);
@@ -87,14 +87,13 @@ void Usage(const char* pszErrorMsg = NULL)
     do { if (i + nExtraArg >= argc) \
         Usage(CPLSPrintf("%s option requires %d argument(s)", argv[i], nExtraArg)); } while(0)
 
-int main( int argc, char ** argv ) 
+int main( int argc, char ** argv )
 
 {
-    int            i;
-    int            bGotSRS = FALSE;
-    int            bPretty = FALSE;
-    int            bValidate = FALSE;
-    int            bFindEPSG = FALSE;
+    bool bGotSRS = false;
+    bool bPretty = false;
+    bool bValidate = false;
+    bool bFindEPSG = false;
     int            nEPSGCode = -1;
     const char     *pszInput = NULL;
     const char     *pszOutputType = "default";
@@ -110,71 +109,6 @@ int main( int argc, char ** argv )
 /*      Register standard GDAL and OGR drivers.                         */
 /* -------------------------------------------------------------------- */
     GDALAllRegister();
-#ifdef OGR_ENABLED
-    OGRRegisterAll();
-#endif
-
-/* -------------------------------------------------------------------- */
-/*      Process --formats option.                                       */
-/*      Code copied from gcore/gdal_misc.cpp and ogr/ogrutils.cpp.      */
-/*      This is not ideal, but is best for more descriptive output and  */
-/*      we don't want to call OGRGeneralCmdLineProcessor().             */
-/* -------------------------------------------------------------------- */ 
-   for( i = 1; i < argc; i++ )
-    {        
-        if( EQUAL(argv[i], "--formats") )
-        {
-            int iDr;
-            
-            /* GDAL formats */
-            printf( "Supported Raster Formats:\n" );
-            for( iDr = 0; iDr < GDALGetDriverCount(); iDr++ )
-            {
-                GDALDriverH hDriver = GDALGetDriver(iDr);
-                const char *pszRWFlag, *pszVirtualIO;
-                
-                if( GDALGetMetadataItem( hDriver, GDAL_DCAP_CREATE, NULL ) )
-                    pszRWFlag = "rw+";
-                else if( GDALGetMetadataItem( hDriver, GDAL_DCAP_CREATECOPY, 
-                                              NULL ) )
-                    pszRWFlag = "rw";
-                else
-                    pszRWFlag = "ro";
-                
-                if( GDALGetMetadataItem( hDriver, GDAL_DCAP_VIRTUALIO, NULL) )
-                    pszVirtualIO = "v";
-                else
-                    pszVirtualIO = "";
-                
-                printf( "  %s (%s%s): %s\n",
-                        GDALGetDriverShortName( hDriver ),
-                        pszRWFlag, pszVirtualIO,
-                        GDALGetDriverLongName( hDriver ) );
-            }
-
-            /* OGR formats */
-#ifdef OGR_ENABLED
-            printf( "\nSupported Vector Formats:\n" );
-            
-            OGRSFDriverRegistrar *poR = OGRSFDriverRegistrar::GetRegistrar();
-            
-            for( iDr = 0; iDr < poR->GetDriverCount(); iDr++ )
-            {
-                OGRSFDriver *poDriver = poR->GetDriver(iDr);
-                
-                if( poDriver->TestCapability( ODrCCreateDataSource ) )
-                    printf( "  -> \"%s\" (read/write)\n", 
-                            poDriver->GetName() );
-                else
-                    printf( "  -> \"%s\" (readonly)\n", 
-                            poDriver->GetName() );
-            }
-            
-#endif
-            exit(1);
-            
-        }
-    }
 
 /* -------------------------------------------------------------------- */
 /*      Register standard GDAL drivers, and process generic GDAL        */
@@ -187,7 +121,7 @@ int main( int argc, char ** argv )
 /* -------------------------------------------------------------------- */
 /*      Parse arguments.                                                */
 /* -------------------------------------------------------------------- */
-    for( i = 1; i < argc; i++ )
+    for( int i = 1; i < argc; i++ )
     {
         CPLDebug( "gdalsrsinfo", "got arg #%d : [%s]", i, argv[i] );
 
@@ -195,26 +129,27 @@ int main( int argc, char ** argv )
         {
             printf("%s was compiled against GDAL %s and is running against GDAL %s\n",
                    argv[0], GDAL_RELEASE_NAME, GDALVersionInfo("RELEASE_NAME"));
+            CSLDestroy( argv );
             return 0;
         }
         else if( EQUAL(argv[i], "-h") || EQUAL(argv[i], "--help") )
             Usage();
         else if( EQUAL(argv[i], "-e") )
-            bFindEPSG = TRUE;
+            bFindEPSG = true;
         else if( EQUAL(argv[i], "-o") )
         {
             CHECK_HAS_ENOUGH_ADDITIONAL_ARGS(1);
             pszOutputType = argv[++i];
         }
         else if( EQUAL(argv[i], "-p") )
-            bPretty = TRUE;
+            bPretty = true;
         else if( EQUAL(argv[i], "-V") )
-            bValidate = TRUE;
+            bValidate = true;
         else if( argv[i][0] == '-' )
         {
             Usage(CPLSPrintf("Unknown option name '%s'", argv[i]));
         }
-        else  
+        else
             pszInput = argv[i];
     }
 
@@ -224,12 +159,15 @@ int main( int argc, char ** argv )
     }
 
     /* Search for SRS */
-    bGotSRS = FindSRS( pszInput, oSRS );
+    /* coverity[tainted_data] */
+    bGotSRS = FindSRS( pszInput, oSRS ) == TRUE;
 
-    CPLDebug( "gdalsrsinfo", 
+    CPLDebug( "gdalsrsinfo",
               "bGotSRS: %d bValidate: %d pszOutputType: %s bPretty: %d",
-              bGotSRS, bValidate, pszOutputType, bPretty  );
-      
+              static_cast<int>(bGotSRS),
+              static_cast<int>(bValidate),
+              pszOutputType,
+              static_cast<int>(bPretty) );
 
     /* Make sure we got a SRS */
     if ( ! bGotSRS ) {
@@ -242,14 +180,14 @@ int main( int argc, char ** argv )
 
         /* Find EPSG code - experimental */
         if ( EQUAL(pszOutputType,"epsg") )
-            bFindEPSG = TRUE;
+            bFindEPSG = true;
         if ( bFindEPSG ) {
             CPLError( CE_Warning, CPLE_AppDefined,
                       "EPSG detection is experimental and requires new data files (see bug #4345)" );
             nEPSGCode = FindEPSG( oSRS );
             /* If found, replace oSRS based on EPSG code */
             if(nEPSGCode != -1) {
-                CPLDebug( "gdalsrsinfo", 
+                CPLDebug( "gdalsrsinfo",
                           "Found EPSG code %d", nEPSGCode );
                 OGRSpatialReference oSRS2;
                 if ( oSRS2.importFromEPSG( nEPSGCode ) == OGRERR_NONE )
@@ -270,25 +208,25 @@ int main( int argc, char ** argv )
             else
                 printf( "\nValidate Succeeds\n" );
         }
-        
+
         /* Output */
         if ( EQUAL("default", pszOutputType ) ) {
             /* does this work in MSVC? */
-            const char* papszOutputTypes[] = 
-                { "proj4", "wkt", NULL };      
-            if ( bFindEPSG ) 
+            const char* papszOutputTypes[] =
+                { "proj4", "wkt", NULL };
+            if ( bFindEPSG )
                 printf("\nEPSG:%d\n",nEPSGCode);
             PrintSRSOutputTypes( oSRS, papszOutputTypes );
         }
         else if ( EQUAL("all", pszOutputType ) ) {
-            if ( bFindEPSG ) 
+            if ( bFindEPSG )
                 printf("\nEPSG:%d\n\n",nEPSGCode);
-            const char* papszOutputTypes[] = 
+            const char* papszOutputTypes[] =
                 {"proj4","wkt","wkt_simple","wkt_noct","wkt_esri","mapinfo","xml",NULL};
             PrintSRSOutputTypes( oSRS, papszOutputTypes );
         }
         else if ( EQUAL("wkt_all", pszOutputType ) ) {
-            const char* papszOutputTypes[] = 
+            const char* papszOutputTypes[] =
                 { "wkt", "wkt_simple", "wkt_noct", "wkt_esri", NULL };
             PrintSRSOutputTypes( oSRS, papszOutputTypes );
         }
@@ -300,16 +238,13 @@ int main( int argc, char ** argv )
             else
                 PrintSRS( oSRS, pszOutputType, bPretty, FALSE );
             if ( bPretty )
-                printf( "\n" );        
+                printf( "\n" );
         }
-
     }
 
     /* cleanup anything left */
     GDALDestroyDriverManager();
-#ifdef OGR_ENABLED
     OGRCleanupAll();
-#endif
     CSLDestroy( argv );
 
     return 0;
@@ -320,98 +255,80 @@ int main( int argc, char ** argv )
 /*                                                                      */
 /*      Search for SRS from pszInput, update oSRS.                      */
 /************************************************************************/
-int FindSRS( const char *pszInput, OGRSpatialReference &oSRS )
+bool FindSRS( const char *pszInput, OGRSpatialReference &oSRS )
 
 {
-    int            bGotSRS = FALSE;
-    VSILFILE      *fp = NULL;
-    GDALDataset	  *poGDALDS = NULL; 
-#ifdef OGR_ENABLED
-    OGRDataSource *poOGRDS = NULL;
+    bool bGotSRS = false;
+    GDALDataset *poGDALDS = NULL;
     OGRLayer      *poLayer = NULL;
-#endif
-    char           *pszProjection = NULL;
+    const char    *pszProjection = NULL;
     CPLErrorHandler oErrorHandler = NULL;
-    int bIsFile = FALSE;
-    OGRErr eErr = CE_None;
-    int bDebug  = FALSE;
+    bool bIsFile = false;
+    OGRErr eErr = OGRERR_NONE;
 
-    /* temporarily supress error messages we may get from xOpen() */
-    bDebug = CSLTestBoolean(CPLGetConfigOption("CPL_DEBUG", "OFF"));
-    if ( ! bDebug )
+    /* temporarily suppress error messages we may get from xOpen() */
+    bool bDebug = CPLTestBool(CPLGetConfigOption("CPL_DEBUG", "OFF"));
+    if( !bDebug )
         oErrorHandler = CPLSetErrorHandler ( CPLQuietErrorHandler );
 
     /* Test if argument is a file */
-    fp = VSIFOpenL( pszInput, "r" );
+    VSILFILE *fp = VSIFOpenL( pszInput, "r" );
     if ( fp )  {
-        bIsFile = TRUE;
+        bIsFile = true;
         VSIFCloseL( fp );
         CPLDebug( "gdalsrsinfo", "argument is a file" );
-    } 
-       
-    /* try to open with GDAL */
-    if( strncmp(pszInput, "http://spatialreference.org/",
-                strlen("http://spatialreference.org/")) != 0 )
-    {
-        CPLDebug( "gdalsrsinfo", "trying to open with GDAL" );
-        poGDALDS = (GDALDataset *) GDALOpen( pszInput, GA_ReadOnly );
     }
-    if ( poGDALDS != NULL && poGDALDS->GetProjectionRef( ) != NULL ) {
-        pszProjection = (char *) poGDALDS->GetProjectionRef( );
-        if( oSRS.importFromWkt( &pszProjection ) == CE_None ) {
-            CPLDebug( "gdalsrsinfo", "got SRS from GDAL" );
-            bGotSRS = TRUE;
-        }
-        GDALClose( (GDALDatasetH) poGDALDS );
-        if ( ! bGotSRS ) 
-            CPLDebug( "gdalsrsinfo", "did not open with GDAL" );
-    }    
-    
-#ifdef OGR_ENABLED
-    /* if unsuccessful, try to open with OGR */
-    if ( ! bGotSRS ) {
-        if( strncmp(pszInput, "http://spatialreference.org/",
-                    strlen("http://spatialreference.org/")) != 0 )
+
+    /* try to open with GDAL */
+    if( !STARTS_WITH(pszInput, "http://spatialreference.org/") )    {
+        CPLDebug( "gdalsrsinfo", "trying to open with GDAL" );
+        poGDALDS = (GDALDataset *) GDALOpenEx( pszInput, 0, NULL, NULL, NULL );
+    }
+    if ( poGDALDS != NULL ) {
+        pszProjection = poGDALDS->GetProjectionRef( );
+        if( pszProjection != NULL && pszProjection[0] != '\0' )
         {
-            CPLDebug( "gdalsrsinfo", "trying to open with OGR" );
-            poOGRDS = OGRSFDriverRegistrar::Open( pszInput, FALSE, NULL );
+            char* pszProjectionTmp = (char*) pszProjection;
+            if( oSRS.importFromWkt( &pszProjectionTmp ) == OGRERR_NONE ) {
+                CPLDebug( "gdalsrsinfo", "got SRS from GDAL" );
+                bGotSRS = true;
+            }
         }
-        if( poOGRDS != NULL ) {
-            poLayer = poOGRDS->GetLayer( 0 );
+        else if( poGDALDS->GetLayerCount() > 0 )
+        {
+            poLayer = poGDALDS->GetLayer( 0 );
             if ( poLayer != NULL ) {
                 OGRSpatialReference *poSRS = poLayer->GetSpatialRef( );
                 if ( poSRS != NULL ) {
                     CPLDebug( "gdalsrsinfo", "got SRS from OGR" );
-                    bGotSRS = TRUE;
+                    bGotSRS = true;
                     OGRSpatialReference* poSRSClone = poSRS->Clone();
                     oSRS = *poSRSClone;
                     OGRSpatialReference::DestroySpatialReference( poSRSClone );
                 }
             }
-            OGRDataSource::DestroyDataSource( poOGRDS );
-            poOGRDS = NULL;
-        } 
-        if ( ! bGotSRS ) 
-            CPLDebug( "gdalsrsinfo", "did not open with OGR" );
+        }
+        GDALClose( (GDALDatasetH) poGDALDS );
+        if ( ! bGotSRS )
+            CPLDebug( "gdalsrsinfo", "did not open with GDAL" );
     }
-#endif // OGR_ENABLED
-    
+
     /* Try ESRI file */
     if ( ! bGotSRS && bIsFile && (strstr(pszInput,".prj") != NULL) ) {
-        CPLDebug( "gdalsrsinfo", 
+        CPLDebug( "gdalsrsinfo",
                   "trying to get SRS from ESRI .prj file [%s]", pszInput );
 
         char **pszTemp;
         if ( strstr(pszInput,"ESRI::") != NULL )
             pszTemp = CSLLoad( pszInput+6 );
-        else 
+        else
             pszTemp = CSLLoad( pszInput );
 
         if( pszTemp ) {
             eErr = oSRS.importFromESRI( pszTemp );
             CSLDestroy( pszTemp );
         }
-        else 
+        else
             eErr = OGRERR_UNSUPPORTED_SRS;
 
         if( eErr != OGRERR_NONE ) {
@@ -419,29 +336,29 @@ int FindSRS( const char *pszInput, OGRSpatialReference &oSRS )
         }
         else {
             CPLDebug( "gdalsrsinfo", "got SRS from ESRI .prj file" );
-            bGotSRS = TRUE;
+            bGotSRS = true;
         }
     }
 
     /* Last resort, try OSRSetFromUserInput() */
     if ( ! bGotSRS ) {
-        CPLDebug( "gdalsrsinfo", 
+        CPLDebug( "gdalsrsinfo",
                   "trying to get SRS from user input [%s]", pszInput );
 
         eErr = oSRS.SetFromUserInput( pszInput );
- 
+
        if(  eErr != OGRERR_NONE ) {
             CPLDebug( "gdalsrsinfo", "did not get SRS from user input" );
         }
         else {
             CPLDebug( "gdalsrsinfo", "got SRS from user input" );
-            bGotSRS = TRUE;
+            bGotSRS = true;
         }
     }
-    
+
     /* restore error messages */
-    if ( ! bDebug )
-        CPLSetErrorHandler ( oErrorHandler );	
+    if( !bDebug )
+        CPLSetErrorHandler ( oErrorHandler );
 
 
     return bGotSRS;
@@ -453,17 +370,18 @@ int FindSRS( const char *pszInput, OGRSpatialReference &oSRS )
 /*                                                                      */
 /*      Print spatial reference in specified format.                    */
 /************************************************************************/
-CPLErr PrintSRS( const OGRSpatialReference &oSRS, 
-                 const char * pszOutputType, 
-                 int bPretty, int bPrintSep )
+CPLErr PrintSRS( const OGRSpatialReference &oSRS,
+                 const char * pszOutputType,
+                 bool bPretty, bool bPrintSep )
 
 {
     if ( ! pszOutputType || EQUAL(pszOutputType,""))
         return CE_None;
 
     CPLDebug( "gdalsrsinfo", "PrintSRS( oSRS, %s, %d, %d )\n",
-              pszOutputType, bPretty, bPrintSep );
-    
+              pszOutputType, static_cast<int>(bPretty),
+              static_cast<int>(bPrintSep) );
+
     char *pszOutput = NULL;
 
     if ( EQUAL("proj4", pszOutputType ) ) {
@@ -474,49 +392,49 @@ CPLErr PrintSRS( const OGRSpatialReference &oSRS,
 
     else if ( EQUAL("wkt", pszOutputType ) ) {
         if ( bPrintSep ) printf("OGC WKT :\n");
-        if ( bPretty ) 
+        if ( bPretty )
             oSRS.exportToPrettyWkt( &pszOutput, FALSE );
         else
             oSRS.exportToWkt( &pszOutput );
         printf("%s\n",pszOutput);
     }
-        
+
     else if (  EQUAL("wkt_simple", pszOutputType ) ) {
         if ( bPrintSep ) printf("OGC WKT (simple) :\n");
         oSRS.exportToPrettyWkt( &pszOutput, TRUE );
         printf("%s\n",pszOutput);
     }
-        
+
     else if ( EQUAL("wkt_noct", pszOutputType ) ) {
         if (  bPrintSep ) printf("OGC WKT (no CT) :\n");
         OGRSpatialReference *poSRS = oSRS.Clone();
         poSRS->StripCTParms( );
-        if ( bPretty ) 
+        if ( bPretty )
             poSRS->exportToPrettyWkt( &pszOutput, FALSE );
         else
             poSRS->exportToWkt( &pszOutput );
         OGRSpatialReference::DestroySpatialReference( poSRS );
         printf("%s\n",pszOutput);
     }
-        
+
     else if ( EQUAL("wkt_esri", pszOutputType ) ) {
         if ( bPrintSep ) printf("ESRI WKT :\n");
         OGRSpatialReference *poSRS = oSRS.Clone();
         poSRS->morphToESRI( );
-        if ( bPretty ) 
+        if ( bPretty )
             poSRS->exportToPrettyWkt( &pszOutput, FALSE );
         else
             poSRS->exportToWkt( &pszOutput );
         OGRSpatialReference::DestroySpatialReference( poSRS );
         printf("%s\n",pszOutput);
     }
-        
+
     else if ( EQUAL("mapinfo", pszOutputType ) ) {
         if ( bPrintSep ) printf("MAPINFO : ");
         oSRS.exportToMICoordSys( &pszOutput );
         printf("\'%s\'\n",pszOutput);
     }
-        
+
     else if ( EQUAL("xml", pszOutputType ) ) {
         if ( bPrintSep ) printf("XML :\n");
         oSRS.exportToXML( &pszOutput, NULL );
@@ -531,7 +449,7 @@ CPLErr PrintSRS( const OGRSpatialReference &oSRS,
     }
 
     CPLFree( pszOutput );
-    
+
     return CE_None;
 }
 
@@ -540,15 +458,15 @@ CPLErr PrintSRS( const OGRSpatialReference &oSRS,
 /*                                                                      */
 /*      Print spatial reference in specified formats.                   */
 /************************************************************************/
-void PrintSRSOutputTypes( const OGRSpatialReference &oSRS, 
+void PrintSRSOutputTypes( const OGRSpatialReference &oSRS,
                           const char ** papszOutputTypes )
-    
+
 {
     int nOutputTypes = CSLCount((char**)papszOutputTypes);
     printf( "\n" );
     for ( int i=0; i<nOutputTypes; i++ ) {
-        PrintSRS( oSRS, papszOutputTypes[i], TRUE, TRUE );
-        printf( "\n" );        
+        PrintSRS( oSRS, papszOutputTypes[i], true, true );
+        printf( "\n" );
     }
 }
 
@@ -572,7 +490,7 @@ int SearchCSVForWKT( const char *pszFileCSV, const char *pszTarget )
     int nCode = 0;
     int nFound = -1;
 
-    CPLDebug( "gdalsrsinfo", 
+    CPLDebug( "gdalsrsinfo",
               "SearchCSVForWKT()\nfile=%s\nWKT=%s\n",
               pszFileCSV, pszTarget);
 
@@ -591,15 +509,15 @@ int SearchCSVForWKT( const char *pszFileCSV, const char *pszTarget )
 
     /* support gzipped file */
     if ( strstr( pszFileCSV,".gz") != NULL )
-        sprintf( szTemp, "/vsigzip/%s", pszFilename);
+        snprintf( szTemp, sizeof(szTemp), "/vsigzip/%s", pszFilename);
     else
-        sprintf( szTemp, "%s", pszFilename);
+        snprintf( szTemp, sizeof(szTemp), "%s", pszFilename);
 
     CPLDebug( "gdalsrsinfo", "SearchCSVForWKT() using file %s",
               szTemp );
 
     fp = VSIFOpenL( szTemp, "r" );
-    if( fp == NULL ) 
+    if( fp == NULL )
     {
         CPLDebug( "gdalsrsinfo", "could not open support file %s",
                   pszFilename );
@@ -622,7 +540,7 @@ int SearchCSVForWKT( const char *pszFileCSV, const char *pszTarget )
             continue;
             /* do nothing */;
 
-        // else if( EQUALN(pszLine,"include ",8) )
+        // else if( STARTS_WITH_CI(pszLine, "include ") )
         // {
         //     eErr = importFromDict( pszLine + 8, pszCode );
         //     if( eErr != OGRERR_UNSUPPORTED_SRS )
@@ -635,18 +553,18 @@ int SearchCSVForWKT( const char *pszFileCSV, const char *pszTarget )
         pszTemp = strstr(pszLine,",");
         if (pszTemp)
         {
-            nPos = pszTemp - pszLine;
+            nPos = static_cast<int>(pszTemp - pszLine);
 
             if ( nPos == 0 )
                 continue;
-            
+
             strncpy( szTemp, pszLine, nPos );
             szTemp[nPos] = '\0';
             nCode = atoi(szTemp);
 
             pszWKT = (char *) pszLine + nPos +1;
 
-            // CPLDebug( "gdalsrsinfo", 
+            // CPLDebug( "gdalsrsinfo",
             //           "code=%d\nWKT=\n[%s]\ntarget=\n[%s]\n",
             //           nCode,pszWKT, pszTarget );
 
@@ -660,15 +578,15 @@ int SearchCSVForWKT( const char *pszFileCSV, const char *pszTarget )
             }
         }
     }
-    
+
     VSIFCloseL( fp );
 
     return nFound;
 
 }
 
-/* TODO 
-   - search for well-known values (AutoIdentifyEPSG()) 
+/* TODO
+   - search for well-known values (AutoIdentifyEPSG())
 
    - should we search .override.csv files?
 
@@ -691,9 +609,8 @@ int FindEPSG( const OGRSpatialReference &oSRS )
 {
     char *pszWKT = NULL;
     char *pszESRI = NULL;
-    int nFound = -1;
     OGRSpatialReference *poSRS = NULL;
- 
+
     poSRS = oSRS.Clone();
     poSRS->StripCTParms();
     poSRS->exportToWkt( &pszWKT );
@@ -709,15 +626,14 @@ int FindEPSG( const OGRSpatialReference &oSRS )
 
     /* search for EPSG code in epsg_*.wkt.gz files */
     /* using ESRI WKT for now, as it seems to work best */
-    nFound = SearchCSVForWKT( "epsg_esri.wkt.gz", pszESRI );
+    int nFound = SearchCSVForWKT( "epsg_esri.wkt.gz", pszESRI );
     if ( nFound == -1 )
         nFound = SearchCSVForWKT( "epsg_ogc_simple.wkt.gz", pszESRI );
     if ( nFound == -1 )
         nFound = SearchCSVForWKT( "epsg_ogc.wkt.gz", pszESRI );
 
-    
     CPLFree( pszWKT );
-    CPLFree( pszESRI );    
-    
+    CPLFree( pszESRI );
+
     return nFound;
 }
