@@ -1,8 +1,8 @@
 /******************************************************************************
- * $Id: multireadtest.cpp 10645 2007-01-18 02:22:39Z warmerdam $
+ * $Id: multireadtest.cpp 33615 2016-03-02 20:19:22Z goatbar $
  *
  * Project:  GDAL Utilities
- * Purpose:  Multithreading test application.
+ * Purpose:  Multi-threading test application.
  * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
  ******************************************************************************
@@ -32,7 +32,7 @@
 #include "cpl_multiproc.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: multireadtest.cpp 10645 2007-01-18 02:22:39Z warmerdam $");
+CPL_CVSID("$Id: multireadtest.cpp 33615 2016-03-02 20:19:22Z goatbar $");
 
 static int nThreadCount = 4, nIterations = 1, bLockOnOpen = TRUE;
 static int nOpenIterations = 1;
@@ -40,7 +40,7 @@ static volatile int nPendingThreads = 0;
 static const char *pszFilename = NULL;
 static int nChecksum = 0;
 
-static void *pGlobalMutex = NULL;
+static CPLMutex *pGlobalMutex = NULL;
 
 static void WorkerFunc( void * );
 
@@ -87,7 +87,7 @@ int main( int argc, char ** argv )
             pszFilename = argv[iArg];
         else
         {
-            printf( "Unrecognised argument: %s\n", argv[iArg] );
+            printf( "Unrecognized argument: %s\n", argv[iArg] );
             Usage();
         }
     }
@@ -112,14 +112,14 @@ int main( int argc, char ** argv )
     if( hDS == NULL )
         exit( 1 );
 
-    nChecksum = GDALChecksumImage( GDALGetRasterBand( hDS, 1 ), 
-                                   0, 0, 
-                                   GDALGetRasterXSize( hDS ), 
+    nChecksum = GDALChecksumImage( GDALGetRasterBand( hDS, 1 ),
+                                   0, 0,
+                                   GDALGetRasterXSize( hDS ),
                                    GDALGetRasterYSize( hDS ) );
-    
+
     GDALClose( hDS );
 
-    printf( "Got checksum %d, launching %d worker threads on %s, %d iterations.\n", 
+    printf( "Got checksum %d, launching %d worker threads on %s, %d iterations.\n",
             nChecksum, nThreadCount, pszFilename, nIterations );
 
 /* -------------------------------------------------------------------- */
@@ -147,10 +147,12 @@ int main( int argc, char ** argv )
     CPLReleaseMutex( pGlobalMutex );
 
     printf( "All threads complete.\n" );
-    
+
     CSLDestroy( argv );
-    
+
     GDALDestroyDriverManager();
+
+    return 0;
 }
 
 
@@ -177,10 +179,10 @@ static void WorkerFunc( void * )
         for( iIter = 0; iIter < nIterations && hDS != NULL; iIter++ )
         {
             int nMyChecksum;
-        
-            nMyChecksum = GDALChecksumImage( GDALGetRasterBand( hDS, 1 ), 
-                                             0, 0, 
-                                             GDALGetRasterXSize( hDS ), 
+
+            nMyChecksum = GDALChecksumImage( GDALGetRasterBand( hDS, 1 ),
+                                             0, 0,
+                                             GDALGetRasterXSize( hDS ),
                                              GDALGetRasterYSize( hDS ) );
 
             if( nMyChecksum != nChecksum )
