@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: aigopen.c 22159 2011-04-14 18:18:54Z warmerdam $
+ * $Id: aigopen.c 33720 2016-03-15 00:39:53Z goatbar $
  *
  * Project:  Arc/Info Binary Grid Translator
  * Purpose:  Grid file access cover API for non-GDAL use.
@@ -7,6 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 1999, Frank Warmerdam
+ * Copyright (c) 2009-2010, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -29,7 +30,9 @@
 
 #include "aigrid.h"
 
-CPL_CVSID("$Id: aigopen.c 22159 2011-04-14 18:18:54Z warmerdam $");
+CPL_CVSID("$Id: aigopen.c 33720 2016-03-15 00:39:53Z goatbar $");
+
+CPL_INLINE static void CPL_IGNORE_RET_VAL_INT(CPL_UNUSED int unused) {}
 
 /************************************************************************/
 /*                              AIGOpen()                               */
@@ -53,7 +56,7 @@ AIGInfo_t *AIGOpen( const char * pszInputName, const char * pszAccess )
     {
         int      i;
 
-        for( i = strlen(pszCoverName)-1; i > 0; i-- )
+        for( i = (int)strlen(pszCoverName)-1; i > 0; i-- )
         {
             if( pszCoverName[i] == '\\' || pszCoverName[i] == '/' )
             {
@@ -98,7 +101,7 @@ AIGInfo_t *AIGOpen( const char * pszInputName, const char * pszAccess )
 /* -------------------------------------------------------------------- */
     if (psInfo->dfCellSizeX <= 0 || psInfo->dfCellSizeY <= 0)
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "Illegal cell size : %f x %f",
                   psInfo->dfCellSizeX, psInfo->dfCellSizeY );
         AIGClose( psInfo );
@@ -106,15 +109,15 @@ AIGInfo_t *AIGOpen( const char * pszInputName, const char * pszAccess )
     }
 
     psInfo->nPixels = (int)
-        ((psInfo->dfURX - psInfo->dfLLX + 0.5 * psInfo->dfCellSizeX) 
+        ((psInfo->dfURX - psInfo->dfLLX + 0.5 * psInfo->dfCellSizeX)
 		/ psInfo->dfCellSizeX);
     psInfo->nLines = (int)
-        ((psInfo->dfURY - psInfo->dfLLY + 0.5 * psInfo->dfCellSizeY) 
+        ((psInfo->dfURY - psInfo->dfLLY + 0.5 * psInfo->dfCellSizeY)
 		/ psInfo->dfCellSizeY);
-    
+
     if (psInfo->nPixels <= 0 || psInfo->nLines <= 0)
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "Invalid raster dimensions : %d x %d",
                   psInfo->nPixels, psInfo->nLines );
         AIGClose( psInfo );
@@ -126,7 +129,7 @@ AIGInfo_t *AIGOpen( const char * pszInputName, const char * pszAccess )
         psInfo->nBlockXSize > INT_MAX / psInfo->nBlocksPerRow ||
         psInfo->nBlockYSize > INT_MAX / psInfo->nBlocksPerColumn)
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "Invalid block characteristics: nBlockXSize=%d, "
                   "nBlockYSize=%d, nBlocksPerRow=%d, nBlocksPerColumn=%d",
                   psInfo->nBlockXSize, psInfo->nBlockYSize,
@@ -135,6 +138,13 @@ AIGInfo_t *AIGOpen( const char * pszInputName, const char * pszAccess )
         return NULL;
     }
 
+    if (psInfo->nBlocksPerRow > INT_MAX / psInfo->nBlocksPerColumn)
+    {
+        CPLError(CE_Failure, CPLE_OutOfMemory, "Too many blocks");
+        AIGClose( psInfo );
+        return NULL;
+    }
+
     psInfo->nTileXSize = psInfo->nBlockXSize * psInfo->nBlocksPerRow;
     psInfo->nTileYSize = psInfo->nBlockYSize * psInfo->nBlocksPerColumn;
 
@@ -144,6 +154,8 @@ AIGInfo_t *AIGOpen( const char * pszInputName, const char * pszAccess )
     if (psInfo->nTilesPerRow > INT_MAX / psInfo->nTilesPerColumn)
     {
         CPLError(CE_Failure, CPLE_OutOfMemory, "Too many tiles");
+        psInfo->nTilesPerRow = 0; /* to avoid int32 overflow in AIGClose() */
+        psInfo->nTilesPerColumn = 0;
         AIGClose( psInfo );
         return NULL;
     }
@@ -151,12 +163,11 @@ AIGInfo_t *AIGOpen( const char * pszInputName, const char * pszAccess )
 /* -------------------------------------------------------------------- */
 /*      Setup tile infos, but defer reading of tile data.               */
 /* -------------------------------------------------------------------- */
-    psInfo->pasTileInfo = (AIGTileInfo *) 
-        VSICalloc(sizeof(AIGTileInfo),
+    psInfo->pasTileInfo = (AIGTileInfo *)
+        VSI_CALLOC_VERBOSE(sizeof(AIGTileInfo),
                   psInfo->nTilesPerRow * psInfo->nTilesPerColumn);
     if (psInfo->pasTileInfo == NULL)
     {
-        CPLError(CE_Failure, CPLE_OutOfMemory, "Cannot allocate tile info array");
         AIGClose( psInfo );
         return NULL;
     }
@@ -183,6 +194,7 @@ CPLErr AIGAccessTile( AIGInfo_t *psInfo, int iTileX, int iTileY )
     char szBasename[20];
     char *pszFilename;
     AIGTileInfo *psTInfo;
+    const size_t nFilenameLen = strlen(psInfo->pszCoverName)+40;
 
 /* -------------------------------------------------------------------- */
 /*      Identify our tile.                                              */
@@ -203,21 +215,21 @@ CPLErr AIGAccessTile( AIGInfo_t *psInfo, int iTileX, int iTileY )
 /*      Compute the basename.                                           */
 /* -------------------------------------------------------------------- */
     if( iTileY == 0 )
-        sprintf( szBasename, "w%03d001", iTileX + 1 );
+        snprintf( szBasename, sizeof(szBasename), "w%03d001", iTileX + 1 );
     else if( iTileY == 1 )
-        sprintf( szBasename, "w%03d000", iTileX + 1 );
+        snprintf( szBasename, sizeof(szBasename), "w%03d000", iTileX + 1 );
     else
-        sprintf( szBasename, "z%03d%03d", iTileX + 1, iTileY - 1 );
-    
+        snprintf( szBasename, sizeof(szBasename), "z%03d%03d", iTileX + 1, iTileY - 1 );
+
 /* -------------------------------------------------------------------- */
 /*      Open the file w001001.adf file itself.                          */
 /* -------------------------------------------------------------------- */
-    pszFilename = (char *) CPLMalloc(strlen(psInfo->pszCoverName)+40);
-    sprintf( pszFilename, "%s/%s.adf", psInfo->pszCoverName, szBasename );
+    pszFilename = (char *) CPLMalloc(nFilenameLen);
+    snprintf( pszFilename, nFilenameLen, "%s/%s.adf", psInfo->pszCoverName, szBasename );
 
     psTInfo->fpGrid = AIGLLOpen( pszFilename, "rb" );
     psTInfo->bTriedToLoad = TRUE;
-    
+
     if( psTInfo->fpGrid == NULL )
     {
         CPLError( CE_Warning, CPLE_OpenFailed,
@@ -229,7 +241,7 @@ CPLErr AIGAccessTile( AIGInfo_t *psInfo, int iTileX, int iTileY )
 
     CPLFree( pszFilename );
     pszFilename = NULL;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Read the block index file.                                      */
 /* -------------------------------------------------------------------- */
@@ -250,7 +262,7 @@ CPLErr AIGReadTile( AIGInfo_t * psInfo, int nBlockXOff, int nBlockYOff,
     AIGTileInfo *psTInfo;
 
 /* -------------------------------------------------------------------- */
-/*      Compute our tile, and ensure it is accessable (open).  Then     */
+/*      Compute our tile, and ensure it is accessible (open).  Then     */
 /*      reduce block x/y values to be the block within that tile.       */
 /* -------------------------------------------------------------------- */
     iTileX = nBlockXOff / psInfo->nBlocksPerRow;
@@ -276,15 +288,15 @@ CPLErr AIGReadTile( AIGInfo_t * psInfo, int nBlockXOff, int nBlockYOff,
             panData[i] = ESRI_GRID_NO_DATA;
         return CE_None;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      validate block id.                                              */
 /* -------------------------------------------------------------------- */
     nBlockID = nBlockXOff + nBlockYOff * psInfo->nBlocksPerRow;
-    if( nBlockID < 0 
+    if( nBlockID < 0
         || nBlockID >= psInfo->nBlocksPerRow * psInfo->nBlocksPerColumn )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "Illegal block requested." );
         return CE_Failure;
     }
@@ -292,14 +304,14 @@ CPLErr AIGReadTile( AIGInfo_t * psInfo, int nBlockXOff, int nBlockYOff,
     if( nBlockID >= psTInfo->nBlocks )
     {
         int i;
-        CPLDebug( "AIG", 
+        CPLDebug( "AIG",
                   "Request legal block, but from beyond end of block map.\n"
                   "Assuming all nodata." );
         for( i = psInfo->nBlockXSize * psInfo->nBlockYSize - 1; i >= 0; i-- )
             panData[i] = ESRI_GRID_NO_DATA;
         return CE_None;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Read block.                                                     */
 /* -------------------------------------------------------------------- */
@@ -340,7 +352,7 @@ CPLErr AIGReadFloatTile( AIGInfo_t * psInfo, int nBlockXOff, int nBlockYOff,
     AIGTileInfo *psTInfo;
 
 /* -------------------------------------------------------------------- */
-/*      Compute our tile, and ensure it is accessable (open).  Then     */
+/*      Compute our tile, and ensure it is accessible (open).  Then     */
 /*      reduce block x/y values to be the block within that tile.       */
 /* -------------------------------------------------------------------- */
     iTileX = nBlockXOff / psInfo->nBlocksPerRow;
@@ -366,15 +378,15 @@ CPLErr AIGReadFloatTile( AIGInfo_t * psInfo, int nBlockXOff, int nBlockYOff,
             pafData[i] = ESRI_GRID_FLOAT_NO_DATA;
         return CE_None;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      validate block id.                                              */
 /* -------------------------------------------------------------------- */
     nBlockID = nBlockXOff + nBlockYOff * psInfo->nBlocksPerRow;
-    if( nBlockID < 0 
+    if( nBlockID < 0
         || nBlockID >= psInfo->nBlocksPerRow * psInfo->nBlocksPerColumn )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "Illegal block requested." );
         return CE_Failure;
     }
@@ -382,14 +394,14 @@ CPLErr AIGReadFloatTile( AIGInfo_t * psInfo, int nBlockXOff, int nBlockYOff,
     if( nBlockID >= psTInfo->nBlocks )
     {
         int i;
-        CPLDebug( "AIG", 
+        CPLDebug( "AIG",
                   "Request legal block, but from beyond end of block map.\n"
                   "Assuming all nodata." );
         for( i = psInfo->nBlockXSize * psInfo->nBlockYSize - 1; i >= 0; i-- )
             pafData[i] = ESRI_GRID_FLOAT_NO_DATA;
         return CE_None;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Read block.                                                     */
 /* -------------------------------------------------------------------- */
@@ -397,7 +409,7 @@ CPLErr AIGReadFloatTile( AIGInfo_t * psInfo, int nBlockXOff, int nBlockYOff,
                          psTInfo->panBlockOffset[nBlockID],
                          psTInfo->panBlockSize[nBlockID],
                          psInfo->nBlockXSize, psInfo->nBlockYSize,
-                         (GInt32 *) pafData, psInfo->nCellType, 
+                         (GInt32 *) pafData, psInfo->nCellType,
                          psInfo->bCompressed );
 
 /* -------------------------------------------------------------------- */
@@ -424,17 +436,20 @@ CPLErr AIGReadFloatTile( AIGInfo_t * psInfo, int nBlockXOff, int nBlockYOff,
 void AIGClose( AIGInfo_t * psInfo )
 
 {
-    int nTileCount = psInfo->nTilesPerRow * psInfo->nTilesPerColumn;
-    int iTile;
-
-    for( iTile = 0; iTile < nTileCount; iTile++ )
+    if( psInfo->pasTileInfo != NULL )
     {
-        if( psInfo->pasTileInfo[iTile].fpGrid )
-        {
-            VSIFCloseL( psInfo->pasTileInfo[iTile].fpGrid );
+        int nTileCount = psInfo->nTilesPerRow * psInfo->nTilesPerColumn;
+        int iTile;
 
-            CPLFree( psInfo->pasTileInfo[iTile].panBlockOffset );
-            CPLFree( psInfo->pasTileInfo[iTile].panBlockSize );
+        for( iTile = 0; iTile < nTileCount; iTile++ )
+        {
+            if( psInfo->pasTileInfo[iTile].fpGrid )
+            {
+                CPL_IGNORE_RET_VAL_INT(VSIFCloseL( psInfo->pasTileInfo[iTile].fpGrid ));
+
+                CPLFree( psInfo->pasTileInfo[iTile].panBlockOffset );
+                CPLFree( psInfo->pasTileInfo[iTile].panBlockSize );
+            }
         }
     }
 
@@ -461,13 +476,13 @@ VSILFILE *AIGLLOpen( const char *pszFilename, const char *pszAccess )
         char *pszUCFilename = CPLStrdup(pszFilename);
         int  i;
 
-        for( i = strlen(pszUCFilename)-1; 
+        for( i = (int)strlen(pszUCFilename)-1;
              pszUCFilename[i] != '/' && pszUCFilename[i] != '\\';
              i-- )
         {
             pszUCFilename[i] = (char) toupper(pszUCFilename[i]);
         }
-        
+
         fp = VSIFOpenL( pszUCFilename, pszAccess );
 
         CPLFree( pszUCFilename );
@@ -475,4 +490,3 @@ VSILFILE *AIGLLOpen( const char *pszFilename, const char *pszAccess )
 
     return fp;
 }
-
