@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ceosdataset.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: ceosdataset.cpp 33720 2016-03-15 00:39:53Z goatbar $
  *
  * Project:  CEOS Translator
  * Purpose:  GDALDataset driver for CEOS translator.
@@ -29,13 +29,10 @@
  ****************************************************************************/
 
 #include "ceosopen.h"
+#include "gdal_frmts.h"
 #include "gdal_pam.h"
 
-CPL_CVSID("$Id: ceosdataset.cpp 27729 2014-09-24 00:40:16Z goatbar $");
-
-CPL_C_START
-void	GDALRegister_CEOS(void);
-CPL_C_END
+CPL_CVSID("$Id: ceosdataset.cpp 33720 2016-03-15 00:39:53Z goatbar $");
 
 /************************************************************************/
 /* ==================================================================== */
@@ -48,7 +45,7 @@ class CEOSRasterBand;
 class CEOSDataset : public GDALPamDataset
 {
     friend class CEOSRasterBand;
-    
+
     CEOSImage	*psCEOS;
 
   public:
@@ -66,11 +63,11 @@ class CEOSDataset : public GDALPamDataset
 class CEOSRasterBand : public GDALPamRasterBand
 {
     friend class CEOSDataset;
-    
+
   public:
 
     		CEOSRasterBand( CEOSDataset *, int );
-    
+
     virtual CPLErr IReadBlock( int, int, void * );
 };
 
@@ -79,12 +76,12 @@ class CEOSRasterBand : public GDALPamRasterBand
 /*                           CEOSRasterBand()                            */
 /************************************************************************/
 
-CEOSRasterBand::CEOSRasterBand( CEOSDataset *poDS, int nBand )
+CEOSRasterBand::CEOSRasterBand( CEOSDataset *poDSIn, int nBandIn )
 
 {
-    this->poDS = poDS;
-    this->nBand = nBand;
-    
+    this->poDS = poDSIn;
+    this->nBand = nBandIn;
+
     eDataType = GDT_Byte;
 
     nBlockXSize = poDS->GetRasterXSize();
@@ -95,9 +92,9 @@ CEOSRasterBand::CEOSRasterBand( CEOSDataset *poDS, int nBand )
 /*                             IReadBlock()                             */
 /************************************************************************/
 
-CPLErr CEOSRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYOff,
-                                  void * pImage )
-
+CPLErr CEOSRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
+                                   int nBlockYOff,
+                                   void * pImage )
 {
     CEOSDataset	*poCEOS_DS = (CEOSDataset *) poDS;
 
@@ -141,9 +138,6 @@ CEOSDataset::~CEOSDataset()
 GDALDataset *CEOSDataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
-    CEOSImage	*psCEOS;
-    int		i;
-    
 /* -------------------------------------------------------------------- */
 /*      Before trying CEOSOpen() we first verify that the first         */
 /*      record is in fact a CEOS file descriptor record.                */
@@ -160,14 +154,13 @@ GDALDataset *CEOSDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Try opening the dataset.                                        */
 /* -------------------------------------------------------------------- */
-    psCEOS = CEOSOpen( poOpenInfo->pszFilename, "rb" );
-    
+    CEOSImage *psCEOS = CEOSOpen( poOpenInfo->pszFilename, "rb" );
     if( psCEOS == NULL )
         return( NULL );
 
     if( psCEOS->nBitsPerPixel != 8 )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The CEOS driver cannot handle nBitsPerPixel = %d",
                   psCEOS->nBitsPerPixel );
         CEOSClose(psCEOS);
@@ -187,7 +180,7 @@ GDALDataset *CEOSDataset::Open( GDALOpenInfo * poOpenInfo )
     if( poOpenInfo->eAccess == GA_Update )
     {
         CEOSClose(psCEOS);
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The CEOS driver does not support update access to existing"
                   " datasets.\n" );
         return NULL;
@@ -195,24 +188,22 @@ GDALDataset *CEOSDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
-    CEOSDataset 	*poDS;
-
-    poDS = new CEOSDataset();
+    CEOSDataset *poDS = new CEOSDataset();
 
     poDS->psCEOS = psCEOS;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Capture some information from the file that is of interest.     */
 /* -------------------------------------------------------------------- */
     poDS->nRasterXSize = psCEOS->nPixels;
     poDS->nRasterYSize = psCEOS->nLines;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Create band information objects.                                */
 /* -------------------------------------------------------------------- */
     poDS->nBands = psCEOS->nBands;;
 
-    for( i = 0; i < poDS->nBands; i++ )
+    for( int i = 0; i < poDS->nBands; i++ )
         poDS->SetBand( i+1, new CEOSRasterBand( poDS, i+1 ) );
 
 /* -------------------------------------------------------------------- */
@@ -236,21 +227,20 @@ GDALDataset *CEOSDataset::Open( GDALOpenInfo * poOpenInfo )
 void GDALRegister_CEOS()
 
 {
-    GDALDriver	*poDriver;
+    if( GDALGetDriverByName( "CEOS" ) != NULL )
+        return;
 
-    if( GDALGetDriverByName( "CEOS" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "CEOS" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "CEOS Image" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_various.html#CEOS" );
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
-        
-        poDriver->pfnOpen = CEOSDataset::Open;
+    GDALDriver *poDriver = new GDALDriver();
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    poDriver->SetDescription( "CEOS" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
+                               "CEOS Image" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,
+                               "frmt_various.html#CEOS" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+
+    poDriver->pfnOpen = CEOSDataset::Open;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
