@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ceostest.c 10645 2007-01-18 02:22:39Z warmerdam $
+ * $Id: ceostest.c 33720 2016-03-15 00:39:53Z goatbar $
  *
  * Project:  CEOS Translator
  * Purpose:  Test mainline.
@@ -46,24 +46,24 @@ int main( int nArgc, char ** papszArgv )
     else
         pszFilename = "imag_01.dat";
 
-    fp = VSIFOpen( pszFilename, "rb" );
+    fp = VSIFOpenL( pszFilename, "rb" );
     if( fp == NULL )
     {
         fprintf( stderr, "Can't open %s at all.\n", pszFilename );
         exit( 1 );
     }
 
-    while( !VSIFEof(fp) 
+    while( !VSIFEofL(fp)
            && (psRecord = CEOSReadRecord( fp )) != NULL )
     {
-        printf( "%9d:%4d:%8x:%d\n", 
-                nPosition, psRecord->nRecordNum, 
+        printf( "%9d:%4d:%8x:%d\n",
+                nPosition, psRecord->nRecordNum,
                 psRecord->nRecordType, psRecord->nLength );
         CEOSDestroyRecord( psRecord );
 
-        nPosition = (int) VSIFTell( fp );
+        nPosition = (int) VSIFTellL( fp );
     }
-    VSIFClose( fp );
+    VSIFCloseL( fp );
 
     exit( 0 );
 }
