@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ctgdataset.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: ctgdataset.cpp 32205 2015-12-17 21:57:20Z goatbar $
  *
  * Project:  CTG driver
  * Purpose:  GDALDataset driver for CTG dataset.
@@ -27,16 +27,13 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
+#include "gdal_frmts.h"
 #include "gdal_pam.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: ctgdataset.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: ctgdataset.cpp 32205 2015-12-17 21:57:20Z goatbar $");
 
-CPL_C_START
-void    GDALRegister_CTG(void);
-CPL_C_END
-
-#define HEADER_LINE_COUNT 5
+static const int HEADER_LINE_COUNT = 5;
 
 typedef struct
 {
@@ -94,7 +91,7 @@ static const LULCDescStruct asLULCDesc[] =
     {92, "Glaciers" }
 };
 
-static const char* apszBandDescription[] =
+static const char* const apszBandDescription[] =
 {
     "Land Use and Land Cover",
     "Political units",
@@ -167,18 +164,16 @@ class CTGRasterBand : public GDALPamRasterBand
 /*                           CTGRasterBand()                            */
 /************************************************************************/
 
-CTGRasterBand::CTGRasterBand( CTGDataset *poDS, int nBand )
-
+CTGRasterBand::CTGRasterBand( CTGDataset *poDSIn, int nBandIn ) :
+    papszCategories(NULL)
 {
-    this->poDS = poDS;
-    this->nBand = nBand;
+    this->poDS = poDSIn;
+    this->nBand = nBandIn;
 
     eDataType = GDT_Int32;
 
     nBlockXSize = poDS->GetRasterXSize();
     nBlockYSize = poDS->GetRasterYSize();
-
-    papszCategories = NULL;
 }
 
 /************************************************************************/
@@ -194,7 +189,8 @@ CTGRasterBand::~CTGRasterBand()
 /*                             IReadBlock()                             */
 /************************************************************************/
 
-CPLErr CTGRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, CPL_UNUSED int nBlockYOff,
+CPLErr CTGRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
+                                  CPL_UNUSED int nBlockYOff,
                                   void * pImage )
 {
     CTGDataset* poGDS = (CTGDataset* ) poDS;
@@ -216,7 +212,7 @@ double CTGRasterBand::GetNoDataValue( int *pbSuccess )
     if (pbSuccess)
         *pbSuccess = TRUE;
 
-    return 0.;
+    return 0.0;
 }
 
 /************************************************************************/
@@ -231,15 +227,14 @@ char **CTGRasterBand::GetCategoryNames()
     if (papszCategories != NULL)
         return papszCategories;
 
-    int i;
     int nasLULCDescSize = (int)(sizeof(asLULCDesc) / sizeof(asLULCDesc[0]));
     int nCategoriesSize = asLULCDesc[nasLULCDescSize - 1].nCode;
     papszCategories = (char**)CPLCalloc(nCategoriesSize + 2, sizeof(char*));
-    for(i=0;i<nasLULCDescSize;i++)
+    for(int i=0;i<nasLULCDescSize;i++)
     {
         papszCategories[asLULCDesc[i].nCode] = CPLStrdup(asLULCDesc[i].pszDesc);
     }
-    for(i=0;i<nCategoriesSize;i++)
+    for(int i=0;i<nCategoriesSize;i++)
     {
         if (papszCategories[i] == NULL)
             papszCategories[i] = CPLStrdup("");
@@ -253,13 +248,16 @@ char **CTGRasterBand::GetCategoryNames()
 /*                            ~CTGDataset()                            */
 /************************************************************************/
 
-CTGDataset::CTGDataset()
-{
-    pszProjection = NULL;
-    bHasReadImagery = FALSE;
-    pabyImage = NULL;
-    fp = NULL;
-}
+CTGDataset::CTGDataset() :
+    fp(NULL),
+    nNWEasting(0),
+    nNWNorthing(0),
+    nCellSize(0),
+    nUTMZone(0),
+    pszProjection(NULL),
+    bHasReadImagery(FALSE),
+    pabyImage(NULL)
+{ }
 
 /************************************************************************/
 /*                            ~CTGDataset()                            */
@@ -335,8 +333,7 @@ int CTGDataset::ReadImagery()
                      nLine, szLine);
             return FALSE;
         }
-        int i;
-        for(i=0;i<6;i++)
+        for(int i=0;i<6;i++)
         {
             int nVal = atoi(ExtractField(szField, szLine, 20 + 10*i, 10));
             if (nVal >= 2000000000)
@@ -360,18 +357,18 @@ int CTGDataset::Identify( GDALOpenInfo * poOpenInfo )
 
     GDALOpenInfo* poOpenInfoToDelete = NULL;
     /*  GZipped grid_cell.gz files are common, so automagically open them */
-    /*  if the /vsigzip/ has not been explicitely passed */
+    /*  if the /vsigzip/ has not been explicitly passed */
     const char* pszFilename = CPLGetFilename(poOpenInfo->pszFilename);
     if ((EQUAL(pszFilename, "grid_cell.gz") ||
          EQUAL(pszFilename, "grid_cell1.gz") ||
          EQUAL(pszFilename, "grid_cell2.gz")) &&
-        !EQUALN(poOpenInfo->pszFilename, "/vsigzip/", 9))
+        !STARTS_WITH_CI(poOpenInfo->pszFilename, "/vsigzip/"))
     {
         osFilename = "/vsigzip/";
         osFilename += poOpenInfo->pszFilename;
         poOpenInfo = poOpenInfoToDelete =
                 new GDALOpenInfo(osFilename.c_str(), GA_ReadOnly,
-                                 poOpenInfo->papszSiblingFiles);
+                                 poOpenInfo->GetSiblingFiles());
     }
 
     if (poOpenInfo->nHeaderBytes < HEADER_LINE_COUNT * 80)
@@ -384,8 +381,7 @@ int CTGDataset::Identify( GDALOpenInfo * poOpenInfo )
 /*      Chech that it looks roughly as a CTG dataset                    */
 /* -------------------------------------------------------------------- */
     const char* pszData = (const char*)poOpenInfo->pabyHeader;
-    int i;
-    for(i=0;i<4 * 80;i++)
+    for(int i=0;i<4 * 80;i++)
     {
         if (!((pszData[i] >= '0' && pszData[i] <= '9') ||
               pszData[i] == ' ' || pszData[i] == '-'))
@@ -423,20 +419,18 @@ int CTGDataset::Identify( GDALOpenInfo * poOpenInfo )
 GDALDataset *CTGDataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
-    int         i;
-
     if (!Identify(poOpenInfo))
         return NULL;
 
     CPLString osFilename(poOpenInfo->pszFilename);
 
     /*  GZipped grid_cell.gz files are common, so automagically open them */
-    /*  if the /vsigzip/ has not been explicitely passed */
+    /*  if the /vsigzip/ has not been explicitly passed */
     const char* pszFilename = CPLGetFilename(poOpenInfo->pszFilename);
     if ((EQUAL(pszFilename, "grid_cell.gz") ||
          EQUAL(pszFilename, "grid_cell1.gz") ||
          EQUAL(pszFilename, "grid_cell2.gz")) &&
-        !EQUALN(poOpenInfo->pszFilename, "/vsigzip/", 9))
+        !STARTS_WITH_CI(poOpenInfo->pszFilename, "/vsigzip/"))
     {
         osFilename = "/vsigzip/";
         osFilename += poOpenInfo->pszFilename;
@@ -465,7 +459,7 @@ GDALDataset *CTGDataset::Open( GDALOpenInfo * poOpenInfo )
         return NULL;
     }
 
-    for(i=HEADER_LINE_COUNT * 80 - 1;i>=0;i--)
+    for(int i=HEADER_LINE_COUNT * 80 - 1;i>=0;i--)
     {
         if (szHeader[i] == ' ')
             szHeader[i] = 0;
@@ -480,9 +474,7 @@ GDALDataset *CTGDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
-    CTGDataset         *poDS;
-
-    poDS = new CTGDataset();
+    CTGDataset *poDS = new CTGDataset();
     poDS->fp = fp;
     fp = NULL;
     poDS->nRasterXSize = nCols;
@@ -530,7 +522,7 @@ GDALDataset *CTGDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Create band information objects.                                */
 /* -------------------------------------------------------------------- */
     poDS->nBands = 6;
-    for( i = 0; i < poDS->nBands; i++ )
+    for( int i = 0; i < poDS->nBands; i++ )
     {
         poDS->SetBand( i+1, new CTGRasterBand( poDS, i+1 ) );
         poDS->GetRasterBand(i+1)->SetDescription(apszBandDescription[i]);
@@ -547,7 +539,7 @@ GDALDataset *CTGDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
 
-    return( poDS );
+    return poDS;
 }
 
 /************************************************************************/
@@ -564,7 +556,7 @@ CPLErr CTGDataset::GetGeoTransform( double * padfTransform )
     padfTransform[4] = 0.;
     padfTransform[5] = -nCellSize;
 
-    return( CE_None );
+    return CE_None;
 }
 
 /************************************************************************/
@@ -584,24 +576,22 @@ const char* CTGDataset::GetProjectionRef()
 void GDALRegister_CTG()
 
 {
-    GDALDriver  *poDriver;
+    if( GDALGetDriverByName( "CTG" ) != NULL )
+      return;
 
-    if( GDALGetDriverByName( "CTG" ) == NULL )
-    {
-        poDriver = new GDALDriver();
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->SetDescription( "CTG" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
-                                   "USGS LULC Composite Theme Grid" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,
-                                   "frmt_various.html#CTG" );
+    poDriver->SetDescription( "CTG" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
+                               "USGS LULC Composite Theme Grid" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,
+                               "frmt_various.html#CTG" );
 
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
-        poDriver->pfnOpen = CTGDataset::Open;
-        poDriver->pfnIdentify = CTGDataset::Identify;
+    poDriver->pfnOpen = CTGDataset::Open;
+    poDriver->pfnIdentify = CTGDataset::Identify;
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
-
