@@ -29,19 +29,18 @@
  ******************************************************************************
  *
  * THE CURRENT IMPLEMENTATION IS WRITE ONLY.
- * 
+ *
  */
 
-#include "gdal_pam.h"
+/* stdio.h needed before including crnlib.h, since the later needs NULL to be defined */
+#include <stdio.h>
 #include "crnlib.h"
 #include "dds_defs.h"
+#include "gdal_frmts.h"
+#include "gdal_pam.h"
 
 CPL_CVSID("$Id: $");
 
-CPL_C_START
-void	GDALRegister_DDS(void);
-CPL_C_END
-
 using namespace crnlib;
 
 enum { DDS_COLOR_TYPE_RGB,
@@ -70,34 +69,32 @@ public:
 /************************************************************************/
 
 GDALDataset *
-DDSDataset::CreateCopy(const char * pszFilename, GDALDataset *poSrcDS, 
-                       int bStrict, char ** papszOptions, 
+DDSDataset::CreateCopy(const char * pszFilename, GDALDataset *poSrcDS,
+                       int bStrict, char ** papszOptions,
                        GDALProgressFunc pfnProgress, void * pProgressData)
 
-{  
+{
     int  nBands = poSrcDS->GetRasterCount();
-    int  nXSize = poSrcDS->GetRasterXSize();
-    int  nYSize = poSrcDS->GetRasterYSize();
-    
+
     /* -------------------------------------------------------------------- */
     /*      Some rudimentary checks                                         */
     /* -------------------------------------------------------------------- */
     if (nBands != 3 && nBands != 4)
     {
-        CPLError(CE_Failure, CPLE_NotSupported, 
+        CPLError(CE_Failure, CPLE_NotSupported,
                  "DDS driver doesn't support %d bands. Must be 3 (rgb) \n"
-                 "or 4 (rgba) bands.\n", 
+                 "or 4 (rgba) bands.\n",
                  nBands);
-        
+
         return NULL;
     }
 
     if (poSrcDS->GetRasterBand(1)->GetRasterDataType() != GDT_Byte)
     {
-        CPLError( (bStrict) ? CE_Failure : CE_Warning, CPLE_NotSupported, 
+        CPLError( (bStrict) ? CE_Failure : CE_Warning, CPLE_NotSupported,
                   "DDS driver doesn't support data type %s. "
-                  "Only eight bit (Byte) bands supported. %s\n", 
-                  GDALGetDataTypeName( 
+                  "Only eight bit (Byte) bands supported. %s\n",
+                  GDALGetDataTypeName(
                                       poSrcDS->GetRasterBand(1)->GetRasterDataType()),
                   (bStrict) ? "" : "Defaulting to Byte" );
 
@@ -118,13 +115,11 @@ DDSDataset::CreateCopy(const char * pszFilename, GDALDataset *poSrcDS,
     /* -------------------------------------------------------------------- */
     /*      Create the dataset.                                             */
     /* -------------------------------------------------------------------- */
-    VSILFILE	*fpImage;
-
-    fpImage = VSIFOpenL(pszFilename, "wb");
+    VSILFILE *fpImage = VSIFOpenL(pszFilename, "wb");
     if (fpImage == NULL)
     {
-        CPLError(CE_Failure, CPLE_OpenFailed, 
-                 "Unable to create dds file %s.\n", 
+        CPLError(CE_Failure, CPLE_OpenFailed,
+                 "Unable to create dds file %s.\n",
                  pszFilename);
         return NULL;
     }
@@ -132,14 +127,15 @@ DDSDataset::CreateCopy(const char * pszFilename, GDALDataset *poSrcDS,
     /* -------------------------------------------------------------------- */
     /*      Create the Crunch compressor                                    */
     /* -------------------------------------------------------------------- */
-    
+
     /* Default values */
-    crn_format fmt = cCRNFmtDXT3; 
-    const uint cDXTBlockSize = 4;    
+    crn_format fmt = cCRNFmtDXT3;
+    const uint cDXTBlockSize = 4;
     crn_dxt_quality dxt_quality = cCRNDXTQualityNormal;
-    bool srgb_colorspace = true;    
+    bool srgb_colorspace = true;
     bool dxt1a_transparency = true;
-    
+    //bool generate_mipmaps = true;
+
     /* Check the texture format */
     const char *pszFormat = CSLFetchNameValue( papszOptions, "FORMAT" );
 
@@ -153,10 +149,12 @@ DDSDataset::CreateCopy(const char * pszFilename, GDALDataset *poSrcDS,
             fmt = cCRNFmtDXT3;
         else if (EQUAL(pszFormat, "dxt5"))
             fmt = cCRNFmtDXT5;
+	else if (EQUAL(pszFormat, "etc1"))
+            fmt = cCRNFmtETC1;
         else
         {
             CPLError( CE_Failure, CPLE_AppDefined,
-                      "Illegal FORMAT value '%s', should be DXT1, DXT1A, DXT3 or DXT5.",
+                      "Illegal FORMAT value '%s', should be DXT1, DXT1A, DXT3, DXT5 or ETC1",
                       pszFormat );
             return NULL;
         }
@@ -168,7 +166,7 @@ DDSDataset::CreateCopy(const char * pszFilename, GDALDataset *poSrcDS,
     if (pszQuality)
     {
         if (EQUAL(pszQuality, "SUPERFAST"))
-            dxt_quality = cCRNDXTQualitySuperFast;            
+            dxt_quality = cCRNDXTQualitySuperFast;
         else if (EQUAL(pszQuality, "FAST"))
             dxt_quality = cCRNDXTQualityFast;
         else if (EQUAL(pszQuality, "NORMAL"))
@@ -176,7 +174,7 @@ DDSDataset::CreateCopy(const char * pszFilename, GDALDataset *poSrcDS,
         else if (EQUAL(pszQuality, "BETTER"))
             dxt_quality = cCRNDXTQualityBetter;
         else if (EQUAL(pszQuality, "UBER"))
-            dxt_quality = cCRNDXTQualityUber;        
+            dxt_quality = cCRNDXTQualityUber;
         else
         {
             CPLError( CE_Failure, CPLE_AppDefined,
@@ -186,35 +184,39 @@ DDSDataset::CreateCopy(const char * pszFilename, GDALDataset *poSrcDS,
         }
     }
 
+    int nXSize = poSrcDS->GetRasterXSize();
+    int nYSize = poSrcDS->GetRasterYSize();
+
     if ((nXSize%4!=0) || (nYSize%4!=0)) {
-      CPLError(CE_Warning, CPLE_AppDefined,
-               "Raster size is not a multiple of 4: %dx%d. "
-               "Extra rows/colums will be ignored during the compression.",
-               nXSize, nYSize);
+      CPLError( CE_Warning, CPLE_AppDefined,
+                "Raster size is not a multiple of 4: %dx%d. "
+                "Extra rows/columns will be ignored during the compression.",
+                nXSize, nYSize );
     }
-    
+
     crn_comp_params comp_params;
     comp_params.m_format = fmt;
     comp_params.m_dxt_quality = dxt_quality;
     comp_params.set_flag(cCRNCompFlagPerceptual, srgb_colorspace);
     comp_params.set_flag(cCRNCompFlagDXT1AForTransparency, dxt1a_transparency);
-    
+
     crn_block_compressor_context_t pContext = crn_create_block_compressor(comp_params);
-    
+
     /* -------------------------------------------------------------------- */
     /*      Write the DDS header to the file.                               */
     /* -------------------------------------------------------------------- */
 
     VSIFWriteL(&crnlib::cDDSFileSignature, 1,
                sizeof(crnlib::cDDSFileSignature), fpImage);
-    
+
     crnlib::DDSURFACEDESC2 ddsDesc;
     memset(&ddsDesc, 0, sizeof(ddsDesc));
     ddsDesc.dwSize = sizeof(ddsDesc);
-    ddsDesc.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH | DDSD_PIXELFORMAT;
+    ddsDesc.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH | DDSD_MIPMAPCOUNT | DDSD_PIXELFORMAT | DDSD_DEPTH ;
     ddsDesc.dwWidth = nXSize;
     ddsDesc.dwHeight = nYSize;
-    
+    ddsDesc.dwMipMapCount = 1;
+
     ddsDesc.ddpfPixelFormat.dwSize = sizeof(crnlib::DDPIXELFORMAT);
     ddsDesc.ddpfPixelFormat.dwFlags = DDPF_FOURCC;
     ddsDesc.ddpfPixelFormat.dwFourCC = crn_get_format_fourcc(fmt);
@@ -222,19 +224,19 @@ DDSDataset::CreateCopy(const char * pszFilename, GDALDataset *poSrcDS,
 
     // Set pitch/linearsize field (some DDS readers require this field to be non-zero).
     uint bits_per_pixel = crn_get_format_bits_per_texel(fmt);
-    ddsDesc.lPitch = (((ddsDesc.dwWidth + 3) & ~3) * ((ddsDesc.dwHeight + 3) & ~3) * bits_per_pixel) >> 3;
+    ddsDesc.lPitch = (((ddsDesc.dwWidth + 3) & ~3U) * ((ddsDesc.dwHeight + 3) & ~3U) * bits_per_pixel) >> 3;
     ddsDesc.dwFlags |= DDSD_LINEARSIZE;
 
     /* Endianness problems when serializing structure?? dds on-disk format
        should be verified */
     VSIFWriteL(&ddsDesc, 1, sizeof(ddsDesc), fpImage);
-    
+
     /* -------------------------------------------------------------------- */
     /*      Loop over image, compressing image data.                        */
     /* -------------------------------------------------------------------- */
     const uint bytesPerBlock = crn_get_bytes_per_dxt_block(fmt);
     CPLErr eErr = CE_None;
-    const uint nYNumBlocks = (nYSize + cDXTBlockSize - 1) / cDXTBlockSize;  
+    const uint nYNumBlocks = (nYSize + cDXTBlockSize - 1) / cDXTBlockSize;
     const uint num_blocks_x = (nXSize + cDXTBlockSize - 1) / cDXTBlockSize;
     const uint total_compressed_size = num_blocks_x * bytesPerBlock;
 
@@ -249,16 +251,16 @@ DDSDataset::CreateCopy(const char * pszFilename, GDALDataset *poSrcDS,
     {
         const uint size_y = (iLine*cDXTBlockSize+cDXTBlockSize) < (uint)nYSize ?
                            cDXTBlockSize : (cDXTBlockSize-((iLine*cDXTBlockSize+cDXTBlockSize)-(uint)nYSize));
-        
-        eErr = poSrcDS->RasterIO(GF_Read, 0, iLine*cDXTBlockSize, nXSize, size_y, 
+
+        eErr = poSrcDS->RasterIO(GF_Read, 0, iLine*cDXTBlockSize, nXSize, size_y,
                                  pabyScanlines, nXSize, size_y, GDT_Byte,
                                  nBands, NULL,
-                                 nBands, 
-                                 nBands * nXSize, 1);
+                                 nBands,
+                                 nBands * nXSize, 1, NULL);
 
         if (eErr != CE_None)
             break;
-        
+
         crn_uint32 *pSrc_image = NULL;
         if (nColorType == DDS_COLOR_TYPE_RGB_ALPHA)
             pSrc_image = (crn_uint32*)pabyScanlines;
@@ -269,9 +271,9 @@ DDSDataset::CreateCopy(const char * pszFilename, GDALDataset *poSrcDS,
             {
                 int y = (i*3);
                 src_image[i] = (255<<24) | (pabyScanlines[y+2]<<16) | (pabyScanlines[y+1]<<8) |
-                  pabyScanlines[y];            
+                  pabyScanlines[y];
             }
-            
+
             pSrc_image = &(src_image[0]);
         }
 
@@ -289,7 +291,7 @@ DDSDataset::CreateCopy(const char * pszFilename, GDALDataset *poSrcDS,
                     *pDst_pixels++ = pSrc_image[actual_x + actual_y * nXSize];
                 }
             }
-            
+
             // Compress the DXTn block.
             crn_compress_block(pContext, pixels, static_cast<crn_uint8 *>(pCompressed_data) + block_x * bytesPerBlock);
         }
@@ -314,14 +316,14 @@ DDSDataset::CreateCopy(const char * pszFilename, GDALDataset *poSrcDS,
     CPLFree(pabyScanlines);
     crn_free_block_compressor(pContext);
     pContext = NULL;
-   
+
     VSIFCloseL(fpImage);
 
     if (eErr != CE_None)
         return NULL;
 
     DDSDataset *poDsDummy = new DDSDataset();
-    
+
     return poDsDummy;
 }
 
@@ -331,40 +333,40 @@ DDSDataset::CreateCopy(const char * pszFilename, GDALDataset *poSrcDS,
 
 void GDALRegister_DDS()
 {
-    GDALDriver	*poDriver;
+    if( GDALGetDriverByName( "DDS" ) != NULL )
+        return;
 
-    if (GDALGetDriverByName( "DDS" ) == NULL)
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription("DDS");
-        poDriver->SetMetadataItem(GDAL_DMD_LONGNAME, 
-                                  "DirectDraw Surface");
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_various.html#DDS" );        
-        poDriver->SetMetadataItem(GDAL_DMD_EXTENSION, "dds");
-        poDriver->SetMetadataItem(GDAL_DMD_MIMETYPE, "image/dds");
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->SetMetadataItem(GDAL_DMD_CREATIONOPTIONLIST, 
-                                  "<CreationOptionList>\n"
-                                  "   <Option name='FORMAT' type='string-select' description='Texture format' default='DXT3'>\n"
-                                  "     <Value>DXT1</Value>\n"
-                                  "     <Value>DXT1A</Value>\n"
-                                  "     <Value>DXT3</Value>\n"
-                                  "     <Value>DXT5</Value>\n"                                                                    
-                                  "   </Option>\n"
-                                  "   <Option name='QUALITY' type='string-select' description='Compression Quality' default='NORMAL'>\n"
-                                  "     <Value>SUPERFAST</Value>\n"
-                                  "     <Value>FAST</Value>\n"
-                                  "     <Value>NORMAL</Value>\n"
-                                  "     <Value>BETTER</Value>\n"
-                                  "     <Value>UBER</Value>\n"                                                                                                      
-                                  "   </Option>\n"                                  
-                                  "</CreationOptionList>\n" );
+    poDriver->SetDescription("DDS");
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem(GDAL_DMD_LONGNAME,
+                              "DirectDraw Surface");
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_various.html#DDS" );
+    poDriver->SetMetadataItem(GDAL_DMD_EXTENSION, "dds");
+    poDriver->SetMetadataItem(GDAL_DMD_MIMETYPE, "image/dds");
 
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
-        
-        poDriver->pfnCreateCopy = DDSDataset::CreateCopy;
-        
-        GetGDALDriverManager()->RegisterDriver(poDriver);
-    }
+    poDriver->SetMetadataItem(
+        GDAL_DMD_CREATIONOPTIONLIST,
+        "<CreationOptionList>\n"
+        "   <Option name='FORMAT' type='string-select' description='Texture format' default='DXT3'>\n"
+        "     <Value>DXT1</Value>\n"
+        "     <Value>DXT1A</Value>\n"
+        "     <Value>DXT3</Value>\n"
+        "     <Value>DXT5</Value>\n"
+        "     <Value>ETC1</Value>\n"
+        "   </Option>\n"
+        "   <Option name='QUALITY' type='string-select' description='Compression Quality' default='NORMAL'>\n"
+        "     <Value>SUPERFAST</Value>\n"
+        "     <Value>FAST</Value>\n"
+        "     <Value>NORMAL</Value>\n"
+        "     <Value>BETTER</Value>\n"
+        "     <Value>UBER</Value>\n"
+        "   </Option>\n"
+        "</CreationOptionList>\n" );
+
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    poDriver->pfnCreateCopy = DDSDataset::CreateCopy;
+
+    GetGDALDriverManager()->RegisterDriver(poDriver);
 }
