@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: dted_api.h 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: dted_api.h 33720 2016-03-15 00:39:53Z goatbar $
  *
  * Project:  DTED Translator
  * Purpose:  Public (C callable) interface for DTED/CDED reading.
@@ -28,8 +28,8 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _DTED_API_H_INCLUDED
-#define _DTED_API_H_INCLUDED
+#ifndef DTED_API_H_INCLUDED
+#define DTED_API_H_INCLUDED
 
 /* -------------------------------------------------------------------- */
 /*      To avoid dependence on CPL, just define AVOID_CPL when          */
@@ -37,6 +37,7 @@
 /* -------------------------------------------------------------------- */
 #ifndef AVOID_CPL
 #  include "cpl_conv.h"
+#  include "cpl_string.h"
 #else
 
 #include <stdio.h>
@@ -133,10 +134,12 @@ typedef struct {
 /* -------------------------------------------------------------------- */
 DTEDInfo *DTEDOpen( const char * pszFilename, const char * pszAccess,
                     int bTestOpen );
+DTEDInfo *DTEDOpenEx( VSILFILE* fp, const char * pszFilename,
+                      const char * pszAccess, int bTestOpen );
 
 /**     Read one single sample. The coordinates are given from the
         top-left corner of the file (contrary to the internal
-        organisation or a DTED file)
+        organization or a DTED file)
 */
 int DTEDReadPoint( DTEDInfo * psDInfo, int nXOff, int nYOff, GInt16* panVal);
 
@@ -146,8 +149,8 @@ int DTEDReadPoint( DTEDInfo * psDInfo, int nXOff, int nYOff, GInt16* panVal);
 int DTEDReadProfile( DTEDInfo * psDInfo, int nColumnOffset,
                      GInt16 * panData );
 
-/* Extented version of DTEDReadProfile that enable the user to specify */
-/* whether he wants the checksums to be verified */
+/* Extended version of DTEDReadProfile that enables the user to specify */
+/* whether the checksums should be verified */
 int DTEDReadProfileEx( DTEDInfo * psDInfo, int nColumnOffset,
                        GInt16 * panData, int bVerifyChecksum );
 
@@ -159,7 +162,7 @@ int DTEDWriteProfile( DTEDInfo *psDInfo, int nColumnOffset, GInt16 *panData);
 
 void DTEDClose( DTEDInfo * );
 
-const char *DTEDCreate( const char *pszFilename, 
+const char *DTEDCreate( const char *pszFilename,
                         int nLevel, int nLLOriginLat, int nLLOriginLong );
 
 /* -------------------------------------------------------------------- */
@@ -167,11 +170,11 @@ const char *DTEDCreate( const char *pszFilename,
 /* -------------------------------------------------------------------- */
 typedef enum {
     DTEDMD_VERTACCURACY_UHL = 1,            /* UHL 29+4, ACC 8+4 */
-    DTEDMD_VERTACCURACY_ACC = 2,           
+    DTEDMD_VERTACCURACY_ACC = 2,
     DTEDMD_SECURITYCODE_UHL = 3,            /* UHL 33+3, DSI 4+1 */
-    DTEDMD_SECURITYCODE_DSI = 4,           
+    DTEDMD_SECURITYCODE_DSI = 4,
     DTEDMD_UNIQUEREF_UHL = 5,               /* UHL 36+12, DSI 65+15*/
-    DTEDMD_UNIQUEREF_DSI = 6,              
+    DTEDMD_UNIQUEREF_DSI = 6,
     DTEDMD_DATA_EDITION = 7,            /* DSI 88+2 */
     DTEDMD_MATCHMERGE_VERSION = 8,      /* DSI 90+1 */
     DTEDMD_MAINT_DATE = 9,              /* DSI 91+4 */
@@ -184,15 +187,14 @@ typedef enum {
     DTEDMD_HORIZACCURACY = 16,          /* ACC 4+4 */
     DTEDMD_REL_HORIZACCURACY = 17,      /* ACC 12+4 */
     DTEDMD_REL_VERTACCURACY = 18,       /* ACC 16+4 */
-    DTEDMD_HORIZDATUM = 19,             /* DSI 145+5 */ 
+    DTEDMD_HORIZDATUM = 19,             /* DSI 145+5 */
     DTEDMD_ORIGINLONG = 20,             /* UHL 5+7 */
     DTEDMD_ORIGINLAT = 21,              /* UHL 13+7 */
     DTEDMD_NIMA_DESIGNATOR = 22,        /* DSI 60 + 5 */
-    DTEDMD_PARTIALCELL_DSI = 23,        /* DSI 289 + 2 */ 
+    DTEDMD_PARTIALCELL_DSI = 23,        /* DSI 289 + 2 */
     DTEDMD_MAX = 23
 } DTEDMetaDataCode;
 
-    
 char *DTEDGetMetadata( DTEDInfo *, DTEDMetaDataCode );
 int   DTEDSetMetadata( DTEDInfo *, DTEDMetaDataCode, const char *);
 
@@ -208,6 +210,4 @@ void  DTEDPtStreamTrimEdgeOnlyTiles( void *hStream );
 
 CPL_C_END
 
-#endif /* ndef _DTED_API_H_INCLUDED */
-
-
+#endif /* ndef DTED_API_H_INCLUDED */
