@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: jp2userbox.cpp 27739 2014-09-25 18:49:52Z goatbar $
+ * $Id: jp2userbox.cpp 33717 2016-03-14 06:29:14Z goatbar $
  *
  * Project:  GDAL ECW Driver
  * Purpose:  JP2UserBox implementation - arbitrary box read/write.
@@ -29,7 +29,7 @@
 
 #include "gdal_ecw.h"
 
-CPL_CVSID("$Id: jp2userbox.cpp 27739 2014-09-25 18:49:52Z goatbar $");
+CPL_CVSID("$Id: jp2userbox.cpp 33717 2016-03-14 06:29:14Z goatbar $");
 
 #if defined(HAVE_COMPRESS)
 
@@ -101,7 +101,7 @@ CNCSError JP2UserBox::Parse( CPL_UNUSED class CNCSJP2File &JP2File,
                              CPL_UNUSED CNCSJPCIOStream &Stream )
 #endif
 {
-    CNCSError Error = NCS_SUCCESS;
+    CNCSError Error(GetCNCSError(NCS_SUCCESS));
 
     return Error;
 }
@@ -113,25 +113,25 @@ CNCSError JP2UserBox::Parse( CPL_UNUSED class CNCSJP2File &JP2File,
 /************************************************************************/
 
 #if ECWSDK_VERSION >= 40
-CNCSError JP2UserBox::UnParse( NCS::SDK::CFileBase &JP2File, 
+CNCSError JP2UserBox::UnParse( NCS::SDK::CFileBase &JP2File,
                                NCS::CIOStream &Stream )
 #else
-CNCSError JP2UserBox::UnParse( class CNCSJP2File &JP2File, 
+CNCSError JP2UserBox::UnParse( class CNCSJP2File &JP2File,
                                CNCSJPCIOStream &Stream )
 #endif
 {
-    CNCSError Error = NCS_SUCCESS;
+    CNCSError Error(GetCNCSError(NCS_SUCCESS));
 
     if( m_nTBox == 0 )
     {
-        Error = NCS_UNKNOWN_ERROR;
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        Error = GetCNCSError(NCS_UNKNOWN_ERROR);
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "No box type set in JP2UserBox::UnParse()" );
         return Error;
     }
 #if ECWSDK_VERSION<50
     Error = CNCSJP2Box::UnParse(JP2File, Stream);
-#else 
+#else
     Error = CNCSSDKBox::UnParse(JP2File, Stream);
 #endif
 //    NCSJP2_CHECKIO_BEGIN(Error, Stream);
