@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: EnvisatFile.h 27098 2014-03-27 00:16:11Z rouault $
+ * $Id: EnvisatFile.h 33720 2016-03-15 00:39:53Z goatbar $
  *
  * Project:  APP ENVISAT Support
  * Purpose:  Low Level Envisat file access (read/write) API.
@@ -27,110 +27,106 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef __ENVISAT_FILE_H__
-#define __ENVISAT_FILE_H__
+#ifndef ENVISAT_FILE_H_
+#define ENVISAT_FILE_H_
 
 typedef struct EnvisatFile_tag EnvisatFile;
 
-typedef enum 
+typedef enum
 {
     MPH = 0,
     SPH = 1
 } EnvisatFile_HeaderFlag;
 
-int EnvisatFile_Open( EnvisatFile **self, const char *filename, 
+int EnvisatFile_Open( EnvisatFile **self, const char *filename,
                       const char *mode );
 void EnvisatFile_Close( EnvisatFile *self );
 const char *EnvisatFile_GetFilename( EnvisatFile *self );
-int EnvisatFile_Create( EnvisatFile **self, const char *filename, 
+int EnvisatFile_Create( EnvisatFile **self, const char *filename,
                         const char *template_file );
 int EnvisatFile_GetCurrentLength( EnvisatFile *self );
 
-const char* EnvisatFile_GetKeyByIndex(  EnvisatFile *self, 
+const char* EnvisatFile_GetKeyByIndex(  EnvisatFile *self,
                                         EnvisatFile_HeaderFlag mph_or_sph,
                                         int key_index );
 
-int EnvisatFile_TestKey(  EnvisatFile *self, 
+int EnvisatFile_TestKey(  EnvisatFile *self,
                           EnvisatFile_HeaderFlag mph_or_sph,
                           const char *key );
 
-const char *EnvisatFile_GetKeyValueAsString( EnvisatFile *self, 
+const char *EnvisatFile_GetKeyValueAsString( EnvisatFile *self,
                                              EnvisatFile_HeaderFlag mph_or_sph,
                                              const char *key,
                                              const char *default_value );
 
-int EnvisatFile_SetKeyValueAsString( EnvisatFile *self, 
+int EnvisatFile_SetKeyValueAsString( EnvisatFile *self,
                                      EnvisatFile_HeaderFlag mph_or_sph,
                                      const char *key,
                                      const char *value );
 
-int EnvisatFile_GetKeyValueAsInt( EnvisatFile *self, 
+int EnvisatFile_GetKeyValueAsInt( EnvisatFile *self,
                                   EnvisatFile_HeaderFlag mph_or_sph,
-                                  const char *key, 
+                                  const char *key,
                                   int default_value );
 
-int EnvisatFile_SetKeyValueAsInt( EnvisatFile *self, 
+int EnvisatFile_SetKeyValueAsInt( EnvisatFile *self,
                                   EnvisatFile_HeaderFlag mph_or_sph,
-                                  const char *key, 
+                                  const char *key,
                                   int value );
 
-double EnvisatFile_GetKeyValueAsDouble( EnvisatFile *self, 
+double EnvisatFile_GetKeyValueAsDouble( EnvisatFile *self,
                                         EnvisatFile_HeaderFlag mph_or_sph,
-                                        const char *key, 
+                                        const char *key,
                                         double default_value );
-int EnvisatFile_SetKeyValueAsDouble( EnvisatFile *self, 
+int EnvisatFile_SetKeyValueAsDouble( EnvisatFile *self,
                                      EnvisatFile_HeaderFlag mph_or_sph,
-                                     const char *key, 
+                                     const char *key,
                                      double value );
 
 int EnvisatFile_GetDatasetIndex( EnvisatFile *self, const char *ds_name );
 
-int EnvisatFile_GetDatasetInfo( EnvisatFile *self, 
+int EnvisatFile_GetDatasetInfo( EnvisatFile *self,
                                 int ds_index,
-                                char **ds_name, 
+                                char **ds_name,
                                 char **ds_type,
                                 char **filename,
                                 int  *ds_offset,
                                 int  *ds_size,
                                 int  *num_dsr,
                                 int  *dsr_size );
-int EnvisatFile_SetDatasetInfo( EnvisatFile *self, 
+int EnvisatFile_SetDatasetInfo( EnvisatFile *self,
                                 int ds_index,
                                 int ds_offset,
                                 int ds_size,
                                 int num_dsr,
                                 int dsr_size );
-                              
-int EnvisatFile_ReadDatasetRecordChunk( EnvisatFile *self, 
+
+int EnvisatFile_ReadDatasetRecordChunk( EnvisatFile *self,
                                     int ds_index,
                                     int record_index,
-                                    void *buffer, 
+                                    void *buffer,
                                     int offset, int size ) ;
-int EnvisatFile_ReadDatasetRecord( EnvisatFile *self, 
+int EnvisatFile_ReadDatasetRecord( EnvisatFile *self,
                                    int ds_index,
-                                   int record_index, 
+                                   int record_index,
                                    void *record_buffer );
-int EnvisatFile_WriteDatasetRecord( EnvisatFile *self, 
+int EnvisatFile_WriteDatasetRecord( EnvisatFile *self,
                                     int ds_index,
-                                    int record_index, 
+                                    int record_index,
                                     void *record_buffer );
-int EnvisatFile_ReadDatasetChunk( EnvisatFile *self, 
+int EnvisatFile_ReadDatasetChunk( EnvisatFile *self,
                                   int ds_index,
-                                  int offset, 
+                                  int offset,
                                   int size,
                                   void *buffer );
-                                
 
 #ifndef FAILURE
 #  define FAILURE 1
 #endif
-#ifndef SUCCESS 
+#ifndef SUCCESS
 #  define SUCCESS 0
 #endif
 
-#endif /* __ENVISAT_FILE_H__ */
+#endif /* ENVISAT_FILE_H_ */
 
 /* EOF */
-
-
-
