@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ershdrnode.cpp 21814 2011-02-23 22:55:14Z warmerdam $
+ * $Id: ershdrnode.cpp 33717 2016-03-14 06:29:14Z goatbar $
  *
  * Project:  ERMapper .ers Driver
  * Purpose:  Implementation of ERSHdrNode class for parsing/accessing .ers hdr.
@@ -31,22 +31,20 @@
 #include "cpl_string.h"
 #include "ershdrnode.h"
 
-CPL_CVSID("$Id: ershdrnode.cpp 21814 2011-02-23 22:55:14Z warmerdam $");
+CPL_CVSID("$Id: ershdrnode.cpp 33717 2016-03-14 06:29:14Z goatbar $");
 
 
 /************************************************************************/
 /*                             ERSHdrNode()                             */
 /************************************************************************/
 
-ERSHdrNode::ERSHdrNode()
-
-{
-    nItemMax = 0;
-    nItemCount = 0;
-    papszItemName = NULL;
-    papszItemValue = NULL;
-    papoItemChild = NULL;
-}
+ERSHdrNode::ERSHdrNode() :
+    nItemMax(0),
+    nItemCount(0),
+    papszItemName(NULL),
+    papszItemValue(NULL),
+    papoItemChild(NULL)
+{ }
 
 /************************************************************************/
 /*                            ~ERSHdrNode()                             */
@@ -55,9 +53,7 @@ ERSHdrNode::ERSHdrNode()
 ERSHdrNode::~ERSHdrNode()
 
 {
-    int i;
-
-    for( i = 0; i < nItemCount; i++ )
+    for( int i = 0; i < nItemCount; i++ )
     {
         if( papoItemChild[i] != NULL )
             delete papoItemChild[i];
@@ -84,11 +80,11 @@ void ERSHdrNode::MakeSpace()
     if( nItemCount == nItemMax )
     {
         nItemMax = (int) (nItemMax * 1.3) + 10;
-        papszItemName = (char **) 
+        papszItemName = (char **)
             CPLRealloc(papszItemName,sizeof(char *) * nItemMax);
-        papszItemValue = (char **) 
+        papszItemValue = (char **)
             CPLRealloc(papszItemValue,sizeof(char *) * nItemMax);
-        papoItemChild = (ERSHdrNode **) 
+        papoItemChild = (ERSHdrNode **)
             CPLRealloc(papoItemChild,sizeof(void *) * nItemMax);
     }
 }
@@ -110,18 +106,17 @@ int ERSHdrNode::ReadLine( VSILFILE * fp, CPLString &osLine )
     do
     {
         const char *pszNewLine = CPLReadLineL( fp );
-        
+
         if( pszNewLine == NULL )
             return FALSE;
 
         osLine += pszNewLine;
 
-        int  bInQuote = FALSE;
-        size_t  i;
+        bool bInQuote = false;
 
         nBracketLevel = 0;
 
-        for( i = 0; i < osLine.length(); i++ )
+        for( size_t i = 0; i < osLine.length(); i++ )
         {
             if( osLine[i] == '"' )
                 bInQuote = !bInQuote;
@@ -156,20 +151,21 @@ int ERSHdrNode::ReadLine( VSILFILE * fp, CPLString &osLine )
 int ERSHdrNode::ParseChildren( VSILFILE * fp )
 
 {
-    while( TRUE )
-    { 
-        size_t iOff;
-        CPLString osLine;
-
+    while( true )
+    {
 /* -------------------------------------------------------------------- */
 /*      Read the next line (or multi-line for bracketed value).         */
 /* -------------------------------------------------------------------- */
+        CPLString osLine;
+
         if( !ReadLine( fp, osLine ) )
             return FALSE;
 
 /* -------------------------------------------------------------------- */
 /*      Got a Name=Value.                                               */
 /* -------------------------------------------------------------------- */
+        size_t iOff;
+
         if( (iOff = osLine.find_first_of( '=' )) != std::string::npos )
         {
             CPLString osName = osLine.substr(0,iOff-1);
@@ -193,14 +189,14 @@ int ERSHdrNode::ParseChildren( VSILFILE * fp )
         {
             CPLString osName = osLine.substr(0,iOff);
             osName.Trim();
-            
+
             MakeSpace();
             papszItemName[nItemCount] = CPLStrdup(osName);
             papszItemValue[nItemCount] = NULL;
             papoItemChild[nItemCount] = new ERSHdrNode();
 
             nItemCount++;
-            
+
             if( !papoItemChild[nItemCount-1]->ParseChildren( fp ) )
                 return FALSE;
         }
@@ -219,8 +215,8 @@ int ERSHdrNode::ParseChildren( VSILFILE * fp )
 /* -------------------------------------------------------------------- */
         else if( osLine.Trim().length() > 0 )
         {
-            CPLError( CE_Failure, CPLE_AppDefined, 
-                      "Unexpected line parsing .ecw:\n%s", 
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "Unexpected line parsing .ecw:\n%s",
                       osLine.c_str() );
             return FALSE;
         }
@@ -237,27 +233,26 @@ int ERSHdrNode::WriteSelf( VSILFILE * fp, int nIndent )
 
 {
     CPLString oIndent;
-    int i;
 
     oIndent.assign( nIndent, '\t' );
 
-    for( i = 0; i < nItemCount; i++ )
+    for( int i = 0; i < nItemCount; i++ )
     {
         if( papszItemValue[i] != NULL )
         {
-            if( VSIFPrintfL( fp, "%s%s\t= %s\n", 
-                             oIndent.c_str(), 
-                             papszItemName[i], 
+            if( VSIFPrintfL( fp, "%s%s\t= %s\n",
+                             oIndent.c_str(),
+                             papszItemName[i],
                              papszItemValue[i] ) < 1 )
                 return FALSE;
         }
         else
         {
-            VSIFPrintfL( fp, "%s%s Begin\n", 
+            VSIFPrintfL( fp, "%s%s Begin\n",
                          oIndent.c_str(), papszItemName[i] );
             if( !papoItemChild[i]->WriteSelf( fp, nIndent+1 ) )
                 return FALSE;
-            if( VSIFPrintfL( fp, "%s%s End\n", 
+            if( VSIFPrintfL( fp, "%s%s End\n",
                              oIndent.c_str(), papszItemName[i] ) < 1 )
                 return FALSE;
         }
@@ -270,21 +265,19 @@ int ERSHdrNode::WriteSelf( VSILFILE * fp, int nIndent )
 /*                                Find()                                */
 /*                                                                      */
 /*      Find the desired entry value.  The input is a path with         */
-/*      components seperated by dots, relative to the current node.     */
+/*      components separated by dots, relative to the current node.     */
 /************************************************************************/
 
 const char *ERSHdrNode::Find( const char *pszPath, const char *pszDefault )
 
 {
-    int i;
-
 /* -------------------------------------------------------------------- */
 /*      If this is the final component of the path, search for a        */
 /*      matching child and return the value.                            */
 /* -------------------------------------------------------------------- */
     if( strchr(pszPath,'.') == NULL )
     {
-        for( i = 0; i < nItemCount; i++ )
+        for( int i = 0; i < nItemCount; i++ )
         {
             if( EQUAL(pszPath,papszItemName[i]) )
             {
@@ -292,9 +285,9 @@ const char *ERSHdrNode::Find( const char *pszPath, const char *pszDefault )
                 {
                     if( papszItemValue[i][0] == '"' )
                     {
-                        // strip off quotes. 
+                        // strip off quotes.
                         osTempReturn = papszItemValue[i];
-                        osTempReturn = 
+                        osTempReturn =
                             osTempReturn.substr( 1, osTempReturn.length()-2 );
                         return osTempReturn;
                     }
@@ -313,20 +306,19 @@ const char *ERSHdrNode::Find( const char *pszPath, const char *pszDefault )
 /*      and recurse.                                                    */
 /* -------------------------------------------------------------------- */
     CPLString osPathFirst, osPathRest, osPath = pszPath;
-    int iDot;
-    
-    iDot = osPath.find_first_of('.');
+
+    size_t iDot = osPath.find_first_of('.');
     osPathFirst = osPath.substr(0,iDot);
     osPathRest = osPath.substr(iDot+1);
 
-    for( i = 0; i < nItemCount; i++ )
+    for( int i = 0; i < nItemCount; i++ )
     {
         if( EQUAL(osPathFirst,papszItemName[i]) )
         {
             if( papoItemChild[i] != NULL )
                 return papoItemChild[i]->Find( osPathRest, pszDefault );
-            else
-                return pszDefault;
+
+            return pszDefault;
         }
     }
 
@@ -339,30 +331,30 @@ const char *ERSHdrNode::Find( const char *pszPath, const char *pszDefault )
 /*      Find a particular element from an array valued item.            */
 /************************************************************************/
 
-const char *ERSHdrNode::FindElem( const char *pszPath, int iElem, 
+const char *ERSHdrNode::FindElem( const char *pszPath, int iElem,
                                   const char *pszDefault )
 
 {
     const char *pszArray = Find( pszPath, NULL );
-    char **papszTokens;
-    int  bDefault = TRUE;
 
     if( pszArray == NULL )
         return pszDefault;
 
-    papszTokens = CSLTokenizeStringComplex( pszArray, "{ \t}", TRUE, FALSE );
+    bool bDefault = true;
+    char **papszTokens
+        = CSLTokenizeStringComplex( pszArray, "{ \t}", TRUE, FALSE );
     if( iElem >= 0 && iElem < CSLCount(papszTokens) )
     {
         osTempReturn = papszTokens[iElem];
-        bDefault = FALSE;
+        bDefault = false;
     }
-    
+
     CSLDestroy( papszTokens );
 
     if( bDefault )
         return pszDefault;
-    else
-        return osTempReturn;
+
+    return osTempReturn;
 }
 
 /************************************************************************/
@@ -374,12 +366,9 @@ const char *ERSHdrNode::FindElem( const char *pszPath, int iElem,
 ERSHdrNode *ERSHdrNode::FindNode( const char *pszPath )
 
 {
-    int i;
     CPLString osPathFirst, osPathRest, osPath = pszPath;
-    int iDot;
-    
-    iDot = osPath.find_first_of('.');
-    if( iDot == -1 )
+    size_t iDot = osPath.find_first_of('.');
+    if( iDot == std::string::npos )
     {
         osPathFirst = osPath;
     }
@@ -389,7 +378,7 @@ ERSHdrNode *ERSHdrNode::FindNode( const char *pszPath )
         osPathRest = osPath.substr(iDot+1);
     }
 
-    for( i = 0; i < nItemCount; i++ )
+    for( int i = 0; i < nItemCount; i++ )
     {
         if( EQUAL(osPathFirst,papszItemName[i]) )
         {
@@ -418,24 +407,22 @@ void ERSHdrNode::Set( const char *pszPath, const char *pszValue )
 
 {
     CPLString  osPath = pszPath;
-    int iDot;
-    
-    iDot = osPath.find_first_of('.');
+    size_t iDot = osPath.find_first_of('.');
 
 /* -------------------------------------------------------------------- */
 /*      We have an intermediate node, find or create it and             */
 /*      recurse.                                                        */
 /* -------------------------------------------------------------------- */
-    if( iDot != -1 )
+    if( iDot != std::string::npos )
     {
         CPLString osPathFirst = osPath.substr(0,iDot);
         CPLString osPathRest = osPath.substr(iDot+1);
         ERSHdrNode *poFirst = FindNode( osPathFirst );
-        
+
         if( poFirst == NULL )
         {
             poFirst = new ERSHdrNode();
-            
+
             MakeSpace();
             papszItemName[nItemCount] = CPLStrdup(osPathFirst);
             papszItemValue[nItemCount] = NULL;
@@ -450,11 +437,9 @@ void ERSHdrNode::Set( const char *pszPath, const char *pszValue )
 /* -------------------------------------------------------------------- */
 /*      This is the final item name.  Find or create it.                */
 /* -------------------------------------------------------------------- */
-    int i;
-
-    for( i = 0; i < nItemCount; i++ )
+    for( int i = 0; i < nItemCount; i++ )
     {
-        if( EQUAL(osPath,papszItemName[i]) 
+        if( EQUAL(osPath,papszItemName[i])
             && papszItemValue[i] != NULL )
         {
             CPLFree( papszItemValue[i] );
