@@ -1,7 +1,7 @@
 
 class ERSHdrNode;
 
-class ERSHdrNode 
+class ERSHdrNode
 {
     CPLString osTempReturn;
 
@@ -17,16 +17,16 @@ public:
     ERSHdrNode();
     ~ERSHdrNode();
 
-    int    ParseChildren( FILE *fp );
-    int    WriteSelf( FILE *fp, int nIndent );
+    int    ParseChildren( VSILFILE *fp );
+    int    WriteSelf( VSILFILE *fp, int nIndent );
 
     const char *Find( const char *pszPath, const char *pszDefault = NULL );
-    const char *FindElem( const char *pszPath, int iElem, 
+    const char *FindElem( const char *pszPath, int iElem,
                           const char *pszDefault = NULL );
     ERSHdrNode *FindNode( const char *pszPath );
 
     void   Set( const char *pszPath, const char *pszValue );
 
 private:
-    int    ReadLine( FILE *, CPLString & );
+    int    ReadLine( VSILFILE *, CPLString & );
 };
