@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: fitdataset.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: fitdataset.cpp 33720 2016-03-15 00:39:53Z goatbar $
  *
  * Project:  FIT Driver
  * Purpose:  Implement FIT Support - not using the SGI iflFIT library.
@@ -28,17 +28,13 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "fit.h"
-#include "gstEndian.h"
-#include "gdal_pam.h"
 #include "cpl_string.h"
+#include "fit.h"
+#include "gdal_frmts.h"
+#include "gdal_pam.h"
+#include "gstEndian.h"
 
-CPL_CVSID("$Id: fitdataset.cpp 27044 2014-03-16 23:41:27Z rouault $");
-
-CPL_C_START
- 
-void	GDALRegister_FIT(void);
-CPL_C_END
+CPL_CVSID("$Id: fitdataset.cpp 33720 2016-03-15 00:39:53Z goatbar $");
 
 #define FIT_WRITE
 
@@ -57,11 +53,11 @@ class FITRasterBand;
 class FITDataset : public GDALPamDataset
 {
     friend class FITRasterBand;
-    
+
     VSILFILE	*fp;
     FITinfo	*info;
     double      adfGeoTransform[6];
-    
+
   public:
     FITDataset();
     ~FITDataset();
@@ -71,8 +67,8 @@ class FITDataset : public GDALPamDataset
 
 #ifdef FIT_WRITE
 static GDALDataset *FITCreateCopy(const char * pszFilename,
-                                  GDALDataset *poSrcDS, 
-                                  int bStrict, char ** papszOptions, 
+                                  GDALDataset *poSrcDS,
+                                  int bStrict, char ** papszOptions,
                                   GDALProgressFunc pfnProgress,
                                   void * pProgressData );
 #endif // FIT_WRITE
@@ -86,7 +82,7 @@ static GDALDataset *FITCreateCopy(const char * pszFilename,
 class FITRasterBand : public GDALPamRasterBand
 {
     friend class FITDataset;
-    
+
     unsigned long recordSize; // number of bytes of a single page/block/record
     unsigned long numXBlocks; // number of pages in the X direction
     unsigned long numYBlocks; // number of pages in the Y direction
@@ -96,13 +92,13 @@ class FITRasterBand : public GDALPamRasterBand
 
 public:
 
-    FITRasterBand( FITDataset *, int );
+    FITRasterBand( FITDataset *, int nBandIn, int nBandsIn );
     ~FITRasterBand();
-    
+
     // should override RasterIO eventually.
-    
+
     virtual CPLErr IReadBlock( int, int, void * );
-//     virtual CPLErr WriteBlock( int, int, void * ); 
+//     virtual CPLErr WriteBlock( int, int, void * );
     virtual double GetMinimum( int *pbSuccess );
     virtual double GetMaximum( int *pbSuccess );
     virtual GDALColorInterp GetColorInterpretation();
@@ -113,39 +109,47 @@ public:
 /*                           FITRasterBand()                            */
 /************************************************************************/
 
-FITRasterBand::FITRasterBand( FITDataset *poDS, int nBand ) : tmpImage( NULL )
+FITRasterBand::FITRasterBand( FITDataset *poDSIn, int nBandIn, int nBandsIn ) :
+        recordSize(0),
+        numXBlocks(0),
+        numYBlocks(0),
+        bytesPerComponent(0),
+        bytesPerPixel(0),
+        tmpImage( NULL )
 
 {
-    this->poDS = poDS;
-    this->nBand = nBand;
-    
+    this->poDS = poDSIn;
+    this->nBand = nBandIn;
+
 /* -------------------------------------------------------------------- */
 /*      Get the GDAL data type.                                         */
 /* -------------------------------------------------------------------- */
-    eDataType = fitDataType(poDS->info->dtype);
+    eDataType = fitDataType(poDSIn->info->dtype);
 
 /* -------------------------------------------------------------------- */
 /*      Get the page sizes.                                             */
 /* -------------------------------------------------------------------- */
-    nBlockXSize = poDS->info->xPageSize;
-    nBlockYSize = poDS->info->yPageSize;
+    nBlockXSize = poDSIn->info->xPageSize;
+    nBlockYSize = poDSIn->info->yPageSize;
 
 /* -------------------------------------------------------------------- */
 /*      Caculate the values for record offset calculations.             */
 /* -------------------------------------------------------------------- */
     bytesPerComponent = (GDALGetDataTypeSize(eDataType) / 8);
-    bytesPerPixel = poDS->nBands * bytesPerComponent;
+    if( bytesPerComponent == 0 )
+        return;
+    bytesPerPixel = nBandsIn * bytesPerComponent;
+    if( nBlockXSize <= 0 || nBlockYSize <= 0 ||
+        nBlockXSize > INT_MAX / (int)bytesPerPixel ||
+        nBlockYSize > INT_MAX / (nBlockXSize * (int)bytesPerPixel) )
+        return;
     recordSize = bytesPerPixel * nBlockXSize * nBlockYSize;
     numXBlocks =
-        (unsigned long) ceil((double) poDS->info->xSize / nBlockXSize);
+        (unsigned long) ceil((double) poDSIn->info->xSize / nBlockXSize);
     numYBlocks =
-        (unsigned long) ceil((double) poDS->info->ySize / nBlockYSize);
-
-    tmpImage = (char *) malloc(recordSize);
-    if (! tmpImage)
-        CPLError(CE_Fatal, CPLE_NotSupported, 
-                 "FITRasterBand couldn't allocate %lu bytes", recordSize);
+        (unsigned long) ceil((double) poDSIn->info->ySize / nBlockYSize);
 
+    tmpImage = (char *) VSI_MALLOC_VERBOSE(recordSize);
 /* -------------------------------------------------------------------- */
 /*      Set the access flag.  For now we set it the same as the         */
 /*      whole dataset, but eventually this should take account of       */
@@ -157,8 +161,7 @@ FITRasterBand::FITRasterBand( FITDataset *poDS, int nBand ) : tmpImage( NULL )
 
 FITRasterBand::~FITRasterBand()
 {
-    if ( tmpImage )
-        free ( tmpImage );
+    VSIFree ( tmpImage );
 }
 
 
@@ -170,10 +173,10 @@ FITRasterBand::~FITRasterBand()
                 t *dstp = (t *) pImage; \
                 t *srcp = (t *) tmpImage; \
                 srcp += nBand-1; \
-                long i = 0; \
+                long imacro = 0; \
                 for(long y=ystart; y != ystop; y+= yinc) \
-                    for(long x=xstart; x != xstop; x+= xinc, i++) { \
-                        dstp[i] = srcp[(y * nBlockXSize + x) * \
+                    for(long x=xstart; x != xstop; x+= xinc, imacro++) { \
+                        dstp[imacro] = srcp[(y * nBlockXSize + x) * \
                                        poFIT_DS->nBands]; \
                     } \
     }
@@ -183,10 +186,10 @@ FITRasterBand::~FITRasterBand()
                 t *dstp = (t *) pImage; \
                 t *srcp = (t *) tmpImage; \
                 srcp += nBand-1; \
-                long i = 0; \
-                for(long x=xstart; x != xstop; x+= xinc, i++) \
+                long imacro = 0; \
+                for(long x=xstart; x != xstop; x+= xinc, imacro++) \
                     for(long y=ystart; y != ystop; y+= yinc) { \
-                        dstp[i] = srcp[(x * nBlockYSize + y) * \
+                        dstp[imacro] = srcp[(x * nBlockYSize + y) * \
                                        poFIT_DS->nBands]; \
                     } \
     }
@@ -246,7 +249,7 @@ CPLErr FITRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
         CPLError(CE_Failure, CPLE_NotSupported,
                  "FIT - unrecognized image space %i",
                  poFIT_DS->info->space);
-        tilenum = 0;
+        return CE_Failure;
     } // switch
 
     uint64 offset = poFIT_DS->info->dataOffset + recordSize * tilenum;
@@ -268,15 +271,21 @@ CPLErr FITRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
     if ((poFIT_DS->nBands == 1) && (poFIT_DS->info->space == 1)) // upper left
         fastpath = TRUE;
 
+    size_t nRead;
     if (! fastpath) {
-        VSIFReadL( tmpImage, recordSize, 1, poFIT_DS->fp );
+        nRead = VSIFReadL( tmpImage, recordSize, 1, poFIT_DS->fp );
         // offset to correct component to swap
         p = (char *) tmpImage + nBand-1;
     }
     else {
-        VSIFReadL( pImage, recordSize, 1, poFIT_DS->fp );
+        nRead = VSIFReadL( pImage, recordSize, 1, poFIT_DS->fp );
         p = (char *) pImage;
     }
+    if( nRead != 1 )
+    {
+        CPLError(CE_Failure, CPLE_FileIO, "Cannot read record");
+        return CE_Failure;
+    }
 
 
 #ifdef swapping
@@ -299,7 +308,7 @@ CPLErr FITRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
             gst_swap64(p + i);
         break;
     default:
-        CPLError(CE_Failure, CPLE_NotSupported, 
+        CPLError(CE_Failure, CPLE_NotSupported,
                  "FITRasterBand::IReadBlock unsupported bytesPerPixel %lu",
                  bytesPerComponent);
     } // switch
@@ -382,7 +391,7 @@ CPLErr FITRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                 COPY_XFIRST(uint64);
                 break;
             default:
-                CPLError(CE_Failure, CPLE_NotSupported, 
+                CPLError(CE_Failure, CPLE_NotSupported,
                          "FITRasterBand::IReadBlock unsupported "
                          "bytesPerComponent %lu", bytesPerComponent);
             } // switch
@@ -460,7 +469,7 @@ CPLErr FITRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                 COPY_YFIRST(uint64);
                 break;
             default:
-                CPLError(CE_Failure, CPLE_NotSupported, 
+                CPLError(CE_Failure, CPLE_NotSupported,
                          "FITRasterBand::IReadBlock unsupported "
                          "bytesPerComponent %lu", bytesPerComponent);
             } // switch
@@ -482,8 +491,6 @@ CPLErr FITRasterBand::ReadBlock( int nBlockXOff, int nBlockYOff,
 {
     FITDataset	*poFIT_DS = (FITDataset *) poDS;
 
-    
-
     return CE_None;
 }
 
@@ -497,8 +504,6 @@ CPLErr FITRasterBand::WriteBlock( int nBlockXOff, int nBlockYOff,
 {
     FITDataset	*poFIT_DS = (FITDataset *) poDS;
 
-    
-
     return CE_None;
 }
 #endif
@@ -518,12 +523,11 @@ double FITRasterBand::GetMinimum( int *pbSuccess )
         *pbSuccess = TRUE;
 
     if (poFIT_DS->info->version &&
-        EQUALN((const char *) &(poFIT_DS->info->version), "02", 2)) {
+        STARTS_WITH_CI((const char *) &(poFIT_DS->info->version), "02")) {
         return poFIT_DS->info->minValue;
     }
-    else {
-        return GDALRasterBand::GetMinimum( pbSuccess );
-    }
+
+    return GDALRasterBand::GetMinimum( pbSuccess );
 }
 
 /************************************************************************/
@@ -540,12 +544,11 @@ double FITRasterBand::GetMaximum( int *pbSuccess )
     if (pbSuccess)
         *pbSuccess = TRUE;
 
-    if (EQUALN((const char *) &poFIT_DS->info->version, "02", 2)) {
+    if (STARTS_WITH_CI((const char *) &poFIT_DS->info->version, "02")) {
         return poFIT_DS->info->maxValue;
     }
-    else {
-        return GDALRasterBand::GetMaximum( pbSuccess );
-    }
+
+    return GDALRasterBand::GetMaximum( pbSuccess );
 }
 
 /************************************************************************/
@@ -561,13 +564,13 @@ GDALColorInterp FITRasterBand::GetColorInterpretation()
 
     switch(poFIT_DS->info->cm) {
     case 1: // iflNegative - inverted luminance (min value is white)
-        CPLError( CE_Warning, CPLE_NotSupported, 
+        CPLError( CE_Warning, CPLE_NotSupported,
                   "FIT - color model Negative not supported - ignoring model");
             return GCI_Undefined;
 
     case 2: // iflLuminance - luminance
         if (poFIT_DS->nBands != 1) {
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model Luminance mismatch with %i bands",
                       poFIT_DS->nBands);
             return GCI_Undefined;
@@ -576,14 +579,14 @@ GDALColorInterp FITRasterBand::GetColorInterpretation()
         case 1:
             return GCI_GrayIndex;
         default:
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model Luminance unknown band %i", nBand);
             return GCI_Undefined;
         } // switch nBand
 
     case 3: // iflRGB - full color (Red, Green, Blue triplets)
         if (poFIT_DS->nBands != 3) {
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model RGB mismatch with %i bands",
                       poFIT_DS->nBands);
             return GCI_Undefined;
@@ -596,20 +599,20 @@ GDALColorInterp FITRasterBand::GetColorInterpretation()
         case 3:
             return GCI_BlueBand;
         default:
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model RGB unknown band %i", nBand);
             return GCI_Undefined;
         } // switch nBand
 
     case 4: // iflRGBPalette - color mapped values
-        CPLError( CE_Warning, CPLE_NotSupported, 
+        CPLError( CE_Warning, CPLE_NotSupported,
                   "FIT - color model  RGBPalette not supported - "
                   "ignoring model");
             return GCI_Undefined;
 
     case 5: // iflRGBA - full color with transparency (alpha channel)
         if (poFIT_DS->nBands != 4) {
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model RGBA mismatch with %i bands",
                       poFIT_DS->nBands);
             return GCI_Undefined;
@@ -624,14 +627,14 @@ GDALColorInterp FITRasterBand::GetColorInterpretation()
         case 4:
             return GCI_AlphaBand;
         default:
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model RGBA unknown band %i", nBand);
             return GCI_Undefined;
         } // switch nBand
 
     case 6: // iflHSV - Hue, Saturation, Value
         if (poFIT_DS->nBands != 3) {
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model HSV mismatch with %i bands",
                       poFIT_DS->nBands);
             return GCI_Undefined;
@@ -644,14 +647,14 @@ GDALColorInterp FITRasterBand::GetColorInterpretation()
         case 3:
             return GCI_LightnessBand;
         default:
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model HSV unknown band %i", nBand);
             return GCI_Undefined;
         } // switch nBand
 
     case 7: // iflCMY - Cyan, Magenta, Yellow
         if (poFIT_DS->nBands != 3) {
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model CMY mismatch with %i bands",
                       poFIT_DS->nBands);
             return GCI_Undefined;
@@ -664,14 +667,14 @@ GDALColorInterp FITRasterBand::GetColorInterpretation()
         case 3:
             return GCI_YellowBand;
         default:
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model CMY unknown band %i", nBand);
             return GCI_Undefined;
         } // switch nBand
 
     case 8: // iflCMYK - Cyan, Magenta, Yellow, Black
         if (poFIT_DS->nBands != 4) {
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model CMYK mismatch with %i bands",
                       poFIT_DS->nBands);
             return GCI_Undefined;
@@ -686,14 +689,14 @@ GDALColorInterp FITRasterBand::GetColorInterpretation()
         case 4:
             return GCI_BlackBand;
         default:
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model CMYK unknown band %i", nBand);
             return GCI_Undefined;
         } // switch nBand
 
     case 9: // iflBGR - full color (ordered Blue, Green, Red)
         if (poFIT_DS->nBands != 3) {
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model BGR mismatch with %i bands",
                       poFIT_DS->nBands);
             return GCI_Undefined;
@@ -706,14 +709,14 @@ GDALColorInterp FITRasterBand::GetColorInterpretation()
         case 3:
             return GCI_RedBand;
         default:
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model BGR unknown band %i", nBand);
             return GCI_Undefined;
         } // switch nBand
 
     case 10: // iflABGR - Alpha, Blue, Green, Red (SGI frame buffers)
         if (poFIT_DS->nBands != 4) {
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model ABGR mismatch with %i bands",
                       poFIT_DS->nBands);
             return GCI_Undefined;
@@ -728,7 +731,7 @@ GDALColorInterp FITRasterBand::GetColorInterpretation()
         case 4:
             return GCI_RedBand;
         default:
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model ABGR unknown band %i", nBand);
             return GCI_Undefined;
         } // switch nBand
@@ -738,13 +741,13 @@ GDALColorInterp FITRasterBand::GetColorInterpretation()
         return GCI_Undefined;
 
     case 12: // iflYCC PhotoCD color model (Luminance, Chrominance)
-        CPLError( CE_Warning, CPLE_NotSupported, 
+        CPLError( CE_Warning, CPLE_NotSupported,
                   "FIT - color model YCC not supported - ignoring model");
             return GCI_Undefined;
 
     case 13: // iflLuminanceAlpha - Luminance plus alpha
         if (poFIT_DS->nBands != 2) {
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model LuminanceAlpha mismatch with "
                       "%i bands",
                       poFIT_DS->nBands);
@@ -756,14 +759,14 @@ GDALColorInterp FITRasterBand::GetColorInterpretation()
         case 2:
             return GCI_AlphaBand;
         default:
-            CPLError( CE_Failure, CPLE_NotSupported, 
+            CPLError( CE_Failure, CPLE_NotSupported,
                       "FIT - color model LuminanceAlpha unknown band %i",
                       nBand);
             return GCI_Undefined;
         } // switch nBand
 
     default:
-        CPLError( CE_Warning, CPLE_NotSupported, 
+        CPLError( CE_Warning, CPLE_NotSupported,
                   "FIT - unrecognized color model %i - ignoring model",
                   poFIT_DS->info->cm);
         return GCI_Undefined;
@@ -776,7 +779,6 @@ GDALColorInterp FITRasterBand::GetColorInterpretation()
 
 FITDataset::FITDataset() : fp( NULL ), info( NULL )
 {
-
     adfGeoTransform[0] = 0.0; // x origin (top left corner)
     adfGeoTransform[1] = 1.0; // x pixel size
     adfGeoTransform[2] = 0.0;
@@ -796,10 +798,15 @@ FITDataset::~FITDataset()
     if (info)
         delete(info);
     if(fp)
-        VSIFCloseL(fp);
+    {
+        if( VSIFCloseL(fp) != 0 )
+        {
+            CPLError(CE_Failure, CPLE_FileIO, "I/O error");
+        }
+    }
 }
 
-// simple guard object to delete memory 
+// simple guard object to delete memory
 // when the guard goes out of scope
 template< class T >
 class DeleteGuard
@@ -814,18 +821,18 @@ public:
     T *take()
     {
         T *tmp = _ptr;
-	    _ptr = 0;
+	    _ptr = NULL;
 	    return tmp;
     }
 
 private:
     T *_ptr;
 	// prevent default copy constructor and assignment operator
-    DeleteGuard( const DeleteGuard & );  
+    DeleteGuard( const DeleteGuard & );
     DeleteGuard &operator=( const DeleteGuard & );
 };
 
-// simple guard object to free memory 
+// simple guard object to free memory
 // when the guard goes out of scope
 template< class T >
 class FreeGuard
@@ -841,14 +848,14 @@ public:
     T *take()
     {
         T *tmp = _ptr;
-	    _ptr = 0;
+	    _ptr = NULL;
 	    return tmp;
     }
 
 private:
     T *_ptr;
 	// prevent default copy constructor and assignment operator
-    FreeGuard( const FreeGuard & );  
+    FreeGuard( const FreeGuard & );
     FreeGuard &operator=( const FreeGuard & );
 };
 
@@ -860,20 +867,20 @@ GDALDataset *FITDataset::Open( GDALOpenInfo * poOpenInfo )
 {
 /* -------------------------------------------------------------------- */
 /*	First we check to see if the file has the expected header	*/
-/*	bytes.								*/    
+/*	bytes.								*/
 /* -------------------------------------------------------------------- */
 
     if( poOpenInfo->nHeaderBytes < 5 )
         return NULL;
 
 
-    if( !EQUALN((const char *) poOpenInfo->pabyHeader, "IT01", 4) &&
-        !EQUALN((const char *) poOpenInfo->pabyHeader, "IT02", 4) )
+    if( !STARTS_WITH_CI((const char *) poOpenInfo->pabyHeader, "IT01") &&
+        !STARTS_WITH_CI((const char *) poOpenInfo->pabyHeader, "IT02") )
         return NULL;
 
     if( poOpenInfo->eAccess == GA_Update )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The FIT driver does not support update access to existing"
                   " files.\n" );
         return NULL;
@@ -882,9 +889,7 @@ GDALDataset *FITDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
-    FITDataset 	*poDS;
-
-    poDS = new FITDataset();
+    FITDataset 	*poDS = new FITDataset();
     DeleteGuard<FITDataset> guard( poDS );
 
 	// re-open file for large file (64bit) access
@@ -911,7 +916,7 @@ GDALDataset *FITDataset::Open( GDALOpenInfo * poOpenInfo )
     FIThead02 *head = (FIThead02 *) poOpenInfo->pabyHeader;
 
     // extract the image attributes from the file header
-    if (EQUALN((const char *) &head->version, "02", 2)) {
+    if (STARTS_WITH_CI((const char *) &head->version, "02")) {
         // incomplete header
         if( poOpenInfo->nHeaderBytes < (signed) sizeof(FIThead02) )
             return NULL;
@@ -927,7 +932,7 @@ GDALDataset *FITDataset::Open( GDALOpenInfo * poOpenInfo )
 
         info->userOffset = sizeof(FIThead02);
     }
-    else if (EQUALN((const char *) &head->version, "01", 2)) {
+    else if (STARTS_WITH_CI((const char *) &head->version, "01")) {
         // incomplete header
         if( poOpenInfo->nHeaderBytes < (signed) sizeof(FIThead01) )
             return NULL;
@@ -943,7 +948,7 @@ GDALDataset *FITDataset::Open( GDALOpenInfo * poOpenInfo )
     }
     else {
         // unrecognized header version
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "FIT - unsupported header version %.2s\n",
                   (const char*) &head->version);
         return NULL;
@@ -981,7 +986,7 @@ GDALDataset *FITDataset::Open( GDALOpenInfo * poOpenInfo )
     info->cPageSize = head->cPageSize;
 
     CPLDebug("FIT", "size %i %i %i %i, pageSize %i %i %i %i",
-             info->xSize, info->ySize, info->zSize, info->cSize, 
+             info->xSize, info->ySize, info->zSize, info->cSize,
              info->xPageSize, info->yPageSize, info->zPageSize,
              info->cPageSize);
 
@@ -992,7 +997,12 @@ GDALDataset *FITDataset::Open( GDALOpenInfo * poOpenInfo )
 
     poDS->nRasterXSize = head->xSize;
     poDS->nRasterYSize = head->ySize;
-    poDS->nBands = head->cSize;
+
+    if (!GDALCheckDatasetDimensions(poDS->nRasterXSize, poDS->nRasterYSize) ||
+        !GDALCheckBandCount(head->cSize, FALSE))
+    {
+        return NULL;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Check if 64 bit seek is needed.                                 */
@@ -1015,7 +1025,7 @@ GDALDataset *FITDataset::Open( GDALOpenInfo * poOpenInfo )
 #ifdef VSI_LARGE_API_SUPPORTED
         CPLDebug("FIT", "Using 64 bit version of fseek");
 #else
-        CPLError(CE_Fatal, CPLE_NotSupported, 
+        CPLError(CE_Fatal, CPLE_NotSupported,
                  "FIT - need 64 bit version of fseek");
 #endif
 
@@ -1025,28 +1035,28 @@ GDALDataset *FITDataset::Open( GDALOpenInfo * poOpenInfo )
 
     if( info->zSize != 1 )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "FIT driver - unsupported zSize %i\n", info->zSize);
         return NULL;
     }
 
     if( info->order != 1 ) // interleaved - RGBRGB
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "FIT driver - unsupported order %i\n", info->order);
         return NULL;
     }
 
     if( info->zPageSize != 1 )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "FIT driver - unsupported zPageSize %i\n", info->zPageSize);
         return NULL;
     }
 
     if( info->cPageSize != info->cSize )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "FIT driver - unsupported cPageSize %i (!= %i)\n",
                   info->cPageSize, info->cSize);
         return NULL;
@@ -1055,9 +1065,12 @@ GDALDataset *FITDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Create band information objects.                                */
 /* -------------------------------------------------------------------- */
-    for( int i = 0; i < poDS->nBands; i++ )
+    for( int i = 0; i < (int)head->cSize; i++ )
     {
-        poDS->SetBand( i+1,  new FITRasterBand( poDS, i+1 ) ) ;
+        FITRasterBand* poBand = new FITRasterBand( poDS, i+1, (int)head->cSize );
+        poDS->SetBand( i+1,  poBand);
+        if( poBand->tmpImage == NULL )
+            return NULL;
     }
 
 /* -------------------------------------------------------------------- */
@@ -1069,7 +1082,7 @@ GDALDataset *FITDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Check for external overviews.                                   */
 /* -------------------------------------------------------------------- */
-    poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename, poOpenInfo->papszSiblingFiles );
+    poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename, poOpenInfo->GetSiblingFiles() );
 
     return guard.take();
 }
@@ -1080,8 +1093,8 @@ GDALDataset *FITDataset::Open( GDALOpenInfo * poOpenInfo )
 
 #ifdef FIT_WRITE
 static GDALDataset *FITCreateCopy(const char * pszFilename,
-                                  GDALDataset *poSrcDS, 
-                                  int bStrict, char ** papszOptions, 
+                                  GDALDataset *poSrcDS,
+                                  int bStrict, char ** papszOptions,
                                   GDALProgressFunc pfnProgress,
                                   void * pProgressData )
 {
@@ -1090,7 +1103,7 @@ static GDALDataset *FITCreateCopy(const char * pszFilename,
     int nBands = poSrcDS->GetRasterCount();
     if (nBands == 0)
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "FIT driver does not support source dataset with zero band.\n");
         return NULL;
     }
@@ -1098,19 +1111,17 @@ static GDALDataset *FITCreateCopy(const char * pszFilename,
 /* -------------------------------------------------------------------- */
 /*      Create the dataset.                                             */
 /* -------------------------------------------------------------------- */
-    VSILFILE	*fpImage;
-
     if( !pfnProgress( 0.0, NULL, pProgressData ) )
     {
         CPLError( CE_Failure, CPLE_UserInterrupt, "User terminated" );
         return NULL;
     }
 
-    fpImage = VSIFOpenL( pszFilename, "wb" );
+    VSILFILE *fpImage = VSIFOpenL( pszFilename, "wb" );
     if( fpImage == NULL )
     {
-        CPLError( CE_Failure, CPLE_OpenFailed, 
-                  "FIT - unable to create file %s.\n", 
+        CPLError( CE_Failure, CPLE_OpenFailed,
+                  "FIT - unable to create file %s.\n",
                   pszFilename );
         return NULL;
     }
@@ -1126,8 +1137,8 @@ static GDALDataset *FITCreateCopy(const char * pszFilename,
     // clean header so padding (past real header) is all zeros
     memset( head, 0, size );
 
-    strncpy((char *) &head->magic, "IT", 2);
-    strncpy((char *) &head->version, "02", 2);
+    memcpy((char *) &head->magic, "IT", 2);
+    memcpy((char *) &head->version, "02", 2);
 
     head->xSize = poSrcDS->GetRasterXSize();
     gst_swapb(head->xSize);
@@ -1141,13 +1152,13 @@ static GDALDataset *FITCreateCopy(const char * pszFilename,
 
     GDALRasterBand *firstBand = poSrcDS->GetRasterBand(1);
     if (! firstBand) {
-        VSIFCloseL(fpImage);
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fpImage));
         return NULL;
     }
 
     head->dtype = fitGetDataType(firstBand->GetRasterDataType());
     if (! head->dtype) {
-        VSIFCloseL(fpImage);
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fpImage));
         return NULL;
     }
     gst_swapb(head->dtype);
@@ -1174,7 +1185,7 @@ static GDALDataset *FITCreateCopy(const char * pszFilename,
             blockY = newBlockY;
         }
         else {
-            CPLError(CE_Failure, CPLE_OpenFailed, 
+            CPLError(CE_Failure, CPLE_OpenFailed,
                      "FIT - Unable to parse option PAGESIZE values [%s]", str);
         }
     }
@@ -1186,7 +1197,7 @@ static GDALDataset *FITCreateCopy(const char * pszFilename,
     // * handle block size bigger than image size
     // * undesirable block size (non power of 2, others?)
     // * mismatched block sizes for different bands
-    // * image that isn't even pages (ie. partially empty pages at edge)
+    // * image that isn't even pages (i.e. partially empty pages at edge)
     CPLDebug("FIT write", "using block size %ix%i", blockX, blockY);
 
     head->xPageSize = blockX;
@@ -1207,7 +1218,7 @@ static GDALDataset *FITCreateCopy(const char * pszFilename,
     head->dataOffset = size;
     gst_swapb(head->dataOffset);
 
-    VSIFWriteL(head, size, 1, fpImage);
+    CPL_IGNORE_RET_VAL(VSIFWriteL(head, size, 1, fpImage));
 
 /* -------------------------------------------------------------------- */
 /*      Loop over image, copying image data.                            */
@@ -1219,7 +1230,7 @@ static GDALDataset *FITCreateCopy(const char * pszFilename,
     unsigned long pageBytes = blockX * blockY * bytesPerPixel;
     char *output = (char *) malloc(pageBytes);
     if (! output)
-        CPLError(CE_Fatal, CPLE_NotSupported, 
+        CPLError(CE_Fatal, CPLE_NotSupported,
                  "FITRasterBand couldn't allocate %lu bytes", pageBytes);
     FreeGuard<char> guardOutput( output );
 
@@ -1255,10 +1266,10 @@ static GDALDataset *FITCreateCopy(const char * pszFilename,
                 GDALRasterBand * poBand = poSrcDS->GetRasterBand( iBand+1 );
                 CPLErr eErr =
                     poBand->RasterIO( GF_Read, // eRWFlag
-                                      x * blockX, // nXOff
-                                      y * blockY, // nYOff
-                                      readX, // nXSize
-                                      readY, // nYSize
+                                      static_cast<int>(x * blockX), // nXOff
+                                      static_cast<int>(y * blockY), // nYOff
+                                      static_cast<int>(readX), // nXSize
+                                      static_cast<int>(readY), // nYSize
                                       output + iBand * bytesPerComponent,
                                       // pData
                                       blockX, // nBufXSize
@@ -1266,9 +1277,9 @@ static GDALDataset *FITCreateCopy(const char * pszFilename,
                                       firstBand->GetRasterDataType(),
                                       // eBufType
                                       bytesPerPixel, // nPixelSpace
-                                      bytesPerPixel * blockX); // nLineSpace
+                                      bytesPerPixel * blockX, NULL); // nLineSpace
                 if (eErr != CE_None)
-                    CPLError(CE_Failure, CPLE_FileIO, 
+                    CPLError(CE_Failure, CPLE_FileIO,
                              "FIT write - CreateCopy got read error %i", eErr);
             } // for iBand
 
@@ -1292,13 +1303,13 @@ static GDALDataset *FITCreateCopy(const char * pszFilename,
                     gst_swap64(p + i);
                 break;
             default:
-                CPLError(CE_Failure, CPLE_NotSupported, 
+                CPLError(CE_Failure, CPLE_NotSupported,
                          "FIT write - unsupported bytesPerPixel %lu",
                          bytesPerComponent);
             } // switch
 #endif // swapping
-            
-            VSIFWriteL(output, pageBytes, 1, fpImage);
+
+            CPL_IGNORE_RET_VAL(VSIFWriteL(output, pageBytes, 1, fpImage));
 
             double perc = ((double) (y * maxx + x)) / (maxx * maxy);
 //             printf("progress %f\n", perc);
@@ -1306,7 +1317,7 @@ static GDALDataset *FITCreateCopy(const char * pszFilename,
             {
                 CPLError( CE_Failure, CPLE_UserInterrupt, "User terminated" );
                 //free(output);
-                VSIFCloseL( fpImage );
+                CPL_IGNORE_RET_VAL(VSIFCloseL( fpImage ));
                 VSIUnlink( pszFilename );
                 return NULL;
             }
@@ -1314,14 +1325,14 @@ static GDALDataset *FITCreateCopy(const char * pszFilename,
 
     //free(output);
 
-    VSIFCloseL( fpImage );
+    CPL_IGNORE_RET_VAL(VSIFCloseL( fpImage ));
 
     pfnProgress( 1.0, NULL, pProgressData );
 
 /* -------------------------------------------------------------------- */
-/*      Re-open dataset, and copy any auxilary pam information.         */
+/*      Re-open dataset, and copy any auxiliary pam information.         */
 /* -------------------------------------------------------------------- */
-    GDALPamDataset *poDS = (GDALPamDataset *) 
+    GDALPamDataset *poDS = (GDALPamDataset *)
         GDALOpen( pszFilename, GA_ReadOnly );
 
     if( poDS )
@@ -1349,27 +1360,25 @@ static GDALDataset *FITCreateCopy(const char * pszFilename,
 void GDALRegister_FIT()
 
 {
-    GDALDriver	*poDriver;
+    if( GDALGetDriverByName( "FIT" ) != NULL )
+        return;
 
-    if( GDALGetDriverByName( "FIT" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "FIT" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "FIT Image" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_various.html#" );
-        poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "" );
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->pfnOpen = FITDataset::Open;
+    poDriver->SetDescription( "FIT" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "FIT Image" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_various.html#" );
+    poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+
+    poDriver->pfnOpen = FITDataset::Open;
 #ifdef FIT_WRITE
-        poDriver->pfnCreateCopy = FITCreateCopy;
-        poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, 
-                                   "Byte UInt16 Int16 UInt32 Int32 Float32 Float64" );
+    poDriver->pfnCreateCopy = FITCreateCopy;
+    poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES,
+                               "Byte UInt16 Int16 UInt32 Int32 "
+                               "Float32 Float64" );
 #endif // FIT_WRITE
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
