@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalallregister.cpp 27196 2014-04-16 11:41:24Z rouault $
+ * $Id: gdalallregister.cpp 33546 2016-02-24 21:02:52Z goatbar $
  *
  * Project:  GDAL Core
  * Purpose:  Implementation of GDALAllRegister(), primary format registration.
@@ -30,8 +30,13 @@
 
 #include "gdal_priv.h"
 #include "gdal_frmts.h"
+#include "ogrsf_frmts.h"
 
-CPL_CVSID("$Id: gdalallregister.cpp 27196 2014-04-16 11:41:24Z rouault $");
+#ifdef GNM_ENABLED
+   #include "gnm_frmts.h"
+#endif
+
+CPL_CVSID("$Id: gdalallregister.cpp 33546 2016-02-24 21:02:52Z goatbar $");
 
 #ifdef notdef
 // we may have a use for this some day
@@ -48,45 +53,26 @@ static char *szConfiguredFormats = "GDAL_FORMATS";
  * Register all known configured GDAL drivers.
  *
  * This function will drive any of the following that are configured into
- * GDAL.  Many others as well haven't been updated in this
- * documentation (see <a href="http://gdal.org/formats_list.html">full list</a>):
+ * GDAL.  See <a href="http://gdal.org/formats_list.html">raster list</a> and
+ * <a href="http://gdal.org/ogr_formats.html">vector full list</a>
  *
- * <ul>
- * <li> GeoTIFF (GTiff)
- * <li> Geosoft GXF (GXF)
- * <li> Erdas Imagine (HFA)
- * <li> CEOS (CEOS)
- * <li> ELAS (ELAS)
- * <li> Arc/Info Binary Grid (AIGrid)
- * <li> SDTS Raster DEM (SDTS)
- * <li> OGDI (OGDI)
- * <li> ESRI Labelled BIL (EHdr)
- * <li> PCI .aux Labelled Raw Raster (PAux)
- * <li> HDF4 Hierachal Data Format Release 4
- * <li> HDF5 Hierachal Data Format Release 5
- * <li> GSAG Golden Software ASCII Grid
- * <li> GSBG Golden Software Binary Grid
- * </ul>
- *
- * This function should generally be called once at the beginning of the application.
+ * This function should generally be called once at the beginning of the
+ * application.
  */
 
 void CPL_STDCALL GDALAllRegister()
 
 {
+    // AutoLoadDrivers is a no-op if compiled with GDAL_NO_AUTOLOAD defined.
     GetGDALDriverManager()->AutoLoadDrivers();
 
 #ifdef FRMT_vrt
     GDALRegister_VRT();
-#endif    
+#endif
 
-#ifdef FRMT_gdb    
-    GDALRegister_GDB();
-#endif    
-
-#ifdef FRMT_gtiff    
+#ifdef FRMT_gtiff
     GDALRegister_GTiff();
-#endif    
+#endif
 
 #ifdef FRMT_nitf
     GDALRegister_NITF();
@@ -97,19 +83,19 @@ void CPL_STDCALL GDALAllRegister()
 #ifdef FRMT_hfa
     GDALRegister_HFA();
 #endif
-    
+
 #ifdef FRMT_ceos2
     GDALRegister_SAR_CEOS();
 #endif
-    
+
 #ifdef FRMT_ceos
     GDALRegister_CEOS();
 #endif
-    
+
 #ifdef FRMT_jaxapalsar
     GDALRegister_PALSARJaxa();
 #endif
-    
+
 #ifdef FRMT_gff
     GDALRegister_GFF();
 #endif
@@ -117,7 +103,7 @@ void CPL_STDCALL GDALAllRegister()
 #ifdef FRMT_elas
     GDALRegister_ELAS();
 #endif
-    
+
 #ifdef FRMT_aigrid
 //    GDALRegister_AIGrid2();
     GDALRegister_AIGrid();
@@ -147,7 +133,7 @@ void CPL_STDCALL GDALAllRegister()
 #ifdef FRMT_dds
     GDALRegister_DDS();
 #endif
-    
+
 #ifdef FRMT_gta
     GDALRegister_GTA();
 #endif
@@ -205,6 +191,10 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_RS2();
 #endif
 
+#ifdef FRMT_safe
+    GDALRegister_SAFE();
+#endif
+
 #ifdef FRMT_pcidsk
     GDALRegister_PCIDSK();
 #endif
@@ -247,6 +237,7 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_ISIS3();
     GDALRegister_ISIS2();
     GDALRegister_PDS();
+    GDALRegister_VICAR();
 #endif
 
 #ifdef FRMT_til
@@ -366,6 +357,46 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_MAP();
 #endif
 
+#ifdef FRMT_kmlsuperoverlay
+    GDALRegister_KMLSUPEROVERLAY();
+#endif
+
+#ifdef FRMT_webp
+    GDALRegister_WEBP();
+#endif
+
+#ifdef FRMT_pdf
+    GDALRegister_PDF();
+#endif
+
+#ifdef FRMT_rasterlite
+    GDALRegister_Rasterlite();
+#endif
+
+#ifdef FRMT_mbtiles
+    GDALRegister_MBTiles();
+#endif
+
+#ifdef FRMT_plmosaic
+    GDALRegister_PLMOSAIC();
+#endif
+
+#ifdef FRMT_cals
+    GDALRegister_CALS();
+#endif
+
+#ifdef FRMT_wmts
+    GDALRegister_WMTS();
+#endif
+
+#ifdef FRMT_sentinel2
+    GDALRegister_SENTINEL2();
+#endif
+
+#ifdef FRMT_mrf
+    GDALRegister_mrf();
+#endif
+
 /* -------------------------------------------------------------------- */
 /*      Put raw formats at the end of the list. These drivers support   */
 /*      various ASCII-header labeled formats, so the driver could be    */
@@ -377,8 +408,6 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_PNM();
     GDALRegister_DOQ1();
     GDALRegister_DOQ2();
-    GDALRegister_ENVI();
-    GDALRegister_EHdr();
     GDALRegister_GenBin();
     GDALRegister_PAux();
     GDALRegister_MFF();
@@ -401,6 +430,12 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_ACE2();
     GDALRegister_SNODAS();
     GDALRegister_KRO();
+    GDALRegister_ROIPAC();
+
+    // Those ones need to look for side car files so put them at end
+    GDALRegister_ENVI();
+    GDALRegister_EHdr();
+    GDALRegister_ISCE();
 #endif
 
 #ifdef FRMT_arg
@@ -422,7 +457,7 @@ void CPL_STDCALL GDALAllRegister()
 
 #ifdef FRMT_gxf
     GDALRegister_GXF();
-#endif    
+#endif
 
 #ifdef FRMT_grass
     GDALRegister_GRASS();
@@ -432,8 +467,9 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_DODS();
 #endif
 
-#ifdef FRMT_wcs
-    GDALRegister_HTTP();
+/* Register KEA before HDF5 */
+#ifdef FRMT_kea
+    GDALRegister_KEA();
 #endif
 
 #ifdef FRMT_hdf5
@@ -456,18 +492,10 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_BLX();
 #endif
 
-#ifdef FRMT_pgchip
-    GDALRegister_PGCHIP();
-#endif
-
 #ifdef FRMT_georaster
     GDALRegister_GEOR();
 #endif
 
-#ifdef FRMT_rasterlite
-    GDALRegister_Rasterlite();
-#endif
-
 #ifdef FRMT_epsilon
     GDALRegister_EPSILON();
 #endif
@@ -480,10 +508,6 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_SAGA();
 #endif
 
-#ifdef FRMT_kmlsuperoverlay
-    GDALRegister_KMLSUPEROVERLAY();
-#endif
-
 #ifdef FRMT_xyz
     GDALRegister_XYZ();
 #endif
@@ -492,10 +516,6 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_HF2();
 #endif
 
-#ifdef FRMT_pdf
-    GDALRegister_PDF();
-#endif
-
 #ifdef FRMT_jpegls
     GDALRegister_JPEGLS();
 #endif
@@ -512,10 +532,6 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_E00GRID();
 #endif
 
-#ifdef FRMT_webp
-    GDALRegister_WEBP();
-#endif
-
 #ifdef FRMT_zmap
     GDALRegister_ZMap();
 #endif
@@ -524,15 +540,22 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_NGSGEOID();
 #endif
 
-#ifdef FRMT_mbtiles
-    GDALRegister_MBTiles();
-#endif
-
 #ifdef FRMT_iris
     GDALRegister_IRIS();
 #endif
+
+#ifdef GNM_ENABLED
+    GNMRegisterAllInternal();
+#endif
+
+    OGRRegisterAllInternal();
+
+#ifdef FRMT_wcs
+    GDALRegister_HTTP();
+#endif
+
 /* -------------------------------------------------------------------- */
-/*      Deregister any drivers explicitly marked as supressed by the    */
+/*      Deregister any drivers explicitly marked as suppressed by the   */
 /*      GDAL_SKIP environment variable.                                 */
 /* -------------------------------------------------------------------- */
     GetGDALDriverManager()->AutoSkipDrivers();
