@@ -1,8 +1,8 @@
 /******************************************************************************
- * $Id: gifdataset.cpp 28284 2015-01-03 20:08:36Z rouault $
+ * $Id: gifdataset.cpp 33717 2016-03-14 06:29:14Z goatbar $
  *
  * Project:  GIF Driver
- * Purpose:  Implement GDAL GIF Support using libungif code.  
+ * Purpose:  Implement GDAL GIF Support using libungif code.
  * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
  ******************************************************************************
@@ -28,15 +28,14 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "gdal_pam.h"
 #include "cpl_string.h"
+#include "gdal_frmts.h"
+#include "gdal_pam.h"
 #include "gifabstractdataset.h"
 
-CPL_CVSID("$Id: gifdataset.cpp 28284 2015-01-03 20:08:36Z rouault $");
+CPL_CVSID("$Id: gifdataset.cpp 33717 2016-03-14 06:29:14Z goatbar $");
 
 CPL_C_START
-void	GDALRegister_GIF(void);
-
 #if !(defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5)
 
 // This prototype seems to have been messed up!
@@ -50,10 +49,9 @@ GifFileType * EGifOpen(void* userData, OutputFunc writeFunc);
 
 CPL_C_END
 
-static const int InterlacedOffset[] = { 0, 4, 2, 1 }; 
-static const int InterlacedJumps[] = { 8, 8, 4, 2 };  
+static const int InterlacedOffset[] = { 0, 4, 2, 1 };
+static const int InterlacedJumps[] = { 8, 8, 4, 2 };
 
-static int VSIGIFReadFunc( GifFileType *, GifByteType *, int);
 static int VSIGIFWriteFunc( GifFileType *, const GifByteType *, int );
 
 /************************************************************************/
@@ -78,7 +76,6 @@ class GIFDataset : public GIFAbstractDataset
                                     int bStrict, char ** papszOptions,
                                     GDALProgressFunc pfnProgress,
                                     void * pProgressData );
-
 };
 
 /************************************************************************/
@@ -87,157 +84,32 @@ class GIFDataset : public GIFAbstractDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class GIFRasterBand : public GDALPamRasterBand
+class GIFRasterBand : public GIFAbstractRasterBand
 {
-    friend class GIFDataset;
-
-    SavedImage	*psImage;
-
-    int		*panInterlaceMap;
-    
-    GDALColorTable *poColorTable;
-
-    int		nTransparentColor;
-
   public:
 
                    GIFRasterBand( GIFDataset *, int, SavedImage *, int );
-    virtual       ~GIFRasterBand();
-
     virtual CPLErr IReadBlock( int, int, void * );
-
-    virtual double GetNoDataValue( int *pbSuccess = NULL );
-    virtual GDALColorInterp GetColorInterpretation();
-    virtual GDALColorTable *GetColorTable();
 };
 
 /************************************************************************/
 /*                           GIFRasterBand()                            */
 /************************************************************************/
 
-GIFRasterBand::GIFRasterBand( GIFDataset *poDS, int nBand, 
-                              SavedImage *psSavedImage, int nBackground )
+GIFRasterBand::GIFRasterBand( GIFDataset *poDSIn, int nBandIn,
+                              SavedImage *psSavedImage, int nBackground ) :
+                GIFAbstractRasterBand(poDSIn, nBandIn, psSavedImage, nBackground, FALSE)
 
 {
-    this->poDS = poDS;
-    this->nBand = nBand;
-
-    eDataType = GDT_Byte;
-
-    nBlockXSize = poDS->nRasterXSize;
-    nBlockYSize = 1;
-
-    psImage = psSavedImage;
-
-    poColorTable = NULL;
-    panInterlaceMap = NULL;
-    nTransparentColor = 0;
-
-    if (psImage == NULL)
-        return;
-
-/* -------------------------------------------------------------------- */
-/*      Setup interlacing map if required.                              */
-/* -------------------------------------------------------------------- */
-    panInterlaceMap = NULL;
-    if( psImage->ImageDesc.Interlace )
-    {
-        int	i, j, iLine = 0;
-
-        panInterlaceMap = (int *) CPLCalloc(poDS->nRasterYSize,sizeof(int));
-
-	for (i = 0; i < 4; i++)
-        {
-	    for (j = InterlacedOffset[i]; 
-                 j < poDS->nRasterYSize;
-                 j += InterlacedJumps[i]) 
-                panInterlaceMap[j] = iLine++;
-        }
-    }
-
-/* -------------------------------------------------------------------- */
-/*      Check for transparency.  We just take the first graphic         */
-/*      control extension block we find, if any.                        */
-/* -------------------------------------------------------------------- */
-    int	iExtBlock;
-
-    nTransparentColor = -1;
-    for( iExtBlock = 0; iExtBlock < psImage->ExtensionBlockCount; iExtBlock++ )
-    {
-        unsigned char *pExtData;
-
-        if( psImage->ExtensionBlocks[iExtBlock].Function != 0xf9 ||
-            psImage->ExtensionBlocks[iExtBlock].ByteCount < 4  )
-            continue;
-
-        pExtData = (unsigned char *) psImage->ExtensionBlocks[iExtBlock].Bytes;
-
-        /* check if transparent color flag is set */
-        if( !(pExtData[0] & 0x1) )
-            continue;
-
-        nTransparentColor = pExtData[3];
-    }
-
-/* -------------------------------------------------------------------- */
-/*      Setup colormap.                                                 */
-/* -------------------------------------------------------------------- */
-    ColorMapObject 	*psGifCT = psImage->ImageDesc.ColorMap;
-    if( psGifCT == NULL )
-        psGifCT = poDS->hGifFile->SColorMap;
-
-    poColorTable = new GDALColorTable();
-    for( int iColor = 0; iColor < psGifCT->ColorCount; iColor++ )
-    {
-        GDALColorEntry	oEntry;
-
-        oEntry.c1 = psGifCT->Colors[iColor].Red;
-        oEntry.c2 = psGifCT->Colors[iColor].Green;
-        oEntry.c3 = psGifCT->Colors[iColor].Blue;
-
-        if( iColor == nTransparentColor )
-            oEntry.c4 = 0;
-        else
-            oEntry.c4 = 255;
-
-        poColorTable->SetColorEntry( iColor, &oEntry );
-    }
-
-/* -------------------------------------------------------------------- */
-/*      If we have a background value, return it here.  Some            */
-/*      applications might want to treat this as transparent, but in    */
-/*      many uses this is inappropriate so we don't return it as        */
-/*      nodata or transparent.                                          */
-/* -------------------------------------------------------------------- */
-    if( nBackground != 255 )
-    {
-        char szBackground[10];
-        
-        sprintf( szBackground, "%d", nBackground );
-        SetMetadataItem( "GIF_BACKGROUND", szBackground );
-    }
-}
-
-/************************************************************************/
-/*                           ~GIFRasterBand()                           */
-/************************************************************************/
-
-GIFRasterBand::~GIFRasterBand()
-
-{
-    if( poColorTable != NULL )
-        delete poColorTable;
-
-    CPLFree( panInterlaceMap );
 }
 
 /************************************************************************/
 /*                             IReadBlock()                             */
 /************************************************************************/
 
-CPLErr GIFRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYOff,
+CPLErr GIFRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
+                                  int nBlockYOff,
                                   void * pImage )
-
 {
     CPLAssert( nBlockXOff == 0 );
 
@@ -250,45 +122,12 @@ CPLErr GIFRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYOff,
     if( panInterlaceMap != NULL )
         nBlockYOff = panInterlaceMap[nBlockYOff];
 
-    memcpy( pImage, psImage->RasterBits + nBlockYOff * nBlockXSize, 
+    memcpy( pImage, psImage->RasterBits + nBlockYOff * nBlockXSize,
             nBlockXSize );
 
     return CE_None;
 }
 
-/************************************************************************/
-/*                       GetColorInterpretation()                       */
-/************************************************************************/
-
-GDALColorInterp GIFRasterBand::GetColorInterpretation()
-
-{
-    return GCI_PaletteIndex;
-}
-
-/************************************************************************/
-/*                           GetColorTable()                            */
-/************************************************************************/
-
-GDALColorTable *GIFRasterBand::GetColorTable()
-
-{
-    return poColorTable;
-}
-
-/************************************************************************/
-/*                           GetNoDataValue()                           */
-/************************************************************************/
-
-double GIFRasterBand::GetNoDataValue( int *pbSuccess )
-
-{
-    if( pbSuccess != NULL )
-        *pbSuccess = nTransparentColor != -1;
-
-    return nTransparentColor;
-}
-
 /************************************************************************/
 /* ==================================================================== */
 /*                             GIFDataset                               */
@@ -311,33 +150,29 @@ GIFDataset::GIFDataset()
 GDALDataset *GIFDataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
-    if( !Identify( poOpenInfo ) )
+    if( !Identify( poOpenInfo ) || poOpenInfo->fpL == NULL )
         return NULL;
 
     if( poOpenInfo->eAccess == GA_Update )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The GIF driver does not support update access to existing"
                   " files.\n" );
         return NULL;
     }
 
 /* -------------------------------------------------------------------- */
-/*      Open the file and ingest.                                       */
+/*      Ingest.                                                         */
 /* -------------------------------------------------------------------- */
-    GifFileType 	*hGifFile;
-    VSILFILE                *fp;
-    int                  nGifErr;
+    VSILFILE *fp = poOpenInfo->fpL;
+    poOpenInfo->fpL = NULL;
 
-    fp = VSIFOpenL( poOpenInfo->pszFilename, "r" );
-    if( fp == NULL )
-        return NULL;
-
-    hGifFile = GIFAbstractDataset::myDGifOpen( fp, VSIGIFReadFunc );
+    GifFileType *hGifFile
+        = GIFAbstractDataset::myDGifOpen( fp, GIFAbstractDataset::ReadFunc );
     if( hGifFile == NULL )
     {
         VSIFCloseL( fp );
-        CPLError( CE_Failure, CPLE_OpenFailed, 
+        CPLError( CE_Failure, CPLE_OpenFailed,
                   "DGifOpen() failed for %s.\n"
                   "Perhaps the gif file is corrupt?\n",
                   poOpenInfo->pszFilename );
@@ -351,27 +186,7 @@ GDALDataset *GIFDataset::Open( GDALOpenInfo * poOpenInfo )
     /* -------------------------------------------------------------------- */
     /*      Find the first image record.                                    */
     /* -------------------------------------------------------------------- */
-    GifRecordType RecordType = TERMINATE_RECORD_TYPE;
-
-    while( DGifGetRecordType(hGifFile, &RecordType) != GIF_ERROR
-        && RecordType != TERMINATE_RECORD_TYPE
-        && RecordType != IMAGE_DESC_RECORD_TYPE )
-    {
-        /* Skip extension records found before IMAGE_DESC_RECORD_TYPE */
-        if (RecordType == EXTENSION_RECORD_TYPE)
-        {
-            int nFunction;
-            GifByteType *pExtData;
-            if (DGifGetExtension(hGifFile, &nFunction, &pExtData) == GIF_ERROR)
-                break;
-            while (pExtData != NULL)
-            {
-                if (DGifGetExtensionNext(hGifFile, &pExtData) == GIF_ERROR)
-                    break;
-            }
-        }
-    }
-
+    GifRecordType RecordType = FindFirstImage(hGifFile);
     if( RecordType == IMAGE_DESC_RECORD_TYPE  &&
         DGifGetImageDesc(hGifFile) != GIF_ERROR)
     {
@@ -383,7 +198,9 @@ GDALDataset *GIFDataset::Open( GDALOpenInfo * poOpenInfo )
                       "Due to limitations of the GDAL GIF driver we deliberately avoid\n"
                       "opening large GIF files (larger than 100 megapixels).");
             GIFAbstractDataset::myDGifCloseFile( hGifFile );
-            VSIFCloseL( fp );
+            /* Reset poOpenInfo->fpL since BIGGIF may need it */
+            poOpenInfo->fpL = fp;
+            VSIFSeekL(fp, 0, SEEK_SET);
             return NULL;
         }
     }
@@ -392,11 +209,11 @@ GDALDataset *GIFDataset::Open( GDALOpenInfo * poOpenInfo )
 
     VSIFSeekL( fp, 0, SEEK_SET);
 
-    hGifFile = GIFAbstractDataset::myDGifOpen( fp, VSIGIFReadFunc );
+    hGifFile = GIFAbstractDataset::myDGifOpen( fp, GIFAbstractDataset::ReadFunc );
     if( hGifFile == NULL )
     {
         VSIFCloseL( fp );
-        CPLError( CE_Failure, CPLE_OpenFailed, 
+        CPLError( CE_Failure, CPLE_OpenFailed,
                   "DGifOpen() failed for %s.\n"
                   "Perhaps the gif file is corrupt?\n",
                   poOpenInfo->pszFilename );
@@ -404,7 +221,7 @@ GDALDataset *GIFDataset::Open( GDALOpenInfo * poOpenInfo )
         return NULL;
     }
 
-    nGifErr = DGifSlurp( hGifFile );
+    int nGifErr = DGifSlurp( hGifFile );
 
     if( nGifErr != GIF_OK || hGifFile->SavedImages == NULL )
     {
@@ -421,7 +238,7 @@ GDALDataset *GIFDataset::Open( GDALOpenInfo * poOpenInfo )
             return NULL;
          }
          else
-            CPLError( CE_Failure, CPLE_OpenFailed, 
+            CPLError( CE_Failure, CPLE_OpenFailed,
                         "DGifSlurp() failed for %s.\n"
                         "Perhaps the gif file is corrupt?\n",
                         poOpenInfo->pszFilename );
@@ -432,9 +249,7 @@ GDALDataset *GIFDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
-    GIFDataset 	*poDS;
-
-    poDS = new GIFDataset();
+    GIFDataset *poDS = new GIFDataset();
 
     poDS->fp = fp;
     poDS->eAccess = GA_ReadOnly;
@@ -463,8 +278,11 @@ GDALDataset *GIFDataset::Open( GDALOpenInfo * poOpenInfo )
             CPLDebug("GIF", "Skipping image without color table");
             continue;
         }
-
-        poDS->SetBand( poDS->nBands+1, 
+#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
+        /* Since giflib 5, de-interlacing is done by DGifSlurp() */
+        psImage->ImageDesc.Interlace = 0;
+#endif
+        poDS->SetBand( poDS->nBands+1,
                        new GIFRasterBand( poDS, poDS->nBands+1, psImage,
                                           hGifFile->SBackGroundColor ));
     }
@@ -483,12 +301,13 @@ GDALDataset *GIFDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Initialize any PAM information.                                 */
 /* -------------------------------------------------------------------- */
     poDS->SetDescription( poOpenInfo->pszFilename );
-    poDS->TryLoadXML();
+    poDS->TryLoadXML( poOpenInfo->GetSiblingFiles() );
 
 /* -------------------------------------------------------------------- */
 /*      Support overviews.                                              */
 /* -------------------------------------------------------------------- */
-    poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
+    poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename,
+                                 poOpenInfo->GetSiblingFiles() );
 
     return poDS;
 }
@@ -497,11 +316,7 @@ GDALDataset *GIFDataset::Open( GDALOpenInfo * poOpenInfo )
 /*                        GDALPrintGifError()                           */
 /************************************************************************/
 
-static void GDALPrintGifError(
-#if GIFLIB_MAJOR < 5
-CPL_UNUSED 
-#endif
-GifFileType *hGifFile, const char* pszMsg)
+static void GDALPrintGifError(CPL_UNUSED GifFileType *hGifFile, const char* pszMsg)
 {
 /* GIFLIB_MAJOR is only defined in libgif >= 4.2.0 */
 /* libgif 4.2.0 has retired PrintGifError() and added GifErrorString() */
@@ -529,47 +344,45 @@ GifFileType *hGifFile, const char* pszMsg)
 /************************************************************************/
 
 GDALDataset *
-GIFDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS, 
-               int bStrict, char ** papszOptions, 
+GIFDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
+               int bStrict, char ** papszOptions,
                GDALProgressFunc pfnProgress, void * pProgressData )
 
 {
-    int  nBands = poSrcDS->GetRasterCount();
-    int  nXSize = poSrcDS->GetRasterXSize();
-    int  nYSize = poSrcDS->GetRasterYSize();
-    int	 bInterlace = FALSE;
-
 /* -------------------------------------------------------------------- */
 /*      Check for interlaced option.                                    */
 /* -------------------------------------------------------------------- */
-    bInterlace = CSLFetchBoolean(papszOptions, "INTERLACING", FALSE);
+    int bInterlace = CSLFetchBoolean(papszOptions, "INTERLACING", FALSE);
 
 /* -------------------------------------------------------------------- */
 /*      Some some rudimentary checks                                    */
 /* -------------------------------------------------------------------- */
+    const int nBands = poSrcDS->GetRasterCount();
     if( nBands != 1 )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "GIF driver only supports one band images.\n" );
 
         return NULL;
     }
 
+    const int nXSize = poSrcDS->GetRasterXSize();
+    const int nYSize = poSrcDS->GetRasterYSize();
     if (nXSize > 65535 || nYSize > 65535)
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "GIF driver only supports datasets up to 65535x65535 size.\n" );
 
         return NULL;
     }
 
-    if( poSrcDS->GetRasterBand(1)->GetRasterDataType() != GDT_Byte 
+    if( poSrcDS->GetRasterBand(1)->GetRasterDataType() != GDT_Byte
         && bStrict )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "GIF driver doesn't support data type %s. "
-                  "Only eight bit bands supported.\n", 
-                  GDALGetDataTypeName( 
+                  "Only eight bit bands supported.\n",
+                  GDALGetDataTypeName(
                       poSrcDS->GetRasterBand(1)->GetRasterDataType()) );
 
         return NULL;
@@ -578,18 +391,16 @@ GIFDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 /* -------------------------------------------------------------------- */
 /*      Open the output file.                                           */
 /* -------------------------------------------------------------------- */
-    GifFileType *hGifFile;
-    VSILFILE *fp;
-
-    fp = VSIFOpenL( pszFilename, "wb" );
+    VSILFILE *fp  = VSIFOpenL( pszFilename, "wb" );
     if( fp == NULL )
     {
-        CPLError( CE_Failure, CPLE_OpenFailed, 
-                  "Failed to create %s:\n%s", 
+        CPLError( CE_Failure, CPLE_OpenFailed,
+                  "Failed to create %s:\n%s",
                   pszFilename, VSIStrerror( errno ) );
         return NULL;
     }
 
+    GifFileType *hGifFile;
 #if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
     int nError;
     hGifFile = EGifOpen( fp, VSIGIFWriteFunc, &nError );
@@ -599,7 +410,7 @@ GIFDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
     if( hGifFile == NULL )
     {
         VSIFCloseL( fp );
-        CPLError( CE_Failure, CPLE_OpenFailed, 
+        CPLError( CE_Failure, CPLE_OpenFailed,
                   "EGifOpenFilename(%s) failed.  Does file already exist?",
                   pszFilename );
 
@@ -611,12 +422,19 @@ GIFDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 /* -------------------------------------------------------------------- */
     GDALRasterBand	*poBand = poSrcDS->GetRasterBand(1);
     ColorMapObject	*psGifCT;
-    int			iColor;
 
     if( poBand->GetColorTable() == NULL )
     {
         psGifCT = GifMakeMapObject( 256, NULL );
-        for( iColor = 0; iColor < 256; iColor++ )
+        if( psGifCT == NULL )
+        {
+            CPLError(CE_Failure, CPLE_AppDefined,
+                     "Cannot allocate color table");
+            GIFAbstractDataset::myEGifCloseFile(hGifFile);
+            VSIFCloseL( fp );
+            return NULL;
+        }
+        for( int iColor = 0; iColor < 256; iColor++ )
         {
             psGifCT->Colors[iColor].Red = (GifByteType) iColor;
             psGifCT->Colors[iColor].Green = (GifByteType) iColor;
@@ -626,13 +444,22 @@ GIFDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
     else
     {
         GDALColorTable	*poCT = poBand->GetColorTable();
-        int nFullCount = 1;
+        int nFullCount = 2;
 
         while( nFullCount < poCT->GetColorEntryCount() )
             nFullCount = nFullCount * 2;
 
         psGifCT = GifMakeMapObject( nFullCount, NULL );
-        for( iColor = 0; iColor < poCT->GetColorEntryCount(); iColor++ )
+        if( psGifCT == NULL )
+        {
+            CPLError(CE_Failure, CPLE_AppDefined,
+                     "Cannot allocate color table");
+            GIFAbstractDataset::myEGifCloseFile(hGifFile);
+            VSIFCloseL( fp );
+            return NULL;
+        }
+        int iColor = 0;
+        for( ; iColor < poCT->GetColorEntryCount(); iColor++ )
         {
             GDALColorEntry	sEntry;
 
@@ -652,8 +479,10 @@ GIFDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 /* -------------------------------------------------------------------- */
 /*      Setup parameters.                                               */
 /* -------------------------------------------------------------------- */
-    if (EGifPutScreenDesc(hGifFile, nXSize, nYSize, 
-                          psGifCT->ColorCount, 255, psGifCT) == GIF_ERROR)
+    if (EGifPutScreenDesc(hGifFile, nXSize, nYSize,
+                          8, /* ColorRes */
+                          255, /* Background */
+                          psGifCT) == GIF_ERROR)
     {
         GifFreeMapObject(psGifCT);
         GDALPrintGifError(hGifFile, "Error writing gif file.");
@@ -661,7 +490,7 @@ GIFDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
         VSIFCloseL( fp );
         return NULL;
     }
-    
+
     GifFreeMapObject(psGifCT);
     psGifCT = NULL;
 
@@ -689,26 +518,26 @@ GIFDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 /* -------------------------------------------------------------------- */
 /*      Loop over image, copying image data.                            */
 /* -------------------------------------------------------------------- */
-    CPLErr      eErr;
-    GDALPamDataset *poDS;
-    GByte      *pabyScanline;
-
-    pabyScanline = (GByte *) CPLMalloc( nXSize );
+    GDALPamDataset *poDS = NULL;
+    GByte *pabyScanline = (GByte *) CPLMalloc( nXSize );
 
     if( !pfnProgress( 0.0, NULL, pProgressData ) )
-        eErr = CE_Failure;
+    {
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "Unable to setup progress." );
+    }
 
     if( !bInterlace )
     {
         for( int iLine = 0; iLine < nYSize; iLine++ )
         {
-            eErr = poBand->RasterIO( GF_Read, 0, iLine, nXSize, 1, 
-                                     pabyScanline, nXSize, 1, GDT_Byte,
-                                     nBands, nBands * nXSize );
+            const CPLErr eErr = poBand->RasterIO( GF_Read, 0, iLine, nXSize, 1,
+                                                  pabyScanline, nXSize, 1, GDT_Byte,
+                                                  nBands, nBands * nXSize, NULL );
 
             if( eErr != CE_None || EGifPutLine( hGifFile, pabyScanline, nXSize ) == GIF_ERROR )
             {
-                CPLError( CE_Failure, CPLE_AppDefined, 
+                CPLError( CE_Failure, CPLE_AppDefined,
                           "Error writing gif file." );
                 goto error;
             }
@@ -722,29 +551,28 @@ GIFDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
     }
     else
     {
-        int 	i, j;
-        int nLinesRead = 0;
         int nLinesToRead = 0;
-        for ( i = 0; i < 4; i++)
+        for ( int i = 0; i < 4; i++)
         {
-            for (j = InterlacedOffset[i]; j < nYSize; j += InterlacedJumps[i]) 
+            for ( int j = InterlacedOffset[i]; j < nYSize; j += InterlacedJumps[i] )
             {
                 nLinesToRead ++;
             }
         }
 
+        int nLinesRead = 0;
         /* Need to perform 4 passes on the images: */
-        for ( i = 0; i < 4; i++)
+        for ( int i = 0; i < 4; i++)
         {
-            for (j = InterlacedOffset[i]; j < nYSize; j += InterlacedJumps[i]) 
+            for ( int j = InterlacedOffset[i]; j < nYSize; j += InterlacedJumps[i] )
             {
-                eErr= poBand->RasterIO( GF_Read, 0, j, nXSize, 1, 
+                const CPLErr eErr= poBand->RasterIO( GF_Read, 0, j, nXSize, 1,
                                         pabyScanline, nXSize, 1, GDT_Byte,
-                                        1, nXSize );
+                                        1, nXSize, NULL );
 
                 if (eErr != CE_None || EGifPutLine(hGifFile, pabyScanline, nXSize) == GIF_ERROR)
                 {
-                    CPLError( CE_Failure, CPLE_AppDefined, 
+                    CPLError( CE_Failure, CPLE_AppDefined,
                             "Error writing gif file." );
                     goto error;
                 }
@@ -766,7 +594,7 @@ GIFDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 /* -------------------------------------------------------------------- */
     if (GIFAbstractDataset::myEGifCloseFile(hGifFile) == GIF_ERROR)
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "EGifCloseFile() failed.\n" );
         hGifFile = NULL;
         goto error;
@@ -781,17 +609,17 @@ GIFDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 /* -------------------------------------------------------------------- */
     if( CSLFetchBoolean( papszOptions, "WORLDFILE", FALSE ) )
     {
-    	double      adfGeoTransform[6];
-	
-	if( poSrcDS->GetGeoTransform( adfGeoTransform ) == CE_None )
+        double      adfGeoTransform[6];
+
+        if( poSrcDS->GetGeoTransform( adfGeoTransform ) == CE_None )
             GDALWriteWorldFile( pszFilename, "wld", adfGeoTransform );
     }
 
 /* -------------------------------------------------------------------- */
-/*      Re-open dataset, and copy any auxilary pam information.         */
+/*      Re-open dataset, and copy any auxiliary pam information.         */
 /* -------------------------------------------------------------------- */
 
-    /* If outputing to stdout, we can't reopen it, so we'll return */
+    /* If writing to stdout, we can't reopen it, so return */
     /* a fake dataset to make the caller happy */
     CPLPushErrorHandler(CPLQuietErrorHandler);
     poDS = (GDALPamDataset*) GDALOpen(pszFilename, GA_ReadOnly);
@@ -823,27 +651,13 @@ error:
     return NULL;
 }
 
-/************************************************************************/
-/*                           VSIGIFReadFunc()                           */
-/*                                                                      */
-/*      Proxy function for reading from GIF file.                       */
-/************************************************************************/
-
-static int VSIGIFReadFunc( GifFileType *psGFile, GifByteType *pabyBuffer, 
-                           int nBytesToRead )
-
-{
-    return VSIFReadL( pabyBuffer, 1, nBytesToRead, 
-                      (VSILFILE *) psGFile->UserData );
-}
-
 /************************************************************************/
 /*                          VSIGIFWriteFunc()                           */
 /*                                                                      */
 /*      Proxy write function.                                           */
 /************************************************************************/
 
-static int VSIGIFWriteFunc( GifFileType *psGFile, 
+static int VSIGIFWriteFunc( GifFileType *psGFile,
                             const GifByteType *pabyBuffer, int nBytesToWrite )
 
 {
@@ -860,50 +674,46 @@ static int VSIGIFWriteFunc( GifFileType *psGFile,
         /* Most readers don't even care if it is GIF87a or GIF89a, but it is */
         /* better to write the right version */
 
-        int nRet = VSIFWriteL("GIF89a", 1, 6, fp);
+        size_t nRet = VSIFWriteL("GIF89a", 1, 6, fp);
         nRet += VSIFWriteL( (char *) pabyBuffer + 6, 1, nBytesToWrite - 6, fp );
-        return nRet;
+        return static_cast<int>(nRet);
     }
     else
-        return VSIFWriteL( (void *) pabyBuffer, 1, nBytesToWrite, fp );
+        return static_cast<int>(VSIFWriteL( (void *) pabyBuffer, 1, nBytesToWrite, fp ));
 }
 
 /************************************************************************/
-/*                          GDALRegister_GIF()                        */
+/*                          GDALRegister_GIF()                          */
 /************************************************************************/
 
 void GDALRegister_GIF()
 
 {
-    GDALDriver	*poDriver;
+    if( GDALGetDriverByName( "GIF" ) != NULL )
+        return;
 
-    if( GDALGetDriverByName( "GIF" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "GIF" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "Graphics Interchange Format (.gif)" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_gif.html" );
-        poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "gif" );
-        poDriver->SetMetadataItem( GDAL_DMD_MIMETYPE, "image/gif" );
-        poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, 
-                                   "Byte" );
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->SetMetadataItem( GDAL_DMD_CREATIONOPTIONLIST, 
+    poDriver->SetDescription( "GIF" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
+                               "Graphics Interchange Format (.gif)" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_gif.html" );
+    poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "gif" );
+    poDriver->SetMetadataItem( GDAL_DMD_MIMETYPE, "image/gif" );
+    poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, "Byte" );
+
+    poDriver->SetMetadataItem( GDAL_DMD_CREATIONOPTIONLIST,
 "<CreationOptionList>\n"
 "   <Option name='INTERLACING' type='boolean'/>\n"
 "   <Option name='WORLDFILE' type='boolean'/>\n"
 "</CreationOptionList>\n" );
 
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
-        poDriver->pfnOpen = GIFDataset::Open;
-        poDriver->pfnCreateCopy = GIFDataset::CreateCopy;
-        poDriver->pfnIdentify = GIFAbstractDataset::Identify;
+    poDriver->pfnOpen = GIFDataset::Open;
+    poDriver->pfnCreateCopy = GIFDataset::CreateCopy;
+    poDriver->pfnIdentify = GIFAbstractDataset::Identify;
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
-
