@@ -1,10 +1,10 @@
 /******************************************************************************
- * $Id: grass57dataset.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: grass57dataset.cpp 33893 2016-04-04 14:08:28Z rouault $
  *
  * Project:  GRASS Driver
  * Purpose:  Implement GRASS raster read/write support
  *           This version is for GRASS 5.7+ and uses GRASS libraries
- *           directly instead of using libgrass. 
+ *           directly instead of using libgrass.
  * Author:   Frank Warmerdam <warmerdam@pobox.com>
  *           Radim Blazek <blazek@itc.it>
  *
@@ -33,6 +33,11 @@
 
 #include <stdlib.h>
 
+#include "cpl_string.h"
+#include "gdal_frmts.h"
+#include "gdal_priv.h"
+#include "ogr_spatialref.h"
+
 extern "C" {
 #ifdef __cplusplus
 #define class _class
@@ -41,7 +46,7 @@ extern "C" {
 #ifdef __cplusplus
 #undef class
 #endif
-    
+
 #include <grass/version.h>
 #include <grass/gprojects.h>
 #include <grass/gis.h>
@@ -57,17 +62,9 @@ char *GPJ_grass_to_wkt(struct Key_Value *,
 #endif
 }
 
-#include "gdal_priv.h"
-#include "cpl_string.h"
-#include "ogr_spatialref.h"
-
 #define GRASS_MAX_COLORS 100000  // what is the right value
 
-CPL_CVSID("$Id: grass57dataset.cpp 27044 2014-03-16 23:41:27Z rouault $");
-
-CPL_C_START
-void	GDALRegister_GRASS(void);
-CPL_C_END
+CPL_CVSID("$Id: grass57dataset.cpp 33893 2016-04-04 14:08:28Z rouault $");
 
 #if GRASS_VERSION_MAJOR  >= 7
 #define G_get_cellhd             Rast_get_cellhd
@@ -92,6 +89,7 @@ CPL_C_END
 #define G_get_d_raster_row       Rast_get_d_row
 #define G_allocate_f_raster_buf  Rast_allocate_f_buf
 #define G_allocate_d_raster_buf  Rast_allocate_d_buf
+#define G__setenv                G_setenv_nogisrc
 #endif
 
 /************************************************************************/
@@ -122,12 +120,12 @@ class GRASSDataset : public GDALDataset
 {
     friend class GRASSRasterBand;
 
-    char	*pszGisdbase;  
+    char	*pszGisdbase;
     char	*pszLocation;  /* LOCATION_NAME */
     char	*pszElement;   /* cellhd or group */
 
-    struct Cell_head sCellInfo; /* raster region */ 
-    
+    struct Cell_head sCellInfo; /* raster region */
+
     char	*pszProjection;
 
     double	adfGeoTransform[6];
@@ -164,7 +162,7 @@ class GRASSRasterBand : public GDALRasterBand
     struct Colors sGrassColors;
     GDALColorTable *poCT;
 
-    struct Cell_head sOpenWindow; /* the region when the raster was opened */ 
+    struct Cell_head sOpenWindow; /* the region when the raster was opened */
 
     int		bHaveMinMax;
     double	dfCellMin;
@@ -176,12 +174,15 @@ class GRASSRasterBand : public GDALRasterBand
 
   public:
 
-                   GRASSRasterBand( GRASSDataset *, int, 
+                   GRASSRasterBand( GRASSDataset *, int,
                                     const char *, const char * );
     virtual        ~GRASSRasterBand();
 
     virtual CPLErr IReadBlock( int, int, void * );
-    virtual CPLErr IRasterIO ( GDALRWFlag, int, int, int, int, void *, int, int, GDALDataType, int, int );
+    virtual CPLErr IRasterIO ( GDALRWFlag, int, int, int, int, void *, int, int, GDALDataType,
+                               GSpacing nPixelSpace,
+                               GSpacing nLineSpace,
+                               GDALRasterIOExtraArg* psExtraArg);
     virtual GDALColorInterp GetColorInterpretation();
     virtual GDALColorTable *GetColorTable();
     virtual double GetMinimum( int *pbSuccess = NULL );
@@ -190,7 +191,7 @@ class GRASSRasterBand : public GDALRasterBand
 
   private:
     CPLErr ResetReading( struct Cell_head * );
-    
+
 };
 
 
@@ -222,7 +223,7 @@ GRASSRasterBand::GRASSRasterBand( GRASSDataset *poDS, int nBand,
 /* -------------------------------------------------------------------- */
     struct FPRange sRange;
 
-    if( G_read_fp_range( (char *) pszCellName, (char *) pszMapset, 
+    if( G_read_fp_range( (char *) pszCellName, (char *) pszMapset,
                          &sRange ) == -1 )
     {
         bHaveMinMax = FALSE;
@@ -236,12 +237,12 @@ GRASSRasterBand::GRASSRasterBand( GRASSDataset *poDS, int nBand,
 /* -------------------------------------------------------------------- */
 /*      Setup band type, and preferred nodata value.                    */
 /* -------------------------------------------------------------------- */
-    // Negative values are also (?) stored as 4 bytes (format = 3) 
+    // Negative values are also (?) stored as 4 bytes (format = 3)
     //       => raster with format < 3 has only positive values
 
-    // GRASS modules usually do not waste space and only the format necessary to keep 
+    // GRASS modules usually do not waste space and only the format necessary to keep
     // full raster values range is used -> no checks if shorter type could be used
-    
+
     if( nGRSType == CELL_TYPE ) {
 	if ( sCellInfo.format == 0 ) {  // 1 byte / cell -> possible range 0,255
 	    if ( bHaveMinMax && dfCellMin > 0 ) {
@@ -264,20 +265,20 @@ GRASSRasterBand::GRASSRasterBand( GRASSDataset *poDS, int nBand,
 		dfNoData = 65535;
 	    } else { // maximum is not known or full range is used
 		CELL cval;
-		this->eDataType = GDT_Int32; 
+		this->eDataType = GDT_Int32;
 		G_set_c_null_value ( &cval, 1);
 		dfNoData = (double) cval;
 		nativeNulls = true;
 	    }
 	    nativeNulls = false;
-	} else {  // 3-4 bytes 
+	} else {  // 3-4 bytes
 	    CELL cval;
 	    this->eDataType = GDT_Int32;
 	    G_set_c_null_value ( &cval, 1);
 	    dfNoData = (double) cval;
 	    nativeNulls = true;
 	}
-    } 
+    }
     else if( nGRSType == FCELL_TYPE ) {
 	FCELL fval;
         this->eDataType = GDT_Float32;
@@ -310,7 +311,7 @@ GRASSRasterBand::GRASSRasterBand( GRASSDataset *poDS, int nBand,
     poCT = NULL;
     if( G_read_colors( (char *) pszCellName, (char *) pszMapset, &sGrassColors ) == 1 )
     {
-	int maxcolor; 
+	int maxcolor;
 	CELL min, max;
 
 	G_get_color_range ( &min, &max, &sGrassColors);
@@ -321,7 +322,7 @@ GRASSRasterBand::GRASSRasterBand( GRASSDataset *poDS, int nBand,
             } else {
 	       maxcolor = (int) ceil ( dfCellMax );
 	    }
-	    if ( maxcolor > GRASS_MAX_COLORS ) { 
+	    if ( maxcolor > GRASS_MAX_COLORS ) {
 		maxcolor = GRASS_MAX_COLORS;
                 CPLDebug( "GRASS", "Too many values, color table cut to %d entries.", maxcolor );
 	    }
@@ -333,7 +334,7 @@ GRASSRasterBand::GRASSRasterBand( GRASSDataset *poDS, int nBand,
                CPLDebug( "GRASS", "Too many values, color table set to %d entries.", maxcolor );
 	    }
         }
-	    
+
         poCT = new GDALColorTable();
         for( int iColor = 0; iColor <= maxcolor; iColor++ )
         {
@@ -363,30 +364,29 @@ GRASSRasterBand::GRASSRasterBand( GRASSDataset *poDS, int nBand,
                 poCT->SetColorEntry( iColor, &sColor );
             }
         }
-	    
-	/* Create metadata enries for color table rules */
-	char key[200], value[200];
-	int rcount = G_colors_count ( &sGrassColors );
 
-	sprintf ( value, "%d", rcount );
-	this->SetMetadataItem( "COLOR_TABLE_RULES_COUNT", value );
+        /* Create metadata entries for color table rules */
+        char key[200], value[200];
+        int rcount = G_colors_count ( &sGrassColors );
 
-	/* Add the rules in reverse order */
-	for ( int i = rcount-1; i >= 0; i-- ) {
-	    DCELL val1, val2;
-	    unsigned char r1, g1, b1, r2, g2, b2;
+        snprintf ( value, sizeof(value), "%d", rcount );
+        this->SetMetadataItem( "COLOR_TABLE_RULES_COUNT", value );
 
-	     G_get_f_color_rule ( &val1, &r1, &g1, &b1, &val2, &r2, &g2, &b2, &sGrassColors, i );
-		
+        /* Add the rules in reverse order */
+        for ( int i = rcount-1; i >= 0; i-- ) {
+            DCELL val1, val2;
+            unsigned char r1, g1, b1, r2, g2, b2;
 
-	     sprintf ( key, "COLOR_TABLE_RULE_RGB_%d", rcount-i-1 );
-	     sprintf ( value, "%e %e %d %d %d %d %d %d", val1, val2, r1, g1, b1, r2, g2, b2 );
-	     this->SetMetadataItem( key, value );
-	}
+            G_get_f_color_rule ( &val1, &r1, &g1, &b1, &val2, &r2, &g2, &b2, &sGrassColors, i );
+
+            snprintf ( key, sizeof(key), "COLOR_TABLE_RULE_RGB_%d", rcount-i-1 );
+            snprintf ( value, sizeof(value), "%e %e %d %d %d %d %d %d", val1, val2, r1, g1, b1, r2, g2, b2 );
+            this->SetMetadataItem( key, value );
+        }
     } else {
-	this->SetMetadataItem( "COLOR_TABLE_RULES_COUNT", "0" );
+        this->SetMetadataItem( "COLOR_TABLE_RULES_COUNT", "0" );
     }
-    
+
     this->valid = true;
 }
 
@@ -403,7 +403,7 @@ GRASSRasterBand::~GRASSRasterBand()
 
     if( hCell >= 0 )
         G_close_cell( hCell );
-    
+
     if ( pszCellName )
         G_free ( pszCellName );
 
@@ -439,10 +439,10 @@ CPLErr GRASSRasterBand::ResetReading ( struct Cell_head *sNewWindow )
 	/* Open raster */
 	G__setenv( "GISDBASE", ((GRASSDataset *)poDS)->pszGisdbase );
 	G__setenv( "LOCATION_NAME", ((GRASSDataset *)poDS)->pszLocation );
-	G__setenv( "MAPSET", pszMapset); 
+	G__setenv( "MAPSET", pszMapset);
 	G_reset_mapsets();
 	G_add_mapset_to_search_path ( pszMapset );
-	
+
 	if ( (hCell = G_open_cell_old( pszCellName, pszMapset)) < 0 ) {
 	    CPLError( CE_Warning, CPLE_AppDefined, "GRASS: Cannot open raster '%s'", pszCellName );
             this->valid = false;
@@ -450,7 +450,7 @@ CPLErr GRASSRasterBand::ResetReading ( struct Cell_head *sNewWindow )
 	}
 
 	G_copy((void *) &sOpenWindow, (void *) sNewWindow, sizeof(struct Cell_head));
-	
+
     }
     else
     {
@@ -479,21 +479,22 @@ CPLErr GRASSRasterBand::ResetReading ( struct Cell_head *sNewWindow )
 /*                                                                      */
 /************************************************************************/
 
-CPLErr GRASSRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff, void * pImage )
+CPLErr GRASSRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
+                                    void *pImage )
 
 {
     if ( ! this->valid ) return CE_Failure;
 
-    // Reset window because IRasterIO could be previosly called
+    // Reset window because IRasterIO could be previously called.
     if ( ResetReading ( &(((GRASSDataset *)poDS)->sCellInfo) ) != CE_None ) {
        return CE_Failure;
-    }       
-    
+    }
+
     if ( eDataType == GDT_Byte || eDataType == GDT_UInt16 ) {
         CELL  *cbuf;
 
 	cbuf = G_allocate_c_raster_buf();
-	G_get_c_raster_row ( hCell, cbuf, nBlockYOff );	
+	G_get_c_raster_row ( hCell, cbuf, nBlockYOff );
 
 	/* Reset NULLs */
 	for ( int col = 0; col < nBlockXSize; col++ ) {
@@ -501,9 +502,9 @@ CPLErr GRASSRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff, void * pImag
 		cbuf[col] = (CELL) dfNoData;
 	}
 
-	GDALCopyWords ( (void *) cbuf, GDT_Int32, sizeof(CELL), 
+	GDALCopyWords ( (void *) cbuf, GDT_Int32, sizeof(CELL),
 	                pImage, eDataType, GDALGetDataTypeSize(eDataType)/8,
-			nBlockXSize );    
+			nBlockXSize );
 
 	G_free ( cbuf );
 
@@ -514,7 +515,7 @@ CPLErr GRASSRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff, void * pImag
     } else if ( eDataType == GDT_Float64 ) {
 	G_get_d_raster_row ( hCell, (DCELL *) pImage, nBlockYOff );
     }
-	
+
     return CE_None;
 }
 
@@ -527,29 +528,31 @@ CPLErr GRASSRasterBand::IRasterIO ( GDALRWFlag eRWFlag,
 	                           int nXOff, int nYOff, int nXSize, int nYSize,
 				   void * pData, int nBufXSize, int nBufYSize,
 				   GDALDataType eBufType,
-				   int nPixelSpace, int nLineSpace )
+				   GSpacing nPixelSpace,
+                   GSpacing nLineSpace,
+                   GDALRasterIOExtraArg* psExtraArg )
 {
     /* GRASS library does that, we have only calculate and reset the region in map units
      * and if the region has changed, reopen the raster */
-    
+
     /* Calculate the region */
     struct Cell_head sWindow;
     struct Cell_head *psDsWindow;
-    
+
     if ( ! this->valid ) return CE_Failure;
 
     psDsWindow = &(((GRASSDataset *)poDS)->sCellInfo);
-    
-    sWindow.north = psDsWindow->north - nYOff * psDsWindow->ns_res; 
-    sWindow.south = sWindow.north - nYSize * psDsWindow->ns_res; 
-    sWindow.west = psDsWindow->west + nXOff * psDsWindow->ew_res; 
-    sWindow.east = sWindow.west + nXSize * psDsWindow->ew_res; 
+
+    sWindow.north = psDsWindow->north - nYOff * psDsWindow->ns_res;
+    sWindow.south = sWindow.north - nYSize * psDsWindow->ns_res;
+    sWindow.west = psDsWindow->west + nXOff * psDsWindow->ew_res;
+    sWindow.east = sWindow.west + nXSize * psDsWindow->ew_res;
     sWindow.proj = psDsWindow->proj;
     sWindow.zone = psDsWindow->zone;
 
     sWindow.cols = nBufXSize;
     sWindow.rows = nBufYSize;
-     
+
     /* Reset resolution */
     G_adjust_Cell_head ( &sWindow, 1, 1);
 
@@ -557,7 +560,7 @@ CPLErr GRASSRasterBand::IRasterIO ( GDALRWFlag eRWFlag,
     {
         return CE_Failure;
     }
-    
+
     /* Read Data */
     CELL  *cbuf = NULL;
     FCELL *fbuf = NULL;
@@ -572,7 +575,7 @@ CPLErr GRASSRasterBand::IRasterIO ( GDALRWFlag eRWFlag,
 	nLineSpace = nBufXSize * nPixelSpace;
 
     if ( nGRSType == CELL_TYPE && ( !nativeNulls || eBufType != GDT_Int32 || sizeof(CELL) != 4 ||
-		                    nPixelSpace != sizeof(CELL) )  ) 
+		                    nPixelSpace != sizeof(CELL) )  )
     {
 	cbuf = G_allocate_c_raster_buf();
     } else if( nGRSType == FCELL_TYPE && ( eBufType != GDT_Float32 || nPixelSpace != sizeof(FCELL) ) ) {
@@ -585,42 +588,42 @@ CPLErr GRASSRasterBand::IRasterIO ( GDALRWFlag eRWFlag,
 
     for ( int row = 0; row < nBufYSize; row++ ) {
         char *pnt = (char *)pData + row * nLineSpace;
-	
+
 	if ( nGRSType == CELL_TYPE ) {
 	    if ( direct ) {
 		G_get_c_raster_row ( hCell, (CELL *) pnt, row );
 	    } else {
 		G_get_c_raster_row ( hCell, cbuf, row );
-		
+
 		/* Reset NULLs */
 		for ( int col = 0; col < nBufXSize; col++ ) {
-		    if ( G_is_c_null_value(&(cbuf[col])) ) 
+		    if ( G_is_c_null_value(&(cbuf[col])) )
 			cbuf[col] = (CELL) dfNoData;
 		}
 
-		GDALCopyWords ( (void *) cbuf, GDT_Int32, sizeof(CELL), 
+		GDALCopyWords ( (void *) cbuf, GDT_Int32, sizeof(CELL),
 			        (void *)  pnt,  eBufType, nPixelSpace,
-				nBufXSize ); 
+				nBufXSize );
 	    }
 	} else if( nGRSType == FCELL_TYPE ) {
 	    if ( direct ) {
 		G_get_f_raster_row ( hCell, (FCELL *) pnt, row );
 	    } else {
 		G_get_f_raster_row ( hCell, fbuf, row );
-		
-		GDALCopyWords ( (void *) fbuf, GDT_Float32, sizeof(FCELL), 
+
+		GDALCopyWords ( (void *) fbuf, GDT_Float32, sizeof(FCELL),
 			        (void *)  pnt,  eBufType, nPixelSpace,
-				nBufXSize ); 
+				nBufXSize );
 	    }
 	} else if( nGRSType == DCELL_TYPE ) {
 	    if ( direct ) {
 		G_get_d_raster_row ( hCell, (DCELL *) pnt, row );
 	    } else {
 		G_get_d_raster_row ( hCell, dbuf, row );
-		
-		GDALCopyWords ( (void *) dbuf, GDT_Float64, sizeof(DCELL), 
+
+		GDALCopyWords ( (void *) dbuf, GDT_Float64, sizeof(DCELL),
 			        (void *)  pnt,  eBufType, nPixelSpace,
-				nBufXSize ); 
+				nBufXSize );
 	    }
 	}
     }
@@ -628,7 +631,7 @@ CPLErr GRASSRasterBand::IRasterIO ( GDALRWFlag eRWFlag,
     if ( cbuf ) G_free ( cbuf );
     if ( fbuf ) G_free ( fbuf );
     if ( dbuf ) G_free ( dbuf );
-    
+
     return CE_None;
 }
 
@@ -693,7 +696,7 @@ double GRASSRasterBand::GetMaximum( int *pbSuccess )
         return 4294967295.0;
     else if( eDataType == GDT_UInt16 )
         return 65535;
-    else 
+    else
         return 255;
 }
 
@@ -739,13 +742,13 @@ GRASSDataset::GRASSDataset()
 
 GRASSDataset::~GRASSDataset()
 {
-    
+
     if ( pszGisdbase )
 	G_free ( pszGisdbase );
-    
+
     if ( pszLocation )
         G_free ( pszLocation );
-    
+
     if ( pszElement )
 	G_free ( pszElement );
 
@@ -756,7 +759,7 @@ GRASSDataset::~GRASSDataset()
 /*                          GetProjectionRef()                          */
 /************************************************************************/
 
-const char *GRASSDataset::GetProjectionRef() 
+const char *GRASSDataset::GetProjectionRef()
 {
     if( pszProjection == NULL )
         return "";
@@ -768,10 +771,10 @@ const char *GRASSDataset::GetProjectionRef()
 /*                          GetGeoTransform()                           */
 /************************************************************************/
 
-CPLErr GRASSDataset::GetGeoTransform( double * padfGeoTransform ) 
+CPLErr GRASSDataset::GetGeoTransform( double * padfGeoTransform )
 {
     memcpy( padfGeoTransform, adfGeoTransform, sizeof(double) * 6 );
-    
+
     return CE_None;
 }
 
@@ -784,22 +787,22 @@ CPLErr GRASSDataset::GetGeoTransform( double * padfGeoTransform )
 /* Returns: true - OK                                                   */
 /*          false - failed                                              */
 /************************************************************************/
-bool GRASSDataset::SplitPath( char *path, char **gisdbase, char **location, 
+bool GRASSDataset::SplitPath( char *path, char **gisdbase, char **location,
 	                      char **mapset, char **element, char **name )
 {
     char *p, *ptr[5], *tmp;
     int  i = 0;
-    
+
     *gisdbase = *location = *mapset = *element = *name = NULL;
-    
-    if ( !path || strlen(path) == 0 ) 
+
+    if ( !path || strlen(path) == 0 )
 	return false;
 
     tmp = G_store ( path );
 
     while ( (p = strrchr(tmp,'/')) != NULL  && i < 4 ) {
 	*p = '\0';
-	
+
 	if ( strlen(p+1) == 0 ) /* repeated '/' */
 	    continue;
 
@@ -847,7 +850,7 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
         && strstr(poOpenInfo->pszFilename,"/group/") == NULL )
         return NULL;
 
-    /* Always init, if no rasters are opened G_no_gisinit resets the projection and 
+    /* Always init, if no rasters are opened G_no_gisinit resets the projection and
      * rasters in different projection may be then opened */
 
     // Don't use GISRC file and read/write GRASS variables (from location G_VAR_GISRC) to memory only.
@@ -858,14 +861,14 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 
     // Set error function
     G_set_error_routine ( (GrassErrorHandler) Grass2CPLErrorHook );
-    
+
 
     // GISBASE is path to the directory where GRASS is installed,
     if ( !getenv( "GISBASE" ) ) {
         static char* gisbaseEnv = NULL;
         const char *gisbase = GRASS_GISBASE;
         CPLError( CE_Warning, CPLE_AppDefined, "GRASS warning: GISBASE "
-                "enviroment variable was not set, using:\n%s", gisbase );
+                "environment variable was not set, using:\n%s", gisbase );
         char buf[2000];
         snprintf ( buf, sizeof(buf), "GISBASE=%s", gisbase );
         buf[sizeof(buf)-1] = '\0';
@@ -883,22 +886,22 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Check element name                                              */
 /* -------------------------------------------------------------------- */
-    if ( strcmp(pszElem,"cellhd") != 0 && strcmp(pszElem,"group") != 0 ) { 
-	G_free(pszGisdb); 
-        G_free(pszLoc); 
-        G_free(pszMapset); 
-        G_free(pszElem); 
+    if ( strcmp(pszElem,"cellhd") != 0 && strcmp(pszElem,"group") != 0 ) {
+	G_free(pszGisdb);
+        G_free(pszLoc);
+        G_free(pszMapset);
+        G_free(pszElem);
         G_free(pszName);
 	return NULL;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Set GRASS variables                                             */
 /* -------------------------------------------------------------------- */
 
     G__setenv( "GISDBASE", pszGisdb );
     G__setenv( "LOCATION_NAME", pszLoc );
-    G__setenv( "MAPSET", pszMapset); // group is searched only in current mapset 
+    G__setenv( "MAPSET", pszMapset); // group is searched only in current mapset
     G_reset_mapsets();
     G_add_mapset_to_search_path ( pszMapset );
 
@@ -906,7 +909,7 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Check if this is a valid grass cell.                            */
 /* -------------------------------------------------------------------- */
     if ( strcmp(pszElem,"cellhd") == 0 ) {
-	
+
         if ( G_find_file2("cell", pszName, pszMapset) == NULL ) {
 	    G_free(pszGisdb); G_free(pszLoc); G_free(pszMapset); G_free(pszElem); G_free(pszName);
 	    return NULL;
@@ -926,8 +929,8 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 	    G_free(pszGisdb); G_free(pszLoc); G_free(pszMapset); G_free(pszElem); G_free(pszName);
 	    return NULL;
 	}
-        
-        for( int iRef = 0; iRef < ref.nfiles; iRef++ ) 
+
+        for( int iRef = 0; iRef < ref.nfiles; iRef++ )
 	{
             papszCells = CSLAddString( papszCells, ref.file[iRef].name );
             papszMapsets = CSLAddString( papszMapsets, ref.file[iRef].mapset );
@@ -936,7 +939,7 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 
         I_free_group_ref( &ref );
     }
-    
+
     G_free( pszMapset );
     G_free( pszName );
 
@@ -953,7 +956,7 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->pszGisdbase = pszGisdb;
     poDS->pszLocation = pszLoc;
     poDS->pszElement = pszElem;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Capture some information from the file that is of interest.     */
 /* -------------------------------------------------------------------- */
@@ -977,7 +980,7 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->adfGeoTransform[3] = poDS->sCellInfo.north;
     poDS->adfGeoTransform[4] = 0.0;
     poDS->adfGeoTransform[5] = -1 * poDS->sCellInfo.ns_res;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Try to get a projection definition.                             */
 /* -------------------------------------------------------------------- */
@@ -994,7 +997,7 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     for( int iBand = 0; papszCells[iBand] != NULL; iBand++ )
     {
-	GRASSRasterBand *rb = new GRASSRasterBand( poDS, iBand+1, papszMapsets[iBand], 
+	GRASSRasterBand *rb = new GRASSRasterBand( poDS, iBand+1, papszMapsets[iBand],
                                                                   papszCells[iBand] );
 
 	if ( !rb->valid ) {
@@ -1002,26 +1005,26 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 	    delete rb;
 	    delete poDS;
 	    return NULL;
-	}
+        }
 
         poDS->SetBand( iBand+1, rb );
     }
 
     CSLDestroy(papszCells);
     CSLDestroy(papszMapsets);
-    
+
 /* -------------------------------------------------------------------- */
 /*      Confirm the requested access is supported.                      */
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->eAccess == GA_Update )
     {
         delete poDS;
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The GRASS driver does not support update access to existing"
                   " datasets.\n" );
         return NULL;
     }
-    
+
     return poDS;
 }
 
@@ -1031,24 +1034,20 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 
 void GDALRegister_GRASS()
 {
-    GDALDriver	*poDriver;
-    
-    if (! GDAL_CHECK_VERSION("GDAL/GRASS57 driver"))
+    if( !GDAL_CHECK_VERSION( "GDAL/GRASS57 driver" ) )
         return;
 
-    if( GDALGetDriverByName( "GRASS" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "GRASS" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "GRASS Database Rasters (5.7+)" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_grass.html" );
-        
-        poDriver->pfnOpen = GRASSDataset::Open;
+    if( GDALGetDriverByName( "GRASS" ) != NULL )
+        return;
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    GDALDriver *poDriver = new GDALDriver();
+
+    poDriver->SetDescription( "GRASS" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "GRASS Rasters (5.7+)" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_grass.html" );
+
+    poDriver->pfnOpen = GRASSDataset::Open;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
-
