@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: grassdataset.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: grassdataset.cpp 33717 2016-03-14 06:29:14Z goatbar $
  *
  * Project:  GRASS Driver
  * Purpose:  Implement GRASS raster read/write support
@@ -30,15 +30,12 @@
 
 #include <libgrass.h>
 
-#include "gdal_priv.h"
 #include "cpl_string.h"
+#include "gdal_frmts.h"
+#include "gdal_priv.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: grassdataset.cpp 27044 2014-03-16 23:41:27Z rouault $");
-
-CPL_C_START
-void	GDALRegister_GRASS(void);
-CPL_C_END
+CPL_CVSID("$Id: grassdataset.cpp 33717 2016-03-14 06:29:14Z goatbar $");
 
 /************************************************************************/
 /*                         Grass2CPLErrorHook()                         */
@@ -104,7 +101,7 @@ class GRASSRasterBand : public GDALRasterBand
 
   public:
 
-                   GRASSRasterBand( GRASSDataset *, int, 
+                   GRASSRasterBand( GRASSDataset *, int,
                                     const char *, const char * );
     virtual        ~GRASSRasterBand();
 
@@ -139,7 +136,7 @@ GRASSRasterBand::GRASSRasterBand( GRASSDataset *poDS, int nBand,
 /* -------------------------------------------------------------------- */
     struct FPRange sRange;
 
-    if( G_read_fp_range( (char *) pszCellName, (char *) pszMapset, 
+    if( G_read_fp_range( (char *) pszCellName, (char *) pszMapset,
                          &sRange ) == -1 )
     {
         bHaveMinMax = FALSE;
@@ -190,7 +187,7 @@ GRASSRasterBand::GRASSRasterBand( GRASSDataset *poDS, int nBand,
         this->eDataType = GDT_Float64;
         dfNoData = -12345.0;
     }
-    
+
     nBlockXSize = poDS->nRasterXSize;;
     nBlockYSize = 1;
 
@@ -202,7 +199,7 @@ GRASSRasterBand::GRASSRasterBand( GRASSDataset *poDS, int nBand,
     struct Colors sGrassColors;
 
     poCT = NULL;
-    if( G_read_colors( (char *) pszCellName, (char *) pszMapset, 
+    if( G_read_colors( (char *) pszCellName, (char *) pszMapset,
                        &sGrassColors ) == 1 )
     {
         poCT = new GDALColorTable();
@@ -263,11 +260,11 @@ CPLErr GRASSRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 
 {
     char *pachNullBuf;
-    
+
     pachNullBuf = (char *) CPLMalloc(nBlockXSize);
     G_get_null_value_row( hCell, pachNullBuf, nBlockYOff );
-            
-    if( eDataType == GDT_Float32 || eDataType == GDT_Float64 
+
+    if( eDataType == GDT_Float32 || eDataType == GDT_Float64
         || eDataType == GDT_UInt32 )
     {
         G_get_raster_row( hCell, pImage, nBlockYOff, nGRSType  );
@@ -280,16 +277,15 @@ CPLErr GRASSRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                     ((GUInt32 *) pImage)[i] = (GUInt32) dfNoData;
                 else if( eDataType == GDT_Float32 )
                     ((float *) pImage)[i] = dfNoData;
-                else 
+                else
                     ((double *) pImage)[i] = dfNoData;
             }
         }
-        
     }
     else
     {
         GUInt32 *panRow = (GUInt32 *) CPLMalloc(4 * nBlockXSize);
-        
+
         G_get_raster_row( hCell, panRow, nBlockYOff, nGRSType  );
 
         for( int i = 0; i < nBlockXSize; i++ )
@@ -298,7 +294,7 @@ CPLErr GRASSRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                 panRow[i] = (GUInt32) dfNoData;
         }
 
-        GDALCopyWords( panRow, GDT_UInt32, 4, 
+        GDALCopyWords( panRow, GDT_UInt32, 4,
                        pImage, eDataType, GDALGetDataTypeSize(eDataType)/8,
                        nBlockXSize );
 
@@ -371,7 +367,7 @@ double GRASSRasterBand::GetMaximum( int *pbSuccess )
         return 4294967295.0;
     else if( eDataType == GDT_UInt16 )
         return 65535;
-    else 
+    else
         return 255;
 }
 
@@ -426,7 +422,7 @@ GRASSDataset::~GRASSDataset()
 /*                          GetProjectionRef()                          */
 /************************************************************************/
 
-const char *GRASSDataset::GetProjectionRef() 
+const char *GRASSDataset::GetProjectionRef()
 {
     if( pszProjection == NULL )
         return "";
@@ -438,10 +434,10 @@ const char *GRASSDataset::GetProjectionRef()
 /*                          GetGeoTransform()                           */
 /************************************************************************/
 
-CPLErr GRASSDataset::GetGeoTransform( double * padfGeoTransform ) 
+CPLErr GRASSDataset::GetGeoTransform( double * padfGeoTransform )
 {
     memcpy( padfGeoTransform, adfGeoTransform, sizeof(double) * 6 );
-    
+
     return CE_None;
 }
 
@@ -476,7 +472,7 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
         G_free( pszMapset );
         G_free( pszCell );
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Check if this is a valid GRASS imagery group.                   */
 /* -------------------------------------------------------------------- */
@@ -486,7 +482,7 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 
         I_init_group_ref( &ref );
         I_get_group_ref( pszCell, &ref );
-        
+
         for( int iRef = 0; iRef < ref.nfiles; iRef++ )
         {
             papszCells = CSLAddString( papszCells, ref.file[iRef].name );
@@ -511,12 +507,12 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 
     /* notdef: should only allow read access to an existing cell, right? */
     poDS->eAccess = poOpenInfo->eAccess;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Capture some information from the file that is of interest.     */
 /* -------------------------------------------------------------------- */
     struct Cell_head	sCellInfo;
-    
+
     if( G_get_cellhd( papszCells[0], papszMapsets[0], &sCellInfo ) != 0 )
     {
         /* notdef: report failure. */
@@ -558,24 +554,24 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     for( int iBand = 0; papszCells[iBand] != NULL; iBand++ )
     {
-        poDS->SetBand( iBand+1, 
-                       new GRASSRasterBand( poDS, iBand+1, 
-                                            papszMapsets[iBand], 
+        poDS->SetBand( iBand+1,
+                       new GRASSRasterBand( poDS, iBand+1,
+                                            papszMapsets[iBand],
                                             papszCells[iBand] ) );
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Confirm the requested access is supported.                      */
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->eAccess == GA_Update )
     {
         delete poDS;
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The GRASS driver does not support update access to existing"
                   " datasets.\n" );
         return NULL;
     }
-    
+
     return poDS;
 }
 
@@ -586,24 +582,20 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 void GDALRegister_GRASS()
 
 {
-    GDALDriver	*poDriver;
-    
-    if (! GDAL_CHECK_VERSION("GDAL/GRASS driver"))
+    if( !GDAL_CHECK_VERSION( "GDAL/GRASS driver" ) )
         return;
 
-    if( GDALGetDriverByName( "GRASS" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "GRASS" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "GRASS Database Rasters" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_grass.html" );
-        
-        poDriver->pfnOpen = GRASSDataset::Open;
+    if( GDALGetDriverByName( "GRASS" ) != NULL )
+        return;
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    GDALDriver *poDriver = new GDALDriver();
+
+    poDriver->SetDescription( "GRASS" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "GRASS Database Rasters" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_grass.html" );
+
+    poDriver->pfnOpen = GRASSDataset::Open;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
-
