@@ -297,10 +297,10 @@ void fillSect2 (enGribMeta *en, uChar *sec2, sInt4 lenSec2)
  * shapeEarth = [Code:3.2] shape of the Earth defined by GRIB2 (Output)
  *    factRad = Scale factor of radius of spherical Earth. (Output)
  *     valRad = Value of radius of spherical Earth (Output)
- *    factMaj = Scale factor of major axis of eliptical Earth. (Output)
- *     valMaj = Value of major axis of eliptical Earth (Output)
- *    factMin = Scale factor of minor axis of eliptical Earth. (Output)
- *     valMin = Value of minor axis of eliptical Earth (Output)
+ *    factMaj = Scale factor of major axis of elliptical Earth. (Output)
+ *     valMaj = Value of major axis of elliptical Earth (Output)
+ *    factMin = Scale factor of minor axis of elliptical Earth. (Output)
+ *     valMin = Value of minor axis of elliptical Earth (Output)
  *
  * RETURNS: void
  *
@@ -387,7 +387,7 @@ static void getShpEarth (double majEarth, double minEarth, sInt4 * shapeEarth,
  * [tmplNum=0]
  *      angle = rule 92.1.6 may not hold, in which case, angle != 0, and
  *              unit = angle/subdivision. Typically 0 (Input)
- *   subDivis = 0 or see angle explaination. (Input)
+ *   subDivis = 0 or see angle explanation. (Input)
  * [tmplNum=10,20,30]
  *    meshLat = Latitude where Dx and Dy are specified (Input)
  *  orientLon = Orientation of the grid (Input)
@@ -580,7 +580,7 @@ int fillSect3 (enGribMeta *en, uShort2 tmplNum, double majEarth,
 static int getCodedTime (uChar timeCode, double time, sInt4 *ans)
 {
    /* Following is a lookup table for unit conversion (see code table 4.4). */
-   static sInt4 unit2sec[] = {
+   static const sInt4 unit2sec[] = {
       60, 3600, 86400L, 0, 0,
       0, 0, 0, 0, 0,
       10800, 21600L, 43200L, 1
@@ -724,7 +724,7 @@ int fillSect4_0 (enGribMeta *en, uShort2 tmplNum, uChar cat, uChar subCat,
  *
  * RETURNS: int
  *    > 0 (length of section 4).
- *    -1 if not template 4.1, or fillSect4_0 wasn't already called.
+ *    -1 if not template 4.1, or fillSect4_0 was not already called.
  *
  *  4/2006 Arthur Taylor (MDL): Created.
  *
@@ -734,7 +734,7 @@ int fillSect4_0 (enGribMeta *en, uShort2 tmplNum, uChar cat, uChar subCat,
 int fillSect4_1 (enGribMeta *en, uShort2 tmplNum, uChar typeEnsemble,
                  uChar perturbNum, uChar numFcsts)
 {
-   /* ensemble tempate (1) */
+   /* Ensemble template (1) */
    if (tmplNum != 1) {
       /* This is specifically for template 4.1 */
       return -1;
@@ -763,7 +763,7 @@ int fillSect4_1 (enGribMeta *en, uShort2 tmplNum, uChar typeEnsemble,
  *
  * RETURNS: int
  *    > 0 (length of section 4).
- *    -1 if not template 4.2, or fillSect4_0 wasn't already called.
+ *    -1 if not template 4.2, or fillSect4_0 was not already called.
  *
  *  4/2006 Arthur Taylor (MDL): Created.
  *
@@ -811,7 +811,7 @@ int fillSect4_2 (enGribMeta *en, uShort2 tmplNum, uChar numFcsts,
  *
  * RETURNS: int
  *    > 0 (length of section 4).
- *    -1 if not template 4.5, or fillSect4_0 wasn't already called.
+ *    -1 if not template 4.5, or fillSect4_0 was not already called.
  *
  *  4/2006 Arthur Taylor (MDL): Created.
  *
@@ -872,7 +872,7 @@ int fillSect4_5 (enGribMeta *en, uShort2 tmplNum, uChar numFcsts,
  *
  * RETURNS: int
  *    > 0 (length of section 4).
- *    -1 if not template 4.8, or fillSect4_0 wasn't already called.
+ *    -1 if not template 4.8, or fillSect4_0 was not already called.
  *    -4 can only handle 1 and only 1 time interval
  *
  *  4/2006 Arthur Taylor (MDL): Created.
@@ -952,7 +952,7 @@ int fillSect4_8 (enGribMeta *en, uShort2 tmplNum, sInt4 endYear, int endMonth,
  *
  * RETURNS: int
  *    > 0 (length of section 4).
- *    -1 if not template 4.9, or fillSect4_0 wasn't already called.
+ *    -1 if not template 4.9, or fillSect4_0 was not already called.
  *    -4 can only handle 1 and only 1 time interval
  *
  *  4/2006 Arthur Taylor (MDL): Created.
@@ -1040,7 +1040,7 @@ int fillSect4_9 (enGribMeta *en, uShort2 tmplNum, uChar numFcsts,
  *
  * RETURNS: int
  *    > 0 (length of section 4).
- *    -1 if not template 4.9, or fillSect4_0 wasn't already called.
+ *    -1 if not template 4.9, or fillSect4_0 was not already called.
  *    -4 can only handle 1 and only 1 time interval
  *
  *  5/2006 Arthur Taylor (MDL): Created.
@@ -1111,7 +1111,7 @@ int fillSect4_10 (enGribMeta *en, uShort2 tmplNum, int percentile,
  *
  * RETURNS: int
  *    > 0 (length of section 4).
- *    -1 if not template 4.12, or fillSect4_0 wasn't already called.
+ *    -1 if not template 4.12, or fillSect4_0 was not already called.
  *    -4 can only handle 1 and only 1 time interval
  *
  *  4/2006 Arthur Taylor (MDL): Created.
