@@ -31,8 +31,6 @@
 #include "metaname.h"
 #include "filedatasource.h"
 
-#include "cpl_port.h"
-
 #define SECT0LEN_BYTE 16
 
 typedef union {
@@ -85,7 +83,7 @@ void GRIB2InventoryFree (inventoryType *inv)
  *
  * PURPOSE
  *   Prints to standard out, an inventory of the file, assuming one has an
- * array of invenories of single grib messages.
+ * array of inventories of single GRIB messages.
  *
  * ARGUMENTS
  *    Inv = Pointer to an Array of inventories to print. (Input)
@@ -244,7 +242,9 @@ static int InventoryParseTime (char *is, double *AnsTime)
  *   May want to put this in degrib2.c
  *****************************************************************************
  */
-static int GRIB2SectToBuffer (DataSource &fp, CPL_UNUSED uInt4 gribLen, sChar *sect,
+static int GRIB2SectToBuffer (DataSource &fp,
+                              CPL_UNUSED uInt4 gribLen,
+                              sChar *sect,
                               uInt4 *secLen, uInt4 *buffLen, char **buff)
 {
    char *buffer = *buff; /* Local ptr to buff to reduce ptr confusion. */
@@ -257,9 +257,20 @@ static int GRIB2SectToBuffer (DataSource &fp, CPL_UNUSED uInt4 gribLen, sChar *s
       }
       return -1;
    }
+   if( *secLen < sizeof(sInt4) )
+   {
+       errSprintf ("ERROR: Wrong secLen in GRIB2SectToBuffer\n");
+       return -1;
+   }
    if (*buffLen < *secLen) {
+      char* buffnew = (char *) realloc ((void *) *buff, *secLen * sizeof (char));
+      if( buffnew == NULL )
+      {
+           errSprintf ("ERROR: Ran out of memory in GRIB2SectToBuffer\n");
+           return -1;
+      }
       *buffLen = *secLen;
-      *buff = (char *) realloc ((void *) *buff, *buffLen * sizeof (char));
+      *buff = buffnew;
       buffer = *buff;
    }
 
@@ -275,7 +286,7 @@ static int GRIB2SectToBuffer (DataSource &fp, CPL_UNUSED uInt4 gribLen, sChar *s
    if (*sect == -1) {
       *sect = buffer[5 - 5];
    } else if (buffer[5 - 5] != *sect) {
-      errSprintf ("ERROR: Section %d misslabeled\n", *sect);
+      errSprintf ("ERROR: Section %d mislabeled\n", *sect);
       return -2;
    }
    return 0;
@@ -316,7 +327,8 @@ static int GRIB2SectToBuffer (DataSource &fp, CPL_UNUSED uInt4 gribLen, sChar *s
  *   May want to put this in degrib2.c
  *****************************************************************************
  */
-static int GRIB2SectJump (DataSource &fp, CPL_UNUSED sInt4 gribLen, sChar *sect, uInt4 *secLen)
+static int GRIB2SectJump (DataSource &fp,
+                          CPL_UNUSED sInt4 gribLen, sChar *sect, uInt4 *secLen)
 {
    char sectNum;        /* Validates that we are on the correct section. */
    int c;               /* Check that the fseek is still inside the file. */
@@ -340,17 +352,17 @@ static int GRIB2SectJump (DataSource &fp, CPL_UNUSED sInt4 gribLen, sChar *sect,
    if (*sect == -1) {
       *sect = sectNum;
    } else if (sectNum != *sect) {
-      errSprintf ("ERROR: Section %d misslabeled\n", *sect);
-      return -2;
+       errSprintf ("ERROR: Section %d mislabeled\n", *sect);
+       return -2;
    }
-   /* Since fseek does not give an error if we jump outside the file, we test 
+   /* Since fseek does not give an error if we jump outside the file, we test
     * it by using fgetc / ungetc. */
    fp.DataSourceFseek (*secLen - 5, SEEK_CUR);
    if ((c = fp.DataSourceFgetc()) == EOF) {
-      errSprintf ("ERROR: Ran out of file in Section %d\n", *sect);
-      return -1;
+       errSprintf ("ERROR: Ran out of file in Section %d\n", *sect);
+       return -1;
    } else {
-		 fp.DataSourceUngetc(c);
+       fp.DataSourceUngetc(c);
    }
    return 0;
 }
@@ -419,12 +431,12 @@ static int GRIB2Inventory2to7 (sChar sectNum, DataSource &fp, sInt4 gribLen,
    double fstSurfValue; /* Value of first fixed surface. */
    sInt4 value;         /* The scaled value from GRIB2 file. */
    sChar factor;        /* The scaled factor from GRIB2 file */
-   sChar scale;         /* Surface scale as opposed to probility factor. */
+   sChar scale;         /* Surface scale as opposed to probability factor. */
    uChar sndSurfType;   /* Type of the second fixed surface. */
    double sndSurfValue; /* Value of second fixed surface. */
    sChar f_sndValue;    /* flag if SndValue is valid. */
    uChar timeRangeUnit;
-   sInt4 lenTime;       /* Used by parseTime to tell difference betweeen 8hr
+   sInt4 lenTime;       /* Used by parseTime to tell difference between 8hr
                          * average and 1hr average ozone. */
    uChar genID;         /* The Generating process ID (used for GFS MOS) */
    uChar probType;      /* The probability type */
@@ -443,7 +455,7 @@ static int GRIB2Inventory2to7 (sChar sectNum, DataSource &fp, sInt4 gribLen,
          return -6;
       }
       if ((sectNum != 2) && (sectNum != 3)) {
-         errSprintf ("ERROR: Section 2 or 3 misslabeled\n");
+         errSprintf ("ERROR: Section 2 or 3 mislabeled\n");
          return -5;
       } else if (sectNum == 2) {
          /* Jump past section 3. */
@@ -627,26 +639,27 @@ enum { GS4_ANALYSIS, GS4_ENSEMBLE, GS4_DERIVED, GS4_PROBABIL_PNT = 5,
    /* Try to convert lenTime to hourly. */
    if (timeRangeUnit == 0) {
       lenTime = (sInt4) (lenTime / 60.);
-      timeRangeUnit = 1;
+      /*timeRangeUnit = 1;*/
    } else if (timeRangeUnit == 1) {
    } else if (timeRangeUnit == 2) {
       lenTime = lenTime * 24;
-      timeRangeUnit = 1;
+      /*timeRangeUnit = 1;*/
    } else if (timeRangeUnit == 10) {
       lenTime = lenTime * 3;
-      timeRangeUnit = 1;
+      /*timeRangeUnit = 1;*/
    } else if (timeRangeUnit == 11) {
       lenTime = lenTime * 6;
-      timeRangeUnit = 1;
+      /*timeRangeUnit = 1;*/
    } else if (timeRangeUnit == 12) {
       lenTime = lenTime * 12;
-      timeRangeUnit = 1;
+      /*timeRangeUnit = 1;*/
    } else if (timeRangeUnit == 13) {
       lenTime = (sInt4) (lenTime / 3600.);
-      timeRangeUnit = 1;
+      /*timeRangeUnit = 1;*/
    } else {
       printf ("Can't handle this timeRangeUnit\n");
-      myAssert (timeRangeUnit == 1);
+      //myAssert (timeRangeUnit == 1);
+      return -8;
    }
    if (lenTime == GRIB2MISSING_s4) {
       lenTime = 0;
@@ -788,7 +801,7 @@ enum { GS4_ANALYSIS, GS4_ENSEMBLE, GS4_DERIVED, GS4_PROBABIL_PNT = 5,
  *   9/2002 Arthur Taylor (MDL/RSIS): Created.
  *  11/2002 AAT: Revised.
  *  12/2002 (TK,AC,TB,&MS): Code Review.
- *   3/2003 AAT: Corrected some satelite type mistakes.
+ *   3/2003 AAT: Corrected some satellite type mistakes.
  *   3/2003 AAT: Implemented multiple grid inventories in the same GRIB2
  *          message.
  *   4/2003 AAT: Started adding GRIB1 support
@@ -800,7 +813,7 @@ enum { GS4_ANALYSIS, GS4_ENSEMBLE, GS4_DERIVED, GS4_PROBABIL_PNT = 5,
  *          after we know we have a GRIB file, we don't want "trailing" bytes
  *          to break the program.
  *   8/2003 AAT: switched fileLen to only be computed for an error message.
- *   8/2003 AAT: curTot no longer serves a purpse.
+ *   8/2003 AAT: curTot no longer serves a purpose.
  *   5/2004 AAT: Added a check for section number 2..8 for the repeated
  *          section (otherwise error)
  *  10/2004 AAT: Added ability to inventory TDLP records.
@@ -900,7 +913,7 @@ int GRIB2Inventory (DataSource &fp, inventoryType **Inv, uInt4 *LenInv,
             free (msg);
             /* find out how big the file is. */
             fp.DataSourceFseek (0L, SEEK_END);
-            fileLen = fp.DataSourceFtell();
+            fileLen = static_cast<int>(fp.DataSourceFtell());
             /* fseek (fp, 0L, SEEK_SET); */
             printf ("There were %d trailing bytes in the file.\n",
                     fileLen - offset);
@@ -1132,7 +1145,7 @@ int GRIB2RefTime (char *filename, double *refTime)
             free (msg);
             /* find out how big the file is. */
             fp.DataSourceFseek (0L, SEEK_END);
-            fileLen = fp.DataSourceFtell();
+            fileLen = static_cast<int>(fp.DataSourceFtell());
             /* fseek (fp, 0L, SEEK_SET); */
             printf ("There were %d trailing bytes in the file.\n",
                     fileLen - offset);
