@@ -55,7 +55,8 @@ g2int jpcunpack(unsigned char *cpack,g2int len,g2int *idrstmpl,g2int ndpts,
 
          ifld=(g2int *)calloc(ndpts,sizeof(g2int));
          if ( ifld == 0 ) {
-            fprintf(stderr,"Could not allocate space in jpcunpack.\n  Data field NOT upacked.\n");
+            fprintf(stderr, "Could not allocate space in jpcunpack.\n"
+                    "Data field NOT unpacked.\n");
             return(1);
          }
          /* iret= (g2int) */ dec_jpeg2000((char *) cpack,len,ifld);
