@@ -16,12 +16,9 @@
 
 #include "cpl_port.h"
 
-typedef g2int integer;
-typedef g2float real;
-
-/* Subroutine */ int reduce(CPL_UNUSED integer *kfildo, integer *jmin, integer *jmax, 
+/* Subroutine */ int reduce(CPL_UNUSED integer *kfildo, integer *jmin, integer *jmax,
 	integer *lbit, integer *nov, integer *lx, integer *ndg, integer *ibit,
-	 integer *jbit, integer *kbit, integer *novref, integer *ibxx2, 
+	 integer *jbit, integer *kbit, integer *novref, integer *ibxx2,
 	integer *ier)
 {
     /* Initialized data */
@@ -158,12 +155,12 @@ typedef g2float real;
     ntotbt[*kbit - 1] = iorigb;
 /*           THIS IS THE VALUE OF TOTAL BITS FOR THE ORIGINAL LX */
 /*           GROUPS, WHICH REQUIRES KBITS TO PACK THE GROUP */
-/*           LENGHTS.  SETTING THIS HERE MAKES ONE LESS LOOPS */
+/*           LENGTHS.  SETTING THIS HERE MAKES ONE LESS LOOPS */
 /*           NECESSARY BELOW. */
 
 /*        COMPUTE BITS NOW USED FOR THE PARAMETERS DEFINED. */
 
-/*        DETERMINE OTHER POSSIBILITES BY INCREASING LX AND DECREASING */
+/*        DETERMINE OTHER POSSIBILITIES BY INCREASING LX AND DECREASING */
 /*        NOV( ) WITH VALUES GREATER THAN THRESHOLDS.  ASSUME A GROUP IS */
 /*        SPLIT INTO 2 OR MORE GROUPS SO THAT KBIT IS REDUCED WITHOUT */
 /*        CHANGING IBIT OR JBIT. */
@@ -387,7 +384,7 @@ L288:
 /*           LX IS NOW THE NEW NUMBER OF GROUPS. */
 	*kbit = jj;
 /*           KBIT IS NOW THE NEW NUMBER OF BITS REQUIRED FOR PACKING */
-/*           GROUP LENGHTS. */
+/*           GROUP LENGTHS. */
     }
 
 /*     WRITE(KFILDO,406)CFEED,LX */
