@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gribdataset.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: gribdataset.cpp 33813 2016-03-29 23:48:36Z goatbar $
  *
  * Project:  GRIB Driver
  * Purpose:  GDALDataset driver for GRIB translator for read support
@@ -27,11 +27,13 @@
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  ******************************************************************************
- * 
+ *
  */
 
-#include "gdal_pam.h"
 #include "cpl_multiproc.h"
+#include "gdal_frmts.h"
+#include "gdal_pam.h"
+#include "ogr_spatialref.h"
 
 #include "degrib18/degrib/degrib2.h"
 #include "degrib18/degrib/inventory.h"
@@ -39,19 +41,13 @@
 #include "degrib18/degrib/filedatasource.h"
 #include "degrib18/degrib/memorydatasource.h"
 
-#include "ogr_spatialref.h"
+CPL_CVSID("$Id: gribdataset.cpp 33813 2016-03-29 23:48:36Z goatbar $");
 
-CPL_CVSID("$Id: gribdataset.cpp 27729 2014-09-24 00:40:16Z goatbar $");
-
-CPL_C_START
-void	GDALRegister_GRIB(void);
-CPL_C_END
-
-static void *hGRIBMutex = NULL;
+static CPLMutex *hGRIBMutex = NULL;
 
 /************************************************************************/
 /* ==================================================================== */
-/*				GRIBDataset				*/
+/*                              GRIBDataset                             */
 /* ==================================================================== */
 /************************************************************************/
 
@@ -62,21 +58,21 @@ class GRIBDataset : public GDALPamDataset
     friend class GRIBRasterBand;
 
   public:
-		GRIBDataset();
-		~GRIBDataset();
-    
+                GRIBDataset();
+                ~GRIBDataset();
+
     static GDALDataset *Open( GDALOpenInfo * );
     static int          Identify( GDALOpenInfo * );
 
-    CPLErr 	GetGeoTransform( double * padfTransform );
+    CPLErr      GetGeoTransform( double * padfTransform );
     const char *GetProjectionRef();
-    
-	private:
-		void SetGribMetaData(grib_MetaData* meta);
-    VSILFILE	*fp;
+
+  private:
+    void SetGribMetaData(grib_MetaData* meta);
+    VSILFILE    *fp;
     char  *pszProjection;
-    OGRCoordinateTransformation *poTransform;
-    double adfGeoTransform[6]; // Calculate and store once as GetGeoTransform may be called multiple times
+    // Calculate and store once as GetGeoTransform may be called multiple times.
+    double adfGeoTransform[6];
 
     GIntBig  nCachedBytes;
     GIntBig  nCachedBytesThreshold;
@@ -93,7 +89,7 @@ class GRIBDataset : public GDALPamDataset
 class GRIBRasterBand : public GDALPamRasterBand
 {
     friend class GRIBDataset;
-    
+
 public:
     GRIBRasterBand( GRIBDataset*, int, inventoryType* );
     virtual ~GRIBRasterBand();
@@ -110,7 +106,8 @@ private:
 
     CPLErr       LoadData();
 
-    static void ReadGribData( DataSource &, sInt4, int, double**, grib_MetaData**);
+    static void ReadGribData( DataSource &, sInt4, int, double**,
+                              grib_MetaData** );
     sInt4 start;
     int subgNum;
     char *longFstLevel;
@@ -126,7 +123,7 @@ private:
 /*                         ConvertUnitInText()                          */
 /************************************************************************/
 
-static CPLString ConvertUnitInText(int bMetricUnits, const char* pszTxt)
+static CPLString ConvertUnitInText( bool bMetricUnits, const char* pszTxt )
 {
     if( !bMetricUnits )
         return pszTxt;
@@ -142,36 +139,41 @@ static CPLString ConvertUnitInText(int bMetricUnits, const char* pszTxt)
 /*                           GRIBRasterBand()                            */
 /************************************************************************/
 
-GRIBRasterBand::GRIBRasterBand( GRIBDataset *poDS, int nBand, 
-                                inventoryType *psInv )
-  : m_Grib_Data(NULL), m_Grib_MetaData(NULL)
+GRIBRasterBand::GRIBRasterBand( GRIBDataset *poDSIn, int nBandIn,
+                                inventoryType *psInv ) :
+    start(psInv->start),
+    subgNum(psInv->subgNum),
+    longFstLevel(CPLStrdup(psInv->longFstLevel)),
+    m_Grib_Data(NULL),
+    m_Grib_MetaData(NULL),
+    nGribDataXSize(poDSIn->nRasterXSize),
+    nGribDataYSize(poDSIn->nRasterYSize)
 {
-    this->poDS = poDS;
-    this->nBand = nBand;
-    this->start = psInv->start;
-    this->subgNum = psInv->subgNum;
-    this->longFstLevel = CPLStrdup(psInv->longFstLevel);
+    poDS = poDSIn;
+    nBand = nBandIn;
 
-    eDataType = GDT_Float64; // let user do -ot Float32 if needed for saving space, GRIB contains Float64 (though not fully utilized most of the time)
+    // Let user do -ot Float32 if needed for saving space, GRIB contains
+    // Float64 (though not fully utilized most of the time).
+    eDataType = GDT_Float64;
 
-    nBlockXSize = poDS->nRasterXSize;
+    nBlockXSize = poDSIn->nRasterXSize;
     nBlockYSize = 1;
 
-    nGribDataXSize = poDS->nRasterXSize;
-    nGribDataYSize = poDS->nRasterYSize;
+    const char* pszGribNormalizeUnits =
+        CPLGetConfigOption("GRIB_NORMALIZE_UNITS", "YES");
+    bool bMetricUnits = CPLTestBool(pszGribNormalizeUnits);
 
-    const char* pszGribNormalizeUnits = CPLGetConfigOption("GRIB_NORMALIZE_UNITS", "YES");
-    int bMetricUnits = CSLTestBoolean(pszGribNormalizeUnits);
-
-    SetMetadataItem( "GRIB_UNIT", ConvertUnitInText(bMetricUnits, psInv->unitName) );
-    SetMetadataItem( "GRIB_COMMENT", ConvertUnitInText(bMetricUnits, psInv->comment) );
+    SetMetadataItem( "GRIB_UNIT",
+                     ConvertUnitInText(bMetricUnits, psInv->unitName) );
+    SetMetadataItem( "GRIB_COMMENT",
+                     ConvertUnitInText(bMetricUnits, psInv->comment) );
     SetMetadataItem( "GRIB_ELEMENT", psInv->element );
     SetMetadataItem( "GRIB_SHORT_NAME", psInv->shortFstLevel );
-    SetMetadataItem( "GRIB_REF_TIME", 
+    SetMetadataItem( "GRIB_REF_TIME",
                      CPLString().Printf("%12.0f sec UTC", psInv->refTime ) );
-    SetMetadataItem( "GRIB_VALID_TIME", 
+    SetMetadataItem( "GRIB_VALID_TIME",
                      CPLString().Printf("%12.0f sec UTC", psInv->validTime ) );
-    SetMetadataItem( "GRIB_FORECAST_SECONDS", 
+    SetMetadataItem( "GRIB_FORECAST_SECONDS",
                      CPLString().Printf("%.0f sec", psInv->foreSec ) );
 }
 
@@ -185,7 +187,7 @@ GRIBRasterBand::GRIBRasterBand( GRIBDataset *poDS, int nBand,
 void GRIBRasterBand::FindPDSTemplate()
 
 {
-    GRIBDataset *poGDS = (GRIBDataset *) poDS;
+    GRIBDataset *poGDS = reinterpret_cast<GRIBDataset *>( poDS );
 
 /* -------------------------------------------------------------------- */
 /*      Collect section 4 octet information ... we read the file        */
@@ -193,12 +195,13 @@ void GRIBRasterBand::FindPDSTemplate()
 /*      this for us.                                                    */
 /* -------------------------------------------------------------------- */
     GIntBig nOffset = VSIFTellL( poGDS->fp );
-    GByte abyHead[5];
-    GUInt32 nSectSize;
 
     VSIFSeekL( poGDS->fp, start+16, SEEK_SET );
+
+    GByte abyHead[5] = { 0 };
     VSIFReadL( abyHead, 5, 1, poGDS->fp );
 
+    GUInt32 nSectSize = 0;
     while( abyHead[4] != 4 )
     {
         memcpy( &nSectSize, abyHead, 4 );
@@ -208,41 +211,38 @@ void GRIBRasterBand::FindPDSTemplate()
             || VSIFReadL( abyHead, 5, 1, poGDS->fp ) != 1 )
             break;
     }
-        
+
     if( abyHead[4] == 4 )
     {
-        GUInt16 nCoordCount;
-        GUInt16 nPDTN;
-        CPLString osOctet;
-        int i;
-        GByte *pabyBody;
-
         memcpy( &nSectSize, abyHead, 4 );
         CPL_MSBPTR32( &nSectSize );
 
-        pabyBody = (GByte *) CPLMalloc(nSectSize-5);
+        GByte *pabyBody = static_cast<GByte *>( CPLMalloc(nSectSize - 5) );
         VSIFReadL( pabyBody, 1, nSectSize-5, poGDS->fp );
 
+        GUInt16 nCoordCount = 0;
         memcpy( &nCoordCount, pabyBody + 5 - 5, 2 );
         CPL_MSBPTR16( &nCoordCount );
 
+        GUInt16 nPDTN = 0;
         memcpy( &nPDTN, pabyBody + 7 - 5, 2 );
         CPL_MSBPTR16( &nPDTN );
 
         SetMetadataItem( "GRIB_PDS_PDTN",
                          CPLString().Printf( "%d", nPDTN ) );
 
-        for( i = 9; i < (int) nSectSize; i++ )
+        CPLString osOctet;
+        for( int i = 9; i < static_cast<int>( nSectSize ); i++ )
         {
-            char szByte[10];
+            char szByte[10] = { '\0' };
 
             if( i == 9 )
-                sprintf( szByte, "%d", pabyBody[i-5] );
+                snprintf( szByte, sizeof(szByte), "%d", pabyBody[i-5] );
             else
-                sprintf( szByte, " %d", pabyBody[i-5] );
+                snprintf( szByte, sizeof(szByte), " %d", pabyBody[i-5] );
             osOctet += szByte;
         }
-        
+
         SetMetadataItem( "GRIB_PDS_TEMPLATE_NUMBERS", osOctet );
 
         CPLFree( pabyBody );
@@ -259,10 +259,10 @@ const char * GRIBRasterBand::GetDescription() const
 {
     if( longFstLevel == NULL )
         return GDALPamRasterBand::GetDescription();
-    else
-        return longFstLevel;
+
+    return longFstLevel;
 }
- 
+
 /************************************************************************/
 /*                             LoadData()                               */
 /************************************************************************/
@@ -272,28 +272,30 @@ CPLErr GRIBRasterBand::LoadData()
 {
     if( !m_Grib_Data )
     {
-        GRIBDataset *poGDS = (GRIBDataset *) poDS;
+        GRIBDataset *poGDS = reinterpret_cast<GRIBDataset *>( poDS );
 
         if (poGDS->bCacheOnlyOneBand)
         {
-            /* In "one-band-at-a-time" strategy, if the last recently used */
-            /* band is not that one, uncache it. We could use a smarter strategy */
-            /* based on a LRU, but that's a bit overkill for now. */
+            // In "one-band-at-a-time" strategy, if the last recently used
+            // band is not that one, uncache it. We could use a smarter strategy
+            // based on a LRU, but that's a bit overkill for now.
             poGDS->poLastUsedBand->UncacheData();
             poGDS->nCachedBytes = 0;
         }
         else
         {
-            /* Once we have cached more than nCachedBytesThreshold bytes, we will switch */
-            /* to "one-band-at-a-time" strategy, instead of caching all bands that have */
-            /* been accessed */
+            // Once we have cached more than nCachedBytesThreshold bytes, we
+            // will switch to "one-band-at-a-time" strategy, instead of caching
+            // all bands that have been accessed.
             if (poGDS->nCachedBytes > poGDS->nCachedBytesThreshold)
             {
-                CPLDebug("GRIB", "Maximum band cache size reached for this dataset. "
-                         "Caching only one band at a time from now");
+                CPLDebug( "GRIB",
+                          "Maximum band cache size reached for this dataset. "
+                          "Caching only one band at a time from now");
                 for(int i=0;i<poGDS->nBands;i++)
                 {
-                    ((GRIBRasterBand*) poGDS->GetRasterBand(i+1))->UncacheData();
+                    reinterpret_cast<GRIBRasterBand*>(
+                        poGDS->GetRasterBand(i+1))->UncacheData();
                 }
                 poGDS->nCachedBytes = 0;
                 poGDS->bCacheOnlyOneBand = TRUE;
@@ -320,14 +322,16 @@ CPLErr GRIBRasterBand::LoadData()
         poGDS->nCachedBytes += nGribDataXSize * nGribDataYSize * sizeof(double);
         poGDS->poLastUsedBand = this;
 
-        if( nGribDataXSize != nRasterXSize 
+        if( nGribDataXSize != nRasterXSize
             || nGribDataYSize != nRasterYSize )
         {
             CPLError( CE_Warning, CPLE_AppDefined,
-                      "Band %d of GRIB dataset is %dx%d, while the first band and dataset is %dx%d.  Georeferencing of band %d may be incorrect, and data access may be incomplete.", 
-                      nBand, 
-                      nGribDataXSize, nGribDataYSize, 
-                      nRasterXSize, nRasterYSize, 
+                      "Band %d of GRIB dataset is %dx%d, while the first band "
+                      "and dataset is %dx%d.  Georeferencing of band %d may "
+                      "be incorrect, and data access may be incomplete.",
+                      nBand,
+                      nGribDataXSize, nGribDataYSize,
+                      nRasterXSize, nRasterYSize,
                       nBand );
         }
     }
@@ -339,7 +343,8 @@ CPLErr GRIBRasterBand::LoadData()
 /*                             IReadBlock()                             */
 /************************************************************************/
 
-CPLErr GRIBRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYOff,
+CPLErr GRIBRasterBand::IReadBlock( int /* nBlockXOff */,
+                                   int nBlockYOff,
                                    void * pImage )
 
 {
@@ -357,27 +362,25 @@ CPLErr GRIBRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYOff,
         && nGribDataYSize == nRasterYSize )
     {
         // Simple 1:1 case.
-        memcpy(pImage, 
-               m_Grib_Data + nRasterXSize * (nRasterYSize - nBlockYOff - 1), 
+        memcpy(pImage,
+               m_Grib_Data + nRasterXSize * (nRasterYSize - nBlockYOff - 1),
                nRasterXSize * sizeof(double));
-        
+
         return CE_None;
     }
-    else
-    {
-        memset( pImage, 0, sizeof(double)*nRasterXSize );
 
-        if( nBlockYOff >= nGribDataYSize ) // off image?
-            return CE_None;
+    memset( pImage, 0, sizeof(double) * nRasterXSize );
 
-        int nCopyWords = MIN(nRasterXSize,nGribDataXSize);
-
-        memcpy( pImage, 
-                m_Grib_Data + nGribDataXSize*(nGribDataYSize-nBlockYOff-1),
-                nCopyWords * sizeof(double) );
-        
+    if( nBlockYOff >= nGribDataYSize ) // off image?
         return CE_None;
-    }
+
+    const int nCopyWords = MIN(nRasterXSize,nGribDataXSize);
+
+    memcpy( pImage,
+            m_Grib_Data + nGribDataXSize*(nGribDataYSize-nBlockYOff-1),
+            nCopyWords * sizeof(double) );
+
+    return CE_None;
 }
 
 /************************************************************************/
@@ -412,30 +415,37 @@ double GRIBRasterBand::GetNoDataValue( int *pbSuccess )
 /*                            ReadGribData()                            */
 /************************************************************************/
 
-void GRIBRasterBand::ReadGribData( DataSource & fp, sInt4 start, int subgNum, double** data, grib_MetaData** metaData)
+void GRIBRasterBand::ReadGribData( DataSource & fp, sInt4 start, int subgNum,
+                                   double** data, grib_MetaData** metaData)
 {
-    /* Initialisation, for calling the ReadGrib2Record function */
-    sInt4 f_endMsg = 1;  /* 1 if we read the last grid in a GRIB message, or we haven't read any messages. */
-    // int subgNum = 0;     /* The subgrid in the message that we are interested in. */
-    sChar f_unit = 2;        /* None = 0, English = 1, Metric = 2 */
-    double majEarth = 0;     /* -radEarth if < 6000 ignore, otherwise use this to
-                              * override the radEarth in the GRIB1 or GRIB2
-                              * message.  Needed because NCEP uses 6371.2 but GRIB1 could only state 6367.47. */
-    double minEarth = 0;     /* -minEarth if < 6000 ignore, otherwise use this to
-                              * override the minEarth in the GRIB1 or GRIB2 message. */
-    sChar f_SimpleVer = 4;   /* Which version of the simple NDFD Weather table to
-                              * use. (1 is 6/2003) (2 is 1/2004) (3 is 2/2004)
-                              * (4 is 11/2004) (default 4) */
-    LatLon lwlf;         /* lower left corner (cookie slicing) -lwlf */
-    LatLon uprt;         /* upper right corner (cookie slicing) -uprt */
-    IS_dataType is;      /* Un-parsed meta data for this GRIB2 message. As well as some memory used by the unpacker. */
+    // Initialization, for calling the ReadGrib2Record function.
+    sInt4 f_endMsg = 1;  // 1 if we read the last grid in a GRIB message, or we
+                         // haven't read any messages.
+    // int subgNum = 0; // The subgrid in the message that we are interested in.
+    sChar f_unit = 2;        // None = 0, English = 1, Metric = 2
+    double majEarth = 0.0;   // -radEarth if < 6000 ignore, otherwise use this
+                             // to override the radEarth in the GRIB1 or GRIB2
+                             // message.  Needed because NCEP uses 6371.2 but
+                             // GRIB1 could only state 6367.47.
+    double minEarth = 0.0;   // -minEarth if < 6000 ignore, otherwise use this
+                             // to override the minEarth in the GRIB1 or GRIB2
+                             // message.
+    sChar f_SimpleVer = 4;   // Which version of the simple NDFD Weather table
+                             // to use. (1 is 6/2003) (2 is 1/2004) (3 is
+                             // 2/2004) (4 is 11/2004) (default 4)
+    LatLon lwlf;         // lower left corner (cookie slicing) -lwlf
+    LatLon uprt;         // upper right corner (cookie slicing) -uprt
+    IS_dataType is;      // Un-parsed meta data for this GRIB2 message. As well
+                         // as some memory used by the unpacker.
 
-    lwlf.lat = -100; // lat == -100 instructs the GRIB decoder that we don't want a subgrid
+    lwlf.lat = -100; // lat == -100 instructs the GRIB decoder that we don't
+                     // want a subgrid
 
     IS_Init (&is);
 
-    const char* pszGribNormalizeUnits = CPLGetConfigOption("GRIB_NORMALIZE_UNITS", "YES");
-    if ( !CSLTestBoolean(pszGribNormalizeUnits) )
+    const char* pszGribNormalizeUnits =
+        CPLGetConfigOption("GRIB_NORMALIZE_UNITS", "YES");
+    if ( !CPLTestBool(pszGribNormalizeUnits) )
         f_unit = 0; /* do not normalize units to metric */
 
     /* Read GRIB message from file position "start". */
@@ -446,7 +456,8 @@ void GRIBRasterBand::ReadGribData( DataSource & fp, sInt4 start, int subgNum, do
     ReadGrib2Record (fp, f_unit, data, &grib_DataLen, *metaData, &is, subgNum,
                      majEarth, minEarth, f_SimpleVer, &f_endMsg, &lwlf, &uprt);
 
-    char * errMsg = errSprintf(NULL); // no intention to show errors, just swallow it and free the memory
+    // no intention to show errors, just swallow it and free the memory.
+    char * errMsg = errSprintf(NULL);
     if( errMsg != NULL )
         CPLDebug( "GRIB", "%s", errMsg );
     free(errMsg);
@@ -482,15 +493,17 @@ GRIBRasterBand::~GRIBRasterBand()
 
 /************************************************************************/
 /* ==================================================================== */
-/*				GRIBDataset				*/
+/*                              GRIBDataset                             */
 /* ==================================================================== */
 /************************************************************************/
 
-GRIBDataset::GRIBDataset()
-
+GRIBDataset::GRIBDataset() :
+    fp(NULL),
+    pszProjection(CPLStrdup("")),
+    nCachedBytes(0),
+    bCacheOnlyOneBand(FALSE),
+    poLastUsedBand(NULL)
 {
-  poTransform = NULL;
-  pszProjection = CPLStrdup("");
   adfGeoTransform[0] = 0.0;
   adfGeoTransform[1] = 1.0;
   adfGeoTransform[2] = 0.0;
@@ -498,12 +511,11 @@ GRIBDataset::GRIBDataset()
   adfGeoTransform[4] = 0.0;
   adfGeoTransform[5] = 1.0;
 
-  nCachedBytes = 0;
   /* Switch caching strategy once 100 MB threshold is reached */
   /* Why 100 MB ? --> why not ! */
-  nCachedBytesThreshold = ((GIntBig)atoi(CPLGetConfigOption("GRIB_CACHEMAX", "100"))) * 1024 * 1024;
-  bCacheOnlyOneBand = FALSE;
-  poLastUsedBand = NULL;
+  nCachedBytesThreshold =
+      static_cast<GIntBig>(atoi(CPLGetConfigOption("GRIB_CACHEMAX", "100")))
+      * 1024 * 1024;
 }
 
 /************************************************************************/
@@ -516,7 +528,7 @@ GRIBDataset::~GRIBDataset()
     FlushCache();
     if( fp != NULL )
         VSIFCloseL( fp );
-		
+
     CPLFree( pszProjection );
 }
 
@@ -549,18 +561,17 @@ int GRIBDataset::Identify( GDALOpenInfo * poOpenInfo )
 {
     if (poOpenInfo->nHeaderBytes < 8)
         return FALSE;
-        
+
 /* -------------------------------------------------------------------- */
 /*      Does a part of what ReadSECT0() but in a thread-safe way.       */
 /* -------------------------------------------------------------------- */
-    int i;
-    for(i=0;i<poOpenInfo->nHeaderBytes-3;i++)
+    for(int i=0;i<poOpenInfo->nHeaderBytes-3;i++)
     {
-        if (EQUALN((const char*)poOpenInfo->pabyHeader + i, "GRIB", 4) ||
-            EQUALN((const char*)poOpenInfo->pabyHeader + i, "TDLP", 4))
+        if (STARTS_WITH_CI((const char*)poOpenInfo->pabyHeader + i, "GRIB") ||
+            STARTS_WITH_CI((const char*)poOpenInfo->pabyHeader + i, "TDLP"))
             return TRUE;
     }
-    
+
     return FALSE;
 }
 
@@ -573,15 +584,15 @@ GDALDataset *GRIBDataset::Open( GDALOpenInfo * poOpenInfo )
 {
     if( !Identify(poOpenInfo) )
         return NULL;
-        
+
 /* -------------------------------------------------------------------- */
 /*      A fast "probe" on the header that is partially read in memory.  */
 /* -------------------------------------------------------------------- */
     char *buff = NULL;
     uInt4 buffLen = 0;
-    sInt4 sect0[SECT0LEN_WORD];
-    uInt4 gribLen;
-    int version;
+    sInt4 sect0[SECT0LEN_WORD] = { 0 };
+    uInt4 gribLen = 0;
+    int version = 0;
 // grib is not thread safe, make sure not to cause problems
 // for other thread safe formats
 
@@ -596,13 +607,13 @@ GDALDataset *GRIBDataset::Open( GDALOpenInfo * poOpenInfo )
         return NULL;
     }
     free(buff);
-    
+
 /* -------------------------------------------------------------------- */
 /*      Confirm the requested access is supported.                      */
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->eAccess == GA_Update )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The GRIB driver does not support update access to existing"
                   " datasets.\n" );
         return NULL;
@@ -610,27 +621,28 @@ GDALDataset *GRIBDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
-    GRIBDataset 	*poDS;
-
-    poDS = new GRIBDataset();
+    GRIBDataset *poDS = new GRIBDataset();
 
     poDS->fp = VSIFOpenL( poOpenInfo->pszFilename, "r" );
 
-	/* Check the return values */    
-	if (!poDS->fp) {
+    /* Check the return values */
+    if (!poDS->fp) {
         // we have no FP, so we don't have anywhere to read from
         char * errMsg = errSprintf(NULL);
         if( errMsg != NULL )
             CPLDebug( "GRIB", "%s", errMsg );
         free(errMsg);
-		
-		CPLError( CE_Failure, CPLE_OpenFailed, "Error (%d) opening file %s", errno, poOpenInfo->pszFilename);
-        CPLReleaseMutex(hGRIBMutex); // Release hGRIBMutex otherwise we'll deadlock with GDALDataset own hGRIBMutex
+
+        CPLError( CE_Failure, CPLE_OpenFailed,
+                  "Error (%d) opening file %s", errno, poOpenInfo->pszFilename);
+        // Release hGRIBMutex otherwise we'll deadlock with GDALDataset own
+        // hGRIBMutex.
+        CPLReleaseMutex(hGRIBMutex);
         delete poDS;
         CPLAcquireMutex(hGRIBMutex, 1000.0);
         return NULL;
-	}
-    
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Read the header.                                                */
 /* -------------------------------------------------------------------- */
@@ -643,26 +655,30 @@ GDALDataset *GRIBDataset::Open( GDALOpenInfo * poOpenInfo )
 /* The band-data that is read is stored into the first RasterBand,      */
 /* simply so that the same portion of the file is not read twice.       */
 /* -------------------------------------------------------------------- */
-    
+
     VSIFSeekL( poDS->fp, 0, SEEK_SET );
 
-    FileDataSource grib_fp (poDS->fp);
+    FileDataSource grib_fp(poDS->fp);
 
-    inventoryType *Inv = NULL;  /* Contains an GRIB2 message inventory of the file */
-    uInt4 LenInv = 0;        /* size of Inv (also # of GRIB2 messages) */
-    int msgNum =0;          /* The messageNumber during the inventory. */
+    // Contains an GRIB2 message inventory of the file.
+    inventoryType *Inv = NULL;
+    uInt4 LenInv = 0;        // Size of Inv (also # of GRIB2 messages).
+    int msgNum = 0;          // The messageNumber during the inventory.
 
-    if (GRIB2Inventory (grib_fp, &Inv, &LenInv, 0, &msgNum) <= 0 )
+    if( GRIB2Inventory (grib_fp, &Inv, &LenInv, 0, &msgNum) <= 0 )
     {
         char * errMsg = errSprintf(NULL);
         if( errMsg != NULL )
             CPLDebug( "GRIB", "%s", errMsg );
         free(errMsg);
 
-        CPLError( CE_Failure, CPLE_OpenFailed, 
-                  "%s is a grib file, but no raster dataset was successfully identified.",
+        CPLError( CE_Failure, CPLE_OpenFailed,
+                  "%s is a grib file, "
+                  "but no raster dataset was successfully identified.",
                   poOpenInfo->pszFilename );
-        CPLReleaseMutex(hGRIBMutex); // Release hGRIBMutex otherwise we'll deadlock with GDALDataset own hGRIBMutex
+        // Release hGRIBMutex otherwise we'll deadlock with GDALDataset own
+        // hGRIBMutex.
+        CPLReleaseMutex(hGRIBMutex);
         delete poDS;
         CPLAcquireMutex(hGRIBMutex, 1000.0);
         return NULL;
@@ -673,36 +689,61 @@ GDALDataset *GRIBDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     for (uInt4 i = 0; i < LenInv; ++i)
     {
+        GRIBRasterBand *gribBand = NULL;
         uInt4 bandNr = i+1;
         if (bandNr == 1)
         {
-            // important: set DataSet extents before creating first RasterBand in it
-            double * data = NULL;
-            grib_MetaData* metaData;
-            GRIBRasterBand::ReadGribData(grib_fp, 0, Inv[i].subgNum, &data, &metaData);
-            if (data == 0 || metaData->gds.Nx < 1 || metaData->gds.Ny < 1)
+            // Important: set DataSet extents before creating first RasterBand
+            // in it.
+            double *data = NULL;
+            grib_MetaData *metaData = NULL;
+            GRIBRasterBand::ReadGribData( grib_fp, 0, Inv[i].subgNum,
+                                          &data, &metaData );
+            if( data == NULL || metaData == NULL || metaData->gds.Nx < 1 ||
+                 metaData->gds.Ny < 1 )
             {
-                CPLError( CE_Failure, CPLE_OpenFailed, 
-                          "%s is a grib file, but no raster dataset was successfully identified.",
+                CPLError( CE_Failure, CPLE_OpenFailed,
+                          "%s is a grib file, "
+                          "but no raster dataset was successfully identified.",
                           poOpenInfo->pszFilename );
-                CPLReleaseMutex(hGRIBMutex); // Release hGRIBMutex otherwise we'll deadlock with GDALDataset own hGRIBMutex
+                // Release hGRIBMutex otherwise we'll deadlock with GDALDataset
+                // own hGRIBMutex.
+                CPLReleaseMutex(hGRIBMutex);
                 delete poDS;
                 CPLAcquireMutex(hGRIBMutex, 1000.0);
+                if (metaData != NULL)
+                {
+                    delete metaData;
+                }
+                if (data != NULL)
+                {
+                    free(data);
+                }
                 return NULL;
             }
 
-            poDS->SetGribMetaData(metaData); // set the DataSet's x,y size, georeference and projection from the first GRIB band
-            GRIBRasterBand* gribBand = new GRIBRasterBand( poDS, bandNr, Inv+i);
+             // Set the DataSet's x,y size, georeference and projection from
+             // the first GRIB band.
+            poDS->SetGribMetaData(metaData);
+            gribBand = new GRIBRasterBand( poDS, bandNr, Inv+i);
 
             if( Inv->GribVersion == 2 )
                 gribBand->FindPDSTemplate();
 
             gribBand->m_Grib_Data = data;
             gribBand->m_Grib_MetaData = metaData;
-            poDS->SetBand( bandNr, gribBand);
         }
         else
-            poDS->SetBand( bandNr, new GRIBRasterBand( poDS, bandNr, Inv+i ));
+        {
+            gribBand = new GRIBRasterBand( poDS, bandNr, Inv+i );
+            if( CPLTestBool(
+                   CPLGetConfigOption( "GRIB_PDS_ALL_BANDS", "ON" ) ) )
+            {
+                if( Inv->GribVersion == 2 )
+                    gribBand->FindPDSTemplate();
+            }
+        }
+        poDS->SetBand( bandNr, gribBand);
         GRIB2InventoryFree (Inv + i);
     }
     free (Inv);
@@ -711,14 +752,17 @@ GDALDataset *GRIBDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Initialize any PAM information.                                 */
 /* -------------------------------------------------------------------- */
     poDS->SetDescription( poOpenInfo->pszFilename );
-    
-    CPLReleaseMutex(hGRIBMutex); // Release hGRIBMutex otherwise we'll deadlock with GDALDataset own hGRIBMutex
+
+    // Release hGRIBMutex otherwise we'll deadlock with GDALDataset own
+    // hGRIBMutex.
+    CPLReleaseMutex(hGRIBMutex);
     poDS->TryLoadXML();
 
 /* -------------------------------------------------------------------- */
 /*      Check for external overviews.                                   */
 /* -------------------------------------------------------------------- */
-    poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename, poOpenInfo->papszSiblingFiles );
+    poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename,
+                                 poOpenInfo->GetSiblingFiles() );
     CPLAcquireMutex(hGRIBMutex, 1000.0);
 
     return( poDS );
@@ -758,14 +802,17 @@ void GRIBDataset::SetGribMetaData(grib_MetaData* meta)
                     meta->gds.meshLat, meta->gds.orientLon,
                     0.0, 0.0); // set projection
         break;
-			
 
       case GS3_ORTHOGRAPHIC:
 
-        //oSRS.SetOrthographic(0.0, meta->gds.orientLon,
-        //											meta->gds.lon2, meta->gds.lat2);
-        //oSRS.SetGEOS(meta->gds.orientLon, meta->gds.stretchFactor, meta->gds.lon2, meta->gds.lat2);
-        oSRS.SetGEOS(  0, 35785831, 0, 0 ); // hardcoded for now, I don't know yet how to parse the meta->gds section
+        // oSRS.SetOrthographic( 0.0, meta->gds.orientLon,
+        //                       meta->gds.lon2, meta->gds.lat2);
+
+        // oSRS.SetGEOS( meta->gds.orientLon, meta->gds.stretchFactor,
+        //               meta->gds.lon2, meta->gds.lat2);
+
+        // TODO: Hardcoded for now. How to parse the meta->gds section?
+        oSRS.SetGEOS(  0, 35785831, 0, 0 );
         break;
       case GS3_EQUATOR_EQUIDIST:
         break;
@@ -793,7 +840,7 @@ void GRIBDataset::SetGribMetaData(grib_MetaData* meta)
     }
     else
     {
-        double fInv = a/(a-b);
+        const double fInv = a / (a - b);
         oSRS.SetGeogCS( "Coordinate System imported from GRIB file",
                         NULL,
                         "Spheroid imported from GRIB file",
@@ -803,15 +850,18 @@ void GRIBDataset::SetGribMetaData(grib_MetaData* meta)
     OGRSpatialReference oLL; // construct the "geographic" part of oSRS
     oLL.CopyGeogCSFrom( &oSRS );
 
-    double rMinX;
-    double rMaxY;
-    double rPixelSizeX;
-    double rPixelSizeY;
+    double rMinX = 0.0;
+    double rMaxY = 0.0;
+    double rPixelSizeX = 0.0;
+    double rPixelSizeY = 0.0;
     if (meta->gds.projType == GS3_ORTHOGRAPHIC)
     {
-        //rMinX = -meta->gds.Dx * (meta->gds.Nx / 2); // This is what should work, but it doesn't .. Dx seems to have an inverse relation with pixel size
-        //rMaxY = meta->gds.Dy * (meta->gds.Ny / 2);
-        const double geosExtentInMeters = 11137496.552; // hardcoded for now, assumption: GEOS projection, full disc (like MSG)
+        // This is what should work, but it doesn't .. Dx seems to have an
+        // inverse relation with pixel size.
+        // rMinX = -meta->gds.Dx * (meta->gds.Nx / 2);
+        // rMaxY = meta->gds.Dy * (meta->gds.Ny / 2);
+        // Hardcoded for now, assumption: GEOS projection, full disc (like MSG).
+        const double geosExtentInMeters = 11137496.552;
         rMinX = -(geosExtentInMeters / 2);
         rMaxY = geosExtentInMeters / 2;
         rPixelSizeX = geosExtentInMeters / meta->gds.Nx;
@@ -819,13 +869,23 @@ void GRIBDataset::SetGribMetaData(grib_MetaData* meta)
     }
     else if( oSRS.IsProjected() )
     {
-        rMinX = meta->gds.lon1; // longitude in degrees, to be transformed to meters (or degrees in case of latlon)
-        rMaxY = meta->gds.lat1; // latitude in degrees, to be transformed to meters 
-        OGRCoordinateTransformation *poTransformLLtoSRS = OGRCreateCoordinateTransformation( &(oLL), &(oSRS) );
-        if ((poTransformLLtoSRS != NULL) && poTransformLLtoSRS->Transform( 1, &rMinX, &rMaxY )) // transform it to meters
+        // Longitude in degrees, to be transformed to meters (or degrees in
+        // case of latlon).
+        rMinX = meta->gds.lon1;
+        // Latitude in degrees, to be transformed to meters.
+        rMaxY = meta->gds.lat1;
+        OGRCoordinateTransformation *poTransformLLtoSRS =
+            OGRCreateCoordinateTransformation( &(oLL), &(oSRS) );
+        // Transform it to meters.
+        if( (poTransformLLtoSRS != NULL) &&
+            poTransformLLtoSRS->Transform( 1, &rMinX, &rMaxY ))
         {
             if (meta->gds.scan == GRIB2BIT_2) // Y is minY, GDAL wants maxY
-                rMaxY += (meta->gds.Ny - 1) * meta->gds.Dy; // -1 because we GDAL needs the coordinates of the centre of the pixel
+            {
+                // -1 because we GDAL needs the coordinates of the centre of
+                // the pixel.
+                rMaxY += (meta->gds.Ny - 1) * meta->gds.Dy;
+            }
             rPixelSizeX = meta->gds.Dx;
             rPixelSizeY = meta->gds.Dy;
         }
@@ -833,23 +893,27 @@ void GRIBDataset::SetGribMetaData(grib_MetaData* meta)
         {
             rMinX = 0.0;
             rMaxY = 0.0;
-            
+
             rPixelSizeX = 1.0;
             rPixelSizeY = -1.0;
-            
+
             oSRS.Clear();
 
             CPLError( CE_Warning, CPLE_AppDefined,
-                      "Unable to perform coordinate transformations, so the correct\n"
-                      "projected geotransform could not be deduced from the lat/long\n"
-                      "control points.  Defaulting to ungeoreferenced." );
+                      "Unable to perform coordinate transformations, so the "
+                      "correct projected geotransform could not be deduced "
+                      "from the lat/long control points.  "
+                      "Defaulting to ungeoreferenced." );
         }
         delete poTransformLLtoSRS;
     }
     else
     {
-        rMinX = meta->gds.lon1; // longitude in degrees, to be transformed to meters (or degrees in case of latlon)
-        rMaxY = meta->gds.lat1; // latitude in degrees, to be transformed to meters 
+        // Longitude in degrees, to be transformed to meters (or degrees in
+        // case of latlon).
+        rMinX = meta->gds.lon1;
+        // Latitude in degrees, to be transformed to meters.
+        rMaxY = meta->gds.lat1;
 
         double rMinY = meta->gds.lat2;
         if (meta->gds.lat2 > rMaxY)
@@ -861,7 +925,8 @@ void GRIBDataset::SetGribMetaData(grib_MetaData* meta)
         if( meta->gds.Nx == 1 )
           rPixelSizeX = meta->gds.Dx;
         else if (meta->gds.lon1 > meta->gds.lon2)
-          rPixelSizeX = (360.0 - (meta->gds.lon1 - meta->gds.lon2)) / (meta->gds.Nx - 1);
+          rPixelSizeX =
+              (360.0 - (meta->gds.lon1 - meta->gds.lon2)) / (meta->gds.Nx - 1);
         else
           rPixelSizeX = (meta->gds.lon2 - meta->gds.lon1) / (meta->gds.Nx - 1);
 
@@ -916,24 +981,21 @@ static void GDALDeregister_GRIB(GDALDriver* )
 void GDALRegister_GRIB()
 
 {
-    GDALDriver	*poDriver;
+    if( GDALGetDriverByName( "GRIB" ) != NULL )
+        return;
 
-    if( GDALGetDriverByName( "GRIB" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "GRIB" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "GRIdded Binary (.grb)" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_grib.html" );
-        poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "grb" );
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->pfnOpen = GRIBDataset::Open;
-        poDriver->pfnIdentify = GRIBDataset::Identify;
-        poDriver->pfnUnloadDriver = GDALDeregister_GRIB;
+    poDriver->SetDescription( "GRIB" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "GRIdded Binary (.grb)" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_grib.html" );
+    poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "grb" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    poDriver->pfnOpen = GRIBDataset::Open;
+    poDriver->pfnIdentify = GRIBDataset::Identify;
+    poDriver->pfnUnloadDriver = GDALDeregister_GRIB;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
