@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gt_overview.cpp 33806 2016-03-28 22:26:19Z goatbar $
+ * $Id: gt_overview.cpp 35041 2016-08-10 14:04:52Z rouault $
  *
  * Project:  GeoTIFF Driver
  * Purpose:  Code to build overviews of external databases as a TIFF file.
@@ -35,7 +35,9 @@
 #include "tifvsi.h"
 #include "xtiffio.h"
 
-CPL_CVSID("$Id: gt_overview.cpp 33806 2016-03-28 22:26:19Z goatbar $");
+#include <algorithm>
+
+CPL_CVSID("$Id: gt_overview.cpp 35041 2016-08-10 14:04:52Z rouault $");
 
 /************************************************************************/
 /*                         GTIFFWriteDirectory()                        */
@@ -714,6 +716,11 @@ GTIFFBuildOverviews( const char * pszFilename,
 /*      Loop writing overview data.                                     */
 /* -------------------------------------------------------------------- */
 
+    int * panOverviewListSorted = static_cast<int*>(
+                                        CPLMalloc(sizeof(int) * nOverviews));
+    memcpy( panOverviewListSorted, panOverviewList, sizeof(int) * nOverviews);
+    std::sort(panOverviewListSorted, panOverviewListSorted + nOverviews);
+
     GTIFFSetInExternalOvr(true);
 
     if (nCompression != COMPRESSION_NONE &&
@@ -725,41 +732,67 @@ GTIFFBuildOverviews( const char * pszFilename,
          EQUAL(pszResampling, "CUBICSPLINE") || EQUAL(pszResampling, "LANCZOS") ||
          EQUAL(pszResampling, "BILINEAR")))
     {
-        /* In the case of pixel interleaved compressed overviews, we want to generate */
-        /* the overviews for all the bands block by block, and not band after band, */
-        /* in order to write the block once and not loose space in the TIFF file */
-        GDALRasterBand ***papapoOverviewBands;
-
-        papapoOverviewBands = (GDALRasterBand ***) CPLCalloc(sizeof(void*),nBands);
+        // In the case of pixel interleaved compressed overviews, we want to
+        // generate the overviews for all the bands block by block, and not
+        // band after band, in order to write the block once and not loose
+        // space in the TIFF file.
+        GDALRasterBand ***papapoOverviewBands =
+            static_cast<GDALRasterBand ***>(
+                CPLCalloc(sizeof(void *), nBands) );
         for( iBand = 0; iBand < nBands && eErr == CE_None; iBand++ )
         {
-            GDALRasterBand    *hSrcBand = papoBandList[iBand];
-            GDALRasterBand    *hDstBand = hODS->GetRasterBand( iBand+1 );
-            papapoOverviewBands[iBand] = (GDALRasterBand **) CPLCalloc(sizeof(void*),nOverviews);
-            papapoOverviewBands[iBand][0] = hDstBand;
+            GDALRasterBand *poSrcBand = papoBandList[iBand];
+            GDALRasterBand *poDstBand = hODS->GetRasterBand( iBand+1 );
+            papapoOverviewBands[iBand] =
+                static_cast<GDALRasterBand **>(
+                    CPLCalloc(sizeof(void *), nOverviews) );
 
-            int bHasNoData;
-            double noDataValue = hSrcBand->GetNoDataValue(&bHasNoData);
-            if (bHasNoData)
-                hDstBand->SetNoDataValue(noDataValue);
+            int bHasNoData = FALSE;
+            const double noDataValue = poSrcBand->GetNoDataValue(&bHasNoData);
+            if( bHasNoData )
+                poDstBand->SetNoDataValue(noDataValue);
 
-            for( int i = 0; i < nOverviews-1 && eErr == CE_None; i++ )
+            for( int i = 0; i < nOverviews && eErr == CE_None; i++ )
             {
-                papapoOverviewBands[iBand][i+1] = hDstBand->GetOverview(i);
-                if (papapoOverviewBands[iBand][i+1] == NULL)
-                    eErr = CE_Failure;
-                else
+                for( int j = -1; j < poDstBand->GetOverviewCount() &&
+                                 eErr == CE_None; j++ )
                 {
-                    if (bHasNoData)
-                        papapoOverviewBands[iBand][i+1]->SetNoDataValue(noDataValue);
+                    int    nOvFactor;
+                    GDALRasterBand * poOverview =
+                            (j < 0 ) ? poDstBand : poDstBand->GetOverview( j );
+                    if( poOverview == NULL )
+                    {
+                        eErr = CE_Failure;
+                        continue;
+                    }
+
+                    nOvFactor = GDALComputeOvFactor(poOverview->GetXSize(),
+                                                    poSrcBand->GetXSize(),
+                                                    poOverview->GetYSize(),
+                                                    poSrcBand->GetYSize());
+
+                    if( nOvFactor == panOverviewListSorted[i]
+                        || nOvFactor == GDALOvLevelAdjust2(
+                                            panOverviewListSorted[i],
+                                            poSrcBand->GetXSize(),
+                                            poSrcBand->GetYSize() ) )
+                    {
+                        papapoOverviewBands[iBand][i] = poOverview;
+                        if( bHasNoData )
+                            poOverview->SetNoDataValue(noDataValue);
+                        break;
+                    }
                 }
+                CPLAssert( papapoOverviewBands[iBand][i] != NULL );
             }
         }
 
-        if (eErr == CE_None)
-            eErr = GDALRegenerateOverviewsMultiBand(nBands, papoBandList,
-                                            nOverviews, papapoOverviewBands,
-                                            pszResampling, pfnProgress, pProgressData );
+        if( eErr == CE_None )
+            eErr =
+                GDALRegenerateOverviewsMultiBand(
+                    nBands, papoBandList,
+                    nOverviews, papapoOverviewBands,
+                    pszResampling, pfnProgress, pProgressData );
 
         for( iBand = 0; iBand < nBands; iBand++ )
         {
@@ -786,6 +819,9 @@ GTIFFBuildOverviews( const char * pszFilename,
             if (bHasNoData)
                 hDstBand->SetNoDataValue(noDataValue);
 
+            // FIXME: this logic regenerates all overview bands, not only the
+            // ones requested
+
             papoOverviews[0] = hDstBand;
             nDstOverviews = hDstBand->GetOverviewCount() + 1;
             CPLAssert( nDstOverviews < 128 );
@@ -834,6 +870,8 @@ GTIFFBuildOverviews( const char * pszFilename,
 
     GTIFFSetInExternalOvr(false);
 
+    CPLFree(panOverviewListSorted);
+
     pfnProgress( 1.0, NULL, pProgressData );
 
     return eErr;
