@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: geo_normalize.h,v 1.12 2005/08/26 16:08:14 fwarmerdam Exp $
+ * $Id: geo_normalize.h 2233 2012-10-09 01:33:11Z warmerdam $
  *
  * Project:  libgeotiff
  * Purpose:  Include file related to geo_normalize.c containing Code to
@@ -26,43 +26,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: geo_normalize.h,v $
- * Revision 1.12  2005/08/26 16:08:14  fwarmerdam
- * Include void in empty argument list for prototype.
- *
- * Revision 1.11  2004/02/03 17:19:50  warmerda
- * export GTIFAngleToDD() - used by GDAL mrsiddataset.cpp
- *
- * Revision 1.10  2003/01/15 04:39:16  warmerda
- * Added GTIFDeaccessCSV
- *
- * Revision 1.9  2003/01/15 03:37:40  warmerda
- * added GTIFFreeMemory()
- *
- * Revision 1.8  2002/11/28 22:27:42  warmerda
- * preliminary upgrade to EPSG 6.2.2 tables
- *
- * Revision 1.7  1999/09/17 00:55:26  warmerda
- * added GTIFGetUOMAngleInfo(), and UOMAngle in GTIFDefn
- *
- * Revision 1.6  1999/05/04 03:13:42  warmerda
- * Added prototype
- *
- * Revision 1.5  1999/04/29 23:02:55  warmerda
- * added docs, and MapSys related stuff
- *
- * Revision 1.4  1999/03/18 21:35:19  geotiff
- * Added PROJ.4 related stuff
- *
- * Revision 1.3  1999/03/17 20:44:04  geotiff
- * added CPL_DLL related support
- *
- * Revision 1.2  1999/03/10 18:24:06  geotiff
- * corrected to use int'
- *
- */
+ *****************************************************************************/
 
 #ifndef GEO_NORMALIZE_H_INCLUDED
 #define GEO_NORMALIZE_H_INCLUDED
@@ -79,7 +43,7 @@ extern "C" {
  *
  * Include file for extended projection definition normalization api.
  */
-    
+
 #define MAX_GTIF_PROJPARMS 	10
 
 /**
@@ -96,7 +60,7 @@ typedef struct {
 
     /** From GeographicTypeGeoKey tag.  For example GCS_WGS_84 or
         GCS_Voirol_1875_Paris.  Includes datum and prime meridian value. */
-    short	GCS;	      
+    short	GCS;
 
     /** From ProjLinearUnitsGeoKey.  For example Linear_Meter. */
     short	UOMLength;
@@ -109,7 +73,7 @@ typedef struct {
 
     /** One UOMAngle = UOMLengthInDegrees degrees. */
     double      UOMAngleInDegrees;
-    
+
     /** Datum from GeogGeodeticDatumGeoKey tag. For example Datum_WGS84 */
     short	Datum;
 
@@ -131,6 +95,16 @@ typedef struct {
     /** The length of the semi minor ellipse axis in meters. */
     double	SemiMinor;
 
+  /* This #if is primary intended to maintain binary compatibility with older
+     versions of libgeotiff for MrSID binaries (for example). */
+#if !defined(GEO_NORMALIZE_DISABLE_TOWGS84)
+    /** TOWGS84 transformation values (0/3/7) */
+    short       TOWGS84Count;
+
+    /** TOWGS84 transformation values */
+    double      TOWGS84[7];
+#endif /* !defined(GEO_NORMALIZE_DISABLE_TOWGS84) */
+
     /** Projection id from ProjectionGeoKey.  For example Proj_UTM_11S. */
     short	ProjCode;
 
@@ -139,9 +113,9 @@ typedef struct {
     short	Projection;
 
     /** GeoTIFF identifier for underlying projection method.  While some of
-      these values have corresponding vlaues in EPSG (Projection field),
+      these values have corresponding values in EPSG (Projection field),
       others do not.  For example CT_TransverseMercator. */
-    short	CTProjection;   
+    short	CTProjection;
 
     /** Number of projection parameters in ProjParm and ProjParmId. */
     int		nParms;
@@ -164,10 +138,13 @@ typedef struct {
     /** UTM, or State Plane Zone number, zero if not known. */
     int		Zone;
 
+    /** Do we have any definition at all?  0 if no geokeys found */
+    int         DefnSet;
+
 } GTIFDefn;
 
 int CPL_DLL GTIFGetPCSInfo( int nPCSCode, char **ppszEPSGName,
-                            short *pnProjOp, 
+                            short *pnProjOp,
                             short *pnUOMLengthCode, short *pnGeogCS );
 int CPL_DLL GTIFGetProjTRFInfo( int nProjTRFCode,
                                 char ** ppszProjTRFName,
@@ -191,7 +168,7 @@ int CPL_DLL GTIFGetUOMAngleInfo( int nUOMAngleCode,
                                  char **ppszUOMName,
                                  double * pdfInDegrees );
 double CPL_DLL GTIFAngleToDD( double dfAngle, int nUOMAngle );
-    
+
 
 /* this should be used to free strings returned by GTIFGet... funcs */
 void CPL_DLL GTIFFreeMemory( char * );
@@ -199,7 +176,8 @@ void CPL_DLL GTIFDeaccessCSV( void );
 
 int CPL_DLL GTIFGetDefn( GTIF *psGTIF, GTIFDefn * psDefn );
 void CPL_DLL GTIFPrintDefn( GTIFDefn *, FILE * );
-void CPL_DLL GTIFFreeDefn( GTIF * );
+GTIFDefn CPL_DLL *GTIFAllocDefn( void );
+void CPL_DLL GTIFFreeDefn( GTIFDefn * );
 
 void CPL_DLL SetCSVFilenameHook( const char *(*CSVFileOverride)(const char *) );
 
@@ -224,9 +202,12 @@ int CPL_DLL   GTIFProjToMapSys( int ProjCode, int * pZone );
  * These are only useful if using libgeotiff with libproj (PROJ.4+).
  */
 char CPL_DLL *GTIFGetProj4Defn( GTIFDefn * );
+
 int  CPL_DLL  GTIFProj4ToLatLong( GTIFDefn *, int, double *, double * );
 int  CPL_DLL  GTIFProj4FromLatLong( GTIFDefn *, int, double *, double * );
 
+int  CPL_DLL  GTIFSetFromProj4( GTIF *gtif, const char *proj4 );
+
 #if defined(HAVE_LIBPROJ) && defined(HAVE_PROJECTS_H)
 #  define HAVE_GTIFPROJ4
 #endif
@@ -234,5 +215,5 @@ int  CPL_DLL  GTIFProj4FromLatLong( GTIFDefn *, int, double *, double * );
 #ifdef __cplusplus
 }
 #endif
-    
+
 #endif /* ndef GEO_NORMALIZE_H_INCLUDED */
