@@ -43,7 +43,7 @@ to read.  At this time all keys except for strings have only one value,
 so <b>index</b> should be zero, and <b>count</b> should be one.<p>
 
 The <b>key</b> indicates the key name to be written to the
-file and should from the geokey_t enumeration 
+file and should from the geokey_t enumeration
 (eg. <tt>ProjectedCSTypeGeoKey</tt>).  The full list of possible geokey_t
 values can be found in geokeys.inc, or in the online documentation for
 GTIFKeyGet().<p>
@@ -61,15 +61,15 @@ are passed by value rather than as pointers when count is 1, but as pointers
 if count is larger than 1.<p>
 
 Note that key values aren't actually flushed to the file until
-GTIFWriteKeys() is called.  Till then 
+GTIFWriteKeys() is called.  Till then
 the new values are just kept with the GTIF structure.<p>
 
 <b>Example:</b><p>
 
 <pre>
-    GTIFKeySet(gtif, GTRasterTypeGeoKey, TYPE_SHORT, 1, 
+    GTIFKeySet(gtif, GTRasterTypeGeoKey, TYPE_SHORT, 1,
                RasterPixelIsArea);
-    GTIFKeySet(gtif, GTCitationGeoKey, TYPE_ASCII, 0, 
+    GTIFKeySet(gtif, GTCitationGeoKey, TYPE_ASCII, 0,
                "UTM 11 North / NAD27" );
 </pre>
 
@@ -78,7 +78,7 @@ the new values are just kept with the GTIF structure.<p>
 int GTIFKeySet(GTIF *gtif, geokey_t keyID, tagtype_t type, int count,...)
 {
     va_list ap;
-    int index = gtif->gt_keyindex[ keyID ];
+    int nIndex = gtif->gt_keyindex[ keyID ];
     int newvalues = 0;
     GeoKey *key;
     char *data = NULL;
@@ -88,7 +88,7 @@ int GTIFKeySet(GTIF *gtif, geokey_t keyID, tagtype_t type, int count,...)
 
     va_start(ap, count);
     /* pass singleton keys by value */
-    if (count>1 && type!=TYPE_ASCII) 
+    if (count>1 && type!=TYPE_ASCII)
     {
         val = va_arg(ap, char*);
     }
@@ -97,21 +97,21 @@ int GTIFKeySet(GTIF *gtif, geokey_t keyID, tagtype_t type, int count,...)
         /* delete the indicated tag */
         va_end(ap);
 
-        if( index < 1 )
+        if( nIndex < 1 )
             return 0;
 
-        if (gtif->gt_keys[index].gk_type == TYPE_ASCII)
+        if (gtif->gt_keys[nIndex].gk_type == TYPE_ASCII)
         {
-            _GTIFFree (gtif->gt_keys[index].gk_data);
+            _GTIFFree (gtif->gt_keys[nIndex].gk_data);
         }
 
-        while( index < gtif->gt_num_keys )
+        while( nIndex < gtif->gt_num_keys )
         {
-            _GTIFmemcpy( gtif->gt_keys + index, 
-                         gtif->gt_keys + index + 1, 
+            _GTIFmemcpy( gtif->gt_keys + nIndex,
+                         gtif->gt_keys + nIndex + 1,
                          sizeof(GeoKey) );
-            gtif->gt_keyindex[gtif->gt_keys[index].gk_key] = index;
-            index++;
+            gtif->gt_keyindex[gtif->gt_keys[nIndex].gk_key] = nIndex;
+            nIndex++;
         }
 
         gtif->gt_num_keys--;
@@ -125,21 +125,21 @@ int GTIFKeySet(GTIF *gtif, geokey_t keyID, tagtype_t type, int count,...)
     {
       case TYPE_SHORT:  sval=(pinfo_t) va_arg(ap, int); val=(char *)&sval;     break;
       case TYPE_DOUBLE: dval=va_arg(ap, dblparam_t); val=(char *)&dval;  break;
-      case TYPE_ASCII: 
+      case TYPE_ASCII:
         val=va_arg(ap, char*);
-        count = strlen(val) + 1; /* force = string length */
+        count = (int)strlen(val) + 1; /* force = string length */
         break;
       default:
         assert( FALSE );
         break;
     }
     va_end(ap);
-    
+
     /* We assume here that there are no multi-valued SHORTS ! */
-    if (index)
+    if (nIndex)
     {
         /* Key already exists */
-        key = gtif->gt_keys+index;
+        key = gtif->gt_keys+nIndex;
         if (type!=key->gk_type || count > key->gk_count)
         {
             /* need to reset data pointer */
@@ -154,15 +154,14 @@ int GTIFKeySet(GTIF *gtif, geokey_t keyID, tagtype_t type, int count,...)
         /* We need to create the key */
         if (gtif->gt_num_keys == MAX_KEYS) return 0;
         key = gtif->gt_keys + ++gtif->gt_num_keys;
-        index = gtif->gt_num_keys;
-        gtif->gt_keyindex[ keyID ] = index;
+        nIndex = gtif->gt_num_keys;
+        gtif->gt_keyindex[ keyID ] = nIndex;
         key->gk_key = keyID;
         key->gk_type = type;
         key->gk_count = count;
         key->gk_size = _gtiff_size[ type ];
-        /* TODO: Make sure that the following two casts are safe. */
-        if ((geokey_t)gtif->gt_keymin > keyID) gtif->gt_keymin=keyID;
-        if ((geokey_t)gtif->gt_keymax < keyID) gtif->gt_keymax=keyID;
+        if ((geokey_t)gtif->gt_keymin > keyID)  gtif->gt_keymin=keyID;
+        if ((geokey_t)gtif->gt_keymax < keyID)  gtif->gt_keymax=keyID;
         newvalues = 1;
     }
 
@@ -170,7 +169,7 @@ int GTIFKeySet(GTIF *gtif, geokey_t keyID, tagtype_t type, int count,...)
     {
         switch (type)
         {
-          case TYPE_SHORT:  
+          case TYPE_SHORT:
             if (count > 1) return 0;
             data = (char *)&key->gk_data; /* store value *in* data */
             break;
@@ -191,12 +190,12 @@ int GTIFKeySet(GTIF *gtif, geokey_t keyID, tagtype_t type, int count,...)
     /* this fixes a bug where if a request is made to write a duplicate
        key, we must initialize the data to a valid value.
        Bryan Wells (bryan@athena.bangor.autometric.com) */
-        
+
     else /* no new values, but still have something to write */
     {
         switch (type)
         {
-          case TYPE_SHORT:  
+          case TYPE_SHORT:
             if (count > 1) return 0;
             data = (char *)&key->gk_data; /* store value *in* data */
             break;
@@ -209,7 +208,7 @@ int GTIFKeySet(GTIF *gtif, geokey_t keyID, tagtype_t type, int count,...)
             return 0;
         }
     }
-        
+
     switch (type)
     {
       case TYPE_ASCII:
@@ -227,7 +226,7 @@ int GTIFKeySet(GTIF *gtif, geokey_t keyID, tagtype_t type, int count,...)
     }
 
     _GTIFmemcpy(data, val, count*key->gk_size);
-    
+
     gtif->gt_flags |= FLAG_FILE_MODIFIED;
     return 1;
 }
