@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tif_float.c 10646 2007-01-18 02:38:10Z warmerdam $
+ * $Id: tif_float.c 33796 2016-03-27 13:21:07Z goatbar $
  *
  * Project:  GeoTIFF Driver
  * Purpose:  Floating point conversion functions. Convert 16- and 24-bit
@@ -11,7 +11,7 @@
  *
  * This code is based on the code from OpenEXR project with the following
  * copyright:
- * 
+ *
  * Copyright (c) 2002, Industrial Light & Magic, a division of Lucas
  * Digital Ltd. LLC
  *
@@ -28,8 +28,8 @@
  * distribution.
  * *       Neither the name of Industrial Light & Magic nor the names of
  * its contributors may be used to endorse or promote products derived
- * from this software without specific prior written permission. 
- * 
+ * from this software without specific prior written permission.
+ *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -44,7 +44,7 @@
  *
  ****************************************************************************/
 
-#include "cpl_port.h"
+#include "tif_float.h"
 
 /************************************************************************/
 /*                           HalfToFloat()                              */
@@ -61,61 +61,62 @@ GUInt32 HalfToFloat( GUInt16 iHalf )
 
     if (iExponent == 0)
     {
-	if (iMantissa == 0)
-	{
+        if (iMantissa == 0)
+        {
 /* -------------------------------------------------------------------- */
-/*	Plus or minus zero.			                        */
+/*      Plus or minus zero.                                             */
 /* -------------------------------------------------------------------- */
 
             return iSign << 31;
-	}
-	else
-	{
+        }
+        else
+        {
 /* -------------------------------------------------------------------- */
-/*	Denormalized number -- renormalize it.			        */
+/*      Denormalized number -- renormalize it.                          */
 /* -------------------------------------------------------------------- */
 
-	    while (!(iMantissa & 0x00000400))
-	    {
-		iMantissa <<= 1;
-		iExponent -=  1;
-	    }
+            while (!(iMantissa & 0x00000400))
+            {
+                iMantissa <<= 1;
+                iExponent -=  1;
+            }
 
-	    iExponent += 1;
-	    iMantissa &= ~0x00000400;
-	}
+            iExponent += 1;
+            iMantissa &= ~0x00000400U;
+        }
     }
     else if (iExponent == 31)
     {
-	if (iMantissa == 0)
-	{
+        if (iMantissa == 0)
+        {
 /* -------------------------------------------------------------------- */
-/*	 Positive or negative infinity.			                */
+/*       Positive or negative infinity.                                 */
 /* -------------------------------------------------------------------- */
 
-	    return (iSign << 31) | 0x7f800000;
-	}
-	else
-	{
+            return (iSign << 31) | 0x7f800000;
+        }
+        else
+        {
 /* -------------------------------------------------------------------- */
-/*	 NaN -- preserve sign and significand bits.	                */
+/*       NaN -- preserve sign and significand bits.                     */
 /* -------------------------------------------------------------------- */
 
-	    return (iSign << 31) | 0x7f800000 | (iMantissa << 13);
-	}
+            return (iSign << 31) | 0x7f800000 | (iMantissa << 13);
+        }
     }
 
 /* -------------------------------------------------------------------- */
-/*	 Normalized number.			                        */
+/*       Normalized number.                                             */
 /* -------------------------------------------------------------------- */
 
     iExponent = iExponent + (127 - 15);
     iMantissa = iMantissa << 13;
 
 /* -------------------------------------------------------------------- */
-/*	 Assemble sign, exponent and mantissa.			        */
+/*       Assemble sign, exponent and mantissa.                          */
 /* -------------------------------------------------------------------- */
 
+    /* coverity[overflow_sink] */
     return (iSign << 31) | (iExponent << 23) | iMantissa;
 }
 
@@ -134,60 +135,61 @@ GUInt32 TripleToFloat( GUInt32 iTriple )
 
     if (iExponent == 0)
     {
-	if (iMantissa == 0)
-	{
+        if (iMantissa == 0)
+        {
 /* -------------------------------------------------------------------- */
-/*	Plus or minus zero.			                        */
+/*      Plus or minus zero.                                             */
 /* -------------------------------------------------------------------- */
 
-	    return iSign << 31;
-	}
-	else
-	{
+            return iSign << 31;
+        }
+        else
+        {
 /* -------------------------------------------------------------------- */
-/*	Denormalized number -- renormalize it.			        */
+/*      Denormalized number -- renormalize it.                          */
 /* -------------------------------------------------------------------- */
 
-	    while (!(iMantissa & 0x00002000))
-	    {
-		iMantissa <<= 1;
-		iExponent -=  1;
-	    }
+            while (!(iMantissa & 0x00002000))
+            {
+                iMantissa <<= 1;
+                iExponent -= 1;
+            }
 
-	    iExponent += 1;
-	    iMantissa &= ~0x00002000;
-	}
+            iExponent += 1;
+            iMantissa &= ~0x00002000U;
+        }
     }
     else if (iExponent == 127)
     {
-	if (iMantissa == 0)
-	{
+        if (iMantissa == 0)
+        {
 /* -------------------------------------------------------------------- */
-/*	 Positive or negative infinity.			                */
+/*       Positive or negative infinity.                                 */
 /* -------------------------------------------------------------------- */
 
-	    return (iSign << 31) | 0x7f800000;
-	}
-	else
-	{
+            return (iSign << 31) | 0x7f800000;
+        }
+        else
+        {
 /* -------------------------------------------------------------------- */
-/*	 NaN -- preserve sign and significand bits.	                */
+/*       NaN -- preserve sign and significand bits.                     */
 /* -------------------------------------------------------------------- */
 
-	    return (iSign << 31) | 0x7f800000 | (iMantissa << 7);
-	}
+            return (iSign << 31) | 0x7f800000 | (iMantissa << 7);
+        }
     }
 
 /* -------------------------------------------------------------------- */
-/*	 Normalized number.			                        */
+/*       Normalized number.                                             */
 /* -------------------------------------------------------------------- */
 
     iExponent = iExponent + (127 - 63);
     iMantissa = iMantissa << 7;
 
 /* -------------------------------------------------------------------- */
-/*	 Assemble sign, exponent and mantissa.			        */
+/*       Assemble sign, exponent and mantissa.                          */
 /* -------------------------------------------------------------------- */
 
+    /* coverity[overflow_sink] */
     return (iSign << 31) | (iExponent << 23) | iMantissa;
 }
