@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hdf4dataset.cpp 33722 2016-03-15 15:15:58Z dron $
+ * $Id: hdf4dataset.cpp 35036 2016-08-10 12:11:37Z rouault $
  *
  * Project:  Hierarchical Data Format Release 4 (HDF4)
  * Purpose:  HDF4 Datasets. Open HDF4 file, fetch metadata and list of
@@ -42,7 +42,7 @@
 
 #include "hdf4dataset.h"
 
-CPL_CVSID("$Id: hdf4dataset.cpp 33722 2016-03-15 15:15:58Z dron $");
+CPL_CVSID("$Id: hdf4dataset.cpp 35036 2016-08-10 12:11:37Z rouault $");
 
 extern const char * const pszGDALSignature;
 
@@ -298,39 +298,29 @@ int HDF4Dataset::GetDataTypeSize( int32 iNumType )
 
 double HDF4Dataset::AnyTypeToDouble( int32 iNumType, void *pData )
 {
+    CPL_STATIC_ASSERT(sizeof(GIntBig) == 8);
     switch ( iNumType )
     {
         case DFNT_INT8:
-            return *reinterpret_cast<double *>(
-                reinterpret_cast<char *>( pData ) );
+            return static_cast<double>(*reinterpret_cast<char *>(pData));
         case DFNT_UINT8:
-            return *reinterpret_cast<double *>(
-                reinterpret_cast<unsigned char *>( pData ) );
+            return static_cast<double>(*reinterpret_cast<GByte *>(pData));
         case DFNT_INT16:
-            return *reinterpret_cast<double *>(
-                reinterpret_cast<short *>( pData ) );
+            return static_cast<double>(*reinterpret_cast<GInt16 *>(pData));
         case DFNT_UINT16:
-            return *reinterpret_cast<double *>(
-                reinterpret_cast<unsigned short *>( pData ) );
+            return static_cast<double>(*reinterpret_cast<GUInt16 *>(pData));
         case DFNT_INT32:
-            return *reinterpret_cast<double *>(
-                reinterpret_cast<int *>( pData ) );
+            return static_cast<double>(*reinterpret_cast<GInt32 *>(pData));
         case DFNT_UINT32:
-            return *reinterpret_cast<double *>(
-                reinterpret_cast<unsigned int *>( pData ) );
+            return static_cast<double>(*reinterpret_cast<GUInt32 *>(pData));
         case DFNT_INT64:
-            // Highly suspicious! Should be GIntBig. But cannot verify.
-            return *reinterpret_cast<double *>(
-                reinterpret_cast<char *>( pData ) );
+            return static_cast<double>(*reinterpret_cast<GIntBig *>(pData));
         case DFNT_UINT64:
-            return *reinterpret_cast<double *>(
-                reinterpret_cast<GIntBig *>( pData ) );
+            return static_cast<double>(*reinterpret_cast<GIntBig *>(pData));
         case DFNT_FLOAT32:
-            return *reinterpret_cast<double *>(
-                reinterpret_cast<float *>( pData ) );
+            return static_cast<double>(*reinterpret_cast<float *>(pData));
         case DFNT_FLOAT64:
-            return *reinterpret_cast<double *>(
-                reinterpret_cast<double *>( pData ) );
+            return static_cast<double>(*reinterpret_cast<double *>(pData));
         default:
         {
             CPLError( CE_Warning, CPLE_AppDefined,
