@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hdf4dataset.h 26569 2013-10-29 23:51:12Z rouault $
+ * $Id: hdf4dataset.h 33794 2016-03-26 13:19:07Z goatbar $
  *
  * Project:  Hierarchical Data Format Release 4 (HDF4)
  * Purpose:  Header file for HDF4 datasets reader.
@@ -27,8 +27,8 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _HDF4DATASET_H_INCLUDED_
-#define _HDF4DATASET_H_INCLUDED_
+#ifndef HDF4DATASET_H_INCLUDED_
+#define HDF4DATASET_H_INCLUDED_
 
 #include "cpl_list.h"
 #include "gdal_pam.h"
@@ -65,16 +65,15 @@ class HDF4Dataset : public GDALPamDataset
 
   private:
 
-    int         bIsHDFEOS;
+    bool bIsHDFEOS;
 
     static char **HDF4EOSTokenizeAttrs( const char *pszString );
     static char **HDF4EOSGetObject( char **papszAttrList, char **ppszAttrName,
-                                    char **ppszAttrValue );
-     
-  protected:
+                                    char **ppszAttrClass, char **ppszAttrValue );
 
-    FILE	*fp;
-    int32	hGR, hSD;
+  protected:
+    int32	hGR;
+    int32	hSD;
     int32	nImages;
     HDF4SubdatasetType iSubdatasetType;
     const char	*pszSubdatasetType;
@@ -95,8 +94,8 @@ class HDF4Dataset : public GDALPamDataset
 
   public:
                 HDF4Dataset();
-		~HDF4Dataset();
-    
+    virtual ~HDF4Dataset();
+
     virtual char      **GetMetadataDomainList();
     virtual char        **GetMetadata( const char * pszDomain = "" );
     static GDALDataset  *Open( GDALOpenInfo * );
@@ -107,5 +106,4 @@ char *SPrintArray( GDALDataType eDataType, const void *paDataArray,
                    int nValues, const char *pszDelimiter );
 
 
-#endif /* _HDF4DATASET_H_INCLUDED_ */
-
+#endif /* HDF4DATASET_H_INCLUDED_ */
