@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: bagdataset.cpp 27394 2014-05-24 16:37:41Z rouault $
+ * $Id: bagdataset.cpp 33720 2016-03-15 00:39:53Z goatbar $
  *
  * Project:  Hierarchical Data Format Release 5 (HDF5)
  * Purpose:  Read BAG datasets.
@@ -30,18 +30,15 @@
 
 #include "gh5_convenience.h"
 
+#include "cpl_string.h"
+#include "gdal_frmts.h"
 #include "gdal_pam.h"
 #include "gdal_priv.h"
 #include "ogr_spatialref.h"
-#include "cpl_string.h"
 
-CPL_CVSID("$Id: bagdataset.cpp 27394 2014-05-24 16:37:41Z rouault $");
+CPL_CVSID("$Id: bagdataset.cpp 33720 2016-03-15 00:39:53Z goatbar $");
 
-CPL_C_START
-void    GDALRegister_BAG(void);
-CPL_C_END
-
-OGRErr OGR_SRS_ImportFromISO19115( OGRSpatialReference *poThis, 
+OGRErr OGR_SRS_ImportFromISO19115( OGRSpatialReference *poThis,
                                    const char *pszISOXML );
 
 /************************************************************************/
@@ -51,7 +48,6 @@ OGRErr OGR_SRS_ImportFromISO19115( OGRSpatialReference *poThis,
 /************************************************************************/
 class BAGDataset : public GDALPamDataset
 {
-
     friend class BAGRasterBand;
 
     hid_t        hHDF5;
@@ -63,11 +59,11 @@ class BAGDataset : public GDALPamDataset
 
     char        *pszXMLMetadata;
     char        *apszMDList[2];
-    
+
 public:
     BAGDataset();
     ~BAGDataset();
-    
+
     virtual CPLErr GetGeoTransform( double * );
     virtual const char *GetProjectionRef(void);
     virtual char      **GetMetadataDomainList();
@@ -97,32 +93,27 @@ class BAGRasterBand : public GDALPamRasterBand
     double      dfMaximum;
 
 public:
-  
     BAGRasterBand( BAGDataset *, int );
     ~BAGRasterBand();
 
     bool                    Initialize( hid_t hDataset, const char *pszName );
 
     virtual CPLErr          IReadBlock( int, int, void * );
-    virtual double	    GetNoDataValue( int * ); 
+    virtual double	    GetNoDataValue( int * );
 
     virtual double GetMinimum( int *pbSuccess = NULL );
-    virtual double GetMaximum(int *pbSuccess = NULL );
+    virtual double GetMaximum( int *pbSuccess = NULL );
 };
 
 /************************************************************************/
 /*                           BAGRasterBand()                            */
 /************************************************************************/
-BAGRasterBand::BAGRasterBand( BAGDataset *poDS, int nBand )
-
+BAGRasterBand::BAGRasterBand( BAGDataset *poDS_, int nBand_ ) :
+    hDatasetID(-1), native(-1), dataspace(-1), bMinMaxSet(false),
+    dfMinimum(0.0), dfMaximum(0.0)
 {
-    this->poDS       = poDS;
-    this->nBand      = nBand;
-    
-    hDatasetID = -1;
-    dataspace = -1;
-    native = -1;
-    bMinMaxSet = false;
+    poDS = poDS_;
+    nBand = nBand_;
 }
 
 /************************************************************************/
@@ -145,18 +136,27 @@ BAGRasterBand::~BAGRasterBand()
 /*                             Initialize()                             */
 /************************************************************************/
 
-bool BAGRasterBand::Initialize( hid_t hDatasetID, const char *pszName )
+bool BAGRasterBand::Initialize( hid_t hDatasetIDIn, const char *pszName )
 
 {
     SetDescription( pszName );
 
-    this->hDatasetID = hDatasetID;
+    hDatasetID = hDatasetIDIn;
 
-    hid_t datatype     = H5Dget_type( hDatasetID );
-    dataspace          = H5Dget_space( hDatasetID );
-    int n_dims         = H5Sget_simple_extent_ndims( dataspace );
-    native             = H5Tget_native_type( datatype, H5T_DIR_ASCEND );
-    hsize_t dims[3], maxdims[3];
+    const hid_t datatype = H5Dget_type( hDatasetIDIn );
+    dataspace = H5Dget_space( hDatasetIDIn );
+    const int n_dims = H5Sget_simple_extent_ndims( dataspace );
+    native = H5Tget_native_type( datatype, H5T_DIR_ASCEND );
+    hsize_t dims[3] = {
+      static_cast<hsize_t>(0),
+      static_cast<hsize_t>(0),
+      static_cast<hsize_t>(0)
+    };
+    hsize_t maxdims[3] = {
+      static_cast<hsize_t>(0),
+      static_cast<hsize_t>(0),
+      static_cast<hsize_t>(0)
+    };
 
     eDataType = GH5_GetDataType( native );
 
@@ -164,8 +164,8 @@ bool BAGRasterBand::Initialize( hid_t hDatasetID, const char *pszName )
     {
         H5Sget_simple_extent_dims( dataspace, dims, maxdims );
 
-        nRasterXSize = (int) dims[1];
-        nRasterYSize = (int) dims[0];
+        nRasterXSize = static_cast<int>(dims[1]);
+        nRasterYSize = static_cast<int>(dims[0]);
     }
     else
     {
@@ -181,27 +181,31 @@ bool BAGRasterBand::Initialize( hid_t hDatasetID, const char *pszName )
 /*      Check for chunksize, and use it as blocksize for optimized      */
 /*      reading.                                                        */
 /* -------------------------------------------------------------------- */
-    hid_t listid = H5Dget_create_plist( hDatasetID );
+    const hid_t listid = H5Dget_create_plist( hDatasetIDIn );
     if (listid>0)
     {
         if(H5Pget_layout(listid) == H5D_CHUNKED)
         {
-            hsize_t panChunkDims[3];
-            int nDimSize = H5Pget_chunk(listid, 3, panChunkDims);
-            nBlockXSize  = (int) panChunkDims[nDimSize-1];
-            nBlockYSize  = (int) panChunkDims[nDimSize-2];
+            hsize_t panChunkDims[3] = {
+              static_cast<hsize_t>(0),
+              static_cast<hsize_t>(0),
+              static_cast<hsize_t>(0)
+            };
+            const int nDimSize = H5Pget_chunk(listid, 3, panChunkDims);
+            nBlockXSize  = static_cast<int>(panChunkDims[nDimSize-1]);
+            nBlockYSize  = static_cast<int>(panChunkDims[nDimSize-2]);
         }
 
         int nfilters = H5Pget_nfilters( listid );
 
-        H5Z_filter_t filter;
-        char         name[120];
+        char name[120];
         size_t       cd_nelmts = 20;
         unsigned int cd_values[20];
         unsigned int flags;
-        for (int i = 0; i < nfilters; i++) 
+        for (int i = 0; i < nfilters; i++)
         {
-          filter = H5Pget_filter(listid, i, &flags, (size_t *)&cd_nelmts, cd_values, 120, name);
+          const H5Z_filter_t filter =
+              H5Pget_filter(listid, i, &flags, &cd_nelmts, cd_values, 120, name);
           if (filter == H5Z_FILTER_DEFLATE)
             poDS->SetMetadataItem( "COMPRESSION", "DEFLATE", "IMAGE_STRUCTURE" );
           else if (filter == H5Z_FILTER_NBIT)
@@ -218,16 +222,16 @@ bool BAGRasterBand::Initialize( hid_t hDatasetID, const char *pszName )
 /* -------------------------------------------------------------------- */
 /*      Load min/max information.                                       */
 /* -------------------------------------------------------------------- */
-    if( EQUAL(pszName,"elevation") 
-        && GH5_FetchAttribute( hDatasetID, "Maximum Elevation Value", 
-                            dfMaximum ) 
-        && GH5_FetchAttribute( hDatasetID, "Minimum Elevation Value", 
+    if( EQUAL(pszName,"elevation")
+        && GH5_FetchAttribute( hDatasetIDIn, "Maximum Elevation Value",
+                            dfMaximum )
+        && GH5_FetchAttribute( hDatasetIDIn, "Minimum Elevation Value",
                                dfMinimum ) )
         bMinMaxSet = true;
-    else if( EQUAL(pszName,"uncertainty") 
-             && GH5_FetchAttribute( hDatasetID, "Maximum Uncertainty Value", 
-                                    dfMaximum ) 
-             && GH5_FetchAttribute( hDatasetID, "Minimum Uncertainty Value", 
+    else if( EQUAL(pszName,"uncertainty")
+             && GH5_FetchAttribute( hDatasetIDIn, "Maximum Uncertainty Value",
+                                    dfMaximum )
+             && GH5_FetchAttribute( hDatasetIDIn, "Minimum Uncertainty Value",
                                     dfMinimum ) )
     {
         /* Some products where uncertainty band is completely set to nodata */
@@ -235,10 +239,10 @@ bool BAGRasterBand::Initialize( hid_t hDatasetID, const char *pszName )
         if( dfMinimum != 0.0 && dfMaximum != 0.0 )
             bMinMaxSet = true;
     }
-    else if( EQUAL(pszName,"nominal_elevation") 
-             && GH5_FetchAttribute( hDatasetID, "max_value", 
-                                    dfMaximum ) 
-             && GH5_FetchAttribute( hDatasetID, "min_value", 
+    else if( EQUAL(pszName,"nominal_elevation")
+             && GH5_FetchAttribute( hDatasetIDIn, "max_value",
+                                    dfMaximum )
+             && GH5_FetchAttribute( hDatasetIDIn, "min_value",
                                     dfMinimum ) )
         bMinMaxSet = true;
 
@@ -258,8 +262,8 @@ double BAGRasterBand::GetMinimum( int * pbSuccess )
             *pbSuccess = TRUE;
         return dfMinimum;
     }
-    else
-        return GDALRasterBand::GetMinimum( pbSuccess );
+
+    return GDALRasterBand::GetMinimum( pbSuccess );
 }
 
 /************************************************************************/
@@ -275,8 +279,8 @@ double BAGRasterBand::GetMaximum( int * pbSuccess )
             *pbSuccess = TRUE;
         return dfMaximum;
     }
-    else
-        return GDALRasterBand::GetMaximum( pbSuccess );
+
+    return GDALRasterBand::GetMaximum( pbSuccess );
 }
 
 /************************************************************************/
@@ -290,12 +294,12 @@ double BAGRasterBand::GetNoDataValue( int * pbSuccess )
 
     if( EQUAL(GetDescription(),"elevation") )
         return  1000000.0;
-    else if( EQUAL(GetDescription(),"uncertainty") )
+    if( EQUAL(GetDescription(),"uncertainty") )
         return 1000000.0;
-    else if( EQUAL(GetDescription(),"nominal_elevation") )
+    if( EQUAL(GetDescription(),"nominal_elevation") )
         return 1000000.0;
-    else
-        return GDALPamRasterBand::GetNoDataValue( pbSuccess );
+
+    return GDALPamRasterBand::GetNoDataValue( pbSuccess );
 }
 
 /************************************************************************/
@@ -304,52 +308,57 @@ double BAGRasterBand::GetNoDataValue( int * pbSuccess )
 CPLErr BAGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                                   void * pImage )
 {
-    herr_t      status;
-    hsize_t     count[3];
-    H5OFFSET_TYPE offset[3];
-    int         nSizeOfData;
-    hid_t       memspace;
-    hsize_t     col_dims[3];
-    hsize_t     rank;
+    H5OFFSET_TYPE offset[3] = {
+      static_cast<H5OFFSET_TYPE>(MAX(0, nRasterYSize - (nBlockYOff+1)*nBlockYSize)),
+      static_cast<H5OFFSET_TYPE>(nBlockXOff * nBlockXSize),
+      static_cast<H5OFFSET_TYPE>(0)
+    };
+    hsize_t count[3] = {
+      static_cast<hsize_t>(nBlockYSize),
+      static_cast<hsize_t>(nBlockXSize),
+      static_cast<hsize_t>(0)
+    };
 
-    rank=2;
-
-    offset[0] = MAX(0,nRasterYSize - (nBlockYOff+1)*nBlockYSize);
-    offset[1] = nBlockXOff*nBlockXSize;
-    count[0]  = nBlockYSize;
-    count[1]  = nBlockXSize;
-
-    nSizeOfData = H5Tget_size( native );
+    const int nSizeOfData = static_cast<int>(H5Tget_size( native ));
     memset( pImage,0,nBlockXSize*nBlockYSize*nSizeOfData );
 
 /*  blocksize may not be a multiple of imagesize */
-    count[0]  = MIN( size_t(nBlockYSize), GetYSize() - offset[0]);
-    count[1]  = MIN( size_t(nBlockXSize), GetXSize() - offset[1]);
+    count[0] = MIN( size_t(nBlockYSize), GetYSize() - offset[0] );
+    count[1] = MIN( size_t(nBlockXSize), GetXSize() - offset[1] );
 
     if( nRasterYSize - (nBlockYOff+1)*nBlockYSize < 0 )
     {
-        count[0] += (nRasterYSize - (nBlockYOff+1)*nBlockYSize);
+        count[0] += (nRasterYSize - (nBlockYOff+1)*nBlockYSize );
     }
 
 /* -------------------------------------------------------------------- */
 /*      Select block from file space                                    */
 /* -------------------------------------------------------------------- */
-    status =  H5Sselect_hyperslab( dataspace,
-                                   H5S_SELECT_SET,
-                                   offset, NULL,
-                                   count, NULL );
+    herr_t status =  H5Sselect_hyperslab( dataspace,
+                                          H5S_SELECT_SET,
+                                          offset, NULL,
+                                          count, NULL );
+    if( status < 0 )
+        return CE_Failure;
 
 /* -------------------------------------------------------------------- */
 /*      Create memory space to receive the data                         */
 /* -------------------------------------------------------------------- */
-    col_dims[0]=nBlockYSize;
-    col_dims[1]=nBlockXSize;
-    memspace = H5Screate_simple( (int) rank, col_dims, NULL );
+    hsize_t col_dims[3] = {
+      static_cast<hsize_t>(nBlockYSize),
+      static_cast<hsize_t>(nBlockXSize),
+      static_cast<hsize_t>(0)
+    };
+    hsize_t rank = 2;
+    const hid_t memspace =
+        H5Screate_simple( static_cast<int>(rank), col_dims, NULL );
     H5OFFSET_TYPE mem_offset[3] = {0, 0, 0};
     status =  H5Sselect_hyperslab(memspace,
                                   H5S_SELECT_SET,
                                   mem_offset, NULL,
                                   count, NULL);
+    if( status < 0 )
+        return CE_Failure;
 
     status = H5Dread ( hDatasetID,
                        native,
@@ -363,19 +372,21 @@ CPLErr BAGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 /* -------------------------------------------------------------------- */
 /*      Y flip the data.                                                */
 /* -------------------------------------------------------------------- */
-    int nLinesToFlip = count[0];
-    int nLineSize = nSizeOfData * nBlockXSize;
-    GByte *pabyTemp = (GByte *) CPLMalloc(nLineSize);
+    const int nLinesToFlip = static_cast<int>(count[0]);
+    const int nLineSize = nSizeOfData * nBlockXSize;
+    GByte *pabyTemp = static_cast<GByte *>(CPLMalloc(nLineSize));
 
     for( int iY = 0; iY < nLinesToFlip/2; iY++ )
     {
-        memcpy( pabyTemp, 
-                ((GByte *)pImage) + iY * nLineSize,
+        memcpy( pabyTemp,
+                reinterpret_cast<GByte *>(pImage) + iY * nLineSize,
                 nLineSize );
-        memcpy( ((GByte *)pImage) + iY * nLineSize,
-                ((GByte *)pImage) + (nLinesToFlip-iY-1) * nLineSize,
+        memcpy( reinterpret_cast<GByte *>(pImage) + iY * nLineSize,
+                (reinterpret_cast<GByte *>(pImage) + (nLinesToFlip-iY-1) *
+                 nLineSize),
                 nLineSize );
-        memcpy( ((GByte *)pImage) + (nLinesToFlip-iY-1) * nLineSize,
+        memcpy( (reinterpret_cast<GByte *>(pImage) + (nLinesToFlip-iY-1) *
+                 nLineSize),
                 pabyTemp,
                 nLineSize );
     }
@@ -391,8 +402,8 @@ CPLErr BAGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                   "H5Dread() failed for block." );
         return CE_Failure;
     }
-    else
-        return CE_None;
+
+    return CE_None;
 }
 
 /************************************************************************/
@@ -405,18 +416,20 @@ CPLErr BAGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 /*                             BAGDataset()                             */
 /************************************************************************/
 
-BAGDataset::BAGDataset()
+BAGDataset::BAGDataset() :
+    hHDF5(-1),
+    pszProjection(NULL),
+    pszXMLMetadata(NULL)
 {
-    hHDF5 = -1;
-    pszXMLMetadata = NULL;
-    pszProjection = NULL;
-
     adfGeoTransform[0] = 0.0;
     adfGeoTransform[1] = 1.0;
     adfGeoTransform[2] = 0.0;
     adfGeoTransform[3] = 0.0;
     adfGeoTransform[4] = 0.0;
     adfGeoTransform[5] = 1.0;
+
+    apszMDList[0] = NULL;
+    apszMDList[1] = NULL;
 }
 
 /************************************************************************/
@@ -429,8 +442,8 @@ BAGDataset::~BAGDataset( )
     if( hHDF5 >= 0 )
         H5Fclose( hHDF5 );
 
-    CPLFree( pszXMLMetadata );
     CPLFree( pszProjection );
+    CPLFree( pszXMLMetadata );
 }
 
 /************************************************************************/
@@ -476,29 +489,27 @@ GDALDataset *BAGDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->eAccess == GA_Update )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The BAG driver does not support update access." );
         return NULL;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Open the file as an HDF5 file.                                  */
 /* -------------------------------------------------------------------- */
-    hid_t hHDF5 = H5Fopen( poOpenInfo->pszFilename, 
+    hid_t hHDF5 = H5Fopen( poOpenInfo->pszFilename,
                            H5F_ACC_RDONLY, H5P_DEFAULT );
 
-    if( hHDF5 < 0 )  
+    if( hHDF5 < 0 )
         return NULL;
 
 /* -------------------------------------------------------------------- */
 /*      Confirm it is a BAG dataset by checking for the                 */
 /*      BAG_Root/Bag Version attribute.                                 */
 /* -------------------------------------------------------------------- */
-    hid_t hBagRoot = H5Gopen( hHDF5, "/BAG_root" );
-    hid_t hVersion = -1;
-
-    if( hBagRoot >= 0 )
-        hVersion = H5Aopen_name( hBagRoot, "Bag Version" );
+    const hid_t hBagRoot = H5Gopen( hHDF5, "/BAG_root" );
+    const hid_t hVersion =
+        ( hBagRoot >= 0 ) ? H5Aopen_name( hBagRoot, "Bag Version" ) : -1;
 
     if( hVersion < 0 )
     {
@@ -512,7 +523,7 @@ GDALDataset *BAGDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding dataset.                                 */
 /* -------------------------------------------------------------------- */
-    BAGDataset *poDS = new BAGDataset();
+    BAGDataset * const poDS = new BAGDataset();
 
     poDS->hHDF5 = hHDF5;
 
@@ -530,7 +541,7 @@ GDALDataset *BAGDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Fetch the elevation dataset and attach as a band.               */
 /* -------------------------------------------------------------------- */
     int nNextBand = 1;
-    hid_t hElevation = H5Dopen( hHDF5, "/BAG_root/elevation" );
+    const hid_t hElevation = H5Dopen( hHDF5, "/BAG_root/elevation" );
     if( hElevation < 0 )
     {
         delete poDS;
@@ -554,7 +565,7 @@ GDALDataset *BAGDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Try to do the same for the uncertainty band.                    */
 /* -------------------------------------------------------------------- */
-    hid_t hUncertainty = H5Dopen( hHDF5, "/BAG_root/uncertainty" );
+    const hid_t hUncertainty = H5Dopen( hHDF5, "/BAG_root/uncertainty" );
     BAGRasterBand *poUBand = new BAGRasterBand( poDS, nNextBand );
 
     if( hUncertainty >= 0 && poUBand->Initialize( hUncertainty, "uncertainty") )
@@ -573,7 +584,7 @@ GDALDataset *BAGDataset::Open( GDALOpenInfo * poOpenInfo )
         hNominal = H5Dopen( hHDF5, "/BAG_root/nominal_elevation" );
     } H5E_END_TRY;
 
-    BAGRasterBand *poNBand = new BAGRasterBand( poDS, nNextBand );
+    BAGRasterBand * const poNBand = new BAGRasterBand( poDS, nNextBand );
     if( hNominal >= 0 && poNBand->Initialize( hNominal,
                                               "nominal_elevation" ) )
     {
@@ -581,7 +592,7 @@ GDALDataset *BAGDataset::Open( GDALOpenInfo * poOpenInfo )
     }
     else
         delete poNBand;
-        
+
 /* -------------------------------------------------------------------- */
 /*      Load the XML metadata.                                          */
 /* -------------------------------------------------------------------- */
@@ -611,15 +622,25 @@ void BAGDataset::LoadMetadata()
 /* -------------------------------------------------------------------- */
 /*      Load the metadata from the file.                                */
 /* -------------------------------------------------------------------- */
-    hid_t hMDDS = H5Dopen( hHDF5, "/BAG_root/metadata" );
-    hid_t datatype     = H5Dget_type( hMDDS );
-    hid_t dataspace    = H5Dget_space( hMDDS );
-    hid_t native       = H5Tget_native_type( datatype, H5T_DIR_ASCEND );
-    hsize_t dims[3], maxdims[3];
+    const hid_t hMDDS = H5Dopen( hHDF5, "/BAG_root/metadata" );
+    const hid_t datatype = H5Dget_type( hMDDS );
+    const hid_t dataspace = H5Dget_space( hMDDS );
+    const hid_t native = H5Tget_native_type( datatype, H5T_DIR_ASCEND );
+    hsize_t dims[3] = {
+      static_cast<hsize_t>(0),
+      static_cast<hsize_t>(0),
+      static_cast<hsize_t>(0)
+    };
+    hsize_t maxdims[3] = {
+      static_cast<hsize_t>(0),
+      static_cast<hsize_t>(0),
+      static_cast<hsize_t>(0)
+    };
 
     H5Sget_simple_extent_dims( dataspace, dims, maxdims );
 
-    pszXMLMetadata = (char *) CPLCalloc((int) (dims[0]+1),1);
+    pszXMLMetadata = static_cast<char *>(
+        CPLCalloc(static_cast<int>(dims[0]+1),1));
 
     H5Dread( hMDDS, native, H5S_ALL, dataspace, H5P_DEFAULT, pszXMLMetadata );
 
@@ -641,21 +662,21 @@ void BAGDataset::LoadMetadata()
 
     CPLStripXMLNamespace( psRoot, NULL, TRUE );
 
-    CPLXMLNode *psGeo = CPLSearchXMLNode( psRoot, "=MD_Georectified" );
+    CPLXMLNode * const psGeo = CPLSearchXMLNode( psRoot, "=MD_Georectified" );
 
     if( psGeo != NULL )
     {
-        char **papszCornerTokens = 
-            CSLTokenizeStringComplex( 
+        char **papszCornerTokens =
+            CSLTokenizeStringComplex(
                 CPLGetXMLValue( psGeo, "cornerPoints.Point.coordinates", "" ),
                 " ,", FALSE, FALSE );
 
         if( CSLCount(papszCornerTokens ) == 4 )
         {
-            double dfLLX = atof( papszCornerTokens[0] );
-            double dfLLY = atof( papszCornerTokens[1] );
-            double dfURX = atof( papszCornerTokens[2] );
-            double dfURY = atof( papszCornerTokens[3] );
+            const double dfLLX = CPLAtof( papszCornerTokens[0] );
+            const double dfLLY = CPLAtof( papszCornerTokens[1] );
+            const double dfURX = CPLAtof( papszCornerTokens[2] );
+            const double dfURY = CPLAtof( papszCornerTokens[3] );
 
             adfGeoTransform[0] = dfLLX;
             adfGeoTransform[1] = (dfURX - dfLLX) / (GetRasterXSize()-1);
@@ -677,7 +698,7 @@ void BAGDataset::LoadMetadata()
         == OGRERR_NONE )
     {
         oSRS.exportToWkt( &pszProjection );
-    } 
+    }
     else
     {
         ParseWKTFromXML( pszXMLMetadata );
@@ -686,7 +707,7 @@ void BAGDataset::LoadMetadata()
 /* -------------------------------------------------------------------- */
 /*      Fetch acquisition date.                                         */
 /* -------------------------------------------------------------------- */
-    CPLXMLNode *psDateTime = CPLSearchXMLNode( psRoot, "=dateTime" );
+    CPLXMLNode * const psDateTime = CPLSearchXMLNode( psRoot, "=dateTime" );
     if( psDateTime != NULL )
     {
         const char *pszDateTimeValue = CPLGetXMLValue( psDateTime, NULL, "" );
@@ -702,14 +723,13 @@ void BAGDataset::LoadMetadata()
 /************************************************************************/
 OGRErr BAGDataset::ParseWKTFromXML( const char *pszISOXML )
 {
-    OGRSpatialReference oSRS;
-    CPLXMLNode *psRoot = CPLParseXMLString( pszISOXML );
+    CPLXMLNode * const psRoot = CPLParseXMLString( pszISOXML );
     OGRErr eOGRErr = OGRERR_FAILURE;
 
     if( psRoot == NULL )
         return eOGRErr;
 
-    CPLStripXMLNamespace( psRoot, NULL, TRUE ); 
+    CPLStripXMLNamespace( psRoot, NULL, TRUE );
 
     CPLXMLNode *psRSI = CPLSearchXMLNode( psRoot, "=referenceSystemInfo" );
     if( psRSI == NULL )
@@ -720,20 +740,25 @@ OGRErr BAGDataset::ParseWKTFromXML( const char *pszISOXML )
         return eOGRErr;
     }
 
+    OGRSpatialReference oSRS;
     oSRS.Clear();
 
-    const char *pszSRCodeString = 
-        CPLGetXMLValue( psRSI, "MD_ReferenceSystem.referenceSystemIdentifier.RS_Identifier.code.CharacterString", NULL );
+    const char *pszSRCodeString =
+        CPLGetXMLValue( psRSI, "MD_ReferenceSystem.referenceSystemIdentifier."
+                        "RS_Identifier.code.CharacterString", NULL );
     if( pszSRCodeString == NULL )
     {
         CPLDebug("BAG",
-          "Unable to find /MI_Metadata/referenceSystemInfo[1]/MD_ReferenceSystem[1]/referenceSystemIdentifier[1]/RS_Identifier[1]/code[1]/CharacterString[1] in metadata." );
+                 "Unable to find /MI_Metadata/referenceSystemInfo[1]/"
+                 "MD_ReferenceSystem[1]/referenceSystemIdentifier[1]/"
+                 "RS_Identifier[1]/code[1]/CharacterString[1] in metadata." );
         CPLDestroyXMLNode( psRoot );
         return eOGRErr;
     }
-    
-    const char *pszSRCodeSpace = 
-        CPLGetXMLValue( psRSI, "MD_ReferenceSystem.referenceSystemIdentifier.RS_Identifier.codeSpace.CharacterString", "" );
+
+    const char *pszSRCodeSpace =
+        CPLGetXMLValue( psRSI, "MD_ReferenceSystem.referenceSystemIdentifier."
+                        "RS_Identifier.codeSpace.CharacterString", "" );
     if( !EQUAL( pszSRCodeSpace, "WKT" ) )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
@@ -742,7 +767,7 @@ OGRErr BAGDataset::ParseWKTFromXML( const char *pszISOXML )
         return eOGRErr;
     }
 
-    char* pszWKT = const_cast< char* >( pszSRCodeString );
+    char* pszWKT = const_cast<char *>( pszSRCodeString );
     if( oSRS.importFromWkt( &pszWKT ) != OGRERR_NONE )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
@@ -758,23 +783,28 @@ OGRErr BAGDataset::ParseWKTFromXML( const char *pszISOXML )
     if( psRSI == NULL )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
-            "Unable to find second instance of <referenceSystemInfo> in metadata." );
+                  "Unable to find second instance of <referenceSystemInfo> "
+                  "in metadata." );
         CPLDestroyXMLNode( psRoot );
         return eOGRErr;
     }
 
-    pszSRCodeString = 
-      CPLGetXMLValue( psRSI, "MD_ReferenceSystem.referenceSystemIdentifier.RS_Identifier.code.CharacterString", NULL );
+    pszSRCodeString =
+      CPLGetXMLValue( psRSI, "MD_ReferenceSystem.referenceSystemIdentifier."
+                      "RS_Identifier.code.CharacterString", NULL );
     if( pszSRCodeString == NULL )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
-            "Unable to find /MI_Metadata/referenceSystemInfo[2]/MD_ReferenceSystem[1]/referenceSystemIdentifier[1]/RS_Identifier[1]/code[1]/CharacterString[1] in metadata." );
+            "Unable to find /MI_Metadata/referenceSystemInfo[2]/"
+                  "MD_ReferenceSystem[1]/referenceSystemIdentifier[1]/"
+                  "RS_Identifier[1]/code[1]/CharacterString[1] in metadata." );
         CPLDestroyXMLNode( psRoot );
         return eOGRErr;
     }
 
-    pszSRCodeSpace = 
-        CPLGetXMLValue( psRSI, "MD_ReferenceSystem.referenceSystemIdentifier.RS_Identifier.codeSpace.CharacterString", "" );
+    pszSRCodeSpace =
+        CPLGetXMLValue( psRSI, "MD_ReferenceSystem.referenceSystemIdentifier."
+                        "RS_Identifier.codeSpace.CharacterString", "" );
     if( !EQUAL( pszSRCodeSpace, "WKT" ) )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
@@ -783,18 +813,17 @@ OGRErr BAGDataset::ParseWKTFromXML( const char *pszISOXML )
         return eOGRErr;
     }
 
-    if( EQUALN(pszSRCodeString, "VERTCS", 6 ) )
+    if( STARTS_WITH_CI(pszSRCodeString, "VERTCS") )
     {
         CPLString oString( pszProjection );
+        CPLFree( pszProjection );
         oString += ",";
         oString += pszSRCodeString;
-        if ( pszProjection )
-            CPLFree( pszProjection );
         pszProjection = CPLStrdup( oString );
     }
 
     CPLDestroyXMLNode( psRoot );
-    
+
     return eOGRErr;
 }
 
@@ -810,8 +839,8 @@ CPLErr BAGDataset::GetGeoTransform( double *padfGeoTransform )
         memcpy( padfGeoTransform, adfGeoTransform, sizeof(double)*6 );
         return CE_None;
     }
-    else
-        return GDALPamDataset::GetGeoTransform( padfGeoTransform );
+
+    return GDALPamDataset::GetGeoTransform( padfGeoTransform );
 }
 
 /************************************************************************/
@@ -823,8 +852,8 @@ const char *BAGDataset::GetProjectionRef()
 {
     if( pszProjection )
         return pszProjection;
-    else 
-        return GDALPamDataset::GetProjectionRef();
+
+    return GDALPamDataset::GetProjectionRef();
 }
 
 /************************************************************************/
@@ -852,8 +881,8 @@ char **BAGDataset::GetMetadata( const char *pszDomain )
 
         return apszMDList;
     }
-    else
-        return GDALPamDataset::GetMetadata( pszDomain );
+
+    return GDALPamDataset::GetMetadata( pszDomain );
 }
 
 /************************************************************************/
@@ -862,23 +891,21 @@ char **BAGDataset::GetMetadata( const char *pszDomain )
 void GDALRegister_BAG( )
 
 {
-    GDALDriver  *poDriver;
-    
-    if (! GDAL_CHECK_VERSION("BAG"))
+    if( !GDAL_CHECK_VERSION( "BAG" ) )
         return;
 
-    if(  GDALGetDriverByName( "BAG" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "BAG" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "Bathymetry Attributed Grid" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_bag.html" );
-        poDriver->pfnOpen = BAGDataset::Open;
-        poDriver->pfnIdentify = BAGDataset::Identify;
+    if( GDALGetDriverByName( "BAG" ) != NULL )
+        return;
 
-        GetGDALDriverManager( )->RegisterDriver( poDriver );
-    }
+    GDALDriver *poDriver = new GDALDriver();
+
+    poDriver->SetDescription( "BAG" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
+                               "Bathymetry Attributed Grid" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_bag.html" );
+    poDriver->pfnOpen = BAGDataset::Open;
+    poDriver->pfnIdentify = BAGDataset::Identify;
+
+    GetGDALDriverManager( )->RegisterDriver( poDriver );
 }
