@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hdf5dataset.h 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: hdf5dataset.h 31777 2015-11-26 14:14:41Z rouault $
  *
  * Project:  Hierarchical Data Format Release 5 (HDF5)
  * Purpose:  Header file for HDF5 datasets reader.
@@ -28,8 +28,8 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _HDF5DATASET_H_INCLUDED_
-#define _HDF5DATASET_H_INCLUDED_
+#ifndef HDF5DATASET_H_INCLUDED_
+#define HDF5DATASET_H_INCLUDED_
 
 #include "gdal_pam.h"
 #include "cpl_list.h"
@@ -62,7 +62,7 @@ herr_t HDF5CreateGroupObjs(hid_t, const char *,void *);
 /************************************************************************/
 class HDF5Dataset : public GDALPamDataset
 {
-  protected:
+protected:
 
   hid_t            hHDF5;
   hid_t            hGroupID; /* H handler interface */
@@ -71,7 +71,6 @@ class HDF5Dataset : public GDALPamDataset
   int              nDatasetType;
   int              nSubDataCount;
 
-
   HDF5GroupObjects *poH5RootGroup; /* Contain hdf5 Groups information */
 
   CPLErr ReadGlobalAttributes(int);
@@ -97,14 +96,14 @@ class HDF5Dataset : public GDALPamDataset
    * 			the attribute name must be the form:
    * 					root attribute name
    * 					SUBDATASET/subdataset attribute name
-   * @param pdfValues pointer wich will store the array of doubles read.
+   * @param pdfValues pointer which will store the array of doubles read.
    * @param nLen it stores the length of the array read. If NULL it doesn't inform
-   *        the lenght of the array.
+   *        the length of the array.
    * @return CPLErr CE_None in case of success, CE_Failure in case of failure
    */
   CPLErr HDF5ReadDoubleAttr(const char* pszAttrName,double **pdfValues,int *nLen=NULL);
 
-  public:
+public:
 
   char	           **papszMetadata;
   HDF5GroupObjects *poH5CurrentObject;
@@ -116,7 +115,4 @@ class HDF5Dataset : public GDALPamDataset
   static int Identify(GDALOpenInfo *);
 };
 
-
-
-#endif /* _HDF5DATASET_H_INCLUDED_ */
-
+#endif /* HDF5DATASET_H_INCLUDED_ */
