@@ -1,10 +1,10 @@
 /******************************************************************************
- * $Id: hfa.h 21687 2011-02-12 03:59:15Z warmerdam $
+ * $Id: hfa.h 33720 2016-03-15 00:39:53Z goatbar $
  *
  * Project:  Erdas Imagine (.img) Translator
  * Purpose:  Public (C callable) interface for the Erdas Imagine reading
  *           code.  This include files, and it's implementing code depends
- *           on CPL, but not GDAL. 
+ *           on CPL, but not GDAL.
  * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
  ******************************************************************************
@@ -29,8 +29,8 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _HFAOPEN_H_INCLUDED
-#define _HFAOPEN_H_INCLUDED
+#ifndef HFAOPEN_H_INCLUDED
+#define HFAOPEN_H_INCLUDED
 
 /* -------------------------------------------------------------------- */
 /*      Include standard portability stuff.                             */
@@ -38,11 +38,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-#ifdef HFA_PRIVATE
-typedef HFAInfo_t *HFAHandle;
-#else
-typedef void *HFAHandle;
-#endif
+typedef struct hfainfo *HFAHandle;
 
 /* -------------------------------------------------------------------- */
 /*      Structure definitions from eprj.h, with some type               */
@@ -109,12 +105,12 @@ typedef struct {
 
 typedef struct {
 	Eprj_ProType proType;		/* projection type */
-	long proNumber;			/* projection number for internal 
+	int proNumber;			/* projection number for internal
 					   projections */
 	char * proExeName;	/* projection executable name for
 					   EXTERNal projections */
 	char * proName;	/* projection name */
-	long proZone;			/* projection zone (UTM, SP only) */
+	int proZone;			/* projection zone (UTM, SP only) */
 	double proParams[15];	/* projection parameters array in the
 					   GCTP form */
 	Eprj_Spheroid proSpheroid;	/* projection spheroid */
@@ -126,6 +122,28 @@ typedef struct {
     double      polycoefvector[2];
 } Efga_Polynomial;
 
+/* -------------------------------------------------------------------- */
+/*      data types.                                                     */
+/* -------------------------------------------------------------------- */
+typedef enum
+{
+    EPT_MIN = 0,
+    EPT_u1 = 0,
+    EPT_u2 = 1,
+    EPT_u4 = 2,
+    EPT_u8 = 3,
+    EPT_s8 = 4,
+    EPT_u16 = 5,
+    EPT_s16 = 6,
+    EPT_u32 = 7,
+    EPT_s32 = 8,
+    EPT_f32 = 9,
+    EPT_f64 = 10,
+    EPT_c64 = 11,
+    EPT_c128 = 12,
+    EPT_MAX = EPT_c128
+} EPTType;
+
 /* -------------------------------------------------------------------- */
 /*      Prototypes                                                      */
 /* -------------------------------------------------------------------- */
@@ -133,13 +151,13 @@ typedef struct {
 CPL_C_START
 
 HFAHandle CPL_DLL HFAOpen( const char * pszFilename, const char * pszMode );
-void	CPL_DLL HFAClose( HFAHandle );
+int	CPL_DLL HFAClose( HFAHandle ); /* 0 = success */
 CPLErr HFADelete( const char *pszFilename );
 CPLErr HFARenameReferences( HFAHandle, const char *, const char * );
 
 HFAHandle CPL_DLL HFACreateLL( const char *pszFilename );
-HFAHandle CPL_DLL HFACreate( const char *pszFilename, int nXSize, int nYSize, 
-                             int nBands, int nDataType, char ** papszOptions );
+HFAHandle CPL_DLL HFACreate( const char *pszFilename, int nXSize, int nYSize,
+                             int nBands, EPTType eDataType, char ** papszOptions );
 const char CPL_DLL *HFAGetIGEFilename( HFAHandle );
 CPLErr  CPL_DLL HFAFlush( HFAHandle );
 int CPL_DLL HFACreateOverview( HFAHandle hHFA, int nBand, int nOverviewLevel,
@@ -158,37 +176,37 @@ CPLErr CPL_DLL HFASetProParameters( HFAHandle, const Eprj_ProParameters * );
 
 CPLErr CPL_DLL HFAGetRasterInfo( HFAHandle hHFA, int *pnXSize, int *pnYSize,
                                  int *pnBands );
-CPLErr CPL_DLL HFAGetBandInfo( HFAHandle hHFA, int nBand, int * pnDataType,
-                               int * pnBlockXSize, int * pnBlockYSize, 
+CPLErr CPL_DLL HFAGetBandInfo( HFAHandle hHFA, int nBand, EPTType* peDataType,
+                               int * pnBlockXSize, int * pnBlockYSize,
                                int *pnCompressionType );
 int    CPL_DLL HFAGetBandNoData( HFAHandle hHFA, int nBand, double *pdfValue );
 CPLErr CPL_DLL HFASetBandNoData( HFAHandle hHFA, int nBand, double dfValue );
 int    CPL_DLL HFAGetOverviewCount( HFAHandle hHFA, int nBand );
-CPLErr CPL_DLL HFAGetOverviewInfo( HFAHandle hHFA, int nBand, int nOverview, 
+CPLErr CPL_DLL HFAGetOverviewInfo( HFAHandle hHFA, int nBand, int nOverview,
                                    int * pnXSize, int * pnYSize,
                                    int * pnBlockXSize, int * pnBlockYSize,
-                                   int * pnHFADataType );
-CPLErr CPL_DLL HFAGetRasterBlock( HFAHandle hHFA, int nBand, int nXBlock, 
+                                   EPTType * peHFADataType );
+CPLErr CPL_DLL HFAGetRasterBlock( HFAHandle hHFA, int nBand, int nXBlock,
                                   int nYBlock, void * pData );
-CPLErr CPL_DLL HFAGetRasterBlockEx( HFAHandle hHFA, int nBand, int nXBlock, 
+CPLErr CPL_DLL HFAGetRasterBlockEx( HFAHandle hHFA, int nBand, int nXBlock,
                                     int nYBlock, void * pData, int nDataSize );
-CPLErr CPL_DLL HFAGetOverviewRasterBlock( HFAHandle hHFA, int nBand, 
+CPLErr CPL_DLL HFAGetOverviewRasterBlock( HFAHandle hHFA, int nBand,
                                           int iOverview,
                                    int nXBlock, int nYBlock, void * pData );
-CPLErr CPL_DLL HFAGetOverviewRasterBlockEx( HFAHandle hHFA, int nBand, 
+CPLErr CPL_DLL HFAGetOverviewRasterBlockEx( HFAHandle hHFA, int nBand,
                                           int iOverview,
                                    int nXBlock, int nYBlock, void * pData, int nDataSize );
-CPLErr CPL_DLL HFASetRasterBlock( HFAHandle hHFA, int nBand, 
+CPLErr CPL_DLL HFASetRasterBlock( HFAHandle hHFA, int nBand,
                                   int nXBlock, int nYBlock,
                                   void * pData );
-CPLErr CPL_DLL HFASetOverviewRasterBlock( 
-    HFAHandle hHFA, int nBand, int iOverview,int nXBlock, int nYBlock, 
+CPLErr CPL_DLL HFASetOverviewRasterBlock(
+    HFAHandle hHFA, int nBand, int iOverview,int nXBlock, int nYBlock,
     void * pData );
 const char * HFAGetBandName( HFAHandle hHFA, int nBand );
 void HFASetBandName( HFAHandle hHFA, int nBand, const char *pszName );
-int     CPL_DLL HFAGetDataTypeBits( int );
-const char CPL_DLL *HFAGetDataTypeName( int );
-CPLErr	CPL_DLL HFAGetPCT( HFAHandle, int, int *, 
+int     CPL_DLL HFAGetDataTypeBits( EPTType eDataType );
+const char CPL_DLL *HFAGetDataTypeName( EPTType eDataType );
+CPLErr	CPL_DLL HFAGetPCT( HFAHandle, int, int *,
                            double **, double **, double ** , double **,
                            double **);
 CPLErr  CPL_DLL HFASetPCT( HFAHandle, int, int, double *, double *, double *, double * );
@@ -198,57 +216,22 @@ CPLErr  CPL_DLL HFAGetDataRange( HFAHandle, int, double *, double * );
 char  CPL_DLL **HFAGetMetadata( HFAHandle hHFA, int nBand );
 CPLErr  CPL_DLL HFASetMetadata( HFAHandle hHFA, int nBand, char ** );
 char  CPL_DLL **HFAGetClassNames( HFAHandle hHFA, int nBand );
-int CPL_DLL 
-HFACreateLayer( HFAHandle psInfo, HFAEntry *poParent,
-                const char *pszLayerName,
-                int bOverview, int nBlockSize, 
-                int bCreateCompressed, int bCreateLargeRaster,
-                int bDependentLayer,
-                int nXSize, int nYSize, int nDataType, 
-                char **papszOptions,
-                
-                // these are only related to external (large) files
-                GIntBig nStackValidFlagsOffset, 
-                GIntBig nStackDataOffset,
-                int nStackCount, int nStackIndex );
 
 int CPL_DLL
-HFAReadXFormStack( HFAHandle psInfo, 
+HFAReadXFormStack( HFAHandle psInfo,
                    Efga_Polynomial **ppasPolyListForward,
                    Efga_Polynomial **ppasPolyListReverse );
 CPLErr CPL_DLL
 HFAWriteXFormStack( HFAHandle psInfo, int nBand, int nXFormCount,
                     Efga_Polynomial **ppasPolyListForward,
                     Efga_Polynomial **ppasPolyListReverse );
-int CPL_DLL 
+int CPL_DLL
 HFAEvaluateXFormStack( int nStepCount, int bForward,
                        Efga_Polynomial *pasPolyList,
                        double *pdfX, double *pdfY );
 
 char CPL_DLL **HFAReadCameraModel( HFAHandle psInfo );
 
-char *
-HFAPCSStructToWKT( const Eprj_Datum *psDatum,
-                   const Eprj_ProParameters *psPro,
-                   const Eprj_MapInfo *psMapInfo,
-                   HFAEntry *poMapInformation );
-
-/* -------------------------------------------------------------------- */
-/*      data types.                                                     */
-/* -------------------------------------------------------------------- */
-#define EPT_u1	0
-#define EPT_u2	1
-#define EPT_u4	2
-#define EPT_u8	3
-#define EPT_s8	4
-#define EPT_u16	5
-#define EPT_s16	6
-#define EPT_u32	7
-#define EPT_s32	8
-#define EPT_f32	9
-#define EPT_f64	10
-#define EPT_c64	11
-#define EPT_c128 12
 
 /* -------------------------------------------------------------------- */
 /*      Projection codes.                                               */
@@ -330,4 +313,4 @@ HFAPCSStructToWKT( const Eprj_Datum *psDatum,
 
 CPL_C_END
 
-#endif /* ndef _HFAOPEN_H_INCLUDED */
+#endif /* ndef HFAOPEN_H_INCLUDED */
