@@ -1,5 +1,5 @@
 /*****************************************************************************
-* $Id: IntergraphBand.h 27044 2014-03-16 23:41:27Z rouault $
+* $Id: IntergraphBand.h 33720 2016-03-15 00:39:53Z goatbar $
 *
 * Project:  Intergraph Raster Format support
 * Purpose:  Read selected types of Intergraph Raster Format
@@ -50,7 +50,7 @@ protected:
     int             nFullBlocksX;
     int             nFullBlocksY;
 
-    GByte	       *pabyBlockBuf;
+    GByte	   *pabyBlockBuf;
     uint32          nTiles;
 
     INGR_TileItem  *pahTiles;
@@ -58,29 +58,29 @@ protected:
     INGR_HeaderOne  hHeaderOne;
     INGR_HeaderTwoA hHeaderTwo;
     INGR_TileHeader hTileDir;
-    
+
     int             nRLEOffset;
 
 public:
-    IntergraphRasterBand( IntergraphDataset *poDS, 
+    IntergraphRasterBand( IntergraphDataset *poDS,
         int nBand,
         int nBandOffset,
         GDALDataType eType = GDT_Unknown);
     ~IntergraphRasterBand();
 
     virtual double GetMinimum( int *pbSuccess = NULL );
-    virtual double GetMaximum( int *pbSuccess = NULL );    
+    virtual double GetMaximum( int *pbSuccess = NULL );
     virtual GDALColorTable *GetColorTable();
     virtual GDALColorInterp GetColorInterpretation();
     virtual CPLErr IReadBlock( int nBlockXOff, int nBlockYOff, void *pImage );
     virtual CPLErr IWriteBlock( int nBlockXOff, int nBlockYOff, void *pImage );
-    virtual CPLErr SetColorTable( GDALColorTable *poColorTable ); 
+    virtual CPLErr SetColorTable( GDALColorTable *poColorTable );
     virtual CPLErr SetStatistics( double dfMin, double dfMax, double dfMean, double dfStdDev );
 
 protected:
     int  HandleUninstantiatedTile( int nBlockXOff, int nBlockYOff, void* pImage);
     int  LoadBlockBuf( int nBlockXOff, int nBlockYOff, int nBlockBytes, GByte *pabyBlock );
-    void ReshapeBlock( int nBlockXOff, int nBlockYOff, int nBlockBytes, GByte *pabyBlock );
+    bool ReshapeBlock( int nBlockXOff, int nBlockYOff, int nBlockBytes, GByte *pabyBlock );
     void FlushBandHeader( void );
     void BlackWhiteCT( bool bReverse = false );
 };
@@ -92,7 +92,7 @@ protected:
 class IntergraphRGBBand : public IntergraphRasterBand
 {
 public:
-    IntergraphRGBBand( IntergraphDataset *poDS, 
+    IntergraphRGBBand( IntergraphDataset *poDS,
         int nBand,
         int nBandOffset,
         int nRGorB );
@@ -115,7 +115,7 @@ private:
     int             nRGBBand;
 
 public:
-    IntergraphBitmapBand( IntergraphDataset *poDS, 
+    IntergraphBitmapBand( IntergraphDataset *poDS,
         int nBand,
         int nBandOffset,
         int nRGorB = 1 );
@@ -140,7 +140,7 @@ private:
     uint32         *panRLELineOffset;
 
 public:
-    IntergraphRLEBand( IntergraphDataset *poDS, 
+    IntergraphRLEBand( IntergraphDataset *poDS,
         int nBand,
         int nBandOffset,
         int nRGorB = 0);
