@@ -2,7 +2,7 @@
  * $Id:  $
  *
  * Project:  Creates a jpeg header
- * Purpose:  Abreviated JPEG support
+ * Purpose:  Abbreviated JPEG support
  * Author:   Ivan Lucena, [lucena_ivan at hotmail.com]
  *
  ******************************************************************************
@@ -124,7 +124,7 @@ static const GByte JPGHLP_DQT_chrominance[64] = {
      59, 59, 59, 59, 59, 59, 59, 59
 };
 
-static const GByte ZIGZAG[64] = { 
+static const GByte ZIGZAG[64] = {
       0,  1,  5,  6, 14, 15, 27, 28,
       2,  4,  7, 13, 16, 26, 29, 42,
       3,  8, 12, 17, 25, 30, 41, 43,
@@ -142,14 +142,12 @@ static const GByte ZIGZAG[64] = {
   { int i; for( i = 0; i < 64; i++ ) tb[i] = (GByte) (tb[i] op vl); }
 
 int JPGHLP_HeaderMaker( GByte *pabyBuffer,
-                        const int nCols, 
-                        const int nRows, 
-                        const int nComponents, 
+                        const int nCols,
+                        const int nRows,
+                        const int nComponents,
                         CPL_UNUSED const int nRestart,
                         const int nQuality )
 {
-    int i;
-
     GByte *pabNext = pabyBuffer;
 
     // ------------------------------------------------------------------------
@@ -163,7 +161,7 @@ int JPGHLP_HeaderMaker( GByte *pabyBuffer,
     // Application Segment
     // ------------------------------------------------------------------------
 
-    *( pabNext++ )      = 0xFF;         // Tag Mark      
+    *( pabNext++ )      = 0xFF;         // Tag Mark
     *( pabNext++ )      = 0xE0;         // APP0
     *( pabNext++ )      = 0x00;         // Segment Length (msb)
     *( pabNext++ )      = 0x10;         // Segment Length (lsb)
@@ -196,7 +194,7 @@ int JPGHLP_HeaderMaker( GByte *pabyBuffer,
         ADJUST( abQuantTables[1], *, 0.5 );
     }
 
-    for( i = 0; i < 2 && i < nComponents; i++ )
+    for( int i = 0; i < 2 && i < nComponents; i++ )
     {
         *( pabNext++ )  = 0xFF;         // Tag Mark
         *( pabNext++ )  = 0xDB;         // DQT
@@ -212,21 +210,21 @@ int JPGHLP_HeaderMaker( GByte *pabyBuffer,
     // ------------------------------------------------------------------------
 
     *( pabNext++ )      = 0xFF;
-    *( pabNext++ )      = 0xC0;         // SOF 
-    *( pabNext++ )      = 0;            // Segment Length (msb) 
+    *( pabNext++ )      = 0xC0;         // SOF
+    *( pabNext++ )      = 0;            // Segment Length (msb)
     if ( nComponents > 1 )
-        *( pabNext++ )  = 17;           // Segment Length (lsb) 
+        *( pabNext++ )  = 17;           // Segment Length (lsb)
     else
         *( pabNext++ )  = 11;           // Segment Length (lsb)
-    *( pabNext++ )      = 8;            // 8-bit Precision 
-    *( pabNext++ )      = (GByte) (nRows >> 8); // Height in rows (msb) 
-    *( pabNext++ )      = (GByte) nRows;// Height in rows (lsb) 
+    *( pabNext++ )      = 8;            // 8-bit Precision
+    *( pabNext++ )      = (GByte) (nRows >> 8); // Height in rows (msb)
+    *( pabNext++ )      = (GByte) nRows;// Height in rows (lsb)
     *( pabNext++ )      = (GByte) (nCols >> 8); // Width in columns (msb)
     *( pabNext++ )      = (GByte) nCols;// Width in columns (lsb)
-    *( pabNext++ )      = (GByte) nComponents;// Number of components 
+    *( pabNext++ )      = (GByte) nComponents;// Number of components
     *( pabNext++ )      = 0;            // Component ID
     *( pabNext++ )      = 0x21;         // Hozontal/Vertical Sampling
-    *( pabNext++ )      = 0;            // Quantization table ID 
+    *( pabNext++ )      = 0;            // Quantization table ID
     if ( nComponents > 1 )
     {
         *( pabNext++ )  = 1;            // Component ID
@@ -263,16 +261,15 @@ int JPGHLP_HeaderMaker( GByte *pabyBuffer,
     pnHTs[1][2]         = sizeof(JPGHLP_2DC_Symbols);
     pnHTs[1][3]         = sizeof(JPGHLP_2AC_Symbols);
 
-    int j, k;
-    for( i = 0; i < 2 && i < nComponents; i++ )
+    for( int i = 0; i < 2 && i < nComponents; i++ )
     {
-        for( j = 0; j < 2; j++ )
+        for( int j = 0; j < 2; j++ )
         {
-            k = j + 2;
-            int nCodes  = pnHTs[i][j];
-            int nSymbols = pnHTs[i][k];
+            const int k = j + 2;
+            const int nCodes  = pnHTs[i][j];
+            const int nSymbols = pnHTs[i][k];
             *( pabNext++ ) = 0xFF;                  // Tag Mark
-            *( pabNext++ ) = 0xc4;                  // DHT 
+            *( pabNext++ ) = 0xc4;                  // DHT
             *( pabNext++ ) = 0;                     // Segment Length (msb)
             *( pabNext++ ) = (GByte) (3 + nCodes + nSymbols); // Segment Length (lsb)
             *( pabNext++ ) = (GByte) ((j << 4) | i);          // Table ID
@@ -288,30 +285,30 @@ int JPGHLP_HeaderMaker( GByte *pabyBuffer,
     // ------------------------------------------------------------------------
 
     *( pabNext++ )      = 0xFF;         // Tag Mark
-    *( pabNext++ )      = 0xDA;         // SOS 
+    *( pabNext++ )      = 0xDA;         // SOS
     if (nComponents > 1 )
     {
-        *( pabNext++ )  = 0;            // Segment Length (msb) 
+        *( pabNext++ )  = 0;            // Segment Length (msb)
         *( pabNext++ )  = 12;           // Segment Length (lsb)
-        *( pabNext++ )  = 3;            // Number of components 
-        *( pabNext++ )  = 0;            // Components 0 
-        *( pabNext++ )  = 0;            // Huffman table ID 
-        *( pabNext++ )  = 1;            // Components 1 
+        *( pabNext++ )  = 3;            // Number of components
+        *( pabNext++ )  = 0;            // Components 0
+        *( pabNext++ )  = 0;            // Huffman table ID
+        *( pabNext++ )  = 1;            // Components 1
         *( pabNext++ )  = 0x11;         // Huffman table ID
-        *( pabNext++ )  = 2;            // Components 2 
+        *( pabNext++ )  = 2;            // Components 2
         *( pabNext++ )  = 0x11;         // Huffman table ID
     }
     else
     {
         *( pabNext++ )  = 0;            // Segment Length (msb)
         *( pabNext++ )  = 8;            // Segment Length (lsb)
-        *( pabNext++ )  = 1;            // Number of components 
+        *( pabNext++ )  = 1;            // Number of components
         *( pabNext++ )  = 0;            // Components 0
         *( pabNext++ )  = 0;            // Huffman table ID
     }
     *( pabNext++ )      = 0;            // First DCT coefficient
-    *( pabNext++ )      = 63;           // Last DCT coefficient 
+    *( pabNext++ )      = 63;           // Last DCT coefficient
     *( pabNext++ )      = 0;            // Spectral selection
 
-    return pabNext - pabyBuffer;
+    return static_cast<int>(pabNext - pabyBuffer);
 }
