@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ddfsubfielddefn.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ddfsubfielddefn.cpp 33717 2016-03-14 06:29:14Z goatbar $
  *
  * Project:  ISO 8211 Access
  * Purpose:  Implements the DDFSubfieldDefn class.
@@ -30,8 +30,9 @@
 
 #include "iso8211.h"
 #include "cpl_conv.h"
+#include "cpl_string.h"
 
-CPL_CVSID("$Id: ddfsubfielddefn.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: ddfsubfielddefn.cpp 33717 2016-03-14 06:29:14Z goatbar $");
 
 /************************************************************************/
 /*                          DDFSubfieldDefn()                           */
@@ -41,13 +42,13 @@ DDFSubfieldDefn::DDFSubfieldDefn()
 
 {
     pszName = NULL;
-    
+
     bIsVariable = TRUE;
     nFormatWidth = 0;
-    chFormatDelimeter = DDF_UNIT_TERMINATOR;
+    chFormatDelimiter = DDF_UNIT_TERMINATOR;
     eBinaryFormat = NotBinary;
     eType = DDFString;
-    
+
     pszFormatString = CPLStrdup("");
 
     nMaxBufChars = 0;
@@ -74,12 +75,12 @@ void DDFSubfieldDefn::SetName( const char * pszNewName )
 
 {
     int         i;
-    
+
     CPLFree( pszName );
 
     pszName = CPLStrdup( pszNewName );
 
-    for( i = strlen(pszName)-1; i > 0 && pszName[i] == ' '; i-- )
+    for( i = static_cast<int>(strlen(pszName))-1; i > 0 && pszName[i] == ' '; i-- )
         pszName[i] = '\0';
 }
 
@@ -106,11 +107,18 @@ int DDFSubfieldDefn::SetFormat( const char * pszFormat )
     if( pszFormatString[1] == '(' )
     {
         nFormatWidth = atoi(pszFormatString+2);
+        if( nFormatWidth < 0 )
+        {
+             CPLError( CE_Failure, CPLE_AppDefined,
+                       "Format width %s is invalid.",
+                       pszFormatString+2 );
+            return FALSE;
+        }
         bIsVariable = nFormatWidth == 0;
     }
     else
         bIsVariable = TRUE;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Interpret the format string.                                    */
 /* -------------------------------------------------------------------- */
@@ -124,7 +132,7 @@ int DDFSubfieldDefn::SetFormat( const char * pszFormat )
       case 'R':
         eType = DDFFloat;
         break;
-        
+
       case 'I':
       case 'S':
         eType = DDFInt;
@@ -136,15 +144,16 @@ int DDFSubfieldDefn::SetFormat( const char * pszFormat )
         bIsVariable = FALSE;
         if( pszFormatString[1] == '(' )
         {
-            if( atoi(pszFormatString+2) % 8 != 0 )
+            nFormatWidth = atoi(pszFormatString+2);
+            if( nFormatWidth < 0 || nFormatWidth % 8 != 0 )
             {
                  CPLError( CE_Failure, CPLE_AppDefined,
                            "Format width %s is invalid.",
                            pszFormatString+2 );
                 return FALSE;
             }
-            
-            nFormatWidth = atoi(pszFormatString+2) / 8;
+
+            nFormatWidth = nFormatWidth / 8;
             eBinaryFormat = SInt; // good default, works for SDTS.
 
             if( nFormatWidth < 5 )
@@ -152,12 +161,19 @@ int DDFSubfieldDefn::SetFormat( const char * pszFormat )
             else
                 eType = DDFBinaryString;
         }
-        
+
         // or do we have a binary type indicator? (is it binary)
         else
         {
             eBinaryFormat = (DDFBinaryFormat) (pszFormatString[1] - '0');
             nFormatWidth = atoi(pszFormatString+2);
+            if( nFormatWidth < 0 )
+            {
+                 CPLError( CE_Failure, CPLE_AppDefined,
+                           "Format width %s is invalid.",
+                           pszFormatString+2 );
+                return FALSE;
+            }
 
             if( eBinaryFormat == SInt || eBinaryFormat == UInt )
                 eType = DDFInt;
@@ -167,22 +183,22 @@ int DDFSubfieldDefn::SetFormat( const char * pszFormat )
         break;
 
       case 'X':
-        // 'X' is extra space, and shouldn't be directly assigned to a
-        // subfield ... I haven't encountered it in use yet though.
+        // 'X' is extra space, and should not be directly assigned to a
+        // subfield ... I have not encountered it in use yet though.
         CPLError( CE_Failure, CPLE_AppDefined,
                   "Format type of `%c' not supported.\n",
                   pszFormatString[0] );
-        
+
         return FALSE;
-        
+
       default:
         CPLError( CE_Failure, CPLE_AppDefined,
                   "Format type of `%c' not recognised.\n",
                   pszFormatString[0] );
-        
+
         return FALSE;
     }
-    
+
     return TRUE;
 }
 
@@ -196,7 +212,7 @@ int DDFSubfieldDefn::SetFormat( const char * pszFormat )
  * A variety of information about this field definition is written to the
  * give debugging file handle.
  *
- * @param fp The standard io file handle to write to.  ie. stderr
+ * @param fp The standard IO file handle to write to.  i.e. stderr
  */
 
 void DDFSubfieldDefn::Dump( FILE * fp )
@@ -228,13 +244,13 @@ void DDFSubfieldDefn::Dump( FILE * fp )
  * @param pachSourceData The pointer to the raw data for this field.  This
  * may have come from DDFRecord::GetData(), taking into account skip factors
  * over previous subfields data.
- * @param nMaxBytes The maximum number of bytes that are accessable after
+ * @param nMaxBytes The maximum number of bytes that are accessible after
  * pachSourceData.
  * @param pnConsumedBytes Pointer to an integer into which the number of
  * bytes consumed by this field should be written.  May be NULL to ignore.
  *
  * @return The number of bytes at pachSourceData which are actual data for
- * this record (not including unit, or field terminator).  
+ * this record (not including unit, or field terminator).
  */
 
 int DDFSubfieldDefn::GetDataLength( const char * pachSourceData,
@@ -245,7 +261,7 @@ int DDFSubfieldDefn::GetDataLength( const char * pachSourceData,
     {
         if( nFormatWidth > nMaxBytes )
         {
-            CPLError( CE_Warning, CPLE_AppDefined, 
+            CPLError( CE_Warning, CPLE_AppDefined,
                       "Only %d bytes available for subfield %s with\n"
                       "format string %s ... returning shortened data.",
                       nMaxBytes, pszName, pszFormatString );
@@ -269,30 +285,30 @@ int DDFSubfieldDefn::GetDataLength( const char * pachSourceData,
         int     bAsciiField = TRUE;
         int     extraConsumedBytes = 0;
 
-        /* We only check for the field terminator because of some buggy 
+        /* We only check for the field terminator because of some buggy
          * datasets with missing format terminators.  However, we have found
-         * the field terminator and unit terminators are legal characters 
+         * the field terminator and unit terminators are legal characters
          * within the fields of some extended datasets (such as JP34NC94.000).
-         * So we don't check for the field terminator and unit terminators as 
-         * a single byte if the field appears to be multi-byte which we 
+         * So we don't check for the field terminator and unit terminators as
+         * a single byte if the field appears to be multi-byte which we
          * establish by checking for the buffer ending with 0x1e 0x00 (a
-         * two byte field terminator). 
+         * two byte field terminator).
          *
-         * In the case of S57, the subfield ATVL of the NATF field can be 
-         * encoded in lexical level 2 (see S57 specification, Edition 3.1, 
-         * paragraph 2.4 and 2.5). In that case the Unit Terminator and Field 
+         * In the case of S57, the subfield ATVL of the NATF field can be
+         * encoded in lexical level 2 (see S57 specification, Edition 3.1,
+         * paragraph 2.4 and 2.5). In that case the Unit Terminator and Field
          * Terminator are followed by the NULL character.
-         * A better fix would be to read the NALL tag in the DSSI to check 
-         * that the lexical level is 2, instead of relying on the value of 
+         * A better fix would be to read the NALL tag in the DSSI to check
+         * that the lexical level is 2, instead of relying on the value of
          * the first byte as we are doing - but that is not information
          * that is available at the libiso8211 level (bug #1526)
          */
 
         // If the whole field ends with 0x1e 0x00 then we assume this
         // field is a double byte character set.
-        if( nMaxBytes > 1 
-            && (pachSourceData[nMaxBytes-2] == chFormatDelimeter
-                || pachSourceData[nMaxBytes-2] == DDF_FIELD_TERMINATOR) 
+        if( nMaxBytes > 1
+            && (pachSourceData[nMaxBytes-2] == chFormatDelimiter
+                || pachSourceData[nMaxBytes-2] == DDF_FIELD_TERMINATOR)
             && pachSourceData[nMaxBytes-1] == 0x00 )
             bAsciiField = FALSE;
 
@@ -303,15 +319,15 @@ int DDFSubfieldDefn::GetDataLength( const char * pachSourceData,
         {
             if (bAsciiField)
             {
-                if (pachSourceData[nLength] == chFormatDelimeter ||
+                if (pachSourceData[nLength] == chFormatDelimiter ||
                     pachSourceData[nLength] == DDF_FIELD_TERMINATOR)
                     break;
             }
             else
             {
-                if (nLength > 0 
-                    && (pachSourceData[nLength-1] == chFormatDelimeter 
-                        || pachSourceData[nLength-1] == DDF_FIELD_TERMINATOR) 
+                if (nLength > 0
+                    && (pachSourceData[nLength-1] == chFormatDelimiter
+                        || pachSourceData[nLength-1] == DDF_FIELD_TERMINATOR)
                     && pachSourceData[nLength] == 0)
                 {
                     // Suck up the field terminator if one follows
@@ -321,9 +337,9 @@ int DDFSubfieldDefn::GetDataLength( const char * pachSourceData,
                         pachSourceData[nLength+1] == DDF_FIELD_TERMINATOR)
                         extraConsumedBytes++;
                     break;
-                } 
+                }
             }
-            
+
             nLength++;
         }
 
@@ -334,7 +350,7 @@ int DDFSubfieldDefn::GetDataLength( const char * pachSourceData,
             else
                 *pnConsumedBytes = nLength + extraConsumedBytes + 1;
         }
-        
+
         return nLength;
     }
 }
@@ -359,7 +375,7 @@ int DDFSubfieldDefn::GetDataLength( const char * pachSourceData,
  * @param pachSourceData The pointer to the raw data for this field.  This
  * may have come from DDFRecord::GetData(), taking into account skip factors
  * over previous subfields data.
- * @param nMaxBytes The maximum number of bytes that are accessable after
+ * @param nMaxBytes The maximum number of bytes that are accessible after
  * pachSourceData.
  * @param pnConsumedBytes Pointer to an integer into which the number of
  * bytes consumed by this field should be written.  May be NULL to ignore.
@@ -388,7 +404,7 @@ DDFSubfieldDefn::ExtractStringData( const char * pachSourceData,
     if( nMaxBufChars < nLength+1 )
     {
         CPLFree( pachBuffer );
-        
+
         nMaxBufChars = nLength+1;
         pachBuffer = (char *) CPLMalloc(nMaxBufChars);
     }
@@ -418,7 +434,7 @@ DDFSubfieldDefn::ExtractStringData( const char * pachSourceData,
  * @param pachSourceData The pointer to the raw data for this field.  This
  * may have come from DDFRecord::GetData(), taking into account skip factors
  * over previous subfields data.
- * @param nMaxBytes The maximum number of bytes that are accessable after
+ * @param nMaxBytes The maximum number of bytes that are accessible after
  * pachSourceData.
  * @param pnConsumedBytes Pointer to an integer into which the number of
  * bytes consumed by this field should be written.  May be NULL to ignore.
@@ -442,7 +458,7 @@ DDFSubfieldDefn::ExtractFloatData( const char * pachSourceData,
       case 'R':
       case 'S':
       case 'C':
-        return atof(ExtractStringData(pachSourceData, nMaxBytes,
+        return CPLAtof(ExtractStringData(pachSourceData, nMaxBytes,
                                       pnConsumedBytes));
 
       case 'B':
@@ -468,9 +484,9 @@ DDFSubfieldDefn::ExtractFloatData( const char * pachSourceData,
           // word aligned.
 #ifdef CPL_LSB
           if( pszFormatString[0] == 'B' )
-#else            
+#else
               if( pszFormatString[0] == 'b' )
-#endif            
+#endif
               {
                   for( int i = 0; i < nFormatWidth; i++ )
                       abyData[nFormatWidth-i-1] = pachSourceData[i];
@@ -495,7 +511,7 @@ DDFSubfieldDefn::ExtractFloatData( const char * pachSourceData,
                   //CPLAssert( FALSE );
                   return 0.0;
               }
-            
+
             case SInt:
               if( nFormatWidth == 1 )
                   return( *((signed char *) abyData) );
@@ -508,7 +524,7 @@ DDFSubfieldDefn::ExtractFloatData( const char * pachSourceData,
                   //CPLAssert( FALSE );
                   return 0.0;
               }
-            
+
             case FloatReal:
               if( nFormatWidth == 4 )
                   return( *((float *) pabyData) );
@@ -520,7 +536,7 @@ DDFSubfieldDefn::ExtractFloatData( const char * pachSourceData,
                   return 0.0;
               }
 
-            case NotBinary:            
+            case NotBinary:
             case FPReal:
             case FloatComplex:
               //CPLAssert( FALSE );
@@ -554,7 +570,7 @@ DDFSubfieldDefn::ExtractFloatData( const char * pachSourceData,
  * @param pachSourceData The pointer to the raw data for this field.  This
  * may have come from DDFRecord::GetData(), taking into account skip factors
  * over previous subfields data.
- * @param nMaxBytes The maximum number of bytes that are accessable after
+ * @param nMaxBytes The maximum number of bytes that are accessible after
  * pachSourceData.
  * @param pnConsumedBytes Pointer to an integer into which the number of
  * bytes consumed by this field should be written.  May be NULL to ignore.
@@ -589,7 +605,7 @@ DDFSubfieldDefn::ExtractIntData( const char * pachSourceData,
 
           if( nFormatWidth > nMaxBytes || nFormatWidth >= (int)sizeof(abyData) )
           {
-              CPLError( CE_Warning, CPLE_AppDefined, 
+              CPLError( CE_Warning, CPLE_AppDefined,
                         "Attempt to extract int subfield %s with format %s\n"
                         "failed as only %d bytes available.  Using zero.",
                         pszName, pszFormatString, MIN(nMaxBytes, (int)sizeof(abyData)) );
@@ -604,9 +620,9 @@ DDFSubfieldDefn::ExtractIntData( const char * pachSourceData,
           // word aligned.
 #ifdef CPL_LSB
           if( pszFormatString[0] == 'B' )
-#else            
+#else
               if( pszFormatString[0] == 'b' )
-#endif            
+#endif
               {
                   for( int i = 0; i < nFormatWidth; i++ )
                       abyData[nFormatWidth-i-1] = pachSourceData[i];
@@ -631,7 +647,7 @@ DDFSubfieldDefn::ExtractIntData( const char * pachSourceData,
                   //CPLAssert( FALSE );
                   return 0;
               }
-            
+
             case SInt:
               if( nFormatWidth == 4 )
                   return( *((GInt32 *) pabyData) );
@@ -644,7 +660,7 @@ DDFSubfieldDefn::ExtractIntData( const char * pachSourceData,
                   //CPLAssert( FALSE );
                   return 0;
               }
-            
+
             case FloatReal:
               if( nFormatWidth == 4 )
                   return( (int) *((float *) pabyData) );
@@ -656,7 +672,7 @@ DDFSubfieldDefn::ExtractIntData( const char * pachSourceData,
                   return 0;
               }
 
-            case NotBinary:            
+            case NotBinary:
             case FPReal:
             case FloatComplex:
               //CPLAssert( FALSE );
@@ -694,6 +710,11 @@ void DDFSubfieldDefn::DumpData( const char * pachData, int nMaxBytes,
                                 FILE * fp )
 
 {
+    if( nMaxBytes < 0 )
+    {
+        fprintf( fp, "      Subfield `%s' = {invalid length}\n", pszName );
+        return;
+    }
     if( eType == DDFFloat )
         fprintf( fp, "      Subfield `%s' = %f\n",
                  pszName,
@@ -727,17 +748,17 @@ void DDFSubfieldDefn::DumpData( const char * pachData, int nMaxBytes,
 /************************************************************************/
 
 /**
- * Get default data. 
+ * Get default data.
  *
  * Returns the default subfield data contents for this subfield definition.
- * For variable length numbers this will normally be "0<unit-terminator>". 
+ * For variable length numbers this will normally be "0<unit-terminator>".
  * For variable length strings it will be "<unit-terminator>".  For fixed
  * length numbers it is zero filled.  For fixed length strings it is space
- * filled.  For binary numbers it is binary zero filled. 
+ * filled.  For binary numbers it is binary zero filled.
  *
  * @param pachData the buffer into which the returned default will be placed.
  * May be NULL if just querying default size.
- * @param nBytesAvailable the size of pachData in bytes. 
+ * @param nBytesAvailable the size of pachData in bytes.
  * @param pnBytesUsed will receive the size of the subfield default data in
  * bytes.
  *
@@ -745,7 +766,7 @@ void DDFSubfieldDefn::DumpData( const char * pachData, int nMaxBytes,
  * small to hold the default.
  */
 
-int DDFSubfieldDefn::GetDefaultValue( char *pachData, int nBytesAvailable, 
+int DDFSubfieldDefn::GetDefaultValue( char *pachData, int nBytesAvailable,
                                       int *pnBytesUsed )
 
 {
@@ -771,15 +792,17 @@ int DDFSubfieldDefn::GetDefaultValue( char *pachData, int nBytesAvailable,
     }
     else
     {
+        char chFillChar;
         if( GetBinaryFormat() == NotBinary )
         {
             if( GetType() == DDFInt || GetType() == DDFFloat )
-                memset( pachData, '0', nDefaultSize );
+                chFillChar = '0'; /* ASCII zero intended */
             else
-                memset( pachData, ' ', nDefaultSize );
+                chFillChar = ' ';
         }
         else
-            memset( pachData, 0, nDefaultSize );
+            chFillChar = 0;
+        memset( pachData, chFillChar, nDefaultSize );
     }
 
     return TRUE;
@@ -793,11 +816,11 @@ int DDFSubfieldDefn::GetDefaultValue( char *pachData, int nBytesAvailable,
  * Format string subfield value.
  *
  * Returns a buffer with the passed in string value reformatted in a way
- * suitable for storage in a DDFField for this subfield.  
+ * suitable for storage in a DDFField for this subfield.
  */
 
-int DDFSubfieldDefn::FormatStringValue( char *pachData, int nBytesAvailable, 
-                                        int *pnBytesUsed, 
+int DDFSubfieldDefn::FormatStringValue( char *pachData, int nBytesAvailable,
+                                        int *pnBytesUsed,
                                         const char *pszValue,
                                         int nValueLength )
 
@@ -805,14 +828,14 @@ int DDFSubfieldDefn::FormatStringValue( char *pachData, int nBytesAvailable,
     int nSize;
 
     if( nValueLength == -1 )
-        nValueLength = strlen(pszValue);
+        nValueLength = static_cast<int>(strlen(pszValue));
 
     if( bIsVariable )
     {
         nSize = nValueLength + 1;
     }
     else
-    {                                                                  
+    {
         nSize = nFormatWidth;
     }
 
@@ -855,24 +878,24 @@ int DDFSubfieldDefn::FormatStringValue( char *pachData, int nBytesAvailable,
  * Format int subfield value.
  *
  * Returns a buffer with the passed in int value reformatted in a way
- * suitable for storage in a DDFField for this subfield.  
+ * suitable for storage in a DDFField for this subfield.
  */
 
-int DDFSubfieldDefn::FormatIntValue( char *pachData, int nBytesAvailable, 
+int DDFSubfieldDefn::FormatIntValue( char *pachData, int nBytesAvailable,
                                      int *pnBytesUsed, int nNewValue )
 
 {
     int nSize;
     char szWork[30];
 
-    sprintf( szWork, "%d", nNewValue );
+    snprintf( szWork, sizeof(szWork), "%d", nNewValue );
 
     if( bIsVariable )
     {
-        nSize = strlen(szWork) + 1;
+        nSize = static_cast<int>(strlen(szWork)) + 1;
     }
     else
-    {                                                                  
+    {
         nSize = nFormatWidth;
 
         if( GetBinaryFormat() == NotBinary && (int) strlen(szWork) > nSize )
@@ -901,10 +924,13 @@ int DDFSubfieldDefn::FormatIntValue( char *pachData, int nBytesAvailable,
         switch( GetBinaryFormat() )
         {
           case NotBinary:
-            memset( pachData, '0', nSize );
+          {
+            char chFillChar = '0'; /* ASCII zero intended */
+            memset( pachData, chFillChar, nSize );
             strncpy( pachData + nSize - strlen(szWork), szWork,
                      strlen(szWork) );
             break;
+          }
 
           case UInt:
           case SInt:
@@ -944,21 +970,21 @@ int DDFSubfieldDefn::FormatIntValue( char *pachData, int nBytesAvailable,
  * Format float subfield value.
  *
  * Returns a buffer with the passed in float value reformatted in a way
- * suitable for storage in a DDFField for this subfield.  
+ * suitable for storage in a DDFField for this subfield.
  */
 
-int DDFSubfieldDefn::FormatFloatValue( char *pachData, int nBytesAvailable, 
+int DDFSubfieldDefn::FormatFloatValue( char *pachData, int nBytesAvailable,
                                        int *pnBytesUsed, double dfNewValue )
 
 {
     int nSize;
     char szWork[120];
 
-    sprintf( szWork, "%.16g", dfNewValue );
+    CPLsnprintf( szWork, sizeof(szWork), "%.16g", dfNewValue );
 
     if( bIsVariable )
     {
-        nSize = strlen(szWork) + 1;
+        nSize = static_cast<int>(strlen(szWork)) + 1;
     }
     else
     {
@@ -986,7 +1012,9 @@ int DDFSubfieldDefn::FormatFloatValue( char *pachData, int nBytesAvailable,
     {
         if( GetBinaryFormat() == NotBinary )
         {
-            memset( pachData, '0', nSize );
+            const char chFillZeroASCII = '0'; /* ASCII zero intended */
+            /* coverity[bad_memset] */
+            memset( pachData, chFillZeroASCII, nSize );
             strncpy( pachData + nSize - strlen(szWork), szWork,
                      strlen(szWork) );
         }
