@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: jaxapalsardataset.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: jaxapalsardataset.cpp 33717 2016-03-14 06:29:14Z goatbar $
  *
  * Project:  PALSAR JAXA imagery reader
  * Purpose:  Support for PALSAR L1.1/1.5 imagery and appropriate metadata from
@@ -30,15 +30,12 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
+#include "gdal_frmts.h"
 #include "gdal_pam.h"
 
-CPL_CVSID("$Id: jaxapalsardataset.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: jaxapalsardataset.cpp 33717 2016-03-14 06:29:14Z goatbar $");
 
-CPL_C_START
-void	GDALRegister_PALSARJaxa(void);
-CPL_C_END
-
-#if defined(WIN32) || defined(WIN32CE)
+#if defined(WIN32)
 #define SEP_STRING "\\"
 #else
 #define SEP_STRING "/"
@@ -80,7 +77,7 @@ CPL_C_END
 		(x) = atoi(psBuf); \
 	} while (0);
 
-/* read string fields 
+/* read string fields
  * note: string must be size of field to be extracted + 1
  */
 #define READ_STRING(s, n, f) \
@@ -140,7 +137,8 @@ CPL_C_END
 enum eFileType {
 	level_11 = 0,
 	level_15,
-    level_10
+        level_10,
+        level_unknown = 999,
 };
 
 enum ePolarization {
@@ -176,18 +174,18 @@ public:
     static void ReadMetadata( PALSARJaxaDataset *poDS, VSILFILE *fp );
 };
 
-PALSARJaxaDataset::PALSARJaxaDataset()
-{
-    pasGCPList = NULL;
-    nGCPCount = 0;
-}
+PALSARJaxaDataset::PALSARJaxaDataset() :
+    pasGCPList(NULL),
+    nGCPCount(0),
+    nFileType(level_unknown)
+{ }
 
 PALSARJaxaDataset::~PALSARJaxaDataset()
 {
-    if( nGCPCount > 0 ) 
+    if( nGCPCount > 0 )
     {
-        GDALDeinitGCPs( nGCPCount, pasGCPList ); 
-        CPLFree( pasGCPList ); 
+        GDALDeinitGCPs( nGCPCount, pasGCPList );
+        CPLFree( pasGCPList );
     }
 }
 
@@ -206,6 +204,7 @@ class PALSARJaxaRasterBand : public GDALRasterBand {
     int nBitsPerSample;
     int nSamplesPerGroup;
     int nRecordSize;
+
 public:
     PALSARJaxaRasterBand( PALSARJaxaDataset *poDS, int nBand, VSILFILE *fp );
     ~PALSARJaxaRasterBand();
@@ -217,10 +216,13 @@ public:
 /*                         PALSARJaxaRasterBand()                       */
 /************************************************************************/
 
-PALSARJaxaRasterBand::PALSARJaxaRasterBand( PALSARJaxaDataset *poDS, 
-	int nBand, VSILFILE *fp )
+PALSARJaxaRasterBand::PALSARJaxaRasterBand( PALSARJaxaDataset *poDSIn,
+                                            int nBandIn, VSILFILE *fpIn ) :
+    nPolarization(hh)
 {
-    this->fp = fp;
+    poDS = poDSIn;
+    nBand = nBandIn;
+    this->fp = fpIn;
 
     /* Read image options record to determine the type of data */
     VSIFSeekL( fp, BITS_PER_SAMPLE_OFFSET, SEEK_SET );
@@ -242,19 +244,23 @@ PALSARJaxaRasterBand::PALSARJaxaRasterBand( PALSARJaxaDataset *poDS,
         nFileType = level_15;
     }
 
-    poDS->nFileType = nFileType;
+    poDSIn->nFileType = nFileType;
 
     /* Read number of range/azimuth lines */
     VSIFSeekL( fp, NUMBER_LINES_OFFSET, SEEK_SET );
     READ_CHAR_VAL( nRasterYSize, NUMBER_LINES_LENGTH, fp );
     VSIFSeekL( fp, SAR_DATA_RECORD_LENGTH_OFFSET, SEEK_SET );
     READ_CHAR_VAL( nRecordSize, SAR_DATA_RECORD_LENGTH_LENGTH, fp );
-    nRasterXSize = (nRecordSize -
+    int nDenom = ((nBitsPerSample / 8) * nSamplesPerGroup);
+    if( nDenom == 0 )
+        nRasterXSize = 0;
+    else
+        nRasterXSize = (nRecordSize -
                     (nFileType != level_15 ? SIG_DAT_REC_OFFSET : PROC_DAT_REC_OFFSET))
-        / ((nBitsPerSample / 8) * nSamplesPerGroup);
+        / nDenom;
 
-    poDS->nRasterXSize = nRasterXSize;
-    poDS->nRasterYSize = nRasterYSize;
+    poDSIn->nRasterXSize = nRasterXSize;
+    poDSIn->nRasterYSize = nRasterYSize;
 
     /* Polarization */
     switch (nBand) {
@@ -274,15 +280,16 @@ PALSARJaxaRasterBand::PALSARJaxaRasterBand( PALSARJaxaDataset *poDS,
         nPolarization = vv;
         SetMetadataItem( "POLARIMETRIC_INTERP", "VV" );
         break;
+      // TODO: What about the default?
     }
-	
+
     /* size of block we can read */
     nBlockXSize = nRasterXSize;
     nBlockYSize = 1;
 
     /* set the file pointer to the first SAR data record */
     VSIFSeekL( fp, IMAGE_OPT_DESC_LENGTH, SEEK_SET );
-}	
+}
 
 /************************************************************************/
 /*                        ~PALSARJaxaRasterBand()                       */
@@ -298,8 +305,9 @@ PALSARJaxaRasterBand::~PALSARJaxaRasterBand()
 /*                             IReadBlock()                             */
 /************************************************************************/
 
-CPLErr PALSARJaxaRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYOff,
-	void *pImage )
+CPLErr PALSARJaxaRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
+                                         int nBlockYOff,
+                                         void *pImage )
 {
     int nNumBytes = 0;
     if (nFileType == level_11) {
@@ -309,7 +317,7 @@ CPLErr PALSARJaxaRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYO
         nNumBytes = 2;
     }
 
-    int nOffset = IMAGE_OPT_DESC_LENGTH + ((nBlockYOff - 1) * nRecordSize) + 
+    int nOffset = IMAGE_OPT_DESC_LENGTH + ((nBlockYOff - 1) * nRecordSize) +
         (nFileType == level_11 ? SIG_DAT_REC_OFFSET : PROC_DAT_REC_OFFSET);
 
     VSIFSeekL( fp, nOffset, SEEK_SET );
@@ -318,7 +326,7 @@ CPLErr PALSARJaxaRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYO
 #ifdef CPL_LSB
     if (nFileType == level_11)
         GDALSwapWords( pImage, 4, nBlockXSize * 2, 4 );
-    else 
+    else
         GDALSwapWords( pImage, 2, nBlockXSize, 2 );
 #endif
 
@@ -368,49 +376,49 @@ void PALSARJaxaDataset::ReadMetadata( PALSARJaxaDataset *poDS, VSILFILE *fp ) {
     else {
         poDS->SetMetadataItem( "PRODUCT_LEVEL", "1.5" );
         /* extract equivalent number of looks */
-        VSIFSeekL( fp, LEADER_FILE_DESCRIPTOR_LENGTH + 
+        VSIFSeekL( fp, LEADER_FILE_DESCRIPTOR_LENGTH +
                   EFFECTIVE_LOOKS_AZIMUTH_OFFSET, SEEK_SET );
-        char pszENL[17];
+        char szENL[17];
         double dfENL;
         READ_CHAR_FLOAT(dfENL, 16, fp);
-        sprintf( pszENL, "%-16.1f", dfENL );
-        poDS->SetMetadataItem( "AZIMUTH_LOOKS", pszENL );
+        snprintf( szENL, sizeof(szENL), "%-16.1f", dfENL );
+        poDS->SetMetadataItem( "AZIMUTH_LOOKS", szENL );
 
         /* extract pixel spacings */
         VSIFSeekL( fp, LEADER_FILE_DESCRIPTOR_LENGTH +
                   DATA_SET_SUMMARY_LENGTH + PIXEL_SPACING_OFFSET, SEEK_SET );
         double dfPixelSpacing;
         double dfLineSpacing;
-        char pszPixelSpacing[33];
-        char pszLineSpacing[33];
+        char szPixelSpacing[33];
+        char szLineSpacing[33];
         READ_CHAR_FLOAT(dfPixelSpacing, 16, fp);
         READ_CHAR_FLOAT(dfLineSpacing, 16, fp);
-        sprintf( pszPixelSpacing, "%-32.1f",dfPixelSpacing );
-        sprintf( pszLineSpacing, "%-32.1f", dfLineSpacing );
-        poDS->SetMetadataItem( "PIXEL_SPACING", pszPixelSpacing );
-        poDS->SetMetadataItem( "LINE_SPACING", pszPixelSpacing );
+        snprintf( szPixelSpacing, sizeof(szPixelSpacing), "%-32.1f",dfPixelSpacing );
+        snprintf( szLineSpacing, sizeof(szLineSpacing), "%-32.1f", dfLineSpacing );
+        poDS->SetMetadataItem( "PIXEL_SPACING", szPixelSpacing );
+        poDS->SetMetadataItem( "LINE_SPACING", szPixelSpacing );
 
         /* Alphanumeric projection name */
         VSIFSeekL( fp, LEADER_FILE_DESCRIPTOR_LENGTH +
                   DATA_SET_SUMMARY_LENGTH + ALPHANUMERIC_PROJECTION_NAME_OFFSET,
                   SEEK_SET );
-        char pszProjName[33];
-        READ_STRING(pszProjName, 32, fp);
-        poDS->SetMetadataItem( "PROJECTION_NAME", pszProjName );
-		
+        char szProjName[33];
+        READ_STRING(szProjName, 32, fp);
+        poDS->SetMetadataItem( "PROJECTION_NAME", szProjName );
+
         /* Extract corner GCPs */
         poDS->nGCPCount = 4;
-        poDS->pasGCPList = (GDAL_GCP *)CPLCalloc( sizeof(GDAL_GCP), 
+        poDS->pasGCPList = (GDAL_GCP *)CPLCalloc( sizeof(GDAL_GCP),
                                                   poDS->nGCPCount );
         GDALInitGCPs( poDS->nGCPCount, poDS->pasGCPList );
 
         /* setup the GCPs */
         int i;
         for (i = 0; i < poDS->nGCPCount; i++) {
-            char pszID[2];
-            sprintf( pszID, "%d", i + 1);
+            char szID[30];
+            snprintf( szID, sizeof(szID), "%d", i + 1);
             CPLFree(poDS->pasGCPList[i].pszId);
-            poDS->pasGCPList[i].pszId = CPLStrdup( pszID );
+            poDS->pasGCPList[i].pszId = CPLStrdup( szID );
             poDS->pasGCPList[i].dfGCPZ = 0.0;
         }
 
@@ -418,7 +426,7 @@ void PALSARJaxaDataset::ReadMetadata( PALSARJaxaDataset *poDS, VSILFILE *fp ) {
         /* seek to start of GCPs */
         VSIFSeekL( fp, LEADER_FILE_DESCRIPTOR_LENGTH +
                   DATA_SET_SUMMARY_LENGTH + TOP_LEFT_LAT_OFFSET, SEEK_SET );
-		
+
         /* top-left GCP */
         READ_CHAR_FLOAT(dfTemp, 16, fp);
         poDS->pasGCPList[0].dfGCPY = dfTemp;
@@ -473,9 +481,8 @@ int PALSARJaxaDataset::Identify( GDALOpenInfo *poOpenInfo ) {
         return 0;
 
     /* First, check that this is a PALSAR image indeed */
-    if ( !EQUALN((char *)(poOpenInfo->pabyHeader + 60),"AL", 2) 
-         || !EQUALN(CPLGetBasename((char *)(poOpenInfo->pszFilename)) + 4, 
-                    "ALPSR", 5) )
+    if ( !STARTS_WITH_CI((char *)(poOpenInfo->pabyHeader + 60), "AL")
+         || !STARTS_WITH_CI(CPLGetBasename((char *)(poOpenInfo->pszFilename)) + 4, "ALPSR") )
     {
         return 0;
     }
@@ -520,18 +527,18 @@ GDALDataset *PALSARJaxaDataset::Open( GDALOpenInfo * poOpenInfo ) {
     /* Check that this actually is a JAXA PALSAR product */
     if ( !PALSARJaxaDataset::Identify(poOpenInfo) )
         return NULL;
-        
+
 /* -------------------------------------------------------------------- */
 /*      Confirm the requested access is supported.                      */
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->eAccess == GA_Update )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The JAXAPALSAR driver does not support update access to existing"
                   " datasets.\n" );
         return NULL;
     }
-    
+
     PALSARJaxaDataset *poDS = new PALSARJaxaDataset();
 
     /* Get the suffix of the filename, we'll need this */
@@ -539,25 +546,26 @@ GDALDataset *PALSARJaxaDataset::Open( GDALOpenInfo * poOpenInfo ) {
                                  (CPLGetFilename( poOpenInfo->pszFilename ) + 3) );
 
     /* Try to read each of the polarizations */
-    char *pszImgFile = (char *)VSIMalloc( 
-        strlen( CPLGetDirname( poOpenInfo->pszFilename ) ) + 
-        strlen( pszSuffix ) + 8 );
+    const size_t nImgFileLen =
+        strlen( CPLGetDirname( poOpenInfo->pszFilename ) ) +
+        strlen( pszSuffix ) + 8;
+    char *pszImgFile = (char *)CPLMalloc( nImgFileLen );
 
     int nBandNum = 1;
 
     /* HH */
     VSILFILE *fpHH;
-    sprintf( pszImgFile, "%s%sIMG-HH%s", 
+    snprintf( pszImgFile, nImgFileLen, "%s%sIMG-HH%s",
              CPLGetDirname(poOpenInfo->pszFilename), SEP_STRING, pszSuffix );
     fpHH = VSIFOpenL( pszImgFile, "rb" );
-    if (fpHH != NULL) { 
+    if (fpHH != NULL) {
         poDS->SetBand( nBandNum, new PALSARJaxaRasterBand( poDS, 0, fpHH ) );
         nBandNum++;
     }
 
     /* HV */
     VSILFILE *fpHV;
-    sprintf( pszImgFile, "%s%sIMG-HV%s", 
+    snprintf( pszImgFile, nImgFileLen, "%s%sIMG-HV%s",
              CPLGetDirname(poOpenInfo->pszFilename), SEP_STRING, pszSuffix );
     fpHV = VSIFOpenL( pszImgFile, "rb" );
     if (fpHV != NULL) {
@@ -567,7 +575,7 @@ GDALDataset *PALSARJaxaDataset::Open( GDALOpenInfo * poOpenInfo ) {
 
     /* VH */
     VSILFILE *fpVH;
-    sprintf( pszImgFile, "%s%sIMG-VH%s", 
+    snprintf( pszImgFile, nImgFileLen, "%s%sIMG-VH%s",
              CPLGetDirname(poOpenInfo->pszFilename), SEP_STRING, pszSuffix );
     fpVH = VSIFOpenL( pszImgFile, "rb" );
     if (fpVH != NULL) {
@@ -577,7 +585,7 @@ GDALDataset *PALSARJaxaDataset::Open( GDALOpenInfo * poOpenInfo ) {
 
     /* VV */
     VSILFILE *fpVV;
-    sprintf( pszImgFile, "%s%sIMG-VV%s",
+    snprintf( pszImgFile, nImgFileLen, "%s%sIMG-VV%s",
              CPLGetDirname(poOpenInfo->pszFilename), SEP_STRING, pszSuffix );
     fpVV = VSIFOpenL( pszImgFile, "rb" );
     if (fpVV != NULL) {
@@ -606,10 +614,10 @@ GDALDataset *PALSARJaxaDataset::Open( GDALOpenInfo * poOpenInfo ) {
     }
 
     /* read metadata from Leader file. */
-    char *pszLeaderFilename = (char *)VSIMalloc( 
-        strlen( CPLGetDirname( poOpenInfo->pszFilename ) ) + 
-        strlen(pszSuffix) + 5 );
-    sprintf( pszLeaderFilename, "%s%sLED%s", 
+    const size_t nLeaderFilenameLen = strlen( CPLGetDirname( poOpenInfo->pszFilename ) ) +
+        strlen(pszSuffix) + 5;
+    char *pszLeaderFilename = (char *)CPLMalloc( nLeaderFilenameLen );
+    snprintf( pszLeaderFilename, nLeaderFilenameLen, "%s%sLED%s",
              CPLGetDirname( poOpenInfo->pszFilename ) , SEP_STRING, pszSuffix );
 
     VSILFILE *fpLeader = VSIFOpenL( pszLeaderFilename, "rb" );
@@ -641,20 +649,23 @@ GDALDataset *PALSARJaxaDataset::Open( GDALOpenInfo * poOpenInfo ) {
 /*                      GDALRegister_PALSARJaxa()                       */
 /************************************************************************/
 
-void GDALRegister_PALSARJaxa() {
-    GDALDriver	*poDriver;
+void GDALRegister_PALSARJaxa()
 
-    if( GDALGetDriverByName( "JAXAPALSAR" ) == NULL ) {
-        poDriver = new GDALDriver();
-        poDriver->SetDescription( "JAXAPALSAR" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "JAXA PALSAR Product Reader (Level 1.1/1.5)" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_palsar.html" );
-        poDriver->pfnOpen = PALSARJaxaDataset::Open;
-        poDriver->pfnIdentify = PALSARJaxaDataset::Identify;
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+{
+    if( GDALGetDriverByName( "JAXAPALSAR" ) != NULL )
+        return;
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    GDALDriver *poDriver = new GDALDriver();
+
+    poDriver->SetDescription( "JAXAPALSAR" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
+                               "JAXA PALSAR Product Reader (Level 1.1/1.5)" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_palsar.html" );
+
+    poDriver->pfnOpen = PALSARJaxaDataset::Open;
+    poDriver->pfnIdentify = PALSARJaxaDataset::Identify;
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
