@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: jpeg2000dataset.cpp 27182 2014-04-14 20:03:08Z rouault $
+ * $Id: jpeg2000dataset.cpp 33720 2016-03-15 00:39:53Z goatbar $
  *
  * Project:  JPEG-2000
  * Purpose:  Partial implementation of the ISO/IEC 15444-1 standard
@@ -28,18 +28,15 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
+#include "cpl_string.h"
+#include "gdal_frmts.h"
 #include "gdaljp2abstractdataset.h"
 #include "gdaljp2metadata.h"
-#include "cpl_string.h"
 
 #include <jasper/jasper.h>
 #include "jpeg2000_vsil_io.h"
 
-CPL_CVSID("$Id: jpeg2000dataset.cpp 27182 2014-04-14 20:03:08Z rouault $");
-
-CPL_C_START
-void    GDALRegister_JPEG2000(void);
-CPL_C_END
+CPL_CVSID("$Id: jpeg2000dataset.cpp 33720 2016-03-15 00:39:53Z goatbar $");
 
 // XXX: Part of code below extracted from the JasPer internal headers and
 // must be in sync with JasPer version (this one works with JasPer 1.900.1)
@@ -175,6 +172,7 @@ class JPEG2000Dataset : public GDALJP2AbstractDataset
     jas_stream_t *psStream;
     jas_image_t *psImage;
     int         iFormat;
+    int         bPromoteTo8Bit;
 
     int         bAlreadyDecoded;
     int         DecodeImage();
@@ -182,7 +180,7 @@ class JPEG2000Dataset : public GDALJP2AbstractDataset
   public:
                 JPEG2000Dataset();
                 ~JPEG2000Dataset();
-    
+
     static int           Identify( GDALOpenInfo * );
     static GDALDataset  *Open( GDALOpenInfo * );
 };
@@ -196,12 +194,12 @@ class JPEG2000Dataset : public GDALJP2AbstractDataset
 class JPEG2000RasterBand : public GDALPamRasterBand
 {
     friend class JPEG2000Dataset;
-    
+
     // NOTE: poDS may be altered for NITF/JPEG2000 files!
     JPEG2000Dataset     *poGDS;
 
     jas_matrix_t        *psMatrix;
-    
+
     int                  iDepth;
     int                  bSignedness;
 
@@ -209,7 +207,7 @@ class JPEG2000RasterBand : public GDALPamRasterBand
 
                 JPEG2000RasterBand( JPEG2000Dataset *, int, int, int );
                 ~JPEG2000RasterBand();
-                
+
     virtual CPLErr IReadBlock( int, int, void * );
     virtual GDALColorInterp GetColorInterpretation();
 };
@@ -219,15 +217,15 @@ class JPEG2000RasterBand : public GDALPamRasterBand
 /*                           JPEG2000RasterBand()                       */
 /************************************************************************/
 
-JPEG2000RasterBand::JPEG2000RasterBand( JPEG2000Dataset *poDS, int nBand,
-                int iDepth, int bSignedness )
+JPEG2000RasterBand::JPEG2000RasterBand( JPEG2000Dataset *poDSIn, int nBandIn,
+                int iDepthIn, int bSignednessIn )
 
 {
-    this->poDS = poDS;
-    poGDS = poDS;
-    this->nBand = nBand;
-    this->iDepth = iDepth;
-    this->bSignedness = bSignedness;
+    this->poDS = poDSIn;
+    poGDS = poDSIn;
+    this->nBand = nBandIn;
+    this->iDepth = iDepthIn;
+    this->bSignedness = bSignednessIn;
 
     // XXX: JasPer can't handle data with depth > 32 bits
     // Maximum possible depth for JPEG2000 is 38!
@@ -255,9 +253,17 @@ JPEG2000RasterBand::JPEG2000RasterBand( JPEG2000Dataset *poDS, int nBand,
     }
     // FIXME: Figure out optimal block size!
     // Should the block size be fixed or determined dynamically?
-    nBlockXSize = MIN(256, poDS->nRasterXSize);
-    nBlockYSize = MIN(256, poDS->nRasterYSize);
+    nBlockXSize = MIN(256, poDSIn->nRasterXSize);
+    nBlockYSize = MIN(256, poDSIn->nRasterYSize);
     psMatrix = jas_matrix_create(nBlockYSize, nBlockXSize);
+
+    if( iDepth % 8 != 0 && !poDSIn->bPromoteTo8Bit )
+    {
+        SetMetadataItem( "NBITS",
+                         CPLString().Printf("%d",iDepth),
+                         "IMAGE_STRUCTURE" );
+    }
+    SetMetadataItem( "COMPRESSION", "JP2000", "IMAGE_STRUCTURE" );
 }
 
 /************************************************************************/
@@ -267,7 +273,7 @@ JPEG2000RasterBand::JPEG2000RasterBand( JPEG2000Dataset *poDS, int nBand,
 JPEG2000RasterBand::~JPEG2000RasterBand()
 {
     if ( psMatrix )
-        jas_matrix_destroy( psMatrix );    
+        jas_matrix_destroy( psMatrix );
 }
 
 /************************************************************************/
@@ -345,6 +351,18 @@ CPLErr JPEG2000RasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
         }
     }
 
+    if( poGDS->bPromoteTo8Bit && nBand == 4 )
+    {
+        ptr = (GByte*)pImage;
+        for( i = 0; i < nHeightToRead; i++, ptr += nLineSize )
+        {
+            for( j = 0; j < nWidthToRead; j++ )
+            {
+                ((GByte*)ptr)[j] *= 255;
+            }
+        }
+    }
+
     return CE_None;
 }
 
@@ -359,11 +377,11 @@ GDALColorInterp JPEG2000RasterBand::GetColorInterpretation()
     {
         return GCI_Undefined;
     }
-    
+
     if ( jas_clrspc_fam( jas_image_clrspc( poGDS->psImage ) ) ==
          JAS_CLRSPC_FAM_GRAY )
         return GCI_GrayIndex;
-    else if ( jas_clrspc_fam( jas_image_clrspc( poGDS->psImage ) ) == 
+    else if ( jas_clrspc_fam( jas_image_clrspc( poGDS->psImage ) ) ==
               JAS_CLRSPC_FAM_RGB )
     {
         switch ( jas_image_cmpttype( poGDS->psImage, nBand - 1 ) )
@@ -394,13 +412,15 @@ GDALColorInterp JPEG2000RasterBand::GetColorInterpretation()
 /*                           JPEG2000Dataset()                          */
 /************************************************************************/
 
-JPEG2000Dataset::JPEG2000Dataset()
+JPEG2000Dataset::JPEG2000Dataset() :
+    iFormat(0)
 {
     psStream = NULL;
     psImage = NULL;
     nBands = 0;
     bAlreadyDecoded = FALSE;
-    
+    bPromoteTo8Bit = FALSE;
+
     poDriver = (GDALDriver *)GDALGetDriverByName("JPEG2000");
 }
 
@@ -426,15 +446,15 @@ int JPEG2000Dataset::DecodeImage()
 {
     if (bAlreadyDecoded)
         return psImage != NULL;
-        
-    bAlreadyDecoded = TRUE;    
-    if ( !( psImage = jas_image_decode(psStream, iFormat, 0) ) )
+
+    bAlreadyDecoded = TRUE;
+    if ( !( psImage = jas_image_decode(psStream, iFormat, NULL) ) )
     {
         CPLDebug( "JPEG2000", "Unable to decode image. Format: %s, %d",
                   jas_image_fmttostr( iFormat ), iFormat );
         return FALSE;
     }
-    
+
     /* Case of a JP2 image : check that the properties given by */
     /* the JP2 boxes match the ones of the code stream */
     if (nBands != 0)
@@ -449,7 +469,7 @@ int JPEG2000Dataset::DecodeImage()
             psImage = NULL;
             return FALSE;
         }
-        
+
         if (nRasterXSize != jas_image_cmptwidth( psImage, 0 ) ||
             nRasterYSize != jas_image_cmptheight( psImage, 0 ) )
         {
@@ -463,7 +483,7 @@ int JPEG2000Dataset::DecodeImage()
             psImage = NULL;
             return FALSE;
         }
-        
+
         int iBand;
         for ( iBand = 0; iBand < nBands; iBand++ )
         {
@@ -481,9 +501,9 @@ int JPEG2000Dataset::DecodeImage()
             }
         }
     }
-    
+
     /* Ask for YCbCr -> RGB translation */
-    if ( jas_clrspc_fam( jas_image_clrspc( psImage ) ) == 
+    if ( jas_clrspc_fam( jas_image_clrspc( psImage ) ) ==
               JAS_CLRSPC_FAM_YCBCR )
     {
         jas_image_t *psRGBImage;
@@ -502,7 +522,7 @@ int JPEG2000Dataset::DecodeImage()
         jas_cmprof_destroy(psRGBProf);
         psImage = psRGBImage;
     }
-    
+
     return TRUE;
 }
 
@@ -525,11 +545,11 @@ int JPEG2000Dataset::Identify( GDALOpenInfo * poOpenInfo )
 {
     static const unsigned char jpc_header[] = {0xff,0x4f};
     static const unsigned char jp2_box_jp[] = {0x6a,0x50,0x20,0x20}; /* 'jP  ' */
-        
-    if( poOpenInfo->nHeaderBytes >= 16 
-        && (memcmp( poOpenInfo->pabyHeader, jpc_header, 
+
+    if( poOpenInfo->nHeaderBytes >= 16
+        && (memcmp( poOpenInfo->pabyHeader, jpc_header,
                     sizeof(jpc_header) ) == 0
-            || memcmp( poOpenInfo->pabyHeader + 4, jp2_box_jp, 
+            || memcmp( poOpenInfo->pabyHeader + 4, jp2_box_jp,
                     sizeof(jp2_box_jp) ) == 0
             /* PGX file*/
             || (memcmp( poOpenInfo->pabyHeader, "PG", 2) == 0 &&
@@ -537,7 +557,7 @@ int JPEG2000Dataset::Identify( GDALOpenInfo * poOpenInfo )
                 (memcmp( poOpenInfo->pabyHeader + 3, "ML", 2) == 0 ||
                  memcmp( poOpenInfo->pabyHeader + 3, "LM", 2) == 0))) )
         return TRUE;
-    
+
     else
         return FALSE;
 }
@@ -552,7 +572,7 @@ GDALDataset *JPEG2000Dataset::Open( GDALOpenInfo * poOpenInfo )
     int         iFormat;
     char        *pszFormatName = NULL;
     jas_stream_t *sS;
-    
+
     if (!Identify(poOpenInfo))
         return NULL;
 
@@ -569,23 +589,23 @@ GDALDataset *JPEG2000Dataset::Open( GDALOpenInfo * poOpenInfo )
         return NULL;
     }
     if ( strlen( pszFormatName ) < 3 ||
-        (!EQUALN( pszFormatName, "jp2", 3 ) &&
-         !EQUALN( pszFormatName, "jpc", 3 ) &&
-         !EQUALN( pszFormatName, "pgx", 3 )) )
+        (!STARTS_WITH_CI(pszFormatName, "jp2") &&
+         !STARTS_WITH_CI(pszFormatName, "jpc") &&
+         !STARTS_WITH_CI(pszFormatName, "pgx")) )
     {
-        CPLDebug( "JPEG2000", "JasPer reports file is format type `%s'.", 
+        CPLDebug( "JPEG2000", "JasPer reports file is format type `%s'.",
                   pszFormatName );
         jas_stream_close( sS );
         return NULL;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Confirm the requested access is supported.                      */
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->eAccess == GA_Update )
     {
         jas_stream_close(sS);
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The JPEG2000 driver does not support update access to existing"
                   " datasets.\n" );
         return NULL;
@@ -602,20 +622,19 @@ GDALDataset *JPEG2000Dataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->psStream = sS;
     poDS->iFormat = iFormat;
 
-    if ( EQUALN( pszFormatName, "jp2", 3 ) )
+    if ( STARTS_WITH_CI(pszFormatName, "jp2") )
     {
         // XXX: Hack to read JP2 boxes from input file. JasPer hasn't public
         // API call for such things, so we will use internal JasPer functions.
         jp2_box_t *box;
-        box = 0;
-        while ( ( box = jp2_box_get(poDS->psStream) ) )
+        while ( ( box = jp2_box_get(poDS->psStream) ) != NULL )
         {
             switch (box->type)
             {
                 case JP2_BOX_IHDR:
-                poDS->nBands = box->data.ihdr.numcmpts;
-                poDS->nRasterXSize = box->data.ihdr.width;
-                poDS->nRasterYSize = box->data.ihdr.height;
+                poDS->nBands = static_cast<int>(box->data.ihdr.numcmpts);
+                poDS->nRasterXSize = static_cast<int>(box->data.ihdr.width);
+                poDS->nRasterYSize = static_cast<int>(box->data.ihdr.height);
                 CPLDebug( "JPEG2000",
                           "IHDR box found. Dump: "
                           "width=%d, height=%d, numcmpts=%d, bpp=%d",
@@ -624,7 +643,7 @@ GDALDataset *JPEG2000Dataset::Open( GDALOpenInfo * poOpenInfo )
                 /* ISO/IEC 15444-1:2004 I.5.3.1 specifies that 255 means that all */
                 /* components have not the same bit depth and/or sign and that a */
                 /* BPCC box must then follow to specify them for each component */
-                if ( box->data.ihdr.bpc != 255 )
+                if ( box->data.ihdr.bpc != 255 && paiDepth == NULL && pabSignedness == NULL )
                 {
                     paiDepth = (int *)CPLMalloc(poDS->nBands * sizeof(int));
                     pabSignedness = (int *)CPLMalloc(poDS->nBands * sizeof(int));
@@ -683,18 +702,22 @@ GDALDataset *JPEG2000Dataset::Open( GDALOpenInfo * poOpenInfo )
                 break;
             }
             jp2_box_destroy( box );
-            box = 0;
+            box = NULL;
         }
         if( !paiDepth || !pabSignedness )
         {
             delete poDS;
             CPLDebug( "JPEG2000", "Unable to read JP2 header boxes.\n" );
+            CPLFree( paiDepth );
+            CPLFree( pabSignedness );
             return NULL;
         }
         if ( jas_stream_rewind( poDS->psStream ) < 0 )
         {
             delete poDS;
             CPLDebug( "JPEG2000", "Unable to rewind input stream.\n" );
+            CPLFree( paiDepth );
+            CPLFree( pabSignedness );
             return NULL;
         }
     }
@@ -707,8 +730,8 @@ GDALDataset *JPEG2000Dataset::Open( GDALOpenInfo * poOpenInfo )
         }
 
         poDS->nBands = jas_image_numcmpts( poDS->psImage );
-        poDS->nRasterXSize = jas_image_cmptwidth( poDS->psImage, 0 );
-        poDS->nRasterYSize = jas_image_cmptheight( poDS->psImage, 0 );
+        poDS->nRasterXSize = static_cast<int>(jas_image_cmptwidth( poDS->psImage, 0 ));
+        poDS->nRasterYSize = static_cast<int>(jas_image_cmptheight( poDS->psImage, 0 ));
         paiDepth = (int *)CPLMalloc( poDS->nBands * sizeof(int) );
         pabSignedness = (int *)CPLMalloc( poDS->nBands * sizeof(int) );
         for ( iBand = 0; iBand < poDS->nBands; iBand++ )
@@ -717,7 +740,7 @@ GDALDataset *JPEG2000Dataset::Open( GDALOpenInfo * poOpenInfo )
             pabSignedness[iBand] = jas_image_cmptsgnd( poDS->psImage, iBand );
         }
     }
-    
+
     if ( !GDALCheckDatasetDimensions(poDS->nRasterXSize, poDS->nRasterYSize) ||
          !GDALCheckBandCount(poDS->nBands, 0) )
     {
@@ -727,21 +750,33 @@ GDALDataset *JPEG2000Dataset::Open( GDALOpenInfo * poOpenInfo )
         return NULL;
     }
 
+/* -------------------------------------------------------------------- */
+/*      Should we promote alpha channel to 8 bits ?                     */
+/* -------------------------------------------------------------------- */
+    poDS->bPromoteTo8Bit = (poDS->nBands == 4 &&
+                            paiDepth[0] == 8 &&
+                            paiDepth[1] == 8 &&
+                            paiDepth[2] == 8 &&
+                            paiDepth[3] == 1 &&
+                            CSLFetchBoolean(poOpenInfo->papszOpenOptions, "1BIT_ALPHA_PROMOTION", TRUE));
+    if( poDS->bPromoteTo8Bit )
+        CPLDebug( "JPEG2000",  "Fourth (alpha) band is promoted from 1 bit to 8 bit");
+
 /* -------------------------------------------------------------------- */
 
 /*      Create band information objects.                                */
 /* -------------------------------------------------------------------- */
+
+
     for( iBand = 1; iBand <= poDS->nBands; iBand++ )
     {
         poDS->SetBand( iBand, new JPEG2000RasterBand( poDS, iBand,
             paiDepth[iBand - 1], pabSignedness[iBand - 1] ) );
-        
+
     }
-    
-    if ( paiDepth )
-        CPLFree( paiDepth );
-    if ( pabSignedness )
-        CPLFree( pabSignedness );
+
+    CPLFree( paiDepth );
+    CPLFree( pabSignedness );
 
     poDS->LoadJP2Metadata(poOpenInfo);
 
@@ -755,7 +790,25 @@ GDALDataset *JPEG2000Dataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Check for overviews.                                            */
 /* -------------------------------------------------------------------- */
     poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
-    
+
+/* -------------------------------------------------------------------- */
+/*      Vector layers                                                   */
+/* -------------------------------------------------------------------- */
+    if( poOpenInfo->nOpenFlags & GDAL_OF_VECTOR )
+    {
+        poDS->LoadVectorLayers(
+            CSLFetchBoolean(poOpenInfo->papszOpenOptions, "OPEN_REMOTE_GML", FALSE));
+
+        // If file opened in vector-only mode and there's no vector,
+        // return
+        if( (poOpenInfo->nOpenFlags & GDAL_OF_RASTER) == 0 &&
+            poDS->GetLayerCount() == 0 )
+        {
+            delete poDS;
+            return NULL;
+        }
+    }
+
     return( poDS );
 }
 
@@ -764,8 +817,8 @@ GDALDataset *JPEG2000Dataset::Open( GDALOpenInfo * poOpenInfo )
 /************************************************************************/
 
 static GDALDataset *
-JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS, 
-                    int bStrict, char ** papszOptions, 
+JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
+                    int bStrict, char ** papszOptions,
                     GDALProgressFunc pfnProgress, void * pProgressData )
 
 {
@@ -784,14 +837,14 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 
     if (poSrcDS->GetRasterBand(1)->GetColorTable() != NULL)
     {
-        CPLError( (bStrict) ? CE_Failure : CE_Warning, CPLE_NotSupported, 
+        CPLError( (bStrict) ? CE_Failure : CE_Warning, CPLE_NotSupported,
                   "JPEG2000 driver ignores color table. "
                   "The source raster band will be considered as grey level.\n"
                   "Consider using color table expansion (-expand option in gdal_translate)\n");
         if (bStrict)
             return NULL;
     }
-    
+
     for ( iBand = 0; iBand < nBands; iBand++ )
     {
         poBand = poSrcDS->GetRasterBand( iBand + 1);
@@ -804,7 +857,7 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                 break;
 
             default:
-                if( !CSLTestBoolean(CPLGetConfigOption("JPEG2000_FORCE_CREATION", "NO")) )
+                if( !CPLTestBool(CPLGetConfigOption("JPEG2000_FORCE_CREATION", "NO")) )
                 {
                     CPLError(CE_Failure, CPLE_AppDefined,
                              "A band of the source dataset is of type %s, which might cause crashes in libjasper. "
@@ -815,7 +868,7 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                 break;
         }
     }
-    
+
     if( !pfnProgress( 0.0, NULL, pProgressData ) )
         return NULL;
 
@@ -826,18 +879,19 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
     jas_image_t         *psImage;
 
     JPEG2000Init();
-    const char* pszAccess = EQUALN(pszFilename, "/vsisubfile/", 12) ? "r+b" : "w+b";
+    const char* pszAccess = STARTS_WITH_CI(pszFilename, "/vsisubfile/") ? "r+b" : "w+b";
     if( !(psStream = JPEG2000_VSIL_fopen( pszFilename, pszAccess) ) )
     {
-        CPLError( CE_Failure, CPLE_FileIO, "Unable to create file %s.\n", 
+        CPLError( CE_Failure, CPLE_FileIO, "Unable to create file %s.\n",
                   pszFilename );
         return NULL;
     }
-    
+
     if ( !(psImage = jas_image_create0()) )
     {
-        CPLError( CE_Failure, CPLE_OutOfMemory, "Unable to create image %s.\n", 
+        CPLError( CE_Failure, CPLE_OutOfMemory, "Unable to create image %s.\n",
                   pszFilename );
+        jas_stream_close( psStream );
         return NULL;
     }
 
@@ -852,22 +906,23 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 
     sComps = (jas_image_cmptparm_t*)
         CPLMalloc( nBands * sizeof(jas_image_cmptparm_t) );
-  
+
     if ( !(psMatrix = jas_matrix_create( 1, nXSize )) )
     {
-        CPLError( CE_Failure, CPLE_OutOfMemory, 
+        CPLError( CE_Failure, CPLE_OutOfMemory,
                   "Unable to create matrix with size %dx%d.\n", 1, nYSize );
         CPLFree( sComps );
         jas_image_destroy( psImage );
+        jas_stream_close( psStream );
         return NULL;
     }
     paiScanline = (GUInt32 *) CPLMalloc( nXSize *
                             GDALGetDataTypeSize(GDT_UInt32) / 8 );
-    
+
     for ( iBand = 0; iBand < nBands; iBand++ )
     {
         poBand = poSrcDS->GetRasterBand( iBand + 1);
-        
+
         sComps[iBand].tlx = sComps[iBand].tly = 0;
         sComps[iBand].hstep = sComps[iBand].vstep = 1;
         sComps[iBand].width = nXSize;
@@ -892,32 +947,33 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 
         for( iLine = 0; eErr == CE_None && iLine < nYSize; iLine++ )
         {
-            eErr = poBand->RasterIO( GF_Read, 0, iLine, nXSize, 1, 
+            eErr = poBand->RasterIO( GF_Read, 0, iLine, nXSize, 1,
                               paiScanline, nXSize, 1, GDT_UInt32,
-                              sizeof(GUInt32), sizeof(GUInt32) * nXSize );
+                              sizeof(GUInt32), sizeof(GUInt32) * nXSize, NULL );
             for ( iPixel = 0; iPixel < nXSize; iPixel++ )
                 jas_matrix_setv( psMatrix, iPixel, paiScanline[iPixel] );
-            
+
             if( (jas_image_writecmpt(psImage, iBand, 0, iLine,
                               nXSize, 1, psMatrix)) < 0 )
             {
-                CPLError( CE_Failure, CPLE_AppDefined, 
-                    "Unable to write scanline %d of the component %d.\n", 
+                CPLError( CE_Failure, CPLE_AppDefined,
+                    "Unable to write scanline %d of the component %d.\n",
                     iLine, iBand );
                 jas_matrix_destroy( psMatrix );
                 CPLFree( paiScanline );
                 CPLFree( sComps );
                 jas_image_destroy( psImage );
+                jas_stream_close( psStream );
                 return NULL;
             }
-            
+
             if( eErr == CE_None &&
             !pfnProgress( ((iLine + 1) + iBand * nYSize) /
                           ((double) nYSize * nBands),
                          NULL, pProgressData) )
             {
                 eErr = CE_Failure;
-                CPLError( CE_Failure, CPLE_UserInterrupt, 
+                CPLError( CE_Failure, CPLE_UserInterrupt,
                       "User terminated CreateCopy()" );
             }
         }
@@ -959,13 +1015,13 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
         "numgbits",
         NULL
     };
-    
+
     pszFormatName = CSLFetchNameValue( papszOptions, "FORMAT" );
     if ( !pszFormatName ||
-         (!EQUALN( pszFormatName, "jp2", 3 ) &&
-          !EQUALN( pszFormatName, "jpc", 3 ) ) )
+         (!STARTS_WITH_CI(pszFormatName, "jp2") &&
+          !STARTS_WITH_CI(pszFormatName, "jpc") ) )
         pszFormatName = "jp2";
-    
+
     pszOptionBuf[0] = '\0';
     if ( papszOptions )
     {
@@ -979,8 +1035,8 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                 {
                     int m, n;
 
-                    n = strlen( pszOptionBuf );
-                    m = n + strlen( papszOptions[i] ) + 1;
+                    n = static_cast<int>(strlen( pszOptionBuf ));
+                    m = n + static_cast<int>(strlen( papszOptions[i] )) + 1;
                     if ( m > OPTSMAX )
                         break;
                     if ( n > 0 )
@@ -1029,7 +1085,7 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
     {
         /* JAS_CLRSPC_UNKNOWN causes crashes in Jasper jp2_enc.c at line 231 */
         /* iccprof = jas_iccprof_createfromcmprof(jas_image_cmprof(image)); */
-        /* but if we explictely set the cmprof, it does not work better */
+        /* but if we explicitly set the cmprof, it does not work better */
         /* since it would abort at line 281 later ... */
         /* So the best option is to switch to gray colorspace */
         /* And we need to switch at the band level too, otherwise Kakadu or */
@@ -1045,18 +1101,20 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 /*      Set the GeoTIFF box if georeferencing is available, and this    */
 /*      is a JP2 file.                                                  */
 /* -------------------------------------------------------------------- */
-    if ( EQUALN( pszFormatName, "jp2", 3 ) )
+    if ( STARTS_WITH_CI(pszFormatName, "jp2") )
     {
 #ifdef HAVE_JASPER_UUID
         double  adfGeoTransform[6];
-        if( ((poSrcDS->GetGeoTransform(adfGeoTransform) == CE_None
-                 && (adfGeoTransform[0] != 0.0 
-                     || adfGeoTransform[1] != 1.0 
-                     || adfGeoTransform[2] != 0.0 
-                     || adfGeoTransform[3] != 0.0 
-                     || adfGeoTransform[4] != 0.0 
+        if( CSLFetchBoolean( papszOptions, "GeoJP2", TRUE ) &&
+            ((poSrcDS->GetGeoTransform(adfGeoTransform) == CE_None
+                 && (adfGeoTransform[0] != 0.0
+                     || adfGeoTransform[1] != 1.0
+                     || adfGeoTransform[2] != 0.0
+                     || adfGeoTransform[3] != 0.0
+                     || adfGeoTransform[4] != 0.0
                      || ABS(adfGeoTransform[5]) != 1.0))
-                || poSrcDS->GetGCPCount() > 0) )
+                || poSrcDS->GetGCPCount() > 0
+                || poSrcDS->GetMetadata("RPC") != NULL ) )
         {
             GDALJP2Metadata oJP2Geo;
 
@@ -1071,6 +1129,8 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                 oJP2Geo.SetGeoTransform( adfGeoTransform );
             }
 
+            oJP2Geo.SetRPCMD(  poSrcDS->GetMetadata("RPC") );
+
             const char* pszAreaOrPoint = poSrcDS->GetMetadataItem(GDALMD_AREA_OR_POINT);
             oJP2Geo.bPixelIsPoint = pszAreaOrPoint != NULL && EQUAL(pszAreaOrPoint, GDALMD_AOP_POINT);
 
@@ -1094,6 +1154,7 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                 CPLFree( paiScanline );
                 CPLFree( sComps );
                 jas_image_destroy( psImage );
+                jas_stream_close( psStream );
                 return NULL;
             }
             jp2_box_destroy( box );
@@ -1109,6 +1170,7 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                 CPLFree( paiScanline );
                 CPLFree( sComps );
                 jas_image_destroy( psImage );
+                jas_stream_close( psStream );
                 return NULL;
             }
 #ifdef HAVE_JASPER_UUID
@@ -1125,12 +1187,13 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
             CPLFree( paiScanline );
             CPLFree( sComps );
             jas_image_destroy( psImage );
+            jas_stream_close( psStream );
             return NULL;
         }
     }
 
     jas_stream_flush( psStream );
-    
+
     jas_matrix_destroy( psMatrix );
     CPLFree( paiScanline );
     CPLFree( sComps );
@@ -1142,25 +1205,138 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
         return NULL;
     }
 
+/* -------------------------------------------------------------------- */
+/*      Add GMLJP2 box at end of file.                                  */
+/* -------------------------------------------------------------------- */
+    if ( STARTS_WITH_CI(pszFormatName, "jp2") )
+    {
+        double  adfGeoTransform[6];
+        if( CSLFetchBoolean( papszOptions, "GMLJP2", TRUE ) &&
+            poSrcDS->GetGeoTransform(adfGeoTransform) == CE_None &&
+            poSrcDS->GetProjectionRef() != NULL &&
+            poSrcDS->GetProjectionRef()[0] != '\0' )
+        {
+            VSILFILE* fp = VSIFOpenL(pszFilename, "rb+");
+            if( fp )
+            {
+                // Look for jp2c box and patch its LBox to be the real box size
+                // instead of zero
+                int bOK = FALSE;
+                GUInt32   nLBox;
+                GUInt32   nTBox;
+
+                while( true )
+                {
+                    if( VSIFReadL(&nLBox, 4, 1, fp) != 1 ||
+                        VSIFReadL(&nTBox, 4, 1, fp) != 1 )
+                        break;
+                    nLBox = CPL_MSBWORD32( nLBox );
+                    if( memcmp(&nTBox, "jp2c", 4) == 0 )
+                    {
+                        if( nLBox >= 8 )
+                        {
+                            bOK = TRUE;
+                            break;
+                        }
+                        if( nLBox == 0 )
+                        {
+                            vsi_l_offset nPos = VSIFTellL(fp);
+                            VSIFSeekL(fp, 0, SEEK_END);
+                            vsi_l_offset nEnd = VSIFTellL(fp);
+                            VSIFSeekL(fp, nPos - 8, SEEK_SET);
+                            nLBox = (GUInt32)(8 + nEnd - nPos);
+                            if( nLBox == (vsi_l_offset)8 + nEnd - nPos )
+                            {
+                                nLBox = CPL_MSBWORD32( nLBox );
+                                VSIFWriteL(&nLBox, 1, 4, fp);
+                                bOK = TRUE;
+                            }
+                        }
+                        break;
+                    }
+                    if( nLBox < 8 )
+                        break;
+                    VSIFSeekL(fp, nLBox - 8, SEEK_CUR);
+                }
+
+                // Can write GMLJP2 box
+                if( bOK )
+                {
+                    GDALJP2Metadata oJP2MD;
+                    oJP2MD.SetProjection( poSrcDS->GetProjectionRef() );
+                    oJP2MD.SetGeoTransform( adfGeoTransform );
+                    GDALJP2Box *poBox;
+                    const char* pszGMLJP2V2Def = CSLFetchNameValue( papszOptions, "GMLJP2V2_DEF" );
+                    if( pszGMLJP2V2Def != NULL )
+                        poBox = oJP2MD.CreateGMLJP2V2(nXSize,nYSize,pszGMLJP2V2Def,poSrcDS);
+                    else
+                        poBox = oJP2MD.CreateGMLJP2(nXSize,nYSize);
+
+                    nLBox = (int) poBox->GetDataLength() + 8;
+                    nLBox = CPL_MSBWORD32( nLBox );
+                    memcpy(&nTBox, poBox->GetType(), 4);
+
+                    VSIFSeekL(fp, 0, SEEK_END);
+                    VSIFWriteL( &nLBox, 4, 1, fp );
+                    VSIFWriteL( &nTBox, 4, 1, fp );
+                    VSIFWriteL(poBox->GetWritableData(), 1, (int) poBox->GetDataLength(), fp);
+                    VSIFCloseL(fp);
+
+                    delete poBox;
+                }
+            }
+        }
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Do we need a world file?                                        */
 /* -------------------------------------------------------------------- */
     if( CSLFetchBoolean( papszOptions, "WORLDFILE", FALSE ) )
     {
         double      adfGeoTransform[6];
-        
+
         poSrcDS->GetGeoTransform( adfGeoTransform );
         GDALWriteWorldFile( pszFilename, "wld", adfGeoTransform );
     }
 
 /* -------------------------------------------------------------------- */
-/*      Re-open dataset, and copy any auxilary pam information.         */
+/*      Re-open dataset, and copy any auxiliary pam information.         */
 /* -------------------------------------------------------------------- */
     GDALOpenInfo oOpenInfo(pszFilename, GA_ReadOnly);
     GDALPamDataset *poDS = (GDALPamDataset*) JPEG2000Dataset::Open(&oOpenInfo);
 
     if( poDS )
-        poDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
+    {
+        poDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT & (~GCIF_METADATA) );
+
+        /* Only write relevant metadata to PAM, and if needed */
+        char** papszSrcMD = CSLDuplicate(poSrcDS->GetMetadata());
+        papszSrcMD = CSLSetNameValue(papszSrcMD, GDALMD_AREA_OR_POINT, NULL);
+        papszSrcMD = CSLSetNameValue(papszSrcMD, "Corder", NULL);
+        for(char** papszSrcMDIter = papszSrcMD;
+                papszSrcMDIter && *papszSrcMDIter; )
+        {
+            /* Remove entries like KEY= (without value) */
+            if( (*papszSrcMDIter)[0] &&
+                (*papszSrcMDIter)[strlen((*papszSrcMDIter))-1] == '=' )
+            {
+                CPLFree(*papszSrcMDIter);
+                memmove(papszSrcMDIter, papszSrcMDIter + 1,
+                        sizeof(char*) * (CSLCount(papszSrcMDIter + 1) + 1));
+            }
+            else
+                ++papszSrcMDIter;
+        }
+        char** papszMD = CSLDuplicate(poDS->GetMetadata());
+        papszMD = CSLSetNameValue(papszMD, GDALMD_AREA_OR_POINT, NULL);
+        if( papszSrcMD && papszSrcMD[0] != NULL &&
+            CSLCount(papszSrcMD) != CSLCount(papszMD) )
+        {
+            poDS->SetMetadata(papszSrcMD);
+        }
+        CSLDestroy(papszSrcMD);
+        CSLDestroy(papszMD);
+    }
 
     return poDS;
 }
@@ -1172,32 +1348,73 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 void GDALRegister_JPEG2000()
 
 {
-    GDALDriver  *poDriver;
-    
-    if (! GDAL_CHECK_VERSION("JPEG2000 driver"))
+    if( !GDAL_CHECK_VERSION( "JPEG2000 driver" ) )
         return;
 
-    if( GDALGetDriverByName( "JPEG2000" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "JPEG2000" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "JPEG-2000 part 1 (ISO/IEC 15444-1)" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_jpeg2000.html" );
-        poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, 
-                                   "Byte Int16 UInt16 Int32 UInt32" );
-        poDriver->SetMetadataItem( GDAL_DMD_MIMETYPE, "image/jp2" );
-        poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "jp2" );
-        
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    if( GDALGetDriverByName( "JPEG2000" ) != NULL )
+        return;
 
-        poDriver->pfnIdentify = JPEG2000Dataset::Identify;
-        poDriver->pfnOpen = JPEG2000Dataset::Open;
-        poDriver->pfnCreateCopy = JPEG2000CreateCopy;
+    GDALDriver *poDriver = new GDALDriver();
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    poDriver->SetDescription( "JPEG2000" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VECTOR, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
+                               "JPEG-2000 part 1 (ISO/IEC 15444-1), "
+                               "based on Jasper library" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_jpeg2000.html" );
+    poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES,
+                               "Byte Int16 UInt16 Int32 UInt32" );
+    poDriver->SetMetadataItem( GDAL_DMD_MIMETYPE, "image/jp2" );
+    poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "jp2" );
+
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+
+    poDriver->SetMetadataItem( GDAL_DMD_OPENOPTIONLIST,
+"<OpenOptionList>"
+"   <Option name='1BIT_ALPHA_PROMOTION' type='boolean' description='Whether a 1-bit alpha channel should be promoted to 8-bit' default='YES'/>"
+"   <Option name='OPEN_REMOTE_GML' type='boolean' description='Whether to load remote vector layers referenced by a link in a GMLJP2 v2 box' default='NO'/>"
+"</OpenOptionList>" );
+
+    poDriver->SetMetadataItem( GDAL_DMD_CREATIONOPTIONLIST,
+"<CreationOptionList>"
+"   <Option name='FORMAT' type='string-select' default='according to file extension. If unknown, default to J2K'>"
+"       <Value>JP2</Value>"
+"       <Value>JPC</Value>"
+"   </Option>"
+"   <Option name='GeoJP2' type='boolean' description='Whether to emit a GeoJP2 box' default='YES'/>"
+"   <Option name='GMLJP2' type='boolean' description='Whether to emit a GMLJP2 v1 box' default='YES'/>"
+"   <Option name='GMLJP2V2_DEF' type='string' description='Definition file to describe how a GMLJP2 v2 box should be generated. If set to YES, a minimal instance will be created'/>"
+"   <Option name='WORLDFILE' type='boolean' description='Whether to write a worldfile .wld' default='NO'/>"
+"   <Option name='imgareatlx' type='string' />"
+"   <Option name='imgareatly' type='string' />"
+"   <Option name='tilegrdtlx' type='string' />"
+"   <Option name='tilegrdtly' type='string' />"
+"   <Option name='tilewidth' type='string' />"
+"   <Option name='tileheight' type='string' />"
+"   <Option name='prcwidth' type='string' />"
+"   <Option name='prcheight' type='string' />"
+"   <Option name='cblkwidth' type='string' />"
+"   <Option name='cblkheight' type='string' />"
+"   <Option name='mode' type='string' />"
+"   <Option name='rate' type='string' />"
+"   <Option name='ilyrrates' type='string' />"
+"   <Option name='prg' type='string' />"
+"   <Option name='numrlvls' type='string' />"
+"   <Option name='sop' type='string' />"
+"   <Option name='eph' type='string' />"
+"   <Option name='lazy' type='string' />"
+"   <Option name='termall' type='string' />"
+"   <Option name='segsym' type='string' />"
+"   <Option name='vcausal' type='string' />"
+"   <Option name='pterm' type='string' />"
+"   <Option name='resetprob' type='string' />"
+"   <Option name='numgbits' type='string' />"
+"</CreationOptionList>"  );
+
+    poDriver->pfnIdentify = JPEG2000Dataset::Identify;
+    poDriver->pfnOpen = JPEG2000Dataset::Open;
+    poDriver->pfnCreateCopy = JPEG2000CreateCopy;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
-
