@@ -38,10 +38,11 @@
 #include <algorithm>
 #include <fstream>
 
+#include "cpl_conv.h"
 #include "cpl_error.h"
 #include "cpl_string.h"
-#include "cpl_conv.h"
 #include "cpl_vsi.h"
+#include "gdal_frmts.h"
 #include "ogr_spatialref.h"
 #include "../vrt/gdal_vrt.h"
 #include "../vrt/vrtdataset.h"
@@ -51,7 +52,7 @@ using namespace std;
 /************************************************************************/
 /*                           GenerateTiles()                            */
 /************************************************************************/
-void GenerateTiles(std::string filename,
+static void GenerateTiles(std::string filename,
                    CPL_UNUSED int zoom,
                    int rxsize,
                    int rysize,
@@ -72,10 +73,10 @@ void GenerateTiles(std::string filename,
 
     if (isJpegDriver && bands == 4)
         bands = 3;
-   
+
     poTmpDataset = poMemDriver->Create("", dxsize, dysize, bands, GDT_Byte, NULL);
-   
-    if (isJpegDriver == false)//Jpeg dataset only has one or three bands
+
+    if (!isJpegDriver)//Jpeg dataset only has one or three bands
     {
         if (bands < 4)//add transparency to files with one band or three bands
         {
@@ -84,11 +85,11 @@ void GenerateTiles(std::string filename,
         }
     }
 
-    int rowOffset = rysize/dysize;
-    int loopCount = rysize/rowOffset;
+    const int rowOffset = rysize/dysize;
+    const int loopCount = rysize/rowOffset;
     for (int row = 0; row < loopCount; row++)
     {
-        if (isJpegDriver == false)
+        if (!isJpegDriver)
         {
             for (int i = 0; i < dxsize; i++)
             {
@@ -99,36 +100,24 @@ void GenerateTiles(std::string filename,
         for (int band = 1; band <= bands; band++)
         {
             GDALRasterBand* poBand = poSrcDs->GetRasterBand(band);
-
             int hasNoData = 0;
-            bool isSigned = false;
-            double noDataValue = poBand->GetNoDataValue(&hasNoData);
+            const double noDataValue = poBand->GetNoDataValue(&hasNoData);
             const char* pixelType = poBand->GetMetadataItem("PIXELTYPE", "IMAGE_STRUCTURE");
-            if (pixelType)
-            {
-                if (strcmp(pixelType, "SIGNEDBYTE") == 0)
-                {
-                    isSigned = true; 
-                }
-            }
+            const bool isSigned = ( pixelType && (strcmp(pixelType, "SIGNEDBYTE") == 0) );
 
             int yOffset = ry + row * rowOffset;
-            bool bReadFailed = false;
-            if (poBand)
-            {
-                CPLErr errTest = 
-                    poBand->RasterIO( GF_Read, rx, yOffset, rxsize, rowOffset, pafScanline, dxsize, 1, GDT_Byte, 0, 0, NULL);
+            CPLErr errTest =
+                poBand->RasterIO( GF_Read, rx, yOffset, rxsize, rowOffset, pafScanline, dxsize, 1, GDT_Byte, 0, 0, NULL);
 
-                if ( errTest == CE_Failure )
-                {
-                    hasNoData = 1;
-                    bReadFailed = true;
-                }
+            const bool bReadFailed = ( errTest == CE_Failure );
+            if ( bReadFailed )
+            {
+                hasNoData = 1;
             }
 
 
             //fill the true or false for hadnoData array if the source data has nodata value
-            if (isJpegDriver == false)
+            if (!isJpegDriver)
             {
                 if (hasNoData == 1)
                 {
@@ -140,7 +129,7 @@ void GenerateTiles(std::string filename,
                         {
                             tmpv -= 128;
                         }
-                        if (tmpv == noDataValue || bReadFailed == true)
+                        if (tmpv == noDataValue || bReadFailed)
                         {
                             hadnoData[j] = true;
                         }
@@ -148,22 +137,22 @@ void GenerateTiles(std::string filename,
                 }
             }
 
-            if (bReadFailed == false)
+            if (!bReadFailed)
             {
                 GDALRasterBand* poBandtmp = poTmpDataset->GetRasterBand(band);
-                poBandtmp->RasterIO(GF_Write, 0, row, dxsize, 1, pafScanline, dxsize, 1, GDT_Byte, 
-                                    0, 0, NULL);
+                CPL_IGNORE_RET_VAL( poBandtmp->RasterIO(GF_Write, 0, row, dxsize, 1, pafScanline, dxsize, 1, GDT_Byte,
+                                    0, 0, NULL) );
             }
-        } 
+        }
 
         //fill the values for alpha band
-        if (isJpegDriver == false)
+        if (!isJpegDriver)
         {
             if (alphaBand)
             {
                 for (int i = 0; i < dxsize; i++)
                 {
-                    if (hadnoData[i] == true)
+                    if (hadnoData[i])
                     {
                         pafScanline[i] = 0;
                     }
@@ -171,10 +160,10 @@ void GenerateTiles(std::string filename,
                     {
                         pafScanline[i] = 255;
                     }
-                }    
+                }
 
-                alphaBand->RasterIO(GF_Write, 0, row, dxsize, 1, pafScanline, dxsize, 1, GDT_Byte, 
-                                    0, 0, NULL);
+                CPL_IGNORE_RET_VAL( alphaBand->RasterIO(GF_Write, 0, row, dxsize, 1, pafScanline, dxsize, 1, GDT_Byte,
+                                    0, 0, NULL) );
             }
         }
     }
@@ -200,12 +189,12 @@ void GenerateTiles(std::string filename,
 /************************************************************************/
 
 static
-int  GenerateRootKml(const char* filename, 
+int  GenerateRootKml(const char* filename,
                      const char* kmlfilename,
-                     double north, 
-                     double south, 
-                     double east, 
-                     double west, 
+                     double north,
+                     double south,
+                     double east,
+                     double west,
                      int tilesize,
                      const char* pszOverlayName,
                      const char* pszOverlayDescription)
@@ -223,7 +212,7 @@ int  GenerateRootKml(const char* filename,
     if( pszOverlayName == NULL )
         pszOverlayName = tmpfilename;
 
-    // If we haven't writen any features yet, output the layer's schema
+    // If we have not written any features yet, output the layer's schema.
     VSIFPrintfL(fp, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
     VSIFPrintfL(fp, "<kml xmlns=\"http://www.opengis.net/kml/2.2\">\n");
     VSIFPrintfL(fp, "\t<Document>\n");
@@ -285,11 +274,11 @@ int  GenerateRootKml(const char* filename,
 /************************************************************************/
 
 static
-int  GenerateChildKml(std::string filename, 
-                      int zoom, int ix, int iy, 
-                      double zoomxpixel, double zoomypixel, int dxsize, int dysize, 
-                      double south, double west, int xsize, 
-                      int ysize, int maxzoom, 
+int  GenerateChildKml(std::string filename,
+                      int zoom, int ix, int iy,
+                      double zoomxpixel, double zoomypixel, int dxsize, int dysize,
+                      double south, double west, int xsize,
+                      int ysize, int maxzoom,
                       OGRCoordinateTransformation * poTransform,
                       std::string fileExt,
                       bool fixAntiMeridian,
@@ -358,7 +347,7 @@ int  GenerateChildKml(std::string filename,
         if (tmp < tmp1)
         {
             ychildern.push_back(iy*2+1);
-        }     
+        }
         maxLodPix = 2048;
     }
 
@@ -490,7 +479,7 @@ int  GenerateChildKml(std::string filename,
     VSIFPrintfL(fp, "\t</Document>\n");
     VSIFPrintfL(fp, "</kml>\n");
     VSIFCloseL(fp);
-    
+
     return TRUE;
 }
 
@@ -520,7 +509,7 @@ GDALDataset *KmlSuperOverlayCreateCopy( const char * pszFilename,
     int bands = poSrcDS->GetRasterCount();
     if (bands != 1 && bands != 3 && bands != 4)
         return NULL;
-   
+
     //correct the file and get the directory
     char* output_dir = NULL;
     if (pszFilename == NULL)
@@ -541,7 +530,7 @@ GDALDataset *KmlSuperOverlayCreateCopy( const char * pszFilename,
         {
             isKmz = true;
         }
-    
+
         output_dir = CPLStrdup(CPLGetPath(pszFilename));
         if (strcmp(output_dir, "") == 0)
         {
@@ -612,16 +601,16 @@ GDALDataset *KmlSuperOverlayCreateCopy( const char * pszFilename,
     if (poSrcDS->GetProjectionRef() != NULL)
     {
         OGRSpatialReference poDsUTM;
-     
+
         char* projStr = (char*)poSrcDS->GetProjectionRef();
-     
+
         if (poDsUTM.importFromWkt(&projStr) == OGRERR_NONE)
         {
             if (poDsUTM.IsProjected())
             {
                 OGRSpatialReference poLatLong;
                 poLatLong.SetWellKnownGeogCS( "WGS84" );
-           
+
                 poTransform = OGRCreateCoordinateTransformation( &poDsUTM, &poLatLong );
                 if( poTransform != NULL )
                 {
@@ -632,7 +621,7 @@ GDALDataset *KmlSuperOverlayCreateCopy( const char * pszFilename,
         }
     }
 
-    bool fixAntiMeridian = CSLFetchBoolean( papszOptions, "FIX_ANTIMERIDIAN", FALSE );
+    bool fixAntiMeridian = CPL_TO_BOOL(CSLFetchBoolean( papszOptions, "FIX_ANTIMERIDIAN", FALSE ));
     if ( fixAntiMeridian && east < west )
     {
         east += 360;
@@ -676,7 +665,7 @@ GDALDataset *KmlSuperOverlayCreateCopy( const char * pszFilename,
         zoomypixels.push_back(fabs(adfGeoTransform[5]) * pow(2.0, (maxzoom - zoom)));
     }
 
-    std::string tmpFileName; 
+    std::string tmpFileName;
     std::vector<std::string> fileVector;
     int nRet;
 
@@ -697,7 +686,7 @@ GDALDataset *KmlSuperOverlayCreateCopy( const char * pszFilename,
                                north, south, east, west, (int)tilexsize,
                                pszOverlayName, pszOverlayDescription);
     }
-    
+
     if (nRet == FALSE)
     {
         OGRCoordinateTransformation::DestroyCT( poTransform );
@@ -766,7 +755,7 @@ GDALDataset *KmlSuperOverlayCreateCopy( const char * pszFilename,
 
         xloop = xloop>0 ? xloop : 1;
         yloop = yloop>0 ? yloop : 1;
-        
+
         std::stringstream zoomStr;
         zoomStr << zoom;
 
@@ -808,7 +797,7 @@ GDALDataset *KmlSuperOverlayCreateCopy( const char * pszFilename,
                     fileVector.push_back(filename);
                 }
 
-                GenerateTiles(filename, zoom, rxsize, rysize, ix, iy, rx, ry, dxsize, 
+                GenerateTiles(filename, zoom, rxsize, rysize, ix, iy, rx, ry, dxsize,
                               dysize, bands, poSrcDS, poOutputTileDriver, poMemDriver, isJpegDriver);
                 std::string childKmlfile = zoomDir + "/" + iyStr.str() + ".kml";
                 if (isKmz)
@@ -829,7 +818,7 @@ GDALDataset *KmlSuperOverlayCreateCopy( const char * pszFilename,
                     zoomypix = 1;
                 }
 
-                GenerateChildKml(childKmlfile, zoom, ix, iy, zoomxpix, zoomypix, 
+                GenerateChildKml(childKmlfile, zoom, ix, iy, zoomxpix, zoomypix,
                                  dxsize, dysize, tmpSouth, adfGeoTransform[0],
                                  xsize, ysize, maxzoom, poTransform, fileExt, fixAntiMeridian,
                                  pszAltitude, pszAltitudeMode);
@@ -865,7 +854,7 @@ static CPLString KMLRemoveSlash(const char* pszPathIn)
 {
     char* pszPath = CPLStrdup(pszPathIn);
 
-    while(TRUE)
+    while( true )
     {
         char* pszSlashDotDot = strstr(pszPath, "/../");
         if (pszSlashDotDot == NULL || pszSlashDotDot == pszPath)
@@ -931,9 +920,9 @@ int KmlSuperOverlayReadDataset::CloseDependentDatasets()
     int bRet = FALSE;
     if( poDSIcon != NULL )
     {
-        CPLString osFilename(poDSIcon->GetDescription());
+        CPLString l_osFilename(poDSIcon->GetDescription());
         delete poDSIcon;
-        VSIUnlink(osFilename);
+        VSIUnlink(l_osFilename);
         poDSIcon = NULL;
         bRet = TRUE;
     }
@@ -992,11 +981,11 @@ CPLErr KmlSuperOverlayReadDataset::GetGeoTransform( double * padfGeoTransform )
 /*                        KmlSuperOverlayRasterBand()                   */
 /************************************************************************/
 
-KmlSuperOverlayRasterBand::KmlSuperOverlayRasterBand(KmlSuperOverlayReadDataset* poDS,
-                                                     CPL_UNUSED int nBand)
+KmlSuperOverlayRasterBand::KmlSuperOverlayRasterBand(KmlSuperOverlayReadDataset* poDSIn,
+                                                     int)
 {
-    nRasterXSize = poDS->nRasterXSize;
-    nRasterYSize = poDS->nRasterYSize;
+    nRasterXSize = poDSIn->nRasterXSize;
+    nRasterYSize = poDSIn->nRasterYSize;
     eDataType = GDT_Byte;
     nBlockXSize = 256;
     nBlockYSize = 256;
@@ -1016,7 +1005,7 @@ CPLErr KmlSuperOverlayRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff, vo
         nXSize = nRasterXSize - nXOff;
     if( nYOff + nYSize > nRasterYSize )
         nYSize = nRasterYSize - nYOff;
-    
+
     GDALRasterIOExtraArg sExtraArg;
     INIT_RASTERIO_EXTRA_ARG(sExtraArg);
 
@@ -1099,14 +1088,14 @@ int KmlSuperOverlayGetBoundingBox(CPLXMLNode* psNode, double* adfExtents)
         psBox = CPLGetXMLNode(psNode, "LatLonAltBox");
     if( psBox == NULL )
         return FALSE;
-    
+
     const char* pszNorth = CPLGetXMLValue(psBox, "north", NULL);
     const char* pszSouth = CPLGetXMLValue(psBox, "south", NULL);
     const char* pszEast = CPLGetXMLValue(psBox, "east", NULL);
     const char* pszWest = CPLGetXMLValue(psBox, "west", NULL);
     if( pszNorth == NULL || pszSouth == NULL || pszEast == NULL || pszWest == NULL )
         return FALSE;
-    
+
     adfExtents[0] = CPLAtof(pszWest);
     adfExtents[1] = CPLAtof(pszSouth);
     adfExtents[2] = CPLAtof(pszEast);
@@ -1128,7 +1117,7 @@ class SubImageDesc
 CPLErr KmlSuperOverlayReadDataset::IRasterIO( GDALRWFlag eRWFlag,
                                int nXOff, int nYOff, int nXSize, int nYSize,
                                void * pData, int nBufXSize, int nBufYSize,
-                               GDALDataType eBufType, 
+                               GDALDataType eBufType,
                                int nBandCount, int *panBandMap,
                                GSpacing nPixelSpace, GSpacing nLineSpace,
                                GSpacing nBandSpace,
@@ -1155,7 +1144,7 @@ CPLErr KmlSuperOverlayReadDataset::IRasterIO( GDALRWFlag eRWFlag,
                                     nXSize * nOvrFactor,
                                     nYSize * nOvrFactor,
                                     pData, nBufXSize, nBufYSize,
-                                    eBufType, 
+                                    eBufType,
                                     nBandCount, panBandMap,
                                     nPixelSpace, nLineSpace, nBandSpace, &sExtraArgs);
     }
@@ -1198,13 +1187,13 @@ CPLErr KmlSuperOverlayReadDataset::IRasterIO( GDALRWFlag eRWFlag,
                     dfRequestYMin < adfExtents[3] && dfRequestYMax > adfExtents[1] )
                 {
                     CPLString osSubFilename;
-                    if( strncmp(pszHref, "http", 4) == 0)
+                    if( STARTS_WITH(pszHref, "http"))
                         osSubFilename = CPLSPrintf("/vsicurl_streaming/%s", pszHref);
                     else
                     {
                         const char* pszBaseFilename = osFilename.c_str();
                         if( EQUAL(CPLGetExtension(pszBaseFilename), "kmz") &&
-                            strncmp(pszBaseFilename, "/vsizip/", 8) != 0 )
+                            !STARTS_WITH(pszBaseFilename, "/vsizip/") )
                         {
                             osSubFilename = "/vsizip/";
                             osSubFilename += CPLGetPath(pszBaseFilename);
@@ -1222,68 +1211,69 @@ CPLErr KmlSuperOverlayReadDataset::IRasterIO( GDALRWFlag eRWFlag,
                     if( EQUAL(CPLGetExtension(osSubFilename), "kml") )
                     {
                         KmlSuperOverlayReadDataset* poRoot = poParent ? poParent : this;
-                        LinkedDataset* psLink = poRoot->oMapChildren[osSubFilename];
-                        if( psLink == NULL )
+                        LinkedDataset* psLinkDS = poRoot->oMapChildren[osSubFilename];
+                        if( psLinkDS == NULL )
                         {
                             if( poRoot->oMapChildren.size() == 64 )
                             {
-                                psLink = poRoot->psLastLink;
-                                CPLAssert(psLink);
-                                poRoot->oMapChildren.erase(psLink->osSubFilename);
-                                GDALClose(psLink->poDS);
-                                if( psLink->psPrev != NULL )
+                                psLinkDS = poRoot->psLastLink;
+                                CPLAssert(psLinkDS);
+                                poRoot->oMapChildren.erase(psLinkDS->osSubFilename);
+                                GDALClose(psLinkDS->poDS);
+                                if( psLinkDS->psPrev != NULL )
                                 {
-                                    poRoot->psLastLink = psLink->psPrev;
-                                    psLink->psPrev->psNext = NULL;
+                                    poRoot->psLastLink = psLinkDS->psPrev;
+                                    psLinkDS->psPrev->psNext = NULL;
                                 }
                                 else
                                 {
-                                    CPLAssert(psLink == poRoot->psFirstLink);
+                                    CPLAssert(psLinkDS == poRoot->psFirstLink);
                                     poRoot->psFirstLink = poRoot->psLastLink = NULL;
                                 }
                             }
                             else
-                                psLink = new LinkedDataset();
+                                psLinkDS = new LinkedDataset();
 
-                            poRoot->oMapChildren[osSubFilename] = psLink;
+                            poRoot->oMapChildren[osSubFilename] = psLinkDS;
                             poSubImageDS = (KmlSuperOverlayReadDataset*)
                                 KmlSuperOverlayReadDataset::Open(osSubFilename, poRoot);
                             if( poSubImageDS )
                                 poSubImageDS->MarkAsShared();
                             else
-                                CPLDebug("KMLSuperOverlay", "Cannt open %s", osSubFilename.c_str());
-                            psLink->osSubFilename = osSubFilename;
-                            psLink->poDS = poSubImageDS;
-                            psLink->psPrev = NULL;
-                            psLink->psNext = poRoot->psFirstLink;
+                                CPLDebug( "KMLSuperOverlay", "Cannot open %s",
+                                          osSubFilename.c_str() );
+                            psLinkDS->osSubFilename = osSubFilename;
+                            psLinkDS->poDS = poSubImageDS;
+                            psLinkDS->psPrev = NULL;
+                            psLinkDS->psNext = poRoot->psFirstLink;
                             if( poRoot->psFirstLink != NULL )
                             {
                                 CPLAssert(poRoot->psFirstLink->psPrev == NULL);
-                                poRoot->psFirstLink->psPrev = psLink;
+                                poRoot->psFirstLink->psPrev = psLinkDS;
                             }
                             else
-                                poRoot->psLastLink = psLink;
-                            poRoot->psFirstLink = psLink;
+                                poRoot->psLastLink = psLinkDS;
+                            poRoot->psFirstLink = psLinkDS;
                         }
                         else
                         {
-                            poSubImageDS = psLink->poDS;
-                            if( psLink != poRoot->psFirstLink )
+                            poSubImageDS = psLinkDS->poDS;
+                            if( psLinkDS != poRoot->psFirstLink )
                             {
-                                if( psLink == poRoot->psLastLink )
+                                if( psLinkDS == poRoot->psLastLink )
                                 {
-                                    poRoot->psLastLink = psLink->psPrev;
+                                    poRoot->psLastLink = psLinkDS->psPrev;
                                     CPLAssert(poRoot->psLastLink != NULL );
                                     poRoot->psLastLink->psNext = NULL;
                                 }
                                 else
-                                    psLink->psNext->psPrev = psLink->psPrev;
-                                CPLAssert( psLink->psPrev != NULL );
-                                psLink->psPrev->psNext = psLink->psNext;
-                                psLink->psPrev = NULL;
-                                poRoot->psFirstLink->psPrev = psLink;
-                                psLink->psNext = poRoot->psFirstLink;
-                                poRoot->psFirstLink = psLink;
+                                    psLinkDS->psNext->psPrev = psLinkDS->psPrev;
+                                CPLAssert( psLinkDS->psPrev != NULL );
+                                psLinkDS->psPrev->psNext = psLinkDS->psNext;
+                                psLinkDS->psPrev = NULL;
+                                poRoot->psFirstLink->psPrev = psLinkDS;
+                                psLinkDS->psNext = poRoot->psFirstLink;
+                                poRoot->psFirstLink = psLinkDS;
                             }
                         }
                     }
@@ -1422,8 +1412,9 @@ CPLErr KmlSuperOverlayReadDataset::IRasterIO( GDALRWFlag eRWFlag,
 
     GDALProgressFunc  pfnProgressGlobal = psExtraArg->pfnProgress;
     void             *pProgressDataGlobal = psExtraArg->pProgressData;
+    CPLErr eErr = CE_None;
 
-    for(int iBandIdx = 0; iBandIdx < nBandCount; iBandIdx++ )
+    for(int iBandIdx = 0; iBandIdx < nBandCount && eErr == CE_None; iBandIdx++ )
     {
         int nBand = panBandMap[iBandIdx];
 
@@ -1433,7 +1424,8 @@ CPLErr KmlSuperOverlayReadDataset::IRasterIO( GDALRWFlag eRWFlag,
             for(int j = 0; j < nBufYSize; j ++ )
             {
                 GDALCopyWords( &nVal, GDT_Byte, 0,
-                            ((GByte*) pData) + j * nLineSpace + iBandIdx * nBandSpace, eBufType, nPixelSpace,
+                            ((GByte*) pData) + j * nLineSpace + iBandIdx * nBandSpace, eBufType,
+                            static_cast<int>(nPixelSpace),
                             nBufXSize );
             }
             continue;
@@ -1454,13 +1446,13 @@ CPLErr KmlSuperOverlayReadDataset::IRasterIO( GDALRWFlag eRWFlag,
         INIT_RASTERIO_EXTRA_ARG(sExtraArgs);
         sExtraArgs.eResampleAlg = psExtraArg->eResampleAlg;
         sExtraArgs.pfnProgress = GDALScaledProgress;
-        sExtraArgs.pProgressData = 
+        sExtraArgs.pProgressData =
             GDALCreateScaledProgress( 1.0 * iBandIdx / nBandCount,
                                       1.0 * (iBandIdx + 1) / nBandCount,
                                       pfnProgressGlobal,
                                       pProgressDataGlobal );
 
-        poDSIcon->GetRasterBand(nIconBand)->RasterIO( eRWFlag,
+        eErr = poDSIcon->GetRasterBand(nIconBand)->RasterIO( eRWFlag,
                                                       nReqXOff,
                                                       nReqYOff,
                                                       nReqXSize,
@@ -1476,7 +1468,7 @@ CPLErr KmlSuperOverlayReadDataset::IRasterIO( GDALRWFlag eRWFlag,
     psExtraArg->pfnProgress = pfnProgressGlobal;
     psExtraArg->pProgressData = pProgressDataGlobal;
 
-    return CE_None;
+    return eErr;
 
 }
 
@@ -1502,7 +1494,8 @@ int KmlSuperOverlayFindRegionStartInternal(CPLXMLNode* psNode,
         *ppsLink = psLink;
         return TRUE;
     }
-    if( strcmp(psNode->pszValue, "Document") == 0 &&
+    if( (strcmp(psNode->pszValue, "Document") == 0 ||
+         strcmp(psNode->pszValue, "Folder") == 0) &&
         (psRegion = CPLGetXMLNode(psNode, "Region")) != NULL &&
         (psGroundOverlay = CPLGetXMLNode(psNode, "GroundOverlay")) != NULL )
     {
@@ -1567,14 +1560,14 @@ int KmlSuperOverlayReadDataset::Identify(GDALOpenInfo * poOpenInfo)
     if( !EQUAL(pszExt, "kml") ||
         strstr((const char*)poOpenInfo->pabyHeader, "<kml") == NULL )
         return FALSE;
-    
+
     for( int i=0;i<2;i++ )
     {
         if( strstr((const char*)poOpenInfo->pabyHeader, "<NetworkLink>") != NULL &&
             strstr((const char*)poOpenInfo->pabyHeader, "<Region>") != NULL &&
             strstr((const char*)poOpenInfo->pabyHeader, "<Link>") != NULL )
             return TRUE;
-        
+
         if( strstr((const char*)poOpenInfo->pabyHeader, "<Document>") != NULL &&
             strstr((const char*)poOpenInfo->pabyHeader, "<Region>") != NULL &&
             strstr((const char*)poOpenInfo->pabyHeader, "<GroundOverlay>") != NULL )
@@ -1583,7 +1576,7 @@ int KmlSuperOverlayReadDataset::Identify(GDALOpenInfo * poOpenInfo)
         if( i == 0 && !poOpenInfo->TryToIngest(1024*10) )
             return FALSE;
     }
-    
+
     return -1;
 }
 
@@ -1616,7 +1609,7 @@ GDALDataset* KmlSuperOverlayLoadIcon(const char* pszBaseFilename, const char* ps
     }
 
     CPLString osSubFilename;
-    if( strncmp(pszIcon, "http", 4) == 0)
+    if( STARTS_WITH(pszIcon, "http"))
         osSubFilename = CPLSPrintf("/vsicurl_streaming/%s", pszIcon);
     else
     {
@@ -1679,7 +1672,7 @@ static void KmlSuperOverlayComputeDepth(CPLString osFilename,
             if( EQUAL(pszExt, "kml") )
             {
                 CPLString osSubFilename;
-                if( strncmp(pszHref, "http", 4) == 0)
+                if( STARTS_WITH(pszHref, "http"))
                     osSubFilename = CPLSPrintf("/vsicurl_streaming/%s", pszHref);
                 else
                 {
@@ -1943,13 +1936,13 @@ void KmlSingleDocRasterDataset::BuildOverviews()
 /*                      KmlSingleDocRasterRasterBand()                  */
 /************************************************************************/
 
-KmlSingleDocRasterRasterBand::KmlSingleDocRasterRasterBand(KmlSingleDocRasterDataset* poDS,
-                                                           int nBand)
+KmlSingleDocRasterRasterBand::KmlSingleDocRasterRasterBand(KmlSingleDocRasterDataset* poDSIn,
+                                                           int nBandIn)
 {
-    this->poDS = poDS;
-    this->nBand = nBand;
-    nBlockXSize = poDS->nTileSize;
-    nBlockYSize = poDS->nTileSize;
+    this->poDS = poDSIn;
+    this->nBand = nBandIn;
+    nBlockXSize = poDSIn->nTileSize;
+    nBlockYSize = poDSIn->nTileSize;
     eDataType = GDT_Byte;
 }
 
@@ -1963,7 +1956,7 @@ CPLErr KmlSingleDocRasterRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
     KmlSingleDocRasterDataset* poGDS = (KmlSingleDocRasterDataset*) poDS;
     const char* pszImageFilename = CPLFormFilename( poGDS->osDirname,
         CPLSPrintf("kml_image_L%d_%d_%d", poGDS->nLevel, nBlockYOff, nBlockXOff), poGDS->osNominalExt );
-    if( poGDS->poCurTileDS == NULL || 
+    if( poGDS->poCurTileDS == NULL ||
         strcmp(CPLGetFilename(poGDS->poCurTileDS->GetDescription()),
                CPLGetFilename(pszImageFilename)) != 0 )
     {
@@ -2026,13 +2019,13 @@ CPLErr KmlSingleDocRasterRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                         if( poEntry != NULL )
                         {
                             if( nBand == 1 )
-                                ((GByte*) pImage)[j * nBlockXSize + i] = poEntry->c1;
+                                ((GByte*) pImage)[j * nBlockXSize + i] = static_cast<GByte>(poEntry->c1);
                             else if( nBand == 2 )
-                                ((GByte*) pImage)[j * nBlockXSize + i] = poEntry->c2;
+                                ((GByte*) pImage)[j * nBlockXSize + i] = static_cast<GByte>(poEntry->c2);
                             else if( nBand == 3 )
-                                ((GByte*) pImage)[j * nBlockXSize + i] = poEntry->c3;
+                                ((GByte*) pImage)[j * nBlockXSize + i] = static_cast<GByte>(poEntry->c3);
                             else
-                                ((GByte*) pImage)[j * nBlockXSize + i] = poEntry->c4;
+                                ((GByte*) pImage)[j * nBlockXSize + i] = static_cast<GByte>(poEntry->c4);
                         }
                     }
                 }
@@ -2126,7 +2119,7 @@ static void KmlSingleDocCollectTiles(CPLXMLNode* psNode,
         int level, j, i;
         char szExt[4];
         const char* pszHref = CPLGetXMLValue(psNode, "", "");
-        if( strncmp(pszHref, "http", 4) == 0 )
+        if( STARTS_WITH(pszHref, "http") )
         {
             osURLBase = CPLGetPath(pszHref);
         }
@@ -2289,9 +2282,9 @@ GDALDataset *KmlSuperOverlayReadDataset::Open(const char* pszFilename,
     const char* pszExt = CPLGetExtension(pszFilename);
     if( EQUAL(pszExt, "kmz") )
     {
-        if( strncmp(pszFilename, "/vsizip/", 8) != 0 )
+        if( !STARTS_WITH(pszFilename, "/vsizip/") )
             osFilename = CPLSPrintf("/vsizip/%s", pszFilename);
-        char** papszFiles = CPLReadDir(osFilename);
+        char** papszFiles = VSIReadDir(osFilename);
         if( papszFiles == NULL )
             return NULL;
         char** papszIter = papszFiles;
@@ -2325,6 +2318,15 @@ GDALDataset *KmlSuperOverlayReadDataset::Open(const char* pszFilename,
     if( psNode == NULL )
         return NULL;
 
+    GDALDataset* psSingleDocDS = KmlSingleDocRasterDataset::Open(pszFilename,
+                                                                 osFilename,
+                                                                 psNode);
+    if( psSingleDocDS != NULL )
+    {
+        CPLDestroyXMLNode(psNode);
+        return psSingleDocDS;
+    }
+
     CPLXMLNode* psRegion = NULL;
     CPLXMLNode* psDocument = NULL;
     CPLXMLNode* psGroundOverlay = NULL;
@@ -2332,11 +2334,8 @@ GDALDataset *KmlSuperOverlayReadDataset::Open(const char* pszFilename,
     if( !KmlSuperOverlayFindRegionStart(psNode, &psRegion,
                                         &psDocument, &psGroundOverlay, &psLink) )
     {
-        GDALDataset* psDS = KmlSingleDocRasterDataset::Open(pszFilename,
-                                                            osFilename,
-                                                            psNode);
         CPLDestroyXMLNode(psNode);
-        return psDS;
+        return NULL;
     }
 
     if( psLink != NULL )
@@ -2349,7 +2348,7 @@ GDALDataset *KmlSuperOverlayReadDataset::Open(const char* pszFilename,
         }
 
         CPLString osSubFilename;
-        if( strncmp(pszHref, "http", 4) == 0)
+        if( STARTS_WITH(pszHref, "http"))
             osSubFilename = CPLSPrintf("/vsicurl_streaming/%s", pszHref);
         else
         {
@@ -2491,23 +2490,22 @@ static CPLErr KmlSuperOverlayDatasetDelete(CPL_UNUSED const char* fileName)
 /*                    GDALRegister_KMLSUPEROVERLAY()                    */
 /************************************************************************/
 
-void GDALRegister_KMLSUPEROVERLAY()
-   
-{
-    GDALDriver	*poDriver;
-   
-    if( GDALGetDriverByName( "KMLSUPEROVERLAY" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-      
-        poDriver->SetDescription( "KMLSUPEROVERLAY" );
-        poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "Kml Super Overlay" );
-        poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, 
-                                   "Byte Int16 UInt16 Int32 UInt32 Float32 Float64 CInt16 CInt32 CFloat32 CFloat64" );
+void CPL_DLL GDALRegister_KMLSUPEROVERLAY()
 
-        poDriver->SetMetadataItem( GDAL_DMD_CREATIONOPTIONLIST,
+{
+    if( GDALGetDriverByName( "KMLSUPEROVERLAY" ) != NULL )
+        return;
+
+    GDALDriver *poDriver = new GDALDriver();
+
+    poDriver->SetDescription( "KMLSUPEROVERLAY" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "Kml Super Overlay" );
+    poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES,
+                               "Byte Int16 UInt16 Int32 UInt32 Float32 Float64 "
+                               "CInt16 CInt32 CFloat32 CFloat64" );
+
+    poDriver->SetMetadataItem( GDAL_DMD_CREATIONOPTIONLIST,
 "<CreationOptionList>"
 "   <Option name='NAME' type='string' description='Overlay name'/>"
 "   <Option name='DESCRIPTION' type='string' description='Overlay description'/>"
@@ -2525,13 +2523,12 @@ void GDALRegister_KMLSUPEROVERLAY()
 "   <Option name='FIX_ANTIMERIDIAN' type='boolean' description='Fix for images crossing the antimeridian causing errors in Google Earth' />"
 "</CreationOptionList>" );
 
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
-        poDriver->pfnIdentify = KmlSuperOverlayReadDataset::Identify;
-        poDriver->pfnOpen = KmlSuperOverlayReadDataset::Open;
-        poDriver->pfnCreateCopy = KmlSuperOverlayCreateCopy;
-        poDriver->pfnDelete = KmlSuperOverlayDatasetDelete;
-      
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    poDriver->pfnIdentify = KmlSuperOverlayReadDataset::Identify;
+    poDriver->pfnOpen = KmlSuperOverlayReadDataset::Open;
+    poDriver->pfnCreateCopy = KmlSuperOverlayCreateCopy;
+    poDriver->pfnDelete = KmlSuperOverlayDatasetDelete;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
