@@ -27,12 +27,13 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
+#include "gdal_frmts.h"
 #include "gdal_pam.h"
 #include "gdal_proxy.h"
-#include "ogr_spatialref.h"
 #include "ogr_geometry.h"
+#include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: mapdataset.cpp 27560 2014-08-04 17:52:58Z rouault $");
+CPL_CVSID("$Id: mapdataset.cpp 33720 2016-03-15 00:39:53Z goatbar $");
 
 /************************************************************************/
 /* ==================================================================== */
@@ -82,9 +83,9 @@ class MAPWrapperRasterBand : public GDALProxyRasterBand
     virtual GDALRasterBand* RefUnderlyingRasterBand() { return poBaseBand; }
 
   public:
-    MAPWrapperRasterBand( GDALRasterBand* poBaseBand )
+    explicit MAPWrapperRasterBand( GDALRasterBand* poBaseBandIn )
         {
-            this->poBaseBand = poBaseBand;
+            this->poBaseBand = poBaseBandIn;
             eDataType = poBaseBand->GetRasterDataType();
             poBaseBand->GetBlockSize(&nBlockXSize, &nBlockYSize);
         }
@@ -97,21 +98,20 @@ class MAPWrapperRasterBand : public GDALProxyRasterBand
 /* ==================================================================== */
 /************************************************************************/
 
-MAPDataset::MAPDataset()
-
+MAPDataset::MAPDataset() :
+    poImageDS(NULL),
+    pszWKT(NULL),
+    bGeoTransformValid(false),
+    nGCPCount(0),
+    pasGCPList(NULL),
+    poNeatLine(NULL)
 {
-    poImageDS = NULL;
-    pszWKT = NULL;
     adfGeoTransform[0] = 0.0;
     adfGeoTransform[1] = 1.0;
     adfGeoTransform[2] = 0.0;
     adfGeoTransform[3] = 0.0;
     adfGeoTransform[4] = 0.0;
     adfGeoTransform[5] = 1.0;
-    nGCPCount = 0;
-    pasGCPList = NULL;
-    poNeatLine = NULL;
-    bGeoTransformValid = false;
 }
 
 /************************************************************************/
@@ -126,11 +126,9 @@ MAPDataset::~MAPDataset()
         GDALClose( poImageDS );
         poImageDS = NULL;
     }
-    if (pszWKT != NULL)
-    {
-        CPLFree(pszWKT);
-        pszWKT = NULL;
-    }
+
+    CPLFree(pszWKT);
+
     if (nGCPCount)
     {
         GDALDeinitGCPs( nGCPCount, pasGCPList );
@@ -167,22 +165,21 @@ int MAPDataset::CloseDependentDatasets()
 int MAPDataset::Identify( GDALOpenInfo *poOpenInfo )
 
 {
-    if( poOpenInfo->nHeaderBytes < 200 
+    if( poOpenInfo->nHeaderBytes < 200
         || !EQUAL(CPLGetExtension(poOpenInfo->pszFilename),"MAP") )
         return FALSE;
 
-    if( strstr((const char *) poOpenInfo->pabyHeader,"OziExplorer Map Data File") == NULL )
+    if( strstr(reinterpret_cast<const char *>( poOpenInfo->pabyHeader ),
+               "OziExplorer Map Data File") == NULL )
         return FALSE;
-    else
-        return TRUE;
+
+    return TRUE;
 }
 
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
 
-#define MAX_GCP 30
-
 GDALDataset *MAPDataset::Open( GDALOpenInfo * poOpenInfo )
 {
     if( !Identify( poOpenInfo ) )
@@ -193,12 +190,12 @@ GDALDataset *MAPDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->eAccess == GA_Update )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The MAP driver does not support update access to existing"
                   " datasets.\n" );
         return NULL;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
@@ -209,29 +206,30 @@ GDALDataset *MAPDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Try to load and parse the .MAP file.                            */
 /* -------------------------------------------------------------------- */
 
-    int bOziFileOK = 
-         GDALLoadOziMapFile( poOpenInfo->pszFilename,
-                             poDS->adfGeoTransform, 
+    bool bOziFileOK =
+         CPL_TO_BOOL(GDALLoadOziMapFile( poOpenInfo->pszFilename,
+                             poDS->adfGeoTransform,
                              &poDS->pszWKT,
-                             &poDS->nGCPCount, &poDS->pasGCPList );
+                             &poDS->nGCPCount, &poDS->pasGCPList ));
 
     if ( bOziFileOK && poDS->nGCPCount == 0 )
          poDS->bGeoTransformValid = TRUE;
 
-   /* We need to read again the .map file because the GDALLoadOziMapFile function
-      does not returns all required data . An API change is necessary : maybe in GDAL 2.0 ? */
+    /* We need to read again the .map file because the GDALLoadOziMapFile function
+       does not returns all required data . An API change is necessary : maybe in GDAL 2.0 ? */
 
-    char	**papszLines;
-    int		iLine, nLines=0;
-
-    papszLines = CSLLoad2( poOpenInfo->pszFilename, 200, 200, NULL );
+    char **papszLines = CSLLoad2( poOpenInfo->pszFilename, 200, 200, NULL );
 
     if ( !papszLines )
+    {
+        delete poDS;
         return NULL;
+    }
 
-    nLines = CSLCount( papszLines );
+    const int nLines = CSLCount( papszLines );
     if( nLines < 2 )
     {
+        delete poDS;
         CSLDestroy(papszLines);
         return NULL;
     }
@@ -244,7 +242,7 @@ GDALDataset *MAPDataset::Open( GDALOpenInfo * poOpenInfo )
     VSIStatBufL sStat;
     if (VSIStatL(poDS->osImgFilename, &sStat) != 0)
     {
-        CPLString osPath = CPLGetPath(poOpenInfo->pszFilename);
+        const CPLString osPath = CPLGetPath(poOpenInfo->pszFilename);
         if (CPLIsFilenameRelative(poDS->osImgFilename))
         {
             poDS->osImgFilename = CPLFormCIFilename(osPath, poDS->osImgFilename, NULL);
@@ -259,7 +257,8 @@ GDALDataset *MAPDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Try and open the file.                                          */
 /* -------------------------------------------------------------------- */
-    poDS->poImageDS = (GDALDataset *) GDALOpen(poDS->osImgFilename, GA_ReadOnly );
+    poDS->poImageDS = reinterpret_cast<GDALDataset *>(
+        GDALOpen(poDS->osImgFilename, GA_ReadOnly ) );
     if( poDS->poImageDS == NULL || poDS->poImageDS->GetRasterCount() == 0)
     {
         CSLDestroy(papszLines);
@@ -270,18 +269,16 @@ GDALDataset *MAPDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Attach the bands.                                               */
 /* -------------------------------------------------------------------- */
-    int iBand;
-
     poDS->nRasterXSize = poDS->poImageDS->GetRasterXSize();
     poDS->nRasterYSize = poDS->poImageDS->GetRasterYSize();
     if (!GDALCheckDatasetDimensions(poDS->nRasterXSize, poDS->nRasterYSize))
     {
-        delete poDS;
         GDALClose( poDS->poImageDS );
+        delete poDS;
         return NULL;
     }
 
-    for( iBand = 1; iBand <= poDS->poImageDS->GetRasterCount(); iBand++ )
+    for( int iBand = 1; iBand <= poDS->poImageDS->GetRasterCount(); iBand++ )
         poDS->SetBand( iBand,
                        new MAPWrapperRasterBand( poDS->poImageDS->GetRasterBand( iBand )) );
 
@@ -291,14 +288,14 @@ GDALDataset *MAPDataset::Open( GDALOpenInfo * poOpenInfo )
 
     /* First, we need to check if it is necessary to define a neatline */
     bool bNeatLine = false;
-    char **papszTok;
-    for ( iLine = 10; iLine < nLines; iLine++ )
+    for ( int iLine = 10; iLine < nLines; iLine++ )
     {
-        if ( EQUALN(papszLines[iLine], "MMPXY,", 6) )
+        if ( STARTS_WITH_CI(papszLines[iLine], "MMPXY,") )
         {
-            papszTok = CSLTokenizeString2( papszLines[iLine], ",",
-                                           CSLT_STRIPLEADSPACES
-                                           | CSLT_STRIPENDSPACES );
+            char **papszTok
+                = CSLTokenizeString2( papszLines[iLine], ",",
+                                      CSLT_STRIPLEADSPACES
+                                      | CSLT_STRIPENDSPACES );
 
             if ( CSLCount(papszTok) != 4 )
             {
@@ -306,8 +303,8 @@ GDALDataset *MAPDataset::Open( GDALOpenInfo * poOpenInfo )
                 continue;
             }
 
-            int x = CPLAtofM(papszTok[2]);
-            int y = CPLAtofM(papszTok[3]);
+            const int x = atoi(papszTok[2]);
+            const int y = atoi(papszTok[3]);
             if (( x != 0 && x != poDS->nRasterXSize) || (y != 0 && y != poDS->nRasterYSize) )
             {
                 bNeatLine = true;
@@ -326,13 +323,14 @@ GDALDataset *MAPDataset::Open( GDALOpenInfo * poOpenInfo )
 
         if ( poDS->bGeoTransformValid )        /* Compute the projected coordinates of the corners */
         {
-            for ( iLine = 10; iLine < nLines; iLine++ )
+            for ( int iLine = 10; iLine < nLines; iLine++ )
             {
-                if ( EQUALN(papszLines[iLine], "MMPXY,", 6) )
+                if ( STARTS_WITH_CI(papszLines[iLine], "MMPXY,") )
                 {
-                    papszTok = CSLTokenizeString2( papszLines[iLine], ",",
-                                                   CSLT_STRIPLEADSPACES
-                                                   | CSLT_STRIPENDSPACES );
+                    char **papszTok
+                        = CSLTokenizeString2( papszLines[iLine], ",",
+                                              CSLT_STRIPLEADSPACES
+                                              | CSLT_STRIPENDSPACES );
 
                     if ( CSLCount(papszTok) != 4 )
                     {
@@ -340,12 +338,12 @@ GDALDataset *MAPDataset::Open( GDALOpenInfo * poOpenInfo )
                         continue;
                     }
 
-                    double x = CPLAtofM(papszTok[2]);
-                    double y = CPLAtofM(papszTok[3]);
-                    double X = poDS->adfGeoTransform[0] + x * poDS->adfGeoTransform[1] +
-                                            y * poDS->adfGeoTransform[2];
-                    double Y = poDS->adfGeoTransform[3] + x * poDS->adfGeoTransform[4] +
-                                            y * poDS->adfGeoTransform[5];
+                    const double x = CPLAtofM(papszTok[2]);
+                    const double y = CPLAtofM(papszTok[3]);
+                    const double X = poDS->adfGeoTransform[0] + x * poDS->adfGeoTransform[1] +
+                        y * poDS->adfGeoTransform[2];
+                    const double Y = poDS->adfGeoTransform[3] + x * poDS->adfGeoTransform[4] +
+                        y * poDS->adfGeoTransform[5];
                     poRing->addPoint(X, Y);
                     CPLDebug( "CORNER MMPXY", "%f, %f, %f, %f", x, y, X, Y);
                     CSLDestroy(papszTok);
@@ -354,24 +352,24 @@ GDALDataset *MAPDataset::Open( GDALOpenInfo * poOpenInfo )
         }
         else /* Convert the geographic coordinates to projected coordinates */
         {
-            OGRSpatialReference oSRS;
-            OGRSpatialReference *poLatLong = NULL;
             OGRCoordinateTransformation *poTransform = NULL;
-            OGRErr eErr;
             char *pszWKT = poDS->pszWKT;
 
-            if ( &poDS->pszWKT != NULL )
+            if ( pszWKT != NULL )
             {
-                eErr = oSRS.importFromWkt ( &pszWKT );
-                if ( eErr == OGRERR_NONE )
+                OGRSpatialReference oSRS;
+                OGRSpatialReference *poLatLong = NULL;
+                if ( OGRERR_NONE == oSRS.importFromWkt ( &pszWKT ))
                     poLatLong = oSRS.CloneGeogCS();
-                    if ( poLatLong )
-                        poTransform = OGRCreateCoordinateTransformation( poLatLong, &oSRS );
+                if ( poLatLong )
+                    poTransform = OGRCreateCoordinateTransformation( poLatLong, &oSRS );
+                if ( poLatLong )
+                    delete poLatLong;
             }
 
-            for ( iLine = 10; iLine < nLines; iLine++ )
+            for ( int iLine = 10; iLine < nLines; iLine++ )
             {
-                if ( EQUALN(papszLines[iLine], "MMPLL,", 6) )
+                if ( STARTS_WITH_CI(papszLines[iLine], "MMPLL,") )
                 {
                     CPLDebug( "MMPLL", "%s", papszLines[iLine] );
                     char **papszTok = NULL;
@@ -396,10 +394,8 @@ GDALDataset *MAPDataset::Open( GDALOpenInfo * poOpenInfo )
                     CSLDestroy(papszTok);
                 }
             }
-            if (poTransform)
+            if ( poTransform )
                 delete poTransform;
-            if (poLatLong)
-                delete poLatLong;
         }
 
         poRing->closeRings();
@@ -435,7 +431,7 @@ CPLErr MAPDataset::GetGeoTransform( double * padfTransform )
 {
     memcpy(padfTransform, adfGeoTransform, 6 * sizeof(double));
 
-    return( (nGCPCount == 0) ? CE_None : CE_Failure );
+    return (nGCPCount == 0) ? CE_None : CE_Failure;
 }
 
 
@@ -475,7 +471,7 @@ char** MAPDataset::GetFileList()
     char **papszFileList = GDALDataset::GetFileList();
 
     papszFileList = CSLAddString( papszFileList, osImgFilename );
-    
+
     return papszFileList;
 }
 
@@ -486,25 +482,20 @@ char** MAPDataset::GetFileList()
 void GDALRegister_MAP()
 
 {
-    GDALDriver	*poDriver;
+    if( GDALGetDriverByName( "MAP" ) != NULL )
+        return;
 
-    if( GDALGetDriverByName( "MAP" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "MAP" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "OziExplorer .MAP" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_map.html" );
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    poDriver->SetDescription( "MAP" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "OziExplorer .MAP" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_map.html" );
 
-        poDriver->pfnOpen = MAPDataset::Open;
-        poDriver->pfnIdentify = MAPDataset::Identify;
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    poDriver->pfnOpen = MAPDataset::Open;
+    poDriver->pfnIdentify = MAPDataset::Identify;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
-
-
