@@ -8,47 +8,46 @@
 * Copyright (c) 2010, LizardTech
 * All rights reserved.
 
-* Redistribution and use in source and binary forms, with or without 
+* Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
-*   Redistributions of source code must retain the above copyright notice, 
+*   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
-* 
-*   Redistributions in binary form must reproduce the above copyright notice, 
+*
+*   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
-*   Neither the name of the LizardTech nor the names of its contributors may 
-*   be used to endorse or promote products derived from this software without 
+*   Neither the name of the LizardTech nor the names of its contributors may
+*   be used to endorse or promote products derived from this software without
 *   specific prior written permission.
 *
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
+* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
-#include "lidar/MG4PointReader.h"
-#include "lidar/FileIO.h"
-#include "lidar/Error.h"
-#include "lidar/Version.h"
+
+#ifdef DEBUG_BOOL
+#define DO_NOT_USE_DEBUG_BOOL
+#endif
+
+#include "mg4lidar_headers.h"
+
 #include <float.h>
 LT_USE_LIDAR_NAMESPACE
 
+#include "gdal_frmts.h"
 #include "gdal_pam.h"
 // #include "gdal_alg.h" // 1.6 and later have gridding algorithms
 
-CPL_C_START
-//void __declspec(dllexport) GDALRegister_MG4Lidar(void);
-void CPL_DLL GDALRegister_MG4Lidar(void);
-CPL_C_END
-
 /************************************************************************/
 /* ==================================================================== */
 /*				MG4LidarDataset				*/
@@ -69,8 +68,8 @@ class CropableMG4PointReader : public MG4PointReader
          setBounds(*bounds);
    }
 };
-CropableMG4PointReader::CropableMG4PointReader() : MG4PointReader() {};   
-CropableMG4PointReader::~CropableMG4PointReader() {};   
+CropableMG4PointReader::CropableMG4PointReader() : MG4PointReader() {};
+CropableMG4PointReader::~CropableMG4PointReader() {};
 
 IMPLEMENT_OBJECT_CREATE(CropableMG4PointReader);
 
@@ -79,12 +78,14 @@ static double MaxBlockSideSize = 1024.0;
 class MG4LidarDataset : public GDALPamDataset
 {
 friend class MG4LidarRasterBand;
+
 public:
    MG4LidarDataset();
    ~MG4LidarDataset();
    static GDALDataset *Open( GDALOpenInfo * );
    CPLErr 	GetGeoTransform( double * padfTransform );
    const char *GetProjectionRef();
+
 protected:
    MG4PointReader *reader;
    FileIO *fileIO;
@@ -94,7 +95,8 @@ protected:
    MG4LidarDataset **papoOverviewDS;
    CPLXMLNode *poXMLPCView;
    bool ownsXML;
-   int nBlockXSize, nBlockYSize;
+   int nBlockXSize;
+   int nBlockYSize;
    int iLevel;
 };
 
@@ -120,7 +122,7 @@ public:
    protected:
    double getMaxValue();
    double nodatavalue;
-   virtual const bool ElementPassesFilter(const PointData &, size_t);
+   virtual bool ElementPassesFilter(const PointData &, size_t);
    template<typename DTYPE>
    CPLErr   doReadBlock(int, int, void *);
    CPLXMLNode *poxmlBand;
@@ -160,7 +162,7 @@ default:
    CPLError(CE_Failure, CPLE_AssertionFailed,
       "Invalid datatype in MG4 file");
    break;
-#undef DO_CASE      
+#undef DO_CASE
    }
    // Coerce datatypes as required.
    const char * ForceDataType =  CPLGetXMLValue(pods->poXMLPCView, "Datatype", NULL);
@@ -190,7 +192,6 @@ default:
    else
       papszFilterReturnNums = CSLTokenizeString(poxmlFilter->psChild->pszValue);
 
-   
    CPLXMLNode * poxmlAggregation = CPLGetXMLNode(poxmlBand, "AggregationMethod");
    if( poxmlAggregation == NULL )
       poxmlAggregation = CPLGetXMLNode(pods->poXMLPCView, "AggregationMethod");
@@ -214,7 +215,7 @@ default:
       {
          papszParams = CSLTokenizeString(poxmlMethod->psChild->pszValue);
          if (!EQUAL(papszParams[0], "MAX"))
-            nodatavalue = atof(papszParams[0]);
+            nodatavalue = CPLAtof(papszParams[0]);
       }
       // else if .... Add support for other interpolation methods here.
       CSLDestroy(papszParams);
@@ -308,7 +309,7 @@ const DTYPE GetChannelElement(const ChannelData &channel, size_t idx)
 }
 
 
-const bool MG4LidarRasterBand::ElementPassesFilter(const PointData &pointdata, size_t i)
+bool MG4LidarRasterBand::ElementPassesFilter(const PointData &pointdata, size_t i)
 {
    bool bClassificationOK = true;
    bool bReturnNumOK = true;
@@ -316,7 +317,7 @@ const bool MG4LidarRasterBand::ElementPassesFilter(const PointData &pointdata, s
    // Check if classification code is ok:  it was requested and it does match one of the requested codes
    const int classcode = GetChannelElement<int>(*pointdata.getChannel(CHANNEL_NAME_ClassId), i);
    char bufCode[16];
-   sprintf(bufCode, "%d", classcode);
+   snprintf(bufCode, sizeof(bufCode), "%d", classcode);
    bClassificationOK = (papszFilterClassCodes == NULL ? true :
       (CSLFindString(papszFilterClassCodes,bufCode)!=-1));
 
@@ -324,7 +325,7 @@ const bool MG4LidarRasterBand::ElementPassesFilter(const PointData &pointdata, s
    {
       // Check if return num is ok:  it was requested and it does match one of the requested return numbers
       const long returnnum= static_cast<const unsigned char *>(pointdata.getChannel(CHANNEL_NAME_ReturnNum)->getData())[i];
-      sprintf(bufCode, "%d", (int)returnnum);
+      snprintf(bufCode, sizeof(bufCode), "%d", (int)returnnum);
       bReturnNumOK = (papszFilterReturnNums == NULL ? true :
          (CSLFindString(papszFilterReturnNums, bufCode)!=-1));
       if (!bReturnNumOK && CSLFindString(papszFilterReturnNums, "Last")!=-1)
@@ -365,7 +366,7 @@ CPLErr   MG4LidarRasterBand::doReadBlock(int nBlockXOff, int nBlockYOff, void *
    }
 
    double geoTrans[6];
-   poGDS->GetGeoTransform(geoTrans);   
+   poGDS->GetGeoTransform(geoTrans);
    double xres = geoTrans[1];
    double yres = geoTrans[5];
 
@@ -374,7 +375,7 @@ CPLErr   MG4LidarRasterBand::doReadBlock(int nBlockXOff, int nBlockYOff, void *
    double xmax = xmin + nBlockXSize* xres;
    double ymax = reader->getBounds().y.max - (nBlockYOff * nBlockYSize* -yres);
    double ymin = ymax - nBlockYSize* -yres;
-   Bounds bounds(xmin, xmax,  ymin, ymax, -HUGE_VAL, +HUGE_VAL); 
+   Bounds bounds(xmin, xmax,  ymin, ymax, -HUGE_VAL, +HUGE_VAL);
    PointData pointdata;
    pointdata.init(reader->getPointInfo(), 4096);
    double fraction = 1.0/pow(RESOLUTION_RATIO, poGDS->iLevel);
@@ -399,13 +400,13 @@ CPLErr   MG4LidarRasterBand::doReadBlock(int nBlockXOff, int nBlockYOff, void *
          col = floor (col);
          row = floor (row);
 
-         if (row < 0) 
+         if (row < 0)
             row = 0;
-         else if (row >= nBlockYSize) 
+         else if (row >= nBlockYSize)
             row = nBlockYSize - 1;
-         if (col < 0) 
+         if (col < 0)
             col = 0;
-         else if (col >= nBlockXSize ) 
+         else if (col >= nBlockXSize )
             col = nBlockXSize - 1;
 
          int iCol = (int) (col);
@@ -425,7 +426,7 @@ CPLErr   MG4LidarRasterBand::doReadBlock(int nBlockXOff, int nBlockYOff, void *
                static_cast<DTYPE *>(pImage)[offset] > value)
                static_cast<DTYPE *>(pImage)[offset] = value;
          }
-         else if (EQUAL(Aggregation, "Mean"))
+         else if (EQUAL(Aggregation, "Mean") && Accumulator != NULL)
          {
             DTYPE value = GetChannelElement<DTYPE>(*channel, i);
             Accumulator[offset].count++;
@@ -434,7 +435,7 @@ CPLErr   MG4LidarRasterBand::doReadBlock(int nBlockXOff, int nBlockYOff, void *
          }
       }
    }
-   
+
    delete[] Accumulator;
    return CE_None;
 }
@@ -497,16 +498,16 @@ CPLErr MG4LidarRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 /************************************************************************/
 
 CPLErr MG4LidarRasterBand::GetStatistics( int bApproxOK, int bForce,
-                                         double *pdfMin, double *pdfMax, 
+                                         double *pdfMin, double *pdfMax,
                                          double *pdfMean, double *pdfStdDev )
 
 {
-   bApproxOK = TRUE; 
+   bApproxOK = TRUE;
    bForce = TRUE;
 
-   return GDALPamRasterBand::GetStatistics( bApproxOK, bForce, 
-      pdfMin, pdfMax, 
-      pdfMean, pdfStdDev );   
+   return GDALPamRasterBand::GetStatistics( bApproxOK, bForce,
+      pdfMin, pdfMax,
+      pdfMean, pdfStdDev );
 
 }
 /************************************************************************/
@@ -520,11 +521,14 @@ double MG4LidarRasterBand::GetNoDataValue( int *pbSuccess )
    return nodatavalue;
 }
 
-
 /************************************************************************/
 /*                            MG4LidarDataset()                             */
 /************************************************************************/
-MG4LidarDataset::MG4LidarDataset()
+
+MG4LidarDataset::MG4LidarDataset() :
+    nBlockXSize(0),
+    nBlockYSize(0),
+    iLevel(0)
 {
    reader = NULL;
    fileIO = NULL;
@@ -533,8 +537,8 @@ MG4LidarDataset::MG4LidarDataset()
    ownsXML = false;
    nOverviewCount = 0;
    papoOverviewDS = NULL;
-
 }
+
 /************************************************************************/
 /*                            ~MG4LidarDataset()                             */
 /************************************************************************/
@@ -564,7 +568,7 @@ CPLErr MG4LidarDataset::GetGeoTransform( double * padfTransform )
 
 {
    padfTransform[0] = reader->getBounds().x.min ;// Upper left X, Y
-   padfTransform[3] = reader->getBounds().y.max; // 
+   padfTransform[3] = reader->getBounds().y.max; //
    padfTransform[1] = reader->getBounds().x.length()/GetRasterXSize(); //xRes
    padfTransform[2] = 0.0;
 
@@ -583,7 +587,7 @@ const char *MG4LidarDataset::GetProjectionRef()
 {
    const char * wkt = CPLGetXMLValue(poXMLPCView, "GeoReference", NULL);
    if (wkt == NULL)
-      wkt = reader->getWKT(); 
+      wkt = reader->getWKT();
    return(wkt);
 }
 
@@ -612,7 +616,7 @@ CPLErr MG4LidarDataset::OpenZoomLevel( int iZoom )
    nRasterYSize  = static_cast<int>(gHeight / yRes + 0.5);
 
    nBlockXSize = static_cast<int>(MIN(MaxBlockSideSize , GetRasterXSize()));
-   nBlockYSize = static_cast<int>(MIN(MaxBlockSideSize , GetRasterYSize())); 
+   nBlockYSize = static_cast<int>(MIN(MaxBlockSideSize , GetRasterYSize()));
 
    CPLDebug( "MG4Lidar", "Opened zoom level %d with size %dx%d.\n",
       iZoom, nRasterXSize, nRasterYSize );
@@ -626,7 +630,7 @@ CPLErr MG4LidarDataset::OpenZoomLevel( int iZoom )
    /* -------------------------------------------------------------------- */
    /*      Create band information objects.                                */
    /* -------------------------------------------------------------------- */
-   size_t BandCount = 0;
+   int BandCount = 0;
    CPLXMLNode* xmlBand = poXMLPCView;
    bool bClass = false;
    bool bNumRets = false;
@@ -637,7 +641,7 @@ CPLErr MG4LidarDataset::OpenZoomLevel( int iZoom )
       const char * name = "Z";
       if (xmlChannel && xmlChannel->psChild && xmlChannel->psChild->pszValue)
          name = xmlChannel->psChild->pszValue;
-      
+
       BandCount++;
       MG4LidarRasterBand *band = new MG4LidarRasterBand(this, BandCount, xmlBand, name);
       SetBand(BandCount, band);
@@ -659,7 +663,7 @@ CPLErr MG4LidarDataset::OpenZoomLevel( int iZoom )
    const ChannelInfo *ci = NULL;
    for (int i=0; i<nBands; i++)
    {
-      ci = reader->getChannel(dynamic_cast<MG4LidarRasterBand*>(papoBands[i])->ChannelName);
+      ci = reader->getChannel(static_cast<MG4LidarRasterBand*>(papoBands[i])->ChannelName);
       requiredChannels.getChannel(i).init(*ci);
    }
    int iSDKChannels = nBands;
@@ -695,13 +699,13 @@ GDALDataset *MG4LidarDataset::Open( GDALOpenInfo * poOpenInfo )
    CPLSetConfigOption( "CPL_LOG", "C:\\ArcGIS_GDAL\\jdem\\cpl.log" );
 #endif
 
-   if( poOpenInfo->fp == NULL || poOpenInfo->nHeaderBytes < 32 )
+   if( poOpenInfo->fpL == NULL || poOpenInfo->nHeaderBytes < 32 )
       return NULL;
 
    CPLXMLNode *pxmlPCView;
 
    // do something sensible for .sid files without a .view
-   if( EQUALN((const char *) poOpenInfo->pabyHeader, "msid", 4) )
+   if( STARTS_WITH_CI((const char *) poOpenInfo->pabyHeader, "msid") )
    {
       int gen;
       bool raster;
@@ -719,7 +723,7 @@ GDALDataset *MG4LidarDataset::Open( GDALOpenInfo * poOpenInfo )
    else
    {
       // support .view xml
-      if( !EQUALN((const char *) poOpenInfo->pabyHeader, "<PointCloudView", 15 ) )
+      if( !STARTS_WITH_CI((const char *) poOpenInfo->pabyHeader, "<PointCloudView") )
          return NULL;
 
       pxmlPCView = CPLParseXMLFile( poOpenInfo->pszFilename );
@@ -730,7 +734,7 @@ GDALDataset *MG4LidarDataset::Open( GDALOpenInfo * poOpenInfo )
    CPLXMLNode *psInputFile = CPLGetXMLNode( pxmlPCView, "InputFile" );
    if( psInputFile == NULL )
    {
-      CPLError( CE_Failure, CPLE_OpenFailed, 
+      CPLError( CE_Failure, CPLE_OpenFailed,
          "Failed to find <InputFile> in document." );
       CPLDestroyXMLNode(pxmlPCView);
       return NULL;
@@ -747,7 +751,7 @@ GDALDataset *MG4LidarDataset::Open( GDALOpenInfo * poOpenInfo )
    /*      Check that particular fields in the header are valid looking    */
    /*      dates.                                                          */
    /* -------------------------------------------------------------------- */
-   if( openinfo.fp == NULL || openinfo.nHeaderBytes < 50 )
+   if( openinfo.fpL == NULL || openinfo.nHeaderBytes < 50 )
    {
       CPLDestroyXMLNode(pxmlPCView);
       return NULL;
@@ -755,7 +759,7 @@ GDALDataset *MG4LidarDataset::Open( GDALOpenInfo * poOpenInfo )
 
    /* check magic */
    // to do:  SDK should provide an API for this.
-   if(  !EQUALN((const char *) openinfo.pabyHeader, "msid", 4)
+   if(  !STARTS_WITH_CI((const char *) openinfo.pabyHeader, "msid")
       || (*(openinfo.pabyHeader+4) != 0x4 )) // Generation 4.  ... is there more we can check?
    {
       CPLDestroyXMLNode(pxmlPCView);
@@ -778,7 +782,8 @@ GDALDataset *MG4LidarDataset::Open( GDALOpenInfo * poOpenInfo )
    FileIO* io = FileIO::create();
 
 #if (defined(WIN32) && _MSC_VER >= 1310) || __MSVCRT_VERSION__ >= 0x0601
-   bool bIsUTF8 = CSLTestBoolean( CPLGetConfigOption( "GDAL_FILENAME_IS_UTF8", "YES" ) );
+   bool bIsUTF8 =
+       CPLTestBool( CPLGetConfigOption( "GDAL_FILENAME_IS_UTF8", "YES" ) );
    wchar_t *pwszFilename = NULL;
    if (bIsUTF8)
    {
@@ -804,32 +809,32 @@ GDALDataset *MG4LidarDataset::Open( GDALOpenInfo * poOpenInfo )
       int cslcount = CSLCount(papszClipExtent);
       if (cslcount != 4 && cslcount != 6)
       {
-         CPLError( CE_Failure, CPLE_OpenFailed, 
+         CPLError( CE_Failure, CPLE_OpenFailed,
             "Invalid ClipBox.  Must contain 4 or 6 floats." );
          CSLDestroy(papszClipExtent);
          delete poDS;
          RELEASE(r);
          RELEASE(io);
 #if (defined(WIN32) && _MSC_VER >= 1310) || __MSVCRT_VERSION__ >= 0x0601
-         if ( pwszFilename ) 
+         if ( pwszFilename )
             CPLFree( pwszFilename );
 #endif
          return NULL;
       }
       if (!EQUAL(papszClipExtent[0], "NOFILTER"))
-         bounds.x.min = atof(papszClipExtent[0]);
+         bounds.x.min = CPLAtof(papszClipExtent[0]);
       if (!EQUAL(papszClipExtent[1], "NOFILTER"))
-         bounds.x.max = atof(papszClipExtent[1]);
+         bounds.x.max = CPLAtof(papszClipExtent[1]);
       if (!EQUAL(papszClipExtent[2], "NOFILTER"))
-         bounds.y.min = atof(papszClipExtent[2]);
+         bounds.y.min = CPLAtof(papszClipExtent[2]);
       if (!EQUAL(papszClipExtent[3], "NOFILTER"))
-         bounds.y.max = atof(papszClipExtent[3]);
+         bounds.y.max = CPLAtof(papszClipExtent[3]);
       if (cslcount == 6)
       {
          if (!EQUAL(papszClipExtent[4], "NOFILTER"))
-            bounds.z.min = atof(papszClipExtent[4]);
+            bounds.z.min = CPLAtof(papszClipExtent[4]);
          if (!EQUAL(papszClipExtent[5], "NOFILTER"))
-            bounds.z.max = atof(papszClipExtent[5]);
+            bounds.z.max = CPLAtof(papszClipExtent[5]);
       }
       CSLDestroy(papszClipExtent);
    }
@@ -847,21 +852,21 @@ GDALDataset *MG4LidarDataset::Open( GDALOpenInfo * poOpenInfo )
 #endif
    dynamic_cast<CropableMG4PointReader *>(poDS->reader)->init(poDS->fileIO, &bounds);
    poDS->SetDescription(poOpenInfo->pszFilename);
-   poDS->TryLoadXML(); 
+   poDS->TryLoadXML();
 
    double pts_per_area = ((double)r->getNumPoints())/(r->getBounds().x.length()*r->getBounds().y.length());
    double average_pt_spacing = sqrt(1.0 / pts_per_area) ;
    double cell_side = average_pt_spacing;
    const char * pszCellSize = CPLGetXMLValue(pxmlPCView, "CellSize", NULL);
    if (pszCellSize)
-      cell_side = atof(pszCellSize);
+      cell_side = CPLAtof(pszCellSize);
    MaxRasterSize = MAX(poDS->reader->getBounds().x.length()/cell_side, poDS->reader->getBounds().y.length()/cell_side);
 
    RELEASE(r);
    RELEASE(io);
 
-   // Calculate the number of levels to expose.  The highest level correpsonds to a
-   // raster size of 256 on the longest side.
+   // Calculate the number of levels to expose.  The highest level corresponds
+   // to a raster size of 256 on the longest side.
    double blocksizefactor = MaxRasterSize/256.0;
    poDS->nOverviewCount = MAX(0, (int)(log(blocksizefactor)/log(RESOLUTION_RATIO) + 0.5));
    if ( poDS->nOverviewCount > 0 )
@@ -878,7 +883,7 @@ GDALDataset *MG4LidarDataset::Open( GDALOpenInfo * poOpenInfo )
          poDS->papoOverviewDS[i]->SetMetadata(poDS->GetMetadata("MG4Lidar"), "MG4Lidar");
          poDS->papoOverviewDS[i]->poXMLPCView = pxmlPCView;
          poDS->papoOverviewDS[i]->OpenZoomLevel( i+1 );
-      }       
+      }
    }
 
    /* -------------------------------------------------------------------- */
@@ -908,34 +913,30 @@ GDALDataset *MG4LidarDataset::Open( GDALOpenInfo * poOpenInfo )
 }
 
 /************************************************************************/
-/*                          GDALRegister_MG4Lidar()                        */
+/*                          GDALRegister_MG4Lidar()                     */
 /************************************************************************/
 
 void GDALRegister_MG4Lidar()
 
 {
-   GDALDriver	*poDriver;
-
-    if (! GDAL_CHECK_VERSION("MG4Lidar driver"))
+    if( !GDAL_CHECK_VERSION( "MG4Lidar driver" ) )
         return;
 
-   if( GDALGetDriverByName( "MG4Lidar" ) == NULL )
-   {
-      poDriver = new GDALDriver();
+    if( GDALGetDriverByName( "MG4Lidar" ) != NULL )
+        return;
 
-      poDriver->SetDescription( "MG4Lidar" );
-      poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-         "MrSID Generation 4 / Lidar (.sid)" );
-      // To do:  update this help file in gdal.org
-      poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-         "frmt_mrsid_lidar.html" );
+    GDALDriver *poDriver = new GDALDriver();
 
-      poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "view" );
-      poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, 
-         "Float64" );
+    poDriver->SetDescription( "MG4Lidar" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
+                               "MrSID Generation 4 / Lidar (.sid)" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,  "frmt_mrsid_lidar.html" );
 
-      poDriver->pfnOpen = MG4LidarDataset::Open;
+    poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "view" );
+    poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, "Float64" );
 
-      GetGDALDriverManager()->RegisterDriver( poDriver );
-   }
+    poDriver->pfnOpen = MG4LidarDataset::Open;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
