@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: msgdataset.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: msgdataset.cpp 33951 2016-04-12 14:51:31Z rouault $
  *
  * Project:  MSG Driver
  * Purpose:  GDALDataset driver for MSG translator for read support.
@@ -28,6 +28,7 @@
  * DEALINGS IN THE SOFTWARE.
  ******************************************************************************/
 
+#include "gdal_frmts.h"
 #include "msgdataset.h"
 #include "prologue.h"
 #include "xritheaderparser.h"
@@ -137,7 +138,7 @@ GDALDataset *MSGDataset::Open( GDALOpenInfo * poOpenInfo )
     if (sErr.length() > 0)
     {
         if (sErr.compare("-") != 0) // this driver does not recognize this format .. be silent and return false so that another driver can try
-          CPLError( CE_Failure, CPLE_AppDefined, (sErr+"\n").c_str() );
+          CPLError( CE_Failure, CPLE_AppDefined, "%s", (sErr+"\n").c_str() );
         return FALSE;
     }
 
@@ -181,18 +182,18 @@ GDALDataset *MSGDataset::Open( GDALOpenInfo * poOpenInfo )
     else
     {
         std::string sErr = "The prologue of the data set could not be found at the location specified:\n" + sPrologueFileName + "\n";
-        CPLError( CE_Failure, CPLE_AppDefined,
+        CPLError( CE_Failure, CPLE_AppDefined, "%s",
                   sErr.c_str() );
         return FALSE;
     }
-      
+
 
 // We're confident the string is formatted as an MSG command_line
 
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
-    
+
     MSGDataset   *poDS;
     poDS = new MSGDataset();
     poDS->command = command; // copy it
@@ -248,16 +249,17 @@ GDALDataset *MSGDataset::Open( GDALOpenInfo * poOpenInfo )
 
     poDS->oSRS.SetGEOS(  0, 35785831, 0, 0 );
     poDS->oSRS.SetWellKnownGeogCS( "WGS84" ); // Temporary line to satisfy ERDAS (otherwise the ellips is "unnamed"). Eventually this should become the custom a and b ellips (CGMS).
+    CPLFree( poDS->pszProjection );
     poDS->oSRS.exportToWkt( &(poDS->pszProjection) );
 
     // The following are 3 different try-outs for also setting the ellips a and b parameters.
     // We leave them out for now however because this does not work. In gdalwarp, when choosing some
     // specific target SRS, the result is an error message:
-    // 
+    //
     // ERROR 1: geocentric transformation missing z or ellps
     // ERROR 1: GDALWarperOperation::ComputeSourceWindow() failed because
     // the pfnTransformer failed.
-    // 
+    //
     // I can't explain the reason for the message at this time (could be a problem in the way the SRS is set here,
     // but also a bug in Proj.4 or GDAL.
     /*
@@ -276,8 +278,13 @@ GDALDataset *MSGDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 
     char *pszLLTemp;
+    char *pszLLTemp_bak;
+
     (poDS->oSRS.GetAttrNode("GEOGCS"))->exportToWkt(&pszLLTemp);
+    pszLLTemp_bak = pszLLTemp; // importFromWkt() changes the pointer
     poDS->oLL.importFromWkt(&pszLLTemp);
+    CPLFree( pszLLTemp_bak );
+
     poDS->poTransform = OGRCreateCoordinateTransformation( &(poDS->oSRS), &(poDS->oLL) );
 
 /* -------------------------------------------------------------------- */
@@ -295,19 +302,19 @@ GDALDataset *MSGDataset::Open( GDALOpenInfo * poOpenInfo )
     {
         poDS->SetBand( iBand+1, new MSGRasterBand( poDS, iBand+1 ) );
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Confirm the requested access is supported.                      */
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->eAccess == GA_Update )
     {
         delete poDS;
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The MSG driver does not support update access to existing"
                   " datasets.\n" );
         return NULL;
     }
-    
+
     return( poDS );
 }
 
@@ -326,7 +333,7 @@ MSGRasterBand::MSGRasterBand( MSGDataset *poDS, int nBand )
 {
     this->poDS = poDS;
     this->nBand = nBand;
-		
+
     // Find if we're dealing with MSG1, MSG2, MSG3 or MSG4
     // Doing this per band is the only way to guarantee time-series when the satellite is changed
 
@@ -469,9 +476,8 @@ CPLErr MSGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                                   void * pImage )
 
 {
-        
-    MSGDataset  *poGDS = (MSGDataset *) poDS;
 
+    MSGDataset  *poGDS = (MSGDataset *) poDS;
 
     int iBytesPerPixel = 1;
     if (eDataType == GDT_UInt16)
@@ -491,7 +497,7 @@ CPLErr MSGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
       strip_number = poGDS->command.iNrStrips(iChannel) - nBlockYOff;
 
     std::string strip_input_file = poGDS->command.sFileName(iSatellite, nBand, strip_number);
-    
+
 /* -------------------------------------------------------------------- */
 /*      Open the input file                                             */
 /* -------------------------------------------------------------------- */
@@ -506,10 +512,10 @@ CPLErr MSGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
         if (xhp.isValid())
         {
           std::vector <short> QualityInfo;
-          unsigned short chunck_height = xhp.nrRows();
-          unsigned short chunck_bpp = xhp.nrBitsPerPixel();
-          unsigned short chunck_width = xhp.nrColumns();
-          unsigned __int8 NR = (unsigned __int8)chunck_bpp;
+          unsigned short chunk_height = xhp.nrRows();
+          unsigned short chunk_bpp = xhp.nrBitsPerPixel();
+          unsigned short chunk_width = xhp.nrColumns();
+          unsigned __int8 NR = (unsigned __int8)chunk_bpp;
           unsigned int nb_ibytes = xhp.dataSize();
           int iShift = 0;
           bool fSplitStrip = false; // in the split strip the "shift" only happens before the split "row"
@@ -523,30 +529,30 @@ CPLErr MSGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
             // When iLowerShift > 0, the lower HRV image is shifted to the right
             // When iLowerShift < 0, the lower HRV image is shifted to the left
             // The available raster may be wider than needed, so that time series don't fall outside the raster.
-            
+
             if (nBlockYOff <= iSplitBlock)
               iShift = -iLowerShift;
             // iShift < 0 means upper image moves to the left
             // iShift > 0 means upper image moves to the right
           }
-          
+
           std::auto_ptr< unsigned char > ibuf( new unsigned char[nb_ibytes]);
-          
+
           if (ibuf.get() == 0)
           {
-             CPLError( CE_Failure, CPLE_AppDefined, 
+             CPLError( CE_Failure, CPLE_AppDefined,
                   "Not enough memory to perform wavelet decompression\n");
             return CE_Failure;
           }
 
           i_file.read( (char *)(ibuf.get()), nb_ibytes);
-          
+
           Util::CDataFieldCompressedImage  img_compressed(ibuf.release(),
                                   nb_ibytes*8,
-                                  (unsigned char)chunck_bpp,
-                                  chunck_width,
-                                  chunck_height      );
-          
+                                  (unsigned char)chunk_bpp,
+                                  chunk_width,
+                                  chunk_height      );
+
           Util::CDataFieldUncompressedImage img_uncompressed;
 
           //****************************************************
@@ -559,20 +565,20 @@ CPLErr MSGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
           // 8 bits -> 8 bits
           // 10 bits -> 16 bits (img_uncompressed contains the 10 bits data in packed form)
           // Geometry:
-          // chunck_width x chunck_height to nBlockXSize x nBlockYSize
+          // chunk_width x chunk_height to nBlockXSize x nBlockYSize
 
           // cases:
           // combination of the following:
           // - scan direction can be north or south
           // - eDataType can be GDT_Byte, GDT_UInt16 or GDT_Float32
-          // - nBlockXSize == chunck_width or nBlockXSize > chunck_width
-          // - when nBlockXSize > chunck_width, fSplitStrip can be true or false
+          // - nBlockXSize == chunk_width or nBlockXSize > chunk_width
+          // - when nBlockXSize > chunk_width, fSplitStrip can be true or false
           // we won't distinguish the following cases:
           // - NR can be == 8 or != 8
-          // - when nBlockXSize > chunck_width, iShift iMinCOff-iMaxCOff <= iShift <= 0
+          // - when nBlockXSize > chunk_width, iShift iMinCOff-iMaxCOff <= iShift <= 0
 
           int nBlockSize = nBlockXSize * nBlockYSize;
-          int y = chunck_width * chunck_height;
+          int y = chunk_width * chunk_height;
           int iStep = -1;
           if (fScanNorth) // image is the other way around
           {
@@ -583,7 +589,7 @@ CPLErr MSGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
           COMP::CImage cimg (img_uncompressed); // unpack
           if (eDataType == GDT_Byte)
           {
-            if (nBlockXSize == chunck_width) // optimized version
+            if (nBlockXSize == chunk_width) // optimized version
             {
               if (poGDS->command.cDataConversion == 'B')
               {
@@ -602,25 +608,25 @@ CPLErr MSGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
               memset(pImage, 0, nBlockXSize * nBlockYSize * iBytesPerPixel);
               if (poGDS->command.cDataConversion == 'B')
               {
-                for( int j = 0; j < chunck_height; ++j ) // assumption: nBlockYSize == chunck_height
-                { 
+                for( int j = 0; j < chunk_height; ++j ) // assumption: nBlockYSize == chunk_height
+                {
                   int iXOffset = j * nBlockXSize + iShift;
                   iXOffset += nBlockXSize - iLowerWestColumnPlanned - 1; // Position the HRV part in the frame; -1 to compensate the pre-increment in the for-loop
                   if (fSplitStrip && (j >= iSplitRow)) // In splitstrip, below splitline, thus do not shift!!
                     iXOffset -= iShift;
-                  for (int i = 0; i < chunck_width; ++i)
+                  for (int i = 0; i < chunk_width; ++i)
                     ((GByte *)pImage)[++iXOffset] = cimg.Get()[y+=iStep] / 4;
                 }
               }
               else
               {
-                for( int j = 0; j < chunck_height; ++j ) // assumption: nBlockYSize == chunck_height
-                { 
+                for( int j = 0; j < chunk_height; ++j ) // assumption: nBlockYSize == chunk_height
+                {
                   int iXOffset = j * nBlockXSize + iShift;
                   iXOffset += nBlockXSize - iLowerWestColumnPlanned - 1; // Position the HRV part in the frame; -1 to compensate the pre-increment in the for-loop
                   if (fSplitStrip && (j >= iSplitRow)) // In splitstrip, below splitline, thus do not shift!!
                     iXOffset -= iShift;
-                  for (int i = 0; i < chunck_width; ++i)
+                  for (int i = 0; i < chunk_width; ++i)
                     ((GByte *)pImage)[++iXOffset] = cimg.Get()[y+=iStep];
                 }
               }
@@ -628,7 +634,7 @@ CPLErr MSGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
           }
           else if (eDataType == GDT_UInt16) // this is our "normal case" if scan direction is South: 10 bit MSG data became 2 bytes per pixel
           {
-            if (nBlockXSize == chunck_width) // optimized version
+            if (nBlockXSize == chunk_width) // optimized version
             {
               for( int i = 0; i < nBlockSize; ++i )
                   ((GUInt16 *)pImage)[i] = cimg.Get()[y+=iStep];
@@ -637,20 +643,20 @@ CPLErr MSGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
             {
               // initialize to 0's (so that it does not have to be done in an 'else' statement <performance>)
               memset(pImage, 0, nBlockXSize * nBlockYSize * iBytesPerPixel);
-              for( int j = 0; j < chunck_height; ++j ) // assumption: nBlockYSize == chunck_height
+              for( int j = 0; j < chunk_height; ++j ) // assumption: nBlockYSize == chunk_height
               {
                 int iXOffset = j * nBlockXSize + iShift;
                 iXOffset += nBlockXSize - iLowerWestColumnPlanned - 1; // Position the HRV part in the frame; -1 to compensate the pre-increment in the for-loop
                 if (fSplitStrip && (j >= iSplitRow)) // In splitstrip, below splitline, thus do not shift!!
                   iXOffset -= iShift;
-                for (int i = 0; i < chunck_width; ++i)
+                for (int i = 0; i < chunk_width; ++i)
                   ((GUInt16 *)pImage)[++iXOffset] = cimg.Get()[y+=iStep];
               }
             }
           }
           else if (eDataType == GDT_Float32) // radiometric calibration is requested
           {
-            if (nBlockXSize == chunck_width) // optimized version
+            if (nBlockXSize == chunk_width) // optimized version
             {
               for( int i = 0; i < nBlockSize; ++i )
                 ((float *)pImage)[i] = (float)rRadiometricCorrection(cimg.Get()[y+=iStep], iChannel, nBlockYOff * nBlockYSize + i / nBlockXSize, i % nBlockXSize, poGDS);
@@ -659,15 +665,15 @@ CPLErr MSGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
             {
               // initialize to 0's (so that it does not have to be done in an 'else' statement <performance>)
               memset(pImage, 0, nBlockXSize * nBlockYSize * iBytesPerPixel);
-              for( int j = 0; j < chunck_height; ++j ) // assumption: nBlockYSize == chunck_height
+              for( int j = 0; j < chunk_height; ++j ) // assumption: nBlockYSize == chunk_height
               {
                 int iXOffset = j * nBlockXSize + iShift;
                 iXOffset += nBlockXSize - iLowerWestColumnPlanned - 1; // Position the HRV part in the frame; -1 to compensate the pre-increment in the for-loop
                 if (fSplitStrip && (j >= iSplitRow)) // In splitstrip, below splitline, thus do not shift!!
                   iXOffset -= iShift;
                 int iXFrom = nBlockXSize - iLowerWestColumnPlanned + iShift; // i is used as the iCol parameter in rRadiometricCorrection
-                int iXTo = nBlockXSize - iLowerWestColumnPlanned + chunck_width + iShift;
-                for (int i = iXFrom; i < iXTo; ++i) // range always equal to chunck_width .. this is to utilize i to get iCol
+                int iXTo = nBlockXSize - iLowerWestColumnPlanned + chunk_width + iShift;
+                for (int i = iXFrom; i < iXTo; ++i) // range always equal to chunk_width .. this is to utilize i to get iCol
                   ((float *)pImage)[++iXOffset] = (float)rRadiometricCorrection(cimg.Get()[y+=iStep], iChannel, nBlockYOff * nBlockYSize + j, (fSplitStrip && (j >= iSplitRow))?(i - iShift):i, poGDS);
               }
             }
@@ -689,12 +695,13 @@ CPLErr MSGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 
 double MSGRasterBand::rRadiometricCorrection(unsigned int iDN, int iChannel, int iRow, int iCol, MSGDataset* poGDS)
 {
-	int iIndex = iChannel - 1; // just for speed optimization
+  int iIndex = iChannel - 1; // just for speed optimization
 
   double rSlope = poGDS->rCalibrationSlope[iIndex];
   double rOffset = poGDS->rCalibrationOffset[iIndex];
-  
-  if (poGDS->command.cDataConversion == 'T') // reflectance for visual bands, temperatore for IR bands
+
+  // Reflectance for visual bands, temperature for IR bands.
+  if (poGDS->command.cDataConversion == 'T')
   {
     double rRadiance = rOffset + (iDN * rSlope);
 
@@ -737,19 +744,16 @@ double MSGRasterBand::rRadiometricCorrection(unsigned int iDN, int iChannel, int
 void GDALRegister_MSG()
 
 {
-    GDALDriver  *poDriver;
+    if( GDALGetDriverByName( "MSG" ) != NULL )
+        return;
 
-    if( GDALGetDriverByName( "MSG" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "MSG" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "MSG HRIT Data" );
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->pfnOpen = MSGDataset::Open;
+    poDriver->SetDescription( "MSG" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,  "MSG HRIT Data" );
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    poDriver->pfnOpen = MSGDataset::Open;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
-
