@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: msgdataset.h 32190 2015-12-16 13:50:27Z goatbar $
  *
  * Project:  MSG Driver
  * Purpose:  GDALDataset driver for MSG translator for read support.
@@ -35,14 +36,16 @@
 #include <fstream>
 
 CPL_C_START
-void  GDALRegister_MSG(void);
+void GDALRegister_MSG();
 CPL_C_END
 
 /************************************************************************/
 /*                            MSGRasterBand                             */
 /************************************************************************/
 
+class MSGDataset;
 class ReflectanceCalculator;
+
 class MSGRasterBand : public GDALRasterBand
 {
   friend class MSGDataset;
@@ -69,7 +72,7 @@ class MSGRasterBand : public GDALRasterBand
 class MSGDataset : public GDALDataset
 {
   friend class MSGRasterBand;
-  
+
   public:
     MSGDataset();
     ~MSGDataset();
