@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: netcdfdataset.h 26612 2013-11-14 17:35:56Z goatbar $
+ * $Id: netcdfdataset.h 33794 2016-03-26 13:19:07Z goatbar $
  *
  * Project:  netCDF read/write Driver
  * Purpose:  GDAL bindings over netCDF library.
@@ -27,17 +27,18 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _NETCDFDATASET_H_INCLUDED_
-#define _NETCDFDATASET_H_INCLUDED_
+#ifndef NETCDFDATASET_H_INCLUDED_
+#define NETCDFDATASET_H_INCLUDED_
 
-#include <float.h>
+#include <cfloat>
+
+#include "cpl_string.h"
+#include "gdal_frmts.h"
 #include "gdal_pam.h"
 #include "gdal_priv.h"
-#include "gdal_frmts.h"
-#include "cpl_string.h"
-#include "ogr_spatialref.h"
 #include "netcdf.h"
-
+#include "ogr_spatialref.h"
+#include "ogrsf_frmts.h"
 
 /************************************************************************/
 /* ==================================================================== */
@@ -72,8 +73,9 @@
 /* -------------------------------------------------------------------- */
 
 /* NETCDF driver defs */
-#define NCDF_MAX_STR_LEN     8192
-#define NCDF_CONVENTIONS_CF  "CF-1.5"
+static const size_t NCDF_MAX_STR_LEN = 8192;
+#define NCDF_CONVENTIONS_CF_V1_5  "CF-1.5"
+#define NCDF_CONVENTIONS_CF_V1_6  "CF-1.6"
 #define NCDF_SPATIAL_REF     "spatial_ref"
 #define NCDF_GEOTRANSFORM    "GeoTransform"
 #define NCDF_DIMNAME_X       "x"
@@ -81,28 +83,34 @@
 #define NCDF_DIMNAME_LON     "lon"
 #define NCDF_DIMNAME_LAT     "lat"
 #define NCDF_LONLAT          "lon lat"
-#define NCDF_PI              3.14159265358979323846
 
 /* netcdf file types, as in libcdi/cdo and compat w/netcdf.h */
-#define NCDF_FORMAT_NONE            0   /* Not a netCDF file */
-#define NCDF_FORMAT_NC              1   /* netCDF classic format */
-#define NCDF_FORMAT_NC2             2   /* netCDF version 2 (64-bit)  */
-#define NCDF_FORMAT_NC4             3   /* netCDF version 4 */
-#define NCDF_FORMAT_NC4C            4   /* netCDF version 4 (classic) */
-#define NCDF_FORMAT_UNKNOWN         10  /* Format not determined (yet) */
+typedef enum
+{
+    NCDF_FORMAT_NONE    = 0,   /* Not a netCDF file */
+    NCDF_FORMAT_NC      = 1,   /* netCDF classic format */
+    NCDF_FORMAT_NC2     = 2,   /* netCDF version 2 (64-bit)  */
+    NCDF_FORMAT_NC4     = 3,   /* netCDF version 4 */
+    NCDF_FORMAT_NC4C    = 4,   /* netCDF version 4 (classic) */
 /* HDF files (HDF5 or HDF4) not supported because of lack of support */
 /* in libnetcdf installation or conflict with other drivers */
-#define NCDF_FORMAT_HDF5            5   /* HDF4 file, not supported */
-#define NCDF_FORMAT_HDF4            6   /* HDF4 file, not supported */
+    NCDF_FORMAT_HDF5    = 5,   /* HDF4 file, not supported */
+    NCDF_FORMAT_HDF4    = 6,   /* HDF4 file, not supported */
+    NCDF_FORMAT_UNKNOWN = 10  /* Format not determined (yet) */
+} NetCDFFormatEnum;
 
 /* compression parameters */
-#define NCDF_COMPRESS_NONE            0   
+typedef enum
+{
+    NCDF_COMPRESS_NONE    = 0,
 /* TODO */
 /* http://www.unidata.ucar.edu/software/netcdf/docs/BestPractices.html#Packed%20Data%20Values */
-#define NCDF_COMPRESS_PACKED          1  
-#define NCDF_COMPRESS_DEFLATE         2   
-#define NCDF_DEFLATE_LEVEL            1  /* best time/size ratio */  
-#define NCDF_COMPRESS_SZIP            3  /* no support for writting */ 
+    NCDF_COMPRESS_PACKED  = 1,
+    NCDF_COMPRESS_DEFLATE = 2,
+    NCDF_COMPRESS_SZIP    = 3  /* no support for writing */
+} NetCDFCompressEnum;
+
+static const int NCDF_DEFLATE_LEVEL    = 1;  /* best time/size ratio */
 
 /* helper for libnetcdf errors */
 #define NCDF_ERR(status) if ( status != NC_NOERR ){ \
@@ -110,8 +118,8 @@ CPLError( CE_Failure,CPLE_AppDefined, \
 "netcdf error #%d : %s .\nat (%s,%s,%d)\n",status, nc_strerror(status), \
 __FILE__, __FUNCTION__, __LINE__ ); }
 
-/* check for NC2 support in case it wasn't enabled at compile time */
-/* NC4 has to be detected at compile as it requires a special build of netcdf-4 */
+/* Check for NC2 support in case it was not enabled at compile time. */
+/* NC4 has to be detected at compile as it requires a special build of netcdf-4. */
 #ifndef NETCDF_HAS_NC2
 #ifdef NC_64BIT_OFFSET
 #define NETCDF_HAS_NC2 1
@@ -133,6 +141,8 @@ __FILE__, __FUNCTION__, __LINE__ ); }
 /* should be SRS_UL_METER but use meter now for compat with gtiff files */
 #define CF_UNITS_M           "metre"
 #define CF_UNITS_D           SRS_UA_DEGREE
+#define CF_PROJ_X_VAR_NAME   "x"
+#define CF_PROJ_Y_VAR_NAME   "y"
 #define CF_PROJ_X_COORD      "projection_x_coordinate"
 #define CF_PROJ_Y_COORD      "projection_y_coordinate"
 #define CF_PROJ_X_COORD_LONG_NAME "x coordinate of projection"
@@ -140,7 +150,17 @@ __FILE__, __FUNCTION__, __LINE__ ); }
 #define CF_GRD_MAPPING_NAME  "grid_mapping_name"
 #define CF_GRD_MAPPING       "grid_mapping"
 #define CF_COORDINATES       "coordinates"
-/* #define CF_AXIS            "axis" */
+
+#define CF_LONGITUDE_VAR_NAME      "lon"
+#define CF_LONGITUDE_STD_NAME      "longitude"
+#define CF_LONGITUDE_LNG_NAME      "longitude"
+#define CF_LATITUDE_VAR_NAME       "lat"
+#define CF_LATITUDE_STD_NAME       "latitude"
+#define CF_LATITUDE_LNG_NAME       "latitude"
+#define CF_DEGREES_NORTH           "degrees_north"
+#define CF_DEGREES_EAST            "degrees_east"
+
+#define CF_AXIS            "axis"
 /* #define CF_BOUNDS          "bounds" */
 /* #define CF_ORIG_UNITS      "original_units" */
 
@@ -163,6 +183,7 @@ __FILE__, __FUNCTION__, __LINE__ ); }
 #define CF_PT_ORTHOGRAPHIC           "orthographic"
 #define CF_PT_POLAR_STEREO           "polar_stereographic"
 #define CF_PT_STEREO                 "stereographic"
+#define CF_PT_GEOS                   "geostationary"
 
 /* projection parameters */
 #define CF_PP_STD_PARALLEL           "standard_parallel"
@@ -191,58 +212,60 @@ __FILE__, __FUNCTION__, __LINE__ ); }
 #define CF_PP_SEMI_MAJOR_AXIS        "semi_major_axis"
 #define CF_PP_SEMI_MINOR_AXIS        "semi_minor_axis"
 #define CF_PP_VERT_PERSP             "vertical_perspective" /*not used yet */
+#define CF_PP_PERSPECTIVE_POINT_HEIGHT "perspective_point_height"
+#define CF_PP_SWEEP_ANGLE_AXIS        "sweep_angle_axis"
 
 
 /* -------------------------------------------------------------------- */
 /*         CF-1 Coordinate Type Naming (Chapter 4.  Coordinate Types )  */
 /* -------------------------------------------------------------------- */
-static const char* papszCFLongitudeVarNames[] = { "lon", "longitude", NULL };
-static const char* papszCFLongitudeAttribNames[] = { "units", CF_STD_NAME, "axis", NULL };
-static const char* papszCFLongitudeAttribValues[] = { "degrees_east", "longitude", "X", NULL };
-static const char* papszCFLatitudeVarNames[] = { "lat", "latitude", NULL };
-static const char* papszCFLatitudeAttribNames[] = { "units", CF_STD_NAME, "axis", NULL };
-static const char* papszCFLatitudeAttribValues[] = { "degrees_north", "latitude", "Y", NULL };
- 
-static const char* papszCFProjectionXVarNames[] = { "x", "xc", NULL };
-static const char* papszCFProjectionXAttribNames[] = { CF_STD_NAME, NULL };
-static const char* papszCFProjectionXAttribValues[] = { CF_PROJ_X_COORD, NULL };
-static const char* papszCFProjectionYVarNames[] = { "y", "yc", NULL };
-static const char* papszCFProjectionYAttribNames[] = { CF_STD_NAME, NULL };
-static const char* papszCFProjectionYAttribValues[] = { CF_PROJ_Y_COORD, NULL };
+static const char* const papszCFLongitudeVarNames[] = { CF_LONGITUDE_VAR_NAME, "longitude", NULL };
+static const char* const papszCFLongitudeAttribNames[] = { CF_UNITS, CF_STD_NAME, CF_AXIS, NULL };
+static const char* const papszCFLongitudeAttribValues[] = { CF_DEGREES_EAST, CF_LONGITUDE_STD_NAME, "X", NULL };
+static const char* const papszCFLatitudeVarNames[] = { CF_LATITUDE_VAR_NAME, "latitude", NULL };
+static const char* const papszCFLatitudeAttribNames[] = { CF_UNITS, CF_STD_NAME, CF_AXIS, NULL };
+static const char* const papszCFLatitudeAttribValues[] = { CF_DEGREES_NORTH, CF_LATITUDE_STD_NAME, "Y", NULL };
 
-static const char* papszCFVerticalAttribNames[] = { "axis", "positive", "positive", NULL };
-static const char* papszCFVerticalAttribValues[] = { "Z", "up", "down", NULL };
-static const char* papszCFVerticalUnitsValues[] = { 
+static const char* const papszCFProjectionXVarNames[] = { CF_PROJ_X_VAR_NAME, "xc", NULL };
+static const char* const papszCFProjectionXAttribNames[] = { CF_STD_NAME, NULL };
+static const char* const papszCFProjectionXAttribValues[] = { CF_PROJ_X_COORD, NULL };
+static const char* const papszCFProjectionYVarNames[] = { CF_PROJ_Y_VAR_NAME, "yc", NULL };
+static const char* const papszCFProjectionYAttribNames[] = { CF_STD_NAME, NULL };
+static const char* const papszCFProjectionYAttribValues[] = { CF_PROJ_Y_COORD, NULL };
+
+static const char* const papszCFVerticalAttribNames[] = { CF_AXIS, "positive", "positive", NULL };
+static const char* const papszCFVerticalAttribValues[] = { "Z", "up", "down", NULL };
+static const char* const papszCFVerticalUnitsValues[] = {
     /* units of pressure */
-    "bar", "bars", "millibar", "millibars", "decibar", "decibars", 
+    "bar", "bars", "millibar", "millibars", "decibar", "decibars",
     "atmosphere", "atmospheres", "atm", "pascal", "pascals", "Pa", "hPa",
     /* units of length */
-    "meter", "meters", "m", "kilometer", "kilometers", "km", 
+    "meter", "meters", "m", "kilometer", "kilometers", "km",
     /* dimensionless vertical coordinates */
     "level", "layer", "sigma_level",
     NULL };
 /* dimensionless vertical coordinates */
-static const char* papszCFVerticalStandardNameValues[] = { 
+static const char* const papszCFVerticalStandardNameValues[] = {
     "atmosphere_ln_pressure_coordinate", "atmosphere_sigma_coordinate",
-    "atmosphere_hybrid_sigma_pressure_coordinate", 
+    "atmosphere_hybrid_sigma_pressure_coordinate",
     "atmosphere_hybrid_height_coordinate",
     "atmosphere_sleve_coordinate", "ocean_sigma_coordinate",
     "ocean_s_coordinate", "ocean_sigma_z_coordinate",
     "ocean_double_sigma_coordinate", "atmosphere_ln_pressure_coordinate",
-    "atmosphere_sigma_coordinate", 
+    "atmosphere_sigma_coordinate",
     "atmosphere_hybrid_sigma_pressure_coordinate",
     "atmosphere_hybrid_height_coordinate",
     "atmosphere_sleve_coordinate", "ocean_sigma_coordinate",
     "ocean_s_coordinate", "ocean_sigma_z_coordinate",
     "ocean_double_sigma_coordinate", NULL };
 
-static const char* papszCFTimeAttribNames[] = { "axis", NULL };
-static const char* papszCFTimeAttribValues[] = { "T", NULL };
-static const char* papszCFTimeUnitsValues[] = { 
-    "days since", "day since", "d since", 
-    "hours since", "hour since", "h since", "hr since", 
-    "minutes since", "minute since", "min since", 
-    "seconds since", "second since", "sec since", "s since", 
+static const char* const papszCFTimeAttribNames[] = { CF_AXIS, NULL };
+static const char* const papszCFTimeAttribValues[] = { "T", NULL };
+static const char* const papszCFTimeUnitsValues[] = {
+    "days since", "day since", "d since",
+    "hours since", "hour since", "h since", "hr since",
+    "minutes since", "minute since", "min since",
+    "seconds since", "second since", "sec since", "s since",
     NULL };
 
 
@@ -258,41 +281,41 @@ static const char* papszCFTimeUnitsValues[] = {
 /* A struct allowing us to map between GDAL(OGC WKT) and CF-1 attributes */
 typedef struct {
     const char *CF_ATT;
-    const char *WKT_ATT; 
+    const char *WKT_ATT;
     // TODO: mappings may need default values, like scale factor?
     //double defval;
 } oNetcdfSRS_PP;
 
 // default mappings, for the generic case
-/* These 'generic' mappings are based on what was previously in the  
-   poNetCDFSRS struct. They will be used as a fallback in case none 
-   of the others match (ie you are exporting a projection that has 
-   no CF-1 equivalent). 
-   They are not used for known CF-1 projections since there is not a 
-   unique 2-way projection-independent 
-   mapping between OGC WKT params and CF-1 ones: it varies per-projection. 
-*/ 
+/* These 'generic' mappings are based on what was previously in the
+   poNetCDFSRS struct. They will be used as a fallback in case none
+   of the others match (i.e. you are exporting a projection that has
+   no CF-1 equivalent).
+   They are not used for known CF-1 projections since there is not a
+   unique 2-way projection-independent
+   mapping between OGC WKT params and CF-1 ones: it varies per-projection.
+*/
 
 static const oNetcdfSRS_PP poGenericMappings[] = {
     /* scale_factor is handled as a special case, write 2 values */
-    {CF_PP_STD_PARALLEL_1, SRS_PP_STANDARD_PARALLEL_1 }, 
-    {CF_PP_STD_PARALLEL_2, SRS_PP_STANDARD_PARALLEL_2 }, 
-    {CF_PP_LONG_CENTRAL_MERIDIAN, SRS_PP_CENTRAL_MERIDIAN }, 
-    {CF_PP_LONG_CENTRAL_MERIDIAN, SRS_PP_LONGITUDE_OF_CENTER }, 
-    {CF_PP_LON_PROJ_ORIGIN, SRS_PP_LONGITUDE_OF_ORIGIN },  
-    //Multiple mappings to LAT_PROJ_ORIGIN 
-    {CF_PP_LAT_PROJ_ORIGIN, SRS_PP_LATITUDE_OF_ORIGIN },  
-    {CF_PP_LAT_PROJ_ORIGIN, SRS_PP_LATITUDE_OF_CENTER },  
-    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },   
-    {CF_PP_FALSE_NORTHING, SRS_PP_FALSE_NORTHING },        
+    {CF_PP_STD_PARALLEL_1, SRS_PP_STANDARD_PARALLEL_1 },
+    {CF_PP_STD_PARALLEL_2, SRS_PP_STANDARD_PARALLEL_2 },
+    {CF_PP_LONG_CENTRAL_MERIDIAN, SRS_PP_CENTRAL_MERIDIAN },
+    {CF_PP_LONG_CENTRAL_MERIDIAN, SRS_PP_LONGITUDE_OF_CENTER },
+    {CF_PP_LON_PROJ_ORIGIN, SRS_PP_LONGITUDE_OF_ORIGIN },
+    //Multiple mappings to LAT_PROJ_ORIGIN
+    {CF_PP_LAT_PROJ_ORIGIN, SRS_PP_LATITUDE_OF_ORIGIN },
+    {CF_PP_LAT_PROJ_ORIGIN, SRS_PP_LATITUDE_OF_CENTER },
+    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },
+    {CF_PP_FALSE_NORTHING, SRS_PP_FALSE_NORTHING },
     {NULL, NULL },
 };
 
-// Albers equal area 
+// Albers equal area
 //
 // grid_mapping_name = albers_conical_equal_area
 // WKT: Albers_Conic_Equal_Area
-// ESPG:9822 
+// ESPG:9822
 //
 // Map parameters:
 //
@@ -307,7 +330,7 @@ static const oNetcdfSRS_PP poAEAMappings[] = {
     {CF_PP_STD_PARALLEL_2, SRS_PP_STANDARD_PARALLEL_2},
     {CF_PP_LAT_PROJ_ORIGIN, SRS_PP_LATITUDE_OF_CENTER},
     {CF_PP_LONG_CENTRAL_MERIDIAN, SRS_PP_LONGITUDE_OF_CENTER},
-    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },  
+    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },
     {CF_PP_FALSE_NORTHING, SRS_PP_FALSE_NORTHING },
     {NULL, NULL}
  };
@@ -327,7 +350,7 @@ static const oNetcdfSRS_PP poAEAMappings[] = {
 static const oNetcdfSRS_PP poAEMappings[] = {
     {CF_PP_LAT_PROJ_ORIGIN, SRS_PP_LATITUDE_OF_CENTER},
     {CF_PP_LON_PROJ_ORIGIN, SRS_PP_LONGITUDE_OF_CENTER},
-    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },  
+    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },
     {CF_PP_FALSE_NORTHING, SRS_PP_FALSE_NORTHING },
     {NULL, NULL}
  };
@@ -347,7 +370,7 @@ static const oNetcdfSRS_PP poAEMappings[] = {
 static const oNetcdfSRS_PP poLAEAMappings[] = {
     {CF_PP_LAT_PROJ_ORIGIN, SRS_PP_LATITUDE_OF_CENTER},
     {CF_PP_LON_PROJ_ORIGIN, SRS_PP_LONGITUDE_OF_CENTER},
-    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },  
+    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },
     {CF_PP_FALSE_NORTHING, SRS_PP_FALSE_NORTHING },
     {NULL, NULL}
  };
@@ -365,7 +388,7 @@ static const oNetcdfSRS_PP poLAEAMappings[] = {
 //    * false_easting
 //    * false_northing
 //
-// See http://www.remotesensing.org/geotiff/proj_list/lambert_conic_conformal_1sp.html 
+// See http://www.remotesensing.org/geotiff/proj_list/lambert_conic_conformal_1sp.html
 
 // Lambert conformal conic - 1SP
 /* See bug # 3324
@@ -380,7 +403,7 @@ static const oNetcdfSRS_PP poLCC1SPMappings[] = {
     {CF_PP_LAT_PROJ_ORIGIN, SRS_PP_LATITUDE_OF_ORIGIN},
     {CF_PP_LONG_CENTRAL_MERIDIAN, SRS_PP_CENTRAL_MERIDIAN},
     {CF_PP_SCALE_FACTOR_ORIGIN, SRS_PP_SCALE_FACTOR}, /* special case */
-    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },  
+    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },
     {CF_PP_FALSE_NORTHING, SRS_PP_FALSE_NORTHING },
     {NULL, NULL}
  };
@@ -391,7 +414,7 @@ static const oNetcdfSRS_PP poLCC2SPMappings[] = {
     {CF_PP_STD_PARALLEL_2, SRS_PP_STANDARD_PARALLEL_2},
     {CF_PP_LAT_PROJ_ORIGIN, SRS_PP_LATITUDE_OF_ORIGIN},
     {CF_PP_LONG_CENTRAL_MERIDIAN, SRS_PP_CENTRAL_MERIDIAN},
-    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },  
+    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },
     {CF_PP_FALSE_NORTHING, SRS_PP_FALSE_NORTHING },
     {NULL, NULL}
  };
@@ -400,7 +423,7 @@ static const oNetcdfSRS_PP poLCC2SPMappings[] = {
 //
 // grid_mapping_name = lambert_cylindrical_equal_area
 // WKT: Cylindrical_Equal_Area
-// EPSG:9834 (Spherical) and EPSG:9835 
+// EPSG:9834 (Spherical) and EPSG:9835
 //
 // Map parameters:
 //
@@ -409,14 +432,14 @@ static const oNetcdfSRS_PP poLCC2SPMappings[] = {
 //    * false_easting
 //    * false_northing
 //
-// NB: CF-1 specifies a 'scale_factor_at_projection' alternative  
-//  to std_parallel ... but no reference to this in EPSG/remotesensing.org 
-//  ignore for now. 
+// NB: CF-1 specifies a 'scale_factor_at_projection' alternative
+//  to std_parallel ... but no reference to this in EPSG/remotesensing.org
+//  ignore for now.
 //
 static const oNetcdfSRS_PP poLCEAMappings[] = {
     {CF_PP_STD_PARALLEL_1, SRS_PP_STANDARD_PARALLEL_1},
     {CF_PP_LONG_CENTRAL_MERIDIAN, SRS_PP_CENTRAL_MERIDIAN},
-    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },  
+    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },
     {CF_PP_FALSE_NORTHING, SRS_PP_FALSE_NORTHING },
     {NULL, NULL}
  };
@@ -444,12 +467,12 @@ static const oNetcdfSRS_PP poLCEAMappings[] = {
 //    * false_easting
 //    * false_northing
 
-// Mercator 1 Standard Parallel (EPSG:9804) 
+// Mercator 1 Standard Parallel (EPSG:9804)
 static const oNetcdfSRS_PP poM1SPMappings[] = {
     {CF_PP_LON_PROJ_ORIGIN, SRS_PP_CENTRAL_MERIDIAN},
-    //LAT_PROJ_ORIGIN is always equator (0) in CF-1 
+    //LAT_PROJ_ORIGIN is always equator (0) in CF-1
     {CF_PP_SCALE_FACTOR_ORIGIN, SRS_PP_SCALE_FACTOR},
-    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },  
+    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },
     {CF_PP_FALSE_NORTHING, SRS_PP_FALSE_NORTHING },
     {NULL, NULL}
  };
@@ -458,10 +481,10 @@ static const oNetcdfSRS_PP poM1SPMappings[] = {
 static const oNetcdfSRS_PP poM2SPMappings[] = {
     {CF_PP_LON_PROJ_ORIGIN, SRS_PP_CENTRAL_MERIDIAN},
     {CF_PP_STD_PARALLEL_1, SRS_PP_STANDARD_PARALLEL_1},
-    //From best understanding of this projection, only  
- 	// actually specify one SP - it is the same N/S of equator. 
-    //{CF_PP_STD_PARALLEL_2, SRS_PP_LATITUDE_OF_ORIGIN}, 
-    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },  
+    //From best understanding of this projection, only
+ 	// actually specify one SP - it is the same N/S of equator.
+    //{CF_PP_STD_PARALLEL_2, SRS_PP_LATITUDE_OF_ORIGIN},
+    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },
     {CF_PP_FALSE_NORTHING, SRS_PP_FALSE_NORTHING },
     {NULL, NULL}
  };
@@ -480,10 +503,10 @@ static const oNetcdfSRS_PP poM2SPMappings[] = {
 static const oNetcdfSRS_PP poOrthoMappings[] = {
     {CF_PP_LAT_PROJ_ORIGIN, SRS_PP_LATITUDE_OF_ORIGIN},
     {CF_PP_LON_PROJ_ORIGIN, SRS_PP_CENTRAL_MERIDIAN},
-    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },  
+    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },
     {CF_PP_FALSE_NORTHING, SRS_PP_FALSE_NORTHING },
     {NULL, NULL}
- }; 
+ };
 
 // Polar stereographic
 //
@@ -498,7 +521,7 @@ static const oNetcdfSRS_PP poOrthoMappings[] = {
 //    * false_easting
 //    * false_northing
 
-/* 
+/*
    (http://www.remotesensing.org/geotiff/proj_list/polar_stereographic.html)
 
    Note: Projection parameters for this projection are quite different in CF-1 from
@@ -510,16 +533,16 @@ static const oNetcdfSRS_PP poOrthoMappings[] = {
        depending on the sign of 'latitude_of_origin' in WKT.
    CF allows the use of standard_parallel (lat_ts in proj4) OR scale_factor (k0 in proj4).
    This is analogous to the B and A variants (resp.) in EPSG guidelines.
-   When importing a CF file with scale_factor, we compute standard_parallel using 
+   When importing a CF file with scale_factor, we compute standard_parallel using
      Snyder eq. 22-7 with k=1 and lat=standard_parallel.
-   Currently OGR does NOT relate the scale factor with the standard parallel, so we 
+   Currently OGR does NOT relate the scale factor with the standard parallel, so we
    use the default. It seems that proj4 uses lat_ts (standard_parallel) and not k0.
 */
 static const oNetcdfSRS_PP poPSmappings[] = {
     {CF_PP_STD_PARALLEL_1, SRS_PP_LATITUDE_OF_ORIGIN},
     /* {CF_PP_SCALE_FACTOR_ORIGIN, SRS_PP_SCALE_FACTOR},   */
     {CF_PP_VERT_LONG_FROM_POLE, SRS_PP_CENTRAL_MERIDIAN},
-    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },  
+    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },
     {CF_PP_FALSE_NORTHING, SRS_PP_FALSE_NORTHING },
     {NULL, NULL}
 };
@@ -536,7 +559,7 @@ static const oNetcdfSRS_PP poPSmappings[] = {
 //    * north_pole_grid_longitude - This parameter is optional (default is 0.).
 
 /* TODO: No GDAL equivalent of rotated pole? Doesn't seem to have an EPSG
-   code or WKT ... so unless some advanced proj4 features can be used 
+   code or WKT ... so unless some advanced proj4 features can be used
    seems to rule out.
    see GDAL bug #4285 for a possible fix or workaround
 */
@@ -559,8 +582,8 @@ static const oNetcdfSRS_PP poPSmappings[] = {
 static const oNetcdfSRS_PP poStMappings[] = {
     {CF_PP_LAT_PROJ_ORIGIN, SRS_PP_LATITUDE_OF_ORIGIN},
     {CF_PP_LON_PROJ_ORIGIN, SRS_PP_CENTRAL_MERIDIAN},
-    {CF_PP_SCALE_FACTOR_ORIGIN, SRS_PP_SCALE_FACTOR},  
-    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },  
+    {CF_PP_SCALE_FACTOR_ORIGIN, SRS_PP_SCALE_FACTOR},
+    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },
     {CF_PP_FALSE_NORTHING, SRS_PP_FALSE_NORTHING },
     {NULL, NULL}
   };
@@ -579,10 +602,10 @@ static const oNetcdfSRS_PP poStMappings[] = {
 //    * false_northing
 //
 static const oNetcdfSRS_PP poTMMappings[] = {
-    {CF_PP_SCALE_FACTOR_MERIDIAN, SRS_PP_SCALE_FACTOR},  
+    {CF_PP_SCALE_FACTOR_MERIDIAN, SRS_PP_SCALE_FACTOR},
     {CF_PP_LONG_CENTRAL_MERIDIAN, SRS_PP_CENTRAL_MERIDIAN},
     {CF_PP_LAT_PROJ_ORIGIN, SRS_PP_LATITUDE_OF_ORIGIN},
-    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },  
+    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },
     {CF_PP_FALSE_NORTHING, SRS_PP_FALSE_NORTHING },
     {NULL, NULL}
   };
@@ -603,13 +626,24 @@ static const oNetcdfSRS_PP poTMMappings[] = {
 // TODO: see how to map this to OGR
 
 
-/* Mappings for various projections, including netcdf and GDAL projection names 
-   and corresponding oNetcdfSRS_PP mapping struct. 
+static const oNetcdfSRS_PP poGEOSMappings[] = {
+    {CF_PP_LON_PROJ_ORIGIN, SRS_PP_CENTRAL_MERIDIAN},
+    {CF_PP_PERSPECTIVE_POINT_HEIGHT, SRS_PP_SATELLITE_HEIGHT},
+    {CF_PP_FALSE_EASTING, SRS_PP_FALSE_EASTING },
+    {CF_PP_FALSE_NORTHING, SRS_PP_FALSE_NORTHING },
+    /* { CF_PP_SWEEP_ANGLE_AXIS, .... } handled as a proj.4 extension */
+    {NULL, NULL}
+  };
+
+
+
+/* Mappings for various projections, including netcdf and GDAL projection names
+   and corresponding oNetcdfSRS_PP mapping struct.
    A NULL mappings value means that the projection is not included in the CF
    standard and the generic mapping (poGenericMappings) will be used. */
 typedef struct {
     const char *CF_SRS;
-    const char *WKT_SRS; 
+    const char *WKT_SRS;
     const oNetcdfSRS_PP* mappings;
 } oNetcdfSRS_PT;
 
@@ -618,16 +652,16 @@ static const oNetcdfSRS_PT poNetcdfSRS_PT[] = {
     {CF_PT_AE, SRS_PT_AZIMUTHAL_EQUIDISTANT, poAEMappings },
     {"cassini_soldner", SRS_PT_CASSINI_SOLDNER, NULL },
     {CF_PT_LCEA, SRS_PT_CYLINDRICAL_EQUAL_AREA, poLCEAMappings },
-    {"eckert_iv", SRS_PT_ECKERT_IV, NULL },      
-    {"eckert_vi", SRS_PT_ECKERT_VI, NULL },  
+    {"eckert_iv", SRS_PT_ECKERT_IV, NULL },
+    {"eckert_vi", SRS_PT_ECKERT_VI, NULL },
     {"equidistant_conic", SRS_PT_EQUIDISTANT_CONIC, NULL },
     {"equirectangular", SRS_PT_EQUIRECTANGULAR, NULL },
     {"gall_stereographic", SRS_PT_GALL_STEREOGRAPHIC, NULL },
-    {"geostationary_satellite", SRS_PT_GEOSTATIONARY_SATELLITE, NULL },
+    {CF_PT_GEOS, SRS_PT_GEOSTATIONARY_SATELLITE, poGEOSMappings },
     {"goode_homolosine", SRS_PT_GOODE_HOMOLOSINE, NULL },
     {"gnomonic", SRS_PT_GNOMONIC, NULL },
     {"hotine_oblique_mercator", SRS_PT_HOTINE_OBLIQUE_MERCATOR, NULL },
-    {"hotine_oblique_mercator_2P", 
+    {"hotine_oblique_mercator_2P",
      SRS_PT_HOTINE_OBLIQUE_MERCATOR_TWO_POINT_NATURAL_ORIGIN, NULL },
     {"laborde_oblique_mercator", SRS_PT_LABORDE_OBLIQUE_MERCATOR, NULL },
     {CF_PT_LCC, SRS_PT_LAMBERT_CONFORMAL_CONIC_1SP, poLCC1SPMappings },
@@ -639,13 +673,13 @@ static const oNetcdfSRS_PT poNetcdfSRS_PT[] = {
     {"mollweide", SRS_PT_MOLLWEIDE, NULL },
     {"new_zealand_map_grid", SRS_PT_NEW_ZEALAND_MAP_GRID, NULL },
     /* for now map to STEREO, see bug #4267 */
-    {"oblique_stereographic", SRS_PT_OBLIQUE_STEREOGRAPHIC, NULL }, 
+    {"oblique_stereographic", SRS_PT_OBLIQUE_STEREOGRAPHIC, NULL },
     /* {STEREO, SRS_PT_OBLIQUE_STEREOGRAPHIC, poStMappings },  */
     {CF_PT_ORTHOGRAPHIC, SRS_PT_ORTHOGRAPHIC, poOrthoMappings },
     {CF_PT_POLAR_STEREO, SRS_PT_POLAR_STEREOGRAPHIC, poPSmappings },
     {"polyconic", SRS_PT_POLYCONIC, NULL },
-    {"robinson", SRS_PT_ROBINSON, NULL }, 
-    {"sinusoidal", SRS_PT_SINUSOIDAL, NULL },  
+    {"robinson", SRS_PT_ROBINSON, NULL },
+    {"sinusoidal", SRS_PT_SINUSOIDAL, NULL },
     {CF_PT_STEREO, SRS_PT_STEREOGRAPHIC, poStMappings },
     {"swiss_oblique_cylindrical", SRS_PT_SWISS_OBLIQUE_CYLINDRICAL, NULL },
     {CF_PT_TM, SRS_PT_TRANSVERSE_MERCATOR, poTMMappings },
@@ -655,15 +689,80 @@ static const oNetcdfSRS_PT poNetcdfSRS_PT[] = {
 
 /************************************************************************/
 /* ==================================================================== */
-/*			     netCDFDataset		                             		*/
+/*                        netCDFWriterConfig classes                    */
+/* ==================================================================== */
+/************************************************************************/
+
+class netCDFWriterConfigAttribute
+{
+    public:
+        CPLString m_osName;
+        CPLString m_osType;
+        CPLString m_osValue;
+
+        bool Parse(CPLXMLNode* psNode);
+};
+
+class netCDFWriterConfigField
+{
+    public:
+        CPLString m_osName;
+        CPLString m_osNetCDFName;
+        CPLString m_osMainDim;
+        std::vector<netCDFWriterConfigAttribute> m_aoAttributes;
+
+        bool Parse(CPLXMLNode* psNode);
+};
+
+class netCDFWriterConfigLayer
+{
+    public:
+        CPLString m_osName;
+        CPLString m_osNetCDFName;
+        std::map<CPLString, CPLString> m_oLayerCreationOptions;
+        std::vector<netCDFWriterConfigAttribute> m_aoAttributes;
+        std::map<CPLString, netCDFWriterConfigField> m_oFields;
+
+        bool Parse(CPLXMLNode* psNode);
+};
+
+class netCDFWriterConfiguration
+{
+    public:
+        bool m_bIsValid;
+        std::map<CPLString, CPLString> m_oDatasetCreationOptions;
+        std::map<CPLString, CPLString> m_oLayerCreationOptions;
+        std::vector<netCDFWriterConfigAttribute> m_aoAttributes;
+        std::map<CPLString, netCDFWriterConfigField> m_oFields;
+        std::map<CPLString, netCDFWriterConfigLayer> m_oLayers;
+
+        netCDFWriterConfiguration() : m_bIsValid(false) {}
+
+        bool Parse(const char* pszFilename);
+        static bool SetNameValue(CPLXMLNode* psNode,
+                                 std::map<CPLString,CPLString>& oMap);
+};
+
+/************************************************************************/
+/* ==================================================================== */
+/*                           netCDFDataset                              */
 /* ==================================================================== */
 /************************************************************************/
 
 class netCDFRasterBand;
+class netCDFLayer;
 
 class netCDFDataset : public GDALPamDataset
 {
     friend class netCDFRasterBand; //TMP
+    friend class netCDFLayer;
+
+    typedef enum
+    {
+        SINGLE_LAYER,
+        SEPARATE_FILES,
+        SEPARATE_GROUPS
+    } MultipleLayerBehaviour;
 
     /* basic dataset vars */
     CPLString     osFilename;
@@ -672,73 +771,92 @@ class netCDFDataset : public GDALPamDataset
     char          **papszMetadata;
     CPLStringList papszDimName;
     bool          bBottomUp;
-    int           nFormat;
-    int           bIsGdalFile; /* was this file created by GDAL? */
-    int           bIsGdalCfFile; /* was this file created by the (new) CF-compliant driver? */
+    NetCDFFormatEnum eFormat;
+    bool          bIsGdalFile; /* was this file created by GDAL? */
+    bool          bIsGdalCfFile; /* was this file created by the (new) CF-compliant driver? */
     char         *pszCFProjection;
     char         *pszCFCoordinates;
+    MultipleLayerBehaviour eMultipleLayerBehaviour;
+    std::vector<netCDFDataset*> apoVectorDatasets;
 
     /* projection/GT */
     double       adfGeoTransform[6];
     char         *pszProjection;
     int          nXDimID;
     int          nYDimID;
-    int          bIsProjected;
-    int          bIsGeographic;
+    bool         bIsProjected;
+    bool         bIsGeographic;
 
     /* state vars */
-    int          status;
-    int          bDefineMode;
-    int          bSetProjection; 
-    int          bSetGeoTransform;
-    int          bAddedProjectionVars;
-    int          bAddedGridMappingRef;
+    bool         bDefineMode;
+    bool         bSetProjection;
+    bool         bSetGeoTransform;
+    bool         bAddedProjectionVars;
+    bool         bAddedGridMappingRef;
 
     /* create vars */
     char         **papszCreationOptions;
-    int          nCompress;
+    NetCDFCompressEnum eCompress;
     int          nZLevel;
-    int          bChunking;
+#ifdef NETCDF_HAS_NC4
+    bool         bChunking;
+#endif
     int          nCreateMode;
-    int          bSignedData;
+    bool         bSignedData;
 
-    double       rint( double );
+    int          nLayers;
+    netCDFLayer   **papoLayers;
 
-    double       FetchCopyParm( const char *pszGridMappingValue, 
+    netCDFWriterConfiguration oWriterConfig;
+
+    static double       rint( double );
+
+    double       FetchCopyParm( const char *pszGridMappingValue,
                                 const char *pszParm, double dfDefault );
 
     char **      FetchStandardParallels( const char *pszGridMappingValue );
 
     void ProcessCreationOptions( );
-    int DefVarDeflate( int nVarId, int bChunkingArg=TRUE );
-    CPLErr AddProjectionVars( GDALProgressFunc pfnProgress=GDALDummyProgress, 
+    int DefVarDeflate( int nVarId, bool bChunkingArg=true );
+    CPLErr AddProjectionVars( GDALProgressFunc pfnProgress=GDALDummyProgress,
                               void * pProgressData=NULL );
-    void AddGridMappingRef(); 
+    void AddGridMappingRef();
 
-    int GetDefineMode() { return bDefineMode; }
-    int SetDefineMode( int bNewDefineMode );
+    bool GetDefineMode() { return bDefineMode; }
+    bool SetDefineMode( bool bNewDefineMode );
 
     CPLErr      ReadAttributes( int, int );
 
     void  CreateSubDatasetList( );
 
-    void  SetProjectionFromVar( int );
+    void  SetProjectionFromVar( int, bool bReadSRSOnly );
 
     int ProcessCFGeolocation( int );
     CPLErr Set1DGeolocation( int nVarId, const char *szDimName );
     double * Get1DGeolocation( const char *szDimName, int &nVarLen );
 
+    static bool CloneAttributes(int old_cdfid, int new_cdfid, int nSrcVarId, int nDstVarId);
+    static bool CloneVariableContent(int old_cdfid, int new_cdfid, int nSrcVarId, int nDstVarId);
+    static bool CloneGrp(int nOldGrpId, int nNewGrpId, bool bIsNC4, int nLayerId,
+                         int nDimIdToGrow, size_t nNewSize);
+    bool GrowDim(int nLayerId, int nDimIdToGrow, size_t nNewSize);
+
   protected:
 
     CPLXMLNode *SerializeToXML( const char *pszVRTPath );
 
+    virtual OGRLayer   *ICreateLayer( const char *pszName,
+                                     OGRSpatialReference *poSpatialRef,
+                                     OGRwkbGeometryType eGType,
+                                     char ** papszOptions );
+
   public:
 
-    netCDFDataset( );
-    ~netCDFDataset( );
-    
+    netCDFDataset();
+    virtual ~netCDFDataset();
+
     /* Projection/GT */
-    CPLErr 	GetGeoTransform( double * );    
+    CPLErr 	GetGeoTransform( double * );
     CPLErr 	SetGeoTransform (double *);
     const char * GetProjectionRef();
     CPLErr 	SetProjection (const char *);
@@ -746,11 +864,16 @@ class netCDFDataset : public GDALPamDataset
     virtual char      **GetMetadataDomainList();
     char ** GetMetadata( const char * );
 
+    virtual int  TestCapability(const char* pszCap);
+
+    virtual int  GetLayerCount() { return nLayers; }
+    virtual OGRLayer* GetLayer(int nIdx);
+
     int GetCDFID() { return cdfid; }
 
     /* static functions */
     static int Identify( GDALOpenInfo * );
-    static int IdentifyFormat( GDALOpenInfo *, bool );
+    static NetCDFFormatEnum IdentifyFormat( GDALOpenInfo *, bool );
     static GDALDataset *Open( GDALOpenInfo * );
 
     static netCDFDataset *CreateLL( const char * pszFilename,
@@ -760,10 +883,132 @@ class netCDFDataset : public GDALPamDataset
                                 int nXSize, int nYSize, int nBands,
                                 GDALDataType eType,
                                 char ** papszOptions );
-    static GDALDataset* CreateCopy( const char * pszFilename, GDALDataset *poSrcDS, 
-                                    int bStrict, char ** papszOptions, 
+    static GDALDataset* CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
+                                    int bStrict, char ** papszOptions,
                                     GDALProgressFunc pfnProgress, void * pProgressData );
-        
 };
 
+class netCDFLayer: public OGRLayer
+{
+        typedef union
+        {
+            signed char chVal;
+            unsigned char uchVal;
+            short   sVal;
+            unsigned short usVal;
+            int     nVal;
+            unsigned int unVal;
+            GIntBig nVal64;
+            GUIntBig unVal64;
+            float   fVal;
+            double  dfVal;
+        } NCDFNoDataUnion;
+
+        typedef struct
+        {
+            NCDFNoDataUnion uNoData;
+            nc_type         nType;
+            int             nVarId;
+            int             nDimCount;
+            bool            bHasWarnedAboutTruncation;
+            int             nMainDimId;
+            int             nSecDimId;
+            bool            bIsDays;
+        } FieldDesc;
+
+        netCDFDataset  *m_poDS;
+        int             m_nLayerCDFId;
+        OGRFeatureDefn *m_poFeatureDefn;
+        CPLString       m_osRecordDimName;
+        int             m_nRecordDimID;
+        int             m_nDefaultWidth;
+        bool            m_bAutoGrowStrings;
+        int             m_nDefaultMaxWidthDimId;
+        int             m_nXVarID;
+        int             m_nYVarID;
+        int             m_nZVarID;
+        nc_type         m_nXVarNCDFType;
+        nc_type         m_nYVarNCDFType;
+        nc_type         m_nZVarNCDFType;
+        NCDFNoDataUnion m_uXVarNoData;
+        NCDFNoDataUnion m_uYVarNoData;
+        NCDFNoDataUnion m_uZVarNoData;
+        CPLString       m_osWKTVarName;
+        int             m_nWKTMaxWidth;
+        int             m_nWKTMaxWidthDimId;
+        int             m_nWKTVarID;
+        nc_type         m_nWKTNCDFType;
+        CPLString       m_osCoordinatesValue;
+        std::vector<FieldDesc> m_aoFieldDesc;
+        int             m_nCurFeatureId;
+        CPLString       m_osGridMapping;
+        bool            m_bWriteGDALTags;
+        bool            m_bUseStringInNC4;
+        bool            m_bNCDumpCompat;
+
+        CPLString       m_osProfileDimName;
+        int             m_nProfileDimID;
+        CPLString       m_osProfileVariables;
+        int             m_nProfileVarID;
+        bool            m_bProfileVarUnlimited;
+        int             m_nParentIndexVarID;
+
+        const netCDFWriterConfigLayer* m_poLayerConfig;
+
+        OGRFeature     *GetNextRawFeature();
+        double          Get1DVarAsDouble( int nVarId, nc_type nVarType,
+                                          size_t nIndex,
+                                          NCDFNoDataUnion noDataVal,
+                                          bool* pbIsNoData );
+        CPLErr          GetFillValue( int nVarID, char **ppszValue );
+        CPLErr          GetFillValue( int nVarID, double *pdfValue );
+        void            GetNoDataValueForFloat( int nVarId, NCDFNoDataUnion* puNoData );
+        void            GetNoDataValueForDouble( int nVarId, NCDFNoDataUnion* puNoData );
+        void            GetNoDataValue( int nVarId, nc_type nVarType, NCDFNoDataUnion* puNoData );
+        bool            FillFeatureFromVar(OGRFeature* poFeature, int nMainDimId, size_t nIndex);
+        bool            FillVarFromFeature(OGRFeature* poFeature, int nMainDimId, size_t nIndex);
+
+    public:
+                netCDFLayer(netCDFDataset* poDS,
+                            int nLayerCDFId,
+                            const char* pszName,
+                            OGRwkbGeometryType eGeomType,
+                            OGRSpatialReference* poSRS);
+        virtual ~netCDFLayer();
+
+        bool            Create(char** papszOptions, const netCDFWriterConfigLayer* poLayerConfig);
+        void            SetRecordDimID(int nRecordDimID);
+        void            SetXYZVars(int nXVarId, int nYVarId, int nZVarId);
+        void            SetWKTGeometryField(const char* pszWKTVarName);
+        void            SetGridMapping(const char* pszGridMapping);
+        void            SetProfile(int nProfileDimID, int nParentIndexVarID);
+        bool            AddField(int nVarId);
+
+        int             GetCDFID() const { return m_nLayerCDFId; }
+        void            SetCDFID(int nId) { m_nLayerCDFId = nId; }
+
+        virtual void ResetReading();
+        virtual OGRFeature* GetNextFeature();
+
+        virtual GIntBig GetFeatureCount(int bForce);
+
+        virtual int  TestCapability(const char* pszCap);
+
+        virtual OGRFeatureDefn* GetLayerDefn();
+
+        virtual OGRErr ICreateFeature(OGRFeature* poFeature);
+        virtual OGRErr CreateField(OGRFieldDefn* poFieldDefn, int bApproxOK);
+};
+
+void NCDFWriteLonLatVarsAttributes(int cdfid, int nVarLonID, int nVarLatID);
+void NCDFWriteXYVarsAttributes(int cdfid, int nVarXID, int nVarYID,
+                                      OGRSpatialReference* poSRS);
+int NCDFWriteSRSVariable(int cdfid, OGRSpatialReference* poSRS,
+                                char** ppszCFProjection, bool bWriteGDALTags);
+CPLErr NCDFGetAttr( int nCdfId, int nVarId, const char *pszAttrName,
+                    double *pdfValue );
+CPLErr NCDFGetAttr( int nCdfId, int nVarId, const char *pszAttrName,
+                    char **pszValue );
+bool NCDFIsUnlimitedDim(bool bIsNC4, int cdfid, int nDimId);
+
 #endif
