@@ -1,5 +1,5 @@
 /***************************************************************************
- * $Id: mgrs.c 10645 2007-01-18 02:22:39Z warmerdam $
+ * $Id: mgrs.c 33717 2016-03-14 06:29:14Z goatbar $
  *
  * Project:  MGRS Converter
  * Purpose:  Geotrans code for MGRS translation (slightly adapted)
@@ -11,24 +11,24 @@
  *
  * ABSTRACT
  *
- *    This component converts between geodetic coordinates (latitude and 
- *    longitude) and Military Grid Reference System (MGRS) coordinates. 
+ *    This component converts between geodetic coordinates (latitude and
+ *    longitude) and Military Grid Reference System (MGRS) coordinates.
  *
  * ERROR HANDLING
  *
  *    This component checks parameters for valid values.  If an invalid value
- *    is found, the error code is combined with the current error code using 
+ *    is found, the error code is combined with the current error code using
  *    the bitwise or.  This combining allows multiple error codes to be
  *    returned. The possible error codes are:
  *
  *          MGRS_NO_ERROR          : No errors occurred in function
- *          MGRS_LAT_ERROR         : Latitude outside of valid range 
+ *          MGRS_LAT_ERROR         : Latitude outside of valid range
  *                                    (-90 to 90 degrees)
  *          MGRS_LON_ERROR         : Longitude outside of valid range
  *                                    (-180 to 360 degrees)
  *          MGRS_STR_ERROR         : An MGRS string error: string too long,
  *                                    too short, or badly formed
- *          MGRS_PRECISION_ERROR   : The precision must be between 0 and 5 
+ *          MGRS_PRECISION_ERROR   : The precision must be between 0 and 5
  *                                    inclusive.
  *          MGRS_A_ERROR           : Semi-major axis less than or equal to zero
  *          MGRS_INV_F_ERROR       : Inverse flattening outside of valid range
@@ -76,7 +76,7 @@
  *    ----              -----------
  *    16-11-94          Original Code
  *    15-09-99          Reengineered upper layers
- *    02-05-03          Corrected latitude band bug in GRID_UTM 
+ *    02-05-03          Corrected latitude band bug in GRID_UTM
  *    08-20-03          Reengineered lower layers
  */
 
@@ -139,50 +139,51 @@
 #define TWOMIL        2000000.e0    /* TWO MILLION                           */
 #define TRUE                      1  /* CONSTANT VALUE FOR TRUE VALUE  */
 #define FALSE                     0  /* CONSTANT VALUE FOR FALSE VALUE */
-#define PI    3.14159265358979323e0  /* PI                             */
-#define PI_OVER_2  (PI / 2.0e0)
+#define PI_OVER_2  (M_PI / 2.0e0)
 
 #define MIN_EASTING  100000
 #define MAX_EASTING  900000
 #define MIN_NORTHING 0
 #define MAX_NORTHING 10000000
 #define MAX_PRECISION           5   /* Maximum precision of easting & northing */
-#define MIN_UTM_LAT      ( (-80 * PI) / 180.0 ) /* -80 degrees in radians    */
-#define MAX_UTM_LAT      ( (84 * PI) / 180.0 )  /* 84 degrees in radians     */
+#define MIN_UTM_LAT      ( (-80 * M_PI) / 180.0 ) /* -80 degrees in radians    */
+#define MAX_UTM_LAT      ( (84 * M_PI) / 180.0 )  /* 84 degrees in radians     */
 
 #define MIN_EAST_NORTH 0
 #define MAX_EAST_NORTH 4000000
 
 
 /* Ellipsoid parameters, default to WGS 84 */
-double MGRS_a = 6378137.0;    /* Semi-major axis of ellipsoid in meters */
-double MGRS_f = 1 / 298.257223563; /* Flattening of ellipsoid           */
-double MGRS_recpf = 298.257223563;
-char   MGRS_Ellipsoid_Code[3] = {'W','E',0};
+static const double MGRS_a = 6378137.0;    /* Semi-major axis of ellipsoid in meters */
+static const double MGRS_f = 1 / 298.257223563; /* Flattening of ellipsoid           */
+#ifdef unused
+static const double MGRS_recpf = 298.257223563;
+#endif
+static const char   MGRS_Ellipsoid_Code[3] = {'W','E',0};
 
 
-/* 
+/*
  *    CLARKE_1866 : Ellipsoid code for CLARKE_1866
  *    CLARKE_1880 : Ellipsoid code for CLARKE_1880
  *    BESSEL_1841 : Ellipsoid code for BESSEL_1841
  *    BESSEL_1841_NAMIBIA : Ellipsoid code for BESSEL 1841 (NAMIBIA)
  */
-const char* CLARKE_1866 = "CC";
-const char* CLARKE_1880 = "CD";
-const char* BESSEL_1841 = "BR";
-const char* BESSEL_1841_NAMIBIA = "BN";
+static const char* const CLARKE_1866 = "CC";
+static const char* const CLARKE_1880 = "CD";
+static const char* const BESSEL_1841 = "BR";
+static const char* const BESSEL_1841_NAMIBIA = "BN";
 
 
 typedef struct Latitude_Band_Value
 {
   long letter;            /* letter representing latitude band  */
-  double min_northing;    /* minimum northing for latitude band */       
+  double min_northing;    /* minimum northing for latitude band */
   double north;           /* upper latitude for latitude band   */
   double south;           /* lower latitude for latitude band   */
-} Latitude_Band; 
+} Latitude_Band;
 
-static const Latitude_Band Latitude_Band_Table[20] = 
-  {{LETTER_C, 1100000.0, -72.0, -80.5}, 
+static const Latitude_Band Latitude_Band_Table[20] =
+  {{LETTER_C, 1100000.0, -72.0, -80.5},
   {LETTER_D, 2000000.0, -64.0, -72.0},
   {LETTER_E, 2800000.0, -56.0, -64.0},
   {LETTER_F, 3700000.0, -48.0, -56.0},
@@ -203,7 +204,7 @@ static const Latitude_Band Latitude_Band_Table[20] =
   {LETTER_W, 7000000.0, 72.0, 64.0},
   {LETTER_X, 7900000.0, 84.5, 72.0}};
 
- 
+
 typedef struct UPS_Constant_Value
 {
   long letter;            /* letter representing latitude band      */
@@ -212,9 +213,9 @@ typedef struct UPS_Constant_Value
   long ltr3_high_value;   /* 3rd letter range - high number (UPS)   */
   double false_easting;   /* False easting based on 2nd letter      */
   double false_northing;  /* False northing based on 3rd letter     */
-} UPS_Constant; 
+} UPS_Constant;
 
-static const UPS_Constant UPS_Constant_Table[4] = 
+static const UPS_Constant UPS_Constant_Table[4] =
   {{LETTER_A, LETTER_J, LETTER_Z, LETTER_Z, 800000.0, 800000.0},
   {LETTER_B, LETTER_A, LETTER_R, LETTER_Z, 2000000.0, 800000.0},
   {LETTER_Y, LETTER_J, LETTER_Z, LETTER_P, 800000.0, 1300000.0},
@@ -222,10 +223,10 @@ static const UPS_Constant UPS_Constant_Table[4] =
 
 /***************************************************************************/
 /*
- *                              FUNCTIONS     
+ *                              FUNCTIONS
  */
 
-long Get_Latitude_Band_Min_Northing(long letter, double* min_northing)
+static long Get_Latitude_Band_Min_Northing(long letter, double* min_northing)
 /*
  * The function Get_Latitude_Band_Min_Northing receives a latitude band letter
  * and uses the Latitude_Band_Table to determine the minimum northing for that
@@ -249,11 +250,11 @@ long Get_Latitude_Band_Min_Northing(long letter, double* min_northing)
   return error_code;
 } /* Get_Latitude_Band_Min_Northing */
 
-
-long Get_Latitude_Range(long letter, double* north, double* south)
+#ifdef unused
+static long Get_Latitude_Range(long letter, double* north, double* south)
 /*
  * The function Get_Latitude_Range receives a latitude band letter
- * and uses the Latitude_Band_Table to determine the latitude band 
+ * and uses the Latitude_Band_Table to determine the latitude band
  * boundaries for that latitude band letter.
  *
  *   letter   : Latitude band letter                        (input)
@@ -283,12 +284,13 @@ long Get_Latitude_Range(long letter, double* north, double* south)
 
   return error_code;
 } /* Get_Latitude_Range */
+#endif
 
-
-long Get_Latitude_Letter(double latitude, int* letter)
+#ifdef unusued
+static long Get_Latitude_Letter(double latitude, int* letter)
 /*
  * The function Get_Latitude_Letter receives a latitude value
- * and uses the Latitude_Band_Table to determine the latitude band 
+ * and uses the Latitude_Band_Table to determine the latitude band
  * letter for that latitude.
  *
  *   latitude   : Latitude              (input)
@@ -311,16 +313,17 @@ long Get_Latitude_Letter(double latitude, int* letter)
 
   return error_code;
 } /* Get_Latitude_Letter */
+#endif
 
-
-long Check_Zone(char* MGRS, long* zone_exists)
+#ifdef unused
+static long Check_Zone(char* MGRS, long* zone_exists)
 /*
  * The function Check_Zone receives an MGRS coordinate string.
  * If a zone is given, TRUE is returned. Otherwise, FALSE
  * is returned.
  *
  *   MGRS           : MGRS coordinate string        (input)
- *   zone_exists    : TRUE if a zone is given, 
+ *   zone_exists    : TRUE if a zone is given,
  *                    FALSE if a zone is not given  (output)
  */
 { /* Check_Zone */
@@ -329,9 +332,9 @@ long Check_Zone(char* MGRS, long* zone_exists)
   int num_digits = 0;
   long error_code = MGRS_NO_ERROR;
 
-  /* skip any leading blanks */  
+  /* skip any leading blanks */
   while (MGRS[i] == ' ')
-    i++;  
+    i++;
   j = i;
   while (isdigit(MGRS[i]))
     i++;
@@ -346,11 +349,11 @@ long Check_Zone(char* MGRS, long* zone_exists)
 
   return error_code;
 } /* Check_Zone */
+#endif
 
-
-long Round_MGRS (double value)
+static long Round_MGRS (double value)
 /*
- * The function Round_MGRS rounds the input value to the 
+ * The function Round_MGRS rounds the input value to the
  * nearest integer, using the standard engineering rule.
  * The rounded integer value is then returned.
  *
@@ -367,14 +370,14 @@ long Round_MGRS (double value)
 } /* Round_MGRS */
 
 
-long Make_MGRS_String (char* MGRS, 
-                       long Zone, 
-                       int Letters[MGRS_LETTERS], 
-                       double Easting, 
+static long Make_MGRS_String (char* MGRS,
+                       long Zone,
+                       int Letters[MGRS_LETTERS],
+                       double Easting,
                        double Northing,
                        long Precision)
 /*
- * The function Make_MGRS_String constructs an MGRS string 
+ * The function Make_MGRS_String constructs an MGRS string
  * from its component parts.
  *
  *   MGRS           : MGRS coordinate string          (output)
@@ -411,19 +414,19 @@ long Make_MGRS_String (char* MGRS,
   if (Northing >= 99999.5)
     Northing = 99999.0;
   north = (long)(Northing/divisor);
-  i += sprintf (MGRS+i, "%*.*ld", (int) Precision, (int) Precision, north);
+  /*i += */sprintf (MGRS+i, "%*.*ld", (int) Precision, (int) Precision, north);
   return (error_code);
 } /* Make_MGRS_String */
 
 
-long Break_MGRS_String (char* MGRS,
+static long Break_MGRS_String (char* MGRS,
                         long* Zone,
                         long Letters[MGRS_LETTERS],
                         double* Easting,
                         double* Northing,
                         long* Precision)
 /*
- * The function Break_MGRS_String breaks down an MGRS  
+ * The function Break_MGRS_String breaks down an MGRS
  * coordinate string into its component parts.
  *
  *   MGRS           : MGRS coordinate string          (input)
@@ -453,7 +456,7 @@ long Break_MGRS_String (char* MGRS,
       /* get zone */
       strncpy (zone_string, MGRS+j, 2);
       zone_string[2] = 0;
-      sscanf (zone_string, "%ld", Zone);  
+      sscanf (zone_string, "%ld", Zone);
       if ((*Zone < 1) || (*Zone > 60))
         error_code |= MGRS_STRING_ERROR;
     }
@@ -521,15 +524,15 @@ long Break_MGRS_String (char* MGRS,
 } /* Break_MGRS_String */
 
 
-void Get_Grid_Values (long zone, 
-                      long* ltr2_low_value, 
-                      long* ltr2_high_value, 
+static void Get_Grid_Values (long zone,
+                      long* ltr2_low_value,
+                      long* ltr2_high_value,
                       double *false_northing)
 /*
- * The function Get_Grid_Values sets the letter range used for 
- * the 2nd letter in the MGRS coordinate string, based on the set 
+ * The function Get_Grid_Values sets the letter range used for
+ * the 2nd letter in the MGRS coordinate string, based on the set
  * number of the utm zone. It also sets the false northing using a
- * value of A for the second letter of the grid square, based on 
+ * value of A for the second letter of the grid square, based on
  * the grid pattern and set number of the utm zone.
  *
  *    zone            : Zone number             (input)
@@ -546,7 +549,7 @@ void Get_Grid_Values (long zone,
   if (!set_number)
     set_number = 6;
 
-  if (!strcmp(MGRS_Ellipsoid_Code,CLARKE_1866) || !strcmp(MGRS_Ellipsoid_Code, CLARKE_1880) || 
+  if (!strcmp(MGRS_Ellipsoid_Code,CLARKE_1866) || !strcmp(MGRS_Ellipsoid_Code, CLARKE_1880) ||
       !strcmp(MGRS_Ellipsoid_Code,BESSEL_1841) || !strcmp(MGRS_Ellipsoid_Code,BESSEL_1841_NAMIBIA))
     aa_pattern = FALSE;
   else
@@ -585,12 +588,12 @@ void Get_Grid_Values (long zone,
   }
 } /* END OF Get_Grid_Values */
 
-
-long UTM_To_MGRS (long Zone,
+#ifdef unused
+static long UTM_To_MGRS (long Zone,
                   double Latitude,
                   double Easting,
                   double Northing,
-                  long Precision, 
+                  long Precision,
                   char *MGRS)
 /*
  * The function UTM_To_MGRS calculates an MGRS coordinate string
@@ -621,23 +624,23 @@ long UTM_To_MGRS (long Zone,
   Get_Grid_Values(Zone, &ltr2_low_value, &ltr2_high_value, &false_northing);
 
   error_code = Get_Latitude_Letter(Latitude, &letters[0]);
-   
+
   if (!error_code)
   {
     grid_northing = Northing;
     if (grid_northing == 1.e7)
-      grid_northing = grid_northing - 1.0; 
+      grid_northing = grid_northing - 1.0;
 
     while (grid_northing >= TWOMIL)
     {
-      grid_northing = grid_northing - TWOMIL; 
+      grid_northing = grid_northing - TWOMIL;
     }
     grid_northing = grid_northing - false_northing;
 
     if (grid_northing < 0.0)
       grid_northing = grid_northing + TWOMIL;
 
-    letters[2] = (long)(grid_northing / ONEHT); 
+    letters[2] = (long)(grid_northing / ONEHT);
     if (letters[2] > LETTER_H)
       letters[2] = letters[2] + 1;
 
@@ -648,7 +651,7 @@ long UTM_To_MGRS (long Zone,
     if (((letters[0] == LETTER_V) && (Zone == 31)) && (grid_easting == 500000.0))
       grid_easting = grid_easting - 1.0; /* SUBTRACT 1 METER */
 
-    letters[1] = ltr2_low_value + ((long)(grid_easting / ONEHT) -1); 
+    letters[1] = ltr2_low_value + ((long)(grid_easting / ONEHT) -1);
     if ((ltr2_low_value == LETTER_J) && (letters[1] > LETTER_N))
       letters[1] = letters[1] + 1;
 
@@ -656,8 +659,9 @@ long UTM_To_MGRS (long Zone,
   }
   return error_code;
 } /* END UTM_To_MGRS */
+#endif
 
-
+#ifdef unused
 long Set_MGRS_Parameters (double a,
                           double f,
                           char   *Ellipsoid_Code)
@@ -688,11 +692,12 @@ long Set_MGRS_Parameters (double a,
     MGRS_a = a;
     MGRS_f = f;
     MGRS_recpf = inv_f;
-    strcpy (MGRS_Ellipsoid_Code, Ellipsoid_Code);
+    strncpy (MGRS_Ellipsoid_Code, Ellipsoid_Code, sizeof(MGRS_Ellipsoid_Code));
+    MGRS_Ellipsoid_Code[sizeof(MGRS_Ellipsoid_Code) - 1] = '\0';
   }
   return (Error_Code);
 }  /* Set_MGRS_Parameters  */
-
+#endif
 
 void Get_MGRS_Parameters (double *a,
                           double *f,
@@ -712,7 +717,7 @@ void Get_MGRS_Parameters (double *a,
   return;
 } /* Get_MGRS_Parameters */
 
-#ifdef notdef
+#ifndef GDAL_COMPILATION
 long Convert_UTM_To_MGRS (long Zone,
                           char Hemisphere,
                           double Easting,
@@ -721,8 +726,8 @@ long Convert_UTM_To_MGRS (long Zone,
                           char* MGRS)
 /*
  * The function Convert_UTM_To_MGRS converts UTM (zone, easting, and
- * northing) coordinates to an MGRS coordinate string, according to the 
- * current ellipsoid parameters.  If any errors occur, the error code(s) 
+ * northing) coordinates to an MGRS coordinate string, according to the
+ * current ellipsoid parameters.  If any errors occur, the error code(s)
  * are returned by the function, otherwise MGRS_NO_ERROR is returned.
  *
  *    Zone       : UTM zone                         (input)
@@ -754,7 +759,7 @@ long Convert_UTM_To_MGRS (long Zone,
     temp_error = Convert_UTM_To_Geodetic (Zone, Hemisphere, Easting, Northing, &latitude, &longitude);
 
 	  /* Special check for rounding to (truncated) eastern edge of zone 31V */
-	  if ((Zone == 31) && (latitude >= 56.0 * DEG_TO_RAD) && (latitude < 64.0 * DEG_TO_RAD) && 
+	  if ((Zone == 31) && (latitude >= 56.0 * DEG_TO_RAD) && (latitude < 64.0 * DEG_TO_RAD) &&
         (longitude >= 3.0 * DEG_TO_RAD))
 	  { /* Reconvert to UTM zone 32 */
       Set_UTM_Parameters (MGRS_a, MGRS_f, 32);
@@ -774,9 +779,9 @@ long Convert_MGRS_To_UTM (char   *MGRS,
                           double *Northing)
 /*
  * The function Convert_MGRS_To_UTM converts an MGRS coordinate string
- * to UTM projection (zone, hemisphere, easting and northing) coordinates 
- * according to the current ellipsoid parameters.  If any errors occur, 
- * the error code(s) are returned by the function, otherwise UTM_NO_ERROR 
+ * to UTM projection (zone, hemisphere, easting and northing) coordinates
+ * according to the current ellipsoid parameters.  If any errors occur,
+ * the error code(s) are returned by the function, otherwise UTM_NO_ERROR
  * is returned.
  *
  *    MGRS       : MGRS coordinate string           (input)
@@ -788,14 +793,14 @@ long Convert_MGRS_To_UTM (char   *MGRS,
 { /* Convert_MGRS_To_UTM */
   double scaled_min_northing;
   double min_northing;
-  long ltr2_low_value;
-  long ltr2_high_value;
+  long ltr2_low_value = 0;
+  long ltr2_high_value = 0;
   double false_northing;
   double grid_easting;        /* Easting for 100,000 meter grid square      */
   double grid_northing;       /* Northing for 100,000 meter grid square     */
   long letters[MGRS_LETTERS];
   long in_precision;
-#ifdef notdef
+#ifndef GDAL_COMPILATION
   double upper_lat_limit;     /* North latitude limits based on 1st letter  */
   double lower_lat_limit;     /* South latitude limits based on 1st letter  */
   double latitude = 0.0;
@@ -823,7 +828,7 @@ long Convert_MGRS_To_UTM (char   *MGRS,
         Get_Grid_Values(*Zone, &ltr2_low_value, &ltr2_high_value, &false_northing);
 
         /* Check that the second letter of the MGRS string is within
-         * the range of valid second letter values 
+         * the range of valid second letter values
          * Also check that the third letter is valid */
         if ((letters[1] < ltr2_low_value) || (letters[1] > ltr2_high_value) || (letters[2] > LETTER_V))
           error_code |= MGRS_STRING_ERROR;
@@ -839,7 +844,7 @@ long Convert_MGRS_To_UTM (char   *MGRS,
             grid_northing = grid_northing - ONEHT;
 
           if (letters[2] > LETTER_I)
-            grid_northing = grid_northing - ONEHT; 
+            grid_northing = grid_northing - ONEHT;
 
           if (grid_northing >= TWOMIL)
             grid_northing = grid_northing - TWOMIL;
@@ -861,7 +866,7 @@ long Convert_MGRS_To_UTM (char   *MGRS,
 
             *Easting = grid_easting + *Easting;
             *Northing = grid_northing + *Northing;
-#ifdef notdef
+#ifndef GDAL_COMPILATION
             /* check that point is within Zone Letter bounds */
             error_code = Set_UTM_Parameters(MGRS_a,MGRS_f,*Zone);
             if (!error_code)
@@ -894,10 +899,10 @@ long Convert_UPS_To_MGRS (char   Hemisphere,
                           long   Precision,
                           char*  MGRS)
 /*
- *  The function Convert_UPS_To_MGRS converts UPS (hemisphere, easting, 
- *  and northing) coordinates to an MGRS coordinate string according to 
+ *  The function Convert_UPS_To_MGRS converts UPS (hemisphere, easting,
+ *  and northing) coordinates to an MGRS coordinate string according to
  *  the current ellipsoid parameters.  If any errors occur, the error
- *  code(s) are returned by the function, otherwise UPS_NO_ERROR is 
+ *  code(s) are returned by the function, otherwise UPS_NO_ERROR is
  *  returned.
  *
  *    Hemisphere    : Hemisphere either 'N' or 'S'     (input)
@@ -914,7 +919,7 @@ long Convert_UPS_To_MGRS (char   Hemisphere,
   long ltr2_low_value;        /* 2nd letter range - low number                */
   int letters[MGRS_LETTERS];  /* Number location of 3 letters in alphabet     */
   double divisor;
-  int index = 0;
+  int l_index = 0;
   long error_code = MGRS_NO_ERROR;
 
   if ((Hemisphere != 'N') && (Hemisphere != 'S'))
@@ -934,14 +939,14 @@ long Convert_UPS_To_MGRS (char   Hemisphere,
     if (Hemisphere == 'N')
     {
       if (Easting >= TWOMIL)
-        letters[0] = LETTER_Z; 
+        letters[0] = LETTER_Z;
       else
         letters[0] = LETTER_Y;
 
-      index = letters[0] - 22;
-      ltr2_low_value = UPS_Constant_Table[index].ltr2_low_value;
-      false_easting = UPS_Constant_Table[index].false_easting;
-      false_northing = UPS_Constant_Table[index].false_northing;
+      l_index = letters[0] - 22;
+      ltr2_low_value = UPS_Constant_Table[l_index].ltr2_low_value;
+      false_easting = UPS_Constant_Table[l_index].false_easting;
+      false_northing = UPS_Constant_Table[l_index].false_northing;
     }
     else
     {
@@ -957,7 +962,7 @@ long Convert_UPS_To_MGRS (char   Hemisphere,
 
     grid_northing = Northing;
     grid_northing = grid_northing - false_northing;
-    letters[2] = (long)(grid_northing / ONEHT);
+    letters[2] = (int)(grid_northing / ONEHT);
 
     if (letters[2] > LETTER_H)
       letters[2] = letters[2] + 1;
@@ -967,26 +972,26 @@ long Convert_UPS_To_MGRS (char   Hemisphere,
 
     grid_easting = Easting;
     grid_easting = grid_easting - false_easting;
-    letters[1] = ltr2_low_value + ((long)(grid_easting / ONEHT)); 
+    letters[1] = (int)(ltr2_low_value + ((long)(grid_easting / ONEHT)));
 
     if (Easting < TWOMIL)
     {
       if (letters[1] > LETTER_L)
-        letters[1] = letters[1] + 3; 
+        letters[1] = letters[1] + 3;
 
       if (letters[1] > LETTER_U)
-        letters[1] = letters[1] + 2; 
+        letters[1] = letters[1] + 2;
     }
     else
     {
       if (letters[1] > LETTER_C)
-        letters[1] = letters[1] + 2; 
+        letters[1] = letters[1] + 2;
 
       if (letters[1] > LETTER_H)
         letters[1] = letters[1] + 1;
-      
+
       if (letters[1] > LETTER_L)
-        letters[1] = letters[1] + 3; 
+        letters[1] = letters[1] + 3;
     }
 
     Make_MGRS_String (MGRS, 0, letters, Easting, Northing, Precision);
@@ -1001,9 +1006,9 @@ long Convert_MGRS_To_UPS ( char   *MGRS,
                            double *Northing)
 /*
  *  The function Convert_MGRS_To_UPS converts an MGRS coordinate string
- *  to UPS (hemisphere, easting, and northing) coordinates, according 
- *  to the current ellipsoid parameters. If any errors occur, the error 
- *  code(s) are returned by the function, otherwide UPS_NO_ERROR is returned.
+ *  to UPS (hemisphere, easting, and northing) coordinates, according
+ *  to the current ellipsoid parameters. If any errors occur, the error
+ *  code(s) are returned by the function, otherwise UPS_NO_ERROR is returned.
  *
  *    MGRS          : MGRS coordinate string           (input)
  *    Hemisphere    : Hemisphere either 'N' or 'S'     (output)
@@ -1018,10 +1023,10 @@ long Convert_MGRS_To_UPS ( char   *MGRS,
   double false_northing;      /* False northing for 3rd letter              */
   double grid_easting;        /* easting for 100,000 meter grid square      */
   double grid_northing;       /* northing for 100,000 meter grid square     */
-  long zone;
+  long zone = 0;
   long letters[MGRS_LETTERS];
   long in_precision;
-  int index = 0;
+  int l_index = 0;
   long error_code = MGRS_NO_ERROR;
 
   error_code = Break_MGRS_String (MGRS, &zone, letters, Easting, Northing, &in_precision);
@@ -1035,12 +1040,12 @@ long Convert_MGRS_To_UPS ( char   *MGRS,
       {
         *Hemisphere = 'N';
 
-        index = letters[0] - 22;
-        ltr2_low_value = UPS_Constant_Table[index].ltr2_low_value;
-        ltr2_high_value = UPS_Constant_Table[index].ltr2_high_value;
-        ltr3_high_value = UPS_Constant_Table[index].ltr3_high_value;
-        false_easting = UPS_Constant_Table[index].false_easting;
-        false_northing = UPS_Constant_Table[index].false_northing;
+        l_index = (int)(letters[0] - 22);
+        ltr2_low_value = UPS_Constant_Table[l_index].ltr2_low_value;
+        ltr2_high_value = UPS_Constant_Table[l_index].ltr2_high_value;
+        ltr3_high_value = UPS_Constant_Table[l_index].ltr3_high_value;
+        false_easting = UPS_Constant_Table[l_index].false_easting;
+        false_northing = UPS_Constant_Table[l_index].false_northing;
       }
       else
       {
@@ -1054,7 +1059,7 @@ long Convert_MGRS_To_UPS ( char   *MGRS,
       }
 
       /* Check that the second letter of the MGRS string is within
-       * the range of valid second letter values 
+       * the range of valid second letter values
        * Also check that the third letter is valid */
       if ((letters[1] < ltr2_low_value) || (letters[1] > ltr2_high_value) ||
           ((letters[1] == LETTER_D) || (letters[1] == LETTER_E) ||
@@ -1065,14 +1070,14 @@ long Convert_MGRS_To_UPS ( char   *MGRS,
 
       if (!error_code)
       {
-        grid_northing = (double)letters[2] * ONEHT + false_northing; 
+        grid_northing = (double)letters[2] * ONEHT + false_northing;
         if (letters[2] > LETTER_I)
           grid_northing = grid_northing - ONEHT;
 
         if (letters[2] > LETTER_O)
           grid_northing = grid_northing - ONEHT;
 
-        grid_easting = (double)((letters[1]) - ltr2_low_value) * ONEHT + false_easting; 
+        grid_easting = (double)((letters[1]) - ltr2_low_value) * ONEHT + false_easting;
         if (ltr2_low_value != LETTER_A)
         {
           if (letters[1] > LETTER_L)
@@ -1100,6 +1105,3 @@ long Convert_MGRS_To_UPS ( char   *MGRS,
   }
   return (error_code);
 } /* Convert_MGRS_To_UPS */
-
-
-
