@@ -1,8 +1,8 @@
 /******************************************************************************
- * $Id: nitflib.h 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: nitflib.h 33717 2016-03-14 06:29:14Z goatbar $
  *
  * Project:  NITF Read/Write Library
- * Purpose:  Main GDAL independent include file for NITF support.  
+ * Purpose:  Main GDAL independent include file for NITF support.
  * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
  **********************************************************************
@@ -38,7 +38,7 @@
 
 CPL_C_START
 
-typedef struct { 
+typedef struct {
     char szSegmentType[3]; /* one of "IM", ... */
 
     GUIntBig nSegmentHeaderStart;
@@ -73,21 +73,22 @@ typedef struct {
     char    **papszMetadata;
 
     CPLXMLNode *psNITFSpecNode;
-    
+
 } NITFFile;
 
 /* -------------------------------------------------------------------- */
 /*      File level prototypes.                                          */
 /* -------------------------------------------------------------------- */
 NITFFile CPL_DLL *NITFOpen( const char *pszFilename, int bUpdatable );
+NITFFile *NITFOpenEx( VSILFILE *fp, const char *pszFilename );
 void     CPL_DLL  NITFClose( NITFFile * );
 
-int      CPL_DLL  NITFCreate( const char *pszFilename, 
-                              int nPixels, int nLines, int nBands, 
+int      CPL_DLL  NITFCreate( const char *pszFilename,
+                              int nPixels, int nLines, int nBands,
                               int nBitsPerSample, const char *pszPVType,
                               char **papszOptions );
 
-const char CPL_DLL *NITFFindTRE( const char *pszTREData, int nTREBytes, 
+const char CPL_DLL *NITFFindTRE( const char *pszTREData, int nTREBytes,
                                  const char *pszTag, int *pnFoundTRESize );
 const char CPL_DLL *NITFFindTREByIndex( const char *pszTREData, int nTREBytes,
                                 const char *pszTag, int nTreIndex,
@@ -109,7 +110,7 @@ typedef struct {
 
 } NITFBandInfo;
 
-typedef struct { 
+typedef struct {
     GUInt16 nLocId;
     GUInt32 nLocOffset;
     GUInt32 nLocSize;
@@ -137,7 +138,7 @@ typedef struct {
     int        nBitsPerSample;
 
     NITFBandInfo *pasBandInfo;
-    
+
     char       chIMODE;
 
     int        nBlocksPerRow;
@@ -148,7 +149,7 @@ typedef struct {
     char       szPVType[4];
     char       szIREP[9];
     char       szICAT[9];
-    int        nABPP; /* signficant bits per pixel */
+    int        nABPP; /* significant bits per pixel */
 
     char       chICORDS;
     int        bHaveIGEOLO;
@@ -173,7 +174,7 @@ typedef struct {
     int        nIALVL;
     int        nIDLVL;
     char       szIMAG[5];
-    
+
     int        bNoDataSet;
     int        nNoDataValue;
 
@@ -181,7 +182,7 @@ typedef struct {
     char    *pachTRE;
 
     /* Internal information not for application use. */
-    
+
     int        nWordSize;
     GUIntBig   nPixelOffset;
     GUIntBig   nLineOffset;
@@ -191,7 +192,7 @@ typedef struct {
     GUIntBig    *panBlockStart;
 
     char       **papszMetadata;
-    
+
     GUInt32 *apanVQLUT[4];
 
     int     nLocCount;
@@ -204,13 +205,13 @@ void      CPL_DLL  NITFImageDeaccess( NITFImage * );
 
 int       CPL_DLL  NITFReadImageBlock( NITFImage *, int nBlockX, int nBlockY,
                                        int nBand, void *pData );
-int       CPL_DLL  NITFReadImageLine( NITFImage *, int nLine, int nBand, 
+int       CPL_DLL  NITFReadImageLine( NITFImage *, int nLine, int nBand,
                                       void *pData );
 int       CPL_DLL  NITFWriteImageBlock( NITFImage *, int nBlockX, int nBlockY,
                                         int nBand, void *pData );
-int       CPL_DLL  NITFWriteImageLine( NITFImage *, int nLine, int nBand, 
+int       CPL_DLL  NITFWriteImageLine( NITFImage *, int nLine, int nBand,
                                        void *pData );
-int       CPL_DLL  NITFWriteLUT( NITFImage *psImage, int nBand, int nColors, 
+int       CPL_DLL  NITFWriteLUT( NITFImage *psImage, int nBand, int nColors,
                                  unsigned char *pabyLUT );
 int       CPL_DLL  NITFWriteIGEOLO( NITFImage *psImage, char chICORDS,
                                     int nZone,
@@ -224,7 +225,7 @@ char      CPL_DLL **NITFReadUSE00A( NITFImage *psImage );
 char      CPL_DLL **NITFReadSTDIDC( NITFImage *psImage );
 char      CPL_DLL **NITFReadBLOCKA( NITFImage *psImage );
 
-GUIntBig  CPL_DLL NITFIHFieldOffset( NITFImage *psImage, 
+GUIntBig  CPL_DLL NITFIHFieldOffset( NITFImage *psImage,
                                      const char *pszFieldName );
 
 #define BLKREAD_OK    0
@@ -234,7 +235,7 @@ GUIntBig  CPL_DLL NITFIHFieldOffset( NITFImage *psImage,
 int NITFUncompressARIDPCM( NITFImage *psImage,
                            GByte *pabyInputData, int nInputBytes,
                            GByte *pabyOutputImage );
-int NITFUncompressBILEVEL( NITFImage *psImage, 
+int NITFUncompressBILEVEL( NITFImage *psImage,
                            GByte *pabyInputData, int nInputBytes,
                            GByte *pabyOutputImage );
 
@@ -267,7 +268,7 @@ int       CPL_DLL  NITFDESExtractShapefile(NITFDES* psDES, const char* pszRadixF
 /*      These are really intended to be private helper stuff for the    */
 /*      library.                                                        */
 /* -------------------------------------------------------------------- */
-char *NITFGetField( char *pszTarget, const char *pszSource, 
+char *NITFGetField( char *pszTarget, const char *pszSource,
                     int nStart, int nLength );
 void NITFExtractMetadata( char ***ppapszMetadata, const char *pachHeader,
                           int nStart, int nLength, const char *pszName );
@@ -375,7 +376,7 @@ typedef struct {
 
 int CPL_DLL NITFReadICHIPB( NITFImage *psImage, NITFICHIPBInfo * );
 
-double CPL_DLL 
+double CPL_DLL
         NITF_WGS84_Geocentric_Latitude_To_Geodetic_Latitude( double dfLat );
 
 
@@ -410,4 +411,3 @@ CPLXMLNode* NITFCreateXMLTre(NITFFile* psFile,
 CPL_C_END
 
 #endif /* ndef NITFLIB_H_INCLUDED */
-
