@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: nitfrasterband.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: nitfrasterband.cpp 33808 2016-03-29 21:15:28Z goatbar $
  *
  * Project:  NITF Read/Write Translator
  * Purpose:  NITFRasterBand (and related proxy band) implementations.
@@ -35,7 +35,7 @@
 #include "cpl_string.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: nitfrasterband.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: nitfrasterband.cpp 33808 2016-03-29 21:15:28Z goatbar $");
 
 /************************************************************************/
 /*                       NITFMakeColorTable()                           */
@@ -47,14 +47,11 @@ static GDALColorTable* NITFMakeColorTable(NITFImage* psImage, NITFBandInfo *psBa
 
     if( psBandInfo->nSignificantLUTEntries > 0 )
     {
-        int  iColor;
-
         poColorTable = new GDALColorTable();
 
-        for( iColor = 0; iColor < psBandInfo->nSignificantLUTEntries; iColor++)
+        for( int iColor = 0; iColor < psBandInfo->nSignificantLUTEntries; iColor++)
         {
             GDALColorEntry sEntry;
-
             sEntry.c1 = psBandInfo->pabyLUT[  0 + iColor];
             sEntry.c2 = psBandInfo->pabyLUT[256 + iColor];
             sEntry.c3 = psBandInfo->pabyLUT[512 + iColor];
@@ -76,10 +73,9 @@ static GDALColorTable* NITFMakeColorTable(NITFImage* psImage, NITFBandInfo *psBa
 /* -------------------------------------------------------------------- */
     if( poColorTable == NULL && psImage->nBitsPerSample == 1 )
     {
-        GDALColorEntry sEntry;
-
         poColorTable = new GDALColorTable();
 
+        GDALColorEntry sEntry;
         sEntry.c1 = 0;
         sEntry.c2 = 0;
         sEntry.c3 = 0;
@@ -92,7 +88,7 @@ static GDALColorTable* NITFMakeColorTable(NITFImage* psImage, NITFBandInfo *psBa
         sEntry.c4 = 255;
         poColorTable->SetColorEntry( 1, &sEntry );
     }
-    
+
     return poColorTable;
 }
 
@@ -113,41 +109,6 @@ NITFProxyPamRasterBand::~NITFProxyPamRasterBand()
 }
 
 
-#define RB_PROXY_METHOD_WITH_RET(retType, retErrValue, methodName, argList, argParams) \
-retType NITFProxyPamRasterBand::methodName argList \
-{ \
-    retType ret; \
-    GDALRasterBand* _poSrcBand = RefUnderlyingRasterBand(); \
-    if (_poSrcBand) \
-    { \
-        ret = _poSrcBand->methodName argParams; \
-        UnrefUnderlyingRasterBand(_poSrcBand); \
-    } \
-    else \
-    { \
-        ret = retErrValue; \
-    } \
-    return ret; \
-}
-
-
-#define RB_PROXY_METHOD_WITH_RET_AND_CALL_OTHER_METHOD(retType, retErrValue, methodName, underlyingMethodName, argList, argParams) \
-retType NITFProxyPamRasterBand::methodName argList \
-{ \
-    retType ret; \
-    GDALRasterBand* _poSrcBand = RefUnderlyingRasterBand(); \
-    if (_poSrcBand) \
-    { \
-        ret = _poSrcBand->underlyingMethodName argParams; \
-        UnrefUnderlyingRasterBand(_poSrcBand); \
-    } \
-    else \
-    { \
-        ret = retErrValue; \
-    } \
-    return ret; \
-}
-
 char      **NITFProxyPamRasterBand::GetMetadata( const char * pszDomain  )
 {
     GDALRasterBand* _poSrcBand = RefUnderlyingRasterBand();
@@ -195,8 +156,6 @@ CPLErr NITFProxyPamRasterBand::GetStatistics( int bApproxOK, int bForce,
                                       double *pdfMin, double *pdfMax,
                                       double *pdfMean, double *pdfStdDev )
 {
-    CPLErr ret;
-
 /* -------------------------------------------------------------------- */
 /*      Do we already have metadata items for the requested values?     */
 /* -------------------------------------------------------------------- */
@@ -213,8 +172,9 @@ CPLErr NITFProxyPamRasterBand::GetStatistics( int bApproxOK, int bForce,
     GDALRasterBand* _poSrcBand = RefUnderlyingRasterBand();
     if (_poSrcBand)
     {
-        ret = _poSrcBand->GetStatistics( bApproxOK, bForce,
-                                         pdfMin, pdfMax, pdfMean, pdfStdDev);
+        CPLErr ret = _poSrcBand->GetStatistics( bApproxOK, bForce,
+                                                pdfMin, pdfMax, pdfMean,
+                                                pdfStdDev);
         if (ret == CE_None)
         {
             /* Report underlying statistics at PAM level */
@@ -228,12 +188,10 @@ CPLErr NITFProxyPamRasterBand::GetStatistics( int bApproxOK, int bForce,
                             _poSrcBand->GetMetadataItem("STATISTICS_STDDEV"));
         }
         UnrefUnderlyingRasterBand(_poSrcBand);
+        return ret;
     }
-    else
-    {
-        ret = CE_Failure;
-    }
-    return ret;
+
+    return CE_Failure;
 }
 
 CPLErr NITFProxyPamRasterBand::ComputeStatistics( int bApproxOK,
@@ -241,13 +199,12 @@ CPLErr NITFProxyPamRasterBand::ComputeStatistics( int bApproxOK,
                                         double *pdfMean, double *pdfStdDev,
                                         GDALProgressFunc pfn, void *pProgressData )
 {
-    CPLErr ret;
     GDALRasterBand* _poSrcBand = RefUnderlyingRasterBand();
     if (_poSrcBand)
     {
-        ret = _poSrcBand->ComputeStatistics( bApproxOK, pdfMin, pdfMax,
-                                             pdfMean, pdfStdDev,
-                                             pfn, pProgressData);
+        CPLErr ret = _poSrcBand->ComputeStatistics( bApproxOK, pdfMin, pdfMax,
+                                                    pdfMean, pdfStdDev,
+                                                    pfn, pProgressData);
         if (ret == CE_None)
         {
             /* Report underlying statistics at PAM level */
@@ -261,12 +218,10 @@ CPLErr NITFProxyPamRasterBand::ComputeStatistics( int bApproxOK,
                             _poSrcBand->GetMetadataItem("STATISTICS_STDDEV"));
         }
         UnrefUnderlyingRasterBand(_poSrcBand);
+        return ret;
     }
-    else
-    {
-        ret = CE_Failure;
-    }
-    return ret;
+
+    return CE_Failure;
 }
 
 
@@ -298,6 +253,23 @@ RB_PROXY_METHOD_GET_DBL_WITH_SUCCESS(GetNoDataValue)
 RB_PROXY_METHOD_GET_DBL_WITH_SUCCESS(GetMinimum)
 RB_PROXY_METHOD_GET_DBL_WITH_SUCCESS(GetMaximum)
 
+#define RB_PROXY_METHOD_WITH_RET_AND_CALL_OTHER_METHOD(retType, retErrValue, methodName, underlyingMethodName, argList, argParams) \
+retType NITFProxyPamRasterBand::methodName argList \
+{ \
+    retType ret; \
+    GDALRasterBand* _poSrcBand = RefUnderlyingRasterBand(); \
+    if (_poSrcBand) \
+    { \
+        ret = _poSrcBand->underlyingMethodName argParams; \
+        UnrefUnderlyingRasterBand(_poSrcBand); \
+    } \
+    else \
+    { \
+        ret = retErrValue; \
+    } \
+    return ret; \
+}
+
 RB_PROXY_METHOD_WITH_RET_AND_CALL_OTHER_METHOD(CPLErr, CE_Failure, IReadBlock, ReadBlock,
                                 ( int nXBlockOff, int nYBlockOff, void* pImage),
                                 (nXBlockOff, nYBlockOff, pImage) )
@@ -309,11 +281,28 @@ RB_PROXY_METHOD_WITH_RET_AND_CALL_OTHER_METHOD(CPLErr, CE_Failure, IRasterIO, Ra
                                 int nXOff, int nYOff, int nXSize, int nYSize,
                                 void * pData, int nBufXSize, int nBufYSize,
                                 GDALDataType eBufType,
-                                int nPixelSpace,
-                                int nLineSpace ),
+                                GSpacing nPixelSpace, GSpacing nLineSpace,
+                                GDALRasterIOExtraArg* psExtraArg ),
                         (eRWFlag, nXOff, nYOff, nXSize, nYSize,
                                 pData, nBufXSize, nBufYSize, eBufType,
-                                nPixelSpace, nLineSpace ) )
+                                nPixelSpace, nLineSpace, psExtraArg ) )
+
+#define RB_PROXY_METHOD_WITH_RET(retType, retErrValue, methodName, argList, argParams) \
+retType NITFProxyPamRasterBand::methodName argList \
+{ \
+    retType ret; \
+    GDALRasterBand* _poSrcBand = RefUnderlyingRasterBand(); \
+    if (_poSrcBand) \
+    { \
+        ret = _poSrcBand->methodName argParams; \
+        UnrefUnderlyingRasterBand(_poSrcBand); \
+    } \
+    else \
+    { \
+        ret = retErrValue; \
+    } \
+    return ret; \
+}
 
 RB_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, FlushCache, (), ())
 
@@ -330,7 +319,7 @@ RB_PROXY_METHOD_WITH_RET(int, 0, HasArbitraryOverviews, (), ())
 RB_PROXY_METHOD_WITH_RET(int, 0,  GetOverviewCount, (), ())
 RB_PROXY_METHOD_WITH_RET(GDALRasterBand*, NULL,  GetOverview, (int arg1), (arg1))
 RB_PROXY_METHOD_WITH_RET(GDALRasterBand*, NULL,  GetRasterSampleOverview,
-                        (int arg1), (arg1))
+                        (GUIntBig arg1), (arg1))
 
 RB_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, BuildOverviews,
                         (const char * arg1, int arg2, int *arg3,
@@ -345,17 +334,14 @@ RB_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, AdviseRead,
 
 RB_PROXY_METHOD_WITH_RET(GDALRasterBand*, NULL, GetMaskBand, (), ())
 RB_PROXY_METHOD_WITH_RET(int, 0, GetMaskFlags, (), ())
-RB_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, CreateMaskBand, ( int nFlags ), (nFlags))
+RB_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, CreateMaskBand, ( int nFlagsIn ), (nFlagsIn))
 
 
 /************************************************************************/
 /*                 UnrefUnderlyingRasterBand()                        */
 /************************************************************************/
 
-void NITFProxyPamRasterBand::UnrefUnderlyingRasterBand(CPL_UNUSED GDALRasterBand* poUnderlyingRasterBand)
-{
-}
-
+void NITFProxyPamRasterBand::UnrefUnderlyingRasterBand(CPL_UNUSED GDALRasterBand* poUnderlyingRasterBand) {}
 
 /************************************************************************/
 /* ==================================================================== */
@@ -367,38 +353,38 @@ void NITFProxyPamRasterBand::UnrefUnderlyingRasterBand(CPL_UNUSED GDALRasterBand
 /*                           NITFRasterBand()                           */
 /************************************************************************/
 
-NITFRasterBand::NITFRasterBand( NITFDataset *poDS, int nBand )
+NITFRasterBand::NITFRasterBand( NITFDataset *poDSIn, int nBandIn )
 
 {
-    NITFBandInfo *psBandInfo = poDS->psImage->pasBandInfo + nBand - 1;
+    NITFBandInfo *psBandInfo = poDSIn->psImage->pasBandInfo + nBandIn - 1;
 
-    this->poDS = poDS;
-    this->nBand = nBand;
+    this->poDS = poDSIn;
+    this->nBand = nBandIn;
 
-    this->eAccess = poDS->eAccess;
-    this->psImage = poDS->psImage;
+    eAccess = poDSIn->eAccess;
+    psImage = poDSIn->psImage;
 
 /* -------------------------------------------------------------------- */
 /*      Translate data type(s).                                         */
 /* -------------------------------------------------------------------- */
     if( psImage->nBitsPerSample <= 8 )
         eDataType = GDT_Byte;
-    else if( psImage->nBitsPerSample == 16 
+    else if( psImage->nBitsPerSample == 16
              && EQUAL(psImage->szPVType,"SI") )
         eDataType = GDT_Int16;
     else if( psImage->nBitsPerSample == 16 )
         eDataType = GDT_UInt16;
     else if( psImage->nBitsPerSample == 12 )
         eDataType = GDT_UInt16;
-    else if( psImage->nBitsPerSample == 32 
+    else if( psImage->nBitsPerSample == 32
              && EQUAL(psImage->szPVType,"SI") )
         eDataType = GDT_Int32;
-    else if( psImage->nBitsPerSample == 32 
+    else if( psImage->nBitsPerSample == 32
              && EQUAL(psImage->szPVType,"R") )
         eDataType = GDT_Float32;
     else if( psImage->nBitsPerSample == 32 )
         eDataType = GDT_UInt32;
-    else if( psImage->nBitsPerSample == 64 
+    else if( psImage->nBitsPerSample == 64
              && EQUAL(psImage->szPVType,"R") )
         eDataType = GDT_Float64;
     else if( psImage->nBitsPerSample == 64
@@ -407,7 +393,7 @@ NITFRasterBand::NITFRasterBand( NITFDataset *poDS, int nBand )
     /* ERO : note I'm not sure if CFloat64 can be transmitted as NBPP is only 2 characters */
     else
     {
-        int bOpenUnderlyingDS = CSLTestBoolean(
+        int bOpenUnderlyingDS = CPLTestBool(
                 CPLGetConfigOption("NITF_OPEN_UNDERLYING_DS", "YES"));
         if (!bOpenUnderlyingDS && psImage->nBitsPerSample > 8 && psImage->nBitsPerSample < 16)
         {
@@ -429,7 +415,7 @@ NITFRasterBand::NITFRasterBand( NITFDataset *poDS, int nBand )
 /*      Work out block size. If the image is all one big block we       */
 /*      handle via the scanline access API.                             */
 /* -------------------------------------------------------------------- */
-    if( psImage->nBlocksPerRow == 1 
+    if( psImage->nBlocksPerRow == 1
         && psImage->nBlocksPerColumn == 1
         && psImage->nBitsPerSample >= 8
         && EQUAL(psImage->szIC,"NC") )
@@ -451,7 +437,7 @@ NITFRasterBand::NITFRasterBand( NITFDataset *poDS, int nBand )
     poColorTable = NITFMakeColorTable(psImage,
                                       psBandInfo);
 
-    if( psImage->nBitsPerSample == 1 
+    if( psImage->nBitsPerSample == 1
     ||  psImage->nBitsPerSample == 3
     ||  psImage->nBitsPerSample == 5
     ||  psImage->nBitsPerSample == 6
@@ -459,12 +445,23 @@ NITFRasterBand::NITFRasterBand( NITFDataset *poDS, int nBand )
     ||  psImage->nBitsPerSample == 12 )
         SetMetadataItem( "NBITS", CPLString().Printf("%d", psImage->nBitsPerSample), "IMAGE_STRUCTURE" );
 
-    pUnpackData = 0;
+    pUnpackData = NULL;
     if (psImage->nBitsPerSample == 3
     ||  psImage->nBitsPerSample == 5
     ||  psImage->nBitsPerSample == 6
     ||  psImage->nBitsPerSample == 7)
-      pUnpackData = new GByte[((nBlockXSize*nBlockYSize+7)/8)*8];
+    {
+        if( nBlockXSize > (INT_MAX - 7) / nBlockYSize )
+        {
+            eDataType = GDT_Unknown;
+        }
+        else
+        {
+            pUnpackData = static_cast<GByte*>(VSI_MALLOC_VERBOSE(((nBlockXSize*nBlockYSize+7)/8)*8));
+            if( pUnpackData == NULL )
+                eDataType = GDT_Unknown;
+        }
+    }
 }
 
 /************************************************************************/
@@ -477,7 +474,7 @@ NITFRasterBand::~NITFRasterBand()
     if( poColorTable != NULL )
         delete poColorTable;
 
-    delete[] pUnpackData;
+    VSIFree(pUnpackData);
 }
 
 /************************************************************************/
@@ -488,8 +485,7 @@ CPLErr NITFRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                                    void * pImage )
 
 {
-    int  nBlockResult;
-    NITFDataset *poGDS = (NITFDataset *) poDS;
+    NITFDataset *poGDS = reinterpret_cast<NITFDataset *>( poDS );
 
 /* -------------------------------------------------------------------- */
 /*      Special case for JPEG blocks.                                   */
@@ -497,14 +493,15 @@ CPLErr NITFRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
     if( EQUAL(psImage->szIC,"C3") || EQUAL(psImage->szIC,"M3") )
     {
         CPLErr eErr = poGDS->ReadJPEGBlock( nBlockXOff, nBlockYOff );
-        int nBlockBandSize = psImage->nBlockWidth*psImage->nBlockHeight*
-                             (GDALGetDataTypeSize(eDataType)/8);
+        const int nBlockBandSize =
+            psImage->nBlockWidth * psImage->nBlockHeight *
+            GDALGetDataTypeSizeBytes(eDataType);
 
         if( eErr != CE_None )
             return eErr;
 
-        memcpy( pImage, 
-                poGDS->pabyJPEGBlock + (nBand - 1) * nBlockBandSize, 
+        memcpy( pImage,
+                poGDS->pabyJPEGBlock + (nBand - 1) * nBlockBandSize,
                 nBlockBandSize );
 
         return eErr;
@@ -513,21 +510,23 @@ CPLErr NITFRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 /* -------------------------------------------------------------------- */
 /*      Read the line/block                                             */
 /* -------------------------------------------------------------------- */
+    int nBlockResult;
+
     if( bScanlineAccess )
     {
-        nBlockResult = 
+        nBlockResult =
             NITFReadImageLine(psImage, nBlockYOff, nBand, pImage);
     }
     else
     {
-        nBlockResult = 
+        nBlockResult =
             NITFReadImageBlock(psImage, nBlockXOff, nBlockYOff, nBand, pImage);
     }
 
     if( nBlockResult == BLKREAD_OK )
     {
         if( psImage->nBitsPerSample % 8 )
-            Unpack((GByte*)pImage);
+            Unpack( reinterpret_cast<GByte *>( pImage ) );
 
         return CE_None;
     }
@@ -541,10 +540,10 @@ CPLErr NITFRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 /*      8bit.                                                           */
 /* -------------------------------------------------------------------- */
     if( psImage->bNoDataSet )
-        memset( pImage, psImage->nNoDataValue, 
+        memset( pImage, psImage->nNoDataValue,
                 psImage->nWordSize*psImage->nBlockWidth*psImage->nBlockHeight);
     else
-        memset( pImage, 0, 
+        memset( pImage, 0,
                 psImage->nWordSize*psImage->nBlockWidth*psImage->nBlockHeight);
 
     return CE_None;
@@ -556,28 +555,28 @@ CPLErr NITFRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 
 CPLErr NITFRasterBand::IWriteBlock( int nBlockXOff, int nBlockYOff,
                                     void * pImage )
-    
-{
-    int  nBlockResult;
 
+{
 /* -------------------------------------------------------------------- */
 /*      Write the line/block                                            */
 /* -------------------------------------------------------------------- */
+    int  nBlockResult;
+
     if( bScanlineAccess )
     {
-        nBlockResult = 
+        nBlockResult =
             NITFWriteImageLine(psImage, nBlockYOff, nBand, pImage);
     }
     else
     {
-        nBlockResult = 
+        nBlockResult =
             NITFWriteImageBlock(psImage, nBlockXOff, nBlockYOff, nBand,pImage);
     }
 
     if( nBlockResult == BLKREAD_OK )
         return CE_None;
-    else
-        return CE_Failure;
+
+    return CE_Failure;
 }
 
 /************************************************************************/
@@ -592,8 +591,8 @@ double NITFRasterBand::GetNoDataValue( int *pbSuccess )
 
     if( psImage->bNoDataSet )
         return psImage->nNoDataValue;
-    else
-        return GDALPamRasterBand::GetNoDataValue( pbSuccess );
+
+    return GDALPamRasterBand::GetNoDataValue( pbSuccess );
 }
 
 /************************************************************************/
@@ -607,7 +606,7 @@ GDALColorInterp NITFRasterBand::GetColorInterpretation()
 
     if( poColorTable != NULL )
         return GCI_PaletteIndex;
-    
+
     if( EQUAL(psBandInfo->szIREPBAND,"R") )
         return GCI_RedBand;
     if( EQUAL(psBandInfo->szIREPBAND,"G") )
@@ -630,14 +629,12 @@ GDALColorInterp NITFRasterBand::GetColorInterpretation()
 /*                     NITFSetColorInterpretation()                     */
 /************************************************************************/
 
-CPLErr NITFSetColorInterpretation( NITFImage *psImage, 
+CPLErr NITFSetColorInterpretation( NITFImage *psImage,
                                    int nBand,
                                    GDALColorInterp eInterp )
-    
+
 {
-    NITFBandInfo *psBandInfo = psImage->pasBandInfo + nBand - 1;
     const char *pszREP = NULL;
-    GUIntBig nOffset;
 
     if( eInterp == GCI_RedBand )
         pszREP = "R";
@@ -658,7 +655,7 @@ CPLErr NITFSetColorInterpretation( NITFImage *psImage,
 
     if( pszREP == NULL )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "Requested color interpretation (%s) not supported in NITF.",
                   GDALGetColorInterpretationName( eInterp ) );
         return CE_Failure;
@@ -667,30 +664,32 @@ CPLErr NITFSetColorInterpretation( NITFImage *psImage,
 /* -------------------------------------------------------------------- */
 /*      Where does this go in the file?                                 */
 /* -------------------------------------------------------------------- */
+    NITFBandInfo *psBandInfo = psImage->pasBandInfo + nBand - 1;
     strcpy( psBandInfo->szIREPBAND, pszREP );
-    nOffset = NITFIHFieldOffset( psImage, "IREPBAND" );
+    GUIntBig nOffset = NITFIHFieldOffset( psImage, "IREPBAND" );
 
     if( nOffset != 0 )
         nOffset += (nBand - 1) * 13;
-    
+
 /* -------------------------------------------------------------------- */
 /*      write it (space padded).                                        */
 /* -------------------------------------------------------------------- */
     char szPadded[4];
     strcpy( szPadded, pszREP );
     strcat( szPadded, " " );
-    
+
     if( nOffset != 0 )
     {
-        if( VSIFSeekL( psImage->psFile->fp, nOffset, SEEK_SET ) != 0 
-            || VSIFWriteL( (void *) szPadded, 1, 2, psImage->psFile->fp ) != 2 )
+        if( VSIFSeekL( psImage->psFile->fp, nOffset, SEEK_SET ) != 0
+            || VSIFWriteL( reinterpret_cast<void *>( szPadded ), 1, 2,
+                           psImage->psFile->fp ) != 2 )
         {
-            CPLError( CE_Failure, CPLE_AppDefined, 
+            CPLError( CE_Failure, CPLE_AppDefined,
                       "IO failure writing new IREPBAND value to NITF file." );
             return CE_Failure;
         }
     }
-    
+
     return CE_None;
 }
 
@@ -721,19 +720,18 @@ GDALColorTable *NITFRasterBand::GetColorTable()
 CPLErr NITFRasterBand::SetColorTable( GDALColorTable *poNewCT )
 
 {
-    NITFDataset *poGDS = (NITFDataset *) poDS;
+    NITFDataset *poGDS = reinterpret_cast<NITFDataset *>( poDS );
     if( poGDS->bInLoadXML )
         return GDALPamRasterBand::SetColorTable(poNewCT);
-        
+
     if( poNewCT == NULL )
         return CE_Failure;
 
     GByte abyNITFLUT[768];
-    int   i;
-    int   nCount = MIN(256,poNewCT->GetColorEntryCount());
-
     memset( abyNITFLUT, 0, 768 );
-    for( i = 0; i < nCount; i++ )
+
+    const int nCount = MIN(256,poNewCT->GetColorEntryCount());
+    for( int i = 0; i < nCount; i++ )
     {
         GDALColorEntry sEntry;
 
@@ -745,8 +743,8 @@ CPLErr NITFRasterBand::SetColorTable( GDALColorTable *poNewCT )
 
     if( NITFWriteLUT( psImage, nBand, nCount, abyNITFLUT ) )
         return CE_None;
-    else
-        return CE_Failure;
+
+    return CE_Failure;
 }
 
 /************************************************************************/
@@ -755,9 +753,7 @@ CPLErr NITFRasterBand::SetColorTable( GDALColorTable *poNewCT )
 
 void NITFRasterBand::Unpack( GByte* pData )
 {
-  long n = nBlockXSize*nBlockYSize;
-  long i;
-  long k;
+  const int n = nBlockXSize*nBlockYSize;
 
   GByte abyTempData[7] = {0, 0, 0, 0, 0, 0, 0};
   const GByte* pDataSrc = pData;
@@ -773,33 +769,36 @@ void NITFRasterBand::Unpack( GByte* pData )
     case 1:
     {
       // unpack 1-bit in-place in reverse
-      for (i = n; --i >= 0; )
+      // DANGER: Non-standard decrement of counter in the test section of for.
+      for( int i = n; --i >= 0; )
         pData[i] = (pData[i>>3] & (0x80 >> (i&7))) != 0;
-       
+
       break;
     }
     case 2:
     {
       static const int s_Shift2[] = {6, 4, 2, 0};
       // unpack 2-bit in-place in reverse
-      for (i = n; --i >= 0; )
+      // DANGER: Non-standard decrement of counter in the test section of for.
+      for (int i = n; --i >= 0; )
         pData[i] = (pData[i>>2] >> (GByte)s_Shift2[i&3]) & 0x03;
-       
+
       break;
     }
     case 4:
     {
       static const int s_Shift4[] = {4, 0};
       // unpack 4-bit in-place in reverse
-      for (i = n; --i >= 0; )
+      // DANGER: Non-standard decrement of counter in the test section of for.
+      for( int i = n; --i >= 0; )
         pData[i] = (pData[i>>1] >> (GByte)s_Shift4[i&1]) & 0x0f;
-       
+
       break;
     }
     case 3:
     {
       // unpacks 8 pixels (3 bytes) at time
-      for (i = 0, k = 0; i < n; i += 8, k += 3)
+      for( int i = 0, k = 0; i < n; i += 8, k += 3 )
       {
         pUnpackData[i+0] = ((pDataSrc[k+0] >> 5));
         pUnpackData[i+1] = ((pDataSrc[k+0] >> 2) & 0x07);
@@ -817,7 +816,7 @@ void NITFRasterBand::Unpack( GByte* pData )
     case 5:
     {
       // unpacks 8 pixels (5 bytes) at time
-      for (i = 0, k = 0; i < n; i += 8, k += 5)
+      for( int i = 0, k = 0; i < n; i += 8, k += 5 )
       {
         pUnpackData[i+0] = ((pDataSrc[k+0] >> 3));
         pUnpackData[i+1] = ((pDataSrc[k+0] << 2) & 0x1f) | (pDataSrc[k+1] >> 6);
@@ -835,7 +834,7 @@ void NITFRasterBand::Unpack( GByte* pData )
     case 6:
     {
       // unpacks 4 pixels (3 bytes) at time
-      for (i = 0, k = 0; i < n; i += 4, k += 3)
+      for( int i = 0, k = 0; i < n; i += 4, k += 3 )
       {
         pUnpackData[i+0] = ((pDataSrc[k+0] >> 2));
         pUnpackData[i+1] = ((pDataSrc[k+0] << 4) & 0x3f) | (pDataSrc[k+1] >> 4);
@@ -849,7 +848,7 @@ void NITFRasterBand::Unpack( GByte* pData )
     case 7:
     {
       // unpacks 8 pixels (7 bytes) at time
-      for (i = 0, k = 0; i < n; i += 8, k += 7)
+      for( int i = 0, k = 0; i < n; i += 8, k += 7 )
       {
         pUnpackData[i+0] = ((pDataSrc[k+0] >> 1));
         pUnpackData[i+1] = ((pDataSrc[k+0] << 6) & 0x7f) | (pDataSrc[k+1] >> 2);
@@ -866,11 +865,12 @@ void NITFRasterBand::Unpack( GByte* pData )
     }
     case 12:
     {
-      GByte*   pabyImage = (GByte  *)pData;
-      GUInt16* panImage  = (GUInt16*)pData;
-      for (i = n; --i >= 0; )
+      GByte *pabyImage = reinterpret_cast<GByte *>( pData );
+      GUInt16 *panImage  = reinterpret_cast<GUInt16 *>( pData );
+      // DANGER: Non-standard decrement of counter in the test section of for.
+      for( int i = n; --i >= 0; )
       {
-        long iOffset = i*3 / 2;
+        const long iOffset = i*3 / 2;
         if (i % 2 == 0)
           panImage[i] = pabyImage[iOffset] + (pabyImage[iOffset+1] & 0xf0) * 16;
         else
@@ -894,16 +894,16 @@ void NITFRasterBand::Unpack( GByte* pData )
 /*                      NITFWrapperRasterBand()                         */
 /************************************************************************/
 
-NITFWrapperRasterBand::NITFWrapperRasterBand( NITFDataset * poDS,
-                                              GDALRasterBand* poBaseBand,
-                                              int nBand)
+NITFWrapperRasterBand::NITFWrapperRasterBand( NITFDataset * poDSIn,
+                                              GDALRasterBand* poBaseBandIn,
+                                              int nBandIn) :
+    poColorTable(NULL)
 {
-    this->poDS = poDS;
-    this->nBand = nBand;
-    this->poBaseBand = poBaseBand;
+    this->poDS = poDSIn;
+    this->nBand = nBandIn;
+    this->poBaseBand = poBaseBandIn;
     eDataType = poBaseBand->GetRasterDataType();
     poBaseBand->GetBlockSize(&nBlockXSize, &nBlockYSize);
-    poColorTable = NULL;
     eInterp = poBaseBand->GetColorInterpretation();
     bIsJPEG = poBaseBand->GetDataset() != NULL &&
               poBaseBand->GetDataset()->GetDriver() != NULL &&
@@ -945,7 +945,7 @@ GDALColorTable *NITFWrapperRasterBand::GetColorTable()
 
 void NITFWrapperRasterBand::SetColorTableFromNITFBandInfo()
 {
-    NITFDataset* poGDS = (NITFDataset* )poDS;
+    NITFDataset* poGDS = reinterpret_cast<NITFDataset *>( poDS );
     poColorTable = NITFMakeColorTable(poGDS->psImage,
                                       poGDS->psImage->pasBandInfo + nBand - 1);
 }
@@ -963,9 +963,9 @@ GDALColorInterp NITFWrapperRasterBand::GetColorInterpretation()
 /*                        SetColorInterpretation()                      */
 /************************************************************************/
 
-CPLErr NITFWrapperRasterBand::SetColorInterpretation( GDALColorInterp eInterp)
+CPLErr NITFWrapperRasterBand::SetColorInterpretation( GDALColorInterp eInterpIn)
 {
-    this->eInterp = eInterp;
+    this->eInterp = eInterpIn;
     if( poBaseBand->GetDataset() != NULL &&
         poBaseBand->GetDataset()->GetDriver() != NULL &&
         EQUAL(poBaseBand->GetDataset()->GetDriver()->GetDescription(), "JP2ECW") )
@@ -981,13 +981,13 @@ int NITFWrapperRasterBand::GetOverviewCount()
 {
     if( bIsJPEG )
     {
-        if( ((NITFDataset*)poDS)->ExposeUnderlyingJPEGDatasetOverviews() )
+        if( (reinterpret_cast<NITFDataset *>( poDS) )->ExposeUnderlyingJPEGDatasetOverviews() )
             return NITFProxyPamRasterBand::GetOverviewCount();
-        else
-            return GDALPamRasterBand::GetOverviewCount();
+
+        return GDALPamRasterBand::GetOverviewCount();
     }
-    else
-        return NITFProxyPamRasterBand::GetOverviewCount();
+
+    return NITFProxyPamRasterBand::GetOverviewCount();
 }
 
 /************************************************************************/
@@ -998,11 +998,11 @@ GDALRasterBand * NITFWrapperRasterBand::GetOverview(int iOverview)
 {
     if( bIsJPEG )
     {
-        if( ((NITFDataset*)poDS)->ExposeUnderlyingJPEGDatasetOverviews() )
+        if( (reinterpret_cast<NITFDataset *>( poDS ) )->ExposeUnderlyingJPEGDatasetOverviews() )
             return NITFProxyPamRasterBand::GetOverview(iOverview);
-        else
-            return GDALPamRasterBand::GetOverview(iOverview);
+
+        return GDALPamRasterBand::GetOverview(iOverview);
     }
-    else
-        return NITFProxyPamRasterBand::GetOverview(iOverview);
+
+    return NITFProxyPamRasterBand::GetOverview(iOverview);
 }
