@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: grcdataset.cpp 28503 2015-02-16 16:56:48Z rouault $
+ * $Id: grcdataset.cpp 33757 2016-03-20 20:22:33Z goatbar $
  *
  * Project:  GRC Reader
  * Purpose:  GDAL driver for Northwood Classified Format
@@ -28,20 +28,16 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
+#include "gdal_frmts.h"
 #include "gdal_pam.h"
 #include "northwood.h"
 
-#ifdef OGR_ENABLED
 #ifdef MSVC
 #include "..\..\ogr\ogrsf_frmts\mitab\mitab.h"
 #else
 #include "../../ogr/ogrsf_frmts/mitab/mitab.h"
 #endif
-#endif
 
-
-CPL_C_START void GDALRegister_NWT_GRC( void );
-CPL_C_END
 /************************************************************************/
 /* ==================================================================== */
 /*                             NWT_GRCDataset                           */
@@ -102,17 +98,17 @@ class NWT_GRCRasterBand : public GDALPamRasterBand
 /*                           NWT_GRCRasterBand()                        */
 /************************************************************************/
 
-NWT_GRCRasterBand::NWT_GRCRasterBand( NWT_GRCDataset * poDS, int nBand )
+NWT_GRCRasterBand::NWT_GRCRasterBand( NWT_GRCDataset * poDSIn, int nBandIn )
 {
-    this->poDS = poDS;
-    this->nBand = nBand;
-    NWT_GRCDataset *poGDS =( NWT_GRCDataset * ) poDS;
+    this->poDS = poDSIn;
+    this->nBand = nBandIn;
+    NWT_GRCDataset *poGDS = reinterpret_cast<NWT_GRCDataset *>( poDS );
 
     if( poGDS->pGrd->nBitsPerPixel == 8 )
         eDataType = GDT_Byte;
     else if( poGDS->pGrd->nBitsPerPixel == 16 )
         eDataType = GDT_UInt16;
-    else if( poGDS->pGrd->nBitsPerPixel == 32 )
+    else /* if( poGDS->pGrd->nBitsPerPixel == 32 ) */
         eDataType = GDT_UInt32;        // this would be funny
 
     nBlockXSize = poDS->GetRasterXSize();
@@ -130,25 +126,26 @@ NWT_GRCRasterBand::NWT_GRCRasterBand( NWT_GRCDataset * poDS, int nBand )
 
     poGDS->poColorTable->SetColorEntry( 0, &oEntry );
 
-    int i;
-    for( i=0; i < (int) poGDS->pGrd->stClassDict->nNumClassifiedItems; i++ )
+    for( int i = 0;
+         i < static_cast<int>( poGDS->pGrd->stClassDict->nNumClassifiedItems );
+         i++ )
     {
         oEntry.c1 = poGDS->pGrd->stClassDict->stClassifedItem[i]->r;
         oEntry.c2 = poGDS->pGrd->stClassDict->stClassifedItem[i]->g;
         oEntry.c3 = poGDS->pGrd->stClassDict->stClassifedItem[i]->b;
         oEntry.c4 = 0;            // alpha 0 = solid
 
-        poGDS->poColorTable->SetColorEntry( poDS->pGrd->
+        poGDS->poColorTable->SetColorEntry( poGDS->pGrd->
                                           stClassDict->stClassifedItem[i]->
                                           usPixVal, &oEntry );
     }
 
     // find the max value used in the grc
     int maxValue = 0;
-    for( i=0; i < (int) poDS->pGrd->stClassDict->nNumClassifiedItems; i++ )
+    for( int i=0; i < static_cast<int>( poGDS->pGrd->stClassDict->nNumClassifiedItems ); i++ )
     {
-        if( poDS->pGrd->stClassDict->stClassifedItem[i]->usPixVal > maxValue )
-            maxValue = poDS->pGrd->stClassDict->stClassifedItem[i]->usPixVal;
+        if( poGDS->pGrd->stClassDict->stClassifedItem[i]->usPixVal > maxValue )
+            maxValue = poGDS->pGrd->stClassDict->stClassifedItem[i]->usPixVal;
     }
 
     // load a value for the null value
@@ -158,42 +155,42 @@ NWT_GRCRasterBand::NWT_GRCRasterBand( NWT_GRCDataset * poDS, int nBand )
     // are not defined
     for( int val = 1; val <= maxValue; val++ )
     {
-        int i;
-        // loop throught the GRC dictionary to see if the value is defined
-        for( i=0; i < (int) poDS->pGrd->stClassDict->nNumClassifiedItems; i++ )
+        int i = 0;
+        // Loop through the GRC dictionary to see if the value is defined.
+        for( ;
+             i < static_cast<int>( poGDS->pGrd->stClassDict->nNumClassifiedItems );
+             i++ )
         {
-            if( (int) poDS->pGrd->stClassDict->stClassifedItem[i]->usPixVal ==
+            if( static_cast<int>( poGDS->pGrd->stClassDict->stClassifedItem[i]->usPixVal ) ==
                 val )
             {
                 poGDS->papszCategories =
                     CSLAddString( poGDS->papszCategories,
-                                    poDS->pGrd->stClassDict->
+                                    poGDS->pGrd->stClassDict->
                                     stClassifedItem[i]->szClassName );
                 break;
             }
         }
-        if( i >= (int) poDS->pGrd->stClassDict->nNumClassifiedItems )
+        if( i >= static_cast<int>( poGDS->pGrd->stClassDict->nNumClassifiedItems ) )
             poGDS->papszCategories = CSLAddString( poGDS->papszCategories, "" );
 
     }
 }
 
-NWT_GRCRasterBand::~NWT_GRCRasterBand()
-{
-}
+NWT_GRCRasterBand::~NWT_GRCRasterBand() {}
 
 double NWT_GRCRasterBand::GetNoDataValue( int *pbSuccess )
 {
     if( pbSuccess != NULL )
         *pbSuccess = TRUE;
 
-    return 0;                        //Northwood grid 0 is always null
+    return 0.0;  // Northwood grid 0 is always null.
 }
 
 // return an array of null terminated strings for the class names
 char **NWT_GRCRasterBand::GetCategoryNames()
 {
-    NWT_GRCDataset *poGDS = (NWT_GRCDataset *) poDS;
+    NWT_GRCDataset *poGDS = reinterpret_cast<NWT_GRCDataset *>( poDS );
 
     return poGDS->papszCategories;
 }
@@ -201,7 +198,7 @@ char **NWT_GRCRasterBand::GetCategoryNames()
 // return the color table
 GDALColorTable *NWT_GRCRasterBand::GetColorTable()
 {
-    NWT_GRCDataset *poGDS = (NWT_GRCDataset *) poDS;
+    NWT_GRCDataset *poGDS = reinterpret_cast<NWT_GRCDataset *>( poDS );
 
     return poGDS->poColorTable;
 }
@@ -210,23 +207,28 @@ GDALColorInterp NWT_GRCRasterBand::GetColorInterpretation()
 {
     if( nBand == 1 )
         return GCI_PaletteIndex;
-    else
-        return GCI_Undefined;
+
+    return GCI_Undefined;
 }
 
 /************************************************************************/
 /*                             IReadBlock()                             */
 /************************************************************************/
-CPLErr NWT_GRCRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYOff,
-                                        void *pImage )
+CPLErr NWT_GRCRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
+                                      int nBlockYOff,
+                                      void *pImage )
 {
-    NWT_GRCDataset *poGDS =(NWT_GRCDataset *) poDS;
-    int nRecordSize = nBlockXSize *( poGDS->pGrd->nBitsPerPixel / 8 );
+    NWT_GRCDataset *poGDS = reinterpret_cast<NWT_GRCDataset *>( poDS );
+    const int nBytesPerPixel = poGDS->pGrd->nBitsPerPixel / 8;
+    if( nBytesPerPixel <= 0 || nBlockXSize > INT_MAX / nBytesPerPixel )
+        return CE_Failure;
+    const int nRecordSize = nBlockXSize * nBytesPerPixel;
 
     if( nBand == 1 )
     {                            //grc's are just one band of indices
         VSIFSeekL( poGDS->fp, 1024 + nRecordSize * (vsi_l_offset)nBlockYOff, SEEK_SET );
-        VSIFReadL( pImage, 1, nRecordSize, poGDS->fp );
+        if( (int)VSIFReadL( pImage, 1, nRecordSize, poGDS->fp ) != nRecordSize )
+            return CE_Failure;
     }
     else
     {
@@ -243,12 +245,10 @@ CPLErr NWT_GRCRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYOff,
 /*                          NWT_GRCDataset                              */
 /* ==================================================================== */
 /************************************************************************/
-NWT_GRCDataset::NWT_GRCDataset()
-{
-    poColorTable = NULL;
-    papszCategories = NULL;
-    pszProjection = NULL;
-}
+NWT_GRCDataset::NWT_GRCDataset() :
+    fp(NULL), pGrd(NULL), papszCategories(NULL), pszProjection(NULL),
+    poColorTable(NULL)
+{ }
 
 
 /************************************************************************/
@@ -290,18 +290,16 @@ CPLErr NWT_GRCDataset::GetGeoTransform( double *padfTransform )
 /************************************************************************/
 const char *NWT_GRCDataset::GetProjectionRef()
 {
-#ifdef OGR_ENABLED
     if (pszProjection == NULL)
     {
-        OGRSpatialReference *poSpatialRef;
-        poSpatialRef = MITABCoordSys2SpatialRef( pGrd->cMICoordSys );
+        OGRSpatialReference *poSpatialRef
+          = MITABCoordSys2SpatialRef( pGrd->cMICoordSys );
         if (poSpatialRef)
         {
             poSpatialRef->exportToWkt( &pszProjection );
             poSpatialRef->Release();
         }
     }
-#endif
     return ( (const char *) pszProjection );
 }
 
@@ -314,7 +312,7 @@ int NWT_GRCDataset::Identify( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*  Look for the header                                                 */
 /* -------------------------------------------------------------------- */
-    if( poOpenInfo->nHeaderBytes < 50 )
+    if( poOpenInfo->nHeaderBytes < 1024 )
         return FALSE;
 
     if( poOpenInfo->pabyHeader[0] != 'H' ||
@@ -339,9 +337,7 @@ GDALDataset *NWT_GRCDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
-    NWT_GRCDataset *poDS;
-
-    poDS = new NWT_GRCDataset();
+    NWT_GRCDataset *poDS = new NWT_GRCDataset();
 
     poDS->fp = VSIFOpenL(poOpenInfo->pszFilename, "rb");
     if (poDS->fp == NULL)
@@ -355,11 +351,11 @@ GDALDataset *NWT_GRCDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     VSIFSeekL( poDS->fp, 0, SEEK_SET );
     VSIFReadL( poDS->abyHeader, 1, 1024, poDS->fp );
-    poDS->pGrd = (NWT_GRID *) malloc( sizeof (NWT_GRID) );
+    poDS->pGrd = reinterpret_cast<NWT_GRID *>( malloc( sizeof (NWT_GRID) ) );
 
     poDS->pGrd->fp = poDS->fp;
 
-    if (!nwt_ParseHeader( poDS->pGrd, (char *) poDS->abyHeader ) ||
+    if (!nwt_ParseHeader( poDS->pGrd, reinterpret_cast<char *>( poDS->abyHeader ) ) ||
         !GDALCheckDatasetDimensions(poDS->pGrd->nXSide, poDS->pGrd->nYSide) ||
         poDS->pGrd->stClassDict == NULL)
     {
@@ -367,6 +363,14 @@ GDALDataset *NWT_GRCDataset::Open( GDALOpenInfo * poOpenInfo )
         return NULL;
     }
 
+    if( poDS->pGrd->nBitsPerPixel != 8 &&
+        poDS->pGrd->nBitsPerPixel != 16 &&
+        poDS->pGrd->nBitsPerPixel != 32 )
+    {
+        delete poDS;
+        return NULL;
+    }
+
     poDS->nRasterXSize = poDS->pGrd->nXSide;
     poDS->nRasterYSize = poDS->pGrd->nYSide;
 
@@ -384,7 +388,9 @@ GDALDataset *NWT_GRCDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Check for external overviews.                                   */
 /* -------------------------------------------------------------------- */
-    poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename, poOpenInfo->papszSiblingFiles );
+    poDS->oOvManager.Initialize( poDS,
+                                 poOpenInfo->pszFilename,
+                                 poOpenInfo->GetSiblingFiles() );
 
     return (poDS);
 }
@@ -394,26 +400,25 @@ GDALDataset *NWT_GRCDataset::Open( GDALOpenInfo * poOpenInfo )
 /*                          GDALRegister_GRC()                          */
 /************************************************************************/
 
-void
-GDALRegister_NWT_GRC()
+void GDALRegister_NWT_GRC()
+
 {
-    GDALDriver *poDriver;
+    if( GDALGetDriverByName( "NWT_GRC" ) != NULL )
+        return;
 
-    if( GDALGetDriverByName( "NWT_GRC" ) == NULL )
-    {
-        poDriver = new GDALDriver();
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->SetDescription( "NWT_GRC" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
-                                 "Northwood Classified Grid Format .grc/.tab");
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,
-                                 "frmt_various.html#northwood_grc" );
-        poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "grc" );
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    poDriver->SetDescription( "NWT_GRC" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
+                               "Northwood Classified Grid Format .grc/.tab");
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,
+                               "frmt_various.html#northwood_grc" );
+    poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "grc" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
-        poDriver->pfnOpen = NWT_GRCDataset::Open;
-        poDriver->pfnIdentify = NWT_GRCDataset::Identify;
+    poDriver->pfnOpen = NWT_GRCDataset::Open;
+    poDriver->pfnIdentify = NWT_GRCDataset::Identify;
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
