@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: northwood.h 21898 2011-03-06 14:27:52Z rouault $
+ * $Id: northwood.h 31749 2015-11-25 02:32:55Z goatbar $
  *
  * Project:  GRC/GRD Reader
  * Purpose:  Northwood Technologies Grid format declarations
@@ -7,6 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2007, Waypoint Information Technology
+ * Copyright (c) 2009-2011, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -38,15 +39,6 @@
 #include "cpl_vsi.h"
 #endif
 
-#ifndef MAX
-#define MAX(a, b) (a > b ? a : b)
-#endif
-
-#ifndef MIN
-#define MIN(a, b) (a < b ? a : b)
-#endif
-
-
 typedef struct
 {
     int row;
@@ -101,7 +93,7 @@ typedef struct
     char szFileName[256];
     VSILFILE *fp;
     float fVersion;
-    unsigned char cFormat;        //0x00 16 bit, 0x01 32 bit, 0x80 8 bit classifed, 0x81 16 bit classified
+    unsigned char cFormat;  // 0x00 16 bit, 0x01 32 bit, 0x80 8 bit classified, 0x81 16 bit classified
     unsigned int nBitsPerPixel;
     unsigned int nXSide;
     unsigned int nYSide;
