@@ -52,14 +52,14 @@ using namespace PCIDSK;
 /*                      CBandInterleavedChannel()                       */
 /************************************************************************/
 
-CBandInterleavedChannel::CBandInterleavedChannel( PCIDSKBuffer &image_header, 
-                                                  uint64 ih_offset, 
+CBandInterleavedChannel::CBandInterleavedChannel( PCIDSKBuffer &image_header,
+                                                  uint64 ih_offsetIn,
                                                   CPL_UNUSED PCIDSKBuffer &file_header,
                                                   int channelnum,
-                                                  CPCIDSKFile *file,
+                                                  CPCIDSKFile *fileIn,
                                                   uint64 image_offset,
-                                                  eChanType pixel_type )
-        : CPCIDSKChannel( image_header, ih_offset, file, pixel_type, channelnum)
+                                                  eChanType pixel_typeIn )
+        : CPCIDSKChannel( image_header, ih_offsetIn, fileIn, pixel_typeIn, channelnum)
 
 {
     io_handle_p = NULL;
@@ -134,7 +134,7 @@ int CBandInterleavedChannel::ReadBlock( int block_index, void *buffer,
     if( xoff < 0 || xoff + xsize > GetBlockWidth()
         || yoff < 0 || yoff + ysize > GetBlockHeight() )
     {
-        ThrowPCIDSKException( 
+        return ThrowPCIDSKException( 0,
             "Invalid window in ReadBloc(): xoff=%d,yoff=%d,xsize=%d,ysize=%d",
             xoff, yoff, xsize, ysize );
     }
@@ -210,7 +210,7 @@ int CBandInterleavedChannel::WriteBlock( int block_index, void *buffer )
     PCIDSKInterfaces *interfaces = file->GetInterfaces();
 
     if( !file->GetUpdatable() )
-        throw PCIDSKException( "File not open for update in WriteBlock()" );
+        return ThrowPCIDSKException(0, "File not open for update in WriteBlock()" );
 
     InvalidateOverviews();
 
@@ -290,13 +290,13 @@ int CBandInterleavedChannel::WriteBlock( int block_index, void *buffer )
 /************************************************************************/
 void CBandInterleavedChannel
 ::GetChanInfo( std::string &filename_ret, uint64 &image_offset, 
-               uint64 &pixel_offset, uint64 &line_offset, 
+               uint64 &pixel_offsetOut, uint64 &line_offsetOut, 
                bool &little_endian ) const
 
 {
     image_offset = start_byte;
-    pixel_offset = this->pixel_offset;
-    line_offset = this->line_offset;
+    pixel_offsetOut = this->pixel_offset;
+    line_offsetOut = this->line_offset;
     little_endian = (byte_order == 'S');
 
 /* -------------------------------------------------------------------- */
@@ -315,13 +315,13 @@ void CBandInterleavedChannel
 /************************************************************************/
 
 void CBandInterleavedChannel
-::SetChanInfo( std::string filename, uint64 image_offset, 
-               uint64 pixel_offset, uint64 line_offset, 
+::SetChanInfo( std::string filenameIn, uint64 image_offset, 
+               uint64 pixel_offsetIn, uint64 line_offsetIn, 
                bool little_endian )
 
 {
     if( ih_offset == 0 )
-        ThrowPCIDSKException( "No Image Header available for this channel." );
+        return ThrowPCIDSKException( "No Image Header available for this channel." );
 
 /* -------------------------------------------------------------------- */
 /*      Fetch the existing image header.                                */
@@ -337,7 +337,7 @@ void CBandInterleavedChannel
 /* -------------------------------------------------------------------- */
     std::string IHi2_filename;
     
-    if( filename.size() > 64 )
+    if( filenameIn.size() > 64 )
     {
         int link_segment;
         
@@ -356,7 +356,7 @@ void CBandInterleavedChannel
                                      "Long external channel filename link.", 
                                      SEG_SYS, 1 );
 
-            sprintf( link_filename, "LNK %4d", link_segment );
+            snprintf( link_filename, sizeof(link_filename), "LNK %4d", link_segment );
             IHi2_filename = link_filename;
         }
 
@@ -365,7 +365,7 @@ void CBandInterleavedChannel
         
         if( link != NULL )
         {
-            link->SetPath( filename );
+            link->SetPath( filenameIn );
             link->Synchronize();
         }
     }
@@ -385,7 +385,7 @@ void CBandInterleavedChannel
             file->DeleteSegment( link_segment );
         }
         
-        IHi2_filename = filename;
+        IHi2_filename = filenameIn;
     }
         
 /* -------------------------------------------------------------------- */
@@ -398,10 +398,10 @@ void CBandInterleavedChannel
     ih.Put( image_offset, 168, 16 );
 
     // IHi.6.2
-    ih.Put( pixel_offset, 184, 8 );
+    ih.Put( pixel_offsetIn, 184, 8 );
 
     // IHi.6.3
-    ih.Put( line_offset, 192, 8 );
+    ih.Put( line_offsetIn, 192, 8 );
 
     // IHi.6.5
     if( little_endian )
@@ -416,11 +416,11 @@ void CBandInterleavedChannel
 /* -------------------------------------------------------------------- */
     this->filename = MergeRelativePath( file->GetInterfaces()->io,
                                         file->GetFilename(), 
-                                        filename );
+                                        filenameIn );
 
     start_byte = image_offset;
-    this->pixel_offset = pixel_offset;
-    this->line_offset = line_offset;
+    this->pixel_offset = pixel_offsetIn;
+    this->line_offset = line_offsetIn;
     
     if( little_endian )
         byte_order = 'S';
@@ -458,15 +458,17 @@ std::string CBandInterleavedChannel::MassageLink( std::string filename_in ) cons
         
         if (seg_num == 0)
         {
-            throw PCIDSKException("Unable to find link segment. Link name: %s",
+            ThrowPCIDSKException("Unable to find link segment. Link name: %s",
                                   filename_in.c_str());
+            return "";
         }
         
         CLinkSegment* link_seg = 
             dynamic_cast<CLinkSegment*>(file->GetSegment(seg_num));
         if (link_seg == NULL)
         {
-            throw PCIDSKException("Failed to get Link Information Segment.");
+            ThrowPCIDSKException("Failed to get Link Information Segment.");
+            return "";
         }
         
         filename_in = link_seg->GetPath();
@@ -474,4 +476,3 @@ std::string CBandInterleavedChannel::MassageLink( std::string filename_in ) cons
 
     return filename_in;
 }
-
