@@ -52,40 +52,42 @@ using namespace PCIDSK;
 /*                          CExternalChannel()                          */
 /************************************************************************/
 
-CExternalChannel::CExternalChannel( PCIDSKBuffer &image_header, 
-                                    uint64 ih_offset, 
-                                    CPL_UNUSED PCIDSKBuffer &file_header,
-                                    std::string filename,
-                                    int channelnum,
-                                    CPCIDSKFile *file,
-                                    eChanType pixel_type )
-        : CPCIDSKChannel( image_header, ih_offset, file, pixel_type, channelnum)
+CExternalChannel::CExternalChannel( PCIDSKBuffer &image_headerIn,
+                                    uint64 ih_offsetIn,
+                                    CPL_UNUSED PCIDSKBuffer &file_headerIn,
+                                    std::string filenameIn,
+                                    int channelnumIn,
+                                    CPCIDSKFile *fileIn,
+                                    eChanType pixel_typeIn )
+        : CPCIDSKChannel( image_headerIn, ih_offsetIn, fileIn, pixel_typeIn, channelnumIn)
 
 {
     db = NULL;
     mutex = NULL;
+    writable = false;
+    blocks_per_row = 0;
 
 /* -------------------------------------------------------------------- */
 /*      Establish the data window.                                      */
 /* -------------------------------------------------------------------- */
-    exoff  = atoi(image_header.Get( 250, 8 ));
-    eyoff  = atoi(image_header.Get( 258, 8 ));
-    exsize = atoi(image_header.Get( 266, 8 ));
-    eysize = atoi(image_header.Get( 274, 8 ));
+    exoff  = atoi(image_headerIn.Get( 250, 8 ));
+    eyoff  = atoi(image_headerIn.Get( 258, 8 ));
+    exsize = atoi(image_headerIn.Get( 266, 8 ));
+    eysize = atoi(image_headerIn.Get( 274, 8 ));
 
-    echannel = atoi(image_header.Get( 282, 8 ));
+    echannel = atoi(image_headerIn.Get( 282, 8 ));
 
     if (echannel == 0) {
-        echannel = channelnum;
+        echannel = channelnumIn;
     }
 
 /* -------------------------------------------------------------------- */
 /*      Establish the file we will be accessing.                        */
 /* -------------------------------------------------------------------- */
-    if( filename != "" )
-        this->filename = filename;
+    if( filenameIn != "" )
+        this->filename = filenameIn;
     else
-        image_header.Get(64,64,this->filename);
+        image_headerIn.Get(64,64,this->filename);
 }
 
 /************************************************************************/
@@ -178,7 +180,7 @@ int CExternalChannel::ReadBlock( int block_index, void *buffer,
     if( xoff < 0 || xoff + xsize > GetBlockWidth()
         || yoff < 0 || yoff + ysize > GetBlockHeight() )
     {
-        ThrowPCIDSKException( 
+        return ThrowPCIDSKException( 0,
             "Invalid window in ReadBlock(): xoff=%d,yoff=%d,xsize=%d,ysize=%d",
             xoff, yoff, xsize, ysize );
     }
@@ -210,7 +212,7 @@ int CExternalChannel::ReadBlock( int block_index, void *buffer,
     int dst_blockx, dst_blocky;
 
     if( temp_buffer == NULL )
-        ThrowPCIDSKException( "Failed to allocate temporary block buffer." );
+        return ThrowPCIDSKException(0, "Failed to allocate temporary block buffer." );
 
     dst_blockx = block_index % blocks_per_row;
     dst_blocky = block_index / blocks_per_row;
@@ -407,7 +409,7 @@ int CExternalChannel::WriteBlock( int block_index, void *buffer )
     AccessDB();
 
     if( !file->GetUpdatable() || !writable )
-        throw PCIDSKException( "File not open for update in WriteBlock()" );
+        return ThrowPCIDSKException(0, "File not open for update in WriteBlock()" );
 
 /* -------------------------------------------------------------------- */
 /*      Pass the request on directly in the simple case.                */
@@ -435,7 +437,7 @@ int CExternalChannel::WriteBlock( int block_index, void *buffer )
     int dst_blockx, dst_blocky;
 
     if( temp_buffer == NULL )
-        ThrowPCIDSKException( "Failed to allocate temporary block buffer." );
+        return ThrowPCIDSKException(0, "Failed to allocate temporary block buffer." );
 
     dst_blockx = block_index % blocks_per_row;
     dst_blocky = block_index / blocks_per_row;
@@ -646,30 +648,30 @@ int CExternalChannel::WriteBlock( int block_index, void *buffer )
 /************************************************************************/
 /*                            GetEChanInfo()                            */
 /************************************************************************/
-void CExternalChannel::GetEChanInfo( std::string &filename, int &echannel,
-                                     int &exoff, int &eyoff, 
-                                     int &exsize, int &eysize ) const
+void CExternalChannel::GetEChanInfo( std::string &filenameOut, int &echannelOut,
+                                     int &exoffOut, int &eyoffOut, 
+                                     int &exsizeOut, int &eysizeOut ) const
 
 {
-    echannel = this->echannel;
-    exoff = this->exoff;
-    eyoff = this->eyoff;
-    exsize = this->exsize;
-    eysize = this->eysize;
-    filename = this->filename;
+    echannelOut = this->echannel;
+    exoffOut = this->exoff;
+    eyoffOut = this->eyoff;
+    exsizeOut = this->exsize;
+    eysizeOut = this->eysize;
+    filenameOut = this->filename;
 }
 
 /************************************************************************/
 /*                            SetEChanInfo()                            */
 /************************************************************************/
 
-void CExternalChannel::SetEChanInfo( std::string filename, int echannel,
-                                     int exoff, int eyoff, 
-                                     int exsize, int eysize )
+void CExternalChannel::SetEChanInfo( std::string filenameIn, int echannelIn,
+                                     int exoffIn, int eyoffIn, 
+                                     int exsizeIn, int eysizeIn )
 
 {
     if( ih_offset == 0 )
-        ThrowPCIDSKException( "No Image Header available for this channel." );
+        return ThrowPCIDSKException( "No Image Header available for this channel." );
 
 /* -------------------------------------------------------------------- */
 /*      Fetch the existing image header.                                */
@@ -685,7 +687,7 @@ void CExternalChannel::SetEChanInfo( std::string filename, int echannel,
 /* -------------------------------------------------------------------- */
     std::string IHi2_filename;
     
-    if( filename.size() > 64 )
+    if( filenameIn.size() > 64 )
     {
         int link_segment;
         
@@ -704,7 +706,7 @@ void CExternalChannel::SetEChanInfo( std::string filename, int echannel,
                                      "Long external channel filename link.", 
                                      SEG_SYS, 1 );
 
-            sprintf( link_filename, "LNK %4d", link_segment );
+            snprintf( link_filename, sizeof(link_filename), "LNK %4d", link_segment );
             IHi2_filename = link_filename;
         }
 
@@ -713,7 +715,7 @@ void CExternalChannel::SetEChanInfo( std::string filename, int echannel,
         
         if( link != NULL )
         {
-            link->SetPath( filename );
+            link->SetPath( filenameIn );
             link->Synchronize();
         }
     }
@@ -733,7 +735,7 @@ void CExternalChannel::SetEChanInfo( std::string filename, int echannel,
             file->DeleteSegment( link_segment );
         }
         
-        IHi2_filename = filename;
+        IHi2_filename = filenameIn;
     }
         
 /* -------------------------------------------------------------------- */
@@ -755,19 +757,19 @@ void CExternalChannel::SetEChanInfo( std::string filename, int echannel,
     ih.Put( "", 201, 1 );
 
     // IHi.6.7
-    ih.Put( exoff, 250, 8 );
+    ih.Put( exoffIn, 250, 8 );
 
     // IHi.6.8
-    ih.Put( eyoff, 258, 8 );
+    ih.Put( eyoffIn, 258, 8 );
 
     // IHi.6.9
-    ih.Put( exsize, 266, 8 );
+    ih.Put( exsizeIn, 266, 8 );
 
     // IHi.6.10
-    ih.Put( eysize, 274, 8 );
+    ih.Put( eysizeIn, 274, 8 );
 
     // IHi.6.11
-    ih.Put( echannel, 282, 8 );
+    ih.Put( echannelIn, 282, 8 );
 
     file->WriteToFile( ih.buffer, ih_offset, 1024 );
 
@@ -776,11 +778,11 @@ void CExternalChannel::SetEChanInfo( std::string filename, int echannel,
 /* -------------------------------------------------------------------- */
     this->filename = MergeRelativePath( file->GetInterfaces()->io,
                                         file->GetFilename(), 
-                                        filename );
+                                        filenameIn );
 
-    this->exoff = exoff;
-    this->eyoff = eyoff;
-    this->exsize = exsize;
-    this->eysize = eysize;
-    this->echannel = echannel;
+    this->exoff = exoffIn;
+    this->eyoff = eyoffIn;
+    this->exsize = exsizeIn;
+    this->eysize = eysizeIn;
+    this->echannel = echannelIn;
 }
