@@ -24,8 +24,8 @@
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
-#ifndef __INCLUDE_CHANNEL_CPIXELINTERLEAVEDCHANNEL_H
-#define __INCLUDE_CHANNEL_CPIXELINTERLEAVEDCHANNEL_H
+#ifndef INCLUDE_CHANNEL_CPIXELINTERLEAVEDCHANNEL_H
+#define INCLUDE_CHANNEL_CPIXELINTERLEAVEDCHANNEL_H
 
 #include "pcidsk_config.h"
 #include "pcidsk_types.h"
@@ -63,4 +63,4 @@ namespace PCIDSK
     };
 } // end namespace PCIDSK
 
-#endif // __INCLUDE_CHANNEL_CPIXELINTERLEAVEDCHANNEL_H
+#endif // INCLUDE_CHANNEL_CPIXELINTERLEAVEDCHANNEL_H
