@@ -98,7 +98,7 @@ std::string &PCIDSK::UCaseStr( std::string &target )
         if( islower(target[i]) )
             target[i] = (char) toupper(target[i]);
     }
-    
+
     return target;
 }
 
@@ -134,9 +134,9 @@ int64 PCIDSK::atoint64( const char *str_value )
 /*                            SwapPixels()                              */
 /************************************************************************/
 /**
- * @brief Perform an endianess swap for a given buffer of pixels
+ * @brief Perform an endianness swap for a given buffer of pixels
  *
- * Baed on the provided data type, do an appropriate endianess swap for
+ * Baed on the provided data type, do an appropriate endianness swap for
  * a buffer of pixels. Deals with the Complex case specially, in
  * particular.
  *
@@ -153,15 +153,15 @@ void PCIDSK::SwapPixels(void* const data,
     case CHN_16U:
     case CHN_16S:
     case CHN_32R:
-        SwapData(data, DataTypeSize(type), count);
+        SwapData(data, DataTypeSize(type), static_cast<int>(count));
         break;
     case CHN_C16U:
     case CHN_C16S:
     case CHN_C32R:
-        SwapData(data, DataTypeSize(type) / 2, count * 2);
+        SwapData(data, DataTypeSize(type) / 2, static_cast<int>(count) * 2);
         break;
     default:
-        ThrowPCIDSKException("Unknown data type passed to SwapPixels."
+        return ThrowPCIDSKException("Unknown data type passed to SwapPixels."
             "This is a software bug. Please contact your vendor.");
     }
 }
@@ -234,7 +234,7 @@ void PCIDSK::SwapData( void* const data, const int size, const int wcount )
         }
     }
     else
-        ThrowPCIDSKException( "Unsupported data size in SwapData()" );
+        return ThrowPCIDSKException( "Unsupported data size in SwapData()" );
 }
 
 /************************************************************************/
@@ -273,7 +273,7 @@ void PCIDSK::ParseTileFormat( std::string full_text,
 /* -------------------------------------------------------------------- */
 /*      Only operate on tiled stuff.                                    */
 /* -------------------------------------------------------------------- */
-    if( strncmp(full_text.c_str(),"TILED",5) != 0 )
+    if( !STARTS_WITH(full_text.c_str(), "TILED") )
         return;
 
 /* -------------------------------------------------------------------- */
@@ -287,7 +287,7 @@ void PCIDSK::ParseTileFormat( std::string full_text,
         while( isdigit(*next_text) )
             next_text++;
     }
-    
+
     while( *next_text == ' ' )
         next_text++;
 
@@ -300,16 +300,16 @@ void PCIDSK::ParseTileFormat( std::string full_text,
         if (compression == "NO_WARNINGS")
             compression = "";
         else if( compression != "RLE"
-            && strncmp(compression.c_str(),"JPEG",4) != 0 
+            && !STARTS_WITH(compression.c_str(), "JPEG") 
             && compression != "NONE"
             && compression != "QUADTREE" )
         {
-            ThrowPCIDSKException( "Unsupported tile compression scheme '%s' requested.",
+            return ThrowPCIDSKException( "Unsupported tile compression scheme '%s' requested.",
                                   compression.c_str() );
         }
     }    
 }
-                      
+
 /************************************************************************/
 /*                           pci_strcasecmp()                           */
 /************************************************************************/
@@ -347,12 +347,10 @@ int PCIDSK::pci_strcasecmp( const char *string1, const char *string2 )
 /*                          pci_strncasecmp()                           */
 /************************************************************************/
 
-int PCIDSK::pci_strncasecmp( const char *string1, const char *string2, int len )
+int PCIDSK::pci_strncasecmp( const char *string1, const char *string2, size_t len )
 
 {
-    int i;
-
-    for( i = 0; i < len; i++ )
+    for( size_t i = 0; i < len; i++ )
     {
         if( string1[i] == '\0' && string2[i] == '\0' )
             return 0;
@@ -391,11 +389,10 @@ std::vector<double> PCIDSK::ProjParmsFromText( std::string geosys,
 
 {
     std::vector<double> dparms;
-    const char *next = sparms.c_str();
 
-    for( next = sparms.c_str(); *next != '\0'; )
+    for( const char* next = sparms.c_str(); *next != '\0'; )
     {
-        dparms.push_back( atof(next) );
+        dparms.push_back( CPLAtof(next) );
 
         // move past this token
         while( *next != '\0' && *next != ' ' )
@@ -409,25 +406,25 @@ std::vector<double> PCIDSK::ProjParmsFromText( std::string geosys,
     dparms.resize(18);
 
     // This is rather iffy!
-    if( EQUALN(geosys.c_str(),"DEGREE",3) )
+    if( STARTS_WITH_CI(geosys.c_str(),"DEG" /* "DEGREE" */) )
         dparms[17] = (double) (int) UNIT_DEGREE;
-    else if( EQUALN(geosys.c_str(),"MET",3) )
+    else if( STARTS_WITH_CI(geosys.c_str(), "MET") )
         dparms[17] = (double) (int) UNIT_METER;
-    else if( EQUALN(geosys.c_str(),"FOOT",4) )
+    else if( STARTS_WITH_CI(geosys.c_str(), "FOOT") )
         dparms[17] = (double) (int) UNIT_US_FOOT;
-    else if( EQUALN(geosys.c_str(),"FEET",4) )
+    else if( STARTS_WITH_CI(geosys.c_str(), "FEET") )
         dparms[17] = (double) (int) UNIT_US_FOOT;
-    else if( EQUALN(geosys.c_str(),"INTL FOOT",5) )
+    else if( STARTS_WITH_CI(geosys.c_str(),"INTL " /* "INTL FOOT" */) )
         dparms[17] = (double) (int) UNIT_INTL_FOOT;
-    else if( EQUALN(geosys.c_str(),"SPCS",4) )
+    else if( STARTS_WITH_CI(geosys.c_str(), "SPCS") )
         dparms[17] = (double) (int) UNIT_METER;
-    else if( EQUALN(geosys.c_str(),"SPIF",4) )
+    else if( STARTS_WITH_CI(geosys.c_str(), "SPIF") )
         dparms[17] = (double) (int) UNIT_INTL_FOOT;
-    else if( EQUALN(geosys.c_str(),"SPAF",4) )
+    else if( STARTS_WITH_CI(geosys.c_str(), "SPAF") )
         dparms[17] = (double) (int) UNIT_US_FOOT;
     else
         dparms[17] = -1.0; /* unknown */
-    
+
     return dparms;
 }
 
@@ -438,10 +435,9 @@ std::vector<double> PCIDSK::ProjParmsFromText( std::string geosys,
 std::string PCIDSK::ProjParmsToText( std::vector<double> dparms )
 
 {
-    unsigned int i;
     std::string sparms;
 
-    for( i = 0; i < 17; i++ )
+    for( unsigned int i = 0; i < 17; i++ )
     {
         char value[64];
         double dvalue;
@@ -452,13 +448,13 @@ std::string PCIDSK::ProjParmsToText( std::vector<double> dparms )
             dvalue = 0.0;
 
         if( dvalue == floor(dvalue) )
-            sprintf( value, "%d", (int) dvalue );
+            CPLsnprintf( value, sizeof(value), "%d", (int) dvalue );
         else
-            sprintf( value, "%.15g", dvalue );
-        
+            CPLsnprintf( value, sizeof(value), "%.15g", dvalue );
+
         if( i > 0 )
             sparms += " ";
-        
+
         sparms += value;
     }
 
@@ -470,7 +466,7 @@ std::string PCIDSK::ProjParmsToText( std::vector<double> dparms )
 /*                                                                      */
 /*      Extract the directory path portion of the passed filename.      */
 /*      It assumes the last component is a filename and should not      */
-/*      be passed a bare path.  The trailing directory delimeter is     */
+/*      be passed a bare path.  The trailing directory delimiter is     */
 /*      removed from the result.  The return result is an empty         */
 /*      string for a simple filename passed in with no directory        */
 /*      component.                                                      */
@@ -481,7 +477,7 @@ std::string PCIDSK::ExtractPath( std::string filename )
 {
     int i;
 
-    for( i = filename.size()-1; i >= 0; i-- )
+    for( i = static_cast<int>(filename.size())-1; i >= 0; i-- )
     {
         if( filename[i] == '\\' || filename[i] == '/' )
             break;
@@ -568,7 +564,7 @@ void PCIDSK::DefaultDebug( const char * message )
 {
     static bool initialized = false;
     static bool enabled = false;
-    
+
     if( !initialized )
     {
         if( getenv( "PCIDSK_DEBUG" ) != NULL )
@@ -622,7 +618,7 @@ static void vDebug( void (*pfnDebug)(const char *),
 #else
     wrk_args = args;
 #endif
-    
+
     nPR = vsnprintf( szModestBuffer, sizeof(szModestBuffer), fmt, 
                      wrk_args );
     if( nPR == -1 || nPR >= (int) sizeof(szModestBuffer)-1 )
