@@ -25,8 +25,8 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef __INCLUDE_SEGMENT_PCIDSKBITMAP_H
-#define __INCLUDE_SEGMENT_PCIDSKBITMAP_H
+#ifndef INCLUDE_SEGMENT_PCIDSKBITMAP_H
+#define INCLUDE_SEGMENT_PCIDSKBITMAP_H
 
 #include "pcidsk_config.h"
 #include "pcidsk_types.h"
@@ -110,4 +110,4 @@ namespace PCIDSK
     };
 } // end namespace PCIDSK
 
-#endif // __INCLUDE_SEGMENT_PCIDSKBITMAP_H
+#endif // INCLUDE_SEGMENT_PCIDSKBITMAP_H
