@@ -24,8 +24,8 @@
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
-#ifndef __INCLUDE_PCIDSK_SEGMENT_PCIDSKRPCMODEL_H
-#define __INCLUDE_PCIDSK_SEGMENT_PCIDSKRPCMODEL_H
+#ifndef INCLUDE_PCIDSK_SEGMENT_PCIDSKRPCMODEL_H
+#define INCLUDE_PCIDSK_SEGMENT_PCIDSKRPCMODEL_H
  
 #include "pcidsk_rpc.h"
 #include "segment/cpcidsksegment.h"
@@ -124,4 +124,4 @@ namespace PCIDSK {
     };
 }
 
-#endif // __INCLUDE_PCIDSK_SEGMENT_PCIDSKRPCMODEL_H
+#endif // INCLUDE_PCIDSK_SEGMENT_PCIDSKRPCMODEL_H
