@@ -24,8 +24,8 @@
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
-#ifndef __INCLUDE_SEGMENT_PCIDSKVECTORSEGMENT_H
-#define __INCLUDE_SEGMENT_PCIDSKVECTORSEGMENT_H
+#ifndef INCLUDE_SEGMENT_PCIDSKVECTORSEGMENT_H
+#define INCLUDE_SEGMENT_PCIDSKVECTORSEGMENT_H
 
 #include "pcidsk_config.h"
 #include "pcidsk_types.h"
