@@ -1,12 +1,12 @@
 /******************************************************************************
- * $Id: pcrasterdataset.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: pcrasterdataset.cpp 33105 2016-01-23 15:27:32Z rouault $
  *
  * Project:  PCRaster Integration
  * Purpose:  PCRaster CSF 2.0 raster file driver
- * Author:   Kor de Jong, k.dejong at geog.uu.nl
+ * Author:   Kor de Jong, Oliver Schmitz
  *
  ******************************************************************************
- * Copyright (c) 2004, Kor de Jong
+ * Copyright (c) PCRaster owners
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,34 +27,14 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "gdal_pam.h"
 #include "cpl_string.h"
+#include "gdal_pam.h"
 
-CPL_CVSID("$Id: pcrasterdataset.cpp 27729 2014-09-24 00:40:16Z goatbar $");
-
-#ifndef INCLUDED_PCRASTERDATASET
-#include "pcrasterdataset.h"
-#define INCLUDED_PCRASTERDATASET
-#endif
-
-#ifndef INCLUDED_IOSTREAM
-#include <iostream>
-#define INCLUDED_IOSTREAM
-#endif
-
-// PCRaster library headers.
-
-// Module headers.
-#ifndef INCLUDED_PCRASTERRASTERBAND
 #include "pcrasterrasterband.h"
-#define INCLUDED_PCRASTERRASTERBAND
-#endif
-
-#ifndef INCLUDED_PCRASTERUTIL
+#include "pcrasterdataset.h"
 #include "pcrasterutil.h"
-#define INCLUDED_PCRASTERUTIL
-#endif
 
+CPL_CVSID("$Id: pcrasterdataset.cpp 33105 2016-01-23 15:27:32Z rouault $");
 
 
 /*!
@@ -73,15 +53,15 @@ CPL_CVSID("$Id: pcrasterdataset.cpp 27729 2014-09-24 00:40:16Z goatbar $");
   \param     info Object with information about the dataset to open.
   \return    Pointer to newly allocated GDALDataset or 0.
 
-  Returns 0 if the file could not be opened.
+  Returns a nullptr if the file could not be opened.
 */
 GDALDataset* PCRasterDataset::open(
          GDALOpenInfo* info)
 {
-  PCRasterDataset* dataset = 0;
+  PCRasterDataset* dataset = NULL;
 
-  if(info->fp && info->nHeaderBytes >= static_cast<int>(CSF_SIZE_SIG) &&
-         strncmp((char*)info->pabyHeader, CSF_SIG, CSF_SIZE_SIG) == 0) {
+  if(info->fpL && info->nHeaderBytes >= static_cast<int>(CSF_SIZE_SIG) &&
+     strncmp(reinterpret_cast<char*>( info->pabyHeader ), CSF_SIG, CSF_SIZE_SIG) == 0) {
     MOPEN_PERM mode = info->eAccess == GA_Update
          ? M_READ_WRITE
          : M_READ;
@@ -89,7 +69,13 @@ GDALDataset* PCRasterDataset::open(
     MAP* map = mapOpen(info->pszFilename, mode);
 
     if(map) {
+      CPLErrorReset();
       dataset = new PCRasterDataset(map);
+      if( CPLGetLastErrorType() != CE_None )
+      {
+          delete dataset;
+          return NULL;
+      }
     }
   }
 
@@ -123,27 +109,24 @@ GDALDataset* PCRasterDataset::open(
   cell representations.
 */
 GDALDataset* PCRasterDataset::createCopy(
-         char const* filename,
-         GDALDataset* source,
-         CPL_UNUSED int strict,
-         CPL_UNUSED char** options,
-         GDALProgressFunc progress,
-         void* progressData)
+    char const* filename,
+    GDALDataset* source,
+    CPL_UNUSED int strict,
+    CPL_UNUSED char** options,
+    GDALProgressFunc progress,
+    void* progressData)
 {
   // Checks.
-  int nrBands = source->GetRasterCount();
+  const int nrBands = source->GetRasterCount();
   if(nrBands != 1) {
     CPLError(CE_Failure, CPLE_NotSupported,
          "PCRaster driver: Too many bands ('%d'): must be 1 band", nrBands);
-    return 0;
+    return NULL;
   }
 
   GDALRasterBand* raster = source->GetRasterBand(1);
 
   // Create PCRaster raster. Determine properties of raster to create.
-  size_t nrRows = raster->GetYSize();
-  size_t nrCols = raster->GetXSize();
-  std::string string;
 
   // The in-file type of the cells.
   CSF_CR fileCellRepresentation = GDALType2CellRepresentation(
@@ -152,27 +135,28 @@ GDALDataset* PCRasterDataset::createCopy(
   if(fileCellRepresentation == CR_UNDEFINED) {
     CPLError(CE_Failure, CPLE_NotSupported,
          "PCRaster driver: Cannot determine a valid cell representation");
-    return 0;
+    return NULL;
   }
 
   // The value scale of the values.
   CSF_VS valueScale = VS_UNDEFINED;
+  std::string osString;
   if(source->GetMetadataItem("PCRASTER_VALUESCALE")) {
-    string = source->GetMetadataItem("PCRASTER_VALUESCALE");
+    osString = source->GetMetadataItem("PCRASTER_VALUESCALE");
   }
 
-  valueScale = !string.empty()
-         ? string2ValueScale(string)
+  valueScale = !osString.empty()
+         ? string2ValueScale(osString)
          : GDALType2ValueScale(raster->GetRasterDataType());
 
   if(valueScale == VS_UNDEFINED) {
     CPLError(CE_Failure, CPLE_NotSupported,
          "PCRaster driver: Cannot determine a valid value scale");
-    return 0;
+    return NULL;
   }
 
   CSF_PT const projection = PT_YDECT2B;
-  REAL8  const angle = 0.0;
+  const REAL8 angle = 0.0;
   REAL8 west = 0.0;
   REAL8 north = 0.0;
   REAL8 cellSize = 1.0;
@@ -194,7 +178,7 @@ GDALDataset* PCRasterDataset::createCopy(
   if(appCellRepresentation == CR_UNDEFINED) {
     CPLError(CE_Failure, CPLE_NotSupported,
          "PCRaster driver: Cannot determine a valid cell representation");
-    return 0;
+    return NULL;
   }
 
   // Check whether value scale fits the cell representation. Adjust when
@@ -202,13 +186,15 @@ GDALDataset* PCRasterDataset::createCopy(
   valueScale = fitValueScale(valueScale, appCellRepresentation);
 
   // Create a raster with the in file cell representation.
+  const size_t nrRows = raster->GetYSize();
+  const size_t nrCols = raster->GetXSize();
   MAP* map = Rcreate(filename, nrRows, nrCols, fileCellRepresentation,
          valueScale, projection, west, north, angle, cellSize);
 
   if(!map) {
     CPLError(CE_Failure, CPLE_OpenFailed,
          "PCRaster driver: Unable to create raster %s", filename);
-    return 0;
+    return NULL;
   }
 
   // Try to convert in app cell representation to the cell representation
@@ -217,7 +203,7 @@ GDALDataset* PCRasterDataset::createCopy(
     CPLError(CE_Failure, CPLE_NotSupported,
          "PCRaster driver: Cannot convert cells: %s", MstrError());
     Mclose(map);
-    return 0;
+    return NULL;
   }
 
   int hasMissingValue;
@@ -232,8 +218,9 @@ GDALDataset* PCRasterDataset::createCopy(
     missingValue = ::missingValue(fileCellRepresentation);
   }
 
-  // TODO conversie van INT2 naar INT4 ondersteunen. zie ruseas.c regel 503.
-  // conversie op r 159.
+  // TODO: Proper translation of TODO.
+  // TODO: Support conversion to INT2 (?) INT4. ruseas.c see line 503.
+  // Conversion r 159.
 
   // Create buffer for one row of values.
   void* buffer = Rmalloc(map, nrCols);
@@ -243,8 +230,10 @@ GDALDataset* PCRasterDataset::createCopy(
   for(size_t row = 0; errorCode == CE_None && row < nrRows; ++row) {
 
     // Get row from source.
-    if(raster->RasterIO(GF_Read, 0, row, nrCols, 1, buffer, nrCols, 1,
-         raster->GetRasterDataType(), 0, 0) != CE_None) {
+    if(raster->RasterIO(GF_Read, 0, static_cast<int>(row),
+        static_cast<int>(nrCols), 1, buffer,
+        static_cast<int>(nrCols), 1,
+         raster->GetRasterDataType(), 0, 0, NULL) != CE_None) {
       CPLError(CE_Failure, CPLE_FileIO,
          "PCRaster driver: Error reading from source raster");
       errorCode = CE_Failure;
@@ -267,7 +256,7 @@ GDALDataset* PCRasterDataset::createCopy(
     // Write row in target.
     RputRow(map, row, buffer);
 
-    if(!progress((row + 1) / (static_cast<double>(nrRows)), 0, progressData)) {
+    if(!progress((row + 1) / (static_cast<double>(nrRows)), NULL, progressData)) {
       CPLError(CE_Failure, CPLE_UserInterrupt,
          "PCRaster driver: User terminated CreateCopy()");
       errorCode = CE_Failure;
@@ -276,23 +265,23 @@ GDALDataset* PCRasterDataset::createCopy(
   }
 
   Mclose(map);
-  map = 0;
+  map = NULL;
 
   free(buffer);
-  buffer = 0;
+  buffer = NULL;
 
   if( errorCode != CE_None )
       return NULL;
 
 /* -------------------------------------------------------------------- */
-/*      Re-open dataset, and copy any auxilary pam information.         */
+/*      Re-open dataset, and copy any auxiliary pam information.        */
 /* -------------------------------------------------------------------- */
-  GDALPamDataset *poDS = (GDALPamDataset *) 
-        GDALOpen( filename, GA_Update );
+  GDALPamDataset *poDS = reinterpret_cast<GDALPamDataset *>(
+      GDALOpen( filename, GA_Update ) );
 
   if( poDS )
       poDS->CloneInfo( source, GCIF_PAM_DEFAULT );
-  
+
   return poDS;
 }
 
@@ -306,24 +295,31 @@ GDALDataset* PCRasterDataset::createCopy(
 /*!
   \param     map PCRaster map handle. It is ours to close.
 */
-PCRasterDataset::PCRasterDataset(
-         MAP* map)
-
-  : GDALPamDataset(),
-    d_map(map), d_west(0.0), d_north(0.0), d_cellSize(0.0)
-
+PCRasterDataset::PCRasterDataset( MAP* mapIn) :
+    GDALPamDataset(),
+    d_map(mapIn),
+    d_west(0.0),
+    d_north(0.0),
+    d_cellSize(0.0),
+    d_location_changed(false)
 {
   // Read header info.
-  nRasterXSize = RgetNrCols(d_map);
-  nRasterYSize = RgetNrRows(d_map);
+  nRasterXSize = static_cast<int>(RgetNrCols(d_map));
+  nRasterYSize = static_cast<int>(RgetNrRows(d_map));
   d_west = static_cast<double>(RgetXUL(d_map));
   d_north = static_cast<double>(RgetYUL(d_map));
   d_cellSize = static_cast<double>(RgetCellSize(d_map));
   d_cellRepresentation = RgetUseCellRepr(d_map);
-  CPLAssert(d_cellRepresentation != CR_UNDEFINED);
+  if( d_cellRepresentation == CR_UNDEFINED )
+  {
+      CPLError(CE_Failure, CPLE_AssertionFailed, "d_cellRepresentation != CR_UNDEFINED");
+  }
   d_valueScale = RgetValueScale(d_map);
-  CPLAssert(d_valueScale != VS_UNDEFINED);
-  d_missingValue = ::missingValue(d_cellRepresentation);
+  if( d_valueScale == VS_UNDEFINED )
+  {
+      CPLError(CE_Failure, CPLE_AssertionFailed, "d_valueScale != VS_UNDEFINED");
+  }
+  d_defaultNoDataValue = ::missingValue(d_cellRepresentation);
 
   // Create band information objects.
   nBands = 1;
@@ -413,22 +409,138 @@ CSF_VS PCRasterDataset::valueScale() const
 /*!
   \return    Missing value
 */
-double PCRasterDataset::missingValue() const
+double PCRasterDataset::defaultNoDataValue() const
 {
-  return d_missingValue;
+  return d_defaultNoDataValue;
 }
 
 
+GDALDataset* PCRasterDataset::create(
+     const char* filename,
+     int nr_cols,
+     int nr_rows,
+     int nrBands,
+     GDALDataType gdalType,
+     char** papszParmList)
+{
+  // Checks
+  if(nrBands != 1){
+    CPLError(CE_Failure, CPLE_NotSupported,
+         "PCRaster driver : "
+         "attempt to create dataset with too many bands (%d); "
+         "must be 1 band.\n", nrBands);
+    return NULL;
+  }
 
-//------------------------------------------------------------------------------
-// DEFINITION OF FREE OPERATORS
-//------------------------------------------------------------------------------
+  const int row_col_max = INT4_MAX - 1;
+  if(nr_cols > row_col_max){
+    CPLError(CE_Failure, CPLE_NotSupported,
+         "PCRaster driver : "
+         "attempt to create dataset with too many columns (%d); "
+         "must be smaller than %d.", nr_cols, row_col_max);
+    return NULL;
+  }
+
+  if(nr_rows > row_col_max){
+    CPLError(CE_Failure, CPLE_NotSupported,
+         "PCRaster driver : "
+         "attempt to create dataset with too many rows (%d); "
+         "must be smaller than %d.", nr_rows, row_col_max);
+    return NULL;
+  }
+
+  if(gdalType != GDT_Byte &&
+     gdalType != GDT_Int32 &&
+     gdalType != GDT_Float32){
+     CPLError( CE_Failure, CPLE_AppDefined,
+       "PCRaster driver: "
+       "attempt to create dataset with an illegal data type (%s); "
+       "use either Byte, Int32 or Float32.",
+       GDALGetDataTypeName(gdalType));
+    return NULL;
+  }
+
+  // value scale must be specified by the user,
+  // determines cell representation
+  const char *valueScale = CSLFetchNameValue(
+    papszParmList,"PCRASTER_VALUESCALE");
+
+  if(valueScale == NULL){
+    CPLError(CE_Failure, CPLE_AppDefined,
+         "PCRaster driver: value scale can not be determined; "
+         "specify PCRASTER_VALUESCALE.");
+    return NULL;
+  }
 
 
+  CSF_VS csf_value_scale = string2ValueScale(valueScale);
 
-//------------------------------------------------------------------------------
-// DEFINITION OF FREE FUNCTIONS
-//------------------------------------------------------------------------------
+  if(csf_value_scale == VS_UNDEFINED){
+    CPLError( CE_Failure, CPLE_AppDefined,
+         "PCRaster driver: value scale can not be determined (%s); "
+         "use either VS_BOOLEAN, VS_NOMINAL, VS_ORDINAL, VS_SCALAR, "
+         "VS_DIRECTION, VS_LDD",
+          valueScale);
+    return NULL;
+  }
+
+  CSF_CR csf_cell_representation = GDALType2CellRepresentation(gdalType, false);
+
+  // default values
+  REAL8 west = 0.0;
+  REAL8 north = 0.0;
+  REAL8 length = 1.0;
+  REAL8 angle = 0.0;
+  CSF_PT projection = PT_YDECT2B;
+
+  // Create a new raster
+  MAP* map = Rcreate(filename, nr_rows, nr_cols, csf_cell_representation,
+         csf_value_scale, projection, west, north, angle, length);
+
+  if(!map){
+    CPLError(CE_Failure, CPLE_OpenFailed,
+         "PCRaster driver: Unable to create raster %s", filename);
+    return NULL;
+  }
+
+  Mclose(map);
+  map = NULL;
+
+/* -------------------------------------------------------------------- */
+/*      Re-open dataset, and copy any auxiliary pam information.        */
+/* -------------------------------------------------------------------- */
+  GDALPamDataset *poDS = reinterpret_cast<GDALPamDataset *>(
+      GDALOpen(filename, GA_Update) );
+
+  return poDS;
+}
 
 
+CPLErr PCRasterDataset::SetGeoTransform(double* transform)
+{
+  if((transform[2] != 0.0) || (transform[4] != 0.0)) {
+    CPLError(CE_Failure, CPLE_NotSupported,
+             "PCRaster driver: "
+             "rotated geotransformations are not supported.");
+    return CE_Failure;
+  }
 
+  if(transform[1] != transform[5] * -1.0 ) {
+    CPLError(CE_Failure, CPLE_NotSupported,
+             "PCRaster driver: "
+             "only the same width and height for cells is supported." );
+    return CE_Failure;
+  }
+
+  d_west = transform[0];
+  d_north = transform[3];
+  d_cellSize = transform[1];
+  d_location_changed = true;
+
+  return CE_None;
+}
+
+
+bool PCRasterDataset::location_changed() const {
+  return d_location_changed;
+}
