@@ -1,12 +1,12 @@
 /******************************************************************************
- * $Id: pcrasterutil.cpp 15450 2008-10-03 10:59:32Z kdejong $
+ * $Id: pcrasterutil.cpp 32957 2016-01-12 18:53:43Z rouault $
  *
  * Project:  PCRaster Integration
  * Purpose:  PCRaster driver support functions.
- * Author:   Kor de Jong, k.dejong at geog.uu.nl
+ * Author:   Kor de Jong, Oliver Schmitz
  *
  ******************************************************************************
- * Copyright (c) 2004, Kor de Jong
+ * Copyright (c) PCRaster owners
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,31 +27,12 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef INCLUDED_IOSTREAM
-#include <iostream>
-#define INCLUDED_IOSTREAM
-#endif
+#include <cfloat>
 
-#ifndef INCLUDED_ALGORITHM
 #include <algorithm>
-#define INCLUDED_ALGORITHM
-#endif
 
-#ifndef INCLUDED_FLOAT
-#include <float.h>
-#define INCLUDED_FLOAT
-#endif
-
-#ifndef INCLUDED_PCRTYPES
-#include "pcrtypes.h"
-#define INCLUDED_PCRTYPES
-#endif
-
-#ifndef INCLUDED_PCRASTERUTIL
 #include "pcrasterutil.h"
-#define INCLUDED_PCRASTERUTIL
-#endif
-
+#include "pcrtypes.h"
 
 
 //! Converts PCRaster data type to GDAL data type.
@@ -108,7 +89,6 @@ GDALDataType cellRepresentation2GDALType(
 }
 
 
-
 CSF_VS string2ValueScale(
          std::string const& string)
 {
@@ -148,7 +128,6 @@ CSF_VS string2ValueScale(
 }
 
 
-
 std::string valueScale2String(
          CSF_VS valueScale)
 {
@@ -202,7 +181,6 @@ std::string valueScale2String(
 }
 
 
-
 std::string cellRepresentation2String(
          CSF_CR cellRepresentation)
 {
@@ -253,7 +231,6 @@ std::string cellRepresentation2String(
 }
 
 
-
 //! Converts GDAL data type to PCRaster value scale.
 /*!
   \param     type GDAL data type.
@@ -303,7 +280,6 @@ CSF_VS GDALType2ValueScale(
 }
 
 
-
 //! Converts a GDAL type to a PCRaster cell representation.
 /*!
   \param     type GDAL type.
@@ -314,8 +290,8 @@ CSF_VS GDALType2ValueScale(
              complex.
 
   If exact is false, conversion to CSF2.0 types will take place. This is
-  usefull for in file cell representations. If exact is true, and exact match
-  is made. This is usefull for in app cell representations.
+  useful for in file cell representations. If exact is true, and exact match
+  is made. This is useful for in app cell representations.
 
   If exact is false, this function always returns one of CR_UINT1, CR_INT4
   or CR_REAL4.
@@ -364,7 +340,6 @@ CSF_CR GDALType2CellRepresentation(
 }
 
 
-
 //! Determines a missing value to use for data of \a cellRepresentation.
 /*!
   \param     cellRepresentation Cell representation of the data.
@@ -420,7 +395,8 @@ double missingValue(
       break;
     }
     default: {
-      CPLAssert(false);
+      CPLError(CE_Failure, CPLE_NotSupported,
+               "Unexpected value for cellRepresentation = %d", cellRepresentation);
       break;
     }
   }
@@ -429,7 +405,6 @@ double missingValue(
 }
 
 
-
 //! Opens the raster in \a filename using mode \a mode.
 /*!
   \param     filename Filename of raster to open.
@@ -448,7 +423,6 @@ MAP* mapOpen(
 }
 
 
-
 void alterFromStdMV(
          void* buffer,
          size_t size,
@@ -514,7 +488,6 @@ void alterFromStdMV(
 }
 
 
-
 void alterToStdMV(
          void* buffer,
          size_t size,
@@ -580,7 +553,6 @@ void alterToStdMV(
 }
 
 
-
 CSF_VS fitValueScale(
          CSF_VS valueScale,
          CSF_CR cellRepresentation)
@@ -648,7 +620,6 @@ CSF_VS fitValueScale(
 }
 
 
-
 void castValuesToBooleanRange(
          void* buffer,
          size_t size,
@@ -712,3 +683,22 @@ void castValuesToBooleanRange(
   }
 }
 
+
+void castValuesToDirectionRange(
+         void* buffer,
+         size_t size)
+{
+  std::for_each(static_cast<REAL4*>(buffer),
+       static_cast<REAL4*>(buffer) + size,
+       CastToDirection());
+}
+
+
+void castValuesToLddRange(
+         void* buffer,
+         size_t size)
+{
+  std::for_each(static_cast<UINT1*>(buffer),
+       static_cast<UINT1*>(buffer) + size,
+       CastToLdd());
+}
