@@ -2,14 +2,14 @@
  * $Id: pdsdataset.cpp 12658 2007-11-07 23:14:33Z warmerdam $
  *
  * Project:  PDS Driver; Planetary Data System Format
- * Purpose:  Implementation of NASAKeywordHandler - a class to read 
+ * Purpose:  Implementation of NASAKeywordHandler - a class to read
  *           keyword data from PDS, ISIS2 and ISIS3 data products.
  * Author:   Frank Warmerdam <warmerdam@pobox.com
  *
  ******************************************************************************
  * Copyright (c) 2006, Frank Warmerdam <warmerdam@pobox.com>
  * Copyright (c) 2008-2010, Even Rouault <even dot rouault at mines-paris dot org>
- * 
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
@@ -32,25 +32,25 @@
  * and other NASA data systems. Refer to Chapter 12 of "PDS Standards
  * Reference" at http://pds.jpl.nasa.gov/tools/standards-reference.shtml for
  * further details about ODL.
- * 
+ *
  * This is also known as PVL (Parameter Value Language) which is written
  * about at http://www.orrery.us/node/44 where it notes:
- * 
- * The PVL syntax that the PDS uses is specified by the Consultative Committee 
- * for Space Data Systems in their Blue Book publication: "Parameter Value 
- * Language Specification (CCSD0006 and CCSD0008)", June 2000 
- * [CCSDS 641.0-B-2], and Green Book publication: "Parameter Value Language - 
+ *
+ * The PVL syntax that the PDS uses is specified by the Consultative Committee
+ * for Space Data Systems in their Blue Book publication: "Parameter Value
+ * Language Specification (CCSD0006 and CCSD0008)", June 2000
+ * [CCSDS 641.0-B-2], and Green Book publication: "Parameter Value Language -
  * A Tutorial", June 2000 [CCSDS 641.0-G-2]. PVL has also been accepted by the
- * International Standards Organization (ISO), as a Final Draft International 
- * Standard (ISO 14961:2002) keyword value type language for naming and 
+ * International Standards Organization (ISO), as a Final Draft International
+ * Standard (ISO 14961:2002) keyword value type language for naming and
  * expressing data values.
  * --
  * also of interest, on PDS ODL:
  *  http://pds.jpl.nasa.gov/documents/sr/Chapter12.pdf
- *  
+ *
  ****************************************************************************/
 
-#include "cpl_string.h" 
+#include "cpl_string.h"
 #include "nasakeywordhandler.h"
 
 /************************************************************************/
@@ -63,10 +63,10 @@
 /*                         NASAKeywordHandler()                         */
 /************************************************************************/
 
-NASAKeywordHandler::NASAKeywordHandler()
-
+NASAKeywordHandler::NASAKeywordHandler() :
+    papszKeywordList(NULL),
+    pszHeaderNext(NULL)
 {
-    papszKeywordList = NULL;
 }
 
 /************************************************************************/
@@ -93,12 +93,11 @@ int NASAKeywordHandler::Ingest( VSILFILE *fp, int nOffset )
     if( VSIFSeekL( fp, nOffset, SEEK_SET ) != 0 )
         return FALSE;
 
-    for( ; TRUE; ) 
+    for( ; true; )
     {
-        const char *pszCheck;
         char szChunk[513];
 
-        int nBytesRead = VSIFReadL( szChunk, 1, 512, fp );
+        int nBytesRead = static_cast<int>(VSIFReadL( szChunk, 1, 512, fp ));
 
         szChunk[nBytesRead] = '\0';
         osHeaderText += szChunk;
@@ -106,14 +105,15 @@ int NASAKeywordHandler::Ingest( VSILFILE *fp, int nOffset )
         if( nBytesRead < 512 )
             break;
 
+        const char *pszCheck;
         if( osHeaderText.size() > 520 )
             pszCheck = osHeaderText.c_str() + (osHeaderText.size() - 520);
         else
             pszCheck = szChunk;
 
-        if( strstr(pszCheck,"\r\nEND\r\n") != NULL 
-            || strstr(pszCheck,"\nEND\n") != NULL 
-            || strstr(pszCheck,"\r\nEnd\r\n") != NULL 
+        if( strstr(pszCheck,"\r\nEND\r\n") != NULL
+            || strstr(pszCheck,"\nEND\n") != NULL
+            || strstr(pszCheck,"\r\nEnd\r\n") != NULL
             || strstr(pszCheck,"\nEnd\n") != NULL )
             break;
     }
@@ -133,10 +133,9 @@ int NASAKeywordHandler::Ingest( VSILFILE *fp, int nOffset )
 int NASAKeywordHandler::ReadGroup( const char *pszPathPrefix )
 
 {
-    CPLString osName, osValue;
-
-    for( ; TRUE; )
+    for( ; true; )
     {
+        CPLString osName, osValue;
         if( !ReadPair( osName, osValue ) )
             return FALSE;
 
@@ -145,7 +144,7 @@ int NASAKeywordHandler::ReadGroup( const char *pszPathPrefix )
             if( !ReadGroup( (CPLString(pszPathPrefix) + osValue + ".").c_str() ) )
                 return FALSE;
         }
-        else if( EQUAL(osName,"END") 
+        else if( EQUAL(osName,"END")
                  || EQUAL(osName,"END_GROUP" )
                  || EQUAL(osName,"END_OBJECT" ) )
         {
@@ -154,7 +153,7 @@ int NASAKeywordHandler::ReadGroup( const char *pszPathPrefix )
         else
         {
             osName = pszPathPrefix + osName;
-            papszKeywordList = CSLSetNameValue( papszKeywordList, 
+            papszKeywordList = CSLSetNameValue( papszKeywordList,
                                                 osName, osValue );
         }
     }
@@ -184,17 +183,17 @@ int NASAKeywordHandler::ReadPair( CPLString &osName, CPLString &osValue )
 
     if( *pszHeaderNext != '=' )
     {
-        // ISIS3 does not have anything after the end group/object keyword. 
+        // ISIS3 does not have anything after the end group/object keyword.
         if( EQUAL(osName,"End_Group") || EQUAL(osName,"End_Object") )
             return TRUE;
-        else
-            return FALSE;
+
+        return FALSE;
     }
-    
+
     pszHeaderNext++;
-    
+
     SkipWhite();
-    
+
     osValue = "";
 
     // Handle value lists like:     Name   = (Red, Red)
@@ -227,35 +226,33 @@ int NASAKeywordHandler::ReadPair( CPLString &osName, CPLString &osValue )
         }
     }
 
-    else // Handle more normal "single word" values. 
+    else // Handle more normal "single word" values.
     {
         if( !ReadWord( osValue ) )
             return FALSE;
-
     }
-        
+
     SkipWhite();
 
-    // No units keyword?   
+    // No units keyword?
     if( *pszHeaderNext != '<' )
         return TRUE;
 
     // Append units keyword.  For lines that like like this:
     //  MAP_RESOLUTION               = 4.0 <PIXEL/DEGREE>
-    
-    CPLString osWord;
-    
+
     osValue += " ";
-    
+
+    CPLString osWord;
     while( ReadWord( osWord ) )
     {
         SkipWhite();
-        
+
         osValue += osWord;
         if( osWord[strlen(osWord)-1] == '>' )
             break;
     }
-    
+
     return TRUE;
 }
 
@@ -271,9 +268,9 @@ int NASAKeywordHandler::ReadWord( CPLString &osWord )
 
     SkipWhite();
 
-    if( !(*pszHeaderNext != '\0' 
-           && *pszHeaderNext != '=' 
-           && !isspace((unsigned char)*pszHeaderNext)) )
+    if( !(*pszHeaderNext != '\0'
+          && *pszHeaderNext != '='
+          && !isspace( static_cast<unsigned char>( *pszHeaderNext ) ) ) )
         return FALSE;
 
     /* Extract a text string delimited by '\"' */
@@ -308,7 +305,7 @@ int NASAKeywordHandler::ReadWord( CPLString &osWord )
     /* These are expected to not have
        '\'' (delimiters),
        format effectors (should fit on a single line) or
-       control characters.  
+       control characters.
     */
     if( *pszHeaderNext == '\'' )
     {
@@ -325,27 +322,27 @@ int NASAKeywordHandler::ReadWord( CPLString &osWord )
     }
 
     /*
-     * Extract normal text.  Terminated by '=' or whitespace. 
+     * Extract normal text.  Terminated by '=' or whitespace.
      *
-     * A special exception is that a line may terminate with a '-' 
+     * A special exception is that a line may terminate with a '-'
      * which is taken as a line extender, and we suck up white space to new
      * text.
      */
-    while( *pszHeaderNext != '\0' 
-           && *pszHeaderNext != '=' 
-           && !isspace((unsigned char)*pszHeaderNext) )
+    while( *pszHeaderNext != '\0'
+           && *pszHeaderNext != '='
+           && !isspace(static_cast<unsigned char>( *pszHeaderNext ) ) )
     {
         osWord += *pszHeaderNext;
         pszHeaderNext++;
 
-        if( *pszHeaderNext == '-' 
+        if( *pszHeaderNext == '-'
             && (pszHeaderNext[1] == 10 || pszHeaderNext[1] == 13) )
         {
             pszHeaderNext += 2;
             SkipWhite();
         }
     }
-    
+
     return TRUE;
 }
 
@@ -357,15 +354,15 @@ int NASAKeywordHandler::ReadWord( CPLString &osWord )
 void NASAKeywordHandler::SkipWhite()
 
 {
-    for( ; TRUE; )
+    for( ; true; )
     {
-        // Skip C style comments 
+        // Skip C style comments
         if( *pszHeaderNext == '/' && pszHeaderNext[1] == '*' )
         {
             pszHeaderNext += 2;
-            
-            while( *pszHeaderNext != '\0' 
-                   && (*pszHeaderNext != '*' 
+
+            while( *pszHeaderNext != '\0'
+                   && (*pszHeaderNext != '*'
                        || pszHeaderNext[1] != '/' ) )
             {
                 pszHeaderNext++;
@@ -384,15 +381,15 @@ void NASAKeywordHandler::SkipWhite()
             continue;
         }
 
-        // Skip # style comments 
-         if( (*pszHeaderNext == 10 || *pszHeaderNext == 13 ||
+        // Skip # style comments
+        if( (*pszHeaderNext == 10 || *pszHeaderNext == 13 ||
  	     *pszHeaderNext == ' ' || *pszHeaderNext == '\t' )
               && pszHeaderNext[1] == '#' )
         {
             pszHeaderNext += 2;
 
             // consume till end of line.
-            while( *pszHeaderNext != '\0' 
+            while( *pszHeaderNext != '\0'
                    && *pszHeaderNext != 10
                    && *pszHeaderNext != 13 )
             {
@@ -402,13 +399,13 @@ void NASAKeywordHandler::SkipWhite()
         }
 
         // Skip white space (newline, space, tab, etc )
-        if( isspace( (unsigned char)*pszHeaderNext ) )
+        if( isspace( static_cast<unsigned char>( *pszHeaderNext ) ) )
         {
-            pszHeaderNext++; 
+            pszHeaderNext++;
             continue;
         }
-        
-        // not white space, return. 
+
+        // not white space, return.
         return;
     }
 }
@@ -421,21 +418,19 @@ const char *NASAKeywordHandler::GetKeyword( const char *pszPath,
                                             const char *pszDefault )
 
 {
-    const char *pszResult;
+    const char *pszResult = CSLFetchNameValue( papszKeywordList, pszPath );
 
-    pszResult = CSLFetchNameValue( papszKeywordList, pszPath );
     if( pszResult == NULL )
         return pszDefault;
-    else
-        return pszResult;
+
+    return pszResult;
 }
 
 /************************************************************************/
 /*                             GetKeywordList()                         */
 /************************************************************************/
- 
+
 char **NASAKeywordHandler::GetKeywordList()
 {
     return papszKeywordList;
 }
-
