@@ -1,14 +1,14 @@
 /******************************************************************************
- * 
+ *
  * Project:  VICAR Driver; JPL/MIPL VICAR Format
  * Purpose:  Implementation of VICARDataset
  * Author:   Sebastian Walter <sebastian dot walter at fu-berlin dot de>
  *
  * NOTE: This driver code is loosely based on the ISIS and PDS drivers.
- * It is not intended to diminish the contribution of the original authors 
+ * It is not intended to diminish the contribution of the original authors
  ******************************************************************************
  * Copyright (c) 2014, Sebastian Walter <sebastian dot walter at fu-berlin dot de>
- * 
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
@@ -28,26 +28,19 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#define NULL1 0
-#define NULL2 -32768
-#define NULL3 -32768.
+static const int NULL1 = 0;
+static const int NULL2 = -32768;
+static const double NULL3 = -32768.0;
 
-#ifndef PI
-#  define PI 3.1415926535897932384626433832795
-#endif
-
-#include "rawdataset.h"
+#include "cpl_string.h"
+#include "gdal_frmts.h"
 #include "ogr_spatialref.h"
-#include "cpl_string.h" 
+#include "rawdataset.h"
 #include "vicarkeywordhandler.h"
 
 #include <string>
 
-CPL_CVSID("$Id: vicardataset.cpp 31757 2015-11-25 13:54:13Z rouault $");
-
-CPL_C_START
-void GDALRegister_VICAR(void);
-CPL_C_END
+CPL_CVSID("$Id: vicardataset.cpp 33720 2016-03-15 00:39:53Z goatbar $");
 
 /************************************************************************/
 /* ==================================================================== */
@@ -61,24 +54,24 @@ class VICARDataset : public RawDataset
 
     GByte	abyHeader[10000];
     CPLString   osExternalCube;
-    
+
     VICARKeywordHandler  oKeywords;
-  
+
     int         bGotTransform;
     double      adfGeoTransform[6];
-  
+
     CPLString   osProjection;
 
-    const char *GetKeyword( const char *pszPath, 
+    const char *GetKeyword( const char *pszPath,
                             const char *pszDefault = "");
-    
+
 public:
     VICARDataset();
     ~VICARDataset();
-  
+
     virtual CPLErr GetGeoTransform( double * padfTransform );
     virtual const char *GetProjectionRef(void);
-  
+
     virtual char **GetFileList();
 
     static int          Identify( GDALOpenInfo * );
@@ -89,16 +82,14 @@ public:
 
 };
 
-
-
 /************************************************************************/
 /*                            VICARDataset()                            */
 /************************************************************************/
 
-VICARDataset::VICARDataset()
+VICARDataset::VICARDataset() :
+    fpImage(NULL),
+    bGotTransform(FALSE)
 {
-    fpImage = NULL;
-    bGotTransform = FALSE;
     adfGeoTransform[0] = 0.0;
     adfGeoTransform[1] = 1.0;
     adfGeoTransform[2] = 0.0;
@@ -126,9 +117,7 @@ VICARDataset::~VICARDataset()
 char **VICARDataset::GetFileList()
 
 {
-    char **papszFileList = NULL;
-
-    papszFileList = GDALPamDataset::GetFileList();
+    char **papszFileList = GDALPamDataset::GetFileList();
 
     if( strlen(osExternalCube) > 0 )
         papszFileList = CSLAddString( papszFileList, osExternalCube );
@@ -145,8 +134,8 @@ const char *VICARDataset::GetProjectionRef()
 {
     if( strlen(osProjection) > 0 )
         return osProjection;
-    else
-        return GDALPamDataset::GetProjectionRef();
+
+    return GDALPamDataset::GetProjectionRef();
 }
 
 /************************************************************************/
@@ -161,10 +150,8 @@ CPLErr VICARDataset::GetGeoTransform( double * padfTransform )
         memcpy( padfTransform, adfGeoTransform, sizeof(double) * 6 );
         return CE_None;
     }
-    else
-    {
-        return GDALPamDataset::GetGeoTransform( padfTransform );
-    }
+
+    return GDALPamDataset::GetGeoTransform( padfTransform );
 }
 
 /************************************************************************/
@@ -177,11 +164,12 @@ int VICARDataset::Identify( GDALOpenInfo * poOpenInfo )
     if( poOpenInfo->pabyHeader == NULL )
         return FALSE;
 
-    return strstr((char*)poOpenInfo->pabyHeader,"LBLSIZE") != NULL &&
-           strstr((char*)poOpenInfo->pabyHeader,"FORMAT") != NULL &&
-           strstr((char*)poOpenInfo->pabyHeader,"NL") != NULL &&
-           strstr((char*)poOpenInfo->pabyHeader,"NS") != NULL &&
-           strstr((char*)poOpenInfo->pabyHeader,"NB") != NULL;
+    return
+        strstr(reinterpret_cast<char *>( poOpenInfo->pabyHeader ), "LBLSIZE" ) != NULL &&
+        strstr(reinterpret_cast<char *>( poOpenInfo->pabyHeader ), "FORMAT" ) != NULL &&
+        strstr(reinterpret_cast<char *>( poOpenInfo->pabyHeader ), "NL" ) != NULL &&
+        strstr(reinterpret_cast<char *>( poOpenInfo->pabyHeader ), "NS" ) != NULL &&
+        strstr(reinterpret_cast<char *>( poOpenInfo->pabyHeader ), "NB" ) != NULL;
 }
 
 /************************************************************************/
@@ -204,83 +192,73 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
     if( fpQube == NULL )
         return NULL;
 
-    VICARDataset 	*poDS;
-
-    poDS = new VICARDataset();
+    VICARDataset *poDS = new VICARDataset();
     if( ! poDS->oKeywords.Ingest( fpQube, poOpenInfo->pabyHeader ) ) {
         VSIFCloseL( fpQube );
         delete poDS;
         return NULL;
     }
-    
+
     VSIFCloseL( fpQube );
 
-    CPLString osQubeFile;
-    osQubeFile = poOpenInfo->pszFilename;
-    GDALDataType eDataType = GDT_Byte;
-
-    int	nRows = -1;
-    int nCols = -1;
-    int nBands = 1;
-    int nSkipBytes = 0;
-    int bIsDTM = FALSE;
-    char chByteOrder = 'M';
-    double dfNoData = 0.0;
-    const char *value;
-    
     /***** CHECK ENDIANNESS **************/
-    
-    value = poDS->GetKeyword( "INTFMT" );
+
+    const char *value = poDS->GetKeyword( "INTFMT" );
     if (!EQUAL(value,"LOW") ) {
-        CPLError( CE_Failure, CPLE_OpenFailed, 
+        CPLError( CE_Failure, CPLE_OpenFailed,
                   "%s layout not supported. Abort\n\n", value);
-        return FALSE;
+        delete poDS;
+        return NULL;
     }
     value = poDS->GetKeyword( "REALFMT" );
     if (!EQUAL(value,"RIEEE") ) {
-        CPLError( CE_Failure, CPLE_OpenFailed, 
+        CPLError( CE_Failure, CPLE_OpenFailed,
                   "%s layout not supported. Abort\n\n", value);
-        return FALSE;
+        delete poDS;
+        return NULL;
     }
+
+    char chByteOrder = 'M';
     value = poDS->GetKeyword( "BREALFMT" );
     if (EQUAL(value,"VAX") ) {
         chByteOrder = 'I';
     }
-    
+
     /************ CHECK INSTRUMENT *****************/
     /************ ONLY HRSC TESTED *****************/
-    
+
+    bool bIsDTM = false;
     value = poDS->GetKeyword( "DTM.DTM_OFFSET" );
     if (!EQUAL(value,"") ) {
-        bIsDTM = TRUE;
+        bIsDTM = true;
     }
-    
-    
+
     value = poDS->GetKeyword( "BLTYPE" );
-    if (!EQUAL(value,"M94_HRSC") && bIsDTM==FALSE ) {
-        CPLError( CE_Failure, CPLE_OpenFailed, 
+    if (!EQUAL(value,"M94_HRSC") && !bIsDTM ) {
+        CPLError( CE_Failure, CPLE_OpenFailed,
                   "%s instrument not tested. Continue with caution!\n\n", value);
     }
-    
+
     /***********   Grab layout type (BSQ, BIP, BIL) ************/
-    
+
     char szLayout[10] = "BSQ"; //default to band seq.
     value = poDS->GetKeyword( "ORG" );
-    if (EQUAL(value,"BSQ") ) {
-        strcpy(szLayout,"BSQ");
-        nCols = atoi(poDS->GetKeyword("NS"));
-        nRows = atoi(poDS->GetKeyword("NL"));
-        nBands = atoi(poDS->GetKeyword("NB"));
-    }
-    else {
-        CPLError( CE_Failure, CPLE_OpenFailed, 
+    if (!EQUAL(value,"BSQ") )
+    {
+        CPLError( CE_Failure, CPLE_OpenFailed,
                   "%s layout not supported. Abort\n\n", value);
-        return FALSE;
+        delete poDS;
+        return NULL;
     }
-    
+
+    strcpy(szLayout,"BSQ");
+    const int nCols = atoi(poDS->GetKeyword("NS"));
+    const int nRows = atoi(poDS->GetKeyword("NL"));
+    const int nBands = atoi(poDS->GetKeyword("NB"));
+
     /***********   Grab record bytes  **********/
-    nSkipBytes = atoi(poDS->GetKeyword("NBB"));
-    
+    GDALDataType eDataType = GDT_Byte;
+    double dfNoData = 0.0;
     if (EQUAL( poDS->GetKeyword( "FORMAT" ), "BYTE" )) {
         eDataType = GDT_Byte;
         dfNoData = NULL1;
@@ -293,14 +271,15 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
     else if (EQUAL( poDS->GetKeyword( "FORMAT" ), "FULL" )) {
         eDataType = GDT_UInt32;
         dfNoData = 0;
-    } 
+    }
     else if (EQUAL( poDS->GetKeyword( "FORMAT" ), "REAL" )) {
         eDataType = GDT_Float32;
         dfNoData = NULL3;
         chByteOrder = 'I';
     }
     else {
-	    printf("Could not find known VICAR label entries!\n");
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "Could not find known VICAR label entries!\n");
         delete poDS;
         return NULL;
     }
@@ -308,9 +287,11 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
     if( nRows < 1 || nCols < 1 || nBands < 1 )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
-                  "File %s appears to be a VICAR file, but failed to find some required keywords.", 
+                  "File %s appears to be a VICAR file, but failed to find some "
+                  "required keywords.",
                   poDS->GetDescription() );
-        return FALSE;
+        delete poDS;
+        return NULL;
     }
 /* -------------------------------------------------------------------- */
 /*      Capture some information from the file that is of interest.     */
@@ -318,108 +299,95 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->nRasterXSize = nCols;
     poDS->nRasterYSize = nRows;
 
-    double dfULXMap=0.5;
-    double dfULYMap = 0.5;
     double dfXDim = 1.0;
     double dfYDim = 1.0;
-    double xulcenter = 0.0;
-    double yulcenter = 0.0;
 
     value = poDS->GetKeyword("MAP.MAP_SCALE");
     if (strlen(value) > 0 ) {
-        dfXDim = CPLAtof(value);
-        dfYDim = CPLAtof(value) * -1;
-        dfXDim = dfXDim * 1000.0;
-        dfYDim = dfYDim * 1000.0;
+        dfXDim = CPLAtof(value) * 1000.0;
+        dfYDim = CPLAtof(value) * -1 * 1000.0;
     }
 
-    double   dfSampleOffset_Shift;
-    double   dfLineOffset_Shift;
-    double   dfSampleOffset_Mult;
-    double   dfLineOffset_Mult;
-    
-    dfSampleOffset_Shift = 
+    const double dfSampleOffset_Shift =
         CPLAtof(CPLGetConfigOption( "PDS_SampleProjOffset_Shift", "0.5" ));
-    
-    dfLineOffset_Shift = 
+
+    const double dfLineOffset_Shift =
         CPLAtof(CPLGetConfigOption( "PDS_LineProjOffset_Shift", "0.5" ));
 
-    dfSampleOffset_Mult =
+    const double dfSampleOffset_Mult =
         CPLAtof(CPLGetConfigOption( "PDS_SampleProjOffset_Mult", "-1.0") );
 
-    dfLineOffset_Mult = 
+    const double dfLineOffset_Mult =
         CPLAtof( CPLGetConfigOption( "PDS_LineProjOffset_Mult", "1.0") );
 
     /***********   Grab LINE_PROJECTION_OFFSET ************/
+    double yulcenter = 0.0;
+    double dfULYMap = 0.5;
+
     value = poDS->GetKeyword("MAP.LINE_PROJECTION_OFFSET");
     if (strlen(value) > 0) {
         yulcenter = CPLAtof(value);
         dfULYMap = ((yulcenter + dfLineOffset_Shift) * -dfYDim * dfLineOffset_Mult);
     }
     /***********   Grab SAMPLE_PROJECTION_OFFSET ************/
+    double xulcenter = 0.0;
+    double dfULXMap=0.5;
+
     value = poDS->GetKeyword("MAP.SAMPLE_PROJECTION_OFFSET");
     if( strlen(value) > 0 ) {
         xulcenter = CPLAtof(value);
         dfULXMap = ((xulcenter + dfSampleOffset_Shift) * dfXDim * dfSampleOffset_Mult);
     }
-    
+
 /* ==================================================================== */
 /*      Get the coordinate system.                                      */
 /* ==================================================================== */
-    int	bProjectionSet = TRUE;
-    double semi_major = 0.0;
-    double semi_minor = 0.0;
-    double iflattening = 0.0;
-    double center_lat = 0.0;
-    double center_lon = 0.0;
-    double first_std_parallel = 0.0;
-    double second_std_parallel = 0.0;    
-    OGRSpatialReference oSRS;
-    
+    bool bProjectionSet = true;
+
     /***********  Grab TARGET_NAME  ************/
     /**** This is the planets name i.e. MARS ***/
-    CPLString target_name = poDS->GetKeyword("MAP.TARGET_NAME");
-     
+    const CPLString target_name = poDS->GetKeyword("MAP.TARGET_NAME");
+
     /**********   Grab MAP_PROJECTION_TYPE *****/
-    CPLString map_proj_name = 
-        poDS->GetKeyword( "MAP.MAP_PROJECTION_TYPE");
-     
+    const CPLString map_proj_name
+        = poDS->GetKeyword( "MAP.MAP_PROJECTION_TYPE");
+
     /******  Grab semi_major & convert to KM ******/
-    semi_major = 
-        CPLAtof(poDS->GetKeyword( "MAP.A_AXIS_RADIUS")) * 1000.0;
-    
+    const double semi_major
+        = CPLAtof(poDS->GetKeyword( "MAP.A_AXIS_RADIUS")) * 1000.0;
+
     /******  Grab semi-minor & convert to KM ******/
-    semi_minor = 
-        CPLAtof(poDS->GetKeyword( "MAP.C_AXIS_RADIUS")) * 1000.0;
+    const double semi_minor
+        = CPLAtof(poDS->GetKeyword( "MAP.C_AXIS_RADIUS")) * 1000.0;
 
     /***********   Grab CENTER_LAT ************/
-    center_lat = 
+    const double center_lat =
         CPLAtof(poDS->GetKeyword( "MAP.CENTER_LATITUDE"));
 
     /***********   Grab CENTER_LON ************/
-    center_lon = 
-        CPLAtof(poDS->GetKeyword( "MAP.CENTER_LONGITUDE"));
+    const double center_lon
+        = CPLAtof(poDS->GetKeyword( "MAP.CENTER_LONGITUDE"));
 
     /**********   Grab 1st std parallel *******/
-    first_std_parallel = 
+    const double first_std_parallel =
         CPLAtof(poDS->GetKeyword( "MAP.FIRST_STANDARD_PARALLEL"));
 
     /**********   Grab 2nd std parallel *******/
-    second_std_parallel = 
+    const double second_std_parallel =
         CPLAtof(poDS->GetKeyword( "MAP.SECOND_STANDARD_PARALLEL"));
-     
+
     /*** grab  PROJECTION_LATITUDE_TYPE = "PLANETOCENTRIC" ****/
     // Need to further study how ocentric/ographic will effect the gdal library.
     // So far we will use this fact to define a sphere or ellipse for some projections
     // Frank - may need to talk this over
-    char bIsGeographic = TRUE;
+    bool bIsGeographic = true;
     value = poDS->GetKeyword("MAP.COORDINATE_SYSTEM_NAME");
     if (EQUAL( value, "PLANETOCENTRIC" ))
-        bIsGeographic = FALSE; 
+        bIsGeographic = false;
 
 /**   Set oSRS projection and parameters --- all PDS supported types added if apparently supported in oSRS
       "AITOFF",  ** Not supported in GDAL??
-      "ALBERS", 
+      "ALBERS",
       "BONNE",
       "BRIESEMEISTER",   ** Not supported in GDAL??
       "CYLINDRICAL EQUAL AREA",
@@ -439,9 +407,11 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
       "STEREOGRAPHIC",
       "TRANSVERSE MERCATOR",
       "VAN DER GRINTEN",     ** Not supported in GDAL??
-      "WERNER"     ** Not supported in GDAL?? 
-**/ 
-    CPLDebug( "PDS","using projection %s\n\n", map_proj_name.c_str());
+      "WERNER"     ** Not supported in GDAL??
+**/
+    CPLDebug( "PDS", "using projection %s\n\n", map_proj_name.c_str());
+
+    OGRSpatialReference oSRS;
 
     if ((EQUAL( map_proj_name, "EQUIRECTANGULAR" )) ||
         (EQUAL( map_proj_name, "SIMPLE_CYLINDRICAL" )) ||
@@ -460,7 +430,7 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
     } else if (EQUAL( map_proj_name, "TRANSVERSE_MERCATOR" )) {
         oSRS.SetTM ( center_lat, center_lon, 1, 0, 0 );
     } else if (EQUAL( map_proj_name, "LAMBERT_CONFORMAL_CONIC" )) {
-        oSRS.SetLCC ( first_std_parallel, second_std_parallel, 
+        oSRS.SetLCC ( first_std_parallel, second_std_parallel,
                       center_lat, center_lon, 0, 0 );
     } else if (EQUAL( map_proj_name, "LAMBERT_AZIMUTHAL_EQUAL_AREA" )) {
         oSRS.SetLAEA( center_lat, center_lon, 0, 0 );
@@ -469,89 +439,93 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
     } else if (EQUAL( map_proj_name, "MOLLWEIDE" )) {
         oSRS.SetMollweide ( center_lon, 0, 0 );
     } else if (EQUAL( map_proj_name, "ALBERS" )) {
-        oSRS.SetACEA ( first_std_parallel, second_std_parallel, 
+        oSRS.SetACEA ( first_std_parallel, second_std_parallel,
                        center_lat, center_lon, 0, 0 );
     } else if (EQUAL( map_proj_name, "BONNE" )) {
         oSRS.SetBonne ( first_std_parallel, center_lon, 0, 0 );
     } else if (EQUAL( map_proj_name, "GNOMONIC" )) {
         oSRS.SetGnomonic ( center_lat, center_lon, 0, 0 );
-    } else if (EQUAL( map_proj_name, "OBLIQUE_CYLINDRICAL" )) { 
+    } else if (EQUAL( map_proj_name, "OBLIQUE_CYLINDRICAL" )) {
         // hope Swiss Oblique Cylindrical is the same
         oSRS.SetSOC ( center_lat, center_lon, 0, 0 );
     } else {
         CPLDebug( "VICAR",
                   "Dataset projection %s is not supported. Continuing...",
                   map_proj_name.c_str() );
-        bProjectionSet = FALSE;
+        bProjectionSet = false;
     }
 
     if (bProjectionSet) {
         //Create projection name, i.e. MERCATOR MARS and set as ProjCS keyword
-        CPLString proj_target_name = map_proj_name + " " + target_name;
+        const CPLString proj_target_name = map_proj_name + " " + target_name;
         oSRS.SetProjCS(proj_target_name); //set ProjCS keyword
-     
+
         //The geographic/geocentric name will be the same basic name as the body name
         //'GCS' = Geographic/Geocentric Coordinate System
-        CPLString geog_name = "GCS_" + target_name;
-        
+        const CPLString geog_name = "GCS_" + target_name;
+
         //The datum and sphere names will be the same basic name aas the planet
-        CPLString datum_name = "D_" + target_name;
+        const CPLString datum_name = "D_" + target_name;
         CPLString sphere_name = target_name; // + "_IAU_IAG");  //Might not be IAU defined so don't add
-          
+
         //calculate inverse flattening from major and minor axis: 1/f = a/(a-b)
-        if ((semi_major - semi_minor) < 0.0000001) 
+        double iflattening = 0.0;
+        if ((semi_major - semi_minor) < 0.0000001)
             iflattening = 0;
         else
             iflattening = semi_major / (semi_major - semi_minor);
-     
+
         //Set the body size but take into consideration which proj is being used to help w/ compatibility
-        //Notice that most PDS projections are spherical based on the fact that ISIS/PICS are spherical 
+        //Notice that most PDS projections are spherical based on the fact that ISIS/PICS are spherical
         //Set the body size but take into consideration which proj is being used to help w/ proj4 compatibility
         //The use of a Sphere, polar radius or ellipse here is based on how ISIS does it internally
-        if ( ( (EQUAL( map_proj_name, "STEREOGRAPHIC" ) && (fabs(center_lat) == 90)) ) || 
-             (EQUAL( map_proj_name, "POLAR_STEREOGRAPHIC" )))  
+        if ( ( (EQUAL( map_proj_name, "STEREOGRAPHIC" ) && (fabs(center_lat) == 90)) ) ||
+             (EQUAL( map_proj_name, "POLAR_STEREOGRAPHIC" )))
         {
-            if (bIsGeographic) { 
+            if (bIsGeographic) {
                 //Geograpraphic, so set an ellipse
                 oSRS.SetGeogCS( geog_name, datum_name, sphere_name,
-                                semi_major, iflattening, 
+                                semi_major, iflattening,
                                 "Reference_Meridian", 0.0 );
             } else {
-                //Geocentric, so force a sphere using the semi-minor axis. I hope... 
+                //Geocentric, so force a sphere using the semi-minor axis. I hope...
                 sphere_name += "_polarRadius";
                 oSRS.SetGeogCS( geog_name, datum_name, sphere_name,
-                                semi_minor, 0.0, 
+                                semi_minor, 0.0,
                                 "Reference_Meridian", 0.0 );
             }
         }
-        else if ( (EQUAL( map_proj_name, "SIMPLE_CYLINDRICAL" )) || 
-                  (EQUAL( map_proj_name, "EQUIDISTANT" )) || 
-                  (EQUAL( map_proj_name, "ORTHOGRAPHIC" )) || 
-                  (EQUAL( map_proj_name, "STEREOGRAPHIC" )) || 
+        else if ( (EQUAL( map_proj_name, "SIMPLE_CYLINDRICAL" )) ||
+                  (EQUAL( map_proj_name, "EQUIDISTANT" )) ||
+                  (EQUAL( map_proj_name, "ORTHOGRAPHIC" )) ||
+                  (EQUAL( map_proj_name, "STEREOGRAPHIC" )) ||
                   (EQUAL( map_proj_name, "SINUSOIDAL" )) ) {
             //isis uses the spherical equation for these projections so force a sphere
             oSRS.SetGeogCS( geog_name, datum_name, sphere_name,
-                            semi_major, 0.0, 
+                            semi_major, 0.0,
                             "Reference_Meridian", 0.0 );
-        } 
-        else if (EQUAL( map_proj_name, "EQUIRECTANGULAR" )) { 
+        }
+        else if (EQUAL( map_proj_name, "EQUIRECTANGULAR" )) {
             //isis uses local radius as a sphere, which is pre-calculated in the PDS label as the semi-major
             sphere_name += "_localRadius";
             oSRS.SetGeogCS( geog_name, datum_name, sphere_name,
-                            semi_major, 0.0, 
+                            semi_major, 0.0,
                             "Reference_Meridian", 0.0 );
-        } 
-        else { 
+        }
+        else
+        {
             //All other projections: Mercator, Transverse Mercator, Lambert Conformal, etc.
             //Geographic, so set an ellipse
             if (bIsGeographic) {
                 oSRS.SetGeogCS( geog_name, datum_name, sphere_name,
-                                semi_major, iflattening, 
+                                semi_major, iflattening,
                                 "Reference_Meridian", 0.0 );
-            } else { 
-                //Geocentric, so force a sphere. I hope... 
+            }
+            else
+            {
+                //Geocentric, so force a sphere. I hope...
                 oSRS.SetGeogCS( geog_name, datum_name, sphere_name,
-                                semi_major, 0.0, 
+                                semi_major, 0.0,
                                 "Reference_Meridian", 0.0 );
             }
         }
@@ -571,18 +545,18 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
         poDS->adfGeoTransform[4] = 0.0;
         poDS->adfGeoTransform[5] = dfYDim;
     }
-    
+
+    const CPLString osQubeFile = poOpenInfo->pszFilename;
     if( !poDS->bGotTransform )
-        poDS->bGotTransform = 
-            GDALReadWorldFile( osQubeFile, "psw", 
+        poDS->bGotTransform =
+            GDALReadWorldFile( osQubeFile, "psw",
                                poDS->adfGeoTransform );
 
     if( !poDS->bGotTransform )
-        poDS->bGotTransform = 
-            GDALReadWorldFile( osQubeFile, "wld", 
+        poDS->bGotTransform =
+            GDALReadWorldFile( osQubeFile, "wld",
                                poDS->adfGeoTransform );
 
-
 /* -------------------------------------------------------------------- */
 /*      Open target binary file.                                        */
 /* -------------------------------------------------------------------- */
@@ -593,8 +567,8 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
 
     if( poDS->fpImage == NULL )
     {
-        CPLError( CE_Failure, CPLE_OpenFailed, 
-                  "Failed to open %s with write permission.\n%s", 
+        CPLError( CE_Failure, CPLE_OpenFailed,
+                  "Failed to open %s with write permission.\n%s",
                   osQubeFile.c_str(),
                   VSIStrerror( errno ) );
         delete poDS;
@@ -607,26 +581,22 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Compute the line offsets.                                        */
 /* -------------------------------------------------------------------- */
 
-    long int        nItemSize = GDALGetDataTypeSize(eDataType)/8;
-    long int	    nLineOffset=0, nPixelOffset=0, nBandOffset=0;
-    nPixelOffset = nItemSize;
-    nLineOffset = nPixelOffset * nCols + atoi(poDS->GetKeyword("NBB")) ;
-    nBandOffset = nLineOffset * nRows;
+    const long int nItemSize = GDALGetDataTypeSize(eDataType)/8;
+    const long int nPixelOffset = nItemSize;
+    const long int nLineOffset = nPixelOffset * nCols + atoi(poDS->GetKeyword("NBB")) ;
+    const long int nBandOffset = nLineOffset * nRows;
 
-    nSkipBytes = atoi(poDS->GetKeyword("LBLSIZE"));
+    int nSkipBytes = atoi(poDS->GetKeyword("LBLSIZE"));
 
 /* -------------------------------------------------------------------- */
 /*      Create band information objects.                                */
 /* -------------------------------------------------------------------- */
-    int i;
-
-    for( i = 0; i < nBands; i++ )
+    for( int i = 0; i < nBands; i++ )
     {
-        GDALRasterBand	*poBand;
-
-        poBand = new RawRasterBand( poDS, i+1, poDS->fpImage, nSkipBytes + nBandOffset * i, 
-                                   nPixelOffset, nLineOffset, eDataType,
-#ifdef CPL_LSB                               
+        GDALRasterBand	*poBand
+            = new RawRasterBand( poDS, i+1, poDS->fpImage, nSkipBytes + nBandOffset * i,
+                                 static_cast<int>(nPixelOffset), static_cast<int>(nLineOffset), eDataType,
+#ifdef CPL_LSB
                                    chByteOrder == 'I' || chByteOrder == 'L',
 #else
                                    chByteOrder == 'M',
@@ -635,9 +605,11 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
 
         poDS->SetBand( i+1, poBand );
         poBand->SetNoDataValue( dfNoData );
-        if (bIsDTM==TRUE) {
-            poBand->SetScale( (double) CPLAtof(poDS->GetKeyword( "DTM.DTM_SCALING_FACTOR") ) );
-            poBand->SetOffset( (double) CPLAtof(poDS->GetKeyword( "DTM.DTM_OFFSET") ) );
+        if (bIsDTM) {
+            poBand->SetScale( static_cast<double>(
+                CPLAtof(poDS->GetKeyword( "DTM.DTM_SCALING_FACTOR") ) ) );
+            poBand->SetOffset( static_cast<double>(
+                CPLAtof(poDS->GetKeyword( "DTM.DTM_OFFSET") ) ) );
             const char* pszMin = poDS->GetKeyword( "DTM.DTM_MINIMUM_DN", NULL );
             const char* pszMax = poDS->GetKeyword( "DTM.DTM_MAXIMUM_DN", NULL );
             if (pszMin != NULL && pszMax != NULL )
@@ -646,12 +618,12 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
             if (pszNoData != NULL )
                 poBand->SetNoDataValue( CPLAtofM(pszNoData) );
         } else if (EQUAL( poDS->GetKeyword( "BLTYPE"), "M94_HRSC" )) {
-            float scale=CPLAtof(poDS->GetKeyword("DLRTO8.REFLECTANCE_SCALING_FACTOR","-1."));
+            double scale=CPLAtof(poDS->GetKeyword("DLRTO8.REFLECTANCE_SCALING_FACTOR","-1."));
             if (scale < 0.) {
                 scale = CPLAtof(poDS->GetKeyword( "HRCAL.REFLECTANCE_SCALING_FACTOR","1."));
             }
             poBand->SetScale( scale );
-            float offset=CPLAtof(poDS->GetKeyword("DLRTO8.REFLECTANCE_OFFSET","-1."));
+            double offset=CPLAtof(poDS->GetKeyword("DLRTO8.REFLECTANCE_OFFSET","-1."));
             if (offset < 0.) {
                 offset = CPLAtof(poDS->GetKeyword( "HRCAL.REFLECTANCE_OFFSET","0."));
             }
@@ -671,27 +643,27 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 
 /******************   HRSC    ******************************/
-   
+
     if (EQUAL( poDS->GetKeyword( "BLTYPE"), "M94_HRSC" ) ) {
         poDS->SetMetadataItem( "SPACECRAFT_NAME", poDS->GetKeyword( "M94_INSTRUMENT.INSTRUMENT_HOST_NAME") );
         poDS->SetMetadataItem( "PRODUCT_TYPE", poDS->GetKeyword( "TYPE"));
-    	
+
         if (EQUAL( poDS->GetKeyword( "M94_INSTRUMENT.DETECTOR_ID"), "MEX_HRSC_SRC" )) {
-            static const char *apszKeywords[] =  {
+            static const char * const apszKeywords[] =  {
                         "M94_ORBIT.IMAGE_TIME",
                         "FILE.EVENT_TYPE",
                         "FILE.PROCESSING_LEVEL_ID",
-                        "M94_INSTRUMENT.DETECTOR_ID", 
+                        "M94_INSTRUMENT.DETECTOR_ID",
                         "M94_CAMERAS.EXPOSURE_DURATION",
                         "HRCONVER.INSTRUMENT_TEMPERATURE", NULL
                     };
-            for( i = 0; apszKeywords[i] != NULL; i++ ) {
+            for( int i = 0; apszKeywords[i] != NULL; i++ ) {
                 const char *pszKeywordValue = poDS->GetKeyword( apszKeywords[i] );
                 if( pszKeywordValue != NULL )
                     poDS->SetMetadataItem( apszKeywords[i], pszKeywordValue );
             }
         } else {
-            static const char *apszKeywords[] =  {
+            static const char * const apszKeywords[] =  {
                 "M94_ORBIT.START_TIME", "M94_ORBIT.STOP_TIME",
                 "M94_INSTRUMENT.DETECTOR_ID",
                 "M94_CAMERAS.MACROPIXEL_SIZE",
@@ -707,21 +679,21 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
                 "HRORTHO.DTM_NAME", "HRORTHO.EXTORI_FILE_NAME", "HRORTHO.GEOMETRIC_CALIB_FILE_NAME",
                 NULL
             };
-            for( i = 0; apszKeywords[i] != NULL; i++ ) {
+            for( int i = 0; apszKeywords[i] != NULL; i++ ) {
                 const char *pszKeywordValue = poDS->GetKeyword( apszKeywords[i], NULL );
                 if( pszKeywordValue != NULL )
                     poDS->SetMetadataItem( apszKeywords[i], pszKeywordValue );
             }
         }
     }
-    if (bIsDTM==TRUE && EQUAL( poDS->GetKeyword( "MAP.TARGET_NAME"), "MARS" )) {
+    if (bIsDTM && EQUAL( poDS->GetKeyword( "MAP.TARGET_NAME"), "MARS" )) {
         poDS->SetMetadataItem( "SPACECRAFT_NAME", "MARS_EXPRESS" );
         poDS->SetMetadataItem( "PRODUCT_TYPE", "DTM");
-        static const char *apszKeywords[] = {
-            "DTM.DTM_MISSING_DN", "DTM.DTM_OFFSET", "DTM.DTM_SCALING_FACTOR", "DTM.DTM_A_AXIS_RADIUS", 
+        static const char * const apszKeywords[] = {
+            "DTM.DTM_MISSING_DN", "DTM.DTM_OFFSET", "DTM.DTM_SCALING_FACTOR", "DTM.DTM_A_AXIS_RADIUS",
             "DTM.DTM_B_AXIS_RADIUS", "DTM.DTM_C_AXIS_RADIUS", "DTM.DTM_DESC", "DTM.DTM_MINIMUM_DN",
             "DTM.DTM_MAXIMUM_DN", NULL };
-        for( i = 0; apszKeywords[i] != NULL; i++ ) {
+        for( int i = 0; apszKeywords[i] != NULL; i++ ) {
             const char *pszKeywordValue = poDS->GetKeyword( apszKeywords[i] );
             if( pszKeywordValue != NULL )
                 poDS->SetMetadataItem( apszKeywords[i], pszKeywordValue );
@@ -732,7 +704,7 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
 /******************   DAWN   ******************************/
     else if (EQUAL( poDS->GetKeyword( "INSTRUMENT_ID"), "FC2" )) {
         poDS->SetMetadataItem( "SPACECRAFT_NAME", "DAWN" );
-        static const char *apszKeywords[] =  {"ORBIT_NUMBER","FILTER_NUMBER",
+        static const char * const apszKeywords[] =  {"ORBIT_NUMBER","FILTER_NUMBER",
         "FRONT_DOOR_STATUS",
         "FIRST_LINE",
         "FIRST_LINE_SAMPLE",
@@ -748,24 +720,24 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
         "SPACECRAFT_EASTERN_LONGITUDE",
         "FOOTPRINT_POSITIVE_LONGITUDE",
             NULL };
-        for( i = 0; apszKeywords[i] != NULL; i++ ) {
+        for( int i = 0; apszKeywords[i] != NULL; i++ ) {
             const char *pszKeywordValue = poDS->GetKeyword( apszKeywords[i] );
             if( pszKeywordValue != NULL )
                 poDS->SetMetadataItem( apszKeywords[i], pszKeywordValue );
         }
-        
+
     }
-    else if (bIsDTM==TRUE && EQUAL( poDS->GetKeyword( "TARGET_NAME"), "VESTA" )) {
+    else if (bIsDTM && EQUAL( poDS->GetKeyword( "TARGET_NAME"), "VESTA" )) {
         poDS->SetMetadataItem( "SPACECRAFT_NAME", "DAWN" );
         poDS->SetMetadataItem( "PRODUCT_TYPE", "DTM");
-        static const char *apszKeywords[] = {
-            "DTM_MISSING_DN", "DTM_OFFSET", "DTM_SCALING_FACTOR", "DTM_A_AXIS_RADIUS", 
+        static const char * const apszKeywords[] = {
+            "DTM_MISSING_DN", "DTM_OFFSET", "DTM_SCALING_FACTOR", "DTM_A_AXIS_RADIUS",
             "DTM_B_AXIS_RADIUS", "DTM_C_AXIS_RADIUS", "DTM_MINIMUM_DN",
             "DTM_MAXIMUM_DN", "MAP_PROJECTION_TYPE", "COORDINATE_SYSTEM_NAME",
             "POSITIVE_LONGITUDE_DIRECTION", "MAP_SCALE",
             "CENTER_LONGITUDE", "LINE_PROJECTION_OFFSET", "SAMPLE_PROJECTION_OFFSET",
             NULL };
-        for( i = 0; apszKeywords[i] != NULL; i++ ) {
+        for( int i = 0; apszKeywords[i] != NULL; i++ ) {
             const char *pszKeywordValue = poDS->GetKeyword( apszKeywords[i] );
             if( pszKeywordValue != NULL )
                 poDS->SetMetadataItem( apszKeywords[i], pszKeywordValue );
@@ -780,6 +752,7 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
     if (EQUAL(poDS->GetKeyword( "EOL"), "1" ))
         poDS->SetMetadataItem( "END-OF-DATASET_LABEL", "PRESENT" );
     poDS->SetMetadataItem( "CONVERSION_DETAILS", "http://www.lpi.usra.edu/meetings/lpsc2014/pdf/1088.pdf" );
+    poDS->SetMetadataItem( "PIXEL-SHIFT-BUG", "CORRECTED" );
 
 /* -------------------------------------------------------------------- */
 /*      Initialize any PAM information.                                 */
@@ -798,15 +771,13 @@ GDALDataset *VICARDataset::Open( GDALOpenInfo * poOpenInfo )
 /*                             GetKeyword()                             */
 /************************************************************************/
 
-const char *VICARDataset::GetKeyword( const char *pszPath, 
+const char *VICARDataset::GetKeyword( const char *pszPath,
                                       const char *pszDefault )
 
 {
     return oKeywords.GetKeyword( pszPath, pszDefault );
 }
 
-
-
 /************************************************************************/
 /*                         GDALRegister_VICAR()                         */
 /************************************************************************/
@@ -814,22 +785,19 @@ const char *VICARDataset::GetKeyword( const char *pszPath,
 void GDALRegister_VICAR()
 
 {
-    GDALDriver	*poDriver;
+    if( GDALGetDriverByName( "VICAR" ) != NULL )
+        return;
 
-    if( GDALGetDriverByName( "VICAR" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "VICAR" );
-        poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "MIPL VICAR file" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_vicar.html" );
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->pfnOpen = VICARDataset::Open;
-        poDriver->pfnIdentify = VICARDataset::Identify;
+    poDriver->SetDescription( "VICAR" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "MIPL VICAR file" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_vicar.html" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    poDriver->pfnOpen = VICARDataset::Open;
+    poDriver->pfnIdentify = VICARDataset::Identify;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
-
