@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: doq2dataset.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: doq2dataset.cpp 33932 2016-04-09 20:01:39Z goatbar $
  *
  * Project:  USGS DOQ Driver (Second Generation Format)
  * Purpose:  Implementation of DOQ2Dataset
@@ -28,52 +28,57 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "rawdataset.h"
 #include "cpl_string.h"
+#include "gdal_frmts.h"
+#include "rawdataset.h"
 
-CPL_CVSID("$Id: doq2dataset.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: doq2dataset.cpp 33932 2016-04-09 20:01:39Z goatbar $");
 
-CPL_C_START
-void	GDALRegister_DOQ2(void);
-CPL_C_END
+static const char UTM_FORMAT[] =
+    "PROJCS[\"%s / UTM zone %dN\",GEOGCS[%s,PRIMEM[\"Greenwich\",0],"
+    "UNIT[\"degree\",0.0174532925199433]],PROJECTION[\"Transverse_Mercator\"],"
+    "PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\",%d],"
+    "PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",500000],"
+    "PARAMETER[\"false_northing\",0],%s]";
 
-#define UTM_FORMAT \
-"PROJCS[\"%s / UTM zone %dN\",GEOGCS[%s,PRIMEM[\"Greenwich\",0],UNIT[\"degree\",0.0174532925199433]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\",%d],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_northing\",0],%s]"
+static const char WGS84_DATUM[] =
+    "\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563]]";
 
-#define WGS84_DATUM \
-"\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563]]"
+static const char WGS72_DATUM[] =
+    "\"WGS 72\",DATUM[\"WGS_1972\",SPHEROID[\"NWL 10D\",6378135,298.26]]";
 
-#define WGS72_DATUM \
-"\"WGS 72\",DATUM[\"WGS_1972\",SPHEROID[\"NWL 10D\",6378135,298.26]]"
+static const char NAD27_DATUM[] =
+    "\"NAD27\",DATUM[\"North_American_Datum_1927\","
+    "SPHEROID[\"Clarke 1866\",6378206.4,294.978698213901]]";
 
-#define NAD27_DATUM \
-"\"NAD27\",DATUM[\"North_American_Datum_1927\",SPHEROID[\"Clarke 1866\",6378206.4,294.978698213901]]"
-
-#define NAD83_DATUM \
-"\"NAD83\",DATUM[\"North_American_Datum_1983\",SPHEROID[\"GRS 1980\",6378137,298.257222101]]"
+static const char NAD83_DATUM[] =
+    "\"NAD83\",DATUM[\"North_American_Datum_1983\","
+    "SPHEROID[\"GRS 1980\",6378137,298.257222101]]";
 
 /************************************************************************/
 /* ==================================================================== */
-/*				DOQ2Dataset				*/
+/*                              DOQ2Dataset                             */
 /* ==================================================================== */
 /************************************************************************/
 
 class DOQ2Dataset : public RawDataset
 {
-    VSILFILE	*fpImage;	// image data file.
-    
-    double	dfULX, dfULY;
-    double	dfXPixelSize, dfYPixelSize;
+    VSILFILE    *fpImage;  // Image data file.
 
-    char	*pszProjection;
+    double      dfULX;
+    double      dfULY;
+    double      dfXPixelSize;
+    double      dfYPixelSize;
+
+    char        *pszProjection;
 
   public:
-    		DOQ2Dataset();
-    	        ~DOQ2Dataset();
+                DOQ2Dataset();
+                ~DOQ2Dataset();
 
-    CPLErr 	GetGeoTransform( double * padfTransform );
+    CPLErr      GetGeoTransform( double * padfTransform );
     const char  *GetProjectionRef( void );
-    
+
     static GDALDataset *Open( GDALOpenInfo * );
 };
 
@@ -81,11 +86,14 @@ class DOQ2Dataset : public RawDataset
 /*                            DOQ2Dataset()                             */
 /************************************************************************/
 
-DOQ2Dataset::DOQ2Dataset()
-{
-    pszProjection = NULL;
-    fpImage = NULL;
-}
+DOQ2Dataset::DOQ2Dataset() :
+    fpImage(NULL),
+    dfULX(0.0),
+    dfULY(0.0),
+    dfXPixelSize(0.0),
+    dfYPixelSize(0.0),
+    pszProjection(NULL)
+{ }
 
 /************************************************************************/
 /*                            ~DOQ2Dataset()                            */
@@ -98,7 +106,7 @@ DOQ2Dataset::~DOQ2Dataset()
 
     CPLFree( pszProjection );
     if( fpImage != NULL )
-        VSIFCloseL( fpImage );
+        CPL_IGNORE_RET_VAL(VSIFCloseL( fpImage ));
 }
 
 /************************************************************************/
@@ -114,8 +122,8 @@ CPLErr DOQ2Dataset::GetGeoTransform( double * padfTransform )
     padfTransform[3] = dfULY;
     padfTransform[4] = 0.0;
     padfTransform[5] = -1 * dfYPixelSize;
-    
-    return( CE_None );
+
+    return CE_None;
 }
 
 /************************************************************************/
@@ -135,56 +143,61 @@ const char *DOQ2Dataset::GetProjectionRef()
 GDALDataset *DOQ2Dataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
-    int		nWidth=0, nHeight=0, nBandStorage=0, nBandTypes=0;
-    
 /* -------------------------------------------------------------------- */
-/*	We assume the user is pointing to the binary (ie. .bil) file.	*/
+/*      We assume the user is pointing to the binary (i.e. .bil) file.  */
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->nHeaderBytes < 212 )
         return NULL;
 
-    int         nLineCount = 0;
-    const char *pszLine;
-    int		nBytesPerPixel=0;
-    const char *pszDatumLong=NULL, *pszDatumShort=NULL;
-    const char *pszUnits=NULL;
-    char *pszQuadname = NULL;
-    char *pszQuadquad = NULL;
-    char *pszState = NULL;
-    int	        nZone=0, nProjType=0;
-    int		nSkipBytes=0, nBytesPerLine, i, nBandCount = 0;
-    double      dfULXMap=0.0, dfULYMap = 0.0;
-    double      dfXDim=0.0, dfYDim=0.0;
-    char	**papszMetadata = NULL;
-
-    if(! EQUALN((const char *) poOpenInfo->pabyHeader,
-                "BEGIN_USGS_DOQ_HEADER", 21) )
+    if(! STARTS_WITH_CI( reinterpret_cast<char *>( poOpenInfo->pabyHeader ),
+                         "BEGIN_USGS_DOQ_HEADER" ) )
         return NULL;
 
     VSILFILE* fp = VSIFOpenL(poOpenInfo->pszFilename, "rb");
     if (fp == NULL)
         return NULL;
 
-    /* read and discard the first line */
-    pszLine = CPLReadLineL( fp );
+    int nLineCount = 0;
+    int nBytesPerPixel = 0;
+    int nWidth = 0;
+    int nHeight = 0;
+    int nBandStorage = 0;
+    int nBandTypes = 0;
+    const char *pszDatumLong = NULL;
+    const char *pszDatumShort = NULL;
+    const char *pszUnits = NULL;
+    char *pszQuadname = NULL;
+    char *pszQuadquad = NULL;
+    char *pszState = NULL;
+    int nZone=0, nProjType=0;
+    int nSkipBytes = 0;
+    int nBandCount = 0;
+    double dfULXMap=0.0;
+    double dfULYMap = 0.0;
+    double dfXDim = 0.0;
+    double dfYDim = 0.0;
+    char **papszMetadata = NULL;
 
+    /* read and discard the first line */
+    CPL_IGNORE_RET_VAL(CPLReadLineL( fp ));
+
+    const char *pszLine = NULL;
     while( (pszLine = CPLReadLineL( fp )) != NULL )
     {
-	char    **papszTokens;
-
         nLineCount++;
 
-	if( EQUAL(pszLine,"END_USGS_DOQ_HEADER") )
+        if( EQUAL(pszLine,"END_USGS_DOQ_HEADER") )
             break;
 
-	papszTokens = CSLTokenizeString( pszLine );
+        char **papszTokens = CSLTokenizeString( pszLine );
         if( CSLCount( papszTokens ) < 2 )
         {
             CSLDestroy( papszTokens );
             break;
         }
-        
-        if( EQUAL(papszTokens[0],"SAMPLES_AND_LINES") && CSLCount(papszTokens) >= 3 )
+
+        if( EQUAL( papszTokens[0], "SAMPLES_AND_LINES" ) &&
+            CSLCount( papszTokens ) >= 3 )
         {
             nWidth = atoi(papszTokens[1]);
             nHeight = atoi(papszTokens[2]);
@@ -193,102 +206,103 @@ GDALDataset *DOQ2Dataset::Open( GDALOpenInfo * poOpenInfo )
         {
             nSkipBytes = atoi(papszTokens[1]);
         }
-        else if( EQUAL(papszTokens[0],"XY_ORIGIN") && CSLCount(papszTokens) >= 3 )
+        else if( EQUAL( papszTokens[0], "XY_ORIGIN" ) &&
+                 CSLCount( papszTokens ) >= 3 )
         {
-            dfULXMap = atof(papszTokens[1]);
-            dfULYMap = atof(papszTokens[2]);
+            dfULXMap = CPLAtof(papszTokens[1]);
+            dfULYMap = CPLAtof(papszTokens[2]);
         }
-        else if( EQUAL(papszTokens[0],"HORIZONTAL_RESOLUTION") )
+        else if( EQUAL( papszTokens[0], "HORIZONTAL_RESOLUTION" ) )
         {
-            dfXDim = dfYDim = atof(papszTokens[1]);
+            dfXDim = CPLAtof(papszTokens[1]);
+            dfYDim = dfXDim;
         }
-	else if( EQUAL(papszTokens[0],"BAND_ORGANIZATION") )
+        else if( EQUAL(papszTokens[0], "BAND_ORGANIZATION") )
         {
-            if( EQUAL(papszTokens[1],"SINGLE FILE") )
+            if( EQUAL(papszTokens[1], "SINGLE FILE") )
                 nBandStorage = 1;
-            if( EQUAL(papszTokens[1],"BSQ") )
+            if( EQUAL(papszTokens[1], "BSQ") )
                 nBandStorage = 1;
-            if( EQUAL(papszTokens[1],"BIL") )
+            if( EQUAL(papszTokens[1], "BIL") )
                 nBandStorage = 1;
-            if( EQUAL(papszTokens[1],"BIP") )
+            if( EQUAL(papszTokens[1], "BIP") )
                 nBandStorage = 4;
-	}
-	else if( EQUAL(papszTokens[0],"BAND_CONTENT") )
+        }
+        else if( EQUAL(papszTokens[0], "BAND_CONTENT") )
         {
-            if( EQUAL(papszTokens[1],"BLACK&WHITE") )
+            if( EQUAL(papszTokens[1], "BLACK&WHITE") )
                 nBandTypes = 1;
-            else if( EQUAL(papszTokens[1],"COLOR") )
+            else if( EQUAL(papszTokens[1], "COLOR") )
                 nBandTypes = 5;
-            else if( EQUAL(papszTokens[1],"RGB") )
+            else if( EQUAL(papszTokens[1], "RGB") )
                 nBandTypes = 5;
-            else if( EQUAL(papszTokens[1],"RED") )
+            else if( EQUAL(papszTokens[1], "RED") )
                 nBandTypes = 5;
-            else if( EQUAL(papszTokens[1],"GREEN") )
+            else if( EQUAL(papszTokens[1], "GREEN") )
                 nBandTypes = 5;
-            else if( EQUAL(papszTokens[1],"BLUE") )
+            else if( EQUAL(papszTokens[1], "BLUE") )
                 nBandTypes = 5;
 
             nBandCount++;
         }
-        else if( EQUAL(papszTokens[0],"BITS_PER_PIXEL") )
+        else if( EQUAL(papszTokens[0], "BITS_PER_PIXEL") )
         {
-	    nBytesPerPixel = (atoi(papszTokens[1]) / 8);
+            nBytesPerPixel = atoi(papszTokens[1]) / 8;
         }
-        else if( EQUAL(papszTokens[0],"HORIZONTAL_COORDINATE_SYSTEM") )
+        else if( EQUAL(papszTokens[0], "HORIZONTAL_COORDINATE_SYSTEM") )
         {
-	    if( EQUAL(papszTokens[1],"UTM") ) 
+            if( EQUAL(papszTokens[1], "UTM") )
                 nProjType = 1;
-	    else if( EQUAL(papszTokens[1],"SPCS") ) 
+            else if( EQUAL(papszTokens[1], "SPCS") )
                 nProjType = 2;
-	    else if( EQUAL(papszTokens[1],"GEOGRAPHIC") ) 
+            else if( EQUAL(papszTokens[1], "GEOGRAPHIC") )
                 nProjType = 0;
         }
-        else if( EQUAL(papszTokens[0],"COORDINATE_ZONE") )
+        else if( EQUAL(papszTokens[0], "COORDINATE_ZONE") )
         {
             nZone = atoi(papszTokens[1]);
         }
-        else if( EQUAL(papszTokens[0],"HORIZONTAL_UNITS") )
+        else if( EQUAL(papszTokens[0], "HORIZONTAL_UNITS") )
         {
-	    if( EQUAL(papszTokens[1],"METERS") )
+            if( EQUAL(papszTokens[1], "METERS") )
                 pszUnits = "UNIT[\"metre\",1]";
-	    else if( EQUAL(papszTokens[1],"FEET") )
+            else if( EQUAL(papszTokens[1], "FEET") )
                 pszUnits = "UNIT[\"US survey foot\",0.304800609601219]";
         }
         else if( EQUAL(papszTokens[0],"HORIZONTAL_DATUM") )
         {
-	    if( EQUAL(papszTokens[1],"NAD27") ) 
+            if( EQUAL(papszTokens[1], "NAD27") )
             {
-		pszDatumLong = NAD27_DATUM;
-		pszDatumShort = "NAD 27";
+                pszDatumLong = NAD27_DATUM;
+                pszDatumShort = "NAD 27";
             }
-	    else if( EQUAL(papszTokens[1],"WGS72") ) 
+            else if( EQUAL(papszTokens[1]," WGS72") )
             {
-		pszDatumLong = WGS72_DATUM;
-		pszDatumShort = "WGS 72";
+                pszDatumLong = WGS72_DATUM;
+                pszDatumShort = "WGS 72";
             }
-	    else if( EQUAL(papszTokens[1],"WGS84") ) 
+            else if( EQUAL(papszTokens[1], "WGS84") )
             {
-		pszDatumLong = WGS84_DATUM;
-		pszDatumShort = "WGS 84";
+                pszDatumLong = WGS84_DATUM;
+                pszDatumShort = "WGS 84";
             }
-	    else if( EQUAL(papszTokens[1],"NAD83") ) 
+            else if( EQUAL(papszTokens[1], "NAD83") )
             {
-		pszDatumLong = NAD83_DATUM;
-		pszDatumShort = "NAD 83";
+                pszDatumLong = NAD83_DATUM;
+                pszDatumShort = "NAD 83";
             }
-	    else
+            else
             {
-		pszDatumLong = "DATUM[\"unknown\"]";
-		pszDatumShort = "unknown";
+                pszDatumLong = "DATUM[\"unknown\"]";
+                pszDatumShort = "unknown";
             }
-        }    
+        }
         else
         {
             /* we want to generically capture all the other metadata */
             CPLString osMetaDataValue;
-            int  iToken;
 
-            for( iToken = 1; papszTokens[iToken] != NULL; iToken++ )
+            for( int iToken = 1; papszTokens[iToken] != NULL; iToken++ )
             {
                 if( EQUAL(papszTokens[iToken],"*") )
                     continue;
@@ -297,11 +311,11 @@ GDALDataset *DOQ2Dataset::Open( GDALOpenInfo * poOpenInfo )
                     osMetaDataValue += " " ;
                 osMetaDataValue += papszTokens[iToken];
             }
-            papszMetadata = CSLAddNameValue( papszMetadata, 
-                                             papszTokens[0], 
+            papszMetadata = CSLAddNameValue( papszMetadata,
+                                             papszTokens[0],
                                              osMetaDataValue );
         }
-	
+
         CSLDestroy( papszTokens );
     }
 
@@ -317,7 +331,7 @@ GDALDataset *DOQ2Dataset::Open( GDALOpenInfo * poOpenInfo )
         || nBandTypes < 1 || nBandTypes > 9 )
     {
         CSLDestroy( papszMetadata );
-        VSIFCloseL(fp);
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
         return NULL;
     }
 
@@ -328,37 +342,36 @@ GDALDataset *DOQ2Dataset::Open( GDALOpenInfo * poOpenInfo )
     if( nBandTypes > 5 )
     {
         CPLError( CE_Failure, CPLE_OpenFailed,
-                  "DOQ Data Type (%d) is not a supported configuration.\n",
+                  "DOQ Data Type (%d) is not a supported configuration.",
                   nBandTypes );
         CSLDestroy( papszMetadata );
-        VSIFCloseL(fp);
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
         return NULL;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Confirm the requested access is supported.                      */
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->eAccess == GA_Update )
     {
         CSLDestroy( papszMetadata );
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The DOQ2 driver does not support update access to existing"
-                  " datasets.\n" );
-        VSIFCloseL(fp);
+                  " datasets." );
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
         return NULL;
     }
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
-    DOQ2Dataset 	*poDS;
-
-    poDS = new DOQ2Dataset();
+    DOQ2Dataset *poDS = new DOQ2Dataset();
 
     poDS->nRasterXSize = nWidth;
     poDS->nRasterYSize = nHeight;
 
     poDS->SetMetadata( papszMetadata );
     CSLDestroy( papszMetadata );
+    papszMetadata = NULL;
 
     poDS->fpImage = fp;
 
@@ -370,28 +383,40 @@ GDALDataset *DOQ2Dataset::Open( GDALOpenInfo * poOpenInfo )
     else
         nBytesPerPixel *= nBandCount;
 
-    nBytesPerLine = nBytesPerPixel * nWidth;
-    
+    const int nBytesPerLine = nBytesPerPixel * nWidth;
+
 /* -------------------------------------------------------------------- */
 /*      Create band information objects.                                */
 /* -------------------------------------------------------------------- */
-    for( i = 0; i < nBandCount; i++ )
+    CPLErrorReset();
+    for( int i = 0; i < nBandCount; i++ )
     {
-        poDS->SetBand( i+1, 
+        poDS->SetBand( i+1,
             new RawRasterBand( poDS, i+1, poDS->fpImage,
                                nSkipBytes + i, nBytesPerPixel, nBytesPerLine,
                                GDT_Byte, TRUE, TRUE ) );
+        if( CPLGetLastErrorType() != CE_None )
+        {
+            delete poDS;
+            CPLFree( pszQuadname );
+            CPLFree( pszQuadquad );
+            CPLFree( pszState );
+            return NULL;
+        }
     }
 
     if (nProjType == 1)
     {
-	poDS->pszProjection = 
-            CPLStrdup(CPLSPrintf( UTM_FORMAT, pszDatumShort, nZone, 
-                                  pszDatumLong, nZone * 6 - 183, pszUnits ));
-    } 
-    else 
+        poDS->pszProjection =
+            CPLStrdup( CPLSPrintf(
+                UTM_FORMAT, pszDatumShort ? pszDatumShort : "", nZone,
+                pszDatumLong ? pszDatumLong : "",
+                nZone >= 1 && nZone <= 60 ? nZone * 6 - 183 : 0,
+                pszUnits ? pszUnits : "" ) );
+    }
+    else
     {
-	poDS->pszProjection = CPLStrdup("");
+        poDS->pszProjection = CPLStrdup("");
     }
 
     poDS->dfULX = dfULXMap;
@@ -400,9 +425,9 @@ GDALDataset *DOQ2Dataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->dfXPixelSize = dfXDim;
     poDS->dfYPixelSize = dfYDim;
 
-    if ( pszQuadname ) CPLFree( pszQuadname );
-    if ( pszQuadquad) CPLFree( pszQuadquad );
-    if ( pszState) CPLFree( pszState );
+    CPLFree( pszQuadname );
+    CPLFree( pszQuadquad );
+    CPLFree( pszState );
 
 /* -------------------------------------------------------------------- */
 /*      Initialize any PAM information.                                 */
@@ -415,7 +440,7 @@ GDALDataset *DOQ2Dataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
 
-    return( poDS );
+    return poDS;
 }
 
 /************************************************************************/
@@ -425,22 +450,18 @@ GDALDataset *DOQ2Dataset::Open( GDALOpenInfo * poOpenInfo )
 void GDALRegister_DOQ2()
 
 {
-    GDALDriver	*poDriver;
+    if( GDALGetDriverByName( "DOQ2" ) != NULL )
+        return;
 
-    if( GDALGetDriverByName( "DOQ2" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "DOQ2" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "USGS DOQ (New Style)" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_various.html#DOQ2" );
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->pfnOpen = DOQ2Dataset::Open;
+    poDriver->SetDescription( "DOQ2" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "USGS DOQ (New Style)" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_various.html#DOQ2" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    poDriver->pfnOpen = DOQ2Dataset::Open;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
-
