@@ -28,16 +28,13 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "rawdataset.h"
-#include "ogr_spatialref.h"
 #include "cpl_string.h"
+#include "gdal_frmts.h"
+#include "ogr_spatialref.h"
+#include "rawdataset.h"
 
 CPL_CVSID("$Id:  $");
 
-CPL_C_START
-void GDALRegister_EIR(void);
-CPL_C_END
-
 /************************************************************************/
 /* ==================================================================== */
 /*              EIRDataset                                              */
@@ -49,23 +46,23 @@ class EIRDataset : public RawDataset
     friend class RawRasterBand;
 
     VSILFILE  *fpImage; // image data file
-    int    bGotTransform;
+    bool   bGotTransform;
     double adfGeoTransform[6];
-    int    bHDRDirty;
+    bool   bHDRDirty;
     char **papszHDR;
     char **papszExtraFiles;
-    
+
     void        ResetKeyValue( const char *pszKey, const char *pszValue );
     const char *GetKeyValue( const char *pszKey, const char *pszDefault = "" );
 
   public:
     EIRDataset();
-    ~EIRDataset();
-    
+    virtual ~EIRDataset();
+
     virtual CPLErr GetGeoTransform( double * padfTransform );
-    
+
     virtual char **GetFileList();
-    
+
     static int          Identify( GDALOpenInfo * );
     static GDALDataset *Open( GDALOpenInfo * );
 };
@@ -81,14 +78,13 @@ class EIRDataset : public RawDataset
 /*                            EIRDataset()                             */
 /************************************************************************/
 
-EIRDataset::EIRDataset()
-{
-    fpImage = NULL;
-    bGotTransform = FALSE;
-    papszHDR = NULL;
-    papszExtraFiles = NULL;
-    bHDRDirty = FALSE;
-}
+EIRDataset::EIRDataset() :
+    fpImage(NULL),
+    bGotTransform(false),
+    bHDRDirty(false),
+    papszHDR(NULL),
+    papszExtraFiles(NULL)
+{}
 
 /************************************************************************/
 /*                            ~EIRDataset()                            */
@@ -101,21 +97,21 @@ EIRDataset::~EIRDataset()
 
     if( nBands > 0 && GetAccess() == GA_Update )
     {
-        int bNoDataSet;
-        double dfNoData;
-        RawRasterBand *poBand = (RawRasterBand *) GetRasterBand( 1 );
+        RawRasterBand *poBand
+            = reinterpret_cast<RawRasterBand *>( GetRasterBand( 1 ) );
 
-        dfNoData = poBand->GetNoDataValue(&bNoDataSet);
+        int bNoDataSet = FALSE;
+        const double dfNoData = poBand->GetNoDataValue(&bNoDataSet);
         if( bNoDataSet )
         {
-            ResetKeyValue( "NODATA", 
+            ResetKeyValue( "NODATA",
                            CPLString().Printf( "%.8g", dfNoData ) );
         }
     }
 
     if( fpImage != NULL )
-        VSIFCloseL( fpImage );
-    
+        CPL_IGNORE_RET_VAL(VSIFCloseL( fpImage ));
+
     CSLDestroy( papszHDR );
     CSLDestroy( papszExtraFiles );
 }
@@ -124,21 +120,19 @@ EIRDataset::~EIRDataset()
 /*                            GetKeyValue()                             */
 /************************************************************************/
 
-const char *EIRDataset::GetKeyValue( const char *pszKey, 
-                                      const char *pszDefault )
+const char *EIRDataset::GetKeyValue( const char *pszKey,
+                                     const char *pszDefault )
 
 {
-    int i;
-
-    for( i = 0; papszHDR[i] != NULL; i++ )
+    for( int i = 0; papszHDR[i] != NULL; i++ )
     {
         if( EQUALN(pszKey,papszHDR[i],strlen(pszKey))
             && isspace((unsigned char)papszHDR[i][strlen(pszKey)]) )
         {
             const char *pszValue = papszHDR[i] + strlen(pszKey);
-            while( isspace((unsigned char)*pszValue) )
+            while( isspace(static_cast<unsigned char>(*pszValue)) )
                 pszValue++;
-            
+
             return pszValue;
         }
     }
@@ -156,18 +150,16 @@ const char *EIRDataset::GetKeyValue( const char *pszKey,
 void EIRDataset::ResetKeyValue( const char *pszKey, const char *pszValue )
 
 {
-    int i;
-    char szNewLine[82];
-
     if( strlen(pszValue) > 65 )
     {
         CPLAssert( strlen(pszValue) <= 65 );
         return;
     }
 
-    sprintf( szNewLine, "%-15s%s", pszKey, pszValue );
+    char szNewLine[82] = { '\0' };
+    snprintf( szNewLine, sizeof(szNewLine), "%-15s%s", pszKey, pszValue );
 
-    for( i = CSLCount(papszHDR)-1; i >= 0; i-- )
+    for( int i = CSLCount(papszHDR)-1; i >= 0; i-- )
     {
         if( EQUALN(papszHDR[i],szNewLine,strlen(pszKey)+1 ) )
         {
@@ -175,13 +167,13 @@ void EIRDataset::ResetKeyValue( const char *pszKey, const char *pszValue )
             {
                 CPLFree( papszHDR[i] );
                 papszHDR[i] = CPLStrdup( szNewLine );
-                bHDRDirty = TRUE;
+                bHDRDirty = true;
             }
             return;
         }
     }
 
-    bHDRDirty = TRUE;
+    bHDRDirty = true;
     papszHDR = CSLAddString( papszHDR, szNewLine );
 }
 
@@ -198,10 +190,8 @@ CPLErr EIRDataset::GetGeoTransform( double * padfTransform )
         memcpy( padfTransform, adfGeoTransform, sizeof(double) * 6 );
         return CE_None;
     }
-    else
-    {
-        return GDALPamDataset::GetGeoTransform( padfTransform );
-    }
+
+    return GDALPamDataset::GetGeoTransform( padfTransform );
 }
 
 /************************************************************************/
@@ -211,17 +201,16 @@ CPLErr EIRDataset::GetGeoTransform( double * padfTransform )
 char **EIRDataset::GetFileList()
 
 {
-    CPLString osPath = CPLGetPath( GetDescription() );
-    CPLString osName = CPLGetBasename( GetDescription() );
-    char **papszFileList = NULL;
-    
-    // Main data file, etc. 
-    papszFileList = GDALPamDataset::GetFileList();
+    const CPLString osPath = CPLGetPath( GetDescription() );
+    const CPLString osName = CPLGetBasename( GetDescription() );
+
+    // Main data file, etc.
+    char **papszFileList = GDALPamDataset::GetFileList();
 
     // Header file.
     papszFileList = CSLInsertStrings( papszFileList, -1,
                                       papszExtraFiles );
-    
+
     return papszFileList;
 }
 
@@ -235,7 +224,7 @@ int EIRDataset::Identify( GDALOpenInfo * poOpenInfo )
     if( poOpenInfo->nHeaderBytes < 100 )
         return FALSE;
 
-    if( strstr((const char *) poOpenInfo->pabyHeader, 
+    if( strstr((const char *) poOpenInfo->pabyHeader,
                "IMAGINE_RAW_FILE" ) == NULL )
         return FALSE;
 
@@ -249,20 +238,15 @@ int EIRDataset::Identify( GDALOpenInfo * poOpenInfo )
 GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
-    int     i;
-    VSILFILE    *fp;
-    const char *    pszLine;
-    
-    
     if( !Identify( poOpenInfo ) )
         return NULL;
-                  
-    fp = VSIFOpenL( poOpenInfo->pszFilename, "r" );
+
+    VSILFILE *fp = VSIFOpenL( poOpenInfo->pszFilename, "r" );
     if( fp == NULL )
         return NULL;
-    
+
     /* header example and description
-    
+
     IMAGINE_RAW_FILE // must be on first line, by itself
     WIDTH 581        // number of columns in the image
     HEIGHT 695       // number of rows in the image
@@ -274,91 +258,91 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
     BYTE_ORDER       // LSB MSB; required for U16 U32 S16 S32 F32 F64
     DATA_OFFSET      // start of image data in raster file; default 0 bytes
     END_RAW_FILE     // end RAW file - stop reading
-    
+
     For a true color image with three bands (R, G, B) stored using 8 bits
     for each pixel in each band, DATA_TYPE equals U8 and NUM_LAYERS equals
     3 for a total of 24 bits per pixel.
-    
+
     Note that the current version of ERDAS Raw Raster Reader/Writer does
-    not support the LAYER_SKIP_BYTES, RECORD_SKIP_BYTES, TILE_WIDTH and 
-    TILE_HEIGHT directives. Since the reader does not read the PIXEL_FILES 
-    directive, the reader always assumes that the raw binary file is the 
-    dataset, and the name of this file is the name of the header without the 
+    not support the LAYER_SKIP_BYTES, RECORD_SKIP_BYTES, TILE_WIDTH and
+    TILE_HEIGHT directives. Since the reader does not read the PIXEL_FILES
+    directive, the reader always assumes that the raw binary file is the
+    dataset, and the name of this file is the name of the header without the
     extension. Currently, the reader does not support multiple raw binary
-    files in one dataset or a single file with both the header and the raw 
+    files in one dataset or a single file with both the header and the raw
     binary data at the same time.
     */
-    
-    bool         bDone = FALSE;
-    int          nRows = -1, nCols = -1, nBands = 1;
-    int          nSkipBytes = 0;
-    int          nLineCount = 0;
-    GDALDataType eDataType = GDT_Byte;
-    int          nBits = 8;
-    char         chByteOrder = 'M';
-    char         szLayout[10] = "BIL";
-    char         **papszHDR = NULL;
-    
-    // default raster file: same name with no extension
-    CPLString osPath = CPLGetPath( poOpenInfo->pszFilename );
-    CPLString osName = CPLGetBasename( poOpenInfo->pszFilename );
-    CPLString osRasterFilename = CPLFormCIFilename( osPath, osName, "" );
-    
-    // parse the header file
-    while( !bDone && (pszLine = CPLReadLineL( fp )) != NULL )
-    {
-        char    **papszTokens;
 
+    int nRows = -1;
+    int nCols = -1;
+    int nBands = 1;
+    int nSkipBytes = 0;
+    int nLineCount = 0;
+    GDALDataType eDataType = GDT_Byte;
+    int nBits = 8;
+    char chByteOrder = 'M';
+    char szLayout[10] = "BIL";
+    char **papszHDR = NULL;
+
+    // default raster file: same name with no extension
+    const CPLString osPath = CPLGetPath( poOpenInfo->pszFilename );
+    const CPLString osName = CPLGetBasename( poOpenInfo->pszFilename );
+    CPLString osRasterFilename = CPLFormCIFilename( osPath, osName, "" );
+
+    // parse the header file
+    const char *pszLine = NULL;
+    while( (pszLine = CPLReadLineL( fp )) != NULL )
+    {
         nLineCount++;
-        
-        if ( (nLineCount == 1) && !EQUAL(pszLine,"IMAGINE_RAW_FILE") ) {
+
+        if ( (nLineCount == 1) && !EQUAL(pszLine, "IMAGINE_RAW_FILE") ) {
             return NULL;
         }
-            
-        if ( (nLineCount > 50) || EQUAL(pszLine,"END_RAW_FILE") ) {
-            bDone = TRUE;
+
+        if ( (nLineCount > 50) || EQUAL(pszLine, "END_RAW_FILE") ) {
             break;
         }
-        
+
         if( strlen(pszLine) > 1000 )
             break;
 
         papszHDR = CSLAddString( papszHDR, pszLine );
 
-        papszTokens = CSLTokenizeStringComplex( pszLine, " \t", TRUE, FALSE );
+        char **papszTokens
+            = CSLTokenizeStringComplex( pszLine, " \t", TRUE, FALSE );
         if( CSLCount( papszTokens ) < 2 )
         {
             CSLDestroy( papszTokens );
             continue;
         }
-        
-        if( EQUAL(papszTokens[0],"WIDTH") )
+
+        if( EQUAL(papszTokens[0], "WIDTH") )
         {
             nCols = atoi(papszTokens[1]);
         }
-        else if( EQUAL(papszTokens[0],"HEIGHT") )
+        else if( EQUAL(papszTokens[0], "HEIGHT") )
         {
             nRows = atoi(papszTokens[1]);
         }
-        else if( EQUAL(papszTokens[0],"NUM_LAYERS") )
+        else if( EQUAL(papszTokens[0], "NUM_LAYERS") )
         {
             nBands = atoi(papszTokens[1]);
         }
-        else if( EQUAL(papszTokens[0],"PIXEL_FILES") )
+        else if( EQUAL(papszTokens[0], "PIXEL_FILES") )
         {
             osRasterFilename = CPLFormCIFilename( osPath, papszTokens[1], "" );
         }
-        else if( EQUAL(papszTokens[0],"FORMAT") )
+        else if( EQUAL(papszTokens[0], "FORMAT") )
         {
             strncpy( szLayout, papszTokens[1], sizeof(szLayout) );
             szLayout[sizeof(szLayout)-1] = '\0';
         }
-        else if( EQUAL(papszTokens[0],"DATATYPE") 
-                 || EQUAL(papszTokens[0],"DATA_TYPE") )
+        else if( EQUAL(papszTokens[0], "DATATYPE")
+                 || EQUAL(papszTokens[0], "DATA_TYPE") )
         {
             if ( EQUAL(papszTokens[1], "U1")
-                 || EQUAL(papszTokens[1], "U2") 
-                 || EQUAL(papszTokens[1], "U4") 
+                 || EQUAL(papszTokens[1], "U2")
+                 || EQUAL(papszTokens[1], "U4")
                  || EQUAL(papszTokens[1], "U8") ) {
                 nBits = 8;
                 eDataType = GDT_Byte;
@@ -388,19 +372,20 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
                 eDataType = GDT_Float64;
             }
             else {
-                CPLError( CE_Failure, CPLE_NotSupported, 
-                  "EIR driver does not support DATATYPE %s.", 
-                  papszTokens[1] );
+                CPLError(
+                    CE_Failure, CPLE_NotSupported,
+                    "EIR driver does not support DATATYPE %s.",
+                    papszTokens[1] );
                 CSLDestroy( papszTokens );
                 CSLDestroy( papszHDR );
-                VSIFCloseL( fp );
+                CPL_IGNORE_RET_VAL(VSIFCloseL( fp ));
                 return NULL;
             }
         }
-        else if( EQUAL(papszTokens[0],"BYTE_ORDER") )
+        else if( EQUAL(papszTokens[0], "BYTE_ORDER") )
         {
             // M for MSB, L for LSB
-            chByteOrder = (char) toupper(papszTokens[1][0]);
+            chByteOrder = static_cast<char>( toupper(papszTokens[1][0]) );
         }
         else if( EQUAL(papszTokens[0],"DATA_OFFSET") )
         {
@@ -409,9 +394,8 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
 
         CSLDestroy( papszTokens );
     }
-    
-    VSIFCloseL( fp );
-    
+
+    CPL_IGNORE_RET_VAL(VSIFCloseL( fp ));
 
 /* -------------------------------------------------------------------- */
 /*      Did we get the required keywords?  If not we return with        */
@@ -430,24 +414,22 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
         CSLDestroy( papszHDR );
         return NULL;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Confirm the requested access is supported.                      */
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->eAccess == GA_Update )
     {
         CSLDestroy( papszHDR );
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The EIR driver does not support update access to existing"
-                  " datasets.\n" );
+                  " datasets." );
         return NULL;
     }
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
-    EIRDataset     *poDS;
-
-    poDS = new EIRDataset();
+    EIRDataset *poDS = new EIRDataset();
 
 /* -------------------------------------------------------------------- */
 /*      Capture some information from the file that is of interest.     */
@@ -455,7 +437,6 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->nRasterXSize = nCols;
     poDS->nRasterYSize = nRows;
     poDS->papszHDR = papszHDR;
-    
 
 /* -------------------------------------------------------------------- */
 /*      Open target binary file.                                        */
@@ -463,45 +444,45 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->fpImage = VSIFOpenL( osRasterFilename.c_str(), "rb" );
     if( poDS->fpImage == NULL )
     {
-        CPLError( CE_Failure, CPLE_OpenFailed, 
-                  "Failed to open %s.\n%s", 
+        CPLError( CE_Failure, CPLE_OpenFailed,
+                  "Failed to open %s: %s",
                   osRasterFilename.c_str(), VSIStrerror( errno ) );
         delete poDS;
         return NULL;
     }
-    poDS->papszExtraFiles = 
-            CSLAddString( poDS->papszExtraFiles, 
+    poDS->papszExtraFiles =
+            CSLAddString( poDS->papszExtraFiles,
                           osRasterFilename );
 
     poDS->eAccess = poOpenInfo->eAccess;
-    
 
 /* -------------------------------------------------------------------- */
 /*      Compute the line offset.                                        */
 /* -------------------------------------------------------------------- */
-    int             nItemSize = GDALGetDataTypeSize(eDataType)/8;
-    int             nPixelOffset, nLineOffset;
-    vsi_l_offset    nBandOffset;
+    const int nItemSize = GDALGetDataTypeSizeBytes(eDataType);
+    int nPixelOffset = 0;
+    int nLineOffset = 0;
+    vsi_l_offset nBandOffset = 0;
 
-    if( EQUAL(szLayout,"BIP") )
+    if( EQUAL(szLayout, "BIP") )
     {
         nPixelOffset = nItemSize * nBands;
         nLineOffset = nPixelOffset * nCols;
-        nBandOffset = (vsi_l_offset)nItemSize;
+        nBandOffset = static_cast<vsi_l_offset>(nItemSize);
     }
-    else if( EQUAL(szLayout,"BSQ") )
+    else if( EQUAL(szLayout, "BSQ") )
     {
         nPixelOffset = nItemSize;
         nLineOffset = nPixelOffset * nCols;
-        nBandOffset = (vsi_l_offset)nLineOffset * nRows;
+        nBandOffset = static_cast<vsi_l_offset>(nLineOffset) * nRows;
     }
     else /* assume BIL */
     {
         nPixelOffset = nItemSize;
         nLineOffset = nItemSize * nBands * nCols;
-        nBandOffset = (vsi_l_offset)nItemSize * nCols;
+        nBandOffset = static_cast<vsi_l_offset>(nItemSize) * nCols;
     }
-    
+
     poDS->SetDescription( poOpenInfo->pszFilename );
     poDS->PamInitialize();
 
@@ -509,77 +490,70 @@ GDALDataset *EIRDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Create band information objects.                                */
 /* -------------------------------------------------------------------- */
     poDS->nBands = nBands;
-    for( i = 0; i < poDS->nBands; i++ )
+    for( int i = 0; i < poDS->nBands; i++ )
     {
-        RawRasterBand   *poBand;
-            
-        poBand = 
-            new RawRasterBand( poDS, i+1, poDS->fpImage,
-                                nSkipBytes + nBandOffset * i, 
+        RawRasterBand *poBand
+            = new RawRasterBand( poDS, i+1, poDS->fpImage,
+                                nSkipBytes + nBandOffset * i,
                                 nPixelOffset, nLineOffset, eDataType,
-#ifdef CPL_LSB                               
+#ifdef CPL_LSB
                                 chByteOrder == 'I' || chByteOrder == 'L',
 #else
                                 chByteOrder == 'M',
-#endif        
+#endif
                                 nBits);
 
-            
         poDS->SetBand( i+1, poBand );
     }
 
-    
 /* -------------------------------------------------------------------- */
 /*      look for a worldfile                                            */
 /* -------------------------------------------------------------------- */
-    
-    if( !poDS->bGotTransform )
-        poDS->bGotTransform = 
-            GDALReadWorldFile( poOpenInfo->pszFilename, 0, 
-                               poDS->adfGeoTransform );
 
     if( !poDS->bGotTransform )
-        poDS->bGotTransform = 
-            GDALReadWorldFile( poOpenInfo->pszFilename, "wld", 
-                               poDS->adfGeoTransform );
-    
+        poDS->bGotTransform = CPL_TO_BOOL(
+            GDALReadWorldFile( poOpenInfo->pszFilename, NULL,
+                               poDS->adfGeoTransform ) );
+
+    if( !poDS->bGotTransform )
+        poDS->bGotTransform = CPL_TO_BOOL(
+            GDALReadWorldFile( poOpenInfo->pszFilename, "wld",
+                               poDS->adfGeoTransform ) );
+
 /* -------------------------------------------------------------------- */
 /*      Initialize any PAM information.                                 */
 /* -------------------------------------------------------------------- */
     poDS->TryLoadXML();
-    
+
 /* -------------------------------------------------------------------- */
 /*      Check for overviews.                                            */
 /* -------------------------------------------------------------------- */
     poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
 
-    return( poDS );
+    return poDS;
 }
 
 
 /************************************************************************/
-/*                         GDALRegister_EIR()                          */
+/*                         GDALRegister_EIR()                           */
 /************************************************************************/
 
 void GDALRegister_EIR()
 
 {
-    GDALDriver  *poDriver;
+    if( GDALGetDriverByName( "EIR" ) != NULL )
+        return;
 
-    if( GDALGetDriverByName( "EIR" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "EIR" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "Erdas Imagine Raw" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_various.html#EIR" );
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->pfnOpen = EIRDataset::Open;
-        poDriver->pfnIdentify = EIRDataset::Identify;
+    poDriver->SetDescription( "EIR" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "Erdas Imagine Raw" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_various.html#EIR" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    poDriver->pfnOpen = EIRDataset::Open;
+    poDriver->pfnIdentify = EIRDataset::Identify;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
