@@ -28,16 +28,13 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "rawdataset.h"
-#include "ogr_spatialref.h"
 #include "cpl_string.h"
+#include "gdal_frmts.h"
+#include "ogr_spatialref.h"
+#include "rawdataset.h"
 
 CPL_CVSID("$Id: ehdrdataset.cpp 12350 2007-10-08 17:41:32Z rouault $");
 
-CPL_C_START
-void	GDALRegister_GenBin(void);
-CPL_C_END
-
 /* ==================================================================== */
 /*      Table relating USGS and ESRI state plane zones.                 */
 /* ==================================================================== */
@@ -187,7 +184,7 @@ static const int anUsgsEsriZones[] =
 
 /************************************************************************/
 /* ==================================================================== */
-/*				GenBinDataset				*/
+/*                              GenBinDataset                           */
 /* ==================================================================== */
 /************************************************************************/
 
@@ -195,9 +192,9 @@ class GenBinDataset : public RawDataset
 {
     friend class GenBinBitRasterBand;
 
-    VSILFILE	*fpImage;	// image data file.
+    VSILFILE    *fpImage;  // image data file.
 
-    int         bGotTransform;
+    bool        bGotTransform;
     double      adfGeoTransform[6];
     char       *pszProjection;
 
@@ -207,11 +204,11 @@ class GenBinDataset : public RawDataset
 
   public:
     GenBinDataset();
-    ~GenBinDataset();
-    
+    virtual ~GenBinDataset();
+
     virtual CPLErr GetGeoTransform( double * padfTransform );
     virtual const char *GetProjectionRef(void);
-    
+
     virtual char **GetFileList();
 
     static GDALDataset *Open( GDALOpenInfo * );
@@ -229,6 +226,7 @@ class GenBinBitRasterBand : public GDALPamRasterBand
 
   public:
     GenBinBitRasterBand( GenBinDataset *poDS, int nBits );
+    virtual ~GenBinBitRasterBand() {}
 
     virtual CPLErr IReadBlock( int, int, void * );
 };
@@ -237,19 +235,19 @@ class GenBinBitRasterBand : public GDALPamRasterBand
 /*                        GenBinBitRasterBand()                         */
 /************************************************************************/
 
-GenBinBitRasterBand::GenBinBitRasterBand( GenBinDataset *poDS, int nBitsIn )
+GenBinBitRasterBand::GenBinBitRasterBand( GenBinDataset *poDSIn, int nBitsIn ) :
+    nBits(nBitsIn)
 {
-    SetMetadataItem( "NBITS", 
-                     CPLString().Printf("%d",nBitsIn), 
+    SetMetadataItem( "NBITS",
+                     CPLString().Printf("%d",nBitsIn),
                      "IMAGE_STRUCTURE" );
 
-    this->poDS = poDS;
-    nBits = nBitsIn;
+    poDS = poDSIn;
     nBand = 1;
 
     eDataType = GDT_Byte;
 
-    nBlockXSize = poDS->nRasterXSize;
+    nBlockXSize = poDSIn->nRasterXSize;
     nBlockYSize = 1;
 }
 
@@ -257,73 +255,74 @@ GenBinBitRasterBand::GenBinBitRasterBand( GenBinDataset *poDS, int nBitsIn )
 /*                             IReadBlock()                             */
 /************************************************************************/
 
-CPLErr GenBinBitRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYOff,
+CPLErr GenBinBitRasterBand::IReadBlock( int /* nBlockXOff */,
+                                        int nBlockYOff,
                                         void * pImage )
 
 {
-    GenBinDataset *poGDS = (GenBinDataset *) poDS;
-    vsi_l_offset   nLineStart;
-    unsigned int   nLineBytes;
-    int            iBitOffset;
-    GByte         *pabyBuffer;
+    GenBinDataset *poGDS = reinterpret_cast<GenBinDataset *>( poDS );
 
 /* -------------------------------------------------------------------- */
 /*      Establish desired position.                                     */
 /* -------------------------------------------------------------------- */
-    nLineStart = (((vsi_l_offset)nBlockXSize) * nBlockYOff * nBits) / 8;
-    iBitOffset = (int)((((vsi_l_offset)nBlockXSize) * nBlockYOff * nBits) % 8);
-    nLineBytes = (int) ((((vsi_l_offset)nBlockXSize) * (nBlockYOff+1) * nBits + 7) / 8 - nLineStart);
+    const vsi_l_offset nLineStart
+        = (static_cast<vsi_l_offset>( nBlockXSize ) * nBlockYOff * nBits) / 8;
+    int iBitOffset = static_cast<int>(
+        ( static_cast<vsi_l_offset>( nBlockXSize ) * nBlockYOff * nBits) % 8 );
+    const unsigned int nLineBytes = static_cast<unsigned int>(
+        ( static_cast<vsi_l_offset>( nBlockXSize ) * (nBlockYOff+1) * nBits + 7)
+        / 8 - nLineStart);
 
 /* -------------------------------------------------------------------- */
 /*      Read data into buffer.                                          */
 /* -------------------------------------------------------------------- */
-    pabyBuffer = (GByte *) CPLCalloc(nLineBytes,1);
+    GByte *pabyBuffer = static_cast<GByte *>( CPLCalloc( nLineBytes, 1 ) );
 
-    if( VSIFSeekL( poGDS->fpImage, nLineStart, SEEK_SET ) != 0 
+    if( VSIFSeekL( poGDS->fpImage, nLineStart, SEEK_SET ) != 0
         || VSIFReadL( pabyBuffer, 1, nLineBytes, poGDS->fpImage) != nLineBytes )
     {
         CPLError( CE_Failure, CPLE_FileIO,
                   "Failed to read %u bytes at offset %lu.\n%s",
-                  nLineBytes, (unsigned long)nLineStart, 
+                  nLineBytes, static_cast<unsigned long>(nLineStart),
                   VSIStrerror( errno ) );
+        CPLFree( pabyBuffer );
         return CE_Failure;
     }
 
 /* -------------------------------------------------------------------- */
 /*      Copy data, promoting to 8bit.                                   */
 /* -------------------------------------------------------------------- */
-    int iX;
-
+    GByte *pafImage = reinterpret_cast<GByte *>( pImage );
     if( nBits == 1 )
     {
-        for( iX = 0; iX < nBlockXSize; iX++, iBitOffset += nBits )
+        for( int iX = 0; iX < nBlockXSize; iX++, iBitOffset += nBits )
         {
             if( pabyBuffer[iBitOffset>>3]  & (0x80 >>(iBitOffset & 7)) )
-                ((GByte *) pImage)[iX] = 1;
+                pafImage[iX] = 1;
             else
-                ((GByte *) pImage)[iX] = 0;
+                pafImage[iX] = 0;
         }
     }
     else if( nBits == 2 )
     {
-        for( iX = 0; iX < nBlockXSize; iX++, iBitOffset += nBits )
+        for( int iX = 0; iX < nBlockXSize; iX++, iBitOffset += nBits )
         {
-            ((GByte *) pImage)[iX] =
-                ((pabyBuffer[iBitOffset>>3]) >> (6-(iBitOffset&0x7)) & 0x3);
+            pafImage[iX] =
+                (pabyBuffer[iBitOffset>>3]) >> (6-(iBitOffset&0x7)) & 0x3;
         }
     }
     else if( nBits == 4 )
     {
-        for( iX = 0; iX < nBlockXSize; iX++, iBitOffset += nBits )
+        for( int iX = 0; iX < nBlockXSize; iX++, iBitOffset += nBits )
         {
             if( iBitOffset == 0 )
-                ((GByte *) pImage)[iX] = (pabyBuffer[iBitOffset>>3]) >> 4;
+                pafImage[iX] = (pabyBuffer[iBitOffset>>3]) >> 4;
             else
-                ((GByte *) pImage)[iX] = (pabyBuffer[iBitOffset>>3]) & 0xf;
+                pafImage[iX] = (pabyBuffer[iBitOffset>>3]) & 0xf;
         }
     }
     else {
-        CPLAssert( FALSE );
+        CPLAssert( false );
     }
 
     CPLFree( pabyBuffer );
@@ -333,7 +332,7 @@ CPLErr GenBinBitRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYOf
 
 /************************************************************************/
 /* ==================================================================== */
-/*				GenBinDataset				*/
+/*                              GenBinDataset                           */
 /* ==================================================================== */
 /************************************************************************/
 
@@ -341,18 +340,18 @@ CPLErr GenBinBitRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYOf
 /*                            GenBinDataset()                             */
 /************************************************************************/
 
-GenBinDataset::GenBinDataset()
+GenBinDataset::GenBinDataset() :
+    fpImage(NULL),
+    bGotTransform(false),
+    pszProjection(CPLStrdup("")),
+    papszHDR(NULL)
 {
-    fpImage = NULL;
-    pszProjection = CPLStrdup("");
-    bGotTransform = FALSE;
     adfGeoTransform[0] = 0.0;
     adfGeoTransform[1] = 1.0;
     adfGeoTransform[2] = 0.0;
     adfGeoTransform[3] = 0.0;
     adfGeoTransform[4] = 0.0;
     adfGeoTransform[5] = 1.0;
-    papszHDR = NULL;
 }
 
 /************************************************************************/
@@ -365,7 +364,7 @@ GenBinDataset::~GenBinDataset()
     FlushCache();
 
     if( fpImage != NULL )
-        VSIFCloseL( fpImage );
+        CPL_IGNORE_RET_VAL(VSIFCloseL( fpImage ));
 
     CPLFree( pszProjection );
     CSLDestroy( papszHDR );
@@ -396,10 +395,8 @@ CPLErr GenBinDataset::GetGeoTransform( double * padfTransform )
         memcpy( padfTransform, adfGeoTransform, sizeof(double) * 6 );
         return CE_None;
     }
-    else
-    {
-        return GDALPamDataset::GetGeoTransform( padfTransform );
-    }
+
+    return GDALPamDataset::GetGeoTransform( padfTransform );
 }
 
 /************************************************************************/
@@ -409,15 +406,14 @@ CPLErr GenBinDataset::GetGeoTransform( double * padfTransform )
 char **GenBinDataset::GetFileList()
 
 {
-    CPLString osPath = CPLGetPath( GetDescription() );
-    CPLString osName = CPLGetBasename( GetDescription() );
-    char **papszFileList = NULL;
+    const CPLString osPath = CPLGetPath( GetDescription() );
+    const CPLString osName = CPLGetBasename( GetDescription() );
 
-    // Main data file, etc. 
-    papszFileList = GDALPamDataset::GetFileList();
+    // Main data file, etc.
+    char **papszFileList = GDALPamDataset::GetFileList();
 
     // Header file.
-    CPLString osFilename = CPLFormCIFilename( osPath, osName, "hdr" );
+    const CPLString osFilename = CPLFormCIFilename( osPath, osName, "hdr" );
     papszFileList = CSLAddString( papszFileList, osFilename );
 
     return papszFileList;
@@ -431,8 +427,6 @@ void GenBinDataset::ParseCoordinateSystem( char **papszHdr )
 
 {
     const char *pszProjName = CSLFetchNameValue( papszHdr, "PROJECTION_NAME" );
-    OGRSpatialReference oSRS;
-
     if( pszProjName == NULL )
         return;
 
@@ -440,60 +434,59 @@ void GenBinDataset::ParseCoordinateSystem( char **papszHdr )
 /*      Translate zone and parameters into numeric form.                */
 /* -------------------------------------------------------------------- */
     int nZone = 0;
-    double adfProjParms[15];
-    const char *pszUnits = CSLFetchNameValue( papszHdr, "MAP_UNITS" );
-    const char *pszDatumName = CSLFetchNameValue( papszHdr, "DATUM_NAME" );
-
     if( CSLFetchNameValue( papszHdr, "PROJECTION_ZONE" ) )
         nZone = atoi(CSLFetchNameValue( papszHdr, "PROJECTION_ZONE" ));
 
-    memset( adfProjParms, 0, sizeof(adfProjParms) );
+#if 0
+    // TODO(schwehr): Why was this being done but not used?
+    double adfProjParms[15] = { 0.0 };
     if( CSLFetchNameValue( papszHdr, "PROJECTION_PARAMETERS" ) )
     {
-        int i;
-        char **papszTokens = CSLTokenizeString( 
+        char **papszTokens = CSLTokenizeString(
             CSLFetchNameValue( papszHdr, "PROJECTION_PARAMETERS" ) );
 
-        for( i = 0; i < 15 && papszTokens[i] != NULL; i++ )
+        for( int i = 0; i < 15 && papszTokens[i] != NULL; i++ )
             adfProjParms[i] = CPLAtofM( papszTokens[i] );
 
         CSLDestroy( papszTokens );
     }
+#endif
 
 /* -------------------------------------------------------------------- */
 /*      Handle projections.                                             */
 /* -------------------------------------------------------------------- */
+    const char *pszDatumName = CSLFetchNameValue( papszHdr, "DATUM_NAME" );
+    OGRSpatialReference oSRS;
+
     if( EQUAL(pszProjName,"UTM") && nZone != 0 )
     {
-        // honestly, I'm just getting that the negative zone for 
-        // southern hemisphere is used.
+        // Just getting that the negative zone for southern hemisphere is used.
         oSRS.SetUTM( ABS(nZone), nZone > 0 );
     }
 
     else if( EQUAL(pszProjName,"State Plane") && nZone != 0 )
     {
-        int		nPairs = sizeof(anUsgsEsriZones) / (2*sizeof(int));
-        int		i;
-        double          dfUnits = 0.0;
-        
-        for( i = 0; i < nPairs; i++ )
+        const int nPairs = sizeof(anUsgsEsriZones) / (2 * sizeof(int));
+
+        for( int i = 0; i < nPairs; i++ )
         {
             if( anUsgsEsriZones[i*2+1] == nZone )
             {
                 nZone = anUsgsEsriZones[i*2];
                 break;
             }
-            
         }
 
+        const char *pszUnits = CSLFetchNameValue( papszHdr, "MAP_UNITS" );
+        double dfUnits = 0.0;
         if( EQUAL(pszUnits,"feet") )
             dfUnits = CPLAtofM(SRS_UL_US_FOOT_CONV);
-        else if( EQUALN(pszUnits,"MET",3) )
+        else if( STARTS_WITH_CI(pszUnits, "MET") )
             dfUnits = 1.0;
         else
             pszUnits = NULL;
 
-        oSRS.SetStatePlane( ABS(nZone), 
+        oSRS.SetStatePlane( ABS(nZone),
                             pszDatumName==NULL || !EQUAL(pszDatumName,"NAD27"),
                             pszUnits, dfUnits );
     }
@@ -503,22 +496,24 @@ void GenBinDataset::ParseCoordinateSystem( char **papszHdr )
 /* -------------------------------------------------------------------- */
     if( oSRS.GetAttrNode( "GEOGCS" ) == NULL )
     {
-        if( pszDatumName != NULL 
+        if( pszDatumName != NULL
             && oSRS.SetWellKnownGeogCS( pszDatumName ) == OGRERR_NONE )
         {
             // good
         }
-        else if( CSLFetchNameValue( papszHdr, "SPHEROID_NAME" ) 
+        else if( CSLFetchNameValue( papszHdr, "SPHEROID_NAME" )
                  && CSLFetchNameValue( papszHdr, "SEMI_MAJOR_AXIS" )
                  && CSLFetchNameValue( papszHdr, "SEMI_MINOR_AXIS" ) )
         {
-            double dfSemiMajor = CPLAtofM(CSLFetchNameValue( papszHdr, "SEMI_MAJOR_AXIS"));
-            double dfSemiMinor = CPLAtofM(CSLFetchNameValue( papszHdr, "SEMI_MINOR_AXIS"));
-            
+            const double dfSemiMajor
+                = CPLAtofM(CSLFetchNameValue( papszHdr, "SEMI_MAJOR_AXIS"));
+            const double dfSemiMinor
+                = CPLAtofM(CSLFetchNameValue( papszHdr, "SEMI_MINOR_AXIS"));
+
             oSRS.SetGeogCS( CSLFetchNameValue( papszHdr, "SPHEROID_NAME" ),
                             CSLFetchNameValue( papszHdr, "SPHEROID_NAME" ),
                             CSLFetchNameValue( papszHdr, "SPHEROID_NAME" ),
-                            dfSemiMajor, 
+                            dfSemiMajor,
                             1.0 / (1.0 - dfSemiMinor/dfSemiMajor) );
         }
         else // fallback default.
@@ -530,7 +525,7 @@ void GenBinDataset::ParseCoordinateSystem( char **papszHdr )
 /* -------------------------------------------------------------------- */
     CPLFree( pszProjection );
     pszProjection = NULL;
-    
+
     oSRS.exportToWkt( &pszProjection );
 }
 
@@ -541,47 +536,42 @@ void GenBinDataset::ParseCoordinateSystem( char **papszHdr )
 GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
-    int		i, bSelectedHDR;
-    
 /* -------------------------------------------------------------------- */
-/*	We assume the user is pointing to the binary (ie. .bil) file.	*/
+/*      We assume the user is pointing to the binary (i.e. .bil) file.  */
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->nHeaderBytes < 2 )
         return NULL;
 
 /* -------------------------------------------------------------------- */
-/*      Now we need to tear apart tfhe filename to form a .HDR           */
+/*      Now we need to tear apart the filename to form a .HDR           */
 /*      filename.                                                       */
 /* -------------------------------------------------------------------- */
-    CPLString osPath = CPLGetPath( poOpenInfo->pszFilename );
-    CPLString osName = CPLGetBasename( poOpenInfo->pszFilename );
+    const CPLString osPath = CPLGetPath( poOpenInfo->pszFilename );
+    const CPLString osName = CPLGetBasename( poOpenInfo->pszFilename );
     CPLString osHDRFilename;
 
-    if( poOpenInfo->papszSiblingFiles )
+    char** papszSiblingFiles = poOpenInfo->GetSiblingFiles();
+    if( papszSiblingFiles )
     {
-        int iFile = CSLFindString(poOpenInfo->papszSiblingFiles, 
-                                  CPLFormFilename( NULL, osName, "hdr" ) );
+        const int iFile = CSLFindString(
+            papszSiblingFiles, CPLFormFilename( NULL, osName, "hdr" ) );
         if( iFile < 0 ) // return if there is no corresponding .hdr file
             return NULL;
 
-        osHDRFilename = 
-            CPLFormFilename( osPath, poOpenInfo->papszSiblingFiles[iFile],
-                             NULL );
+        osHDRFilename =
+            CPLFormFilename( osPath, papszSiblingFiles[iFile], NULL );
     }
     else
     {
         osHDRFilename = CPLFormCIFilename( osPath, osName, "hdr" );
     }
 
-    bSelectedHDR = EQUAL( osHDRFilename, poOpenInfo->pszFilename );
+    const bool bSelectedHDR = EQUAL( osHDRFilename, poOpenInfo->pszFilename );
 
 /* -------------------------------------------------------------------- */
 /*      Do we have a .hdr file?                                         */
 /* -------------------------------------------------------------------- */
-    VSILFILE	*fp;
-
-    fp = VSIFOpenL( osHDRFilename, "r" );
-    
+    VSILFILE *fp = VSIFOpenL( osHDRFilename, "r" );
     if( fp == NULL )
     {
         return NULL;
@@ -590,17 +580,18 @@ GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Read a chunk to skim for expected keywords.                     */
 /* -------------------------------------------------------------------- */
-    char achHeader[1000];
-    
-    int nRead = VSIFReadL( achHeader, 1, sizeof(achHeader) - 1, fp );
-    achHeader[nRead] = '\0';
-    VSIFSeekL( fp, 0, SEEK_SET );
+    char achHeader[1000] = { '\0' };
 
-    if( strstr( achHeader, "BANDS:" ) == NULL 
-        || strstr( achHeader, "ROWS:" ) == NULL 
+    const int nRead =
+        static_cast<int>(VSIFReadL( achHeader, 1, sizeof(achHeader) - 1, fp ));
+    achHeader[nRead] = '\0';
+    CPL_IGNORE_RET_VAL(VSIFSeekL( fp, 0, SEEK_SET ));
+
+    if( strstr( achHeader, "BANDS:" ) == NULL
+        || strstr( achHeader, "ROWS:" ) == NULL
         || strstr( achHeader, "COLS:" ) == NULL )
     {
-        VSIFCloseL( fp );
+        CPL_IGNORE_RET_VAL(VSIFCloseL( fp ));
         return NULL;
     }
 
@@ -609,13 +600,14 @@ GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     if( bSelectedHDR )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
-                  "The selected file is an Generic Binary header file, but to\n"
-                  "open Generic Binary datasets, the data file should be selected\n"
-                  "instead of the .hdr file.  Please try again selecting\n"
-                  "the raw data file corresponding to the header file: %s\n", 
-                  poOpenInfo->pszFilename );
-        VSIFCloseL( fp );
+        CPLError(
+            CE_Failure, CPLE_AppDefined,
+            "The selected file is an Generic Binary header file, but to "
+            "open Generic Binary datasets, the data file should be selected "
+            "instead of the .hdr file.  Please try again selecting"
+            "the raw data file corresponding to the header file: %s",
+            poOpenInfo->pszFilename );
+        CPL_IGNORE_RET_VAL(VSIFCloseL( fp ));
         return NULL;
     }
 
@@ -632,7 +624,7 @@ GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
             CPLString osPP = pszLine;
 
             pszLine = CPLReadLineL(fp);
-            while( pszLine != NULL 
+            while( pszLine != NULL
                    && (*pszLine == '\t' || *pszLine == ' ') )
             {
                 osPP += pszLine;
@@ -642,20 +634,18 @@ GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
         }
         else
         {
-            char *pszName;
-            CPLString osValue;
-            
-            osValue = CPLParseNameValue( pszLine, &pszName );
+            char *pszName = NULL;
+            CPLString osValue = CPLParseNameValue( pszLine, &pszName );
             osValue.Trim();
-            
+
             papszHdr = CSLSetNameValue( papszHdr, pszName, osValue );
             CPLFree( pszName );
-            
+
             pszLine = CPLReadLineL( fp );
         }
     }
 
-    VSIFCloseL( fp );
+    CPL_IGNORE_RET_VAL(VSIFCloseL( fp ));
 
     if( CSLFetchNameValue( papszHdr, "COLS" ) == NULL
         || CSLFetchNameValue( papszHdr, "ROWS" ) == NULL
@@ -668,14 +658,12 @@ GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
-    GenBinDataset     *poDS;
-
-    poDS = new GenBinDataset();
+    GenBinDataset *poDS = new GenBinDataset();
 
 /* -------------------------------------------------------------------- */
 /*      Capture some information from the file that is of interest.     */
 /* -------------------------------------------------------------------- */
-    int nBands = atoi(CSLFetchNameValue( papszHdr, "BANDS" ));
+    const int nBands = atoi(CSLFetchNameValue( papszHdr, "BANDS" ));
 
     poDS->nRasterXSize = atoi(CSLFetchNameValue( papszHdr, "COLS" ));
     poDS->nRasterYSize = atoi(CSLFetchNameValue( papszHdr, "ROWS" ));
@@ -698,8 +686,8 @@ GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
 
     if( poDS->fpImage == NULL )
     {
-        CPLError( CE_Failure, CPLE_OpenFailed, 
-                  "Failed to open %s with write permission.\n%s", 
+        CPLError( CE_Failure, CPLE_OpenFailed,
+                  "Failed to open %s with write permission.\n%s",
                   osName.c_str(), VSIStrerror( errno ) );
         delete poDS;
         return NULL;
@@ -711,7 +699,7 @@ GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Figure out the data type.                                       */
 /* -------------------------------------------------------------------- */
     const char *pszDataType = CSLFetchNameValue( papszHdr, "DATATYPE" );
-    GDALDataType eDataType;
+    GDALDataType eDataType = GDT_Unknown;
     int nBits = -1; // Only needed for partial byte types
 
     if( pszDataType == NULL )
@@ -726,7 +714,7 @@ GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
         eDataType = GDT_Float64;
     else if( EQUAL(pszDataType,"U8") )
         eDataType = GDT_Byte;
-    else if( EQUAL(pszDataType,"U1") 
+    else if( EQUAL(pszDataType,"U1")
              || EQUAL(pszDataType,"U2")
              || EQUAL(pszDataType,"U4") )
     {
@@ -734,7 +722,7 @@ GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
         eDataType = GDT_Byte;
         if( nBands != 1 )
         {
-            CPLError( CE_Failure, CPLE_OpenFailed, 
+            CPLError( CE_Failure, CPLE_OpenFailed,
                       "Only one band is supported for U1/U2/U4 data type" );
             delete poDS;
             return NULL;
@@ -752,40 +740,44 @@ GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Do we need byte swapping?                                       */
 /* -------------------------------------------------------------------- */
     const char *pszBYTE_ORDER = CSLFetchNameValue(papszHdr,"BYTE_ORDER");
-    int bNative = TRUE;
-    
+    bool bNative = true;
+
     if( pszBYTE_ORDER != NULL )
     {
 #ifdef CPL_LSB
-        bNative = EQUALN(pszBYTE_ORDER,"LSB",3);
+        bNative = STARTS_WITH_CI(pszBYTE_ORDER, "LSB");
 #else
-        bNative = !EQUALN(pszBYTE_ORDER,"LSB",3);
-#endif        
+        bNative = !STARTS_WITH_CI(pszBYTE_ORDER, "LSB");
+#endif
     }
 
 /* -------------------------------------------------------------------- */
-/*	Work out interleaving info.					*/
+/*      Work out interleaving info.                                     */
 /* -------------------------------------------------------------------- */
-    int nItemSize = GDALGetDataTypeSize(eDataType)/8;
-    const char *pszInterleaving = CSLFetchNameValue(papszHdr,"INTERLEAVING");
-    int             nPixelOffset, nLineOffset;
-    vsi_l_offset    nBandOffset;
-    int bIntOverflow = FALSE;
+    const int nItemSize = GDALGetDataTypeSizeBytes(eDataType);
+    int nPixelOffset = 0;
+    int nLineOffset = 0;
+    vsi_l_offset nBandOffset = 0;
+    bool bIntOverflow = false;
 
+    const char *pszInterleaving = CSLFetchNameValue(papszHdr, "INTERLEAVING");
     if( pszInterleaving == NULL )
         pszInterleaving = "BIL";
-    
+
     if( EQUAL(pszInterleaving,"BSQ") || EQUAL(pszInterleaving,"NA") )
     {
         nPixelOffset = nItemSize;
-        if (poDS->nRasterXSize > INT_MAX / nItemSize) bIntOverflow = TRUE;
+        if( poDS->nRasterXSize > INT_MAX / nItemSize )
+            bIntOverflow = true;
         nLineOffset = nItemSize * poDS->nRasterXSize;
-        nBandOffset = nLineOffset * poDS->nRasterYSize;
+        nBandOffset =
+            nLineOffset * static_cast<vsi_l_offset>(poDS->nRasterYSize);
     }
     else if( EQUAL(pszInterleaving,"BIP") )
     {
         nPixelOffset = nItemSize * nBands;
-        if (poDS->nRasterXSize > INT_MAX / nPixelOffset) bIntOverflow = TRUE;
+        if( poDS->nRasterXSize > INT_MAX / nPixelOffset )
+            bIntOverflow = true;
         nLineOffset = nPixelOffset * poDS->nRasterXSize;
         nBandOffset = nItemSize;
     }
@@ -797,16 +789,17 @@ GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
                       pszInterleaving );
 
         nPixelOffset = nItemSize;
-        if (poDS->nRasterXSize > INT_MAX / (nPixelOffset * nBands)) bIntOverflow = TRUE;
+        if( poDS->nRasterXSize > INT_MAX / (nPixelOffset * nBands) )
+            bIntOverflow = true;
         nLineOffset = nPixelOffset * nBands * poDS->nRasterXSize;
-        nBandOffset = nItemSize * poDS->nRasterXSize;
+        nBandOffset = nItemSize * static_cast<vsi_l_offset>(poDS->nRasterXSize);
     }
 
     if (bIntOverflow)
     {
         delete poDS;
-        CPLError( CE_Failure, CPLE_AppDefined, 
-                  "Int overflow occured.");
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "Int overflow occurred." );
         return NULL;
     }
 
@@ -817,18 +810,20 @@ GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Create band information objects.                                */
 /* -------------------------------------------------------------------- */
     poDS->nBands = nBands;
-    for( i = 0; i < poDS->nBands; i++ )
+    for( int i = 0; i < poDS->nBands; i++ )
     {
         if( nBits != -1 )
         {
             poDS->SetBand( i+1, new GenBinBitRasterBand( poDS, nBits ) );
         }
         else
-            poDS->SetBand( 
-                i+1, 
+        {
+            poDS->SetBand(
+                i+1,
                 new RawRasterBand( poDS, i+1, poDS->fpImage,
                                    nBandOffset * i, nPixelOffset, nLineOffset,
                                    eDataType, bNative, TRUE ) );
+        }
     }
 
 /* -------------------------------------------------------------------- */
@@ -839,10 +834,14 @@ GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
         && CSLFetchNameValue(papszHdr,"LR_X_COORDINATE") != NULL
         && CSLFetchNameValue(papszHdr,"LR_Y_COORDINATE") != NULL )
     {
-        double dfULX = CPLAtofM(CSLFetchNameValue(papszHdr,"UL_X_COORDINATE"));
-        double dfULY = CPLAtofM(CSLFetchNameValue(papszHdr,"UL_Y_COORDINATE"));
-        double dfLRX = CPLAtofM(CSLFetchNameValue(papszHdr,"LR_X_COORDINATE"));
-        double dfLRY = CPLAtofM(CSLFetchNameValue(papszHdr,"LR_Y_COORDINATE"));
+        const double dfULX =
+            CPLAtofM(CSLFetchNameValue(papszHdr,"UL_X_COORDINATE"));
+        const double dfULY =
+            CPLAtofM(CSLFetchNameValue(papszHdr,"UL_Y_COORDINATE"));
+        const double dfLRX =
+            CPLAtofM(CSLFetchNameValue(papszHdr,"LR_X_COORDINATE"));
+        const double dfLRY =
+            CPLAtofM(CSLFetchNameValue(papszHdr,"LR_Y_COORDINATE"));
 
         poDS->adfGeoTransform[1] = (dfLRX - dfULX) / (poDS->nRasterXSize-1);
         poDS->adfGeoTransform[2] = 0.0;
@@ -852,7 +851,7 @@ GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
         poDS->adfGeoTransform[0] = dfULX - poDS->adfGeoTransform[1] * 0.5;
         poDS->adfGeoTransform[3] = dfULY - poDS->adfGeoTransform[5] * 0.5;
 
-        poDS->bGotTransform = TRUE;
+        poDS->bGotTransform = true;
     }
 
 /* -------------------------------------------------------------------- */
@@ -864,37 +863,36 @@ GDALDataset *GenBinDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Initialize any PAM information.                                 */
 /* -------------------------------------------------------------------- */
     poDS->TryLoadXML();
-    
+
 /* -------------------------------------------------------------------- */
 /*      Check for overviews.                                            */
 /* -------------------------------------------------------------------- */
     poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
 
-    return( poDS );
+    return poDS;
 }
 
 /************************************************************************/
-/*                         GDALRegister_GenBin()                          */
+/*                         GDALRegister_GenBin()                        */
 /************************************************************************/
 
 void GDALRegister_GenBin()
 
 {
-    GDALDriver	*poDriver;
+    if( GDALGetDriverByName( "GenBin" ) != NULL )
+        return;
 
-    if( GDALGetDriverByName( "GenBin" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "GenBin" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "Generic Binary (.hdr Labelled)" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_various.html#GenBin" );
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->pfnOpen = GenBinDataset::Open;
+    poDriver->SetDescription( "GenBin" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
+                               "Generic Binary (.hdr Labelled)" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,
+                               "frmt_various.html#GenBin" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    poDriver->pfnOpen = GenBinDataset::Open;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
