@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: roipacdataset.cpp 34205 2016-05-10 14:14:49Z rouault $
+ * $Id: roipacdataset.cpp 34639 2016-07-12 08:44:29Z rouault $
  *
  * Project:  ROI_PAC Raster Reader
  * Purpose:  Implementation of the ROI_PAC raster reader
@@ -31,7 +31,7 @@
 #include "ogr_spatialref.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: roipacdataset.cpp 34205 2016-05-10 14:14:49Z rouault $");
+CPL_CVSID("$Id: roipacdataset.cpp 34639 2016-07-12 08:44:29Z rouault $");
 
 /************************************************************************/
 /* ==================================================================== */
@@ -361,8 +361,25 @@ GDALDataset *ROIPACDataset::Open( GDALOpenInfo *poOpenInfo )
     }
     else { /* PIXEL */
         nPixelOffset = GDALGetDataTypeSize(eDataType)/8 * nBands;
-        nLineOffset = nPixelOffset * nWidth * nBands;
+        nLineOffset = nPixelOffset * nWidth;
         nBandOffset = GDALGetDataTypeSize(eDataType)/8;
+
+        if( nBands > 1 )
+        {
+            // GDAL 2.0.[0-3] and 2.1.0  had a value of nLineOffset that was 
+            // equal to the theoretical nLineOffset multiplied by nBands...
+            VSIFSeekL( poDS->fpImage, 0, SEEK_END );
+            const GUIntBig nWrongFileSize = GDALGetDataTypeSizeBytes(eDataType) *
+                                            nWidth * (static_cast<GUIntBig>(nFileLength - 1) * nBands * nBands + nBands);
+            if( VSIFTellL( poDS->fpImage ) == nWrongFileSize )
+            {
+                CPLError(CE_Warning, CPLE_AppDefined,
+                         "This file has been incorrectly generated by an older "
+                         "GDAL version whose line offset computation was erroneous. "
+                         "Taking that into account, but the file should be re-encoded ideally");
+                nLineOffset = nLineOffset * nBands;
+            }
+        }
     }
     poDS->nBands = nBands;
     for (int b = 0; b < nBands; b++)
