@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: snodasdataset.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: snodasdataset.cpp 33864 2016-04-02 11:50:14Z goatbar $
  *
  * Project:  SNODAS driver
  * Purpose:  Implementation of SNODASDataset
@@ -27,18 +27,15 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "rawdataset.h"
-#include "ogr_srs_api.h"
 #include "cpl_string.h"
+#include "gdal_frmts.h"
+#include "ogr_srs_api.h"
+#include "rawdataset.h"
 
-CPL_CVSID("$Id: snodasdataset.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: snodasdataset.cpp 33864 2016-04-02 11:50:14Z goatbar $");
 
 // g++ -g -Wall -fPIC frmts/raw/snodasdataset.cpp -shared -o gdal_SNODAS.so -Iport -Igcore -Ifrmts/raw -Iogr -L. -lgdal
 
-CPL_C_START
-void    GDALRegister_SNODAS(void);
-CPL_C_END
-
 /************************************************************************/
 /* ==================================================================== */
 /*                            SNODASDataset                             */
@@ -63,7 +60,7 @@ class SNODASDataset : public RawDataset
 
   public:
                     SNODASDataset();
-                    ~SNODASDataset();
+    virtual ~SNODASDataset();
 
     virtual CPLErr GetGeoTransform( double * padfTransform );
     virtual const char *GetProjectionRef(void);
@@ -83,12 +80,13 @@ class SNODASDataset : public RawDataset
 
 class SNODASRasterBand : public RawRasterBand
 {
-    public:
+  public:
             SNODASRasterBand(VSILFILE* fpRaw, int nXSize, int nYSize);
+    virtual ~SNODASRasterBand() {}
 
-        virtual double GetNoDataValue( int *pbSuccess = NULL );
-        virtual double GetMinimum( int *pbSuccess = NULL );
-        virtual double GetMaximum(int *pbSuccess = NULL );
+    virtual double GetNoDataValue( int *pbSuccess = NULL );
+    virtual double GetMinimum( int *pbSuccess = NULL );
+    virtual double GetMaximum(int *pbSuccess = NULL );
 };
 
 
@@ -96,9 +94,9 @@ class SNODASRasterBand : public RawRasterBand
 /*                         SNODASRasterBand()                           */
 /************************************************************************/
 
-SNODASRasterBand::SNODASRasterBand(VSILFILE* fpRaw,
+SNODASRasterBand::SNODASRasterBand(VSILFILE* fpRawIn,
                                    int nXSize, int nYSize) :
-    RawRasterBand( fpRaw, 0, 2,
+    RawRasterBand( fpRawIn, 0, 2,
                    nXSize * 2, GDT_Int16,
                    !CPL_IS_LSB, nXSize, nYSize, TRUE, TRUE)
 {
@@ -110,13 +108,14 @@ SNODASRasterBand::SNODASRasterBand(VSILFILE* fpRaw,
 
 double SNODASRasterBand::GetNoDataValue( int *pbSuccess )
 {
-    SNODASDataset* poGDS = (SNODASDataset*) poDS;
+    SNODASDataset* poGDS = reinterpret_cast<SNODASDataset *>( poDS );
     if (pbSuccess)
         *pbSuccess = poGDS->bHasNoData;
+
     if (poGDS->bHasNoData)
         return poGDS->dfNoData;
-    else
-        return RawRasterBand::GetNoDataValue(pbSuccess);
+
+    return RawRasterBand::GetNoDataValue(pbSuccess);
 }
 
 /************************************************************************/
@@ -125,13 +124,14 @@ double SNODASRasterBand::GetNoDataValue( int *pbSuccess )
 
 double SNODASRasterBand::GetMinimum( int *pbSuccess )
 {
-    SNODASDataset* poGDS = (SNODASDataset*) poDS;
+    SNODASDataset* poGDS = reinterpret_cast<SNODASDataset *>( poDS );
     if (pbSuccess)
         *pbSuccess = poGDS->bHasMin;
+
     if (poGDS->bHasMin)
         return poGDS->dfMin;
-    else
-        return RawRasterBand::GetMinimum(pbSuccess);
+
+    return RawRasterBand::GetMinimum(pbSuccess);
 }
 
 /************************************************************************/
@@ -140,13 +140,14 @@ double SNODASRasterBand::GetMinimum( int *pbSuccess )
 
 double SNODASRasterBand::GetMaximum( int *pbSuccess )
 {
-    SNODASDataset* poGDS = (SNODASDataset*) poDS;
+    SNODASDataset* poGDS = reinterpret_cast<SNODASDataset *>( poDS );
     if (pbSuccess)
         *pbSuccess = poGDS->bHasMax;
+
     if (poGDS->bHasMax)
         return poGDS->dfMax;
-    else
-        return RawRasterBand::GetMaximum(pbSuccess);
+
+    return RawRasterBand::GetMaximum(pbSuccess);
 }
 
 /************************************************************************/
@@ -159,21 +160,21 @@ double SNODASRasterBand::GetMaximum( int *pbSuccess )
 /*                           SNODASDataset()                            */
 /************************************************************************/
 
-SNODASDataset::SNODASDataset()
+SNODASDataset::SNODASDataset() :
+    bGotTransform(FALSE),
+    bHasNoData(FALSE),
+    dfNoData(0.0),
+    bHasMin(FALSE),
+    dfMin(0.0),
+    bHasMax(FALSE),
+    dfMax(0.0)
 {
-    bGotTransform = FALSE;
     adfGeoTransform[0] = 0.0;
     adfGeoTransform[1] = 1.0;
     adfGeoTransform[2] = 0.0;
     adfGeoTransform[3] = 0.0;
     adfGeoTransform[4] = 0.0;
     adfGeoTransform[5] = 1.0;
-    bHasNoData = FALSE;
-    dfNoData = 0.0;
-    bHasMin = FALSE;
-    dfMin = 0.0;
-    bHasMax = FALSE;
-    dfMax = 0.0;
 }
 
 /************************************************************************/
@@ -208,10 +209,9 @@ CPLErr SNODASDataset::GetGeoTransform( double * padfTransform )
         memcpy( padfTransform, adfGeoTransform, sizeof(double) * 6 );
         return CE_None;
     }
-    else
-    {
-        return GDALPamDataset::GetGeoTransform( padfTransform );
-    }
+
+    return GDALPamDataset::GetGeoTransform( padfTransform );
+
 }
 
 
@@ -238,10 +238,7 @@ int SNODASDataset::Identify( GDALOpenInfo * poOpenInfo )
     if (poOpenInfo->nHeaderBytes == 0)
         return FALSE;
 
-    return EQUALN((const char*)poOpenInfo->pabyHeader,
-                  "Format version: NOHRSC GIS/RS raster file v1.1",
-                  strlen("Format version: NOHRSC GIS/RS raster file v1.1"));
-}
+    return STARTS_WITH_CI((const char*)poOpenInfo->pabyHeader, "Format version: NOHRSC GIS/RS raster file v1.1");}
 
 /************************************************************************/
 /*                                Open()                                */
@@ -253,34 +250,36 @@ GDALDataset *SNODASDataset::Open( GDALOpenInfo * poOpenInfo )
     if( !Identify(poOpenInfo) )
         return NULL;
 
-    VSILFILE    *fp;
-
-    fp = VSIFOpenL( poOpenInfo->pszFilename, "r" );
-
+    VSILFILE *fp = VSIFOpenL( poOpenInfo->pszFilename, "r" );
     if( fp == NULL )
     {
         return NULL;
     }
 
-    const char *    pszLine;
     int             nRows = -1, nCols = -1;
     CPLString       osDataFilename;
-    int             bIsInteger = FALSE, bIs2Bytes = FALSE;
+    bool bIsInteger = false;
+    bool bIs2Bytes = false;
     double          dfNoData = 0;
-    int             bHasNoData = FALSE;
+    bool bHasNoData = false;
     double          dfMin = 0;
-    int             bHasMin = FALSE;
+    bool bHasMin = false;
     double          dfMax = 0;
-    int             bHasMax = FALSE;
+    bool bHasMax = false;
     double          dfMinX = 0.0, dfMinY = 0.0, dfMaxX = 0.0, dfMaxY = 0.0;
-    int             bHasMinX = FALSE, bHasMinY = FALSE, bHasMaxX = FALSE, bHasMaxY = FALSE;
-    int             bNotProjected = FALSE, bIsWGS84 = FALSE;
+    bool bHasMinX = false;
+    bool bHasMinY = false;
+    bool bHasMaxX = false;
+    bool bHasMaxY = false;
+    bool bNotProjected = false;
+    bool bIsWGS84 = false;
     CPLString       osDescription, osDataUnits;
     int             nStartYear = -1, nStartMonth = -1, nStartDay = -1,
                     nStartHour = -1, nStartMinute = -1, nStartSecond = -1;
     int             nStopYear = -1, nStopMonth = -1, nStopDay = -1,
                     nStopHour = -1, nStopMinute = -1, nStopSecond = -1;
 
+    const char *pszLine;
     while( (pszLine = CPLReadLine2L( fp, 256, NULL )) != NULL )
     {
         char** papszTokens = CSLTokenizeStringComplex( pszLine, ":", TRUE, FALSE );
@@ -357,44 +356,44 @@ GDALDataset *SNODASDataset::Open( GDALOpenInfo * poOpenInfo )
         }
         else if( EQUAL(papszTokens[0],"No data value"))
         {
-            bHasNoData = TRUE;
+            bHasNoData = true;
             dfNoData = CPLAtofM(papszTokens[1]);
         }
         else if( EQUAL(papszTokens[0],"Minimum data value"))
         {
-            bHasMin = TRUE;
+            bHasMin = true;
             dfMin = CPLAtofM(papszTokens[1]);
         }
         else if( EQUAL(papszTokens[0],"Maximum data value"))
         {
-            bHasMax = TRUE;
+            bHasMax = true;
             dfMax = CPLAtofM(papszTokens[1]);
         }
         else if( EQUAL(papszTokens[0],"Minimum x-axis coordinate") )
         {
-            bHasMinX = TRUE;
+            bHasMinX = true;
             dfMinX = CPLAtofM(papszTokens[1]);
         }
         else if( EQUAL(papszTokens[0],"Minimum y-axis coordinate") )
         {
-            bHasMinY = TRUE;
+            bHasMinY = true;
             dfMinY = CPLAtofM(papszTokens[1]);
         }
         else if( EQUAL(papszTokens[0],"Maximum x-axis coordinate") )
         {
-            bHasMaxX = TRUE;
+            bHasMaxX = true;
             dfMaxX = CPLAtofM(papszTokens[1]);
         }
         else if( EQUAL(papszTokens[0],"Maximum y-axis coordinate") )
         {
-            bHasMaxY = TRUE;
+            bHasMaxY = true;
             dfMaxY = CPLAtofM(papszTokens[1]);
         }
 
         CSLDestroy( papszTokens );
     }
 
-    VSIFCloseL( fp );
+    CPL_IGNORE_RET_VAL(VSIFCloseL( fp ));
 
 /* -------------------------------------------------------------------- */
 /*      Did we get the required keywords?  If not we return with        */
@@ -427,9 +426,7 @@ GDALDataset *SNODASDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
-    SNODASDataset     *poDS;
-
-    poDS = new SNODASDataset();
+    SNODASDataset *poDS = new SNODASDataset();
 
     poDS->nRasterXSize = nCols;
     poDS->nRasterYSize = nRows;
@@ -483,7 +480,7 @@ GDALDataset *SNODASDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Check for overviews.                                            */
 /* -------------------------------------------------------------------- */
     poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
-   
+
     return( poDS );
 }
 
@@ -494,23 +491,21 @@ GDALDataset *SNODASDataset::Open( GDALOpenInfo * poOpenInfo )
 void GDALRegister_SNODAS()
 
 {
-    GDALDriver  *poDriver;
+    if( GDALGetDriverByName( "SNODAS" ) != NULL )
+        return;
 
-    if( GDALGetDriverByName( "SNODAS" ) == NULL )
-    {
-        poDriver = new GDALDriver();
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->SetDescription( "SNODAS" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
-                                   "Snow Data Assimilation System" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,
-                                   "frmt_various.html#SNODAS" );
-        poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "hdr" );
+    poDriver->SetDescription( "SNODAS" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
+                               "Snow Data Assimilation System" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_various.html#SNODAS" );
+    poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "hdr" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
-        poDriver->pfnOpen = SNODASDataset::Open;
-        poDriver->pfnIdentify = SNODASDataset::Identify;
+    poDriver->pfnOpen = SNODASDataset::Open;
+    poDriver->pfnIdentify = SNODASDataset::Identify;
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
