@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: rikdataset.cpp 29720 2015-08-21 20:57:27Z rouault $
+ * $Id: rikdataset.cpp 33717 2016-03-14 06:29:14Z goatbar $
  *
  * Project:  RIK Reader
  * Purpose:  All code for RIK Reader
@@ -28,15 +28,12 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include <float.h>
+#include <cfloat>
 #include <zlib.h>
+#include "gdal_frmts.h"
 #include "gdal_pam.h"
 
-CPL_CVSID("$Id: rikdataset.cpp 29720 2015-08-21 20:57:27Z rouault $");
-
-CPL_C_START
-void	GDALRegister_RIK(void);
-CPL_C_END
+CPL_CVSID("$Id: rikdataset.cpp 33717 2016-03-14 06:29:14Z goatbar $");
 
 #define RIK_HEADER_DEBUG 0
 #define RIK_CLEAR_DEBUG 0
@@ -169,16 +166,16 @@ class RIKRasterBand : public GDALPamRasterBand
 /*                           RIKRasterBand()                            */
 /************************************************************************/
 
-RIKRasterBand::RIKRasterBand( RIKDataset *poDS, int nBand )
+RIKRasterBand::RIKRasterBand( RIKDataset *poDSIn, int nBandIn )
 
 {
-    this->poDS = poDS;
-    this->nBand = nBand;
+    poDS = poDSIn;
+    nBand = nBandIn;
 
     eDataType = GDT_Byte;
 
-    nBlockXSize = poDS->nBlockXSize;
-    nBlockYSize = poDS->nBlockYSize;
+    nBlockXSize = poDSIn->nBlockXSize;
+    nBlockYSize = poDSIn->nBlockYSize;
 }
 
 /************************************************************************/
@@ -206,9 +203,7 @@ static int GetNextLZWCode( int codeBits,
 
     while( bitsLeftToGo > 0 )
     {
-        int tmp;
-
-        tmp = blockData[filePos];
+        int tmp = blockData[filePos];
         tmp = tmp >> bitsTaken;
 
         if( bitsLeftToGo < 8 )
@@ -248,7 +243,8 @@ static void OutputPixel( GByte pixel,
 
 {
     if( imagePos < imageWidth && imageLine >= 0)
-        ((GByte *) image)[imagePos + imageLine * imageWidth] = pixel;
+      reinterpret_cast<GByte *>( image )[imagePos + imageLine * imageWidth]
+          = pixel;
 
     imagePos++;
 
@@ -278,18 +274,13 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                                   void * pImage )
 
 {
-    RIKDataset *poRDS = (RIKDataset *) poDS;
-    GByte *blockData;
-    GUInt32 blocks;
-    GUInt32 nBlockIndex;
-    GUInt32 nBlockOffset;
-    GUInt32 nBlockSize;
+    RIKDataset *poRDS = reinterpret_cast<RIKDataset *>( poDS );
 
-    blocks = poRDS->nHorBlocks * poRDS->nVertBlocks;
-    nBlockIndex = nBlockXOff + nBlockYOff * poRDS->nHorBlocks;
-    nBlockOffset = poRDS->pOffsets[nBlockIndex];
+    const GUInt32 blocks = poRDS->nHorBlocks * poRDS->nVertBlocks;
+    const GUInt32 nBlockIndex = nBlockXOff + nBlockYOff * poRDS->nHorBlocks;
+    const GUInt32 nBlockOffset = poRDS->pOffsets[nBlockIndex];
 
-    nBlockSize = poRDS->nFileSize;
+    GUInt32 nBlockSize = poRDS->nFileSize;
     for( GUInt32 bi = nBlockIndex + 1; bi < blocks; bi++ )
     {
         if( poRDS->pOffsets[bi] )
@@ -311,7 +302,7 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
         )
     {
         for( GUInt32 i = 0; i < pixels; i++ )
-        ((GByte *) pImage)[i] = 0;
+            reinterpret_cast<GByte *>( pImage )[i] = 0;
         return CE_None;
     }
 
@@ -323,32 +314,40 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 
     if( poRDS->options == 0x00 || poRDS->options == 0x40 )
     {
-        VSIFReadL( pImage, 1, nBlockSize, poRDS->fp );
+        VSIFReadL( pImage, 1, nBlockXSize * nBlockYSize, poRDS->fp );
         return CE_None;
     }
 
     // Read block to memory
-    blockData = (GByte *) CPLMalloc(nBlockSize);
-    VSIFReadL( blockData, 1, nBlockSize, poRDS->fp );
-
-    GUInt32 filePos = 0;
-    GUInt32 imagePos = 0;
+    GByte *blockData = reinterpret_cast<GByte *>( VSI_MALLOC_VERBOSE(nBlockSize) );
+    if( blockData == NULL )
+        return CE_Failure;
+    if( VSIFReadL( blockData, 1, nBlockSize, poRDS->fp ) != nBlockSize )
+    {
+        VSIFree(blockData);
+        return CE_Failure;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Read RLE block.                                                 */
 /* -------------------------------------------------------------------- */
+    GUInt32 filePos = 0;
+    GUInt32 imagePos = 0;
 
     if( poRDS->options == 0x01 ||
-        poRDS->options == 0x41 ) do
+        poRDS->options == 0x41 )
     {
-        GByte count = blockData[filePos++];
-        GByte color = blockData[filePos++];
-
-        for (GByte i = 0; i <= count; i++)
+        while( filePos+1 < nBlockSize && imagePos < pixels )
         {
-            ((GByte *) pImage)[imagePos++] = color;
+            GByte count = blockData[filePos++];
+            GByte color = blockData[filePos++];
+
+            for (GByte i = 0; imagePos < pixels && i <= count; i++)
+            {
+                reinterpret_cast<GByte *>( pImage )[imagePos++] = color;
+            }
         }
-    } while( filePos < nBlockSize && imagePos < pixels );
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Read LZW block.                                                 */
@@ -358,6 +357,14 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
     {
         const bool LZW_HAS_CLEAR_CODE = !!(blockData[4] & 0x80);
         const int LZW_MAX_BITS = blockData[4] & 0x1f; // Max 13
+        if( LZW_MAX_BITS > 13 )
+        {
+            CPLFree( blockData );
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "RIK decompression failed. "
+                      "Invalid LZW_MAX_BITS." );
+            return CE_Failure;
+        }
         const int LZW_BITS_PER_PIXEL = 8;
         const int LZW_OFFSET = 5;
 
@@ -376,11 +383,9 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
         int prefix[8192];      // only need LZW_CODES for size.
         GByte character[8192]; // only need LZW_CODES for size.
 
-        int i;
-
-        for( i = 0; i < LZW_CLEAR; i++ )
-            character[i] = (GByte)i;
-        for( i = 0; i < LZW_CODES; i++ )
+        for( int i = 0; i < LZW_CLEAR; i++ )
+          character[i] = static_cast<GByte>( i );
+        for( int i = 0; i < LZW_CODES; i++ )
             prefix[i] = LZW_NO_SUCH_CODE;
 
         filePos = LZW_OFFSET;
@@ -396,9 +401,9 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
         code = GetNextLZWCode( codeBits, blockData, filePos,
                                fileAlign, bitsTaken );
 
-        OutputPixel( (GByte)code, pImage, poRDS->nBlockXSize,
+        OutputPixel( static_cast<GByte>( code ), pImage, poRDS->nBlockXSize,
                      lineBreak, imageLine, imagePos );
-        lastOutput = (GByte)code;
+        lastOutput = static_cast<GByte>( code );
 
         while( imageLine >= 0 &&
                (imageLine || imagePos < poRDS->nBlockXSize) &&
@@ -429,7 +434,7 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 #endif
 
                 // Clear prefix table
-                for( i = LZW_CLEAR; i < LZW_CODES; i++ )
+                for( int i = LZW_CLEAR; i < LZW_CODES; i++ )
                     prefix[i] = LZW_NO_SUCH_CODE;
                 lastAdded = LZW_CLEAR;
                 codeBits = LZW_BITS_PER_PIXEL + 1;
@@ -472,7 +477,7 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 
                 // Decode
 
-                i = 0;
+                int i = 0;
                 while( ++i < LZW_CODES &&
        	               decodeCode >= LZW_CLEAR &&
        	               decodeCode < LZW_NO_SUCH_CODE )
@@ -480,7 +485,7 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                     stack[stackPtr++] = character[decodeCode];
                     decodeCode = prefix[decodeCode];
                 }
-                stack[stackPtr++] = (GByte)decodeCode;
+                stack[stackPtr++] = static_cast<GByte>( decodeCode );
 
                 if( i == LZW_CODES || decodeCode >= LZW_NO_SUCH_CODE )
                 {
@@ -547,13 +552,13 @@ CPLErr RIKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
     else if( poRDS->options == 0x0d )
     {
         uLong destLen = pixels;
-        Byte *upsideDown = (Byte *) CPLMalloc( pixels );
+        Byte *upsideDown = static_cast<Byte *>( CPLMalloc( pixels ) );
 
         uncompress( upsideDown, &destLen, blockData, nBlockSize );
 
         for (GUInt32 i = 0; i < poRDS->nBlockYSize; i++)
         {
-            memcpy( ((Byte *)pImage) + poRDS->nBlockXSize * i,
+          memcpy( reinterpret_cast<Byte *>( pImage ) + poRDS->nBlockXSize * i,
                     upsideDown + poRDS->nBlockXSize *
                                  (poRDS->nBlockYSize - i - 1),
                     poRDS->nBlockXSize );
@@ -584,7 +589,7 @@ GDALColorInterp RIKRasterBand::GetColorInterpretation()
 GDALColorTable *RIKRasterBand::GetColorTable()
 
 {
-    RIKDataset *poRDS = (RIKDataset *) poDS;
+    RIKDataset *poRDS = reinterpret_cast<RIKDataset *>( poDS );
 
     return poRDS->poColorTable;
 }
@@ -669,7 +674,7 @@ int RIKDataset::Identify( GDALOpenInfo * poOpenInfo )
     if( poOpenInfo->fpL == NULL || poOpenInfo->nHeaderBytes < 50 )
         return FALSE;
 
-    if( EQUALN((const char *) poOpenInfo->pabyHeader, "RIK3", 4) )
+    if( STARTS_WITH_CI((const char *) poOpenInfo->pabyHeader, "RIK3") )
     {
         return TRUE;
     }
@@ -686,7 +691,7 @@ int RIKDataset::Identify( GDALOpenInfo * poOpenInfo )
         }
         if( actLength == 0 )
             return -1;
-        
+
         for( int i=0;i<actLength;i++ )
         {
             if( poOpenInfo->pabyHeader[2+i] == 0 )
@@ -713,7 +718,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
 
     bool rik3header = false;
 
-    if( EQUALN((const char *) poOpenInfo->pabyHeader, "RIK3", 4) )
+    if( STARTS_WITH_CI((const char *) poOpenInfo->pabyHeader, "RIK3") )
     {
         rik3header = true;
         VSIFSeekL( poOpenInfo->fpL, 4, SEEK_SET );
@@ -770,7 +775,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
 
         // Read unknown string
 
-        projLength = GetRikString( poOpenInfo->fpL, projection, sizeof(projection) );
+        /*projLength =*/ GetRikString( poOpenInfo->fpL, projection, sizeof(projection) );
 
         // Read map north edge
 
@@ -851,15 +856,13 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
         CPL_SWAP32PTR( &header.iMPPNum );
 #endif
 
-        if (!CPLIsFinite(header.fSouth) |
-            !CPLIsFinite(header.fWest) |
-            !CPLIsFinite(header.fNorth) |
+        if (!CPLIsFinite(header.fSouth) ||
+            !CPLIsFinite(header.fWest) ||
+            !CPLIsFinite(header.fNorth) ||
             !CPLIsFinite(header.fEast))
             return NULL;
 
-        bool offsetBounds;
-
-        offsetBounds = header.fSouth < 4000000;
+        const bool offsetBounds = header.fSouth < 4000000;
 
         header.iMPPDen = 1;
 
@@ -882,7 +885,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
             headerType = "RIK2";
         }
 
-        metersPerPixel = header.iMPPNum / double(header.iMPPDen);
+        metersPerPixel = header.iMPPNum / static_cast<double>( header.iMPPDen );
 
         VSIFReadL( &header.iBlockWidth, 1, sizeof(header.iBlockWidth), poOpenInfo->fpL );
         VSIFReadL( &header.iBlockHeight, 1, sizeof(header.iBlockHeight), poOpenInfo->fpL );
@@ -907,9 +910,11 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
 
         if( offsetBounds || !header.iVertBlocks )
         {
-            header.iVertBlocks = (GUInt32)
-                ceil( (header.fNorth - header.fSouth) /
+            double dfVertBlocks = ceil( (header.fNorth - header.fSouth) /
                       (header.iBlockHeight * metersPerPixel) );
+            if( dfVertBlocks < 1 || dfVertBlocks > INT_MAX )
+                return NULL;
+            header.iVertBlocks = static_cast<GUInt32>(dfVertBlocks);
         }
 
 #if RIK_HEADER_DEBUG
@@ -930,9 +935,6 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
 
         VSIFReadL( &header.iOptions, 1, sizeof(header.iOptions), poOpenInfo->fpL );
 
-        if( !header.iHorBlocks || !header.iVertBlocks )
-           return NULL;
-
         if( header.iOptions != 0x00 && // Uncompressed
             header.iOptions != 0x40 && // Uncompressed
             header.iOptions != 0x01 && // RLE
@@ -947,14 +949,26 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
         }
     }
 
+    if( header.iBlockWidth == 0 ||
+        header.iHorBlocks == 0 ||
+        header.iBlockWidth >= INT_MAX / header.iHorBlocks ||
+        header.iBlockHeight == 0 ||
+        header.iVertBlocks == 0 ||
+        header.iBlockHeight >= INT_MAX / header.iVertBlocks ||
+        header.iBlockHeight >= INT_MAX / header.iBlockWidth ||
+        header.iVertBlocks >= INT_MAX / (int)sizeof(GUInt32) ||
+        header.iHorBlocks >= INT_MAX / (header.iVertBlocks * (int)sizeof(GUInt32)) )
+    {
+        return NULL;
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Read the palette.                                               */
 /* -------------------------------------------------------------------- */
 
     GByte palette[768];
 
-    GUInt16 i;
-    for( i = 0; i < 256; i++ )
+    for( GUInt16 i = 0; i < 256; i++ )
     {
         VSIFReadL( &palette[i * 3 + 2], 1, 1, poOpenInfo->fpL );
         VSIFReadL( &palette[i * 3 + 1], 1, 1, poOpenInfo->fpL );
@@ -965,11 +979,9 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Find block offsets.                                             */
 /* -------------------------------------------------------------------- */
 
-    GUInt32 blocks;
-    GUInt32 *offsets;
-
-    blocks = header.iHorBlocks * header.iVertBlocks;
-    offsets = (GUInt32 *)CPLMalloc( blocks * sizeof(GUInt32) );
+    GUInt32 blocks = header.iHorBlocks * header.iVertBlocks;
+    GUInt32 *offsets = reinterpret_cast<GUInt32 *>(
+        VSIMalloc( blocks * sizeof(GUInt32) ) );
 
     if( !offsets )
     {
@@ -981,7 +993,38 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
 
     if( header.iOptions == 0x00 )
     {
-        offsets[0] = VSIFTellL( poOpenInfo->fpL );
+        offsets[0] = static_cast<GUInt32>(VSIFTellL( poOpenInfo->fpL ));
+
+        if( VSIFEofL( poOpenInfo->fpL ) )
+        {
+            CPLError( CE_Failure, CPLE_OpenFailed,
+                    "File %s. Read past end of file.\n",
+                    poOpenInfo->pszFilename );
+            CPLFree(offsets);
+            return NULL;
+        }
+
+        VSIFSeekL( poOpenInfo->fpL, 0, SEEK_END );
+        vsi_l_offset nBigFileSize = VSIFTellL( poOpenInfo->fpL );
+        if( nBigFileSize > 0xFFFFFFFFU )
+            nBigFileSize = 0xFFFFFFFFU;
+        GUInt32 fileSize = static_cast<GUInt32>(nBigFileSize);
+
+        GUInt32 nBlocksFromFileSize = (fileSize - offsets[0]) / (header.iBlockWidth * header.iBlockHeight);
+        if( nBlocksFromFileSize < blocks )
+        {
+            blocks = nBlocksFromFileSize;
+            header.iVertBlocks = blocks / header.iHorBlocks;
+        }
+
+        if( header.iVertBlocks == 0 )
+        {
+            CPLError( CE_Failure, CPLE_OpenFailed,
+                        "File %s too short.\n",
+                        poOpenInfo->pszFilename );
+            CPLFree( offsets );
+            return NULL;
+        }
 
         for( GUInt32 i = 1; i < blocks; i++ )
         {
@@ -993,14 +1036,16 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
     {
         for( GUInt32 i = 0; i < blocks; i++ )
         {
-            VSIFReadL( &offsets[i], 1, sizeof(offsets[i]), poOpenInfo->fpL );
+            if( VSIFReadL( &offsets[i], sizeof(offsets[i]), 1, poOpenInfo->fpL ) != 1 )
+                break;
 #ifdef CPL_MSB
             CPL_SWAP32PTR( &offsets[i] );
 #endif
             if( rik3header )
             {
                 GUInt32 blockSize;
-                VSIFReadL( &blockSize, 1, sizeof(blockSize), poOpenInfo->fpL );
+                if( VSIFReadL( &blockSize, sizeof(blockSize), 1, poOpenInfo->fpL ) != 1 )
+                    break;
 #ifdef CPL_MSB
                 CPL_SWAP32PTR( &blockSize );
 #endif
@@ -1019,11 +1064,12 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
         CPLError( CE_Failure, CPLE_OpenFailed,
                   "File %s. Read past end of file.\n",
                   poOpenInfo->pszFilename );
+        CPLFree(offsets);
         return NULL;
     }
 
     VSIFSeekL( poOpenInfo->fpL, 0, SEEK_END );
-    GUInt32 fileSize = VSIFTellL( poOpenInfo->fpL );
+    GUInt32 fileSize = static_cast<GUInt32>(VSIFTellL( poOpenInfo->fpL ));
 
 #if RIK_HEADER_DEBUG
     CPLDebug( "RIK",
@@ -1051,6 +1097,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
                     CPLError( CE_Failure, CPLE_OpenFailed,
                               "File %s too short.\n",
                               poOpenInfo->pszFilename );
+                    CPLFree( offsets );
                     return NULL;
                 }
                 header.iVertBlocks = y;
@@ -1064,6 +1111,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
                     CPLError( CE_Failure, CPLE_OpenFailed,
                               "File %s. Corrupt offset table.\n",
                               poOpenInfo->pszFilename );
+                    CPLFree( offsets );
                     return NULL;
                 }
                 header.iVertBlocks = y;
@@ -1121,9 +1169,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
 
-    RIKDataset 	*poDS;
-
-    poDS = new RIKDataset();
+    RIKDataset 	*poDS = new RIKDataset();
 
     poDS->fp = poOpenInfo->fpL;
     poOpenInfo->fpL = NULL;
@@ -1150,7 +1196,7 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
 
     GDALColorEntry oEntry;
     poDS->poColorTable = new GDALColorTable();
-    for( i = 0; i < 256; i++ )
+    for( GUInt16 i = 0; i < 256; i++ )
     {
         oEntry.c1 = palette[i * 3 + 2]; // Red
         oEntry.c2 = palette[i * 3 + 1]; // Green
@@ -1184,12 +1230,12 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
     if( poOpenInfo->eAccess == GA_Update )
     {
         delete poDS;
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The RIK driver does not support update access to existing"
                   " datasets.\n" );
         return NULL;
     }
-    
+
     return( poDS );
 }
 
@@ -1200,25 +1246,20 @@ GDALDataset *RIKDataset::Open( GDALOpenInfo * poOpenInfo )
 void GDALRegister_RIK()
 
 {
-    GDALDriver	*poDriver;
+    if( GDALGetDriverByName( "RIK" ) != NULL )
+        return;
 
-    if( GDALGetDriverByName( "RIK" ) == NULL )
-    {
-        poDriver = new GDALDriver();
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->SetDescription( "RIK" );
-        poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
-                                   "Swedish Grid RIK (.rik)" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,
-                                   "frmt_various.html#RIK" );
-        poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "rik" );
+    poDriver->SetDescription( "RIK" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "Swedish Grid RIK (.rik)" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_various.html#RIK" );
+    poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "rik" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    poDriver->pfnOpen = RIKDataset::Open;
+    poDriver->pfnIdentify = RIKDataset::Identify;
 
-        poDriver->pfnOpen = RIKDataset::Open;
-        poDriver->pfnIdentify = RIKDataset::Identify;
-
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
