@@ -1,8 +1,8 @@
 /******************************************************************************
- * $Id: sdedataset.cpp 27384 2014-05-24 12:28:12Z rouault $
+ * $Id: sdedataset.cpp 33720 2016-03-15 00:39:53Z goatbar $
  *
  * Project:  ESRI ArcSDE Raster reader
- * Purpose:  Dataset implementaion for ESRI ArcSDE Rasters
+ * Purpose:  Dataset implementation for ESRI ArcSDE Rasters
  * Author:   Howard Butler, hobu@hobu.net
  *
  * This work was sponsored by the Geological Survey of Canada, Natural
@@ -30,9 +30,9 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
+#include "gdal_frmts.h"
 #include "sdedataset.h"
 
-
 /************************************************************************/
 /*                          GetRastercount()                            */
 /************************************************************************/
@@ -41,7 +41,7 @@ int SDEDataset::GetRasterCount( void )
 
 {
     return nBands;
-}    
+}
 
 /************************************************************************/
 /*                          GetRasterXSize()                            */
@@ -51,7 +51,7 @@ int SDEDataset::GetRasterXSize( void )
 
 {
     return nRasterXSize;
-}  
+}
 
 /************************************************************************/
 /*                          GetRasterYSize()                            */
@@ -70,38 +70,38 @@ int SDEDataset::GetRasterYSize( void )
 CPLErr SDEDataset::ComputeRasterInfo() {
     long nSDEErr;
     SE_RASTERINFO raster;
-    
+
     nSDEErr = SE_rasterinfo_create(&raster);
     if( nSDEErr != SE_SUCCESS )
     {
         IssueSDEError( nSDEErr, "SE_rasterinfo_create" );
         return CE_Fatal;
     }
-    
+
     LONG nRasterColumnId = 0;
 
-    nSDEErr = SE_rascolinfo_get_id( hRasterColumn, 
+    nSDEErr = SE_rascolinfo_get_id( hRasterColumn,
                                     &nRasterColumnId);
     if( nSDEErr != SE_SUCCESS )
     {
         IssueSDEError( nSDEErr, "SE_rascolinfo_get_id" );
         return CE_Fatal;
-    }        
+    }
 
-    nSDEErr = SE_raster_get_info_by_id( hConnection, 
-                                        nRasterColumnId, 
-                                        1, 
+    nSDEErr = SE_raster_get_info_by_id( hConnection,
+                                        nRasterColumnId,
+                                        1,
                                         raster);
     if( nSDEErr != SE_SUCCESS )
     {
         IssueSDEError( nSDEErr, "SE_rascolinfo_get_id" );
         return CE_Fatal;
     }
-    
+
     LONG nBandsRet;
-    nSDEErr = SE_raster_get_bands(  hConnection, 
-                                    raster, 
-                                    &paohSDERasterBands, 
+    nSDEErr = SE_raster_get_bands(  hConnection,
+                                    raster,
+                                    &paohSDERasterBands,
                                     &nBandsRet);
     if( nSDEErr != SE_SUCCESS )
     {
@@ -110,12 +110,12 @@ CPLErr SDEDataset::ComputeRasterInfo() {
     }
 
     nBands = nBandsRet;
-    
+
     SE_RASBANDINFO band;
-    
+
     // grab our other stuff from the first band and hope for the best
     band = paohSDERasterBands[0];
-    
+
     LONG nXSRet, nYSRet;
     nSDEErr = SE_rasbandinfo_get_band_size( band, &nXSRet, &nYSRet );
     if( nSDEErr != SE_SUCCESS )
@@ -126,7 +126,7 @@ CPLErr SDEDataset::ComputeRasterInfo() {
 
     nRasterXSize = nXSRet;
     nRasterYSize = nYSRet;
-    
+
     SE_ENVELOPE extent;
     nSDEErr = SE_rasbandinfo_get_extent(band, &extent);
     if( nSDEErr != SE_SUCCESS )
@@ -143,22 +143,22 @@ CPLErr SDEDataset::ComputeRasterInfo() {
 
     // x0 roughly corresponds to dfMinX
     // y0 roughly corresponds to dfMaxY
-    // They can be different than the extent parameters because 
-    // SDE uses offsets.  The following info is from Duarte Carreira 
+    // They can be different than the extent parameters because
+    // SDE uses offsets.  The following info is from Duarte Carreira
     // in relation to bug #2063 http://trac.osgeo.org/gdal/ticket/2063 :
 
-    // Depending on how the data was loaded, the pixel width 
+    // Depending on how the data was loaded, the pixel width
     // or pixel height may include a pixel offset from the nearest
-    // tile boundary. An offset will be indicated by aplus sign 
-    // "+" followed by a value. The value indicates the number 
+    // tile boundary. An offset will be indicated by aplus sign
+    // "+" followed by a value. The value indicates the number
     // of pixels the nearest tile boundary is to the left of
     // the image for the x dimension or the number of
-    // pixels the nearest tile boundary is above the image for 
-    // the y dimension. The offset information is only useful 
-    // for advanced application developers who need to know 
+    // pixels the nearest tile boundary is above the image for
+    // the y dimension. The offset information is only useful
+    // for advanced application developers who need to know
     // where the image begins in relation to the underlying tile structure
 
-    LFLOAT x0, y0; 
+    LFLOAT x0, y0;
     nSDEErr = SE_rasbandinfo_get_tile_origin(band, &x0, &y0);
     if( nSDEErr != SE_SUCCESS )
     {
@@ -181,7 +181,7 @@ CPLErr SDEDataset::ComputeRasterInfo() {
         IssueSDEError( nSDEErr, "SE_rasterattr_create" );
         return CE_Fatal;
     }
-    
+
     // Grab the pointer for our member variable
 
     nSDEErr = SE_stream_create(hConnection, &hStream);
@@ -191,15 +191,15 @@ CPLErr SDEDataset::ComputeRasterInfo() {
         return CE_Fatal;
     }
 
-    
+
     for (int i=0; i < nBands; i++) {
         SetBand( i+1, new SDERasterBand( this, i+1, -1, &(paohSDERasterBands[i]) ));
     }
 
     GDALRasterBand* b = GetRasterBand(1);
-    
+
     eDataType = b->GetRasterDataType();
-    
+
     SE_rasterinfo_free(raster);
 
     return CE_None;
@@ -213,19 +213,19 @@ CPLErr SDEDataset::ComputeRasterInfo() {
 CPLErr SDEDataset::GetGeoTransform( double * padfTransform )
 
 {
-    
+
     if (dfMinX == 0.0 && dfMinY == 0.0 && dfMaxX == 0.0 && dfMaxY == 0.0)
         return CE_Fatal;
- 
+
     padfTransform[0] = dfMinX - 0.5*(dfMaxX - dfMinX) / (GetRasterXSize()-1);
     padfTransform[3] = dfMaxY + 0.5*(dfMaxY - dfMinY) / (GetRasterYSize()-1);
-    
+
     padfTransform[1] = (dfMaxX - dfMinX) / (GetRasterXSize()-1);
     padfTransform[2] = 0.0;
-        
+
     padfTransform[4] = 0.0;
     padfTransform[5] = -1 * (dfMaxY - dfMinY) / (GetRasterYSize()-1);
-    
+
     return CE_None;
 }
 
@@ -245,24 +245,24 @@ const char *SDEDataset::GetProjectionRef()
         IssueSDEError( nSDEErr, "SE_coordref_create" );
         return FALSE;
     }
-    
+
     if (!hRasterColumn){
         CPLError ( CE_Failure, CPLE_AppDefined,
-                   "Raster Column not defined");        
-        return ("");   
+                   "Raster Column not defined");
+        return ("");
     }
-    
+
     nSDEErr = SE_rascolinfo_get_coordref(hRasterColumn, coordref);
 
     if (nSDEErr == SE_NO_COORDREF) {
         return ("");
     }
-    
+
     if( nSDEErr != SE_SUCCESS )
     {
         IssueSDEError( nSDEErr, "SE_rascolinfo_get_coordref" );
-    }    
-    
+    }
+
     char szWKT[SE_MAX_SPATIALREF_SRTEXT_LEN];
     nSDEErr = SE_coordref_get_description(coordref, szWKT);
     if (nSDEErr != SE_SUCCESS )
@@ -278,7 +278,7 @@ const char *SDEDataset::GetProjectionRef()
 
     poSRS->exportToWkt(&pszWKT);
     poSRS->Release();
-    
+
     return pszWKT;
 }
 
@@ -304,7 +304,7 @@ SDEDataset::SDEDataset(  )
     nBands              = 0;
     nRasterXSize        = 0;
     nRasterYSize        = 0;
-    
+
     dfMinX              = 0.0;
     dfMinY              = 0.0;
     dfMaxX              = 0.0;
@@ -328,19 +328,19 @@ SDEDataset::~SDEDataset()
 
     if (hRasterColumn)
         SE_rascolinfo_free(hRasterColumn);
-    
+
     if (hStream)
         SE_stream_free(hStream);
-        
+
     if (hAttributes)
         SE_rasterattr_free(hAttributes);
 
     if (hConnection)
         SE_connection_free(hConnection);
-        
+
     if (pszWKT)
         CPLFree(pszWKT);
-    
+
     if (pszLayerName)
         CPLFree(pszLayerName);
 
@@ -356,33 +356,33 @@ SDEDataset::~SDEDataset()
 GDALDataset *SDEDataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
-    
+
 /* -------------------------------------------------------------------- */
 /*      If we aren't prefixed with SDE: then ignore this datasource.    */
 /* -------------------------------------------------------------------- */
-    if( !EQUALN(poOpenInfo->pszFilename,"SDE:",4) )
+    if( !STARTS_WITH_CI(poOpenInfo->pszFilename, "SDE:") )
         return NULL;
 
 /* -------------------------------------------------------------------- */
 /*      Parse arguments on comma.  We expect (layer is optional):       */
 /*        SDE:server,instance,database,username,password,layer          */
 /* -------------------------------------------------------------------- */
-    char **papszTokens = CSLTokenizeStringComplex(  poOpenInfo->pszFilename+4, 
+    char **papszTokens = CSLTokenizeStringComplex(  poOpenInfo->pszFilename+4,
                                                     ",",
-                                                    TRUE, 
+                                                    TRUE,
                                                     TRUE );
-    CPLDebug(   "SDERASTER", "Open(\"%s\") revealed %d tokens.", 
+    CPLDebug(   "SDERASTER", "Open(\"%s\") revealed %d tokens.",
                 poOpenInfo->pszFilename,
                 CSLCount( papszTokens ) );
 
 
     if( CSLCount( papszTokens ) < 5 || CSLCount( papszTokens ) > 7 )
     {
-        CPLError( CE_Failure, CPLE_OpenFailed, 
+        CPLError( CE_Failure, CPLE_OpenFailed,
                   "SDE connect string had wrong number of arguments.\n"
                   "Expected 'SDE:server,instance,database,username,password,layer'\n"
                   "The layer name value is optional.\n"
-                  "Got '%s'", 
+                  "Got '%s'",
                   poOpenInfo->pszFilename );
         return NULL;
     }
@@ -399,12 +399,12 @@ GDALDataset *SDEDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     int         nSDEErr;
     SE_ERROR    hSDEErrorInfo;
-    nSDEErr = SE_connection_create( papszTokens[0], 
-                                    papszTokens[1], 
-                                    papszTokens[2], 
+    nSDEErr = SE_connection_create( papszTokens[0],
+                                    papszTokens[1],
+                                    papszTokens[2],
                                     papszTokens[3],
                                     papszTokens[4],
-                                    &(hSDEErrorInfo), 
+                                    &(hSDEErrorInfo),
                                     &(poDS->hConnection) );
 
     if( nSDEErr != SE_SUCCESS )
@@ -438,14 +438,13 @@ GDALDataset *SDEDataset::Open( GDALOpenInfo * poOpenInfo )
     else {
         poDS->pszColumnName = CPLStrdup( "RASTER" );
     }
-    
+
     CPLDebug ("SDERASTER", "SDE Column name is '%s'", poDS->pszColumnName);
 
     if (CSLCount( papszTokens ) >= 6 ) {
 
         poDS->pszLayerName = CPLStrdup( papszTokens[5] );
 
-        
         nSDEErr =   SE_rascolinfo_create  (&(poDS->hRasterColumn));
         if( nSDEErr != SE_SUCCESS )
         {
@@ -453,12 +452,12 @@ GDALDataset *SDEDataset::Open( GDALOpenInfo * poOpenInfo )
             return NULL;
         }
         CPLDebug( "SDERASTER", "'%s' raster layer specified... "\
-                               "using it directly with '%s' as the raster column name.", 
+                               "using it directly with '%s' as the raster column name.",
                   poDS->pszLayerName,
                   poDS->pszColumnName);
-        nSDEErr = SE_rastercolumn_get_info_by_name( poDS->hConnection, 
-                                                    poDS->pszLayerName, 
-                                                    poDS->pszColumnName, 
+        nSDEErr = SE_rastercolumn_get_info_by_name( poDS->hConnection,
+                                                    poDS->pszLayerName,
+                                                    poDS->pszColumnName,
                                                     poDS->hRasterColumn);
         if( nSDEErr != SE_SUCCESS )
         {
@@ -470,9 +469,9 @@ GDALDataset *SDEDataset::Open( GDALOpenInfo * poOpenInfo )
 
 
     } else {
- 
-        nSDEErr = SE_rastercolumn_get_info_list(poDS->hConnection, 
-                                                &(poDS->paohSDERasterColumns), 
+
+        nSDEErr = SE_rastercolumn_get_info_list(poDS->hConnection,
+                                                &(poDS->paohSDERasterColumns),
                                                 &(poDS->nSubDataCount));
         if( nSDEErr != SE_SUCCESS )
         {
@@ -480,20 +479,19 @@ GDALDataset *SDEDataset::Open( GDALOpenInfo * poOpenInfo )
             return NULL;
         }
 
-        CPLDebug( "SDERASTER", "No layername specified, %d subdatasets available.", 
+        CPLDebug( "SDERASTER", "No layername specified, %d subdatasets available.",
                   poDS->nSubDataCount);
-                  
 
         for (int i = 0; i < poDS->nSubDataCount; i++) {
 
               char         szTableName[SE_QUALIFIED_TABLE_NAME+1];
               char         szColumnName[SE_MAX_COLUMN_LEN+1];
-            nSDEErr = SE_rascolinfo_get_raster_column (poDS->paohSDERasterColumns[i], 
-                                                       szTableName, 
-                                                       szColumnName); 
-            CPLDebug(   "SDERASTER", 
-                        "Layer '%s' with column '%s' found.", 
-                        szTableName, 
+            nSDEErr = SE_rascolinfo_get_raster_column (poDS->paohSDERasterColumns[i],
+                                                       szTableName,
+                                                       szColumnName);
+            CPLDebug(   "SDERASTER",
+                        "Layer '%s' with column '%s' found.",
+                        szTableName,
                         szColumnName);
 
             if( nSDEErr != SE_SUCCESS )
@@ -516,24 +514,20 @@ GDALDataset *SDEDataset::Open( GDALOpenInfo * poOpenInfo )
 void GDALRegister_SDE()
 
 {
-    GDALDriver  *poDriver;
-    
-    if (! GDAL_CHECK_VERSION("SDE driver"))
+    if( !GDAL_CHECK_VERSION( "SDE driver" ) )
         return;
 
-    if( GDALGetDriverByName( "SDE" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "SDE" );
-        poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "ESRI ArcSDE" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_various.html#SDE" );
+    if( GDALGetDriverByName( "SDE" ) != NULL )
+        return;
 
-        poDriver->pfnOpen = SDEDataset::Open;
+    GDALDriver *poDriver = new GDALDriver();
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    poDriver->SetDescription( "SDE" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "ESRI ArcSDE" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,  "frmt_various.html#SDE" );
+
+    poDriver->pfnOpen = SDEDataset::Open;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
