@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: sdts_al.h 26613 2013-11-14 17:42:08Z goatbar $
+ * $Id: sdts_al.h 32067 2015-12-07 15:34:53Z goatbar $
  *
  * Project:  SDTS Translator
  * Purpose:  Include file for entire SDTS Abstraction Layer functions.
@@ -48,12 +48,12 @@ char **SDTSScanModuleReferences( DDFModule *, const char * );
 /**
   Class holding SDTS IREF (internal reference) information, internal
   coordinate system format, scaling and resolution.  This object isn't
-  normally needed by applications. 
+  normally needed by applications.
 */
 class SDTS_IREF
 {
     int         nDefaultSADRFormat;
-    
+
   public:
                 SDTS_IREF();
                 ~SDTS_IREF();
@@ -97,15 +97,14 @@ class SDTS_XREF
 
     /** Projection system name, from the RSNM field.  One of GEO, SPCS, UTM,
         UPS, OTHR, UNSP. */
-    char        *pszSystemName;         
-                                  
+    char        *pszSystemName;
 
     /** Horizontal datum name, from the HDAT field.  One of NAS, NAX, WGA,
         WGB, WGC, WGE. */
-    char        *pszDatum;              
-                                  
+    char        *pszDatum;
+
     /** Zone number for UTM and SPCS projections, from the ZONE field. */
-    int         nZone;                        
+    int         nZone;
 };
 
 /************************************************************************/
@@ -136,7 +135,7 @@ class SDTS_CATD
 
     int         nEntries;
     SDTS_CATDEntry **papoEntries;
-    
+
   public:
                 SDTS_CATD();
                 ~SDTS_CATD();
@@ -177,16 +176,14 @@ class SDTSModId
 
     /** The record within the module referred to.  This is -1 for unused
         SDTSModIds. */
-    long        nRecord;
+    int         nRecord;
 
     /** The "role" of this record within the module.  This is normally empty
         for references, but set in the oModId member of a feature.  */
-    char        szOBRP[8]; 
+    char        szOBRP[8];
 
     /** String "szModule:nRecord" */
     char        szName[20];
-
-
 };
 
 /************************************************************************/
@@ -216,8 +213,7 @@ public:
 
     void        ApplyATID( DDFField * );
 
-
-    /** Dump reable description of feature to indicated stream. */
+    /** Dump readable description of feature to indicated stream. */
     virtual void Dump( FILE * ) = 0;
 };
 
@@ -237,19 +233,19 @@ class SDTSIndexedReader
 
     int                 iCurrentFeature;
 
-protected:    
+protected:
     DDFModule           oDDFModule;
 
 public:
                         SDTSIndexedReader();
     virtual            ~SDTSIndexedReader();
-    
+
     virtual SDTSFeature  *GetNextRawFeature() = 0;
-    
+
     SDTSFeature        *GetNextFeature();
 
     virtual void        Rewind();
-    
+
     void                FillIndex();
     void                ClearIndex();
     int                 IsIndexed();
@@ -260,7 +256,6 @@ public:
     DDFModule          *GetModule() { return &oDDFModule; }
 };
 
-
 /************************************************************************/
 /*                             SDTSRawLine                              */
 /************************************************************************/
@@ -287,7 +282,7 @@ class SDTSRawLine : public SDTSFeature
 
     /** Identifier of polygon to left of this line.  This is the SDTS PIDL
         subfield. */
-    SDTSModId   oLeftPoly;             
+    SDTSModId   oLeftPoly;
 
     /** Identifier of polygon to right of this line.  This is the SDTS PIDR
         subfield. */
@@ -321,15 +316,15 @@ class SDTSRawLine : public SDTSFeature
 class SDTSLineReader : public SDTSIndexedReader
 {
     SDTS_IREF   *poIREF;
-    
+
   public:
-                SDTSLineReader( SDTS_IREF * );
+                explicit SDTSLineReader( SDTS_IREF * );
                 ~SDTSLineReader();
 
     int         Open( const char * );
     SDTSRawLine *GetNextLine( void );
     void        Close();
-    
+
     SDTSFeature *GetNextRawFeature( void ) { return GetNextLine(); }
 
     void        AttachToPolygons( SDTSTransfer *, int iPolyLayer  );
@@ -376,12 +371,10 @@ class SDTSAttrRecord : public SDTSFeature
 
 class SDTSAttrReader : public SDTSIndexedReader
 {
-    SDTS_IREF   *poIREF;
-
     int         bIsSecondary;
-    
+
   public:
-                SDTSAttrReader( SDTS_IREF * );
+                SDTSAttrReader();
    virtual     ~SDTSAttrReader();
 
     int         Open( const char * );
@@ -396,7 +389,7 @@ class SDTSAttrReader : public SDTSIndexedReader
       an Attribute Primary layer.
       */
     int         IsSecondary() { return bIsSecondary; }
-    
+
     SDTSFeature *GetNextRawFeature( void ) { return GetNextAttrRecord(); }
 };
 
@@ -422,7 +415,7 @@ class SDTSRawPoint : public SDTSFeature
     /** Z coordinate of point. */
     double      dfZ;
 
-    /** Optional identifier of area marked by this point (ie. PC01:27). */
+    /** Optional identifier of area marked by this point (i.e. PC01:27). */
     SDTSModId   oAreaId;                /* ARID */
 
     virtual void Dump( FILE * );
@@ -440,9 +433,9 @@ class SDTSRawPoint : public SDTSFeature
 class SDTSPointReader : public SDTSIndexedReader
 {
     SDTS_IREF   *poIREF;
-    
+
   public:
-                SDTSPointReader( SDTS_IREF * );
+                explicit SDTSPointReader( SDTS_IREF * );
     virtual    ~SDTSPointReader();
 
     int         Open( const char * );
@@ -470,13 +463,13 @@ class SDTSPointReader : public SDTSIndexedReader
   ring geometry.
 
   Note that the rings may not appear in any particular order, nor with any
-  meaningful direction (clockwise or counterclockwise).  
+  meaningful direction (clockwise or counterclockwise).
   */
 
 class SDTSRawPolygon : public SDTSFeature
 {
     void        AddEdgeToRing( int, double *, double *, double *, int, int );
-    
+
   public:
                 SDTSRawPolygon();
     virtual    ~SDTSRawPolygon();
@@ -522,7 +515,7 @@ class SDTSRawPolygon : public SDTSFeature
 class SDTSPolygonReader : public SDTSIndexedReader
 {
     int         bRingsAssembled;
-    
+
   public:
                 SDTSPolygonReader();
     virtual    ~SDTSPolygonReader();
@@ -543,7 +536,7 @@ class SDTSPolygonReader : public SDTSIndexedReader
 /**
   Class for reading raster data from a raster layer.
 
-  This class is somewhat unique amoung the reader classes in that it isn't
+  This class is somewhat unique among the reader classes in that it isn't
   derived from SDTSIndexedFeature, and it doesn't return "features".  Instead
   it is used to read raster blocks, in the natural block size of the dataset.
   */
@@ -563,7 +556,7 @@ class SDTSRasterReader
     int         nYStart;                /* SORI */
 
     double      adfTransform[6];
-    
+
   public:
     char        szINTR[4];              /* CE is center, TL is top left */
     char        szFMT[32];
@@ -578,7 +571,7 @@ class SDTSRasterReader
     void        Close();
 
     int         GetRasterType();        /* 1 = int16, see GDAL types */
-#define SDTS_RT_INT16   1   
+#define SDTS_RT_INT16   1
 #define SDTS_RT_FLOAT32 6
 
     int         GetTransform( double * );
@@ -660,12 +653,12 @@ class SDTSTransfer
 
     SDTSFeature *GetIndexedFeatureRef( SDTSModId *,
                                        SDTSLayerType *peType = NULL);
-                
+
     DDFField *GetAttr( SDTSModId * );
 
     int          GetBounds( double *pdfMinX, double *pdfMinY,
                             double *pdfMaxX, double *pdfMaxY );
-    
+
   private:
 
     SDTS_CATD   oCATD;
