@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tildataset.cpp 29199 2015-05-15 08:46:03Z rouault $
+ * $Id: tildataset.cpp 33720 2016-03-15 00:39:53Z goatbar $
  *
  * Project:  EarthWatch .TIL Driver
  * Purpose:  Implementation of the TILDataset class.
@@ -28,15 +28,17 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
+#include "cpl_multiproc.h"
+#include "cpl_string.h"
+#include "cplkeywordparser.h"
+#include "gdal_mdreader.h"
+#include "gdal_frmts.h"
 #include "gdal_pam.h"
 #include "gdal_proxy.h"
 #include "ogr_spatialref.h"
-#include "cpl_string.h"
 #include "vrtdataset.h"
-#include "cpl_multiproc.h"
-#include "cplkeywordparser.h"
 
-CPL_CVSID("$Id: tildataset.cpp 29199 2015-05-15 08:46:03Z rouault $");
+CPL_CVSID("$Id: tildataset.cpp 33720 2016-03-15 00:39:53Z goatbar $");
 
 /************************************************************************/
 /* ==================================================================== */
@@ -49,8 +51,7 @@ class CPL_DLL TILDataset : public GDALPamDataset
     VRTDataset *poVRTDS;
     std::vector<GDALDataset *> apoTileDS;
 
-    CPLString                  osRPBFilename;
-    CPLString                  osIMDFilename;
+    char **papszMetadataFiles;
 
   protected:
     virtual int         CloseDependentDatasets();
@@ -84,23 +85,24 @@ class TILRasterBand : public GDALPamRasterBand
     virtual CPLErr IReadBlock( int, int, void * );
     virtual CPLErr IRasterIO( GDALRWFlag, int, int, int, int,
                               void *, int, int, GDALDataType,
-                              int, int );
+                              GSpacing nPixelSpace, GSpacing nLineSpace,
+                              GDALRasterIOExtraArg* psExtraArg );
 };
 
 /************************************************************************/
 /*                           TILRasterBand()                            */
 /************************************************************************/
 
-TILRasterBand::TILRasterBand( TILDataset *poTILDS, int nBand, 
-                              VRTSourcedRasterBand *poVRTBand )
+TILRasterBand::TILRasterBand( TILDataset *poTILDS, int nBandIn,
+                              VRTSourcedRasterBand *poVRTBandIn )
 
 {
-    this->poDS = poTILDS;
-    this->poVRTBand = poVRTBand;
-    this->nBand = nBand;
-    this->eDataType = poVRTBand->GetRasterDataType();
+    poDS = poTILDS;
+    poVRTBand = poVRTBandIn;
+    nBand = nBandIn;
+    eDataType = poVRTBandIn->GetRasterDataType();
 
-    poVRTBand->GetBlockSize( &nBlockXSize, &nBlockYSize );
+    poVRTBandIn->GetBlockSize( &nBlockXSize, &nBlockYSize );
 }
 
 /************************************************************************/
@@ -121,21 +123,21 @@ CPLErr TILRasterBand::IRasterIO( GDALRWFlag eRWFlag,
                                  int nXOff, int nYOff, int nXSize, int nYSize,
                                  void * pData, int nBufXSize, int nBufYSize,
                                  GDALDataType eBufType,
-                                 int nPixelSpace, int nLineSpace )
+                                 GSpacing nPixelSpace, GSpacing nLineSpace,
+                                 GDALRasterIOExtraArg* psExtraArg )
 
 {
     if(GetOverviewCount() > 0)
     {
         return GDALPamRasterBand::IRasterIO( eRWFlag, nXOff, nYOff, nXSize, nYSize,
                                  pData, nBufXSize, nBufYSize, eBufType,
-                                 nPixelSpace, nLineSpace );
+                                 nPixelSpace, nLineSpace, psExtraArg );
     }
-    else //if not exist TIL overviews, try to use band source overviews
-    {
-        return poVRTBand->IRasterIO( eRWFlag, nXOff, nYOff, nXSize, nYSize,
+
+    // If not exist TIL overviews, try to use band source overviews.
+    return poVRTBand->IRasterIO( eRWFlag, nXOff, nYOff, nXSize, nYSize,
                                  pData, nBufXSize, nBufYSize, eBufType,
-                                 nPixelSpace, nLineSpace );
-    }
+                                 nPixelSpace, nLineSpace, psExtraArg );
 }
 
 /************************************************************************/
@@ -148,11 +150,10 @@ CPLErr TILRasterBand::IRasterIO( GDALRWFlag eRWFlag,
 /*                             TILDataset()                             */
 /************************************************************************/
 
-TILDataset::TILDataset()
-
-{
-    poVRTDS = NULL;
-}
+TILDataset::TILDataset() :
+    poVRTDS(NULL),
+    papszMetadataFiles(NULL)
+{}
 
 /************************************************************************/
 /*                            ~TILDataset()                             */
@@ -162,6 +163,7 @@ TILDataset::~TILDataset()
 
 {
     CloseDependentDatasets();
+    CSLDestroy(papszMetadataFiles);
 }
 
 /************************************************************************/
@@ -195,14 +197,14 @@ int TILDataset::CloseDependentDatasets()
 int TILDataset::Identify( GDALOpenInfo *poOpenInfo )
 
 {
-    if( poOpenInfo->nHeaderBytes < 200 
+    if( poOpenInfo->nHeaderBytes < 200
         || !EQUAL(CPLGetExtension(poOpenInfo->pszFilename),"TIL") )
         return FALSE;
 
     if( strstr((const char *) poOpenInfo->pabyHeader,"numTiles") == NULL )
         return FALSE;
-    else
-        return TRUE;
+
+    return TRUE;
 }
 
 /************************************************************************/
@@ -220,24 +222,30 @@ GDALDataset *TILDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     if( poOpenInfo->eAccess == GA_Update )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The TIL driver does not support update access to existing"
                   " datasets.\n" );
         return NULL;
     }
-    
+
     CPLString osDirname = CPLGetDirname(poOpenInfo->pszFilename);
 
+// get metadata reader
+
+    GDALMDReaderManager mdreadermanager;
+    GDALMDReaderBase* mdreader = mdreadermanager.GetReader(poOpenInfo->pszFilename,
+                                         poOpenInfo->GetSiblingFiles(), MDR_DG);
+
+    if(NULL == mdreader)
+    {
+        CPLError( CE_Failure, CPLE_OpenFailed,
+                  "Unable to open .TIL dataset due to missing metadata file." );
+        return NULL;
+    }
 /* -------------------------------------------------------------------- */
 /*      Try to find the corresponding .IMD file.                        */
 /* -------------------------------------------------------------------- */
-    char **papszIMD = NULL;
-    CPLString osIMDFilename = 
-        GDALFindAssociatedFile( poOpenInfo->pszFilename, "IMD", 
-                                poOpenInfo->papszSiblingFiles, 0 );
-
-    if( osIMDFilename != "" )
-        papszIMD = GDALLoadIMDFile( osIMDFilename, NULL );
+    char **papszIMD = mdreader->GetMetadataDomain(MD_DOMAIN_IMD);
 
     if( papszIMD == NULL )
     {
@@ -252,7 +260,6 @@ GDALDataset *TILDataset::Open( GDALOpenInfo * poOpenInfo )
     {
         CPLError( CE_Failure, CPLE_OpenFailed,
                   "Missing a required field in the .IMD file." );
-        CSLDestroy( papszIMD );
         return NULL;
     }
 
@@ -260,10 +267,9 @@ GDALDataset *TILDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Try to load and parse the .TIL file.                            */
 /* -------------------------------------------------------------------- */
     VSILFILE *fp = VSIFOpenL( poOpenInfo->pszFilename, "r" );
-    
+
     if( fp == NULL )
     {
-        CSLDestroy( papszIMD );
         return NULL;
     }
 
@@ -272,7 +278,6 @@ GDALDataset *TILDataset::Open( GDALOpenInfo * poOpenInfo )
     if( !oParser.Ingest( fp ) )
     {
         VSIFCloseL( fp );
-        CSLDestroy( papszIMD );
         return NULL;
     }
 
@@ -283,18 +288,14 @@ GDALDataset *TILDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
-    TILDataset 	*poDS;
-
-    poDS = new TILDataset();
-
-    poDS->osIMDFilename = osIMDFilename; 
-    poDS->SetMetadata( papszIMD, "IMD" );
+    TILDataset *poDS = new TILDataset();
+    poDS->papszMetadataFiles = mdreader->GetMetadataFiles();
+    mdreader->FillMetadata(&poDS->oMDMD);
     poDS->nRasterXSize = atoi(CSLFetchNameValueDef(papszIMD,"numColumns","0"));
     poDS->nRasterYSize = atoi(CSLFetchNameValueDef(papszIMD,"numRows","0"));
     if (!GDALCheckDatasetDimensions(poDS->nRasterXSize, poDS->nRasterYSize))
     {
         delete poDS;
-        CSLDestroy( papszIMD );
         return NULL;
     }
 
@@ -302,37 +303,35 @@ GDALDataset *TILDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      We need to open one of the images in order to establish         */
 /*      details like the band count and types.                          */
 /* -------------------------------------------------------------------- */
-    GDALDataset *poTemplateDS;
     const char *pszFilename = CSLFetchNameValue( papszTIL, "TILE_1.filename" );
     if( pszFilename == NULL )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "Missing TILE_1.filename in .TIL file." );
         delete poDS;
-        CSLDestroy( papszIMD );
         return NULL;
     }
 
-    // trim double quotes. 
+    // trim double quotes.
     if( pszFilename[0] == '"' )
         pszFilename++;
     if( pszFilename[strlen(pszFilename)-1] == '"' )
-        ((char *) pszFilename)[strlen(pszFilename)-1] = '\0';
+        const_cast<char *>( pszFilename )[strlen(pszFilename)-1] = '\0';
 
     CPLString osFilename = CPLFormFilename(osDirname, pszFilename, NULL);
-    poTemplateDS = (GDALDataset *) GDALOpen( osFilename, GA_ReadOnly );
+    GDALDataset *poTemplateDS = reinterpret_cast<GDALDataset *>(
+        GDALOpen( osFilename, GA_ReadOnly ) );
     if( poTemplateDS == NULL || poTemplateDS->GetRasterCount() == 0)
     {
         delete poDS;
-        CSLDestroy( papszIMD );
         if (poTemplateDS != NULL)
             GDALClose( poTemplateDS );
         return NULL;
     }
 
     GDALRasterBand *poTemplateBand = poTemplateDS->GetRasterBand(1);
-    GDALDataType eDT = poTemplateBand->GetRasterDataType();
-    int          nBandCount = poTemplateDS->GetRasterCount();
+    const GDALDataType eDT = poTemplateBand->GetRasterDataType();
+    const int nBandCount = poTemplateDS->GetRasterCount();
 
     //we suppose the first tile have the same projection as others (usually so)
     CPLString pszProjection(poTemplateDS->GetProjectionRef());
@@ -343,7 +342,7 @@ GDALDataset *TILDataset::Open( GDALOpenInfo * poOpenInfo )
     double      adfGeoTransform[6];
     if( poTemplateDS->GetGeoTransform( adfGeoTransform ) == CE_None )
     {
-        // According to https://www.digitalglobe.com/sites/default/files/ISD_External.pdf, ulx=originX and 
+        // According to https://www.digitalglobe.com/sites/default/files/ISD_External.pdf, ulx=originX and
         // is "Easting of the center of the upper left pixel of the image."
         adfGeoTransform[0] = CPLAtof(CSLFetchNameValueDef(papszIMD,"MAP_PROJECTED_PRODUCT.ULX","0")) - adfGeoTransform[1] / 2;
         adfGeoTransform[3] = CPLAtof(CSLFetchNameValueDef(papszIMD,"MAP_PROJECTED_PRODUCT.ULY","0")) - adfGeoTransform[5] / 2;
@@ -357,11 +356,9 @@ GDALDataset *TILDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Create and initialize the corresponding VRT dataset used to     */
 /*      manage the tiled data access.                                   */
 /* -------------------------------------------------------------------- */
-    int iBand;
-     
     poDS->poVRTDS = new VRTDataset(poDS->nRasterXSize,poDS->nRasterYSize);
 
-    for( iBand = 0; iBand < nBandCount; iBand++ )
+    for( int iBand = 0; iBand < nBandCount; iBand++ )
         poDS->poVRTDS->AddBand( eDT, NULL );
 
     /* Don't try to write a VRT file */
@@ -370,21 +367,24 @@ GDALDataset *TILDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Create band information objects.                                */
 /* -------------------------------------------------------------------- */
-    for( iBand = 1; iBand <= nBandCount; iBand++ )
-        poDS->SetBand( iBand, 
-                       new TILRasterBand( poDS, iBand, 
-                (VRTSourcedRasterBand *) poDS->poVRTDS->GetRasterBand(iBand)));
+    for( int iBand = 1; iBand <= nBandCount; iBand++ )
+        poDS->SetBand(
+            iBand,
+            new TILRasterBand(
+                poDS,
+                iBand,
+                reinterpret_cast<VRTSourcedRasterBand *>(
+                    poDS->poVRTDS->GetRasterBand(iBand) ) ) );
 
 /* -------------------------------------------------------------------- */
 /*      Add tiles as sources for each band.                             */
 /* -------------------------------------------------------------------- */
-    int nTileCount = atoi(CSLFetchNameValueDef(papszTIL,"numTiles","0"));
+    const int nTileCount = atoi(CSLFetchNameValueDef(papszTIL,"numTiles","0"));
     int iTile = 0;
 
     for( iTile = 1; iTile <= nTileCount; iTile++ )
     {
         CPLString osKey;
-
         osKey.Printf( "TILE_%d.filename", iTile );
         pszFilename = CSLFetchNameValue( papszTIL, osKey );
         if( pszFilename == NULL )
@@ -392,78 +392,55 @@ GDALDataset *TILDataset::Open( GDALOpenInfo * poOpenInfo )
             CPLError( CE_Failure, CPLE_AppDefined,
                       "Missing TILE_%d.filename in .TIL file.", iTile );
             delete poDS;
-            CSLDestroy( papszIMD );
             return NULL;
         }
-        
-        // trim double quotes. 
+
+        // trim double quotes.
         if( pszFilename[0] == '"' )
             pszFilename++;
         if( pszFilename[strlen(pszFilename)-1] == '"' )
-            ((char *) pszFilename)[strlen(pszFilename)-1] = '\0';
+            const_cast<char *>( pszFilename )[strlen(pszFilename)-1] = '\0';
         osFilename = CPLFormFilename(osDirname, pszFilename, NULL);
 
         osKey.Printf( "TILE_%d.ULColOffset", iTile );
-        int nULX = atoi(CSLFetchNameValueDef(papszTIL, osKey, "0"));
-        
+        const int nULX = atoi(CSLFetchNameValueDef(papszTIL, osKey, "0"));
+
         osKey.Printf( "TILE_%d.ULRowOffset", iTile );
-        int nULY = atoi(CSLFetchNameValueDef(papszTIL, osKey, "0"));
+        const int nULY = atoi(CSLFetchNameValueDef(papszTIL, osKey, "0"));
 
         osKey.Printf( "TILE_%d.LRColOffset", iTile );
-        int nLRX = atoi(CSLFetchNameValueDef(papszTIL, osKey, "0"));
-        
-        osKey.Printf( "TILE_%d.LRRowOffset", iTile );
-        int nLRY = atoi(CSLFetchNameValueDef(papszTIL, osKey, "0"));
+        const int nLRX = atoi(CSLFetchNameValueDef(papszTIL, osKey, "0"));
 
-        GDALDataset *poTileDS = 
-            new GDALProxyPoolDataset( osFilename, 
+        osKey.Printf( "TILE_%d.LRRowOffset", iTile );
+        const int nLRY = atoi(CSLFetchNameValueDef(papszTIL, osKey, "0"));
+
+        GDALDataset *poTileDS =
+            new GDALProxyPoolDataset( osFilename,
                                       nLRX - nULX + 1, nLRY - nULY + 1 );
         if( poTileDS == NULL )
             continue;
 
         poDS->apoTileDS.push_back( poTileDS );
 
-        for( iBand = 1; iBand <= nBandCount; iBand++ )
+        for( int iBand = 1; iBand <= nBandCount; iBand++ )
         {
-            ((GDALProxyPoolDataset *) poTileDS)->
+            reinterpret_cast<GDALProxyPoolDataset *>( poTileDS )->
                 AddSrcBandDescription( eDT, nLRX - nULX + 1, 1 );
 
             GDALRasterBand *poSrcBand = poTileDS->GetRasterBand(iBand);
 
-            VRTSourcedRasterBand *poVRTBand = 
-                (VRTSourcedRasterBand *) poDS->poVRTDS->GetRasterBand(iBand);
-            
+            VRTSourcedRasterBand *poVRTBand =
+                reinterpret_cast<VRTSourcedRasterBand *>(
+                    poDS->poVRTDS->GetRasterBand(iBand) );
+
             poVRTBand->AddSimpleSource( poSrcBand,
-                                        0, 0, 
-                                        nLRX - nULX + 1, nLRY - nULY + 1, 
-                                        nULX, nULY, 
+                                        0, 0,
+                                        nLRX - nULX + 1, nLRY - nULY + 1,
+                                        nULX, nULY,
                                         nLRX - nULX + 1, nLRY - nULY + 1 );
         }
     }
 
-/* -------------------------------------------------------------------- */
-/*      Set RPC and IMD metadata.                                       */
-/* -------------------------------------------------------------------- */
-    poDS->osRPBFilename = 
-        GDALFindAssociatedFile( poOpenInfo->pszFilename, "RPB", 
-                                poOpenInfo->papszSiblingFiles, 0 );
-    if( poDS->osRPBFilename != "" )
-    {
-        char **papszRPCMD = GDALLoadRPBFile( poOpenInfo->pszFilename,
-                                             poOpenInfo->papszSiblingFiles );
-        
-        if( papszRPCMD != NULL )
-        {
-            poDS->SetMetadata( papszRPCMD, "RPC" );
-            CSLDestroy( papszRPCMD );
-        }
-    }
-
-/* -------------------------------------------------------------------- */
-/*      Cleanup                                                         */
-/* -------------------------------------------------------------------- */
-    CSLDestroy( papszIMD );
-
 /* -------------------------------------------------------------------- */
 /*      Initialize any PAM information.                                 */
 /* -------------------------------------------------------------------- */
@@ -475,7 +452,7 @@ GDALDataset *TILDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
 
-    return( poDS );
+    return poDS;
 }
 
 /************************************************************************/
@@ -485,18 +462,19 @@ GDALDataset *TILDataset::Open( GDALOpenInfo * poOpenInfo )
 char **TILDataset::GetFileList()
 
 {
-    unsigned int  i;
     char **papszFileList = GDALPamDataset::GetFileList();
 
-    for( i = 0; i < apoTileDS.size(); i++ )
+    for( unsigned int i = 0; i < apoTileDS.size(); i++ )
         papszFileList = CSLAddString( papszFileList,
                                       apoTileDS[i]->GetDescription() );
-    
-    papszFileList = CSLAddString( papszFileList, osIMDFilename );
 
-
-    if( osRPBFilename != "" )
-        papszFileList = CSLAddString( papszFileList, osRPBFilename );
+    if(NULL != papszMetadataFiles)
+    {
+        for( int i = 0; papszMetadataFiles[i] != NULL; i++ )
+        {
+            papszFileList = CSLAddString( papszFileList, papszMetadataFiles[i] );
+        }
+    }
 
     return papszFileList;
 }
@@ -508,25 +486,19 @@ char **TILDataset::GetFileList()
 void GDALRegister_TIL()
 
 {
-    GDALDriver	*poDriver;
+    if( GDALGetDriverByName( "TIL" ) != NULL )
+        return;
 
-    if( GDALGetDriverByName( "TIL" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "TIL" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "EarthWatch .TIL" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_til.html" );
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    poDriver->SetDescription( "TIL" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "EarthWatch .TIL" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_til.html" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
-        poDriver->pfnOpen = TILDataset::Open;
-        poDriver->pfnIdentify = TILDataset::Identify;
+    poDriver->pfnOpen = TILDataset::Open;
+    poDriver->pfnIdentify = TILDataset::Identify;
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
-
-
