@@ -45,27 +45,23 @@ static std::map<CPLString, GDALDerivedPixelFunc> osMapPixelFunction;
 /*                        VRTDerivedRasterBand()                        */
 /************************************************************************/
 
-VRTDerivedRasterBand::VRTDerivedRasterBand(GDALDataset *poDS, int nBand)
-    : VRTSourcedRasterBand( poDS, nBand )
-
-{
-    this->pszFuncName = NULL;
-    this->eSourceTransferType = GDT_Unknown;
-}
+VRTDerivedRasterBand::VRTDerivedRasterBand(GDALDataset *poDSIn, int nBandIn) :
+    VRTSourcedRasterBand( poDSIn, nBandIn ),
+    pszFuncName(NULL),
+    eSourceTransferType(GDT_Unknown)
+{}
 
 /************************************************************************/
 /*                        VRTDerivedRasterBand()                        */
 /************************************************************************/
 
-VRTDerivedRasterBand::VRTDerivedRasterBand(GDALDataset *poDS, int nBand,
-					   GDALDataType eType, 
-					   int nXSize, int nYSize)
-    : VRTSourcedRasterBand(poDS, nBand, eType, nXSize, nYSize)
-
-{
-    this->pszFuncName = NULL;
-    this->eSourceTransferType = GDT_Unknown;
-}
+VRTDerivedRasterBand::VRTDerivedRasterBand(GDALDataset *poDSIn, int nBandIn,
+					   GDALDataType eType,
+					   int nXSize, int nYSize) :
+    VRTSourcedRasterBand(poDSIn, nBandIn, eType, nXSize, nYSize),
+    pszFuncName(NULL),
+    eSourceTransferType(GDT_Unknown)
+{}
 
 /************************************************************************/
 /*                       ~VRTDerivedRasterBand()                        */
@@ -74,10 +70,7 @@ VRTDerivedRasterBand::VRTDerivedRasterBand(GDALDataset *poDS, int nBand,
 VRTDerivedRasterBand::~VRTDerivedRasterBand()
 
 {
-    if (this->pszFuncName != NULL) {
-        CPLFree(this->pszFuncName);
-        this->pszFuncName = NULL;
-    }
+    CPLFree( pszFuncName );
 }
 
 /************************************************************************/
@@ -89,7 +82,7 @@ VRTDerivedRasterBand::~VRTDerivedRasterBand()
  * functions for derived bands.  Pixel functions must be registered
  * in this way before a derived band tries to access data.
  *
- * Derived bands are stored with only the name of the pixel function 
+ * Derived bands are stored with only the name of the pixel function
  * that it will apply, and if a pixel function matching the name is not
  * found the IRasterIO() call will do nothing.
  *
@@ -114,6 +107,7 @@ CPLErr CPL_STDCALL GDALAddDerivedBandPixelFunc
 
     return CE_None;
 }
+
 /**
  * This adds a pixel function to the global list of available pixel
  * functions for derived bands.
@@ -175,9 +169,9 @@ GDALDerivedPixelFunc VRTDerivedRasterBand::GetPixelFunction
  * @param pszFuncName Name of pixel function to be applied to this derived
  * band.
  */
-void VRTDerivedRasterBand::SetPixelFunctionName(const char *pszFuncName)
+void VRTDerivedRasterBand::SetPixelFunctionName(const char *pszFuncNameIn)
 {
-    this->pszFuncName = CPLStrdup( pszFuncName );
+    pszFuncName = CPLStrdup( pszFuncNameIn );
 }
 
 /************************************************************************/
@@ -195,9 +189,9 @@ void VRTDerivedRasterBand::SetPixelFunctionName(const char *pszFuncName)
  * @param eDataType Data type to use to obtain pixel information from
  * the sources to be passed to the derived band pixel function.
  */
-void VRTDerivedRasterBand::SetSourceTransferType(GDALDataType eDataType)
+void VRTDerivedRasterBand::SetSourceTransferType(GDALDataType eDataTypeIn)
 {
-    this->eSourceTransferType = eDataType;
+    this->eSourceTransferType = eDataTypeIn;
 }
 
 /************************************************************************/
@@ -258,53 +252,48 @@ void VRTDerivedRasterBand::SetSourceTransferType(GDALDataType eDataType)
  * @return CE_Failure if the access fails, otherwise CE_None.
  */
 CPLErr VRTDerivedRasterBand::IRasterIO(GDALRWFlag eRWFlag,
-				       int nXOff, int nYOff, int nXSize, 
+				       int nXOff, int nYOff, int nXSize,
 				       int nYSize, void * pData, int nBufXSize,
 				       int nBufYSize, GDALDataType eBufType,
-				       int nPixelSpace, int nLineSpace )
+				       GSpacing nPixelSpace,
+                                       GSpacing nLineSpace,
+                                       GDALRasterIOExtraArg* psExtraArg )
 {
-    GDALDerivedPixelFunc pfnPixelFunc;
-    void **pBuffers;
-    CPLErr eErr = CE_None;
-    int iSource, ii, typesize, sourcesize;
-    GDALDataType eSrcType;
-
     if( eRWFlag == GF_Write )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "Writing through VRTSourcedRasterBand is not supported." );
         return CE_Failure;
     }
 
-    typesize = GDALGetDataTypeSize(eBufType) / 8;
+    int typesize = GDALGetDataTypeSizeBytes(eBufType);
     if (GDALGetDataTypeSize(eBufType) % 8 > 0) typesize++;
-    eSrcType = this->eSourceTransferType;
+    GDALDataType eSrcType = eSourceTransferType;
     if ((eSrcType == GDT_Unknown) || (eSrcType >= GDT_TypeCount)) {
 	eSrcType = eBufType;
     }
-    sourcesize = GDALGetDataTypeSize(eSrcType) / 8;
+    const int sourcesize = GDALGetDataTypeSizeBytes(eSrcType);
 
 /* -------------------------------------------------------------------- */
 /*      Initialize the buffer to some background value. Use the         */
 /*      nodata value if available.                                      */
 /* -------------------------------------------------------------------- */
     if ( nPixelSpace == typesize &&
-         (!bNoDataValueSet || dfNoDataValue == 0) ) {
-        memset( pData, 0, nBufXSize * nBufYSize * nPixelSpace );
+         (!m_bNoDataValueSet || m_dfNoDataValue == 0) ) {
+        memset( pData, 0, static_cast<size_t>(nBufXSize * nBufYSize * nPixelSpace) );
     }
-    else if ( !bEqualAreas || bNoDataValueSet )
+    else if ( !bEqualAreas || m_bNoDataValueSet )
     {
         double dfWriteValue = 0.0;
-        int    iLine;
+        if( m_bNoDataValueSet )
+            dfWriteValue = m_dfNoDataValue;
 
-        if( bNoDataValueSet )
-            dfWriteValue = dfNoDataValue;
-
-        for( iLine = 0; iLine < nBufYSize; iLine++ )
+        for( int iLine = 0; iLine < nBufYSize; iLine++ )
         {
-            GDALCopyWords( &dfWriteValue, GDT_Float64, 0, 
-                           ((GByte *)pData) + nLineSpace * iLine, 
-                           eBufType, nPixelSpace, nBufXSize );
+            GDALCopyWords(
+                &dfWriteValue, GDT_Float64, 0,
+                reinterpret_cast<GByte *>( pData ) + nLineSpace * iLine,
+                eBufType, static_cast<int>(nPixelSpace), nBufXSize );
         }
     }
 
@@ -315,17 +304,18 @@ CPLErr VRTDerivedRasterBand::IRasterIO(GDALRWFlag eRWFlag,
     if( (nBufXSize < nXSize || nBufYSize < nYSize)
         && GetOverviewCount() > 0 )
     {
-        if( OverviewRasterIO( eRWFlag, nXOff, nYOff, nXSize, nYSize, 
-                              pData, nBufXSize, nBufYSize, 
-                              eBufType, nPixelSpace, nLineSpace ) == CE_None )
+        if( OverviewRasterIO( eRWFlag, nXOff, nYOff, nXSize, nYSize,
+                              pData, nBufXSize, nBufYSize,
+                              eBufType, nPixelSpace, nLineSpace, psExtraArg ) == CE_None )
             return CE_None;
     }
 
     /* ---- Get pixel function for band ---- */
-    pfnPixelFunc = VRTDerivedRasterBand::GetPixelFunction(this->pszFuncName);
+    GDALDerivedPixelFunc pfnPixelFunc
+        = VRTDerivedRasterBand::GetPixelFunction(this->pszFuncName);
     if (pfnPixelFunc == NULL) {
-	CPLError( CE_Failure, CPLE_IllegalArg, 
-		  "VRTDerivedRasterBand::IRasterIO:" \
+	CPLError( CE_Failure, CPLE_IllegalArg,
+		  "VRTDerivedRasterBand::IRasterIO:"
 		  "Derived band pixel function '%s' not registered.\n",
 		  this->pszFuncName);
 	return CE_Failure;
@@ -339,19 +329,17 @@ CPLErr VRTDerivedRasterBand::IRasterIO(GDALRWFlag eRWFlag,
        of the allocated blocks. */
 
     /* ---- Get buffers for each source ---- */
-    pBuffers = (void **) CPLMalloc(sizeof(void *) * nSources);
-    for (iSource = 0; iSource < nSources; iSource++) {
-        pBuffers[iSource] = (void *) 
-            VSIMalloc(sourcesize * nBufXSize * nBufYSize);
+    void **pBuffers
+        = reinterpret_cast<void **>( CPLMalloc(sizeof(void *) * nSources) );
+    for( int iSource = 0; iSource < nSources; iSource++ ) {
+        pBuffers[iSource] =
+            VSI_MALLOC_VERBOSE(sourcesize * nBufXSize * nBufYSize);
         if (pBuffers[iSource] == NULL)
         {
-            for (ii = 0; ii < iSource; ii++) {
-                VSIFree(pBuffers[iSource]);
+            for (int i = 0; i < iSource; i++) {
+                VSIFree(pBuffers[i]);
             }
-            CPLError( CE_Failure, CPLE_OutOfMemory,
-                "VRTDerivedRasterBand::IRasterIO:" \
-                "Out of memory allocating %d bytes.\n",
-                nPixelSpace * nBufXSize * nBufYSize);
+            CPLFree(pBuffers);
             return CE_Failure;
         }
 
@@ -361,36 +349,40 @@ CPLErr VRTDerivedRasterBand::IRasterIO(GDALRWFlag eRWFlag,
         /* don't need any special line-by-line handling when a nonzero  */
         /* nodata value is set.                                         */
         /* ------------------------------------------------------------ */
-        if ( !bNoDataValueSet || dfNoDataValue == 0 )
+        if ( !m_bNoDataValueSet || m_dfNoDataValue == 0 )
         {
             memset( pBuffers[iSource], 0, sourcesize * nBufXSize * nBufYSize );
         }
         else
         {
-            GDALCopyWords( &dfNoDataValue, GDT_Float64, 0,
-                           (GByte *) pBuffers[iSource], eSrcType, sourcesize,
+            GDALCopyWords( &m_dfNoDataValue, GDT_Float64, 0,
+                           reinterpret_cast<GByte *>( pBuffers[iSource] ), eSrcType, sourcesize,
                            nBufXSize * nBufYSize);
         }
     }
 
+    GDALRasterIOExtraArg sExtraArg;
+    INIT_RASTERIO_EXTRA_ARG(sExtraArg);
+
     /* ---- Load values for sources into packed buffers ---- */
-    for(iSource = 0; iSource < nSources; iSource++) {
-        eErr = ((VRTSource *)papoSources[iSource])->RasterIO
-	    (nXOff, nYOff, nXSize, nYSize, 
-	     pBuffers[iSource], nBufXSize, nBufYSize, 
-	     eSrcType, GDALGetDataTypeSize( eSrcType ) / 8,
-             (GDALGetDataTypeSize( eSrcType ) / 8) * nBufXSize);
+    CPLErr eErr = CE_None;
+    for( int iSource = 0; iSource < nSources && eErr == CE_None; iSource++ ) {
+        eErr = reinterpret_cast<VRTSource *>( papoSources[iSource] )->RasterIO(
+	    nXOff, nYOff, nXSize, nYSize,
+            pBuffers[iSource], nBufXSize, nBufYSize,
+            eSrcType, GDALGetDataTypeSizeBytes( eSrcType ),
+            GDALGetDataTypeSizeBytes( eSrcType ) * nBufXSize, &sExtraArg);
     }
 
     /* ---- Apply pixel function ---- */
     if (eErr == CE_None) {
-	eErr = pfnPixelFunc((void **)pBuffers, nSources,
-			    pData, nBufXSize, nBufYSize,
-			    eSrcType, eBufType, nPixelSpace, nLineSpace);
+        eErr = pfnPixelFunc( reinterpret_cast<void **>( pBuffers ), nSources,
+			     pData, nBufXSize, nBufYSize,
+			     eSrcType, eBufType, static_cast<int>(nPixelSpace), static_cast<int>(nLineSpace) );
     }
 
     /* ---- Release buffers ---- */
-    for (iSource = 0; iSource < nSources; iSource++) {
+    for ( int iSource = 0; iSource < nSources; iSource++ ) {
         VSIFree(pBuffers[iSource]);
     }
     CPLFree(pBuffers);
@@ -402,25 +394,21 @@ CPLErr VRTDerivedRasterBand::IRasterIO(GDALRWFlag eRWFlag,
 /*                              XMLInit()                               */
 /************************************************************************/
 
-CPLErr VRTDerivedRasterBand::XMLInit(CPLXMLNode *psTree, 
-				     const char *pszVRTPath)
+CPLErr VRTDerivedRasterBand::XMLInit( CPLXMLNode *psTree,
+				      const char *pszVRTPath )
 
 {
-    CPLErr eErr;
-    const char *pszTypeName;
-
-    eErr = VRTSourcedRasterBand::XMLInit( psTree, pszVRTPath );
+    CPLErr eErr = VRTSourcedRasterBand::XMLInit( psTree, pszVRTPath );
     if( eErr != CE_None )
         return eErr;
 
     /* ---- Read derived pixel function type ---- */
-    this->SetPixelFunctionName
-	(CPLGetXMLValue(psTree, "PixelFunctionType", NULL));
+    SetPixelFunctionName( CPLGetXMLValue( psTree, "PixelFunctionType", NULL ) );
 
     /* ---- Read optional source transfer data type ---- */
-    pszTypeName = CPLGetXMLValue(psTree, "SourceTransferType", NULL);
+    const char *pszTypeName = CPLGetXMLValue(psTree, "SourceTransferType", NULL);
     if (pszTypeName != NULL) {
-	this->eSourceTransferType = GDALGetDataTypeByName(pszTypeName);
+	eSourceTransferType = GDALGetDataTypeByName( pszTypeName );
     }
 
     return CE_None;
@@ -432,24 +420,21 @@ CPLErr VRTDerivedRasterBand::XMLInit(CPLXMLNode *psTree,
 
 CPLXMLNode *VRTDerivedRasterBand::SerializeToXML(const char *pszVRTPath)
 {
-    CPLXMLNode *psTree;
-
-    psTree = VRTSourcedRasterBand::SerializeToXML( pszVRTPath );
+    CPLXMLNode *psTree = VRTSourcedRasterBand::SerializeToXML( pszVRTPath );
 
 /* -------------------------------------------------------------------- */
 /*      Set subclass.                                                   */
 /* -------------------------------------------------------------------- */
-    CPLCreateXMLNode( 
-        CPLCreateXMLNode( psTree, CXT_Attribute, "subClass" ), 
+    CPLCreateXMLNode(
+        CPLCreateXMLNode( psTree, CXT_Attribute, "subClass" ),
         CXT_Text, "VRTDerivedRasterBand" );
 
     /* ---- Encode DerivedBand-specific fields ---- */
     if( pszFuncName != NULL && strlen(pszFuncName) > 0 )
-        CPLSetXMLValue(psTree, "PixelFunctionType", this->pszFuncName);
+        CPLSetXMLValue( psTree, "PixelFunctionType", pszFuncName );
     if( this->eSourceTransferType != GDT_Unknown)
-        CPLSetXMLValue(psTree, "SourceTransferType", 
-		       GDALGetDataTypeName(this->eSourceTransferType));
+        CPLSetXMLValue( psTree, "SourceTransferType",
+		        GDALGetDataTypeName( eSourceTransferType ) );
 
     return psTree;
 }
-
