@@ -1,5 +1,5 @@
-/***************************************************************************** 
- * $Id: gdalhttp.h 18020 2009-11-14 14:33:20Z rouault $
+/*****************************************************************************
+ * $Id: gdalhttp.h 33717 2016-03-14 06:29:14Z goatbar $
  *
  * Project:  WMS Client Driver
  * Purpose:  Implementation of Dataset and RasterBand classes for WMS
@@ -51,6 +51,6 @@ typedef struct {
     char *m_curl_error;
 } CPLHTTPRequest;
 
-void CPL_DLL CPLHTTPInitializeRequest(CPLHTTPRequest *psRequest, const char *pszURL = 0, const char *const *papszOptions = 0);
+void CPL_DLL CPLHTTPInitializeRequest(CPLHTTPRequest *psRequest, const char *pszURL = NULL, const char *const *papszOptions = NULL);
 void CPL_DLL CPLHTTPCleanupRequest(CPLHTTPRequest *psRequest);
-CPLErr CPL_DLL CPLHTTPFetchMulti(CPLHTTPRequest *pasRequest, int nRequestCount = 1, const char *const *papszOptions = 0);
+CPLErr CPL_DLL CPLHTTPFetchMulti(CPLHTTPRequest *pasRequest, int nRequestCount = 1, const char *const *papszOptions = NULL);
