@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalwmsdataset.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: gdalwmsdataset.cpp 33717 2016-03-14 06:29:14Z goatbar $
  *
  * Project:  WMS Client Driver
  * Purpose:  Implementation of Dataset and RasterBand classes for WMS
@@ -30,10 +30,10 @@
  ****************************************************************************
  *
  * dataset.cpp:
- * Initialization of the GDALWMSdriver, parsing the XML configuration file, 
- * instantiation of the minidrivers and accessors used by minidrivers
+ * Initialization of the GDALWMSdriver, parsing the XML configuration file,
+ * instantiation of the minidrivers and accessors used by minidrivers.
  *
- ***************************************************************************/ 
+ ***************************************************************************/
 
 
 #include "wmsdriver.h"
@@ -48,9 +48,17 @@
 /************************************************************************/
 /*                           GDALWMSDataset()                           */
 /************************************************************************/
-GDALWMSDataset::GDALWMSDataset() {
-    m_mini_driver = 0;
-    m_cache = 0;
+GDALWMSDataset::GDALWMSDataset() :
+    m_block_size_x(0),
+    m_block_size_y(0),
+    m_use_advise_read(0),
+    m_verify_advise_read(0),
+    m_offline_mode(0),
+    m_http_max_conn(0),
+    m_http_timeout(0)
+{
+    m_mini_driver = NULL;
+    m_cache = NULL;
     m_hint.m_valid = false;
     m_data_type = GDT_Byte;
     m_clamp_requests = true;
@@ -219,10 +227,10 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config) {
                 {
                     if ((ulx[0] != '\0') && (uly[0] != '\0') && (lrx[0] != '\0') && (lry[0] != '\0'))
                     {
-                        m_data_window.m_x0 = atof(ulx);
-                        m_data_window.m_y0 = atof(uly);
-                        m_data_window.m_x1 = atof(lrx);
-                        m_data_window.m_y1 = atof(lry);
+                        m_data_window.m_x0 = CPLAtof(ulx);
+                        m_data_window.m_y0 = CPLAtof(uly);
+                        m_data_window.m_x1 = CPLAtof(lrx);
+                        m_data_window.m_y1 = CPLAtof(lry);
                     }
                     else
                     {
@@ -364,8 +372,8 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config) {
             }
         }
     }
-    
-    // UserPwd 
+
+    // UserPwd
     const char *pszUserPwd = CPLGetXMLValue(config, "UserPwd", "");
     if (pszUserPwd[0] != '\0')
         m_osUserPwd = pszUserPwd;
@@ -373,14 +381,14 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config) {
     const char *pszUserAgent = CPLGetXMLValue(config, "UserAgent", "");
     if (pszUserAgent[0] != '\0')
         m_osUserAgent = pszUserAgent;
-    
+
     const char *pszReferer = CPLGetXMLValue(config, "Referer", "");
     if (pszReferer[0] != '\0')
         m_osReferer = pszReferer;
-    
+
     if (ret == CE_None) {
         const char *pszHttpZeroBlockCodes = CPLGetXMLValue(config, "ZeroBlockHttpCodes", "");
-        if(pszHttpZeroBlockCodes == '\0') {
+        if(pszHttpZeroBlockCodes[0] == '\0') {
             m_http_zeroblock_codes.push_back(204);
         } else {
             char **kv = CSLTokenizeString2(pszHttpZeroBlockCodes,",",CSLT_HONOURSTRINGS);
@@ -492,7 +500,7 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config) {
     if (ret == CE_None) {
         // Data values are attributes, they include NoData Min and Max
         // TODO: document those options
-        if (0!=CPLGetXMLNode(config,"DataValues")) {
+        if (NULL!=CPLGetXMLNode(config,"DataValues")) {
             const char *nodata=CPLGetXMLValue(config,"DataValues.NoData",NULL);
             if (nodata!=NULL) WMSSetNoDataValue(nodata);
             const char *min=CPLGetXMLValue(config,"DataValues.min",NULL);
@@ -514,19 +522,19 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config) {
             }
         }
     }
-    
+
     if (ret == CE_None) {
     	const int v = StrToBool(CPLGetXMLValue(config, "UnsafeSSL", "false"));
     	if (v == -1) {
 	    CPLError(CE_Failure, CPLE_AppDefined, "GDALWMS: Invalid value of UnsafeSSL: true or false expected.");
 	    ret = CE_Failure;
-	} else {
+        } else {
 	    m_unsafeSsl = v;
-	}
+        }
     }
 
     if (ret == CE_None) {
-        /* If we dont have projection already set ask mini-driver. */
+        /* If we do not have projection already set ask mini-driver. */
         if (!m_projection.size()) {
             const char *proj = m_mini_driver->GetProjectionInWKT();
             if (proj != NULL) {
@@ -541,7 +549,12 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config) {
 /************************************************************************/
 /*                             IRasterIO()                              */
 /************************************************************************/
-CPLErr GDALWMSDataset::IRasterIO(GDALRWFlag rw, int x0, int y0, int sx, int sy, void *buffer, int bsx, int bsy, GDALDataType bdt, int band_count, int *band_map, int pixel_space, int line_space, int band_space) {
+CPLErr GDALWMSDataset::IRasterIO(GDALRWFlag rw, int x0, int y0, int sx, int sy,
+                                 void *buffer, int bsx, int bsy, GDALDataType bdt,
+                                 int band_count, int *band_map,
+                                 GSpacing nPixelSpace, GSpacing nLineSpace,
+                                 GSpacing nBandSpace,
+                                 GDALRasterIOExtraArg* psExtraArg) {
     CPLErr ret;
 
     if (rw != GF_Read) return CE_Failure;
@@ -555,7 +568,8 @@ CPLErr GDALWMSDataset::IRasterIO(GDALRWFlag rw, int x0, int y0, int sx, int sy,
     m_hint.m_overview = -1;
     m_hint.m_valid = true;
     //	printf("[%p] GDALWMSDataset::IRasterIO(x0: %d, y0: %d, sx: %d, sy: %d, bsx: %d, bsy: %d, band_count: %d, band_map: %p)\n", this, x0, y0, sx, sy, bsx, bsy, band_count, band_map);
-    ret = GDALDataset::IRasterIO(rw, x0, y0, sx, sy, buffer, bsx, bsy, bdt, band_count, band_map, pixel_space, line_space, band_space);
+    ret = GDALDataset::IRasterIO(rw, x0, y0, sx, sy, buffer, bsx, bsy, bdt, band_count, band_map,
+                                 nPixelSpace, nLineSpace, nBandSpace, psExtraArg);
     m_hint.m_valid = false;
 
     return ret;
@@ -579,6 +593,16 @@ CPLErr GDALWMSDataset::SetProjection(CPL_UNUSED const char *proj) {
 /*                          GetGeoTransform()                           */
 /************************************************************************/
 CPLErr GDALWMSDataset::GetGeoTransform(double *gt) {
+    if( !(m_mini_driver_caps.m_has_geotransform) )
+    {
+        gt[0] = 0;
+        gt[1] = 1;
+        gt[2] = 0;
+        gt[3] = 0;
+        gt[4] = 0;
+        gt[5] = 1;
+        return CE_Failure;
+    }
     gt[0] = m_data_window.m_x0;
     gt[1] = (m_data_window.m_x1 - m_data_window.m_x0) / static_cast<double>(m_data_window.m_sx);
     gt[2] = 0.0;
@@ -601,8 +625,11 @@ CPLErr GDALWMSDataset::SetGeoTransform(CPL_UNUSED double *gt) {
 CPLErr GDALWMSDataset::AdviseRead(int x0, int y0,
                                   int sx, int sy,
                                   int bsx, int bsy,
-                                  GDALDataType bdt, CPL_UNUSED int band_count, CPL_UNUSED int *band_map,
+                                  GDALDataType bdt,
+                                  CPL_UNUSED int band_count,
+                                  CPL_UNUSED int *band_map,
                                   char **options) {
+//    printf("AdviseRead(%d, %d, %d, %d)\n", x0, y0, sx, sy);
     if (m_offline_mode || !m_use_advise_read) return CE_None;
     if (m_cache == NULL) return CE_Failure;
 
