@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: minidriver_wms.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: minidriver_wms.cpp 33717 2016-03-14 06:29:14Z goatbar $
  *
  * Project:  WMS Client Driver
  * Purpose:  Implementation of Dataset and RasterBand classes for WMS
@@ -34,7 +34,7 @@
 
 CPP_GDALWMSMiniDriverFactory(WMS)
 
-GDALWMSMiniDriver_WMS::GDALWMSMiniDriver_WMS() {
+GDALWMSMiniDriver_WMS::GDALWMSMiniDriver_WMS() : m_iversion(0) {
 }
 
 GDALWMSMiniDriver_WMS::~GDALWMSMiniDriver_WMS() {
@@ -152,7 +152,6 @@ void GDALWMSMiniDriver_WMS::GetCapabilities(GDALWMSMiniDriverCapabilities *caps)
 
 void GDALWMSMiniDriver_WMS::BuildURL(CPLString *url, const GDALWMSImageRequestInfo &iri, const char* pszRequest) {
     // http://onearth.jpl.nasa.gov/wms.cgi?request=GetMap&width=1000&height=500&layers=modis,global_mosaic&styles=&srs=EPSG:4326&format=image/jpeg&bbox=-180.000000,-90.000000,180.000000,090.000000
-    CPLLocaleC oLocaleEnforcer;
     *url = m_base_url;
     if (m_base_url.ifind( "service=") == std::string::npos)
         URLAppend(url, "&service=WMS");
@@ -166,8 +165,8 @@ void GDALWMSMiniDriver_WMS::BuildURL(CPLString *url, const GDALWMSImageRequestIn
     URLAppendF(url, "&format=%s", m_image_format.c_str());
     URLAppendF(url, "&width=%d", iri.m_sx);
     URLAppendF(url, "&height=%d", iri.m_sy);
-    URLAppendF(url, "&bbox=%.8f,%.8f,%.8f,%.8f", 
-        GetBBoxCoord(iri, m_bbox_order[0]), GetBBoxCoord(iri, m_bbox_order[1]), 
+    URLAppendF(url, "&bbox=%.8f,%.8f,%.8f,%.8f",
+        GetBBoxCoord(iri, m_bbox_order[0]), GetBBoxCoord(iri, m_bbox_order[1]),
         GetBBoxCoord(iri, m_bbox_order[2]), GetBBoxCoord(iri, m_bbox_order[3]));
 }
 
