@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: xpmdataset.cpp 31998 2015-12-04 20:03:08Z rouault $
+ * $Id: xpmdataset.cpp 33720 2016-03-15 00:39:53Z goatbar $
  *
  * Project:  XPM Driver
  * Purpose:  Implement GDAL XPM Support
@@ -31,17 +31,15 @@
 #include "gdal_pam.h"
 #include "cpl_string.h"
 #include "memdataset.h"
-#include "gdal_frmts.h"						      
+#include "gdal_frmts.h"
 
-
-CPL_CVSID("$Id: xpmdataset.cpp 31998 2015-12-04 20:03:08Z rouault $");
+CPL_CVSID("$Id: xpmdataset.cpp 33720 2016-03-15 00:39:53Z goatbar $");
 
 static unsigned char *ParseXPM( const char *pszInput,
                                 unsigned int nFileSize,
-                                int *pnXSize, int *pnYSize, 
+                                int *pnXSize, int *pnYSize,
                                 GDALColorTable **ppoRetTable );
 
-
 /************************************************************************/
 /* ==================================================================== */
 /*				XPMDataset				*/
@@ -51,21 +49,13 @@ static unsigned char *ParseXPM( const char *pszInput,
 class XPMDataset : public GDALPamDataset
 {
   public:
-                 XPMDataset();
+                 XPMDataset() {};
                  ~XPMDataset();
 
     static GDALDataset *Open( GDALOpenInfo * );
+    static int          Identify( GDALOpenInfo * );
 };
 
-/************************************************************************/
-/*                            XPMDataset()                            */
-/************************************************************************/
-
-XPMDataset::XPMDataset()
-
-{
-}
-
 /************************************************************************/
 /*                            ~XPMDataset()                             */
 /************************************************************************/
@@ -77,10 +67,10 @@ XPMDataset::~XPMDataset()
 }
 
 /************************************************************************/
-/*                                Open()                                */
+/*                            Identify()                                */
 /************************************************************************/
 
-GDALDataset *XPMDataset::Open( GDALOpenInfo * poOpenInfo )
+int XPMDataset::Identify( GDALOpenInfo * poOpenInfo )
 
 {
 /* -------------------------------------------------------------------- */
@@ -89,14 +79,26 @@ GDALDataset *XPMDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      containing the letters XPM, and to have "static" in the         */
 /*      header.                                                         */
 /* -------------------------------------------------------------------- */
-    if( poOpenInfo->nHeaderBytes < 32 
-        || strstr((const char *) poOpenInfo->pabyHeader,"XPM") == NULL 
-        || strstr((const char *) poOpenInfo->pabyHeader,"static") == NULL )
+    return poOpenInfo->nHeaderBytes >= 32 &&
+           strstr(reinterpret_cast<const char *>( poOpenInfo->pabyHeader ),
+                  "XPM") != NULL &&
+           strstr(reinterpret_cast<const char *>( poOpenInfo->pabyHeader ),
+                  "static") != NULL;
+}
+
+/************************************************************************/
+/*                                Open()                                */
+/************************************************************************/
+
+GDALDataset *XPMDataset::Open( GDALOpenInfo * poOpenInfo )
+
+{
+    if( !Identify(poOpenInfo) )
         return NULL;
 
     if( poOpenInfo->eAccess == GA_Update )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "The XPM driver does not support update access to existing"
                   " files." );
         return NULL;
@@ -105,51 +107,50 @@ GDALDataset *XPMDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Read the whole file into a memory strings.                      */
 /* -------------------------------------------------------------------- */
-    unsigned int nFileSize;
-    char *pszFileContents;
     VSILFILE *fp = VSIFOpenL( poOpenInfo->pszFilename, "rb" );
     if( fp == NULL )
         return NULL;
 
-    VSIFSeekL( fp, 0, SEEK_END );
-    nFileSize = (unsigned int) VSIFTellL( fp );
-    
-    pszFileContents = (char *) VSIMalloc(nFileSize+1);
-    if( pszFileContents == NULL )
+    if( VSIFSeekL( fp, 0, SEEK_END ) != 0 )
     {
-        CPLError( CE_Failure, CPLE_OutOfMemory, 
-                  "Insufficient memory for loading XPM file %s into memory.", 
-                  poOpenInfo->pszFilename );
-        VSIFCloseL(fp);
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
+        return NULL;
+    }
+    unsigned int nFileSize = static_cast<unsigned int>( VSIFTellL( fp ) );
+
+    char *pszFileContents = reinterpret_cast<char *>( VSI_MALLOC_VERBOSE(nFileSize+1) );
+    if( pszFileContents == NULL  )
+    {
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
         return NULL;
     }
     pszFileContents[nFileSize] = '\0';
-    
-    VSIFSeekL( fp, 0, SEEK_SET );
 
-    if( VSIFReadL( pszFileContents, 1, nFileSize, fp ) != nFileSize)
+    if( VSIFSeekL( fp, 0, SEEK_SET ) != 0 ||
+        VSIFReadL( pszFileContents, 1, nFileSize, fp ) != nFileSize)
     {
         CPLFree( pszFileContents );
-        CPLError( CE_Failure, CPLE_FileIO, 
+        CPLError( CE_Failure, CPLE_FileIO,
                   "Failed to read all %d bytes from file %s.",
                   nFileSize, poOpenInfo->pszFilename );
-        VSIFCloseL(fp);
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
         return NULL;
     }
-    
-    VSIFCloseL(fp);
+
+    CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
     fp = NULL;
 
 /* -------------------------------------------------------------------- */
 /*      Convert into a binary image.                                    */
 /* -------------------------------------------------------------------- */
-    GByte *pabyImage;
-    int   nXSize, nYSize;
-    GDALColorTable *poCT = NULL;
-
     CPLErrorReset();
 
-    pabyImage = ParseXPM( pszFileContents, nFileSize, &nXSize, &nYSize, &poCT );
+    int nXSize;
+    int nYSize;
+    GDALColorTable *poCT = NULL;
+
+    GByte *pabyImage = ParseXPM( pszFileContents, nFileSize, &nXSize, &nYSize, &poCT );
+
     CPLFree( pszFileContents );
 
     if( pabyImage == NULL )
@@ -160,9 +161,7 @@ GDALDataset *XPMDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
-    XPMDataset 	*poDS;
-
-    poDS = new XPMDataset();
+    XPMDataset 	*poDS = new XPMDataset();
 
 /* -------------------------------------------------------------------- */
 /*      Capture some information from the file that is of interest.     */
@@ -173,10 +172,8 @@ GDALDataset *XPMDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Create band information objects.                                */
 /* -------------------------------------------------------------------- */
-    MEMRasterBand *poBand;
-
-    poBand = new MEMRasterBand( poDS, 1, pabyImage, GDT_Byte, 1, nXSize, 
-                                TRUE );
+    MEMRasterBand *poBand = new MEMRasterBand( poDS, 1, pabyImage, GDT_Byte, 1,
+                                               nXSize, TRUE );
     poBand->SetColorTable( poCT );
     poDS->SetBand( 1, poBand );
 
@@ -202,35 +199,31 @@ GDALDataset *XPMDataset::Open( GDALOpenInfo * poOpenInfo )
 
 static GDALDataset *
 XPMCreateCopy( const char * pszFilename,
-               CPL_UNUSED GDALDataset *poSrcDS,
+               GDALDataset *poSrcDS,
                int bStrict,
-               CPL_UNUSED char ** papszOptions,
-               CPL_UNUSED GDALProgressFunc pfnProgress,
-               CPL_UNUSED void * pProgressData )
+               char ** /* papszOptions */,
+               GDALProgressFunc /* pfnProgress */,
+               void * /* pProgressData */)
 {
-    int  nBands = poSrcDS->GetRasterCount();
-    int  nXSize = poSrcDS->GetRasterXSize();
-    int  nYSize = poSrcDS->GetRasterYSize();
-    GDALColorTable *poCT;
-
 /* -------------------------------------------------------------------- */
 /*      Some some rudimentary checks                                    */
 /* -------------------------------------------------------------------- */
+    const int nBands = poSrcDS->GetRasterCount();
     if( nBands != 1 )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "XPM driver only supports one band images.\n" );
 
         return NULL;
     }
 
-    if( poSrcDS->GetRasterBand(1)->GetRasterDataType() != GDT_Byte 
+    if( poSrcDS->GetRasterBand(1)->GetRasterDataType() != GDT_Byte
         && bStrict )
     {
-        CPLError( CE_Failure, CPLE_NotSupported, 
+        CPLError( CE_Failure, CPLE_NotSupported,
                   "XPM driver doesn't support data type %s. "
-                  "Only eight bit bands supported.\n", 
-                  GDALGetDataTypeName( 
+                  "Only eight bit bands supported.\n",
+                  GDALGetDataTypeName(
                       poSrcDS->GetRasterBand(1)->GetRasterDataType()) );
 
         return NULL;
@@ -240,16 +233,16 @@ XPMCreateCopy( const char * pszFilename,
 /*      If there is no colortable on the source image, create a         */
 /*      greyscale one with 64 levels of grey.                           */
 /* -------------------------------------------------------------------- */
-    GDALRasterBand	*poBand = poSrcDS->GetRasterBand(1);
-    int                 i;
-    GDALColorTable      oGreyTable;
+    GDALRasterBand *poBand = poSrcDS->GetRasterBand(1);
+
+    GDALColorTable oGreyTable;
+    GDALColorTable *poCT = poBand->GetColorTable();
 
-    poCT = poBand->GetColorTable();
     if( poCT == NULL )
     {
         poCT = &oGreyTable;
 
-        for( i = 0; i < 256; i++ )
+        for( int i = 0; i < 256; i++ )
         {
             GDALColorEntry sColor;
 
@@ -266,15 +259,17 @@ XPMCreateCopy( const char * pszFilename,
 /*      Build list of active colors, and the mapping from pixels to     */
 /*      our active colormap.                                            */
 /* -------------------------------------------------------------------- */
-    const char *pszColorCodes = " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*()-+=[]|:;,.<>?/";
+    const char *pszColorCodes
+        = " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@"
+        "#$%^&*()-+=[]|:;,.<>?/";
 
     int  anPixelMapping[256];
     GDALColorEntry asPixelColor[256];
-    int  nActiveColors = MIN(poCT->GetColorEntryCount(),256);
+    int nActiveColors = MIN(poCT->GetColorEntryCount(),256);
 
-    // Setup initial colortable and pixel value mapping. 
+    // Setup initial colortable and pixel value mapping.
     memset( anPixelMapping+0, 0, sizeof(int) * 256 );
-    for( i = 0; i < nActiveColors; i++ )
+    for( int i = 0; i < nActiveColors; i++ )
     {
         poCT->GetColorEntryAsRGB( i, asPixelColor + i );
         anPixelMapping[i] = i;
@@ -283,24 +278,24 @@ XPMCreateCopy( const char * pszFilename,
 /* ==================================================================== */
 /*      Iterate merging colors until we are under our limit (about 85). */
 /* ==================================================================== */
-    while( nActiveColors > (int) strlen(pszColorCodes) )
+    while( nActiveColors > static_cast<int>( strlen(pszColorCodes) ) )
     {
         int nClosestDistance = 768;
-        int iClose1 = -1, iClose2 = -1;
-        int iColor1, iColor2;
+        int iClose1 = -1;
+        int iClose2 = -1;
 
-        // Find the closest pair of colors. 
-        for( iColor1 = 0; iColor1 < nActiveColors; iColor1++ )
+        // Find the closest pair of colors.
+        for( int iColor1 = 0; iColor1 < nActiveColors; iColor1++ )
         {
-            for( iColor2 = iColor1+1; iColor2 < nActiveColors; iColor2++ )
+            for( int iColor2 = iColor1+1; iColor2 < nActiveColors; iColor2++ )
             {
-                int	nDistance;
+                int nDistance;
 
-                if( asPixelColor[iColor1].c4 < 128 
+                if( asPixelColor[iColor1].c4 < 128
                     && asPixelColor[iColor2].c4 < 128 )
                     nDistance = 0;
                 else
-                    nDistance = 
+                    nDistance =
                         ABS(asPixelColor[iColor1].c1-asPixelColor[iColor2].c1)
                       + ABS(asPixelColor[iColor1].c2-asPixelColor[iColor2].c2)
                       + ABS(asPixelColor[iColor1].c3-asPixelColor[iColor2].c3);
@@ -322,8 +317,8 @@ XPMCreateCopy( const char * pszFilename,
             break;
 
         // Merge two selected colors - shift icolor2 into icolor1 and
-        // move the last active color into icolor2's slot. 
-        for( i = 0; i < 256; i++ )
+        // move the last active color into icolor2's slot.
+        for( int i = 0; i < 256; i++ )
         {
             if( anPixelMapping[i] == iClose2 )
                 anPixelMapping[i] = iClose1;
@@ -334,17 +329,15 @@ XPMCreateCopy( const char * pszFilename,
         asPixelColor[iClose2] = asPixelColor[nActiveColors-1];
         nActiveColors--;
     }
-        
+
 /* ==================================================================== */
 /*      Write the output image.                                         */
 /* ==================================================================== */
-    VSILFILE	*fpPBM;
-
-    fpPBM = VSIFOpenL( pszFilename, "wb+" );
+    VSILFILE *fpPBM = VSIFOpenL( pszFilename, "wb+" );
     if( fpPBM == NULL )
     {
-        CPLError( CE_Failure, CPLE_OpenFailed, 
-                  "Unable to create file `%s'.", 
+        CPLError( CE_Failure, CPLE_OpenFailed,
+                  "Unable to create file `%s'.",
                   pszFilename );
 
         return NULL;
@@ -353,62 +346,76 @@ XPMCreateCopy( const char * pszFilename,
 /* -------------------------------------------------------------------- */
 /*      Write the header lines.                                         */
 /* -------------------------------------------------------------------- */
-    VSIFPrintfL( fpPBM, "/* XPM */\n" );
-    VSIFPrintfL( fpPBM, "static char *%s[] = {\n", 
-             CPLGetBasename( pszFilename ) );
-    VSIFPrintfL( fpPBM, "/* width height num_colors chars_per_pixel */\n" );
-    VSIFPrintfL( fpPBM, "\"  %3d   %3d     %3d             1\",\n",
-             nXSize, nYSize, nActiveColors );
-    VSIFPrintfL( fpPBM, "/* colors */\n" );
+    bool bOK = VSIFPrintfL( fpPBM, "/* XPM */\n" ) >= 0;
+    bOK &= VSIFPrintfL( fpPBM, "static char *%s[] = {\n",
+             CPLGetBasename( pszFilename ) ) >= 0;
+    bOK &= VSIFPrintfL( fpPBM, "/* width height num_colors chars_per_pixel */\n" ) >= 0;
+
+    const int nXSize = poSrcDS->GetRasterXSize();
+    const int nYSize = poSrcDS->GetRasterYSize();
+
+    bOK &= VSIFPrintfL( fpPBM, "\"  %3d   %3d     %3d             1\",\n",
+             nXSize, nYSize, nActiveColors ) >= 0;
+
+    bOK &= VSIFPrintfL( fpPBM, "/* colors */\n" ) >= 0;
 
 /* -------------------------------------------------------------------- */
 /*      Write the color table.                                          */
 /* -------------------------------------------------------------------- */
-    for( i = 0; i < nActiveColors; i++ )
+    for( int i = 0; bOK && i < nActiveColors; i++ )
     {
         if( asPixelColor[i].c4 < 128 )
-            VSIFPrintfL( fpPBM, "\"%c c None\",\n", pszColorCodes[i] );
+            bOK &= VSIFPrintfL( fpPBM, "\"%c c None\",\n", pszColorCodes[i] ) >= 0;
         else
-            VSIFPrintfL( fpPBM, 
+            bOK &= VSIFPrintfL( fpPBM,
                      "\"%c c #%02x%02x%02x\",\n",
                      pszColorCodes[i],
-                     asPixelColor[i].c1, 
-                     asPixelColor[i].c2, 
-                     asPixelColor[i].c3 );
+                     asPixelColor[i].c1,
+                     asPixelColor[i].c2,
+                     asPixelColor[i].c3 ) >= 0;
     }
 
 /* -------------------------------------------------------------------- */
 /*	Dump image.							*/
 /* -------------------------------------------------------------------- */
-    int iLine;
-    GByte 	*pabyScanline;
+    GByte *pabyScanline = reinterpret_cast<GByte *>( CPLMalloc( nXSize ) );
 
-    pabyScanline = (GByte *) CPLMalloc( nXSize );
-    for( iLine = 0; iLine < nYSize; iLine++ )
+    for( int iLine = 0; bOK && iLine < nYSize; iLine++ )
     {
-        poBand->RasterIO( GF_Read, 0, iLine, nXSize, 1, 
-                          (void *) pabyScanline, nXSize, 1, GDT_Byte, 0, 0, NULL );
-        
-        VSIFPutcL( '"', fpPBM );
+        if( poBand->RasterIO(
+               GF_Read, 0, iLine, nXSize, 1,
+               reinterpret_cast<void *>( pabyScanline), nXSize, 1, GDT_Byte,
+               0, 0, NULL ) != CE_None )
+        {
+            CPLFree( pabyScanline );
+            CPL_IGNORE_RET_VAL(VSIFCloseL( fpPBM ));
+            return NULL;
+        }
+
+        bOK &= VSIFPutcL( '"', fpPBM ) >= 0;
         for( int iPixel = 0; iPixel < nXSize; iPixel++ )
-            VSIFPutcL( pszColorCodes[anPixelMapping[pabyScanline[iPixel]]], 
-                   fpPBM);
-        VSIFPrintfL( fpPBM, "\",\n" );
+            bOK &= VSIFPutcL( pszColorCodes[anPixelMapping[pabyScanline[iPixel]]],
+                   fpPBM) >= 0;
+        bOK &= VSIFPrintfL( fpPBM, "\",\n" ) >= 0;
     }
-    
+
     CPLFree( pabyScanline );
 
 /* -------------------------------------------------------------------- */
 /*      cleanup                                                         */
 /* -------------------------------------------------------------------- */
-    VSIFPrintfL( fpPBM, "};\n" );
-    VSIFCloseL( fpPBM );
+    bOK &= VSIFPrintfL( fpPBM, "};\n" ) >= 0;
+    if( VSIFCloseL( fpPBM ) != 0 )
+        bOK = false;
+
+    if( !bOK )
+        return NULL;
 
 /* -------------------------------------------------------------------- */
 /*      Re-open dataset, and copy any auxiliary pam information.         */
 /* -------------------------------------------------------------------- */
-    GDALPamDataset *poDS = (GDALPamDataset *)
-        GDALOpen( pszFilename, GA_ReadOnly );
+    GDALPamDataset *poDS = reinterpret_cast<GDALPamDataset *>(
+        GDALOpen( pszFilename, GA_ReadOnly ) );
 
     if( poDS )
         poDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
@@ -423,29 +430,25 @@ XPMCreateCopy( const char * pszFilename,
 void GDALRegister_XPM()
 
 {
-    GDALDriver	*poDriver;
+    if( GDALGetDriverByName( "XPM" ) != NULL )
+        return;
 
-    if( GDALGetDriverByName( "XPM" ) == NULL )
-    {
-        poDriver = new GDALDriver();
-        
-        poDriver->SetDescription( "XPM" );
-        poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
-        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
-                                   "X11 PixMap Format" );
-        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_various.html#XPM" );
-        poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "xpm" );
-        poDriver->SetMetadataItem( GDAL_DMD_MIMETYPE, "image/x-xpixmap" );
-        poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, 
-                                   "Byte" );
-        poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    GDALDriver *poDriver = new GDALDriver();
 
-        poDriver->pfnOpen = XPMDataset::Open;
-        poDriver->pfnCreateCopy = XPMCreateCopy;
+    poDriver->SetDescription( "XPM" );
+    poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "X11 PixMap Format" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_various.html#XPM" );
+    poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "xpm" );
+    poDriver->SetMetadataItem( GDAL_DMD_MIMETYPE, "image/x-xpixmap" );
+    poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, "Byte" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
-        GetGDALDriverManager()->RegisterDriver( poDriver );
-    }
+    poDriver->pfnOpen = XPMDataset::Open;
+    poDriver->pfnIdentify = XPMDataset::Identify;
+    poDriver->pfnCreateCopy = XPMCreateCopy;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
 
 /************************************************************************/
@@ -455,7 +458,7 @@ void GDALRegister_XPM()
 static unsigned char *
 ParseXPM( const char *pszInput,
           unsigned int nFileSize,
-          int *pnXSize, int *pnYSize, 
+          int *pnXSize, int *pnYSize,
           GDALColorTable **ppoRetTable )
 
 {
@@ -463,12 +466,10 @@ ParseXPM( const char *pszInput,
 /*      Parse input into an array of strings from within the first C    */
 /*      initializer (list os comma separated strings in braces).        */
 /* ==================================================================== */
-    char **papszXPMList = NULL;
     const char *pszNext = pszInput;
-    int  i;
 
     // Skip till after open brace.
-    while( *pszNext != '\0' && *pszNext != '{' ) 
+    while( *pszNext != '\0' && *pszNext != '{' )
         pszNext++;
 
     if( *pszNext == '\0' )
@@ -477,22 +478,23 @@ ParseXPM( const char *pszInput,
     pszNext++;
 
     // Read lines till close brace.
-    
+
+    char **papszXPMList = NULL;
+    int  i = 0;
+
     while( *pszNext != '\0' && *pszNext != '}' )
     {
-        // skip whole comment. 
-        if( EQUALN(pszNext,"/*",2) )
+        // skip whole comment.
+        if( STARTS_WITH_CI(pszNext, "/*") )
         {
             pszNext += 2;
-            while( *pszNext != '\0' && !EQUALN(pszNext,"*/",2) )
+            while( *pszNext != '\0' && !STARTS_WITH_CI(pszNext, "*/") )
                 pszNext++;
         }
 
         // reading string constants
         else if( *pszNext == '"' )
         {
-            char   *pszLine;
-
             pszNext++;
             i = 0;
 
@@ -505,7 +507,7 @@ ParseXPM( const char *pszInput,
                 return NULL;
             }
 
-            pszLine = (char *) CPLMalloc(i+1);
+            char *pszLine = reinterpret_cast<char *>( CPLMalloc(i+1) );
             strncpy( pszLine, pszNext, i );
             pszLine[i] = '\0';
 
@@ -519,18 +521,18 @@ ParseXPM( const char *pszInput,
             pszNext++;
     }
 
-    if( CSLCount(papszXPMList) < 3 || *pszNext != '}' )
+    if( papszXPMList == NULL || CSLCount(papszXPMList) < 3 || *pszNext != '}' )
     {
         CSLDestroy( papszXPMList );
         return NULL;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Get the image information.                                      */
 /* -------------------------------------------------------------------- */
     int nColorCount, nCharsPerPixel;
 
-    if( sscanf( papszXPMList[0], "%d %d %d %d", 
+    if( sscanf( papszXPMList[0], "%d %d %d %d",
                 pnXSize, pnYSize, &nColorCount, &nCharsPerPixel ) != 4 ||
         *pnXSize <= 0 || *pnYSize <= 0 || nColorCount <= 0 || nColorCount > 256 ||
         static_cast<GUIntBig>(*pnXSize) * static_cast<GUIntBig>(*pnYSize) > nFileSize )
@@ -553,32 +555,29 @@ ParseXPM( const char *pszInput,
 /* -------------------------------------------------------------------- */
 /*      Parse out colors.                                               */
 /* -------------------------------------------------------------------- */
-    int iColor;
     int anCharLookup[256];
     GDALColorTable oCTable;
 
-    for( i = 0; i < 256; i++ ) 
+    for( i = 0; i < 256; i++ )
         anCharLookup[i] = -1;
 
-    for( iColor = 0; iColor < nColorCount; iColor++ )
+    for( int iColor = 0; iColor < nColorCount; iColor++ )
     {
         if( papszXPMList[iColor+1] == NULL )
         {
-            CPLError( CE_Failure, CPLE_AppDefined, 
-                      "Missing color definition for %d in XPM header.", 
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "Missing color definition for %d in XPM header.",
                       iColor+1 );
             CSLDestroy( papszXPMList );
             return NULL;
         }
 
         char **papszTokens = CSLTokenizeString( papszXPMList[iColor+1]+1 );
-        GDALColorEntry sColor;
-        int            nRed, nGreen, nBlue;
 
         if( CSLCount(papszTokens) != 2 || !EQUAL(papszTokens[0],"c") )
         {
-            CPLError( CE_Failure, CPLE_AppDefined, 
-                      "Ill formed color definition (%s) in XPM header.", 
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "Ill formed color definition (%s) in XPM header.",
                       papszXPMList[iColor+1] );
             CSLDestroy( papszXPMList );
             CSLDestroy( papszTokens );
@@ -586,7 +585,10 @@ ParseXPM( const char *pszInput,
         }
 
         anCharLookup[*(reinterpret_cast<GByte*>(papszXPMList[iColor+1]))] = iColor;
-        
+
+        GDALColorEntry sColor;
+        int nRed, nGreen, nBlue;
+
         if( EQUAL(papszTokens[1],"None") )
         {
             sColor.c1 = 0;
@@ -594,11 +596,11 @@ ParseXPM( const char *pszInput,
             sColor.c3 = 0;
             sColor.c4 = 0;
         }
-        else if( sscanf( papszTokens[1], "#%02x%02x%02x", 
+        else if( sscanf( papszTokens[1], "#%02x%02x%02x",
                          &nRed, &nGreen, &nBlue ) != 3 )
         {
-            CPLError( CE_Failure, CPLE_AppDefined, 
-                      "Ill formed color definition (%s) in XPM header.", 
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "Ill formed color definition (%s) in XPM header.",
                       papszXPMList[iColor+1] );
             CSLDestroy( papszXPMList );
             CSLDestroy( papszTokens );
@@ -606,9 +608,9 @@ ParseXPM( const char *pszInput,
         }
         else
         {
-            sColor.c1 = (short) nRed;
-            sColor.c2 = (short) nGreen;
-            sColor.c3 = (short) nBlue;
+            sColor.c1 = static_cast<short>( nRed );
+            sColor.c2 = static_cast<short>( nGreen );
+            sColor.c3 = static_cast<short>( nBlue );
             sColor.c4 = 255;
         }
 
@@ -620,20 +622,14 @@ ParseXPM( const char *pszInput,
 /* -------------------------------------------------------------------- */
 /*      Prepare image buffer.                                           */
 /* -------------------------------------------------------------------- */
-    GByte *pabyImage;
-
-    pabyImage = (GByte *) VSIMalloc2(*pnXSize, *pnYSize);
+    GByte *pabyImage
+        = reinterpret_cast<GByte *>( VSI_CALLOC_VERBOSE(*pnXSize, *pnYSize) );
     if( pabyImage == NULL )
     {
-        CPLError( CE_Failure, CPLE_OutOfMemory, 
-                  "Insufficient memory for %dx%d XPM image buffer.", 
-                  *pnXSize, *pnYSize );
         CSLDestroy( papszXPMList );
         return NULL;
     }
 
-    memset( pabyImage, 0, *pnXSize * *pnYSize );
-
 /* -------------------------------------------------------------------- */
 /*      Parse image.                                                    */
 /* -------------------------------------------------------------------- */
@@ -646,7 +642,7 @@ ParseXPM( const char *pszInput,
         {
             CPLFree( pabyImage );
             CSLDestroy( papszXPMList );
-            CPLError( CE_Failure, CPLE_AppDefined, 
+            CPLError( CE_Failure, CPLE_AppDefined,
                       "Insufficient imagery lines in XPM image." );
             return NULL;
         }
@@ -658,7 +654,8 @@ ParseXPM( const char *pszInput,
             const int nPixelValue
                 = anCharLookup[pabyInLine[iPixel]];
             if( nPixelValue != -1 )
-                pabyImage[iLine * *pnXSize + iPixel] = (GByte) nPixelValue;
+                pabyImage[iLine * *pnXSize + iPixel]
+                    = static_cast<GByte>( nPixelValue );
         }
     }
 
