@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdaldrivermanager.cpp 27121 2014-04-03 22:08:55Z rouault $
+ * $Id: gdaldrivermanager.cpp 34979 2016-08-08 09:30:34Z rouault $
  *
  * Project:  GDAL Core
  * Purpose:  Implementation of GDALDriverManager class.
@@ -28,13 +28,12 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "gdal_priv.h"
+#include "cpl_multiproc.h"
 #include "cpl_string.h"
-#include "cpl_multiproc.h"
-#include "ogr_srs_api.h"
-#include "cpl_multiproc.h"
-#include "gdal_pam.h"
 #include "gdal_alg_priv.h"
+#include "gdal_pam.h"
+#include "gdal_priv.h"
+#include "ogr_srs_api.h"
 
 #ifdef _MSC_VER
 #  ifdef MSVC_USE_VLD
@@ -43,10 +42,7 @@
 #  endif
 #endif
 
-CPL_CVSID("$Id: gdaldrivermanager.cpp 27121 2014-04-03 22:08:55Z rouault $");
-
-static const char *pszUpdatableINST_DATA = 
-"__INST_DATA_TARGET:                                                                                                                                      ";
+CPL_CVSID("$Id: gdaldrivermanager.cpp 34979 2016-08-08 09:30:34Z rouault $");
 
 /************************************************************************/
 /* ==================================================================== */
@@ -55,9 +51,9 @@ static const char *pszUpdatableINST_DATA =
 /************************************************************************/
 
 static volatile GDALDriverManager        *poDM = NULL;
-static void *hDMMutex = NULL;
+static CPLMutex *hDMMutex = NULL;
 
-void** GDALGetphDMMutex() { return &hDMMutex; }
+CPLMutex** GDALGetphDMMutex() { return &hDMMutex; }
 
 /************************************************************************/
 /*                        GetGDALDriverManager()                        */
@@ -96,41 +92,30 @@ GDALDriverManager * GetGDALDriverManager()
 /*                         GDALDriverManager()                          */
 /************************************************************************/
 
-GDALDriverManager::GDALDriverManager()
-
+GDALDriverManager::GDALDriverManager() :
+    nDrivers(0),
+    papoDrivers(NULL)
 {
-    nDrivers = 0;
-    papoDrivers = NULL;
-    pszHome = CPLStrdup("");
-
     CPLAssert( poDM == NULL );
 
 /* -------------------------------------------------------------------- */
 /*      We want to push a location to search for data files             */
 /*      supporting GDAL/OGR such as EPSG csv files, S-57 definition     */
-/*      files, and so forth.  The static pszUpdateableINST_DATA         */
-/*      string can be updated within the shared library or              */
-/*      executable during an install to point installed data            */
-/*      directory.  If it isn't burned in here then we use the          */
-/*      INST_DATA macro (setup at configure time) if                    */
-/*      available. Otherwise we don't push anything and we hope         */
-/*      other mechanisms such as environment variables will have        */
-/*      been employed.                                                  */
+/*      files, and so forth.  Use the INST_DATA macro (setup at         */
+/*      configure time) if available. Otherwise we don't push anything  */
+/*      and we hope other mechanisms such as environment variables will */
+/*      have been employed.                                             */
 /* -------------------------------------------------------------------- */
     if( CPLGetConfigOption( "GDAL_DATA", NULL ) != NULL )
     {
         // this one is picked up automatically by finder initialization.
     }
-    else if( pszUpdatableINST_DATA[19] != ' ' )
-    {
-        CPLPushFinderLocation( pszUpdatableINST_DATA + 19 );
-    }
+#ifdef INST_DATA
     else
     {
-#ifdef INST_DATA
         CPLPushFinderLocation( INST_DATA );
-#endif
     }
+#endif
 }
 
 /************************************************************************/
@@ -146,12 +131,6 @@ GDALDriverManager::~GDALDriverManager()
 /* -------------------------------------------------------------------- */
 /*      Cleanup any open datasets.                                      */
 /* -------------------------------------------------------------------- */
-    int i, nDSCount;
-    GDALDataset **papoDSList;
-
-    /* First begin by requesting each reamining dataset to drop any reference */
-    /* to other datasets */
-    int bHasDroppedRef;
 
     /* We have to prevent the destroying of the dataset pool during this first */
     /* phase, otherwise it cause crashes with a VRT B referencing a VRT A, and if */
@@ -166,28 +145,34 @@ GDALDriverManager::~GDALDriverManager()
     /* datasets, which defeat some "design" of the proxy pool */
     GDALDatasetPoolPreventDestroy();
 
+    /* First begin by requesting each remaining dataset to drop any reference */
+    /* to other datasets */
+    bool bHasDroppedRef = false;
+
     do
     {
-        papoDSList = GDALDataset::GetOpenDatasets(&nDSCount);
+        int nDSCount;
+        GDALDataset **papoDSList = GDALDataset::GetOpenDatasets(&nDSCount);
         /* If a dataset has dropped a reference, the list might have become */
         /* invalid, so go out of the loop and try again with the new valid */
         /* list */
-        bHasDroppedRef = FALSE;
-        for(i=0;i<nDSCount && !bHasDroppedRef;i++)
+        bHasDroppedRef = false;
+        for(int i=0;i<nDSCount && !bHasDroppedRef;i++)
         {
             //CPLDebug("GDAL", "Call CloseDependentDatasets() on %s",
             //      papoDSList[i]->GetDescription() );
-            bHasDroppedRef = papoDSList[i]->CloseDependentDatasets();
+            bHasDroppedRef = CPL_TO_BOOL(papoDSList[i]->CloseDependentDatasets());
         }
     } while(bHasDroppedRef);
 
-    /* Now let's destroy the dataset pool. Nobody shoud use it afterwards */
+    /* Now let's destroy the dataset pool. Nobody should use it afterwards */
     /* if people have well released their dependent datasets above */
     GDALDatasetPoolForceDestroy();
 
     /* Now close the stand-alone datasets */
-    papoDSList = GDALDataset::GetOpenDatasets(&nDSCount);
-    for(i=0;i<nDSCount;i++)
+    int nDSCount;
+    GDALDataset **papoDSList = GDALDataset::GetOpenDatasets(&nDSCount);
+    for(int i=0;i<nDSCount;i++)
     {
         CPLDebug( "GDAL", "force close of %s (%p) in GDALDriverManager cleanup.",
                   papoDSList[i]->GetDescription(), papoDSList[i] );
@@ -202,7 +187,7 @@ GDALDriverManager::~GDALDriverManager()
 /* -------------------------------------------------------------------- */
     while( GetDriverCount() > 0 )
     {
-        GDALDriver      *poDriver = GetDriver(0);
+        GDALDriver *poDriver = GetDriver(0);
 
         DeregisterDriver(poDriver);
         delete poDriver;
@@ -214,7 +199,6 @@ GDALDriverManager::~GDALDriverManager()
 /*      Cleanup local memory.                                           */
 /* -------------------------------------------------------------------- */
     VSIFree( papoDrivers );
-    VSIFree( pszHome );
 
 /* -------------------------------------------------------------------- */
 /*      Cleanup any Proxy related memory.                               */
@@ -222,7 +206,7 @@ GDALDriverManager::~GDALDriverManager()
     PamCleanProxyDB();
 
 /* -------------------------------------------------------------------- */
-/*      Blow away all the finder hints paths.  We really shouldn't      */
+/*      Blow away all the finder hints paths.  We really should not     */
 /*      be doing all of them, but it is currently hard to keep track    */
 /*      of those that actually belong to us.                            */
 /* -------------------------------------------------------------------- */
@@ -259,11 +243,11 @@ GDALDriverManager::~GDALDriverManager()
 /* -------------------------------------------------------------------- */
 /*      Cleanup dataset list mutex                                      */
 /* -------------------------------------------------------------------- */
-    if ( *GDALGetphDLMutex() != NULL ) 
-    { 
-        CPLDestroyMutex( *GDALGetphDLMutex() ); 
-        *GDALGetphDLMutex() = NULL; 
-    } 
+    if ( *GDALGetphDLMutex() != NULL )
+    {
+        CPLDestroyMutex( *GDALGetphDLMutex() );
+        *GDALGetphDLMutex() = NULL;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Cleanup raster block mutex                                      */
@@ -285,9 +269,14 @@ GDALDriverManager::~GDALDriverManager()
 /* -------------------------------------------------------------------- */
     CPLCleanupSetlocaleMutex();
 
+/* -------------------------------------------------------------------- */
+/*      Cleanup QHull mutex                                             */
+/* -------------------------------------------------------------------- */
+    GDALTriangulationTerminate();
+
 /* -------------------------------------------------------------------- */
 /*      Cleanup the master CPL mutex, which governs the creation        */
-/*      of all other mutexes.                                           */ 
+/*      of all other mutexes.                                           */
 /* -------------------------------------------------------------------- */
     CPLCleanupMasterMutex();
 
@@ -351,10 +340,7 @@ GDALDriver * GDALDriverManager::GetDriver( int iDriver )
 {
     CPLMutexHolderD( &hDMMutex );
 
-    if( iDriver < 0 || iDriver >= nDrivers )
-        return NULL;
-    else
-        return papoDrivers[iDriver];
+    return GetDriver_unlocked(iDriver);
 }
 
 /************************************************************************/
@@ -405,11 +391,9 @@ int GDALDriverManager::RegisterDriver( GDALDriver * poDriver )
 /*      If it is already registered, just return the existing           */
 /*      index.                                                          */
 /* -------------------------------------------------------------------- */
-    if( GetDriverByName( poDriver->GetDescription() ) != NULL )
+    if( GetDriverByName_unlocked( poDriver->GetDescription() ) != NULL )
     {
-        int             i;
-
-        for( i = 0; i < nDrivers; i++ )
+        for( int i = 0; i < nDrivers; i++ )
         {
             if( papoDrivers[i] == poDriver )
             {
@@ -419,22 +403,53 @@ int GDALDriverManager::RegisterDriver( GDALDriver * poDriver )
 
         CPLAssert( FALSE );
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Otherwise grow the list to hold the new entry.                  */
 /* -------------------------------------------------------------------- */
-    papoDrivers = (GDALDriver **)
-        VSIRealloc(papoDrivers, sizeof(GDALDriver *) * (nDrivers+1));
+    GDALDriver** papoNewDrivers = (GDALDriver **)
+        VSI_REALLOC_VERBOSE(papoDrivers, sizeof(GDALDriver *) * (nDrivers+1));
+    if( papoNewDrivers == NULL )
+        return -1;
+    papoDrivers = papoNewDrivers;
 
     papoDrivers[nDrivers] = poDriver;
     nDrivers++;
 
+    if( poDriver->pfnOpen != NULL ||
+        poDriver->pfnOpenWithDriverArg != NULL )
+        poDriver->SetMetadataItem( GDAL_DCAP_OPEN, "YES" );
+
     if( poDriver->pfnCreate != NULL )
         poDriver->SetMetadataItem( GDAL_DCAP_CREATE, "YES" );
-    
+
     if( poDriver->pfnCreateCopy != NULL )
         poDriver->SetMetadataItem( GDAL_DCAP_CREATECOPY, "YES" );
 
+    /* Backward compatibility for GDAL raster out-of-tree drivers: */
+    /* if a driver hasn't explicitly set a vector capability, assume it is */
+    /* a raster driver (legacy OGR drivers will have DCAP_VECTOR set before */
+    /* calling RegisterDriver() ) */
+    if( poDriver->GetMetadataItem( GDAL_DCAP_RASTER ) == NULL &&
+        poDriver->GetMetadataItem( GDAL_DCAP_VECTOR ) == NULL &&
+        poDriver->GetMetadataItem( GDAL_DCAP_GNM ) == NULL )
+    {
+        CPLDebug("GDAL", "Assuming DCAP_RASTER for driver %s. Please fix it.",
+                 poDriver->GetDescription() );
+        poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
+    }
+
+    if( poDriver->GetMetadataItem( GDAL_DMD_OPENOPTIONLIST ) != NULL &&
+        poDriver->pfnIdentify == NULL &&
+        !STARTS_WITH_CI(poDriver->GetDescription(), "Interlis") )
+    {
+        CPLDebug("GDAL", "Driver %s that defines GDAL_DMD_OPENOPTIONLIST must also "
+                 "implement Identify(), so that it can be used",
+                 poDriver->GetDescription() );
+    }
+
+    oMapNameToDrivers[CPLString(poDriver->GetDescription()).toupper()] = poDriver;
+
     int iResult = nDrivers - 1;
 
     return iResult;
@@ -476,10 +491,10 @@ int CPL_STDCALL GDALRegisterDriver( GDALDriverH hDriver )
 void GDALDriverManager::DeregisterDriver( GDALDriver * poDriver )
 
 {
-    int         i;
     CPLMutexHolderD( &hDMMutex );
 
-    for( i = 0; i < nDrivers; i++ )
+    int i = 0;
+    for( ; i < nDrivers; i++ )
     {
         if( papoDrivers[i] == poDriver )
             break;
@@ -488,12 +503,14 @@ void GDALDriverManager::DeregisterDriver( GDALDriver * poDriver )
     if( i == nDrivers )
         return;
 
-    while( i < nDrivers-1 )
+    oMapNameToDrivers.erase(CPLString(poDriver->GetDescription()).toupper());
+    nDrivers--;
+    // Move all following drivers down by one to pack the list.
+    while( i < nDrivers )
     {
         papoDrivers[i] = papoDrivers[i+1];
         i++;
     }
-    nDrivers--;
 }
 
 /************************************************************************/
@@ -532,17 +549,13 @@ void CPL_STDCALL GDALDeregisterDriver( GDALDriverH hDriver )
 GDALDriver * GDALDriverManager::GetDriverByName( const char * pszName )
 
 {
-    int         i;
-
     CPLMutexHolderD( &hDMMutex );
 
-    for( i = 0; i < nDrivers; i++ )
-    {
-        if( EQUAL(papoDrivers[i]->GetDescription(), pszName) )
-            return papoDrivers[i];
-    }
+    // Alias old name to new name
+    if( EQUAL(pszName, "CartoDB") )
+        pszName = "Carto";
 
-    return NULL;
+    return oMapNameToDrivers[CPLString(pszName).toupper()];
 }
 
 /************************************************************************/
@@ -563,29 +576,6 @@ GDALDriverH CPL_STDCALL GDALGetDriverByName( const char * pszName )
     return( GetGDALDriverManager()->GetDriverByName( pszName ) );
 }
 
-/************************************************************************/
-/*                              GetHome()                               */
-/************************************************************************/
-
-const char *GDALDriverManager::GetHome()
-
-{
-    return pszHome;
-}
-
-/************************************************************************/
-/*                              SetHome()                               */
-/************************************************************************/
-
-void GDALDriverManager::SetHome( const char * pszNewHome )
-
-{
-    CPLMutexHolderD( &hDMMutex );
-
-    CPLFree( pszHome );
-    pszHome = CPLStrdup(pszNewHome);
-}
-
 /************************************************************************/
 /*                          AutoSkipDrivers()                           */
 /************************************************************************/
@@ -593,40 +583,58 @@ void GDALDriverManager::SetHome( const char * pszNewHome )
 /**
  * \brief This method unload undesirable drivers.
  *
- * All drivers specified in the space delimited list in the GDAL_SKIP 
- * environment variable) will be deregistered and destroyed.  This method 
- * should normally be called after registration of standard drivers to allow 
+ * All drivers specified in the comma delimited list in the GDAL_SKIP
+ * environment variable) will be deregistered and destroyed.  This method
+ * should normally be called after registration of standard drivers to allow
  * the user a way of unloading undesired drivers.  The GDALAllRegister()
  * function already invokes AutoSkipDrivers() at the end, so if that functions
  * is called, it should not be necessary to call this method from application
- * code. 
+ * code.
+ *
+ * Note: space separator is also accepted for backward compatibility, but some
+ * vector formats have spaces in their names, so it is encouraged to use comma
+ * to avoid issues.
  */
 
 void GDALDriverManager::AutoSkipDrivers()
 
 {
-    if( CPLGetConfigOption( "GDAL_SKIP", NULL ) == NULL )
-        return;
-
-    char **papszList = CSLTokenizeString( CPLGetConfigOption("GDAL_SKIP","") );
-
-    for( int i = 0; i < CSLCount(papszList); i++ )
+    char **apapszList[2] = { NULL, NULL };
+    const char* pszGDAL_SKIP = CPLGetConfigOption( "GDAL_SKIP", NULL );
+    if( pszGDAL_SKIP != NULL )
     {
-        GDALDriver *poDriver = GetDriverByName( papszList[i] );
+        /* Favour comma as a separator. If not found, then use space */
+        const char* pszSep = (strchr(pszGDAL_SKIP, ',') != NULL) ? "," : " ";
+        apapszList[0] = CSLTokenizeStringComplex( pszGDAL_SKIP, pszSep, FALSE, FALSE);
+    }
+    const char* pszOGR_SKIP = CPLGetConfigOption( "OGR_SKIP", NULL );
+    if( pszOGR_SKIP != NULL )
+    {
+        /* OGR has always used comma as a separator */
+        apapszList[1] = CSLTokenizeStringComplex(pszOGR_SKIP, ",", FALSE, FALSE);
+    }
 
-        if( poDriver == NULL )
-            CPLError( CE_Warning, CPLE_AppDefined, 
-                      "Unable to find driver %s to unload from GDAL_SKIP environment variable.", 
-                      papszList[i] );
-        else
+    for( int j = 0; j < 2; j++ )
+    {
+        for( int i = 0; apapszList[j] != NULL &&  apapszList[j][i] != NULL; i++ )
         {
-            CPLDebug( "GDAL", "AutoSkipDriver(%s)", papszList[i] );
-            DeregisterDriver( poDriver );
-            delete poDriver;
+            GDALDriver *poDriver = GetDriverByName( apapszList[j][i] );
+
+            if( poDriver == NULL )
+                CPLError( CE_Warning, CPLE_AppDefined,
+                        "Unable to find driver %s to unload from GDAL_SKIP environment variable.",
+                        apapszList[j][i] );
+            else
+            {
+                CPLDebug( "GDAL", "AutoSkipDriver(%s)", apapszList[j][i] );
+                DeregisterDriver( poDriver );
+                delete poDriver;
+            }
         }
     }
 
-    CSLDestroy( papszList );
+    CSLDestroy( apapszList[0] );
+    CSLDestroy( apapszList[1] );
 }
 
 /************************************************************************/
@@ -657,15 +665,19 @@ void GDALDriverManager::AutoSkipDrivers()
 void GDALDriverManager::AutoLoadDrivers()
 
 {
-    char     **papszSearchPath = NULL;
-    const char *pszGDAL_DRIVER_PATH = 
+#ifdef GDAL_NO_AUTOLOAD
+    CPLDebug( "GDAL", "GDALDriverManager::AutoLoadDrivers() not compiled in." );
+#else
+    const char *pszGDAL_DRIVER_PATH =
         CPLGetConfigOption( "GDAL_DRIVER_PATH", NULL );
+    if( pszGDAL_DRIVER_PATH == NULL )
+        pszGDAL_DRIVER_PATH = CPLGetConfigOption( "OGR_DRIVER_PATH", NULL );
 
 /* -------------------------------------------------------------------- */
 /*      Allow applications to completely disable this search by         */
 /*      setting the driver path to the special string "disable".        */
 /* -------------------------------------------------------------------- */
-    if( pszGDAL_DRIVER_PATH != NULL && EQUAL(pszGDAL_DRIVER_PATH,"disable")) 
+    if( pszGDAL_DRIVER_PATH != NULL && EQUAL(pszGDAL_DRIVER_PATH,"disable"))
     {
         CPLDebug( "GDAL", "GDALDriverManager::AutoLoadDrivers() disabled." );
         return;
@@ -674,20 +686,22 @@ void GDALDriverManager::AutoLoadDrivers()
 /* -------------------------------------------------------------------- */
 /*      Where should we look for stuff?                                 */
 /* -------------------------------------------------------------------- */
+    char **papszSearchPath = NULL;
+
     if( pszGDAL_DRIVER_PATH != NULL )
     {
 #ifdef WIN32
-        papszSearchPath = 
+        papszSearchPath =
             CSLTokenizeStringComplex( pszGDAL_DRIVER_PATH, ";", TRUE, FALSE );
 #else
-        papszSearchPath = 
+        papszSearchPath =
             CSLTokenizeStringComplex( pszGDAL_DRIVER_PATH, ":", TRUE, FALSE );
 #endif
     }
     else
     {
 #ifdef GDAL_PREFIX
-        papszSearchPath = CSLAddString( papszSearchPath, 
+        papszSearchPath = CSLAddString( papszSearchPath,
     #ifdef MACOSX_FRAMEWORK
                                         GDAL_PREFIX "/PlugIns");
     #else
@@ -705,33 +719,20 @@ void GDALDriverManager::AutoLoadDrivers()
         }
         else
         {
-            papszSearchPath = CSLAddString( papszSearchPath, 
+            papszSearchPath = CSLAddString( papszSearchPath,
                                             "/usr/local/lib/gdalplugins" );
         }
 #endif
 
    #ifdef MACOSX_FRAMEWORK
    #define num2str(x) str(x)
-   #define str(x) #x 
-     papszSearchPath = CSLAddString( papszSearchPath, 
+   #define str(x) #x
+     papszSearchPath = CSLAddString( papszSearchPath,
                                      "/Library/Application Support/GDAL/"
-                                     num2str(GDAL_VERSION_MAJOR) "."  
+                                     num2str(GDAL_VERSION_MAJOR) "."
                                      num2str(GDAL_VERSION_MINOR) "/PlugIns" );
    #endif
 
-
-        if( strlen(GetHome()) > 0 )
-        {
-            papszSearchPath = CSLAddString( papszSearchPath, 
-                                  CPLFormFilename( GetHome(),
-    #ifdef MACOSX_FRAMEWORK 
-                                     "/Library/Application Support/GDAL/"
-                                     num2str(GDAL_VERSION_MAJOR) "."  
-                                     num2str(GDAL_VERSION_MINOR) "/PlugIns", NULL ) );
-    #else
-                                                    "lib/gdalplugins", NULL ) );
-    #endif                                           
-        }
     }
 
 /* -------------------------------------------------------------------- */
@@ -746,42 +747,52 @@ void GDALDriverManager::AutoLoadDrivers()
 /* -------------------------------------------------------------------- */
     for( int iDir = 0; iDir < CSLCount(papszSearchPath); iDir++ )
     {
-        char **papszFiles = NULL;
-        VSIStatBufL sStatBuf;
         CPLString osABISpecificDir =
             CPLFormFilename( papszSearchPath[iDir], osABIVersion, NULL );
-        
+
+        VSIStatBufL sStatBuf;
         if( VSIStatL( osABISpecificDir, &sStatBuf ) != 0 )
             osABISpecificDir = papszSearchPath[iDir];
 
-        papszFiles = CPLReadDir( osABISpecificDir );
+        char **papszFiles = VSIReadDir( osABISpecificDir );
+        const int nFileCount = CSLCount(papszFiles);
 
-        for( int iFile = 0; iFile < CSLCount(papszFiles); iFile++ )
+        for( int iFile = 0; iFile < nFileCount; iFile++ )
         {
-            char   *pszFuncName;
-            const char *pszFilename;
             const char *pszExtension = CPLGetExtension( papszFiles[iFile] );
-            void   *pRegister;
 
-            if( !EQUALN(papszFiles[iFile],"gdal_",5) )
-                continue;
-
-            if( !EQUAL(pszExtension,"dll") 
-                && !EQUAL(pszExtension,"so") 
+            if( !EQUAL(pszExtension,"dll")
+                && !EQUAL(pszExtension,"so")
                 && !EQUAL(pszExtension,"dylib") )
                 continue;
 
-            pszFuncName = (char *) CPLCalloc(strlen(papszFiles[iFile])+20,1);
-            sprintf( pszFuncName, "GDALRegister_%s", 
-                     CPLGetBasename(papszFiles[iFile]) + 5 );
-            
-            pszFilename = 
-                CPLFormFilename( osABISpecificDir, 
-                                 papszFiles[iFile], NULL );
+            char *pszFuncName;
+            if( STARTS_WITH_CI(papszFiles[iFile], "gdal_") )
+            {
+                pszFuncName = (char *) CPLCalloc(strlen(papszFiles[iFile])+20,1);
+                snprintf( pszFuncName,
+                          strlen(papszFiles[iFile])+20,
+                          "GDALRegister_%s",
+                        CPLGetBasename(papszFiles[iFile]) + strlen("gdal_") );
+            }
+            else if ( STARTS_WITH_CI(papszFiles[iFile], "ogr_") )
+            {
+                pszFuncName = (char *) CPLCalloc(strlen(papszFiles[iFile])+20,1);
+                snprintf( pszFuncName,
+                         strlen(papszFiles[iFile])+20,
+                         "RegisterOGR%s",
+                         CPLGetBasename(papszFiles[iFile]) + strlen("ogr_") );
+            }
+            else
+                continue;
+
+            const char *pszFilename
+                = CPLFormFilename( osABISpecificDir,
+                                   papszFiles[iFile], NULL );
 
             CPLErrorReset();
             CPLPushErrorHandler(CPLQuietErrorHandler);
-            pRegister = CPLGetSymbol( pszFilename, pszFuncName );
+            void *pRegister = CPLGetSymbol( pszFilename, pszFuncName );
             CPLPopErrorHandler();
             if( pRegister == NULL )
             {
@@ -794,10 +805,10 @@ void GDALDriverManager::AutoLoadDrivers()
                               "%s", osLastErrorMsg.c_str() );
                 }
             }
-            
+
             if( pRegister != NULL )
             {
-                CPLDebug( "GDAL", "Auto register %s using %s.", 
+                CPLDebug( "GDAL", "Auto register %s using %s.",
                           pszFilename, pszFuncName );
 
                 ((void (*)()) pRegister)();
@@ -810,6 +821,9 @@ void GDALDriverManager::AutoLoadDrivers()
     }
 
     CSLDestroy( papszSearchPath );
+
+#endif  // GDAL_NO_AUTOLOAD
+
 }
 
 /************************************************************************/
@@ -819,20 +833,18 @@ void GDALDriverManager::AutoLoadDrivers()
 /**
  * \brief Destroy the driver manager.
  *
- * Incidently unloads all managed drivers.
+ * Incidentally unloads all managed drivers.
  *
  * NOTE: This function is not thread safe.  It should not be called while
- * other threads are actively using GDAL. 
+ * other threads are actively using GDAL.
  */
 
 void CPL_STDCALL GDALDestroyDriverManager( void )
 
 {
-    // THREADSAFETY: We would like to lock the mutex here, but it 
+    // THREADSAFETY: We would like to lock the mutex here, but it
     // needs to be reacquired within the destructor during driver
     // deregistration.
     if( poDM != NULL )
         delete poDM;
 }
-
-
