@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_geos.h 17469 2009-07-28 18:28:27Z warmerdam $
+ * $Id: ogr_geos.h 33631 2016-03-04 06:28:09Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Definitions related to support for use of GEOS in OGR.
@@ -32,11 +32,15 @@
 #ifndef OGR_GEOS_H_INCLUDED
 #define OGR_GEOS_H_INCLUDED
 
-#ifdef HAVE_GEOS 
+#ifdef HAVE_GEOS
+// To avoid accidental use of non reentrant GEOS API.
+// (check only effective in GEOS >= 3.5)
+#  define GEOS_USE_ONLY_R_API
+
 #  include <geos_c.h>
 #else
 
-namespace geos { 
+namespace geos {
     class Geometry;
 };
 
