@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_srs_ozi.cpp 25256 2012-11-26 20:19:03Z rouault $
+ * $Id: ogr_srs_ozi.cpp 33631 2016-03-04 06:28:09Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  OGRSpatialReference translation from OziExplorer
@@ -8,6 +8,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2009, Andrey Kiselev <dron@ak4719.spb.edu>
+ * Copyright (c) 2009-2012, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,74 +33,37 @@
 #include "cpl_conv.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: ogr_srs_ozi.cpp 25256 2012-11-26 20:19:03Z rouault $");
+CPL_CVSID("$Id: ogr_srs_ozi.cpp 33631 2016-03-04 06:28:09Z goatbar $");
 
 /************************************************************************/
 /*                          OSRImportFromOzi()                          */
 /************************************************************************/
 
-OGRErr OSRImportFromOzi( OGRSpatialReferenceH hSRS,
-                         const char *pszDatum, const char *pszProj,
-                         const char *pszProjParms )
-
-{
-    VALIDATE_POINTER1( hSRS, "OSRImportFromOzi", CE_Failure );
-
-    return ((OGRSpatialReference *) hSRS)->importFromOzi( pszDatum, pszProj,
-                                                          pszProjParms );
-}
-
-/************************************************************************/
-/*                            importFromOzi()                           */
-/************************************************************************/
-
-/**
- * Note : This method is obsolete, but has been kept to avoid breaking the API.
- *        It can be removed in GDAL 2.0
- */
-
 /**
  * Import coordinate system from OziExplorer projection definition.
  *
- * This method will import projection definition in style, used by
+ * This function will import projection definition in style, used by
  * OziExplorer software.
  *
- * This function is the equivalent of the C function OSRImportFromOzi().
+ * Note: another version of this function with a different signature existed
+ * in GDAL 1.X.
  *
- * @param pszDatum Datum string. This is a fifth string in the
- * OziExplorer .MAP file.
+ * @param hSRS spatial reference object.
+ * @param papszLines Map file lines. This is an array of strings containing
+ * the whole OziExplorer .MAP file. The array is terminated by a NULL pointer.
  *
- * @param pszProj Projection string. Search for line starting with
- * "Map Projection" name in the OziExplorer .MAP file and supply it as a
- * whole in this parameter.
+ * @return OGRERR_NONE on success or an error code in case of failure.
  *
- * @param pszProjParms String containing projection parameters. Search for
- * "Projection Setup" name in the OziExplorer .MAP file and supply it as a
- * whole in this parameter.
- * 
- * @return OGRERR_NONE on success or an error code in case of failure. 
- *
- * @deprecated Use importFromOzi( const char * const* papszLines ) instead
+ * @since OGR 2.0
  */
 
-OGRErr OGRSpatialReference::importFromOzi( const char *pszDatum,
-                                           const char *pszProj,
-                                           const char *pszProjParms )
+OGRErr OSRImportFromOzi( OGRSpatialReferenceH hSRS,
+                         const char * const* papszLines )
 
 {
-    const char* papszLines[8];
+    VALIDATE_POINTER1( hSRS, "OSRImportFromOzi", OGRERR_FAILURE );
 
-    // Fake
-    papszLines[0] = "";
-    papszLines[1] = "";
-    papszLines[2] = "";
-    papszLines[3] = "";
-    papszLines[4] = pszDatum; /* Must be in that position */
-    papszLines[5] = pszProj; /* Must be after 5th line */
-    papszLines[6] = pszProjParms; /* Must be after 5th line */
-    papszLines[7] = NULL;
-
-    return importFromOzi(papszLines);
+    return ((OGRSpatialReference *) hSRS)->importFromOzi( papszLines );
 }
 
 /************************************************************************/
@@ -114,8 +78,8 @@ OGRErr OGRSpatialReference::importFromOzi( const char *pszDatum,
  *
  * @param papszLines Map file lines. This is an array of strings containing
  * the whole OziExplorer .MAP file. The array is terminated by a NULL pointer.
- * 
- * @return OGRERR_NONE on success or an error code in case of failure. 
+ *
+ * @return OGRERR_NONE on success or an error code in case of failure.
  *
  * @since OGR 1.10
  */
@@ -135,11 +99,11 @@ OGRErr OGRSpatialReference::importFromOzi( const char * const* papszLines )
 
     for ( iLine = 5; iLine < nLines; iLine++ )
     {
-        if ( EQUALN(papszLines[iLine], "Map Projection", 14) )
+        if ( STARTS_WITH_CI(papszLines[iLine], "Map Projection") )
         {
             pszProj = papszLines[iLine];
         }
-        else if ( EQUALN(papszLines[iLine], "Projection Setup", 16) )
+        else if ( STARTS_WITH_CI(papszLines[iLine], "Projection Setup") )
         {
             pszProjParms = papszLines[iLine];
         }
@@ -152,20 +116,20 @@ OGRErr OGRSpatialReference::importFromOzi( const char * const* papszLines )
 /*      Operate on the basis of the projection name.                    */
 /* -------------------------------------------------------------------- */
     char    **papszProj = CSLTokenizeStringComplex( pszProj, ",", TRUE, TRUE );
-    char    **papszProjParms = CSLTokenizeStringComplex( pszProjParms, ",", 
+    char    **papszProjParms = CSLTokenizeStringComplex( pszProjParms, ",",
                                                          TRUE, TRUE );
     char    **papszDatum = NULL;
-                                                         
+
     if (CSLCount(papszProj) < 2)
     {
         goto not_enough_data;
     }
 
-    if ( EQUALN(papszProj[1], "Latitude/Longitude", 18) )
+    if ( STARTS_WITH_CI(papszProj[1], "Latitude/Longitude") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "Mercator", 8) )
+    else if ( STARTS_WITH_CI(papszProj[1], "Mercator") )
     {
         if (CSLCount(papszProjParms) < 6) goto not_enough_data;
         double dfScale = CPLAtof(papszProjParms[3]);
@@ -175,7 +139,7 @@ OGRErr OGRSpatialReference::importFromOzi( const char * const* papszLines )
                      CPLAtof(papszProjParms[4]), CPLAtof(papszProjParms[5]) );
     }
 
-    else if ( EQUALN(papszProj[1], "Transverse Mercator", 19) )
+    else if ( STARTS_WITH_CI(papszProj[1], "Transverse Mercator") )
     {
         if (CSLCount(papszProjParms) < 6) goto not_enough_data;
         SetTM( CPLAtof(papszProjParms[1]), CPLAtof(papszProjParms[2]),
@@ -183,7 +147,7 @@ OGRErr OGRSpatialReference::importFromOzi( const char * const* papszLines )
                CPLAtof(papszProjParms[4]), CPLAtof(papszProjParms[5]) );
     }
 
-    else if ( EQUALN(papszProj[1], "Lambert Conformal Conic", 23) )
+    else if ( STARTS_WITH_CI(papszProj[1], "Lambert Conformal Conic") )
     {
         if (CSLCount(papszProjParms) < 8) goto not_enough_data;
         SetLCC( CPLAtof(papszProjParms[6]), CPLAtof(papszProjParms[7]),
@@ -191,14 +155,14 @@ OGRErr OGRSpatialReference::importFromOzi( const char * const* papszLines )
                 CPLAtof(papszProjParms[4]), CPLAtof(papszProjParms[5]) );
     }
 
-    else if ( EQUALN(papszProj[1], "Sinusoidal", 10) )
+    else if ( STARTS_WITH_CI(papszProj[1], "Sinusoidal") )
     {
         if (CSLCount(papszProjParms) < 6) goto not_enough_data;
         SetSinusoidal( CPLAtof(papszProjParms[2]),
                        CPLAtof(papszProjParms[4]), CPLAtof(papszProjParms[5]) );
     }
 
-    else if ( EQUALN(papszProj[1], "Albers Equal Area", 17) )
+    else if ( STARTS_WITH_CI(papszProj[1], "Albers Equal Area") )
     {
         if (CSLCount(papszProjParms) < 8) goto not_enough_data;
         SetACEA( CPLAtof(papszProjParms[6]), CPLAtof(papszProjParms[7]),
@@ -206,12 +170,12 @@ OGRErr OGRSpatialReference::importFromOzi( const char * const* papszLines )
                  CPLAtof(papszProjParms[4]), CPLAtof(papszProjParms[5]) );
     }
 
-    else if ( EQUALN(papszProj[1], "(UTM) Universal Transverse Mercator", 35) && nLines > 5 )
+    else if ( STARTS_WITH_CI(papszProj[1], "(UTM) Universal Transverse Mercator") && nLines > 5 )
     {
         /* Look for the UTM zone in the calibration point data */
         for ( iLine = 5; iLine < nLines; iLine++ )
         {
-            if ( EQUALN(papszLines[iLine], "Point", 5) )
+            if ( STARTS_WITH_CI(papszLines[iLine], "Point") )
             {
                 char    **papszTok = NULL;
                 papszTok = CSLTokenizeString2( papszLines[iLine], ",",
@@ -228,21 +192,21 @@ OGRErr OGRSpatialReference::importFromOzi( const char * const* papszLines )
                     CSLDestroy(papszTok);
                     continue;
                 }
-                SetUTM( CPLAtofM(papszTok[13]), EQUAL(papszTok[16], "N") );
+                SetUTM( atoi(papszTok[13]), EQUAL(papszTok[16], "N") );
                 CSLDestroy(papszTok);
                 break;
             }
         }
         if ( iLine == nLines )    /* Try to guess the UTM zone */
         {
-            float fMinLongitude = INT_MAX;
-            float fMaxLongitude = INT_MIN;
-            float fMinLatitude = INT_MAX;
-            float fMaxLatitude = INT_MIN;
-            int bFoundMMPLL = FALSE;
+            float fMinLongitude = 1000.0f;
+            float fMaxLongitude = -1000.0f;;
+            float fMinLatitude = 1000.0f;
+            float fMaxLatitude = -1000.0f;
+            bool bFoundMMPLL = false;
             for ( iLine = 5; iLine < nLines; iLine++ )
             {
-                if ( EQUALN(papszLines[iLine], "MMPLL", 5) )
+                if ( STARTS_WITH_CI(papszLines[iLine], "MMPLL") )
                 {
                     char    **papszTok = NULL;
                     papszTok = CSLTokenizeString2( papszLines[iLine], ",",
@@ -254,11 +218,11 @@ OGRErr OGRSpatialReference::importFromOzi( const char * const* papszLines )
                         CSLDestroy(papszTok);
                         continue;
                     }
-                    float fLongitude = CPLAtofM(papszTok[2]);
-                    float fLatitude = CPLAtofM(papszTok[3]);
+                    float fLongitude = static_cast<float>(CPLAtofM(papszTok[2]));
+                    float fLatitude = static_cast<float>(CPLAtofM(papszTok[3]));
                     CSLDestroy(papszTok);
 
-                    bFoundMMPLL = TRUE;
+                    bFoundMMPLL = true;
 
                     if ( fMinLongitude > fLongitude )
                         fMinLongitude = fLongitude;
@@ -275,10 +239,10 @@ OGRErr OGRSpatialReference::importFromOzi( const char * const* papszLines )
             if ( bFoundMMPLL && fMaxLatitude <= 90 )
             {
                 int nUtmZone;
-                if ( fMedianLatitude >= 56 && fMedianLatitude <= 64 && 
+                if ( fMedianLatitude >= 56 && fMedianLatitude <= 64 &&
                      fMedianLongitude >= 3 && fMedianLongitude <= 12 )
                     nUtmZone = 32;                                             /* Norway exception */
-                else if ( fMedianLatitude >= 72 && fMedianLatitude <= 84 && 
+                else if ( fMedianLatitude >= 72 && fMedianLatitude <= 84 &&
                          fMedianLongitude >= 0 && fMedianLongitude <= 42 )
                     nUtmZone = (int) ((fMedianLongitude + 3 ) / 12) * 2 + 31;  /* Svalbard exception */
                 else
@@ -290,22 +254,22 @@ OGRErr OGRSpatialReference::importFromOzi( const char * const* papszLines )
         }
     }
 
-    else if ( EQUALN(papszProj[1], "(I) France Zone I", 17) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(I) France Zone I") )
     {
         SetLCC1SP( 49.5, 2.337229167, 0.99987734, 600000, 1200000 );
     }
 
-    else if ( EQUALN(papszProj[1], "(II) France Zone II", 19) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(II) France Zone II") )
     {
         SetLCC1SP( 46.8, 2.337229167, 0.99987742, 600000, 2200000 );
     }
 
-    else if ( EQUALN(papszProj[1], "(III) France Zone III", 21) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(III) France Zone III") )
     {
         SetLCC1SP( 44.1, 2.337229167, 0.99987750, 600000, 3200000 );
     }
 
-    else if ( EQUALN(papszProj[1], "(IV) France Zone IV", 19) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(IV) France Zone IV") )
     {
         SetLCC1SP( 42.165, 2.337229167, 0.99994471, 234.358, 4185861.369 );
     }
@@ -316,83 +280,83 @@ OGRErr OGRSpatialReference::importFromOzi( const char * const* papszLines )
  */
 
 /*
-    else if ( EQUALN(papszProj[1], "(BNG) British National Grid", 27) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(BNG) British National Grid") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "(IG) Irish Grid", 15) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(IG) Irish Grid") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "(NZG) New Zealand Grid", 22) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(NZG) New Zealand Grid") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "(NZTM2) New Zealand TM 2000", 27) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(NZTM2) New Zealand TM 2000") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "(SG) Swedish Grid", 27) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(SG) Swedish Grid") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "(SUI) Swiss Grid", 26) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(SUI) Swiss Grid") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "(A)Lambert Azimuthual Equal Area", 32) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(A)Lambert Azimuthual Equal Area") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "(EQC) Equidistant Conic", 23) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(EQC) Equidistant Conic") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "Polyconic (American)", 20) )
+    else if ( STARTS_WITH_CI(papszProj[1], "Polyconic (American)") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "Van Der Grinten", 15) )
+    else if ( STARTS_WITH_CI(papszProj[1], "Van Der Grinten") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "Vertical Near-Sided Perspective", 31) )
+    else if ( STARTS_WITH_CI(papszProj[1], "Vertical Near-Sided Perspective") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "(WIV) Wagner IV", 15) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(WIV) Wagner IV") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "Bonne", 5) )
+    else if ( STARTS_WITH_CI(papszProj[1], "Bonne") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "(MT0) Montana State Plane Zone 2500", 35) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(MT0) Montana State Plane Zone 2500") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "ITA1) Italy Grid Zone 1", 23) )
+    else if ( STARTS_WITH_CI(papszProj[1], "ITA1) Italy Grid Zone 1") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "ITA2) Italy Grid Zone 2", 23) )
+    else if ( STARTS_WITH_CI(papszProj[1], "ITA2) Italy Grid Zone 2") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "(VICMAP-TM) Victoria Aust.(pseudo AMG)", 38) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(VICMAP-TM) Victoria Aust.(pseudo AMG)") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "VICGRID) Victoria Australia", 27) )
+    else if ( STARTS_WITH_CI(papszProj[1], "VICGRID) Victoria Australia") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "(VG94) VICGRID94 Victoria Australia", 35) )
+    else if ( STARTS_WITH_CI(papszProj[1], "(VG94) VICGRID94 Victoria Australia") )
     {
     }
 
-    else if ( EQUALN(papszProj[1], "Gnomonic", 8) )
+    else if ( STARTS_WITH_CI(papszProj[1], "Gnomonic") )
     {
     }
 */
@@ -413,7 +377,7 @@ OGRErr OGRSpatialReference::importFromOzi( const char * const* papszLines )
                                                | CSLT_STRIPENDSPACES );
     if ( papszDatum == NULL)
         goto not_enough_data;
-        
+
     if ( !IsLocal() )
     {
 
@@ -542,4 +506,3 @@ other_error:
 
     return OGRERR_FAILURE;
 }
-
