@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_srs_usgs.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogr_srs_usgs.cpp 33631 2016-03-04 06:28:09Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  OGRSpatialReference translation to/from USGS georeferencing
@@ -34,7 +34,7 @@
 #include "cpl_conv.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: ogr_srs_usgs.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: ogr_srs_usgs.cpp 33631 2016-03-04 06:28:09Z goatbar $");
 
 /************************************************************************/
 /*  GCTP projection codes.                                              */
@@ -60,11 +60,11 @@ CPL_CVSID("$Id: ogr_srs_usgs.cpp 27044 2014-03-16 23:41:27Z rouault $");
 #define EQRECT  17L     // Equirectangular
 #define MILLER  18L     // Miller Cylindrical
 #define VGRINT  19L     // Van der Grinten
-#define HOM     20L     // (Hotine) Oblique Mercator 
+#define HOM     20L     // (Hotine) Oblique Mercator
 #define ROBIN   21L     // Robinson
 #define SOM     22L     // Space Oblique Mercator (SOM)
 #define ALASKA  23L     // Alaska Conformal
-#define GOODE   24L     // Interrupted Goode Homolosine 
+#define GOODE   24L     // Interrupted Goode Homolosine
 #define MOLL    25L     // Mollweide
 #define IMOLL   26L     // Interrupted Mollweide
 #define HAMMER  27L     // Hammer
@@ -73,11 +73,11 @@ CPL_CVSID("$Id: ogr_srs_usgs.cpp 27044 2014-03-16 23:41:27Z rouault $");
 #define OBEQA   30L     // Oblated Equal Area
 #define ISINUS1 31L     // Integerized Sinusoidal Grid (the same as 99)
 #define CEA     97L     // Cylindrical Equal Area (Grid corners set
-                        // in meters for EASE grid) 
+                        // in meters for EASE grid)
 #define BCEA    98L     // Cylindrical Equal Area (Grid corners set
-                        // in DMS degs for EASE grid) 
+                        // in DMS degs for EASE grid)
 #define ISINUS  99L     // Integerized Sinusoidal Grid
-                        // (added by Raj Gejjagaraguppe ARC for MODIS) 
+                        // (added by Raj Gejjagaraguppe ARC for MODIS)
 
 /************************************************************************/
 /*  GCTP ellipsoid codes.                                               */
@@ -108,7 +108,7 @@ CPL_CVSID("$Id: ogr_srs_usgs.cpp 27044 2014-03-16 23:41:27Z rouault $");
 /*  Correspondence between GCTP and EPSG ellipsoid codes.               */
 /************************************************************************/
 
-static const long aoEllips[] =
+static const int aoEllips[] =
 {
     7008,   // Clarke, 1866 (NAD1927)
     7034,   // Clarke, 1880
@@ -158,7 +158,7 @@ OGRErr OSRImportFromUSGS( OGRSpatialReferenceH hSRS, long iProjsys,
                           long iZone, double *padfPrjParams, long iDatum )
 
 {
-    VALIDATE_POINTER1( hSRS, "OSRImportFromUSGS", CE_Failure );
+    VALIDATE_POINTER1( hSRS, "OSRImportFromUSGS", OGRERR_FAILURE );
 
     return ((OGRSpatialReference *) hSRS)->importFromUSGS( iProjsys, iZone,
                                                            padfPrjParams,
@@ -204,13 +204,13 @@ static double OGRSpatialReferenceUSGSUnpackRadian(double dfVal)
  *        <h4>Projection Transformation Package Projection Parameters</h4>
  * <pre>
  * ----------------------------------------------------------------------------
- *                         |                    Array Element                  
+ *                         |                    Array Element
  *  Code & Projection Id   |---------------------------------------------------
  *                         |   0  |   1  |  2   |  3   |   4   |    5    |6 | 7
  * ----------------------------------------------------------------------------
- *  0 Geographic           |      |      |      |      |       |         |  |  
- *  1 U T M                |Lon/Z |Lat/Z |      |      |       |         |  |  
- *  2 State Plane          |      |      |      |      |       |         |  |  
+ *  0 Geographic           |      |      |      |      |       |         |  |
+ *  1 U T M                |Lon/Z |Lat/Z |      |      |       |         |  |
+ *  2 State Plane          |      |      |      |      |       |         |  |
  *  3 Albers Equal Area    |SMajor|SMinor|STDPR1|STDPR2|CentMer|OriginLat|FE|FN
  *  4 Lambert Conformal C  |SMajor|SMinor|STDPR1|STDPR2|CentMer|OriginLat|FE|FN
  *  5 Mercator             |SMajor|SMinor|      |      |CentMer|TrueScale|FE|FN
@@ -235,19 +235,19 @@ static double OGRSpatialReferenceUSGSUnpackRadian(double dfVal)
  * 22 Space Oblique Merc A |SMajor|SMinor|      |IncAng|AscLong|         |FE|FN
  *    Space Oblique Merc B |SMajor|SMinor|Satnum|Path  |       |         |FE|FN
  * 23 Alaska Conformal     |SMajor|SMinor|      |      |       |         |FE|FN
- * 24 Interrupted Goode    |Sphere|      |      |      |       |         |  |  
+ * 24 Interrupted Goode    |Sphere|      |      |      |       |         |  |
  * 25 Mollweide            |Sphere|      |      |      |CentMer|         |FE|FN
- * 26 Interrupt Mollweide  |Sphere|      |      |      |       |         |  |  
+ * 26 Interrupt Mollweide  |Sphere|      |      |      |       |         |  |
  * 27 Hammer               |Sphere|      |      |      |CentMer|         |FE|FN
  * 28 Wagner IV            |Sphere|      |      |      |CentMer|         |FE|FN
  * 29 Wagner VII           |Sphere|      |      |      |CentMer|         |FE|FN
  * 30 Oblated Equal Area   |Sphere|      |Shapem|Shapen|CentLon|CenterLat|FE|FN
  * ----------------------------------------------------------------------------
- * 
+ *
  *       ----------------------------------------------------
  *                               |      Array Element       |
  *         Code & Projection Id  |---------------------------
- *                               |  8  |  9 |  10 | 11 | 12 |  
+ *                               |  8  |  9 |  10 | 11 | 12 |
  *       ----------------------------------------------------
  *        0 Geographic           |     |    |     |    |    |
  *        1 U T M                |     |    |     |    |    |
@@ -257,12 +257,12 @@ static double OGRSpatialReferenceUSGSUnpackRadian(double dfVal)
  *        5 Mercator             |     |    |     |    |    |
  *        6 Polar Stereographic  |     |    |     |    |    |
  *        7 Polyconic            |     |    |     |    |    |
- *        8 Equid. Conic A       |zero |    |     |    |    |   
+ *        8 Equid. Conic A       |zero |    |     |    |    |
  *          Equid. Conic B       |one  |    |     |    |    |
  *        9 Transverse Mercator  |     |    |     |    |    |
  *       10 Stereographic        |     |    |     |    |    |
- *       11 Lambert Azimuthal    |     |    |     |    |    |    
- *       12 Azimuthal            |     |    |     |    |    |    
+ *       11 Lambert Azimuthal    |     |    |     |    |    |
+ *       12 Azimuthal            |     |    |     |    |    |
  *       13 Gnomonic             |     |    |     |    |    |
  *       14 Orthographic         |     |    |     |    |    |
  *       15 Gen. Vert. Near Per  |     |    |     |    |    |
@@ -270,10 +270,10 @@ static double OGRSpatialReferenceUSGSUnpackRadian(double dfVal)
  *       17 Equirectangular      |     |    |     |    |    |
  *       18 Miller Cylindrical   |     |    |     |    |    |
  *       19 Van der Grinten      |     |    |     |    |    |
- *       20 Hotin Oblique Merc A |Long1|Lat1|Long2|Lat2|zero|   
+ *       20 Hotin Oblique Merc A |Long1|Lat1|Long2|Lat2|zero|
  *          Hotin Oblique Merc B |     |    |     |    |one |
  *       21 Robinson             |     |    |     |    |    |
- *       22 Space Oblique Merc A |PSRev|LRat|PFlag|    |zero|    
+ *       22 Space Oblique Merc A |PSRev|LRat|PFlag|    |zero|
  *          Space Oblique Merc B |     |    |     |    |one |
  *       23 Alaska Conformal     |     |    |     |    |    |
  *       24 Interrupted Goode    |     |    |     |    |    |
@@ -405,12 +405,12 @@ static double OGRSpatialReferenceUSGSUnpackRadian(double dfVal)
  *
  * @param nUSGSAngleFormat one of USGS_ANGLE_DECIMALDEGREES, USGS_ANGLE_PACKEDDMS, or USGS_ANGLE_RADIANS (default is USGS_ANGLE_PACKEDDMS).
  *
- * @return OGRERR_NONE on success or an error code in case of failure. 
+ * @return OGRERR_NONE on success or an error code in case of failure.
  */
 
 OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
                                             double *padfPrjParams,
-                                            long iDatum, 
+                                            long iDatum,
                                             int nUSGSAngleFormat  )
 
 {
@@ -456,7 +456,7 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
                     iZone = -iZone;
                     bNorth = FALSE;
                 }
-                SetUTM( iZone, bNorth );
+                SetUTM( (int)iZone, bNorth );
             }
             break;
 
@@ -470,8 +470,8 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
                     CPLError( CE_Warning, CPLE_AppDefined,
                               "Wrong datum for State Plane projection %d. "
                               "Should be 0 or 8.", (int) iDatum );
-                
-                SetStatePlane( iZone, bNAD83 );
+
+                SetStatePlane( (int)iZone, bNAD83 );
             }
             break;
 
@@ -646,7 +646,7 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
         // FIXME: OBEQA --- Oblated Equal Area skipped
 
         // FIXME: ISINUS1 --- Integerized Sinusoidal Grid (the same as 99) skipped
-        
+
         // FIXME: CEA --- Cylindrical Equal Area skipped (Grid corners set in meters for EASE grid)
 
         // FIXME: BCEA --- Cylindrical Equal Area skipped (Grid corners set in DMS degs for EASE grid)
@@ -657,7 +657,7 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
             CPLDebug( "OSR_USGS", "Unsupported projection: %ld", iProjSys );
             SetLocalCS( CPLString().Printf("GCTP projection number %ld", iProjSys) );
             break;
-            
+
     }
 
 /* -------------------------------------------------------------------- */
@@ -675,13 +675,7 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
             {
                 if ( padfPrjParams[1] > 1.0 )
                 {
-                    if( ABS(padfPrjParams[0] - padfPrjParams[1]) < 0.01 )
-                        dfInvFlattening = 0.0;
-                    else
-                    {
-                        dfInvFlattening = padfPrjParams[0]
-                            / ( padfPrjParams[0] - padfPrjParams[1] );
-                    }
+                    dfInvFlattening = OSRCalcInvFlattening(padfPrjParams[0], padfPrjParams[1] );
                 }
                 else if ( padfPrjParams[1] > 0.0 )
                 {
@@ -704,7 +698,7 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
                     SetGeogCS( CPLString().Printf(
                                     "Unknown datum based upon the %s ellipsoid",
                                     pszName ),
-                               CPLString().Printf( 
+                               CPLString().Printf(
                                     "Not specified (based on %s spheroid)",
                                     pszName ),
                                pszName, dfSemiMajor, dfInvFlattening,
@@ -747,7 +741,7 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
             {
                 CPLError( CE_Warning, CPLE_AppDefined,
                           "Failed to lookup datum code %d, likely due to missing GDAL gcs.csv\n"
-                          " file.  Falling back to use WGS84.", 
+                          " file.  Falling back to use WGS84.",
                           (int) iDatum );
                 SetWellKnownGeogCS("WGS84" );
             }
@@ -779,7 +773,7 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
 /************************************************************************/
 /*                          OSRExportToUSGS()                           */
 /************************************************************************/
-/** 
+/**
  * \brief Export coordinate system in USGS GCTP projection definition.
  *
  * This function is the same as OGRSpatialReference::exportToUSGS().
@@ -790,7 +784,7 @@ OGRErr OSRExportToUSGS( OGRSpatialReferenceH hSRS,
                         double **ppadfPrjParams, long *piDatum )
 
 {
-    VALIDATE_POINTER1( hSRS, "OSRExportToUSGS", CE_Failure );
+    VALIDATE_POINTER1( hSRS, "OSRExportToUSGS", OGRERR_FAILURE );
 
     *ppadfPrjParams = NULL;
 
@@ -813,15 +807,15 @@ OGRErr OSRExportToUSGS( OGRSpatialReferenceH hSRS,
  *
  * @param piZone Pointer to variable, where the zone for UTM and State Plane
  * projection systems will be returned.
- * 
+ *
  * @param ppadfPrjParams Pointer to which dynamically allocated array of
  * 15 projection parameters will be assigned. See importFromUSGS() for
  * the list of parameters. Caller responsible to free this array.
  *
  * @param piDatum Pointer to variable, where the datum code will
  * be returned.
- * 
- * @return OGRERR_NONE on success or an error code on failure. 
+ *
+ * @return OGRERR_NONE on success or an error code on failure.
  */
 
 OGRErr OGRSpatialReference::exportToUSGS( long *piProjSys, long *piZone,
@@ -947,7 +941,7 @@ OGRErr OGRSpatialReference::exportToUSGS( long *piProjSys, long *piZone,
             *piProjSys = UTM;
             if( !bNorth )
                 *piZone = - *piZone;
-        }            
+        }
         else
         {
             *piProjSys = TM;
@@ -1133,7 +1127,7 @@ OGRErr OGRSpatialReference::exportToUSGS( long *piProjSys, long *piZone,
                   "Geographic system will be used.", pszProjection );
         *piProjSys = GEO;
     }
- 
+
 /* -------------------------------------------------------------------- */
 /*      Translate the datum.                                            */
 /* -------------------------------------------------------------------- */
@@ -1161,7 +1155,7 @@ OGRErr OGRSpatialReference::exportToUSGS( long *piProjSys, long *piZone,
                       "Datum \"%s\" unsupported by USGS GCTP. "
                       "Try to translate ellipsoid definition.", pszDatum );
 #endif
-            
+
             for ( i = 0; i < NUMBER_OF_ELLIPSOIDS; i++ )
             {
                 double  dfSM;
@@ -1204,4 +1198,3 @@ OGRErr OGRSpatialReference::exportToUSGS( long *piProjSys, long *piZone,
 
     return OGRERR_NONE;
 }
-
