@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_srs_validate.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogr_srs_validate.cpp 33631 2016-03-04 06:28:09Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implementation of the OGRSpatialReference::Validate() method and
@@ -33,12 +33,12 @@
 #include "ogr_p.h"
 #include "osr_cs_wkt.h"
 
-CPL_CVSID("$Id: ogr_srs_validate.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: ogr_srs_validate.cpp 33631 2016-03-04 06:28:09Z goatbar $");
 
-/* why would fipszone and zone be paramers when they relate to a composite
+/* why would fipszone and zone be parameters when they relate to a composite
    projection which renders done into a non-zoned projection? */
 
-static const char *papszParameters[] =
+static const char * const papszParameters[] =
 {
     SRS_PP_CENTRAL_MERIDIAN,
     SRS_PP_SCALE_FACTOR,
@@ -69,6 +69,10 @@ static const char *papszParameters[] =
     SRS_PP_LONGITUDE_OF_1ST_POINT,
     SRS_PP_LATITUDE_OF_2ND_POINT,
     SRS_PP_LONGITUDE_OF_2ND_POINT,
+    SRS_PP_PEG_POINT_LATITUDE, /* for SCH */
+    SRS_PP_PEG_POINT_LONGITUDE, /* for SCH */
+    SRS_PP_PEG_POINT_HEADING, /* for SCH */
+    SRS_PP_PEG_POINT_HEIGHT, /* for SCH */
     NULL
 };
 
@@ -77,7 +81,7 @@ static const char *papszParameters[] =
 // underscores instead of spaces.  Should we use the EPSG names were available?
 // Plate-Caree has an accent in the spec!
 
-static const char *papszProjectionSupported[] =
+static const char * const papszProjectionSupported[] =
 {
     SRS_PT_CASSINI_SOLDNER,
     SRS_PT_BONNE,
@@ -127,15 +131,17 @@ static const char *papszProjectionSupported[] =
     SRS_PT_WAGNER_V,
     SRS_PT_WAGNER_VI,
     SRS_PT_WAGNER_VII,
+    SRS_PT_QSC,
+    SRS_PT_SCH,
     SRS_PT_GAUSSSCHREIBERTMERCATOR,
     SRS_PT_KROVAK,
     SRS_PT_CYLINDRICAL_EQUAL_AREA,
-    SRS_PT_GOODE_HOMOLOSINE, 
+    SRS_PT_GOODE_HOMOLOSINE,
     SRS_PT_IGH,
     NULL
 };
 
-static const char *papszProjectionUnsupported[] =
+static const char * const papszProjectionUnsupported[] =
 {
     SRS_PT_NEW_ZEALAND_MAP_GRID,
     SRS_PT_TUNISIA_MINING_GRID,
@@ -145,7 +151,7 @@ static const char *papszProjectionUnsupported[] =
 /*
 ** List of supported projections with the PARAMETERS[] acceptable for each.
 */
-static const char *papszProjWithParms[] = {
+static const char * const papszProjWithParms[] = {
 
     SRS_PT_TRANSVERSE_MERCATOR,
     SRS_PP_LATITUDE_OF_ORIGIN,
@@ -465,47 +471,59 @@ static const char *papszProjWithParms[] = {
     SRS_PT_IMW_POLYCONIC,
     SRS_PP_LATITUDE_OF_1ST_POINT,
     SRS_PP_LATITUDE_OF_2ND_POINT,
-    SRS_PP_CENTRAL_MERIDIAN, 
-    SRS_PP_FALSE_EASTING, 
+    SRS_PP_CENTRAL_MERIDIAN,
+    SRS_PP_FALSE_EASTING,
     SRS_PP_FALSE_NORTHING,
     NULL,
 
     SRS_PT_WAGNER_I,
-    SRS_PP_FALSE_EASTING, 
+    SRS_PP_FALSE_EASTING,
     SRS_PP_FALSE_NORTHING,
     NULL,
 
     SRS_PT_WAGNER_II,
-    SRS_PP_FALSE_EASTING, 
+    SRS_PP_FALSE_EASTING,
     SRS_PP_FALSE_NORTHING,
     NULL,
 
     SRS_PT_WAGNER_III,
     SRS_PP_LATITUDE_OF_ORIGIN,
-    SRS_PP_FALSE_EASTING, 
+    SRS_PP_FALSE_EASTING,
     SRS_PP_FALSE_NORTHING,
     NULL,
 
     SRS_PT_WAGNER_IV,
-    SRS_PP_FALSE_EASTING, 
+    SRS_PP_FALSE_EASTING,
     SRS_PP_FALSE_NORTHING,
     NULL,
 
     SRS_PT_WAGNER_V,
-    SRS_PP_FALSE_EASTING, 
+    SRS_PP_FALSE_EASTING,
     SRS_PP_FALSE_NORTHING,
     NULL,
 
     SRS_PT_WAGNER_VI,
-    SRS_PP_FALSE_EASTING, 
+    SRS_PP_FALSE_EASTING,
     SRS_PP_FALSE_NORTHING,
     NULL,
 
     SRS_PT_WAGNER_VII,
-    SRS_PP_FALSE_EASTING, 
+    SRS_PP_FALSE_EASTING,
     SRS_PP_FALSE_NORTHING,
     NULL,
 
+    SRS_PT_QSC,
+    SRS_PP_LATITUDE_OF_ORIGIN,
+    SRS_PP_CENTRAL_MERIDIAN,
+    NULL,
+
+    SRS_PT_SCH,
+    SRS_PP_PEG_POINT_LATITUDE,
+    SRS_PP_PEG_POINT_LONGITUDE,
+    SRS_PP_PEG_POINT_HEADING,
+    SRS_PP_PEG_POINT_HEIGHT,
+    NULL,
+
     SRS_PT_GAUSSSCHREIBERTMERCATOR,
     SRS_PP_LATITUDE_OF_ORIGIN,
     SRS_PP_CENTRAL_MERIDIAN,
@@ -526,7 +544,7 @@ static const char *papszProjWithParms[] = {
     NULL
 };
 
-static const char *papszAliasGroupList[] = {
+static const char * const papszAliasGroupList[] = {
     SRS_PP_LATITUDE_OF_ORIGIN,
     SRS_PP_LATITUDE_OF_CENTER,
     NULL,
@@ -547,7 +565,7 @@ static const char *papszAliasGroupList[] = {
  *
  * This method attempts to verify that the spatial reference system is
  * well formed, and consists of known tokens.  The validation is not
- * comprehensive. 
+ * comprehensive.
  *
  * This method is the same as the C function OSRValidate().
  *
@@ -575,7 +593,7 @@ OGRErr OGRSpatialReference::Validate()
     /* using the CT spec grammar */
     static int bUseCTGrammar = -1;
     if( bUseCTGrammar < 0 )
-        bUseCTGrammar = CSLTestBoolean(CPLGetConfigOption("OSR_USE_CT_GRAMMAR", "TRUE"));
+        bUseCTGrammar = CPLTestBool(CPLGetConfigOption("OSR_USE_CT_GRAMMAR", "TRUE"));
 
     if( eErr == OGRERR_NONE && bUseCTGrammar )
     {
@@ -612,7 +630,7 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
         && !EQUAL(poRoot->GetValue(),"COMPD_CS"))
     {
         CPLDebug( "OGRSpatialReference::Validate",
-                  "Unrecognised root node `%s'\n",
+                  "Unrecognized root node `%s'\n",
                   poRoot->GetValue() );
         return OGRERR_CORRUPT_DATA;
     }
@@ -648,7 +666,7 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
             }
             else if( EQUAL(poNode->GetValue(),"EXTENSION") )
             {
-                // We do not try to control the sub-organization of 
+                // We do not try to control the sub-organization of
                 // EXTENSION nodes.
             }
             else
@@ -671,8 +689,8 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
     {
         OGR_SRSNode     *poNode;
         int             i;
-        int             bGotVertDatum = FALSE;
-        int             bGotUnit = FALSE;
+        bool            bGotVertDatum = false;
+        bool            bGotUnit = false;
         int             nCountAxis = 0;
 
         for( i = 1; i < poRoot->GetChildCount(); i++ )
@@ -684,14 +702,14 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
                 OGRErr eErr = ValidateVertDatum(poNode);
                 if (eErr != OGRERR_NONE)
                     return eErr;
-                bGotVertDatum = TRUE;
+                bGotVertDatum = true;
             }
             else if( EQUAL(poNode->GetValue(),"UNIT") )
             {
                 OGRErr eErr = ValidateUnit(poNode);
                 if (eErr != OGRERR_NONE)
                     return eErr;
-                bGotUnit = TRUE;
+                bGotUnit = true;
             }
             else if( EQUAL(poNode->GetValue(),"AXIS") )
             {
@@ -749,9 +767,9 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
     {
         OGR_SRSNode     *poNode;
         int             i;
-        int             bGotDatum = FALSE;
-        int             bGotPrimeM = FALSE;
-        int             bGotUnit = FALSE;
+        bool            bGotDatum = false;
+        bool            bGotPrimeM = false;
+        bool            bGotUnit = false;
         int             nCountAxis = 0;
 
         for( i = 1; i < poRoot->GetChildCount(); i++ )
@@ -760,20 +778,20 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
 
             if( EQUAL(poNode->GetValue(),"DATUM") )
             {
-                bGotDatum = TRUE;
+                bGotDatum = true;
             }
             else if( EQUAL(poNode->GetValue(),"PRIMEM") )
             {
-                bGotPrimeM = TRUE;
+                bGotPrimeM = true;
 
-                if( poNode->GetChildCount() < 2 
+                if( poNode->GetChildCount() < 2
                     || poNode->GetChildCount() > 3 )
                 {
                     CPLDebug( "OGRSpatialReference::Validate",
                               "PRIMEM has wrong number of children (%d),"
                               "not 2 or 3 as expected.\n",
                               poNode->GetChildCount() );
-                    
+
                     return OGRERR_CORRUPT_DATA;
                 }
             }
@@ -782,7 +800,7 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
                 OGRErr eErr = ValidateUnit(poNode);
                 if (eErr != OGRERR_NONE)
                     return eErr;
-                bGotUnit = TRUE;
+                bGotUnit = true;
             }
             else if( EQUAL(poNode->GetValue(),"AXIS") )
             {
@@ -870,16 +888,16 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
                               "PARAMETER has wrong number of children (%d),"
                               "not 2 as expected.\n",
                               poNode->GetChildCount() );
-                    
+
                     return OGRERR_CORRUPT_DATA;
                 }
                 else if( CSLFindString( (char **)papszParameters,
                                         poNode->GetChild(0)->GetValue()) == -1)
                 {
                     CPLDebug( "OGRSpatialReference::Validate",
-                              "Unrecognised PARAMETER `%s'.\n",
+                              "Unrecognized PARAMETER `%s'.\n",
                               poNode->GetChild(0)->GetValue() );
-                    
+
                     return OGRERR_UNSUPPORTED_SRS;
                 }
             }
@@ -891,7 +909,7 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
                               "PROJECTION has wrong number of children (%d),"
                               "not 1 or 2 as expected.\n",
                               poNode->GetChildCount() );
-                    
+
                     return OGRERR_CORRUPT_DATA;
                 }
                 else if( CSLFindString( (char **)papszProjectionSupported,
@@ -900,18 +918,18 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
                                         poNode->GetChild(0)->GetValue()) == -1)
                 {
                     CPLDebug( "OGRSpatialReference::Validate",
-                              "Unrecognised PROJECTION `%s'.\n",
+                              "Unrecognized PROJECTION `%s'.\n",
                               poNode->GetChild(0)->GetValue() );
-                    
+
                     return OGRERR_UNSUPPORTED_SRS;
                 }
                 else if( CSLFindString( (char **)papszProjectionSupported,
                                         poNode->GetChild(0)->GetValue()) == -1)
                 {
                     CPLDebug( "OGRSpatialReference::Validate",
-                              "Unsupported, but recognised PROJECTION `%s'.\n",
+                              "Unsupported, but recognized PROJECTION `%s'.\n",
                               poNode->GetChild(0)->GetValue() );
-                    
+
                     return OGRERR_UNSUPPORTED_SRS;
                 }
 
@@ -948,7 +966,7 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
             }
             else if( EQUAL(poNode->GetValue(),"EXTENSION") )
             {
-                // We do not try to control the sub-organization of 
+                // We do not try to control the sub-organization of
                 // EXTENSION nodes.
             }
             else
@@ -956,12 +974,12 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
                 CPLDebug( "OGRSpatialReference::Validate",
                           "Unexpected child for PROJCS `%s'.\n",
                           poNode->GetValue() );
-                
+
                 return OGRERR_CORRUPT_DATA;
             }
         }
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Validate GEOGCS if found.                                       */
 /* -------------------------------------------------------------------- */
@@ -982,14 +1000,14 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
             }
             else if( EQUAL(poNode->GetValue(),"PRIMEM") )
             {
-                if( poNode->GetChildCount() < 2 
+                if( poNode->GetChildCount() < 2
                     || poNode->GetChildCount() > 3 )
                 {
                     CPLDebug( "OGRSpatialReference::Validate",
                               "PRIMEM has wrong number of children (%d),"
                               "not 2 or 3 as expected.\n",
                               poNode->GetChildCount() );
-                    
+
                     return OGRERR_CORRUPT_DATA;
                 }
             }
@@ -1005,6 +1023,11 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
                 if (eErr != OGRERR_NONE)
                     return eErr;
             }
+            else if( EQUAL(poNode->GetValue(),"EXTENSION") )
+            {
+                // We do not try to control the sub-organization of
+                // EXTENSION nodes.
+            }
             else if( EQUAL(poNode->GetValue(),"AUTHORITY") )
             {
                 OGRErr eErr = ValidateAuthority(poNode);
@@ -1016,7 +1039,7 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
                 CPLDebug( "OGRSpatialReference::Validate",
                           "Unexpected child for GEOGCS `%s'.\n",
                           poNode->GetValue() );
-                
+
                 return OGRERR_CORRUPT_DATA;
             }
         }
@@ -1025,7 +1048,7 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
         {
             CPLDebug( "OGRSpatialReference::Validate",
                       "No DATUM child in GEOGCS.\n" );
-            
+
             return OGRERR_CORRUPT_DATA;
         }
     }
@@ -1038,14 +1061,14 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
     if( poDATUM != NULL )
     {
         OGR_SRSNode     *poSPHEROID;
-        int             bGotSpheroid = FALSE;
+        bool            bGotSpheroid = false;
         int             i;
 
         if( poDATUM->GetChildCount() == 0 )
         {
             CPLDebug( "OGRSpatialReference::Validate",
                       "DATUM has no children." );
-            
+
             return OGRERR_CORRUPT_DATA;
         }
 
@@ -1057,16 +1080,16 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
             if( EQUAL(poNode->GetValue(),"SPHEROID") )
             {
                 poSPHEROID = poDATUM->GetChild(1);
-                bGotSpheroid = TRUE;
+                bGotSpheroid = true;
 
-                if( poSPHEROID->GetChildCount() != 3 
+                if( poSPHEROID->GetChildCount() != 3
                     && poSPHEROID->GetChildCount() != 4 )
                 {
                     CPLDebug( "OGRSpatialReference::Validate",
                               "SPHEROID has wrong number of children (%d),"
                               "not 3 or 4 as expected.\n",
                               poSPHEROID->GetChildCount() );
-                    
+
                     return OGRERR_CORRUPT_DATA;
                 }
                 else if( CPLAtof(poSPHEROID->GetChild(1)->GetValue()) == 0.0 )
@@ -1085,7 +1108,7 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
             }
             else if( EQUAL(poNode->GetValue(),"TOWGS84") )
             {
-                if( poNode->GetChildCount() != 3 
+                if( poNode->GetChildCount() != 3
                     && poNode->GetChildCount() != 7)
                 {
                     CPLDebug( "OGRSpatialReference::Validate",
@@ -1096,7 +1119,7 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
             }
             else if( EQUAL(poNode->GetValue(),"EXTENSION") )
             {
-                // We do not try to control the sub-organization of 
+                // We do not try to control the sub-organization of
                 // EXTENSION nodes.
             }
             else
@@ -1104,7 +1127,7 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
                 CPLDebug( "OGRSpatialReference::Validate",
                           "Unexpected child for DATUM `%s'.\n",
                           poNode->GetValue() );
-                
+
                 return OGRERR_CORRUPT_DATA;
             }
         }
@@ -1113,10 +1136,10 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
         {
             CPLDebug( "OGRSpatialReference::Validate",
                       "No SPHEROID child in DATUM.\n" );
-            
+
             return OGRERR_CORRUPT_DATA;
         }
-    }        
+    }
 
 /* -------------------------------------------------------------------- */
 /*      If this is projected, try to validate the detailed set of       */
@@ -1134,7 +1157,7 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
 /************************************************************************/
 /*                            OSRValidate()                             */
 /************************************************************************/
-/** 
+/**
  * \brief Validate SRS tokens.
  *
  * This function is the same as the C++ method OGRSpatialReference::Validate().
@@ -1142,7 +1165,7 @@ OGRErr OGRSpatialReference::Validate(OGR_SRSNode *poRoot)
 OGRErr OSRValidate( OGRSpatialReferenceH hSRS )
 
 {
-    VALIDATE_POINTER1( hSRS, "OSRValidate", CE_Failure );
+    VALIDATE_POINTER1( hSRS, "OSRValidate", OGRERR_FAILURE );
 
     return ((OGRSpatialReference *) hSRS)->Validate();
 }
@@ -1160,7 +1183,7 @@ OGRErr OSRValidate( OGRSpatialReferenceH hSRS )
  *
  * @return TRUE if both strings are aliases according to the AliasGroupList, FALSE otherwise
  */
-int OGRSpatialReference::IsAliasFor( const char *pszParm1, 
+int OGRSpatialReference::IsAliasFor( const char *pszParm1,
                                      const char *pszParm2 )
 
 {
@@ -1216,7 +1239,7 @@ OGRErr OGRSpatialReference::ValidateProjection(OGR_SRSNode *poRoot)
 
     if( poPROJCS->GetNode( "PROJECTION" ) == NULL )
     {
-        CPLDebug( "OGRSpatialReference::Validate", 
+        CPLDebug( "OGRSpatialReference::Validate",
                   "PROJCS does not have PROJECTION subnode." );
         return OGRERR_CORRUPT_DATA;
     }
@@ -1226,10 +1249,10 @@ OGRErr OGRSpatialReference::ValidateProjection(OGR_SRSNode *poRoot)
 /* -------------------------------------------------------------------- */
     const char *pszProjection;
     int        iOffset;
-    
+
     pszProjection = poPROJCS->GetNode("PROJECTION")->GetChild(0)->GetValue();
 
-    for( iOffset = 0; 
+    for( iOffset = 0;
          papszProjWithParms[iOffset] != NULL
              && !EQUAL(papszProjWithParms[iOffset],pszProjection); )
     {
@@ -1277,14 +1300,14 @@ OGRErr OGRSpatialReference::ValidateProjection(OGR_SRSNode *poRoot)
 
             if( papszProjWithParms[i] == NULL )
             {
-                CPLDebug( "OGRSpatialReference::Validate", 
+                CPLDebug( "OGRSpatialReference::Validate",
                           "PARAMETER %s for PROJECTION %s is not permitted.",
                           pszParmName, pszProjection );
                 return OGRERR_CORRUPT_DATA;
             }
             else
             {
-                CPLDebug( "OGRSpatialReference::Validate", 
+                CPLDebug( "OGRSpatialReference::Validate",
                           "PARAMETER %s for PROJECTION %s is an alias for %s.",
                           pszParmName, pszProjection,
                           papszProjWithParms[i] );
@@ -1444,4 +1467,3 @@ OGRErr OGRSpatialReference::ValidateUnit(OGR_SRSNode *poRoot)
 
     return OGRERR_NONE;
 }
-
