@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_srsnode.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogr_srsnode.cpp 33674 2016-03-07 20:58:21Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  The OGR_SRSNode class.
@@ -28,10 +28,10 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "ogr_spatialref.h"
 #include "ogr_p.h"
+#include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: ogr_srsnode.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: ogr_srsnode.cpp 33674 2016-03-07 20:58:21Z goatbar $");
 
 /************************************************************************/
 /*                            OGR_SRSNode()                             */
@@ -45,16 +45,12 @@ CPL_CVSID("$Id: ogr_srsnode.cpp 27044 2014-03-16 23:41:27Z rouault $");
  * with a value of "".  Newly created OGR_SRSNodes have no children.
  */
 
-OGR_SRSNode::OGR_SRSNode( const char * pszValueIn )
-
-{
-    pszValue = CPLStrdup( pszValueIn );
-
-    nChildren = 0;
-    papoChildNodes = NULL;
-
-    poParent = NULL;
-}
+OGR_SRSNode::OGR_SRSNode( const char * pszValueIn ) :
+    pszValue(CPLStrdup(pszValueIn)),
+    papoChildNodes(NULL),
+    poParent(NULL),
+    nChildren(0)
+{}
 
 /************************************************************************/
 /*                            ~OGR_SRSNode()                            */
@@ -95,7 +91,7 @@ void OGR_SRSNode::ClearChildren()
  *
  * Get number of children nodes.
  *
- * @return 0 for leaf nodes, or the number of children nodes. 
+ * @return 0 for leaf nodes, or the number of children nodes.
  */
 
 /************************************************************************/
@@ -109,7 +105,7 @@ void OGR_SRSNode::ClearChildren()
  * GetChildCount() - 1.
  *
  * @return a pointer to the child OGR_SRSNode, or NULL if there is no such
- * child. 
+ * child.
  */
 
 OGR_SRSNode *OGR_SRSNode::GetChild( int iChild )
@@ -117,8 +113,8 @@ OGR_SRSNode *OGR_SRSNode::GetChild( int iChild )
 {
     if( iChild < 0 || iChild >= nChildren )
         return NULL;
-    else
-        return papoChildNodes[iChild];
+
+    return papoChildNodes[iChild];
 }
 
 const OGR_SRSNode *OGR_SRSNode::GetChild( int iChild ) const
@@ -126,8 +122,8 @@ const OGR_SRSNode *OGR_SRSNode::GetChild( int iChild ) const
 {
     if( iChild < 0 || iChild >= nChildren )
         return NULL;
-    else
-        return papoChildNodes[iChild];
+
+    return papoChildNodes[iChild];
 }
 
 /************************************************************************/
@@ -144,7 +140,7 @@ const OGR_SRSNode *OGR_SRSNode::GetChild( int iChild ) const
  *
  * If a node appears more than once in the tree (such as UNIT for instance),
  * the first encountered will be returned.  Use GetNode() on a subtree to be
- * more specific. 
+ * more specific.
  *
  * @param pszName the name of the node to search for.
  *
@@ -154,11 +150,6 @@ const OGR_SRSNode *OGR_SRSNode::GetChild( int iChild ) const
 OGR_SRSNode *OGR_SRSNode::GetNode( const char * pszName )
 
 {
-    int  i;
-
-    if( this == NULL )
-        return NULL;
-    
     if( nChildren > 0 && EQUAL(pszName,pszValue) )
         return this;
 
@@ -166,9 +157,9 @@ OGR_SRSNode *OGR_SRSNode::GetNode( const char * pszName )
 /*      First we check the immediate children so we will get an         */
 /*      immediate child in preference to a subchild.                    */
 /* -------------------------------------------------------------------- */
-    for( i = 0; i < nChildren; i++ )
+    for( int i = 0; i < nChildren; i++ )
     {
-        if( EQUAL(papoChildNodes[i]->pszValue,pszName) 
+        if( EQUAL(papoChildNodes[i]->pszValue,pszName)
             && papoChildNodes[i]->nChildren > 0 )
             return papoChildNodes[i];
     }
@@ -176,11 +167,9 @@ OGR_SRSNode *OGR_SRSNode::GetNode( const char * pszName )
 /* -------------------------------------------------------------------- */
 /*      Then get each child to check their children.                    */
 /* -------------------------------------------------------------------- */
-    for( i = 0; i < nChildren; i++ )
+    for( int i = 0; i < nChildren; i++ )
     {
-        OGR_SRSNode *poNode;
-
-        poNode = papoChildNodes[i]->GetNode( pszName );
+        OGR_SRSNode *poNode = papoChildNodes[i]->GetNode( pszName );
         if( poNode != NULL )
             return poNode;
     }
@@ -191,7 +180,7 @@ OGR_SRSNode *OGR_SRSNode::GetNode( const char * pszName )
 const OGR_SRSNode *OGR_SRSNode::GetNode( const char * pszName ) const
 
 {
-    return ((OGR_SRSNode *) this)->GetNode( pszName );
+    return const_cast<OGR_SRSNode *>(this)->GetNode( pszName );
 }
 
 /************************************************************************/
@@ -220,15 +209,15 @@ void OGR_SRSNode::AddChild( OGR_SRSNode * poNew )
 
 /**
  * Insert the passed node as a child of target node, at the indicated
- * position. 
+ * position.
  *
  * Note that ownership of the passed node is assumed by the node on which
  * the method is invoked ... use the Clone() method if the original is to
  * be preserved.  All existing children at location iChild and beyond are
- * push down one space to make space for the new child. 
+ * push down one space to make space for the new child.
  *
  * @param poNew the node to add as a child.
- * @param iChild position to insert, use 0 to insert at the beginning. 
+ * @param iChild position to insert, use 0 to insert at the beginning.
  */
 
 void OGR_SRSNode::InsertChild( OGR_SRSNode * poNew, int iChild )
@@ -238,12 +227,12 @@ void OGR_SRSNode::InsertChild( OGR_SRSNode * poNew, int iChild )
         iChild = nChildren;
 
     nChildren++;
-    papoChildNodes = (OGR_SRSNode **)
-        CPLRealloc( papoChildNodes, sizeof(void*) * nChildren );
+    papoChildNodes = static_cast<OGR_SRSNode **>(
+        CPLRealloc( papoChildNodes, sizeof(void*) * nChildren ) );
 
     memmove( papoChildNodes + iChild + 1, papoChildNodes + iChild,
              sizeof(void*) * (nChildren - iChild - 1) );
-    
+
     papoChildNodes[iChild] = poNew;
     poNew->poParent = this;
 }
@@ -287,17 +276,17 @@ void OGR_SRSNode::DestroyChild( int iChild )
  * Note that the node value must match pszValue with the exception of
  * case.  The comparison is case insensitive.
  *
- * @param pszValue the node value being searched for.
+ * @param pszValueIn the node value being searched for.
  *
- * @return the child index, or -1 on failure. 
+ * @return the child index, or -1 on failure.
  */
 
-int OGR_SRSNode::FindChild( const char * pszValue ) const
+int OGR_SRSNode::FindChild( const char * pszValueIn ) const
 
 {
     for( int i = 0; i < nChildren; i++ )
     {
-        if( EQUAL(papoChildNodes[i]->pszValue,pszValue) )
+        if( EQUAL(papoChildNodes[i]->pszValue, pszValueIn) )
             return i;
     }
 
@@ -342,15 +331,13 @@ void OGR_SRSNode::SetValue( const char * pszNewValue )
 /**
  * Make a duplicate of this node, and it's children.
  *
- * @return a new node tree, which becomes the responsiblity of the caller.
+ * @return a new node tree, which becomes the responsibility of the caller.
  */
 
 OGR_SRSNode *OGR_SRSNode::Clone() const
 
 {
-    OGR_SRSNode *poNew;
-
-    poNew = new OGR_SRSNode( pszValue );
+    OGR_SRSNode *poNew = new OGR_SRSNode( pszValue );
 
     for( int i = 0; i < nChildren; i++ )
     {
@@ -375,22 +362,22 @@ int OGR_SRSNode::NeedsQuoting() const
 
     // As per bugzilla bug 201, the OGC spec says the authority code
     // needs to be quoted even though it appears well behaved.
-    if( poParent != NULL && EQUAL(poParent->GetValue(),"AUTHORITY") )
+    if( poParent != NULL && EQUAL(poParent->GetValue(), "AUTHORITY") )
         return TRUE;
-    
+
     // As per bugzilla bug 294, the OGC spec says the direction
     // values for the AXIS keywords should *not* be quoted.
-    if( poParent != NULL && EQUAL(poParent->GetValue(),"AXIS") 
+    if( poParent != NULL && EQUAL(poParent->GetValue(), "AXIS")
         && this != poParent->GetChild(0) )
         return FALSE;
 
     // Strings starting with e or E are not valid numeric values, so they
-    // need quoting, like in AXIS["E",EAST] 
+    // need quoting, like in AXIS["E",EAST]
     if( (pszValue[0] == 'e' || pszValue[0] == 'E') )
         return TRUE;
 
     // Non-numeric tokens are generally quoted while clean numeric values
-    // are generally not. 
+    // are generally not.
     for( int i = 0; pszValue[i] != '\0'; i++ )
     {
         if( (pszValue[i] < '0' || pszValue[i] > '9')
@@ -416,23 +403,21 @@ int OGR_SRSNode::NeedsQuoting() const
  * @param ppszResult the resulting string is returned in this pointer.
  *
  * @return currently OGRERR_NONE is always returned, but the future it
- * is possible error conditions will develop. 
+ * is possible error conditions will develop.
  */
- 
+
 
 OGRErr OGR_SRSNode::exportToWkt( char ** ppszResult ) const
 
 {
-    char        **papszChildrenWkt = NULL;
-    int         nLength = strlen(pszValue)+4;
-    int         i;
-
 /* -------------------------------------------------------------------- */
 /*      Build a list of the WKT format for the children.                */
 /* -------------------------------------------------------------------- */
-    papszChildrenWkt = (char **) CPLCalloc(sizeof(char*),(nChildren+1));
-    
-    for( i = 0; i < nChildren; i++ )
+    char **papszChildrenWkt = static_cast<char **>(
+        CPLCalloc(sizeof(char*),(nChildren+1)) );
+    size_t nLength = strlen(pszValue)+4;
+
+    for( int i = 0; i < nChildren; i++ )
     {
         papoChildNodes[i]->exportToWkt( papszChildrenWkt + i );
         nLength += strlen(papszChildrenWkt[i]) + 1;
@@ -441,9 +426,9 @@ OGRErr OGR_SRSNode::exportToWkt( char ** ppszResult ) const
 /* -------------------------------------------------------------------- */
 /*      Allocate the result string.                                     */
 /* -------------------------------------------------------------------- */
-    *ppszResult = (char *) CPLMalloc(nLength);
+    *ppszResult = static_cast<char *>( CPLMalloc(nLength) );
     *ppszResult[0] = '\0';
-    
+
 /* -------------------------------------------------------------------- */
 /*      Capture this nodes value.  We put it in double quotes if        */
 /*      this is a leaf node, otherwise we assume it is a well formed    */
@@ -463,8 +448,8 @@ OGRErr OGR_SRSNode::exportToWkt( char ** ppszResult ) const
 /* -------------------------------------------------------------------- */
     if( nChildren > 0 )
         strcat( *ppszResult, "[" );
-    
-    for( i = 0; i < nChildren; i++ )
+
+    for( int i = 0; i < nChildren; i++ )
     {
         strcat( *ppszResult, papszChildrenWkt[i] );
         if( i == nChildren-1 )
@@ -485,16 +470,14 @@ OGRErr OGR_SRSNode::exportToWkt( char ** ppszResult ) const
 OGRErr OGR_SRSNode::exportToPrettyWkt( char ** ppszResult, int nDepth ) const
 
 {
-    char        **papszChildrenWkt = NULL;
-    int         nLength = strlen(pszValue)+4;
-    int         i;
-
 /* -------------------------------------------------------------------- */
 /*      Build a list of the WKT format for the children.                */
 /* -------------------------------------------------------------------- */
-    papszChildrenWkt = (char **) CPLCalloc(sizeof(char*),(nChildren+1));
-    
-    for( i = 0; i < nChildren; i++ )
+    char **papszChildrenWkt = static_cast<char **>(
+        CPLCalloc(sizeof(char*),(nChildren+1)) );
+    size_t nLength = strlen(pszValue)+4;
+
+    for( int i = 0; i < nChildren; i++ )
     {
         papoChildNodes[i]->exportToPrettyWkt( papszChildrenWkt + i,
                                               nDepth + 1);
@@ -504,9 +487,9 @@ OGRErr OGR_SRSNode::exportToPrettyWkt( char ** ppszResult, int nDepth ) const
 /* -------------------------------------------------------------------- */
 /*      Allocate the result string.                                     */
 /* -------------------------------------------------------------------- */
-    *ppszResult = (char *) CPLMalloc(nLength);
+    *ppszResult = static_cast<char *>( CPLMalloc(nLength) );
     *ppszResult[0] = '\0';
-    
+
 /* -------------------------------------------------------------------- */
 /*      Capture this nodes value.  We put it in double quotes if        */
 /*      this is a leaf node, otherwise we assume it is a well formed    */
@@ -526,15 +509,13 @@ OGRErr OGR_SRSNode::exportToPrettyWkt( char ** ppszResult, int nDepth ) const
 /* -------------------------------------------------------------------- */
     if( nChildren > 0 )
         strcat( *ppszResult, "[" );
-    
-    for( i = 0; i < nChildren; i++ )
+
+    for( int i = 0; i < nChildren; i++ )
     {
         if( papoChildNodes[i]->GetChildCount() > 0 )
         {
-            int  j;
-
             strcat( *ppszResult, "\n" );
-            for( j = 0; j < 4*nDepth; j++ )
+            for( int j = 0; j < 4*nDepth; j++ )
                 strcat( *ppszResult, " " );
         }
         strcat( *ppszResult, papszChildrenWkt[i] );
@@ -546,7 +527,7 @@ OGRErr OGR_SRSNode::exportToPrettyWkt( char ** ppszResult, int nDepth ) const
     {
         if( (*ppszResult)[strlen(*ppszResult)-1] == ',' )
             (*ppszResult)[strlen(*ppszResult)-1] = '\0';
-        
+
         strcat( *ppszResult, "]" );
     }
 
@@ -586,7 +567,7 @@ OGRErr OGR_SRSNode::importFromWkt( char ** ppszInput, int nRecLevel, int* pnNode
 
 {
     const char  *pszInput = *ppszInput;
-    int         bInQuotedString = FALSE;
+    bool bInQuotedString = false;
 
     /* Sanity checks */
     if( nRecLevel == 10 )
@@ -597,19 +578,21 @@ OGRErr OGR_SRSNode::importFromWkt( char ** ppszInput, int nRecLevel, int* pnNode
     {
         return OGRERR_CORRUPT_DATA;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Clear any existing children of this node.                       */
 /* -------------------------------------------------------------------- */
     ClearChildren();
-    
+
 /* -------------------------------------------------------------------- */
 /*      Read the ``value'' for this node.                               */
 /* -------------------------------------------------------------------- */
     char        szToken[512];
-    int         nTokenLen = 0;
-    
-    while( *pszInput != '\0' && nTokenLen < (int) sizeof(szToken)-1 )
+    szToken[0] = '\0';
+    size_t nTokenLen = 0;
+
+    while( *pszInput != '\0' &&
+           nTokenLen + 1 < sizeof(szToken) )
     {
         if( *pszInput == '"' )
         {
@@ -621,12 +604,12 @@ OGRErr OGR_SRSNode::importFromWkt( char ** ppszInput, int nRecLevel, int* pnNode
         {
             break;
         }
-        else if( !bInQuotedString 
-                 && (*pszInput == ' ' || *pszInput == '\t' 
+        else if( !bInQuotedString
+                 && (*pszInput == ' ' || *pszInput == '\t'
                      || *pszInput == 10 || *pszInput == 13) )
         {
             /* just skip over whitespace */
-        } 
+        }
         else
         {
             szToken[nTokenLen++] = *pszInput;
@@ -648,15 +631,16 @@ OGRErr OGR_SRSNode::importFromWkt( char ** ppszInput, int nRecLevel, int* pnNode
     {
         do
         {
-            OGR_SRSNode *poNewChild;
-            OGRErr      eErr;
 
             pszInput++; // Skip bracket or comma.
 
-            poNewChild = new OGR_SRSNode();
+            OGR_SRSNode *poNewChild = new OGR_SRSNode();
 
             (*pnNodes) ++;
-            eErr = poNewChild->importFromWkt( (char **) &pszInput, nRecLevel + 1, pnNodes );
+            const OGRErr eErr =
+                poNewChild->importFromWkt(
+                    const_cast<char **>( &pszInput ),
+                    nRecLevel + 1, pnNodes );
             if( eErr != OGRERR_NONE )
             {
                 delete poNewChild;
@@ -664,9 +648,9 @@ OGRErr OGR_SRSNode::importFromWkt( char ** ppszInput, int nRecLevel, int* pnNode
             }
 
             AddChild( poNewChild );
-            
+
             // swallow whitespace
-            while( isspace(*pszInput) ) 
+            while( isspace(*pszInput) )
                 pszInput++;
 
         } while( *pszInput == ',' );
@@ -697,8 +681,6 @@ OGRErr OGR_SRSNode::importFromWkt( char ** ppszInput, int nRecLevel, int* pnNode
 void OGR_SRSNode::MakeValueSafe()
 
 {
-    int         i, j;
-
 /* -------------------------------------------------------------------- */
 /*      First process subnodes.                                         */
 /* -------------------------------------------------------------------- */
@@ -712,11 +694,11 @@ void OGR_SRSNode::MakeValueSafe()
 /* -------------------------------------------------------------------- */
     if( (pszValue[0] >= '0' && pszValue[0] <= '9') || pszValue[0] != '.' )
         return;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Translate non-alphanumeric values to underscores.               */
 /* -------------------------------------------------------------------- */
-    for( i = 0; pszValue[i] != '\0'; i++ )
+    for( int i = 0; pszValue[i] != '\0'; i++ )
     {
         if( !(pszValue[i] >= 'A' && pszValue[i] <= 'Z')
             && !(pszValue[i] >= 'a' && pszValue[i] <= 'z')
@@ -729,14 +711,15 @@ void OGR_SRSNode::MakeValueSafe()
 /* -------------------------------------------------------------------- */
 /*      Remove repeated and trailing underscores.                       */
 /* -------------------------------------------------------------------- */
-    for( i = 1, j = 0; pszValue[i] != '\0'; i++ )
+    int j = 0;
+    for( int i = 1; pszValue[i] != '\0'; i++ )
     {
         if( pszValue[j] == '_' && pszValue[i] == '_' )
             continue;
 
         pszValue[++j] = pszValue[i];
     }
-    
+
     if( pszValue[j] == '_' )
         pszValue[j] = '\0';
     else
@@ -756,8 +739,8 @@ void OGR_SRSNode::MakeValueSafe()
  * parent node matches that value.  Even if a replacement occurs, searching
  * continues.
  *
- * @param pszNode Restrict remapping to children of this type of node 
- *                (eg. "PROJECTION")
+ * @param pszNode Restrict remapping to children of this type of node
+ *                (e.g. "PROJECTION")
  * @param papszSrcValues a NULL terminated array of source string.  If the
  * node value matches one of these (case insensitive) then replacement occurs.
  * @param papszDstValues an array of destination strings.  On a match, the
@@ -768,27 +751,25 @@ void OGR_SRSNode::MakeValueSafe()
  * @param bChildOfHit Only TRUE if we the current node is the child of a match,
  * and so needs to be set.  Application code would normally pass FALSE for this
  * argument.
- * 
+ *
  * @return returns OGRERR_NONE unless something bad happens.  There is no
- * indication returned about whether any replacement occured.  
+ * indication returned about whether any replacement occurred.
  */
 
-OGRErr OGR_SRSNode::applyRemapper( const char *pszNode, 
-                                   char **papszSrcValues, 
-                                   char **papszDstValues, 
+OGRErr OGR_SRSNode::applyRemapper( const char *pszNode,
+                                   char **papszSrcValues,
+                                   char **papszDstValues,
                                    int nStepSize, int bChildOfHit )
 
 {
-    int i;
-
 /* -------------------------------------------------------------------- */
 /*      Scan for value, and replace if our parent was a "hit".          */
 /* -------------------------------------------------------------------- */
     if( bChildOfHit || pszNode == NULL )
     {
-        for( i = 0; papszSrcValues[i] != NULL; i += nStepSize )
+        for( int i = 0; papszSrcValues[i] != NULL; i += nStepSize )
         {
-            if( EQUAL(papszSrcValues[i],pszValue) && 
+            if( EQUAL(papszSrcValues[i],pszValue) &&
                 ! EQUAL(papszDstValues[i],"") )
             {
                 SetValue( papszDstValues[i] );
@@ -798,7 +779,7 @@ OGRErr OGR_SRSNode::applyRemapper( const char *pszNode,
     }
 
 /* -------------------------------------------------------------------- */
-/*      Are the the target node?                                        */
+/*      Is this the target node?                                        */
 /* -------------------------------------------------------------------- */
     if( pszNode != NULL )
         bChildOfHit = EQUAL(pszValue,pszNode);
@@ -806,15 +787,15 @@ OGRErr OGR_SRSNode::applyRemapper( const char *pszNode,
 /* -------------------------------------------------------------------- */
 /*      Recurse                                                         */
 /* -------------------------------------------------------------------- */
-    for( i = 0; i < GetChildCount(); i++ )
+    for( int i = 0; i < GetChildCount(); i++ )
     {
-        GetChild(i)->applyRemapper( pszNode, papszSrcValues, 
+        GetChild(i)->applyRemapper( pszNode, papszSrcValues,
                                     papszDstValues, nStepSize, bChildOfHit );
     }
 
     return OGRERR_NONE;
 }
-                                   
+
 /************************************************************************/
 /*                             StripNodes()                             */
 /************************************************************************/
@@ -822,7 +803,7 @@ OGRErr OGR_SRSNode::applyRemapper( const char *pszNode,
 /**
  * Strip child nodes matching name.
  *
- * Removes any decendent nodes of this node that match the given name. 
+ * Removes any descendant nodes of this node that match the given name.
  * Of course children of removed nodes are also discarded.
  *
  * @param pszName the name for nodes that should be removed.
@@ -849,24 +830,25 @@ void OGR_SRSNode::StripNodes( const char * pszName )
 /************************************************************************/
 
 /* EXTENSION ... being a OSR extension... is arbitrary placed before the AUTHORITY */
-static const char * const apszPROJCSRule[] = 
-{ "PROJCS", "GEOGCS", "PROJECTION", "PARAMETER", "UNIT", "AXIS", "EXTENSION", "AUTHORITY", 
-  NULL };
+static const char * const apszPROJCSRule[] =
+{ "PROJCS", "GEOGCS", "PROJECTION", "PARAMETER", "UNIT", "AXIS", "EXTENSION",
+  "AUTHORITY", NULL };
 
-static const char * const apszDATUMRule[] = 
-{ "DATUM", "SPHEROID", "TOWGS84", "AUTHORITY", NULL };
+static const char * const apszDATUMRule[] =
+{ "DATUM", "SPHEROID", "TOWGS84", "EXTENSION", "AUTHORITY", NULL };
 
-static const char * const apszGEOGCSRule[] = 
-{ "GEOGCS", "DATUM", "PRIMEM", "UNIT", "AXIS", "AUTHORITY", NULL };
+static const char * const apszGEOGCSRule[] =
+{ "GEOGCS", "DATUM", "PRIMEM", "UNIT", "AXIS", "EXTENSION", "AUTHORITY", NULL };
 
-static const char * const apszGEOCCSRule[] = 
+static const char * const apszGEOCCSRule[] =
 { "GEOCCS", "DATUM", "PRIMEM", "UNIT", "AXIS", "AUTHORITY", NULL };
 
-static const char * const apszVERTCSRule[] = 
-{ "VERT_CS", "VERT_DATUM", "UNIT", "AXIS", "AUTHORITY", NULL };
+static const char * const apszVERTCSRule[] =
+{ "VERT_CS", "VERT_DATUM", "UNIT", "AXIS", "EXTENSION", "AUTHORITY", NULL };
 
-static const char * const *apszOrderingRules[] = {
-    apszPROJCSRule, apszGEOGCSRule, apszDATUMRule, apszGEOCCSRule, apszVERTCSRule, NULL };
+static const char * const * const apszOrderingRules[] = {
+    apszPROJCSRule, apszGEOGCSRule, apszDATUMRule, apszGEOCCSRule,
+    apszVERTCSRule, NULL };
 
 /**
  * Correct parameter ordering to match CT Specification.
@@ -879,19 +861,17 @@ static const char * const *apszOrderingRules[] = {
  * This method will reorder the children of the node it is invoked on and
  * then recurse to all children to fix up their children.
  *
- * @return OGRERR_NONE on success or an error code if something goes 
- * wrong.  
+ * @return OGRERR_NONE on success or an error code if something goes
+ * wrong.
  */
 
 OGRErr OGR_SRSNode::FixupOrdering()
 
 {
-    int    i;
-
 /* -------------------------------------------------------------------- */
 /*      Recurse ordering children.                                      */
 /* -------------------------------------------------------------------- */
-    for( i = 0; i < GetChildCount(); i++ )
+    for( int i = 0; i < GetChildCount(); i++ )
         GetChild(i)->FixupOrdering();
 
     if( GetChildCount() < 3 )
@@ -902,7 +882,7 @@ OGRErr OGR_SRSNode::FixupOrdering()
 /* -------------------------------------------------------------------- */
     const char * const * papszRule = NULL;
 
-    for( i = 0; apszOrderingRules[i] != NULL; i++ )
+    for( int i = 0; apszOrderingRules[i] != NULL; i++ )
     {
         if( EQUAL(apszOrderingRules[i][0],pszValue) )
         {
@@ -919,15 +899,16 @@ OGRErr OGR_SRSNode::FixupOrdering()
 /*      (panChildPr) with the priority code for each child (derived     */
 /*      from the rule) and we then bubble sort based on this.           */
 /* -------------------------------------------------------------------- */
-    int  *panChildKey = (int *) CPLCalloc(sizeof(int),GetChildCount());
+    int  *panChildKey = static_cast<int *>(
+        CPLCalloc(sizeof(int), GetChildCount()));
 
-    for( i = 1; i < GetChildCount(); i++ )
+    for( int i = 1; i < GetChildCount(); i++ )
     {
-        panChildKey[i] = CSLFindString( (char**) papszRule, 
+        panChildKey[i] = CSLFindString( (char**) papszRule,
                                         GetChild(i)->GetValue() );
         if( panChildKey[i] == -1 )
         {
-            CPLDebug( "OGRSpatialReference", 
+            CPLDebug( "OGRSpatialReference",
                       "Found unexpected key %s when trying to order SRS nodes.",
                       GetChild(i)->GetValue() );
         }
@@ -937,12 +918,12 @@ OGRErr OGR_SRSNode::FixupOrdering()
 /*      Sort - Note we don't try to do anything with the first child    */
 /*      which we assume is a name string.                               */
 /* -------------------------------------------------------------------- */
-    int j, bChange = TRUE;
+    bool bChange = true;
 
-    for( i = 1; bChange && i < GetChildCount()-1; i++ )
+    for( int i = 1; bChange && i < GetChildCount()-1; i++ )
     {
-        bChange = FALSE;
-        for( j = 1; j < GetChildCount()-i; j++ )
+        bChange = false;
+        for( int j = 1; j < GetChildCount()-i; j++ )
         {
             if( panChildKey[j] == -1 || panChildKey[j+1] == -1 )
                 continue;
@@ -950,16 +931,14 @@ OGRErr OGR_SRSNode::FixupOrdering()
             if( panChildKey[j] > panChildKey[j+1] )
             {
                 OGR_SRSNode *poTemp = papoChildNodes[j];
-                int          nKeyTemp = panChildKey[j];
-
                 papoChildNodes[j] = papoChildNodes[j+1];
                 papoChildNodes[j+1] = poTemp;
 
-                nKeyTemp = panChildKey[j];
+                int nKeyTemp = panChildKey[j];
                 panChildKey[j] = panChildKey[j+1];
                 panChildKey[j+1] = nKeyTemp;
 
-                bChange = TRUE;
+                bChange = true;
             }
         }
     }
@@ -968,5 +947,3 @@ OGRErr OGR_SRSNode::FixupOrdering()
 
     return OGRERR_NONE;
 }
-
-
