@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ograssemblepolygon.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ograssemblepolygon.cpp 33631 2016-03-04 06:28:09Z goatbar $
  *
  * Project:  S-57 Reader
  * Purpose:  Implements polygon assembly from a bunch of arcs.
@@ -33,7 +33,7 @@
 #include "ogr_api.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ograssemblepolygon.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: ograssemblepolygon.cpp 33631 2016-03-04 06:28:09Z goatbar $");
 
 /************************************************************************/
 /*                            CheckPoints()                             */
@@ -42,35 +42,33 @@ CPL_CVSID("$Id: ograssemblepolygon.cpp 27044 2014-03-16 23:41:27Z rouault $");
 /*      distance.  Update the current best distance if they are.        */
 /************************************************************************/
 
-static int CheckPoints( OGRLineString *poLine1, int iPoint1,
+static bool CheckPoints( OGRLineString *poLine1, int iPoint1,
                         OGRLineString *poLine2, int iPoint2,
                         double *pdfDistance )
 
 {
-    double      dfDeltaX, dfDeltaY, dfDistance;
-
     if( pdfDistance == NULL || *pdfDistance == 0 )
         return poLine1->getX(iPoint1) == poLine2->getX(iPoint2)
             && poLine1->getY(iPoint1) == poLine2->getY(iPoint2);
 
-    dfDeltaX = poLine1->getX(iPoint1) - poLine2->getX(iPoint2);
-    dfDeltaY = poLine1->getY(iPoint1) - poLine2->getY(iPoint2);
+    double dfDeltaX = poLine1->getX(iPoint1) - poLine2->getX(iPoint2);
+    double dfDeltaY = poLine1->getY(iPoint1) - poLine2->getY(iPoint2);
 
     dfDeltaX = ABS(dfDeltaX);
     dfDeltaY = ABS(dfDeltaY);
-    
+
     if( dfDeltaX > *pdfDistance || dfDeltaY > *pdfDistance )
-        return FALSE;
-    
-    dfDistance = sqrt(dfDeltaX*dfDeltaX + dfDeltaY*dfDeltaY);
+        return false;
+
+    double dfDistance = sqrt(dfDeltaX*dfDeltaX + dfDeltaY*dfDeltaY);
 
     if( dfDistance < *pdfDistance )
     {
         *pdfDistance = dfDistance;
-        return TRUE;
+        return true;
     }
     else
-        return FALSE;
+        return false;
 }
 
 /************************************************************************/
@@ -103,8 +101,8 @@ static void AddEdgeToRing( OGRLinearRing * poRing, OGRLineString * poLine,
 /* -------------------------------------------------------------------- */
 /*      Should we skip a repeating vertex?                              */
 /* -------------------------------------------------------------------- */
-    if( poRing->getNumPoints() > 0 
-        && CheckPoints( poRing, poRing->getNumPoints()-1, 
+    if( poRing->getNumPoints() > 0
+        && CheckPoints( poRing, poRing->getNumPoints()-1,
                         poLine, iStart, NULL ) )
     {
         iStart += iStep;
@@ -132,9 +130,9 @@ static void AddEdgeToRing( OGRLinearRing * poRing, OGRLineString * poLine,
  */
 
 OGRGeometryH OGRBuildPolygonFromEdges( OGRGeometryH hLines,
-                                       int bBestEffort, 
+                                       int bBestEffort,
                                        int bAutoClose,
-                                       double dfTolerance, 
+                                       double dfTolerance,
                                        OGRErr * peErr )
 
 {
@@ -144,7 +142,7 @@ OGRGeometryH OGRBuildPolygonFromEdges( OGRGeometryH hLines,
             *peErr = OGRERR_NONE;
         return NULL;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Check for the case of a geometrycollection that can be          */
 /*      promoted to MultiLineString.                                    */
@@ -178,7 +176,7 @@ OGRGeometryH OGRBuildPolygonFromEdges( OGRGeometryH hLines,
         return NULL;
     }
 
-    int         bSuccess = TRUE;
+    bool bSuccess = true;
     OGRGeometryCollection *poLines = (OGRGeometryCollection *) hLines;
     std::vector<OGRLinearRing*> aoRings;
 
@@ -200,7 +198,7 @@ OGRGeometryH OGRBuildPolygonFromEdges( OGRGeometryH hLines,
     {
         int             iEdge;
         OGRLineString   *poLine;
-        
+
 /* -------------------------------------------------------------------- */
 /*      Find the first unconsumed edge.                                 */
 /* -------------------------------------------------------------------- */
@@ -220,30 +218,31 @@ OGRGeometryH OGRBuildPolygonFromEdges( OGRGeometryH hLines,
 /*      Start a new ring, copying in the current line directly          */
 /* -------------------------------------------------------------------- */
         OGRLinearRing   *poRing = new OGRLinearRing();
-        
+
         AddEdgeToRing( poRing, poLine, FALSE );
 
 /* ==================================================================== */
 /*      Loop adding edges to this ring until we make a whole pass       */
 /*      within finding anything to add.                                 */
 /* ==================================================================== */
-        int             bWorkDone = TRUE;
+        bool            bWorkDone = true;
         double          dfBestDist = dfTolerance;
 
         while( !CheckPoints(poRing,0,poRing,poRing->getNumPoints()-1,NULL)
                && nRemainingEdges > 0
                && bWorkDone )
         {
-            int         iBestEdge = -1, bReverse = FALSE;
+            int         iBestEdge = -1;
+            int         bReverse = FALSE;
 
-            bWorkDone = FALSE;
+            bWorkDone = false;
             dfBestDist = dfTolerance;
 
             // We consider linking the end to the beginning.  If this is
             // closer than any other option we will just close the loop.
 
             //CheckPoints(poRing,0,poRing,poRing->getNumPoints()-1,&dfBestDist);
-            
+
             // Find unused edge with end point closest to our loose end.
             for( iEdge = 0; iEdge < nEdges; iEdge++ )
             {
@@ -275,14 +274,14 @@ OGRGeometryH OGRBuildPolygonFromEdges( OGRGeometryH hLines,
             // We found one within tolerance - add it.
             if( iBestEdge != -1 )
             {
-                poLine = (OGRLineString *) 
+                poLine = (OGRLineString *)
                     poLines->getGeometryRef(iBestEdge);
-                
+
                 AddEdgeToRing( poRing, poLine, bReverse );
-                    
+
                 panEdgeConsumed[iBestEdge] = TRUE;
                 nRemainingEdges--;
-                bWorkDone = TRUE;
+                bWorkDone = true;
             }
         }
 
@@ -294,15 +293,15 @@ OGRGeometryH OGRBuildPolygonFromEdges( OGRGeometryH hLines,
         if( !CheckPoints(poRing,0,poRing,poRing->getNumPoints()-1,
                          &dfBestDist) )
         {
-            CPLDebug( "OGR", 
+            CPLDebug( "OGR",
                       "Failed to close ring %d.\n"
                       "End Points are: (%.8f,%.7f) and (%.7f,%.7f)\n",
                       (int)aoRings.size(),
-                      poRing->getX(0), poRing->getY(0), 
-                      poRing->getX(poRing->getNumPoints()-1), 
+                      poRing->getX(0), poRing->getY(0),
+                      poRing->getX(poRing->getNumPoints()-1),
                       poRing->getY(poRing->getNumPoints()-1) );
 
-            bSuccess = FALSE;
+            bSuccess = false;
         }
 
 /* -------------------------------------------------------------------- */
@@ -311,8 +310,8 @@ OGRGeometryH OGRBuildPolygonFromEdges( OGRGeometryH hLines,
         if( bAutoClose
             && !CheckPoints(poRing,0,poRing,poRing->getNumPoints()-1,NULL) )
         {
-            poRing->addPoint( poRing->getX(0), 
-                              poRing->getY(0), 
+            poRing->addPoint( poRing->getX(0),
+                              poRing->getY(0),
                               poRing->getZ(0));
         }
 
@@ -361,9 +360,6 @@ OGRGeometryH OGRBuildPolygonFromEdges( OGRGeometryH hLines,
         else
             *peErr = OGRERR_FAILURE;
     }
-    
+
     return (OGRGeometryH) poPolygon;
 }
-
-
-
