@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ograrcgendriver.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: ograrcgendriver.cpp 32119 2015-12-11 05:47:58Z goatbar $
  *
  * Project:  Arc/Info Generate Translator
  * Purpose:  Implements OGRARCGENDriver.
@@ -30,39 +30,73 @@
 #include "ogr_arcgen.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ograrcgendriver.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: ograrcgendriver.cpp 32119 2015-12-11 05:47:58Z goatbar $");
 
 extern "C" void RegisterOGRARCGEN();
 
-/************************************************************************/
-/*                           ~OGRARCGENDriver()                            */
-/************************************************************************/
-
-OGRARCGENDriver::~OGRARCGENDriver()
-
-{
-}
-
-/************************************************************************/
-/*                              GetName()                               */
-/************************************************************************/
-
-const char *OGRARCGENDriver::GetName()
-
-{
-    return "ARCGEN";
-}
 
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
 
-OGRDataSource *OGRARCGENDriver::Open( const char * pszFilename, int bUpdate )
+static GDALDataset *OGRARCGENDriverOpen( GDALOpenInfo* poOpenInfo )
 
 {
-    OGRARCGENDataSource   *poDS = new OGRARCGENDataSource();
+    if( poOpenInfo->eAccess == GA_Update ||
+        poOpenInfo->fpL == NULL )
+    {
+        return NULL;
+    }
 
-    if( !poDS->Open( pszFilename, bUpdate ) )
+    /* Check that the first line is compatible with a generate file */
+    /* and in particular contain >= 32 && <= 127 bytes */
+    bool bFoundEOL = false;
+    char* szFirstLine
+        = CPLStrdup(reinterpret_cast<char *>( poOpenInfo->pabyHeader ) );
+    for(int i=0;szFirstLine[i] != '\0';i++)
+    {
+        if (szFirstLine[i] == '\n' || szFirstLine[i] == '\r')
+        {
+            bFoundEOL = true;
+            szFirstLine[i] = '\0';
+            break;
+        }
+        if (szFirstLine[i] < 32)
+        {
+            CPLFree(szFirstLine);
+            return NULL;
+        }
+    }
+
+    if (!bFoundEOL)
+    {
+        CPLFree(szFirstLine);
+        return NULL;
+    }
+
+    char** papszTokens = CSLTokenizeString2( szFirstLine, " ,", 0 );
+    const int nTokens = CSLCount(papszTokens);
+    if (nTokens != 1 && nTokens != 3 && nTokens != 4)
+    {
+        CSLDestroy(papszTokens);
+        CPLFree(szFirstLine);
+        return NULL;
+    }
+    for(int i=0;i<nTokens;i++)
+    {
+        if( CPLGetValueType(papszTokens[i]) == CPL_VALUE_STRING )
+        {
+            CSLDestroy(papszTokens);
+            CPLFree(szFirstLine);
+            return NULL;
+        }
+    }
+    CSLDestroy(papszTokens);
+    CPLFree(szFirstLine);
+
+    OGRARCGENDataSource *poDS = new OGRARCGENDataSource();
+
+    if( !poDS->Open( poOpenInfo->pszFilename ) )
     {
         delete poDS;
         poDS = NULL;
@@ -71,15 +105,6 @@ OGRDataSource *OGRARCGENDriver::Open( const char * pszFilename, int bUpdate )
     return poDS;
 }
 
-/************************************************************************/
-/*                           TestCapability()                           */
-/************************************************************************/
-
-int OGRARCGENDriver::TestCapability( CPL_UNUSED const char * pszCap )
-{
-    return FALSE;
-}
-
 /************************************************************************/
 /*                           RegisterOGRARCGEN()                           */
 /************************************************************************/
@@ -87,6 +112,18 @@ int OGRARCGENDriver::TestCapability( CPL_UNUSED const char * pszCap )
 void RegisterOGRARCGEN()
 
 {
-    OGRSFDriverRegistrar::GetRegistrar()->RegisterDriver( new OGRARCGENDriver );
+    if( GDALGetDriverByName( "ARCGEN" ) != NULL )
+        return;
+
+    GDALDriver  *poDriver = new GDALDriver();
+
+    poDriver->SetDescription( "ARCGEN" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VECTOR, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "Arc/Info Generate" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drv_arcgen.html" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+    poDriver->pfnOpen = OGRARCGENDriverOpen;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
 
