@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ograrcgenlayer.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: ograrcgenlayer.cpp 32330 2015-12-20 14:24:33Z rouault $
  *
  * Project:  Arc/Info Generate Translator
  * Purpose:  Implements OGRARCGENLayer class.
@@ -33,26 +33,26 @@
 #include "ogr_p.h"
 #include "ogr_srs_api.h"
 
-CPL_CVSID("$Id: ograrcgenlayer.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: ograrcgenlayer.cpp 32330 2015-12-20 14:24:33Z rouault $");
 
 /************************************************************************/
 /*                            OGRARCGENLayer()                             */
 /************************************************************************/
 
 OGRARCGENLayer::OGRARCGENLayer( const char* pszFilename,
-                          VSILFILE* fp, OGRwkbGeometryType eType )
-
+                                VSILFILE* fpIn, OGRwkbGeometryType eType ) :
+    poFeatureDefn(NULL),
+    fp(fpIn),
+    bEOF(FALSE),
+    nNextFID(0)
 {
-    this->fp = fp;
-    nNextFID = 0;
-    bEOF = FALSE;
-
     poFeatureDefn = new OGRFeatureDefn( CPLGetBasename(pszFilename) );
     poFeatureDefn->Reference();
     poFeatureDefn->SetGeomType( eType );
 
-    OGRFieldDefn    oField1( "ID", OFTInteger);
+    OGRFieldDefn oField1( "ID", OFTInteger);
     poFeatureDefn->AddFieldDefn( &oField1 );
+    SetDescription( poFeatureDefn->GetName() );
 }
 
 /************************************************************************/
@@ -87,11 +87,9 @@ void OGRARCGENLayer::ResetReading()
 
 OGRFeature *OGRARCGENLayer::GetNextFeature()
 {
-    OGRFeature  *poFeature;
-
-    while(TRUE)
+    while( true )
     {
-        poFeature = GetNextRawFeature();
+        OGRFeature *poFeature = GetNextRawFeature();
         if (poFeature == NULL)
             return NULL;
 
@@ -102,8 +100,8 @@ OGRFeature *OGRARCGENLayer::GetNextFeature()
         {
             return poFeature;
         }
-        else
-            delete poFeature;
+
+        delete poFeature;
     }
 }
 
@@ -116,14 +114,13 @@ OGRFeature *OGRARCGENLayer::GetNextRawFeature()
     if (bEOF)
         return NULL;
 
-    const char* pszLine;
     OGRwkbGeometryType eType = poFeatureDefn->GetGeomType();
 
     if (wkbFlatten(eType) == wkbPoint)
     {
-        while(TRUE)
+        while( true )
         {
-            pszLine = CPLReadLine2L(fp,256,NULL);
+            const char* pszLine = CPLReadLine2L(fp,256,NULL);
             if (pszLine == NULL || EQUAL(pszLine, "END"))
             {
                 bEOF = TRUE;
@@ -138,29 +135,28 @@ OGRFeature *OGRARCGENLayer::GetNextRawFeature()
                 poFeature->SetField(0, papszTokens[0]);
                 if (nTokens == 3)
                     poFeature->SetGeometryDirectly(
-                        new OGRPoint(atof(papszTokens[1]),
-                                     atof(papszTokens[2])));
+                        new OGRPoint(CPLAtof(papszTokens[1]),
+                                     CPLAtof(papszTokens[2])));
                 else
                     poFeature->SetGeometryDirectly(
-                        new OGRPoint(atof(papszTokens[1]),
-                                     atof(papszTokens[2]),
-                                     atof(papszTokens[3])));
+                        new OGRPoint(CPLAtof(papszTokens[1]),
+                                     CPLAtof(papszTokens[2]),
+                                     CPLAtof(papszTokens[3])));
                 CSLDestroy(papszTokens);
                 return poFeature;
             }
-            else
-                CSLDestroy(papszTokens);
+
+            CSLDestroy(papszTokens);
         }
     }
 
     CPLString osID;
-    OGRLinearRing* poLR =
-        (wkbFlatten(eType) == wkbPolygon) ? new OGRLinearRing() : NULL;
-    OGRLineString* poLS =
-        (wkbFlatten(eType) == wkbLineString) ? new OGRLineString() : poLR;
-    while(TRUE)
+    const bool bIsPoly = (wkbFlatten(eType) == wkbPolygon);
+    OGRLineString* poLS = static_cast<OGRLineString*>(
+        OGRGeometryFactory::createGeometry( (bIsPoly) ? wkbLinearRing : wkbLineString ));
+    while( true )
     {
-        pszLine = CPLReadLine2L(fp,256,NULL);
+        const char* pszLine = CPLReadLine2L(fp,256,NULL);
         if (pszLine == NULL)
             break;
 
@@ -172,10 +168,10 @@ OGRFeature *OGRARCGENLayer::GetNextRawFeature()
             OGRFeature* poFeature = new OGRFeature(poFeatureDefn);
             poFeature->SetFID(nNextFID ++);
             poFeature->SetField(0, osID.c_str());
-            if (wkbFlatten(eType) == wkbPolygon)
+            if( bIsPoly )
             {
                 OGRPolygon* poPoly = new OGRPolygon();
-                poPoly->addRingDirectly(poLR);
+                poPoly->addRingDirectly((OGRLinearRing*)poLS);
                 poFeature->SetGeometryDirectly(poPoly);
             }
             else
@@ -199,14 +195,14 @@ OGRFeature *OGRARCGENLayer::GetNextRawFeature()
         {
             if (nTokens == 2)
             {
-                poLS->addPoint(atof(papszTokens[0]),
-                               atof(papszTokens[1]));
+                poLS->addPoint(CPLAtof(papszTokens[0]),
+                               CPLAtof(papszTokens[1]));
             }
             else if (nTokens == 3)
             {
-                poLS->addPoint(atof(papszTokens[0]),
-                               atof(papszTokens[1]),
-                               atof(papszTokens[2]));
+                poLS->addPoint(CPLAtof(papszTokens[0]),
+                               CPLAtof(papszTokens[1]),
+                               CPLAtof(papszTokens[2]));
             }
             else
             {
@@ -225,8 +221,7 @@ OGRFeature *OGRARCGENLayer::GetNextRawFeature()
 /*                           TestCapability()                           */
 /************************************************************************/
 
-int OGRARCGENLayer::TestCapability( CPL_UNUSED const char * pszCap )
+int OGRARCGENLayer::TestCapability( const char * /* pszCap */ )
 {
     return FALSE;
 }
-
