@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: avc.h,v 1.22 2006/06/27 18:38:43 dmorissette Exp $
+ * $Id: avc.h,v 1.25 2008/07/23 20:51:38 dmorissette Exp $
  *
  * Name:     avc.h
  * Project:  Arc/Info Vector coverage (AVC) BIN<->E00 conversion library
@@ -16,20 +16,31 @@
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  *
  * $Log: avc.h,v $
+ * Revision 1.25  2008/07/23 20:51:38  dmorissette
+ * Fixed GCC 4.1.x compile warnings related to use of char vs unsigned char
+ * (GDAL/OGR ticket http://trac.osgeo.org/gdal/ticket/2495)
+ *
+ * Revision 1.24  2006/08/17 20:09:45  dmorissette
+ * Update for 2.0.0 release
+ *
+ * Revision 1.23  2006/08/17 18:56:42  dmorissette
+ * Support for reading standalone info tables (just tables, no coverage
+ * data) by pointing AVCE00ReadOpen() to the info directory (bug 1549).
+ *
  * Revision 1.22  2006/06/27 18:38:43  dmorissette
  * Cleaned up E00 reading (bug 1497, patch from James F.)
  *
@@ -101,13 +112,18 @@
  *
  **********************************************************************/
 
-#ifndef _AVC_H_INCLUDED_
-#define _AVC_H_INCLUDED_
+#ifndef AVC_H_INCLUDED_
+#define AVC_H_INCLUDED_
 
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
+#ifdef GDAL_COMPILATION
+#include "shapefil.h"
+#else
 #include "dbfopen.h"
+#endif
+
 #include "avc_mbyte.h"
 
 CPL_C_START
@@ -115,7 +131,7 @@ CPL_C_START
 /*---------------------------------------------------------------------
  * Current version of the AVCE00 library... always useful!
  *--------------------------------------------------------------------*/
-#define AVC_VERSION "2.0.0-dev (2006-06-16)"
+#define AVC_VERSION "2.0.0 (2006-08-17)"
 
 
 /* Coverage precision
@@ -165,7 +181,8 @@ typedef enum
     AVCCoverV7,
     AVCCoverPC,
     AVCCoverPC2,   /* Unknown version... hybrid between V7 and PC !!! */
-    AVCCoverWeird  /* Unknown version... hybrid between V7 and PC !!! */
+    AVCCoverWeird, /* Unknown version... hybrid between V7 and PC !!! */
+    AVCCoverV7Tables /* Standalone tables, only an info directory     */
 } AVCCoverType;
 
 /* Enum for byte ordering
@@ -211,11 +228,11 @@ typedef struct AVCArc_t
     GInt32      nLPoly;
     GInt32      nRPoly;
     GInt32      numVertices;
-    AVCVertex   *pasVertices;   
+    AVCVertex   *pasVertices;
 }AVCArc;
 
 /*---------------------------------------------------------------------
- * AVCPal: A PAL (Polygon Arc List) references all the arcs that 
+ * AVCPal: A PAL (Polygon Arc List) references all the arcs that
  *         constitute a polygon.
  *--------------------------------------------------------------------*/
 typedef struct AVCPalArc_t
@@ -290,7 +307,7 @@ typedef struct AVCTxt_t
     double      dV2;    /* ??? */
     double      dV3;    /* ??? */
 
-    char        *pszText;
+    GByte       *pszText; /* Needs to be unsigned char for DBCS */
 
     AVCVertex   *pasVertices;
 }AVCTxt;
@@ -306,8 +323,8 @@ typedef struct AVCRxp_t
 
 
 /*---------------------------------------------------------------------
- * AVCTableDef: Definition of an INFO table's structure.  
- *               This info is read from several files: 
+ * AVCTableDef: Definition of an INFO table's structure.
+ *               This info is read from several files:
  *                   info/arc.dir
  *                   info/arc####.dat
  *                   info/arc####.nit
@@ -346,7 +363,7 @@ typedef struct AVCFieldInfo_t
 
 typedef struct AVCTableDef_t
 {
-    /* Stuff read from the arc.dir file 
+    /* Stuff read from the arc.dir file
      * (1 record, corresponding to this table, from the arc.dir file)
      */
     char        szTableName[33];
@@ -373,7 +390,7 @@ typedef struct AVCField_t
     GInt32      nInt32;
     float       fFloat;
     double      dDouble;
-    char        *pszStr;
+    GByte       *pszStr;
 }AVCField;
 
 /*---------------------------------------------------------------------
@@ -433,7 +450,7 @@ typedef struct AVCBinFile_t
         AVCTableDef  *psTableDef;
     }hdr;
 
-    /* cur.* : temp. storage used to read one record (ARC, PAL, ... or 
+    /* cur.* : temp. storage used to read one record (ARC, PAL, ... or
      *         Table record) from the file.
      */
     union
@@ -466,7 +483,7 @@ typedef struct AVCE00GenInfo_t
 {
     char        *pszBuf;
     int         nBufSize;
-    
+
     int         nPrecision;     /* AVC_SINGLE/DOUBLE_PREC       */
     int         iCurItem;
     int         numItems;
@@ -505,7 +522,7 @@ typedef struct AVCE00ParseInfo_t
                                      /* finished parsing */
     int         nTableE00RecLength;
 
-    /* cur.* : temp. storage used to store current object (ARC, PAL, ... or 
+    /* cur.* : temp. storage used to store current object (ARC, PAL, ... or
      *         Table record) from the file.
      */
     union
@@ -546,7 +563,7 @@ typedef struct AVCE00ReadInfo_t
     AVCCoverType eCoverType;
 
     /* pasSections is built when the coverage is opened and describes
-     * the squeleton of the E00 file.
+     * the skeleton of the E00 file.
      */
     AVCE00Section *pasSections;
     int            numSections;
@@ -555,7 +572,7 @@ typedef struct AVCE00ReadInfo_t
      * the next section when a section finishes. (This is the default)
      * Otherwise, you can use AVCE00ReadGotoSection() to read one section
      * at a time... this will set bReadAllSections=FALSE.
-     */ 
+     */
     GBool         bReadAllSections;
 
     /* Info about the file (or E00 section) currently being processed
@@ -590,7 +607,7 @@ typedef struct AVCE00ReadInfoE00_t
      * the next section when a section finishes. (This is the default)
      * Otherwise, you can use AVCE00ReadGotoSectionE00() to read one
      * section at a time.
-     */ 
+     */
     GBool         bReadAllSections;
 
     /* File handle of the E00 file currently being processed
@@ -658,30 +675,30 @@ void        AVCRawBinFSeek(AVCRawBinFile *psInfo, int nOffset, int nFrom);
 GBool       AVCRawBinEOF(AVCRawBinFile *psInfo);
 void        AVCRawBinSetFileDataSize(AVCRawBinFile *psInfo, int nDataSize);
 
-void        AVCRawBinReadBytes(AVCRawBinFile *psInfo, int nBytesToRead, 
+void        AVCRawBinReadBytes(AVCRawBinFile *psInfo, int nBytesToRead,
                                GByte *pBuf);
 GInt16      AVCRawBinReadInt16(AVCRawBinFile *psInfo);
 GInt32      AVCRawBinReadInt32(AVCRawBinFile *psInfo);
 float       AVCRawBinReadFloat(AVCRawBinFile *psInfo);
 double      AVCRawBinReadDouble(AVCRawBinFile *psInfo);
-void        AVCRawBinReadString(AVCRawBinFile *psFile, int nBytesToRead, 
+void        AVCRawBinReadString(AVCRawBinFile *psFile, int nBytesToRead,
                                 GByte *pBuf);
 
 void        AVCRawBinWriteBytes(AVCRawBinFile *psFile, int nBytesToWrite,
-                                GByte *pBuf);
+                                const GByte *pBuf);
 void        AVCRawBinWriteInt16(AVCRawBinFile *psFile, GInt16 n16Value);
 void        AVCRawBinWriteInt32(AVCRawBinFile *psFile, GInt32 n32Value);
 void        AVCRawBinWriteFloat(AVCRawBinFile *psFile, float fValue);
 void        AVCRawBinWriteDouble(AVCRawBinFile *psFile, double dValue);
 void        AVCRawBinWriteZeros(AVCRawBinFile *psFile, int nBytesToWrite);
-void        AVCRawBinWritePaddedString(AVCRawBinFile *psFile, int nFieldSize, 
-                                       const char *pszString);
+void        AVCRawBinWritePaddedString(AVCRawBinFile *psFile, int nFieldSize,
+                                       const GByte *pszString);
 
 /*---------------------------------------------------------------------
  * Functions related to reading the binary coverage files
  *--------------------------------------------------------------------*/
 
-AVCBinFile *AVCBinReadOpen(const char *pszPath, const char *pszName, 
+AVCBinFile *AVCBinReadOpen(const char *pszPath, const char *pszName,
                            AVCCoverType eCoverType, AVCFileType eType,
                            AVCDBCSInfo *psDBCSInfo);
 void        AVCBinReadClose(AVCBinFile *psFile);
@@ -700,7 +717,7 @@ AVCRxp     *AVCBinReadNextRxp(AVCBinFile *psFile);
 AVCField   *AVCBinReadNextTableRec(AVCBinFile *psFile);
 char      **AVCBinReadNextPrj(AVCBinFile *psFile);
 
-char      **AVCBinReadListTables(const char *pszInfoPath, 
+char      **AVCBinReadListTables(const char *pszInfoPath,
                                  const char *pszCoverName,
                                  char ***ppapszArcDatFiles,
                                  AVCCoverType eCoverType,
@@ -709,11 +726,11 @@ char      **AVCBinReadListTables(const char *pszInfoPath,
 /*---------------------------------------------------------------------
  * Functions related to writing the binary coverage files
  *--------------------------------------------------------------------*/
-AVCBinFile *AVCBinWriteCreate(const char *pszPath, const char *pszName, 
+AVCBinFile *AVCBinWriteCreate(const char *pszPath, const char *pszName,
                               AVCCoverType eCoverType,
                               AVCFileType eType, int nPrecision,
                               AVCDBCSInfo *psDBCSInfo);
-AVCBinFile *AVCBinWriteCreateTable(const char *pszInfoPath, 
+AVCBinFile *AVCBinWriteCreateTable(const char *pszInfoPath,
                                    const char *pszCoverName,
                                    AVCTableDef *psSrcTableDef,
                                    AVCCoverType eCoverType,
@@ -739,12 +756,12 @@ AVCE00GenInfo  *AVCE00GenInfoAlloc(int nCoverPrecision);
 void        AVCE00GenInfoFree(AVCE00GenInfo *psInfo);
 void        AVCE00GenReset(AVCE00GenInfo  *psInfo);
 
-const char *AVCE00GenStartSection(AVCE00GenInfo *psInfo, AVCFileType eType, 
+const char *AVCE00GenStartSection(AVCE00GenInfo *psInfo, AVCFileType eType,
                                   const char *pszFilename);
 const char *AVCE00GenEndSection(AVCE00GenInfo *psInfo, AVCFileType eType,
                                 GBool bCont);
 
-const char *AVCE00GenObject(AVCE00GenInfo *psInfo, 
+const char *AVCE00GenObject(AVCE00GenInfo *psInfo,
                             AVCFileType eType, void *psObj, GBool bCont);
 const char *AVCE00GenArc(AVCE00GenInfo *psInfo, AVCArc *psArc, GBool bCont);
 const char *AVCE00GenPal(AVCE00GenInfo *psInfo, AVCPal *psPal, GBool bCont);
@@ -765,11 +782,11 @@ const char *AVCE00GenTableRec(AVCE00GenInfo *psInfo, int numFields,
 /*---------------------------------------------------------------------
  * Functions related to parsing E00 lines
  *--------------------------------------------------------------------*/
-AVCE00ParseInfo  *AVCE00ParseInfoAlloc();
+AVCE00ParseInfo  *AVCE00ParseInfoAlloc(void);
 void    AVCE00ParseInfoFree(AVCE00ParseInfo *psInfo);
 void    AVCE00ParseReset(AVCE00ParseInfo  *psInfo);
 
-AVCFileType AVCE00ParseSectionHeader(AVCE00ParseInfo  *psInfo, 
+AVCFileType AVCE00ParseSectionHeader(AVCE00ParseInfo  *psInfo,
                                      const char *pszLine);
 GBool   AVCE00ParseSectionEnd(AVCE00ParseInfo  *psInfo, const char *pszLine,
                               GBool bResetParseInfo);
@@ -788,16 +805,16 @@ AVCTxt  *AVCE00ParseNextTxtLine(AVCE00ParseInfo *psInfo, const char *pszLine);
 AVCTxt  *AVCE00ParseNextTx6Line(AVCE00ParseInfo *psInfo, const char *pszLine);
 char   **AVCE00ParseNextPrjLine(AVCE00ParseInfo *psInfo, const char *pszLine);
 AVCRxp  *AVCE00ParseNextRxpLine(AVCE00ParseInfo *psInfo, const char *pszLine);
-AVCTableDef *AVCE00ParseNextTableDefLine(AVCE00ParseInfo *psInfo, 
+AVCTableDef *AVCE00ParseNextTableDefLine(AVCE00ParseInfo *psInfo,
                                          const char *pszLine);
-AVCField    *AVCE00ParseNextTableRecLine(AVCE00ParseInfo *psInfo, 
+AVCField    *AVCE00ParseNextTableRecLine(AVCE00ParseInfo *psInfo,
                                          const char *pszLine);
 
 
 /*---------------------------------------------------------------------
  * Misc. functions shared by several parts of the lib.
  *--------------------------------------------------------------------*/
-int _AVCE00ComputeRecSize(int numFields, AVCFieldInfo *pasDef, 
+int _AVCE00ComputeRecSize(int numFields, AVCFieldInfo *pasDef,
                           GBool bMapType40ToDouble);
 
 void _AVCDestroyTableFields(AVCTableDef *psTableDef, AVCField *pasFields);
@@ -806,7 +823,7 @@ AVCTableDef *_AVCDupTableDef(AVCTableDef *psSrcDef);
 
 GBool AVCFileExists(const char *pszPath, const char *pszName);
 char *AVCAdjustCaseSensitiveFilename(char *pszFname);
-int  AVCPrintRealValue(char *pszBuf, int nPrecision, AVCFileType eType,
+int  AVCPrintRealValue(char *pszBuf, size_t nBufLen, int nPrecision, AVCFileType eType,
                        double dValue);
 
 /*=====================================================================
@@ -822,7 +839,7 @@ int              AVCE00ReadRewindE00(AVCE00ReadE00Ptr psRead);
 void            *AVCE00ReadNextObjectE00(AVCE00ReadE00Ptr psRead);
 
 AVCE00Section  *AVCE00ReadSectionsListE00(AVCE00ReadE00Ptr psRead, int *numSect);
-int             AVCE00ReadGotoSectionE00(AVCE00ReadE00Ptr psRead, 
+int             AVCE00ReadGotoSectionE00(AVCE00ReadE00Ptr psRead,
                                          AVCE00Section *psSect,
                                          GBool bContinue);
 
@@ -836,7 +853,7 @@ const char     *AVCE00ReadNextLine(AVCE00ReadPtr psInfo);
 int             AVCE00ReadRewind(AVCE00ReadPtr psInfo);
 
 AVCE00Section  *AVCE00ReadSectionsList(AVCE00ReadPtr psInfo, int *numSect);
-int             AVCE00ReadGotoSection(AVCE00ReadPtr psInfo, 
+int             AVCE00ReadGotoSection(AVCE00ReadPtr psInfo,
                                       AVCE00Section *psSect,
                                       GBool bContinue);
 
@@ -844,15 +861,13 @@ int             AVCE00ReadGotoSection(AVCE00ReadPtr psInfo,
  * Functions to write E00 lines to a binary coverage
  *--------------------------------------------------------------------*/
 
-AVCE00WritePtr  AVCE00WriteOpen(const char *pszCoverPath, 
+AVCE00WritePtr  AVCE00WriteOpen(const char *pszCoverPath,
                                 AVCCoverType eNewCoverType, int nPrecision);
 void            AVCE00WriteClose(AVCE00WritePtr psInfo);
-int             AVCE00WriteNextLine(AVCE00WritePtr psInfo, 
+int             AVCE00WriteNextLine(AVCE00WritePtr psInfo,
                                     const char *pszLine);
 int             AVCE00DeleteCoverage(const char *pszCoverPath);
 
 CPL_C_END
 
-#endif /* _AVC_H_INCLUDED_ */
-
-
+#endif /* AVC_H_INCLUDED_ */
