@@ -16,16 +16,16 @@
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  *
@@ -80,7 +80,7 @@
 
 /*---------------------------------------------------------------------
  * Define a static flag and set it with the byte ordering on this machine
- * we will then compare with this value to decide if we nned to swap
+ * we will then compare with this value to decide if we need to swap
  * bytes or not.
  *
  * CPL_MSB or CPL_LSB should be set in the makefile... the default is
@@ -104,7 +104,7 @@ static AVCByteOrder geSystemByteOrder = AVCLittleEndian;
  * Returns a valid AVCRawBinFile structure, or NULL if the file could
  * not be opened or created.
  *
- * AVCRawBinClose() will eventually have to be called to release the 
+ * AVCRawBinClose() will eventually have to be called to release the
  * resources used by the AVCRawBinFile structure.
  **********************************************************************/
 AVCRawBinFile *AVCRawBinOpen(const char *pszFname, const char *pszAccess,
@@ -116,29 +116,29 @@ AVCRawBinFile *AVCRawBinOpen(const char *pszFname, const char *pszAccess,
     psFile = (AVCRawBinFile*)CPLCalloc(1, sizeof(AVCRawBinFile));
 
     /*-----------------------------------------------------------------
-     * Validate access mode and open/create file.  
+     * Validate access mode and open/create file.
      * For now we support only: "r" for read-only or "w" for write-only
      * or "a" for append.
      *
      * A case for "r+" is included here, but random access is not
      * properly supported yet... so this option should be used with care.
      *----------------------------------------------------------------*/
-    if (EQUALN(pszAccess, "r+", 2))
+    if (STARTS_WITH_CI(pszAccess, "r+"))
     {
         psFile->eAccess = AVCReadWrite;
         psFile->fp = VSIFOpen(pszFname, "r+b");
     }
-    else if (EQUALN(pszAccess, "r", 1))
+    else if (STARTS_WITH_CI(pszAccess, "r"))
     {
         psFile->eAccess = AVCRead;
         psFile->fp = VSIFOpen(pszFname, "rb");
     }
-    else if (EQUALN(pszAccess, "w", 1))
+    else if (STARTS_WITH_CI(pszAccess, "w"))
     {
         psFile->eAccess = AVCWrite;
         psFile->fp = VSIFOpen(pszFname, "wb");
     }
-    else if (EQUALN(pszAccess, "a", 1))
+    else if (STARTS_WITH_CI(pszAccess, "a"))
     {
         psFile->eAccess = AVCWrite;
         psFile->fp = VSIFOpen(pszFname, "ab");
@@ -146,13 +146,13 @@ AVCRawBinFile *AVCRawBinOpen(const char *pszFname, const char *pszAccess,
     else
     {
         CPLError(CE_Failure, CPLE_IllegalArg,
-                 "Acces mode \"%s\" not supported.", pszAccess);
+                 "Access mode \"%s\" not supported.", pszAccess);
         CPLFree(psFile);
         return NULL;
     }
 
     /*-----------------------------------------------------------------
-     * Check that file was opened succesfully, and init struct.
+     * Check that file was opened successfully, and init struct.
      *----------------------------------------------------------------*/
     if (psFile->fp == NULL)
     {
@@ -206,7 +206,7 @@ void AVCRawBinClose(AVCRawBinFile *psFile)
  * PC Arc/Info where the physical file size is always a multiple of
  * 256 bytes padded with some junk at the end.
  *
- * The default value is -1 which just looks for the real EOF. 
+ * The default value is -1 which just looks for the real EOF.
  **********************************************************************/
 void AVCRawBinSetFileDataSize(AVCRawBinFile *psFile, int nFileDataSize)
 {
@@ -219,7 +219,7 @@ void AVCRawBinSetFileDataSize(AVCRawBinFile *psFile, int nFileDataSize)
 /**********************************************************************
  *                          AVCRawBinReadBytes()
  *
- * Copy the number of bytes from the input file to the specified 
+ * Copy the number of bytes from the input file to the specified
  * memory location.
  **********************************************************************/
 static GBool bDisableReadBytesEOFError = FALSE;
@@ -230,7 +230,7 @@ void AVCRawBinReadBytes(AVCRawBinFile *psFile, int nBytesToRead, GByte *pBuf)
 
     /* Make sure file is opened with Read access
      */
-    if (psFile == NULL || 
+    if (psFile == NULL ||
         (psFile->eAccess != AVCRead && psFile->eAccess != AVCReadWrite))
     {
         CPLError(CE_Failure, CPLE_FileIO,
@@ -248,7 +248,7 @@ void AVCRawBinReadBytes(AVCRawBinFile *psFile, int nBytesToRead, GByte *pBuf)
         return;
     }
 
-    /* This is the long method... it supports reading data that 
+    /* This is the long method... it supports reading data that
      * overlaps the input buffer boundaries.
      */
     while(nBytesToRead > 0)
@@ -260,7 +260,7 @@ void AVCRawBinReadBytes(AVCRawBinFile *psFile, int nBytesToRead, GByte *pBuf)
         if (psFile->nCurPos == psFile->nCurSize)
         {
             psFile->nOffset += psFile->nCurSize;
-            psFile->nCurSize = VSIFRead(psFile->abyBuf, sizeof(GByte),
+            psFile->nCurSize = (int)VSIFRead(psFile->abyBuf, sizeof(GByte),
                                         AVCRAWBIN_READBUFSIZE, psFile->fp);
             psFile->nCurPos = 0;
         }
@@ -270,7 +270,7 @@ void AVCRawBinReadBytes(AVCRawBinFile *psFile, int nBytesToRead, GByte *pBuf)
             /* Attempt to read past EOF... generate an error.
              *
              * Note: AVCRawBinEOF() can set bDisableReadBytesEOFError=TRUE
-             *       to disable the error message whils it is testing
+             *       to disable the error message while it is testing
              *       for EOF.
              *
              * TODO: We are not resetting the buffer. Also, there is no easy
@@ -286,7 +286,7 @@ void AVCRawBinReadBytes(AVCRawBinFile *psFile, int nBytesToRead, GByte *pBuf)
         }
 
         /* If the requested bytes are not all in the current buffer then
-         * just read the part that's in memory for now... the loop will 
+         * just read the part that's in memory for now... the loop will
          * take care of the rest.
          */
         if (psFile->nCurPos + nBytesToRead > psFile->nCurSize)
@@ -300,7 +300,7 @@ void AVCRawBinReadBytes(AVCRawBinFile *psFile, int nBytesToRead, GByte *pBuf)
         }
         else
         {
-            /* All the requested bytes are now in the buffer... 
+            /* All the requested bytes are now in the buffer...
              * simply copy them and return.
              */
             memcpy(pBuf, psFile->abyBuf+psFile->nCurPos, nBytesToRead);
@@ -328,7 +328,7 @@ void AVCRawBinReadString(AVCRawBinFile *psFile, int nBytesToRead, GByte *pBuf)
     pBuf[nBytesToRead] = '\0';
 
     pszConvBuf = AVCE00ConvertFromArcDBCS(psFile->psDBCSInfo,
-                                          pBuf, 
+                                          pBuf,
                                           nBytesToRead);
 
     if (pszConvBuf != pBuf)
@@ -342,7 +342,7 @@ void AVCRawBinReadString(AVCRawBinFile *psFile, int nBytesToRead, GByte *pBuf)
  *
  * Move the read pointer to the specified location.
  *
- * As with fseek(), the specified position can be relative to the 
+ * As with fseek(), the specified position can be relative to the
  * beginning of the file (SEEK_SET), or the current position (SEEK_CUR).
  * SEEK_END is not supported.
  **********************************************************************/
@@ -358,7 +358,7 @@ void AVCRawBinFSeek(AVCRawBinFile *psFile, int nOffset, int nFrom)
     if (psFile == NULL || psFile->eAccess == AVCWrite)
         return;
 
-    /* Compute destination relative to current memory buffer 
+    /* Compute destination relative to current memory buffer
      */
     if (nFrom == SEEK_SET)
         nTarget = nOffset - psFile->nOffset;
@@ -369,7 +369,7 @@ void AVCRawBinFSeek(AVCRawBinFile *psFile, int nOffset, int nFrom)
      */
     if (nTarget > 0 && nTarget <= psFile->nCurSize)
     {
-        /* Requested location is already in memory... just move the 
+        /* Requested location is already in memory... just move the
          * read pointer
          */
         psFile->nCurPos = nTarget;
@@ -377,13 +377,14 @@ void AVCRawBinFSeek(AVCRawBinFile *psFile, int nOffset, int nFrom)
     else
     {
         /* Requested location is not part of the memory buffer...
-         * move the FILE * to the right location and be ready to 
+         * move the FILE * to the right location and be ready to
          * read from there.
          */
-        VSIFSeek(psFile->fp, psFile->nOffset+nTarget, SEEK_SET);
         psFile->nCurPos = 0;
         psFile->nCurSize = 0;
         psFile->nOffset = psFile->nOffset+nTarget;
+        if( VSIFSeek(psFile->fp, psFile->nOffset+nTarget, SEEK_SET) < 0 )
+            return;
     }
 
 }
@@ -405,7 +406,7 @@ GBool AVCRawBinEOF(AVCRawBinFile *psFile)
     if (psFile->eAccess != AVCRead && psFile->eAccess != AVCReadWrite)
         return TRUE;
 
-    /* If file data size was specified, then check that we have not 
+    /* If file data size was specified, then check that we have not
      * passed that point yet...
      */
     if (psFile->nFileDataSize > 0 &&
@@ -414,14 +415,14 @@ GBool AVCRawBinEOF(AVCRawBinFile *psFile)
 
     /* If the file pointer has been moved by AVCRawBinFSeek(), then
      * we may be at a position past EOF, but VSIFeof() would still
-     * return FALSE. It also returns false if we have read just upto
+     * return FALSE. It also returns false if we have read just up to
      * the end of the file. EOF marker would not have been set unless
      * we try to read past that.
-     * 
+     *
      * To prevent this situation, if the memory buffer is empty,
      * we will try to read 1 byte from the file to force the next
-     * chunk of data to be loaded (and we'll move the the read pointer
-     * back by 1 char after of course!).  
+     * chunk of data to be loaded (and we'll move the read pointer
+     * back by 1 char after of course!).
      * If we are at the end of the file, this will trigger the EOF flag.
      */
     if ((psFile->nCurPos == 0 && psFile->nCurSize == 0) ||
@@ -429,7 +430,7 @@ GBool AVCRawBinEOF(AVCRawBinFile *psFile)
          psFile->nCurSize == AVCRAWBIN_READBUFSIZE))
     {
         GByte c;
-        /* Set bDisableReadBytesEOFError=TRUE to temporarily disable 
+        /* Set bDisableReadBytesEOFError=TRUE to temporarily disable
          * the EOF error message from AVCRawBinReadBytes().
          */
         bDisableReadBytesEOFError = TRUE;
@@ -440,7 +441,7 @@ GBool AVCRawBinEOF(AVCRawBinFile *psFile)
             AVCRawBinFSeek(psFile, -1, SEEK_CUR);
     }
 
-    return (psFile->nCurPos == psFile->nCurSize && 
+    return (psFile->nCurPos == psFile->nCurSize &&
             VSIFEof(psFile->fp));
 }
 
@@ -448,9 +449,9 @@ GBool AVCRawBinEOF(AVCRawBinFile *psFile)
 /**********************************************************************
  *                          AVCRawBinRead<datatype>()
  *
- * Arc/Info files are binary files with MSB first (Motorola) byte 
+ * Arc/Info files are binary files with MSB first (Motorola) byte
  * ordering.  The following functions will read from the input file
- * and return a value with the bytes ordered properly for the current 
+ * and return a value with the bytes ordered properly for the current
  * platform.
  **********************************************************************/
 GInt16  AVCRawBinReadInt16(AVCRawBinFile *psFile)
@@ -516,17 +517,17 @@ double  AVCRawBinReadDouble(AVCRawBinFile *psFile)
  *
  * Write the number of bytes from the buffer to the file.
  *
- * If a problem happens, then CPLError() will be called and 
- * CPLGetLastErrNo() can be used to test if a write operation was 
- * succesful.
+ * If a problem happens, then CPLError() will be called and
+ * CPLGetLastErrNo() can be used to test if a write operation was
+ * successful.
  **********************************************************************/
-void AVCRawBinWriteBytes(AVCRawBinFile *psFile, int nBytesToWrite, 
+void AVCRawBinWriteBytes(AVCRawBinFile *psFile, int nBytesToWrite,
                          const GByte *pBuf)
 {
     /*----------------------------------------------------------------
      * Make sure file is opened with Write access
      *---------------------------------------------------------------*/
-    if (psFile == NULL || 
+    if (psFile == NULL ||
         (psFile->eAccess != AVCWrite && psFile->eAccess != AVCReadWrite))
     {
         CPLError(CE_Failure, CPLE_FileIO,
@@ -549,13 +550,13 @@ void AVCRawBinWriteBytes(AVCRawBinFile *psFile, int nBytesToWrite,
 /**********************************************************************
  *                          AVCRawBinWrite<datatype>()
  *
- * Arc/Info files are binary files with MSB first (Motorola) byte 
+ * Arc/Info files are binary files with MSB first (Motorola) byte
  * ordering.  The following functions will reorder the byte for the
  * value properly and write that to the output file.
  *
- * If a problem happens, then CPLError() will be called and 
- * CPLGetLastErrNo() can be used to test if a write operation was 
- * succesful.
+ * If a problem happens, then CPLError() will be called and
+ * CPLGetLastErrNo() can be used to test if a write operation was
+ * successful.
  **********************************************************************/
 void  AVCRawBinWriteInt16(AVCRawBinFile *psFile, GInt16 n16Value)
 {
@@ -601,23 +602,23 @@ void  AVCRawBinWriteDouble(AVCRawBinFile *psFile, double dValue)
 /**********************************************************************
  *                          AVCRawBinWriteZeros()
  *
- * Write a number of zeros (sepcified in bytes) at the current position 
+ * Write a number of zeros (specified in bytes) at the current position
  * in the file.
  *
- * If a problem happens, then CPLError() will be called and 
- * CPLGetLastErrNo() can be used to test if a write operation was 
- * succesful.
+ * If a problem happens, then CPLError() will be called and
+ * CPLGetLastErrNo() can be used to test if a write operation was
+ * successful.
  **********************************************************************/
 void AVCRawBinWriteZeros(AVCRawBinFile *psFile, int nBytesToWrite)
 {
     char acZeros[8] = {0, 0, 0, 0, 0, 0, 0, 0};
     int i;
 
-    /* Write by 8 bytes chunks.  The last chunk may be less than 8 bytes 
+    /* Write by 8 bytes chunks.  The last chunk may be less than 8 bytes
      */
     for(i=0; i< nBytesToWrite; i+=8)
     {
-        AVCRawBinWriteBytes(psFile, MIN(8,(nBytesToWrite-i)), 
+        AVCRawBinWriteBytes(psFile, MIN(8,(nBytesToWrite-i)),
                             (GByte*)acZeros);
     }
 }
@@ -628,9 +629,9 @@ void AVCRawBinWriteZeros(AVCRawBinFile *psFile, int nBytesToWrite)
  * Write a string and pad the end of the field (up to nFieldSize) with
  * spaces number of spaces at the current position in the file.
  *
- * If a problem happens, then CPLError() will be called and 
- * CPLGetLastErrNo() can be used to test if a write operation was 
- * succesful.
+ * If a problem happens, then CPLError() will be called and
+ * CPLGetLastErrNo() can be used to test if a write operation was
+ * successful.
  **********************************************************************/
 void AVCRawBinWritePaddedString(AVCRawBinFile *psFile, int nFieldSize,
                                 const GByte *pszString)
@@ -644,7 +645,7 @@ void AVCRawBinWritePaddedString(AVCRawBinFile *psFile, int nFieldSize,
     pszString = AVCE00Convert2ArcDBCS(psFile->psDBCSInfo,
                                       pszString, nFieldSize);
 
-    nLen = strlen((const char *)pszString);
+    nLen = (int)strlen((const char *)pszString);
     nLen = MIN(nLen, nFieldSize);
     numSpaces = nFieldSize - nLen;
 
@@ -655,8 +656,7 @@ void AVCRawBinWritePaddedString(AVCRawBinFile *psFile, int nFieldSize,
      */
     for(i=0; i< numSpaces; i+=8)
     {
-        AVCRawBinWriteBytes(psFile, MIN(8,(numSpaces-i)), 
+        AVCRawBinWriteBytes(psFile, MIN(8,(numSpaces-i)),
                             (GByte*)acSpaces);
     }
 }
-
