@@ -1,12 +1,12 @@
 /******************************************************************************
- * $Id: ogrbnalayer.cpp 21684 2011-02-11 22:14:01Z warmerdam $
+ * $Id: ogrbnalayer.cpp 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  BNA Translator
  * Purpose:  Implements OGRBNALayer class.
  * Author:   Even Rouault, even dot rouault at mines dash paris dot org
  *
  ******************************************************************************
- * Copyright (c) 2007, Even Rouault
+ * Copyright (c) 2007-2010, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -33,10 +33,6 @@
 #include "cpl_csv.h"
 #include "ogr_p.h"
 
-#ifndef M_PI
-# define M_PI  3.1415926535897932384626433832795
-#endif
-
 /************************************************************************/
 /*                            OGRBNALayer()                             */
 /*                                                                      */
@@ -46,50 +42,49 @@
 
 OGRBNALayer::OGRBNALayer( const char *pszFilename,
                           const char* layerName,
-                          BNAFeatureType bnaFeatureType,
+                          BNAFeatureType bnaFeatureTypeIn,
                           OGRwkbGeometryType eLayerGeomType,
-                          int bWriter,
-                          OGRBNADataSource* poDS,
-                          int nIDs)
-
+                          int bWriterIn,
+                          OGRBNADataSource* poDSIn,
+                          int nIDsIn) :
+    poDS(poDSIn),
+    bWriter(bWriterIn),
+    nIDs(nIDsIn),
+    eof(FALSE),
+    failed(FALSE),
+    curLine(0),
+    nNextFID(0),
+    nFeatures(0),
+    partialIndexTable(TRUE),
+    offsetAndLineFeaturesTable(NULL)
 {
-    eof = FALSE;
-    failed = FALSE;
-    curLine = 0;
-    nNextFID = 0;
-    
-    this->bWriter = bWriter;
-    this->poDS = poDS;
-    this->nIDs = nIDs;
-
-    nFeatures = 0;
-    partialIndexTable = TRUE;
-    offsetAndLineFeaturesTable = NULL;
-
-    const char* iKnowHowToCount[] = { "Primary", "Secondary", "Third", "Fourth", "Fifth" };
+    static const char* const iKnowHowToCount[]
+        = { "Primary", "Secondary", "Third", "Fourth", "Fifth" };
     char tmp[32];
 
-    poFeatureDefn = new OGRFeatureDefn( CPLSPrintf("%s_%s", 
-                                                   CPLGetBasename( pszFilename ) , 
-                                                   layerName ));
+    poFeatureDefn = new OGRFeatureDefn(
+        CPLSPrintf( "%s_%s",
+                    CPLGetBasename( pszFilename ),
+                    layerName ));
     poFeatureDefn->Reference();
     poFeatureDefn->SetGeomType( eLayerGeomType );
-    this->bnaFeatureType = bnaFeatureType;
+    SetDescription( poFeatureDefn->GetName() );
+    this->bnaFeatureType = bnaFeatureTypeIn;
 
     if (! bWriter )
     {
-        int i;
-        for(i=0;i<nIDs;i++)
+        for(int i=0;i<nIDs;i++)
         {
-            if (i < (int) (sizeof(iKnowHowToCount)/sizeof(iKnowHowToCount[0])) )
+            if (i < static_cast<int>(
+                        sizeof(iKnowHowToCount)/sizeof(iKnowHowToCount[0]) ) )
             {
-                sprintf(tmp, "%s ID", iKnowHowToCount[i]);
+                snprintf(tmp, sizeof(tmp), "%s ID", iKnowHowToCount[i]);
                 OGRFieldDefn oFieldID(tmp, OFTString );
                 poFeatureDefn->AddFieldDefn( &oFieldID );
             }
             else
             {
-                sprintf(tmp, "%dth ID", i+1);
+                snprintf(tmp, sizeof(tmp), "%dth ID", i+1);
                 OGRFieldDefn oFieldID(tmp, OFTString );
                 poFeatureDefn->AddFieldDefn( &oFieldID );
             }
@@ -132,11 +127,14 @@ OGRBNALayer::~OGRBNALayer()
 /************************************************************************/
 /*                         SetFeatureIndexTable()                       */
 /************************************************************************/
-void  OGRBNALayer::SetFeatureIndexTable(int nFeatures, OffsetAndLine* offsetAndLineFeaturesTable, int partialIndexTable)
+void OGRBNALayer::SetFeatureIndexTable(
+    int nFeaturesIn,
+    OffsetAndLine* offsetAndLineFeaturesTableIn,
+    int partialIndexTableIn )
 {
-    this->nFeatures = nFeatures;
-    this->offsetAndLineFeaturesTable = offsetAndLineFeaturesTable;
-    this->partialIndexTable = partialIndexTable;
+    nFeatures = nFeaturesIn;
+    offsetAndLineFeaturesTable = offsetAndLineFeaturesTableIn;
+    partialIndexTable = partialIndexTableIn;
 }
 
 /************************************************************************/
@@ -146,11 +144,13 @@ void  OGRBNALayer::SetFeatureIndexTable(int nFeatures, OffsetAndLine* offsetAndL
 void OGRBNALayer::ResetReading()
 
 {
+    if( fpBNA == NULL )
+        return;
     eof = FALSE;
     failed = FALSE;
     curLine = 0;
     nNextFID = 0;
-    VSIFSeekL( fpBNA, 0, SEEK_SET );
+    CPL_IGNORE_RET_VAL(VSIFSeekL( fpBNA, 0, SEEK_SET ));
 }
 
 
@@ -160,23 +160,23 @@ void OGRBNALayer::ResetReading()
 
 OGRFeature *OGRBNALayer::GetNextFeature()
 {
-    OGRFeature  *poFeature;
-    BNARecord* record;
-    int offset, line;
-
-    if (failed || eof) return NULL;
+    if (failed || eof || fpBNA == NULL)
+        return NULL;
 
     while(1)
     {
         int ok = FALSE;
-        offset = (int) VSIFTellL(fpBNA);
-        line = curLine;
+        const int offset = static_cast<int>( VSIFTellL(fpBNA) );
+        const int line = curLine;
         if (nNextFID < nFeatures)
         {
-            VSIFSeekL( fpBNA, offsetAndLineFeaturesTable[nNextFID].offset, SEEK_SET );
+            if( VSIFSeekL( fpBNA, offsetAndLineFeaturesTable[nNextFID].offset,
+                           SEEK_SET ) < 0 )
+                return NULL;
             curLine = offsetAndLineFeaturesTable[nNextFID].line;
         }
-        record =  BNA_GetNextRecord(fpBNA, &ok, &curLine, TRUE, bnaFeatureType);
+        BNARecord* record
+            = BNA_GetNextRecord(fpBNA, &ok, &curLine, TRUE, bnaFeatureType);
         if (ok == FALSE)
         {
             BNA_FreeRecord(record);
@@ -199,12 +199,15 @@ OGRFeature *OGRBNALayer::GetNextFeature()
             {
                 nFeatures++;
                 offsetAndLineFeaturesTable =
-                    (OffsetAndLine*)CPLRealloc(offsetAndLineFeaturesTable, nFeatures * sizeof(OffsetAndLine));
+                    static_cast<OffsetAndLine *>(
+                        CPLRealloc( offsetAndLineFeaturesTable,
+                                    nFeatures * sizeof(OffsetAndLine) ) );
                 offsetAndLineFeaturesTable[nFeatures-1].offset = offset;
                 offsetAndLineFeaturesTable[nFeatures-1].line = line;
             }
 
-            poFeature = BuildFeatureFromBNARecord(record, nNextFID++);
+            OGRFeature *poFeature
+                = BuildFeatureFromBNARecord(record, nNextFID++);
 
             BNA_FreeRecord(record);
 
@@ -232,13 +235,12 @@ OGRFeature *OGRBNALayer::GetNextFeature()
 
 void OGRBNALayer::WriteFeatureAttributes(VSILFILE* fp, OGRFeature *poFeature )
 {
-    int i;
     OGRFieldDefn *poFieldDefn;
     int nbOutID = poDS->GetNbOutId();
     if (nbOutID < 0)
         nbOutID = poFeatureDefn->GetFieldCount();
-    for(i=0;i<nbOutID;i++)
-    { 
+    for(int i = 0; i < nbOutID; i++ )
+    {
         if (i < poFeatureDefn->GetFieldCount())
         {
             poFieldDefn = poFeatureDefn->GetFieldDefn( i );
@@ -249,22 +251,22 @@ void OGRBNALayer::WriteFeatureAttributes(VSILFILE* fp, OGRFeature *poFeature )
                     char szBuffer[64];
                     OGRFormatDouble(szBuffer, sizeof(szBuffer),
                                     poFeature->GetFieldAsDouble(i), '.');
-                    VSIFPrintfL( fp, "\"%s\",", szBuffer);
+                    CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "\"%s\",", szBuffer));
                 }
                 else
                 {
                     const char *pszRaw = poFeature->GetFieldAsString( i );
-                    VSIFPrintfL( fp, "\"%s\",", pszRaw);
+                    CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "\"%s\",", pszRaw));
                 }
             }
             else
             {
-                VSIFPrintfL( fp, "\"\",");
+                CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "\"\","));
             }
         }
         else
         {
-            VSIFPrintfL( fp, "\"\",");
+            CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "\"\","));
         }
     }
 }
@@ -277,20 +279,19 @@ void OGRBNALayer::WriteCoord(VSILFILE* fp, double dfX, double dfY)
 {
     char szBuffer[64];
     OGRFormatDouble(szBuffer, sizeof(szBuffer), dfX, '.', poDS->GetCoordinatePrecision());
-    VSIFPrintfL( fp, "%s", szBuffer);
-    VSIFPrintfL( fp, "%s", poDS->GetCoordinateSeparator());
+    CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", szBuffer));
+    CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", poDS->GetCoordinateSeparator()));
     OGRFormatDouble(szBuffer, sizeof(szBuffer), dfY, '.', poDS->GetCoordinatePrecision());
-    VSIFPrintfL( fp, "%s", szBuffer);
+    CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", szBuffer));
 }
 
 /************************************************************************/
-/*                           CreateFeature()                            */
+/*                           ICreateFeature()                            */
 /************************************************************************/
 
-OGRErr OGRBNALayer::CreateFeature( OGRFeature *poFeature )
+OGRErr OGRBNALayer::ICreateFeature( OGRFeature *poFeature )
 
 {
-    int i,j,k,n;
     OGRGeometry     *poGeom = poFeature->GetGeometryRef();
     char eol[3];
     const char* partialEol = (poDS->GetMultiLine()) ? eol : poDS->GetCoordinateSeparator();
@@ -313,15 +314,15 @@ OGRErr OGRBNALayer::CreateFeature( OGRFeature *poFeature )
         eol[0] = 10;
         eol[1] = 0;
     }
-    
+
     if ( ! bWriter )
     {
         return OGRERR_FAILURE;
     }
-    
+
     if( poFeature->GetFID() == OGRNullFID )
         poFeature->SetFID( nFeatures++ );
-    
+
     VSILFILE* fp = poDS->GetOutputFP();
     int nbPairPerLine = poDS->GetNbPairPerLine();
 
@@ -332,13 +333,13 @@ OGRErr OGRBNALayer::CreateFeature( OGRFeature *poFeature )
         {
             OGRPoint* point = (OGRPoint*)poGeom;
             WriteFeatureAttributes(fp, poFeature);
-            VSIFPrintfL( fp, "1");
-            VSIFPrintfL( fp, "%s", partialEol);
+            CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "1"));
+            CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", partialEol));
             WriteCoord(fp, point->getX(), point->getY());
-            VSIFPrintfL( fp, "%s", eol);
+            CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", eol));
             break;
         }
-            
+
         case wkbPolygon:
         case wkbPolygon25D:
         {
@@ -348,12 +349,12 @@ OGRErr OGRBNALayer::CreateFeature( OGRFeature *poFeature )
             {
                 return OGRERR_FAILURE;
             }
-            
+
             double firstX = ring->getX(0);
             double firstY = ring->getY(0);
             int nBNAPoints = ring->getNumPoints();
             int is_ellipse = FALSE;
-            
+
             /* This code tries to detect an ellipse in a polygon geometry */
             /* This will only work presumably on ellipses already read from a BNA file */
             /* Mostly a BNA to BNA feature... */
@@ -378,7 +379,7 @@ OGRErr OGRBNALayer::CreateFeature( OGRFeature *poFeature )
                     double major_radius = fabs(firstX - center1X);
                     double minor_radius = fabs(quarterY - center1Y);
                     is_ellipse = TRUE;
-                    for(i=0;i<360;i++)
+                    for(int i=0;i<360;i++)
                     {
                         if (!(fabs(center1X + major_radius * cos(i * (M_PI / 180)) - ring->getX(i)) < 1e-5 &&
                               fabs(center1Y + minor_radius * sin(i * (M_PI / 180)) - ring->getY(i)) < 1e-5))
@@ -390,12 +391,12 @@ OGRErr OGRBNALayer::CreateFeature( OGRFeature *poFeature )
                     if ( is_ellipse == TRUE )
                     {
                         WriteFeatureAttributes(fp, poFeature);
-                        VSIFPrintfL( fp, "2");
-                        VSIFPrintfL( fp, "%s", partialEol);
+                        CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "2"));
+                        CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", partialEol));
                         WriteCoord(fp, center1X, center1Y);
-                        VSIFPrintfL( fp, "%s", partialEol);
+                        CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", partialEol));
                         WriteCoord(fp, major_radius, minor_radius);
-                        VSIFPrintfL( fp, "%s", eol);
+                        CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", eol));
                     }
                 }
             }
@@ -403,7 +404,7 @@ OGRErr OGRBNALayer::CreateFeature( OGRFeature *poFeature )
             if ( is_ellipse == FALSE)
             {
                 int nInteriorRings = polygon->getNumInteriorRings();
-                for(i=0;i<nInteriorRings;i++)
+                for(int i=0;i<nInteriorRings;i++)
                 {
                     nBNAPoints += polygon->getInteriorRing(i)->getNumPoints() + 1;
                 }
@@ -413,30 +414,30 @@ OGRErr OGRBNALayer::CreateFeature( OGRFeature *poFeature )
                     return OGRERR_FAILURE;
                 }
                 WriteFeatureAttributes(fp, poFeature);
-                VSIFPrintfL( fp, "%d", nBNAPoints);
-                n = ring->getNumPoints();
+                CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%d", nBNAPoints));
+                int n = ring->getNumPoints();
                 int nbPair = 0;
-                for(i=0;i<n;i++)
+                for(int i=0;i<n;i++)
                 {
-                    VSIFPrintfL( fp, "%s", ((nbPair % nbPairPerLine) == 0) ? partialEol : " ");
+                    CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", ((nbPair % nbPairPerLine) == 0) ? partialEol : " "));
                     WriteCoord(fp, ring->getX(i), ring->getY(i));
                     nbPair++;
                 }
-                for(i=0;i<nInteriorRings;i++)
+                for(int i=0;i<nInteriorRings;i++)
                 {
                     ring = polygon->getInteriorRing(i);
                     n = ring->getNumPoints();
-                    for(j=0;j<n;j++)
+                    for(int j=0;j<n;j++)
                     {
-                        VSIFPrintfL( fp, "%s", ((nbPair % nbPairPerLine) == 0) ? partialEol : " ");
+                        CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", ((nbPair % nbPairPerLine) == 0) ? partialEol : " "));
                         WriteCoord(fp, ring->getX(j), ring->getY(j));
                         nbPair++;
                     }
-                    VSIFPrintfL( fp, "%s", ((nbPair % nbPairPerLine) == 0) ? partialEol : " ");
+                    CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", ((nbPair % nbPairPerLine) == 0) ? partialEol : " "));
                     WriteCoord(fp, firstX, firstY);
                     nbPair++;
                 }
-                VSIFPrintfL( fp, "%s", eol);
+                CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", eol));
             }
             break;
         }
@@ -447,8 +448,8 @@ OGRErr OGRBNALayer::CreateFeature( OGRFeature *poFeature )
             OGRMultiPolygon* multipolygon = (OGRMultiPolygon*)poGeom;
             int N = multipolygon->getNumGeometries();
             int nBNAPoints = 0;
-            double firstX = 0, firstY = 0; 
-            for(i=0;i<N;i++)
+            double firstX = 0, firstY = 0;
+            for(int i=0;i<N;i++)
             {
                 OGRPolygon* polygon = (OGRPolygon*)multipolygon->getGeometryRef(i);
                 OGRLinearRing* ring = polygon->getExteriorRing();
@@ -464,7 +465,7 @@ OGRErr OGRBNALayer::CreateFeature( OGRFeature *poFeature )
                 }
                 nBNAPoints += ring->getNumPoints();
                 int nInteriorRings = polygon->getNumInteriorRings();
-                for(j=0;j<nInteriorRings;j++)
+                for(int j=0;j<nInteriorRings;j++)
                 {
                     nBNAPoints += polygon->getInteriorRing(j)->getNumPoints() + 1;
                 }
@@ -475,45 +476,45 @@ OGRErr OGRBNALayer::CreateFeature( OGRFeature *poFeature )
                 return OGRERR_FAILURE;
             }
             WriteFeatureAttributes(fp, poFeature);
-            VSIFPrintfL( fp, "%d", nBNAPoints);
+            CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%d", nBNAPoints));
             int nbPair = 0;
-            for(i=0;i<N;i++)
+            for(int i=0;i<N;i++)
             {
                 OGRPolygon* polygon = (OGRPolygon*)multipolygon->getGeometryRef(i);
                 OGRLinearRing* ring = polygon->getExteriorRing();
                 if (ring == NULL)
                     continue;
 
-                n = ring->getNumPoints();
+                int n = ring->getNumPoints();
                 int nInteriorRings = polygon->getNumInteriorRings();
-                for(j=0;j<n;j++)
+                for(int j=0;j<n;j++)
                 {
-                    VSIFPrintfL( fp, "%s", ((nbPair % nbPairPerLine) == 0) ? partialEol : " ");
+                    CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", ((nbPair % nbPairPerLine) == 0) ? partialEol : " "));
                     WriteCoord(fp, ring->getX(j), ring->getY(j));
                     nbPair++;
                 }
                 if (i != 0)
                 {
-                    VSIFPrintfL( fp, "%s", ((nbPair % nbPairPerLine) == 0) ? partialEol : " ");
+                    CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", ((nbPair % nbPairPerLine) == 0) ? partialEol : " "));
                     WriteCoord(fp, firstX, firstY);
                     nbPair++;
                 }
-                for(j=0;j<nInteriorRings;j++)
+                for(int j=0;j<nInteriorRings;j++)
                 {
                     ring = polygon->getInteriorRing(j);
                     n = ring->getNumPoints();
-                    for(k=0;k<n;k++)
+                    for(int k=0;k<n;k++)
                     {
-                        VSIFPrintfL( fp, "%s", ((nbPair % nbPairPerLine) == 0) ? partialEol : " ");
+                        CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", ((nbPair % nbPairPerLine) == 0) ? partialEol : " "));
                         WriteCoord(fp, ring->getX(k), ring->getY(k));
                         nbPair++;
                     }
-                    VSIFPrintfL( fp, "%s", ((nbPair % nbPairPerLine) == 0) ? partialEol : " ");
+                    CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", ((nbPair % nbPairPerLine) == 0) ? partialEol : " "));
                     WriteCoord(fp, firstX, firstY);
                     nbPair++;
                 }
             }
-            VSIFPrintfL( fp, "%s", eol);
+            CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", eol));
             break;
         }
 
@@ -522,25 +523,24 @@ OGRErr OGRBNALayer::CreateFeature( OGRFeature *poFeature )
         {
             OGRLineString* line = (OGRLineString*)poGeom;
             int n = line->getNumPoints();
-            int i;
             if (n < 2)
             {
                 CPLError( CE_Failure, CPLE_AppDefined, "Invalid geometry" );
                 return OGRERR_FAILURE;
             }
             WriteFeatureAttributes(fp, poFeature);
-            VSIFPrintfL( fp, "-%d", n);
+            CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "-%d", n));
             int nbPair = 0;
-            for(i=0;i<n;i++)
+            for(int i=0;i<n;i++)
             {
-                VSIFPrintfL( fp, "%s", partialEol);
+                CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", partialEol));
                 WriteCoord(fp, line->getX(i), line->getY(i));
                 nbPair++;
             }
-            VSIFPrintfL( fp, "%s", eol);
+            CPL_IGNORE_RET_VAL(VSIFPrintfL( fp, "%s", eol));
             break;
         }
-            
+
         default:
         {
             CPLError( CE_Failure, CPLE_AppDefined,
@@ -550,7 +550,7 @@ OGRErr OGRBNALayer::CreateFeature( OGRFeature *poFeature )
             return OGRERR_UNSUPPORTED_GEOMETRY_TYPE;
         }
     }
-    
+
     return OGRERR_NONE;
 }
 
@@ -560,8 +560,7 @@ OGRErr OGRBNALayer::CreateFeature( OGRFeature *poFeature )
 /*                            CreateField()                             */
 /************************************************************************/
 
-OGRErr OGRBNALayer::CreateField( OGRFieldDefn *poField, int bApproxOK )
-
+OGRErr OGRBNALayer::CreateField( OGRFieldDefn *poField, int /* bApproxOK */ )
 {
     if( !bWriter || nFeatures != 0)
         return OGRERR_FAILURE;
@@ -575,29 +574,28 @@ OGRErr OGRBNALayer::CreateField( OGRFieldDefn *poField, int bApproxOK )
 /************************************************************************/
 /*                           BuildFeatureFromBNARecord()                */
 /************************************************************************/
-OGRFeature *    OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long fid)
+OGRFeature *OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long fid)
 {
-    OGRFeature  *poFeature;
-    int i;
-
-    poFeature = new OGRFeature( poFeatureDefn );
-    for(i=0;i<nIDs;i++)
+    OGRFeature  *poFeature = new OGRFeature( poFeatureDefn );
+    for( int i = 0; i < nIDs; i++ )
     {
         poFeature->SetField( i, record->ids[i] ? record->ids[i] : "");
     }
     poFeature->SetFID( fid );
     if (bnaFeatureType == BNA_POINT)
     {
-        poFeature->SetGeometryDirectly( new OGRPoint( record->tabCoords[0][0], record->tabCoords[0][1] ) );
+        poFeature->SetGeometryDirectly(
+            new OGRPoint( record->tabCoords[0][0], record->tabCoords[0][1] ) );
     }
     else if (bnaFeatureType == BNA_POLYLINE)
     {
         OGRLineString* lineString = new OGRLineString ();
         lineString->setCoordinateDimension(2);
         lineString->setNumPoints(record->nCoords);
-        for(i=0;i<record->nCoords;i++)
+        for( int i = 0; i < record->nCoords; i++)
         {
-            lineString->setPoint(i, record->tabCoords[i][0], record->tabCoords[i][1] );
+            lineString->setPoint( i, record->tabCoords[i][0],
+                                  record->tabCoords[i][1] );
         }
         poFeature->SetGeometryDirectly(lineString);
     }
@@ -607,17 +605,19 @@ OGRFeature *    OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long
         double firstY = record->tabCoords[0][1];
         int isFirstPolygon = 1;
         double secondaryFirstX = 0, secondaryFirstY = 0;
-  
+
         OGRLinearRing* ring = new OGRLinearRing ();
         ring->setCoordinateDimension(2);
         ring->addPoint(record->tabCoords[0][0], record->tabCoords[0][1] );
-  
+
         /* record->nCoords is really a safe upper bound */
         int nbPolygons = 0;
         OGRPolygon** tabPolygons =
-            (OGRPolygon**)CPLMalloc(record->nCoords * sizeof(OGRPolygon*));
+            static_cast<OGRPolygon**>(
+                CPLMalloc( record->nCoords * sizeof(OGRPolygon*) ) );
 
-        for(i=1;i<record->nCoords;i++)
+        int i = 1;
+        for( ; i < record->nCoords; i++ )
         {
             ring->addPoint(record->tabCoords[i][0], record->tabCoords[i][1] );
             if (isFirstPolygon == 1 &&
@@ -628,20 +628,21 @@ OGRFeature *    OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long
                 polygon->addRingDirectly(ring);
                 tabPolygons[nbPolygons] = polygon;
                 nbPolygons++;
-    
+
                 if (i == record->nCoords - 1)
                 {
                     break;
                 }
-    
+
                 isFirstPolygon = 0;
-    
-                i ++;
+
+                i++;
                 secondaryFirstX = record->tabCoords[i][0];
                 secondaryFirstY = record->tabCoords[i][1];
                 ring = new OGRLinearRing ();
                 ring->setCoordinateDimension(2);
-                ring->addPoint(record->tabCoords[i][0], record->tabCoords[i][1] );
+                ring->addPoint( record->tabCoords[i][0],
+                                record->tabCoords[i][1] );
             }
             else if (isFirstPolygon == 0 &&
                     record->tabCoords[i][0] == secondaryFirstX &&
@@ -655,9 +656,11 @@ OGRFeature *    OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long
 
                 if (i < record->nCoords - 1)
                 {
-                    /* After the closing of a subpolygon, the first coordinates of the first polygon */
-                    /* should be recalled... in theory */
-                    if (record->tabCoords[i+1][0] == firstX &&  record->tabCoords[i+1][1] == firstY)
+                    // After the closing of a subpolygon, the first
+                    // coordinates of the first polygon should be
+                    // recalled... in theory
+                    if (record->tabCoords[i+1][0] == firstX
+                        && record->tabCoords[i+1][1] == firstY)
                     {
                         if (i + 1 == record->nCoords - 1)
                             break;
@@ -666,8 +669,9 @@ OGRFeature *    OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long
                     else
                     {
 #if 0
-                        CPLError(CE_Warning, CPLE_AppDefined, 
-                                 "Geometry of polygon of fid %d starting at line %d is not strictly conformant. "
+                        CPLError(CE_Warning, CPLE_AppDefined,
+                                 "Geometry of polygon of fid %d starting at "
+                                 "line %d is not strictly conformant. "
                                  "Trying to go on...\n",
                                  fid,
                                  offsetAndLineFeaturesTable[fid].line + 1);
@@ -679,25 +683,29 @@ OGRFeature *    OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long
                     secondaryFirstY = record->tabCoords[i][1];
                     ring = new OGRLinearRing ();
                     ring->setCoordinateDimension(2);
-                    ring->addPoint(record->tabCoords[i][0], record->tabCoords[i][1] );
+                    ring->addPoint( record->tabCoords[i][0],
+                                    record->tabCoords[i][1] );
                 }
                 else
                 {
 #if 0
-                    CPLError(CE_Warning, CPLE_AppDefined, 
-                        "Geometry of polygon of fid %d starting at line %d is not strictly conformant. Trying to go on...\n",
-                        fid,
-                        offsetAndLineFeaturesTable[fid].line + 1);
+                    CPLError( CE_Warning, CPLE_AppDefined,
+                              "Geometry of polygon of fid %d starting at "
+                              "line %d is not strictly conformant. "
+                              "Trying to go on...\n",
+                              fid,
+                              offsetAndLineFeaturesTable[fid].line + 1);
 #endif
                 }
             }
         }
         if (i == record->nCoords)
         {
-            /* Let's be a bit tolerant abount non closing polygons */
+            /* Let's be a bit tolerant about non-closing polygons. */
             if (isFirstPolygon)
             {
-                ring->addPoint(record->tabCoords[0][0], record->tabCoords[0][1] );
+                ring->addPoint( record->tabCoords[0][0],
+                                record->tabCoords[0][1] );
 
                 OGRPolygon* polygon = new OGRPolygon ();
                 polygon->addRingDirectly(ring);
@@ -705,11 +713,11 @@ OGRFeature *    OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long
                 nbPolygons++;
             }
         }
-        
+
         if (nbPolygons == 1)
         {
-            /* Special optimization here : we directly put the polygon into the multipolygon. */
-            /* This should save quite a few useless copies */
+            // Special optimization here : we directly put the polygon into
+            // the multipolygon.  This should save quite a few useless copies
             OGRMultiPolygon* multipolygon = new OGRMultiPolygon();
             multipolygon->addGeometryDirectly(tabPolygons[0]);
             poFeature->SetGeometryDirectly(multipolygon);
@@ -718,15 +726,18 @@ OGRFeature *    OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long
         {
             int isValidGeometry;
             poFeature->SetGeometryDirectly(
-                OGRGeometryFactory::organizePolygons((OGRGeometry**)tabPolygons, nbPolygons, &isValidGeometry, NULL));
-            
+                OGRGeometryFactory::organizePolygons(
+                    reinterpret_cast<OGRGeometry**>( tabPolygons ),
+                    nbPolygons, &isValidGeometry, NULL ) );
+
             if (!isValidGeometry)
             {
-                CPLError(CE_Warning, CPLE_AppDefined, 
-                        "Geometry of polygon of fid %ld starting at line %d cannot be translated to Simple Geometry. "
-                        "All polygons will be contained in a multipolygon.\n",
-                        fid,
-                        offsetAndLineFeaturesTable[fid].line + 1);
+                CPLError( CE_Warning, CPLE_AppDefined,
+                          "Geometry of polygon of fid %ld starting at line %d "
+                          "cannot be translated to Simple Geometry. "
+                          "All polygons will be contained in a multipolygon.\n",
+                          fid,
+                          offsetAndLineFeaturesTable[fid].line + 1);
             }
         }
 
@@ -734,8 +745,8 @@ OGRFeature *    OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long
     }
     else
     {
-        /* Circle or ellipses are not part of the OGR Simple Geometry, so we discretize them
-           into polygons by 1 degree step */
+        // Circle or ellipses are not part of the OGR Simple Geometry, so we
+        // discretize them into polygons by 1 degree step.
         OGRPolygon* polygon = new OGRPolygon ();
         OGRLinearRing* ring = new OGRLinearRing ();
         ring->setCoordinateDimension(2);
@@ -745,7 +756,7 @@ OGRFeature *    OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long
         double minor_radius = record->tabCoords[1][1];
         if (minor_radius == 0)
             minor_radius = major_radius;
-        for(i=0;i<360;i++)
+        for( int i = 0; i < 360; i++ )
         {
             ring->addPoint(center_x + major_radius * cos(i * (M_PI / 180)),
                            center_y + minor_radius * sin(i * (M_PI / 180)) );
@@ -757,7 +768,7 @@ OGRFeature *    OGRBNALayer::BuildFeatureFromBNARecord (BNARecord* record, long
         poFeature->SetField( nIDs, major_radius);
         poFeature->SetField( nIDs+1, minor_radius);
     }
-    
+
     return poFeature;
 }
 
@@ -771,25 +782,30 @@ void OGRBNALayer::FastParseUntil ( int interestFID)
     {
         ResetReading();
 
-        BNARecord* record;
+        BNARecord *record = NULL;
 
         if (nFeatures > 0)
         {
-            VSIFSeekL( fpBNA, offsetAndLineFeaturesTable[nFeatures-1].offset, SEEK_SET );
+            if( VSIFSeekL( fpBNA,
+                           offsetAndLineFeaturesTable[nFeatures-1].offset,
+                           SEEK_SET ) < 0 )
+                return;
             curLine = offsetAndLineFeaturesTable[nFeatures-1].line;
 
             /* Just skip the last read one */
             int ok = FALSE;
-            record =  BNA_GetNextRecord(fpBNA, &ok, &curLine, TRUE, BNA_READ_NONE);
+            record = BNA_GetNextRecord(fpBNA, &ok, &curLine, TRUE,
+                                       BNA_READ_NONE);
             BNA_FreeRecord(record);
         }
 
         while(1)
         {
             int ok = FALSE;
-            int offset = (int) VSIFTellL(fpBNA);
+            int offset = static_cast<int>( VSIFTellL(fpBNA) );
             int line = curLine;
-            record =  BNA_GetNextRecord(fpBNA, &ok, &curLine, TRUE, BNA_READ_NONE);
+            record = BNA_GetNextRecord(fpBNA, &ok, &curLine, TRUE,
+                                       BNA_READ_NONE);
             if (ok == FALSE)
             {
                 failed = TRUE;
@@ -809,7 +825,9 @@ void OGRBNALayer::FastParseUntil ( int interestFID)
             {
                 nFeatures++;
                 offsetAndLineFeaturesTable =
-                    (OffsetAndLine*)CPLRealloc(offsetAndLineFeaturesTable, nFeatures * sizeof(OffsetAndLine));
+                    static_cast<OffsetAndLine *>(
+                        CPLRealloc( offsetAndLineFeaturesTable,
+                                    nFeatures * sizeof(OffsetAndLine) ) );
                 offsetAndLineFeaturesTable[nFeatures-1].offset = offset;
                 offsetAndLineFeaturesTable[nFeatures-1].line = line;
 
@@ -830,25 +848,25 @@ void OGRBNALayer::FastParseUntil ( int interestFID)
 /*                           GetFeature()                               */
 /************************************************************************/
 
-OGRFeature *  OGRBNALayer::GetFeature( long nFID )
+OGRFeature *  OGRBNALayer::GetFeature( GIntBig nFID )
 {
-    OGRFeature  *poFeature;
-    BNARecord* record;
-    int ok;
-    
-    if (nFID < 0)
+    if (nFID < 0 || !CPL_INT64_FITS_ON_INT32(nFID))
         return NULL;
 
-    FastParseUntil(nFID);
+    FastParseUntil( static_cast<int>( nFID ) );
 
     if (nFID >= nFeatures)
         return NULL;
 
-    VSIFSeekL( fpBNA, offsetAndLineFeaturesTable[nFID].offset, SEEK_SET );
-    curLine = offsetAndLineFeaturesTable[nFID].line;
-    record =  BNA_GetNextRecord(fpBNA, &ok, &curLine, TRUE, bnaFeatureType);
+    int ok;
+    if( VSIFSeekL( fpBNA, offsetAndLineFeaturesTable[nFID].offset, SEEK_SET ) < 0 )
+        return NULL;
 
-    poFeature = BuildFeatureFromBNARecord(record, nFID);
+    curLine = offsetAndLineFeaturesTable[nFID].line;
+    BNARecord* record
+        = BNA_GetNextRecord(fpBNA, &ok, &curLine, TRUE, bnaFeatureType);
+
+    OGRFeature *poFeature = BuildFeatureFromBNARecord(record, (int)nFID);
 
     BNA_FreeRecord(record);
 
@@ -869,4 +887,3 @@ int OGRBNALayer::TestCapability( const char * pszCap )
     else
         return FALSE;
 }
-
