@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrbnaparser.cpp 27710 2014-09-21 15:30:29Z goatbar $
+ * $Id: ogrbnaparser.cpp 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  BNA Parser
  * Purpose:  Parse a BNA record
@@ -36,8 +36,7 @@ void BNA_FreeRecord(BNARecord* record)
 {
   if (record)
   {
-    int i;
-    for(i=0;i<NB_MAX_BNA_IDS;i++)
+    for( int i = 0; i < NB_MAX_BNA_IDS; i++)
     {
         if (record->ids[i]) VSIFree(record->ids[i]);
         record->ids[i] = NULL;
@@ -67,14 +66,14 @@ const char* BNA_FeatureTypeToStr(BNAFeatureType featureType)
 
 void BNA_Display(BNARecord* record)
 {
-  int i;
   fprintf(stderr, "\"%s\", \"%s\", \"%s\", %s\n",
           record->ids[0] ? record->ids[0] : "",
           record->ids[1] ? record->ids[1] : "",
           record->ids[2] ? record->ids[2] : "",
           BNA_FeatureTypeToStr(record->featureType));
-  for(i=0;i<record->nCoords;i++)
-    fprintf(stderr, "%f, %f\n", record->tabCoords[i][0], record->tabCoords[i][1]);
+  for( int i = 0; i < record->nCoords; i++ )
+    fprintf( stderr, "%f, %f\n", record->tabCoords[i][0],
+             record->tabCoords[i][1] );
 }
 
 /*
@@ -113,31 +112,32 @@ and http://64.145.236.125/forum/topic.asp?topic_id=1930&forum_id=1&Topic_Title=h
 -73.000000, 41.087010 -73.035597, 41.114420 -73.049337, 41.125000
 */
 
-/* We are (and must be) a bit tolerant : BNA files format has several variations */
-/* and most don't follow strictly the 'specification' */
-/* Extra spaces, tabulations or line feed are accepted and ignored */
-/* We allow one line format and several line format in the same file */
-/* We allow from NB_MIN_BNA_IDS to NB_MAX_BNA_IDS ids */
-/* We allow that couples of coordinates on the same line may be separated only by spaces */
-/* (instead of being separated by a comma) */
+// We are (and must be) a bit tolerant : BNA files format has several variations
+// and most don't follow strictly the 'specification'.
+// Extra spaces, tabulations or line feed are accepted and ignored.
+// We allow one line format and several line format in the same file.
+// We allow from NB_MIN_BNA_IDS to NB_MAX_BNA_IDS ids.
+// We allow that couples of coordinates on the same line may be separated only
+// by spaces (instead of being separated by a comma).
 
-#define STRING_NOT_TERMINATED      "string not terminated when end of line occured"
-#define MISSING_FIELDS             "missing fields"
-#define BAD_INTEGER_NUMBER_FORMAT  "bad integer number format"
-#define BAD_FLOAT_NUMBER_FORMAT    "bad float number format"
-#define PRIMARY_ID_MANDATORY       "primary ID can't be empty or missing"
-#define STRING_EXPECTED            "string expected"
-#define NUMBER_EXPECTED            "number expected"
-#define INTEGER_NUMBER_EXPECTED    "integer number expected"
-#define FLOAT_NUMBER_EXPECTED      "float number expected"
-#define INVALID_GEOMETRY_TYPE      "invalid geometry type"
-#define TOO_LONG_ID                "too long id (> 256 characters)"
-#define MAX_BNA_IDS_REACHED        "maximum number of IDs reached"
-#define NOT_ENOUGH_MEMORY          "not enough memory for request number of coordinates"
-#define LINE_TOO_LONG              "line too long"
+static const char *const STRING_NOT_TERMINATED
+    = "string not terminated when end of line occurred";
+static const char *const MISSING_FIELDS            = "missing fields";
+static const char *const BAD_INTEGER_NUMBER_FORMAT = "bad integer number format";
+static const char *const BAD_FLOAT_NUMBER_FORMAT   = "bad float number format";
+static const char *const STRING_EXPECTED           = "string expected";
+static const char *const NUMBER_EXPECTED           = "number expected";
+static const char *const INTEGER_NUMBER_EXPECTED   = "integer number expected";
+static const char *const FLOAT_NUMBER_EXPECTED     = "float number expected";
+static const char *const INVALID_GEOMETRY_TYPE     = "invalid geometry type";
+static const char *const TOO_LONG_ID               = "too long id (> 256 characters)";
+static const char *const MAX_BNA_IDS_REACHED       = "maximum number of IDs reached";
+static const char *const NOT_ENOUGH_MEMORY
+    = "not enough memory for request number of coordinates";
+static const char *const LINE_TOO_LONG             = "line too long";
 
-#define TMP_BUFFER_SIZE             256
-#define LINE_BUFFER_SIZE            1024
+static const int TMP_BUFFER_SIZE = 256;
+static const int LINE_BUFFER_SIZE = 1024;
 
 enum
 {
@@ -149,7 +149,8 @@ enum
 static int BNA_GetLine(char szLineBuffer[LINE_BUFFER_SIZE+1], VSILFILE* f)
 {
     char* ptrCurLine = szLineBuffer;
-    int nRead = VSIFReadL(szLineBuffer, 1, LINE_BUFFER_SIZE, f);
+    int nRead
+        = static_cast<int>( VSIFReadL(szLineBuffer, 1, LINE_BUFFER_SIZE, f) );
     szLineBuffer[nRead] = 0;
     if (nRead == 0)
     {
@@ -171,8 +172,8 @@ static int BNA_GetLine(char szLineBuffer[LINE_BUFFER_SIZE+1], VSILFILE* f)
     {
         if (nRead < LINE_BUFFER_SIZE)
             return BNA_LINE_OK;
-        else
-            return BNA_LINE_TOO_LONG;
+
+        return BNA_LINE_TOO_LONG;
     }
 
     if (*ptrCurLine == 0x0d)
@@ -180,7 +181,7 @@ static int BNA_GetLine(char szLineBuffer[LINE_BUFFER_SIZE+1], VSILFILE* f)
         if (ptrCurLine == szLineBuffer + LINE_BUFFER_SIZE - 1)
         {
             char c;
-            nRead = VSIFReadL(&c, 1, 1, f);
+            nRead = static_cast<int>(VSIFReadL(&c, 1, 1, f));
             if (nRead == 1)
             {
                 if (c == 0x0a)
@@ -189,22 +190,34 @@ static int BNA_GetLine(char szLineBuffer[LINE_BUFFER_SIZE+1], VSILFILE* f)
                 }
                 else
                 {
-                    VSIFSeekL(f, VSIFTellL(f) - 1, SEEK_SET);
+                    if( VSIFSeekL(f, VSIFTellL(f) - 1, SEEK_SET) != 0 )
+                        return BNA_LINE_EOF;
                 }
             }
         }
         else if (ptrCurLine[1] == 0x0a)
         {
-            VSIFSeekL(f, VSIFTellL(f) + ptrCurLine + 2 - (szLineBuffer + nRead), SEEK_SET);
+            if( VSIFSeekL( f,
+                           VSIFTellL(f) + ptrCurLine + 2
+                           - (szLineBuffer + nRead),
+                           SEEK_SET ) != 0 )
+                return BNA_LINE_EOF;
         }
         else
         {
-            VSIFSeekL(f, VSIFTellL(f) + ptrCurLine + 1 - (szLineBuffer + nRead), SEEK_SET);
+            if( VSIFSeekL( f,
+                           VSIFTellL(f) + ptrCurLine + 1
+                           - (szLineBuffer + nRead),
+                           SEEK_SET ) != 0 )
+                return BNA_LINE_EOF;
         }
     }
     else /* *ptrCurLine == 0x0a */
     {
-        VSIFSeekL(f, VSIFTellL(f) + ptrCurLine + 1 - (szLineBuffer + nRead), SEEK_SET);
+        if( VSIFSeekL( f,
+                       VSIFTellL(f) + ptrCurLine + 1 - (szLineBuffer + nRead),
+                       SEEK_SET) != 0 )
+            return BNA_LINE_EOF;
     }
     *ptrCurLine = 0;
 
@@ -218,14 +231,12 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
                              int verbose,
                              BNAFeatureType interestFeatureType)
 {
-    BNARecord* record;
-    char c;
-    int inQuotes = FALSE;
+    bool inQuotes = false;
     int numField = 0;
     char* ptrBeginningOfNumber = NULL;
-    int exponentFound = 0;
-    int exponentSignFound = 0;
-    int dotFound = 0;
+    bool exponentFound = false;
+    bool exponentSignFound = false;
+    bool dotFound = false;
     int numChar = 0;
     const char* detailedErrorMsg = NULL;
     BNAFeatureType currentFeatureType = BNA_UNKNOWN;
@@ -234,10 +245,11 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
     int  tmpBufferLength[NB_MAX_BNA_IDS] = {0, 0, 0};
     char szLineBuffer[LINE_BUFFER_SIZE + 1];
 
-    record = (BNARecord*)CPLMalloc(sizeof(BNARecord));
+    BNARecord* record
+        = static_cast<BNARecord *>( CPLMalloc(sizeof(BNARecord)) );
     memset(record, 0, sizeof(BNARecord));
 
-    while (TRUE)
+    while( true )
     {
       numChar = 0;
       (*curLine)++;
@@ -254,15 +266,16 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
       }
 
       char* ptrCurLine = szLineBuffer;
-      const char* ptrBeginLine = szLineBuffer;
 
       if (*ptrCurLine == 0)
         continue;
 
+      const char* ptrBeginLine = szLineBuffer;
+
       while(1)
       {
-        numChar = ptrCurLine - ptrBeginLine;
-        c = *ptrCurLine;
+        numChar = static_cast<int>(ptrCurLine - ptrBeginLine);
+        char c = *ptrCurLine;
         if (c == 0) c = 10;
         if (inQuotes)
         {
@@ -275,7 +288,7 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
           {
             if (tmpBufferLength[numField] == TMP_BUFFER_SIZE)
             {
-              detailedErrorMsg = TOO_LONG_ID; 
+              detailedErrorMsg = TOO_LONG_ID;
               goto error;
             }
             tmpBuffer[numField][tmpBufferLength[numField]++] = c;
@@ -290,7 +303,7 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
           {
             if (tmpBufferLength[numField] == TMP_BUFFER_SIZE)
             {
-              detailedErrorMsg = TOO_LONG_ID; 
+              detailedErrorMsg = TOO_LONG_ID;
               goto error;
             }
             tmpBuffer[numField][tmpBufferLength[numField]++] = c;
@@ -303,7 +316,7 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
             do
             {
               ptrCurLine++;
-              numChar = ptrCurLine - ptrBeginLine;
+              numChar = static_cast<int>(ptrCurLine - ptrBeginLine);
               c = *ptrCurLine;
               if (!(c == ' ' || c == '\t'))
                 break;
@@ -328,9 +341,10 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
               {
                 if (verbose)
                 {
-                  CPLError(CE_Warning, CPLE_AppDefined, 
-                           "At line %d, at char %d, extra data will be ignored!\n",
-                           *curLine, numChar+1);
+                  CPLError( CE_Warning, CPLE_AppDefined,
+                            "At line %d, at char %d, extra data will be "
+                            "ignored",
+                            *curLine, numChar+1 );
                 }
               }
               *ok = 1;
@@ -338,9 +352,9 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
             }
 
             ptrBeginningOfNumber = NULL;
-            exponentFound = 0;
-            exponentSignFound = 0;
-            dotFound = 0;
+            exponentFound = false;
+            exponentSignFound = false;
+            dotFound = false;
             numField++;
 
             if (c == 10)
@@ -348,7 +362,7 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
 
             if (c != ',')
             {
-              /* don't increment ptrCurLine */
+              // Do not increment ptrCurLine.
               continue;
             }
           }
@@ -377,7 +391,8 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
 
           if (numField == 0)
           {
-            /* Maybe not so mandatory.. Atlas MapMaker(TM) exports BNA files with empty primaryID */
+            // Maybe not so mandatory...
+            // Atlas MapMaker(TM) exports BNA files with empty primaryID.
             /*
             if (record->primaryID == NULL || *(record->primaryID) == 0)
             {
@@ -395,7 +410,8 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
               goto error;
             }
             nCoords = atoi(ptrBeginningOfNumber);
-            if (nCoords == 0 || nCoords == -1)
+            if (nCoords == 0 || nCoords == -1 || nCoords >= INT_MAX / 16 ||
+                nCoords <= INT_MIN / 16 )
             {
               detailedErrorMsg = INVALID_GEOMETRY_TYPE;
               goto error;
@@ -426,8 +442,7 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
             if (interestFeatureType == BNA_READ_ALL ||
                 interestFeatureType == currentFeatureType)
             {
-              int i;
-              for(i=0;i<NB_MAX_BNA_IDS;i++)
+              for( int i=0; i < NB_MAX_BNA_IDS; i++ )
               {
                 if (tmpBufferLength[i] && tmpBuffer[i][0])
                 {
@@ -438,7 +453,7 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
               }
 
               record->tabCoords =
-                  (double(*)[2])VSIMalloc(record->nCoords * 2 * sizeof(double));
+                  (double(*)[2])VSI_MALLOC2_VERBOSE(record->nCoords, 2 * sizeof(double));
               if (record->tabCoords == NULL)
               {
                   detailedErrorMsg = NOT_ENOUGH_MEMORY;
@@ -465,15 +480,17 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
               if (pszComma)
                   *pszComma = ',';
             }
-            if (numField == NB_MIN_BNA_IDS + 1 + nbExtraId + 2 * record->nCoords - 1)
+            if (numField ==
+                NB_MIN_BNA_IDS + 1 + nbExtraId + 2 * record->nCoords - 1)
             {
               if (c != 10)
               {
                 if (verbose)
                 {
-                  CPLError(CE_Warning, CPLE_AppDefined, 
-                           "At line %d, at char %d, extra data will be ignored!\n",
-                           *curLine, numChar+1);
+                  CPLError( CE_Warning, CPLE_AppDefined,
+                            "At line %d, at char %d, extra data will be "
+                            "ignored",
+                            *curLine, numChar+1);
                 }
               }
               *ok = 1;
@@ -482,9 +499,9 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
           }
 
           ptrBeginningOfNumber = NULL;
-          exponentFound = 0;
-          exponentSignFound = 0;
-          dotFound = 0;
+          exponentFound = false;
+          exponentSignFound = false;
+          dotFound = false;
           numField++;
 
           if (c == 10)
@@ -496,7 +513,8 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
           {
             inQuotes = TRUE;
           }
-          else if (numField >= NB_MIN_BNA_IDS && currentFeatureType == BNA_UNKNOWN)
+          else if ( numField >= NB_MIN_BNA_IDS
+                    && currentFeatureType == BNA_UNKNOWN)
           {
             if (ptrBeginningOfNumber == NULL)
             {
@@ -522,7 +540,8 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
         }
         else
         {
-          if (numField < NB_MIN_BNA_IDS || (numField == NB_MIN_BNA_IDS + nbExtraId - 1))
+          if( numField < NB_MIN_BNA_IDS
+              || (numField == NB_MIN_BNA_IDS + nbExtraId - 1) )
           {
             detailedErrorMsg = STRING_EXPECTED;
             goto error;
@@ -560,7 +579,7 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
                 detailedErrorMsg = BAD_FLOAT_NUMBER_FORMAT;
                 goto error;
               }
-              dotFound = 1;
+              dotFound = true;
             }
             else if (c == '+' || c == '-')
             {
@@ -569,11 +588,11 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
               }
               else if (exponentFound)
               {
-                if (exponentSignFound == 0 && ptrCurLine > ptrBeginLine &&
+                if (!exponentSignFound && ptrCurLine > ptrBeginLine &&
                     (ptrCurLine[-1] == 'e' || ptrCurLine[-1] == 'E' ||
                      ptrCurLine[-1] == 'd' || ptrCurLine[-1] == 'D'))
                 {
-                  exponentSignFound = 1;
+                  exponentSignFound = true;
                 }
                 else
                 {
@@ -591,12 +610,12 @@ BNARecord* BNA_GetNextRecord(VSILFILE* f,
             {
               if (ptrBeginningOfNumber == NULL ||
                   !(ptrCurLine[-1] >= '0' && ptrCurLine[-1] <= '9') ||
-                  exponentFound == 1)
+                  exponentFound )
               {
                 detailedErrorMsg = BAD_FLOAT_NUMBER_FORMAT;
                 goto error;
               }
-              exponentFound = 1;
+              exponentFound = true;
             }
             else
             {
@@ -628,15 +647,15 @@ error:
     {
       if (detailedErrorMsg)
       {
-        CPLError(CE_Failure, CPLE_AppDefined, 
-                "Parsing failed at line %d, at char %d : %s!\n",
-                 *curLine, numChar+1, detailedErrorMsg);
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "Parsing failed at line %d, at char %d : %s",
+                  *curLine, numChar+1, detailedErrorMsg );
       }
       else
       {
-        CPLError(CE_Failure, CPLE_AppDefined, 
-                "Parsing failed at line %d, at char %d!\n",
-                 *curLine, numChar+1);
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "Parsing failed at line %d, at char %d",
+                  *curLine, numChar+1 );
       }
     }
     BNA_FreeRecord(record);
