@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrcouchdblayer.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogrcouchdblayer.cpp 32247 2015-12-19 07:05:52Z goatbar $
  *
  * Project:  CouchDB Translator
  * Purpose:  Implements OGRCouchDBLayer class.
@@ -31,30 +31,22 @@
 #include "ogrgeojsonreader.h"
 #include "ogrgeojsonutils.h"
 
-CPL_CVSID("$Id: ogrcouchdblayer.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: ogrcouchdblayer.cpp 32247 2015-12-19 07:05:52Z goatbar $");
 
 /************************************************************************/
 /*                            OGRCouchDBLayer()                             */
 /************************************************************************/
 
-OGRCouchDBLayer::OGRCouchDBLayer(OGRCouchDBDataSource* poDS)
-
-{
-    this->poDS = poDS;
-
-    nNextInSeq = 0;
-
-    poSRS = NULL;
-
-    poFeatureDefn = NULL;
-
-    nOffset = 0;
-    bEOF = FALSE;
-
-    poFeatures = NULL;
-
-    bGeoJSONDocument = TRUE;
-}
+OGRCouchDBLayer::OGRCouchDBLayer(OGRCouchDBDataSource* poDSIn) :
+    poDS(poDSIn),
+    poFeatureDefn(NULL),
+    poSRS(NULL),
+    nNextInSeq(0),
+    nOffset(0),
+    bEOF(FALSE),
+    poFeatures(NULL),
+    bGeoJSONDocument(TRUE)
+{}
 
 /************************************************************************/
 /*                            ~OGRCouchDBLayer()                            */
@@ -104,15 +96,15 @@ OGRFeature *OGRCouchDBLayer::GetNextFeature()
 
     GetLayerDefn();
 
-    while(TRUE)
+    while( true )
     {
         if (nNextInSeq < nOffset ||
-            nNextInSeq >= nOffset + (int)aoFeatures.size())
+            nNextInSeq >= nOffset + static_cast<int>(aoFeatures.size()))
         {
             if (bEOF)
                 return NULL;
 
-            nOffset += aoFeatures.size();
+            nOffset += static_cast<int>(aoFeatures.size());
             if (!FetchNextRows())
                 return NULL;
         }
@@ -156,12 +148,12 @@ OGRFeature *OGRCouchDBLayer::GetNextRawFeature()
 /*                          SetNextByIndex()                            */
 /************************************************************************/
 
-OGRErr OGRCouchDBLayer::SetNextByIndex( long nIndex )
+OGRErr OGRCouchDBLayer::SetNextByIndex( GIntBig nIndex )
 {
-    if (nIndex < 0)
+    if (nIndex < 0 || nIndex >= INT_MAX )
         return OGRERR_FAILURE;
     bEOF = FALSE;
-    nNextInSeq = nIndex;
+    nNextInSeq = (int)nIndex;
     return OGRERR_NONE;
 }
 
@@ -192,7 +184,7 @@ OGRFeature* OGRCouchDBLayer::TranslateFeature( json_object* poObj )
     const char* pszId = json_object_get_string(poId);
     if (pszId)
     {
-        poFeature->SetField(_ID_FIELD, pszId);
+        poFeature->SetField(COUCHDB_ID_FIELD, pszId);
 
         int nFID = atoi(pszId);
         const char* pszFID = CPLSPrintf("%09d", nFID);
@@ -203,7 +195,7 @@ OGRFeature* OGRCouchDBLayer::TranslateFeature( json_object* poObj )
     json_object* poRev = json_object_object_get(poObj, "_rev");
     const char* pszRev = json_object_get_string(poRev);
     if (pszRev)
-        poFeature->SetField(_REV_FIELD, pszRev);
+        poFeature->SetField(COUCHDB_REV_FIELD, pszRev);
 
 /* -------------------------------------------------------------------- */
 /*      Translate GeoJSON "properties" object to feature attributes.    */
@@ -366,8 +358,9 @@ void OGRCouchDBLayer::BuildFeatureDefnFromDoc(json_object* poDoc)
         {
             if( -1 == poFeatureDefn->GetFieldIndex( it.key ) )
             {
+                OGRFieldSubType eSubType;
                 OGRFieldDefn fldDefn( it.key,
-                    GeoJSONPropertyToFieldType( it.val ) );
+                    GeoJSONPropertyToFieldType( it.val, eSubType ) );
                 poFeatureDefn->AddFieldDefn( &fldDefn );
             }
         }
@@ -383,8 +376,9 @@ void OGRCouchDBLayer::BuildFeatureDefnFromDoc(json_object* poDoc)
                 strcmp(it.key, "geometry") != 0 &&
                 -1 == poFeatureDefn->GetFieldIndex( it.key ) )
             {
+                OGRFieldSubType eSubType;
                 OGRFieldDefn fldDefn( it.key,
-                    GeoJSONPropertyToFieldType( it.val ) );
+                    GeoJSONPropertyToFieldType( it.val, eSubType ) );
                 poFeatureDefn->AddFieldDefn( &fldDefn );
             }
         }
@@ -524,7 +518,7 @@ int OGRCouchDBLayer::FetchNextRowsAnalyseDocs(json_object* poAnswerObj)
 
         json_object* poId = json_object_object_get(poDoc, "_id");
         const char* pszId = json_object_get_string(poId);
-        if (pszId != NULL && strncmp(pszId, "_design/", 8) != 0)
+        if (pszId != NULL && !STARTS_WITH(pszId, "_design/"))
         {
             aoFeatures.push_back(poDoc);
         }
@@ -536,3 +530,13 @@ int OGRCouchDBLayer::FetchNextRowsAnalyseDocs(json_object* poAnswerObj)
 
     return TRUE;
 }
+
+/************************************************************************/
+/*              	     GetSpatialRef()                            */
+/************************************************************************/
+
+OGRSpatialReference* OGRCouchDBLayer::GetSpatialRef()
+{
+    GetLayerDefn();
+    return poSRS;
+}
