@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: dgnfloat.cpp 10646 2007-01-18 02:38:10Z warmerdam $
+ * $Id: dgnfloat.cpp 33713 2016-03-12 17:41:57Z goatbar $
  *
  * Project:  Microstation DGN Access Library
  * Purpose:  Functions for translating DGN floats into IEEE floats.
@@ -29,7 +29,7 @@
 
 #include "dgnlibp.h"
 
-CPL_CVSID("$Id: dgnfloat.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
+CPL_CVSID("$Id: dgnfloat.cpp 33713 2016-03-12 17:41:57Z goatbar $");
 
 typedef struct dbl {
     GUInt32 hi;
@@ -82,7 +82,7 @@ void    DGN2IEEEDouble(void * dbl)
     sign         = dt.hi & 0x80000000;
 
 /* -------------------------------------------------------------------- */
-/*      Adjust the exponent so that we may work with it                 */      
+/*      Adjust the exponent so that we may work with it                 */
 /* -------------------------------------------------------------------- */
     exponent = dt.hi >> 23;
     exponent = exponent & 0x000000ff;
@@ -94,7 +94,7 @@ void    DGN2IEEEDouble(void * dbl)
 /*      Save the bits that we are discarding so we can round properly   */
 /* -------------------------------------------------------------------- */
     rndbits = dt.lo & 0x00000007;
-        
+
     dt.lo = dt.lo >> 3;
     dt.lo = (dt.lo & 0x1fffffff) | (dt.hi << 29);
 
@@ -117,14 +117,8 @@ void    DGN2IEEEDouble(void * dbl)
     src = (unsigned char *) &dt;
     dest = (unsigned char *) dbl;
 
-    dest[0] = src[4];
-    dest[1] = src[5];
-    dest[2] = src[6];
-    dest[3] = src[7];
-    dest[4] = src[0];
-    dest[5] = src[1];
-    dest[6] = src[2];
-    dest[7] = src[3];
+    memcpy(dest + 0, src + 4, 4);
+    memcpy(dest + 4, src + 0, 4);
 #else
     memcpy( dbl, &dt, 8 );
 #endif
@@ -141,7 +135,7 @@ void    IEEE2DGNDouble(void * dbl)
     GInt32      exponent;
     GInt32      sign;
     GByte       *src,*dest;
-        
+
 #ifdef CPL_LSB
     src  = (GByte *) dbl;
     dest = (GByte *) &dt;
@@ -175,7 +169,7 @@ void    IEEE2DGNDouble(void * dbl)
     {
         dest = (GByte *) dbl;
 
-        if (sign) 
+        if (sign)
             dest[1] = 0xff;
         else
             dest[1] = 0x7f;
@@ -189,7 +183,7 @@ void    IEEE2DGNDouble(void * dbl)
         dest[7] = 0xff;
 
         return;
-    }   
+    }
 
 /* -------------------------------------------------------------------- */
 /*      In the case of of underflow return zero                         */
@@ -219,7 +213,7 @@ void    IEEE2DGNDouble(void * dbl)
         dt.hi = dt.hi | (dt.lo >> 29);
         dt.hi = dt.hi & 0x007fffff;
         dt.hi = dt.hi | (exponent << 23) | sign;
-            
+
         dt.lo = dt.lo << 3;
     }
 
@@ -230,14 +224,10 @@ void    IEEE2DGNDouble(void * dbl)
     dest = (GByte *) dbl;
 
 #ifdef CPL_LSB
-    dest[2] = src[0];
-    dest[3] = src[1];
-    dest[0] = src[2];
-    dest[1] = src[3];
-    dest[6] = src[4];
-    dest[7] = src[5];
-    dest[4] = src[6];
-    dest[5] = src[7];
+    memcpy(dest + 2, src + 0, 2);
+    memcpy(dest + 0, src + 2, 2);
+    memcpy(dest + 6, src + 4, 2);
+    memcpy(dest + 4, src + 6, 2);
 #else
     dest[1] = src[0];
     dest[0] = src[1];
