@@ -1,8 +1,8 @@
 /******************************************************************************
- * $Id: dgnlibp.h 22381 2011-05-16 21:14:22Z rouault $
+ * $Id: dgnlibp.h 33713 2016-03-12 17:41:57Z goatbar $
  *
  * Project:  Microstation DGN Access Library
- * Purpose:  Internal (privatE) datastructures, and prototypes for DGN Access 
+ * Purpose:  Internal (privatE) datastructures, and prototypes for DGN Access
  *           Library.
  * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
@@ -28,16 +28,11 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _DGNLIBP_H_INCLUDED
-#define _DGNLIBP_H_INCLUDED
+#ifndef DGNLIBP_H_INCLUDED
+#define DGNLIBP_H_INCLUDED
 
 #include "dgnlib.h"
 
-
-#ifndef PI
-#define PI  3.1415926535897932384626433832795
-#endif
-
 typedef struct {
     FILE        *fp;
     int         next_element_id;
@@ -90,7 +85,7 @@ typedef struct {
                         + ((p)[3] << 8) \
                         + ((p)[1] << 24) \
                         + ((p)[0] << 16))
-#define DGN_WRITE_INT32( n, p ) { GInt32 nMacroWork = (n);                   \
+#define DGN_WRITE_INT32( n, p ) { GInt32 nMacroWork = (GInt32)(n);                   \
  ((unsigned char *)p)[0] = (unsigned char)((nMacroWork & 0x00ff0000) >> 16); \
  ((unsigned char *)p)[1] = (unsigned char)((nMacroWork & 0xff000000) >> 24); \
  ((unsigned char *)p)[2] = (unsigned char)((nMacroWork & 0x000000ff) >> 0);  \
@@ -108,4 +103,4 @@ void DGNAsciiToRad50( const char *str, unsigned short *rad50 );
 void DGNSpatialFilterToUOR( DGNInfo *);
 int  DGNLoadRawElement( DGNInfo *psDGN, int *pnType, int *pnLevel );
 
-#endif /* ndef _DGNLIBP_H_INCLUDED */
+#endif /* ndef DGNLIBP_H_INCLUDED */
