@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrdgndatasource.cpp 23431 2011-11-27 15:02:24Z rouault $
+ * $Id: ogrdgndatasource.cpp 33105 2016-01-23 15:27:32Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRPGDataSource class.
@@ -31,21 +31,19 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrdgndatasource.cpp 23431 2011-11-27 15:02:24Z rouault $");
+CPL_CVSID("$Id: ogrdgndatasource.cpp 33105 2016-01-23 15:27:32Z rouault $");
 
 /************************************************************************/
 /*                         OGRDGNDataSource()                           */
 /************************************************************************/
 
-OGRDGNDataSource::OGRDGNDataSource()
-
-{
-    papoLayers = NULL;
-    nLayers = 0;
-    hDGN = NULL;
-    pszName = NULL;
-    papszOptions = NULL;
-}
+OGRDGNDataSource::OGRDGNDataSource() :
+    papoLayers(NULL),
+    nLayers(0),
+    pszName(NULL),
+    hDGN(NULL),
+    papszOptions(NULL)
+{}
 
 /************************************************************************/
 /*                        ~OGRDGNDataSource()                           */
@@ -56,7 +54,7 @@ OGRDGNDataSource::~OGRDGNDataSource()
 {
     for( int i = 0; i < nLayers; i++ )
         delete papoLayers[i];
-    
+
     CPLFree( papoLayers );
     CPLFree( pszName );
     CSLDestroy( papszOptions );
@@ -69,8 +67,8 @@ OGRDGNDataSource::~OGRDGNDataSource()
 /*                                Open()                                */
 /************************************************************************/
 
-int OGRDGNDataSource::Open( const char * pszNewName, 
-                            int bTestOpen, 
+int OGRDGNDataSource::Open( const char * pszNewName,
+                            int bTestOpen,
                             int bUpdate )
 
 {
@@ -83,15 +81,14 @@ int OGRDGNDataSource::Open( const char * pszNewName,
 /* -------------------------------------------------------------------- */
     if( bTestOpen )
     {
-        FILE    *fp;
-        GByte   abyHeader[512];
-        int     nHeaderBytes = 0;
 
-        fp = VSIFOpen( pszNewName, "rb" );
+        FILE *fp = VSIFOpen( pszNewName, "rb" );
         if( fp == NULL )
             return FALSE;
 
-        nHeaderBytes = (int) VSIFRead( abyHeader, 1, sizeof(abyHeader), fp );
+        GByte abyHeader[512];
+        const int nHeaderBytes = static_cast<int>(
+            VSIFRead( abyHeader, 1, sizeof(abyHeader), fp ) );
 
         VSIFClose( fp );
 
@@ -109,8 +106,8 @@ int OGRDGNDataSource::Open( const char * pszNewName,
     if( hDGN == NULL )
     {
         if( !bTestOpen )
-            CPLError( CE_Failure, CPLE_AppDefined, 
-                      "Unable to open %s as a Microstation .dgn file.\n", 
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "Unable to open %s as a Microstation .dgn file.",
                       pszNewName );
         return FALSE;
     }
@@ -118,18 +115,16 @@ int OGRDGNDataSource::Open( const char * pszNewName,
 /* -------------------------------------------------------------------- */
 /*      Create the layer object.                                        */
 /* -------------------------------------------------------------------- */
-    OGRDGNLayer *poLayer;
-
-    poLayer = new OGRDGNLayer( "elements", hDGN, bUpdate );
+    OGRDGNLayer *poLayer = new OGRDGNLayer( "elements", hDGN, bUpdate );
     pszName = CPLStrdup( pszNewName );
 
 /* -------------------------------------------------------------------- */
 /*      Add layer to data source layer list.                            */
 /* -------------------------------------------------------------------- */
-    papoLayers = (OGRDGNLayer **)
-        CPLRealloc( papoLayers,  sizeof(OGRDGNLayer *) * (nLayers+1) );
+    papoLayers = static_cast<OGRDGNLayer **>(
+        CPLRealloc( papoLayers,  sizeof(OGRDGNLayer *) * (nLayers+1) ) );
     papoLayers[nLayers++] = poLayer;
-    
+
     return TRUE;
 }
 
@@ -142,8 +137,8 @@ int OGRDGNDataSource::TestCapability( const char * pszCap )
 {
     if( EQUAL(pszCap,ODsCCreateLayer) )
         return TRUE;
-    else
-        return FALSE;
+
+    return FALSE;
 }
 
 /************************************************************************/
@@ -155,8 +150,8 @@ OGRLayer *OGRDGNDataSource::GetLayer( int iLayer )
 {
     if( iLayer < 0 || iLayer >= nLayers )
         return NULL;
-    else
-        return papoLayers[iLayer];
+
+    return papoLayers[iLayer];
 }
 
 
@@ -165,44 +160,37 @@ OGRLayer *OGRDGNDataSource::GetLayer( int iLayer )
 /*                                                                      */
 /*      Called by OGRDGNDriver::Create() method to setup a stub         */
 /*      OGRDataSource object without the associated file created        */
-/*      yet.  It will be created by the CreateLayer() call.             */
+/*      yet.  It will be created by theICreateLayer() call.             */
 /************************************************************************/
 
-int OGRDGNDataSource::PreCreate( const char *pszFilename, 
-                                 char **papszOptions )
+int OGRDGNDataSource::PreCreate( const char *pszFilename,
+                                 char **papszOptionsIn )
 
 {
-    this->papszOptions = CSLDuplicate( papszOptions );
+    this->papszOptions = CSLDuplicate( papszOptionsIn );
     pszName = CPLStrdup( pszFilename );
 
     return TRUE;
 }
 
 /************************************************************************/
-/*                            CreateLayer()                             */
+/*                           ICreateLayer()                             */
 /************************************************************************/
 
-OGRLayer *OGRDGNDataSource::CreateLayer( const char *pszLayerName, 
-                                         OGRSpatialReference *poSRS, 
-                                         OGRwkbGeometryType eGeomType, 
+OGRLayer *OGRDGNDataSource::ICreateLayer( const char *pszLayerName,
+                                         OGRSpatialReference *poSRS,
+                                         OGRwkbGeometryType eGeomType,
                                          char **papszExtraOptions )
 
 {
-    const char *pszSeed, *pszMasterUnit = "m", *pszSubUnit = "cm";
-    const char *pszValue;
-    int nUORPerSU=1, nSUPerMU=100;
-    int nCreationFlags = 0, b3DRequested;
-    double dfOriginX = -21474836.0,  /* default origin centered on zero */
-           dfOriginY = -21474836.0,  /* with two decimals of precision */
-           dfOriginZ = -21474836.0;
-
 /* -------------------------------------------------------------------- */
 /*      Ensure only one layer gets created.                             */
 /* -------------------------------------------------------------------- */
     if( nLayers > 0 )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
-                  "DGN driver only supports one layer will all the elements in it." );
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "DGN driver only supports one layer will all the elements "
+                  "in it." );
         return NULL;
     }
 
@@ -210,11 +198,21 @@ OGRLayer *OGRDGNDataSource::CreateLayer( const char *pszLayerName,
 /*      If the coordinate system is geographic, we should use a         */
 /*      localized default origin and resolution.                        */
 /* -------------------------------------------------------------------- */
+    const char *pszMasterUnit = "m";
+    const char *pszSubUnit = "cm";
+
+    int nUORPerSU = 1;
+    int nSUPerMU = 100;
+
+    double dfOriginX = -21474836.0;  // Default origin centered on zero
+    double dfOriginY = -21474836.0;  // with two decimals of precision.
+    double dfOriginZ = -21474836.0;
+
     if( poSRS != NULL && poSRS->IsGeographic() )
     {
         dfOriginX = -200.0;
         dfOriginY = -200.0;
-        
+
         pszMasterUnit = "d";
         pszSubUnit = "s";
         nSUPerMU = 3600;
@@ -226,10 +224,12 @@ OGRLayer *OGRDGNDataSource::CreateLayer( const char *pszLayerName,
 /* -------------------------------------------------------------------- */
     papszOptions = CSLInsertStrings( papszOptions, 0, papszExtraOptions );
 
-    b3DRequested = CSLFetchBoolean( papszOptions, "3D", 
-                                    (((int) eGeomType) & wkb25DBit) );
+    const bool b3DRequested
+        = CPL_TO_BOOL(CSLFetchBoolean( papszOptions, "3D",
+                           wkbHasZ(eGeomType) ));
 
-    pszSeed = CSLFetchNameValue( papszOptions, "SEED" );
+    const char *pszSeed = CSLFetchNameValue( papszOptions, "SEED" );
+    int nCreationFlags = 0;
     if( pszSeed )
         nCreationFlags |= DGNCF_USE_SEED_ORIGIN | DGNCF_USE_SEED_UNITS;
     else if( b3DRequested )
@@ -239,23 +239,24 @@ OGRLayer *OGRDGNDataSource::CreateLayer( const char *pszLayerName,
 
     if( pszSeed == NULL )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "No seed file provided, and unable to find seed_2d.dgn." );
         return NULL;
     }
-    
+
     if( CSLFetchBoolean( papszOptions, "COPY_WHOLE_SEED_FILE", TRUE ) )
         nCreationFlags |= DGNCF_COPY_WHOLE_SEED_FILE;
     if( CSLFetchBoolean( papszOptions, "COPY_SEED_FILE_COLOR_TABLE", TRUE ) )
         nCreationFlags |= DGNCF_COPY_SEED_FILE_COLOR_TABLE;
-    
-    pszValue = CSLFetchNameValue( papszOptions, "MASTER_UNIT_NAME" );
+
+    const char *pszValue
+        = CSLFetchNameValue( papszOptions, "MASTER_UNIT_NAME" );
     if( pszValue != NULL )
     {
         nCreationFlags &= ~DGNCF_USE_SEED_UNITS;
         pszMasterUnit = pszValue;
     }
-    
+
     pszValue = CSLFetchNameValue( papszOptions, "SUB_UNIT_NAME" );
     if( pszValue != NULL )
     {
@@ -263,7 +264,6 @@ OGRLayer *OGRDGNDataSource::CreateLayer( const char *pszLayerName,
         pszSubUnit = pszValue;
     }
 
-
     pszValue = CSLFetchNameValue( papszOptions, "SUB_UNITS_PER_MASTER_UNIT" );
     if( pszValue != NULL )
     {
@@ -281,29 +281,29 @@ OGRLayer *OGRDGNDataSource::CreateLayer( const char *pszLayerName,
     pszValue = CSLFetchNameValue( papszOptions, "ORIGIN" );
     if( pszValue != NULL )
     {
-        char **papszTuple = CSLTokenizeStringComplex( pszValue, " ,", 
+        char **papszTuple = CSLTokenizeStringComplex( pszValue, " ,",
                                                       FALSE, FALSE );
 
         nCreationFlags &= ~DGNCF_USE_SEED_ORIGIN;
         if( CSLCount(papszTuple) == 3 )
         {
-            dfOriginX = atof(papszTuple[0]);
-            dfOriginY = atof(papszTuple[1]);
-            dfOriginZ = atof(papszTuple[2]);
+            dfOriginX = CPLAtof(papszTuple[0]);
+            dfOriginY = CPLAtof(papszTuple[1]);
+            dfOriginZ = CPLAtof(papszTuple[2]);
         }
         else if( CSLCount(papszTuple) == 2 )
         {
-            dfOriginX = atof(papszTuple[0]);
-            dfOriginY = atof(papszTuple[1]);
+            dfOriginX = CPLAtof(papszTuple[0]);
+            dfOriginY = CPLAtof(papszTuple[1]);
             dfOriginZ = 0.0;
         }
         else
         {
             CSLDestroy(papszTuple);
-            CPLError( CE_Failure, CPLE_AppDefined, 
+            CPLError( CE_Failure, CPLE_AppDefined,
                       "ORIGIN is not a valid 2d or 3d tuple.\n"
                       "Separate tuple values with comma." );
-            return FALSE;
+            return NULL;
         }
         CSLDestroy(papszTuple);
     }
@@ -311,8 +311,8 @@ OGRLayer *OGRDGNDataSource::CreateLayer( const char *pszLayerName,
 /* -------------------------------------------------------------------- */
 /*      Try creating the base file.                                     */
 /* -------------------------------------------------------------------- */
-    hDGN = DGNCreate( pszName, pszSeed, nCreationFlags, 
-                      dfOriginX, dfOriginY, dfOriginZ, 
+    hDGN = DGNCreate( pszName, pszSeed, nCreationFlags,
+                      dfOriginX, dfOriginY, dfOriginZ,
                       nSUPerMU, nUORPerSU, pszMasterUnit, pszSubUnit );
     if( hDGN == NULL )
         return NULL;
@@ -320,16 +320,14 @@ OGRLayer *OGRDGNDataSource::CreateLayer( const char *pszLayerName,
 /* -------------------------------------------------------------------- */
 /*      Create the layer object.                                        */
 /* -------------------------------------------------------------------- */
-    OGRDGNLayer *poLayer;
-
-    poLayer = new OGRDGNLayer( pszLayerName, hDGN, TRUE );
+    OGRDGNLayer *poLayer = new OGRDGNLayer( pszLayerName, hDGN, TRUE );
 
 /* -------------------------------------------------------------------- */
 /*      Add layer to data source layer list.                            */
 /* -------------------------------------------------------------------- */
-    papoLayers = (OGRDGNLayer **)
-        CPLRealloc( papoLayers,  sizeof(OGRDGNLayer *) * (nLayers+1) );
+    papoLayers = static_cast<OGRDGNLayer **>(
+        CPLRealloc( papoLayers,  sizeof(OGRDGNLayer *) * (nLayers+1) ) );
     papoLayers[nLayers++] = poLayer;
-    
+
     return poLayer;
 }
