@@ -1,9 +1,9 @@
 /******************************************************************************
- * $Id: ogrdxf_hatch.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogrdxf_hatch.cpp 33713 2016-03-12 17:41:57Z goatbar $
  *
  * Project:  DXF Translator
  * Purpose:  Implements translation support for HATCH elements as part
- *           of the OGRDXFLayer class.  
+ *           of the OGRDXFLayer class.
  * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
  ******************************************************************************
@@ -35,11 +35,7 @@
 
 #include "ogrdxf_polyline_smooth.h"
 
-CPL_CVSID("$Id: ogrdxf_hatch.cpp 27044 2014-03-16 23:41:27Z rouault $");
-
-#ifndef PI
-#define PI  3.14159265358979323846
-#endif 
+CPL_CVSID("$Id: ogrdxf_hatch.cpp 33713 2016-03-12 17:41:57Z goatbar $");
 
 /************************************************************************/
 /*                           TranslateHATCH()                           */
@@ -57,7 +53,7 @@ OGRFeature *OGRDXFLayer::TranslateHATCH()
     OGRFeature *poFeature = new OGRFeature( poFeatureDefn );
 
     CPLString osHatchPattern;
-    int nFillFlag = 0;
+    /* int nFillFlag = 0; */
     OGRGeometryCollection oGC;
 
     while( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) > 0 )
@@ -65,7 +61,7 @@ OGRFeature *OGRDXFLayer::TranslateHATCH()
         switch( nCode )
         {
           case 70:
-            nFillFlag = atoi(szLineBuf);
+            /* nFillFlag = atoi(szLineBuf); */
             break;
 
           case 2:
@@ -91,6 +87,12 @@ OGRFeature *OGRDXFLayer::TranslateHATCH()
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        delete poFeature;
+        return NULL;
+    }
 
     if( nCode == 0 )
         poDS->UnreadValue();
@@ -127,14 +129,14 @@ OGRFeature *OGRDXFLayer::TranslateHATCH()
     if( oStyleProperties.count("Color") > 0 )
         nColor = atoi(oStyleProperties["Color"]);
 
-    // Use layer color? 
+    // Use layer color?
     if( nColor < 1 || nColor > 255 )
     {
         const char *pszValue = poDS->LookupLayerProperty( osLayer, "Color" );
         if( pszValue != NULL )
             nColor = atoi(pszValue);
     }
-        
+
 /* -------------------------------------------------------------------- */
 /*      Setup the style string.                                         */
 /* -------------------------------------------------------------------- */
@@ -142,12 +144,12 @@ OGRFeature *OGRDXFLayer::TranslateHATCH()
     {
         CPLString osStyle;
         const unsigned char *pabyDXFColors = ACGetColorTable();
-        
+
         osStyle.Printf( "BRUSH(fc:#%02x%02x%02x)",
                         pabyDXFColors[nColor*3+0],
                         pabyDXFColors[nColor*3+1],
                         pabyDXFColors[nColor*3+2] );
-        
+
         poFeature->SetStyleString( osStyle );
     }
 
@@ -169,7 +171,10 @@ OGRErr OGRDXFLayer::CollectBoundaryPath( OGRGeometryCollection *poGC )
 /* -------------------------------------------------------------------- */
     nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf));
     if( nCode != 92 )
+    {
+        DXF_LAYER_READER_ERROR();
         return OGRERR_FAILURE;
+    }
 
     int  nBoundaryPathType = atoi(szLineBuf);
 
@@ -188,10 +193,13 @@ OGRErr OGRDXFLayer::CollectBoundaryPath( OGRGeometryCollection *poGC )
 /* -------------------------------------------------------------------- */
     nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf));
     if( nCode != 93 )
+    {
+        DXF_LAYER_READER_ERROR();
         return OGRERR_FAILURE;
+    }
 
     int nEdgeCount = atoi(szLineBuf);
-    
+
 /* -------------------------------------------------------------------- */
 /*      Loop reading edges.                                             */
 /* -------------------------------------------------------------------- */
@@ -209,10 +217,13 @@ OGRErr OGRDXFLayer::CollectBoundaryPath( OGRGeometryCollection *poGC )
 
         nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf));
         if( nCode != 72 )
+        {
+            DXF_LAYER_READER_ERROR();
             return OGRERR_FAILURE;
+        }
 
         int nEdgeType = atoi(szLineBuf);
-        
+
 /* -------------------------------------------------------------------- */
 /*      Process a line edge.                                            */
 /* -------------------------------------------------------------------- */
@@ -223,22 +234,22 @@ OGRErr OGRDXFLayer::CollectBoundaryPath( OGRGeometryCollection *poGC )
             double dfEndX;
             double dfEndY;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 10 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 10 )
                 dfStartX = CPLAtof(szLineBuf);
             else
                 break;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 20 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 20 )
                 dfStartY = CPLAtof(szLineBuf);
             else
                 break;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 11 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 11 )
                 dfEndX = CPLAtof(szLineBuf);
             else
                 break;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 21 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 21 )
                 dfEndY = CPLAtof(szLineBuf);
             else
                 break;
@@ -262,27 +273,27 @@ OGRErr OGRDXFLayer::CollectBoundaryPath( OGRGeometryCollection *poGC )
             double dfEndAngle;
             int    bCounterClockwise = FALSE;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 10 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 10 )
                 dfCenterX = CPLAtof(szLineBuf);
             else
                 break;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 20 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 20 )
                 dfCenterY = CPLAtof(szLineBuf);
             else
                 break;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 40 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 40 )
                 dfRadius = CPLAtof(szLineBuf);
             else
                 break;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 50 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 50 )
                 dfStartAngle = CPLAtof(szLineBuf);
             else
                 break;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 51 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 51 )
                 dfEndAngle = CPLAtof(szLineBuf);
             else
                 break;
@@ -291,16 +302,18 @@ OGRErr OGRDXFLayer::CollectBoundaryPath( OGRGeometryCollection *poGC )
                 bCounterClockwise = atoi(szLineBuf);
             else if (nCode >= 0)
                 poDS->UnreadValue();
+            else
+                break;
 
             if( dfStartAngle > dfEndAngle )
                 dfEndAngle += 360.0;
             if( bCounterClockwise )
             {
-                dfStartAngle *= -1; 
-                dfEndAngle *= -1; 
+                dfStartAngle *= -1;
+                dfEndAngle *= -1;
             }
 
-            OGRGeometry *poArc = OGRGeometryFactory::approximateArcAngles( 
+            OGRGeometry *poArc = OGRGeometryFactory::approximateArcAngles(
                 dfCenterX, dfCenterY, 0.0,
                 dfRadius, dfRadius, 0.0,
                 dfStartAngle, dfEndAngle, 0.0 );
@@ -325,37 +338,37 @@ OGRErr OGRDXFLayer::CollectBoundaryPath( OGRGeometryCollection *poGC )
             double dfRatio;
             int    bCounterClockwise = FALSE;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 10 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 10 )
                 dfCenterX = CPLAtof(szLineBuf);
             else
                 break;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 20 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 20 )
                 dfCenterY = CPLAtof(szLineBuf);
             else
                 break;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 11 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 11 )
                 dfMajorX = CPLAtof(szLineBuf);
             else
                 break;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 21 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 21 )
                 dfMajorY = CPLAtof(szLineBuf);
             else
                 break;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 40 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 40 )
                 dfRatio = CPLAtof(szLineBuf) / 100.0;
             else
                 break;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 50 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 50 )
                 dfStartAngle = CPLAtof(szLineBuf);
             else
                 break;
 
-            if( poDS->ReadValue(szLineBuf,sizeof(szLineBuf)) == 51 )
+            if( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) == 51 )
                 dfEndAngle = CPLAtof(szLineBuf);
             else
                 break;
@@ -364,21 +377,23 @@ OGRErr OGRDXFLayer::CollectBoundaryPath( OGRGeometryCollection *poGC )
                 bCounterClockwise = atoi(szLineBuf);
             else if (nCode >= 0)
                 poDS->UnreadValue();
+            else
+                break;
 
             if( dfStartAngle > dfEndAngle )
                 dfEndAngle += 360.0;
             if( bCounterClockwise )
             {
-                dfStartAngle *= -1; 
-                dfEndAngle *= -1; 
+                dfStartAngle *= -1;
+                dfEndAngle *= -1;
             }
 
             dfMajorRadius = sqrt( dfMajorX * dfMajorX + dfMajorY * dfMajorY );
             dfMinorRadius = dfMajorRadius * dfRatio;
 
-            dfRotation = -1 * atan2( dfMajorY, dfMajorX ) * 180 / PI;
+            dfRotation = -1 * atan2( dfMajorY, dfMajorX ) * 180 / M_PI;
 
-            OGRGeometry *poArc = OGRGeometryFactory::approximateArcAngles( 
+            OGRGeometry *poArc = OGRGeometryFactory::approximateArcAngles(
                 dfCenterX, dfCenterY, 0.0,
                 dfMajorRadius, dfMinorRadius, dfRotation,
                 dfStartAngle, dfEndAngle, 0.0 );
@@ -395,6 +410,12 @@ OGRErr OGRDXFLayer::CollectBoundaryPath( OGRGeometryCollection *poGC )
         }
     }
 
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        return OGRERR_FAILURE;
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Skip through source boundary objects if present.                */
 /* -------------------------------------------------------------------- */
@@ -500,8 +521,13 @@ OGRErr OGRDXFLayer::CollectPolylinePath( OGRGeometryCollection *poGC )
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        return OGRERR_FAILURE;
+    }
 
-    if( nCode != 10 && nCode != 20 && nCode != 42 && nCode >= 0)
+    if( nCode != 10 && nCode != 20 && nCode != 42 )
         poDS->UnreadValue();
 
     if( bHaveX && bHaveY )
@@ -510,6 +536,11 @@ OGRErr OGRDXFLayer::CollectPolylinePath( OGRGeometryCollection *poGC )
     if( bIsClosed )
         oSmoothPolyline.Close();
 
+    if(oSmoothPolyline.IsEmpty())
+    {
+        return OGRERR_FAILURE;
+    }
+
     poGC->addGeometryDirectly( oSmoothPolyline.Tesselate() );
 
 /* -------------------------------------------------------------------- */
@@ -534,5 +565,3 @@ OGRErr OGRDXFLayer::CollectPolylinePath( OGRGeometryCollection *poGC )
     }
     return OGRERR_NONE;
 }
-
-    
