@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrdxfdatasource.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: ogrdxfdatasource.cpp 33713 2016-03-12 17:41:57Z goatbar $
  *
  * Project:  DXF Translator
  * Purpose:  Implements OGRDXFDataSource class
@@ -32,17 +32,17 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrdxfdatasource.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: ogrdxfdatasource.cpp 33713 2016-03-12 17:41:57Z goatbar $");
 
 /************************************************************************/
 /*                          OGRDXFDataSource()                          */
 /************************************************************************/
 
-OGRDXFDataSource::OGRDXFDataSource()
-
-{
-    fp = NULL;
-}
+OGRDXFDataSource::OGRDXFDataSource() :
+    fp(NULL),
+    iEntitiesSectionOffset(0),
+    bInlineBlocks(FALSE)
+{ }
 
 /************************************************************************/
 /*                         ~OGRDXFDataSource()                          */
@@ -100,17 +100,14 @@ OGRLayer *OGRDXFDataSource::GetLayer( int iLayer )
 int OGRDXFDataSource::Open( const char * pszFilename, int bHeaderOnly )
 
 {
-    if( !EQUAL(CPLGetExtension(pszFilename),"dxf") )
-        return FALSE;
-
     osEncoding = CPL_ENC_ISO8859_1;
 
     osName = pszFilename;
 
-    bInlineBlocks = CSLTestBoolean(
+    bInlineBlocks = CPLTestBool(
         CPLGetConfigOption( "DXF_INLINE_BLOCKS", "TRUE" ) );
 
-    if( CSLTestBoolean(
+    if( CPLTestBool(
             CPLGetConfigOption( "DXF_HEADER_ONLY", "FALSE" ) ) )
         bHeaderOnly = TRUE;
 
@@ -122,7 +119,7 @@ int OGRDXFDataSource::Open( const char * pszFilename, int bHeaderOnly )
         return FALSE;
 
     oReader.Initialize( fp );
-    
+
 /* -------------------------------------------------------------------- */
 /*      Confirm we have a header section.                               */
 /* -------------------------------------------------------------------- */
@@ -133,7 +130,7 @@ int OGRDXFDataSource::Open( const char * pszFilename, int bHeaderOnly )
     if( ReadValue( szLineBuf ) != 0 || !EQUAL(szLineBuf,"SECTION") )
         return FALSE;
 
-    if( ReadValue( szLineBuf ) != 2 
+    if( ReadValue( szLineBuf ) != 2
         || (!EQUAL(szLineBuf,"HEADER") && !EQUAL(szLineBuf,"ENTITIES") && !EQUAL(szLineBuf,"TABLES")) )
         return FALSE;
 
@@ -146,8 +143,13 @@ int OGRDXFDataSource::Open( const char * pszFilename, int bHeaderOnly )
         if( CPLGetConfigOption( "DXF_ENCODING", NULL ) != NULL )
             osEncoding = CPLGetConfigOption( "DXF_ENCODING", NULL );
 
-        ReadTablesSection();
-        ReadValue(szLineBuf);
+        if( !ReadTablesSection() )
+            return FALSE;
+        if( ReadValue(szLineBuf) < 0 )
+        {
+            DXF_READER_ERROR();
+            return FALSE;
+        }
     }
 
 /* -------------------------------------------------------------------- */
@@ -156,21 +158,38 @@ int OGRDXFDataSource::Open( const char * pszFilename, int bHeaderOnly )
 /* -------------------------------------------------------------------- */
     else /* if( EQUAL(szLineBuf,"HEADER") ) */
     {
-        ReadHeaderSection();
-        ReadValue(szLineBuf);
+        if( !ReadHeaderSection() )
+            return FALSE;
+        if( ReadValue(szLineBuf) < 0 )
+        {
+            DXF_READER_ERROR();
+            return FALSE;
+        }
 
 /* -------------------------------------------------------------------- */
 /*      Process the CLASSES section, if present.                        */
 /* -------------------------------------------------------------------- */
         if( EQUAL(szLineBuf,"ENDSEC") )
-            ReadValue(szLineBuf);
+        {
+            if( ReadValue(szLineBuf) < 0 )
+            {
+                DXF_READER_ERROR();
+                return FALSE;
+            }
+        }
 
         if( EQUAL(szLineBuf,"SECTION") )
-            ReadValue(szLineBuf);
-        
+        {
+            if( ReadValue(szLineBuf) < 0 )
+            {
+                DXF_READER_ERROR();
+                return FALSE;
+            }
+        }
+
         if( EQUAL(szLineBuf,"CLASSES") )
         {
-            while( (nCode = ReadValue( szLineBuf,sizeof(szLineBuf) )) > -1 
+            while( (nCode = ReadValue( szLineBuf,sizeof(szLineBuf) )) > -1
                    && !EQUAL(szLineBuf,"ENDSEC") )
             {
                 //printf("C:%d/%s\n", nCode, szLineBuf );
@@ -181,15 +200,32 @@ int OGRDXFDataSource::Open( const char * pszFilename, int bHeaderOnly )
 /*      Process the TABLES section, if present.                         */
 /* -------------------------------------------------------------------- */
         if( EQUAL(szLineBuf,"ENDSEC") )
-            ReadValue(szLineBuf);
-        
+        {
+            if( ReadValue(szLineBuf) < 0 )
+            {
+                DXF_READER_ERROR();
+                return FALSE;
+            }
+        }
+
         if( EQUAL(szLineBuf,"SECTION") )
-            ReadValue(szLineBuf);
-        
+        {
+            if( ReadValue(szLineBuf) < 0 )
+            {
+                DXF_READER_ERROR();
+                return FALSE;
+            }
+        }
+
         if( EQUAL(szLineBuf,"TABLES") )
         {
-            ReadTablesSection();
-            ReadValue(szLineBuf);
+            if( !ReadTablesSection() )
+                return FALSE;
+            if( ReadValue(szLineBuf) < 0 )
+            {
+                DXF_READER_ERROR();
+                return FALSE;
+            }
         }
     }
 
@@ -211,15 +247,32 @@ int OGRDXFDataSource::Open( const char * pszFilename, int bHeaderOnly )
     if( !bEntitiesOnly )
     {
         if( EQUAL(szLineBuf,"ENDSEC") )
-            ReadValue(szLineBuf);
-        
+        {
+            if( ReadValue(szLineBuf) < 0 )
+            {
+                DXF_READER_ERROR();
+                return FALSE;
+            }
+        }
+
         if( EQUAL(szLineBuf,"SECTION") )
-            ReadValue(szLineBuf);
-        
+        {
+            if( ReadValue(szLineBuf) < 0 )
+            {
+                DXF_READER_ERROR();
+                return FALSE;
+            }
+        }
+
         if( EQUAL(szLineBuf,"BLOCKS") )
         {
-            ReadBlocksSection();
-            ReadValue(szLineBuf);
+            if( !ReadBlocksSection() )
+                return FALSE;
+            if( ReadValue(szLineBuf) < 0 )
+            {
+                DXF_READER_ERROR();
+                return FALSE;
+            }
         }
     }
 
@@ -230,10 +283,19 @@ int OGRDXFDataSource::Open( const char * pszFilename, int bHeaderOnly )
 /*      Now we are at the entities section, hopefully.  Confirm.        */
 /* -------------------------------------------------------------------- */
     if( EQUAL(szLineBuf,"SECTION") )
-        ReadValue(szLineBuf);
+    {
+        if( ReadValue(szLineBuf) < 0 )
+        {
+            DXF_READER_ERROR();
+            return FALSE;
+        }
+    }
 
     if( !EQUAL(szLineBuf,"ENTITIES") )
+    {
+        DXF_READER_ERROR();
         return FALSE;
+    }
 
     iEntitiesSectionOffset = oReader.iSrcBufferFileOffset + oReader.iSrcBufferOffset;
     apoLayers[0]->ResetReading();
@@ -245,13 +307,13 @@ int OGRDXFDataSource::Open( const char * pszFilename, int bHeaderOnly )
 /*                         ReadTablesSection()                          */
 /************************************************************************/
 
-void OGRDXFDataSource::ReadTablesSection()
+bool OGRDXFDataSource::ReadTablesSection()
 
 {
     char szLineBuf[257];
     int  nCode;
 
-    while( (nCode = ReadValue( szLineBuf, sizeof(szLineBuf) )) > -1 
+    while( (nCode = ReadValue( szLineBuf, sizeof(szLineBuf) )) > -1
            && !EQUAL(szLineBuf,"ENDSEC") )
     {
         // We are only interested in extracting tables.
@@ -260,6 +322,11 @@ void OGRDXFDataSource::ReadTablesSection()
 
         // Currently we are only interested in the LAYER table.
         nCode = ReadValue( szLineBuf, sizeof(szLineBuf) );
+        if( nCode < 0 )
+        {
+            DXF_READER_ERROR();
+            return false;
+        }
 
         if( nCode != 2 )
             continue;
@@ -270,20 +337,32 @@ void OGRDXFDataSource::ReadTablesSection()
                && !EQUAL(szLineBuf,"ENDTAB") )
         {
             if( nCode == 0 && EQUAL(szLineBuf,"LAYER") )
-                ReadLayerDefinition();
+            {
+                if( !ReadLayerDefinition() )
+                    return false;
+            }
             if( nCode == 0 && EQUAL(szLineBuf,"LTYPE") )
-                ReadLineTypeDefinition();
+            {
+                if( !ReadLineTypeDefinition() )
+                    return false;
+            }
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_READER_ERROR();
+        return false;
+    }
 
     CPLDebug( "DXF", "Read %d layer definitions.", (int) oLayerTable.size() );
+    return true;
 }
 
 /************************************************************************/
 /*                        ReadLayerDefinition()                         */
 /************************************************************************/
 
-void OGRDXFDataSource::ReadLayerDefinition()
+bool OGRDXFDataSource::ReadLayerDefinition()
 
 {
     char szLineBuf[257];
@@ -306,14 +385,14 @@ void OGRDXFDataSource::ReadLayerDefinition()
             oLayerProperties["Linetype"] = ACTextUnescape(szLineBuf,
                                                           GetEncoding());
             break;
-            
+
           case 62:
             oLayerProperties["Color"] = szLineBuf;
 
             if( atoi(szLineBuf) < 0 ) // Is layer off?
                 oLayerProperties["Hidden"] = "1";
             break;
-            
+
           case 70:
             oLayerProperties["Flags"] = szLineBuf;
             if( atoi(szLineBuf) & 0x01 ) // Is layer frozen?
@@ -329,12 +408,18 @@ void OGRDXFDataSource::ReadLayerDefinition()
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_READER_ERROR();
+        return false;
+    }
 
     if( oLayerProperties.size() > 0 )
         oLayerTable[osLayerName] = oLayerProperties;
-    
+
     if( nCode == 0 )
         UnreadValue();
+    return true;
 }
 
 /************************************************************************/
@@ -359,7 +444,7 @@ const char *OGRDXFDataSource::LookupLayerProperty( const char *pszLayer,
 /*                       ReadLineTypeDefinition()                       */
 /************************************************************************/
 
-void OGRDXFDataSource::ReadLineTypeDefinition()
+bool OGRDXFDataSource::ReadLineTypeDefinition()
 
 {
     char szLineBuf[257];
@@ -388,17 +473,23 @@ void OGRDXFDataSource::ReadLineTypeDefinition()
               osLineTypeDef += "g";
           }
           break;
-            
+
           default:
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_READER_ERROR();
+        return false;
+    }
 
     if( osLineTypeDef != "" )
         oLineTypeTable[osLineTypeName] = osLineTypeDef;
-    
+
     if( nCode == 0 )
         UnreadValue();
+    return true;
 }
 
 /************************************************************************/
@@ -418,37 +509,48 @@ const char *OGRDXFDataSource::LookupLineType( const char *pszName )
 /*                         ReadHeaderSection()                          */
 /************************************************************************/
 
-void OGRDXFDataSource::ReadHeaderSection()
+bool OGRDXFDataSource::ReadHeaderSection()
 
 {
     char szLineBuf[257];
     int  nCode;
 
-    while( (nCode = ReadValue( szLineBuf, sizeof(szLineBuf) )) > -1 
+    while( (nCode = ReadValue( szLineBuf, sizeof(szLineBuf) )) > -1
            && !EQUAL(szLineBuf,"ENDSEC") )
     {
         if( nCode != 9 )
             continue;
 
-        CPLString osName = szLineBuf;
+        CPLString l_osName = szLineBuf;
 
-        ReadValue( szLineBuf, sizeof(szLineBuf) );
+        if(ReadValue( szLineBuf, sizeof(szLineBuf) )<0)
+        {
+            DXF_READER_ERROR();
+            return false;
+        }
 
         CPLString osValue = szLineBuf;
 
-        oHeaderVariables[osName] = osValue;
+        oHeaderVariables[l_osName] = osValue;
+    }
+    if( nCode < 0 )
+    {
+        DXF_READER_ERROR();
+        return false;
     }
 
-    if (nCode != -1)
+    nCode = ReadValue( szLineBuf, sizeof(szLineBuf) );
+    if( nCode < 0 )
     {
-        nCode = ReadValue( szLineBuf, sizeof(szLineBuf) );
-        UnreadValue();
+        DXF_READER_ERROR();
+        return false;
     }
+    UnreadValue();
 
     /* Unusual DXF files produced by dxflib */
     /* such as http://www.ribbonsoft.com/library/architecture/plants/decd5.dxf */
     /* where there is a spurious ENDSEC in the middle of the header variables */
-    if (nCode == 9 && EQUALN(szLineBuf,"$", 1) )
+    if (nCode == 9 && STARTS_WITH_CI(szLineBuf, "$") )
     {
         while( (nCode = ReadValue( szLineBuf, sizeof(szLineBuf) )) > -1
             && !EQUAL(szLineBuf,"ENDSEC") )
@@ -456,17 +558,26 @@ void OGRDXFDataSource::ReadHeaderSection()
             if( nCode != 9 )
                 continue;
 
-            CPLString osName = szLineBuf;
+            CPLString l_osName = szLineBuf;
 
-            ReadValue( szLineBuf, sizeof(szLineBuf) );
+            if( ReadValue( szLineBuf, sizeof(szLineBuf) ) < 0 )
+            {
+                DXF_READER_ERROR();
+                return false;
+            }
 
             CPLString osValue = szLineBuf;
 
-            oHeaderVariables[osName] = osValue;
+            oHeaderVariables[l_osName] = osValue;
+        }
+        if( nCode < 0 )
+        {
+            DXF_READER_ERROR();
+            return false;
         }
     }
 
-    CPLDebug( "DXF", "Read %d header variables.", 
+    CPLDebug( "DXF", "Read %d header variables.",
               (int) oHeaderVariables.size() );
 
 /* -------------------------------------------------------------------- */
@@ -477,24 +588,25 @@ void OGRDXFDataSource::ReadHeaderSection()
 
     // not strictly accurate but works even without iconv.
     if( osCodepage == "ANSI_1252" )
-        osEncoding = CPL_ENC_ISO8859_1; 
-    else if( EQUALN(osCodepage,"ANSI_",5) )
+        osEncoding = CPL_ENC_ISO8859_1;
+    else if( STARTS_WITH_CI(osCodepage, "ANSI_") )
     {
         osEncoding = "CP";
         osEncoding += osCodepage + 5;
     }
     else
     {
-        // fallback to the default 
+        // fallback to the default
         osEncoding = CPL_ENC_ISO8859_1;
     }
-                                       
+
     if( CPLGetConfigOption( "DXF_ENCODING", NULL ) != NULL )
         osEncoding = CPLGetConfigOption( "DXF_ENCODING", NULL );
 
     if( osEncoding != CPL_ENC_ISO8859_1 )
-        CPLDebug( "DXF", "Treating DXF as encoding '%s', $DWGCODEPAGE='%s'", 
+        CPLDebug( "DXF", "Treating DXF as encoding '%s', $DWGCODEPAGE='%s'",
                   osEncoding.c_str(), osCodepage.c_str() );
+    return true;
 }
 
 /************************************************************************/
@@ -503,13 +615,13 @@ void OGRDXFDataSource::ReadHeaderSection()
 /*      Fetch a variable that came from the HEADER section.             */
 /************************************************************************/
 
-const char *OGRDXFDataSource::GetVariable( const char *pszName, 
+const char *OGRDXFDataSource::GetVariable( const char *pszName,
                                            const char *pszDefault )
 
 {
     if( oHeaderVariables.count(pszName) == 0 )
         return pszDefault;
-    else 
+    else
         return oHeaderVariables[pszName];
 }
 
@@ -540,7 +652,7 @@ void OGRDXFDataSource::AddStandardFields( OGRFeatureDefn *poFeatureDefn )
 
     if( !bInlineBlocks )
     {
-        OGRFieldDefn  oTextField( "BlockName", OFTString );
-        poFeatureDefn->AddFieldDefn( &oTextField );
+        OGRFieldDefn  oBlockNameField( "BlockName", OFTString );
+        poFeatureDefn->AddFieldDefn( &oBlockNameField );
     }
 }
