@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrdxflayer.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogrdxflayer.cpp 33981 2016-04-16 18:31:19Z rouault $
  *
  * Project:  DXF Translator
  * Purpose:  Implements OGRDXFLayer class.
@@ -33,24 +33,17 @@
 #include "ogrdxf_polyline_smooth.h"
 #include "ogr_api.h"
 
-CPL_CVSID("$Id: ogrdxflayer.cpp 27044 2014-03-16 23:41:27Z rouault $");
-
-#ifndef PI
-#define PI  3.14159265358979323846
-#endif 
+CPL_CVSID("$Id: ogrdxflayer.cpp 33981 2016-04-16 18:31:19Z rouault $");
 
 /************************************************************************/
 /*                            OGRDXFLayer()                             */
 /************************************************************************/
 
-OGRDXFLayer::OGRDXFLayer( OGRDXFDataSource *poDS )
-
+OGRDXFLayer::OGRDXFLayer( OGRDXFDataSource *poDSIn ) :
+    poDS(poDSIn),
+    poFeatureDefn(new OGRFeatureDefn( "entities" )),
+    iNextFID(0)
 {
-    this->poDS = poDS;
-
-    iNextFID = 0;
-
-    poFeatureDefn = new OGRFeatureDefn( "entities" );
     poFeatureDefn->Reference();
 
     poDS->AddStandardFields( poFeatureDefn );
@@ -63,6 +56,8 @@ OGRDXFLayer::OGRDXFLayer( OGRDXFDataSource *poDS )
         OGRFieldDefn  oBlockAngleField( "BlockAngle", OFTReal );
         poFeatureDefn->AddFieldDefn( &oBlockAngleField );
     }
+
+    SetDescription( poFeatureDefn->GetName() );
 }
 
 /************************************************************************/
@@ -76,7 +71,7 @@ OGRDXFLayer::~OGRDXFLayer()
     if( m_nFeaturesRead > 0 && poFeatureDefn != NULL )
     {
         CPLDebug( "DXF", "%d features read on layer '%s'.",
-                  (int) m_nFeaturesRead, 
+                  (int) m_nFeaturesRead,
                   poFeatureDefn->GetName() );
     }
 
@@ -117,17 +112,17 @@ void OGRDXFLayer::ResetReading()
 /*      or all entity types.                                            */
 /************************************************************************/
 
-void OGRDXFLayer::TranslateGenericProperty( OGRFeature *poFeature, 
+void OGRDXFLayer::TranslateGenericProperty( OGRFeature *poFeature,
                                             int nCode, char *pszValue )
 
 {
     switch( nCode )
     {
-      case 8: 
+      case 8:
         poFeature->SetField( "Layer", TextUnescape(pszValue) );
         break;
-            
-      case 100: 
+
+      case 100:
       {
           CPLString osSubClass = poFeature->GetFieldAsString("SubClasses");
           if( osSubClass.size() > 0 )
@@ -169,7 +164,7 @@ void OGRDXFLayer::TranslateGenericProperty( OGRFeature *poFeature,
           if( osAggregate.size() > 0 )
               osAggregate += " ";
           osAggregate += pszValue;
-            
+
           poFeature->SetField( "ExtendedEntity", osAggregate );
       }
       break;
@@ -178,11 +173,11 @@ void OGRDXFLayer::TranslateGenericProperty( OGRFeature *poFeature,
       case 210:
         oStyleProperties["210_N.dX"] = pszValue;
         break;
-        
+
       case 220:
         oStyleProperties["220_N.dY"] = pszValue;
         break;
-        
+
       case 230:
         oStyleProperties["230_N.dZ"] = pszValue;
         break;
@@ -205,7 +200,7 @@ void OGRDXFLayer::PrepareLineStyle( OGRFeature *poFeature )
 /* -------------------------------------------------------------------- */
 /*      Is the layer disabled/hidden/frozen/off?                        */
 /* -------------------------------------------------------------------- */
-    int bHidden = 
+    int bHidden =
         EQUAL(poDS->LookupLayerProperty( osLayer, "Hidden" ), "1");
 
 /* -------------------------------------------------------------------- */
@@ -216,14 +211,14 @@ void OGRDXFLayer::PrepareLineStyle( OGRFeature *poFeature )
     if( oStyleProperties.count("Color") > 0 )
         nColor = atoi(oStyleProperties["Color"]);
 
-    // Use layer color? 
+    // Use layer color?
     if( nColor < 1 || nColor > 255 )
     {
         const char *pszValue = poDS->LookupLayerProperty( osLayer, "Color" );
         if( pszValue != NULL )
             nColor = atoi(pszValue);
     }
-        
+
     if( nColor < 1 || nColor > 255 )
         return;
 
@@ -254,21 +249,18 @@ void OGRDXFLayer::PrepareLineStyle( OGRFeature *poFeature )
     CPLString osStyle;
     const unsigned char *pabyDXFColors = ACGetColorTable();
 
-    osStyle.Printf( "PEN(c:#%02x%02x%02x", 
+    osStyle.Printf( "PEN(c:#%02x%02x%02x",
                     pabyDXFColors[nColor*3+0],
                     pabyDXFColors[nColor*3+1],
                     pabyDXFColors[nColor*3+2] );
 
     if( bHidden )
-        osStyle += "00"; 
+        osStyle += "00";
 
     if( dfWeight > 0.0 )
     {
         char szBuffer[64];
-        snprintf(szBuffer, sizeof(szBuffer), "%.2g", dfWeight);
-        char* pszComma = strchr(szBuffer, ',');
-        if (pszComma)
-            *pszComma = '.';
+        CPLsnprintf(szBuffer, sizeof(szBuffer), "%.2g", dfWeight);
         osStyle += CPLString().Printf( ",w:%sg", szBuffer );
     }
 
@@ -280,7 +272,7 @@ void OGRDXFLayer::PrepareLineStyle( OGRFeature *poFeature )
     }
 
     osStyle += ")";
-    
+
     poFeature->SetStyleString( osStyle );
 }
 
@@ -294,14 +286,24 @@ private:
     double adfN[3];
     double adfAX[3];
     double adfAY[3];
-    
+
+    double dfDeterminant;
+    double aadfInverse[4][4];
+
+    double Det2x2( double a, double b, double c, double d )
+    {
+        return (a*d - b*c);
+    }
+
 public:
-    OCSTransformer( double adfN[3] ) {
+    OCSTransformer( double adfNIn[3], bool bInverse = false ) {
         static const double dSmall = 1.0 / 64.0;
         static const double adfWZ[3] = {0, 0, 1};
         static const double adfWY[3] = {0, 1, 0};
 
-        memcpy( this->adfN, adfN, sizeof(double)*3 );
+        dfDeterminant = 0.0;
+        Scale2Unit( adfNIn );
+        memcpy( this->adfN, adfNIn, sizeof(double)*3 );
 
     if ((ABS(adfN[0]) < dSmall) && (ABS(adfN[1]) < dSmall))
             CrossProduct(adfWY, adfN, adfAX);
@@ -311,6 +313,35 @@ public:
     Scale2Unit( adfAX );
     CrossProduct(adfN, adfAX, adfAY);
     Scale2Unit( adfAY );
+
+    if( bInverse == true ) {
+        double a[4] = { 0.0, adfAX[0], adfAY[0], adfN[0] };
+        double b[4] = { 0.0, adfAX[1], adfAY[1], adfN[1] };
+        double c[4] = { 0.0, adfAX[2], adfAY[2], adfN[2] };
+
+        dfDeterminant = a[1]*b[2]*c[3] - a[1]*b[3]*c[2]
+                      + a[2]*b[3]*c[1] - a[2]*b[1]*c[3]
+                      + a[3]*b[1]*c[2] - a[3]*b[2]*c[1];
+
+        if( dfDeterminant != 0.0 ) {
+            double k = 1 / dfDeterminant;
+            double a11 = adfAX[0], a12 = adfAY[0], a13 = adfN[0];
+            double a21 = adfAX[1], a22 = adfAY[1], a23 = adfN[1];
+            double a31 = adfAX[2], a32 = adfAY[2], a33 = adfN[2];
+
+            aadfInverse[1][1] = k * Det2x2( a22,a23,a32,a33 );
+            aadfInverse[1][2] = k * Det2x2( a13,a12,a33,a32 );
+            aadfInverse[1][3] = k * Det2x2( a12,a13,a22,a23 );
+
+            aadfInverse[2][1] = k * Det2x2( a23,a21,a33,a31 );
+            aadfInverse[2][2] = k * Det2x2( a11,a13,a31,a33 );
+            aadfInverse[2][3] = k * Det2x2( a13,a11,a23,a21 );
+
+            aadfInverse[3][1] = k * Det2x2( a21,a22,a31,a32 );
+            aadfInverse[3][2] = k * Det2x2( a12,a11,a32,a31 );
+            aadfInverse[3][3] = k * Det2x2( a11,a12,a21,a22 );
+        }
+    }
     }
 
     void CrossProduct(const double *a, const double *b, double *vResult) {
@@ -330,19 +361,19 @@ public:
     }
     OGRSpatialReference *GetSourceCS() { return NULL; }
     OGRSpatialReference *GetTargetCS() { return NULL; }
-    int Transform( int nCount, 
+    int Transform( int nCount,
                    double *x, double *y, double *z )
         { return TransformEx( nCount, x, y, z, NULL ); }
-    
-    int TransformEx( int nCount, 
-                     double *adfX, double *adfY, double *adfZ = NULL,
+
+    int TransformEx( int nCount,
+                     double *adfX, double *adfY, double *adfZ,
                      int *pabSuccess = NULL )
         {
             int i;
             for( i = 0; i < nCount; i++ )
             {
                 double x = adfX[i], y = adfY[i], z = adfZ[i];
-                
+
                 adfX[i] = x * adfAX[0] + y * adfAY[0] + z * adfN[0];
                 adfY[i] = x * adfAX[1] + y * adfAY[1] + z * adfN[1];
                 adfZ[i] = x * adfAX[2] + y * adfAY[2] + z * adfN[2];
@@ -352,6 +383,26 @@ public:
             }
             return TRUE;
         }
+
+    int InverseTransform( int nCount,
+                          double *adfX, double *adfY, double *adfZ )
+    {
+        if( dfDeterminant == 0.0 )
+            return FALSE;
+
+        for( int i = 0; i < nCount; i++ )
+        {
+            double x = adfX[i], y = adfY[i], z = adfZ[i];
+
+            adfX[i] = x * aadfInverse[1][1] + y * aadfInverse[1][2]
+                    + z * aadfInverse[1][3];
+            adfY[i] = x * aadfInverse[2][1] + y * aadfInverse[2][2]
+                    + z * aadfInverse[2][3];
+            adfZ[i] = x * aadfInverse[3][1] + y * aadfInverse[3][2]
+                    + z * aadfInverse[3][3];
+        }
+        return TRUE;
+    }
 };
 
 /************************************************************************/
@@ -445,7 +496,7 @@ OGRFeature *OGRDXFLayer::TranslateMTEXT()
 
           case 21:
             dfYDirection = CPLAtof(szLineBuf);
-            dfAngle = atan2( dfYDirection, dfXDirection ) * 180.0 / PI;
+            dfAngle = atan2( dfYDirection, dfXDirection ) * 180.0 / M_PI;
             break;
 
           case 1:
@@ -464,6 +515,12 @@ OGRFeature *OGRDXFLayer::TranslateMTEXT()
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        delete poFeature;
+        return NULL;
+    }
 
     if( nCode == 0 )
         poDS->UnreadValue();
@@ -484,7 +541,7 @@ OGRFeature *OGRDXFLayer::TranslateMTEXT()
 
     poFeature->SetField( "Text", osText );
 
-    
+
 /* -------------------------------------------------------------------- */
 /*      We need to escape double quotes with backslashes before they    */
 /*      can be inserted in the style string.                            */
@@ -512,7 +569,7 @@ OGRFeature *OGRDXFLayer::TranslateMTEXT()
     if( oStyleProperties.count("Color") > 0 )
         nColor = atoi(oStyleProperties["Color"]);
 
-    // Use layer color? 
+    // Use layer color?
     if( nColor < 1 || nColor > 255 )
     {
         CPLString osLayer = poFeature->GetFieldAsString("Layer");
@@ -520,48 +577,41 @@ OGRFeature *OGRDXFLayer::TranslateMTEXT()
         if( pszValue != NULL )
             nColor = atoi(pszValue);
     }
-        
+
 /* -------------------------------------------------------------------- */
 /*      Prepare style string.                                           */
 /* -------------------------------------------------------------------- */
     CPLString osStyle;
     char szBuffer[64];
-    char* pszComma;
 
     osStyle.Printf("LABEL(f:\"Arial\",t:\"%s\"",osText.c_str());
 
     if( dfAngle != 0.0 )
     {
-        snprintf(szBuffer, sizeof(szBuffer), "%.3g", dfAngle);
-        pszComma = strchr(szBuffer, ',');
-        if (pszComma)
-            *pszComma = '.';
+        CPLsnprintf(szBuffer, sizeof(szBuffer), "%.3g", dfAngle);
         osStyle += CPLString().Printf(",a:%s", szBuffer);
     }
 
     if( dfHeight != 0.0 )
     {
-        snprintf(szBuffer, sizeof(szBuffer), "%.3g", dfHeight);
-        pszComma = strchr(szBuffer, ',');
-        if (pszComma)
-            *pszComma = '.';
+        CPLsnprintf(szBuffer, sizeof(szBuffer), "%.3g", dfHeight);
         osStyle += CPLString().Printf(",s:%sg", szBuffer);
     }
 
     if( nAttachmentPoint >= 0 && nAttachmentPoint <= 9 )
     {
-        const static int anAttachmentMap[10] = 
+        const static int anAttachmentMap[10] =
             { -1, 7, 8, 9, 4, 5, 6, 1, 2, 3 };
-        
-        osStyle += 
+
+        osStyle +=
             CPLString().Printf(",p:%d", anAttachmentMap[nAttachmentPoint]);
     }
 
     if( nColor > 0 && nColor < 256 )
     {
         const unsigned char *pabyDXFColors = ACGetColorTable();
-        osStyle += 
-            CPLString().Printf( ",c:#%02x%02x%02x", 
+        osStyle +=
+            CPLString().Printf( ",c:#%02x%02x%02x",
                                 pabyDXFColors[nColor*3+0],
                                 pabyDXFColors[nColor*3+1],
                                 pabyDXFColors[nColor*3+2] );
@@ -589,6 +639,8 @@ OGRFeature *OGRDXFLayer::TranslateTEXT()
     double dfHeight = 0.0;
     CPLString osText;
     int bHaveZ = FALSE;
+    int nAnchorPosition = 1;
+    int nHorizontalAlignment = 0, nVerticalAlignment = 0;
 
     while( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) > 0 )
     {
@@ -620,11 +672,25 @@ OGRFeature *OGRDXFLayer::TranslateTEXT()
             dfAngle = CPLAtof(szLineBuf);
             break;
 
+          case 72:
+            nHorizontalAlignment = atoi(szLineBuf);
+            break;
+
+          case 73:
+            nVerticalAlignment = atoi(szLineBuf);
+            break;
+
           default:
             TranslateGenericProperty( poFeature, nCode, szLineBuf );
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        delete poFeature;
+        return NULL;
+    }
 
     if( nCode == 0 )
         poDS->UnreadValue();
@@ -637,6 +703,33 @@ OGRFeature *OGRDXFLayer::TranslateTEXT()
     ApplyOCSTransformer( poGeom );
     poFeature->SetGeometryDirectly( poGeom );
 
+/* -------------------------------------------------------------------- */
+/*      Determine anchor position.                                      */
+/* -------------------------------------------------------------------- */
+    if( nHorizontalAlignment > 0 || nVerticalAlignment > 0 )
+    {
+        switch( nVerticalAlignment )
+        {
+          case 1: // bottom
+            nAnchorPosition = 10;
+            break;
+
+          case 2: // middle
+            nAnchorPosition = 4;
+            break;
+
+          case 3: // top
+            nAnchorPosition = 7;
+            break;
+
+          default:
+            break;
+        }
+        if( nHorizontalAlignment < 3 )
+            nAnchorPosition += nHorizontalAlignment;
+        // TODO other alignment options
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Translate text from Win-1252 to UTF8.  We approximate this      */
 /*      by treating Win-1252 as Latin-1.                                */
@@ -669,7 +762,7 @@ OGRFeature *OGRDXFLayer::TranslateTEXT()
 /* -------------------------------------------------------------------- */
     CPLString osLayer = poFeature->GetFieldAsString("Layer");
 
-    int bHidden = 
+    int bHidden =
         EQUAL(poDS->LookupLayerProperty( osLayer, "Hidden" ), "1");
 
 /* -------------------------------------------------------------------- */
@@ -680,14 +773,14 @@ OGRFeature *OGRDXFLayer::TranslateTEXT()
     if( oStyleProperties.count("Color") > 0 )
         nColor = atoi(oStyleProperties["Color"]);
 
-    // Use layer color? 
+    // Use layer color?
     if( nColor < 1 || nColor > 255 )
     {
         const char *pszValue = poDS->LookupLayerProperty( osLayer, "Color" );
         if( pszValue != NULL )
             nColor = atoi(pszValue);
     }
-        
+
     if( nColor < 1 || nColor > 255 )
         nColor = 8;
 
@@ -696,38 +789,33 @@ OGRFeature *OGRDXFLayer::TranslateTEXT()
 /* -------------------------------------------------------------------- */
     CPLString osStyle;
     char szBuffer[64];
-    char* pszComma;
 
     osStyle.Printf("LABEL(f:\"Arial\",t:\"%s\"",osText.c_str());
 
+    osStyle += CPLString().Printf(",p:%d", nAnchorPosition);
+
     if( dfAngle != 0.0 )
     {
-        snprintf(szBuffer, sizeof(szBuffer), "%.3g", dfAngle);
-        pszComma = strchr(szBuffer, ',');
-        if (pszComma)
-            *pszComma = '.';
+        CPLsnprintf(szBuffer, sizeof(szBuffer), "%.3g", dfAngle);
         osStyle += CPLString().Printf(",a:%s", szBuffer);
     }
 
     if( dfHeight != 0.0 )
     {
-        snprintf(szBuffer, sizeof(szBuffer), "%.3g", dfHeight);
-        pszComma = strchr(szBuffer, ',');
-        if (pszComma)
-            *pszComma = '.';
+        CPLsnprintf(szBuffer, sizeof(szBuffer), "%.3g", dfHeight);
         osStyle += CPLString().Printf(",s:%sg", szBuffer);
     }
 
     const unsigned char *pabyDWGColors = ACGetColorTable();
 
-    snprintf( szBuffer, sizeof(szBuffer), ",c:#%02x%02x%02x", 
+    snprintf( szBuffer, sizeof(szBuffer), ",c:#%02x%02x%02x",
               pabyDWGColors[nColor*3+0],
               pabyDWGColors[nColor*3+1],
               pabyDWGColors[nColor*3+2] );
     osStyle += szBuffer;
 
     if( bHidden )
-        osStyle += "00"; 
+        osStyle += "00";
 
     osStyle += ")";
 
@@ -771,21 +859,27 @@ OGRFeature *OGRDXFLayer::TranslatePOINT()
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        delete poFeature;
+        return NULL;
+    }
+
+    if( nCode == 0 )
+        poDS->UnreadValue();
 
     OGRPoint* poGeom;
     if( bHaveZ )
         poGeom = new OGRPoint( dfX, dfY, dfZ );
     else
         poGeom = new OGRPoint( dfX, dfY );
-    ApplyOCSTransformer( poGeom );
+
     poFeature->SetGeometryDirectly( poGeom );
 
-    if( nCode == 0 )
-        poDS->UnreadValue();
-    
     // Set style pen color
     PrepareLineStyle( poFeature );
- 
+
     return poFeature;
 }
 
@@ -841,6 +935,12 @@ OGRFeature *OGRDXFLayer::TranslateLINE()
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        delete poFeature;
+        return NULL;
+    }
 
     if( nCode == 0 )
         poDS->UnreadValue();
@@ -860,7 +960,6 @@ OGRFeature *OGRDXFLayer::TranslateLINE()
         poLS->addPoint( dfX2, dfY2 );
     }
 
-    ApplyOCSTransformer( poLS );
     poFeature->SetGeometryDirectly( poLS );
 
     PrepareLineStyle( poFeature );
@@ -959,6 +1058,12 @@ OGRFeature *OGRDXFLayer::TranslateLWPOLYLINE()
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        delete poFeature;
+        return NULL;
+    }
 
     if( nCode == 0 )
         poDS->UnreadValue();
@@ -966,7 +1071,7 @@ OGRFeature *OGRDXFLayer::TranslateLWPOLYLINE()
     if( bHaveX && bHaveY )
         smoothPolyline.AddPoint(dfX, dfY, dfZ, dfBulge);
 
-    
+
     if(smoothPolyline.IsEmpty())
     {
         delete poFeature;
@@ -1019,6 +1124,19 @@ OGRFeature *OGRDXFLayer::TranslatePOLYLINE()
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        delete poFeature;
+        return NULL;
+    }
+
+    if( (nPolylineFlag & (16 ^ 64)) != 0 )
+    {
+        CPLDebug( "DXF", "Polygon/polyface mesh not supported." );
+        delete poFeature;
+        return NULL;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Collect VERTEXes as a smooth polyline.                          */
@@ -1036,6 +1154,13 @@ OGRFeature *OGRDXFLayer::TranslatePOLYLINE()
         if( !EQUAL(szLineBuf,"VERTEX") )
         {
             while( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf)))>0 ) {}
+            if( nCode < 0 )
+            {
+                DXF_LAYER_READER_ERROR();
+                delete poFeature;
+                return NULL;
+            }
+
             continue;
         }
 
@@ -1047,11 +1172,11 @@ OGRFeature *OGRDXFLayer::TranslatePOLYLINE()
               case 10:
                 dfX = CPLAtof(szLineBuf);
                 break;
-                
+
               case 20:
                 dfY = CPLAtof(szLineBuf);
                 break;
-                
+
               case 30:
                 dfZ = CPLAtof(szLineBuf);
                 smoothPolyline.setCoordinateDimension(3);
@@ -1069,6 +1194,13 @@ OGRFeature *OGRDXFLayer::TranslatePOLYLINE()
                 break;
             }
         }
+        if( nCode < 0 )
+        {
+            DXF_LAYER_READER_ERROR();
+            delete poFeature;
+            return NULL;
+        }
+
 
         // Ignore Spline frame control points ( see #4683 )
         if ((nVertexFlag & 16) == 0)
@@ -1081,7 +1213,7 @@ OGRFeature *OGRDXFLayer::TranslatePOLYLINE()
         delete poFeature;
         return NULL;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Close polyline if necessary.                                    */
 /* -------------------------------------------------------------------- */
@@ -1089,7 +1221,9 @@ OGRFeature *OGRDXFLayer::TranslatePOLYLINE()
         smoothPolyline.Close();
 
     OGRGeometry* poGeom = smoothPolyline.Tesselate();
-    ApplyOCSTransformer( poGeom );
+
+    if( (nPolylineFlag & 8) == 0 )
+        ApplyOCSTransformer( poGeom );
     poFeature->SetGeometryDirectly( poGeom );
 
     PrepareLineStyle( poFeature );
@@ -1139,6 +1273,12 @@ OGRFeature *OGRDXFLayer::TranslateCIRCLE()
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        delete poFeature;
+        return NULL;
+    }
 
     if( nCode == 0 )
         poDS->UnreadValue();
@@ -1146,10 +1286,10 @@ OGRFeature *OGRDXFLayer::TranslateCIRCLE()
 /* -------------------------------------------------------------------- */
 /*      Create geometry                                                 */
 /* -------------------------------------------------------------------- */
-    OGRGeometry *poCircle = 
-        OGRGeometryFactory::approximateArcAngles( dfX1, dfY1, dfZ1, 
+    OGRGeometry *poCircle =
+        OGRGeometryFactory::approximateArcAngles( dfX1, dfY1, dfZ1,
                                                   dfRadius, dfRadius, 0.0,
-                                                  0.0, 360.0, 
+                                                  0.0, 360.0,
                                                   0.0 );
 
     if( !bHaveZ )
@@ -1176,6 +1316,8 @@ OGRFeature *OGRDXFLayer::TranslateELLIPSE()
     double dfStartAngle = 0.0, dfEndAngle = 360.0;
     double dfAxisX=0.0, dfAxisY=0.0, dfAxisZ=0.0;
     int bHaveZ = FALSE;
+    double adfN[3];
+    bool bApplyOCSTransform = false;
 
 /* -------------------------------------------------------------------- */
 /*      Process values.                                                 */
@@ -1215,12 +1357,12 @@ OGRFeature *OGRDXFLayer::TranslateELLIPSE()
 
           case 41:
             // These *seem* to always be in radians regardless of $AUNITS
-            dfEndAngle = -1 * CPLAtof(szLineBuf) * 180.0 / PI;
+            dfEndAngle = -1 * CPLAtof(szLineBuf) * 180.0 / M_PI;
             break;
 
           case 42:
             // These *seem* to always be in radians regardless of $AUNITS
-            dfStartAngle = -1 * CPLAtof(szLineBuf) * 180.0 / PI;
+            dfStartAngle = -1 * CPLAtof(szLineBuf) * 180.0 / M_PI;
             break;
 
           default:
@@ -1228,10 +1370,42 @@ OGRFeature *OGRDXFLayer::TranslateELLIPSE()
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        delete poFeature;
+        return NULL;
+    }
 
     if( nCode == 0 )
         poDS->UnreadValue();
 
+/* -------------------------------------------------------------------- */
+/*      Setup coordinate system                                         */
+/* -------------------------------------------------------------------- */
+    if( oStyleProperties.count("210_N.dX") != 0
+        && oStyleProperties.count("220_N.dY") != 0
+        && oStyleProperties.count("230_N.dZ") != 0 )
+    {
+        adfN[0] = CPLAtof(oStyleProperties["210_N.dX"]);
+        adfN[1] = CPLAtof(oStyleProperties["220_N.dY"]);
+        adfN[2] = CPLAtof(oStyleProperties["230_N.dZ"]);
+
+        if( (adfN[0] == 0.0 && adfN[1] == 0.0 && adfN[2] == 1.0) == false )
+        {
+            double *x, *y, *z;
+            OCSTransformer oTransformer( adfN, true );
+
+            bApplyOCSTransform = true;
+
+            x = &dfX1; y = &dfY1, z = &dfZ1;
+            oTransformer.InverseTransform( 1, x, y, z );
+
+            x = &dfAxisX; y = &dfAxisY, z = &dfAxisZ;
+            oTransformer.InverseTransform( 1, x, y, z );
+        }
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Compute primary and secondary axis lengths, and the angle of    */
 /*      rotation for the ellipse.                                       */
@@ -1239,32 +1413,33 @@ OGRFeature *OGRDXFLayer::TranslateELLIPSE()
     double dfPrimaryRadius, dfSecondaryRadius;
     double dfRotation;
 
-    if( dfStartAngle > dfEndAngle )
-        dfEndAngle += 360.0;
-
-    dfPrimaryRadius = sqrt( dfAxisX * dfAxisX 
+    dfPrimaryRadius = sqrt( dfAxisX * dfAxisX
                             + dfAxisY * dfAxisY
                             + dfAxisZ * dfAxisZ );
 
     dfSecondaryRadius = dfRatio * dfPrimaryRadius;
 
-    dfRotation = -1 * atan2( dfAxisY, dfAxisX ) * 180 / PI;
+    dfRotation = -1 * atan2( dfAxisY, dfAxisX ) * 180 / M_PI;
 
 /* -------------------------------------------------------------------- */
 /*      Create geometry                                                 */
 /* -------------------------------------------------------------------- */
-    OGRGeometry *poEllipse = 
-        OGRGeometryFactory::approximateArcAngles( dfX1, dfY1, dfZ1, 
-                                                  dfPrimaryRadius, 
+    if( dfStartAngle > dfEndAngle )
+        dfEndAngle += 360.0;
+
+    OGRGeometry *poEllipse =
+        OGRGeometryFactory::approximateArcAngles( dfX1, dfY1, dfZ1,
+                                                  dfPrimaryRadius,
                                                   dfSecondaryRadius,
-                                                  dfRotation, 
+                                                  dfRotation,
                                                   dfStartAngle, dfEndAngle,
                                                   0.0 );
 
     if( !bHaveZ )
         poEllipse->flattenTo2D();
 
-    ApplyOCSTransformer( poEllipse );
+    if( bApplyOCSTransform == true )
+        ApplyOCSTransformer( poEllipse );
     poFeature->SetGeometryDirectly( poEllipse );
 
     PrepareLineStyle( poFeature );
@@ -1325,6 +1500,12 @@ OGRFeature *OGRDXFLayer::TranslateARC()
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        delete poFeature;
+        return NULL;
+    }
 
     if( nCode == 0 )
         poDS->UnreadValue();
@@ -1335,8 +1516,8 @@ OGRFeature *OGRDXFLayer::TranslateARC()
     if( dfStartAngle > dfEndAngle )
         dfEndAngle += 360.0;
 
-    OGRGeometry *poArc = 
-        OGRGeometryFactory::approximateArcAngles( dfX1, dfY1, dfZ1, 
+    OGRGeometry *poArc =
+        OGRGeometryFactory::approximateArcAngles( dfX1, dfY1, dfZ1,
                                                   dfRadius, dfRadius, 0.0,
                                                   dfStartAngle, dfEndAngle,
                                                   0.0 );
@@ -1355,18 +1536,24 @@ OGRFeature *OGRDXFLayer::TranslateARC()
 /*                          TranslateSPLINE()                           */
 /************************************************************************/
 
-void rbspline(int npts,int k,int p1,double b[],double h[], double p[]);
-void rbsplinu(int npts,int k,int p1,double b[],double h[], double p[]);
+void rbspline2(int npts,int k,int p1,double b[],double h[],
+        bool xflag, double x[], double p[]);
 
 OGRFeature *OGRDXFLayer::TranslateSPLINE()
 
 {
     char szLineBuf[257];
-    int nCode, nDegree = -1, nFlags = -1, bClosed = FALSE, i;
+    int nCode, nDegree = -1, nOrder = -1, i;
+    int nControlPoints = -1, nKnots = -1;
+    bool bResult = false, bCalculateKnots = false;
     OGRFeature *poFeature = new OGRFeature( poFeatureDefn );
     std::vector<double> adfControlPoints;
+    std::vector<double> adfKnots;
+    std::vector<double> adfWeights;
 
     adfControlPoints.push_back(0.0);
+    adfKnots.push_back(0.0);
+    adfWeights.push_back(0.0);
 
 /* -------------------------------------------------------------------- */
 /*      Process values.                                                 */
@@ -1384,60 +1571,125 @@ OGRFeature *OGRDXFLayer::TranslateSPLINE()
             adfControlPoints.push_back( 0.0 );
             break;
 
+          case 40:
+            adfKnots.push_back( CPLAtof(szLineBuf) );
+            break;
+
+          case 41:
+            adfWeights.push_back( CPLAtof(szLineBuf) );
+            break;
+
           case 70:
-            nFlags = atoi(szLineBuf);
-            if( nFlags & 1 )
-                bClosed = TRUE;
             break;
 
           case 71:
             nDegree = atoi(szLineBuf);
             break;
 
+          case 72:
+            nKnots = atoi(szLineBuf);
+            break;
+
+          case 73:
+            nControlPoints = atoi(szLineBuf);
+            break;
+
           default:
             TranslateGenericProperty( poFeature, nCode, szLineBuf );
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        delete poFeature;
+        return NULL;
+    }
 
     if( nCode == 0 )
         poDS->UnreadValue();
 
-    if( bClosed )
+/* -------------------------------------------------------------------- */
+/*      Sanity checks                                                   */
+/* -------------------------------------------------------------------- */
+    nOrder = nDegree + 1;
+
+    bResult = ( nOrder >= 2 );
+    if( bResult == true )
     {
-        for( i = 0; i < nDegree; i++ )
+        // Check whether nctrlpts value matches number of vertices read
+        int nCheck = (static_cast<int>(adfControlPoints.size()) - 1) / 3;
+
+        if( nControlPoints == -1 )
+            nControlPoints =
+                (static_cast<int>(adfControlPoints.size()) - 1) / 3;
+
+        // min( num(ctrlpts) ) = order
+        bResult = ( nControlPoints >= nOrder && nControlPoints == nCheck);
+    }
+
+    if( bResult == true )
+    {
+        int nCheck = static_cast<int>(adfKnots.size()) - 1;
+
+        // Recalculate knots when:
+        // - no knots data present, nknots is -1 and ncheck is 0
+        // - nknots value present, no knot vertices
+        //   nknots is (nctrlpts + order), ncheck is 0
+        if( nCheck == 0 )
         {
-            adfControlPoints.push_back( adfControlPoints[i*3+1] );
-            adfControlPoints.push_back( adfControlPoints[i*3+2] );
-            adfControlPoints.push_back( adfControlPoints[i*3+3] );
+            bCalculateKnots = true;
+            for( i = 0; i < (nControlPoints + nOrder); i++ )
+                adfKnots.push_back( 0.0 );
+
+            nCheck = static_cast<int>(adfKnots.size()) - 1;
         }
+        // Adjust nknots value when:
+        // - nknots value not present, knot vertices present
+        //   nknots is -1, ncheck is (nctrlpts + order)
+        if( nKnots == -1 )
+            nKnots = static_cast<int>(adfKnots.size()) - 1;
+
+        // num(knots) = num(ctrlpts) + order
+        bResult = ( nKnots == (nControlPoints + nOrder) && nKnots == nCheck );
+    }
+
+    if( bResult == true )
+    {
+        int nWeights = static_cast<int>(adfWeights.size()) - 1;
+
+        if( nWeights == 0 )
+        {
+            for( i = 0; i < nControlPoints; i++ )
+                adfWeights.push_back( 1.0 );
+
+            nWeights = static_cast<int>(adfWeights.size()) - 1;
+        }
+
+        // num(weights) = num(ctrlpts)
+        bResult = ( nWeights == nControlPoints );
+    }
+
+    if( bResult == false )
+    {
+        DXF_LAYER_READER_ERROR();
+        delete poFeature;
+        return NULL;
     }
 
 /* -------------------------------------------------------------------- */
 /*      Interpolate spline                                              */
 /* -------------------------------------------------------------------- */
-    int nControlPoints = adfControlPoints.size() / 3;
-    std::vector<double> h, p;
-
-    h.push_back(1.0);
-    for( i = 0; i < nControlPoints; i++ )
-        h.push_back( 1.0 );
-    
-    // resolution:
-    //int p1 = getGraphicVariableInt("$SPLINESEGS", 8) * npts;
     int p1 = nControlPoints * 8;
+    std::vector<double> p;
 
     p.push_back( 0.0 );
     for( i = 0; i < 3*p1; i++ )
         p.push_back( 0.0 );
 
-    if( bClosed )
-        rbsplinu( nControlPoints, nDegree+1, p1, &(adfControlPoints[0]), 
-                  &(h[0]), &(p[0]) );
-    else
-        rbspline( nControlPoints, nDegree+1, p1, &(adfControlPoints[0]), 
-                  &(h[0]), &(p[0]) );
-    
+    rbspline2( nControlPoints, nOrder, p1, &(adfControlPoints[0]),
+            &(adfWeights[0]), bCalculateKnots, &(adfKnots[0]), &(p[0]) );
+
 /* -------------------------------------------------------------------- */
 /*      Turn into OGR geometry.                                         */
 /* -------------------------------------------------------------------- */
@@ -1447,7 +1699,6 @@ OGRFeature *OGRDXFLayer::TranslateSPLINE()
     for( i = 0; i < p1; i++ )
         poLS->setPoint( i, p[i*3+1], p[i*3+2] );
 
-    ApplyOCSTransformer( poLS );
     poFeature->SetGeometryDirectly( poLS );
 
     PrepareLineStyle( poFeature );
@@ -1530,6 +1781,12 @@ OGRFeature *OGRDXFLayer::Translate3DFACE()
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        delete poFeature;
+        return NULL;
+    }
 
     if( nCode == 0 )
         poDS->UnreadValue();
@@ -1665,6 +1922,12 @@ OGRFeature *OGRDXFLayer::TranslateSOLID()
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        delete poFeature;
+        return NULL;
+    }
 
     CPLDebug("Corner coordinates are", "%f,%f,%f,%f,%f,%f,%f,%f", dfX1, dfY1,
             dfX2, dfY2, dfX3, dfY3, dfX4, dfY4);
@@ -1723,7 +1986,7 @@ OGRFeature *OGRDXFLayer::TranslateSOLID()
 class GeometryInsertTransformer : public OGRCoordinateTransformation
 {
 public:
-    GeometryInsertTransformer() : 
+    GeometryInsertTransformer() :
             dfXOffset(0),dfYOffset(0),dfZOffset(0),
             dfXScale(1.0),dfYScale(1.0),dfZScale(1.0),
             dfAngle(0.0) {}
@@ -1738,11 +2001,11 @@ public:
 
     OGRSpatialReference *GetSourceCS() { return NULL; }
     OGRSpatialReference *GetTargetCS() { return NULL; }
-    int Transform( int nCount, 
+    int Transform( int nCount,
                    double *x, double *y, double *z )
         { return TransformEx( nCount, x, y, z, NULL ); }
-    
-    int TransformEx( int nCount, 
+
+    int TransformEx( int nCount,
                      double *x, double *y, double *z = NULL,
                      int *pabSuccess = NULL )
         {
@@ -1819,20 +2082,26 @@ OGRFeature *OGRDXFLayer::TranslateINSERT()
 
           case 50:
             dfAngle = CPLAtof(szLineBuf);
-            // We want to transform this to radians. 
+            // We want to transform this to radians.
             // It is apparently always in degrees regardless of $AUNITS
-            oTransformer.dfAngle = dfAngle * PI / 180.0;
+            oTransformer.dfAngle = dfAngle * M_PI / 180.0;
             break;
 
-          case 2: 
+          case 2:
             osBlockName = szLineBuf;
             break;
-            
+
           default:
             TranslateGenericProperty( poFeature, nCode, szLineBuf );
             break;
         }
     }
+    if( nCode < 0 )
+    {
+        DXF_LAYER_READER_ERROR();
+        delete poFeature;
+        return NULL;
+    }
 
     if( nCode == 0 )
         poDS->UnreadValue();
@@ -1845,7 +2114,7 @@ OGRFeature *OGRDXFLayer::TranslateINSERT()
     {
         // ApplyOCSTransformer( poGeom ); ?
         poFeature->SetGeometryDirectly(
-            new OGRPoint( oTransformer.dfXOffset, 
+            new OGRPoint( oTransformer.dfXOffset,
                           oTransformer.dfYOffset,
                           oTransformer.dfZOffset ) );
 
@@ -1861,7 +2130,7 @@ OGRFeature *OGRDXFLayer::TranslateINSERT()
 /*      Lookup the block.                                               */
 /* -------------------------------------------------------------------- */
     DXFBlockDefinition *poBlock = poDS->LookupBlock( osBlockName );
-    
+
     if( poBlock == NULL )
     {
         delete poFeature;
@@ -1960,13 +2229,12 @@ OGRFeature *OGRDXFLayer::GetNextUnfilteredFeature()
     {
         // read ahead to an entity.
         while( (nCode = poDS->ReadValue(szLineBuf,sizeof(szLineBuf))) > 0 ) {}
-
-        if( nCode == -1 )
+        if( nCode < 0 )
         {
-            CPLDebug( "DXF", "Unexpected end of data without ENDSEC." );
+            DXF_LAYER_READER_ERROR();
             return NULL;
         }
-        
+
         if( EQUAL(szLineBuf,"ENDSEC") )
         {
             //CPLDebug( "DXF", "Clean end of features at ENDSEC." );
@@ -1994,7 +2262,7 @@ OGRFeature *OGRDXFLayer::GetNextUnfilteredFeature()
         {
             poFeature = TranslateMTEXT();
         }
-        else if( EQUAL(szLineBuf,"TEXT") 
+        else if( EQUAL(szLineBuf,"TEXT")
                  || EQUAL(szLineBuf,"ATTDEF") )
         {
             poFeature = TranslateTEXT();
@@ -2052,7 +2320,7 @@ OGRFeature *OGRDXFLayer::GetNextUnfilteredFeature()
             if( oIgnoredEntities.count(szLineBuf) == 0 )
             {
                 oIgnoredEntities.insert( szLineBuf );
-                CPLDebug( "DWG", "Ignoring one or more of entity '%s'.", 
+                CPLDebug( "DWG", "Ignoring one or more of entity '%s'.",
                           szLineBuf );
             }
         }
@@ -2074,7 +2342,7 @@ OGRFeature *OGRDXFLayer::GetNextUnfilteredFeature()
 OGRFeature *OGRDXFLayer::GetNextFeature()
 
 {
-    while( TRUE )
+    while( true )
     {
         OGRFeature *poFeature = GetNextUnfilteredFeature();
 
@@ -2105,4 +2373,3 @@ int OGRDXFLayer::TestCapability( const char * pszCap )
     else
         return FALSE;
 }
-
