@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrdxfwriterds.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogrdxfwriterds.cpp 33713 2016-03-12 17:41:57Z goatbar $
  *
  * Project:  DXF Translator
  * Purpose:  Implements OGRDXFWriterDS - the OGRDataSource class used for
@@ -33,7 +33,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrdxfwriterds.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: ogrdxfwriterds.cpp 33713 2016-03-12 17:41:57Z goatbar $");
 
 /************************************************************************/
 /*                          OGRDXFWriterDS()                          */
@@ -83,7 +83,7 @@ OGRDXFWriterDS::~OGRDXFWriterDS()
                 VSIFWriteL( pszLine, 1, strlen(pszLine), fp );
                 VSIFWriteL( "\n", 1, 1, fp );
             }
-                    
+
 /* -------------------------------------------------------------------- */
 /*      Cleanup temporary file.                                         */
 /* -------------------------------------------------------------------- */
@@ -128,7 +128,8 @@ int OGRDXFWriterDS::TestCapability( const char * pszCap )
 
 {
     if( EQUAL(pszCap,ODsCCreateLayer) )
-        return TRUE;
+        // Unable to have more than one OGR entities layer in a DXF file, with one options blocks layer.
+        return poBlocksLayer == NULL || poLayer == NULL;
     else
         return FALSE;
 }
@@ -177,7 +178,7 @@ int OGRDXFWriterDS::Open( const char * pszFilename, char **papszOptions )
         const char *pszValue = CPLFindFile( "gdal", "header.dxf" );
         if( pszValue == NULL )
         {
-            CPLError( CE_Failure, CPLE_OpenFailed, 
+            CPLError( CE_Failure, CPLE_OpenFailed,
                       "Failed to find template header file header.dxf for reading,\nis GDAL_DATA set properly?" );
             return FALSE;
         }
@@ -230,8 +231,8 @@ int OGRDXFWriterDS::Open( const char * pszFilename, char **papszOptions )
 
     if( fp == NULL )
     {
-        CPLError( CE_Failure, CPLE_OpenFailed, 
-                  "Failed to open '%s' for writing.", 
+        CPLError( CE_Failure, CPLE_OpenFailed,
+                  "Failed to open '%s' for writing.",
                   pszFilename );
         return FALSE;
     }
@@ -245,8 +246,8 @@ int OGRDXFWriterDS::Open( const char * pszFilename, char **papszOptions )
     fpTemp = VSIFOpenL( osTempFilename, "w" );
     if( fpTemp == NULL )
     {
-        CPLError( CE_Failure, CPLE_OpenFailed, 
-                  "Failed to open '%s' for writing.", 
+        CPLError( CE_Failure, CPLE_OpenFailed,
+                  "Failed to open '%s' for writing.",
                   osTempFilename.c_str() );
         return FALSE;
     }
@@ -255,12 +256,12 @@ int OGRDXFWriterDS::Open( const char * pszFilename, char **papszOptions )
 }
 
 /************************************************************************/
-/*                            CreateLayer()                             */
+/*                           ICreateLayer()                             */
 /************************************************************************/
 
-OGRLayer *OGRDXFWriterDS::CreateLayer( const char *pszName, 
-                                       OGRSpatialReference *, 
-                                       OGRwkbGeometryType, 
+OGRLayer *OGRDXFWriterDS::ICreateLayer( const char *pszName,
+                                       OGRSpatialReference *,
+                                       OGRwkbGeometryType,
                                        char ** )
 
 {
@@ -312,10 +313,7 @@ static int WriteValue( VSILFILE *fp, int nCode, double dfValue )
 {
     char szLinePair[64];
 
-    snprintf(szLinePair, sizeof(szLinePair), "%3d\n%.15g\n", nCode, dfValue );
-    char* pszComma = strchr(szLinePair, ',');
-    if (pszComma)
-        *pszComma = '.';
+    CPLsnprintf(szLinePair, sizeof(szLinePair), "%3d\n%.15g\n", nCode, dfValue );
     size_t nLen = strlen(szLinePair);
     if( VSIFWriteL( szLinePair, 1, nLen, fp ) != nLen )
     {
@@ -342,12 +340,12 @@ int OGRDXFWriterDS::TransferUpdateHeader( VSILFILE *fpOut )
     int nCode;
     CPLString osSection, osTable, osEntity;
 
-    while( (nCode = oHeaderDS.ReadValue( szLineBuf, sizeof(szLineBuf) )) != -1 
+    while( (nCode = oHeaderDS.ReadValue( szLineBuf, sizeof(szLineBuf) )) != -1
            && osSection != "ENTITIES" )
     {
         if( nCode == 0 && EQUAL(szLineBuf,"ENDTAB") )
         {
-            // If we are at the end of the LAYER TABLE consider inserting 
+            // If we are at the end of the LAYER TABLE consider inserting
             // missing definitions.
             if( osTable == "LAYER" )
             {
@@ -355,7 +353,7 @@ int OGRDXFWriterDS::TransferUpdateHeader( VSILFILE *fpOut )
                     return FALSE;
             }
 
-            // If at the end of the BLOCK_RECORD TABLE consider inserting 
+            // If at the end of the BLOCK_RECORD TABLE consider inserting
             // missing definitions.
             if( osTable == "BLOCK_RECORD" && poBlocksLayer )
             {
@@ -363,7 +361,7 @@ int OGRDXFWriterDS::TransferUpdateHeader( VSILFILE *fpOut )
                     return FALSE;
             }
 
-            // If at the end of the LTYPE TABLE consider inserting 
+            // If at the end of the LTYPE TABLE consider inserting
             // missing layer type definitions.
             if( osTable == "LTYPE" )
             {
@@ -375,8 +373,8 @@ int OGRDXFWriterDS::TransferUpdateHeader( VSILFILE *fpOut )
         }
 
         // If we are at the end of the BLOCKS section, consider inserting
-        // suplementary blocks. 
-        if( nCode == 0 && osSection == "BLOCKS" && EQUAL(szLineBuf,"ENDSEC") 
+        // supplementary blocks.
+        if( nCode == 0 && osSection == "BLOCKS" && EQUAL(szLineBuf,"ENDSEC")
             && poBlocksLayer != NULL )
         {
             if( !WriteNewBlockDefinitions( fp ) )
@@ -384,7 +382,7 @@ int OGRDXFWriterDS::TransferUpdateHeader( VSILFILE *fpOut )
         }
 
         // We need to keep track of where $HANDSEED is so that we can
-        // come back and fix it up when we have generated all entity ids. 
+        // come back and fix it up when we have generated all entity ids.
         if( nCode == 9 && EQUAL(szLineBuf,"$HANDSEED") )
         {
             if( !WriteValue( fpOut, nCode, szLineBuf ) )
@@ -465,7 +463,7 @@ int OGRDXFWriterDS::TransferUpdateHeader( VSILFILE *fpOut )
 
             if( !WriteValue( fpOut, nCode, szLineBuf ) )
                 return FALSE;
-                
+
             osSection = szLineBuf;
         }
 
@@ -478,14 +476,14 @@ int OGRDXFWriterDS::TransferUpdateHeader( VSILFILE *fpOut )
 
             osTable = szLineBuf;
         }
-        
+
         // If we are starting the first layer, then capture
         // the layer contents while copying so we can duplicate
         // it for any new layer definitions.
         if( nCode == 0 && EQUAL(szLineBuf,"LAYER")
             && osTable == "LAYER" && aosDefaultLayerText.size() == 0 )
         {
-            do { 
+            do {
                 anDefaultLayerCode.push_back( nCode );
                 aosDefaultLayerText.push_back( szLineBuf );
 
@@ -507,7 +505,7 @@ int OGRDXFWriterDS::TransferUpdateHeader( VSILFILE *fpOut )
     }
 
     return TRUE;
-}    
+}
 
 /************************************************************************/
 /*                       TransferUpdateTrailer()                        */
@@ -516,17 +514,17 @@ int OGRDXFWriterDS::TransferUpdateHeader( VSILFILE *fpOut )
 int OGRDXFWriterDS::TransferUpdateTrailer( VSILFILE *fpOut )
 {
     OGRDXFReader oReader;
-    VSILFILE *fp;
+    VSILFILE *l_fp;
 
 /* -------------------------------------------------------------------- */
 /*      Open the file and setup a reader.                               */
 /* -------------------------------------------------------------------- */
-    fp = VSIFOpenL( osTrailerFile, "r" );
+    l_fp = VSIFOpenL( osTrailerFile, "r" );
 
-    if( fp == NULL )
+    if( l_fp == NULL )
         return FALSE;
 
-    oReader.Initialize( fp );
+    oReader.Initialize( l_fp );
 
 /* -------------------------------------------------------------------- */
 /*      Scan ahead to find the OBJECTS section.                         */
@@ -572,7 +570,7 @@ int OGRDXFWriterDS::TransferUpdateTrailer( VSILFILE *fpOut )
         }
     }
 
-    VSIFCloseL( fp );
+    VSIFCloseL( l_fp );
 
     return TRUE;
 }
@@ -584,7 +582,7 @@ int OGRDXFWriterDS::TransferUpdateTrailer( VSILFILE *fpOut )
 /*      variable.                                                       */
 /************************************************************************/
 
-int OGRDXFWriterDS::FixupHANDSEED( VSILFILE *fp )
+int OGRDXFWriterDS::FixupHANDSEED( VSILFILE *fpIn )
 
 {
 /* -------------------------------------------------------------------- */
@@ -593,7 +591,7 @@ int OGRDXFWriterDS::FixupHANDSEED( VSILFILE *fp )
     unsigned int   nHighestHandle = 0;
     std::set<CPLString>::iterator it;
 
-    for( it = aosUsedEntities.begin(); it != aosUsedEntities.end(); it++ ) 
+    for( it = aosUsedEntities.begin(); it != aosUsedEntities.end(); it++ )
     {
         unsigned int nHandle;
         if( sscanf( (*it).c_str(), "%x", &nHandle ) == 1 )
@@ -612,22 +610,22 @@ int OGRDXFWriterDS::FixupHANDSEED( VSILFILE *fp )
     if( nHANDSEEDOffset == 0 )
         return FALSE;
 
-    VSIFSeekL( fp, nHANDSEEDOffset, SEEK_SET );
-    VSIFReadL( szWorkBuf, 1, sizeof(szWorkBuf), fp );
-    
+    VSIFSeekL( fpIn, nHANDSEEDOffset, SEEK_SET );
+    VSIFReadL( szWorkBuf, 1, sizeof(szWorkBuf), fpIn );
+
     while( szWorkBuf[i] != '\n' )
         i++;
 
     i++;
     if( szWorkBuf[i] == '\r' )
         i++;
-    
+
     CPLString osNewValue;
 
     osNewValue.Printf( "%08X", nHighestHandle + 1 );
     strncpy( szWorkBuf + i, osNewValue.c_str(), osNewValue.size() );
 
-    VSIFSeekL( fp, nHANDSEEDOffset, SEEK_SET );
+    VSIFSeekL( fpIn, nHANDSEEDOffset, SEEK_SET );
     VSIFWriteL( szWorkBuf, 1, sizeof(szWorkBuf), fp );
 
     return TRUE;
@@ -639,7 +637,7 @@ int OGRDXFWriterDS::FixupHANDSEED( VSILFILE *fp )
 
 int  OGRDXFWriterDS::WriteNewLayerDefinitions( VSILFILE * fpOut )
 
-{                                               
+{
     int iLayer, nNewLayers = CSLCount(papszLayersToCreate);
 
     for( iLayer = 0; iLayer < nNewLayers; iLayer++ )
@@ -672,30 +670,30 @@ int  OGRDXFWriterDS::WriteNewLayerDefinitions( VSILFILE * fpOut )
 /*                      WriteNewLineTypeRecords()                       */
 /************************************************************************/
 
-int OGRDXFWriterDS::WriteNewLineTypeRecords( VSILFILE *fp )
+int OGRDXFWriterDS::WriteNewLineTypeRecords( VSILFILE *fpIn )
 
 {
     if( poLayer == NULL )
         return TRUE;
 
     std::map<CPLString,CPLString>::iterator oIt;
-    std::map<CPLString,CPLString>& oNewLineTypes = 
+    std::map<CPLString,CPLString>& oNewLineTypes =
         poLayer->GetNewLineTypeMap();
 
-    for( oIt = oNewLineTypes.begin(); 
+    for( oIt = oNewLineTypes.begin();
          oIt != oNewLineTypes.end(); oIt++ )
     {
-        WriteValue( fp, 0, "LTYPE" );
-        WriteEntityID( fp );
-        WriteValue( fp, 100, "AcDbSymbolTableRecord" );
-        WriteValue( fp, 100, "AcDbLinetypeTableRecord" );
-        WriteValue( fp, 2, (*oIt).first );
-        WriteValue( fp, 70, "0" );
-        WriteValue( fp, 3, "" );
-        WriteValue( fp, 72, "65" );
-        VSIFWriteL( (*oIt).second.c_str(), 1, (*oIt).second.size(), fp );
+        WriteValue( fpIn, 0, "LTYPE" );
+        WriteEntityID( fpIn );
+        WriteValue( fpIn, 100, "AcDbSymbolTableRecord" );
+        WriteValue( fpIn, 100, "AcDbLinetypeTableRecord" );
+        WriteValue( fpIn, 2, (*oIt).first );
+        WriteValue( fpIn, 70, "0" );
+        WriteValue( fpIn, 3, "" );
+        WriteValue( fpIn, 72, "65" );
+        VSIFWriteL( (*oIt).second.c_str(), 1, (*oIt).second.size(), fpIn );
 
-        CPLDebug( "DXF", "Define Line type '%s'.", 
+        CPLDebug( "DXF", "Define Line type '%s'.",
                   (*oIt).first.c_str() );
     }
 
@@ -706,7 +704,7 @@ int OGRDXFWriterDS::WriteNewLineTypeRecords( VSILFILE *fp )
 /*                        WriteNewBlockRecords()                        */
 /************************************************************************/
 
-int OGRDXFWriterDS::WriteNewBlockRecords( VSILFILE * fp )
+int OGRDXFWriterDS::WriteNewBlockRecords( VSILFILE * fpIn )
 
 {
     std::set<CPLString> aosAlreadyHandled;
@@ -737,12 +735,12 @@ int OGRDXFWriterDS::WriteNewBlockRecords( VSILFILE * fp )
 /* -------------------------------------------------------------------- */
 /*      Write the block record.                                         */
 /* -------------------------------------------------------------------- */
-        WriteValue( fp, 0, "BLOCK_RECORD" );
-        WriteEntityID( fp );
-        WriteValue( fp, 100, "AcDbSymbolTableRecord" );
-        WriteValue( fp, 100, "AcDbBlockTableRecord" );
-        WriteValue( fp, 2, poThisBlockFeat->GetFieldAsString("BlockName") );
-        if( !WriteValue( fp, 340, "0" ) )
+        WriteValue( fpIn, 0, "BLOCK_RECORD" );
+        WriteEntityID( fpIn );
+        WriteValue( fpIn, 100, "AcDbSymbolTableRecord" );
+        WriteValue( fpIn, 100, "AcDbBlockTableRecord" );
+        WriteValue( fpIn, 2, poThisBlockFeat->GetFieldAsString("BlockName") );
+        if( !WriteValue( fpIn, 340, "0" ) )
             return FALSE;
     }
 
@@ -753,10 +751,10 @@ int OGRDXFWriterDS::WriteNewBlockRecords( VSILFILE * fp )
 /*                      WriteNewBlockDefinitions()                      */
 /************************************************************************/
 
-int OGRDXFWriterDS::WriteNewBlockDefinitions( VSILFILE * fp )
+int OGRDXFWriterDS::WriteNewBlockDefinitions( VSILFILE * fpIn )
 
 {
-    poLayer->ResetFP( fp );
+    poLayer->ResetFP( fpIn );
 
 /* ==================================================================== */
 /*      Loop over all block objects written via the blocks layer.       */
@@ -779,24 +777,24 @@ int OGRDXFWriterDS::WriteNewBlockDefinitions( VSILFILE * fp )
         CPLDebug( "DXF", "Writing BLOCK definition for '%s'.",
                   poThisBlockFeat->GetFieldAsString("BlockName") );
 
-        WriteValue( fp, 0, "BLOCK" );
-        WriteEntityID( fp );
-        WriteValue( fp, 100, "AcDbEntity" );
+        WriteValue( fpIn, 0, "BLOCK" );
+        WriteEntityID( fpIn );
+        WriteValue( fpIn, 100, "AcDbEntity" );
         if( strlen(poThisBlockFeat->GetFieldAsString("Layer")) > 0 )
-            WriteValue( fp, 8, poThisBlockFeat->GetFieldAsString("Layer") );
+            WriteValue( fpIn, 8, poThisBlockFeat->GetFieldAsString("Layer") );
         else
-            WriteValue( fp, 8, "0" );
-        WriteValue( fp, 100, "AcDbBlockBegin" );
-        WriteValue( fp, 2, poThisBlockFeat->GetFieldAsString("BlockName") );
-        WriteValue( fp, 70, "0" );
+            WriteValue( fpIn, 8, "0" );
+        WriteValue( fpIn, 100, "AcDbBlockBegin" );
+        WriteValue( fpIn, 2, poThisBlockFeat->GetFieldAsString("BlockName") );
+        WriteValue( fpIn, 70, "0" );
 
         // Origin
-        WriteValue( fp, 10, "0.0" );
-        WriteValue( fp, 20, "0.0" );
-        WriteValue( fp, 30, "0.0" );
+        WriteValue( fpIn, 10, "0.0" );
+        WriteValue( fpIn, 20, "0.0" );
+        WriteValue( fpIn, 30, "0.0" );
 
-        WriteValue( fp, 3, poThisBlockFeat->GetFieldAsString("BlockName") );
-        WriteValue( fp, 1, "" );
+        WriteValue( fpIn, 3, poThisBlockFeat->GetFieldAsString("BlockName") );
+        WriteValue( fpIn, 1, "" );
 
 /* -------------------------------------------------------------------- */
 /*      Write out the feature entities.                                 */
@@ -807,28 +805,28 @@ int OGRDXFWriterDS::WriteNewBlockDefinitions( VSILFILE * fp )
 /* -------------------------------------------------------------------- */
 /*      Write out following features if they are the same block.        */
 /* -------------------------------------------------------------------- */
-        while( iBlock < poBlocksLayer->apoBlocks.size()-1 
+        while( iBlock < poBlocksLayer->apoBlocks.size()-1
             && EQUAL(poBlocksLayer->apoBlocks[iBlock+1]->GetFieldAsString("BlockName"),
                      osBlockName) )
         {
             iBlock++;
-            
+
             if( poLayer->CreateFeature( poBlocksLayer->apoBlocks[iBlock] )
                 != OGRERR_NONE )
                 return FALSE;
         }
-        
+
 /* -------------------------------------------------------------------- */
 /*      Write out the block definition postamble.                       */
 /* -------------------------------------------------------------------- */
-        WriteValue( fp, 0, "ENDBLK" );
-        WriteEntityID( fp );
-        WriteValue( fp, 100, "AcDbEntity" );
+        WriteValue( fpIn, 0, "ENDBLK" );
+        WriteEntityID( fpIn );
+        WriteValue( fpIn, 100, "AcDbEntity" );
         if( strlen(poThisBlockFeat->GetFieldAsString("Layer")) > 0 )
-            WriteValue( fp, 8, poThisBlockFeat->GetFieldAsString("Layer") );
+            WriteValue( fpIn, 8, poThisBlockFeat->GetFieldAsString("Layer") );
         else
-            WriteValue( fp, 8, "0" );
-        WriteValue( fp, 100, "AcDbBlockEnd" );
+            WriteValue( fpIn, 8, "0" );
+        WriteValue( fpIn, 100, "AcDbBlockEnd" );
     }
 
     return TRUE;
@@ -847,17 +845,17 @@ void OGRDXFWriterDS::ScanForEntities( const char *pszFilename,
 
 {
     OGRDXFReader oReader;
-    VSILFILE *fp;
+    VSILFILE *l_fp;
 
 /* -------------------------------------------------------------------- */
 /*      Open the file and setup a reader.                               */
 /* -------------------------------------------------------------------- */
-    fp = VSIFOpenL( pszFilename, "r" );
+    l_fp = VSIFOpenL( pszFilename, "r" );
 
-    if( fp == NULL )
+    if( l_fp == NULL )
         return;
 
-    oReader.Initialize( fp );
+    oReader.Initialize( l_fp );
 
 /* -------------------------------------------------------------------- */
 /*      Add every code "5" line to our entities list.                   */
@@ -873,7 +871,7 @@ void OGRDXFWriterDS::ScanForEntities( const char *pszFilename,
             CPLString osEntity( szLineBuf );
 
             if( CheckEntityID( osEntity ) )
-                CPLDebug( "DXF", "Encounted entity '%s' multiple times.",
+                CPLDebug( "DXF", "Encountered entity '%s' multiple times.",
                           osEntity.c_str() );
             else
                 aosUsedEntities.insert( osEntity );
@@ -889,7 +887,7 @@ void OGRDXFWriterDS::ScanForEntities( const char *pszFilename,
         }
     }
 
-    VSIFCloseL( fp );
+    VSIFCloseL( l_fp );
 }
 
 /************************************************************************/
@@ -914,31 +912,31 @@ int OGRDXFWriterDS::CheckEntityID( const char *pszEntityID )
 /*                           WriteEntityID()                            */
 /************************************************************************/
 
-long OGRDXFWriterDS::WriteEntityID( VSILFILE *fp, long nPreferredFID )
+long OGRDXFWriterDS::WriteEntityID( VSILFILE *fpIn, long nPreferredFID )
 
 {
     CPLString osEntityID;
 
     if( nPreferredFID != OGRNullFID )
     {
-        
+
         osEntityID.Printf( "%X", (unsigned int) nPreferredFID );
         if( !CheckEntityID( osEntityID ) )
         {
             aosUsedEntities.insert( osEntityID );
-            WriteValue( fp, 5, osEntityID );
+            WriteValue( fpIn, 5, osEntityID );
             return nPreferredFID;
         }
     }
 
-    do 
+    do
     {
         osEntityID.Printf( "%X", nNextFID++ );
     }
     while( CheckEntityID( osEntityID ) );
-    
+
     aosUsedEntities.insert( osEntityID );
-    WriteValue( fp, 5, osEntityID );
+    WriteValue( fpIn, 5, osEntityID );
 
     return nNextFID - 1;
 }
