@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrdxfwriterlayer.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: ogrdxfwriterlayer.cpp 33713 2016-03-12 17:41:57Z goatbar $
  *
  * Project:  DXF Translator
  * Purpose:  Implements OGRDXFWriterLayer - the OGRLayer class used for
@@ -34,24 +34,20 @@
 #include "cpl_string.h"
 #include "ogr_featurestyle.h"
 
-CPL_CVSID("$Id: ogrdxfwriterlayer.cpp 27729 2014-09-24 00:40:16Z goatbar $");
-
-#ifndef PI
-#define PI  3.14159265358979323846
-#endif 
+CPL_CVSID("$Id: ogrdxfwriterlayer.cpp 33713 2016-03-12 17:41:57Z goatbar $");
 
 /************************************************************************/
 /*                         OGRDXFWriterLayer()                          */
 /************************************************************************/
 
-OGRDXFWriterLayer::OGRDXFWriterLayer( OGRDXFWriterDS *poDS, VSILFILE *fp )
+OGRDXFWriterLayer::OGRDXFWriterLayer( OGRDXFWriterDS *poDSIn, VSILFILE *fpIn )
 
 {
-    this->fp = fp;
-    this->poDS = poDS;
+    this->fp = fpIn;
+    this->poDS = poDSIn;
 
     nNextAutoID = 1;
-    bWriteHatch = CSLTestBoolean(CPLGetConfigOption("DXF_WRITE_HATCH", "YES"));
+    bWriteHatch = CPLTestBool(CPLGetConfigOption("DXF_WRITE_HATCH", "YES"));
 
     poFeatureDefn = new OGRFeatureDefn( "entities" );
     poFeatureDefn->Reference();
@@ -76,10 +72,10 @@ OGRDXFWriterLayer::OGRDXFWriterLayer( OGRDXFWriterDS *poDS, VSILFILE *fp )
 
     OGRFieldDefn  oBlockField( "BlockName", OFTString );
     poFeatureDefn->AddFieldDefn( &oBlockField );
-    
+
     OGRFieldDefn  oScaleField( "BlockScale", OFTRealList );
     poFeatureDefn->AddFieldDefn( &oScaleField );
-    
+
     OGRFieldDefn  oBlockAngleField( "BlockAngle", OFTReal );
     poFeatureDefn->AddFieldDefn( &oBlockAngleField );
 }
@@ -118,7 +114,7 @@ int OGRDXFWriterLayer::TestCapability( const char * pszCap )
         return TRUE;
     else if( EQUAL(pszCap,OLCSequentialWrite) )
         return TRUE;
-    else 
+    else
         return FALSE;
 }
 
@@ -161,7 +157,7 @@ int OGRDXFWriterLayer::WriteValue( int nCode, const char *pszValue )
 
     osLinePair += "\n";
 
-    return VSIFWriteL( osLinePair.c_str(), 
+    return VSIFWriteL( osLinePair.c_str(),
                        1, osLinePair.size(), fp ) == osLinePair.size();
 }
 
@@ -176,7 +172,7 @@ int OGRDXFWriterLayer::WriteValue( int nCode, int nValue )
 
     osLinePair.Printf( "%3d\n%d\n", nCode, nValue );
 
-    return VSIFWriteL( osLinePair.c_str(), 
+    return VSIFWriteL( osLinePair.c_str(),
                        1, osLinePair.size(), fp ) == osLinePair.size();
 }
 
@@ -189,13 +185,10 @@ int OGRDXFWriterLayer::WriteValue( int nCode, double dfValue )
 {
     char szLinePair[64];
 
-    snprintf(szLinePair, sizeof(szLinePair), "%3d\n%.15g\n", nCode, dfValue );
-    char* pszComma = strchr(szLinePair, ',');
-    if (pszComma)
-        *pszComma = '.';
+    CPLsnprintf(szLinePair, sizeof(szLinePair), "%3d\n%.15g\n", nCode, dfValue );
     size_t nLen = strlen(szLinePair);
 
-    return VSIFWriteL( szLinePair, 
+    return VSIFWriteL( szLinePair,
                        1, nLen, fp ) == nLen;
 }
 
@@ -215,7 +208,7 @@ OGRErr OGRDXFWriterLayer::WriteCore( OGRFeature *poFeature )
 /*      Also, for reasons I don't understand these ids seem to have     */
 /*      to start somewhere around 0x50 hex (80 decimal).                */
 /* -------------------------------------------------------------------- */
-    poFeature->SetFID( poDS->WriteEntityID(fp,poFeature->GetFID()) );
+    poFeature->SetFID( poDS->WriteEntityID(fp,(int)poFeature->GetFID()) );
 
 /* -------------------------------------------------------------------- */
 /*      For now we assign everything to the default layer - layer       */
@@ -228,12 +221,12 @@ OGRErr OGRDXFWriterLayer::WriteCore( OGRFeature *poFeature )
     }
     else
     {
-        const char *pszExists = 
+        const char *pszExists =
             poDS->oHeaderDS.LookupLayerProperty( pszLayer, "Exists" );
         if( (pszExists == NULL || strlen(pszExists) == 0)
             && CSLFindString( poDS->papszLayersToCreate, pszLayer ) == -1 )
         {
-            poDS->papszLayersToCreate = 
+            poDS->papszLayersToCreate =
                 CSLAddString( poDS->papszLayersToCreate, pszLayer );
         }
 
@@ -275,14 +268,14 @@ OGRErr OGRDXFWriterLayer::WriteINSERT( OGRFeature *poFeature )
             WriteValue( 62, ColorStringToDXFColor( poSymbol->Color(bDefault) ) );
     }
     delete poTool;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Write location.                                                 */
 /* -------------------------------------------------------------------- */
     OGRPoint *poPoint = (OGRPoint *) poFeature->GetGeometryRef();
 
     WriteValue( 10, poPoint->getX() );
-    if( !WriteValue( 20, poPoint->getY() ) ) 
+    if( !WriteValue( 20, poPoint->getY() ) )
         return OGRERR_FAILURE;
 
     if( poPoint->getGeometryType() == wkbPoint25D )
@@ -290,12 +283,12 @@ OGRErr OGRDXFWriterLayer::WriteINSERT( OGRFeature *poFeature )
         if( !WriteValue( 30, poPoint->getZ() ) )
             return OGRERR_FAILURE;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Write scaling.                                                  */
 /* -------------------------------------------------------------------- */
     int nScaleCount;
-    const double *padfScale = 
+    const double *padfScale =
         poFeature->GetFieldAsDoubleList( "BlockScale", &nScaleCount );
 
     if( nScaleCount == 3 )
@@ -353,7 +346,7 @@ OGRErr OGRDXFWriterLayer::WritePOINT( OGRFeature *poFeature )
     OGRPoint *poPoint = (OGRPoint *) poFeature->GetGeometryRef();
 
     WriteValue( 10, poPoint->getX() );
-    if( !WriteValue( 20, poPoint->getY() ) ) 
+    if( !WriteValue( 20, poPoint->getY() ) )
         return OGRERR_FAILURE;
 
     if( poPoint->getGeometryType() == wkbPoint25D )
@@ -361,7 +354,7 @@ OGRErr OGRDXFWriterLayer::WritePOINT( OGRFeature *poFeature )
         if( !WriteValue( 30, poPoint->getZ() ) )
             return OGRERR_FAILURE;
     }
-    
+
     return OGRERR_NONE;
 }
 
@@ -378,8 +371,8 @@ CPLString OGRDXFWriterLayer::TextEscape( const char *pszInput )
 
 {
     CPLString osResult;
-    wchar_t *panInput = CPLRecodeToWChar( pszInput, 
-                                          CPL_ENC_UTF8, 
+    wchar_t *panInput = CPLRecodeToWChar( pszInput,
+                                          CPL_ENC_UTF8,
                                           CPL_ENC_UCS2 );
     int i;
 
@@ -403,7 +396,7 @@ CPLString OGRDXFWriterLayer::TextEscape( const char *pszInput )
     }
 
     CPLFree(panInput);
-    
+
     return osResult;
 }
 
@@ -445,7 +438,7 @@ OGRErr OGRDXFWriterLayer::WriteTEXT( OGRFeature *poFeature )
 /*      Color                                                           */
 /* -------------------------------------------------------------------- */
         if( poLabel->ForeColor(bDefault) != NULL && !bDefault )
-            WriteValue( 62, ColorStringToDXFColor( 
+            WriteValue( 62, ColorStringToDXFColor(
                             poLabel->ForeColor(bDefault) ) );
 
 /* -------------------------------------------------------------------- */
@@ -472,10 +465,10 @@ OGRErr OGRDXFWriterLayer::WriteTEXT( OGRFeature *poFeature )
 /*      Anchor / Attachment Point                                       */
 /* -------------------------------------------------------------------- */
         int nAnchor = poLabel->Anchor(bDefault);
-        
+
         if( !bDefault )
         {
-            const static int anAnchorMap[] = 
+            const static int anAnchorMap[] =
                 { -1, 7, 8, 9, 4, 5, 6, 1, 2, 3, 7, 8, 9 };
 
             if( nAnchor > 0 && nAnchor < 13 )
@@ -502,7 +495,7 @@ OGRErr OGRDXFWriterLayer::WriteTEXT( OGRFeature *poFeature )
     OGRPoint *poPoint = (OGRPoint *) poFeature->GetGeometryRef();
 
     WriteValue( 10, poPoint->getX() );
-    if( !WriteValue( 20, poPoint->getY() ) ) 
+    if( !WriteValue( 20, poPoint->getY() ) )
         return OGRERR_FAILURE;
 
     if( poPoint->getGeometryType() == wkbPoint25D )
@@ -510,7 +503,7 @@ OGRErr OGRDXFWriterLayer::WriteTEXT( OGRFeature *poFeature )
         if( !WriteValue( 30, poPoint->getZ() ) )
             return OGRERR_FAILURE;
     }
-    
+
     return OGRERR_NONE;
 
 }
@@ -518,7 +511,7 @@ OGRErr OGRDXFWriterLayer::WriteTEXT( OGRFeature *poFeature )
 /************************************************************************/
 /*                     PrepareLineTypeDefinition()                      */
 /************************************************************************/
-CPLString 
+CPLString
 OGRDXFWriterLayer::PrepareLineTypeDefinition( CPL_UNUSED OGRFeature *poFeature,
                                               OGRStyleTool *poTool )
 {
@@ -526,12 +519,12 @@ OGRDXFWriterLayer::PrepareLineTypeDefinition( CPL_UNUSED OGRFeature *poFeature,
     OGRStylePen *poPen = (OGRStylePen *) poTool;
     GBool  bDefault;
     const char *pszPattern;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Fetch pattern.                                                  */
 /* -------------------------------------------------------------------- */
     pszPattern = poPen->Pattern( bDefault );
-    if( bDefault || strlen(pszPattern) == 0 ) 
+    if( bDefault || strlen(pszPattern) == 0 )
         return "";
 
 /* -------------------------------------------------------------------- */
@@ -549,35 +542,33 @@ OGRDXFWriterLayer::PrepareLineTypeDefinition( CPL_UNUSED OGRFeature *poFeature,
         CPLString osDXFEntry;
 
         // Split amount and unit.
-        for( pszUnit = pszToken; 
+        for( pszUnit = pszToken;
              strchr( "0123456789.", *pszUnit) != NULL;
              pszUnit++ ) {}
 
         osAmount.assign(pszToken,(int) (pszUnit-pszToken));
-        
-        // If the unit is other than 'g' we really should be trying to 
+
+        // If the unit is other than 'g' we really should be trying to
         // do some type of transformation - but what to do?  Pretty hard.
-        
-        // 
 
         // Even entries are "pen down" represented as negative in DXF.
         if( i%2 == 0 )
             osDXFEntry.Printf( " 49\n-%s\n 74\n0\n", osAmount.c_str() );
         else
             osDXFEntry.Printf( " 49\n%s\n 74\n0\n", osAmount.c_str() );
-        
+
         osDef += osDXFEntry;
 
-        dfTotalLength += atof(osAmount);
+        dfTotalLength += CPLAtof(osAmount);
     }
 
 /* -------------------------------------------------------------------- */
 /*      Prefix 73 and 40 items to the definition.                       */
 /* -------------------------------------------------------------------- */
     CPLString osPrefix;
-    
-    osPrefix.Printf( " 73\n%d\n 40\n%.6g\n", 
-                     CSLCount(papszTokens), 
+
+    osPrefix.Printf( " 73\n%d\n 40\n%.6g\n",
+                     CSLCount(papszTokens),
                      dfTotalLength );
     osDef = osPrefix + osDef;
 
@@ -605,16 +596,16 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
     {
         return OGRERR_NONE;
     }
-            
-    if( wkbFlatten(poGeom->getGeometryType()) == wkbMultiPolygon 
+
+    if( wkbFlatten(poGeom->getGeometryType()) == wkbMultiPolygon
         || wkbFlatten(poGeom->getGeometryType()) == wkbMultiLineString )
     {
         OGRGeometryCollection *poGC = (OGRGeometryCollection *) poGeom;
         int iGeom;
         OGRErr eErr = OGRERR_NONE;
 
-        for( iGeom = 0; 
-             eErr == OGRERR_NONE && iGeom < poGC->getNumGeometries(); 
+        for( iGeom = 0;
+             eErr == OGRERR_NONE && iGeom < poGC->getNumGeometries();
              iGeom++ )
         {
             eErr = WritePOLYLINE( poFeature, poGC->getGeometryRef( iGeom ) );
@@ -633,8 +624,8 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
         OGRErr eErr;
 
         eErr = WritePOLYLINE( poFeature, poPoly->getExteriorRing() );
-        for( iGeom = 0; 
-             eErr == OGRERR_NONE && iGeom < poPoly->getNumInteriorRings(); 
+        for( iGeom = 0;
+             eErr == OGRERR_NONE && iGeom < poPoly->getNumInteriorRings();
              iGeom++ )
         {
             eErr = WritePOLYLINE( poFeature, poPoly->getInteriorRing(iGeom) );
@@ -716,7 +707,7 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
         if( poPen->Color(bDefault) != NULL && !bDefault )
             WriteValue( 62, ColorStringToDXFColor( poPen->Color(bDefault) ) );
 
-        // we want to fetch the width in ground units. 
+        // we want to fetch the width in ground units.
         poPen->SetUnit( OGRSTUGround, 1.0 );
         double dfWidth = poPen->Width(bDefault);
 
@@ -729,8 +720,8 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
 /* -------------------------------------------------------------------- */
     CPLString osLineType = poFeature->GetFieldAsString( "Linetype" );
 
-    if( osLineType.size() > 0 
-        && (poDS->oHeaderDS.LookupLineType( osLineType ) != NULL 
+    if( osLineType.size() > 0
+        && (poDS->oHeaderDS.LookupLineType( osLineType ) != NULL
             || oNewLineTypes.count(osLineType) > 0 ) )
     {
         // Already define -> just reference it.
@@ -738,7 +729,7 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
     }
     else if( poTool != NULL && poTool->GetType() == OGRSTCPen )
     {
-        CPLString osDefinition = PrepareLineTypeDefinition( poFeature, 
+        CPLString osDefinition = PrepareLineTypeDefinition( poFeature,
                                                             poTool );
 
         if( osDefinition != "" && osLineType == "" )
@@ -760,8 +751,8 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
             // create an automatic name for it.
             if( osLineType == "" )
             {
-                do 
-                { 
+                do
+                {
                     osLineType.Printf( "AutoLineType-%d", nNextAutoID++ );
                 }
                 while( poDS->oHeaderDS.LookupLineType(osLineType) != NULL );
@@ -791,7 +782,7 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
 
     for( iVert = 0; iVert < poLS->getNumPoints(); iVert++ )
     {
-        if( bHasDifferentZ ) 
+        if( bHasDifferentZ )
         {
             WriteValue( 0, "VERTEX" );
             WriteValue( 100, "AcDbEntity" );
@@ -800,7 +791,7 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
             WriteCore( poFeature );
         }
         WriteValue( 10, poLS->getX(iVert) );
-        if( !WriteValue( 20, poLS->getY(iVert) ) ) 
+        if( !WriteValue( 20, poLS->getY(iVert) ) )
             return OGRERR_FAILURE;
 
         if( bHasDifferentZ )
@@ -817,7 +808,7 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
         WriteCore( poFeature );
         WriteValue( 100, "AcDbEntity" );
     }
-    
+
     delete poTool;
 
     return OGRERR_NONE;
@@ -843,7 +834,7 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
         WriteValue( 0, "VERTEX" );
         WriteValue( 8, "0" );
         WriteValue( 10, poLS->getX(iVert) );
-        if( !WriteValue( 20, poLS->getY(iVert) ) ) 
+        if( !WriteValue( 20, poLS->getY(iVert) ) )
             return OGRERR_FAILURE;
 
         if( poLS->getGeometryType() == wkbLineString25D )
@@ -855,7 +846,7 @@ OGRErr OGRDXFWriterLayer::WritePOLYLINE( OGRFeature *poFeature,
 
     WriteValue( 0, "SEQEND" );
     WriteValue( 8, "0" );
-    
+
     return OGRERR_NONE;
 #endif
 }
@@ -879,15 +870,15 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
     {
         return OGRERR_NONE;
     }
-            
+
     if( wkbFlatten(poGeom->getGeometryType()) == wkbMultiPolygon )
     {
         OGRGeometryCollection *poGC = (OGRGeometryCollection *) poGeom;
         int iGeom;
         OGRErr eErr = OGRERR_NONE;
 
-        for( iGeom = 0; 
-             eErr == OGRERR_NONE && iGeom < poGC->getNumGeometries(); 
+        for( iGeom = 0;
+             eErr == OGRERR_NONE && iGeom < poGC->getNumGeometries();
              iGeom++ )
         {
             eErr = WriteHATCH( poFeature, poGC->getGeometryRef( iGeom ) );
@@ -919,7 +910,7 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
 
     WriteValue( 2, "SOLID" ); // fill pattern
     WriteValue( 70, 1 ); // solid fill
-    WriteValue( 71, 0 ); // associativity 
+    WriteValue( 71, 0 ); // associativity
 
 /* -------------------------------------------------------------------- */
 /*      Do we have styling information?                                 */
@@ -957,7 +948,7 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
 
         if( poPen->Color(bDefault) != NULL && !bDefault )
             WriteValue( 62, ColorStringToDXFColor( poPen->Color(bDefault) ) );
-        
+
         double dfWidthInMM = poPen->Width(bDefault);
 
         if( !bDefault )
@@ -969,8 +960,8 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
 /* -------------------------------------------------------------------- */
     CPLString osLineType = poFeature->GetFieldAsString( "Linetype" );
 
-    if( osLineType.size() > 0 
-        && (poDS->oHeaderDS.LookupLineType( osLineType ) != NULL 
+    if( osLineType.size() > 0
+        && (poDS->oHeaderDS.LookupLineType( osLineType ) != NULL
             || oNewLineTypes.count(osLineType) > 0 ) )
     {
         // Already define -> just reference it.
@@ -978,7 +969,7 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
     }
     else if( poTool != NULL && poTool->GetType() == OGRSTCPen )
     {
-        CPLString osDefinition = PrepareLineTypeDefinition( poFeature, 
+        CPLString osDefinition = PrepareLineTypeDefinition( poFeature,
                                                             poTool );
 
         if( osDefinition != "" && osLineType == "" )
@@ -1000,8 +991,8 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
             // create an automatic name for it.
             if( osLineType == "" )
             {
-                do 
-                { 
+                do
+                {
                     osLineType.Printf( "AutoLineType-%d", nNextAutoID++ );
                 }
                 while( poDS->oHeaderDS.LookupLineType(osLineType) != NULL );
@@ -1038,7 +1029,7 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
         WriteValue( 72, 0 ); // has bulge
         WriteValue( 73, 1 ); // is closed
         WriteValue( 93, poLR->getNumPoints() );
-        
+
         for( int iVert = 0; iVert < poLR->getNumPoints(); iVert++ )
         {
             WriteValue( 10, poLR->getX(iVert) );
@@ -1051,7 +1042,7 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
     WriteValue( 75, 0 ); // hatch style = Hatch "odd parity" area (Normal style)
     WriteValue( 76, 1 ); // hatch pattern type = predefined
     WriteValue( 98, 0 ); // 0 seed points
-    
+
     return OGRERR_NONE;
 
 #ifdef notdef
@@ -1075,7 +1066,7 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
         WriteValue( 0, "VERTEX" );
         WriteValue( 8, "0" );
         WriteValue( 10, poLS->getX(iVert) );
-        if( !WriteValue( 20, poLS->getY(iVert) ) ) 
+        if( !WriteValue( 20, poLS->getY(iVert) ) )
             return OGRERR_FAILURE;
 
         if( poLS->getGeometryType() == wkbLineString25D )
@@ -1087,21 +1078,21 @@ OGRErr OGRDXFWriterLayer::WriteHATCH( OGRFeature *poFeature,
 
     WriteValue( 0, "SEQEND" );
     WriteValue( 8, "0" );
-    
+
     return OGRERR_NONE;
 #endif
 }
 
 /************************************************************************/
-/*                           CreateFeature()                            */
+/*                           ICreateFeature()                            */
 /************************************************************************/
 
-OGRErr OGRDXFWriterLayer::CreateFeature( OGRFeature *poFeature )
+OGRErr OGRDXFWriterLayer::ICreateFeature( OGRFeature *poFeature )
 
 {
     OGRGeometry *poGeom = poFeature->GetGeometryRef();
     OGRwkbGeometryType eGType = wkbNone;
-    
+
     if( poGeom != NULL )
     {
         if( !poGeom->IsEmpty() )
@@ -1119,33 +1110,33 @@ OGRErr OGRDXFWriterLayer::CreateFeature( OGRFeature *poFeature )
 
         // we don't want to treat as a block ref if we are writing blocks layer
         if( pszBlockName != NULL
-            && poDS->poBlocksLayer != NULL 
+            && poDS->poBlocksLayer != NULL
             && poFeature->GetDefnRef() == poDS->poBlocksLayer->GetLayerDefn())
             pszBlockName = NULL;
 
         // We don't want to treat as a blocks ref if the block is not defined
-        if( pszBlockName 
+        if( pszBlockName
             && poDS->oHeaderDS.LookupBlock(pszBlockName) == NULL )
         {
             if( poDS->poBlocksLayer == NULL
                 || poDS->poBlocksLayer->FindBlock(pszBlockName) == NULL )
                 pszBlockName = NULL;
         }
-                                  
+
         if( pszBlockName != NULL )
             return WriteINSERT( poFeature );
-            
+
         else if( poFeature->GetStyleString() != NULL
-            && EQUALN(poFeature->GetStyleString(),"LABEL",5) )
+            && STARTS_WITH_CI(poFeature->GetStyleString(), "LABEL") )
             return WriteTEXT( poFeature );
         else
             return WritePOINT( poFeature );
     }
-    else if( eGType == wkbLineString 
+    else if( eGType == wkbLineString
              || eGType == wkbMultiLineString )
         return WritePOLYLINE( poFeature );
 
-    else if( eGType == wkbPolygon 
+    else if( eGType == wkbPolygon
              || eGType == wkbMultiPolygon )
     {
         if( bWriteHatch )
@@ -1164,18 +1155,18 @@ OGRErr OGRDXFWriterLayer::CreateFeature( OGRFeature *poFeature )
         for( iGeom = 0; iGeom < poGC->getNumGeometries(); iGeom++ )
         {
             poFeature->SetGeometry( poGC->getGeometryRef(iGeom) );
-                                    
+
             OGRErr eErr = CreateFeature( poFeature );
-            
+
             if( eErr != OGRERR_NONE )
                 return eErr;
 
         }
-        
+
         poFeature->SetGeometryDirectly( poGC );
         return OGRERR_NONE;
     }
-    else 
+    else
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "No known way to write feature with geometry '%s'.",
@@ -1199,9 +1190,9 @@ int OGRDXFWriterLayer::ColorStringToDXFColor( const char *pszRGB )
 
     int nRed, nGreen, nBlue, nTransparency = 255;
 
-    int nCount  = sscanf(pszRGB,"#%2x%2x%2x%2x",&nRed,&nGreen,&nBlue, 
+    int nCount  = sscanf(pszRGB,"#%2x%2x%2x%2x",&nRed,&nGreen,&nBlue,
                          &nTransparency);
-   
+
     if (nCount < 3 )
         return -1;
 
@@ -1225,6 +1216,6 @@ int OGRDXFWriterLayer::ColorStringToDXFColor( const char *pszRGB )
             nMinDist = nDist;
         }
     }
-    
+
     return nBestColor;
 }
