@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogredigeodriver.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: ogredigeodriver.cpp 32110 2015-12-10 17:19:40Z goatbar $
  *
  * Project:  EDIGEO Translator
  * Purpose:  Implements OGREDIGEODriver.
@@ -30,41 +30,37 @@
 #include "ogr_edigeo.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogredigeodriver.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: ogredigeodriver.cpp 32110 2015-12-10 17:19:40Z goatbar $");
 
 extern "C" void RegisterOGREDIGEO();
 
 // g++ -fPIC -g -Wall ogr/ogrsf_frmts/edigeo/*.cpp -shared -o ogr_EDIGEO.so -Iport -Igcore -Iogr -Iogr/ogrsf_frmts -Iogr/ogrsf_frmts/generic -Iogr/ogrsf_frmts/edigeo -L. -lgdal
 
 /************************************************************************/
-/*                         ~OGREDIGEODriver()                           */
+/*                        OGREDIGEODriverIdentify()                     */
 /************************************************************************/
 
-OGREDIGEODriver::~OGREDIGEODriver()
+static int OGREDIGEODriverIdentify( GDALOpenInfo * poOpenInfo )
 
 {
-}
-
-/************************************************************************/
-/*                              GetName()                               */
-/************************************************************************/
-
-const char *OGREDIGEODriver::GetName()
-
-{
-    return "EDIGEO";
+    return poOpenInfo->fpL != NULL &&
+           EQUAL(CPLGetExtension(poOpenInfo->pszFilename), "thf");
 }
 
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
 
-OGRDataSource *OGREDIGEODriver::Open( const char * pszFilename, int bUpdate )
+static GDALDataset *OGREDIGEODriverOpen( GDALOpenInfo * poOpenInfo )
 
 {
+    if( poOpenInfo->eAccess == GA_Update ||
+        !OGREDIGEODriverIdentify(poOpenInfo) )
+        return NULL;
+
     OGREDIGEODataSource   *poDS = new OGREDIGEODataSource();
 
-    if( !poDS->Open( pszFilename, bUpdate ) )
+    if( !poDS->Open( poOpenInfo->pszFilename ) )
     {
         delete poDS;
         poDS = NULL;
@@ -73,15 +69,6 @@ OGRDataSource *OGREDIGEODriver::Open( const char * pszFilename, int bUpdate )
     return poDS;
 }
 
-/************************************************************************/
-/*                           TestCapability()                           */
-/************************************************************************/
-
-int OGREDIGEODriver::TestCapability( CPL_UNUSED const char * pszCap )
-{
-    return FALSE;
-}
-
 /************************************************************************/
 /*                           RegisterOGREDIGEO()                        */
 /************************************************************************/
@@ -89,6 +76,23 @@ int OGREDIGEODriver::TestCapability( CPL_UNUSED const char * pszCap )
 void RegisterOGREDIGEO()
 
 {
-    OGRSFDriverRegistrar::GetRegistrar()->RegisterDriver( new OGREDIGEODriver );
+    if( GDALGetDriverByName( "EDIGEO" ) != NULL )
+        return;
+
+    GDALDriver  *poDriver = new GDALDriver();
+
+    poDriver->SetDescription( "EDIGEO" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VECTOR, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
+                               "French EDIGEO exchange format" );
+    poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "thf" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drv_edigeo.html" );
+
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+
+    poDriver->pfnOpen = OGREDIGEODriverOpen;
+    poDriver->pfnIdentify = OGREDIGEODriverIdentify;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
 
