@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: FGdbDatasource.cpp 26310 2013-08-13 19:54:26Z rouault $
+ * $Id: FGdbDatasource.cpp 33563 2016-02-26 14:57:06Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements FileGDB OGR Datasource.
@@ -9,6 +9,7 @@
  ******************************************************************************
  * Copyright (c) 2010, Ragi Yaser Burhum
  * Copyright (c) 2011, Paul Ramsey <pramsey at cleverelephant.ca>
+ * Copyright (c) 2011-2013, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -34,8 +35,9 @@
 #include "cpl_string.h"
 #include "gdal.h"
 #include "FGdbUtils.h"
+#include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: FGdbDatasource.cpp 26310 2013-08-13 19:54:26Z rouault $");
+CPL_CVSID("$Id: FGdbDatasource.cpp 33563 2016-02-26 14:57:06Z rouault $");
 
 using std::vector;
 using std::wstring;
@@ -44,10 +46,13 @@ using std::wstring;
 /*                          FGdbDataSource()                           */
 /************************************************************************/
 
-FGdbDataSource::FGdbDataSource(FGdbDriver* poDriver):
+FGdbDataSource::FGdbDataSource(FGdbDriver* poDriverIn, 
+                               FGdbDatabaseConnection* pConnection):
 OGRDataSource(),
-m_poDriver(poDriver), m_pszName(0), m_pGeodatabase(NULL)
+m_poDriver(poDriverIn), m_pConnection(pConnection), m_pGeodatabase(NULL), m_bUpdate(false),
+m_poOpenFileGDBDrv(NULL)
 {
+    bPerLayerCopyingForTransaction = -1;
 }
 
 /************************************************************************/
@@ -56,23 +61,107 @@ m_poDriver(poDriver), m_pszName(0), m_pGeodatabase(NULL)
 
 FGdbDataSource::~FGdbDataSource()
 {
+    CPLMutexHolderOptionalLockD(m_poDriver ? m_poDriver->GetMutex() : NULL);
+    
+    if( m_pConnection && m_pConnection->IsLocked() )
+        CommitTransaction();
+
+    //Close();
     size_t count = m_layers.size();
     for(size_t i = 0; i < count; ++i )
-        delete m_layers[i];
+    {
+        m_layers[i]->CloseGDBObjects();
+    }
 
-    m_poDriver->Release( m_pszName );
-    CPLFree( m_pszName );
+    FixIndexes();
+
+    if( m_poDriver )
+        m_poDriver->Release( m_osPublicName );
+
+    //size_t count = m_layers.size();
+    for(size_t i = 0; i < count; ++i )
+    {
+        delete m_layers[i];
+    }
 }
 
+/************************************************************************/
+/*                             FixIndexes()                             */
+/************************************************************************/
+
+int FGdbDataSource::FixIndexes()
+{
+    int bRet = TRUE;
+    if( m_pConnection && m_pConnection->IsFIDHackInProgress() )
+    {
+        m_pConnection->CloseGeodatabase();
+
+        char* apszDrivers[2];
+        apszDrivers[0] = (char*) "OpenFileGDB";
+        apszDrivers[1] = NULL;
+        const char* pszSystemCatalog = CPLFormFilename(m_osFSName, "a00000001.gdbtable", NULL);
+        GDALDataset* poOpenFileGDBDS = (GDALDataset*)
+            GDALOpenEx(pszSystemCatalog, GDAL_OF_VECTOR,
+                       apszDrivers, NULL, NULL);
+        if( poOpenFileGDBDS == NULL || poOpenFileGDBDS->GetLayer(0) == NULL )
+        {
+            CPLError(CE_Failure, CPLE_AppDefined,
+                     "Cannot open %s with OpenFileGDB driver. "
+                     "Should not happen. Some layers will be corrupted",
+                     pszSystemCatalog);
+            bRet = FALSE;
+        }
+        else
+        {
+            OGRLayer* poLayer = poOpenFileGDBDS->GetLayer(0);
+            size_t count = m_layers.size();
+            for(size_t i = 0; i < count; ++i )
+            {
+                if( m_layers[i]->m_oMapOGRFIDToFGDBFID.size() == 0)
+                    continue;
+                CPLString osFilter = "name = '";
+                osFilter += m_layers[i]->GetName();
+                osFilter += "'";
+                poLayer->SetAttributeFilter(osFilter);
+                poLayer->ResetReading();
+                OGRFeature* poF = poLayer->GetNextFeature();
+                if( poF == NULL )
+                {
+                    CPLError(CE_Failure, CPLE_AppDefined,
+                             "Cannot find filename for layer %s",
+                             m_layers[i]->GetName());
+                    bRet = FALSE;
+                }
+                else
+                {
+                    if( !m_layers[i]->EditIndexesForFIDHack(CPLFormFilename(m_osFSName,
+                                        CPLSPrintf("a%08x", (int)poF->GetFID()), NULL)) )
+                    {
+                        bRet = FALSE;
+                    }
+                }
+                delete poF;
+            }
+        }
+        GDALClose(poOpenFileGDBDS);
+
+        m_pConnection->SetFIDHackInProgress(FALSE);
+    }
+    return bRet;
+}
 
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
 
-int FGdbDataSource::Open(Geodatabase* pGeodatabase, const char * pszNewName, int bUpdate )
+int FGdbDataSource::Open(const char * pszNewName, int bUpdate,
+                         const char* pszPublicName )
 {
-    m_pszName = CPLStrdup( pszNewName );
-    m_pGeodatabase = pGeodatabase;
+    m_osFSName = pszNewName;
+    m_osPublicName = (pszPublicName) ? pszPublicName : pszNewName;
+    m_pGeodatabase = m_pConnection->GetGDB();
+    m_bUpdate = CPL_TO_BOOL(bUpdate);
+    m_poOpenFileGDBDrv = (GDALDriver*) GDALGetDriverByName("OpenFileGDB");
 
     std::vector<std::wstring> typesRequested;
 
@@ -86,6 +175,85 @@ int FGdbDataSource::Open(Geodatabase* pGeodatabase, const char * pszNewName, int
     return rv;
 }
 
+/************************************************************************/
+/*                               Close()                                */
+/************************************************************************/
+
+int FGdbDataSource::Close(int bCloseGeodatabase)
+{
+    size_t count = m_layers.size();
+    for(size_t i = 0; i < count; ++i )
+    {
+        m_layers[i]->CloseGDBObjects();
+    }
+
+    int bRet = FixIndexes();
+    if( m_pConnection && bCloseGeodatabase )
+        m_pConnection->CloseGeodatabase();
+    m_pGeodatabase = NULL;
+    return bRet;
+}
+
+/************************************************************************/
+/*                               ReOpen()                               */
+/************************************************************************/
+
+int FGdbDataSource::ReOpen()
+{
+    CPLAssert(m_pGeodatabase == NULL);
+
+    if( EQUAL(CPLGetConfigOption("FGDB_SIMUL_FAIL_REOPEN", ""), "CASE1") ||
+        !m_pConnection->OpenGeodatabase(m_osFSName) )
+    {
+        CPLError(CE_Failure, CPLE_AppDefined, "Cannot reopen %s",
+                 m_osFSName.c_str());
+        return FALSE;
+    }
+
+    FGdbDataSource* pDS = new FGdbDataSource(m_poDriver, m_pConnection);
+    if( EQUAL(CPLGetConfigOption("FGDB_SIMUL_FAIL_REOPEN", ""), "CASE2") ||
+        !pDS->Open(m_osPublicName, TRUE, m_osFSName) )
+    {
+        pDS->m_poDriver = NULL;
+        delete pDS;
+        CPLError(CE_Failure, CPLE_AppDefined, "Cannot reopen %s",
+                 m_osFSName.c_str());
+        return FALSE;
+    }
+    
+    int bRet = TRUE;
+    size_t count = m_layers.size();
+    for(size_t i = 0; i < count; ++i )
+    {
+        FGdbLayer* pNewLayer = (FGdbLayer*)pDS->GetLayerByName(m_layers[i]->GetName());
+        if( pNewLayer &&
+            !EQUAL(CPLGetConfigOption("FGDB_SIMUL_FAIL_REOPEN", ""), "CASE3") )
+        {
+            m_layers[i]->m_pTable = pNewLayer->m_pTable;
+            pNewLayer->m_pTable = NULL;
+            m_layers[i]->m_pEnumRows = pNewLayer->m_pEnumRows;
+            pNewLayer->m_pEnumRows = NULL;
+        }
+        else
+        {
+            CPLError(CE_Failure, CPLE_AppDefined, "Cannot reopen %s",
+                     m_layers[i]->GetName());
+            bRet = FALSE;
+        }
+        m_layers[i]->m_oMapOGRFIDToFGDBFID.clear();
+        m_layers[i]->m_oMapFGDBFIDToOGRFID.clear();
+    }
+    
+    m_pGeodatabase = pDS->m_pGeodatabase;
+    pDS->m_pGeodatabase = NULL;
+
+    pDS->m_poDriver = NULL;
+    pDS->m_pConnection = NULL;
+    delete pDS;
+    
+    return bRet;
+}
+
 /************************************************************************/
 /*                          OpenFGDBTables()                            */
 /************************************************************************/
@@ -101,10 +269,26 @@ bool FGdbDataSource::OpenFGDBTables(const std::wstring &type,
         if (FAILED(hr = m_pGeodatabase->OpenTable(layers[i], *pTable)))
         {
             delete pTable;
-            GDBDebug(hr, "Error opening " + WStringToString(layers[i]) + ". Skipping it");
+            
+            std::wstring fgdb_error_desc_w;
+            fgdbError er;
+            er = FileGDBAPI::ErrorInfo::GetErrorDescription(hr, fgdb_error_desc_w);
+            const char* pszLikelyReason = "Might be due to unsupported spatial reference system. Using OpenFileGDB driver or FileGDB SDK >= 1.4 should solve it";
+            if ( er == S_OK )
+            {
+                std::string fgdb_error_desc = WStringToString(fgdb_error_desc_w);
+                if( fgdb_error_desc == "FileGDB compression is not installed." )
+                {
+                    pszLikelyReason = "Using FileGDB SDK 1.4 or later should solve this issue.";
+                }
+            }
+
+            GDBErr(hr, "Error opening " + WStringToString(layers[i]),
+                   CE_Warning,
+                   (". Skipping it. " + CPLString(pszLikelyReason)).c_str());
             continue;
         }
-        FGdbLayer* pLayer = new FGdbLayer;
+        FGdbLayer* pLayer = new FGdbLayer();
         if (!pLayer->Initialize(this, pTable, layers[i], type))
         {
             delete pLayer;
@@ -172,7 +356,7 @@ bool FGdbDataSource::LoadLayers(const std::wstring &root)
 /* Old recursive LoadLayers. Removed in favor of simple one that only
    looks at FeatureClasses and Tables. */
 
-// Flattens out hierarchichal GDB structure
+// Flattens out hierarchical GDB structure.
 bool FGdbDataSource::LoadLayersOld(const std::vector<wstring> & datasetTypes,
                                 const wstring & parent)
 {
@@ -203,7 +387,7 @@ bool FGdbDataSource::LoadLayersOld(const std::vector<wstring> & datasetTypes,
 
                 // do something with it
                 // For now, we just ignore dataset containers and only open the children
-                //std::wcout << datasetTypes[dsTypeIndex] << L" " << childDatasets[childDatasetIndex] << std::endl;
+                // std::wcout << datasetTypes[dsTypeIndex] << L" " << childDatasets[childDatasetIndex] << std::endl;
 
                 if (!LoadLayersOld(datasetTypes, childDatasets[childDatasetIndex]))
                     errorsEncountered = true;
@@ -215,7 +399,7 @@ bool FGdbDataSource::LoadLayersOld(const std::vector<wstring> & datasetTypes,
 
     if ((!childrenFound) && parent != L"\\")
     {
-        //wcout << "Opening " << parent << "...";
+        // wcout << "Opening " << parent << "...";
         Table* pTable = new Table;
         if (FAILED(hr = m_pGeodatabase->OpenTable(parent,*pTable)))
         {
@@ -248,21 +432,26 @@ bool FGdbDataSource::LoadLayersOld(const std::vector<wstring> & datasetTypes,
 
 OGRErr FGdbDataSource::DeleteLayer( int iLayer )
 {
+    if( !m_bUpdate || m_pGeodatabase == NULL )
+        return OGRERR_FAILURE;
+
     if( iLayer < 0 || iLayer >= static_cast<int>(m_layers.size()) )
         return OGRERR_FAILURE;
+    
+    FGdbLayer* poBaseLayer = m_layers[iLayer];
 
     // Fetch FGDBAPI Table before deleting OGR layer object
 
-    Table* pTable = m_layers[iLayer]->GetTable();
+    //Table* pTable = poBaseLayer->GetTable();
 
-    std::string name = m_layers[iLayer]->GetLayerDefn()->GetName();
-    std::wstring strPath = m_layers[iLayer]->GetTablePath();
-    std::wstring strType = m_layers[iLayer]->GetType();
+    std::string name = poBaseLayer->GetLayerDefn()->GetName();
+    std::wstring strPath = poBaseLayer->GetTablePath();
+    std::wstring strType = poBaseLayer->GetType();
 
     // delete OGR layer
     delete m_layers[iLayer];
 
-    pTable = NULL; // OGR Layer had ownership of FGDB Table
+    //pTable = NULL; // OGR Layer had ownership of FGDB Table
 
     m_layers.erase(m_layers.begin() + iLayer);
 
@@ -286,11 +475,12 @@ OGRErr FGdbDataSource::DeleteLayer( int iLayer )
 int FGdbDataSource::TestCapability( const char * pszCap )
 {
     if( EQUAL(pszCap,ODsCCreateLayer) )
-        return TRUE;
+        return m_bUpdate;
 
     else if( EQUAL(pszCap,ODsCDeleteLayer) )
+        return m_bUpdate;
+    else if EQUAL(pszCap,ODsCCreateGeomFieldAfterCreateLayer)
         return TRUE;
-
     return FALSE;
 }
 
@@ -310,18 +500,21 @@ OGRLayer *FGdbDataSource::GetLayer( int iLayer )
 }
 
 /************************************************************************/
-/*                              CreateLayer()                           */
+/*                             ICreateLayer()                           */
 /*                                                                      */
 /* See FGdbLayer::Create for creation options                           */
 /************************************************************************/
 
 OGRLayer *
-FGdbDataSource::CreateLayer( const char * pszLayerName,
+FGdbDataSource::ICreateLayer( const char * pszLayerName,
                               OGRSpatialReference *poSRS,
                               OGRwkbGeometryType eType,
                               char ** papszOptions )
 {
-    FGdbLayer* pLayer = new FGdbLayer;
+    if( !m_bUpdate || m_pGeodatabase == NULL )
+        return NULL;
+
+    FGdbLayer* pLayer = new FGdbLayer();
     if (!pLayer->Create(this, pszLayerName, poSRS, eType, papszOptions))
     {
         delete pLayer;
@@ -361,15 +554,16 @@ class OGRFGdbSingleFeatureLayer : public OGRLayer
 /************************************************************************/
 
 OGRFGdbSingleFeatureLayer::OGRFGdbSingleFeatureLayer(const char* pszLayerName,
-                                                     const char *pszVal )
+                                                     const char *pszValIn )
 {
     poFeatureDefn = new OGRFeatureDefn( pszLayerName );
+    SetDescription( poFeatureDefn->GetName() );
     poFeatureDefn->Reference();
     OGRFieldDefn oField( "FIELD_1", OFTString );
     poFeatureDefn->AddFieldDefn( &oField );
 
     iNextShapeId = 0;
-    this->pszVal = pszVal ? CPLStrdup(pszVal) : NULL;
+    this->pszVal = pszValIn ? CPLStrdup(pszValIn) : NULL;
 }
 
 /************************************************************************/
@@ -409,7 +603,24 @@ OGRLayer * FGdbDataSource::ExecuteSQL( const char *pszSQLCommand,
                                        const char *pszDialect )
 
 {
-    if ( pszDialect != NULL && EQUAL(pszDialect, "OGRSQL") )
+    if( m_pConnection && m_pConnection->IsFIDHackInProgress() )
+    {
+        if( Close() )
+            ReOpen();
+    }
+    if( m_pGeodatabase == NULL )
+         return NULL;
+
+    size_t count = m_layers.size();
+    for(size_t i = 0; i < count; ++i )
+    {
+        m_layers[i]->EndBulkLoad();
+    }
+    
+/* -------------------------------------------------------------------- */
+/*      Use generic implementation for recognized dialects              */
+/* -------------------------------------------------------------------- */
+    if( IsGenericSQLDialect(pszDialect) )
         return OGRDataSource::ExecuteSQL( pszSQLCommand,
                                           poSpatialFilter,
                                           pszDialect );
@@ -417,7 +628,7 @@ OGRLayer * FGdbDataSource::ExecuteSQL( const char *pszSQLCommand,
 /* -------------------------------------------------------------------- */
 /*      Special case GetLayerDefinition                                 */
 /* -------------------------------------------------------------------- */
-    if (EQUALN(pszSQLCommand, "GetLayerDefinition ", strlen("GetLayerDefinition ")))
+    if (STARTS_WITH_CI(pszSQLCommand, "GetLayerDefinition "))
     {
         FGdbLayer* poLayer = (FGdbLayer*) GetLayerByName(pszSQLCommand + strlen("GetLayerDefinition "));
         if (poLayer)
@@ -435,7 +646,7 @@ OGRLayer * FGdbDataSource::ExecuteSQL( const char *pszSQLCommand,
 /* -------------------------------------------------------------------- */
 /*      Special case GetLayerMetadata                                   */
 /* -------------------------------------------------------------------- */
-    if (EQUALN(pszSQLCommand, "GetLayerMetadata ", strlen("GetLayerMetadata ")))
+    if (STARTS_WITH_CI(pszSQLCommand, "GetLayerMetadata "))
     {
         FGdbLayer* poLayer = (FGdbLayer*) GetLayerByName(pszSQLCommand + strlen("GetLayerMetadata "));
         if (poLayer)
@@ -452,16 +663,19 @@ OGRLayer * FGdbDataSource::ExecuteSQL( const char *pszSQLCommand,
 
     /* TODO: remove that workaround when the SDK has finally a decent */
     /* SQL support ! */
-    if( EQUALN(pszSQLCommand, "SELECT ", 7) && pszDialect == NULL )
+    if( STARTS_WITH_CI(pszSQLCommand, "SELECT ") && pszDialect == NULL )
     {
         CPLDebug("FGDB", "Support for SELECT is known to be partially "
                          "non-compliant with FileGDB SDK API v1.2.\n"
                          "So for now, we use default OGR SQL engine. "
-                         "Explicitely specify -dialect FileGDB\n"
+                         "Explicitly specify -dialect FileGDB\n"
                          "to use the SQL engine from the FileGDB SDK API");
-        return OGRDataSource::ExecuteSQL( pszSQLCommand,
+        OGRLayer* poLayer = OGRDataSource::ExecuteSQL( pszSQLCommand,
                                         poSpatialFilter,
                                         pszDialect );
+        if( poLayer )
+            m_oSetSelectLayers.insert(poLayer);
+        return poLayer;
     }
 
 /* -------------------------------------------------------------------- */
@@ -469,17 +683,32 @@ OGRLayer * FGdbDataSource::ExecuteSQL( const char *pszSQLCommand,
 /* -------------------------------------------------------------------- */
     EnumRows* pEnumRows = new EnumRows;
     long hr;
-    if (FAILED(hr = m_pGeodatabase->ExecuteSQL(
-                                StringToWString(pszSQLCommand), true, *pEnumRows)))
+    try
+    {
+        hr = m_pGeodatabase->ExecuteSQL(
+                                StringToWString(pszSQLCommand), true, *pEnumRows);
+    }
+    catch(...)
+    {
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "Exception occurred at executing '%s'. Application may "
+                  "become unstable", pszSQLCommand );
+        delete pEnumRows;
+        return NULL;
+    }
+
+    if (FAILED(hr))
     {
         GDBErr(hr, CPLSPrintf("Failed at executing '%s'", pszSQLCommand));
         delete pEnumRows;
         return NULL;
     }
 
-    if( EQUALN(pszSQLCommand, "SELECT ", 7) )
+    if( STARTS_WITH_CI(pszSQLCommand, "SELECT ") )
     {
-        return new FGdbResultLayer(this, pszSQLCommand, pEnumRows);
+        OGRLayer* poLayer = new FGdbResultLayer(this, pszSQLCommand, pEnumRows);
+        m_oSetSelectLayers.insert(poLayer);
+        return poLayer;
     }
     else
     {
@@ -494,5 +723,38 @@ OGRLayer * FGdbDataSource::ExecuteSQL( const char *pszSQLCommand,
 
 void FGdbDataSource::ReleaseResultSet( OGRLayer * poResultsSet )
 {
+    if( poResultsSet )
+        m_oSetSelectLayers.erase(poResultsSet);
     delete poResultsSet;
 }
+
+/************************************************************************/
+/*                      HasPerLayerCopyingForTransaction()              */
+/************************************************************************/
+
+int FGdbDataSource::HasPerLayerCopyingForTransaction()
+{
+    if( bPerLayerCopyingForTransaction >= 0 )
+        return bPerLayerCopyingForTransaction;
+#ifdef WIN32
+    bPerLayerCopyingForTransaction = FALSE;
+#else
+    bPerLayerCopyingForTransaction =
+        m_poOpenFileGDBDrv != NULL &&
+        CPLTestBool(CPLGetConfigOption("FGDB_PER_LAYER_COPYING_TRANSACTION", "TRUE"));
+#endif
+    return bPerLayerCopyingForTransaction;
+}
+
+/************************************************************************/
+/*                        SetSymlinkFlagOnAllLayers()                    */
+/************************************************************************/
+
+void FGdbDataSource::SetSymlinkFlagOnAllLayers()
+{
+    size_t count = m_layers.size();
+    for(size_t i = 0; i < count; ++i )
+    {
+        m_layers[i]->SetSymlinkFlag();
+    }
+}
