@@ -1,5 +1,5 @@
 /******************************************************************************
-* $Id: FGdbUtils.cpp 27044 2014-03-16 23:41:27Z rouault $
+* $Id: FGdbUtils.cpp 33563 2016-02-26 14:57:06Z rouault $
 *
 * Project:  OpenGIS Simple Features Reference Implementation
 * Purpose:  Different utility functions used in FileGDB OGR driver.
@@ -36,7 +36,7 @@
 #include "ogr_api.h"
 #include "ogrpgeogeometry.h"
 
-CPL_CVSID("$Id: FGdbUtils.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: FGdbUtils.cpp 33563 2016-02-26 14:57:06Z rouault $");
 
 using std::string;
 
@@ -68,21 +68,21 @@ std::string WStringToString(const std::wstring& utf16string)
 /*                                GDBErr()                               */
 /*************************************************************************/
 
-bool GDBErr(long int hr, std::string desc)
+bool GDBErr(long int hr, std::string desc, CPLErr errType, const char* pszAddMsg)
 {
     std::wstring fgdb_error_desc_w;
     fgdbError er;
-    er = FileGDBAPI::ErrorInfo::GetErrorDescription(hr, fgdb_error_desc_w);
+    er = FileGDBAPI::ErrorInfo::GetErrorDescription(static_cast<fgdbError>(hr), fgdb_error_desc_w);
     if ( er == S_OK )
     {
         std::string fgdb_error_desc = WStringToString(fgdb_error_desc_w);
-        CPLError( CE_Failure, CPLE_AppDefined,
-                  "Error: %s (%s)", desc.c_str(), fgdb_error_desc.c_str());
+        CPLError( errType, CPLE_AppDefined,
+                  "%s (%s)%s", desc.c_str(), fgdb_error_desc.c_str(), pszAddMsg);
     }
     else
     {
-        CPLError( CE_Failure, CPLE_AppDefined,
-                  "Error (%ld): %s", hr, desc.c_str());
+        CPLError( errType, CPLE_AppDefined,
+                  "Error (%ld): %s%s", hr, desc.c_str(), pszAddMsg);
     }
     // FIXME? EvenR: not sure if ClearErrors() is really necessary, but as it, it causes crashes in case of
     // repeated errors
@@ -99,7 +99,7 @@ bool GDBDebug(long int hr, std::string desc)
 {
     std::wstring fgdb_error_desc_w;
     fgdbError er;
-    er = FileGDBAPI::ErrorInfo::GetErrorDescription(hr, fgdb_error_desc_w);
+    er = FileGDBAPI::ErrorInfo::GetErrorDescription(static_cast<fgdbError>(hr), fgdb_error_desc_w);
     if ( er == S_OK )
     {
         std::string fgdb_error_desc = WStringToString(fgdb_error_desc_w);
@@ -120,28 +120,28 @@ bool GDBDebug(long int hr, std::string desc)
 /*                            GDBToOGRGeometry()                         */
 /*************************************************************************/
 
-bool GDBToOGRGeometry(string geoType, bool hasZ, OGRwkbGeometryType* pOut)
+bool GDBToOGRGeometry(string geoType, bool hasZ, bool hasM, OGRwkbGeometryType* pOut)
 {
     if (geoType == "esriGeometryPoint")
     {
-        *pOut = hasZ? wkbPoint25D : wkbPoint;
+        *pOut = wkbPoint;
     }
     else if (geoType == "esriGeometryMultipoint")
     {
-        *pOut = hasZ? wkbMultiPoint25D : wkbMultiPoint;
+        *pOut = wkbMultiPoint;
     }
     else if (geoType == "esriGeometryLine")
     {
-        *pOut = hasZ? wkbLineString25D : wkbLineString;
+        *pOut = wkbLineString;
     }
     else if (geoType == "esriGeometryPolyline")
     {
-        *pOut = hasZ? wkbMultiLineString25D : wkbMultiLineString;
+        *pOut = wkbMultiLineString;
     }
     else if (geoType == "esriGeometryPolygon" ||
             geoType == "esriGeometryMultiPatch")
     {
-        *pOut = hasZ? wkbMultiPolygon25D : wkbMultiPolygon; // no mapping to single polygon
+        *pOut = wkbMultiPolygon; // no mapping to single polygon
     }
     else
     {
@@ -149,6 +149,10 @@ bool GDBToOGRGeometry(string geoType, bool hasZ, OGRwkbGeometryType* pOut)
                 "Cannot map esriGeometryType(%s) to OGRwkbGeometryType", geoType.c_str());
         return false;
     }
+    if( hasZ )
+        *pOut = wkbSetZ(*pOut);
+    if( hasM )
+        *pOut = wkbSetM(*pOut);
 
     return true;
 }
@@ -157,53 +161,21 @@ bool GDBToOGRGeometry(string geoType, bool hasZ, OGRwkbGeometryType* pOut)
 /*                            OGRGeometryToGDB()                         */
 /*************************************************************************/
 
-bool OGRGeometryToGDB(OGRwkbGeometryType ogrType, std::string *gdbType, bool *hasZ)
+bool OGRGeometryToGDB(OGRwkbGeometryType ogrType, std::string *gdbType, bool *hasZ, bool *hasM)
 {
-    switch (ogrType)
+    *hasZ = wkbHasZ(ogrType);
+    *hasM = wkbHasM(ogrType);
+    switch (wkbFlatten(ogrType))
     {
-        /* 3D forms */
-        case wkbPoint25D:
-        {
-            *gdbType = "esriGeometryPoint";
-            *hasZ = true;
-            break;
-        }
-
-        case wkbMultiPoint25D:
-        {
-            *gdbType = "esriGeometryMultipoint";
-            *hasZ = true;
-            break;
-        }
-
-        case wkbLineString25D:
-        case wkbMultiLineString25D:
-        {
-            *gdbType = "esriGeometryPolyline";
-            *hasZ = true;
-            break;
-        }
-
-        case wkbPolygon25D:
-        case wkbMultiPolygon25D:
-        {
-            *gdbType = "esriGeometryPolygon";
-            *hasZ = true;
-            break;
-        }
-
-        /* 2D forms */
         case wkbPoint:
         {
             *gdbType = "esriGeometryPoint";
-            *hasZ = false;
             break;
         }
 
         case wkbMultiPoint:
         {
             *gdbType = "esriGeometryMultipoint";
-            *hasZ = false;
             break;
         }
 
@@ -211,7 +183,6 @@ bool OGRGeometryToGDB(OGRwkbGeometryType ogrType, std::string *gdbType, bool *ha
         case wkbMultiLineString:
         {
             *gdbType = "esriGeometryPolyline";
-            *hasZ = false;
             break;
         }
 
@@ -219,7 +190,6 @@ bool OGRGeometryToGDB(OGRwkbGeometryType ogrType, std::string *gdbType, bool *ha
         case wkbMultiPolygon:
         {
             *gdbType = "esriGeometryPolygon";
-            *hasZ = false;
             break;
         }
         
@@ -237,9 +207,10 @@ bool OGRGeometryToGDB(OGRwkbGeometryType ogrType, std::string *gdbType, bool *ha
 /*                            GDBToOGRFieldType()                        */
 /*************************************************************************/
 
-// We could make this function far more robust by doing automatic coertion of types,
-// and/or skipping fields we do not know. But our purposes this works fine
-bool GDBToOGRFieldType(std::string gdbType, OGRFieldType* pOut)
+// We could make this function far more robust by doing automatic coercion of
+// types, and/or skipping fields we do not know. But, for our purposes. this
+// works fine.
+bool GDBToOGRFieldType(std::string gdbType, OGRFieldType* pOut, OGRFieldSubType* pSubType)
 {
     /*
     ESRI types
@@ -274,14 +245,25 @@ bool GDBToOGRFieldType(std::string gdbType, OGRFieldType* pOut)
     /** Time *///                                   OFTTime = 10,               NO
     /** Date and Time *///                          OFTDateTime = 11            YES
 
-    if (gdbType == "esriFieldTypeSmallInteger" ||
-        gdbType == "esriFieldTypeInteger")
+    *pSubType = OFSTNone;
+    if (gdbType == "esriFieldTypeSmallInteger" )
+    {
+        *pSubType = OFSTInt16;
+        *pOut = OFTInteger;
+        return true;
+    }
+    else if (gdbType == "esriFieldTypeInteger")
     {
         *pOut = OFTInteger;
         return true;
     }
-    else if (gdbType == "esriFieldTypeSingle" ||
-        gdbType == "esriFieldTypeDouble")
+    else if (gdbType == "esriFieldTypeSingle" )
+    {
+        *pSubType = OFSTFloat32;
+        *pOut = OFTReal;
+        return true;
+    }
+    else if (gdbType == "esriFieldTypeDouble")
     {
         *pOut = OFTReal;
         return true;
@@ -321,18 +303,25 @@ bool GDBToOGRFieldType(std::string gdbType, OGRFieldType* pOut)
 /*                            OGRToGDBFieldType()                        */
 /*************************************************************************/
 
-bool OGRToGDBFieldType(OGRFieldType ogrType, std::string* gdbType)
+bool OGRToGDBFieldType(OGRFieldType ogrType, OGRFieldSubType eSubType, std::string* gdbType)
 {
     switch(ogrType)
     {
         case OFTInteger:
         {
-            *gdbType = "esriFieldTypeInteger";
+            if( eSubType == OFSTInt16 )
+                *gdbType = "esriFieldTypeSmallInteger";
+            else
+                *gdbType = "esriFieldTypeInteger";
             break;
         }
         case OFTReal:
+        case OFTInteger64:
         {
-            *gdbType = "esriFieldTypeDouble";
+             if( eSubType == OFSTFloat32 )
+                *gdbType = "esriFieldTypeSingle";
+            else
+                *gdbType = "esriFieldTypeDouble";
             break;
         }
         case OFTString:
@@ -412,6 +401,10 @@ bool GDBFieldTypeToWidthPrecision(std::string &gdbType, int *width, int *precisi
     {
         *width = 0;
     }
+    else if(gdbType == "esriFieldTypeGlobalID" )
+    {
+        *width = 38;
+    }
     else
     {
         CPLError( CE_Warning, CPLE_AppDefined,
@@ -434,7 +427,7 @@ bool GDBGeometryToOGRGeometry(bool forceMulti, FileGDBAPI::ShapeBuffer* pGdbGeom
 
     OGRErr eErr = OGRCreateFromShapeBin( pGdbGeometry->shapeBuffer,
                                 &pOGRGeometry,
-                                pGdbGeometry->inUseLength);
+                                static_cast<int>(pGdbGeometry->inUseLength));
 
     //OGRErr eErr = OGRGeometryFactory::createFromWkb(pGdbGeometry->shapeBuffer, pOGRSR, &pOGRGeometry, pGdbGeometry->inUseLength );
 
@@ -501,7 +494,7 @@ bool GDBToOGRSpatialReference(const string & wkt, OGRSpatialReference** ppSR)
         *ppSR = NULL;
 
         CPLError( CE_Failure, CPLE_AppDefined,
-                  "Failed morhping from ESRI Geometry: %s", wkt.c_str());
+                  "Failed morphing from ESRI Geometry: %s", wkt.c_str());
 
         return false;
     }
@@ -554,7 +547,7 @@ std::string FGDBEscapeUnsupportedPrefixes(const std::string className)
     std::string newName = className;
     // From ESRI docs
     // Feature classes starting with these strings are unsupported.
-    static const char* UNSUPPORTED_PREFIXES[] = {"sde_", "gdb_", "delta_", NULL};
+    static const char* const UNSUPPORTED_PREFIXES[] = {"sde_", "gdb_", "delta_", NULL};
 
     for (int i = 0; UNSUPPORTED_PREFIXES[i] != NULL; i++)
     {
@@ -579,7 +572,7 @@ std::string FGDBEscapeReservedKeywords(const std::string name)
     std::transform(upperName.begin(), upperName.end(), upperName.begin(), ::toupper);
 
     // From ESRI docs
-    static const char* RESERVED_WORDS[] = {FGDB_OID_NAME, "ADD", "ALTER", "AND", "AS", "ASC", "BETWEEN",
+    static const char* const RESERVED_WORDS[] = {FGDB_OID_NAME, "ADD", "ALTER", "AND", "AS", "ASC", "BETWEEN",
                                     "BY", "COLUMN", "CREATE", "DATE", "DELETE", "DESC",
                                     "DROP", "EXISTS", "FOR", "FROM", "IN", "INSERT", "INTO",
                                     "IS", "LIKE", "NOT", "NULL", "OR", "ORDER", "SELECT",
