@@ -1,11 +1,11 @@
 /******************************************************************************
- * $Id: ogrfmelayerdb.cpp 12123 2007-09-11 23:57:40Z warmerdam $
+ * $Id: ogrfmelayerdb.cpp 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  FMEObjects Translator
  * Purpose:  Implementation of the OGRFMELayerDB class.  This is the
  *           class implementing behaviour for layers that are built on
- *           smart readers representing databases with spatial constraints, 
- *           and where clause support. 
+ *           smart readers representing databases with spatial constraints,
+ *           and where clause support.
  * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
  ******************************************************************************
@@ -17,16 +17,16 @@
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
@@ -34,14 +34,14 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrfmelayerdb.cpp 12123 2007-09-11 23:57:40Z warmerdam $");
+CPL_CVSID("$Id: ogrfmelayerdb.cpp 33714 2016-03-13 05:42:13Z goatbar $");
 
 /************************************************************************/
 /*                           OGRFMELayerDB()                            */
 /************************************************************************/
 
-OGRFMELayerDB::OGRFMELayerDB( OGRFMEDataSource *poDSIn, 
-                              const char *pszReaderNameIn, 
+OGRFMELayerDB::OGRFMELayerDB( OGRFMEDataSource *poDSIn,
+                              const char *pszReaderNameIn,
                               const char *pszDatasetIn,
                               IFMEStringArray *poUserDirectivesIn )
         : OGRFMELayer( poDSIn )
@@ -58,7 +58,7 @@ OGRFMELayerDB::OGRFMELayerDB( OGRFMEDataSource *poDSIn,
 
     for( FME_UInt32 i = 0; i < poUserDirectivesIn->entries(); i++ )
     {
-        CPLDebug( "FMEOLEDB", "userDirective[%d] = %s\n", 
+        CPLDebug( "FMEOLEDB", "userDirective[%d] = %s\n",
                   i, (const char *) (*poUserDirectivesIn)(i) );
 
         poUserDirectives->append( (*poUserDirectivesIn)(i) );
@@ -89,7 +89,7 @@ int OGRFMELayerDB::TestCapability( const char * pszCap )
     if( EQUAL(pszCap,OLCRandomRead) )
         return FALSE;
 
-    else if( EQUAL(pszCap,OLCSequentialWrite) 
+    else if( EQUAL(pszCap,OLCSequentialWrite)
              || EQUAL(pszCap,OLCRandomWrite) )
         return FALSE;
 
@@ -99,7 +99,7 @@ int OGRFMELayerDB::TestCapability( const char * pszCap )
     else if( EQUAL(pszCap,OLCFastSpatialFilter) )
         return TRUE;
 
-    else 
+    else
         return FALSE;
 }
 
@@ -143,7 +143,7 @@ OGRFeature *OGRFMELayerDB::GetNextFeature()
     poFeature = poDS->ProcessFeature( this, poFMEFeature );
 
     if( nPreviousFeature == -1 )
-        CPLDebug( "FMEOLEDB", "Fetching first feature from layer `%s'.", 
+        CPLDebug( "FMEOLEDB", "Fetching first feature from layer `%s'.",
                   GetLayerDefn()->GetName() );
 
     poFeature->SetFID( ++nPreviousFeature );
@@ -157,7 +157,7 @@ OGRFeature *OGRFMELayerDB::GetNextFeature()
 /************************************************************************/
 /*                            ResetReading()                            */
 /************************************************************************/
- 
+
 void OGRFMELayerDB::ResetReading()
 
 {
@@ -179,30 +179,30 @@ void OGRFMELayerDB::ResetReading()
 /*                              SetMacro()                              */
 /*                                                                      */
 /*      Set the value of one macro within a set of macros stored in     */
-/*      comma delimeted name value pairs (as per RUNTIME_MACROS in      */
+/*      comma delimited name value pairs (as per RUNTIME_MACROS in      */
 /*      user directives).                                               */
 /************************************************************************/
 
-static void SetMacro( IFMEString *poMacros, const char *pszTarget, 
+static void SetMacro( IFMEString *poMacros, const char *pszTarget,
                       const char *pszNewValue )
 
 {
     char      *pszWorking, *pszValStart;
     int       nOldValLength;
 
-    pszWorking = (char *) CPLMalloc(strlen(poMacros->data()) 
-                                    + strlen(pszNewValue) 
+    pszWorking = (char *) CPLMalloc(strlen(poMacros->data())
+                                    + strlen(pszNewValue)
                                     + strlen(pszTarget) + 20 );
     strcpy( pszWorking, poMacros->data() );
 
     pszValStart = strstr( pszWorking, pszTarget );
-    if( pszValStart == NULL 
+    if( pszValStart == NULL
         || pszValStart[strlen(pszTarget)] != ',' )
     {
         if( strlen(pszWorking) > 0 )
             strcat( pszWorking, "," );
 
-        sprintf( pszWorking + strlen(pszWorking), "%s,%s", 
+        sprintf( pszWorking + strlen(pszWorking), "%s,%s",
                  pszTarget, pszNewValue );
         *poMacros = pszWorking;
         CPLFree( pszWorking );
@@ -211,13 +211,13 @@ static void SetMacro( IFMEString *poMacros, const char *pszTarget,
 
     pszValStart += strlen(pszTarget) + 1;
 
-    for( nOldValLength = 0; 
-         pszValStart[nOldValLength] != ',' 
-             && pszValStart[nOldValLength] != '\0'; 
+    for( nOldValLength = 0;
+         pszValStart[nOldValLength] != ','
+             && pszValStart[nOldValLength] != '\0';
          nOldValLength++ ) {}
 
-    memmove( pszValStart + strlen(pszNewValue), 
-            pszValStart + nOldValLength, 
+    memmove( pszValStart + strlen(pszNewValue),
+            pszValStart + nOldValLength,
             strlen(pszValStart + nOldValLength)+1 );
 
     memcpy( pszValStart, pszNewValue, strlen( pszNewValue ) );
@@ -265,11 +265,11 @@ int OGRFMELayerDB::CreateReader()
 
     for( i = 0; i < poUserDirectives->entries(); i++ )
         poUDC->append( (*poUserDirectives)(i) );
-    
+
 /* -------------------------------------------------------------------- */
 /*      Update the IDLIST to just select the desired table.             */
 /* -------------------------------------------------------------------- */
-    
+
     for( i = 0; i < poUDC->entries(); i++ )
     {
         if( EQUAL((const char *) (*poUDC)(i),"IDLIST") )
@@ -310,18 +310,18 @@ int OGRFMELayerDB::CreateReader()
                 poUDC->getElement( i+1, *poMacroValue );
 
                 m_poFilterGeom->getEnvelope( &oEnvelope );
-                
-                if( EQUALN(pszReaderName,"SDE",3) )
+
+                if( STARTS_WITH_CI(pszReaderName, "SDE") )
                 {
                     sprintf( szSEARCH_ENVELOPE, "%.16f", oEnvelope.MinX );
                     SetMacro( poMacroValue, "_SDE3MINX", szSEARCH_ENVELOPE );
-                    
+
                     sprintf( szSEARCH_ENVELOPE, "%.16f", oEnvelope.MinY );
                     SetMacro( poMacroValue, "_SDE3MINY", szSEARCH_ENVELOPE );
-                    
+
                     sprintf( szSEARCH_ENVELOPE, "%.16f", oEnvelope.MaxX );
                     SetMacro( poMacroValue, "_SDE3MAXX", szSEARCH_ENVELOPE );
-                    
+
                     sprintf( szSEARCH_ENVELOPE, "%.16f", oEnvelope.MaxY );
                     SetMacro( poMacroValue, "_SDE3MAXY", szSEARCH_ENVELOPE );
                 }
@@ -329,20 +329,20 @@ int OGRFMELayerDB::CreateReader()
                 {
                     sprintf( szSEARCH_ENVELOPE, "%.16f", oEnvelope.MinX );
                     SetMacro( poMacroValue, "_ORACLE_MINX", szSEARCH_ENVELOPE);
-                    
+
                     sprintf( szSEARCH_ENVELOPE, "%.16f", oEnvelope.MinY );
                     SetMacro( poMacroValue, "_ORACLE_MINY", szSEARCH_ENVELOPE);
-                    
+
                     sprintf( szSEARCH_ENVELOPE, "%.16f", oEnvelope.MaxX );
                     SetMacro( poMacroValue, "_ORACLE_MAXX", szSEARCH_ENVELOPE);
-                    
+
                     sprintf( szSEARCH_ENVELOPE, "%.16f", oEnvelope.MaxY );
                     SetMacro( poMacroValue, "_ORACLE_MAXY", szSEARCH_ENVELOPE);
                 }
 
                 poUDC->setElement( i+1, *poMacroValue );
 
-                CPLDebug( "FMEOLEDB", "Update %s to:\n%s", 
+                CPLDebug( "FMEOLEDB", "Update %s to:\n%s",
                           pszDirective, poMacroValue->data() );
 
                 poSession->destroyString( poMacroValue );
@@ -350,7 +350,7 @@ int OGRFMELayerDB::CreateReader()
             }
         }
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Create new reader with desired constraints.                     */
 /* -------------------------------------------------------------------- */
@@ -371,7 +371,7 @@ int OGRFMELayerDB::CreateReader()
 
     if( pszAttributeFilter != NULL && strlen(pszAttributeFilter) > 0 )
     {
-        if( EQUALN(pszReaderName,"SDE",3) )
+        if( STARTS_WITH_CI(pszReaderName, "SDE") )
             poParms->append( "WHERE" );
         else
             poParms->append( "WHERE_CLAUSE" );
@@ -406,7 +406,7 @@ int OGRFMELayerDB::CreateReader()
 
     for( i = 0; i < poParms->entries(); i++ )
     {
-        CPLDebug( "FMEOLEDB", "openParms[%d] = %s", 
+        CPLDebug( "FMEOLEDB", "openParms[%d] = %s",
                   i, (const char *) (*poParms)(i) );
     }
 
@@ -431,7 +431,7 @@ int OGRFMELayerDB::CreateReader()
 /*                          GetFeatureCount()                           */
 /************************************************************************/
 
-int OGRFMELayerDB::GetFeatureCount( int bForce )
+GIntBig OGRFMELayerDB::GetFeatureCount( int bForce )
 
 {
     /*
