@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_gensql.cpp 31016 2015-10-16 08:11:46Z rouault $
+ * $Id: ogr_gensql.cpp 34092 2016-04-25 09:09:46Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRGenSQLResultsLayer.
@@ -36,7 +36,7 @@
 #include "cpl_time.h"
 #include <vector>
 
-CPL_CVSID("$Id: ogr_gensql.cpp 31016 2015-10-16 08:11:46Z rouault $");
+CPL_CVSID("$Id: ogr_gensql.cpp 34092 2016-04-25 09:09:46Z rouault $");
 
 
 class OGRGenSQLGeomFieldDefn: public OGRGeomFieldDefn
@@ -84,36 +84,28 @@ int OGRGenSQLResultsLayerHasSpecialField(swq_expr_node* expr,
 /*                       OGRGenSQLResultsLayer()                        */
 /************************************************************************/
 
-OGRGenSQLResultsLayer::OGRGenSQLResultsLayer( GDALDataset *poSrcDS,
-                                              void *pSelectInfo, 
+OGRGenSQLResultsLayer::OGRGenSQLResultsLayer( GDALDataset *poSrcDSIn,
+                                              void *pSelectInfoIn,
                                               OGRGeometry *poSpatFilter,
-                                              const char *pszWHERE,
-                                              const char *pszDialect )
-
+                                              const char *pszWHEREIn,
+                                              const char *pszDialect ) :
+    poSrcLayer(NULL), pszWHERE(NULL), papoTableLayers(NULL), poDefn(NULL),
+    panGeomFieldToSrcGeomField(NULL), nIndexSize(0),
+    panFIDIndex(NULL), bOrderByValid(FALSE), nNextIndexFID(0),
+    poSummaryFeature(NULL), iFIDFieldIndex(), nExtraDSCount(0), papoExtraDS(NULL)
 {
-    swq_select *psSelectInfo = (swq_select *) pSelectInfo;
+    swq_select *psSelectInfo = (swq_select *) pSelectInfoIn;
 
-    this->poSrcDS = poSrcDS;
-    this->pSelectInfo = pSelectInfo;
-    poDefn = NULL;
-    poSummaryFeature = NULL;
-    panFIDIndex = NULL;
-    bOrderByValid = FALSE;
-    nIndexSize = 0;
-    nNextIndexFID = 0;
-    nExtraDSCount = 0;
-    papoExtraDS = NULL;
-    panGeomFieldToSrcGeomField = NULL;
+    poSrcDS = poSrcDSIn;
+    pSelectInfo = pSelectInfoIn;
 
 /* -------------------------------------------------------------------- */
 /*      Identify all the layers involved in the SELECT.                 */
 /* -------------------------------------------------------------------- */
-    int iTable;
-
-    papoTableLayers = (OGRLayer **) 
+    papoTableLayers = (OGRLayer **)
         CPLCalloc( sizeof(OGRLayer *), psSelectInfo->table_count );
 
-    for( iTable = 0; iTable < psSelectInfo->table_count; iTable++ )
+    for( int iTable = 0; iTable < psSelectInfo->table_count; iTable++ )
     {
         swq_table_def *psTableDef = psSelectInfo->table_defs + iTable;
         GDALDataset *poTableDS = poSrcDS;
@@ -125,7 +117,7 @@ OGRGenSQLResultsLayer::OGRGenSQLResultsLayer( GDALDataset *poSrcDS,
             if( poTableDS == NULL )
             {
                 if( strlen(CPLGetLastErrorMsg()) == 0 )
-                    CPLError( CE_Failure, CPLE_AppDefined, 
+                    CPLError( CE_Failure, CPLE_AppDefined,
                               "Unable to open secondary datasource\n"
                               "`%s' required by JOIN.",
                               psTableDef->data_source );
@@ -138,16 +130,17 @@ OGRGenSQLResultsLayer::OGRGenSQLResultsLayer( GDALDataset *poSrcDS,
             papoExtraDS[nExtraDSCount-1] = poTableDS;
         }
 
-        papoTableLayers[iTable] = 
+        papoTableLayers[iTable] =
             poTableDS->GetLayerByName( psTableDef->table_name );
-        
+
         CPLAssert( papoTableLayers[iTable] != NULL );
 
         if( papoTableLayers[iTable] == NULL )
             return;
     }
-    
+
     poSrcLayer = papoTableLayers[0];
+    SetMetadata( poSrcLayer->GetMetadata( "NATIVE_DATA" ), "NATIVE_DATA" );
 
 /* -------------------------------------------------------------------- */
 /*      If the user has explicitly requested a OGRSQL dialect, then    */
@@ -155,7 +148,7 @@ OGRGenSQLResultsLayer::OGRGenSQLResultsLayer( GDALDataset *poSrcDS,
 /*      when there is a risk it cannot understand it (#4022)            */
 /* -------------------------------------------------------------------- */
     int bForwardWhereToSourceLayer = TRUE;
-    if( pszWHERE )
+    if( pszWHEREIn )
     {
         if( psSelectInfo->where_expr && pszDialect != NULL &&
             EQUAL(pszDialect, "OGRSQL") )
@@ -165,12 +158,12 @@ OGRGenSQLResultsLayer::OGRGenSQLResultsLayer( GDALDataset *poSrcDS,
                             (psSelectInfo->where_expr, nMinIndexForSpecialField);
         }
         if (bForwardWhereToSourceLayer)
-            this->pszWHERE = CPLStrdup(pszWHERE);
+            pszWHERE = CPLStrdup(pszWHEREIn);
         else
-            this->pszWHERE = NULL;
+            pszWHERE = NULL;
     }
     else
-        this->pszWHERE = NULL;
+        pszWHERE = NULL;
 
 /* -------------------------------------------------------------------- */
 /*      Prepare a feature definition based on the query.                */
@@ -199,10 +192,10 @@ OGRGenSQLResultsLayer::OGRGenSQLResultsLayer( GDALDataset *poSrcDS,
         int iSrcGeomField = -1;
 
         if( psColDef->table_index != -1 )
-            poLayerDefn = 
+            poLayerDefn =
                 papoTableLayers[psColDef->table_index]->GetLayerDefn();
 
-        if( psColDef->field_index > -1 
+        if( psColDef->field_index > -1
             && poLayerDefn != NULL
             && psColDef->field_index < poLayerDefn->GetFieldCount() )
         {
@@ -228,7 +221,7 @@ OGRGenSQLResultsLayer::OGRGenSQLResultsLayer( GDALDataset *poSrcDS,
         {
             CPLFree( psColDef->field_name );
             psColDef->field_name = (char *) CPLMalloc(40);
-            sprintf( psColDef->field_name, "FIELD_%d", poDefn->GetFieldCount()+1 );
+            snprintf( psColDef->field_name, 40, "FIELD_%d", poDefn->GetFieldCount()+1 );
         }
 
         if( psColDef->field_alias != NULL )
@@ -240,7 +233,7 @@ OGRGenSQLResultsLayer::OGRGenSQLResultsLayer( GDALDataset *poSrcDS,
         }
         else if( psColDef->col_func != SWQCF_NONE )
         {
-            const swq_operation *op = swq_op_registrar::GetOperator( 
+            const swq_operation *op = swq_op_registrar::GetOperator(
                 (swq_op) psColDef->col_func );
 
             oFDefn.SetName( CPLSPrintf( "%s_%s",
@@ -311,6 +304,12 @@ OGRGenSQLResultsLayer::OGRGenSQLResultsLayer( GDALDataset *poSrcDS,
                 oFDefn.SetType( OFTString );
                 break;
             }
+            if( psColDef->field_index-iFIDFieldIndex == SPF_FID &&
+                poSrcLayer->GetMetadataItem(OLMD_FID64) != NULL &&
+                EQUAL(poSrcLayer->GetMetadataItem(OLMD_FID64), "YES") )
+            {
+                oFDefn.SetType( OFTInteger64 );
+            }
         }
         else
         {
@@ -435,7 +434,7 @@ OGRGenSQLResultsLayer::OGRGenSQLResultsLayer( GDALDataset *poSrcDS,
     {
         psSelectInfo->result_columns++;
 
-        psSelectInfo->column_defs = (swq_col_def *) 
+        psSelectInfo->column_defs = (swq_col_def *)
             CPLRealloc( psSelectInfo->column_defs, sizeof(swq_col_def) * psSelectInfo->result_columns );
 
         swq_col_def *col_def = psSelectInfo->column_defs + psSelectInfo->result_columns - 1;
@@ -478,7 +477,7 @@ OGRGenSQLResultsLayer::OGRGenSQLResultsLayer( GDALDataset *poSrcDS,
     FindAndSetIgnoredFields();
 
     if( !bForwardWhereToSourceLayer )
-        SetAttributeFilter( pszWHERE );
+        SetAttributeFilter( pszWHEREIn );
 }
 
 /************************************************************************/
@@ -491,7 +490,7 @@ OGRGenSQLResultsLayer::~OGRGenSQLResultsLayer()
     if( m_nFeaturesRead > 0 && poDefn != NULL )
     {
         CPLDebug( "GenSQL", "%d features read on layer '%s'.",
-                  (int) m_nFeaturesRead, 
+                  (int) m_nFeaturesRead,
                   poDefn->GetName() );
     }
 
@@ -502,7 +501,7 @@ OGRGenSQLResultsLayer::~OGRGenSQLResultsLayer()
 /* -------------------------------------------------------------------- */
     CPLFree( papoTableLayers );
     papoTableLayers = NULL;
-             
+
     CPLFree( panFIDIndex );
     CPLFree( panGeomFieldToSrcGeomField );
 
@@ -555,9 +554,9 @@ void OGRGenSQLResultsLayer::ClearFilters()
         for( iJoin = 0; iJoin < psSelectInfo->join_count; iJoin++ )
         {
             swq_join_def *psJoinInfo = psSelectInfo->join_defs + iJoin;
-            OGRLayer *poJoinLayer = 
+            OGRLayer *poJoinLayer =
                 papoTableLayers[psJoinInfo->secondary_table];
-            
+
             poJoinLayer->SetAttributeFilter( "" );
         }
     }
@@ -615,7 +614,7 @@ void OGRGenSQLResultsLayer::ApplyFiltersToSource()
 /*                            ResetReading()                            */
 /************************************************************************/
 
-void OGRGenSQLResultsLayer::ResetReading() 
+void OGRGenSQLResultsLayer::ResetReading()
 
 {
     swq_select *psSelectInfo = (swq_select *) pSelectInfo;
@@ -631,7 +630,7 @@ void OGRGenSQLResultsLayer::ResetReading()
 /************************************************************************/
 /*                           SetNextByIndex()                           */
 /*                                                                      */
-/*      If we already have an FID list, we can easily resposition       */
+/*      If we already have an FID list, we can easily reposition        */
 /*      ourselves in it.                                                */
 /************************************************************************/
 
@@ -642,8 +641,8 @@ OGRErr OGRGenSQLResultsLayer::SetNextByIndex( GIntBig nIndex )
 
     CreateOrderByIndex();
 
-    if( psSelectInfo->query_mode == SWQM_SUMMARY_RECORD 
-        || psSelectInfo->query_mode == SWQM_DISTINCT_LIST 
+    if( psSelectInfo->query_mode == SWQM_SUMMARY_RECORD
+        || psSelectInfo->query_mode == SWQM_DISTINCT_LIST
         || panFIDIndex != NULL )
     {
         nNextIndexFID = nIndex;
@@ -660,7 +659,7 @@ OGRErr OGRGenSQLResultsLayer::SetNextByIndex( GIntBig nIndex )
 /************************************************************************/
 
 OGRErr OGRGenSQLResultsLayer::GetExtent( int iGeomField,
-                                         OGREnvelope *psExtent, 
+                                         OGREnvelope *psExtent,
                                          int bForce )
 
 {
@@ -733,17 +732,17 @@ int OGRGenSQLResultsLayer::TestCapability( const char *pszCap )
 
     if( EQUAL(pszCap,OLCFastSetNextByIndex) )
     {
-        if( psSelectInfo->query_mode == SWQM_SUMMARY_RECORD 
-            || psSelectInfo->query_mode == SWQM_DISTINCT_LIST 
+        if( psSelectInfo->query_mode == SWQM_SUMMARY_RECORD
+            || psSelectInfo->query_mode == SWQM_DISTINCT_LIST
             || panFIDIndex != NULL )
             return TRUE;
-        else 
+        else
             return poSrcLayer->TestCapability( pszCap );
     }
 
     if( psSelectInfo->query_mode == SWQM_RECORDSET
-        && (EQUAL(pszCap,OLCFastFeatureCount) 
-            || EQUAL(pszCap,OLCRandomRead) 
+        && (EQUAL(pszCap,OLCFastFeatureCount)
+            || EQUAL(pszCap,OLCRandomRead)
             || EQUAL(pszCap,OLCFastGetExtent)) )
         return poSrcLayer->TestCapability( pszCap );
 
@@ -857,14 +856,14 @@ int OGRGenSQLResultsLayer::PrepareSummary()
 /*      GetFeatureCount().                                            */
 /* -------------------------------------------------------------------- */
 
-    if( psSelectInfo->result_columns == 1 
+    if( psSelectInfo->result_columns == 1
         && psSelectInfo->column_defs[0].col_func == SWQCF_COUNT
         && psSelectInfo->column_defs[0].field_index < 0 )
     {
         GIntBig nRes = poSrcLayer->GetFeatureCount( TRUE );
         poSummaryFeature->SetField( 0, nRes );
 
-        if( (GIntBig)(int)nRes == nRes )
+        if( CPL_INT64_FITS_ON_INT32(nRes) )
         {
             poDefn->GetFieldDefn(0)->SetType(OFTInteger);
             delete poSummaryFeature;
@@ -920,7 +919,7 @@ int OGRGenSQLResultsLayer::PrepareSummary()
                                                 psColDef->field_index );
                 pszError = swq_select_summarize( psSelectInfo, iField, pszVal );
             }
-            
+
             if( pszError != NULL )
             {
                 delete poSrcFeature;
@@ -944,7 +943,7 @@ int OGRGenSQLResultsLayer::PrepareSummary()
     {
         delete poSummaryFeature;
         poSummaryFeature = NULL;
-        
+
         CPLError( CE_Failure, CPLE_AppDefined, "%s", pszError );
         return FALSE;
     }
@@ -971,7 +970,7 @@ int OGRGenSQLResultsLayer::PrepareSummary()
                 swq_summary *psSummary = psSelectInfo->column_summary + iField;
                 if( psColDef->col_func == SWQCF_COUNT )
                 {
-                    if( (GIntBig)(int)psSummary->count == psSummary->count ) 
+                    if( CPL_INT64_FITS_ON_INT32(psSummary->count) )
                     {
                         delete poSummaryFeature;
                         poSummaryFeature = NULL;
@@ -1009,7 +1008,8 @@ int OGRGenSQLResultsLayer::PrepareSummary()
                                                     brokendowntime.tm_mday,
                                                     brokendowntime.tm_hour,
                                                     brokendowntime.tm_min,
-                                                    brokendowntime.tm_sec + fmod(dfAvg, 1), 0);
+                                                    static_cast<float>(brokendowntime.tm_sec + fmod(dfAvg, 1)),
+                                                    0);
                     }
                     else
                         poSummaryFeature->SetField( iField,
@@ -1050,11 +1050,11 @@ int OGRGenSQLResultsLayer::PrepareSummary()
 /*                       OGRMultiFeatureFetcher()                       */
 /************************************************************************/
 
-static swq_expr_node *OGRMultiFeatureFetcher( swq_expr_node *op, 
+static swq_expr_node *OGRMultiFeatureFetcher( swq_expr_node *op,
                                               void *pFeatureList )
 
 {
-    std::vector<OGRFeature*> *papoFeatures = 
+    std::vector<OGRFeature*> *papoFeatures =
         (std::vector<OGRFeature*> *) pFeatureList;
     OGRFeature *poFeature;
     swq_expr_node *poRetNode = NULL;
@@ -1071,7 +1071,7 @@ static swq_expr_node *OGRMultiFeatureFetcher( swq_expr_node *op,
                   op->table_index );
         return NULL;
     }
-    
+
     poFeature = (*papoFeatures)[op->table_index];
 
 /* -------------------------------------------------------------------- */
@@ -1081,41 +1081,41 @@ static swq_expr_node *OGRMultiFeatureFetcher( swq_expr_node *op,
     {
       case SWQ_INTEGER:
       case SWQ_BOOLEAN:
-        if( poFeature == NULL 
+        if( poFeature == NULL
             || !poFeature->IsFieldSet(op->field_index) )
         {
             poRetNode = new swq_expr_node(0);
             poRetNode->is_null = TRUE;
         }
         else
-            poRetNode = new swq_expr_node( 
+            poRetNode = new swq_expr_node(
                 poFeature->GetFieldAsInteger(op->field_index) );
         break;
 
       case SWQ_INTEGER64:
-        if( poFeature == NULL 
+        if( poFeature == NULL
             || !poFeature->IsFieldSet(op->field_index) )
         {
             poRetNode = new swq_expr_node((GIntBig)0);
             poRetNode->is_null = TRUE;
         }
         else
-            poRetNode = new swq_expr_node( 
+            poRetNode = new swq_expr_node(
                 poFeature->GetFieldAsInteger64(op->field_index) );
         break;
 
       case SWQ_FLOAT:
-        if( poFeature == NULL 
+        if( poFeature == NULL
             || !poFeature->IsFieldSet(op->field_index) )
         {
             poRetNode = new swq_expr_node( 0.0 );
             poRetNode->is_null = TRUE;
         }
         else
-            poRetNode = new swq_expr_node( 
+            poRetNode = new swq_expr_node(
                 poFeature->GetFieldAsDouble(op->field_index) );
         break;
-        
+
       case SWQ_GEOMETRY:
         if( poFeature == NULL )
         {
@@ -1131,14 +1131,14 @@ static swq_expr_node *OGRMultiFeatureFetcher( swq_expr_node *op,
         break;
 
       default:
-        if( poFeature == NULL 
+        if( poFeature == NULL
             || !poFeature->IsFieldSet(op->field_index) )
         {
             poRetNode = new swq_expr_node("");
             poRetNode->is_null = TRUE;
         }
         else
-            poRetNode = new swq_expr_node( 
+            poRetNode = new swq_expr_node(
                 poFeature->GetFieldAsString(op->field_index) );
         break;
     }
@@ -1175,7 +1175,7 @@ static CPLString GetFilterForJoin(swq_expr_node* poExpr, OGRFeature* poSrcFeat,
             }
             OGRFieldType ePrimaryFieldType =
                     poSrcFeat->GetFieldDefnRef(poExpr->field_index)->GetType();
-            OGRField *psSrcField = 
+            OGRField *psSrcField =
                     poSrcFeat->GetRawFieldRef(poExpr->field_index);
 
             switch( ePrimaryFieldType )
@@ -1194,8 +1194,8 @@ static CPLString GetFilterForJoin(swq_expr_node* poExpr, OGRFeature* poSrcFeat,
 
             case OFTString:
             {
-                char *pszEscaped = CPLEscapeString( psSrcField->String, 
-                                                    strlen(psSrcField->String),
+                char *pszEscaped = CPLEscapeString( psSrcField->String,
+                                                    static_cast<int>(strlen(psSrcField->String)),
                                                     CPLES_SQL );
                 CPLString osRes = "'";
                 osRes += pszEscaped;
@@ -1210,7 +1210,7 @@ static CPLString GetFilterForJoin(swq_expr_node* poExpr, OGRFeature* poSrcFeat,
                 return "";
             }
         }
-        
+
         if(  poExpr->table_index == secondary_table )
         {
             OGRFieldDefn* poSecondaryFieldDefn =
@@ -1224,13 +1224,11 @@ static CPLString GetFilterForJoin(swq_expr_node* poExpr, OGRFeature* poSrcFeat,
 
     if( poExpr->eNodeType == SNT_OPERATION )
     {
-        /* -------------------------------------------------------------------- */
-        /*      Operation - start by unparsing all the subexpressions.          */
-        /* -------------------------------------------------------------------- */
+        /* ----------------------------------------------------------------- */
+        /*      Operation - start by unparsing all the subexpressions.       */
+        /* ----------------------------------------------------------------- */
         std::vector<char*> apszSubExpr;
-        int i;
-
-        for( i = 0; i < poExpr->nSubExprCount; i++ )
+        for( int i = 0; i < poExpr->nSubExprCount; i++ )
         {
             CPLString osSubExpr = GetFilterForJoin(poExpr->papoSubExpr[i], poSrcFeat,
                                                    poJoinLayer, secondary_table);
@@ -1245,10 +1243,10 @@ static CPLString GetFilterForJoin(swq_expr_node* poExpr, OGRFeature* poSrcFeat,
 
         CPLString osExpr = poExpr->UnparseOperationFromUnparsedSubExpr(&apszSubExpr[0]);
 
-        /* -------------------------------------------------------------------- */
-        /*      cleanup subexpressions.                                         */
-        /* -------------------------------------------------------------------- */
-        for( i = 0; i < poExpr->nSubExprCount; i++ )
+        /* ----------------------------------------------------------------- */
+        /*      cleanup subexpressions.                                      */
+        /* ----------------------------------------------------------------- */
+        for( int i = 0; i < poExpr->nSubExprCount; i++ )
             CPLFree( apszSubExpr[i] );
 
         return osExpr;
@@ -1292,8 +1290,8 @@ OGRFeature *OGRGenSQLResultsLayer::TranslateFeature( OGRFeature *poSrcFeat )
         CPLAssert(psJoinInfo->secondary_table == iJoin + 1);
 
         OGRLayer *poJoinLayer = papoTableLayers[psJoinInfo->secondary_table];
-        
-        osFilter = GetFilterForJoin(psJoinInfo->poExpr, poSrcFeat, poJoinLayer, 
+
+        osFilter = GetFilterForJoin(psJoinInfo->poExpr, poSrcFeat, poJoinLayer,
                                     psJoinInfo->secondary_table);
         //CPLDebug("OGR", "Filter = %s\n", osFilter.c_str());
 
@@ -1321,6 +1319,8 @@ OGRFeature *OGRGenSQLResultsLayer::TranslateFeature( OGRFeature *poSrcFeat )
     poDstFeat->SetFID( poSrcFeat->GetFID() );
 
     poDstFeat->SetStyleString( poSrcFeat->GetStyleString() );
+    poDstFeat->SetNativeData( poSrcFeat->GetNativeData() );
+    poDstFeat->SetNativeMediaType( poSrcFeat->GetNativeMediaType() );
 
 /* -------------------------------------------------------------------- */
 /*      Evaluate fields that are complex expressions.                   */
@@ -1342,9 +1342,9 @@ OGRFeature *OGRGenSQLResultsLayer::TranslateFeature( OGRFeature *poSrcFeat )
             continue;
         }
 
-        poResult = psColDef->expr->Evaluate( OGRMultiFeatureFetcher, 
+        poResult = psColDef->expr->Evaluate( OGRMultiFeatureFetcher,
                                              (void *) &apoFeatures );
-        
+
         if( poResult == NULL )
         {
             delete poDstFeat;
@@ -1371,11 +1371,11 @@ OGRFeature *OGRGenSQLResultsLayer::TranslateFeature( OGRFeature *poSrcFeat )
           case SWQ_INTEGER64:
             poDstFeat->SetField( iRegularField++, poResult->int_value );
             break;
-            
+
           case SWQ_FLOAT:
             poDstFeat->SetField( iRegularField++, poResult->float_value );
             break;
-            
+
           case SWQ_GEOMETRY:
           {
             OGRGenSQLGeomFieldDefn* poGeomFieldDefn =
@@ -1413,7 +1413,7 @@ OGRFeature *OGRGenSQLResultsLayer::TranslateFeature( OGRFeature *poSrcFeat )
             poDstFeat->SetGeomField( iGeomField++, poResult->geometry_value );
             break;
           }
-            
+
           default:
             poDstFeat->SetField( iRegularField++, poResult->string_value );
             break;
@@ -1421,7 +1421,7 @@ OGRFeature *OGRGenSQLResultsLayer::TranslateFeature( OGRFeature *poSrcFeat )
 
         delete poResult;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Copy fields from primary record to the destination feature.     */
 /* -------------------------------------------------------------------- */
@@ -1454,8 +1454,6 @@ OGRFeature *OGRGenSQLResultsLayer::TranslateFeature( OGRFeature *poSrcFeat )
             switch (SpecialFieldTypes[psColDef->field_index - iFIDFieldIndex])
             {
               case SWQ_INTEGER:
-                poDstFeat->SetField( iRegularField, poSrcFeat->GetFieldAsInteger(psColDef->field_index) );
-                break;
               case SWQ_INTEGER64:
                 poDstFeat->SetField( iRegularField, poSrcFeat->GetFieldAsInteger64(psColDef->field_index) );
                 break;
@@ -1482,14 +1480,14 @@ OGRFeature *OGRGenSQLResultsLayer::TranslateFeature( OGRFeature *poSrcFeat )
               case SWQ_FLOAT:
                 poDstFeat->SetField( iRegularField, poSrcFeat->GetFieldAsDouble(psColDef->field_index) );
                 break;
-              
+
               case SWQ_STRING:
               case SWQ_TIMESTAMP:
               case SWQ_DATE:
               case SWQ_TIME:
                 poDstFeat->SetField( iRegularField, poSrcFeat->GetFieldAsString(psColDef->field_index) );
                 break;
-                
+
               case SWQ_GEOMETRY:
                   CPLAssert(0);
                   break;
@@ -1515,7 +1513,7 @@ OGRFeature *OGRGenSQLResultsLayer::TranslateFeature( OGRFeature *poSrcFeat )
         if( poJoinFeature == NULL )
             continue;
 
-        // Copy over selected field values. 
+        // Copy over selected field values.
         iRegularField = 0;
         for( int iField = 0; iField < psSelectInfo->result_columns; iField++ )
         {
@@ -1527,7 +1525,7 @@ OGRFeature *OGRGenSQLResultsLayer::TranslateFeature( OGRFeature *poSrcFeat )
 
             if( psColDef->table_index == psJoinInfo->secondary_table )
                 poDstFeat->SetField( iRegularField,
-                                     poJoinFeature->GetRawFieldRef( 
+                                     poJoinFeature->GetRawFieldRef(
                                          psColDef->field_index ) );
 
             iRegularField ++;
@@ -1553,7 +1551,7 @@ OGRFeature *OGRGenSQLResultsLayer::GetNextFeature()
 /* -------------------------------------------------------------------- */
 /*      Handle summary sets.                                            */
 /* -------------------------------------------------------------------- */
-    if( psSelectInfo->query_mode == SWQM_SUMMARY_RECORD 
+    if( psSelectInfo->query_mode == SWQM_SUMMARY_RECORD
         || psSelectInfo->query_mode == SWQM_DISTINCT_LIST )
         return GetFeature( nNextIndexFID++ );
 
@@ -1562,7 +1560,7 @@ OGRFeature *OGRGenSQLResultsLayer::GetNextFeature()
 /* -------------------------------------------------------------------- */
 /*      Handle ordered sets.                                            */
 /* -------------------------------------------------------------------- */
-    while( TRUE )
+    while( true )
     {
         OGRFeature *poFeature;
 
@@ -1574,7 +1572,7 @@ OGRFeature *OGRGenSQLResultsLayer::GetNextFeature()
 
             if( poSrcFeat == NULL )
                 return NULL;
-            
+
             poFeature = TranslateFeature( poSrcFeat );
             delete poSrcFeat;
         }
@@ -1664,7 +1662,7 @@ OGRFeature *OGRGenSQLResultsLayer::GetFeature( GIntBig nFID )
 
     poResult = TranslateFeature( poSrcFeature );
     poResult->SetFID( nFID );
-    
+
     delete poSrcFeature;
 
     return poResult;
@@ -1674,7 +1672,7 @@ OGRFeature *OGRGenSQLResultsLayer::GetFeature( GIntBig nFID )
 /*                          GetSpatialFilter()                          */
 /************************************************************************/
 
-OGRGeometry *OGRGenSQLResultsLayer::GetSpatialFilter() 
+OGRGeometry *OGRGenSQLResultsLayer::GetSpatialFilter()
 
 {
     return NULL;
@@ -1751,7 +1749,7 @@ void OGRGenSQLResultsLayer::CreateOrderByIndex()
     size_t nFeaturesAlloc = 100;
 
     panFIDIndex = NULL;
-    pasIndexFields = (OGRField *) 
+    pasIndexFields = (OGRField *)
         CPLCalloc(sizeof(OGRField), nOrderItems * nFeaturesAlloc);
     panFIDList = (GIntBig *) CPLMalloc(sizeof(GIntBig) * nFeaturesAlloc);
 
@@ -1778,7 +1776,7 @@ void OGRGenSQLResultsLayer::CreateOrderByIndex()
                 return;
             }
             OGRField* pasNewIndexFields = (OGRField *)
-                VSIRealloc(pasIndexFields,
+                VSI_REALLOC_VERBOSE(pasIndexFields,
                            sizeof(OGRField) * nOrderItems * (size_t)nNewFeaturesAlloc);
             if (pasNewIndexFields == NULL)
             {
@@ -1791,7 +1789,7 @@ void OGRGenSQLResultsLayer::CreateOrderByIndex()
             pasIndexFields = pasNewIndexFields;
 
             GIntBig* panNewFIDList = (GIntBig *)
-                VSIRealloc(panFIDList, sizeof(GIntBig) *  (size_t)nNewFeaturesAlloc);
+                VSI_REALLOC_VERBOSE(panFIDList, sizeof(GIntBig) *  (size_t)nNewFeaturesAlloc);
             if (panNewFIDList == NULL)
             {
                 VSIFree(pasIndexFields);
@@ -1822,9 +1820,6 @@ void OGRGenSQLResultsLayer::CreateOrderByIndex()
                     switch (SpecialFieldTypes[psKeyDef->field_index - iFIDFieldIndex])
                     {
                       case SWQ_INTEGER:
-                        psDstField->Integer = poSrcFeat->GetFieldAsInteger(psKeyDef->field_index);
-                        break;
-
                       case SWQ_INTEGER64:
                         psDstField->Integer64 = poSrcFeat->GetFieldAsInteger64(psKeyDef->field_index);
                         break;
@@ -1840,14 +1835,14 @@ void OGRGenSQLResultsLayer::CreateOrderByIndex()
                 }
                 continue;
             }
-            
-            poFDefn = poSrcLayer->GetLayerDefn()->GetFieldDefn( 
+
+            poFDefn = poSrcLayer->GetLayerDefn()->GetFieldDefn(
                 psKeyDef->field_index );
 
             psSrcField = poSrcFeat->GetRawFieldRef( psKeyDef->field_index );
 
-            if( poFDefn->GetType() == OFTInteger 
-                || poFDefn->GetType() == OFTInteger64 
+            if( poFDefn->GetType() == OFTInteger
+                || poFDefn->GetType() == OFTInteger64
                 || poFDefn->GetType() == OFTReal
                 || poFDefn->GetType() == OFTDate
                 || poFDefn->GetType() == OFTTime
@@ -1873,10 +1868,9 @@ void OGRGenSQLResultsLayer::CreateOrderByIndex()
 /* -------------------------------------------------------------------- */
 /*      Initialize panFIDIndex                                          */
 /* -------------------------------------------------------------------- */
-    panFIDIndex = (GIntBig *) VSIMalloc(sizeof(GIntBig) * (size_t)nIndexSize);
+    panFIDIndex = (GIntBig *) VSI_MALLOC_VERBOSE(sizeof(GIntBig) * (size_t)nIndexSize);
     if( panFIDIndex == NULL )
     {
-        CPLError(CE_Failure, CPLE_AppDefined, "Cannot allocate panFIDIndex");
         VSIFree(pasIndexFields);
         VSIFree(panFIDList);
         nIndexSize = 0;
@@ -1934,7 +1928,7 @@ void OGRGenSQLResultsLayer::CreateOrderByIndex()
             continue;
         }
 
-        poFDefn = poSrcLayer->GetLayerDefn()->GetFieldDefn( 
+        poFDefn = poSrcLayer->GetLayerDefn()->GetFieldDefn(
             psKeyDef->field_index );
 
         if( poFDefn->GetType() == OFTString )
@@ -1942,8 +1936,8 @@ void OGRGenSQLResultsLayer::CreateOrderByIndex()
             for( i = 0; i < nIndexSize; i++ )
             {
                 OGRField *psField = pasIndexFields + iKey + i * nOrderItems;
-                
-                if( psField->Set.nMarker1 != OGRUnsetMarker 
+
+                if( psField->Set.nMarker1 != OGRUnsetMarker
                     || psField->Set.nMarker2 != OGRUnsetMarker )
                     CPLFree( psField->String );
             }
@@ -1974,7 +1968,7 @@ void OGRGenSQLResultsLayer::CreateOrderByIndex()
 /*      Sort the records in a section of the index.                     */
 /************************************************************************/
 
-int OGRGenSQLResultsLayer::SortIndexSection( OGRField *pasIndexFields, 
+int OGRGenSQLResultsLayer::SortIndexSection( OGRField *pasIndexFields,
                                               GIntBig nStart, GIntBig nEntries )
 
 {
@@ -1995,13 +1989,12 @@ int OGRGenSQLResultsLayer::SortIndexSection( OGRField *pasIndexFields,
         !SortIndexSection( pasIndexFields, nSecondStart, nSecondGroup ) )
         return FALSE;
 
-    panMerged = (GIntBig *) VSIMalloc( sizeof(GIntBig) * (size_t)nEntries );
+    panMerged = (GIntBig *) VSI_MALLOC_VERBOSE( sizeof(GIntBig) * (size_t)nEntries );
     if( panMerged == NULL )
     {
-        CPLError(CE_Failure, CPLE_AppDefined, "Cannot allocated panMerged");
         return FALSE;
     }
-        
+
     while( iMerge < nEntries )
     {
         int  nResult;
@@ -2011,9 +2004,9 @@ int OGRGenSQLResultsLayer::SortIndexSection( OGRField *pasIndexFields,
         else if( nSecondGroup == 0 )
             nResult = 1;
         else
-            nResult = Compare( pasIndexFields 
-                               + panFIDIndex[nFirstStart] * nOrderItems, 
-                               pasIndexFields 
+            nResult = Compare( pasIndexFields
+                               + panFIDIndex[nFirstStart] * nOrderItems,
+                               pasIndexFields
                                + panFIDIndex[nSecondStart] * nOrderItems );
 
         if( nResult < 0 )
@@ -2032,7 +2025,7 @@ int OGRGenSQLResultsLayer::SortIndexSection( OGRField *pasIndexFields,
 
     memcpy( panFIDIndex + nStart, panMerged, sizeof(GIntBig) * (size_t)nEntries );
     CPLFree( panMerged );
-    
+
     return TRUE;
 }
 
@@ -2060,19 +2053,19 @@ int OGRGenSQLResultsLayer::Compare( OGRField *pasFirstTuple,
         else if( psKeyDef->field_index >= iFIDFieldIndex )
             poFDefn = NULL;
         else
-            poFDefn = poSrcLayer->GetLayerDefn()->GetFieldDefn( 
+            poFDefn = poSrcLayer->GetLayerDefn()->GetFieldDefn(
                 psKeyDef->field_index );
-        
-        if( pasFirstTuple[iKey].Set.nMarker1 == OGRUnsetMarker 
+
+        if( pasFirstTuple[iKey].Set.nMarker1 == OGRUnsetMarker
              && pasFirstTuple[iKey].Set.nMarker2 == OGRUnsetMarker )
         {
-            if( pasSecondTuple[iKey].Set.nMarker1 == OGRUnsetMarker 
+            if( pasSecondTuple[iKey].Set.nMarker1 == OGRUnsetMarker
                 && pasSecondTuple[iKey].Set.nMarker2 == OGRUnsetMarker )
                 nResult = 0;
             else
                 nResult = -1;
         }
-        else if ( pasSecondTuple[iKey].Set.nMarker1 == OGRUnsetMarker 
+        else if ( pasSecondTuple[iKey].Set.nMarker1 == OGRUnsetMarker
                 && pasSecondTuple[iKey].Set.nMarker2 == OGRUnsetMarker )
         {
             nResult = 1;
@@ -2082,11 +2075,6 @@ int OGRGenSQLResultsLayer::Compare( OGRField *pasFirstTuple,
             switch (SpecialFieldTypes[psKeyDef->field_index - iFIDFieldIndex])
             {
               case SWQ_INTEGER:
-                if( pasFirstTuple[iKey].Integer < pasSecondTuple[iKey].Integer )
-                    nResult = -1;
-                else if( pasFirstTuple[iKey].Integer > pasSecondTuple[iKey].Integer )
-                    nResult = 1;
-                break;
               case SWQ_INTEGER64:
                 if( pasFirstTuple[iKey].Integer64 < pasSecondTuple[iKey].Integer64 )
                     nResult = -1;
@@ -2113,7 +2101,7 @@ int OGRGenSQLResultsLayer::Compare( OGRField *pasFirstTuple,
         {
             if( pasFirstTuple[iKey].Integer < pasSecondTuple[iKey].Integer )
                 nResult = -1;
-            else if( pasFirstTuple[iKey].Integer 
+            else if( pasFirstTuple[iKey].Integer
                      > pasSecondTuple[iKey].Integer )
                 nResult = 1;
         }
@@ -2121,7 +2109,7 @@ int OGRGenSQLResultsLayer::Compare( OGRField *pasFirstTuple,
         {
             if( pasFirstTuple[iKey].Integer64 < pasSecondTuple[iKey].Integer64 )
                 nResult = -1;
-            else if( pasFirstTuple[iKey].Integer64 
+            else if( pasFirstTuple[iKey].Integer64
                      > pasSecondTuple[iKey].Integer64 )
                 nResult = 1;
         }
