@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrlayerpool.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogrlayerpool.cpp 31925 2015-12-01 15:02:41Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Defines OGRLayerPool and OGRProxiedLayer class
@@ -29,18 +29,18 @@
 
 #include "ogrlayerpool.h"
 
-CPL_CVSID("$Id: ogrlayerpool.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: ogrlayerpool.cpp 31925 2015-12-01 15:02:41Z rouault $");
 
 /************************************************************************/
 /*                      OGRAbstractProxiedLayer()                       */
 /************************************************************************/
 
-OGRAbstractProxiedLayer::OGRAbstractProxiedLayer(OGRLayerPool* poPool)
+OGRAbstractProxiedLayer::OGRAbstractProxiedLayer(OGRLayerPool* poPoolIn) :
+    poPrevLayer(NULL),
+    poNextLayer(NULL)
 {
-    CPLAssert(poPool != NULL);
-    this->poPool = poPool;
-    poPrevLayer = NULL;
-    poNextLayer = NULL;
+    CPLAssert(poPoolIn != NULL);
+    poPool = poPoolIn;
 }
 
 /************************************************************************/
@@ -59,12 +59,12 @@ OGRAbstractProxiedLayer::~OGRAbstractProxiedLayer()
 /*                            OGRLayerPool()                            */
 /************************************************************************/
 
-OGRLayerPool::OGRLayerPool(int nMaxSimultaneouslyOpened)
+OGRLayerPool::OGRLayerPool(int nMaxSimultaneouslyOpenedIn) :
+    poMRULayer(NULL),
+    poLRULayer(NULL),
+    nMRUListSize(0)
 {
-    poMRULayer = NULL;
-    poLRULayer = NULL;
-    nMRUListSize = 0;
-    this->nMaxSimultaneouslyOpened = nMaxSimultaneouslyOpened;
+    nMaxSimultaneouslyOpened = nMaxSimultaneouslyOpenedIn;
 }
 
 /************************************************************************/
@@ -154,16 +154,16 @@ void OGRLayerPool::UnchainLayer(OGRAbstractProxiedLayer* poLayer)
 /*                          OGRProxiedLayer()                           */
 /************************************************************************/
 
-OGRProxiedLayer::OGRProxiedLayer(OGRLayerPool* poPool,
-                                 OpenLayerFunc pfnOpenLayer,
-                                 FreeUserDataFunc pfnFreeUserData,
-                                 void* pUserData) : OGRAbstractProxiedLayer(poPool)
+OGRProxiedLayer::OGRProxiedLayer(OGRLayerPool* poPoolIn,
+                                 OpenLayerFunc pfnOpenLayerIn,
+                                 FreeUserDataFunc pfnFreeUserDataIn,
+                                 void* pUserDataIn) : OGRAbstractProxiedLayer(poPoolIn)
 {
-    CPLAssert(pfnOpenLayer != NULL);
+    CPLAssert(pfnOpenLayerIn != NULL);
 
-    this->pfnOpenLayer = pfnOpenLayer;
-    this->pfnFreeUserData = pfnFreeUserData;
-    this->pUserData = pUserData;
+    pfnOpenLayer = pfnOpenLayerIn;
+    pfnFreeUserData = pfnFreeUserDataIn;
+    pUserData = pUserDataIn;
     poUnderlyingLayer = NULL;
     poFeatureDefn = NULL;
     poSRS = NULL;
@@ -216,6 +216,22 @@ void OGRProxiedLayer::CloseUnderlyingLayer()
     poUnderlyingLayer = NULL;
 }
 
+/************************************************************************/
+/*                          GetUnderlyingLayer()                        */
+/************************************************************************/
+
+OGRLayer* OGRProxiedLayer::GetUnderlyingLayer()
+{
+    if( poUnderlyingLayer == NULL )
+    {
+        //  If the open fails, poUnderlyingLayer will still be a nullptr
+        // and the user will be warned by the open call.
+        // coverity[check_return]
+        OpenUnderlyingLayer();
+    }
+    return poUnderlyingLayer;
+}
+
 /************************************************************************/
 /*                          GetSpatialFilter()                          */
 /************************************************************************/
@@ -279,7 +295,7 @@ OGRFeature *OGRProxiedLayer::GetNextFeature()
 /*                           SetNextByIndex()                           */
 /************************************************************************/
 
-OGRErr      OGRProxiedLayer::SetNextByIndex( long nIndex )
+OGRErr      OGRProxiedLayer::SetNextByIndex( GIntBig nIndex )
 {
     if( poUnderlyingLayer == NULL && !OpenUnderlyingLayer() ) return OGRERR_FAILURE;
     return poUnderlyingLayer->SetNextByIndex(nIndex);
@@ -289,27 +305,27 @@ OGRErr      OGRProxiedLayer::SetNextByIndex( long nIndex )
 /*                             GetFeature()                             */
 /************************************************************************/
 
-OGRFeature *OGRProxiedLayer::GetFeature( long nFID )
+OGRFeature *OGRProxiedLayer::GetFeature( GIntBig nFID )
 {
     if( poUnderlyingLayer == NULL && !OpenUnderlyingLayer() ) return NULL;
     return poUnderlyingLayer->GetFeature(nFID);
 }
 
 /************************************************************************/
-/*                             SetFeature()                             */
+/*                             ISetFeature()                             */
 /************************************************************************/
 
-OGRErr      OGRProxiedLayer::SetFeature( OGRFeature *poFeature )
+OGRErr      OGRProxiedLayer::ISetFeature( OGRFeature *poFeature )
 {
     if( poUnderlyingLayer == NULL && !OpenUnderlyingLayer() ) return OGRERR_FAILURE;
     return poUnderlyingLayer->SetFeature(poFeature);
 }
 
 /************************************************************************/
-/*                            CreateFeature()                           */
+/*                            ICreateFeature()                           */
 /************************************************************************/
 
-OGRErr      OGRProxiedLayer::CreateFeature( OGRFeature *poFeature )
+OGRErr      OGRProxiedLayer::ICreateFeature( OGRFeature *poFeature )
 {
     if( poUnderlyingLayer == NULL && !OpenUnderlyingLayer() ) return OGRERR_FAILURE;
     return poUnderlyingLayer->CreateFeature(poFeature);
@@ -319,7 +335,7 @@ OGRErr      OGRProxiedLayer::CreateFeature( OGRFeature *poFeature )
 /*                           DeleteFeature()                            */
 /************************************************************************/
 
-OGRErr      OGRProxiedLayer::DeleteFeature( long nFID )
+OGRErr      OGRProxiedLayer::DeleteFeature( GIntBig nFID )
 {
     if( poUnderlyingLayer == NULL && !OpenUnderlyingLayer() ) return OGRERR_FAILURE;
     return poUnderlyingLayer->DeleteFeature(nFID);
@@ -390,7 +406,7 @@ OGRSpatialReference *OGRProxiedLayer::GetSpatialRef()
 /*                          GetFeatureCount()                           */
 /************************************************************************/
 
-int         OGRProxiedLayer::GetFeatureCount( int bForce )
+GIntBig         OGRProxiedLayer::GetFeatureCount( int bForce )
 {
     if( poUnderlyingLayer == NULL && !OpenUnderlyingLayer() ) return 0;
     return poUnderlyingLayer->GetFeatureCount(bForce);
@@ -461,10 +477,10 @@ OGRErr      OGRProxiedLayer::ReorderFields( int* panMap )
 /*                           AlterFieldDefn()                           */
 /************************************************************************/
 
-OGRErr      OGRProxiedLayer::AlterFieldDefn( int iField, OGRFieldDefn* poNewFieldDefn, int nFlags )
+OGRErr      OGRProxiedLayer::AlterFieldDefn( int iField, OGRFieldDefn* poNewFieldDefn, int nFlagsIn )
 {
     if( poUnderlyingLayer == NULL && !OpenUnderlyingLayer() ) return OGRERR_FAILURE;
-    return poUnderlyingLayer->AlterFieldDefn(iField, poNewFieldDefn, nFlags);
+    return poUnderlyingLayer->AlterFieldDefn(iField, poNewFieldDefn, nFlagsIn);
 }
 
 /************************************************************************/
@@ -566,4 +582,3 @@ OGRErr      OGRProxiedLayer::SetIgnoredFields( const char **papszFields )
     if( poUnderlyingLayer == NULL && !OpenUnderlyingLayer() ) return OGRERR_FAILURE;
     return poUnderlyingLayer->SetIgnoredFields(papszFields);
 }
-
