@@ -29,10 +29,10 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "ogrgeoconceptlayer.h"
-#include "ogrgeoconceptdatasource.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
+#include "ogrgeoconceptdatasource.h"
+#include "ogrgeoconceptlayer.h"
 
 CPL_CVSID("$Id: ogrgeoconceptdatasource.cpp 00000 2007-11-03 11:49:22Z drichard $");
 
@@ -40,22 +40,18 @@ CPL_CVSID("$Id: ogrgeoconceptdatasource.cpp 00000 2007-11-03 11:49:22Z drichard
 /*                         OGRGeoconceptDataSource()                    */
 /************************************************************************/
 
-OGRGeoconceptDataSource::OGRGeoconceptDataSource()
-
-{
-    _papoLayers = NULL;
-    _nLayers = 0;
-
-    _pszGCT = NULL;
-    _pszName = NULL;
-    _pszDirectory = NULL;
-    _pszExt = NULL;
-    _papszOptions = NULL;
-
-    _bSingleNewFile = FALSE;
-    _bUpdate = FALSE;
-    _hGXT = NULL;
-}
+OGRGeoconceptDataSource::OGRGeoconceptDataSource() :
+    _papoLayers(NULL),
+    _nLayers(0),
+    _pszGCT(NULL),
+    _pszName(NULL),
+    _pszDirectory(NULL),
+    _pszExt(NULL),
+    _papszOptions(NULL),
+    _bSingleNewFile(false),
+    _bUpdate(false),
+    _hGXT(NULL)
+{}
 
 /************************************************************************/
 /*                        ~OGRGeoconceptDataSource()                    */
@@ -64,42 +60,21 @@ OGRGeoconceptDataSource::OGRGeoconceptDataSource()
 OGRGeoconceptDataSource::~OGRGeoconceptDataSource()
 
 {
-    if ( _pszGCT )
+    for( int i = 0; i < _nLayers; i++ )
     {
-      CPLFree( _pszGCT );
-    }
-    if ( _pszName )
-    {
-      CPLFree( _pszName );
-    }
-    if ( _pszDirectory )
-    {
-      CPLFree( _pszDirectory );
-    }
-    if ( _pszExt )
-    {
-      CPLFree( _pszExt );
-    }
-
-    if ( _papoLayers )
-    {
-      for( int i = 0; i < _nLayers; i++ )
-      {
         delete _papoLayers[i];
-      }
-
-      CPLFree( _papoLayers );
     }
+    CPLFree( _papoLayers );
+    CPLFree( _pszGCT );
+    CPLFree( _pszName );
+    CPLFree( _pszDirectory );
+    CPLFree( _pszExt );
+    CSLDestroy( _papszOptions );
 
     if( _hGXT )
     {
       Close_GCIO(&_hGXT);
     }
-
-    if ( _papszOptions )
-    {
-      CSLDestroy( _papszOptions );
-    }
 }
 
 /************************************************************************/
@@ -108,18 +83,10 @@ OGRGeoconceptDataSource::~OGRGeoconceptDataSource()
 /*      Open an existing file.                                          */
 /************************************************************************/
 
-int OGRGeoconceptDataSource::Open( const char* pszName, int bTestOpen, int bUpdate )
+int OGRGeoconceptDataSource::Open( const char* pszName, bool bTestOpen,
+                                   bool bUpdate )
 
 {
-/* -------------------------------------------------------------------- */
-/*      We will only consider .gxt and .txt files.                      */
-/* -------------------------------------------------------------------- */
-    const char* pszExtension = CPLGetExtension(pszName);
-    if( !EQUAL(pszExtension,"gxt") && !EQUAL(pszExtension,"txt") )
-    {
-        return FALSE;
-    }
-
 /* -------------------------------------------------------------------- */
 /*      Is the given path a directory or a regular file?                */
 /* -------------------------------------------------------------------- */
@@ -131,8 +98,9 @@ int OGRGeoconceptDataSource::Open( const char* pszName, int bTestOpen, int bUpda
         if( !bTestOpen )
         {
             CPLError( CE_Failure, CPLE_AppDefined,
-                    "%s is neither a file or directory, Geoconcept access failed.",
-                    pszName );
+                      "%s is neither a file or directory, "
+                      "Geoconcept access failed.",
+                      pszName );
         }
 
         return FALSE;
@@ -149,10 +117,10 @@ int OGRGeoconceptDataSource::Open( const char* pszName, int bTestOpen, int bUpda
 
     if( VSI_ISREG(stat.st_mode) )
     {
-        _bSingleNewFile= FALSE;
-        _bUpdate= bUpdate;
-        _pszName= CPLStrdup( pszName );
-        if( !LoadFile( _bUpdate? "a+t":"rt" ) )
+        _bSingleNewFile = false;
+        _bUpdate = bUpdate;
+        _pszName = CPLStrdup( pszName );
+        if( !LoadFile( _bUpdate ? "a+t":"rt" ) )
         {
             CPLDebug( "GEOCONCEPT",
                       "Failed to open Geoconcept %s."
@@ -175,8 +143,6 @@ int OGRGeoconceptDataSource::Open( const char* pszName, int bTestOpen, int bUpda
 int OGRGeoconceptDataSource::LoadFile( const char *pszMode )
 
 {
-    OGRGeoconceptLayer *poFile;
-
     if( _pszExt == NULL)
     {
       const char* pszExtension = CPLGetExtension(_pszName);
@@ -196,24 +162,24 @@ int OGRGeoconceptDataSource::LoadFile( const char *pszMode )
     GCExportFileMetadata* Meta= GetGCMeta_GCIO(_hGXT);
     if( Meta )
     {
-      int nC, iC, nS, iS;
+      const int nC = CountMetaTypes_GCIO(Meta);
 
-      if( (nC= CountMetaTypes_GCIO(Meta))>0 )
+      if( nC > 0 )
       {
-        GCType* aClass;
-        GCSubType* aSubclass;
-
-        for( iC= 0; iC<nC; iC++ )
+        for( int iC= 0; iC<nC; iC++ )
         {
-          if( (aClass= GetMetaType_GCIO(Meta,iC)) )
+          GCType* aClass = GetMetaType_GCIO(Meta,iC);
+          if( aClass )
           {
-            if( (nS= CountTypeSubtypes_GCIO(aClass)) )
+            const int nS = CountTypeSubtypes_GCIO(aClass);
+            if( nS )
             {
-              for( iS= 0; iS<nS; iS++ )
+              for( int iS = 0; iS<nS; iS++ )
               {
-                if( (aSubclass= GetTypeSubtype_GCIO(aClass,iS)) )
+                GCSubType *aSubclass = GetTypeSubtype_GCIO(aClass,iS);
+                if( aSubclass )
                 {
-                  poFile = new OGRGeoconceptLayer;
+                  OGRGeoconceptLayer *poFile = new OGRGeoconceptLayer;
                   if( poFile->Open(aSubclass) != OGRERR_NONE )
                   {
                     delete poFile;
@@ -221,13 +187,15 @@ int OGRGeoconceptDataSource::LoadFile( const char *pszMode )
                   }
 
                   /* Add layer to data source layers list */
-                  _papoLayers = (OGRGeoconceptLayer **)
-                      CPLRealloc( _papoLayers,  sizeof(OGRGeoconceptLayer *) * (_nLayers+1) );
+                  _papoLayers = static_cast<OGRGeoconceptLayer **>(
+                      CPLRealloc( _papoLayers,
+                                  sizeof(OGRGeoconceptLayer *)
+                                  * (_nLayers+1) ) );
                   _papoLayers[_nLayers++] = poFile;
 
-                  CPLDebug("GEOCONCEPT",
-                           "nLayers=%d - last=[%s]",
-                           _nLayers, poFile->GetLayerDefn()->GetName());
+                  CPLDebug( "GEOCONCEPT",
+                            "nLayers=%d - last=[%s]",
+                            _nLayers, poFile->GetLayerDefn()->GetName());
                 }
               }
             }
@@ -252,20 +220,17 @@ int OGRGeoconceptDataSource::LoadFile( const char *pszMode )
 int OGRGeoconceptDataSource::Create( const char *pszName, char** papszOptions )
 
 {
-    const char *pszConf;
-    const char *pszExtension;
-
-    if( _pszName ) CPLFree(_pszName);
+    CPLFree( _pszName );
     _papszOptions = CSLDuplicate( papszOptions );
 
-    pszConf= CSLFetchNameValue(papszOptions,"CONFIG");
+    const char *pszConf = CSLFetchNameValue(papszOptions,"CONFIG");
     if( pszConf != NULL )
     {
       _pszGCT = CPLStrdup(pszConf);
     }
 
     _pszExt = (char *)CSLFetchNameValue(papszOptions,"EXTENSION");
-    pszExtension = CSLFetchNameValue(papszOptions,"EXTENSION");
+    const char *pszExtension = CSLFetchNameValue( papszOptions, "EXTENSION" );
     if( pszExtension == NULL )
     {
         _pszExt = CPLStrdup(CPLGetExtension(pszName));
@@ -309,9 +274,9 @@ int OGRGeoconceptDataSource::Create( const char *pszName, char** papszOptions )
 
 /* -------------------------------------------------------------------- */
 /*      Create a new single file.                                       */
-/*      OGRGeoconceptDriver::CreateLayer() will do the job.             */
+/*      OGRGeoconceptDriver::ICreateLayer() will do the job.             */
 /* -------------------------------------------------------------------- */
-    _bSingleNewFile = TRUE;
+    _bSingleNewFile = true;
 
     if( !LoadFile( "wt" ) )
     {
@@ -326,26 +291,18 @@ int OGRGeoconceptDataSource::Create( const char *pszName, char** papszOptions )
 }
 
 /************************************************************************/
-/*                            CreateLayer()                             */
+/*                           ICreateLayer()                             */
 /*                                                                      */
 /* Options (-lco) :                                                     */
 /*   FEATURETYPE : TYPE.SUBTYPE                                         */
 /************************************************************************/
 
-OGRLayer *OGRGeoconceptDataSource::CreateLayer( const char * pszLayerName,
+OGRLayer *OGRGeoconceptDataSource::ICreateLayer( const char * pszLayerName,
                                                 OGRSpatialReference *poSRS /* = NULL */,
                                                 OGRwkbGeometryType eType /* = wkbUnknown */,
                                                 char ** papszOptions /* = NULL */ )
 
 {
-    GCTypeKind gcioFeaType;
-    GCDim gcioDim;
-    OGRGeoconceptLayer *poFile= NULL;
-    const char *pszFeatureType; 
-    char **ft;
-    int iLayer;
-    char pszln[512];
-
     if( _hGXT == NULL )
     {
         CPLError( CE_Failure, CPLE_NotSupported,
@@ -365,6 +322,9 @@ OGRLayer *OGRGeoconceptDataSource::CreateLayer( const char * pszLayerName,
     /*
      * pszLayerName Class.Subclass if -nln option used, otherwise file name
      */
+    const char *pszFeatureType;
+    char pszln[512];
+
     if( !(pszFeatureType = CSLFetchNameValue(papszOptions,"FEATURETYPE")) )
     {
       if( !pszLayerName || !strchr(pszLayerName,'.') )
@@ -378,6 +338,7 @@ OGRLayer *OGRGeoconceptDataSource::CreateLayer( const char * pszLayerName,
         pszFeatureType= pszLayerName;
     }
 
+    char **ft;
     if( !(ft= CSLTokenizeString2(pszFeatureType,".",0)) ||
         CSLCount(ft)!=2 )
     {
@@ -392,7 +353,9 @@ OGRLayer *OGRGeoconceptDataSource::CreateLayer( const char * pszLayerName,
 /* -------------------------------------------------------------------- */
 /*      Figure out what type of layer we need.                          */
 /* -------------------------------------------------------------------- */
-    gcioDim= v2D_GCIO;
+    GCTypeKind gcioFeaType;
+    GCDim gcioDim = v2D_GCIO;
+
     if( eType == wkbUnknown )
         gcioFeaType = vUnknownItemType_GCIO;
     else if( eType == wkbPoint )
@@ -451,11 +414,13 @@ OGRLayer *OGRGeoconceptDataSource::CreateLayer( const char * pszLayerName,
      * layer name to exist in the CONFIG as "Class.Subclass".
      * Removing the CONFIG, implies on-the-fly-creation of layers...
      */
+    OGRGeoconceptLayer *poFile= NULL;
+
     if( _nLayers > 0 )
-      for( iLayer= 0; iLayer<_nLayers; iLayer++)
+      for( int iLayer= 0; iLayer<_nLayers; iLayer++)
       {
-        poFile= (OGRGeoconceptLayer*)GetLayer(iLayer);
-        if( EQUAL(poFile->GetLayerDefn()->GetName(),pszFeatureType) )
+        poFile= reinterpret_cast<OGRGeoconceptLayer *>( GetLayer(iLayer) );
+        if( poFile != NULL && EQUAL(poFile->GetLayerDefn()->GetName(),pszFeatureType) )
         {
           break;
         }
@@ -533,8 +498,9 @@ OGRLayer *OGRGeoconceptDataSource::CreateLayer( const char * pszLayerName,
         return NULL;
       }
 
-      _papoLayers = (OGRGeoconceptLayer **)
-                      CPLRealloc( _papoLayers,  sizeof(OGRGeoconceptLayer *) * (_nLayers+1) );
+      _papoLayers = static_cast<OGRGeoconceptLayer **>(
+          CPLRealloc( _papoLayers,
+                      sizeof(OGRGeoconceptLayer *) * (_nLayers+1) ) );
       _papoLayers[_nLayers++] = poFile;
 
       CPLDebug("GEOCONCEPT",
@@ -561,8 +527,8 @@ int OGRGeoconceptDataSource::TestCapability( const char * pszCap )
 {
     if( EQUAL(pszCap,ODsCCreateLayer) )
         return TRUE;
-    else
-        return FALSE;
+
+    return FALSE;
 }
 
 /************************************************************************/
@@ -572,10 +538,9 @@ int OGRGeoconceptDataSource::TestCapability( const char * pszCap )
 OGRLayer *OGRGeoconceptDataSource::GetLayer( int iLayer )
 
 {
-    OGRLayer *poFile;
     if( iLayer < 0 || iLayer >= GetLayerCount() )
-        poFile= NULL;
-    else
-        poFile= _papoLayers[iLayer];
+      return NULL;
+
+    OGRLayer *poFile = _papoLayers[iLayer];
     return poFile;
 }
