@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: ogrgeoconceptlayer.h 
+ * $Id: ogrgeoconceptlayer.h
  *
  * Name:     ogrgeoconceptlayer.h
  * Project:  OpenGIS Simple Features Reference Implementation
@@ -15,24 +15,24 @@
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************/
 
 #include "ogrsf_frmts.h"
 #include "geoconcept.h"
 
-#ifndef _GEOCONCEPT_OGR_LAYER_H_INCLUDED_
-#define _GEOCONCEPT_OGR_LAYER_H_INCLUDED_
+#ifndef GEOCONCEPT_OGR_LAYER_H_INCLUDED_
+#define GEOCONCEPT_OGR_LAYER_H_INCLUDED_
 
 /**********************************************************************/
 /*            OGCGeoconceptLayer Class                           */
@@ -56,16 +56,18 @@ class OGRGeoconceptLayer : public OGRLayer
 //    OGRErr               SetAttributeFilter( const char* pszQuery );
     void                 ResetReading();
     OGRFeature*          GetNextFeature();
-//    OGRErr               SetNextByIndex( long nIndex );
+//    OGRErr               SetNextByIndex( GIntBig nIndex );
 
-//    OGRFeature*          GetFeature( long nFID );
-//    OGRErr               SetFeature( OGRFeature* poFeature );
-//    OGRErr               DeleteFeature( long nFID );
-    OGRErr               CreateFeature( OGRFeature* poFeature );
+//    OGRFeature*          GetFeature( GIntBig nFID );
+//    OGRErr               ISetFeature( OGRFeature* poFeature );
+//    OGRErr               DeleteFeature( GIntBig nFID );
+    OGRErr               ICreateFeature( OGRFeature* poFeature );
     OGRFeatureDefn*      GetLayerDefn( ) { return _poFeatureDefn; } // FIXME
     OGRSpatialReference* GetSpatialRef( );
-    int                  GetFeatureCount( int bForce = TRUE );
+    GIntBig              GetFeatureCount( int bForce = TRUE );
     OGRErr               GetExtent( OGREnvelope *psExtent, int bForce = TRUE );
+    virtual OGRErr      GetExtent(int iGeomField, OGREnvelope *psExtent, int bForce)
+                { return OGRLayer::GetExtent(iGeomField, psExtent, bForce); }
     int                  TestCapability( const char* pszCap );
 //    const char*          GetInfo( const char* pszTag );
     OGRErr               CreateField( OGRFieldDefn* poField, int bApproxOK = TRUE );
@@ -79,4 +81,4 @@ class OGRGeoconceptLayer : public OGRLayer
     void                   SetSpatialRef( OGRSpatialReference *poSpatialRef );
 };
 
-#endif /* _GEOCONCEPT_OGR_LAYER_H_INCLUDED_ */
+#endif /* GEOCONCEPT_OGR_LAYER_H_INCLUDED_ */
