@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrgeojsondatasource.cpp 28492 2015-02-15 14:16:35Z rouault $
+ * $Id: ogrgeojsondatasource.cpp 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implementation of OGRGeoJSONDataSource class (OGR GeoJSON Driver).
@@ -30,8 +30,10 @@
 #include "ogr_geojson.h"
 #include "ogrgeojsonutils.h"
 #include "ogrgeojsonreader.h"
+#include "gdal_utils.h"
 #include <cpl_http.h>
 #include <json.h> // JSON-C
+#include "ogrgeojsonwriter.h"
 #include <cstddef>
 #include <cstdlib>
 using namespace std;
@@ -41,15 +43,15 @@ using namespace std;
 /************************************************************************/
 
 OGRGeoJSONDataSource::OGRGeoJSONDataSource()
-    : pszName_(NULL), pszGeoData_(NULL),
-        papoLayers_(NULL), nLayers_(0), fpOut_(NULL),
+    : pszName_(NULL), pszGeoData_(NULL), nGeoDataLen_(0),
+        papoLayers_(NULL), papoLayersWriter_(NULL), nLayers_(0), fpOut_(NULL),
         flTransGeom_( OGRGeoJSONDataSource::eGeometryPreserve ),
-        flTransAttrs_( OGRGeoJSONDataSource::eAtributesPreserve ),
-        bFpOutputIsSeekable_( FALSE ),
-        nBBOXInsertLocation_(0)
-{
-    // I've got constructed. Lunch time!
-}
+        flTransAttrs_( OGRGeoJSONDataSource::eAttributesPreserve ),
+        bOtherPages_(false),
+        bFpOutputIsSeekable_( false ),
+        nBBOXInsertLocation_(0),
+        bUpdatable_( false )
+{ }
 
 /************************************************************************/
 /*                           ~OGRGeoJSONDataSource()                    */
@@ -57,57 +59,35 @@ OGRGeoJSONDataSource::OGRGeoJSONDataSource()
 
 OGRGeoJSONDataSource::~OGRGeoJSONDataSource()
 {
+    FlushCache();
     Clear();
-    
-    if( NULL != fpOut_ )
-    {
-        VSIFCloseL( fpOut_ );
-        fpOut_ = NULL;
-    }
 }
 
 /************************************************************************/
 /*                           Open()                                     */
 /************************************************************************/
 
-int OGRGeoJSONDataSource::Open( const char* pszName )
+int OGRGeoJSONDataSource::Open( GDALOpenInfo* poOpenInfo,
+                                GeoJSONSourceType nSrcType )
 {
-    CPLAssert( NULL != pszName );
-
-/* -------------------------------------------------------------------- */
-/*      Release resources allocated during previous request.            */
-/* -------------------------------------------------------------------- */
-    if( NULL != papoLayers_ )
-    {
-        CPLAssert( nLayers_ > 0 );
-        Clear();
-    }
-
-/* -------------------------------------------------------------------- */
-/*      Determine type of data source: text file (.geojson, .json),     */
-/*      Web Service or text passed directly and load data.              */
-/* -------------------------------------------------------------------- */
-    GeoJSONSourceType nSrcType;
-    
-    VSILFILE* fp = NULL;
-
-    nSrcType = GeoJSONGetSourceType( pszName, &fp );
     if( eGeoJSONSourceService == nSrcType )
     {
-        if( (strstr(pszName, "SERVICE=WFS") || strstr(pszName, "service=WFS") ||
-             strstr(pszName, "service=wfs")) && !strstr(pszName, "json"))
+        if( !ReadFromService( poOpenInfo->pszFilename ) )
             return FALSE;
-
-        if( !ReadFromService( pszName ) )
+        if( poOpenInfo->eAccess == GA_Update )
+        {
+            CPLError(CE_Failure, CPLE_NotSupported,
+                     "Update from remote service not supported");
             return FALSE;
+        }
     }
     else if( eGeoJSONSourceText == nSrcType )
     {
-        pszGeoData_ = CPLStrdup( pszName );
+        pszGeoData_ = CPLStrdup( poOpenInfo->pszFilename );
     }
     else if( eGeoJSONSourceFile == nSrcType )
     {
-        if( !ReadFromFile( pszName, fp ) )
+        if( !ReadFromFile( poOpenInfo ) )
             return FALSE;
     }
     else
@@ -121,22 +101,46 @@ int OGRGeoJSONDataSource::Open( const char* pszName )
 /*      GeoJSON text tree.                                              */
 /* -------------------------------------------------------------------- */
     if( NULL == pszGeoData_ ||
-        strncmp(pszGeoData_, "{\"couchdb\":\"Welcome\"", strlen("{\"couchdb\":\"Welcome\"")) == 0 ||
-        strncmp(pszGeoData_, "{\"db_name\":\"", strlen("{\"db_name\":\"")) == 0 ||
-        strncmp(pszGeoData_, "{\"total_rows\":", strlen("{\"total_rows\":")) == 0 ||
-        strncmp(pszGeoData_, "{\"rows\":[", strlen("{\"rows\":[")) == 0)
+        STARTS_WITH(pszGeoData_, "{\"couchdb\":\"Welcome\"") ||
+        STARTS_WITH(pszGeoData_, "{\"db_name\":\"") ||
+        STARTS_WITH(pszGeoData_, "{\"total_rows\":") ||
+        STARTS_WITH(pszGeoData_, "{\"rows\":["))
     {
         Clear();
         return FALSE;
     }
 
-    LoadLayers();
+    LoadLayers(poOpenInfo->papszOpenOptions);
     if( nLayers_ == 0 )
     {
+        bool bEmitError = true;
+        if( eGeoJSONSourceService == nSrcType )
+        {
+            CPLString osTmpFilename = CPLSPrintf("/vsimem/%p/%s", this,
+                                        CPLGetFilename(poOpenInfo->pszFilename));
+            VSIFCloseL(VSIFileFromMemBuffer( osTmpFilename,
+                                             (GByte*)pszGeoData_,
+                                             nGeoDataLen_,
+                                             TRUE ));
+            pszGeoData_ = NULL;
+            if( GDALIdentifyDriver(osTmpFilename, NULL) )
+                bEmitError = false;
+            VSIUnlink(osTmpFilename);
+        }
         Clear();
-        
-        CPLError( CE_Failure, CPLE_OpenFailed, 
-                  "Failed to read GeoJSON data" );
+
+        if( bEmitError )
+        {
+            CPLError( CE_Failure, CPLE_OpenFailed,
+                    "Failed to read GeoJSON data" );
+        }
+        return FALSE;
+    }
+
+    if( eGeoJSONSourceText == nSrcType && poOpenInfo->eAccess == GA_Update )
+    {
+        CPLError(CE_Failure, CPLE_NotSupported,
+                    "Update from inline definition not supported");
         return FALSE;
     }
 
@@ -169,17 +173,20 @@ OGRLayer* OGRGeoJSONDataSource::GetLayer( int nLayer )
 {
     if( 0 <= nLayer && nLayer < nLayers_ )
     {
-        return papoLayers_[nLayer];
+        if( papoLayers_ )
+            return papoLayers_[nLayer];
+        else
+            return papoLayersWriter_[nLayer];
     }
 
     return NULL;
 }
 
 /************************************************************************/
-/*                            CreateLayer()                             */
+/*                           ICreateLayer()                             */
 /************************************************************************/
 
-OGRLayer* OGRGeoJSONDataSource::CreateLayer( const char* pszName_,
+OGRLayer* OGRGeoJSONDataSource::ICreateLayer( const char* pszNameIn,
                                              OGRSpatialReference* poSRS,
                                              OGRwkbGeometryType eGType,
                                              char** papszOptions )
@@ -198,26 +205,60 @@ OGRLayer* OGRGeoJSONDataSource::CreateLayer( const char* pszName_,
         return NULL;
     }
 
-    OGRGeoJSONWriteLayer* poLayer = NULL;
-    poLayer = new OGRGeoJSONWriteLayer( pszName_, eGType, papszOptions, this );
-
-/* -------------------------------------------------------------------- */
-/*      Add layer to data source layer list.                            */
-/* -------------------------------------------------------------------- */
-    
-    papoLayers_ = (OGRLayer **)
-        CPLRealloc( papoLayers_,  sizeof(OGRLayer*) * (nLayers_ + 1) );
-
-    papoLayers_[nLayers_++] = poLayer;
-
     VSIFPrintfL( fpOut_, "{\n\"type\": \"FeatureCollection\",\n" );
 
+    bool bWriteFC_BBOX =
+        CPLTestBool(CSLFetchNameValueDef(papszOptions, "WRITE_BBOX", "FALSE"));
+
+    const char* pszNativeData = CSLFetchNameValue(papszOptions, "NATIVE_DATA");
+    const char* pszNativeMediaType = CSLFetchNameValue(papszOptions, "NATIVE_MEDIA_TYPE");
+    bool bWriteCRSIfWGS84 = true;
+    if( pszNativeMediaType && EQUAL(pszNativeMediaType, "application/vnd.geo+json") )
+    {
+        json_object *poObj = NULL;
+        if( OGRJSonParse(pszNativeData, &poObj) &&
+            json_object_get_type(poObj) == json_type_object )
+        {
+            json_object_iter it;
+            it.key = NULL;
+            it.val = NULL;
+            it.entry = NULL;
+            CPLString osNativeData;
+            bWriteCRSIfWGS84 = false;
+            json_object_object_foreachC(poObj, it)
+            {
+                if( strcmp(it.key, "type") == 0 ||
+                    strcmp(it.key, "features") == 0  )
+                {
+                    continue;
+                }
+                if( strcmp(it.key, "bbox") == 0 )
+                {
+                    if( CSLFetchNameValue(papszOptions, "WRITE_BBOX") == NULL )
+                        bWriteFC_BBOX = true;
+                    continue;
+                }
+                if( strcmp(it.key, "crs") == 0 )
+                {
+                    bWriteCRSIfWGS84 = true;
+                    continue;
+                }
+                json_object* poKey = json_object_new_string(it.key);
+                VSIFPrintfL( fpOut_, "%s: ", json_object_to_json_string(poKey) );
+                json_object_put(poKey);
+                VSIFPrintfL( fpOut_, "%s,\n", json_object_to_json_string(it.val) );
+            }
+            json_object_put(poObj);
+        }
+    }
+
     if (poSRS)
     {
         const char* pszAuthority = poSRS->GetAuthorityName(NULL);
         const char* pszAuthorityCode = poSRS->GetAuthorityCode(NULL);
         if (pszAuthority != NULL && pszAuthorityCode != NULL &&
-            EQUAL(pszAuthority, "EPSG"))
+            EQUAL(pszAuthority, "EPSG") &&
+            (bWriteCRSIfWGS84 || !EQUAL(pszAuthorityCode, "4326")) )
         {
             json_object* poObjCRS = json_object_new_object();
             json_object_object_add(poObjCRS, "type",
@@ -243,7 +284,7 @@ OGRLayer* OGRGeoJSONDataSource::CreateLayer( const char* pszName_,
         }
     }
 
-    if (bFpOutputIsSeekable_)
+    if (bFpOutputIsSeekable_ && bWriteFC_BBOX)
     {
         nBBOXInsertLocation_ = (int) VSIFTellL( fpOut_ );
 
@@ -255,6 +296,18 @@ OGRLayer* OGRGeoJSONDataSource::CreateLayer( const char* pszName_,
 
     VSIFPrintfL( fpOut_, "\"features\": [\n" );
 
+    OGRGeoJSONWriteLayer* poLayer
+        = new OGRGeoJSONWriteLayer( pszNameIn, eGType, papszOptions, bWriteFC_BBOX, this );
+
+/* -------------------------------------------------------------------- */
+/*      Add layer to data source layer list.                            */
+/* -------------------------------------------------------------------- */
+    CPLAssert(papoLayers_ == NULL);
+    papoLayersWriter_ = (OGRGeoJSONWriteLayer **)
+        CPLRealloc( papoLayers_,  sizeof(OGRGeoJSONWriteLayer*) * (nLayers_ + 1) );
+
+    papoLayersWriter_[nLayers_++] = poLayer;
+
     return poLayer;
 }
 
@@ -265,29 +318,29 @@ OGRLayer* OGRGeoJSONDataSource::CreateLayer( const char* pszName_,
 int OGRGeoJSONDataSource::TestCapability( const char* pszCap )
 {
     if( EQUAL( pszCap, ODsCCreateLayer ) )
-        return fpOut_ != NULL /* && nLayers_ == 0 */;
-    else if( EQUAL( pszCap, ODsCDeleteLayer ) )
-        return FALSE;
-    else
-        return FALSE;
+        return fpOut_ != NULL && nLayers_ == 0;
+
+    // if( EQUAL( pszCap, ODsCDeleteLayer ) )
+    //     return FALSE;
+
+    return FALSE;
 }
 
 /************************************************************************/
 /*                              Create()                                */
 /************************************************************************/
 
-int OGRGeoJSONDataSource::Create( const char* pszName, char** papszOptions )
+int OGRGeoJSONDataSource::Create( const char* pszName,
+                                  char** /* papszOptions */ )
 {
-    UNREFERENCED_PARAM(papszOptions);
-
     CPLAssert( NULL == fpOut_ );
 
     if (strcmp(pszName, "/dev/stdout") == 0)
         pszName = "/vsistdout/";
 
     bFpOutputIsSeekable_ =  !(strcmp(pszName,"/vsistdout/") == 0 ||
-                              strncmp(pszName,"/vsigzip/", 9) == 0 ||
-                              strncmp(pszName,"/vsizip/", 8) == 0);
+                              STARTS_WITH(pszName, "/vsigzip/") ||
+                              STARTS_WITH(pszName, "/vsizip/"));
 
 /* -------------------------------------------------------------------- */
 /*     File overwrite not supported.                                    */
@@ -299,15 +352,15 @@ int OGRGeoJSONDataSource::Create( const char* pszName, char** papszOptions )
                   "The GeoJSON driver does not overwrite existing files." );
         return FALSE;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Create the output file.                                         */
 /* -------------------------------------------------------------------- */
     fpOut_ = VSIFOpenL( pszName, "w" );
     if( NULL == fpOut_)
     {
-        CPLError( CE_Failure, CPLE_OpenFailed, 
-                  "Failed to create GeoJSON datasource: %s.", 
+        CPLError( CE_Failure, CPLE_OpenFailed,
+                  "Failed to create GeoJSON datasource: %s.",
                   pszName );
         return FALSE;
     }
@@ -344,12 +397,16 @@ void OGRGeoJSONDataSource::Clear()
 {
     for( int i = 0; i < nLayers_; i++ )
     {
-        CPLAssert( NULL != papoLayers_ );
-        delete papoLayers_[i];
+        if( papoLayers_ != NULL )
+            delete papoLayers_[i];
+        else
+            delete papoLayersWriter_[i];
     }
 
     CPLFree( papoLayers_ );
     papoLayers_ = NULL;
+    CPLFree( papoLayersWriter_ );
+    papoLayersWriter_ = NULL;
     nLayers_ = 0;
 
     CPLFree( pszName_ );
@@ -357,34 +414,52 @@ void OGRGeoJSONDataSource::Clear()
 
     CPLFree( pszGeoData_ );
     pszGeoData_ = NULL;
+    nGeoDataLen_ = 0;
 
-    if( NULL != fpOut_ )
+    if( fpOut_ )
     {
         VSIFCloseL( fpOut_ );
+        fpOut_ = NULL;
     }
-    fpOut_ = NULL;
 }
 
 /************************************************************************/
 /*                           ReadFromFile()                             */
 /************************************************************************/
 
-int OGRGeoJSONDataSource::ReadFromFile( const char* pszSource, VSILFILE* fpIn )
+int OGRGeoJSONDataSource::ReadFromFile( GDALOpenInfo* poOpenInfo )
 {
     GByte* pabyOut = NULL;
-    if( !VSIIngestFile( fpIn, pszSource, &pabyOut, NULL, -1) )
+    if( poOpenInfo->fpL == NULL ||
+        !VSIIngestFile( poOpenInfo->fpL, poOpenInfo->pszFilename, &pabyOut, NULL, -1) )
     {
-        if( fpIn != NULL )
-            VSIFCloseL(fpIn);
         return FALSE;
     }
-    if( fpIn != NULL )
-        VSIFCloseL(fpIn);
+
+    VSIFCloseL(poOpenInfo->fpL);
+    poOpenInfo->fpL = NULL;
     pszGeoData_ = (char*) pabyOut;
 
-    pszName_ = CPLStrdup( pszSource );
+    pszName_ = CPLStrdup( poOpenInfo->pszFilename );
 
     CPLAssert( NULL != pszGeoData_ );
+
+    if( poOpenInfo->eAccess == GA_Update )
+    {
+        VSILFILE* fp = VSIFOpenL(poOpenInfo->pszFilename, "rb+");
+        if( fp == NULL )
+        {
+            CPLError(CE_Failure, CPLE_FileIO,
+                     "Update not supported because file is not writable");
+            return FALSE;
+        }
+        else
+        {
+            bUpdatable_ = true;
+        }
+        VSIFCloseL(fp);
+    }
+
     return TRUE;
 }
 
@@ -408,10 +483,10 @@ int OGRGeoJSONDataSource::ReadFromService( const char* pszSource )
 /* -------------------------------------------------------------------- */
     CPLErrorReset();
 
-    CPLHTTPResult* pResult = NULL;
     char* papsOptions[] = { (char*) "HEADERS=Accept: text/plain, application/json", NULL };
 
-    pResult = CPLHTTPFetch( pszSource, papsOptions );
+    CPLHTTPResult* pResult
+        = CPLHTTPFetch( pszSource, papsOptions );
 
 /* -------------------------------------------------------------------- */
 /*      Try to handle CURL/HTTP errors.                                 */
@@ -425,7 +500,7 @@ int OGRGeoJSONDataSource::ReadFromService( const char* pszSource )
 
    if( 0 != pResult->nStatus )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "Curl reports error: %d: %s",
                   pResult->nStatus, pResult->pszErrBuf );
         CPLHTTPDestroyResult( pResult );
@@ -439,7 +514,7 @@ int OGRGeoJSONDataSource::ReadFromService( const char* pszSource )
 
     if ( eGeoJSONProtocolUnknown != GeoJSONGetProtocolType( pszData ) )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
             "The data that was downloaded also starts with "
             "protocol prefix (http://, https:// or ftp://) "
             "and cannot be processed as GeoJSON data.");
@@ -449,7 +524,9 @@ int OGRGeoJSONDataSource::ReadFromService( const char* pszSource )
 
     // Directly assign CPLHTTPResult::pabyData to pszGeoData_
     pszGeoData_ = (char*) pszData;
+    nGeoDataLen_ = pResult->nDataLen;
     pResult->pabyData = NULL;
+    pResult->nDataLen = 0;
 
     pszName_ = CPLStrdup( pszSource );
 
@@ -466,7 +543,7 @@ int OGRGeoJSONDataSource::ReadFromService( const char* pszSource )
 /*                           LoadLayers()                               */
 /************************************************************************/
 
-void OGRGeoJSONDataSource::LoadLayers()
+void OGRGeoJSONDataSource::LoadLayers(char** papszOpenOptionsIn)
 {
     if( NULL == pszGeoData_ )
     {
@@ -474,7 +551,7 @@ void OGRGeoJSONDataSource::LoadLayers()
                   "GeoJSON data buffer empty" );
         return;
     }
-    
+
     const char* const apszPrefix[] = { "loadGeoJSON(", "jsonp(" };
     for(size_t iP = 0; iP < sizeof(apszPrefix) / sizeof(apszPrefix[0]); iP++ )
     {
@@ -500,8 +577,6 @@ void OGRGeoJSONDataSource::LoadLayers()
         return;
     }
 
-    OGRErr err = OGRERR_NONE;
-
 /* -------------------------------------------------------------------- */
 /*      Is it ESRI Feature Service data ?                               */
 /* -------------------------------------------------------------------- */
@@ -509,9 +584,18 @@ void OGRGeoJSONDataSource::LoadLayers()
          strstr(pszGeoData_, "esriFieldType") )
     {
         OGRESRIJSONReader reader;
-        err = reader.Parse( pszGeoData_ );
+        OGRErr err = reader.Parse( pszGeoData_ );
         if( OGRERR_NONE == err )
         {
+            json_object* poObj = reader.GetJSonObject();
+            if( poObj && json_object_get_type(poObj) == json_type_object )
+            {
+                json_object* poExceededTransferLimit =
+                    json_object_object_get(poObj, "exceededTransferLimit");
+                if( poExceededTransferLimit && json_object_get_type(poExceededTransferLimit) == json_type_boolean )
+                    bOtherPages_ = CPL_TO_BOOL(
+                        json_object_get_boolean(poExceededTransferLimit) );
+            }
             reader.ReadLayers( this );
         }
         return;
@@ -524,7 +608,7 @@ void OGRGeoJSONDataSource::LoadLayers()
          strstr(pszGeoData_, "\"Topology\"")  )
     {
         OGRTopoJSONReader reader;
-        err = reader.Parse( pszGeoData_ );
+        OGRErr err = reader.Parse( pszGeoData_ );
         if( OGRERR_NONE == err )
         {
             reader.ReadLayers( this );
@@ -542,19 +626,45 @@ void OGRGeoJSONDataSource::LoadLayers()
         reader.SetPreserveGeometryType( false );
         CPLDebug( "GeoJSON", "Geometry as OGRGeometryCollection type." );
     }
-    
-    if( eAtributesSkip == flTransAttrs_ )
+
+    if( eAttributesSkip == flTransAttrs_ )
     {
         reader.SetSkipAttributes( true );
         CPLDebug( "GeoJSON", "Skip all attributes." );
     }
-    
+
+    reader.SetFlattenNestedAttributes(
+        CPL_TO_BOOL(CSLFetchBoolean(papszOpenOptionsIn, "FLATTEN_NESTED_ATTRIBUTES", FALSE)),
+        CSLFetchNameValueDef(papszOpenOptionsIn, "NESTED_ATTRIBUTE_SEPARATOR", "_")[0]);
+
+    const int bDefaultNativeData = bUpdatable_ ? TRUE : FALSE ;
+    reader.SetStoreNativeData(
+        CPL_TO_BOOL(CSLFetchBoolean(papszOpenOptionsIn, "NATIVE_DATA", bDefaultNativeData)));
+
+    reader.SetArrayAsString(
+        CPLTestBool(CSLFetchNameValueDef(papszOpenOptionsIn, "ARRAY_AS_STRING",
+                CPLGetConfigOption("OGR_GEOJSON_ARRAY_AS_STRING", "NO"))));
+
 /* -------------------------------------------------------------------- */
 /*      Parse GeoJSON and build valid OGRLayer instance.                */
 /* -------------------------------------------------------------------- */
-    err = reader.Parse( pszGeoData_ );
+    OGRErr err = reader.Parse( pszGeoData_ );
     if( OGRERR_NONE == err )
     {
+        json_object* poObj = reader.GetJSonObject();
+        if( poObj && json_object_get_type(poObj) == json_type_object )
+        {
+            json_object* poProperties = json_object_object_get(poObj, "properties");
+            if( poProperties && json_object_get_type(poProperties) == json_type_object )
+            {
+                json_object* poExceededTransferLimit =
+                    json_object_object_get(poProperties, "exceededTransferLimit");
+                if( poExceededTransferLimit && json_object_get_type(poExceededTransferLimit) == json_type_boolean )
+                  bOtherPages_ = CPL_TO_BOOL(
+                      json_object_get_boolean(poExceededTransferLimit) );
+            }
+        }
+
         reader.ReadLayers( this );
     }
 
@@ -567,13 +677,114 @@ void OGRGeoJSONDataSource::LoadLayers()
 
 void OGRGeoJSONDataSource::AddLayer( OGRGeoJSONLayer* poLayer )
 {
+    CPLAssert(papoLayersWriter_ == NULL);
 
     poLayer->DetectGeometryType();
 
     /* Return layer in readable state. */
     poLayer->ResetReading();
 
-    papoLayers_ = (OGRLayer**)CPLRealloc( papoLayers_, sizeof(OGRLayer*) * (nLayers_ + 1));
-    papoLayers_[nLayers_] = poLayer; 
+    papoLayers_ = (OGRGeoJSONLayer**)CPLRealloc( papoLayers_, sizeof(OGRGeoJSONLayer*) * (nLayers_ + 1));
+    papoLayers_[nLayers_] = poLayer;
     nLayers_ ++;
 }
+
+/************************************************************************/
+/*                            FlushCache()                              */
+/************************************************************************/
+
+void OGRGeoJSONDataSource::FlushCache()
+{
+    if( papoLayersWriter_ != NULL )
+        return;
+
+    for( int i=0; i < nLayers_; i++ )
+    {
+        if( papoLayers_[i]->HasBeenUpdated() )
+        {
+            papoLayers_[i]->SetUpdated(false);
+
+            CPLString osBackup(pszName_);
+            osBackup += ".bak";
+            if( VSIRename(pszName_, osBackup) < 0 )
+            {
+                CPLError(CE_Failure, CPLE_AppDefined,
+                         "Cannot create backup copy");
+                return;
+            }
+
+            bool bOK = false;
+
+            // Disable all filters
+            OGRFeatureQuery *poAttrQueryBak = papoLayers_[i]->m_poAttrQuery;
+            papoLayers_[i]->m_poAttrQuery = NULL;
+            OGRGeometry* poFilterGeomBak = papoLayers_[i]->m_poFilterGeom;
+            papoLayers_[i]->m_poFilterGeom = NULL;
+
+            // If the source data only contained one single feature and
+            // that's still the case, then do not use a FeatureCollection
+            // on writing
+            bool bAlreadyDone = false;
+            if( papoLayers_[i]->GetFeatureCount(TRUE) == 1 &&
+                papoLayers_[i]->GetMetadata("NATIVE_DATA") == NULL )
+            {
+                papoLayers_[i]->ResetReading();
+                OGRFeature* poFeature = papoLayers_[i]->GetNextFeature();
+                if( poFeature != NULL )
+                {
+                    if( poFeature->GetNativeData() != NULL )
+                    {
+                        bAlreadyDone = true;
+                        json_object* poObj = OGRGeoJSONWriteFeature( poFeature,
+                                                                     FALSE/* bWriteBBOX */,
+                                                                     -1 /*nCoordPrecision*/,
+                                                                     -1 /* nSignificatnFigures*/);
+                        VSILFILE* fp = VSIFOpenL(pszName_, "wb");
+                        if( fp != NULL )
+                        {
+                            bOK = VSIFPrintfL( fp, "%s", json_object_to_json_string( poObj ) ) > 0;
+                            VSIFCloseL( fp );
+                        }
+                        json_object_put( poObj );
+                    }
+                    delete poFeature;
+                }
+            }
+
+            // Otherwise do layer translation
+            if( !bAlreadyDone )
+            {
+                char** papszOptions = NULL;
+                papszOptions = CSLAddString(papszOptions, "-f");
+                papszOptions = CSLAddString(papszOptions, "GeoJSON");
+                GDALVectorTranslateOptions* psOptions =
+                    GDALVectorTranslateOptionsNew(papszOptions, NULL);
+                CSLDestroy(papszOptions);
+                GDALDatasetH hSrcDS = this;
+                GDALDatasetH hOutDS =
+                    GDALVectorTranslate(pszName_, NULL, 1, &hSrcDS, psOptions, NULL);
+                GDALVectorTranslateOptionsFree(psOptions);
+
+                if( hOutDS != NULL )
+                {
+                    CPLErrorReset();
+                    GDALClose(hOutDS);
+                    bOK = (CPLGetLastErrorType() == CE_None);
+                }
+            }
+
+            // Restore filters
+            papoLayers_[i]->m_poAttrQuery = poAttrQueryBak;
+            papoLayers_[i]->m_poFilterGeom = poFilterGeomBak;
+
+            if( bOK )
+            {
+                VSIUnlink(osBackup);
+            }
+            else
+            {
+                VSIRename(osBackup, pszName_);
+            }
+        }
+    }
+}
