@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrgeojsonlayer.cpp 33714 2016-03-13 05:42:13Z goatbar $
+ * $Id: ogrgeojsonlayer.cpp 34334 2016-06-10 07:37:16Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implementation of OGRGeoJSONLayer class (OGR GeoJSON Driver).
@@ -41,7 +41,6 @@
 /************************************************************************/
 
 const char* const OGRGeoJSONLayer::DefaultName = "OGRGeoJSON";
-const char* const OGRGeoJSONLayer::DefaultFIDColumn = "id";
 const OGRwkbGeometryType OGRGeoJSONLayer::DefaultGeometryType = wkbUnknown;
 
 /************************************************************************/
@@ -52,7 +51,8 @@ OGRGeoJSONLayer::OGRGeoJSONLayer( const char* pszName,
                                   OGRSpatialReference* poSRSIn,
                                   OGRwkbGeometryType eGType,
                                   OGRGeoJSONDataSource* poDS )
-  : OGRMemLayer( pszName, poSRSIn, eGType), poDS_(poDS), bUpdated_(false)
+  : OGRMemLayer( pszName, poSRSIn, eGType), poDS_(poDS), bUpdated_(false),
+    bOriginalIdModified_(false)
 {
     SetAdvertizeUTF8(true);
     SetUpdatable( poDS->IsUpdatable() );
@@ -112,22 +112,44 @@ OGRErr OGRGeoJSONLayer::SyncToDisk()
 
 void OGRGeoJSONLayer::AddFeature( OGRFeature* poFeature )
 {
-    if( -1 == poFeature->GetFID() )
+    GIntBig nFID = poFeature->GetFID();
+    
+    // Detect potential FID duplicates and make sure they are eventually
+    // unique
+    if( -1 == nFID )
     {
-        GIntBig nFID = GetFeatureCount(FALSE);
-        poFeature->SetFID( nFID );
-
-        // TODO - mloskot: We need to redesign creation of FID column
-        int nField = poFeature->GetFieldIndex( DefaultFIDColumn );
-        if( -1 != nField &&
-            (GetLayerDefn()->GetFieldDefn(nField)->GetType() == OFTInteger ||
-             GetLayerDefn()->GetFieldDefn(nField)->GetType() == OFTInteger64 ))
+        nFID = GetFeatureCount(FALSE);
+        OGRFeature* poTryFeature;
+        while( (poTryFeature = GetFeature(nFID) ) != NULL )
         {
-            poFeature->SetField( nField, nFID );
+            nFID ++;
+            delete poTryFeature;
         }
     }
+    else
+    {
+        OGRFeature* poTryFeature;
+        if( (poTryFeature = GetFeature(nFID) ) != NULL )
+        {
+            if( !bOriginalIdModified_ )
+            {
+                CPLError(CE_Warning, CPLE_AppDefined,
+                         "Several features with id = " CPL_FRMT_GIB " have been found. "
+                         "Altering it to be unique. This warning will not be emitted for this layer",
+                         nFID);
+                bOriginalIdModified_ = true;
+            }
+            delete poTryFeature;
+            nFID = GetFeatureCount(FALSE);
+            while( (poTryFeature = GetFeature(nFID) ) != NULL )
+            {
+                nFID ++;
+                delete poTryFeature;
+            }
+        }
+    }
+    poFeature->SetFID( nFID );
 
-    GIntBig nFID = poFeature->GetFID();
     if( !CPL_INT64_FITS_ON_INT32(nFID) )
         SetMetadataItem(OLMD_FID64, "YES");
 
