@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrgeojsonutils.cpp 27613 2014-08-30 15:55:30Z rouault $
+ * $Id: ogrgeojsonutils.cpp 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implementation of private utilities used within OGR GeoJSON Driver.
@@ -67,7 +67,7 @@ int GeoJSONIsObject( const char* pszText )
         return FALSE;
 
     return ((strstr(pszText, "\"type\"") != NULL && strstr(pszText, "\"coordinates\"") != NULL)
-        || (strstr(pszText, "\"type\"") != NULL && strstr(pszText, "\"Topology\"") != NULL) 
+        || (strstr(pszText, "\"type\"") != NULL && strstr(pszText, "\"Topology\"") != NULL)
         || strstr(pszText, "\"FeatureCollection\"") != NULL
         || strstr(pszText, "\"Feature\"") != NULL
         || (strstr(pszText, "\"geometryType\"") != NULL && strstr(pszText, "\"esriGeometry") != NULL));
@@ -78,84 +78,61 @@ int GeoJSONIsObject( const char* pszText )
 /************************************************************************/
 
 static
-int GeoJSONFileIsObject( const char* pszSource, VSILFILE** pfp ) 
-{ 
-    CPLAssert( NULL != pszSource ); 
- 
-    VSILFILE* fp = NULL; 
-    fp = VSIFOpenL( pszSource, "rb" ); 
-    if( NULL == fp ) 
-    { 
-        return FALSE; 
-    } 
-    
-    // by default read first 6000 bytes 
-    // 6000 was chosen as enough bytes to  
-    // enable all current tests to pass 
-    vsi_l_offset nToReadLen = 6000; 
-    vsi_l_offset nDataLen = 0; 
+bool GeoJSONFileIsObject( GDALOpenInfo* poOpenInfo )
+{
+    // by default read first 6000 bytes
+    // 6000 was chosen as enough bytes to
+    // enable all current tests to pass
 
-    char* pszGeoData = (char*)VSIMalloc((size_t)(nToReadLen + 1)); 
-    if( NULL == pszGeoData ) 
-    { 
-        VSIFCloseL(fp); 
-        return FALSE; 
-    } 
-
-    nDataLen = VSIFReadL( pszGeoData, 1, (size_t)nToReadLen, fp );
-    pszGeoData[nDataLen] = '\0'; 
-    if( nDataLen == 0 ) 
-    { 
-        VSIFCloseL( fp ); 
-        CPLFree( pszGeoData ); 
-        return FALSE; 
-    }
-
-    if( !GeoJSONIsObject(pszGeoData) )
+    if( poOpenInfo->fpL == NULL ||
+        !poOpenInfo->TryToIngest(6000) )
     {
-        CPLFree( pszGeoData ); 
-        VSIFCloseL( fp ); 
-        return FALSE;
+        return false;
     }
 
-    *pfp = fp;
+    if( !GeoJSONIsObject((const char*)poOpenInfo->pabyHeader) )
+    {
+        return false;
+    }
 
-    CPLFree( pszGeoData ); 
-
-    return TRUE; 
-} 
+    return true;
+}
 
 /************************************************************************/
 /*                           GeoJSONGetSourceType()                     */
 /************************************************************************/
 
-GeoJSONSourceType GeoJSONGetSourceType( const char* pszSource, VSILFILE** pfp )
+GeoJSONSourceType GeoJSONGetSourceType( GDALOpenInfo* poOpenInfo )
 {
     GeoJSONSourceType srcType = eGeoJSONSourceUnknown;
 
-    *pfp = NULL;
-
     // NOTE: Sometimes URL ends with .geojson token, for example
     //       http://example/path/2232.geojson
     //       It's important to test beginning of source first.
-    if ( eGeoJSONProtocolUnknown != GeoJSONGetProtocolType( pszSource ) )
+    if ( eGeoJSONProtocolUnknown != GeoJSONGetProtocolType( poOpenInfo->pszFilename ) )
     {
+        if( (strstr(poOpenInfo->pszFilename, "SERVICE=WFS") ||
+             strstr(poOpenInfo->pszFilename, "service=WFS") ||
+             strstr(poOpenInfo->pszFilename, "service=wfs")) &&
+             !strstr(poOpenInfo->pszFilename, "json") )
+            return srcType;
         srcType = eGeoJSONSourceService;
     }
-    else if( EQUAL( CPLGetExtension( pszSource ), "geojson" )
-             || EQUAL( CPLGetExtension( pszSource ), "json" )
-             || EQUAL( CPLGetExtension( pszSource ), "topojson" )
-             || ((EQUALN( pszSource, "/vsigzip/", 9) || EQUALN( pszSource, "/vsizip/", 8)) &&
-                 (strstr( pszSource, ".json") || strstr( pszSource, ".JSON") ||
-                  strstr( pszSource, ".geojson") || strstr( pszSource, ".GEOJSON")) ))
+    else if( EQUAL( CPLGetExtension( poOpenInfo->pszFilename ), "geojson" )
+             || EQUAL( CPLGetExtension( poOpenInfo->pszFilename ), "json" )
+             || EQUAL( CPLGetExtension( poOpenInfo->pszFilename ), "topojson" )
+             || ((STARTS_WITH_CI(poOpenInfo->pszFilename, "/vsigzip/") || STARTS_WITH_CI(poOpenInfo->pszFilename, "/vsizip/")) &&
+                 (strstr( poOpenInfo->pszFilename, ".json") || strstr( poOpenInfo->pszFilename, ".JSON") ||
+                  strstr( poOpenInfo->pszFilename, ".geojson") || strstr( poOpenInfo->pszFilename, ".GEOJSON")) ))
     {
-        srcType = eGeoJSONSourceFile;
+        if( poOpenInfo->fpL != NULL )
+            srcType = eGeoJSONSourceFile;
     }
-    else if( GeoJSONIsObject( pszSource ) )
+    else if( GeoJSONIsObject( poOpenInfo->pszFilename ) )
     {
         srcType = eGeoJSONSourceText;
     }
-    else if( GeoJSONFileIsObject( pszSource, pfp ) )
+    else if( GeoJSONFileIsObject( poOpenInfo ) )
     {
         srcType = eGeoJSONSourceFile;
     }
@@ -171,11 +148,11 @@ GeoJSONProtocolType GeoJSONGetProtocolType( const char* pszSource )
 {
     GeoJSONProtocolType ptclType = eGeoJSONProtocolUnknown;
 
-    if( EQUALN( pszSource, "http:", 5 ) )
+    if( STARTS_WITH_CI(pszSource, "http:") )
         ptclType = eGeoJSONProtocolHTTP;
-    else if( EQUALN( pszSource, "https:", 6 ) )
+    else if( STARTS_WITH_CI(pszSource, "https:") )
         ptclType = eGeoJSONProtocolHTTPS;
-    else if( EQUALN( pszSource, "ftp:", 4 ) )
+    else if( STARTS_WITH_CI(pszSource, "ftp:") )
         ptclType = eGeoJSONProtocolFTP;
 
     return ptclType;
@@ -188,43 +165,40 @@ GeoJSONProtocolType GeoJSONGetProtocolType( const char* pszSource )
 #define MY_INT64_MAX ((((GIntBig)0x7FFFFFFF) << 32) | 0xFFFFFFFF)
 #define MY_INT64_MIN ((((GIntBig)0x80000000) << 32))
 
-OGRFieldType GeoJSONPropertyToFieldType( json_object* poObject )
+OGRFieldType GeoJSONPropertyToFieldType( json_object* poObject,
+                                         OGRFieldSubType& eSubType,
+                                         bool bArrayAsString )
 {
+    eSubType = OFSTNone;
+
     if (poObject == NULL) { return OFTString; }
 
     json_type type = json_object_get_type( poObject );
 
     if( json_type_boolean == type )
+    {
+        eSubType = OFSTBoolean;
         return OFTInteger;
+    }
     else if( json_type_double == type )
         return OFTReal;
     else if( json_type_int == type )
     {
         GIntBig nVal = json_object_get_int64(poObject);
-        if( nVal == MY_INT64_MIN || nVal == MY_INT64_MAX )
+        if( !CPL_INT64_FITS_ON_INT32(nVal) )
         {
-            static int bWarned = FALSE;
-            if( !bWarned )
+            if( nVal == MY_INT64_MIN || nVal == MY_INT64_MAX )
             {
-                bWarned = TRUE;
-                CPLError(CE_Warning, CPLE_AppDefined,
-                         "Integer values ranging out of 64bit integer range "
-                         "have been found. Will be clamped to INT64_MIN/INT64_MAX");
+                static bool bWarned = false;
+                if( !bWarned )
+                {
+                    bWarned = true;
+                    CPLError(CE_Warning, CPLE_AppDefined,
+                             "Integer values probably ranging out of 64bit integer range "
+                             "have been found. Will be clamped to INT64_MIN/INT64_MAX");
+                }
             }
-            return OFTString;
-        }
-        // FIXME when we have 64bit integer
-        if( nVal != (int) nVal )
-        {
-            static int bWarned = FALSE;
-            if( !bWarned )
-            {
-                bWarned = TRUE;
-                CPLDebug("GeoJSON",
-                         "64b-bit integer have been found. Will be reported as "
-                         "strings");
-            }
-            return OFTString;
+            return OFTInteger64;
         }
         else
         {
@@ -235,31 +209,74 @@ OGRFieldType GeoJSONPropertyToFieldType( json_object* poObject )
         return OFTString;
     else if( json_type_array == type )
     {
+        if( bArrayAsString )
+            return OFTString;
         int nSize = json_object_array_length(poObject);
         if (nSize == 0)
             return OFTStringList; /* we don't know, so let's assume it's a string list */
         OGRFieldType eType = OFTIntegerList;
+        int bOnlyBoolean = TRUE;
         for(int i=0;i<nSize;i++)
         {
             json_object* poRow = json_object_array_get_idx(poObject, i);
             if (poRow != NULL)
             {
                 type = json_object_get_type( poRow );
+                bOnlyBoolean &= (type == json_type_boolean);
                 if (type == json_type_string)
                     return OFTStringList;
                 else if (type == json_type_double)
                     eType = OFTRealList;
+                else if (eType == OFTIntegerList &&
+                         type == json_type_int)
+                {
+                    GIntBig nVal = json_object_get_int64(poRow);
+                    if( !CPL_INT64_FITS_ON_INT32(nVal) )
+                        eType = OFTInteger64List;
+                }
                 else if (type != json_type_int &&
                          type != json_type_boolean)
                     return OFTString;
             }
         }
+        if( bOnlyBoolean )
+            eSubType = OFSTBoolean;
         return eType;
     }
     else
         return OFTString; /* null, object */
 }
 
+/************************************************************************/
+/*                        GeoJSONStringPropertyToFieldType()            */
+/************************************************************************/
+
+OGRFieldType GeoJSONStringPropertyToFieldType( json_object* poObject )
+{
+    if (poObject == NULL) { return OFTString; }
+    const char* pszStr = json_object_get_string( poObject );
+
+    OGRField sWrkField;
+    CPLPushErrorHandler(CPLQuietErrorHandler);
+    const bool bSuccess = CPL_TO_BOOL(OGRParseDate( pszStr, &sWrkField, 0 ));
+    CPLPopErrorHandler();
+    CPLErrorReset();
+    if( bSuccess )
+    {
+        const bool bHasDate = strchr( pszStr, '/' ) != NULL ||
+            strchr( pszStr, '-' ) != NULL;
+        const bool  bHasTime = strchr( pszStr, ':' ) != NULL;
+        if( bHasDate && bHasTime )
+            return OFTDateTime;
+        else if( bHasDate )
+            return OFTDate;
+        else
+            return OFTTime;
+        // TODO: What if both are false?
+    }
+    return OFTString;
+}
+
 /************************************************************************/
 /*                           OGRGeoJSONGetGeometryName()                */
 /************************************************************************/
@@ -267,8 +284,8 @@ OGRFieldType GeoJSONPropertyToFieldType( json_object* poObject )
 const char* OGRGeoJSONGetGeometryName( OGRGeometry const* poGeometry )
 {
     CPLAssert( NULL != poGeometry );
-    
-    OGRwkbGeometryType eType = poGeometry->getGeometryType();
+
+    const OGRwkbGeometryType eType = poGeometry->getGeometryType();
 
     if( wkbPoint == eType || wkbPoint25D == eType )
         return "Point";
@@ -284,7 +301,6 @@ const char* OGRGeoJSONGetGeometryName( OGRGeometry const* poGeometry )
         return "MultiPolygon";
     else if( wkbGeometryCollection == eType || wkbGeometryCollection25D == eType )
         return "GeometryCollection";
-    else
-        return "Unknown";
-}
 
+    return "Unknown";
+}
