@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrgeomediaselectlayer.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogrgeomediaselectlayer.cpp 28375 2015-01-30 12:06:11Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRGeomediaSelectLayer class, layer access to the results
@@ -32,7 +32,7 @@
 #include "cpl_conv.h"
 #include "ogr_geomedia.h"
 
-CPL_CVSID("$Id: ogrgeomediaselectlayer.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: ogrgeomediaselectlayer.cpp 28375 2015-01-30 12:06:11Z rouault $");
 
 /************************************************************************/
 /*                        OGRGeomediaSelectLayer()                      */
@@ -134,7 +134,7 @@ void OGRGeomediaSelectLayer::ResetReading()
 /*                             GetFeature()                             */
 /************************************************************************/
 
-OGRFeature *OGRGeomediaSelectLayer::GetFeature( long nFeatureId )
+OGRFeature *OGRGeomediaSelectLayer::GetFeature( GIntBig nFeatureId )
 
 {
     return OGRGeomediaLayer::GetFeature( nFeatureId );
@@ -159,7 +159,7 @@ int OGRGeomediaSelectLayer::TestCapability( const char * pszCap )
 /*      way of counting features matching a spatial query.              */
 /************************************************************************/
 
-int OGRGeomediaSelectLayer::GetFeatureCount( int bForce )
+GIntBig OGRGeomediaSelectLayer::GetFeatureCount( int bForce )
 
 {
     return OGRGeomediaLayer::GetFeatureCount( bForce );
