@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrgftlayer.cpp 27741 2014-09-26 19:20:02Z goatbar $
+ * $Id: ogrgftlayer.cpp 32370 2015-12-20 19:40:13Z rouault $
  *
  * Project:  GFT Translator
  * Purpose:  Implements OGRGFTLayer class.
@@ -30,16 +30,16 @@
 #include "ogr_gft.h"
 #include "cpl_minixml.h"
 
-CPL_CVSID("$Id: ogrgftlayer.cpp 27741 2014-09-26 19:20:02Z goatbar $");
+CPL_CVSID("$Id: ogrgftlayer.cpp 32370 2015-12-20 19:40:13Z rouault $");
 
 /************************************************************************/
 /*                            OGRGFTLayer()                             */
 /************************************************************************/
 
-OGRGFTLayer::OGRGFTLayer(OGRGFTDataSource* poDS)
+OGRGFTLayer::OGRGFTLayer(OGRGFTDataSource* poDSIn)
 
 {
-    this->poDS = poDS;
+    this->poDS = poDSIn;
 
     nNextInSeq = 0;
 
@@ -103,15 +103,15 @@ OGRFeature *OGRGFTLayer::GetNextFeature()
 
     GetLayerDefn();
 
-    while(TRUE)
+    while( true )
     {
         if (nNextInSeq < nOffset ||
-            nNextInSeq >= nOffset + (int)aosRows.size())
+            nNextInSeq >= nOffset + static_cast<int>(aosRows.size()))
         {
             if (bEOF)
                 return NULL;
 
-            nOffset += aosRows.size();
+            nOffset += static_cast<int>(aosRows.size());
             if (!FetchNextRows())
                 return NULL;
         }
@@ -157,11 +157,11 @@ char **OGRGFTCSVSplitLine( const char *pszString, char chDelimiter )
 
         nTokenLen = 0;
 
-        /* Try to find the next delimeter, marking end of token */
+        /* Try to find the next delimiter, marking end of token */
         for( ; *pszString != '\0'; pszString++ )
         {
 
-            /* End if this is a delimeter skip it and break. */
+            /* End if this is a delimiter skip it and break. */
             if( !bInString && *pszString == chDelimiter )
             {
                 pszString++;
@@ -222,10 +222,10 @@ static void ParseLineString(OGRLineString* poLS,
     {
         char** papszTokens = CSLTokenizeString2(papszTuples[iTuple], ",", 0);
         if (CSLCount(papszTokens) == 2)
-            poLS->addPoint(atof(papszTokens[0]), atof(papszTokens[1]));
+            poLS->addPoint(CPLAtof(papszTokens[0]), CPLAtof(papszTokens[1]));
         else if (CSLCount(papszTokens) == 3)
-            poLS->addPoint(atof(papszTokens[0]), atof(papszTokens[1]),
-                            atof(papszTokens[2]));
+            poLS->addPoint(CPLAtof(papszTokens[0]), CPLAtof(papszTokens[1]),
+                            CPLAtof(papszTokens[2]));
         CSLDestroy(papszTokens);
     }
     CSLDestroy(papszTuples);
@@ -244,10 +244,10 @@ static OGRGeometry* ParseKMLGeometry(/* const */ CPLXMLNode* psXML)
         {
             char** papszTokens = CSLTokenizeString2(pszCoordinates, ",", 0);
             if (CSLCount(papszTokens) == 2)
-                poGeom = new OGRPoint(atof(papszTokens[0]), atof(papszTokens[1]));
+                poGeom = new OGRPoint(CPLAtof(papszTokens[0]), CPLAtof(papszTokens[1]));
             else if (CSLCount(papszTokens) == 3)
-                poGeom = new OGRPoint(atof(papszTokens[0]), atof(papszTokens[1]),
-                                      atof(papszTokens[2]));
+                poGeom = new OGRPoint(CPLAtof(papszTokens[0]), CPLAtof(papszTokens[1]),
+                                      CPLAtof(papszTokens[2]));
             CSLDestroy(papszTokens);
         }
     }
@@ -287,8 +287,8 @@ static OGRGeometry* ParseKMLGeometry(/* const */ CPLXMLNode* psXML)
                     if (psIter->eType == CXT_Element &&
                         strcmp(psIter->pszValue, "innerBoundaryIs") == 0)
                     {
-                        CPLXMLNode* psLinearRing = CPLGetXMLNode(psIter, "LinearRing");
-                        const char* pszCoordinates = CPLGetXMLValue(
+                        psLinearRing = CPLGetXMLNode(psIter, "LinearRing");
+                        pszCoordinates = CPLGetXMLValue(
                             psLinearRing ? psLinearRing : psIter, "coordinates", NULL);
                         if (pszCoordinates)
                         {
@@ -344,7 +344,6 @@ static OGRGeometry* ParseKMLGeometry(/* const */ CPLXMLNode* psXML)
             CPLAssert(0);
         }
 
-        psIter = psXML->psChild;
         for(psIter = psXML->psChild; psIter; psIter = psIter->psNext)
         {
             if (psIter->eType == CXT_Element)
@@ -426,8 +425,8 @@ OGRFeature *OGRGFTLayer::BuildFeatureFromSQL(const char* pszLine)
                             CPLGetValueType(papszLatLon[0]) != CPL_VALUE_STRING &&
                             CPLGetValueType(papszLatLon[1]) != CPL_VALUE_STRING)
                         {
-                            OGRPoint* poPoint = new OGRPoint(atof( papszLatLon[1]),
-                                                            atof( papszLatLon[0]));
+                            OGRPoint* poPoint = new OGRPoint(CPLAtof( papszLatLon[1]),
+                                                            CPLAtof( papszLatLon[0]));
                             poPoint->assignSpatialReference(poSRS);
                             poFeature->SetGeometryDirectly(poPoint);
                         }
@@ -460,7 +459,7 @@ OGRFeature *OGRGFTLayer::BuildFeatureFromSQL(const char* pszLine)
                 CPLGetValueType(pszLat) != CPL_VALUE_STRING &&
                 CPLGetValueType(pszLong) != CPL_VALUE_STRING)
             {
-                OGRPoint* poPoint = new OGRPoint(atof(pszLong), atof(pszLat));
+                OGRPoint* poPoint = new OGRPoint(CPLAtof(pszLong), CPLAtof(pszLat));
                 poPoint->assignSpatialReference(poSRS);
                 poFeature->SetGeometryDirectly(poPoint);
             }
@@ -502,12 +501,12 @@ OGRFeature *OGRGFTLayer::GetNextRawFeature()
 /*                          SetNextByIndex()                            */
 /************************************************************************/
 
-OGRErr OGRGFTLayer::SetNextByIndex( long nIndex )
+OGRErr OGRGFTLayer::SetNextByIndex( GIntBig nIndex )
 {
-    if (nIndex < 0)
+    if (nIndex < 0 || nIndex >= INT_MAX )
         return OGRERR_FAILURE;
     bEOF = FALSE;
-    nNextInSeq = nIndex;
+    nNextInSeq = (int)nIndex;
     return OGRERR_NONE;
 }
 
@@ -608,7 +607,7 @@ CPLString OGRGFTLayer::PatchSQL(const char* pszSQL)
 
     while(*pszSQL)
     {
-        if (EQUALN(pszSQL, "COUNT(", 5) && strchr(pszSQL, ')'))
+        if (STARTS_WITH_CI(pszSQL, "COUNT(") && strchr(pszSQL, ')'))
         {
             const char* pszNext = strchr(pszSQL, ')');
             osSQL += "COUNT()";
