@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: gmlfeatureclass.cpp 27132 2014-04-05 21:48:58Z rouault $
+ * $Id: gmlfeatureclass.cpp 33702 2016-03-11 06:20:16Z goatbar $
  *
  * Project:  GML Reader
  * Purpose:  Implementation of GMLFeatureClass.
@@ -15,47 +15,52 @@
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
 #include "gmlreader.h"
 #include "cpl_conv.h"
+#include "cpl_string.h"
 #include "ogr_core.h"
 #include "ogr_geometry.h"
+#include "cpl_string.h"
 
 /************************************************************************/
 /*                          GMLFeatureClass()                           */
 /************************************************************************/
 
-GMLFeatureClass::GMLFeatureClass( const char *pszName )
-
+GMLFeatureClass::GMLFeatureClass( const char *pszName ) :
+    m_pszName(NULL),
+    m_pszElementName(NULL),
+    n_nNameLen(0),
+    n_nElementNameLen(0),
+    m_nPropertyCount(0),
+    m_papoProperty(NULL),
+    m_nGeometryPropertyCount(0),
+    m_papoGeometryProperty(NULL),
+    m_bSchemaLocked(false),
+    m_nFeatureCount(-1), // unknown
+    m_pszExtraInfo(NULL),
+    m_bHaveExtents(false),
+    m_dfXMin(0.0),
+    m_dfXMax(0.0),
+    m_dfYMin(0.0),
+    m_dfYMax(0.0),
+    m_pszSRSName(NULL),
+    m_bSRSNameConsistent(true)
 {
     m_pszName = CPLStrdup( pszName );
-    n_nNameLen = strlen( m_pszName );
-    m_pszElementName = NULL;
-    n_nElementNameLen = 0;
-    m_nPropertyCount = 0;
-    m_papoProperty = NULL;
-    m_nGeometryPropertyCount = 0;
-    m_papoGeometryProperty = NULL;
-    m_bSchemaLocked = FALSE;
-
-    m_pszExtraInfo = NULL;
-    m_bHaveExtents = FALSE;
-    m_nFeatureCount = -1; // unknown
-
-    m_pszSRSName = NULL;
-    m_bSRSNameConsistant = TRUE;
+    n_nNameLen = static_cast<int>(strlen( m_pszName ));
 }
 
 /************************************************************************/
@@ -77,6 +82,28 @@ GMLFeatureClass::~GMLFeatureClass()
     CPLFree( m_pszSRSName );
 }
 
+/************************************************************************/
+/*                         StealProperties()                            */
+/************************************************************************/
+
+void GMLFeatureClass::StealProperties()
+{
+    m_nPropertyCount = 0;
+    CPLFree( m_papoProperty );
+    m_papoProperty = NULL;
+}
+
+/************************************************************************/
+/*                       StealGeometryProperties()                      */
+/************************************************************************/
+
+void GMLFeatureClass::StealGeometryProperties()
+{
+    m_nGeometryPropertyCount = 0;
+    CPLFree( m_papoGeometryProperty );
+    m_papoGeometryProperty = NULL;
+}
+
 /************************************************************************/
 /*                            SetName()                                 */
 /************************************************************************/
@@ -220,15 +247,15 @@ void GMLFeatureClass::ClearGeometryProperties()
 /*                         HasFeatureProperties()                       */
 /************************************************************************/
 
-int GMLFeatureClass::HasFeatureProperties()
+bool GMLFeatureClass::HasFeatureProperties()
 {
     for( int i = 0; i < m_nPropertyCount; i++ )
     {
         if( m_papoProperty[i]->GetType() == GMLPT_FeatureProperty ||
             m_papoProperty[i]->GetType() == GMLPT_FeaturePropertyList )
-            return TRUE;
+            return true;
     }
-    return FALSE;
+    return false;
 }
 
 /************************************************************************/
@@ -240,7 +267,7 @@ void GMLFeatureClass::SetElementName( const char *pszElementName )
 {
     CPLFree( m_pszElementName );
     m_pszElementName = CPLStrdup( pszElementName );
-    n_nElementNameLen = strlen(pszElementName);
+    n_nElementNameLen = static_cast<int>(strlen(pszElementName));
 }
 
 /************************************************************************/
@@ -270,10 +297,10 @@ size_t GMLFeatureClass::GetElementNameLen() const
 }
 
 /************************************************************************/
-/*                          GetFeatureCount()                           */
+/*                         GetFeatureCount()                          */
 /************************************************************************/
 
-int GMLFeatureClass::GetFeatureCount()
+GIntBig GMLFeatureClass::GetFeatureCount()
 
 {
     return m_nFeatureCount;
@@ -283,7 +310,7 @@ int GMLFeatureClass::GetFeatureCount()
 /*                          SetFeatureCount()                           */
 /************************************************************************/
 
-void GMLFeatureClass::SetFeatureCount( int nNewCount )
+void GMLFeatureClass::SetFeatureCount( GIntBig nNewCount )
 
 {
     m_nFeatureCount = nNewCount;
@@ -317,7 +344,7 @@ void GMLFeatureClass::SetExtraInfo( const char *pszExtraInfo )
 /*                             SetExtents()                             */
 /************************************************************************/
 
-void GMLFeatureClass::SetExtents( double dfXMin, double dfXMax, 
+void GMLFeatureClass::SetExtents( double dfXMin, double dfXMax,
                                   double dfYMin, double dfYMax )
 
 {
@@ -326,14 +353,14 @@ void GMLFeatureClass::SetExtents( double dfXMin, double dfXMax,
     m_dfYMin = dfYMin;
     m_dfYMax = dfYMax;
 
-    m_bHaveExtents = TRUE;
+    m_bHaveExtents = true;
 }
 
 /************************************************************************/
 /*                             GetExtents()                             */
 /************************************************************************/
 
-int GMLFeatureClass::GetExtents( double *pdfXMin, double *pdfXMax, 
+bool GMLFeatureClass::GetExtents( double *pdfXMin, double *pdfXMax,
                                  double *pdfYMin, double *pdfYMax )
 
 {
@@ -355,7 +382,7 @@ int GMLFeatureClass::GetExtents( double *pdfXMin, double *pdfXMax,
 void GMLFeatureClass::SetSRSName( const char* pszSRSName )
 
 {
-    m_bSRSNameConsistant = TRUE;
+    m_bSRSNameConsistent = true;
     CPLFree(m_pszSRSName);
     m_pszSRSName = (pszSRSName) ? CPLStrdup(pszSRSName) : NULL;
 }
@@ -367,7 +394,7 @@ void GMLFeatureClass::SetSRSName( const char* pszSRSName )
 void GMLFeatureClass::MergeSRSName( const char* pszSRSName )
 
 {
-    if(!m_bSRSNameConsistant)
+    if(!m_bSRSNameConsistent)
         return;
 
     if( m_pszSRSName == NULL )
@@ -377,9 +404,9 @@ void GMLFeatureClass::MergeSRSName( const char* pszSRSName )
     }
     else
     {
-        m_bSRSNameConsistant = pszSRSName != NULL &&
+        m_bSRSNameConsistent = pszSRSName != NULL &&
                                   strcmp(m_pszSRSName, pszSRSName) == 0;
-        if (!m_bSRSNameConsistant)
+        if (!m_bSRSNameConsistent)
         {
             CPLFree(m_pszSRSName);
             m_pszSRSName = NULL;
@@ -391,28 +418,28 @@ void GMLFeatureClass::MergeSRSName( const char* pszSRSName )
 /*                         InitializeFromXML()                          */
 /************************************************************************/
 
-int GMLFeatureClass::InitializeFromXML( CPLXMLNode *psRoot )
+bool GMLFeatureClass::InitializeFromXML( CPLXMLNode *psRoot )
 
 {
 /* -------------------------------------------------------------------- */
 /*      Do some rudimentary checking that this is a well formed         */
 /*      node.                                                           */
 /* -------------------------------------------------------------------- */
-    if( psRoot == NULL 
-        || psRoot->eType != CXT_Element 
+    if( psRoot == NULL
+        || psRoot->eType != CXT_Element
         || !EQUAL(psRoot->pszValue,"GMLFeatureClass") )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "GMLFeatureClass::InitializeFromXML() called on %s node!",
                   psRoot->pszValue );
-        return FALSE;
+        return false;
     }
 
     if( CPLGetXMLValue( psRoot, "Name", NULL ) == NULL )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "GMLFeatureClass has no <Name> element." );
-        return FALSE;
+        return false;
     }
 
 /* -------------------------------------------------------------------- */
@@ -420,7 +447,7 @@ int GMLFeatureClass::InitializeFromXML( CPLXMLNode *psRoot )
 /* -------------------------------------------------------------------- */
     CPLFree( m_pszName );
     m_pszName = CPLStrdup( CPLGetXMLValue( psRoot, "Name", NULL ) );
-    n_nNameLen = strlen(m_pszName);
+    n_nNameLen = static_cast<int>(strlen(m_pszName));
 
     SetElementName( CPLGetXMLValue( psRoot, "ElementPath", m_pszName ) );
 
@@ -429,10 +456,10 @@ int GMLFeatureClass::InitializeFromXML( CPLXMLNode *psRoot )
 /* -------------------------------------------------------------------- */
     CPLXMLNode *psThis;
 
-    int bHasValidGeometryName = FALSE;
-    int bHasValidGeometryElementPath = FALSE;
-    int bHasFoundGeomType = FALSE;
-    int bHasFoundGeomElements = FALSE;
+    bool bHasValidGeometryName = false;
+    bool bHasValidGeometryElementPath = false;
+    bool bHasFoundGeomType = false;
+    bool bHasFoundGeomElements = false;
     const char* pszGName = "";
     const char* pszGPath = "";
     int nGeomType = wkbUnknown;
@@ -445,99 +472,99 @@ int GMLFeatureClass::InitializeFromXML( CPLXMLNode *psRoot )
             const char *pszName = CPLGetXMLValue( psThis, "Name", "" );
             const char *pszElementPath = CPLGetXMLValue( psThis, "ElementPath", "" );
             const char *pszType = CPLGetXMLValue( psThis, "Type", NULL );
+            bool bNullable = CPLTestBool(CPLGetXMLValue( psThis, "Nullable", "true") );
             nGeomType = wkbUnknown;
             if( pszType != NULL && !EQUAL(pszType, "0") )
             {
                 nGeomType = atoi(pszType);
-                int nFlattenGeomType = nGeomType & (~wkb25DBit);
-                if( nGeomType != 0 && !(nFlattenGeomType >= 0 && nFlattenGeomType <= 7) )
+                OGRwkbGeometryType nFlattenGeomType = wkbFlatten(nGeomType);
+                if( nGeomType != 0 && !(nFlattenGeomType >= wkbPoint && nFlattenGeomType <= wkbMultiSurface) )
                 {
                     nGeomType = wkbUnknown;
-                    CPLError(CE_Warning, CPLE_AppDefined, "Unrecognised geometry type : %s",
-                            pszType);
+                    CPLError( CE_Warning, CPLE_AppDefined,
+                              "Unrecognized geometry type : %s",
+                              pszType);
                 }
                 else if( nGeomType == 0 )
                     nGeomType = OGRFromOGCGeomType(pszType);
             }
-            bHasFoundGeomElements = TRUE;
-            AddGeometryProperty( new GMLGeometryPropertyDefn( pszName, pszElementPath, nGeomType ) );
-            bHasValidGeometryName = FALSE;
-            bHasValidGeometryElementPath = FALSE;
-            bHasFoundGeomType = FALSE;
+            bHasFoundGeomElements = true;
+            AddGeometryProperty( new GMLGeometryPropertyDefn( pszName, pszElementPath, nGeomType, -1, bNullable ) );
+            bHasValidGeometryName = false;
+            bHasValidGeometryElementPath = false;
+            bHasFoundGeomType = false;
         }
         else if( psThis->eType == CXT_Element &&
             strcmp(psThis->pszValue, "GeometryName") == 0 )
         {
-            bHasFoundGeomElements = TRUE;
+            bHasFoundGeomElements = true;
 
             if( bHasValidGeometryName )
             {
-                AddGeometryProperty( new GMLGeometryPropertyDefn( pszGName, pszGPath, nGeomType ) );
-                bHasValidGeometryName = FALSE;
-                bHasValidGeometryElementPath = FALSE;
-                bHasFoundGeomType = FALSE;
-                pszGName = "";
+                AddGeometryProperty( new GMLGeometryPropertyDefn( pszGName, pszGPath, nGeomType, -1, true ) );
+                /*bHasValidGeometryName = false;*/
+                bHasValidGeometryElementPath = false;
+                bHasFoundGeomType = false;
                 pszGPath = "";
                 nGeomType = wkbUnknown;
             }
             pszGName = CPLGetXMLValue( psThis, NULL, "" );
-            bHasValidGeometryName = TRUE;
+            bHasValidGeometryName = true;
         }
         else if( psThis->eType == CXT_Element &&
             strcmp(psThis->pszValue, "GeometryElementPath") == 0 )
         {
-            bHasFoundGeomElements = TRUE;
+            bHasFoundGeomElements = true;
 
             if( bHasValidGeometryElementPath )
             {
-                AddGeometryProperty( new GMLGeometryPropertyDefn( pszGName, pszGPath, nGeomType ) );
-                bHasValidGeometryName = FALSE;
-                bHasValidGeometryElementPath = FALSE;
-                bHasFoundGeomType = FALSE;
+                AddGeometryProperty( new GMLGeometryPropertyDefn( pszGName, pszGPath, nGeomType, -1, true ) );
+                bHasValidGeometryName = false;
+                /*bHasValidGeometryElementPath = false;*/
+                bHasFoundGeomType = false;
                 pszGName = "";
-                pszGPath = "";
                 nGeomType = wkbUnknown;
             }
             pszGPath = CPLGetXMLValue( psThis, NULL, "" );
-            bHasValidGeometryElementPath = TRUE;
+            bHasValidGeometryElementPath = true;
         }
         else if( psThis->eType == CXT_Element &&
                  strcmp(psThis->pszValue, "GeometryType") == 0 )
         {
-            bHasFoundGeomElements = TRUE;
+            bHasFoundGeomElements = true;
 
             if( bHasFoundGeomType )
             {
-                AddGeometryProperty( new GMLGeometryPropertyDefn( pszGName, pszGPath, nGeomType ) );
-                bHasValidGeometryName = FALSE;
-                bHasValidGeometryElementPath = FALSE;
-                bHasFoundGeomType = FALSE;
+                AddGeometryProperty( new GMLGeometryPropertyDefn( pszGName, pszGPath, nGeomType, -1, true ) );
+                bHasValidGeometryName = false;
+                bHasValidGeometryElementPath = false;
+                /*bHasFoundGeomType = false;*/
                 pszGName = "";
                 pszGPath = "";
-                nGeomType = wkbUnknown;
             }
             const char* pszGeometryType = CPLGetXMLValue( psThis, NULL, NULL );
             nGeomType = wkbUnknown;
             if( pszGeometryType != NULL && !EQUAL(pszGeometryType, "0") )
             {
                 nGeomType = atoi(pszGeometryType);
-                int nFlattenGeomType = nGeomType & (~wkb25DBit);
+                OGRwkbGeometryType nFlattenGeomType = wkbFlatten(nGeomType);
                 if( nGeomType == 100 || EQUAL(pszGeometryType, "NONE") )
                 {
-                    bHasValidGeometryElementPath = FALSE;
-                    bHasFoundGeomType = FALSE;
+                    bHasValidGeometryElementPath = false;
+                    bHasFoundGeomType = false;
                     break;
                 }
-                else if( nGeomType != 0 && !(nFlattenGeomType >= 0 && nFlattenGeomType <= 7) )
+                else if( nGeomType != 0 && !(nFlattenGeomType >= wkbPoint && nFlattenGeomType <= wkbMultiSurface) )
                 {
                     nGeomType = wkbUnknown;
-                    CPLError(CE_Warning, CPLE_AppDefined, "Unrecognised geometry type : %s",
-                            pszGeometryType);
+                    CPLError( CE_Warning, CPLE_AppDefined,
+                              "Unrecognized geometry type : %s",
+                              pszGeometryType);
                 }
                 else if( nGeomType == 0 )
                     nGeomType = OGRFromOGCGeomType(pszGeometryType);
             }
-            bHasFoundGeomType = TRUE;
+            bHasFoundGeomType = true;
         }
     }
 
@@ -546,7 +573,7 @@ int GMLFeatureClass::InitializeFromXML( CPLXMLNode *psRoot )
     /* a geometry field */
     if( bHasValidGeometryElementPath || bHasFoundGeomType || !bHasFoundGeomElements )
     {
-        AddGeometryProperty( new GMLGeometryPropertyDefn( pszGName, pszGPath, nGeomType ) );
+        AddGeometryProperty( new GMLGeometryPropertyDefn( pszGName, pszGPath, nGeomType, -1, true ) );
     }
 
     SetSRSName( CPLGetXMLValue( psRoot, "SRSName", NULL ) );
@@ -561,14 +588,14 @@ int GMLFeatureClass::InitializeFromXML( CPLXMLNode *psRoot )
 
         pszValue = CPLGetXMLValue( psDSI, "FeatureCount", NULL );
         if( pszValue != NULL )
-            SetFeatureCount( atoi(pszValue) );
+            SetFeatureCount( CPLAtoGIntBig(pszValue) );
 
         // Eventually we should support XML subtrees.
         pszValue = CPLGetXMLValue( psDSI, "ExtraInfo", NULL );
         if( pszValue != NULL )
             SetExtraInfo( pszValue );
 
-        if( CPLGetXMLValue( psDSI, "ExtentXMin", NULL ) != NULL 
+        if( CPLGetXMLValue( psDSI, "ExtentXMin", NULL ) != NULL
             && CPLGetXMLValue( psDSI, "ExtentXMax", NULL ) != NULL
             && CPLGetXMLValue( psDSI, "ExtentYMin", NULL ) != NULL
             && CPLGetXMLValue( psDSI, "ExtentYMax", NULL ) != NULL )
@@ -579,7 +606,7 @@ int GMLFeatureClass::InitializeFromXML( CPLXMLNode *psRoot )
                         CPLAtof(CPLGetXMLValue( psDSI, "ExtentYMax", "0.0" )) );
         }
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Collect property definitions.                                   */
 /* -------------------------------------------------------------------- */
@@ -590,42 +617,81 @@ int GMLFeatureClass::InitializeFromXML( CPLXMLNode *psRoot )
         {
             const char *pszName = CPLGetXMLValue( psThis, "Name", NULL );
             const char *pszType = CPLGetXMLValue( psThis, "Type", "Untyped" );
+            const char *pszSubType = CPLGetXMLValue( psThis, "Subtype", "" );
             const char *pszCondition = CPLGetXMLValue( psThis, "Condition", NULL );
+            bool bNullable = CPLTestBool(CPLGetXMLValue( psThis, "Nullable", "true") );
             GMLPropertyDefn *poPDefn;
 
             if( pszName == NULL )
             {
-                CPLError( CE_Failure, CPLE_AppDefined, 
+                CPLError( CE_Failure, CPLE_AppDefined,
                           "GMLFeatureClass %s has a PropertyDefn without a <Name>..",
                           m_pszName );
-                return FALSE;
+                return false;
             }
 
-            poPDefn = new GMLPropertyDefn( 
+            poPDefn = new GMLPropertyDefn(
                 pszName, CPLGetXMLValue( psThis, "ElementPath", NULL ) );
-            
+
+            poPDefn->SetNullable(bNullable);
             if( EQUAL(pszType,"Untyped") )
                 poPDefn->SetType( GMLPT_Untyped );
-            else if( EQUAL(pszType,"String") ) 
+            else if( EQUAL(pszType,"String") )
             {
-                poPDefn->SetType( GMLPT_String );
-                poPDefn->SetWidth( atoi( CPLGetXMLValue( psThis, "Width", "0" ) ) );
+                if( EQUAL(pszSubType, "Boolean") )
+                {
+                    poPDefn->SetType( GMLPT_Boolean );
+                    poPDefn->SetWidth( 1 );
+                }
+                else
+                {
+                    poPDefn->SetType( GMLPT_String );
+                    poPDefn->SetWidth( atoi( CPLGetXMLValue( psThis, "Width", "0" ) ) );
+                }
             }
             else if( EQUAL(pszType,"Integer") )
             {
-                poPDefn->SetType( GMLPT_Integer );
+                if( EQUAL(pszSubType, "Short") )
+                {
+                    poPDefn->SetType( GMLPT_Short );
+                }
+                else if( EQUAL(pszSubType, "Integer64") )
+                {
+                    poPDefn->SetType( GMLPT_Integer64 );
+                }
+                else
+                {
+                    poPDefn->SetType( GMLPT_Integer );
+                }
                 poPDefn->SetWidth( atoi( CPLGetXMLValue( psThis, "Width", "0" ) ) );
             }
             else if( EQUAL(pszType,"Real") )
             {
-                poPDefn->SetType( GMLPT_Real );
+                if( EQUAL(pszSubType, "Float") )
+                {
+                    poPDefn->SetType( GMLPT_Float );
+                }
+                else
+                {
+                    poPDefn->SetType( GMLPT_Real );
+                }
                 poPDefn->SetWidth( atoi( CPLGetXMLValue( psThis, "Width", "0" ) ) );
                 poPDefn->SetPrecision( atoi( CPLGetXMLValue( psThis, "Precision", "0" ) ) );
             }
-            else if( EQUAL(pszType,"StringList") ) 
-                poPDefn->SetType( GMLPT_StringList );
+            else if( EQUAL(pszType,"StringList") )
+            {
+                if( EQUAL(pszSubType, "Boolean") )
+                    poPDefn->SetType( GMLPT_BooleanList );
+                else
+                    poPDefn->SetType( GMLPT_StringList );
+            }
             else if( EQUAL(pszType,"IntegerList") )
-                poPDefn->SetType( GMLPT_IntegerList );
+            {
+                if( EQUAL(pszSubType, "Integer64") )
+                    poPDefn->SetType( GMLPT_Integer64List );
+                else
+                    poPDefn->SetType( GMLPT_IntegerList );
+            }
             else if( EQUAL(pszType,"RealList") )
                 poPDefn->SetType( GMLPT_RealList );
             else if( EQUAL(pszType,"Complex") )
@@ -636,20 +702,21 @@ int GMLFeatureClass::InitializeFromXML( CPLXMLNode *psRoot )
                 poPDefn->SetType( GMLPT_FeaturePropertyList );
             else
             {
-                CPLError( CE_Failure, CPLE_AppDefined, 
-                          "Unrecognised property type %s.", 
+                CPLError( CE_Failure, CPLE_AppDefined,
+                          "Unrecognized property type %s.",
                           pszType );
                 delete poPDefn;
-                return FALSE;
+                return false;
             }
             if( pszCondition != NULL )
                 poPDefn->SetCondition(pszCondition);
 
-            AddProperty( poPDefn );
+            if( AddProperty( poPDefn ) < 0 )
+                delete poPDefn;
         }
     }
 
-    return TRUE;
+    return true;
 }
 
 /************************************************************************/
@@ -669,24 +736,60 @@ CPLXMLNode *GMLFeatureClass::SerializeToXML()
 
     CPLCreateXMLElementAndValue( psRoot, "Name", GetName() );
     CPLCreateXMLElementAndValue( psRoot, "ElementPath", GetElementName() );
-    
-    if( m_nGeometryPropertyCount > 0 )
+
+    if( m_nGeometryPropertyCount > 1 )
+    {
+        for(int i=0; i < m_nGeometryPropertyCount; i++)
+        {
+            GMLGeometryPropertyDefn* poGeomFDefn = m_papoGeometryProperty[i];
+
+            CPLXMLNode *psPDefnNode;
+            psPDefnNode = CPLCreateXMLNode( psRoot, CXT_Element, "GeomPropertyDefn" );
+            if( strlen(poGeomFDefn->GetName()) > 0 )
+                CPLCreateXMLElementAndValue( psPDefnNode, "Name",
+                                             poGeomFDefn->GetName() );
+            if( poGeomFDefn->GetSrcElement() != NULL && strlen(poGeomFDefn->GetSrcElement()) > 0 )
+                CPLCreateXMLElementAndValue( psPDefnNode, "ElementPath",
+                                             poGeomFDefn->GetSrcElement() );
+
+            if( poGeomFDefn->GetType() != 0 /* wkbUnknown */ )
+            {
+                char szValue[128];
+
+                OGRwkbGeometryType eType = (OGRwkbGeometryType)poGeomFDefn->GetType();
+
+                CPLString osStr(OGRToOGCGeomType(eType));
+                if( wkbHasZ(eType) ) osStr += "Z";
+                CPLCreateXMLNode( psPDefnNode, CXT_Comment, osStr.c_str() );
+
+                snprintf( szValue, sizeof(szValue), "%d", eType );
+                CPLCreateXMLElementAndValue( psPDefnNode, "Type", szValue );
+            }
+        }
+    }
+    else if( m_nGeometryPropertyCount == 1 )
     {
         GMLGeometryPropertyDefn* poGeomFDefn = m_papoGeometryProperty[0];
-        
+
         if( strlen(poGeomFDefn->GetName()) > 0 )
-            CPLCreateXMLElementAndValue( psRoot, "GeometryName", 
+            CPLCreateXMLElementAndValue( psRoot, "GeometryName",
                                          poGeomFDefn->GetName() );
 
         if( poGeomFDefn->GetSrcElement() != NULL && strlen(poGeomFDefn->GetSrcElement()) > 0 )
-            CPLCreateXMLElementAndValue( psRoot, "GeometryElementPath", 
+            CPLCreateXMLElementAndValue( psRoot, "GeometryElementPath",
                                          poGeomFDefn->GetSrcElement() );
-        
+
         if( poGeomFDefn->GetType() != 0 /* wkbUnknown */ )
         {
             char szValue[128];
 
-            sprintf( szValue, "%d", poGeomFDefn->GetType() );
+            OGRwkbGeometryType eType = (OGRwkbGeometryType)poGeomFDefn->GetType();
+
+            CPLString osStr(OGRToOGCGeomType(eType));
+            if( wkbHasZ(eType) ) osStr += "Z";
+            CPLCreateXMLNode( psRoot, CXT_Comment, osStr.c_str() );
+
+            snprintf( szValue, sizeof(szValue), "%d", eType );
             CPLCreateXMLElementAndValue( psRoot, "GeometryType", szValue );
         }
     }
@@ -714,7 +817,7 @@ CPLXMLNode *GMLFeatureClass::SerializeToXML()
         {
             char szValue[128];
 
-            sprintf( szValue, "%d", m_nFeatureCount );
+            snprintf( szValue, sizeof(szValue), CPL_FRMT_GIB, m_nFeatureCount );
             CPLCreateXMLElementAndValue( psDSI, "FeatureCount", szValue );
         }
 
@@ -726,23 +829,23 @@ CPLXMLNode *GMLFeatureClass::SerializeToXML()
         {
             char szValue[128];
 
-            snprintf( szValue, sizeof(szValue), "%.5f", m_dfXMin );
+            CPLsnprintf( szValue, sizeof(szValue), "%.5f", m_dfXMin );
             CPLCreateXMLElementAndValue( psDSI, "ExtentXMin", szValue );
 
-            snprintf( szValue, sizeof(szValue), "%.5f", m_dfXMax );
+            CPLsnprintf( szValue, sizeof(szValue), "%.5f", m_dfXMax );
             CPLCreateXMLElementAndValue( psDSI, "ExtentXMax", szValue );
 
-            snprintf( szValue, sizeof(szValue), "%.5f", m_dfYMin );
+            CPLsnprintf( szValue, sizeof(szValue), "%.5f", m_dfYMin );
             CPLCreateXMLElementAndValue( psDSI, "ExtentYMin", szValue );
 
-            snprintf( szValue, sizeof(szValue), "%.5f", m_dfYMax );
+            CPLsnprintf( szValue, sizeof(szValue), "%.5f", m_dfYMax );
             CPLCreateXMLElementAndValue( psDSI, "ExtentYMax", szValue );
         }
 
         if( m_pszExtraInfo )
             CPLCreateXMLElementAndValue( psDSI, "ExtraInfo", m_pszExtraInfo );
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      emit property information.                                      */
 /* -------------------------------------------------------------------- */
@@ -753,33 +856,38 @@ CPLXMLNode *GMLFeatureClass::SerializeToXML()
         const char *pszTypeName = "Unknown";
 
         psPDefnNode = CPLCreateXMLNode( psRoot, CXT_Element, "PropertyDefn" );
-        CPLCreateXMLElementAndValue( psPDefnNode, "Name", 
+        CPLCreateXMLElementAndValue( psPDefnNode, "Name",
                                      poPDefn->GetName() );
-        CPLCreateXMLElementAndValue( psPDefnNode, "ElementPath", 
+        CPLCreateXMLElementAndValue( psPDefnNode, "ElementPath",
                                      poPDefn->GetSrcElement() );
         switch( poPDefn->GetType() )
         {
           case GMLPT_Untyped:
             pszTypeName = "Untyped";
             break;
-            
+
           case GMLPT_String:
+          case GMLPT_Boolean:
             pszTypeName = "String";
             break;
-            
+
           case GMLPT_Integer:
+          case GMLPT_Short:
+          case GMLPT_Integer64:
             pszTypeName = "Integer";
             break;
-            
+
           case GMLPT_Real:
+          case GMLPT_Float:
             pszTypeName = "Real";
             break;
-            
+
           case GMLPT_Complex:
             pszTypeName = "Complex";
             break;
 
           case GMLPT_IntegerList:
+          case GMLPT_Integer64List:
             pszTypeName = "IntegerList";
             break;
 
@@ -788,46 +896,55 @@ CPLXMLNode *GMLFeatureClass::SerializeToXML()
             break;
 
           case GMLPT_StringList:
+          case GMLPT_BooleanList:
             pszTypeName = "StringList";
             break;
 
-          /* should not happen in practise for now because this is not */
+          /* should not happen in practice for now because this is not */
           /* autodetected */
           case GMLPT_FeatureProperty:
             pszTypeName = "FeatureProperty";
             break;
 
-          /* should not happen in practise for now because this is not */
+          /* should not happen in practice for now because this is not */
           /* autodetected */
           case GMLPT_FeaturePropertyList:
             pszTypeName = "FeaturePropertyList";
             break;
         }
         CPLCreateXMLElementAndValue( psPDefnNode, "Type", pszTypeName );
+        if( poPDefn->GetType() == GMLPT_Boolean || poPDefn->GetType() == GMLPT_BooleanList )
+            CPLCreateXMLElementAndValue( psPDefnNode, "Subtype", "Boolean" );
+        else if( poPDefn->GetType() == GMLPT_Short )
+            CPLCreateXMLElementAndValue( psPDefnNode, "Subtype", "Short" );
+        else if( poPDefn->GetType() == GMLPT_Float )
+            CPLCreateXMLElementAndValue( psPDefnNode, "Subtype", "Float" );
+        else if( poPDefn->GetType() == GMLPT_Integer64 ||
+                 poPDefn->GetType() == GMLPT_Integer64List )
+            CPLCreateXMLElementAndValue( psPDefnNode, "Subtype", "Integer64" );
 
         if( EQUAL(pszTypeName,"String") )
         {
             char szMaxLength[48];
-            sprintf(szMaxLength, "%d", poPDefn->GetWidth());
+            snprintf(szMaxLength, sizeof(szMaxLength), "%d", poPDefn->GetWidth());
             CPLCreateXMLElementAndValue ( psPDefnNode, "Width", szMaxLength );
         }
         if( poPDefn->GetWidth() > 0 && EQUAL(pszTypeName,"Integer") )
         {
             char szLength[48];
-            sprintf(szLength, "%d", poPDefn->GetWidth());
+            snprintf(szLength, sizeof(szLength), "%d", poPDefn->GetWidth());
             CPLCreateXMLElementAndValue ( psPDefnNode, "Width", szLength );
         }
         if( poPDefn->GetWidth() > 0 && EQUAL(pszTypeName,"Real") )
         {
             char szLength[48];
-            sprintf(szLength, "%d", poPDefn->GetWidth());
+            snprintf(szLength, sizeof(szLength), "%d", poPDefn->GetWidth());
             CPLCreateXMLElementAndValue ( psPDefnNode, "Width", szLength );
             char szPrecision[48];
-            sprintf(szPrecision, "%d", poPDefn->GetPrecision());
+            snprintf(szPrecision, sizeof(szPrecision), "%d", poPDefn->GetPrecision());
             CPLCreateXMLElementAndValue ( psPDefnNode, "Precision", szPrecision );
         }
     }
 
     return psRoot;
 }
-
