@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: gmlpropertydefn.cpp 27132 2014-04-05 21:48:58Z rouault $
+ * $Id: gmlpropertydefn.cpp 33702 2016-03-11 06:20:16Z goatbar $
  *
  * Project:  GML Reader
  * Purpose:  Implementation of GMLPropertyDefn
@@ -15,16 +15,16 @@
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
@@ -36,7 +36,7 @@
 /*                           GMLPropertyDefn                            */
 /************************************************************************/
 
-GMLPropertyDefn::GMLPropertyDefn( const char *pszName, 
+GMLPropertyDefn::GMLPropertyDefn( const char *pszName,
                                   const char *pszSrcElement )
 
 {
@@ -52,9 +52,10 @@ GMLPropertyDefn::GMLPropertyDefn( const char *pszName,
         m_pszSrcElement = NULL;
     }
     m_eType = GMLPT_Untyped;
-    m_nWidth = 0; 
+    m_nWidth = 0;
     m_nPrecision = 0;
     m_pszCondition = NULL;
+    m_bNullable = true;
 }
 
 /************************************************************************/
@@ -107,21 +108,22 @@ void GMLPropertyDefn::SetCondition( const char *pszCondition )
 /************************************************************************/
 
 void GMLPropertyDefn::AnalysePropertyValue( const GMLProperty* psGMLProperty,
-                                            int bSetWidth )
+                                            bool bSetWidth )
 
 {
 /* -------------------------------------------------------------------- */
 /*      Does the string consist entirely of numeric values?             */
 /* -------------------------------------------------------------------- */
-    int bIsReal = FALSE;
+    bool bIsReal = false;
 
-    int j;
-    for(j=0;j<psGMLProperty->nSubProperties;j++)
+    for( int j=0; j < psGMLProperty->nSubProperties; j++ )
     {
         if (j > 0)
         {
             if( m_eType == GMLPT_Integer )
                 m_eType = GMLPT_IntegerList;
+            else if( m_eType == GMLPT_Integer64 )
+                m_eType = GMLPT_Integer64List;
             else if( m_eType == GMLPT_Real )
                 m_eType = GMLPT_RealList;
             else if( m_eType == GMLPT_String )
@@ -129,6 +131,8 @@ void GMLPropertyDefn::AnalysePropertyValue( const GMLProperty* psGMLProperty,
                 m_eType = GMLPT_StringList;
                 m_nWidth = 0;
             }
+            else if( m_eType == GMLPT_Boolean )
+                m_eType = GMLPT_BooleanList;
         }
         const char* pszValue = psGMLProperty->papszSubProperties[j];
 /* -------------------------------------------------------------------- */
@@ -140,56 +144,66 @@ void GMLPropertyDefn::AnalysePropertyValue( const GMLProperty* psGMLProperty,
 
         CPLValueType valueType = CPLGetValueType(pszValue);
 
-        /* This might not fit into a int32. For now, let's */
-        /* consider this as a real value then. */
-        /* FIXME once RFC31 / 64 bit support is set, we could */
-        /* choose a different behaviour */
-        if (valueType == CPL_VALUE_INTEGER && strlen(pszValue) >= 10)
-        {
-            /* Skip leading spaces */
-            while( isspace( (unsigned char)*pszValue ) )
-                pszValue ++;
-            char szVal[32];
-            sprintf(szVal, "%d", atoi(pszValue));
-            if (strcmp(pszValue, szVal) != 0)
-                valueType = CPL_VALUE_REAL;
-        }
-
         if (valueType == CPL_VALUE_STRING
-            && m_eType != GMLPT_String 
+            && m_eType != GMLPT_String
             && m_eType != GMLPT_StringList )
         {
-            if( m_eType == GMLPT_IntegerList
-                || m_eType == GMLPT_RealList )
+            if( (m_eType == GMLPT_Untyped || m_eType == GMLPT_Boolean) &&
+                (strcmp(pszValue, "true") == 0 ||
+                 strcmp(pszValue, "false") == 0) )
+                m_eType = GMLPT_Boolean;
+            else if( m_eType == GMLPT_BooleanList )
+            {
+                if( !(strcmp(pszValue, "true") == 0 ||
+                      strcmp(pszValue, "false") == 0) )
+                    m_eType = GMLPT_StringList;
+            }
+            else if( m_eType == GMLPT_IntegerList ||
+                     m_eType == GMLPT_Integer64List ||
+                     m_eType == GMLPT_RealList )
                 m_eType = GMLPT_StringList;
             else
                 m_eType = GMLPT_String;
         }
         else
             bIsReal = (valueType == CPL_VALUE_REAL);
-    
+
         if( m_eType == GMLPT_String )
         {
             if( bSetWidth )
             {
                 /* grow the Width to the length of the string passed in */
                 int nWidth;
-                nWidth = strlen(pszValue);
-                if ( m_nWidth < nWidth ) 
+                nWidth = static_cast<int>(strlen(pszValue));
+                if ( m_nWidth < nWidth )
                     SetWidth( nWidth );
             }
         }
-        else if( m_eType == GMLPT_Untyped || m_eType == GMLPT_Integer )
+        else if( m_eType == GMLPT_Untyped || m_eType == GMLPT_Integer ||
+                 m_eType == GMLPT_Integer64 )
         {
             if( bIsReal )
                 m_eType = GMLPT_Real;
-            else
-                m_eType = GMLPT_Integer;
+            else if( m_eType != GMLPT_Integer64 )
+            {
+                GIntBig nVal = CPLAtoGIntBig(pszValue);
+                if( !CPL_INT64_FITS_ON_INT32(nVal) )
+                    m_eType = GMLPT_Integer64;
+                else
+                    m_eType = GMLPT_Integer;
+            }
         }
-        else if( m_eType == GMLPT_IntegerList && bIsReal )
+        else if( (m_eType == GMLPT_IntegerList ||
+                  m_eType == GMLPT_Integer64List) && bIsReal )
         {
             m_eType = GMLPT_RealList;
         }
+        else if( m_eType == GMLPT_IntegerList && valueType == CPL_VALUE_INTEGER )
+        {
+            GIntBig nVal = CPLAtoGIntBig(pszValue);
+            if( !CPL_INT64_FITS_ON_INT32(nVal) )
+                m_eType = GMLPT_Integer64List;
+        }
     }
 }
 
@@ -200,13 +214,15 @@ void GMLPropertyDefn::AnalysePropertyValue( const GMLProperty* psGMLProperty,
 GMLGeometryPropertyDefn::GMLGeometryPropertyDefn( const char *pszName,
                                                   const char *pszSrcElement,
                                                   int nType,
-                                                  int nAttributeIndex )
+                                                  int nAttributeIndex,
+                                                  bool bNullable )
 {
     m_pszName = (pszName == NULL || pszName[0] == '\0') ?
                         CPLStrdup(pszSrcElement) : CPLStrdup(pszName);
     m_pszSrcElement = CPLStrdup(pszSrcElement);
     m_nGeometryType = nType;
     m_nAttributeIndex = nAttributeIndex;
+    m_bNullable = bNullable;
 }
 
 /************************************************************************/
