@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gmlreaderp.h 33702 2016-03-11 06:20:16Z goatbar $
+ * $Id: gmlreaderp.h 35690 2016-10-11 09:56:31Z rouault $
  *
  * Project:  GML Reader
  * Purpose:  Private Declarations for OGR free GML Reader code.
@@ -41,6 +41,7 @@
 #include "ogr_api.h"
 #include "cpl_vsi.h"
 #include "cpl_multiproc.h"
+#include "gmlutils.h"
 
 #include <string>
 #include <vector>
@@ -146,6 +147,8 @@ class GMLHandler
 
     std::vector<NodeLastChild> apsXMLNode;
 
+    int        m_nSRSDimensionIfMissing;
+
     OGRErr     startElementTop(const char *pszName, int nLenName, void* attr);
 
     OGRErr     endElementIgnoredFeature();
@@ -429,6 +432,7 @@ private:
 
     bool          m_bInvertAxisOrderIfLatLong;
     bool          m_bConsiderEPSGAsURN;
+    GMLSwapCoordinatesEnum m_eSwapCoordinates;
     bool          m_bGetSecondaryGeometryOption;
 
     int           ParseFeatureType(CPLXMLNode *psSchemaNode,
@@ -461,7 +465,9 @@ private:
 
 public:
                 GMLReader(bool bExpatReader, bool bInvertAxisOrderIfLatLong,
-                          bool bConsiderEPSGAsURN, bool bGetSecondaryGeometryOption);
+                          bool bConsiderEPSGAsURN,
+                          GMLSwapCoordinatesEnum eSwapCoordinates,
+                          bool bGetSecondaryGeometryOption);
     virtual     ~GMLReader();
 
     bool             IsClassListLocked() const { return m_bClassListLocked; }
