@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gmlutils.cpp 33702 2016-03-11 06:20:16Z goatbar $
+ * $Id: gmlutils.cpp 35690 2016-10-11 09:56:31Z rouault $
  *
  * Project:  GML Utils
  * Purpose:  GML reader
@@ -83,24 +83,25 @@ bool GML_IsSRSLatLongOrder(const char* pszSRSName)
     if (pszSRSName == NULL)
         return false;
 
-    if (STARTS_WITH(pszSRSName, "urn:"))
+    if (STARTS_WITH(pszSRSName, "urn:") && strstr(pszSRSName, ":4326") != NULL)
     {
-        if (strstr(pszSRSName, ":4326") != NULL)
+        /* Shortcut ... */
+        return true;
+    }
+    else if( !EQUALN(pszSRSName, "EPSG:", 5) )
+    {
+        OGRSpatialReference oSRS;
+        if (oSRS.SetFromUserInput(pszSRSName) == OGRERR_NONE)
         {
-            /* Shortcut ... */
-            return true;
+            if (oSRS.EPSGTreatsAsLatLong() || oSRS.EPSGTreatsAsNorthingEasting())
+                return true;
         }
-        else
-        {
-            OGRSpatialReference oSRS;
-            if (oSRS.importFromURN(pszSRSName) == OGRERR_NONE)
-            {
-                if (oSRS.EPSGTreatsAsLatLong() || oSRS.EPSGTreatsAsNorthingEasting())
-                    return true;
-            }
-        }
+        return false;
+    }
+    else
+    {
+        return false;
     }
-    return false;
 }
 
 
@@ -189,6 +190,7 @@ OGRGeometry* GML_BuildOGRGeometryFromList(const CPLXMLNode* const * papsGeometry
                                           bool bInvertAxisOrderIfLatLong,
                                           const char* pszDefaultSRSName,
                                           bool bConsiderEPSGAsURN,
+                                          GMLSwapCoordinatesEnum eSwapCoordinates,
                                           int nPseudoBoolGetSecondaryGeometryOption,
                                           void* hCacheSRS,
                                           bool bFaceHoleNegative)
@@ -247,6 +249,7 @@ OGRGeometry* GML_BuildOGRGeometryFromList(const CPLXMLNode* const * papsGeometry
                                                             bInvertAxisOrderIfLatLong,
                                                             pszDefaultSRSName,
                                                             bConsiderEPSGAsURN,
+                                                            eSwapCoordinates,
                                                             nPseudoBoolGetSecondaryGeometryOption,
                                                             hCacheSRS);
                     }
@@ -280,8 +283,12 @@ OGRGeometry* GML_BuildOGRGeometryFromList(const CPLXMLNode* const * papsGeometry
         SRSCache* poSRSCache = (SRSCache*)hCacheSRS;
         SRSDesc& oSRSDesc = poSRSCache->Get(pszNameLookup);
         poGeom->assignSpatialReference(oSRSDesc.poSRS);
-        if (oSRSDesc.bAxisInvert && bInvertAxisOrderIfLatLong)
+        if( (eSwapCoordinates == GML_SWAP_AUTO &&
+            oSRSDesc.bAxisInvert && bInvertAxisOrderIfLatLong) ||
+            eSwapCoordinates == GML_SWAP_YES )
+        {
             poGeom->swapXY();
+        }
     }
 
     return poGeom;
