@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: parsexsd.cpp 29256 2015-05-27 20:45:11Z rouault $
+ * $Id: parsexsd.cpp 33702 2016-03-11 06:20:16Z goatbar $
  *
  * Project:  GML Reader
  * Purpose:  Implementation of GMLParseXSD()
@@ -15,16 +15,16 @@
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
@@ -43,7 +43,7 @@
 /*      stripped off if there is one.  Returns pointer into             */
 /*      original string.                                                */
 /************************************************************************/
-
+static
 const char *StripNS( const char *pszFullValue )
 
 {
@@ -59,7 +59,7 @@ const char *StripNS( const char *pszFullValue )
 /************************************************************************/
 
 static
-int GetSimpleTypeProperties(CPLXMLNode *psTypeNode,
+bool GetSimpleTypeProperties(CPLXMLNode *psTypeNode,
                             GMLPropertyType *pGMLType,
                             int *pnWidth,
                             int *pnPrecision)
@@ -79,14 +79,19 @@ int GetSimpleTypeProperties(CPLXMLNode *psTypeNode,
                         "restriction.fractionDigits.value", "0" );
         *pnWidth = atoi(pszWidth);
         *pnPrecision = atoi(pszPrecision);
-        return TRUE;
+        return true;
     }
 
-    else if( EQUAL(pszBase,"float")
-                || EQUAL(pszBase,"double") )
+     else if( EQUAL(pszBase,"float") )
+    {
+        *pGMLType = GMLPT_Float;
+        return true;
+    }
+
+    else if( EQUAL(pszBase,"double") )
     {
         *pGMLType = GMLPT_Real;
-        return TRUE;
+        return true;
     }
 
     else if( EQUAL(pszBase,"integer") )
@@ -96,7 +101,27 @@ int GetSimpleTypeProperties(CPLXMLNode *psTypeNode,
             CPLGetXMLValue( psTypeNode,
                         "restriction.totalDigits.value", "0" );
         *pnWidth = atoi(pszWidth);
-        return TRUE;
+        return true;
+    }
+
+    else if( EQUAL(pszBase,"long") )
+    {
+        *pGMLType = GMLPT_Integer64;
+        const char *pszWidth =
+            CPLGetXMLValue( psTypeNode,
+                        "restriction.totalDigits.value", "0" );
+        *pnWidth = atoi(pszWidth);
+        return true;
+    }
+
+    else if( EQUAL(pszBase,"long") )
+    {
+        *pGMLType = GMLPT_Integer64;
+        const char *pszWidth =
+            CPLGetXMLValue( psTypeNode,
+                        "restriction.totalDigits.value", "0" );
+        *pnWidth = atoi(pszWidth);
+        return true;
     }
 
     else if( EQUAL(pszBase,"string") )
@@ -106,7 +131,7 @@ int GetSimpleTypeProperties(CPLXMLNode *psTypeNode,
             CPLGetXMLValue( psTypeNode,
                         "restriction.maxLength.value", "0" );
         *pnWidth = atoi(pszWidth);
-        return TRUE;
+        return true;
     }
 
     /* TODO: Would be nice to have a proper date type */
@@ -114,9 +139,22 @@ int GetSimpleTypeProperties(CPLXMLNode *psTypeNode,
              EQUAL(pszBase,"dateTime") )
     {
         *pGMLType = GMLPT_String;
-        return TRUE;
+        return true;
     }
-    return FALSE;
+
+    else if( EQUAL(pszBase,"boolean") )
+    {
+        *pGMLType = GMLPT_Boolean;
+        return true;
+    }
+
+    else if( EQUAL(pszBase,"short") )
+    {
+        *pGMLType = GMLPT_Short;
+        return true;
+    }
+
+    return false;
 }
 
 /************************************************************************/
@@ -124,7 +162,7 @@ int GetSimpleTypeProperties(CPLXMLNode *psTypeNode,
 /************************************************************************/
 
 static
-int LookForSimpleType(CPLXMLNode *psSchemaNode,
+bool LookForSimpleType(CPLXMLNode *psSchemaNode,
                       const char* pszStrippedNSType,
                       GMLPropertyType *pGMLType,
                       int *pnWidth,
@@ -142,7 +180,7 @@ int LookForSimpleType(CPLXMLNode *psSchemaNode,
         }
     }
     if (psThis == NULL)
-        return FALSE;
+        return false;
 
     return GetSimpleTypeProperties(psThis, pGMLType, pnWidth, pnPrecision);
 }
@@ -202,10 +240,14 @@ static GMLPropertyType GetListTypeFromSingleType(GMLPropertyType eType)
 {
     if( eType == GMLPT_String )
         return GMLPT_StringList;
-    if( eType == GMLPT_Integer )
+    if( eType == GMLPT_Integer || eType == GMLPT_Short )
         return GMLPT_IntegerList;
-    if( eType == GMLPT_Real )
+    if( eType == GMLPT_Integer64 )
+        return GMLPT_Integer64List;
+    if( eType == GMLPT_Real || eType == GMLPT_Float )
         return GMLPT_RealList;
+    if( eType == GMLPT_Boolean )
+        return GMLPT_BooleanList;
     if( eType == GMLPT_FeatureProperty )
         return GMLPT_FeaturePropertyList;
     return eType;
@@ -226,14 +268,14 @@ static const AssocNameType apsPropertyTypes [] =
     {"GeometryPropertyType", wkbUnknown},
     {"PointPropertyType", wkbPoint},
     {"LineStringPropertyType", wkbLineString},
-    {"CurvePropertyType", wkbLineString},
+    {"CurvePropertyType", wkbCompoundCurve},
     {"PolygonPropertyType", wkbPolygon},
-    {"SurfacePropertyType", wkbPolygon},
+    {"SurfacePropertyType", wkbCurvePolygon},
     {"MultiPointPropertyType", wkbMultiPoint},
     {"MultiLineStringPropertyType", wkbMultiLineString},
-    {"MultiCurvePropertyType", wkbMultiLineString},
+    {"MultiCurvePropertyType", wkbMultiCurve},
     {"MultiPolygonPropertyType", wkbMultiPolygon},
-    {"MultiSurfacePropertyType", wkbMultiPolygon},
+    {"MultiSurfacePropertyType", wkbMultiSurface},
     {"MultiGeometryPropertyType", wkbGeometryCollection},
     {"GeometryAssociationType", wkbUnknown},
     {NULL, wkbUnknown},
@@ -243,11 +285,11 @@ static const AssocNameType apsPropertyTypes [] =
 static const AssocNameType apsRefTypes [] =
 {
     {"pointProperty", wkbPoint},
-    {"curveProperty", wkbLineString},
-    {"surfaceProperty", wkbPolygon},
+    {"curveProperty", wkbLineString}, /* should we promote to wkbCompoundCurve ? */
+    {"surfaceProperty", wkbPolygon},  /* should we promote to wkbCurvePolygon ? */
     {"multiPointProperty", wkbMultiPoint},
     {"multiCurveProperty", wkbMultiLineString},
-    {"multiSurfaceProperty", wkbMultiPolygon},
+    {"multiSurfaceProperty", wkbMultiPolygon}, /* should we promote to wkbMultiSurface ? */
     {NULL, wkbUnknown},
 };
 
@@ -310,7 +352,7 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
     CPLXMLNode *psAttrDef;
     int nAttributeIndex = 0;
 
-    int bGotUnrecognizedType = FALSE;
+    bool bGotUnrecognizedType = false;
 
     for( psAttrDef = psAttrSeq->psChild;
             psAttrDef != NULL;
@@ -322,7 +364,7 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
             delete poClass;
             return NULL;
         }
-        
+
         /* Parse stuff like :
         <xs:choice>
             <xs:element ref="gml:polygonProperty"/>
@@ -333,8 +375,8 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
         if( strcmp(psAttrDef->pszValue,"choice") == 0 )
         {
             CPLXMLNode* psChild = psAttrDef->psChild;
-            int bPolygon = FALSE;
-            int bMultiPolygon = FALSE;
+            bool bPolygon = false;
+            bool bMultiPolygon = false;
             for( ; psChild; psChild = psChild->psNext )
             {
                 if( psChild->eType != CXT_Element )
@@ -345,9 +387,9 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
                     if( pszRef != NULL )
                     {
                         if( strcmp(pszRef, "gml:polygonProperty") == 0 )
-                            bPolygon = TRUE;
+                            bPolygon = true;
                         else if( strcmp(pszRef, "gml:multiPolygonProperty") == 0 )
-                            bMultiPolygon = TRUE;
+                            bMultiPolygon = true;
                         else
                         {
                             delete poClass;
@@ -364,7 +406,7 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
             if( bPolygon && bMultiPolygon )
             {
                 poClass->AddGeometryProperty( new GMLGeometryPropertyDefn(
-                    "", "", wkbMultiPolygon, nAttributeIndex ) );
+                    "", "", wkbMultiPolygon, nAttributeIndex, true ) );
 
                 nAttributeIndex ++;
             }
@@ -378,6 +420,7 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
         /* not as a simpleType definition */
         const char* pszType = CPLGetXMLValue( psAttrDef, "type", NULL );
         const char* pszElementName = CPLGetXMLValue( psAttrDef, "name", NULL );
+        bool bNullable = EQUAL(CPLGetXMLValue( psAttrDef, "minOccurs", "1" ), "0");
         const char* pszMaxOccurs = CPLGetXMLValue( psAttrDef, "maxOccurs", NULL );
         if (pszType != NULL)
         {
@@ -386,8 +429,7 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
 
             GMLPropertyType gmlType = GMLPT_Untyped;
             if (EQUAL(pszStrippedNSType, "string") ||
-                EQUAL(pszStrippedNSType, "Character") ||
-                EQUAL(pszStrippedNSType, "boolean"))
+                EQUAL(pszStrippedNSType, "Character"))
                 gmlType = GMLPT_String;
             /* TODO: Would be nice to have a proper date type */
             else if (EQUAL(pszStrippedNSType, "date") ||
@@ -395,27 +437,47 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
                 gmlType = GMLPT_String;
             else if (EQUAL(pszStrippedNSType, "real") ||
                      EQUAL(pszStrippedNSType, "double") ||
-                     EQUAL(pszStrippedNSType, "float") ||
                      EQUAL(pszStrippedNSType, "decimal"))
                 gmlType = GMLPT_Real;
-            else if (EQUAL(pszStrippedNSType, "short") ||
-                     EQUAL(pszStrippedNSType, "int") ||
-                     EQUAL(pszStrippedNSType, "integer") ||
-                     EQUAL(pszStrippedNSType, "long"))
+            else if (EQUAL(pszStrippedNSType, "float") )
+                gmlType = GMLPT_Float;
+            else if (EQUAL(pszStrippedNSType, "int") ||
+                     EQUAL(pszStrippedNSType, "integer"))
                 gmlType = GMLPT_Integer;
+            else if (EQUAL(pszStrippedNSType, "long"))
+                gmlType = GMLPT_Integer64;
+            else if (EQUAL(pszStrippedNSType, "short") )
+                gmlType = GMLPT_Short;
+            else if (EQUAL(pszStrippedNSType, "boolean") )
+                gmlType = GMLPT_Boolean;
             else if (strcmp(pszType, "gml:FeaturePropertyType") == 0 )
             {
                 gmlType = GMLPT_FeatureProperty;
             }
-            else if (strncmp(pszType, "gml:", 4) == 0)
+            else if (STARTS_WITH(pszType, "gml:"))
             {
                 const AssocNameType* psIter = apsPropertyTypes;
                 while(psIter->pszName)
                 {
                     if (strncmp(pszType + 4, psIter->pszName, strlen(psIter->pszName)) == 0)
                     {
+                        OGRwkbGeometryType eType = psIter->eType;
+
+                        /* Look if there's a comment restricting to subclasses */
+                        if( psAttrDef->psNext != NULL && psAttrDef->psNext->eType == CXT_Comment )
+                        {
+                            if( strstr(psAttrDef->psNext->pszValue, "restricted to Polygon") )
+                                eType = wkbPolygon;
+                            else if( strstr(psAttrDef->psNext->pszValue, "restricted to LineString") )
+                                eType = wkbLineString;
+                            else if( strstr(psAttrDef->psNext->pszValue, "restricted to MultiPolygon") )
+                                eType = wkbMultiPolygon;
+                            else if( strstr(psAttrDef->psNext->pszValue, "restricted to MultiLineString") )
+                                eType = wkbMultiLineString;
+                        }
+
                         poClass->AddGeometryProperty( new GMLGeometryPropertyDefn(
-                            pszElementName, pszElementName, psIter->eType, nAttributeIndex ) );
+                            pszElementName, pszElementName, eType, nAttributeIndex, bNullable ) );
 
                         nAttributeIndex ++;
 
@@ -434,7 +496,7 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
                 }
 
                 if (poClass->GetGeometryPropertyCount() == 0)
-                    bGotUnrecognizedType = TRUE;
+                    bGotUnrecognizedType = true;
 
                 continue;
             }
@@ -444,7 +506,8 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
                      strcmp(pszType, "gmgml:Point_MultiPointPropertyType") == 0)
             {
                 poClass->AddGeometryProperty( new GMLGeometryPropertyDefn(
-                    pszElementName, pszElementName, wkbMultiPoint, nAttributeIndex ) );
+                    pszElementName, pszElementName, wkbMultiPoint, nAttributeIndex,
+                    bNullable ) );
 
                 nAttributeIndex ++;
                 continue;
@@ -453,7 +516,8 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
                      strcmp(pszType, "gmgml:LineString_MultiLineStringPropertyType") == 0)
             {
                 poClass->AddGeometryProperty( new GMLGeometryPropertyDefn(
-                    pszElementName, pszElementName, wkbMultiLineString, nAttributeIndex ) );
+                    pszElementName, pszElementName, wkbMultiLineString, nAttributeIndex,
+                    bNullable ) );
 
                 nAttributeIndex ++;
                 continue;
@@ -463,7 +527,8 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
                      strcmp(pszType, "gmgml:Polygon_Surface_MultiSurface_CompositeSurfacePropertyType") == 0)
             {
                 poClass->AddGeometryProperty( new GMLGeometryPropertyDefn(
-                    pszElementName, pszElementName, wkbMultiPolygon, nAttributeIndex ) );
+                    pszElementName, pszElementName, wkbMultiPolygon, nAttributeIndex,
+                    bNullable ) );
 
                 nAttributeIndex ++;
                 continue;
@@ -473,7 +538,8 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
             else if (strcmp(pszType, "wfs:MixedPolygonPropertyType") == 0)
             {
                 poClass->AddGeometryProperty( new GMLGeometryPropertyDefn(
-                    pszElementName, pszElementName, wkbMultiPolygon, nAttributeIndex ) );
+                    pszElementName, pszElementName, wkbMultiPolygon, nAttributeIndex,
+                    bNullable) );
 
                 nAttributeIndex ++;
                 continue;
@@ -501,12 +567,13 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
             GMLPropertyDefn *poProp = new GMLPropertyDefn(
                 pszPropertyName, pszElementName );
 
-            if( pszMaxOccurs != NULL && strcmp(pszMaxOccurs, "unbounded") == 0 )
+            if( pszMaxOccurs != NULL && strcmp(pszMaxOccurs, "1") != 0 )
                 gmlType = GetListTypeFromSingleType(gmlType);
 
             poProp->SetType( gmlType );
             poProp->SetWidth( nWidth );
             poProp->SetPrecision( nPrecision );
+            poProp->SetNullable( bNullable );
 
             if (poClass->AddProperty( poProp ) < 0)
                 delete poProp;
@@ -523,7 +590,7 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
             const char* pszRef = CPLGetXMLValue( psAttrDef, "ref", NULL );
 
             /* FME .xsd */
-            if (pszRef != NULL && strncmp(pszRef, "gml:", 4) == 0)
+            if (pszRef != NULL && STARTS_WITH(pszRef, "gml:"))
             {
                 const AssocNameType* psIter = apsRefTypes;
                 while(psIter->pszName)
@@ -548,7 +615,7 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
                         else
                         {
                             poClass->AddGeometryProperty( new GMLGeometryPropertyDefn(
-                                pszElementName, pszElementName, psIter->eType, nAttributeIndex ) );
+                                pszElementName, pszElementName, psIter->eType, nAttributeIndex, true ) );
 
                             nAttributeIndex ++;
                         }
@@ -568,7 +635,7 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
                 }
 
                 if (poClass->GetGeometryPropertyCount() == 0)
-                    bGotUnrecognizedType = TRUE;
+                    bGotUnrecognizedType = true;
 
                 continue;
             }
@@ -582,8 +649,8 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
                 </xs:complexType>
             </xs:element>
             */
-            CPLXMLNode* psComplexType = GetSingleChildElement( psAttrDef, "complexType" );
-            CPLXMLNode* psComplexTypeSequence = GetSingleChildElement( psComplexType, "sequence" );
+            CPLXMLNode* l_psComplexType = GetSingleChildElement( psAttrDef, "complexType" );
+            CPLXMLNode* psComplexTypeSequence = GetSingleChildElement( l_psComplexType, "sequence" );
             CPLXMLNode* psComplexTypeSequenceElement = GetSingleChildElement( psComplexTypeSequence, "element" );
 
             if( pszElementName != NULL &&
@@ -593,7 +660,7 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
                 strcmp(CPLGetXMLValue( psComplexTypeSequenceElement, "ref", "" ), "gml:_Geometry") == 0 )
             {
                 poClass->AddGeometryProperty( new GMLGeometryPropertyDefn(
-                    pszElementName, pszElementName, wkbUnknown, nAttributeIndex ) );
+                    pszElementName, pszElementName, wkbUnknown, nAttributeIndex, bNullable ) );
 
                 nAttributeIndex ++;
 
@@ -616,12 +683,13 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
         int nWidth = 0, nPrecision = 0;
         GetSimpleTypeProperties(psSimpleType, &eType, &nWidth, &nPrecision);
 
-        if( pszMaxOccurs != NULL && strcmp(pszMaxOccurs, "unbounded") == 0 )
+        if( pszMaxOccurs != NULL && strcmp(pszMaxOccurs, "1") != 0 )
             eType = GetListTypeFromSingleType(eType);
 
         poProp->SetType( eType );
         poProp->SetWidth( nWidth );
         poProp->SetPrecision( nPrecision );
+        poProp->SetNullable( bNullable );
 
         if (poClass->AddProperty( poProp ) < 0)
             delete poProp;
@@ -634,13 +702,13 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
     if( poClass->GetGeometryPropertyCount() == 0 &&
         bGotUnrecognizedType )
     {
-        poClass->AddGeometryProperty( new GMLGeometryPropertyDefn( "", "", wkbUnknown ) );
+        poClass->AddGeometryProperty( new GMLGeometryPropertyDefn( "", "", wkbUnknown, -1, true ) );
     }
 
 /* -------------------------------------------------------------------- */
 /*      Class complete, add to reader class list.                       */
 /* -------------------------------------------------------------------- */
-    poClass->SetSchemaLocked( TRUE );
+    poClass->SetSchemaLocked( true );
 
     return poClass;
 }
@@ -652,8 +720,8 @@ GMLFeatureClass* GMLParseFeatureType(CPLXMLNode *psSchemaNode,
 static
 CPLXMLNode* GMLParseXMLFile(const char* pszFilename)
 {
-    if( strncmp(pszFilename, "http://", 7) == 0 ||
-        strncmp(pszFilename, "https://", 8) == 0 )
+    if( STARTS_WITH(pszFilename, "http://") ||
+        STARTS_WITH(pszFilename, "https://") )
     {
         CPLXMLNode* psRet = NULL;
         CPLHTTPResult* psResult = CPLHTTPFetch( pszFilename, NULL );
@@ -699,8 +767,6 @@ CPLXMLNode* CPLGetFirstChildNode( CPLXMLNode* psNode )
 static
 CPLXMLNode* CPLGetLastNode( CPLXMLNode* psNode )
 {
-    if( psNode == NULL )
-        return NULL;
     CPLXMLNode* psIter = psNode;
     while( psIter->psNext != NULL )
         psIter = psIter->psNext;
@@ -717,28 +783,28 @@ void CPLXMLSchemaResolveInclude( const char* pszMainSchemaLocation,
 {
     std::set<CPLString> osAlreadyIncluded;
 
-    int bTryAgain;
+    bool bTryAgain;
     do
     {
         CPLXMLNode *psThis;
         CPLXMLNode *psLast = NULL;
-        bTryAgain = FALSE;
+        bTryAgain = false;
 
-        for( psThis = psSchemaNode->psChild; 
+        for( psThis = psSchemaNode->psChild;
             psThis != NULL; psThis = psThis->psNext )
         {
             if( psThis->eType == CXT_Element &&
                 EQUAL(psThis->pszValue,"include") )
             {
-                const char* pszSchemaLocation = 
+                const char* pszSchemaLocation =
                         CPLGetXMLValue(psThis, "schemaLocation", NULL);
                 if( pszSchemaLocation != NULL &&
                     osAlreadyIncluded.count( pszSchemaLocation) == 0 )
                 {
                     osAlreadyIncluded.insert( pszSchemaLocation );
 
-                    if( strncmp(pszSchemaLocation, "http://", 7) != 0 &&
-                        strncmp(pszSchemaLocation, "https://", 8) != 0 &&
+                    if( !STARTS_WITH(pszSchemaLocation, "http://") &&
+                        !STARTS_WITH(pszSchemaLocation, "https://") &&
                         CPLIsFilenameRelative(pszSchemaLocation ) )
                     {
                         pszSchemaLocation = CPLFormFilename(
@@ -772,7 +838,7 @@ void CPLXMLSchemaResolveInclude( const char* pszMainSchemaLocation,
 
                                 /* In case the included schema also contains */
                                 /* includes */
-                                bTryAgain = TRUE;
+                                bTryAgain = true;
                             }
 
                         }
@@ -797,20 +863,23 @@ void CPLXMLSchemaResolveInclude( const char* pszMainSchemaLocation,
 /*                          GMLParseXSD()                               */
 /************************************************************************/
 
-int GMLParseXSD( const char *pszFile,
-                 std::vector<GMLFeatureClass*> & aosClasses)
+bool GMLParseXSD( const char *pszFile,
+                 std::vector<GMLFeatureClass*> & aosClasses,
+                 bool& bFullyUnderstood)
 
 {
+    bFullyUnderstood = false;
+
     if( pszFile == NULL )
-        return FALSE;
+        return false;
 
 /* -------------------------------------------------------------------- */
 /*      Load the raw XML file.                                          */
 /* -------------------------------------------------------------------- */
     CPLXMLNode *psXSDTree = GMLParseXMLFile( pszFile );
-    
+
     if( psXSDTree == NULL )
-        return FALSE;
+        return false;
 
 /* -------------------------------------------------------------------- */
 /*      Strip off any namespace qualifiers.                             */
@@ -824,7 +893,7 @@ int GMLParseXSD( const char *pszFile,
     if( psSchemaNode == NULL )
     {
         CPLDestroyXMLNode( psXSDTree );
-        return FALSE;
+        return false;
     }
 
 /* ==================================================================== */
@@ -834,25 +903,27 @@ int GMLParseXSD( const char *pszFile,
 
     //CPLSerializeXMLTreeToFile(psSchemaNode, "/vsistdout/");
 
+    bFullyUnderstood = true;
+
 /* ==================================================================== */
 /*      Process each feature class definition.                          */
 /* ==================================================================== */
     CPLXMLNode *psThis;
 
-    for( psThis = psSchemaNode->psChild; 
+    for( psThis = psSchemaNode->psChild;
          psThis != NULL; psThis = psThis->psNext )
     {
 /* -------------------------------------------------------------------- */
 /*      Check for <xs:element> node.                                    */
 /* -------------------------------------------------------------------- */
-        if( psThis->eType != CXT_Element 
+        if( psThis->eType != CXT_Element
             || !EQUAL(psThis->pszValue,"element") )
             continue;
 
 /* -------------------------------------------------------------------- */
 /*      Check the substitution group.                                   */
 /* -------------------------------------------------------------------- */
-        const char *pszSubGroup = 
+        const char *pszSubGroup =
             StripNS(CPLGetXMLValue(psThis,"substitutionGroup",""));
 
         // Old OGR produced elements for the feature collection.
@@ -864,7 +935,7 @@ int GMLParseXSD( const char *pszFile,
         {
             continue;
         }
-        
+
 /* -------------------------------------------------------------------- */
 /*      Get name                                                        */
 /* -------------------------------------------------------------------- */
@@ -891,6 +962,8 @@ int GMLParseXSD( const char *pszFile,
                         GMLParseFeatureType(psSchemaNode, pszName, psComplexType);
                 if (poClass)
                     aosClasses.push_back(poClass);
+                else
+                    bFullyUnderstood = false;
             }
             continue;
         }
@@ -909,8 +982,7 @@ int GMLParseXSD( const char *pszFile,
                   strlen(pszName) > strlen(pszType) - 4 &&
                   strncmp(pszName + strlen(pszName) - (strlen(pszType) - 4),
                           pszType,
-                          strlen(pszType) - 4) == 0 )
-        {
+                          strlen(pszType) - 4) == 0 )        {
         }
 
         else if( !EQUALN(pszType,pszName,strlen(pszName))
@@ -921,23 +993,25 @@ int GMLParseXSD( const char *pszFile,
         }
 
         /* CanVec .xsd contains weird types that are not used in the related GML */
-        if (strncmp(pszName, "XyZz", 4) == 0 ||
-            strncmp(pszName, "XyZ1", 4) == 0 ||
-            strncmp(pszName, "XyZ2", 4) == 0)
+        if (STARTS_WITH(pszName, "XyZz") ||
+            STARTS_WITH(pszName, "XyZ1") ||
+            STARTS_WITH(pszName, "XyZ2"))
             continue;
 
         GMLFeatureClass* poClass =
                 GMLParseFeatureType(psSchemaNode, pszName, pszType);
         if (poClass)
             aosClasses.push_back(poClass);
+        else
+            bFullyUnderstood = false;
     }
 
     CPLDestroyXMLNode( psXSDTree );
 
     if( aosClasses.size() > 0 )
     {
-        return TRUE;
+        return true;
     }
     else
-        return FALSE;
+        return false;
 }
