@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id$
+ * $Id: ogrgeopackageutility.h 35679 2016-10-10 14:15:22Z rouault $
  *
  * Project:  GeoPackage Translator
  * Purpose:  Utility header for OGR GeoPackage driver.
@@ -84,4 +84,8 @@ OGRErr              GPkgEnvelopeToOGR(GByte *pabyGpkg, size_t szGpkg, OGREnvelop
 
 OGRErr              GPkgHeaderFromWKB(const GByte *pabyGpkg, size_t szGpkg, GPkgHeader *poHeader);
 
+CPLString           SQLEscapeDoubleQuote(const char* pszStr);
+CPLString           SQLUnescapeDoubleQuote(const char* pszStr);
+char**              SQLTokenize( const char* pszSQL );
+
 #endif
