@@ -1,12 +1,12 @@
 /******************************************************************************
- * $Id: ogrgpsbabeldatasource.cpp 25598 2013-02-05 22:24:35Z rouault $
+ * $Id: ogrgpsbabeldatasource.cpp 32982 2016-01-14 16:53:57Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRGPSBabelDataSource class.
  * Author:   Even Rouault, <even dot rouault at mines dash paris dot org>
  *
  ******************************************************************************
- * Copyright (c) 2010, Even Rouault
+ * Copyright (c) 2010-2013, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,28 +27,31 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "ogr_gpsbabel.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
 #include "cpl_error.h"
 #include "cpl_spawn.h"
+#include "ogr_gpsbabel.h"
 
-#include <string.h>
+#include <cstring>
 
-CPL_CVSID("$Id: ogrgpsbabeldatasource.cpp 25598 2013-02-05 22:24:35Z rouault $");
+CPL_CVSID("$Id: ogrgpsbabeldatasource.cpp 32982 2016-01-14 16:53:57Z goatbar $");
 
 /************************************************************************/
 /*                      OGRGPSBabelDataSource()                         */
 /************************************************************************/
 
-OGRGPSBabelDataSource::OGRGPSBabelDataSource()
-
+OGRGPSBabelDataSource::OGRGPSBabelDataSource() :
+    nLayers(0),
+    pszName(NULL),
+    pszGPSBabelDriverName(NULL),
+    pszFilename(NULL),
+    poGPXDS(NULL)
 {
-    nLayers = 0;
-    pszName = NULL;
-    pszGPSBabelDriverName = NULL;
-    pszFilename = NULL;
-    poGPXDS = NULL;
+  for(int i=0; i<5; ++i)
+  {
+    apoLayers[i] = NULL;
+  }
 }
 
 /************************************************************************/
@@ -61,24 +64,36 @@ OGRGPSBabelDataSource::~OGRGPSBabelDataSource()
     CPLFree(pszName);
     CPLFree(pszGPSBabelDriverName);
     CPLFree(pszFilename);
-    
-    if (poGPXDS)
-        OGRDataSource::DestroyDataSource(poGPXDS);
-    
+
+    CloseDependentDatasets();
+
     if (osTmpFileName.size() > 0)
         VSIUnlink(osTmpFileName.c_str());
 }
 
+/************************************************************************/
+/*                     CloseDependentDatasets()                         */
+/************************************************************************/
+
+int OGRGPSBabelDataSource::CloseDependentDatasets()
+{
+    if( poGPXDS == NULL )
+        return FALSE;
+
+    GDALClose( poGPXDS );
+    poGPXDS = NULL;
+    return TRUE;
+}
+
 /************************************************************************/
 /*                             GetArgv()                                */
 /************************************************************************/
 
-static char** GetArgv(int bExplicitFeatures, int bWaypoints, int bRoutes,
-                      int bTracks, const char* pszGPSBabelDriverName,
-                      const char* pszFilename)
+static char** GetArgv( int bExplicitFeatures, int bWaypoints, int bRoutes,
+                       int bTracks, const char* pszGPSBabelDriverName,
+                       const char* pszFilename )
 {
-    char** argv = NULL;
-    argv = CSLAddString(argv, "gpsbabel");
+    char** argv = CSLAddString(NULL, "gpsbabel");
     if (bExplicitFeatures)
     {
         if (bWaypoints) argv = CSLAddString(argv, "-w");
@@ -103,9 +118,9 @@ static char** GetArgv(int bExplicitFeatures, int bWaypoints, int bRoutes,
 
 int OGRGPSBabelDataSource::IsSpecialFile(const char* pszFilename)
 {
-    return (strncmp(pszFilename, "/dev/", 5) == 0 ||
-            strncmp(pszFilename, "usb:", 4) == 0 ||
-            (strncmp(pszFilename, "COM", 3) == 0  && atoi(pszFilename + 3) > 0));
+    return (STARTS_WITH(pszFilename, "/dev/") ||
+            STARTS_WITH(pszFilename, "usb:") ||
+            (STARTS_WITH(pszFilename, "COM")  && atoi(pszFilename + 3) > 0));
 }
 
 /************************************************************************/
@@ -114,15 +129,16 @@ int OGRGPSBabelDataSource::IsSpecialFile(const char* pszFilename)
 
 int OGRGPSBabelDataSource::IsValidDriverName(const char* pszGPSBabelDriverName)
 {
-    int i;
-    for(i=0;pszGPSBabelDriverName[i] != '\0';i++)
+    for( int i = 0; pszGPSBabelDriverName[i] != '\0'; i++ )
     {
         char ch = pszGPSBabelDriverName[i];
-        if (!((ch >= 'A' && ch <= 'Z') || (ch >= 'a' && ch <= 'z') ||
-            (ch >= '0' && ch <= '9') || ch == '_' || ch == '=' || ch == '.'  || ch == ','))
+        if( !((ch >= 'A' && ch <= 'Z') ||
+              (ch >= 'a' && ch <= 'z') ||
+              (ch >= '0' && ch <= '9') ||
+              ch == '_' || ch == '=' || ch == '.'  || ch == ',') )
         {
-            CPLError(CE_Failure, CPLE_AppDefined,
-                "Invalid GPSBabel driver name");
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "Invalid GPSBabel driver name");
             return FALSE;
         }
     }
@@ -133,70 +149,54 @@ int OGRGPSBabelDataSource::IsValidDriverName(const char* pszGPSBabelDriverName)
 /*                                Open()                                */
 /************************************************************************/
 
-int OGRGPSBabelDataSource::Open( const char * pszDatasourceName, int bUpdateIn)
+int OGRGPSBabelDataSource::Open( const char * pszDatasourceName,
+                                 const char* pszGPSBabelDriverNameIn,
+                                 char** papszOpenOptionsIn )
 
 {
-    int bExplicitFeatures = FALSE;
-    int bWaypoints = TRUE, bTracks = TRUE, bRoutes = TRUE;
-    if (bUpdateIn)
+    if (!STARTS_WITH_CI(pszDatasourceName, "GPSBABEL:"))
     {
-        CPLError(CE_Failure, CPLE_NotSupported,
-                    "OGR/GPSBabel driver does not support opening a file in update mode");
-        return FALSE;
-    }
-
-    if (!EQUALN(pszDatasourceName, "GPSBABEL:", 9))
-    {
-        VSILFILE* fp = VSIFOpenL(pszDatasourceName, "rb");
-        if (fp == NULL)
-            return FALSE;
-
-        char szHeader[1024 + 1];
-        memset(szHeader, 0, 1024+1);
-        VSIFReadL(szHeader, 1, 1024, fp);
-        if (memcmp(szHeader, "MsRcd", 5) == 0)
-            pszGPSBabelDriverName = CPLStrdup("mapsource");
-        else if (memcmp(szHeader, "MsRcf", 5) == 0)
-            pszGPSBabelDriverName = CPLStrdup("gdb");
-        else if (strstr(szHeader, "<osm") != NULL)
-            pszGPSBabelDriverName = CPLStrdup("osm");
-        else if (strstr(szHeader, "$GPGSA") != NULL ||
-                 strstr(szHeader, "$GPGGA") != NULL)
-            pszGPSBabelDriverName = CPLStrdup("nmea");
-        else if (EQUALN(szHeader, "OziExplorer",11))
-            pszGPSBabelDriverName = CPLStrdup("ozi");
-        else if (strstr(szHeader, "Grid") && strstr(szHeader, "Datum") && strstr(szHeader, "Header"))
-            pszGPSBabelDriverName = CPLStrdup("garmin_txt");
-        else if (szHeader[0] == 13 && szHeader[10] == 'M' && szHeader[11] == 'S' &&
-                 (szHeader[12] >= '0' && szHeader[12] <= '9') &&
-                 (szHeader[13] >= '0' && szHeader[13] <= '9') &&
-                 szHeader[12] * 10 + szHeader[13] >= 30 &&
-                 (szHeader[14] == 1 || szHeader[14] == 2) && szHeader[15] == 0 &&
-                 szHeader[16] == 0 && szHeader[17] == 0)
-            pszGPSBabelDriverName = CPLStrdup("mapsend");
-        else if (strstr(szHeader, "$PMGNWPL") != NULL ||
-                 strstr(szHeader, "$PMGNRTE") != NULL)
-            pszGPSBabelDriverName = CPLStrdup("magellan");
-
-        VSIFCloseL(fp);
-
-        if (pszGPSBabelDriverName == NULL)
-        {
-            return FALSE;
-        }
-
+        CPLAssert(pszGPSBabelDriverNameIn);
+        pszGPSBabelDriverName = CPLStrdup(pszGPSBabelDriverNameIn);
         pszFilename = CPLStrdup(pszDatasourceName);
     }
+    else
+    {
+        if( CSLFetchNameValue(papszOpenOptionsIn, "FILENAME") )
+            pszFilename = CPLStrdup(CSLFetchNameValue(papszOpenOptionsIn,
+                                                      "FILENAME"));
+
+        if( CSLFetchNameValue(papszOpenOptionsIn, "GPSBABEL_DRIVER") )
+        {
+            if( pszFilename == NULL )
+            {
+                CPLError(CE_Failure, CPLE_AppDefined, "Missing FILENAME");
+                return FALSE;
+            }
+
+            pszGPSBabelDriverName
+                = CPLStrdup(CSLFetchNameValue(papszOpenOptionsIn, "DRIVER"));
+
+            /* A bit of validation to avoid command line injection */
+            if (!IsValidDriverName(pszGPSBabelDriverName))
+                return FALSE;
+        }
+    }
 
     pszName = CPLStrdup( pszDatasourceName );
 
+    bool bExplicitFeatures = false;
+    bool bWaypoints = true;
+    bool bTracks = true;
+    bool bRoutes = true;
+
     if (pszGPSBabelDriverName == NULL)
     {
         const char* pszSep = strchr(pszDatasourceName + 9, ':');
         if (pszSep == NULL)
         {
-            CPLError(CE_Failure, CPLE_AppDefined,
-                    "Wrong syntax. Expected GPSBabel:driver_name:file_name");
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "Wrong syntax. Expected GPSBabel:driver_name:file_name");
             return FALSE;
         }
 
@@ -207,14 +207,16 @@ int OGRGPSBabelDataSource::Open( const char * pszDatasourceName, int bUpdateIn)
         if (!IsValidDriverName(pszGPSBabelDriverName))
             return FALSE;
 
-        /* Parse optionnal features= option */
-        if (EQUALN(pszSep+1, "features=", 9))
+        /* Parse optional features= option */
+        if (STARTS_WITH_CI(pszSep+1, "features="))
         {
             const char* pszNextSep = strchr(pszSep+1, ':');
             if (pszNextSep == NULL)
             {
                 CPLError(CE_Failure, CPLE_AppDefined,
-                        "Wrong syntax. Expected GPSBabel:driver_name[,options]*:[features=waypoints,tracks,routes:]file_name");
+                        "Wrong syntax. Expected "
+                         "GPSBabel:driver_name[,options]*:["
+                         "features=waypoints,tracks,routes:]file_name");
                 return FALSE;
             }
 
@@ -222,23 +224,26 @@ int OGRGPSBabelDataSource::Open( const char * pszDatasourceName, int bUpdateIn)
             *strchr(pszFeatures, ':') = 0;
             char** papszTokens = CSLTokenizeString(pszFeatures);
             char** papszIter = papszTokens;
-            int bErr = FALSE;
-            bExplicitFeatures = TRUE;
-            bWaypoints = bTracks = bRoutes = FALSE;
+            bool bErr = false;
+            bExplicitFeatures = true;
+            bWaypoints = false;
+            bTracks = false;
+            bRoutes = false;
             while(papszIter && *papszIter)
             {
                 if (EQUAL(*papszIter, "waypoints"))
-                    bWaypoints = TRUE;
+                    bWaypoints = true;
                 else if (EQUAL(*papszIter, "tracks"))
-                    bTracks = TRUE;
+                    bTracks = true;
                 else if (EQUAL(*papszIter, "routes"))
-                    bRoutes = TRUE;
+                    bRoutes = true;
                 else
                 {
-                    CPLError(CE_Failure, CPLE_AppDefined, "Wrong value for 'features' options");
-                    bErr = TRUE;
+                    CPLError( CE_Failure, CPLE_AppDefined,
+                              "Wrong value for 'features' options");
+                    bErr = true;
                 }
-                papszIter ++;
+                papszIter++;
             }
             CSLDestroy(papszTokens);
             CPLFree(pszFeatures);
@@ -249,16 +254,17 @@ int OGRGPSBabelDataSource::Open( const char * pszDatasourceName, int bUpdateIn)
             pszSep = pszNextSep;
         }
 
-        pszFilename = CPLStrdup(pszSep+1);
+        if( pszFilename == NULL )
+            pszFilename = CPLStrdup(pszSep+1);
     }
 
     const char* pszOptionUseTempFile = CPLGetConfigOption("USE_TEMPFILE", NULL);
-    if (pszOptionUseTempFile && CSLTestBoolean(pszOptionUseTempFile))
+    if (pszOptionUseTempFile && CPLTestBool(pszOptionUseTempFile))
         osTmpFileName = CPLGenerateTempFilename(NULL);
     else
         osTmpFileName.Printf("/vsimem/ogrgpsbabeldatasource_%p", this);
 
-    int bRet = FALSE;
+    bool bRet = false;
     if (IsSpecialFile(pszFilename))
     {
         /* Special file : don't try to open it */
@@ -294,7 +300,7 @@ int OGRGPSBabelDataSource::Open( const char * pszDatasourceName, int bUpdateIn)
         argv = NULL;
 
         CPLErr nLastErrorType = CPLGetLastErrorType();
-        int nLastErrorNo = CPLGetLastErrorNo();
+        CPLErrorNum nLastErrorNo = CPLGetLastErrorNo();
         CPLString osLastErrorMsg = CPLGetLastErrorMsg();
 
         VSIFCloseL(tmpfp);
@@ -305,17 +311,21 @@ int OGRGPSBabelDataSource::Open( const char * pszDatasourceName, int bUpdateIn)
 
         if (!bRet)
         {
-            if (strstr(osLastErrorMsg.c_str(), "This format cannot be used in piped commands") == NULL)
+            if ( strstr(osLastErrorMsg.c_str(),
+                        "This format cannot be used in piped commands") == NULL)
             {
-                CPLError(nLastErrorType, nLastErrorNo, "%s", osLastErrorMsg.c_str());
+                CPLError( nLastErrorType, nLastErrorNo, "%s",
+                          osLastErrorMsg.c_str());
             }
             else
             {
                 VSIStatBuf sStatBuf;
                 if (VSIStat(pszFilename, &sStatBuf) != 0)
                 {
-                    CPLError(CE_Failure, CPLE_NotSupported,
-                            "Driver %s only supports real (non virtual) files", pszGPSBabelDriverName);
+                    CPLError( CE_Failure, CPLE_NotSupported,
+                              "Driver %s only supports real (non virtual) "
+                              "files",
+                              pszGPSBabelDriverName );
                     return FALSE;
                 }
 
@@ -336,21 +346,21 @@ int OGRGPSBabelDataSource::Open( const char * pszDatasourceName, int bUpdateIn)
 
     if (bRet)
     {
-        poGPXDS = OGRSFDriverRegistrar::Open(osTmpFileName.c_str());
+        poGPXDS = static_cast<GDALDataset *>(
+            GDALOpenEx( osTmpFileName.c_str(),
+                        GDAL_OF_VECTOR, NULL, NULL, NULL ) );
         if (poGPXDS)
         {
-            OGRLayer* poLayer;
-
             if (bWaypoints)
             {
-                poLayer = poGPXDS->GetLayerByName("waypoints");
+                OGRLayer* poLayer = poGPXDS->GetLayerByName("waypoints");
                 if (poLayer != NULL && poLayer->GetFeatureCount() != 0)
                     apoLayers[nLayers++] = poLayer;
             }
 
             if (bRoutes)
             {
-                poLayer = poGPXDS->GetLayerByName("routes");
+                OGRLayer* poLayer = poGPXDS->GetLayerByName("routes");
                 if (poLayer != NULL && poLayer->GetFeatureCount() != 0)
                     apoLayers[nLayers++] = poLayer;
                 poLayer = poGPXDS->GetLayerByName("route_points");
@@ -360,7 +370,7 @@ int OGRGPSBabelDataSource::Open( const char * pszDatasourceName, int bUpdateIn)
 
             if (bTracks)
             {
-                poLayer = poGPXDS->GetLayerByName("tracks");
+                OGRLayer* poLayer = poGPXDS->GetLayerByName("tracks");
                 if (poLayer != NULL && poLayer->GetFeatureCount() != 0)
                     apoLayers[nLayers++] = poLayer;
                 poLayer = poGPXDS->GetLayerByName("track_points");
@@ -377,8 +387,7 @@ int OGRGPSBabelDataSource::Open( const char * pszDatasourceName, int bUpdateIn)
 /*                           TestCapability()                           */
 /************************************************************************/
 
-int OGRGPSBabelDataSource::TestCapability( const char * pszCap )
-
+int OGRGPSBabelDataSource::TestCapability( const char * /* pszCap */ )
 {
     return FALSE;
 }
@@ -392,6 +401,6 @@ OGRLayer *OGRGPSBabelDataSource::GetLayer( int iLayer )
 {
     if( iLayer < 0 || iLayer >= nLayers )
         return NULL;
-    else
-        return apoLayers[iLayer];
+
+    return apoLayers[iLayer];
 }
