@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrhtfdriver.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: ogrhtfdriver.cpp 32744 2016-01-04 22:26:42Z goatbar $
  *
  * Project:  HTF Translator
  * Purpose:  Implements OGRHTFDriver.
@@ -27,42 +27,30 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "ogr_htf.h"
 #include "cpl_conv.h"
+#include "ogr_htf.h"
+#include "ogrsf_frmts.h"
 
-CPL_CVSID("$Id: ogrhtfdriver.cpp 27729 2014-09-24 00:40:16Z goatbar $");
-
-extern "C" void RegisterOGRHTF();
-
-/************************************************************************/
-/*                           ~OGRHTFDriver()                            */
-/************************************************************************/
-
-OGRHTFDriver::~OGRHTFDriver()
-
-{
-}
-
-/************************************************************************/
-/*                              GetName()                               */
-/************************************************************************/
-
-const char *OGRHTFDriver::GetName()
-
-{
-    return "HTF";
-}
+CPL_CVSID("$Id: ogrhtfdriver.cpp 32744 2016-01-04 22:26:42Z goatbar $");
 
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
 
-OGRDataSource *OGRHTFDriver::Open( const char * pszFilename, int bUpdate )
+static GDALDataset *OGRHTFDriverOpen( GDALOpenInfo* poOpenInfo )
 
 {
-    OGRHTFDataSource   *poDS = new OGRHTFDataSource();
+    if( poOpenInfo->eAccess == GA_Update ||
+        poOpenInfo->fpL == NULL )
+        return NULL;
 
-    if( !poDS->Open( pszFilename, bUpdate ) )
+    if( !STARTS_WITH( reinterpret_cast<char *>(poOpenInfo->pabyHeader),
+                      "HTF HEADER") )
+        return NULL;
+
+    OGRHTFDataSource *poDS = new OGRHTFDataSource();
+
+    if( !poDS->Open( poOpenInfo->pszFilename ) )
     {
         delete poDS;
         poDS = NULL;
@@ -71,15 +59,6 @@ OGRDataSource *OGRHTFDriver::Open( const char * pszFilename, int bUpdate )
     return poDS;
 }
 
-/************************************************************************/
-/*                           TestCapability()                           */
-/************************************************************************/
-
-int OGRHTFDriver::TestCapability( CPL_UNUSED const char * pszCap )
-{
-    return FALSE;
-}
-
 /************************************************************************/
 /*                           RegisterOGRHTF()                           */
 /************************************************************************/
@@ -87,6 +66,20 @@ int OGRHTFDriver::TestCapability( CPL_UNUSED const char * pszCap )
 void RegisterOGRHTF()
 
 {
-    OGRSFDriverRegistrar::GetRegistrar()->RegisterDriver( new OGRHTFDriver );
+    if( GDALGetDriverByName( "HTF" ) != NULL )
+        return;
+
+    GDALDriver *poDriver = new GDALDriver();
+
+    poDriver->SetDescription( "HTF" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VECTOR, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
+                                   "Hydrographic Transfer Vector" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drv_htf.html" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+
+    poDriver->pfnOpen = OGRHTFDriverOpen;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
 
