@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ili1reader.cpp 33076 2016-01-22 09:45:53Z pka $
+ * $Id: ili1reader.cpp 33829 2016-03-31 16:21:01Z rouault $
  *
  * Project:  Interlis 1 Reader
  * Purpose:  Implementation of ILI1Reader class.
@@ -28,10 +28,10 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "ogr_ili1.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
 #include "ogr_api.h"
+#include "ogr_ili1.h"
 #include "ogr_geos.h"
 
 #include "ili1reader.h"
@@ -45,33 +45,32 @@
 
 #ifndef POLYGONIZE_AREAS
 #  if defined(__GNUC_PREREQ)
-#    warning Interlis 1 Area polygonizing disabled. Needs GEOS >= 3.1.0
+//#    warning Interlis 1 Area polygonizing disabled. Needs GEOS >= 3.1.0
 #  endif
 #endif
 
-CPL_CVSID("$Id: ili1reader.cpp 33076 2016-01-22 09:45:53Z pka $");
+CPL_CVSID("$Id: ili1reader.cpp 33829 2016-03-31 16:21:01Z rouault $");
 
 //
 // ILI1Reader
 //
-IILI1Reader::~IILI1Reader() {
-}
+IILI1Reader::~IILI1Reader() {}
 
-ILI1Reader::ILI1Reader() {
-  fpItf = NULL;
-  nLayers = 0;
-  papoLayers = NULL;
-  curLayer = NULL;
-  codeBlank = '_';
-  codeUndefined = '@';
-  codeContinue = '\\';
-}
+ILI1Reader::ILI1Reader() :
+  fpItf(NULL),
+  nLayers(0),
+  papoLayers(NULL),
+  curLayer(NULL),
+  codeBlank('_'),
+  codeUndefined('@'),
+  codeContinue('\\')
+{}
 
-ILI1Reader::~ILI1Reader() {
-  int i;
+ILI1Reader::~ILI1Reader()
+{
   if (fpItf) VSIFClose( fpItf );
 
-  for(i=0;i<nLayers;i++)
+  for( int i=0; i < nLayers; i++)
      delete papoLayers[i];
   CPLFree(papoLayers);
 }
@@ -79,7 +78,8 @@ ILI1Reader::~ILI1Reader() {
 /* -------------------------------------------------------------------- */
 /*      Open the source file.                                           */
 /* -------------------------------------------------------------------- */
-int ILI1Reader::OpenFile( const char *pszFilename ) {
+int ILI1Reader::OpenFile( const char *pszFilename )
+{
     fpItf = VSIFOpen( pszFilename, "r" );
     if( fpItf == NULL )
     {
@@ -92,31 +92,48 @@ int ILI1Reader::OpenFile( const char *pszFilename ) {
     return TRUE;
 }
 
-const char* ILI1Reader::GetLayerNameString(const char* topicname, const char* tablename) {
+const char* ILI1Reader::GetLayerNameString( const char* topicname,
+                                            const char* tablename)
+{
 
     return CPLSPrintf("%s__%s", topicname, tablename);
 }
 
-int ILI1Reader::ReadModel(ImdReader *poImdReader, const char *pszModelFilename, OGRILI1DataSource *poDS) {
+int ILI1Reader::ReadModel( ImdReader *poImdReader,
+                           const char *pszModelFilename,
+                           OGRILI1DataSource *poDS)
+{
 
   poImdReader->ReadModel(pszModelFilename);
   for (FeatureDefnInfos::const_iterator it = poImdReader->featureDefnInfos.begin(); it != poImdReader->featureDefnInfos.end(); ++it)
   {
-    //CPLDebug( "OGR_ILI", "Adding OGRILI1Layer with table '%s'", it->poTableDefn->GetName() );
-    OGRILI1Layer* layer = new OGRILI1Layer(it->poTableDefn, it->poGeomFieldInfos, poDS);
+#if DEBUG_VERBOSE
+    CPLDebug( "OGR_ILI", "Adding OGRILI1Layer with table '%s'",
+              it->poTableDefn->GetName() );
+#endif
+    OGRILI1Layer* layer = new OGRILI1Layer( it->GetTableDefnRef(),
+                                            it->poGeomFieldInfos, poDS);
     AddLayer(layer);
-    //Create additional layers for surface and area geometries
-    for (GeomFieldInfos::const_iterator it2 = it->poGeomFieldInfos.begin(); it2 != it->poGeomFieldInfos.end(); ++it2)
+    // Create additional layers for surface and area geometries.
+    for (GeomFieldInfos::const_iterator it2 = it->poGeomFieldInfos.begin();
+         it2 != it->poGeomFieldInfos.end();
+         ++it2)
     {
-      if (it2->second.geomTable)
+      if (it2->second.GetGeomTableDefnRef())
       {
-        OGRFeatureDefn* poGeomTableDefn = it2->second.geomTable;
-        OGRGeomFieldDefn* poOGRGeomFieldDefn = poGeomTableDefn->GetGeomFieldDefn(0);
+        OGRFeatureDefn* poGeomTableDefn = it2->second.GetGeomTableDefnRef();
+        OGRGeomFieldDefn* poOGRGeomFieldDefn
+            = poGeomTableDefn->GetGeomFieldDefn(0);
         GeomFieldInfos oGeomFieldInfos;
         // We add iliGeomType to recognize Ili1 geom tables
-        oGeomFieldInfos[poOGRGeomFieldDefn->GetNameRef()].iliGeomType = it2->second.iliGeomType;
-        //CPLDebug( "OGR_ILI", "Adding OGRILI1Layer with geometry table '%s'", it2->second.geomTable->GetName() );
-        OGRILI1Layer* geomlayer = new OGRILI1Layer(poGeomTableDefn, oGeomFieldInfos, poDS);
+        oGeomFieldInfos[poOGRGeomFieldDefn->GetNameRef()].iliGeomType
+            = it2->second.iliGeomType;
+#if DEBUG_VERBOSE
+        CPLDebug( "OGR_ILI", "Adding OGRILI1Layer with geometry table '%s'",
+                  it2->second.geomTable->GetName() );
+#endif
+        OGRILI1Layer* geomlayer
+            = new OGRILI1Layer(poGeomTableDefn, oGeomFieldInfos, poDS);
         AddLayer(geomlayer);
       }
     }
@@ -125,22 +142,23 @@ int ILI1Reader::ReadModel(ImdReader *poImdReader, const char *pszModelFilename,
   codeBlank = poImdReader->codeBlank;
   CPLDebug( "OGR_ILI", "Ili1Format blankCode '%c'", poImdReader->codeBlank );
   codeUndefined = poImdReader->codeUndefined;
-  CPLDebug( "OGR_ILI", "Ili1Format undefinedCode '%c'", poImdReader->codeUndefined );
+  CPLDebug( "OGR_ILI", "Ili1Format undefinedCode '%c'",
+            poImdReader->codeUndefined );
   codeContinue = poImdReader->codeContinue;
-  CPLDebug( "OGR_ILI", "Ili1Format continueCode '%c'", poImdReader->codeContinue );
+  CPLDebug( "OGR_ILI", "Ili1Format continueCode '%c'",
+            poImdReader->codeContinue );
   return 0;
 }
 
 int ILI1Reader::ReadFeatures() {
     char **tokens = NULL;
-    const char *firsttok = NULL;
-    const char *pszLine;
-    char *topic = NULL;
+    const char *pszLine = NULL;
+    char *topic = CPLStrdup("(null)");
     int ret = TRUE;
 
-    while (ret && (tokens = ReadParseLine()))
+    while (ret && (tokens = ReadParseLine()) != NULL)
     {
-      firsttok = tokens[0];
+      const char *firsttok = tokens[0];
       if (EQUAL(firsttok, "SCNT"))
       {
         //read description
@@ -148,7 +166,7 @@ int ILI1Reader::ReadFeatures() {
         {
           pszLine = CPLReadLine( fpItf );
         }
-        while (pszLine && !EQUALN(pszLine, "////", 4));
+        while (pszLine && !STARTS_WITH_CI(pszLine, "////"));
         ret = (pszLine != NULL);
       }
       else if (EQUAL(firsttok, "MOTR"))
@@ -158,7 +176,7 @@ int ILI1Reader::ReadFeatures() {
         {
           pszLine = CPLReadLine( fpItf );
         }
-        while (pszLine && !EQUALN(pszLine, "////", 4));
+        while (pszLine && !STARTS_WITH_CI(pszLine, "////"));
         ret = (pszLine != NULL);
       }
       else if (EQUAL(firsttok, "MTID"))
@@ -167,21 +185,25 @@ int ILI1Reader::ReadFeatures() {
       else if (EQUAL(firsttok, "MODL"))
       {
       }
-      else if (EQUAL(firsttok, "TOPI"))
+      else if (EQUAL(firsttok, "TOPI") && CSLCount(tokens) >= 2)
       {
         CPLFree(topic);
         topic = CPLStrdup(CSLGetField(tokens, 1));
       }
-      else if (EQUAL(firsttok, "TABL"))
+      else if (EQUAL(firsttok, "TABL") && CSLCount(tokens) >= 2)
       {
-        const char *layername = GetLayerNameString(topic, CSLGetField(tokens, 1));
+        const char *layername
+            = GetLayerNameString(topic, CSLGetField(tokens, 1));
         CPLDebug( "OGR_ILI", "Reading table '%s'", layername );
         curLayer = GetLayerByName(layername);
 
         if (curLayer == NULL) { //create one
-          CPLError(CE_Warning, CPLE_AppDefined,
-              "No model definition for table '%s' found, using default field names.", layername );
-          OGRFeatureDefn* poFeatureDefn = new OGRFeatureDefn(GetLayerNameString(topic, CSLGetField(tokens, 1)));
+          CPLError( CE_Warning, CPLE_AppDefined,
+                    "No model definition for table '%s' found, "
+                    "using default field names.", layername );
+          OGRFeatureDefn* poFeatureDefn
+            = new OGRFeatureDefn(
+                GetLayerNameString(topic, CSLGetField(tokens, 1)));
           poFeatureDefn->SetGeomType( wkbUnknown );
           GeomFieldInfos oGeomFieldInfos;
           curLayer = new OGRILI1Layer(poFeatureDefn, oGeomFieldInfos, NULL);
@@ -189,7 +211,8 @@ int ILI1Reader::ReadFeatures() {
         }
         if(curLayer != NULL) {
           for (int i=0; i < curLayer->GetLayerDefn()->GetFieldCount(); i++) {
-            CPLDebug( "OGR_ILI", "Field %d: %s", i,  curLayer->GetLayerDefn()->GetFieldDefn(i)->GetNameRef());
+            CPLDebug( "OGR_ILI", "Field %d: %s", i,
+                      curLayer->GetLayerDefn()->GetFieldDefn(i)->GetNameRef());
           }
         }
         ret = ReadTable(layername);
@@ -208,7 +231,8 @@ int ILI1Reader::ReadFeatures() {
       }
       else
       {
-        CPLError(CE_Warning, CPLE_AppDefined, "Unexpected token: %s", firsttok );
+        CPLError( CE_Warning, CPLE_AppDefined,
+                  "Unexpected token: %s", firsttok );
       }
 
       CSLDestroy(tokens);
@@ -223,46 +247,56 @@ int ILI1Reader::ReadFeatures() {
 
 int ILI1Reader::ReadTable(CPL_UNUSED const char *layername) {
     char **tokens = NULL;
-    const char *firsttok = NULL;
     int ret = TRUE;
     int warned = FALSE;
-    int fIndex;
     int geomIdx = -1;
 
     OGRFeatureDefn *featureDef = curLayer->GetLayerDefn();
     OGRFeature *feature = NULL;
+    bool bFeatureAdded = false;
 
-    while (ret && (tokens = ReadParseLine()))
+    while (ret && (tokens = ReadParseLine()) != NULL)
     {
-      firsttok = CSLGetField(tokens, 0);
+      const char *firsttok = CSLGetField(tokens, 0);
       if (EQUAL(firsttok, "OBJE"))
       {
         if (featureDef->GetFieldCount() == 0)
         {
-          CPLError(CE_Warning, CPLE_AppDefined,
-              "No field definition found for table: %s", featureDef->GetName() );
-          //Model not read - use heuristics
-          for (fIndex=1; fIndex<CSLCount(tokens); fIndex++)
+          CPLError( CE_Warning, CPLE_AppDefined,
+                    "No field definition found for table: %s",
+                    featureDef->GetName() );
+          // Model not read - use heuristics.
+          for( int fIndex=1; fIndex<CSLCount(tokens); fIndex++ )
           {
             char szFieldName[32];
-            sprintf(szFieldName, "Field%02d", fIndex);
+            snprintf(szFieldName, sizeof(szFieldName), "Field%02d", fIndex);
             OGRFieldDefn oFieldDefn(szFieldName, OFTString);
             featureDef->AddFieldDefn(&oFieldDefn);
           }
         }
         //start new feature
+        if( !bFeatureAdded )
+            delete feature;
         feature = new OGRFeature(featureDef);
 
-        int fieldno = 0;
-        for (fIndex=1; fIndex<CSLCount(tokens) && fieldno < featureDef->GetFieldCount(); fIndex++, fieldno++)
+        for( int fIndex=1, fieldno = 0;
+             fIndex<CSLCount(tokens) && fieldno < featureDef->GetFieldCount();
+             fIndex++, fieldno++ )
         {
           if (!(tokens[fIndex][0] == codeUndefined && tokens[fIndex][1] == '\0')) {
-            //CPLDebug( "READ TABLE OGR_ILI", "Setting Field %d (Type %d): %s", fieldno, featureDef->GetFieldDefn(fieldno)->GetType(), tokens[fIndex]);
+#ifdef DEBUG_VERBOSE
+            CPLDebug( "READ TABLE OGR_ILI", "Setting Field %d (Type %d): %s",
+                      fieldno, featureDef->GetFieldDefn(fieldno)->GetType(),
+                      tokens[fIndex] );
+#endif
             if (featureDef->GetFieldDefn(fieldno)->GetType() == OFTString) {
-                //Interlis 1 encoding is ISO 8859-1 (Latin1) -> Recode to UTF-8
-                char* pszRecoded = CPLRecode(tokens[fIndex], CPL_ENC_ISO8859_1, CPL_ENC_UTF8);
-                //Replace space marks
-                for(char* pszString = pszRecoded; *pszString != '\0'; pszString++ ) {
+                // Interlis 1 encoding is ISO 8859-1 (Latin1) -> Recode to UTF-8
+                char* pszRecoded = CPLRecode(
+                    tokens[fIndex], CPL_ENC_ISO8859_1, CPL_ENC_UTF8);
+                // Replace space marks
+                for( char* pszString = pszRecoded;
+                     *pszString != '\0';
+                     pszString++ ) {
                     if (*pszString == codeBlank) *pszString = ' ';
                 }
                 feature->SetField(fieldno, pszRecoded);
@@ -273,31 +307,42 @@ int ILI1Reader::ReadTable(CPL_UNUSED const char *layername) {
             if (featureDef->GetFieldDefn(fieldno)->GetType() == OFTReal
                 && fieldno > 0
                 && featureDef->GetFieldDefn(fieldno-1)->GetType() == OFTReal) {
-              //check for Point geometry (Coord type)
-              // if there is no ili model read,
+              // Check for Point geometry (Coord type).
+              // If there is no ili model read,
               // we have no chance to detect the
-              // geometry column!!
-              CPLString geomfldname = featureDef->GetFieldDefn(fieldno)->GetNameRef();
-              //Check if name ends with _1
-              if (geomfldname.size() >= 2 && geomfldname[geomfldname.size()-2] == '_') {
+              // geometry column.
+              CPLString geomfldname
+                  = featureDef->GetFieldDefn(fieldno)->GetNameRef();
+              // Check if name ends with _1.
+              if (geomfldname.size() >= 2 && geomfldname[geomfldname.size()-2]
+                  == '_') {
                 geomfldname = geomfldname.substr(0, geomfldname.size()-2);
                 geomIdx = featureDef->GetGeomFieldIndex(geomfldname.c_str());
                 if (geomIdx == -1)
                 {
-                  CPLError(CE_Warning, CPLE_AppDefined,
-                      "No matching definition for field '%s' of table %s found", geomfldname.c_str(), featureDef->GetName());
+                  CPLError( CE_Warning, CPLE_AppDefined,
+                            "No matching definition for field '%s' of "
+                            "table %s found",
+                            geomfldname.c_str(), featureDef->GetName() );
                 }
               } else {
                 geomIdx = -1;
               }
               if (geomIdx >= 0) {
-                if (featureDef->GetGeomFieldDefn(geomIdx)->GetType() == wkbPoint) {
-                  //add Point geometry
-                  OGRPoint *ogrPoint = new OGRPoint(CPLAtof(tokens[fIndex-1]), CPLAtof(tokens[fIndex]));
+                if (featureDef->GetGeomFieldDefn(geomIdx)->GetType() ==
+                    wkbPoint) {
+                  // Add Point geometry.
+                  OGRPoint *ogrPoint = new OGRPoint(
+                      CPLAtof(tokens[fIndex-1]), CPLAtof(tokens[fIndex]));
                   feature->SetGeomFieldDirectly(geomIdx, ogrPoint);
-                } else if (featureDef->GetGeomFieldDefn(geomIdx)->GetType() == wkbPoint25D && fieldno > 1 && featureDef->GetFieldDefn(fieldno-2)->GetType() == OFTReal) {
-                  //add 3D Point geometry
-                  OGRPoint *ogrPoint = new OGRPoint(CPLAtof(tokens[fIndex-2]), CPLAtof(tokens[fIndex-1]), CPLAtof(tokens[fIndex]));
+                } else if (featureDef->GetGeomFieldDefn(geomIdx)->GetType() ==
+                           wkbPoint25D && fieldno > 1 &&
+                           featureDef->GetFieldDefn(fieldno-2)->GetType() ==
+                           OFTReal) {
+                  // Add 3D Point geometry.
+                  OGRPoint *ogrPoint = new OGRPoint(
+                      CPLAtof(tokens[fIndex-2]), CPLAtof(tokens[fIndex-1]),
+                      CPLAtof(tokens[fIndex]) );
                   feature->SetGeomFieldDirectly(geomIdx, ogrPoint);
                 }
               }
@@ -305,34 +350,47 @@ int ILI1Reader::ReadTable(CPL_UNUSED const char *layername) {
           }
         }
         if (!warned && featureDef->GetFieldCount() != CSLCount(tokens)-1) {
-          CPLError(CE_Warning, CPLE_AppDefined,
-              "Field count of table %s doesn't match. %d declared, %d found (e.g. ignored LINEATTR)", featureDef->GetName(), featureDef->GetFieldCount(), CSLCount(tokens)-1);
+          CPLError( CE_Warning, CPLE_AppDefined,
+                    "Field count of table %s doesn't match. %d declared, "
+                    "%d found (e.g. ignored LINEATTR)",
+                    featureDef->GetName(), featureDef->GetFieldCount(),
+                    CSLCount(tokens) - 1 );
           warned = TRUE;
         }
         if (feature->GetFieldCount() > 0) {
-          // USE _TID as FID. TODO: respect IDENT field from model
+          // USE _TID as FID. TODO: respect IDENT field from model.
           feature->SetFID(feature->GetFieldAsInteger64(0));
         }
         curLayer->AddFeature(feature);
+        bFeatureAdded = true;
         geomIdx = -1; //Reset
       }
-      else if (EQUAL(firsttok, "STPT"))
+      else if (EQUAL(firsttok, "STPT") && feature != NULL)
       {
         //Find next non-Point geometry
         if (geomIdx < 0) geomIdx = 0;
-        while (geomIdx < featureDef->GetGeomFieldCount() && featureDef->GetGeomFieldDefn(geomIdx)->GetType() == wkbPoint) { geomIdx++; }
-        OGRwkbGeometryType geomType = (geomIdx < featureDef->GetGeomFieldCount()) ? featureDef->GetGeomFieldDefn(geomIdx)->GetType() : wkbNone;
+        while (geomIdx < featureDef->GetGeomFieldCount() &&
+               featureDef->GetGeomFieldDefn(geomIdx)->GetType() == wkbPoint) {
+            geomIdx++;
+        }
+        OGRwkbGeometryType geomType
+            = (geomIdx < featureDef->GetGeomFieldCount()) ?
+               featureDef->GetGeomFieldDefn(geomIdx)->GetType() : wkbNone;
         ReadGeom(tokens, geomIdx, geomType, feature);
       }
       else if (EQUAL(firsttok, "ELIN"))
       {
-        //empty geom
+        // Empty geom.
       }
-      else if (EQUAL(firsttok, "EDGE"))
+      else if (EQUAL(firsttok, "EDGE") && feature != NULL)
       {
+        CSLDestroy(tokens);
         tokens = ReadParseLine(); //STPT
         //Find next non-Point geometry
-        do { geomIdx++; } while (geomIdx < featureDef->GetGeomFieldCount() && featureDef->GetGeomFieldDefn(geomIdx)->GetType() == wkbPoint);
+        do {
+            geomIdx++;
+        } while (geomIdx < featureDef->GetGeomFieldCount() &&
+                 featureDef->GetGeomFieldDefn(geomIdx)->GetType() == wkbPoint);
         ReadGeom(tokens, geomIdx, wkbMultiLineString, feature);
       }
       else if (EQUAL(firsttok, "PERI"))
@@ -340,41 +398,46 @@ int ILI1Reader::ReadTable(CPL_UNUSED const char *layername) {
       }
       else if (EQUAL(firsttok, "ETAB"))
       {
-        CPLDebug( "OGR_ILI", "Total features: " CPL_FRMT_GIB, curLayer->GetFeatureCount() );
+        CPLDebug( "OGR_ILI", "Total features: " CPL_FRMT_GIB,
+                  curLayer->GetFeatureCount() );
         CSLDestroy(tokens);
+        if( !bFeatureAdded )
+            delete feature;
         return TRUE;
       }
       else
       {
-        CPLError(CE_Warning, CPLE_AppDefined, "Unexpected token: %s", firsttok );
+        CPLError( CE_Warning, CPLE_AppDefined,
+                  "Unexpected token: %s", firsttok );
       }
 
       CSLDestroy(tokens);
     }
 
+    if( !bFeatureAdded )
+        delete feature;
+
     return ret;
 }
 
-void ILI1Reader::ReadGeom(char **stgeom, int geomIdx, OGRwkbGeometryType eType, OGRFeature *feature) {
-    char **tokens = NULL;
-    const char *firsttok = NULL;
-    int end = FALSE;
-    OGRCompoundCurve *ogrCurve = NULL; //current compound curve
-    OGRLineString *ogrLine = NULL; //current line
-    OGRCircularString *arc = NULL; //current arc
-    OGRCurvePolygon *ogrPoly = NULL; //current polygon
-    OGRPoint ogrPoint; //current point
-    OGRMultiCurve *ogrMultiLine = NULL; //current multi line
-
-    //CPLDebug( "OGR_ILI", "ILI1Reader::ReadGeom geomIdx: %d OGRGeometryType: %s", geomIdx, OGRGeometryTypeToName(eType));
+void ILI1Reader::ReadGeom( char **stgeom, int geomIdx, OGRwkbGeometryType eType,
+                           OGRFeature *feature ) {
+#ifdef DEBUG_VERBOSE
+    CPLDebug( "OGR_ILI",
+              "ILI1Reader::ReadGeom geomIdx: %d OGRGeometryType: %s",
+              geomIdx, OGRGeometryTypeToName(eType) );
+#endif
     if (eType == wkbNone)
     {
-      CPLError(CE_Warning, CPLE_AppDefined, "Calling ILI1Reader::ReadGeom with wkbNone" );
+      CPLError( CE_Warning, CPLE_AppDefined,
+                "Calling ILI1Reader::ReadGeom with wkbNone" );
     }
 
-    //Initialize geometry
+    // Initialize geometry.
 
-    ogrCurve = new OGRCompoundCurve();
+    OGRCompoundCurve *ogrCurve = new OGRCompoundCurve();
+    OGRCurvePolygon *ogrPoly = NULL; //current polygon
+    OGRMultiCurve *ogrMultiLine = NULL; //current multi line
 
     if (eType == wkbMultiCurve || eType == wkbMultiLineString)
     {
@@ -385,15 +448,21 @@ void ILI1Reader::ReadGeom(char **stgeom, int geomIdx, OGRwkbGeometryType eType,
       ogrPoly = new OGRCurvePolygon();
     }
 
-    //tokens = ["STPT", "1111", "22222"]
+    OGRPoint ogrPoint; // Current point.
     ogrPoint.setX(CPLAtof(stgeom[1])); ogrPoint.setY(CPLAtof(stgeom[2]));
-    ogrLine = new OGRLineString();
+
+    OGRLineString *ogrLine = new OGRLineString();
     ogrLine->addPoint(&ogrPoint);
 
-    //Parse geometry
-    while (!end && (tokens = ReadParseLine()))
+    // Parse geometry.
+
+    char **tokens = NULL;
+    bool end = false;
+    OGRCircularString *arc = NULL; //current arc
+
+    while (!end && (tokens = ReadParseLine()) != NULL)
     {
-      firsttok = CSLGetField(tokens, 0);
+      const char *firsttok = CSLGetField(tokens, 0);
       if (EQUAL(firsttok, "LIPT"))
       {
         ogrPoint.setX(CPLAtof(tokens[1])); ogrPoint.setY(CPLAtof(tokens[2]));
@@ -431,6 +500,7 @@ void ILI1Reader::ReadGeom(char **stgeom, int geomIdx, OGRwkbGeometryType eType,
           if (error != OGRERR_NONE) {
             CPLError(CE_Warning, CPLE_AppDefined, "Added geometry: %s", ogrLine->exportToJson() );
           }
+          ogrLine = NULL;
         }
         if (!ogrCurve->IsEmpty()) {
           if (ogrMultiLine)
@@ -439,6 +509,7 @@ void ILI1Reader::ReadGeom(char **stgeom, int geomIdx, OGRwkbGeometryType eType,
             if (error != OGRERR_NONE) {
               CPLError(CE_Warning, CPLE_AppDefined, "Added geometry: %s", ogrCurve->exportToJson() );
             }
+            ogrCurve = NULL;
           }
           if (ogrPoly)
           {
@@ -446,13 +517,14 @@ void ILI1Reader::ReadGeom(char **stgeom, int geomIdx, OGRwkbGeometryType eType,
             if (error != OGRERR_NONE) {
               CPLError(CE_Warning, CPLE_AppDefined, "Added geometry: %s", ogrCurve->exportToJson() );
             }
+            ogrCurve = NULL;
           }
         }
-        end = TRUE;
+        end = true;
       }
       else if (EQUAL(firsttok, "EEDG"))
       {
-        end = TRUE;
+        end = true;
       }
       else if (EQUAL(firsttok, "LATT"))
       {
@@ -460,38 +532,45 @@ void ILI1Reader::ReadGeom(char **stgeom, int geomIdx, OGRwkbGeometryType eType,
       }
       else if (EQUAL(firsttok, "EFLA"))
       {
-        end = TRUE;
+        end = true;
       }
       else if (EQUAL(firsttok, "ETAB"))
       {
-        end = TRUE;
+        end = true;
       }
       else
       {
-        CPLError(CE_Warning, CPLE_AppDefined, "Unexpected token: %s", firsttok );
+        CPLError( CE_Warning, CPLE_AppDefined,
+                  "Unexpected token: %s", firsttok );
       }
 
       CSLDestroy(tokens);
     }
 
+    delete ogrLine;
+
     //Set feature geometry
     if (eType == wkbMultiCurve)
     {
       feature->SetGeomFieldDirectly(geomIdx, ogrMultiLine);
+      delete ogrCurve;
     }
     else if (eType == wkbMultiLineString)
     {
       feature->SetGeomFieldDirectly(geomIdx, ogrMultiLine->getLinearGeometry());
       delete ogrMultiLine;
+      delete ogrCurve;
     }
     else if (eType == wkbCurvePolygon)
     {
       feature->SetGeomFieldDirectly(geomIdx, ogrPoly);
+      delete ogrCurve;
     }
     else if (eType == wkbPolygon)
     {
       feature->SetGeomFieldDirectly(geomIdx, ogrPoly->getLinearGeometry());
       delete ogrPoly;
+      delete ogrCurve;
     }
     else
     {
@@ -508,8 +587,8 @@ void ILI1Reader::AddLayer( OGRILI1Layer * poNewLayer )
 {
     nLayers++;
 
-    papoLayers = (OGRILI1Layer **)
-        CPLRealloc( papoLayers, sizeof(void*) * nLayers );
+    papoLayers = static_cast<OGRILI1Layer **>(
+        CPLRealloc( papoLayers, sizeof(void*) * nLayers ) );
 
     papoLayers[nLayers-1] = poNewLayer;
 }
@@ -523,8 +602,8 @@ OGRILI1Layer *ILI1Reader::GetLayer( int iLayer )
 {
     if( iLayer < 0 || iLayer >= nLayers )
         return NULL;
-    else
-        return papoLayers[iLayer];
+
+    return papoLayers[iLayer];
 }
 
 OGRILI1Layer *ILI1Reader::GetLayerByName( const char* pszLayerName )
@@ -556,23 +635,18 @@ int ILI1Reader::GetLayerCount()
 
 char ** ILI1Reader::ReadParseLine()
 {
-    const char  *pszLine;
-    char **tokens;
-    char **conttok;
-    char *token;
-
     CPLAssert( fpItf != NULL );
     if( fpItf == NULL )
         return( NULL );
 
-    pszLine = CPLReadLine( fpItf );
+    const char  *pszLine = CPLReadLine( fpItf );
     if( pszLine == NULL )
         return( NULL );
 
     if (strlen(pszLine) == 0) return NULL;
 
-    tokens = CSLTokenizeString2( pszLine, " ", CSLT_PRESERVEESCAPES );
-    token = tokens[CSLCount(tokens)-1];
+    char **tokens = CSLTokenizeString2( pszLine, " ", CSLT_PRESERVEESCAPES );
+    char *token = tokens[CSLCount(tokens)-1];
 
     //Append CONT lines
     while (strlen(pszLine) && token[0] == codeContinue && token[1] == '\0')
@@ -582,7 +656,11 @@ char ** ILI1Reader::ReadParseLine()
       tokens[CSLCount(tokens)-1] = NULL;
 
       pszLine = CPLReadLine( fpItf );
-      conttok = CSLTokenizeString2( pszLine, " ", CSLT_PRESERVEESCAPES );
+      if( pszLine == NULL )
+      {
+          break;
+      }
+      char **conttok = CSLTokenizeString2( pszLine, " ", CSLT_PRESERVEESCAPES );
       if (!conttok || !EQUAL(conttok[0], "CONT"))
       {
           CSLDestroy(conttok);
