@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ili2handler.cpp 27741 2014-09-26 19:20:02Z goatbar $
+ * $Id: ili2handler.cpp 33109 2016-01-23 16:25:42Z rouault $
  *
  * Project:  Interlis 2 Reader
  * Purpose:  Implementation of ILI2Handler class.
@@ -28,55 +28,56 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "ogr_ili2.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
 #include "ili2readerp.h"
+#include "ogr_ili2.h"
+
 #include <xercesc/sax2/Attributes.hpp>
 
-CPL_CVSID("$Id: ili2handler.cpp 27741 2014-09-26 19:20:02Z goatbar $");
+CPL_CVSID("$Id: ili2handler.cpp 33109 2016-01-23 16:25:42Z rouault $");
 
-// 
+//
 // constants
-// 
-static const char* ILI2_DATASECTION = "DATASECTION";
+//
+static const char* const ILI2_DATASECTION = "DATASECTION";
 
 //
 // ILI2Handler
-// 
-ILI2Handler::ILI2Handler( ILI2Reader *poReader ) {
+//
+ILI2Handler::ILI2Handler( ILI2Reader *poReader ) :
+    level(0),
+    m_nEntityCounter(0)
+{
   m_poReader = poReader;
-  
+
   XMLCh *tmpCh = XMLString::transcode("CORE");
   DOMImplementation *impl = DOMImplementationRegistry::getDOMImplementation(tmpCh);
   XMLString::release(&tmpCh);
 
   // the root element
   tmpCh = XMLString::transcode("ROOT");
-  dom_doc = impl->createDocument(0,tmpCh,0);
+  dom_doc = impl->createDocument(NULL,tmpCh,NULL);
   XMLString::release(&tmpCh);
 
   // the first element is root
   dom_elem = dom_doc->getDocumentElement();
-
 }
 
 ILI2Handler::~ILI2Handler() {
-  
   // remove all elements
   DOMNode *tmpNode = dom_doc->getFirstChild();
   while (tmpNode != NULL) {
-    tmpNode = dom_doc->removeChild(tmpNode);
+    /*tmpNode = */dom_doc->removeChild(tmpNode);
     tmpNode = dom_doc->getFirstChild();
   }
-  
+
   // release the dom tree
   dom_doc->release();
-
 }
 
-    
+
 void ILI2Handler::startDocument() {
   // the level counter starts with DATASECTION
   level = -1;
@@ -88,24 +89,26 @@ void ILI2Handler::endDocument() {
 }
 
 void ILI2Handler::startElement(
-    CPL_UNUSED const XMLCh* const uri,
-    CPL_UNUSED const XMLCh* const localname,
+    const XMLCh* const /* uri */,
+    const XMLCh* const /* localname */,
     const XMLCh* const qname,
     const Attributes& attrs
     ) {
   // start to add the layers, features with the DATASECTION
   char *tmpC = NULL;
   m_nEntityCounter = 0;
-  if ((level >= 0) || (cmpStr(ILI2_DATASECTION, tmpC = XMLString::transcode(qname)) == 0)) {
+  if ((level >= 0) || (cmpStr(ILI2_DATASECTION,
+                              tmpC = XMLString::transcode(qname)) == 0)) {
     level++;
 
     if (level >= 2) {
 
       // create the dom tree
-      DOMElement *elem = (DOMElement*)dom_doc->createElement(qname);
-      
+      DOMElement *elem = reinterpret_cast<DOMElement *>(
+          dom_doc->createElement(qname) );
+
       // add all attributes
-      unsigned int len = attrs.getLength();
+      unsigned int len = static_cast<unsigned int>(attrs.getLength());
       for (unsigned int index = 0; index < len; index++)
         elem->setAttribute(attrs.getQName(index), attrs.getValue(index));
       dom_elem->appendChild(elem);
@@ -131,11 +134,11 @@ void ILI2Handler::endElement(
       m_poReader->AddFeature(childElem);
 
       // remove the child element
-      childElem = (DOMElement*)dom_elem->removeChild(childElem);
+      /*childElem = (DOMElement*)*/dom_elem->removeChild(childElem);
     } else if (level >= 3) {
 
       // go to the parent element
-      dom_elem = (DOMElement*)dom_elem->getParentNode();
+      dom_elem = reinterpret_cast<DOMElement *>( dom_elem->getParentNode() );
     }
     level--;
   }
@@ -166,12 +169,12 @@ void ILI2Handler::characters( const XMLCh *const chars,
 /************************************************************************/
 
 void ILI2Handler::characters( const XMLCh *const chars,
-                     const unsigned int length ) {
-  
+                     CPL_UNUSED const unsigned int length ) {
+
   // add the text element
   if (level >= 3) {
     char *tmpC = XMLString::transcode(chars);
-    
+
     // only add the text if it is not empty
     if (trim(tmpC) != "")
       dom_elem->appendChild(dom_doc->createTextNode(chars));
@@ -186,7 +189,8 @@ void ILI2Handler::startEntity (CPL_UNUSED const XMLCh *const name)
     m_nEntityCounter++;
     if (m_nEntityCounter > 1000)
     {
-        throw SAXNotSupportedException ("File probably corrupted (million laugh pattern)");
+        throw SAXNotSupportedException (
+            "File probably corrupted (million laugh pattern)" );
     }
 }
 
