@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrili1datasource.cpp 27741 2014-09-26 19:20:02Z goatbar $
+ * $Id: ogrili1datasource.cpp 32746 2016-01-05 05:18:43Z goatbar $
  *
  * Project:  Interlis 1 Translator
  * Purpose:  Implements OGRILI1DataSource class.
@@ -28,31 +28,29 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "ogr_ili1.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
 #include "ili1reader.h"
+#include "ogr_ili1.h"
 
 #include <string>
 
-CPL_CVSID("$Id: ogrili1datasource.cpp 27741 2014-09-26 19:20:02Z goatbar $");
+CPL_CVSID("$Id: ogrili1datasource.cpp 32746 2016-01-05 05:18:43Z goatbar $");
 
 /************************************************************************/
 /*                         OGRILI1DataSource()                         */
 /************************************************************************/
 
-OGRILI1DataSource::OGRILI1DataSource()
-
-{
-    pszName = NULL;
-    poImdReader = new ImdReader(1);
-    poReader = NULL;
-    fpTransfer = NULL;
-    pszTopic = NULL;
-    nLayers = 0;
-    papoLayers = NULL;
-}
+OGRILI1DataSource::OGRILI1DataSource() :
+    pszName(NULL),
+    poImdReader(new ImdReader(1)),
+    poReader(NULL),
+    fpTransfer(NULL),
+    pszTopic(NULL),
+    nLayers(0),
+    papoLayers(NULL)
+{}
 
 /************************************************************************/
 /*                        ~OGRILI1DataSource()                         */
@@ -61,9 +59,7 @@ OGRILI1DataSource::OGRILI1DataSource()
 OGRILI1DataSource::~OGRILI1DataSource()
 
 {
-    int i;
-
-    for(i=0;i<nLayers;i++)
+    for( int i=0; i<nLayers; i++)
     {
         delete papoLayers[i];
     }
@@ -87,31 +83,38 @@ OGRILI1DataSource::~OGRILI1DataSource()
 /*                                Open()                                */
 /************************************************************************/
 
-int OGRILI1DataSource::Open( const char * pszNewName, int bTestOpen )
+int OGRILI1DataSource::Open( const char * pszNewName,
+                             char** papszOpenOptionsIn, int bTestOpen )
 
 {
-    FILE        *fp;
-    char        szHeader[1000];
-    std::string osBasename, osModelFilename;
-
     if (strlen(pszNewName) == 0)
     {
         return FALSE;
     }
 
-    char **filenames = CSLTokenizeString2( pszNewName, ",", 0 );
+    std::string osBasename;
+    std::string osModelFilename;
+    if( CSLFetchNameValue(papszOpenOptionsIn, "MODEL") != NULL )
+    {
+        osBasename = pszNewName;
+        osModelFilename = CSLFetchNameValue(papszOpenOptionsIn, "MODEL");
+    }
+    else
+    {
+        char **filenames = CSLTokenizeString2( pszNewName, ",", 0 );
 
-    osBasename = filenames[0];
+        osBasename = filenames[0];
 
-    if( CSLCount(filenames) > 1 )
-        osModelFilename = filenames[1];
+        if( CSLCount(filenames) > 1 )
+            osModelFilename = filenames[1];
 
-    CSLDestroy( filenames );
+        CSLDestroy( filenames );
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Open the source file.                                           */
 /* -------------------------------------------------------------------- */
-    fp = VSIFOpen( osBasename.c_str(), "r" );
+    FILE *fp = VSIFOpen( osBasename.c_str(), "r" );
     if( fp == NULL )
     {
         if( !bTestOpen )
@@ -126,6 +129,8 @@ int OGRILI1DataSource::Open( const char * pszNewName, int bTestOpen )
 /*      If we aren't sure it is ILI1, load a header chunk and check      */
 /*      for signs it is ILI1                                             */
 /* -------------------------------------------------------------------- */
+    char szHeader[1000];
+
     if( bTestOpen )
     {
         int nLen = (int)VSIFRead( szHeader, 1, sizeof(szHeader), fp );
@@ -151,8 +156,8 @@ int OGRILI1DataSource::Open( const char * pszNewName, int bTestOpen )
     if( poReader == NULL )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
-                  "File %s appears to be ILI1 but the ILI1 reader can't\n"
-                  "be instantiated, likely because Xerces support wasn't\n"
+                  "File %s appears to be ILI1 but the ILI1 reader cannot\n"
+                  "be instantiated, likely because Xerces support was not\n"
                   "configured in.",
                   pszNewName );
         return FALSE;
@@ -165,14 +170,19 @@ int OGRILI1DataSource::Open( const char * pszNewName, int bTestOpen )
     if (osModelFilename.length() > 0 )
         poReader->ReadModel( poImdReader, osModelFilename.c_str(), this );
 
-    if( getenv( "ARC_DEGREES" ) != NULL ) {
-      //No better way to pass arguments to the reader (it could even be an -lco arg)
-      poReader->SetArcDegrees( atof( getenv("ARC_DEGREES") ) );
+    int bResetConfigOption = FALSE;
+    if (EQUAL(CPLGetConfigOption("OGR_ARC_STEPSIZE", ""), ""))
+    {
+        bResetConfigOption = TRUE;
+        CPLSetThreadLocalConfigOption("OGR_ARC_STEPSIZE", "0.96");
     }
 
-    //Parse model and read data - without surface joing and polygonizing
+    // Parse model and read data - without surface join and area polygonizing.
     poReader->ReadFeatures();
 
+    if( bResetConfigOption )
+        CPLSetThreadLocalConfigOption("OGR_ARC_STEPSIZE", NULL);
+
     return TRUE;
 }
 
@@ -181,13 +191,13 @@ int OGRILI1DataSource::Open( const char * pszNewName, int bTestOpen )
 /************************************************************************/
 
 int OGRILI1DataSource::Create( const char *pszFilename,
-                               CPL_UNUSED char **papszOptions )
+                               char ** /* papszOptions */)
 {
-    std::string osBasename, osModelFilename;
     char **filenames = CSLTokenizeString2( pszFilename, ",", 0 );
 
-    osBasename = filenames[0];
+    std::string osBasename = filenames[0];
 
+    std::string osModelFilename;
     if( CSLCount(filenames) > 1 )
         osModelFilename = filenames[1];
 
@@ -207,7 +217,6 @@ int OGRILI1DataSource::Create( const char *pszFilename,
         return FALSE;
     }
 
-
 /* -------------------------------------------------------------------- */
 /*      Parse model                                                     */
 /* -------------------------------------------------------------------- */
@@ -225,7 +234,8 @@ int OGRILI1DataSource::Create( const char *pszFilename,
 /*      Write headers                                                   */
 /* -------------------------------------------------------------------- */
     VSIFPrintf( fpTransfer, "SCNT\n" );
-    VSIFPrintf( fpTransfer, "OGR/GDAL %s, INTERLIS Driver\n", GDAL_RELEASE_NAME );
+    VSIFPrintf( fpTransfer, "OGR/GDAL %s, INTERLIS Driver\n",
+                GDAL_RELEASE_NAME );
     VSIFPrintf( fpTransfer, "////\n" );
     VSIFPrintf( fpTransfer, "MTID INTERLIS1\n" );
     const char* modelname = poImdReader->mainModelName.c_str();
@@ -238,20 +248,22 @@ static char *ExtractTopic(const char * pszLayerName)
 {
   const char *table = strchr(pszLayerName, '_');
   while (table && table[1] !=  '_') table = strchr(table+1, '_');
-  return (table) ? CPLScanString(pszLayerName, table-pszLayerName, FALSE, FALSE) : NULL;
+  return (table) ? CPLScanString(
+      pszLayerName, static_cast<int>(table-pszLayerName), FALSE, FALSE) : NULL;
 }
 
 /************************************************************************/
-/*                            CreateLayer()                             */
+/*                           ICreateLayer()                             */
 /************************************************************************/
 
 OGRLayer *
-OGRILI1DataSource::CreateLayer( const char * pszLayerName,
-                                CPL_UNUSED OGRSpatialReference *poSRS,
-                                OGRwkbGeometryType eType,
-                                CPL_UNUSED char ** papszOptions )
+OGRILI1DataSource::ICreateLayer( const char * pszLayerName,
+                               CPL_UNUSED OGRSpatialReference *poSRS,
+                               OGRwkbGeometryType eType,
+                               CPL_UNUSED char ** papszOptions )
 {
-    FeatureDefnInfo featureDefnInfo = poImdReader->GetFeatureDefnInfo(pszLayerName);
+    FeatureDefnInfo featureDefnInfo
+        = poImdReader->GetFeatureDefnInfo(pszLayerName);
     const char *table = pszLayerName;
     char * topic = ExtractTopic(pszLayerName);
     if (nLayers) VSIFPrintf( fpTransfer, "ETAB\n" );
@@ -282,12 +294,15 @@ OGRILI1DataSource::CreateLayer( const char * pszLayerName,
 
     OGRFeatureDefn* poFeatureDefn = new OGRFeatureDefn(table);
     poFeatureDefn->SetGeomType( eType );
-    OGRILI1Layer *poLayer = new OGRILI1Layer(poFeatureDefn, featureDefnInfo.poGeomFieldInfos, this);
+    OGRILI1Layer *poLayer
+        = new OGRILI1Layer(poFeatureDefn, featureDefnInfo.poGeomFieldInfos,
+                           this);
 
     nLayers ++;
-    papoLayers = (OGRILI1Layer**)CPLRealloc(papoLayers, sizeof(OGRILI1Layer*) * nLayers);
+    papoLayers = static_cast<OGRILI1Layer **>(
+        CPLRealloc(papoLayers, sizeof(OGRILI1Layer*) * nLayers) );
     papoLayers[nLayers-1] = poLayer;
-    
+
     return poLayer;
 }
 
@@ -300,8 +315,10 @@ int OGRILI1DataSource::TestCapability( const char * pszCap )
 {
     if( EQUAL(pszCap,ODsCCreateLayer) )
         return TRUE;
-    else
-        return FALSE;
+    if( EQUAL(pszCap,ODsCCurveGeometries) )
+        return TRUE;
+
+    return FALSE;
 }
 
 /************************************************************************/
@@ -319,5 +336,6 @@ OGRLayer *OGRILI1DataSource::GetLayer( int iLayer )
 
 OGRILI1Layer *OGRILI1DataSource::GetLayerByName( const char* pszLayerName )
 {
-  return (OGRILI1Layer*)poReader->GetLayerByName( pszLayerName );
+  return reinterpret_cast<OGRILI1Layer *>(
+      poReader->GetLayerByName( pszLayerName ) );
 }
