@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrili2driver.cpp 13906 2008-03-01 13:08:28Z rouault $
+ * $Id: ogrili2driver.cpp 33109 2016-01-23 16:25:42Z rouault $
  *
  * Project:  Interlis 2 Translator
  * Purpose:  Implements OGRILI2Layer class.
@@ -27,89 +27,105 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "ogr_ili2.h"
+#include "xercesc_headers.h"
+
 #include "cpl_conv.h"
+#include "ogr_ili2.h"
+#include "ogrsf_frmts.h"
 
-CPL_CVSID("$Id: ogrili2driver.cpp 13906 2008-03-01 13:08:28Z rouault $");
+CPL_CVSID("$Id: ogrili2driver.cpp 33109 2016-01-23 16:25:42Z rouault $");
 
-/************************************************************************/
-/*                          ~OGRILI2Driver()                           */
-/************************************************************************/
-
-OGRILI2Driver::~OGRILI2Driver() {
-}
-
-/************************************************************************/
-/*                              GetName()                               */
-/************************************************************************/
-
-const char *OGRILI2Driver::GetName() {
-    return "Interlis 2";
-}
 
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
 
-OGRDataSource *OGRILI2Driver::Open( const char * pszFilename,
-                                   int bUpdate )
+static GDALDataset *OGRILI2DriverOpen( GDALOpenInfo* poOpenInfo )
 
 {
-    OGRILI2DataSource    *poDS;
-
-    if( bUpdate )
+    if( poOpenInfo->eAccess == GA_Update ||
+        (!poOpenInfo->bStatOK && strchr(poOpenInfo->pszFilename, ',') == NULL) )
         return NULL;
 
-    poDS = new OGRILI2DataSource();
+    if( poOpenInfo->fpL != NULL )
+    {
+        if( poOpenInfo->pabyHeader[0] != '<'
+            || strstr((const char*)poOpenInfo->pabyHeader,"interlis.ch/INTERLIS2") == NULL )
+        {
+            return NULL;
+        }
+    }
+    else if( poOpenInfo->bIsDirectory )
+        return NULL;
 
-    if( !poDS->Open( pszFilename, TRUE )
+    OGRILI2DataSource *poDS = new OGRILI2DataSource();
+
+    if( !poDS->Open( poOpenInfo->pszFilename, poOpenInfo->papszOpenOptions,
+                     TRUE )
         || poDS->GetLayerCount() == 0 )
     {
         delete poDS;
         return NULL;
     }
-    else
-        return poDS;
+
+    return poDS;
 }
 
 /************************************************************************/
-/*                          CreateDataSource()                          */
+/*                               Create()                               */
 /************************************************************************/
 
-OGRDataSource *OGRILI2Driver::CreateDataSource( const char * pszName,
-                                               char **papszOptions )
-
+static GDALDataset *OGRILI2DriverCreate( const char * pszName,
+                                         int /* nBands */,
+                                         int /* nXSize */,
+                                         int /* nYSize */,
+                                         GDALDataType /* eDT */,
+                                         char **papszOptions )
 {
-    OGRILI2DataSource    *poDS = new OGRILI2DataSource();
+    OGRILI2DataSource *poDS = new OGRILI2DataSource();
 
     if( !poDS->Create( pszName, papszOptions ) )
     {
         delete poDS;
         return NULL;
     }
-    else
-        return poDS;
+
+    return poDS;
 }
 
 /************************************************************************/
-/*                           TestCapability()                           */
+/*                         OGRILI2DriverUnload()                        */
 /************************************************************************/
 
-int OGRILI2Driver::TestCapability( const char * pszCap ) {
-    if( EQUAL(pszCap,ODrCCreateDataSource) )
-        return TRUE;
-    else if( EQUAL(pszCap,ODrCDeleteDataSource) )
-        return FALSE;
-    else
-        return FALSE;
+static void OGRILI2DriverUnload ( GDALDriver* )
+{
+    if( getenv("ILI2_TERMINATE_XERCES") )
+        XMLPlatformUtils::Terminate();
 }
 
 /************************************************************************/
 /*                           RegisterOGRILI2()                           */
 /************************************************************************/
 
-void RegisterOGRILI2()
-{
-    OGRSFDriverRegistrar::GetRegistrar()->RegisterDriver( new OGRILI2Driver );
-}
+void RegisterOGRILI2() {
+    if( GDALGetDriverByName( "Interlis 2" ) != NULL )
+        return;
 
+    GDALDriver *poDriver = new GDALDriver();
+
+    poDriver->SetDescription( "Interlis 2" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VECTOR, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "Interlis 2" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drv_ili.html" );
+    poDriver->SetMetadataItem( GDAL_DMD_EXTENSIONS, "xtf xml ili" );
+    poDriver->SetMetadataItem( GDAL_DMD_OPENOPTIONLIST,
+"<OpenOptionList>"
+"  <Option name='MODEL' type='string' description='Filename of the model in IlisMeta format (.imd)'/>"
+"</OpenOptionList>" );
+
+    poDriver->pfnOpen = OGRILI2DriverOpen;
+    poDriver->pfnCreate = OGRILI2DriverCreate;
+    poDriver->pfnUnloadDriver = OGRILI2DriverUnload;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
+}
