@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogringreslayer.cpp 19509 2010-04-23 16:49:33Z warmerdam $
+ * $Id: ogringreslayer.cpp 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRIngresLayer class.
@@ -31,7 +31,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogringreslayer.cpp 19509 2010-04-23 16:49:33Z warmerdam $");
+CPL_CVSID("$Id: ogringreslayer.cpp 33714 2016-03-13 05:42:13Z goatbar $");
 
 /************************************************************************/
 /*                           OGRIngresLayer()                            */
@@ -46,7 +46,7 @@ OGRIngresLayer::OGRIngresLayer()
     nResultOffset = 0;
 
     poSRS = NULL;
-    nSRSId = -2; // we haven't even queried the database for it yet. 
+    nSRSId = -2; // we haven't even queried the database for it yet.
 
     poFeatureDefn = NULL;
 
@@ -63,7 +63,7 @@ OGRIngresLayer::~OGRIngresLayer()
     if( m_nFeaturesRead > 0 && poFeatureDefn != NULL )
     {
         CPLDebug( "Ingres", "%d features read on layer '%s'.",
-                  (int) m_nFeaturesRead, 
+                  (int) m_nFeaturesRead,
                   poFeatureDefn->GetName() );
     }
 
@@ -99,7 +99,7 @@ void OGRIngresLayer::ResetReading()
 OGRFeature *OGRIngresLayer::GetNextFeature()
 
 {
-    for( ; TRUE; )
+    while( true )
     {
         OGRFeature      *poFeature;
 
@@ -139,8 +139,8 @@ static int ParseXY( const char **ppszNext, double *padfXY )
         }
     }
 
-    padfXY[0] = atof(pszNext);
-    padfXY[1] = atof(pszNext + iStartY);
+    padfXY[0] = CPLAtof(pszNext);
+    padfXY[1] = CPLAtof(pszNext + iStartY);
 
     int iEnd;
 
@@ -151,7 +151,7 @@ static int ParseXY( const char **ppszNext, double *padfXY )
         if( pszNext[iEnd] == '\0' )
             return FALSE;
     }
-    
+
     *ppszNext += iEnd;
 
     return TRUE;
@@ -209,18 +209,18 @@ OGRGeometry *OGRIngresLayer::TranslateGeometry( const char *pszGeom )
         if( nVertCount == nVertMax )
         {
             nVertMax = nVertMax * 2 + 1;
-            padfXY = (double *) 
+            padfXY = (double *)
                 CPLRealloc(padfXY, sizeof(double) * nVertMax * 2 );
         }
 
         if( !ParseXY( &pszNext, padfXY + nVertCount*2 ) )
         {
-            CPLDebug( "INGRES", "Error parsing geometry: %s", 
+            CPLDebug( "INGRES", "Error parsing geometry: %s",
                       pszGeom );
             CPLFree( padfXY );
             return NULL;
         }
-        
+
         CPLAssert( *pszNext == ')' );
         nVertCount++;
         pszNext++;
@@ -332,28 +332,28 @@ OGRFeature *OGRIngresLayer::RecordToFeature( char **papszRow )
 /* ==================================================================== */
 /*      Transfer all result fields we can.                              */
 /* ==================================================================== */
-    for( iField = 0; 
+    for( iField = 0;
          iField < (int) poResultSet->getDescrParm.gd_descriptorCount;
          iField++ )
     {
-        IIAPI_DATAVALUE *psDV = 
+        IIAPI_DATAVALUE *psDV =
             poResultSet->pasDataBuffer + iField;
-        IIAPI_DESCRIPTOR *psFDesc = 
+        IIAPI_DESCRIPTOR *psFDesc =
             poResultSet->getDescrParm.gd_descriptor + iField;
         int     iOGRField;
 
 /* -------------------------------------------------------------------- */
 /*      Ignore NULL fields.                                             */
 /* -------------------------------------------------------------------- */
-        if( psDV->dv_null ) 
+        if( psDV->dv_null )
             continue;
 
 /* -------------------------------------------------------------------- */
 /*      Handle FID.                                                     */
 /* -------------------------------------------------------------------- */
-        if( osFIDColumn.size() 
-            && EQUAL(psFDesc->ds_columnName,osFIDColumn) 
-            && psFDesc->ds_dataType == IIAPI_INT_TYPE 
+        if( osFIDColumn.size()
+            && EQUAL(psFDesc->ds_columnName,osFIDColumn)
+            && psFDesc->ds_dataType == IIAPI_INT_TYPE
             && psDV->dv_length == 4 )
         {
             if( papszRow[iField] == NULL )
@@ -371,7 +371,7 @@ OGRFeature *OGRIngresLayer::RecordToFeature( char **papszRow )
 /* -------------------------------------------------------------------- */
 /*      Handle Ingres geometry                                           */
 /* -------------------------------------------------------------------- */
-        if( osGeomColumn.size() 
+        if( osGeomColumn.size()
             && EQUAL(psFDesc->ds_columnName,osGeomColumn))
         {
         	if( poDS->IsNewIngres() )
@@ -458,17 +458,17 @@ OGRFeature *OGRIngresLayer::RecordToFeature( char **papszRow )
                 poFeature->SetField( iOGRField, dfValue );
             }
             break;
-            
+
           case IIAPI_DEC_TYPE:
           {
               IIAPI_CONVERTPARM sCParm;
               char szFormatBuf[30];
 
               memset( &sCParm, 0, sizeof(sCParm) );
-              
-              memcpy( &(sCParm.cv_srcDesc), psFDesc, 
+
+              memcpy( &(sCParm.cv_srcDesc), psFDesc,
                       sizeof(IIAPI_DESCRIPTOR) );
-              memcpy( &(sCParm.cv_srcValue), psDV, 
+              memcpy( &(sCParm.cv_srcValue), psDV,
                       sizeof(IIAPI_DATAVALUE) );
 
               sCParm.cv_dstDesc.ds_dataType = IIAPI_CHA_TYPE;
@@ -478,7 +478,7 @@ OGRFeature *OGRIngresLayer::RecordToFeature( char **papszRow )
               sCParm.cv_dstValue.dv_value = szFormatBuf;
 
               IIapi_convertData( &sCParm );
-              
+
               poFeature->SetField( iOGRField, szFormatBuf );
               break;
           }
@@ -536,7 +536,7 @@ OGRFeature *OGRIngresLayer::GetNextRawFeature()
 /*      Note that we actually override this in OGRIngresTableLayer.      */
 /************************************************************************/
 
-OGRFeature *OGRIngresLayer::GetFeature( long nFeatureId )
+OGRFeature *OGRIngresLayer::GetFeature( GIntBig nFeatureId )
 
 {
     return OGRLayer::GetFeature( nFeatureId );
@@ -569,13 +569,11 @@ int OGRIngresLayer::TestCapability( const char * pszCap )
 }
 
 
-
-    
 /************************************************************************/
 /*                            GetFIDColumn()                            */
 /************************************************************************/
 
-const char *OGRIngresLayer::GetFIDColumn() 
+const char *OGRIngresLayer::GetFIDColumn()
 
 {
     return osFIDColumn;
@@ -585,7 +583,7 @@ const char *OGRIngresLayer::GetFIDColumn()
 /*                         GetGeometryColumn()                          */
 /************************************************************************/
 
-const char *OGRIngresLayer::GetGeometryColumn() 
+const char *OGRIngresLayer::GetGeometryColumn()
 
 {
     return osGeomColumn;
@@ -614,17 +612,17 @@ int OGRIngresLayer::FetchSRSId(OGRFeatureDefn *poDefn)
         char         szCommand[1024];
         char           **papszRow;
         OGRIngresStatement oStatement(poDS->GetConn());
-        
-        sprintf( szCommand, 
+
+        sprintf( szCommand,
                  "SELECT srid FROM geometry_columns "
                  "WHERE f_table_name = '%s' AND f_geometry_column = '%s'",
                  poDefn->GetName(),
                  GetGeometryColumn());
-        
+
         oStatement.ExecuteSQL(szCommand);
-        
+
         papszRow = oStatement.GetRow();
-        
+
         if( papszRow != NULL && papszRow[0] != NULL )
         {
             nSRSId = *((II_INT4 *) papszRow[0]);
