@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogringrestablelayer.cpp 28375 2015-01-30 12:06:11Z rouault $
+ * $Id: ogringrestablelayer.cpp 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRIngresTableLayer class.
@@ -31,13 +31,13 @@
 #include "cpl_string.h"
 #include "ogr_ingres.h"
 
-CPL_CVSID("$Id: ogringrestablelayer.cpp 28375 2015-01-30 12:06:11Z rouault $");
+CPL_CVSID("$Id: ogringrestablelayer.cpp 33714 2016-03-13 05:42:13Z goatbar $");
 
 /************************************************************************/
 /*                         OGRIngresTableLayer()                         */
 /************************************************************************/
 
-OGRIngresTableLayer::OGRIngresTableLayer( OGRIngresDataSource *poDSIn, 
+OGRIngresTableLayer::OGRIngresTableLayer( OGRIngresDataSource *poDSIn,
                                   const char * pszTableName,
                                   int bUpdate, int nSRSIdIn )
 
@@ -74,7 +74,7 @@ OGRIngresTableLayer::~OGRIngresTableLayer()
 
 OGRErr  OGRIngresTableLayer::Initialize(const char * pszTableName)
 {
-    poFeatureDefn = ReadTableDefinition( pszTableName );   
+    poFeatureDefn = ReadTableDefinition( pszTableName );
     if (poFeatureDefn)
     {
         ResetReading();
@@ -107,7 +107,7 @@ OGRFeatureDefn *OGRIngresTableLayer::ReadTableDefinition( const char *pszTable )
     osCommand.Printf( "select column_name, column_datatype, column_length, "
                       "column_scale, column_ingdatatype, "
                       "column_internal_datatype "
-                      "from iicolumns where table_name = '%s'", 
+                      "from iicolumns where table_name = '%s'",
                       pszTable );
 
     if( !oStatement.ExecuteSQL( osCommand ) )
@@ -164,7 +164,7 @@ OGRFeatureDefn *OGRIngresTableLayer::ReadTableDefinition( const char *pszTable )
         {
             osGeomColumn = osFieldName;
             osIngresGeomType = osInternalType;
-            
+
             if( strstr(osInternalType,"POINT") )
                 poDefn->SetGeomType( wkbPoint );
             else if( strstr(osInternalType,"LINE")
@@ -184,19 +184,19 @@ OGRFeatureDefn *OGRIngresTableLayer::ReadTableDefinition( const char *pszTable )
                 poDefn->SetGeomType( wkbPolygon );
             continue;
         }
-        else if( EQUALN(osIngresType,"byte",4) 
-            || EQUALN(osIngresType,"long byte",9) )
+        else if( STARTS_WITH_CI(osIngresType, "byte")
+            || STARTS_WITH_CI(osIngresType, "long byte") )
         {
             oField.SetType( OFTBinary );
         }
-        else if( EQUALN(osIngresType,"varchar",7) 
-                 || EQUAL(osIngresType,"text") 
-                 || EQUALN(osIngresType,"long varchar",12) )
+        else if( STARTS_WITH_CI(osIngresType, "varchar")
+                 || EQUAL(osIngresType,"text")
+                 || STARTS_WITH_CI(osIngresType, "long varchar") )
         {
             oField.SetType( OFTString );
             oField.SetWidth( nWidth );
         }
-        else if( EQUALN(osIngresType,"char",4) || EQUAL(osIngresType,"c") )
+        else if( STARTS_WITH_CI(osIngresType, "char") || EQUAL(osIngresType,"c") )
         {
             oField.SetType( OFTString );
             oField.SetWidth( nWidth );
@@ -205,7 +205,7 @@ OGRFeatureDefn *OGRIngresTableLayer::ReadTableDefinition( const char *pszTable )
         {
             oField.SetType( OFTInteger );
         }
-        else if( EQUALN(osIngresType,"decimal", 7) )
+        else if( STARTS_WITH_CI(osIngresType, "decimal") )
         {
             if( nScale != 0 )
             {
@@ -219,13 +219,13 @@ OGRFeatureDefn *OGRIngresTableLayer::ReadTableDefinition( const char *pszTable )
                 oField.SetWidth( nWidth );
             }
         }
-        else if( EQUALN(osIngresType,"float", 5) )
+        else if( STARTS_WITH_CI(osIngresType, "float") )
         {
             oField.SetType( OFTReal );
         }
 #ifdef notdef
-        else if( EQUAL(osIngresType,"date") 
-                 || EQUAL(osIngresType,"ansidate") 
+        else if( EQUAL(osIngresType,"date")
+                 || EQUAL(osIngresType,"ansidate")
                  || EQUAL(osIngresType,"ingresdate") )
         {
             oField.SetType( OFTDate );
@@ -233,8 +233,8 @@ OGRFeatureDefn *OGRIngresTableLayer::ReadTableDefinition( const char *pszTable )
 #endif
 
         // Is this an integer primary key field?
-        if( osFIDColumn.size() == 0 
-            && oField.GetType() == OFTInteger 
+        if( osFIDColumn.size() == 0
+            && oField.GetType() == OFTInteger
             && EQUAL(oField.GetNameRef(),"ogr_fid") )
         {
             osFIDColumn = oField.GetNameRef();
@@ -248,7 +248,7 @@ OGRFeatureDefn *OGRIngresTableLayer::ReadTableDefinition( const char *pszTable )
         CPLDebug( "Ingres", "table %s has FID column %s.",
                   pszTable, osFIDColumn.c_str() );
     else
-        CPLDebug( "Ingres", 
+        CPLDebug( "Ingres",
                   "table %s has no FID column, FIDs will not be reliable!",
                   pszTable );
 
@@ -297,10 +297,10 @@ void OGRIngresTableLayer::BuildWhere()
         char szEnvelope[4096];
         OGREnvelope  sEnvelope;
         szEnvelope[0] = '\0';
-        
+
         //POLYGON((MINX MINY, MAXX MINY, MAXX MAXY, MINX MAXY, MINX MINY))
         m_poFilterGeom->getEnvelope( &sEnvelope );
-        
+
         sprintf(szEnvelope,
                 "POLYGON((%.12f %.12f, %.12f %.12f, %.12f %.12f, %.12f %.12f, %.12f %.12f))",
                 sEnvelope.MinX, sEnvelope.MinY,
@@ -334,10 +334,10 @@ void OGRIngresTableLayer::BuildFullQueryStatement()
 {
     char *pszFields = BuildFields();
 
-    osQueryStatement.Printf( "SELECT %s FROM %s %s", 
-                             pszFields, poFeatureDefn->GetName(), 
+    osQueryStatement.Printf( "SELECT %s FROM %s %s",
+                             pszFields, poFeatureDefn->GetName(),
                              osWHERE.c_str() );
-    
+
     CPLFree( pszFields );
 }
 
@@ -457,7 +457,7 @@ int OGRIngresTableLayer::TestCapability( const char * pszCap )
     else if( EQUAL(pszCap,OLCDeleteFeature) )
         return bUpdateAccess && osFIDColumn.size() != 0;
 
-    else 
+    else
         return OGRIngresLayer::TestCapability( pszCap );
 }
 
@@ -516,13 +516,13 @@ OGRErr OGRIngresTableLayer::DeleteFeature( GIntBig nFID )
 /* -------------------------------------------------------------------- */
     osCommand.Printf( "DELETE FROM %s WHERE %s = %ld",
                       poFeatureDefn->GetName(), osFIDColumn.c_str(), nFID );
-                      
+
 /* -------------------------------------------------------------------- */
 /*      Execute the delete.                                             */
 /* -------------------------------------------------------------------- */
     poDS->EstablishActiveLayer( NULL );
     OGRIngresStatement oStmt( poDS->GetConn() );
-    
+
     if( !oStmt.ExecuteSQL( osCommand ) )
         return OGRERR_FAILURE;
     else
@@ -536,7 +536,7 @@ OGRErr OGRIngresTableLayer::DeleteFeature( GIntBig nFID )
 /*      a form suitable to include in an INSERT command.                */
 /************************************************************************/
 
-OGRErr OGRIngresTableLayer::PrepareOldStyleGeometry( 
+OGRErr OGRIngresTableLayer::PrepareOldStyleGeometry(
     OGRGeometry *poGeom, CPLString &osRetGeomText )
 
 {
@@ -562,8 +562,8 @@ OGRErr OGRIngresTableLayer::PrepareOldStyleGeometry(
     {
         OGRPoint *poPoint = (OGRPoint *) poGeom;
 
-        osRetGeomText.Printf( "(%d,%d)", 
-                              (int) floor(poPoint->getX()), 
+        osRetGeomText.Printf( "(%d,%d)",
+                              (int) floor(poPoint->getX()),
                               (int) floor(poPoint->getY()) );
         return OGRERR_NONE;
     }
@@ -577,31 +577,31 @@ OGRErr OGRIngresTableLayer::PrepareOldStyleGeometry(
         CPLString osLastPoint;
         int i;
 
-        if( (EQUAL(osIngresGeomType,"LSEG") 
-             || EQUAL(osIngresGeomType,"ILSEG")) 
+        if( (EQUAL(osIngresGeomType,"LSEG")
+             || EQUAL(osIngresGeomType,"ILSEG"))
             && poLS->getNumPoints() != 2 )
         {
             CPLError( CE_Failure, CPLE_AppDefined,
-                      "Attempt to place %d vertex linestring in %s field.", 
-                      poLS->getNumPoints(), 
+                      "Attempt to place %d vertex linestring in %s field.",
+                      poLS->getNumPoints(),
                       osIngresGeomType.c_str() );
             return OGRERR_FAILURE;
         }
-        else if( EQUAL(osIngresGeomType,"LINESTRING") 
+        else if( EQUAL(osIngresGeomType,"LINESTRING")
                  && poLS->getNumPoints() > 124 )
         {
             CPLError( CE_Failure, CPLE_AppDefined,
-                      "Attempt to place %d vertex linestring in %s field.", 
-                      poLS->getNumPoints(), 
+                      "Attempt to place %d vertex linestring in %s field.",
+                      poLS->getNumPoints(),
                       osIngresGeomType.c_str() );
             return OGRERR_FAILURE;
         }
-        else if( EQUAL(osIngresGeomType,"ILINESTRING") 
+        else if( EQUAL(osIngresGeomType,"ILINESTRING")
                  && poLS->getNumPoints() > 248 )
         {
             CPLError( CE_Failure, CPLE_AppDefined,
-                      "Attempt to place %d vertex linestring in %s field.", 
-                      poLS->getNumPoints(), 
+                      "Attempt to place %d vertex linestring in %s field.",
+                      poLS->getNumPoints(),
                       osIngresGeomType.c_str() );
             return OGRERR_FAILURE;
         }
@@ -611,20 +611,20 @@ OGRErr OGRIngresTableLayer::PrepareOldStyleGeometry(
         {
             CPLString osPoint;
 
-            if( i > 0 
+            if( i > 0
                 && poLS->getX(i) == poLS->getX(i-1)
                 && poLS->getY(i) == poLS->getY(i-1) )
             {
                 CPLDebug( "INGRES", "Dropping duplicate point in linestring.");
                 continue;
             }
-            
-            if( EQUALN(osIngresGeomType,"I",1) )
+
+            if( STARTS_WITH_CI(osIngresGeomType, "I") )
                 osPoint.Printf( "(%d,%d)",
-                                (int) floor(poLS->getX(i)), 
+                                (int) floor(poLS->getX(i)),
                                 (int) floor(poLS->getY(i)) );
             else
-                osPoint.Printf( "(%.15g,%.15g)", 
+                osPoint.Printf( "(%.15g,%.15g)",
                                 poLS->getX(i), poLS->getY(i) );
 
             if( osPoint == osLastPoint )
@@ -666,21 +666,21 @@ OGRErr OGRIngresTableLayer::PrepareOldStyleGeometry(
                       osIngresGeomType.c_str() );
         }
 
-        if( EQUAL(osIngresGeomType,"POLYGON") 
+        if( EQUAL(osIngresGeomType,"POLYGON")
             && poLS->getNumPoints() > 124 )
         {
             CPLError( CE_Failure, CPLE_AppDefined,
-                      "Attempt to place %d vertex linestring in %s field.", 
-                      poLS->getNumPoints(), 
+                      "Attempt to place %d vertex linestring in %s field.",
+                      poLS->getNumPoints(),
                       osIngresGeomType.c_str() );
             return OGRERR_FAILURE;
         }
-        else if( EQUAL(osIngresGeomType,"IPOLYGON") 
+        else if( EQUAL(osIngresGeomType,"IPOLYGON")
                  && poLS->getNumPoints() > 248 )
         {
             CPLError( CE_Failure, CPLE_AppDefined,
-                      "Attempt to place %d vertex linestring in %s field.", 
-                      poLS->getNumPoints(), 
+                      "Attempt to place %d vertex linestring in %s field.",
+                      poLS->getNumPoints(),
                       osIngresGeomType.c_str() );
             return OGRERR_FAILURE;
         }
@@ -688,7 +688,7 @@ OGRErr OGRIngresTableLayer::PrepareOldStyleGeometry(
         // INGRES geometries use *implied* closure of rings.
         nPoints = poLS->getNumPoints();
         if( poLS->getX(0) == poLS->getX(nPoints-1)
-            && poLS->getY(0) == poLS->getY(nPoints-1) 
+            && poLS->getY(0) == poLS->getY(nPoints-1)
             && nPoints > 1 )
             nPoints--;
 
@@ -696,21 +696,21 @@ OGRErr OGRIngresTableLayer::PrepareOldStyleGeometry(
         for( i = 0; i < nPoints; i++ )
         {
             CPLString osPoint;
-            
-            if( i > 0 
+
+            if( i > 0
                 && poLS->getX(i) == poLS->getX(i-1)
                 && poLS->getY(i) == poLS->getY(i-1) )
             {
                 CPLDebug( "INGRES", "Dropping duplicate point in linestring.");
                 continue;
             }
-            
-            if( EQUALN(osIngresGeomType,"I",1) )
+
+            if( STARTS_WITH_CI(osIngresGeomType, "I") )
                 osPoint.Printf( "(%d,%d)",
-                                (int) floor(poLS->getX(i)), 
+                                (int) floor(poLS->getX(i)),
                                 (int) floor(poLS->getY(i)) );
             else
-                osPoint.Printf( "(%.15g,%.15g)", 
+                osPoint.Printf( "(%.15g,%.15g)",
                                 poLS->getX(i), poLS->getY(i) );
 
             if( osRetGeomText.size() > 1 )
@@ -796,9 +796,9 @@ OGRErr OGRIngresTableLayer::PrepareNewStyleGeometry(
 /* -------------------------------------------------------------------- */
 /*      Fallback generic geometry handling.                             */
 /* -------------------------------------------------------------------- */
-    else 
+    else
     {
-        CPLDebug( 
+        CPLDebug(
             "INGRES",
             "Unexpected geometry type (%s), attempting to treat generically.",
             poGeom->getGeometryName() );
@@ -838,7 +838,7 @@ OGRErr OGRIngresTableLayer::ICreateFeature( OGRFeature *poFeature )
     {
         if( bNeedComma )
             osCommand += ", ";
-        
+
         osCommand = osCommand + osFIDColumn + " ";
         bNeedComma = TRUE;
     }
@@ -853,7 +853,7 @@ OGRErr OGRIngresTableLayer::ICreateFeature( OGRFeature *poFeature )
         else
             osCommand += ", ";
 
-        osCommand = osCommand 
+        osCommand = osCommand
              + poFeatureDefn->GetFieldDefn(i)->GetNameRef();
     }
 
@@ -864,7 +864,7 @@ OGRErr OGRIngresTableLayer::ICreateFeature( OGRFeature *poFeature )
 /* -------------------------------------------------------------------- */
     CPLString osGeomText;
 
-    // Set the geometry 
+    // Set the geometry
     bNeedComma = FALSE;
     if( poFeature->GetGeometryRef() != NULL && osGeomColumn.size() )
     {
@@ -881,7 +881,7 @@ OGRErr OGRIngresTableLayer::ICreateFeature( OGRFeature *poFeature )
         }
         if( localErr == OGRERR_NONE )
         {
-            if( CSLTestBoolean( 
+            if( CPLTestBool(
                      CPLGetConfigOption( "INGRES_INSERT_SUB", "NO") ) )
             {
                 osCommand += " ~V";
@@ -938,7 +938,7 @@ OGRErr OGRIngresTableLayer::ICreateFeature( OGRFeature *poFeature )
         {
             int         iChar;
 
-            //We need to quote and escape string fields. 
+            //We need to quote and escape string fields.
             osCommand += "'";
 
             for( iChar = 0; pszStrValue[iChar] != '\0'; iChar++ )
@@ -992,7 +992,7 @@ OGRErr OGRIngresTableLayer::ICreateFeature( OGRFeature *poFeature )
     poDS->EstablishActiveLayer( NULL );
     OGRIngresStatement oStmt( poDS->GetConn() );
 
-    oStmt.bDebug = FALSE; 
+    oStmt.bDebug = FALSE;
 
     if( osGeomText.size() > 0  && poDS->IsNewIngres() == FALSE )
         oStmt.addInputParameter( IIAPI_LVCH_TYPE, osGeomText.size(),
@@ -1016,7 +1016,7 @@ OGRErr OGRIngresTableLayer::ICreateFeature( OGRFeature *poFeature )
 
     if( !oStmt.ExecuteSQL( osCommand ) )
         return OGRERR_FAILURE;
-    
+
     return OGRERR_NONE;
 
 }
@@ -1025,7 +1025,7 @@ OGRErr OGRIngresTableLayer::ICreateFeature( OGRFeature *poFeature )
 /*                            CreateField()                             */
 /************************************************************************/
 
-OGRErr OGRIngresTableLayer::CreateField( OGRFieldDefn *poFieldIn, 
+OGRErr OGRIngresTableLayer::CreateField( OGRFieldDefn *poFieldIn,
                                          int bApproxOK )
 
 {
@@ -1117,14 +1117,14 @@ OGRErr OGRIngresTableLayer::CreateField( OGRFieldDefn *poFieldIn,
     }
 
     osCommand.Printf( "ALTER TABLE %s ADD COLUMN %s %s",
-                      poFeatureDefn->GetName(), oField.GetNameRef(), 
+                      poFeatureDefn->GetName(), oField.GetNameRef(),
                       szFieldType );
 
     if( !oStatement.ExecuteSQL( osCommand ) )
         return OGRERR_FAILURE;
 
-    poFeatureDefn->AddFieldDefn( &oField );    
-    
+    poFeatureDefn->AddFieldDefn( &oField );
+
     return OGRERR_NONE;
 }
 
@@ -1150,9 +1150,9 @@ OGRFeature *OGRIngresTableLayer::GetFeature( GIntBig nFeatureId )
     char        *pszFieldList = BuildFields();
     char        *pszCommand = (char *) CPLMalloc(strlen(pszFieldList)+2000);
 
-    sprintf( pszCommand, 
-             "SELECT %s FROM %s WHERE %s = %ld", 
-             pszFieldList, poFeatureDefn->GetName(), pszFIDColumn, 
+    sprintf( pszCommand,
+             "SELECT %s FROM %s WHERE %s = %ld",
+             pszFieldList, poFeatureDefn->GetName(), pszFIDColumn,
              nFeatureId );
     CPLFree( pszFieldList );
 
@@ -1222,14 +1222,14 @@ GIntBig OGRIngresTableLayer::GetFeatureCount( int bForce )
 /*      Ensure any active long result is interrupted.                   */
 /* -------------------------------------------------------------------- */
     poDS->InterruptLongResult();
-    
+
 /* -------------------------------------------------------------------- */
 /*      Issue the appropriate select command.                           */
 /* -------------------------------------------------------------------- */
     INGRES_RES    *hResult;
     const char         *pszCommand;
 
-    pszCommand = CPLSPrintf( "SELECT COUNT(*) FROM %s %s", 
+    pszCommand = CPLSPrintf( "SELECT COUNT(*) FROM %s %s",
                              poFeatureDefn->GetName(), pszWHERE );
 
     if( ingres_query( poDS->GetConn(), pszCommand ) )
@@ -1244,7 +1244,7 @@ GIntBig OGRIngresTableLayer::GetFeatureCount( int bForce )
         poDS->ReportError( "ingres_store_result() failed on SELECT COUNT(*)." );
         return FALSE;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Capture the result.                                             */
 /* -------------------------------------------------------------------- */
@@ -1257,7 +1257,7 @@ GIntBig OGRIngresTableLayer::GetFeatureCount( int bForce )
     if( hResultSet != NULL )
         ingres_free_result( hResultSet );
  		hResultSet = NULL;
-    
+
     return nCount;
 }
 #endif
@@ -1279,7 +1279,7 @@ OGRErr OGRIngresTableLayer::GetExtent(OGREnvelope *psExtent, int bForce )
         psExtent->MaxX = 0.0;
         psExtent->MinY = 0.0;
         psExtent->MaxY = 0.0;
-        
+
         return OGRERR_FAILURE;
     }
 
@@ -1298,7 +1298,7 @@ OGRErr OGRIngresTableLayer::GetExtent(OGREnvelope *psExtent, int bForce )
             return OGRERR_FAILURE;
         }
 
-		INGRES_ROW row; 
+		INGRES_ROW row;
 		unsigned long *panLengths = NULL;
 		while ((row = ingres_fetch_row(result)))
 		{
@@ -1314,7 +1314,7 @@ OGRErr OGRIngresTableLayer::GetExtent(OGREnvelope *psExtent, int bForce )
 
 			OGRGeometry *poGeometry = NULL;
 			// Geometry columns will have the first 4 bytes contain the SRID.
-			OGRGeometryFactory::createFromWkb(((GByte *)row[0]) + 4, 
+			OGRGeometryFactory::createFromWkb(((GByte *)row[0]) + 4,
 											  NULL,
 											  &poGeometry,
 											  panLengths[0] - 4 );
@@ -1329,23 +1329,22 @@ OGRErr OGRIngresTableLayer::GetExtent(OGREnvelope *psExtent, int bForce )
 				else if (poGeometry)
 				{
 					poGeometry->getEnvelope(&oEnv);
-					if (oEnv.MinX < psExtent->MinX) 
+					if (oEnv.MinX < psExtent->MinX)
 						psExtent->MinX = oEnv.MinX;
-					if (oEnv.MinY < psExtent->MinY) 
+					if (oEnv.MinY < psExtent->MinY)
 						psExtent->MinY = oEnv.MinY;
-					if (oEnv.MaxX > psExtent->MaxX) 
+					if (oEnv.MaxX > psExtent->MaxX)
 						psExtent->MaxX = oEnv.MaxX;
-					if (oEnv.MaxY > psExtent->MaxY) 
+					if (oEnv.MaxY > psExtent->MaxY)
 						psExtent->MaxY = oEnv.MaxY;
 				}
 				delete poGeometry;
 			}
 		}
 
-		ingres_free_result(result);      
+		ingres_free_result(result);
 	}
 
 	return (bExtentSet ? OGRERR_NONE : OGRERR_FAILURE);
 }
 #endif
-
