@@ -6,6 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2010, Brian Case
+ * Copyright (c) 2010-2014, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -26,14 +27,14 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
+#include "libkml_headers.h"
+
 #include <ogrsf_frmts.h>
 #include <ogr_featurestyle.h>
 #include <string>
 #include <iostream>
 using namespace std;
 
-#include <kml/dom.h>
-
 using kmldom::KmlFactory;;
 using kmldom::IconStylePtr;
 using kmldom::PolyStylePtr;
@@ -55,7 +56,7 @@ args:
             poOgrLayer      the layer the feature is in
             poOgrFeat       the feature
             poKmlFactory    the kml dom factory
-            poKmlPlacemark  the placemark to add it to
+            poKmlFeature  the placemark to add it to
 
 returns:
             nothing
@@ -66,7 +67,7 @@ void featurestyle2kml (
     OGRLayer * poOgrLayer,
     OGRFeature * poOgrFeat,
     KmlFactory * poKmlFactory,
-    PlacemarkPtr poKmlPlacemark )
+    FeaturePtr poKmlFeature )
 {
 
     /***** get the style table *****/
@@ -77,26 +78,28 @@ void featurestyle2kml (
 
     /***** does the feature have style? *****/
 
-    if ( pszStyleString ) {
+    if ( pszStyleString && pszStyleString[0] != '\0' ) {
 
         /***** does it ref a style table? *****/
 
         if ( *pszStyleString == '@' ) {
 
+            const char* pszStyleName = pszStyleString + 1;
+
             /***** is the name in the layer style table *****/
 
-            OGRStyleTable *hSTBLLayer;
+            OGRStyleTable *hSTBLLayer = poOgrLayer->GetStyleTable (  );
             const char *pszTest = NULL;
 
-            if ( ( hSTBLLayer = poOgrLayer->GetStyleTable (  ) ) )
-                pszTest = hSTBLLayer->Find ( pszStyleString );
+            if ( hSTBLLayer != NULL )
+                pszTest = hSTBLLayer->Find ( pszStyleName );
 
             if ( pszTest ) {
                 string oTmp = "#";
 
-                oTmp.append ( pszStyleString + 1 );
+                oTmp.append ( pszStyleName );
 
-                poKmlPlacemark->set_styleurl ( oTmp );
+                poKmlFeature->set_styleurl ( oTmp );
             }
 
 
@@ -108,73 +111,71 @@ void featurestyle2kml (
                 if ( poOgrDS->GetStylePath (  ) )
                     oTmp.append ( poOgrDS->GetStylePath (  ) );
                 oTmp.append ( "#" );
-                oTmp.append ( pszStyleString + 1 );
+                oTmp.append ( pszStyleName );
 
-                poKmlPlacemark->set_styleurl ( oTmp );
+                poKmlFeature->set_styleurl ( oTmp );
             }
         }
 
         /***** no style table ref *****/
 
         else {
-            StylePtr poKmlStyle = poKmlFactory->CreateStyle (  );
-
             /***** parse the style string *****/
 
-            addstylestring2kml ( pszStyleString, poKmlStyle, poKmlFactory,
-                                 poKmlPlacemark, poOgrFeat );
+            StylePtr poKmlStyle = addstylestring2kml ( pszStyleString, NULL, poKmlFactory,
+                                 poKmlFeature );
 
             /***** add the style to the placemark *****/
-
-            poKmlPlacemark->set_styleselector ( poKmlStyle );
+            if( poKmlStyle != NULL )
+                poKmlFeature->set_styleselector ( poKmlStyle );
 
         }
     }
 
     /***** get the style table *****/
 
-    else if ( ( poOgrSTBL = poOgrFeat->GetStyleTable (  ) ) ) {
+    else if ( ( poOgrSTBL = poOgrFeat->GetStyleTable (  ) ) != NULL ) {
 
 
-        StylePtr poKmlStyle = poKmlFactory->CreateStyle (  );
+        StylePtr poKmlStyle = NULL;
 
         /***** parse the style table *****/
 
         poOgrSTBL->ResetStyleStringReading (  );
-        const char *pszStyleString;
 
-        while ( ( pszStyleString = poOgrSTBL->GetNextStyle (  ) ) ) {
+        while ( ( pszStyleString = poOgrSTBL->GetNextStyle (  ) ) != NULL ) {
 
             if ( *pszStyleString == '@' ) {
 
+                const char* pszStyleName = pszStyleString + 1;
+
                 /***** is the name in the layer style table *****/
 
                 OGRStyleTable *poOgrSTBLLayer;
-                const char *pszTest = NULL;
+                // const char *pszTest = NULL;
 
-                if ( ( poOgrSTBLLayer = poOgrLayer->GetStyleTable (  ) ) )
-                    poOgrSTBLLayer->Find ( pszStyleString );
+                if ( ( poOgrSTBLLayer = poOgrLayer->GetStyleTable (  ) ) != NULL )
+                    poOgrSTBLLayer->Find ( pszStyleName );
 
-                if ( pszTest ) {
-                    string oTmp = "#";
-
-                    oTmp.append ( pszStyleString + 1 );
-
-                    poKmlPlacemark->set_styleurl ( oTmp );
-                }
+                // if ( pszTest ) {
+                //     string oTmp = "#";
+                //     oTmp.append ( pszStyleName );
+                //     poKmlFeature->set_styleurl ( oTmp );
+                // }
 
                 /***** assume its a dataset style,      *****/
                 /***** mayby the user will add it later *****/
 
-                else {
+                /* else */
+                {
                     string oTmp;
 
                     if ( poOgrDS->GetStylePath (  ) )
                         oTmp.append ( poOgrDS->GetStylePath (  ) );
                     oTmp.append ( "#" );
-                    oTmp.append ( pszStyleString + 1 );
+                    oTmp.append ( pszStyleName );
 
-                    poKmlPlacemark->set_styleurl ( oTmp );
+                    poKmlFeature->set_styleurl ( oTmp );
                 }
             }
 
@@ -182,12 +183,14 @@ void featurestyle2kml (
 
                 /***** parse the style string *****/
 
-                addstylestring2kml ( pszStyleString, poKmlStyle,
-                                     poKmlFactory, poKmlPlacemark, poOgrFeat );
+                poKmlStyle = addstylestring2kml ( pszStyleString, poKmlStyle,
+                                     poKmlFactory, poKmlFeature );
+                if( poKmlStyle != NULL )
+                {
+                    /***** add the style to the placemark *****/
 
-                /***** add the style to the placemark *****/
-
-                poKmlPlacemark->set_styleselector ( poKmlStyle );
+                    poKmlFeature->set_styleselector ( poKmlStyle );
+                }
 
             }
         }
@@ -200,7 +203,7 @@ void featurestyle2kml (
 ******************************************************************************/
 
 void kml2featurestyle (
-    PlacemarkPtr poKmlPlacemark,
+    FeaturePtr poKmlFeature,
     OGRLIBKMLDataSource * poOgrDS,
     OGRLayer * poOgrLayer,
     OGRFeature * poOgrFeat )
@@ -208,9 +211,9 @@ void kml2featurestyle (
 
     /***** does the placemark have a style url? *****/
 
-    if (    poKmlPlacemark->has_styleurl (  ) ) {
+    if (    poKmlFeature->has_styleurl (  ) ) {
 
-        const string poKmlStyleUrl = poKmlPlacemark->get_styleurl (  );
+        const string poKmlStyleUrl = poKmlFeature->get_styleurl (  );
 
         /***** is the name in the layer style table *****/
 
@@ -222,7 +225,7 @@ void kml2featurestyle (
         /***** is it a layer style ? *****/
 
         if ( *pszUrl == '#'
-             && ( poOgrSTBLLayer = poOgrLayer->GetStyleTable (  ) ) )
+             && ( poOgrSTBLLayer = poOgrLayer->GetStyleTable (  ) ) != NULL )
         {
              pszTest = poOgrSTBLLayer->Find ( pszUrl + 1 );
         }
@@ -233,7 +236,7 @@ void kml2featurestyle (
 
             const char *pszResolve = CPLGetConfigOption ( "LIBKML_RESOLVE_STYLE", "no" );
 
-            if (EQUAL(pszResolve, "yes")) {
+            if (CPLTestBool(pszResolve)) {
 
                 poOgrFeat->SetStyleString ( pszTest );
             }
@@ -249,10 +252,10 @@ void kml2featurestyle (
         }
 
         /***** is it a dataset style? *****/
-        
+
         else {
 
-            int nPathLen = strlen ( poOgrDS->GetStylePath (  ) );
+            int nPathLen = static_cast<int>(strlen ( poOgrDS->GetStylePath (  ) ));
 
             if (    nPathLen == 0
                  || EQUALN ( pszUrl, poOgrDS->GetStylePath (  ), nPathLen ))
@@ -262,9 +265,9 @@ void kml2featurestyle (
 
                 const char *pszResolve = CPLGetConfigOption ( "LIBKML_RESOLVE_STYLE", "no" );
 
-                if (    EQUAL(pszResolve, "yes")
-                     && ( poOgrSTBLLayer = poOgrDS->GetStyleTable (  ) )
-                     && ( pszTest = poOgrSTBLLayer->Find ( pszUrl + nPathLen + 1) )
+                if ( CPLTestBool(pszResolve)
+                     && ( poOgrSTBLLayer = poOgrDS->GetStyleTable (  ) ) != NULL
+                     && ( pszTest = poOgrSTBLLayer->Find ( pszUrl + nPathLen + 1) ) != NULL
                    )
                 {
 
@@ -276,23 +279,22 @@ void kml2featurestyle (
                     pszUrl[nPathLen] = '@';
                     poOgrFeat->SetStyleString ( pszUrl + nPathLen );
                 }
-       
+
             }
-            
+
             /**** its someplace else *****/
 
             else {
 
                 const char *pszFetch = CPLGetConfigOption ( "LIBKML_EXTERNAL_STYLE", "no" );
 
-                if ( EQUAL(pszFetch, "yes") ) {
+                if ( CPLTestBool(pszFetch) ) {
 
                     /***** load up the style table *****/
 
                     char *pszUrlTmp = CPLStrdup(pszUrl);
-                    char *pszPound;
-                    if ((pszPound = strchr(pszUrlTmp, '#'))) {
-                        
+                    char *pszPound = strchr(pszUrlTmp, '#');
+                    if (pszPound != NULL) {
                         *pszPound = '\0';
                     }
 
@@ -303,8 +305,8 @@ void kml2featurestyle (
                     if (    (fp = VSIFOpenL( CPLFormFilename( "/vsicurl/",
                                                               pszUrlTmp,
                                                               NULL),
-                                             "r" ))
-                        ||  (fp = VSIFOpenL( pszUrlTmp, "r" )))
+                                             "r" )) != NULL
+                        ||  (fp = VSIFOpenL( pszUrlTmp, "r" )) != NULL )
                     {
 
                         char szbuf[1025];
@@ -313,11 +315,11 @@ void kml2featurestyle (
                         /***** loop, read and copy to a string *****/
 
                         size_t nRead;
-                    
+
                         do {
-                            
+
                             nRead = VSIFReadL(szbuf, 1, sizeof(szbuf) - 1, fp);
-                            
+
                             if (nRead == 0)
                                 break;
 
@@ -334,7 +336,7 @@ void kml2featurestyle (
 
                         if ( poOgrDS->ParseIntoStyleTable (&oStyle, pszUrlTmp)) {
 
-                            kml2featurestyle (poKmlPlacemark,
+                            kml2featurestyle (poKmlFeature,
                                               poOgrDS,
                                               poOgrLayer,
                                               poOgrFeat );
@@ -363,10 +365,10 @@ void kml2featurestyle (
 
     /***** does the placemark have a style selector *****/
 
-   if ( poKmlPlacemark->has_styleselector (  ) ) {
+   if ( poKmlFeature->has_styleselector (  ) ) {
 
         StyleSelectorPtr poKmlStyleSelector =
-            poKmlPlacemark->get_styleselector (  );
+            poKmlFeature->get_styleselector (  );
 
         /***** is the style a style? *****/
 
@@ -375,17 +377,17 @@ void kml2featurestyle (
 
             OGRStyleMgr *poOgrSM = new OGRStyleMgr;
 
-            /***** if were resolveing style the feature  *****/
+            /***** if were resolving style the feature *****/
             /***** might already have styling to add too *****/
-            
+
             const char *pszResolve = CPLGetConfigOption ( "LIBKML_RESOLVE_STYLE", "no" );
-            if (EQUAL(pszResolve, "yes")) {
+            if (CPLTestBool(pszResolve)) {
                  poOgrSM->InitFromFeature ( poOgrFeat );
             }
             else {
 
                 /***** if featyurestyle gets a name tool this needs changed to the above *****/
-                
+
                 poOgrSM->InitStyleString ( NULL );
             }
 
@@ -394,7 +396,7 @@ void kml2featurestyle (
             kml2stylestring ( poKmlStyle, poOgrSM );
 
             /***** add the style to the feature *****/
-            
+
             poOgrFeat->SetStyleString(poOgrSM->GetStyleString(NULL));
 
             delete poOgrSM;
