@@ -6,6 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2010, Brian Case
+ * Copyright (c) 2011-2014, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -26,11 +27,11 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
+#include "libkml_headers.h"
+
 #include <ogr_featurestyle.h>
 
-#include <kml/dom.h>
-#include <kml/engine.h>
-#include <kml/base/color32.h>
+#include <set>
 
 using kmldom::KmlFactory;;
 using kmldom::ElementPtr;
@@ -46,43 +47,46 @@ using kmldom::PolyStylePtr;
 using kmldom::IconStylePtr;
 using kmldom::IconStyleIconPtr;
 using kmldom::LabelStylePtr;
+using kmldom::BalloonStylePtr;
 using kmldom::HotSpotPtr;
 using kmlbase::Color32;
 using kmldom::PairPtr;
 using kmldom::KmlPtr;
+using kmldom::ListStylePtr;
+using kmldom::ItemIconPtr;
 
 #include "ogrlibkmlstyle.h"
+#include "ogr_libkml.h"
 
 /******************************************************************************
  generic function to parse a stylestring and add to a kml style
 
 args:
             pszStyleString  the stylestring to parse
-            poKmlStyle      the kml style to add to
+            poKmlStyle      the kml style to add to (or NULL)
             poKmlFactory    the kml dom factory
 
 returns:
-            nothing
+            the kml style
 
 ******************************************************************************/
 
-void addstylestring2kml (
+StylePtr addstylestring2kml (
     const char *pszStyleString,
     StylePtr poKmlStyle,
     KmlFactory * poKmlFactory,
-    PlacemarkPtr poKmlPlacemark,
-    OGRFeature * poOgrFeat )
+    FeaturePtr poKmlFeature )
 {
 
     LineStylePtr poKmlLineStyle = NULL;
     PolyStylePtr poKmlPolyStyle = NULL;
     IconStylePtr poKmlIconStyle = NULL;
     LabelStylePtr poKmlLabelStyle = NULL;
-    
+
     /***** just bail now if stylestring is empty *****/
 
     if ( !pszStyleString || !*pszStyleString ) {
-        return;
+        return poKmlStyle;
     }
 
     /***** create and init a style mamager with the style string *****/
@@ -101,7 +105,7 @@ void addstylestring2kml (
         if ( !poOgrST ) {
             continue;
         }
-        
+
         switch ( poOgrST->GetType (  ) ) {
         case OGRSTCPen:
             {
@@ -122,23 +126,25 @@ void addstylestring2kml (
 
                 if ( !nullcheck
                      && poStylePen->GetRGBFromString ( pszcolor, nR, nG, nB, nA ) ) {
-                    poKmlLineStyle->set_color ( Color32 ( nA, nB, nG, nR ) );
+                    poKmlLineStyle->set_color ( Color32 ( static_cast<GByte>(nA),
+                                                          static_cast<GByte>(nB),
+                                                          static_cast<GByte>(nG),
+                                                          static_cast<GByte>(nR) ) );
                 }
+                poStylePen->SetUnit(OGRSTUPixel);
                 double dfWidth = poStylePen->Width ( nullcheck );
 
                 if ( nullcheck )
                     dfWidth = 1.0;
 
                 poKmlLineStyle->set_width ( dfWidth );
-                
+
                 break;
             }
         case OGRSTCBrush:
             {
                 GBool nullcheck;
 
-                poKmlPolyStyle = poKmlFactory->CreatePolyStyle (  );
-
                 OGRStyleBrush *poStyleBrush = ( OGRStyleBrush * ) poOgrST;
 
                 /***** brush color *****/
@@ -152,9 +158,13 @@ void addstylestring2kml (
 
                 if ( !nullcheck
                      && poStyleBrush->GetRGBFromString ( pszcolor, nR, nG, nB, nA ) ) {
-                    poKmlPolyStyle->set_color ( Color32 ( nA, nB, nG, nR ) );
+                    poKmlPolyStyle = poKmlFactory->CreatePolyStyle (  );
+                    poKmlPolyStyle->set_color ( Color32 ( static_cast<GByte>(nA),
+                                                          static_cast<GByte>(nB),
+                                                          static_cast<GByte>(nG),
+                                                          static_cast<GByte>(nR) ) );
                 }
-                
+
 
                 break;
             }
@@ -230,7 +240,10 @@ void addstylestring2kml (
                 const char *pszcolor = poStyleSymbol->Color ( nullcheck );
 
                 if ( !nullcheck && poOgrST->GetRGBFromString ( pszcolor, nR, nG, nB, nA ) ) {
-                    poKmlIconStyle->set_color ( Color32 ( nA, nB, nG, nR ) );
+                    poKmlIconStyle->set_color ( Color32 ( static_cast<GByte>(nA),
+                                                          static_cast<GByte>(nB),
+                                                          static_cast<GByte>(nG),
+                                                          static_cast<GByte>(nR) ) );
                 }
 
                 /***** hotspot *****/
@@ -249,15 +262,13 @@ void addstylestring2kml (
 
                     poKmlIconStyle->set_hotspot ( poKmlHotSpot );
                 }
-                
+
                 break;
             }
         case OGRSTCLabel:
             {
                 GBool nullcheck;
                 GBool nullcheck2;
-                
-                poKmlLabelStyle = poKmlFactory->CreateLabelStyle (  );
 
                 OGRStyleLabel *poStyleLabel = ( OGRStyleLabel * ) poOgrST;
 
@@ -272,7 +283,12 @@ void addstylestring2kml (
 
                 if ( !nullcheck
                      && poStyleLabel->GetRGBFromString ( pszcolor, nR, nG, nB, nA ) ) {
-                    poKmlLabelStyle->set_color ( Color32 ( nA, nB, nG, nR ) );
+                    if( poKmlLabelStyle == NULL )
+                        poKmlLabelStyle = poKmlFactory->CreateLabelStyle (  );
+                    poKmlLabelStyle->set_color ( Color32 ( static_cast<GByte>(nA),
+                                                           static_cast<GByte>(nB),
+                                                           static_cast<GByte>(nG),
+                                                           static_cast<GByte>(nR) ) );
                 }
 
                 /***** scale *****/
@@ -281,9 +297,11 @@ void addstylestring2kml (
 
                 if ( !nullcheck ) {
                     dfScale /= 100.0;
+                    if( poKmlLabelStyle == NULL )
+                        poKmlLabelStyle = poKmlFactory->CreateLabelStyle (  );
                     poKmlLabelStyle->set_scale ( dfScale );
                 }
-                
+
                 /***** heading *****/
 
                 double heading = poStyleLabel->Angle ( nullcheck );
@@ -294,7 +312,7 @@ void addstylestring2kml (
                         IconStyleIconPtr poKmlIcon = poKmlFactory->CreateIconStyleIcon (  );
                         poKmlIconStyle->set_icon ( poKmlIcon );
                     }
-                    
+
                     poKmlIconStyle->set_heading ( heading );
                 }
 
@@ -309,7 +327,7 @@ void addstylestring2kml (
                         IconStyleIconPtr poKmlIcon = poKmlFactory->CreateIconStyleIcon (  );
                         poKmlIconStyle->set_icon ( poKmlIcon );
                     }
-                    
+
                     HotSpotPtr poKmlHotSpot = poKmlFactory->CreateHotSpot (  );
 
                     poKmlHotSpot->set_x ( dfDx );
@@ -323,12 +341,12 @@ void addstylestring2kml (
                 const char *pszText = poStyleLabel->TextString ( nullcheck );
 
                 if ( !nullcheck ) {
-                    if ( poKmlPlacemark ) {
+                    if ( poKmlFeature ) {
 
-                        poKmlPlacemark->set_name( pszText );
+                        poKmlFeature->set_name( pszText );
                     }
                 }
-                    
+
                 break;
             }
         case OGRSTCNone:
@@ -339,19 +357,27 @@ void addstylestring2kml (
         delete poOgrST;
     }
 
-    if ( poKmlLineStyle )
-        poKmlStyle->set_linestyle ( poKmlLineStyle );
+    if ( poKmlLineStyle || poKmlPolyStyle || poKmlIconStyle || poKmlLabelStyle )
+    {
+        if( poKmlStyle == NULL )
+            poKmlStyle = poKmlFactory->CreateStyle (  );
 
-    if ( poKmlPolyStyle )
-        poKmlStyle->set_polystyle ( poKmlPolyStyle );
+        if ( poKmlLineStyle )
+            poKmlStyle->set_linestyle ( poKmlLineStyle );
 
-    if ( poKmlIconStyle )
-        poKmlStyle->set_iconstyle ( poKmlIconStyle );
+        if ( poKmlPolyStyle )
+            poKmlStyle->set_polystyle ( poKmlPolyStyle );
+
+        if ( poKmlIconStyle )
+            poKmlStyle->set_iconstyle ( poKmlIconStyle );
+
+        if ( poKmlLabelStyle )
+            poKmlStyle->set_labelstyle ( poKmlLabelStyle );
+    }
 
-    if ( poKmlLabelStyle )
-        poKmlStyle->set_labelstyle ( poKmlLabelStyle );
-    
     delete poOgrSM;
+
+    return poKmlStyle;
 }
 
 /******************************************************************************
@@ -402,13 +428,13 @@ void kml2stylestring (
     int i;
 
     poOgrNewSM = new OGRStyleMgr( NULL );
-    
+
     /***** linestyle / pen *****/
 
     if ( poKmlStyle->has_linestyle (  ) ) {
 
         poOgrNewSM->InitStyleString ( NULL );
-        
+
         LineStylePtr poKmlLineStyle = poKmlStyle->get_linestyle (  );
 
         poOgrTmpST = NULL;
@@ -417,7 +443,7 @@ void kml2stylestring (
 
             if ( !poOgrST )
                 continue;
-        
+
             if ( poOgrST->GetType ( ) == OGRSTCPen ) {
                 poOgrTmpST = poOgrST;
             }
@@ -426,15 +452,15 @@ void kml2stylestring (
                 delete poOgrST;
             }
         }
-        
+
         OGRStylePen *poOgrStylePen = kml2pen ( poKmlLineStyle,
                                                ( OGRStylePen *) poOgrTmpST);
-        
+
         poOgrNewSM->AddPart ( poOgrStylePen );
 
         delete poOgrStylePen;
         poOgrSM->InitStyleString ( poOgrNewSM->GetStyleString(NULL) );
-        
+
     }
 
     /***** polystyle / brush *****/
@@ -451,7 +477,7 @@ void kml2stylestring (
 
             if ( !poOgrST )
                 continue;
-        
+
             if ( poOgrST->GetType ( ) == OGRSTCBrush ) {
                 poOgrTmpST = poOgrST;
             }
@@ -474,7 +500,7 @@ void kml2stylestring (
     /***** iconstyle / symbol *****/
 
     if ( poKmlStyle->has_iconstyle (  ) ) {
-        
+
         poOgrNewSM->InitStyleString ( NULL );
 
         IconStylePtr poKmlIconStyle = poKmlStyle->get_iconstyle (  );
@@ -485,7 +511,7 @@ void kml2stylestring (
 
             if ( !poOgrST )
                 continue;
-        
+
             if ( poOgrST->GetType ( ) == OGRSTCSymbol ) {
                 poOgrTmpST = poOgrST;
             }
@@ -508,7 +534,7 @@ void kml2stylestring (
     /***** labelstyle / label *****/
 
     if ( poKmlStyle->has_labelstyle (  ) ) {
-        
+
         poOgrNewSM->InitStyleString ( NULL );
 
         LabelStylePtr poKmlLabelStyle = poKmlStyle->get_labelstyle (  );
@@ -519,7 +545,7 @@ void kml2stylestring (
 
             if ( !poOgrST )
                 continue;
-        
+
             if ( poOgrST->GetType ( ) == OGRSTCLabel ) {
                 poOgrTmpST = poOgrST;
             }
@@ -558,7 +584,7 @@ OGRStylePen *kml2pen (
         poOgrStylePen = new OGRStylePen (  );
 
     /***** <LineStyle> should always have a width in pixels *****/
-    
+
     poOgrStylePen->SetUnit(OGRSTUPixel);
 
     /***** width *****/
@@ -701,7 +727,7 @@ OGRStyleLabel *kml2label (
 
         poOgrStyleLabel->SetStretch(dfScale);
     }
-    
+
     return poOgrStyleLabel;
 }
 
@@ -709,7 +735,7 @@ OGRStyleLabel *kml2label (
  function to add a kml style to a style table
 ******************************************************************************/
 
-void kml2styletable (
+static void kml2styletable (
     OGRStyleTable * poOgrStyleTable,
     StylePtr poKmlStyle )
 {
@@ -732,7 +758,7 @@ void kml2styletable (
         const std::string oName = poKmlStyle->get_id (  );
 
 
-        poOgrSM->AddStyle ( CPLString (  ).Printf ( "@%s",
+        poOgrSM->AddStyle ( CPLString (  ).Printf ( "%s",
                                                     oName.c_str (  ) ), NULL );
 
         /***** cleanup the style manager *****/
@@ -742,7 +768,7 @@ void kml2styletable (
 
     else {
         CPLError ( CE_Failure, CPLE_AppDefined,
-                   "ERROR Parseing kml Style: No id" );
+                   "ERROR parsing kml Style: No id" );
     }
 
     return;
@@ -753,31 +779,31 @@ void kml2styletable (
 ******************************************************************************/
 
 StyleSelectorPtr StyleFromStyleSelector(
-    const StyleSelectorPtr& poKmlStyleSelector, 
-    OGRStyleTable * poStyleTable) 
+    const StyleSelectorPtr& poKmlStyleSelector,
+    OGRStyleTable * poStyleTable)
 {
-    
+
     /***** is it a style? *****/
 
     if ( poKmlStyleSelector->IsA( kmldom::Type_Style) )
         return poKmlStyleSelector;
 
     /***** is it a style map? *****/
-    
+
     else if ( poKmlStyleSelector->IsA( kmldom::Type_StyleMap) )
         return StyleFromStyleMap(kmldom::AsStyleMap(poKmlStyleSelector), poStyleTable);
 
     /***** not a style or a style map *****/
-    
+
     return NULL;
 }
 
 
 /******************************************************************************
  function to get the container from the kmlroot
- 
+
  Args:          poKmlRoot   the root element
- 
+
  Returns:       root if its a container, if its a kml the container it
                 contains, or NULL
 
@@ -818,23 +844,22 @@ static ContainerPtr MyGetContainerFromRoot (
 
 
 
-StyleSelectorPtr StyleFromStyleURL(
+static StyleSelectorPtr StyleFromStyleURL(
     const StyleMapPtr& stylemap,
     const string styleurl,
-    OGRStyleTable * poStyleTable) 
+    OGRStyleTable * poStyleTable)
 {
     // TODO:: Parse the styleURL
 
     char *pszUrl = CPLStrdup ( styleurl.c_str (  ) );
     char *pszStyleMapId = CPLStrdup ( stylemap->get_id().c_str (  ) );
-    
 
-    /***** is it an interenal style ref that starts with a # *****/
+    /***** Is it an internal style ref that starts with a #? *****/
 
     if ( *pszUrl == '#' && poStyleTable ) {
 
-        /***** searh the style table for the style we *****/
-        /***** want and copy it back into the table   *****/
+        /***** Search the style table for the style we *****/
+        /***** want and copy it back into the table.   *****/
 
         const char *pszTest = NULL;
         pszTest = poStyleTable->Find ( pszUrl + 1 );
@@ -845,19 +870,19 @@ StyleSelectorPtr StyleFromStyleURL(
 
     /***** We have a real URL and need to go out and fetch it *****/
     /***** FIXME this could be a relative path in a kmz *****/
-    
+
     else if ( strchr(pszUrl, '#') ) {
 
         const char *pszFetch = CPLGetConfigOption ( "LIBKML_EXTERNAL_STYLE", "no" );
-        if ( EQUAL(pszFetch, "yes") ) {
+        if ( CPLTestBool(pszFetch) ) {
 
             /***** Lets go out and fetch the style from the external URL *****/
 
             char *pszUrlTmp = CPLStrdup(pszUrl);
-            char *pszPound;
+            char *pszPound = strchr(pszUrlTmp, '#');
             char *pszRemoteStyleName = NULL;
             // Chop off the stuff (style id) after the URL
-            if ((pszPound = strchr(pszUrlTmp, '#'))) {
+            if (pszPound != NULL) {
                 *pszPound = '\0';
                 pszRemoteStyleName = pszPound + 1;
             }
@@ -867,8 +892,8 @@ StyleSelectorPtr StyleFromStyleURL(
             VSILFILE *fp = NULL;
             if ( (fp = VSIFOpenL( CPLFormFilename( "/vsicurl/",
                                                    pszUrlTmp,
-                                                  NULL), "r" ))
-                 ||  (fp = VSIFOpenL( pszUrlTmp, "r" )) )
+                                                  NULL), "r" )) != NULL
+                 ||  (fp = VSIFOpenL( pszUrlTmp, "r" )) != NULL )
             {
                 char szbuf[1025];
                 std::string oStyle = "";
@@ -876,27 +901,27 @@ StyleSelectorPtr StyleFromStyleURL(
                 /***** loop, read and copy to a string *****/
 
                 size_t nRead;
-                do {                        
+                do {
                     nRead = VSIFReadL(szbuf, 1, sizeof(szbuf) - 1, fp);
                     if (nRead == 0)
                         break;
-                    
+
                     /***** copy buf to the string *****/
-                    
+
                     szbuf[nRead] = '\0';
-                    oStyle.append( szbuf );                        
+                    oStyle.append( szbuf );
                 } while (!VSIFEofL(fp));
-                
+
                 VSIFCloseL(fp);
 
                 /***** parse the kml into the dom *****/
-                
+
                 std::string oKmlErrors;
                 ElementPtr poKmlRoot = kmldom::Parse ( oStyle, &oKmlErrors );
 
                 if ( !poKmlRoot ) {
                     CPLError ( CE_Failure, CPLE_OpenFailed,
-                               "ERROR Parseing style kml %s :%s",
+                               "ERROR parsing style kml %s :%s",
                                pszUrlTmp, oKmlErrors.c_str (  ) );
                     CPLFree(pszUrlTmp);
                     CPLFree ( pszUrl );
@@ -906,7 +931,7 @@ StyleSelectorPtr StyleFromStyleURL(
                 }
 
                 /***** get the root container *****/
-                
+
                 ContainerPtr poKmlContainer;
                 kmldom::KmlFactory* poKmlFactory = kmldom::KmlFactory::GetFactory();
                 if ( !( poKmlContainer = MyGetContainerFromRoot ( poKmlFactory, poKmlRoot ) ) ) {
@@ -918,10 +943,10 @@ StyleSelectorPtr StyleFromStyleURL(
                 }
 
                 /**** parse the styles into the table *****/
-                
+
                 ParseStyles ( AsDocument ( poKmlContainer ), &poStyleTable );
-                    
-                /***** look for the style we leed to map to in the table *****/
+
+                /***** look for the style we need to map to in the table *****/
 
                 const char *pszTest = NULL;
                 pszTest = poStyleTable->Find(pszRemoteStyleName);
@@ -935,8 +960,8 @@ StyleSelectorPtr StyleFromStyleURL(
         }
     }
 
-    /***** FIXME add suport here for relative links inside kml *****/
-    
+    /***** FIXME Add support here for relative links inside KML. *****/
+
     CPLFree ( pszUrl );
     CPLFree ( pszStyleMapId );
 
@@ -945,7 +970,7 @@ StyleSelectorPtr StyleFromStyleURL(
 
 StyleSelectorPtr StyleFromStyleMap(
     const StyleMapPtr& poKmlStyleMap,
-    OGRStyleTable * poStyleTable) 
+    OGRStyleTable * poStyleTable)
 {
 
     /***** check the config option to see if the    *****/
@@ -962,9 +987,9 @@ StyleSelectorPtr StyleFromStyleMap(
         PairPtr myPair = poKmlStyleMap->get_pair_array_at(i);
 
         /***** is it the right one of the pair? *****/
-        
+
         if ( myPair->get_key() == nStyleMapKey ) {
-            
+
             if (myPair->has_styleselector())
                 return StyleFromStyleSelector(myPair->get_styleselector(), poStyleTable);
 
@@ -1020,8 +1045,8 @@ void ParseStyles (
         kml2styletable ( *poStyleTable, AsStyle ( poKmlElement ) );
     }
 
-    /***** Now we have to loop back around and get the style maps. We    *****/ 
-    /***** have to do this a second time since the stylemap might matter *****/ 
+    /***** Now we have to loop back around and get the style maps. We    *****/
+    /***** have to do this a second time since the stylemap might matter *****/
     /***** and we are just looping reference styles that are farther     *****/
     /***** down in the file. Order through the XML as it is parsed.      *****/
 
@@ -1041,7 +1066,7 @@ void ParseStyles (
             *poStyleTable = new OGRStyleTable (  );
 
         /***** copy the style the style map points to since *****/
-        
+
         char *pszStyleMapId = CPLStrdup ( poKmlStyle->get_id().c_str (  ) );
         poKmlStyle = StyleFromStyleMap(kmldom::AsStyleMap(poKmlStyle), *poStyleTable);
         if (poKmlStyle == NULL) {
@@ -1079,7 +1104,8 @@ void ParseStyles (
 void styletable2kml (
     OGRStyleTable * poOgrStyleTable,
     KmlFactory * poKmlFactory,
-    ContainerPtr poKmlContainer )
+    ContainerPtr poKmlContainer,
+    char** papszOptions )
 {
 
     /***** just return if the styletable is null *****/
@@ -1087,33 +1113,163 @@ void styletable2kml (
     if ( !poOgrStyleTable )
         return;
 
-    /***** parse the style table *****/
-
+    std::set<CPLString> aoSetNormalStyles;
+    std::set<CPLString> aoSetHighlightStyles;
     poOgrStyleTable->ResetStyleStringReading (  );
     const char *pszStyleString;
 
-    while ( ( pszStyleString = poOgrStyleTable->GetNextStyle (  ) ) ) {
+    /* Collect styles that end with _normal or _highlight */
+    while ( poOgrStyleTable->GetNextStyle (  ) != NULL ) {
         const char *pszStyleName = poOgrStyleTable->GetLastStyleName (  );
 
+        if( strlen(pszStyleName) > strlen("_normal") &&
+            EQUAL(pszStyleName + strlen(pszStyleName) - strlen("_normal"), "_normal") )
+        {
+            CPLString osName(pszStyleName);
+            osName.resize(strlen(pszStyleName) - strlen("_normal"));
+            aoSetNormalStyles.insert(osName);
+        }
+        else if( strlen(pszStyleName) > strlen("_highlight") &&
+                  EQUAL(pszStyleName + strlen(pszStyleName) - strlen("_highlight"), "_highlight") )
+        {
+            CPLString osName(pszStyleName);
+            osName.resize(strlen(pszStyleName) - strlen("_highlight"));
+            aoSetHighlightStyles.insert(osName);
+        }
+    }
+
+    /***** parse the style table *****/
+
+    poOgrStyleTable->ResetStyleStringReading (  );
+
+    while ( ( pszStyleString = poOgrStyleTable->GetNextStyle (  ) ) != NULL ) {
+        const char *pszStyleName = poOgrStyleTable->GetLastStyleName (  );
+
+        if( aoSetNormalStyles.find(pszStyleName) != aoSetNormalStyles.end() &&
+            aoSetHighlightStyles.find(pszStyleName) != aoSetHighlightStyles.end() )
+        {
+            continue;
+        }
+
         /***** add the style header to the kml *****/
 
         StylePtr poKmlStyle = poKmlFactory->CreateStyle (  );
 
-        poKmlStyle->set_id ( pszStyleName + 1 );
+        poKmlStyle->set_id ( pszStyleName );
 
         /***** parse the style string *****/
 
-        addstylestring2kml ( pszStyleString, poKmlStyle, poKmlFactory, NULL, NULL );
+        addstylestring2kml ( pszStyleString, poKmlStyle, poKmlFactory, NULL );
+
+        /***** add balloon style *****/
+        const char* pszBalloonStyleBgColor = CSLFetchNameValue(papszOptions, CPLSPrintf("%s_balloonstyle_bgcolor", pszStyleName));
+        const char* pszBalloonStyleText = CSLFetchNameValue(papszOptions, CPLSPrintf("%s_balloonstyle_text", pszStyleName));
+        int nR, nG, nB, nA;
+        OGRStylePen oStyleTool;
+        if( (pszBalloonStyleBgColor != NULL &&
+             oStyleTool.GetRGBFromString ( pszBalloonStyleBgColor, nR, nG, nB, nA )  ) ||
+            pszBalloonStyleText != NULL )
+        {
+            BalloonStylePtr poKmlBalloonStyle = poKmlFactory->CreateBalloonStyle();
+            if( pszBalloonStyleBgColor != NULL &&
+                oStyleTool.GetRGBFromString ( pszBalloonStyleBgColor, nR, nG, nB, nA ) )
+                poKmlBalloonStyle->set_bgcolor ( Color32 ( static_cast<GByte>(nA),
+                                                           static_cast<GByte>(nB),
+                                                           static_cast<GByte>(nG),
+                                                           static_cast<GByte>(nR) ) );
+            if( pszBalloonStyleText != NULL )
+                poKmlBalloonStyle->set_text(pszBalloonStyleText);
+            poKmlStyle->set_balloonstyle ( poKmlBalloonStyle );
+        }
 
         /***** add the style to the container *****/
 
         DocumentPtr poKmlDocument = AsDocument ( poKmlContainer );
-
-        //ObjectPtr pokmlObject = boost::static_pointer_cast <kmldom::Object> () ;
-        //poKmlContainer->add_feature ( AsFeature( poKmlStyle) );
         poKmlDocument->add_styleselector ( poKmlStyle );
 
     }
 
+    /* Find style name that end with _normal and _highlight to create */
+    /* a StyleMap from both */
+    std::set<CPLString>::iterator aoSetNormalStylesIter =
+        aoSetNormalStyles.begin();
+    for( ; aoSetNormalStylesIter != aoSetNormalStyles.end(); ++aoSetNormalStylesIter )
+    {
+        CPLString osStyleName(*aoSetNormalStylesIter);
+        if( aoSetHighlightStyles.find(osStyleName) !=
+                aoSetHighlightStyles.end() )
+        {
+            StyleMapPtr poKmlStyleMap = poKmlFactory->CreateStyleMap (  );
+            poKmlStyleMap->set_id ( osStyleName );
+
+            PairPtr poKmlPairNormal = poKmlFactory->CreatePair (  );
+            poKmlPairNormal->set_key(STYLESTATE_NORMAL);
+            poKmlPairNormal->set_styleurl(CPLSPrintf("#%s_normal", osStyleName.c_str()));
+            poKmlStyleMap->add_pair(poKmlPairNormal);
+
+            PairPtr poKmlPairHightlight = poKmlFactory->CreatePair (  );
+            poKmlPairHightlight->set_key(STYLESTATE_HIGHLIGHT);
+            poKmlPairHightlight->set_styleurl(CPLSPrintf("#%s_highlight", osStyleName.c_str()));
+            poKmlStyleMap->add_pair(poKmlPairHightlight);
+
+            /***** add the style to the container *****/
+            DocumentPtr poKmlDocument = AsDocument ( poKmlContainer );
+            poKmlDocument->add_styleselector ( poKmlStyleMap );
+        }
+    }
+
     return;
 }
+
+
+/******************************************************************************
+ function to add a ListStyle and select it to a container
+******************************************************************************/
+
+void createkmlliststyle (
+    KmlFactory * poKmlFactory,
+    const char* pszBaseName,
+    ContainerPtr poKmlLayerContainer,
+    DocumentPtr poKmlDocument,
+    const CPLString& osListStyleType,
+    const CPLString& osListStyleIconHref)
+{
+    if( osListStyleType.size() || osListStyleIconHref.size() )
+    {
+        StylePtr poKmlStyle = poKmlFactory->CreateStyle (  );
+
+        const char* pszStyleName = CPLSPrintf("%s_liststyle", OGRLIBKMLGetSanitizedNCName(pszBaseName).c_str());
+        poKmlStyle->set_id ( pszStyleName );
+
+        ListStylePtr poKmlListStyle = poKmlFactory->CreateListStyle (  );
+        poKmlStyle->set_liststyle ( poKmlListStyle );
+        if( osListStyleType.size() )
+        {
+            if( EQUAL(osListStyleType, "check") )
+                poKmlListStyle->set_listitemtype( kmldom::LISTITEMTYPE_CHECK );
+            else if( EQUAL(osListStyleType, "radioFolder") )
+                poKmlListStyle->set_listitemtype( kmldom::LISTITEMTYPE_RADIOFOLDER );
+            else if( EQUAL(osListStyleType, "checkOffOnly") )
+                poKmlListStyle->set_listitemtype( kmldom::LISTITEMTYPE_CHECKOFFONLY );
+            else if( EQUAL(osListStyleType, "checkHideChildren") )
+                poKmlListStyle->set_listitemtype( kmldom::LISTITEMTYPE_CHECKHIDECHILDREN );
+            else
+            {
+                CPLError(CE_Warning, CPLE_AppDefined,
+                         "Invalid value for list style type: %s. Defaulting to Check",
+                         osListStyleType.c_str());
+                poKmlListStyle->set_listitemtype( kmldom::LISTITEMTYPE_CHECK );
+            }
+        }
+
+        if( osListStyleIconHref.size() )
+        {
+            ItemIconPtr poItemIcon = poKmlFactory->CreateItemIcon (  );
+            poItemIcon->set_href( osListStyleIconHref.c_str() );
+            poKmlListStyle->add_itemicon(poItemIcon);
+        }
+
+        poKmlDocument->add_styleselector ( poKmlStyle );
+        poKmlLayerContainer->set_styleurl( CPLSPrintf("#%s", pszStyleName) );
+    }
+}
