@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_mdb.h 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogr_mdb.h 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Private definitions for MDB driver.
@@ -27,8 +27,8 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _OGR_MDB_H_INCLUDED
-#define _OGR_MDB_H_INCLUDED
+#ifndef OGR_MDB_H_INCLUDED
+#define OGR_MDB_H_INCLUDED
 
 #include <jni.h>
 #include <vector>
@@ -54,7 +54,7 @@ class OGRMDBJavaEnv
     JNIEnv *env;
     int bCalledFromJava;
 
-    int ExceptionOccured();
+    int ExceptionOccurred();
 
     jclass byteArray_class;
 
@@ -105,7 +105,7 @@ class OGRMDBJavaEnv
 
     jclass short_class;
     jmethodID short_shortValue;
-    
+
     jclass integer_class;
     jmethodID integer_intValue;
 
@@ -188,7 +188,7 @@ public:
 
 };
 
-typedef enum 
+typedef enum
 {
     MDB_Boolean = 0x01,
     MDB_Byte = 0x02,
@@ -219,7 +219,7 @@ typedef enum
 /************************************************************************/
 
 class OGRMDBDataSource;
-    
+
 class OGRMDBLayer : public OGRLayer
 {
   protected:
@@ -269,12 +269,12 @@ class OGRMDBLayer : public OGRLayer
                                     OGRSpatialReference* poSRS );
 
     virtual void        ResetReading();
-    virtual int         GetFeatureCount( int bForce );
+    virtual GIntBig     GetFeatureCount( int bForce );
     virtual OGRFeature *GetNextRawFeature();
     virtual OGRFeature *GetNextFeature();
 
-    virtual OGRFeature *GetFeature( long nFeatureId );
-    
+    virtual OGRFeature *GetFeature( GIntBig nFeatureId );
+
     OGRFeatureDefn *    GetLayerDefn() { return poFeatureDefn; }
 
     virtual int         TestCapability( const char * );
@@ -282,6 +282,8 @@ class OGRMDBLayer : public OGRLayer
     virtual const char *GetFIDColumn();
 
     virtual OGRErr      GetExtent( OGREnvelope *psExtent, int bForce );
+    virtual OGRErr      GetExtent(int iGeomField, OGREnvelope *psExtent, int bForce)
+                { return OGRLayer::GetExtent(iGeomField, psExtent, bForce); }
 };
 
 /************************************************************************/
@@ -311,8 +313,8 @@ class OGRMDBDataSource : public OGRDataSource
                         OGRMDBDataSource();
                         ~OGRMDBDataSource();
 
-    int                 Open( const char *, int bUpdate, int bTestOpen );
-    int                 OpenTable( const char *pszTableName, 
+    int                 Open( const char * );
+    int                 OpenTable( const char *pszTableName,
                                    const char *pszGeomCol,
                                    int bUpdate );
 
@@ -339,4 +341,4 @@ class OGRMDBDriver : public OGRSFDriver
     int          TestCapability( const char * );
 };
 
-#endif /* ndef _OGR_MDB_H_INCLUDED */
+#endif /* ndef OGR_MDB_H_INCLUDED */
