@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrmdblayer.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogrmdblayer.cpp 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRMDBLayer class
@@ -33,17 +33,17 @@
 #include "ogrpgeogeometry.h"
 #include "ogrgeomediageometry.h"
 
-CPL_CVSID("$Id: ogrmdblayer.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: ogrmdblayer.cpp 33714 2016-03-13 05:42:13Z goatbar $");
 
 /************************************************************************/
 /*                            OGRMDBLayer()                            */
 /************************************************************************/
 
-OGRMDBLayer::OGRMDBLayer(OGRMDBDataSource* poDS, OGRMDBTable* poMDBTable)
+OGRMDBLayer::OGRMDBLayer(OGRMDBDataSource* poDSIn, OGRMDBTable* poMDBTableIn)
 
 {
-    this->poDS = poDS;
-    this->poMDBTable = poMDBTable;
+    this->poDS = poDSIn;
+    this->poMDBTable = poMDBTableIn;
 
     eGeometryType = MDB_GEOM_NONE;
 
@@ -73,7 +73,7 @@ OGRMDBLayer::~OGRMDBLayer()
     if( m_nFeaturesRead > 0 && poFeatureDefn != NULL )
     {
         CPLDebug( "MDB", "%d features read on layer '%s'.",
-                  (int) m_nFeaturesRead, 
+                  (int) m_nFeaturesRead,
                   poFeatureDefn->GetName() );
     }
 
@@ -108,6 +108,7 @@ CPLErr OGRMDBLayer::BuildFeatureDefn()
 
 {
     poFeatureDefn = new OGRFeatureDefn( poMDBTable->GetName() );
+    SetDescription( poFeatureDefn->GetName() );
 
     poFeatureDefn->Reference();
 
@@ -132,13 +133,13 @@ CPLErr OGRMDBLayer::BuildFeatureDefn()
         }
 
         if( eGeometryType == MDB_GEOM_PGEO
-            && pszGeomColumn == NULL 
+            && pszGeomColumn == NULL
             && EQUAL(pszColName,"Shape") )
         {
             pszGeomColumn = CPLStrdup(pszColName);
             continue;
         }
-        
+
         switch( poMDBTable->GetColumnType(iCol) )
         {
           case MDB_Boolean:
@@ -200,7 +201,7 @@ void OGRMDBLayer::ResetReading()
 /*                          GetFeatureCount()                           */
 /************************************************************************/
 
-int OGRMDBLayer::GetFeatureCount(int bForce)
+GIntBig OGRMDBLayer::GetFeatureCount(int bForce)
 {
     if (m_poFilterGeom != NULL || m_poAttrQuery != NULL)
         return OGRLayer::GetFeatureCount(bForce);
@@ -214,7 +215,7 @@ int OGRMDBLayer::GetFeatureCount(int bForce)
 OGRFeature *OGRMDBLayer::GetNextFeature()
 
 {
-    for( ; TRUE; )
+    while( true )
     {
         OGRFeature      *poFeature;
 
@@ -251,7 +252,7 @@ OGRFeature *OGRMDBLayer::GetNextRawFeature()
     OGRFeature *poFeature = new OGRFeature( poFeatureDefn );
 
     if( pszFIDColumn != NULL && poMDBTable->GetColumnIndex(pszFIDColumn) > -1 )
-        poFeature->SetFID( 
+        poFeature->SetFID(
             poMDBTable->GetColumnAsInt(poMDBTable->GetColumnIndex(pszFIDColumn)) );
     else
         poFeature->SetFID( iNextShapeId );
@@ -274,7 +275,7 @@ OGRFeature *OGRMDBLayer::GetNextRawFeature()
         {
             int nBytes = 0;
             GByte* pData = poMDBTable->GetColumnAsBinary( iSrcField, &nBytes);
-            poFeature->SetField( iField, 
+            poFeature->SetField( iField,
                                  nBytes,
                                  pData );
             CPLFree(pData);
@@ -356,7 +357,7 @@ OGRFeature *OGRMDBLayer::GetNextRawFeature()
 /*                             GetFeature()                             */
 /************************************************************************/
 
-OGRFeature *OGRMDBLayer::GetFeature( long nFeatureId )
+OGRFeature *OGRMDBLayer::GetFeature( GIntBig nFeatureId )
 
 {
     /* This should be implemented directly! */
@@ -445,7 +446,7 @@ void OGRMDBLayer::LookupSRID( int nSRID )
     char* pszSRTextPtr = pszSRText;
     if( poSRS->importFromWkt( &pszSRTextPtr ) != OGRERR_NONE )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "importFromWKT() failed on SRS '%s'.",
                   pszSRText);
         delete poSRS;
@@ -453,7 +454,7 @@ void OGRMDBLayer::LookupSRID( int nSRID )
     }
     else if( poSRS->morphFromESRI() != OGRERR_NONE )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "morphFromESRI() failed on SRS." );
         delete poSRS;
         poSRS = NULL;
@@ -482,7 +483,7 @@ const char *OGRMDBLayer::GetFIDColumn()
 /*                             Initialize()                             */
 /************************************************************************/
 
-CPLErr OGRMDBLayer::Initialize( const char *pszTableName,
+CPLErr OGRMDBLayer::Initialize( CPL_UNUSED const char *pszTableName,
                                 const char *pszGeomCol,
                                 int nShapeType,
                                 double dfExtentLeft,
@@ -557,7 +558,7 @@ CPLErr OGRMDBLayer::Initialize( const char *pszTableName,
     }
 
     if( eOGRType != wkbUnknown && eOGRType != wkbNone && bHasZ )
-        eOGRType = (OGRwkbGeometryType)(((int) eOGRType) | wkb25DBit);
+        eOGRType = wkbSetZ(eOGRType);
 
     poFeatureDefn->SetGeomType(eOGRType);
 
@@ -569,9 +570,9 @@ CPLErr OGRMDBLayer::Initialize( const char *pszTableName,
 /*                             Initialize()                             */
 /************************************************************************/
 
-CPLErr OGRMDBLayer::Initialize( const char *pszTableName,
+CPLErr OGRMDBLayer::Initialize( const char * /*pszTableName */,
                                 const char *pszGeomCol,
-                                OGRSpatialReference* poSRS )
+                                OGRSpatialReference* poSRSIn )
 
 
 {
@@ -590,7 +591,7 @@ CPLErr OGRMDBLayer::Initialize( const char *pszTableName,
 
     eGeometryType = MDB_GEOM_GEOMEDIA;
 
-    this->poSRS = poSRS;
+    this->poSRS = poSRSIn;
 
     CPLErr eErr;
     eErr = BuildFeatureDefn();
