@@ -486,10 +486,12 @@ OGRFeature *IMapInfoFile::GetFeature(GIntBig nFeatureId)
 
 int IMapInfoFile::GetTABType( OGRFieldDefn *poField,
                               TABFieldType* peTABType,
-                              int *pnWidth)
+                              int *pnWidth,
+                              int *pnPrecision)
 {
     TABFieldType        eTABType;
     int                 nWidth = poField->GetWidth();
+    int                 nPrecision = poField->GetPrecision();
 
     if( poField->GetType() == OFTInteger )
     {
@@ -507,6 +509,21 @@ int IMapInfoFile::GetTABType( OGRFieldDefn *poField,
         else
         {
             eTABType = TABFDecimal;
+            // Enforce Mapinfo limits, otherwise MapInfo will crash (#6392)
+            if( nWidth > 20 || nWidth - nPrecision < 2 || nPrecision > 16 )
+            {
+                if( nWidth > 20 )
+                    nWidth = 20;
+                if( nWidth - nPrecision < 2 )
+                    nPrecision = nWidth - 2;
+                if( nPrecision > 16 )
+                    nPrecision = 16;
+                CPLDebug( "MITAB",
+                          "Adjusting initial width,precision of %s from %d,%d to %d,%d",
+                          poField->GetNameRef(),
+                          poField->GetWidth(), poField->GetPrecision(),
+                          nWidth, nPrecision );
+            }
         }
     }
     else if( poField->GetType() == OFTDate )
@@ -548,6 +565,7 @@ int IMapInfoFile::GetTABType( OGRFieldDefn *poField,
 
     *peTABType = eTABType;
     *pnWidth = nWidth;
+    *pnPrecision = nPrecision;
 
     return 0;
 }
@@ -563,12 +581,13 @@ OGRErr IMapInfoFile::CreateField( OGRFieldDefn *poField, int bApproxOK )
 {
     TABFieldType        eTABType;
     int                 nWidth;
+    int                 nPrecision;
 
-    if( GetTABType( poField, &eTABType, &nWidth ) < 0 )
+    if( GetTABType( poField, &eTABType, &nWidth, &nPrecision ) < 0 )
         return OGRERR_FAILURE;
 
     if( AddFieldNative( poField->GetNameRef(), eTABType,
-                        nWidth, poField->GetPrecision(), FALSE, FALSE, bApproxOK ) > -1 )
+                        nWidth, nPrecision, FALSE, FALSE, bApproxOK ) > -1 )
         return OGRERR_NONE;
     else
         return OGRERR_FAILURE;
