@@ -18,16 +18,16 @@
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  *
@@ -38,9 +38,6 @@
  * Revision 1.11  2006/01/27 14:27:35  fwarmerdam
  * fixed ogr bounds setting problems (bug 1198)
  *
- * Revision 1.10  2005/09/20 04:40:02  fwarmerdam
- * fixed CPLReadDir memory leak
- *
  * Revision 1.9  2004/10/15 01:52:30  fwarmerdam
  * ModifiedICreateLayer() to use  -1000,-1000,1000,1000 bounds for GEOGCS
  * much like in mitab_bounds.cpp.  This ensures that geographic files in
@@ -112,7 +109,7 @@ OGRTABDataSource::OGRTABDataSource()
 OGRTABDataSource::~OGRTABDataSource()
 
 {
-    CPLFree( m_pszName ); 
+    CPLFree( m_pszName );
     CPLFree( m_pszDirectory );
 
     for( int i = 0; i < m_nLayerCount; i++ )
@@ -135,12 +132,12 @@ int OGRTABDataSource::Create( const char * pszName, char **papszOptions )
     const char *pszOpt;
 
     CPLAssert( m_pszName == NULL );
-    
+
     m_pszName = CPLStrdup( pszName );
     m_papszOptions = CSLDuplicate( papszOptions );
     m_bUpdate = TRUE;
 
-    if( (pszOpt=CSLFetchNameValue(papszOptions,"FORMAT")) != NULL 
+    if( (pszOpt=CSLFetchNameValue(papszOptions,"FORMAT")) != NULL
         && EQUAL(pszOpt, "MIF") )
         m_bCreateMIF = TRUE;
     else if( EQUAL(CPLGetExtension(pszName),"mif")
@@ -183,7 +180,7 @@ int OGRTABDataSource::Create( const char * pszName, char **papszOptions )
                 return FALSE;
             }
         }
-        
+
         m_pszDirectory = CPLStrdup(pszName);
     }
 
@@ -213,11 +210,11 @@ int OGRTABDataSource::Create( const char * pszName, char **papszOptions )
             }
             poFile = poTabFile;
         }
-        
+
         m_nLayerCount = 1;
         m_papoLayers = (IMapInfoFile **) CPLMalloc(sizeof(void*));
         m_papoLayers[0] = poFile;
-        
+
         m_pszDirectory = CPLStrdup( CPLGetPath(pszName) );
         m_bSingleFile = TRUE;
     }
@@ -268,8 +265,8 @@ int OGRTABDataSource::Open( GDALOpenInfo* poOpenInfo, int bTestOpen )
 /* -------------------------------------------------------------------- */
     else
     {
-        char    **papszFileList = CPLReadDir( m_pszName );
-        
+        char    **papszFileList = VSIReadDir( m_pszName );
+
         m_pszDirectory = CPLStrdup( m_pszName );
 
         for( int iFile = 0;
@@ -288,7 +285,7 @@ int OGRTABDataSource::Open( GDALOpenInfo* poOpenInfo, int bTestOpen )
 
             poFile = IMapInfoFile::SmartOpen( pszSubFilename, m_bUpdate, bTestOpen );
             CPLFree( pszSubFilename );
-            
+
             if( poFile == NULL )
             {
                 CSLDestroy( papszFileList );
@@ -310,7 +307,7 @@ int OGRTABDataSource::Open( GDALOpenInfo* poOpenInfo, int bTestOpen )
                 CPLError( CE_Failure, CPLE_OpenFailed,
                           "No mapinfo files found in directory %s.\n",
                           m_pszDirectory );
-            
+
             return FALSE;
         }
     }
@@ -361,7 +358,7 @@ OGRTABDataSource::ICreateLayer( const char * pszLayerName,
 
     if( !m_bUpdate )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                     "Cannot create layer on read-only dataset.");
         return NULL;
     }
@@ -375,7 +372,7 @@ OGRTABDataSource::ICreateLayer( const char * pszLayerName,
     {
         if( m_bSingleLayerAlreadyCreated )
         {
-            CPLError( CE_Failure, CPLE_AppDefined, 
+            CPLError( CE_Failure, CPLE_AppDefined,
                       "Unable to create new layers in this single file dataset.");
             return NULL;
         }
@@ -391,24 +388,32 @@ OGRTABDataSource::ICreateLayer( const char * pszLayerName,
         {
             pszFullFilename = CPLStrdup( CPLFormFilename( m_pszDirectory,
                                                           pszLayerName, "mif" ) );
-            
+
             poFile = new MIFFile;
+
+            if( poFile->Open( pszFullFilename, TABWrite, FALSE ) != 0 )
+            {
+                CPLFree( pszFullFilename );
+                delete poFile;
+                return NULL;
+            }
         }
         else
         {
             pszFullFilename = CPLStrdup( CPLFormFilename( m_pszDirectory,
                                                           pszLayerName, "tab" ) );
-            
-            poFile = new TABFile;
-        }
-        
-        if( poFile->Open( pszFullFilename, TABWrite, FALSE ) != 0 )
-        {
-            CPLFree( pszFullFilename );
-            delete poFile;
-            return FALSE;
+
+            TABFile* poTABFile = new TABFile;
+            if( poTABFile->Open( pszFullFilename, TABWrite, FALSE, m_nBlockSize ) != 0 )
+            {
+                CPLFree( pszFullFilename );
+                delete poTABFile;
+                return NULL;
+            }
+            poFile = poTABFile;
         }
 
+
         m_nLayerCount++;
         m_papoLayers = (IMapInfoFile **)
             CPLRealloc(m_papoLayers,sizeof(void*)*m_nLayerCount);
@@ -433,9 +438,9 @@ OGRTABDataSource::ICreateLayer( const char * pszLayerName,
     // Pull out the bounds if supplied
     if( (pszOpt=CSLFetchNameValue(papszOptions, "BOUNDS")) != NULL ) {
         double dfBounds[4];
-        if( CPLsscanf(pszOpt, "%lf,%lf,%lf,%lf", &dfBounds[0], 
-                                          &dfBounds[1], 
-                                          &dfBounds[2], 
+        if( CPLsscanf(pszOpt, "%lf,%lf,%lf,%lf", &dfBounds[0],
+                                          &dfBounds[1],
+                                          &dfBounds[2],
                                           &dfBounds[3]) != 4 )
         {
             CPLError( CE_Failure, CPLE_IllegalArg,
@@ -458,12 +463,12 @@ OGRTABDataSource::ICreateLayer( const char * pszLayerName,
 
     if (m_bQuickSpatialIndexMode == TRUE && poFile->SetQuickSpatialIndexMode(TRUE) != 0)
     {
-        CPLError( CE_Warning, CPLE_AppDefined, 
+        CPLError( CE_Warning, CPLE_AppDefined,
                   "Setting Quick Spatial Index Mode failed.");
     }
     else if (m_bQuickSpatialIndexMode == FALSE && poFile->SetQuickSpatialIndexMode(FALSE) != 0)
     {
-        CPLError( CE_Warning, CPLE_AppDefined, 
+        CPLError( CE_Warning, CPLE_AppDefined,
                   "Setting Normal Spatial Index Mode failed.");
     }
 
@@ -492,23 +497,23 @@ char **OGRTABDataSource::GetFileList()
     VSIStatBufL sStatBuf;
     CPLStringList osList;
 
-    VSIStatL( m_pszName, &sStatBuf );
-    if( VSI_ISDIR(sStatBuf.st_mode) )
+    if( VSIStatL( m_pszName, &sStatBuf ) == 0 &&
+        VSI_ISDIR(sStatBuf.st_mode) )
     {
-        static const char *apszExtensions[] = 
+        static const char * const apszExtensions[] =
             { "mif", "mid", "tab", "map", "ind", "dat", "id", NULL };
-        char **papszDirEntries = CPLReadDir( m_pszName );
+        char **papszDirEntries = VSIReadDir( m_pszName );
         int  iFile;
 
-        for( iFile = 0; 
+        for( iFile = 0;
              papszDirEntries != NULL && papszDirEntries[iFile] != NULL;
              iFile++ )
         {
-            if( CSLFindString( (char **) apszExtensions, 
+            if( CSLFindString( (char **) apszExtensions,
                                CPLGetExtension(papszDirEntries[iFile])) != -1)
             {
-                osList.AddString( CPLFormFilename( m_pszName, 
-                                            papszDirEntries[iFile], 
+                osList.AddString( CPLFormFilename( m_pszName,
+                                            papszDirEntries[iFile],
                                             NULL ) );
             }
         }
@@ -517,9 +522,9 @@ char **OGRTABDataSource::GetFileList()
     }
     else
     {
-        static const char* apszMIFExtensions[] = { "mif", "mid", NULL };
-        static const char* apszTABExtensions[] = { "tab", "map", "ind", "dat", "id", NULL };
-        const char** papszExtensions;
+        static const char* const apszMIFExtensions[] = { "mif", "mid", NULL };
+        static const char* const apszTABExtensions[] = { "tab", "map", "ind", "dat", "id", NULL };
+        const char* const * papszExtensions;
         if( EQUAL(CPLGetExtension(m_pszName), "mif") ||
             EQUAL(CPLGetExtension(m_pszName), "mid") )
         {
@@ -529,7 +534,7 @@ char **OGRTABDataSource::GetFileList()
         {
             papszExtensions = apszTABExtensions;
         }
-        const char** papszIter = papszExtensions;
+        const char* const * papszIter = papszExtensions;
         while( *papszIter )
         {
             const char *pszFile = CPLResetExtension(m_pszName, *papszIter );
