@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_mssqlspatial.h 27344 2014-05-15 21:13:54Z tamas $
+ * $Id: ogr_mssqlspatial.h 33807 2016-03-29 19:06:42Z tamas $
  *
  * Project:  MSSQL Spatial driver
  * Purpose:  Definition of classes for OGR MSSQL Spatial driver.
@@ -27,8 +27,8 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _OGR_MSSQLSPATIAL_H_INCLUDED
-#define _OGR_MSSQLSPATIAL_H_INCLUDED
+#ifndef OGR_MSSQLSPATIAL_H_INCLUDED
+#define OGR_MSSQLSPATIAL_H_INCLUDED
 
 #include "ogrsf_frmts.h"
 #include "cpl_odbc.h"
@@ -36,6 +36,12 @@
 
 class OGRMSSQLSpatialDataSource;
 
+/* layer status */
+#define MSSQLLAYERSTATUS_ORIGINAL  0
+#define MSSQLLAYERSTATUS_INITIAL 1
+#define MSSQLLAYERSTATUS_CREATED 2
+#define MSSQLLAYERSTATUS_DISABLED 3
+
 /* geometry format to transfer geometry column */
 #define MSSQLGEOMETRY_NATIVE 0
 #define MSSQLGEOMETRY_WKB 1
@@ -48,6 +54,25 @@ class OGRMSSQLSpatialDataSource;
 #define MSSQLCOLTYPE_BINARY 2
 #define MSSQLCOLTYPE_TEXT 3
 
+/* sqlgeometry constants */
+
+#define SP_NONE 0
+#define SP_HASZVALUES 1
+#define SP_HASMVALUES 2
+#define SP_ISVALID 4
+#define SP_ISSINGLEPOINT 8
+#define SP_ISSINGLELINESEGMENT 0x10
+#define SP_ISWHOLEGLOBE 0x20
+
+#define ST_UNKNOWN 0
+#define ST_POINT 1
+#define ST_LINESTRING 2
+#define ST_POLYGON 3
+#define ST_MULTIPOINT 4
+#define ST_MULTILINESTRING 5
+#define ST_MULTIPOLYGON 6
+#define ST_GEOMETRYCOLLECTION 7
+
 /************************************************************************/
 /*                         OGRMSSQLAppendEscaped( )                     */
 /************************************************************************/
@@ -91,7 +116,7 @@ class OGRMSSQLGeometryParser
 {
 protected:    
     unsigned char* pszData;
-    /* serialization propeties */
+    /* serialization properties */
     char chProps;
     /* point array */
     int nPointSize;
@@ -124,6 +149,52 @@ public:
 };
 
 
+/************************************************************************/
+/*                           OGRMSSQLGeometryWriter                     */
+/************************************************************************/
+
+class OGRMSSQLGeometryWriter
+{
+protected:
+    OGRGeometry *poGeom2;
+    unsigned char* pszData;
+    int nLen;
+    /* serialization propeties */
+    char chProps;
+    /* point array */
+    int nPointSize;
+    int nPointPos;
+    int nNumPoints;
+    int iPoint;
+    /* figure array */
+    int nFigurePos;
+    int nNumFigures;
+    int iFigure;
+    /* shape array */
+    int nShapePos;
+    int nNumShapes;
+    int iShape;
+    int nSRSId;
+    /* geometry or geography */
+    int nColType;
+
+protected:
+    void             WritePoint(OGRPoint* poGeom);
+    void             WritePoint(double x, double y);
+    void             WritePoint(double x, double y, double z);
+    void             WriteLineString(OGRLineString* poGeom);
+    void             WritePolygon(OGRPolygon* poGeom);
+    void             WriteGeometryCollection(OGRGeometryCollection* poGeom, int iParent);
+    void             WriteGeometry(OGRGeometry* poGeom, int iParent);
+    void             TrackGeometry(OGRGeometry* poGeom);
+
+public:
+                     OGRMSSQLGeometryWriter(OGRGeometry *poGeometry, int nGeomColumnType, int nSRS);
+    OGRErr           WriteSqlGeometry(unsigned char* pszBuffer, int nBufLen);
+    int              GetDataLen() { return nLen; }
+};
+
+
 /************************************************************************/
 /*                             OGRMSSQLSpatialLayer                     */
 /************************************************************************/
@@ -132,6 +203,7 @@ class OGRMSSQLSpatialLayer : public OGRLayer
 {
     protected:
     OGRFeatureDefn     *poFeatureDefn;
+    int                 nRawColumns;
 
     CPLODBCStatement   *poStmt;
 
@@ -139,16 +211,20 @@ class OGRMSSQLSpatialLayer : public OGRLayer
     OGRSpatialReference *poSRS;
     int                 nSRSId;
 
-    int                 iNextShapeId;
+    GIntBig             iNextShapeId;
 
     OGRMSSQLSpatialDataSource    *poDS;
 
     int                nGeomColumnType;
     char               *pszGeomColumn;
+    int                nGeomColumnIndex;
     char               *pszFIDColumn;
+    int                nFIDColumnIndex;
 
     int                bIsIdentityFid;
 
+    int                nLayerStatus;
+
     int                *panFieldOrdinals;
 
     CPLErr              BuildFeatureDefn( const char *pszLayerName,
@@ -164,7 +240,7 @@ class OGRMSSQLSpatialLayer : public OGRLayer
     virtual OGRFeature *GetNextRawFeature();
     virtual OGRFeature *GetNextFeature();
 
-    virtual OGRFeature *GetFeature( long nFeatureId );
+    virtual OGRFeature *GetFeature( GIntBig nFeatureId );
     
     virtual OGRFeatureDefn *GetLayerDefn() { return poFeatureDefn; }
 
@@ -179,20 +255,63 @@ class OGRMSSQLSpatialLayer : public OGRLayer
 
     virtual int         TestCapability( const char * );
     char*               GByteArrayToHexString( const GByte* pabyData, int nLen);
+
+    void               SetLayerStatus( int nStatus ) { nLayerStatus = nStatus; }
+    int                GetLayerStatus() { return nLayerStatus; }
 };
 
 /************************************************************************/
 /*                       OGRMSSQLSpatialTableLayer                      */
 /************************************************************************/
 
+typedef union {
+    struct {
+        int     iIndicator;
+        int     Value;
+    } Integer;
+
+    struct {
+        int     iIndicator;
+        GIntBig     Value;
+    } Integer64;
+    
+    struct {
+        int     iIndicator;
+        double  Value;
+    } Float;
+
+    struct {
+        int     nSize;
+        char* pData[8000];
+    } VarChar;
+
+    struct {
+        int     nSize;
+        GByte*  pData;
+    } RawData;
+
+} BCPData;
+
 class OGRMSSQLSpatialTableLayer : public OGRMSSQLSpatialLayer
 {
     int                 bUpdateAccess;
     int                 bLaunderColumnNames;
     int                 bPreservePrecision;
+    int                 bNeedSpatialIndex;
+    int                 bUseCopy;
+    int                 nBCPSize;
 
+    int                 nUploadGeometryFormat;
+    
     char                *pszQuery;
 
+    SQLHANDLE           hEnvBCP;
+    SQLHANDLE           hDBCBCP;
+    int                 nBCPCount;
+    BCPData             **papstBindBuffer;
+
+    int                 bIdentityInsert;
+
     void		ClearStatement();
     CPLODBCStatement* BuildStatement(const char* pszColumns);
 
@@ -206,6 +325,7 @@ class OGRMSSQLSpatialTableLayer : public OGRMSSQLSpatialLayer
 
     OGRwkbGeometryType eGeomType;
 
+
   public:
                         OGRMSSQLSpatialTableLayer( OGRMSSQLSpatialDataSource * );
                         ~OGRMSSQLSpatialTableLayer();
@@ -222,7 +342,7 @@ class OGRMSSQLSpatialTableLayer : public OGRMSSQLSpatialLayer
     void                DropSpatialIndex();
 
     virtual void        ResetReading();
-    virtual int         GetFeatureCount( int );
+    virtual GIntBig     GetFeatureCount( int );
 
     virtual OGRFeatureDefn *GetLayerDefn();
 
@@ -230,9 +350,9 @@ class OGRMSSQLSpatialTableLayer : public OGRMSSQLSpatialLayer
 
     virtual OGRErr      SetAttributeFilter( const char * );
 
-    virtual OGRErr      SetFeature( OGRFeature *poFeature );
-    virtual OGRErr      DeleteFeature( long nFID );
-    virtual OGRErr      CreateFeature( OGRFeature *poFeature );
+    virtual OGRErr      ISetFeature( OGRFeature *poFeature );
+    virtual OGRErr      DeleteFeature( GIntBig nFID );
+    virtual OGRErr      ICreateFeature( OGRFeature *poFeature );
 
     const char*         GetTableName() { return pszTableName; }
     const char*         GetLayerName() { return pszLayerName; }
@@ -241,7 +361,7 @@ class OGRMSSQLSpatialTableLayer : public OGRMSSQLSpatialLayer
     virtual OGRErr      CreateField( OGRFieldDefn *poField,
                                      int bApproxOK = TRUE );
    
-    virtual OGRFeature *GetFeature( long nFeatureId );
+    virtual OGRFeature *GetFeature( GIntBig nFeatureId );
 
     virtual int         TestCapability( const char * );
 
@@ -249,10 +369,23 @@ class OGRMSSQLSpatialTableLayer : public OGRMSSQLSpatialLayer
                                 { bLaunderColumnNames = bFlag; }
     void                SetPrecisionFlag( int bFlag )
                                 { bPreservePrecision = bFlag; }
+    void                SetSpatialIndexFlag( int bFlag )
+                                { bNeedSpatialIndex = bFlag; }
+    void                SetUploadGeometryFormat( int nGeometryFormat )
+                                { nUploadGeometryFormat = nGeometryFormat; }
     void                AppendFieldValue(CPLODBCStatement *poStatement,
-                                       OGRFeature* poFeature, int i);
+                                       OGRFeature* poFeature, int i, int *bind_num, void **bind_buffer);
 
     int                 FetchSRSId();
+
+    void                SetUseCopy(int bcpSize) { bUseCopy = TRUE; nBCPSize = bcpSize; }
+    int                 Failed( int nRetCode );
+#ifdef MSSQL_BCP_SUPPORTED
+    OGRErr              CreateFeatureBCP( OGRFeature *poFeature );
+    int                 Failed2( int nRetCode );
+    int                 InitBCP( const char* pszDSN );
+    void                CloseBCP();
+#endif
 };
 
 /************************************************************************/
@@ -274,11 +407,13 @@ class OGRMSSQLSpatialSelectLayer : public OGRMSSQLSpatialLayer
                         ~OGRMSSQLSpatialSelectLayer();
 
     virtual void        ResetReading();
-    virtual int         GetFeatureCount( int );
+    virtual GIntBig     GetFeatureCount( int );
 
-    virtual OGRFeature *GetFeature( long nFeatureId );
+    virtual OGRFeature *GetFeature( GIntBig nFeatureId );
     
     virtual OGRErr      GetExtent(OGREnvelope *psExtent, int bForce = TRUE);
+    virtual OGRErr      GetExtent(int iGeomField, OGREnvelope *psExtent, int bForce)
+                { return OGRLayer::GetExtent(iGeomField, psExtent, bForce); }
 
     virtual int         TestCapability( const char * );
 };
@@ -305,11 +440,16 @@ class OGRMSSQLSpatialDataSource : public OGRDataSource
 
     int                 bListAllTables;
 
+    int                 nBCPSize;
+    int                 bUseCopy;
+
     // We maintain a list of known SRID to reduce the number of trips to
     // the database to get SRSes. 
     int                 nKnownSRID;
     int                *panSRID;
     OGRSpatialReference **papoSRS;
+
+    char                *pszConnection;
     
   public:
                         OGRMSSQLSpatialDataSource();
@@ -334,8 +474,8 @@ class OGRMSSQLSpatialDataSource : public OGRDataSource
     int                 GetGeometryFormat() { return nGeometryFormat; }
     int                 UseGeometryColumns() { return bUseGeometryColumns; }
 
-    virtual int         DeleteLayer( int iLayer );
-    virtual OGRLayer    *CreateLayer( const char *,
+    virtual OGRErr       DeleteLayer( int iLayer );
+    virtual OGRLayer    *ICreateLayer( const char *,
                                       OGRSpatialReference * = NULL,
                                       OGRwkbGeometryType = wkbUnknown,
                                       char ** = NULL );
@@ -353,9 +493,13 @@ class OGRMSSQLSpatialDataSource : public OGRDataSource
     OGRSpatialReference* FetchSRS( int nId );
     int                 FetchSRSId( OGRSpatialReference * poSRS );
 
+    OGRErr              StartTransaction(CPL_UNUSED int bForce);
+    OGRErr              CommitTransaction();
+    OGRErr              RollbackTransaction();
+
     // Internal use
     CPLODBCSession     *GetSession() { return &oSession; }
-
+    const char         *GetConnectionString() { return pszConnection; }
 };
 
 /************************************************************************/
@@ -376,4 +520,4 @@ class OGRMSSQLSpatialDriver : public OGRSFDriver
     int                 TestCapability( const char * );
 };
 
-#endif /* ndef _OGR_MSSQLSPATIAL_H_INCLUDED */
+#endif /* ndef OGR_MSSQLSPATIAL_H_INCLUDED */
