@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrmysqlresultlayer.cpp 27741 2014-09-26 19:20:02Z goatbar $
+ * $Id: ogrmysqlresultlayer.cpp 33713 2016-03-12 17:41:57Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRMySQLResultLayer class.
@@ -32,13 +32,13 @@
 #include "cpl_conv.h"
 #include "ogr_mysql.h"
 
-CPL_CVSID("$Id: ogrmysqlresultlayer.cpp 27741 2014-09-26 19:20:02Z goatbar $");
+CPL_CVSID("$Id: ogrmysqlresultlayer.cpp 33713 2016-03-12 17:41:57Z goatbar $");
 
 /************************************************************************/
 /*                        OGRMySQLResultLayer()                         */
 /************************************************************************/
 
-OGRMySQLResultLayer::OGRMySQLResultLayer( OGRMySQLDataSource *poDSIn, 
+OGRMySQLResultLayer::OGRMySQLResultLayer( OGRMySQLDataSource *poDSIn,
                                           const char * pszRawQueryIn,
                                           MYSQL_RES *hResultSetIn )
 {
@@ -79,6 +79,7 @@ OGRFeatureDefn *OGRMySQLResultLayer::ReadResultDefinition()
 /*      Parse the returned table information.                           */
 /* -------------------------------------------------------------------- */
     OGRFeatureDefn *poDefn = new OGRFeatureDefn( "sql_statement" );
+    SetDescription( poDefn->GetName() );
     int            iRawField;
 
     poDefn->Reference();
@@ -86,8 +87,8 @@ OGRFeatureDefn *OGRMySQLResultLayer::ReadResultDefinition()
     int precision;
 
     mysql_field_seek( hResultSet, 0 );
-    for( iRawField = 0; 
-         iRawField < (int) mysql_num_fields(hResultSet); 
+    for( iRawField = 0;
+         iRawField < (int) mysql_num_fields(hResultSet);
          iRawField++ )
     {
         MYSQL_FIELD *psMSField = mysql_fetch_field( hResultSet );
@@ -111,15 +112,15 @@ OGRFeatureDefn *OGRMySQLResultLayer::ReadResultDefinition()
           case FIELD_TYPE_NEWDECIMAL:
 #endif
             oField.SetType( OFTReal );
-            
-            // a bunch of hackery to munge the widths that MySQL gives 
+
+            // a bunch of hackery to munge the widths that MySQL gives
             // us into corresponding widths and precisions for OGR
             precision =    (int)psMSField->decimals;
             width = (int)psMSField->length;
             if (!precision)
                 width = width - 1;
             width = width - precision;
-            
+
             oField.SetWidth(width);
             oField.SetPrecision(precision);
             poDefn->AddFieldDefn( &oField );
@@ -182,7 +183,7 @@ OGRFeatureDefn *OGRMySQLResultLayer::ReadResultDefinition()
             oField.SetWidth((int)psMSField->max_length);
             poDefn->AddFieldDefn( &oField );
             break;
-                        
+
           case FIELD_TYPE_GEOMETRY:
             if (pszGeomColumn == NULL)
             {
@@ -190,24 +191,24 @@ OGRFeatureDefn *OGRMySQLResultLayer::ReadResultDefinition()
                 pszGeomColumn = CPLStrdup( psMSField->name);
             }
             break;
-            
+
           default:
-            // any other field we ignore. 
+            // any other field we ignore.
             break;
         }
-        
+
         // assume a FID name first, and if it isn't there
-        // take a field that is not null, a primary key, 
+        // take a field that is not null, a primary key,
         // and is an integer-like field
         if( EQUAL(psMSField->name,"ogc_fid") )
         {
             bHasFid = TRUE;
             pszFIDColumn = CPLStrdup(oField.GetNameRef());
             continue;
-        } else  
+        } else
         if (IS_NOT_NULL(psMSField->flags)
             && IS_PRI_KEY(psMSField->flags)
-            && 
+            &&
                 (
                     psMSField->type == FIELD_TYPE_TINY
                     || psMSField->type == FIELD_TYPE_SHORT
@@ -226,15 +227,15 @@ OGRFeatureDefn *OGRMySQLResultLayer::ReadResultDefinition()
 
     poDefn->SetGeomType( wkbNone );
 
-    if (pszGeomColumn) 
+    if (pszGeomColumn)
     {
         char*        pszType=NULL;
         CPLString    osCommand;
-        char           **papszRow;  
-         
+        char           **papszRow;
+
         // set to unknown first
         poDefn->SetGeomType( wkbUnknown );
-        
+
         osCommand.Printf(
                 "SELECT type FROM geometry_columns WHERE f_table_name='%s'",
                 pszGeomColumnTable );
@@ -255,14 +256,13 @@ OGRFeatureDefn *OGRMySQLResultLayer::ReadResultDefinition()
         {
             pszType = papszRow[0];
 
-            OGRwkbGeometryType nGeomType = OGRFromOGCGeomType(pszType);
+            OGRwkbGeometryType l_nGeomType = OGRFromOGCGeomType(pszType);
 
-            poDefn->SetGeomType( nGeomType );
-
-        } 
+            poDefn->SetGeomType( l_nGeomType );
+        }
 
 		nSRSId = FetchSRSId();
-    } 
+    }
 
 
     return poDefn;
@@ -298,7 +298,7 @@ void OGRMySQLResultLayer::ResetReading()
 /*                          GetFeatureCount()                           */
 /************************************************************************/
 
-int OGRMySQLResultLayer::GetFeatureCount( int bForce )
+GIntBig OGRMySQLResultLayer::GetFeatureCount( int bForce )
 
 {
     // I wonder if we could do anything smart here...
