@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ntf_generic.cpp 10645 2007-01-18 02:22:39Z warmerdam $
+ * $Id: ntf_generic.cpp 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  NTF Translator
  * Purpose:  Handle NTF products that aren't recognised generically.
@@ -31,7 +31,7 @@
 #include "ntf.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ntf_generic.cpp 10645 2007-01-18 02:22:39Z warmerdam $");
+CPL_CVSID("$Id: ntf_generic.cpp 33714 2016-03-13 05:42:13Z goatbar $");
 
 #define MAX_LINK        5000
 
@@ -96,7 +96,7 @@ void NTFGenericClass::CheckAddAttr( const char * pszName,
         pszName = "FEAT_CODE";
 
     iAttrOffset = CSLFindString( papszAttrNames, pszName );
-    
+
     if( iAttrOffset == -1 )
     {
         nAttrCount++;
@@ -169,7 +169,7 @@ void OGRNTFDataSource::WorkupGeneric( NTFFileReader * poReader )
 /* ==================================================================== */
 /*      Read all record groups in the file.                             */
 /* ==================================================================== */
-    while( TRUE )
+    while( true )
     {
 /* -------------------------------------------------------------------- */
 /*      Read a record group                                             */
@@ -181,7 +181,7 @@ void OGRNTFDataSource::WorkupGeneric( NTFFileReader * poReader )
 
         if( papoGroup == NULL || papoGroup[0]->GetType() == 99 )
             break;
-        
+
 /* -------------------------------------------------------------------- */
 /*      Get the class corresponding to the anchor record.               */
 /* -------------------------------------------------------------------- */
@@ -189,7 +189,7 @@ void OGRNTFDataSource::WorkupGeneric( NTFFileReader * poReader )
         char           **papszFullAttList = NULL;
 
         poClass->nFeatureCount++;
-        
+
 /* -------------------------------------------------------------------- */
 /*      Loop over constituent records collecting attributes.            */
 /* -------------------------------------------------------------------- */
@@ -215,15 +215,15 @@ void OGRNTFDataSource::WorkupGeneric( NTFFileReader * poReader )
                       {
                           poClass->CheckAddAttr( poAttDesc->val_type,
                                                  poAttDesc->finter,
-                                                 strlen(papszValues[iAtt]) );
+                                                 static_cast<int>(strlen(papszValues[iAtt])) );
                       }
 
-                      if( CSLFindString( papszFullAttList, 
+                      if( CSLFindString( papszFullAttList,
                                          papszTypes[iAtt] ) == -1 )
-                          papszFullAttList = 
-                              CSLAddString( papszFullAttList, 
+                          papszFullAttList =
+                              CSLAddString( papszFullAttList,
                                             papszTypes[iAtt] );
-                      else
+                      else if( poAttDesc != NULL )
                           poClass->SetMultiple( poAttDesc->val_type );
                   }
 
@@ -260,7 +260,7 @@ void OGRNTFDataSource::WorkupGeneric( NTFFileReader * poReader )
                 if( poReader->GetNTFLevel() < 3 )
                 {
                     NTFAttDesc  *poAttDesc;
-                      
+
                     poAttDesc = poReader->GetAttDesc(poRecord->GetField(9,10));
                     if( poAttDesc != NULL )
                         poClass->CheckAddAttr( poAttDesc->val_type,
@@ -270,7 +270,7 @@ void OGRNTFDataSource::WorkupGeneric( NTFFileReader * poReader )
                         poClass->CheckAddAttr( "FEAT_CODE", "A4", 4 );
                 }
                 break;
-                
+
               default:
                 break;
             }
@@ -303,7 +303,7 @@ static void AddGenericAttributes( NTFFileReader * poReader,
     for( int iAtt = 0; papszTypes != NULL && papszTypes[iAtt] != NULL; iAtt++ )
     {
         int             iField;
-        
+
         if( EQUAL(papszTypes[iAtt],"TX") )
             iField = poFeature->GetFieldIndex("TEXT");
         else if( EQUAL(papszTypes[iAtt],"FC") )
@@ -324,7 +324,7 @@ static void AddGenericAttributes( NTFFileReader * poReader,
         char  szListName[128];
         int   iListField;
 
-        sprintf( szListName, "%s_LIST", 
+        snprintf( szListName, sizeof(szListName), "%s_LIST",
                  poFeature->GetFieldDefnRef(iField)->GetNameRef() );
         iListField = poFeature->GetFieldIndex( szListName );
 
@@ -335,16 +335,16 @@ static void AddGenericAttributes( NTFFileReader * poReader,
         if( iListField != -1 )
         {
             char        *pszAttLongName, *pszAttValue, *pszCodeDesc;
-            
+
             poReader->ProcessAttValue( papszTypes[iAtt], papszValues[iAtt],
-                                       &pszAttLongName, &pszAttValue, 
+                                       &pszAttLongName, &pszAttValue,
                                        &pszCodeDesc );
 
             if( poFeature->IsFieldSet( iListField ) )
             {
-                poFeature->SetField( iListField, 
-                    CPLSPrintf( "%s,%s", 
-                                poFeature->GetFieldAsString( iListField ), 
+                poFeature->SetField( iListField,
+                    CPLSPrintf( "%s,%s",
+                                poFeature->GetFieldAsString( iListField ),
                                 pszAttValue ) );
             }
             else
@@ -374,7 +374,7 @@ static OGRFeature *TranslateGenericNode( NTFFileReader *poReader,
     {
         return NULL;
     }
-        
+
     OGRFeature  *poFeature = new OGRFeature( poLayer->GetLayerDefn() );
 
     // NODE_ID
@@ -427,10 +427,10 @@ static OGRFeature *TranslateGenericCollection( NTFFileReader *poReader,
                                                NTFRecord **papoGroup )
 
 {
-    if( CSLCount((char **) papoGroup) < 1 
+    if( CSLCount((char **) papoGroup) < 1
         || papoGroup[0]->GetType() != NRT_COLLECT )
         return NULL;
-        
+
     OGRFeature  *poFeature = new OGRFeature( poLayer->GetLayerDefn() );
 
     // COLL_ID
@@ -481,11 +481,11 @@ static OGRFeature *TranslateGenericText( NTFFileReader *poReader,
 
 {
     int         iRec;
-    
+
     if( CSLCount((char **) papoGroup) < 2
         || papoGroup[0]->GetType() != NRT_TEXTREC )
         return NULL;
-        
+
     OGRFeature  *poFeature = new OGRFeature( poLayer->GetLayerDefn() );
 
     // TEXT_ID
@@ -511,7 +511,7 @@ static OGRFeature *TranslateGenericText( NTFFileReader *poReader,
     for( iRec = 0; papoGroup[iRec] != NULL; iRec++ )
     {
         NTFRecord       *poRecord = papoGroup[iRec];
-        
+
         if( poRecord->GetType() == NRT_TEXTREP )
         {
             poFeature->SetField( "FONT", atoi(poRecord->GetField(9,12)) );
@@ -541,11 +541,11 @@ static OGRFeature *TranslateGenericName( NTFFileReader *poReader,
 
 {
     int         iRec;
-    
+
     if( CSLCount((char **) papoGroup) < 2
         || papoGroup[0]->GetType() != NRT_NAMEREC )
         return NULL;
-        
+
     OGRFeature  *poFeature = new OGRFeature( poLayer->GetLayerDefn() );
 
     // NAME_ID
@@ -579,7 +579,7 @@ static OGRFeature *TranslateGenericName( NTFFileReader *poReader,
     for( iRec = 0; papoGroup[iRec] != NULL; iRec++ )
     {
         NTFRecord       *poRecord = papoGroup[iRec];
-        
+
         if( poRecord->GetType() == NRT_NAMEPOSTN )
         {
             poFeature->SetField( "FONT", atoi(poRecord->GetField(3,6)) );
@@ -615,7 +615,7 @@ static OGRFeature *TranslateGenericPoint( NTFFileReader *poReader,
     {
         return NULL;
     }
-        
+
     OGRFeature  *poFeature = new OGRFeature( poLayer->GetLayerDefn() );
 
     // POINT_ID
@@ -633,7 +633,7 @@ static OGRFeature *TranslateGenericPoint( NTFFileReader *poReader,
     {
         char    szValType[3];
 
-        strcpy( szValType, papoGroup[0]->GetField(9,10) );
+        snprintf( szValType, sizeof(szValType), "%s", papoGroup[0]->GetField(9,10) );
         if( !EQUAL(szValType,"  ") )
         {
             char        *pszProcessedValue;
@@ -667,7 +667,7 @@ static OGRFeature *TranslateGenericLine( NTFFileReader *poReader,
         || (papoGroup[1]->GetType() != NRT_GEOMETRY
             && papoGroup[1]->GetType() != NRT_GEOMETRY3D) )
         return NULL;
-        
+
     OGRFeature  *poFeature = new OGRFeature( poLayer->GetLayerDefn() );
 
     // LINE_ID
@@ -685,7 +685,7 @@ static OGRFeature *TranslateGenericLine( NTFFileReader *poReader,
     {
         char    szValType[3];
 
-        strcpy( szValType, papoGroup[0]->GetField(9,10) );
+        snprintf( szValType, sizeof(szValType), "%s", papoGroup[0]->GetField(9,10) );
         if( !EQUAL(szValType,"  ") )
         {
             char        *pszProcessedValue;
@@ -717,7 +717,7 @@ static OGRFeature *TranslateGenericPoly( NTFFileReader *poReader,
 /* ==================================================================== */
 /*      Traditional POLYGON record groups.                              */
 /* ==================================================================== */
-    if( CSLCount((char **) papoGroup) >= 2 
+    if( CSLCount((char **) papoGroup) >= 2
         && papoGroup[0]->GetType() == NRT_POLYGON
         && papoGroup[1]->GetType() == NRT_CHAIN )
     {
@@ -728,14 +728,14 @@ static OGRFeature *TranslateGenericPoly( NTFFileReader *poReader,
 
         // NUM_PARTS
         int             nNumLinks = atoi(papoGroup[1]->GetField( 9, 12 ));
-    
+
         if( nNumLinks > MAX_LINK )
         {
-            CPLError( CE_Failure, CPLE_AppDefined, 
+            CPLError( CE_Failure, CPLE_AppDefined,
                       "MAX_LINK exceeded in ntf_generic.cpp." );
             return poFeature;
         }
-    
+
         poFeature->SetField( "NUM_PARTS", nNumLinks );
 
         // DIR
@@ -789,13 +789,13 @@ static OGRFeature *TranslateGenericCPoly( NTFFileReader *poReader,
 /* -------------------------------------------------------------------- */
     if( papoGroup[0]->GetType() != NRT_CPOLY )
         return NULL;
-    
-    if( papoGroup[1] == NULL || 
-        (papoGroup[1]->GetType() != NRT_GEOMETRY 
-         && papoGroup[1]->GetType() != NRT_GEOMETRY3D) ) 
+
+    if( papoGroup[1] == NULL ||
+        (papoGroup[1]->GetType() != NRT_GEOMETRY
+         && papoGroup[1]->GetType() != NRT_GEOMETRY3D) )
         return NULL;
-    
-    if( papoGroup[1] != NULL 
+
+    if( papoGroup[1] != NULL
         && papoGroup[2]->GetType() != NRT_ATTREC )
         return NULL;
 
@@ -806,25 +806,25 @@ static OGRFeature *TranslateGenericCPoly( NTFFileReader *poReader,
 
     // CPOLY_ID
     poFeature->SetField( "CPOLY_ID", atoi(papoGroup[0]->GetField( 3, 8 )) );
-    
+
     // ATTREC Attributes
     AddGenericAttributes( poReader, papoGroup, poFeature );
-    
+
     // Read point geometry
-    if( papoGroup[1] != NULL 
+    if( papoGroup[1] != NULL
         && (papoGroup[1]->GetType() == NRT_GEOMETRY
             || papoGroup[1]->GetType() == NRT_GEOMETRY3D) )
     {
         poFeature->SetGeometryDirectly(
             poReader->ProcessGeometry(papoGroup[1]));
-        poFeature->SetField( "GEOM_ID", 
+        poFeature->SetField( "GEOM_ID",
                              atoi(papoGroup[1]->GetField(3,8)) );
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Collect the chains for each of the rings, and just aggregate    */
 /*      these into the master list without any concept of where the     */
-/*      boundaries are.  The boundary information will be emmitted      */
+/*      boundaries are.  The boundary information will be emitted      */
 /*      in the RingStart field.                                         */
 /* -------------------------------------------------------------------- */
     int         nNumLink = 0, iLink;
@@ -854,15 +854,15 @@ void OGRNTFDataSource::EstablishGenericLayers()
 
 {
     int         iType;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Pick an initial NTFFileReader to build the layers against.      */
 /* -------------------------------------------------------------------- */
     for( int iFile = 0; iFile < nNTFFileCount; iFile++ )
     {
         NTFFileReader   *poPReader = NULL;
-        int             n3DFlag = 0;
-        
+        int              bHasZ = FALSE;
+
         poPReader = papoNTFFileReader[iFile];
         if( poPReader->GetProductId() != NPC_UNKNOWN )
             continue;
@@ -874,26 +874,26 @@ void OGRNTFDataSource::EstablishGenericLayers()
         for( iType = 0; iType < 99; iType++ )
         {
             NTFGenericClass     *poClass = aoGenericClass + iType;
-        
+
             if( poClass->nFeatureCount > 0 && poClass->b3D )
-                n3DFlag = wkb25DBit;
+                bHasZ = TRUE;
         }
-        
+
 /* -------------------------------------------------------------------- */
 /*      Create layers for all recognised layer types with features.     */
 /* -------------------------------------------------------------------- */
         for( iType = 0; iType < 99; iType++ )
         {
             NTFGenericClass     *poClass = aoGenericClass + iType;
-        
+
             if( poClass->nFeatureCount == 0 )
                 continue;
 
             if( iType == NRT_POINTREC )
             {
                 poPReader->
-                    EstablishLayer( "GENERIC_POINT", 
-                                    (OGRwkbGeometryType) (wkbPoint | n3DFlag),
+                    EstablishLayer( "GENERIC_POINT",
+                                    OGR_GT_SetModifier(wkbPoint, bHasZ, FALSE),
                                     TranslateGenericPoint,
                                     NRT_POINTREC, poClass,
                                     "POINT_ID", OFTInteger, 6, 0,
@@ -902,9 +902,8 @@ void OGRNTFDataSource::EstablishGenericLayers()
             else if( iType == NRT_LINEREC )
             {
                 poPReader->
-                    EstablishLayer( "GENERIC_LINE", 
-                                    (OGRwkbGeometryType) 
-                                    (wkbLineString | n3DFlag),
+                    EstablishLayer( "GENERIC_LINE",
+                                    OGR_GT_SetModifier(wkbLineString, bHasZ, FALSE),
                                     TranslateGenericLine,
                                     NRT_LINEREC, poClass,
                                     "LINE_ID", OFTInteger, 6, 0,
@@ -913,9 +912,8 @@ void OGRNTFDataSource::EstablishGenericLayers()
             else if( iType == NRT_TEXTREC )
             {
                 poPReader->
-                    EstablishLayer( "GENERIC_TEXT", 
-                                    (OGRwkbGeometryType) 
-                                    (wkbPoint | n3DFlag),
+                    EstablishLayer( "GENERIC_TEXT",
+                                    OGR_GT_SetModifier(wkbPoint, bHasZ, FALSE),
                                     TranslateGenericText,
                                     NRT_TEXTREC, poClass,
                                     "TEXT_ID", OFTInteger, 6, 0,
@@ -924,9 +922,8 @@ void OGRNTFDataSource::EstablishGenericLayers()
             else if( iType == NRT_NAMEREC )
             {
                 poPReader->
-                    EstablishLayer( "GENERIC_NAME", 
-                                    (OGRwkbGeometryType) 
-                                    (wkbPoint | n3DFlag),
+                    EstablishLayer( "GENERIC_NAME",
+                                    OGR_GT_SetModifier(wkbPoint, bHasZ, FALSE),
                                     TranslateGenericName,
                                     NRT_NAMEREC, poClass,
                                     "NAME_ID", OFTInteger, 6, 0,
@@ -936,8 +933,7 @@ void OGRNTFDataSource::EstablishGenericLayers()
             {
                 poPReader->
                     EstablishLayer( "GENERIC_NODE",
-                                    (OGRwkbGeometryType) 
-                                    (wkbPoint | n3DFlag),
+                                    OGR_GT_SetModifier(wkbPoint, bHasZ, FALSE),
                                     TranslateGenericNode,
                                     NRT_NODEREC, poClass,
                                     "NODE_ID", OFTInteger, 6, 0,
@@ -961,12 +957,12 @@ void OGRNTFDataSource::EstablishGenericLayers()
             else if( iType == NRT_POLYGON )
             {
                 poPReader->
-                    EstablishLayer( "GENERIC_POLY", 
-                                    (OGRwkbGeometryType) (wkbPoint | n3DFlag),
+                    EstablishLayer( "GENERIC_POLY",
+                                    OGR_GT_SetModifier(wkbPoint, bHasZ, FALSE),
                                     TranslateGenericPoly,
                                     NRT_POLYGON, poClass,
                                     "POLY_ID", OFTInteger, 6, 0,
-                                    "NUM_PARTS", OFTInteger, 4, 0, 
+                                    "NUM_PARTS", OFTInteger, 4, 0,
                                     "DIR", OFTIntegerList, 1, 0,
                                     "GEOM_ID_OF_LINK", OFTIntegerList, 6, 0,
                                     "RingStart", OFTIntegerList, 6, 0,
@@ -975,16 +971,15 @@ void OGRNTFDataSource::EstablishGenericLayers()
             else if( iType == NRT_CPOLY )
             {
                 poPReader->
-                    EstablishLayer( "GENERIC_CPOLY", 
-                                    (OGRwkbGeometryType) (wkbPoint | n3DFlag),
+                    EstablishLayer( "GENERIC_CPOLY",
+                                    OGR_GT_SetModifier(wkbPoint, bHasZ, FALSE),
                                     TranslateGenericCPoly,
                                     NRT_CPOLY, poClass,
                                     "CPOLY_ID", OFTInteger, 6, 0,
-                                    "NUM_PARTS", OFTInteger, 4, 0, 
+                                    "NUM_PARTS", OFTInteger, 4, 0,
                                     "POLY_ID", OFTIntegerList, 1, 0,
                                     NULL );
             }
         }
     }
 }
-
