@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ntf_raster.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: ntf_raster.cpp 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  NTF Translator
  * Purpose:  Handle UK Ordnance Survey Raster DTM products.  Includes some
@@ -31,7 +31,7 @@
 
 #include "ntf.h"
 
-CPL_CVSID("$Id: ntf_raster.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: ntf_raster.cpp 33714 2016-03-13 05:42:13Z goatbar $");
 
 /************************************************************************/
 /* ==================================================================== */
@@ -69,7 +69,8 @@ void NTFFileReader::EstablishRasterAccess()
         delete poRecord;
     }
 
-    if( poRecord->GetType() != NRT_GRIDHREC )
+    if( poRecord == NULL ||
+        poRecord->GetType() != NRT_GRIDHREC )
     {
         delete poRecord;
         CPLError( CE_Failure, CPLE_AppDefined,
@@ -94,7 +95,7 @@ void NTFFileReader::EstablishRasterAccess()
         adfGeoTransform[3] = atoi(poRecord->GetField(35,44));
         adfGeoTransform[4] = 0;
         adfGeoTransform[5] = 50;
-        
+
         nRasterDataType = 3; /* GDT_Int16 */
     }
 
@@ -116,7 +117,7 @@ void NTFFileReader::EstablishRasterAccess()
                            + GetYOrigin();
         adfGeoTransform[4] = 0;
         adfGeoTransform[5] = atoi(poRecord->GetField(43,46));
-        
+
         nRasterDataType = 3; /* GDT_Int16 */
     }
 
@@ -153,13 +154,13 @@ CPLErr NTFFileReader::ReadRasterColumn( int iColumn, float *pafElev )
     if( panColumnOffset[iColumn] == 0 )
     {
         int     iPrev;
-        
+
         for( iPrev = 0; iPrev < iColumn-1; iPrev++ )
         {
             if( panColumnOffset[iPrev+1] == 0 )
             {
                 CPLErr  eErr;
-                
+
                 eErr = ReadRasterColumn( iPrev, NULL );
                 if( eErr != CE_None )
                     return eErr;
@@ -172,12 +173,12 @@ CPLErr NTFFileReader::ReadRasterColumn( int iColumn, float *pafElev )
 /* -------------------------------------------------------------------- */
     if( GetFP() == NULL )
         Open();
-    
+
 /* -------------------------------------------------------------------- */
 /*      Read requested record.                                          */
 /* -------------------------------------------------------------------- */
     NTFRecord   *poRecord;
-    
+
     SetFPPos( panColumnOffset[iColumn], iColumn );
     poRecord = ReadRecord();
 
@@ -185,7 +186,7 @@ CPLErr NTFFileReader::ReadRasterColumn( int iColumn, float *pafElev )
     {
         GetFPPos( panColumnOffset+iColumn+1, NULL );
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Handle LANDRANGER DTM columns.                                  */
 /* -------------------------------------------------------------------- */
@@ -202,7 +203,7 @@ CPLErr NTFFileReader::ReadRasterColumn( int iColumn, float *pafElev )
                 atoi(poRecord->GetField(84+iPixel*4,87+iPixel*4)));
         }
     }
-            
+
 /* -------------------------------------------------------------------- */
 /*      Handle PROFILE                                                  */
 /* -------------------------------------------------------------------- */
@@ -210,11 +211,11 @@ CPLErr NTFFileReader::ReadRasterColumn( int iColumn, float *pafElev )
     {
         for( int iPixel = 0; iPixel < nRasterXSize; iPixel++ )
         {
-            pafElev[iPixel] = (float) 
+            pafElev[iPixel] = (float)
            (atoi(poRecord->GetField(19+iPixel*5,23+iPixel*5)) * GetZMult());
         }
     }
-    
+
     delete poRecord;
 
     return CE_None;
@@ -236,7 +237,7 @@ OGRNTFRasterLayer::OGRNTFRasterLayer( OGRNTFDataSource *poDSIn,
 {
     char        szLayerName[128];
 
-    sprintf( szLayerName, "DTM_%s", poReaderIn->GetTileName() );
+    snprintf( szLayerName, sizeof(szLayerName), "DTM_%s", poReaderIn->GetTileName() );
     poFeatureDefn = new OGRFeatureDefn( szLayerName );
     poFeatureDefn->Reference();
     poFeatureDefn->SetGeomType( wkbPoint25D );
@@ -262,7 +263,7 @@ OGRNTFRasterLayer::OGRNTFRasterLayer( OGRNTFDataSource *poDSIn,
         nDEMSample = 1;
     else
         nDEMSample = MAX(1,atoi(poDS->GetOption("DEM_SAMPLE")));
-    
+
     nFeatureCount = (poReader->GetRasterXSize() / nDEMSample)
                   * (poReader->GetRasterYSize() / nDEMSample);
 }
@@ -321,7 +322,7 @@ OGRFeature *OGRNTFRasterLayer::GetNextFeature()
     else
     {
         int     iReqColumn, iReqRow;
-        
+
         iReqColumn = (iCurrentFC - 1) / poReader->GetRasterYSize();
         iReqRow = iCurrentFC - iReqColumn * poReader->GetRasterXSize() - 1;
 
@@ -346,16 +347,16 @@ OGRFeature *OGRNTFRasterLayer::GetNextFeature()
 /*                             GetFeature()                             */
 /************************************************************************/
 
-OGRFeature *OGRNTFRasterLayer::GetFeature( long nFeatureId )
+OGRFeature *OGRNTFRasterLayer::GetFeature( GIntBig nFeatureId )
 
 {
     int         iReqColumn, iReqRow;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Is this in the range of legal feature ids (pixels)?             */
 /* -------------------------------------------------------------------- */
     if( nFeatureId < 1
-        || nFeatureId > poReader->GetRasterXSize()*poReader->GetRasterYSize() )
+        || nFeatureId > static_cast<GIntBig>(poReader->GetRasterXSize())*poReader->GetRasterYSize() )
     {
         return NULL;
     }
@@ -363,16 +364,16 @@ OGRFeature *OGRNTFRasterLayer::GetFeature( long nFeatureId )
 /* -------------------------------------------------------------------- */
 /*      Do we need to load a different column.                          */
 /* -------------------------------------------------------------------- */
-    iReqColumn = (nFeatureId - 1) / poReader->GetRasterYSize();
-    iReqRow = nFeatureId - iReqColumn * poReader->GetRasterXSize() - 1;
-    
+    iReqColumn = ((int)nFeatureId - 1) / poReader->GetRasterYSize();
+    iReqRow = (int)nFeatureId - iReqColumn * poReader->GetRasterXSize() - 1;
+
     if( iReqColumn != iColumnOffset )
     {
         iColumnOffset = iReqColumn;
         if( poReader->ReadRasterColumn( iReqColumn, pafColumn ) != CE_None )
             return NULL;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding feature.                                 */
 /* -------------------------------------------------------------------- */
@@ -388,7 +389,7 @@ OGRFeature *OGRNTFRasterLayer::GetFeature( long nFeatureId )
                       padfGeoTransform[3] + padfGeoTransform[5] * iReqRow,
                       pafColumn[iReqRow] ) );
     poFeature->SetField( 0, pafColumn[iReqRow] );
-    
+
     return poFeature;
 }
 
@@ -401,7 +402,7 @@ OGRFeature *OGRNTFRasterLayer::GetFeature( long nFeatureId )
 /*      way of counting features matching a spatial query.              */
 /************************************************************************/
 
-int OGRNTFRasterLayer::GetFeatureCount( CPL_UNUSED int bForce )
+GIntBig OGRNTFRasterLayer::GetFeatureCount( CPL_UNUSED int bForce )
 {
     return nFeatureCount;
 }
@@ -416,7 +417,7 @@ int OGRNTFRasterLayer::TestCapability( const char * pszCap )
     if( EQUAL(pszCap,OLCRandomRead) )
         return TRUE;
 
-    else if( EQUAL(pszCap,OLCSequentialWrite) 
+    else if( EQUAL(pszCap,OLCSequentialWrite)
              || EQUAL(pszCap,OLCRandomWrite) )
         return FALSE;
 
@@ -426,8 +427,6 @@ int OGRNTFRasterLayer::TestCapability( const char * pszCap )
     else if( EQUAL(pszCap,OLCFastSpatialFilter) )
         return FALSE;
 
-    else 
+    else
         return FALSE;
 }
-
-
