@@ -1,9 +1,9 @@
 /******************************************************************************
- * $Id: ogrocistatement.cpp 14387 2008-05-07 16:14:03Z tamas $
+ * $Id: ogrocistatement.cpp 33713 2016-03-12 17:41:57Z goatbar $
  *
  * Project:  Oracle Spatial Driver
- * Purpose:  Implementation of OGROCIStatement, which encapsulates the 
- *           preparation, executation and fetching from an SQL statement.
+ * Purpose:  Implementation of OGROCIStatement, which encapsulates the
+ *           preparation, execution and fetching from an SQL statement.
  * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
  ******************************************************************************
@@ -31,7 +31,7 @@
 #include "ogr_oci.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrocistatement.cpp 14387 2008-05-07 16:14:03Z tamas $");
+CPL_CVSID("$Id: ogrocistatement.cpp 33713 2016-03-12 17:41:57Z goatbar $");
 
 /************************************************************************/
 /*                          OGROCIStatement()                           */
@@ -51,6 +51,7 @@ OGROCIStatement::OGROCIStatement( OGROCISession *poSessionIn )
     panFieldMap = NULL;
 
     pszCommandText = NULL;
+    nAffectedRows = 0;
 }
 
 /************************************************************************/
@@ -85,7 +86,7 @@ void OGROCIStatement::Clean()
 
     CPLFree( papszCurImage );
     papszCurImage = NULL;
-    
+
     CPLFree( panCurColumnInd );
     panCurColumnInd = NULL;
 
@@ -120,7 +121,7 @@ CPLErr OGROCIStatement::Prepare( const char *pszSQLStatement )
 
     if( hStatement != NULL )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "Statement already executed once on this OGROCIStatement." );
         return CE_Failure;
     }
@@ -128,9 +129,9 @@ CPLErr OGROCIStatement::Prepare( const char *pszSQLStatement )
 /* -------------------------------------------------------------------- */
 /*      Allocate a statement handle.                                    */
 /* -------------------------------------------------------------------- */
-    if( poSession->Failed( 
-        OCIHandleAlloc( poSession->hEnv, (dvoid **) &hStatement, 
-                        (ub4)OCI_HTYPE_STMT,(size_t)0, (dvoid **)0 ), 
+    if( poSession->Failed(
+        OCIHandleAlloc( poSession->hEnv, (dvoid **) &hStatement,
+                        (ub4)OCI_HTYPE_STMT,(size_t)0, (dvoid **)0 ),
         "OCIHandleAlloc(Statement)" ) )
         return CE_Failure;
 
@@ -138,8 +139,8 @@ CPLErr OGROCIStatement::Prepare( const char *pszSQLStatement )
 /*      Prepare the statement.                                          */
 /* -------------------------------------------------------------------- */
     if( poSession->Failed(
-        OCIStmtPrepare( hStatement, poSession->hError, 
-                        (text *) pszSQLStatement, strlen(pszSQLStatement),
+        OCIStmtPrepare( hStatement, poSession->hError,
+                        (text *) pszSQLStatement, static_cast<unsigned int>(strlen(pszSQLStatement)),
                         (ub4)OCI_NTV_SYNTAX, (ub4)OCI_DEFAULT ),
         "OCIStmtPrepare" ) )
         return CE_Failure;
@@ -151,25 +152,25 @@ CPLErr OGROCIStatement::Prepare( const char *pszSQLStatement )
 /*                             BindObject()                             */
 /************************************************************************/
 
-CPLErr OGROCIStatement::BindObject( const char *pszPlaceName, 
+CPLErr OGROCIStatement::BindObject( const char *pszPlaceName,
                                     void *pahObjects, OCIType *hTDO,
                                     void **papIndicators )
 
 {
     OCIBind *hBindOrd = NULL;
 
-    if( poSession->Failed( 
+    if( poSession->Failed(
             OCIBindByName( hStatement, &hBindOrd, poSession->hError,
-                           (text *) pszPlaceName, (sb4) strlen(pszPlaceName), 
-                           (dvoid *) 0, (sb4) 0, SQLT_NTY, (dvoid *)0, 
-                           (ub2 *)0, (ub2 *)0, (ub4)0, (ub4 *)0, 
+                           (text *) pszPlaceName, (sb4) strlen(pszPlaceName),
+                           (dvoid *) 0, (sb4) 0, SQLT_NTY, (dvoid *)0,
+                           (ub2 *)0, (ub2 *)0, (ub4)0, (ub4 *)0,
                            (ub4)OCI_DEFAULT),
             "OCIBindByName()") )
         return CE_Failure;
-    
+
     if( poSession->Failed(
             OCIBindObject( hBindOrd, poSession->hError, hTDO,
-                           (dvoid **) pahObjects, (ub4 *)0, 
+                           (dvoid **) pahObjects, (ub4 *)0,
                            (dvoid **)papIndicators, (ub4 *)0),
             "OCIBindObject()" ) )
         return CE_Failure;
@@ -181,19 +182,19 @@ CPLErr OGROCIStatement::BindObject( const char *pszPlaceName,
 /*                             BindScalar()                             */
 /************************************************************************/
 
-CPLErr OGROCIStatement::BindScalar( const char *pszPlaceName, 
+CPLErr OGROCIStatement::BindScalar( const char *pszPlaceName,
                                     void *pData, int nDataLen,
                                     int nSQLType, sb2 *paeInd )
 
 {
     OCIBind *hBindOrd = NULL;
 
-    if( poSession->Failed( 
+    if( poSession->Failed(
             OCIBindByName( hStatement, &hBindOrd, poSession->hError,
-                           (text *) pszPlaceName, (sb4) strlen(pszPlaceName), 
-                           (dvoid *) pData, (sb4) nDataLen, 
-                           (ub2) nSQLType, (dvoid *)paeInd, (ub2 *)0, 
-                           (ub2 *)0, (ub4)0, (ub4 *)0, 
+                           (text *) pszPlaceName, (sb4) strlen(pszPlaceName),
+                           (dvoid *) pData, (sb4) nDataLen,
+                           (ub2) nSQLType, (dvoid *)paeInd, (ub2 *)0,
+                           (ub2 *)0, (ub4)0, (ub4 *)0,
                            (ub4)OCI_DEFAULT),
             "OCIBindByName()") )
         return CE_Failure;
@@ -221,7 +222,7 @@ CPLErr OGROCIStatement::Execute( const char *pszSQLStatement,
 
     if( hStatement == NULL )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "No prepared statement in call to OGROCIStatement::Execute(NULL)" );
         return CE_Failure;
     }
@@ -231,12 +232,12 @@ CPLErr OGROCIStatement::Execute( const char *pszSQLStatement,
 /* -------------------------------------------------------------------- */
     ub2  nStmtType;
 
-    if( poSession->Failed( 
+    if( poSession->Failed(
         OCIAttrGet( hStatement, OCI_HTYPE_STMT,
                     &nStmtType, 0, OCI_ATTR_STMT_TYPE, poSession->hError ),
         "OCIAttrGet(ATTR_STMT_TYPE)") )
         return CE_Failure;
-    
+
     int bSelect = (nStmtType == OCI_STMT_SELECT);
 
 /* -------------------------------------------------------------------- */
@@ -253,43 +254,54 @@ CPLErr OGROCIStatement::Execute( const char *pszSQLStatement,
 /* -------------------------------------------------------------------- */
 /*      Execute the statement.                                          */
 /* -------------------------------------------------------------------- */
-    if( poSession->Failed( 
-        OCIStmtExecute( poSession->hSvcCtx, hStatement, 
-                        poSession->hError, (ub4)bSelect ? 0 : 1, (ub4)0, 
+    if( poSession->Failed(
+        OCIStmtExecute( poSession->hSvcCtx, hStatement,
+                        poSession->hError, (ub4)bSelect ? 0 : 1, (ub4)0,
                         (OCISnapshot *)NULL, (OCISnapshot *)NULL, nMode ),
         pszCommandText ) )
         return CE_Failure;
 
     if( !bSelect )
+    {
+        ub4 row_count;
+        if( poSession->Failed(
+            OCIAttrGet( hStatement, OCI_HTYPE_STMT,
+                        &row_count, 0, OCI_ATTR_ROW_COUNT, poSession->hError ),
+                        "OCIAttrGet(OCI_ATTR_ROW_COUNT)") )
+            return CE_Failure;
+        nAffectedRows = row_count;
+
         return CE_None;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Count the columns.                                              */
 /* -------------------------------------------------------------------- */
-    for( nRawColumnCount = 0; TRUE; nRawColumnCount++ )
-    {                                                           
+    for( nRawColumnCount = 0; true; nRawColumnCount++ )
+    {
         OCIParam     *hParmDesc;
 
-        if( OCIParamGet( hStatement, OCI_HTYPE_STMT, poSession->hError, 
-                         (dvoid**)&hParmDesc, 
+        if( OCIParamGet( hStatement, OCI_HTYPE_STMT, poSession->hError,
+                         (dvoid**)&hParmDesc,
                          (ub4) nRawColumnCount+1 ) != OCI_SUCCESS )
             break;
     }
-    
+
     panFieldMap = (int *) CPLCalloc(sizeof(int),nRawColumnCount);
-    
+
     papszCurColumn = (char **) CPLCalloc(sizeof(char*),nRawColumnCount+1);
     panCurColumnInd = (sb2 *) CPLCalloc(sizeof(sb2),nRawColumnCount+1);
-        
+
 /* ==================================================================== */
 /*      Establish result column definitions, and setup parameter        */
 /*      defines.                                                        */
 /* ==================================================================== */
     poDefn = new OGRFeatureDefn( pszCommandText );
+    poDefn->SetGeomType(wkbNone);
     poDefn->Reference();
 
     for( int iParm = 0; iParm < nRawColumnCount; iParm++ )
-    {                                                           
+    {
         OGRFieldDefn oField( "", OFTString );
         OCIParam     *hParmDesc;
         ub2          nOCIType;
@@ -298,8 +310,8 @@ CPLErr OGROCIStatement::Execute( const char *pszSQLStatement,
 /* -------------------------------------------------------------------- */
 /*      Get parameter definition.                                       */
 /* -------------------------------------------------------------------- */
-        if( poSession->Failed( 
-            OCIParamGet( hStatement, OCI_HTYPE_STMT, poSession->hError, 
+        if( poSession->Failed(
+            OCIParamGet( hStatement, OCI_HTYPE_STMT, poSession->hError,
                          (dvoid**)&hParmDesc, (ub4) iParm+1 ),
             "OCIParamGet") )
             return CE_Failure;
@@ -310,8 +322,17 @@ CPLErr OGROCIStatement::Execute( const char *pszSQLStatement,
 
         if( oField.GetType() == OFTBinary )
         {
-            panFieldMap[iParm] = -1;
-            continue;                   
+            /* We could probably generalize that, but at least it works in that */
+            /* use case */
+            if( EQUAL(oField.GetNameRef(), "DATA_DEFAULT") && nOCIType == SQLT_LNG )
+            {
+                oField.SetType(OFTString);
+            }
+            else
+            {
+                panFieldMap[iParm] = -1;
+                continue;
+            }
         }
 
         poDefn->AddFieldDefn( &oField );
@@ -324,7 +345,7 @@ CPLErr OGROCIStatement::Execute( const char *pszSQLStatement,
         OCIDefine *hDefn = NULL;
 
         if( oField.GetWidth() > 0 )
-            /* extra space needed for the decimal separator the string 
+            /* extra space needed for the decimal separator the string
             terminator and the negative sign (Tamas Szekeres)*/
             nBufWidth = oField.GetWidth() + 3;
         else if( oField.GetType() == OFTInteger )
@@ -341,9 +362,9 @@ CPLErr OGROCIStatement::Execute( const char *pszSQLStatement,
 
         if( poSession->Failed(
             OCIDefineByPos( hStatement, &hDefn, poSession->hError,
-                            iParm+1, 
+                            iParm+1,
                             (ub1 *) papszCurColumn[nOGRField], nBufWidth,
-                            SQLT_STR, panCurColumnInd + nOGRField, 
+                            SQLT_STR, panCurColumnInd + nOGRField,
                             NULL, NULL, OCI_DEFAULT ),
             "OCIDefineByPos" ) )
             return CE_Failure;
@@ -360,14 +381,14 @@ char **OGROCIStatement::SimpleFetchRow()
 
 {
     int nStatus, i;
-    
+
     if( papszCurImage == NULL )
     {
-        papszCurImage = (char **) 
+        papszCurImage = (char **)
             CPLCalloc(sizeof(char *), nRawColumnCount+1 );
     }
 
-    nStatus = OCIStmtFetch( hStatement, poSession->hError, 1, 
+    nStatus = OCIStmtFetch( hStatement, poSession->hError, 1,
                             OCI_FETCH_NEXT, OCI_DEFAULT );
 
     if( nStatus == OCI_NO_DATA )
@@ -385,4 +406,3 @@ char **OGROCIStatement::SimpleFetchRow()
 
     return papszCurImage;
 }
-
