@@ -1,8 +1,8 @@
 /******************************************************************************
- * $Id: ogrocistringbuf.cpp 10646 2007-01-18 02:38:10Z warmerdam $
+ * $Id: ogrocistringbuf.cpp 33713 2016-03-12 17:41:57Z goatbar $
  *
  * Project:  Oracle Spatial Driver
- * Purpose:  Simple string buffer used to accumulate text of commands 
+ * Purpose:  Simple string buffer used to accumulate text of commands
  *           efficiently.
  * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
@@ -31,7 +31,7 @@
 #include "ogr_oci.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrocistringbuf.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
+CPL_CVSID("$Id: ogrocistringbuf.cpp 33713 2016-03-12 17:41:57Z goatbar $");
 
 /************************************************************************/
 /*                          OGROCIStringBuf()                           */
@@ -78,7 +78,7 @@ void OGROCIStringBuf::MakeRoomFor( int nCharacters )
 void OGROCIStringBuf::Append( const char *pszNewText )
 
 {
-    int  nNewLen = strlen(pszNewText);
+    int  nNewLen = static_cast<int>(strlen(pszNewText));
 
     MakeRoomFor( nNewLen );
     strcat( pszString+nLen, pszNewText );
@@ -96,13 +96,13 @@ void OGROCIStringBuf::Appendf( int nMax, const char *pszFormat, ... )
     char    szSimpleBuf[100];
     char    *pszBuffer;
 
-    if( nMax > (int) sizeof(szSimpleBuf-1) )
+    if( nMax > (int) sizeof(szSimpleBuf)-1 )
         pszBuffer = (char *) CPLMalloc(nMax+1);
     else
         pszBuffer = szSimpleBuf;
 
     va_start(args, pszFormat);
-    vsprintf(pszBuffer, pszFormat, args);
+    CPLvsnprintf(pszBuffer, nMax, pszFormat, args);
     va_end(args);
 
     Append( pszBuffer );
@@ -117,7 +117,7 @@ void OGROCIStringBuf::Appendf( int nMax, const char *pszFormat, ... )
 void OGROCIStringBuf::UpdateEnd()
 
 {
-    nLen += strlen(pszString+nLen);
+    nLen += static_cast<int>(strlen(pszString+nLen));
 }
 
 /************************************************************************/
