@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrodbcdriver.cpp 24957 2012-09-23 17:03:30Z rouault $
+ * $Id: ogrodbcdriver.cpp 33713 2016-03-12 17:41:57Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRODBCDriver class.
@@ -30,7 +30,7 @@
 #include "ogr_odbc.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrodbcdriver.cpp 24957 2012-09-23 17:03:30Z rouault $");
+CPL_CVSID("$Id: ogrodbcdriver.cpp 33713 2016-03-12 17:41:57Z goatbar $");
 
 /************************************************************************/
 /*                            ~OGRODBCDriver()                            */
@@ -61,7 +61,7 @@ OGRDataSource *OGRODBCDriver::Open( const char * pszFilename,
 {
     OGRODBCDataSource     *poDS;
 
-    if( !EQUALN(pszFilename,"ODBC:",5) 
+    if( !STARTS_WITH_CI(pszFilename, "ODBC:")
 #ifdef WIN32
         && !EQUAL(CPLGetExtension(pszFilename), "MDB")
 #endif
@@ -89,7 +89,7 @@ OGRDataSource *OGRODBCDriver::CreateDataSource( const char * pszName,
 {
     OGRODBCDataSource     *poDS;
 
-    if( !EQUALN(pszName,"ODBC:",5) )
+    if( !STARTS_WITH_CI(pszName, "ODBC:") )
         return NULL;
 
     poDS = new OGRODBCDataSource();
@@ -98,7 +98,7 @@ OGRDataSource *OGRODBCDriver::CreateDataSource( const char * pszName,
     if( !poDS->Open( pszName, TRUE, TRUE ) )
     {
         delete poDS;
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
          "ODBC driver doesn't currently support database creation.\n"
                   "Please create database with the `createdb' command." );
         return NULL;
@@ -129,4 +129,3 @@ void RegisterOGRODBC()
 {
     OGRSFDriverRegistrar::GetRegistrar()->RegisterDriver( new OGRODBCDriver );
 }
-
