@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ods_formula.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ods_formula.cpp 32967 2016-01-13 14:40:01Z goatbar $
  *
  * Component: ODS formula Engine
  * Purpose:
@@ -28,13 +28,21 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include <ctype.h>
-#include <math.h>
+#include <cctype>
+#include <cmath>
 
 #include "cpl_conv.h"
 #include "ods_formula.h"
+
+namespace {
 #include "ods_formula_parser.hpp"
 
+int ods_formulalex( ods_formula_node **ppNode,
+                    ods_formula_parse_context *context );
+
+#include "ods_formula_parser.cpp"
+} /* end of anonymous namespace */
+
 #define YYSTYPE  ods_formula_node*
 
 static const SingleOpStruct apsSingleOp[] =
@@ -78,7 +86,7 @@ const SingleOpStruct* ODSGetSingleOpEntry(ods_formula_op eOp)
 /*                                                                      */
 /*      Read back a token from the input.                               */
 /************************************************************************/
-
+namespace {
 int ods_formulalex( YYSTYPE *ppNode, ods_formula_parse_context *context )
 {
     const char *pszInput = context->pszNext;
@@ -105,7 +113,7 @@ int ods_formulalex( YYSTYPE *ppNode, ods_formula_parse_context *context )
     if( *pszInput == '\0' )
     {
         context->pszNext = pszInput;
-        return EOF; 
+        return EOF;
     }
 
 /* -------------------------------------------------------------------- */
@@ -113,13 +121,10 @@ int ods_formulalex( YYSTYPE *ppNode, ods_formula_parse_context *context )
 /* -------------------------------------------------------------------- */
     if( *pszInput == '"' )
     {
-        char *token;
-        int i_token;
-
         pszInput++;
 
-        token = (char *) CPLMalloc(strlen(pszInput)+1);
-        i_token = 0;
+        char *token = static_cast<char *>( CPLMalloc(strlen(pszInput)+1) );
+        int i_token = 0;
 
         while( *pszInput != '\0' )
         {
@@ -139,7 +144,7 @@ int ods_formulalex( YYSTYPE *ppNode, ods_formula_parse_context *context )
                 pszInput++;
                 break;
             }
-            
+
             token[i_token++] = *(pszInput++);
         }
         token[i_token] = '\0';
@@ -157,9 +162,9 @@ int ods_formulalex( YYSTYPE *ppNode, ods_formula_parse_context *context )
 /* -------------------------------------------------------------------- */
     else if( *pszInput >= '0' && *pszInput <= '9' )
     {
-        CPLString osToken;
         const char *pszNext = pszInput + 1;
 
+        CPLString osToken;
         osToken += *pszInput;
 
         // collect non-decimal part of number
@@ -186,8 +191,8 @@ int ods_formulalex( YYSTYPE *ppNode, ods_formula_parse_context *context )
 
         context->pszNext = pszNext;
 
-        if( strstr(osToken,".") 
-            || strstr(osToken,"e") 
+        if( strstr(osToken,".")
+            || strstr(osToken,"e")
             || strstr(osToken,"E") )
         {
             *ppNode = new ods_formula_node( CPLAtof(osToken) );
@@ -206,13 +211,13 @@ int ods_formulalex( YYSTYPE *ppNode, ods_formula_parse_context *context )
     else if( *pszInput == '.' || isalnum( *pszInput ) )
     {
         int nReturn = ODST_IDENTIFIER;
-        CPLString osToken;
         const char *pszNext = pszInput + 1;
 
+        CPLString osToken;
         osToken += *pszInput;
 
         // collect text characters
-        while( isalnum( *pszNext ) || *pszNext == '_' 
+        while( isalnum( *pszNext ) || *pszNext == '_'
                || ((unsigned char) *pszNext) > 127 )
             osToken += *(pszNext++);
 
@@ -258,7 +263,7 @@ int ods_formulalex( YYSTYPE *ppNode, ods_formula_parse_context *context )
             *ppNode = new ods_formula_node( ODS_T );
             return ODST_FUNCTION_SINGLE_ARG;
         }*/
-        
+
         /* Tow-arg functions */
         else if( EQUAL(osToken,"MOD") )
         {
@@ -342,6 +347,7 @@ int ods_formulalex( YYSTYPE *ppNode, ods_formula_parse_context *context )
         return *pszInput;
     }
 }
+} /* end of anonymous namespace */
 
 /************************************************************************/
 /*                        ods_formula_compile()                         */
@@ -360,9 +366,7 @@ ods_formula_node* ods_formula_compile( const char *expr )
     {
         return context.poRoot;
     }
-    else
-    {
-        delete context.poRoot;
-        return NULL;
-    }
+
+    delete context.poRoot;
+    return NULL;
 }
