@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ods_formula.h 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ods_formula.h 32967 2016-01-13 14:40:01Z goatbar $
  *
  * Component: ODS formula Engine
  * Purpose: Implementation of the ods_formula_node class used to represent a
@@ -29,18 +29,16 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _ODS_FORMULA_H_INCLUDED_
-#define _ODS_FORMULA_H_INCLUDED_
+#ifndef ODS_FORMULA_H_INCLUDED_
+#define ODS_FORMULA_H_INCLUDED_
 
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
 #include <vector>
 
-#if defined(_WIN32) && !defined(_WIN32_WCE)
+#if defined(_WIN32)
 #  define strcasecmp stricmp
-#elif defined(_WIN32_WCE)
-#  define strcasecmp _stricmp
 #endif
 
 typedef enum {
@@ -95,6 +93,8 @@ typedef enum {
 
     ODS_CELL,
     ODS_CELL_RANGE,
+
+    ODS_INVALID
 } ods_formula_op;
 
 typedef enum {
@@ -116,6 +116,7 @@ private:
     void           FreeSubExpr();
     std::string    TransformToString() const;
 
+    // TODO: It appears that all the Evaluate methods should return bool.
     int            EvaluateOR(IODSCellEvaluator* poEvaluator);
     int            EvaluateAND(IODSCellEvaluator* poEvaluator);
     int            EvaluateNOT(IODSCellEvaluator* poEvaluator);
@@ -178,7 +179,11 @@ public:
 
 class ods_formula_parse_context {
 public:
-    ods_formula_parse_context() : nStartToken(0), poRoot(NULL) {}
+    ods_formula_parse_context() :
+        nStartToken(0),
+        pszInput(NULL),
+        pszNext(NULL),
+        poRoot(NULL) {}
 
     int        nStartToken;
     const char *pszInput;
@@ -195,8 +200,6 @@ public:
     virtual ~IODSCellEvaluator() {}
 };
 
-int ods_formulaparse( ods_formula_parse_context *context );
-int ods_formulalex( ods_formula_node **ppNode, ods_formula_parse_context *context );
 ods_formula_node* ods_formula_compile( const char *expr );
 
 typedef struct
@@ -209,4 +212,4 @@ typedef struct
 const SingleOpStruct* ODSGetSingleOpEntry(const char* pszName);
 const SingleOpStruct* ODSGetSingleOpEntry(ods_formula_op eOp);
 
-#endif /* def _ODS_FORMULA_H_INCLUDED_ */
+#endif /* def ODS_FORMULA_H_INCLUDED_ */
