@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_ods.h 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogr_ods.h 32989 2016-01-14 21:28:07Z rouault $
  *
  * Project:  ODS Translator
  * Purpose:  Definition of classes for OGR OpenOfficeSpreadsheet .ods driver.
@@ -27,8 +27,8 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _OGR_ODS_H_INCLUDED
-#define _OGR_ODS_H_INCLUDED
+#ifndef OGR_ODS_H_INCLUDED
+#define OGR_ODS_H_INCLUDED
 
 #include "ogrsf_frmts.h"
 #include "ogr_mem.h"
@@ -39,6 +39,8 @@
 #include <string>
 #include <set>
 
+namespace OGRODS {
+
 /************************************************************************/
 /*                             OGRODSLayer                              */
 /************************************************************************/
@@ -48,18 +50,18 @@ class OGRODSDataSource;
 class OGRODSLayer : public OGRMemLayer
 {
     OGRODSDataSource* poDS;
-    int               bUpdated;
-    int               bHasHeaderLine;
+    bool              bUpdated;
+    bool              bHasHeaderLine;
 
     public:
         OGRODSLayer( OGRODSDataSource* poDSIn,
                       const char * pszName,
-                      int bUpdateIn = FALSE);
+                      bool bUpdateIn = FALSE);
 
-    void                SetUpdated(int bUpdatedIn = TRUE);
+    void                SetUpdated(bool bUpdatedIn = true);
 
-    int                 GetHasHeaderLine() { return bHasHeaderLine; }
-    void                SetHasHeaderLine(int bIn) { bHasHeaderLine = bIn; }
+    bool                GetHasHeaderLine() { return bHasHeaderLine; }
+    void                SetHasHeaderLine(bool bIn) { bHasHeaderLine = bIn; }
 
     const char         *GetName() { return OGRMemLayer::GetLayerDefn()->GetName(); };
     OGRwkbGeometryType  GetGeomType() { return wkbNone; }
@@ -67,16 +69,16 @@ class OGRODSLayer : public OGRMemLayer
 
     /* For external usage. Mess with FID */
     virtual OGRFeature *        GetNextFeature();
-    virtual OGRFeature         *GetFeature( long nFeatureId );
-    virtual OGRErr              SetFeature( OGRFeature *poFeature );
-    virtual OGRErr              DeleteFeature( long nFID );
+    virtual OGRFeature         *GetFeature( GIntBig nFeatureId );
+    virtual OGRErr              ISetFeature( OGRFeature *poFeature );
+    virtual OGRErr              DeleteFeature( GIntBig nFID );
 
     /* For internal usage, for cell resolver */
     OGRFeature *        GetNextFeatureWithoutFIDHack() { return OGRMemLayer::GetNextFeature(); }
-    OGRErr              SetFeatureWithoutFIDHack( OGRFeature *poFeature ) { SetUpdated(); return OGRMemLayer::SetFeature(poFeature); }
+    OGRErr              SetFeatureWithoutFIDHack( OGRFeature *poFeature ) { SetUpdated(); return OGRMemLayer::ISetFeature(poFeature); }
 
-    OGRErr              CreateFeature( OGRFeature *poFeature )
-    { SetUpdated(); return OGRMemLayer::CreateFeature(poFeature); }
+    OGRErr              ICreateFeature( OGRFeature *poFeature )
+    { SetUpdated(); return OGRMemLayer::ICreateFeature(poFeature); }
 
     virtual OGRErr      CreateField( OGRFieldDefn *poField,
                                      int bApproxOK = TRUE )
@@ -88,8 +90,8 @@ class OGRODSLayer : public OGRMemLayer
     virtual OGRErr      ReorderFields( int* panMap )
     { SetUpdated(); return OGRMemLayer::ReorderFields(panMap); }
 
-    virtual OGRErr      AlterFieldDefn( int iField, OGRFieldDefn* poNewFieldDefn, int nFlags )
-    { SetUpdated(); return OGRMemLayer::AlterFieldDefn(iField, poNewFieldDefn, nFlags); }
+    virtual OGRErr      AlterFieldDefn( int iField, OGRFieldDefn* poNewFieldDefn, int nFlagsIn )
+    { SetUpdated(); return OGRMemLayer::AlterFieldDefn(iField, poNewFieldDefn, nFlagsIn); }
 
     virtual OGRErr      SyncToDisk();
 };
@@ -117,9 +119,9 @@ typedef struct
 class OGRODSDataSource : public OGRDataSource
 {
     char*               pszName;
-    int                 bUpdatable;
-    int                 bUpdated;
-    int                 bAnalysedFile;
+    bool                bUpdatable;
+    bool                bUpdated;
+    bool                bAnalysedFile;
 
     int                 nLayers;
     OGRLayer          **papoLayers;
@@ -134,11 +136,11 @@ class OGRODSDataSource : public OGRDataSource
     VSILFILE*           fpContent;
     void                AnalyseFile();
 
-    int                 bFirstLineIsHeaders;
+    bool                bFirstLineIsHeaders;
     int                 bAutodetectTypes;
 
     XML_Parser          oParser;
-    int                 bStopParsing;
+    bool                bStopParsing;
     int                 nWithoutEventCounter;
     int                 nDataHandlerCounter;
     int                 nCurLine;
@@ -197,13 +199,13 @@ class OGRODSDataSource : public OGRDataSource
 
     virtual int                 TestCapability( const char * );
 
-    virtual OGRLayer* CreateLayer( const char * pszLayerName,
+    virtual OGRLayer* ICreateLayer( const char * pszLayerName,
                                 OGRSpatialReference *poSRS,
                                 OGRwkbGeometryType eType,
                                 char ** papszOptions );
     virtual OGRErr      DeleteLayer(int iLayer);
 
-    virtual OGRErr      SyncToDisk();
+    virtual void        FlushCache();
 
     void startElementCbk(const char *pszName, const char **ppszAttr);
     void endElementCbk(const char *pszName);
@@ -213,10 +215,12 @@ class OGRODSDataSource : public OGRDataSource
     void endElementStylesCbk(const char *pszName);
     void dataHandlerStylesCbk(const char *data, int nLen);
 
-    int                 GetUpdatable() { return bUpdatable; }
-    void                SetUpdated() { bUpdated = TRUE; }
+    bool                GetUpdatable() { return bUpdatable; }
+    void                SetUpdated() { bUpdated = true; }
 };
 
+} /* end of OGRODS namespace */
+
 /************************************************************************/
 /*                             OGRODSDriver                             */
 /************************************************************************/
@@ -235,5 +239,4 @@ class OGRODSDriver : public OGRSFDriver
     virtual OGRErr      DeleteDataSource( const char *pszName );
 };
 
-
-#endif /* ndef _OGR_ODS_H_INCLUDED */
+#endif /* ndef OGR_ODS_H_INCLUDED */
