@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogropenairlabellayer.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: ogropenairlabellayer.cpp 32976 2016-01-14 00:07:34Z goatbar $
  *
  * Project:  OpenAir Translator
  * Purpose:  Implements OGROpenAirLabelLayer class.
@@ -27,27 +27,25 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "ogr_openair.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
+#include "ogr_openair.h"
 #include "ogr_p.h"
 #include "ogr_srs_api.h"
 
-CPL_CVSID("$Id: ogropenairlabellayer.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: ogropenairlabellayer.cpp 32976 2016-01-14 00:07:34Z goatbar $");
 
 /************************************************************************/
 /*                      OGROpenAirLabelLayer()                          */
 /************************************************************************/
 
-OGROpenAirLabelLayer::OGROpenAirLabelLayer( VSILFILE* fp )
-
+OGROpenAirLabelLayer::OGROpenAirLabelLayer( VSILFILE* fp ) :
+    poFeatureDefn(new OGRFeatureDefn("labels")),
+    poSRS(new OGRSpatialReference(SRS_WKT_WGS84)),
+    fpOpenAir(fp),
+    nNextFID(0)
 {
-    fpOpenAir = fp;
-    nNextFID = 0;
-
-    poSRS = new OGRSpatialReference(SRS_WKT_WGS84);
-
-    poFeatureDefn = new OGRFeatureDefn( "labels"  );
+    SetDescription( poFeatureDefn->GetName() );
     poFeatureDefn->Reference();
     poFeatureDefn->SetGeomType( wkbPoint );
     poFeatureDefn->GetGeomFieldDefn(0)->SetSpatialRef(poSRS);
@@ -96,11 +94,9 @@ void OGROpenAirLabelLayer::ResetReading()
 
 OGRFeature *OGROpenAirLabelLayer::GetNextFeature()
 {
-    OGRFeature  *poFeature;
-
-    while(TRUE)
+    while( true )
     {
-        poFeature = GetNextRawFeature();
+        OGRFeature *poFeature = GetNextRawFeature();
         if (poFeature == NULL)
             return NULL;
 
@@ -111,8 +107,8 @@ OGRFeature *OGROpenAirLabelLayer::GetNextFeature()
         {
             return poFeature;
         }
-        else
-            delete poFeature;
+
+        delete poFeature;
     }
 }
 
@@ -122,20 +118,20 @@ OGRFeature *OGROpenAirLabelLayer::GetNextFeature()
 
 OGRFeature *OGROpenAirLabelLayer::GetNextRawFeature()
 {
-    const char* pszLine;
-    double dfLat = 0, dfLon = 0;
-    int bHasCoord = FALSE;
+    double dfLat = 0;
+    double dfLon = 0;
+    bool bHasCoord = false;
 
-    while(TRUE)
+    while( true )
     {
-        pszLine = CPLReadLine2L(fpOpenAir, 1024, NULL);
+        const char* pszLine = CPLReadLine2L(fpOpenAir, 1024, NULL);
         if (pszLine == NULL)
             return NULL;
 
         if (pszLine[0] == '*' || pszLine[0] == '\0')
             continue;
 
-        if (EQUALN(pszLine, "AC ", 3))
+        if (STARTS_WITH_CI(pszLine, "AC "))
         {
             if (osCLASS.size() != 0)
             {
@@ -145,13 +141,13 @@ OGRFeature *OGROpenAirLabelLayer::GetNextRawFeature()
             }
             osCLASS = pszLine + 3;
         }
-        else if (EQUALN(pszLine, "AN ", 3))
+        else if (STARTS_WITH_CI(pszLine, "AN "))
             osNAME = pszLine + 3;
-        else if (EQUALN(pszLine, "AH ", 3))
+        else if (STARTS_WITH_CI(pszLine, "AH "))
             osCEILING = pszLine + 3;
-        else if (EQUALN(pszLine, "AL ", 3))
+        else if (STARTS_WITH_CI(pszLine, "AL "))
             osFLOOR = pszLine + 3;
-        else if (EQUALN(pszLine, "AT ", 3))
+        else if (STARTS_WITH_CI(pszLine, "AT "))
         {
             bHasCoord = OGROpenAirGetLatLon(pszLine + 3, dfLat, dfLon);
             break;
@@ -184,8 +180,7 @@ OGRFeature *OGROpenAirLabelLayer::GetNextRawFeature()
 /*                           TestCapability()                           */
 /************************************************************************/
 
-int OGROpenAirLabelLayer::TestCapability( CPL_UNUSED const char * pszCap )
+int OGROpenAirLabelLayer::TestCapability( const char * /* pszCap */ )
 {
     return FALSE;
 }
-
