@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: filegdbindex.cpp 27751 2014-09-28 16:33:48Z rouault $
+ * $Id: filegdbindex.cpp 33027 2016-01-17 18:47:10Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements reading of FileGDB indexes
@@ -41,7 +41,8 @@ namespace OpenFileGDB
 /*                     FileGDBOGRDateToDoubleDate()                     */
 /************************************************************************/
 
-static int FileGDBOGRDateToDoubleDate( const OGRField* psField, double *pdfVal )
+static bool FileGDBOGRDateToDoubleDate( const OGRField* psField,
+                                        double *pdfVal )
 {
     struct tm brokendowntime;
 
@@ -50,20 +51,20 @@ static int FileGDBOGRDateToDoubleDate( const OGRField* psField, double *pdfVal )
     brokendowntime.tm_mday = psField->Date.Day;
     brokendowntime.tm_hour = psField->Date.Hour;
     brokendowntime.tm_min = psField->Date.Minute;
-    brokendowntime.tm_sec = psField->Date.Second;
+    brokendowntime.tm_sec = static_cast<int>(psField->Date.Second);
 
-    GIntBig nTime = CPLYMDHMSToUnixTime(&brokendowntime);
+    const GIntBig nTime = CPLYMDHMSToUnixTime(&brokendowntime);
 
     *pdfVal = nTime / 3600. / 24 + 25569;
 
-    return TRUE;
+    return true;
 }
 
 /************************************************************************/
 /*                        FileGDBTrivialIterator                        */
 /************************************************************************/
 
-class FileGDBTrivialIterator : public FileGDBIterator
+class FileGDBTrivialIterator CPL_FINAL : public FileGDBIterator
 {
         FileGDBIterator            *poParentIter;
         FileGDBTable               *poTable;
@@ -95,7 +96,7 @@ class FileGDBTrivialIterator : public FileGDBIterator
 /*                        FileGDBNotIterator                            */
 /************************************************************************/
 
-class FileGDBNotIterator : public FileGDBIterator
+class FileGDBNotIterator CPL_FINAL : public FileGDBIterator
 {
         FileGDBIterator            *poIterBase;
         FileGDBTable               *poTable;
@@ -117,7 +118,7 @@ class FileGDBNotIterator : public FileGDBIterator
 /*                        FileGDBAndIterator                            */
 /************************************************************************/
 
-class FileGDBAndIterator : public FileGDBIterator
+class FileGDBAndIterator CPL_FINAL : public FileGDBIterator
 {
         FileGDBIterator             *poIter1;
         FileGDBIterator             *poIter2;
@@ -138,14 +139,14 @@ class FileGDBAndIterator : public FileGDBIterator
 /*                        FileGDBOrIterator                             */
 /************************************************************************/
 
-class FileGDBOrIterator : public FileGDBIterator
+class FileGDBOrIterator CPL_FINAL : public FileGDBIterator
 {
         FileGDBIterator             *poIter1;
         FileGDBIterator             *poIter2;
         int                          bIteratorAreExclusive;
         int                          iNextRow1;
         int                          iNextRow2;
-        int                          bHasJustReset;
+        bool                         bHasJustReset;
 
     public:
                                      FileGDBOrIterator(FileGDBIterator* poIter1,
@@ -163,16 +164,16 @@ class FileGDBOrIterator : public FileGDBIterator
 /*                        FileGDBIndexIterator                          */
 /************************************************************************/
 
-#define MAX_DEPTH               3
-#define UUID_LEN_AS_STRING      38
-#define MAX_CAR_COUNT_STR       80
-#define MAX_UTF8_LEN_STR        (4 * MAX_CAR_COUNT_STR)
-#define FGDB_PAGE_SIZE          4096
+static const int MAX_DEPTH = 3;
+static const int UUID_LEN_AS_STRING = 38;
+static const int MAX_CAR_COUNT_STR = 80;
+static const int MAX_UTF8_LEN_STR = 4 * MAX_CAR_COUNT_STR;
+static const int FGDB_PAGE_SIZE = 4096;
 
-class FileGDBIndexIterator : public FileGDBIterator
+class FileGDBIndexIterator CPL_FINAL : public FileGDBIterator
 {
         FileGDBTable        *poParent;
-        int                  bAscending;
+        bool                 bAscending;
         VSILFILE            *fpCurIdx;
         FileGDBFieldType     eFieldType;
         GUInt32              nMaxPerPages;
@@ -382,8 +383,8 @@ int FileGDBIterator::GetRowCount()
 /*                         FileGDBTrivialIterator()                     */
 /************************************************************************/
 
-FileGDBTrivialIterator::FileGDBTrivialIterator(FileGDBIterator* poParentIter) :
-        poParentIter(poParentIter), poTable(poParentIter->GetTable()), iRow(0)
+FileGDBTrivialIterator::FileGDBTrivialIterator(FileGDBIterator* poParentIterIn) :
+        poParentIter(poParentIterIn), poTable(poParentIterIn->GetTable()), iRow(0)
 {
 }
 
@@ -403,8 +404,8 @@ int FileGDBTrivialIterator::GetNextRowSortedByFID()
 /*                           FileGDBNotIterator()                       */
 /************************************************************************/
 
-FileGDBNotIterator::FileGDBNotIterator(FileGDBIterator* poIterBase) :
-    poIterBase(poIterBase), poTable(poIterBase->GetTable()), iRow(0), iNextRowBase(-1)
+FileGDBNotIterator::FileGDBNotIterator(FileGDBIterator* poIterBaseIn) :
+    poIterBase(poIterBaseIn), poTable(poIterBaseIn->GetTable()), iRow(0), iNextRowBase(-1)
 {
     bNoHoles = (poTable->GetValidRecordCount() == poTable->GetTotalRecordCount());
 }
@@ -442,7 +443,7 @@ int FileGDBNotIterator::GetNextRowSortedByFID()
             iNextRowBase = poTable->GetTotalRecordCount();
     }
 
-    while( TRUE )
+    while( true )
     {
         if( iRow < iNextRowBase )
         {
@@ -481,9 +482,9 @@ int FileGDBNotIterator::GetRowCount()
 /*                          FileGDBAndIterator()                        */
 /************************************************************************/
 
-FileGDBAndIterator::FileGDBAndIterator(FileGDBIterator* poIter1,
-                                       FileGDBIterator* poIter2) :
-                                       poIter1(poIter1), poIter2(poIter2),
+FileGDBAndIterator::FileGDBAndIterator(FileGDBIterator* poIter1In,
+                                       FileGDBIterator* poIter2In) :
+                                       poIter1(poIter1In), poIter2(poIter2In),
                                        iNextRow1(-1), iNextRow2(-1)
 {
     CPLAssert(poIter1->GetTable() == poIter2->GetTable());
@@ -527,7 +528,7 @@ int FileGDBAndIterator::GetNextRowSortedByFID()
         }
     }
 
-    while( TRUE )
+    while( true )
     {
         if( iNextRow1 < iNextRow2 )
         {
@@ -551,12 +552,15 @@ int FileGDBAndIterator::GetNextRowSortedByFID()
 /************************************************************************/
 
 
-FileGDBOrIterator::FileGDBOrIterator(FileGDBIterator* poIter1,
-                                     FileGDBIterator* poIter2,
-                                     int bIteratorAreExclusive) :
-                            poIter1(poIter1), poIter2(poIter2),
-                            bIteratorAreExclusive(bIteratorAreExclusive),
-                            iNextRow1(-1), iNextRow2(-1), bHasJustReset(TRUE)
+FileGDBOrIterator::FileGDBOrIterator(FileGDBIterator* poIter1In,
+                                     FileGDBIterator* poIter2In,
+                                     int bIteratorAreExclusiveIn) :
+    poIter1(poIter1In),
+    poIter2(poIter2In),
+    bIteratorAreExclusive(bIteratorAreExclusiveIn),
+    iNextRow1(-1),
+    iNextRow2(-1),
+    bHasJustReset(true)
 {
     CPLAssert(poIter1->GetTable() == poIter2->GetTable());
 }
@@ -581,7 +585,7 @@ void FileGDBOrIterator::Reset()
     poIter2->Reset();
     iNextRow1 = -1;
     iNextRow2 = -1;
-    bHasJustReset = TRUE;
+    bHasJustReset = true;
 }
 
 /************************************************************************/
@@ -592,7 +596,7 @@ int FileGDBOrIterator::GetNextRowSortedByFID()
 {
     if( bHasJustReset )
     {
-        bHasJustReset = FALSE;
+        bHasJustReset = false;
         iNextRow1 = poIter1->GetNextRowSortedByFID();
         iNextRow2 = poIter2->GetNextRowSortedByFID();
     }
@@ -641,15 +645,25 @@ int FileGDBOrIterator::GetRowCount()
 /*                         FileGDBIndexIterator()                       */
 /************************************************************************/
 
-FileGDBIndexIterator::FileGDBIndexIterator(FileGDBTable* poParent, int bAscending) :
-                    poParent(poParent), bAscending(bAscending),
-                    fpCurIdx(NULL), eFieldType(FGFT_UNDEFINED),
-                    nMaxPerPages(0), nOffsetFirstValInPage(0),
-                    nValueCountInIdx(0), nIndexDepth(0), eOp(FGSO_ISNOTNULL),
-                    iCurFeatureInPage(-1), nFeaturesInPage(0),
-                    bEvaluateToFALSE(FALSE), bEOF(FALSE),
-                    iSorted(0), nSortedCount(-1), panSortedRows(NULL),
-                    nStrLen(0)
+FileGDBIndexIterator::FileGDBIndexIterator( FileGDBTable* poParentIn,
+                                            int bAscendingIn) :
+  poParent(poParentIn),
+  bAscending(CPL_TO_BOOL(bAscendingIn)),
+  fpCurIdx(NULL),
+  eFieldType(FGFT_UNDEFINED),
+  nMaxPerPages(0),
+  nOffsetFirstValInPage(0),
+  nValueCountInIdx(0),
+  nIndexDepth(0),
+  eOp(FGSO_ISNOTNULL),
+  iCurFeatureInPage(-1),
+  nFeaturesInPage(0),
+  bEvaluateToFALSE(FALSE),
+  bEOF(FALSE),
+  iSorted(0),
+  nSortedCount(-1),
+  panSortedRows(NULL),
+  nStrLen(0)
 {
     memset(iFirstPageIdx, 0xFF, MAX_DEPTH * sizeof(int));
     memset(iLastPageIdx, 0xFF, MAX_DEPTH * sizeof(int));
@@ -731,7 +745,7 @@ static const char* FileGDBValueToStr(OGRFieldType eOGRFieldType,
                                             psValue->Date.Day,
                                             psValue->Date.Hour,
                                             psValue->Date.Minute,
-                                            psValue->Date.Second);
+                                            (int)psValue->Date.Second);
         case OFTDate: return CPLSPrintf("%04d/%02d/%02d",
                                             psValue->Date.Year,
                                             psValue->Date.Month,
@@ -739,7 +753,7 @@ static const char* FileGDBValueToStr(OGRFieldType eOGRFieldType,
         case OFTTime: return CPLSPrintf("%02d:%02d:%02d",
                                             psValue->Date.Hour,
                                             psValue->Date.Minute,
-                                            psValue->Date.Second);
+                                            (int)psValue->Date.Second);
         default:
             break;
     }
@@ -903,8 +917,10 @@ int FileGDBIndexIterator::SetConstraint(int nFieldIdx,
                 returnErrorIf(eOGRFieldType != OFTString);
                 memset(szUUID, 0, UUID_LEN_AS_STRING + 1);
                 strncpy(szUUID, psValue->String, UUID_LEN_AS_STRING);
-                bEvaluateToFALSE = (eOp == FGSO_EQ &&
-                        strlen(psValue->String) != UUID_LEN_AS_STRING);
+                bEvaluateToFALSE =
+                    eOp == FGSO_EQ &&
+                    strlen(psValue->String) !=
+                    static_cast<size_t>(UUID_LEN_AS_STRING);
             }
             break;
         }
@@ -1046,7 +1062,7 @@ int FileGDBIndexIterator::FindPages(int iLevel, int nPage)
             case FGFT_STRING:
             {
                 GUInt16* pasMax;
-#ifdef CPL_MSB
+#if defined(CPL_MSB) || defined(CPL_CPU_REQUIRES_ALIGNED_ACCESS)
                 GUInt16 asMax[MAX_CAR_COUNT_STR];
                 pasMax = asMax;
                 memcpy(asMax, abyPage[iLevel] + nOffsetFirstValInPage +
@@ -1289,7 +1305,7 @@ int FileGDBIndexIterator::GetNextRow()
     if( bEOF )
         return -1;
 
-    while( TRUE )
+    while( true )
     {
         if( iCurFeatureInPage >= nFeaturesInPage || iCurFeatureInPage < 0 )
         {
@@ -1308,7 +1324,6 @@ int FileGDBIndexIterator::GetNextRow()
         else
         {
             int nComp;
-            bMatch = FALSE;
             switch( eFieldType )
             {
                 case FGFT_INT16:
@@ -1346,7 +1361,7 @@ int FileGDBIndexIterator::GetNextRow()
 
                 case FGFT_STRING:
                 {
-#ifdef CPL_MSB
+#if defined(CPL_MSB) || defined(CPL_CPU_REQUIRES_ALIGNED_ACCESS)
                     GUInt16 asVal[MAX_CAR_COUNT_STR];
                     memcpy(asVal, abyPageFeature + nOffsetFirstValInPage +
                                     nStrLen * 2 * iCurFeatureInPage, nStrLen * 2);
@@ -1377,6 +1392,8 @@ int FileGDBIndexIterator::GetNextRow()
                     break;
             }
 
+            bMatch = FALSE;
+            CPL_IGNORE_RET_VAL(bMatch);
             switch( eOp )
             {
                 case FGSO_LT:
@@ -1451,7 +1468,7 @@ int FileGDBIndexIterator::SortRows()
     iSorted = 0;
     int nSortedAlloc = 0;
     Reset();
-    while( TRUE )
+    while( true )
     {
         int nRow = GetNextRow();
         if( nRow < 0 )
@@ -1459,7 +1476,7 @@ int FileGDBIndexIterator::SortRows()
         if( nSortedCount == nSortedAlloc )
         {
             int nNewSortedAlloc = 4 * nSortedAlloc / 3 + 16;
-            int* panNewSortedRows = (int*)VSIRealloc(panSortedRows,
+            int* panNewSortedRows = (int*)VSI_REALLOC_VERBOSE(panSortedRows,
                                             sizeof(int) * nNewSortedAlloc);
             if( panNewSortedRows == NULL )
             {
@@ -1474,8 +1491,10 @@ int FileGDBIndexIterator::SortRows()
     if( nSortedCount == 0 )
         return FALSE;
     std::sort(panSortedRows, panSortedRows + nSortedCount);
+#ifdef nValueCountInIdx_reliable
     if( eOp == FGSO_ISNOTNULL && (int)nValueCountInIdx != nSortedCount )
         PrintError();
+#endif
     return TRUE;
 }
 
@@ -1509,15 +1528,22 @@ int FileGDBIndexIterator::GetNextRowSortedByFID()
 
 int FileGDBIndexIterator::GetRowCount()
 {
+    // The nValueCountInIdx value has been found to be unreliable when the index is built
+    // as features are inserted (and when they are not in increasing order)
+    // (with FileGDB SDK 1.3)
+    // So disable this optimization as there's no fast way to know
+    // if the value is reliable or not.
+#ifdef nValueCountInIdx_reliable
     if( eOp == FGSO_ISNOTNULL )
         return (int)nValueCountInIdx;
+#endif
 
     if( nSortedCount >= 0 )
         return nSortedCount;
 
     int nRowCount = 0;
-    int bSaveAscending = bAscending;
-    bAscending = TRUE; /* for a tiny bit of more efficiency */
+    bool bSaveAscending = bAscending;
+    bAscending = true; /* for a tiny bit of more efficiency */
     Reset();
     while( GetNextRow() >= 0 )
         nRowCount ++;
@@ -1539,26 +1565,26 @@ const OGRField* FileGDBIndexIterator::GetMinMaxValue(OGRField* psField,
     if( nValueCountInIdx == 0 )
         return NULL;
 
-    GByte abyPage[FGDB_PAGE_SIZE];
+    GByte l_abyPage[FGDB_PAGE_SIZE];
     GUInt32 nPage = 1;
     for( GUInt32 iLevel = 0; iLevel < nIndexDepth - 1; iLevel ++ )
     {
         VSIFSeekL(fpCurIdx, (nPage - 1) * FGDB_PAGE_SIZE, SEEK_SET);
-        returnErrorIf(VSIFReadL( abyPage, FGDB_PAGE_SIZE, 1, fpCurIdx ) != 1 );
-        GUInt32 nSubPagesCount = GetUInt32(abyPage + 4, 0);
-        returnErrorIf(nSubPagesCount == 0 || nSubPagesCount > nMaxPerPages);
+        returnErrorIf(VSIFReadL( l_abyPage, FGDB_PAGE_SIZE, 1, fpCurIdx ) != 1 );
+        GUInt32 l_nSubPagesCount = GetUInt32(l_abyPage + 4, 0);
+        returnErrorIf(l_nSubPagesCount == 0 || l_nSubPagesCount > nMaxPerPages);
 
         if( bIsMin )
-            nPage = GetUInt32(abyPage + 8, 0);
+            nPage = GetUInt32(l_abyPage + 8, 0);
         else
-            nPage = GetUInt32(abyPage + 8, nSubPagesCount);
+            nPage = GetUInt32(l_abyPage + 8, l_nSubPagesCount);
         returnErrorIf(nPage < 2 );
     }
 
     VSIFSeekL(fpCurIdx, (nPage - 1) * FGDB_PAGE_SIZE, SEEK_SET);
-    returnErrorIf(VSIFReadL( abyPage, FGDB_PAGE_SIZE, 1, fpCurIdx ) != 1);
+    returnErrorIf(VSIFReadL( l_abyPage, FGDB_PAGE_SIZE, 1, fpCurIdx ) != 1);
 
-    GUInt32 nFeatures = GetUInt32(abyPage + 4, 0);
+    GUInt32 nFeatures = GetUInt32(l_abyPage + 4, 0);
     returnErrorIf(nFeatures < 1 || nFeatures > nMaxPerPages);
 
     int iFeature = (bIsMin) ? 0 : nFeatures-1;
@@ -1567,7 +1593,7 @@ const OGRField* FileGDBIndexIterator::GetMinMaxValue(OGRField* psField,
     {
         case FGFT_INT16:
         {
-            GInt16 nVal = GetInt16(abyPage + nOffsetFirstValInPage, iFeature);
+            GInt16 nVal = GetInt16(l_abyPage + nOffsetFirstValInPage, iFeature);
             psField->Integer = nVal;
             eOutType = OFTInteger;
             return psField;
@@ -1575,7 +1601,7 @@ const OGRField* FileGDBIndexIterator::GetMinMaxValue(OGRField* psField,
 
         case FGFT_INT32:
         {
-            GInt32 nVal = GetInt32(abyPage + nOffsetFirstValInPage, iFeature);
+            GInt32 nVal = GetInt32(l_abyPage + nOffsetFirstValInPage, iFeature);
             psField->Integer = nVal;
             eOutType = OFTInteger;
             return psField;
@@ -1583,7 +1609,7 @@ const OGRField* FileGDBIndexIterator::GetMinMaxValue(OGRField* psField,
 
         case FGFT_FLOAT32:
         {
-            float fVal = GetFloat32(abyPage + nOffsetFirstValInPage, iFeature);
+            float fVal = GetFloat32(l_abyPage + nOffsetFirstValInPage, iFeature);
             psField->Real = fVal;
             eOutType = OFTReal;
             return psField;
@@ -1591,7 +1617,7 @@ const OGRField* FileGDBIndexIterator::GetMinMaxValue(OGRField* psField,
 
         case FGFT_FLOAT64:
         {
-            double dfVal = GetFloat64(abyPage + nOffsetFirstValInPage, iFeature);
+            double dfVal = GetFloat64(l_abyPage + nOffsetFirstValInPage, iFeature);
             psField->Real = dfVal;
             eOutType = OFTReal;
             return psField;
@@ -1599,7 +1625,7 @@ const OGRField* FileGDBIndexIterator::GetMinMaxValue(OGRField* psField,
 
         case FGFT_DATETIME:
         {
-            double dfVal = GetFloat64(abyPage + nOffsetFirstValInPage, iFeature);
+            double dfVal = GetFloat64(l_abyPage + nOffsetFirstValInPage, iFeature);
             FileGDBDoubleDateToOGRDate(dfVal, psField);
             eOutType = OFTDateTime;
             return psField;
@@ -1610,15 +1636,16 @@ const OGRField* FileGDBIndexIterator::GetMinMaxValue(OGRField* psField,
             wchar_t awsVal[MAX_CAR_COUNT_STR+1];
             for(int j=0;j<nStrLen;j++)
             {
-                GUInt16 nCh = GetUInt16(abyPage + nOffsetFirstValInPage +
+                GUInt16 nCh = GetUInt16(l_abyPage + nOffsetFirstValInPage +
                     nStrLen * sizeof(GUInt16) * iFeature, j);
                 awsVal[j] = nCh;
             }
             awsVal[nStrLen] = 0;
             char* pszOut = CPLRecodeFromWChar(awsVal, CPL_ENC_UCS2, CPL_ENC_UTF8);
             returnErrorIf(pszOut == NULL );
-            returnErrorAndCleanupIf(strlen(pszOut) >
-                                        MAX_UTF8_LEN_STR, VSIFree(pszOut) );
+            returnErrorAndCleanupIf(
+                strlen(pszOut) > static_cast<size_t>(MAX_UTF8_LEN_STR),
+                VSIFree(pszOut) );
             strcpy(psField->String, pszOut);
             CPLFree(pszOut);
             eOutType = OFTString;
@@ -1628,7 +1655,7 @@ const OGRField* FileGDBIndexIterator::GetMinMaxValue(OGRField* psField,
         case FGFT_UUID_1:
         case FGFT_UUID_2:
         {
-            memcpy(psField->String, abyPage + nOffsetFirstValInPage +
+            memcpy(psField->String, l_abyPage + nOffsetFirstValInPage +
                    UUID_LEN_AS_STRING *iFeature, UUID_LEN_AS_STRING);
             psField->String[UUID_LEN_AS_STRING] = 0;
             eOutType = OFTString;
@@ -1717,7 +1744,7 @@ template <class Getter> void FileGDBIndexIterator::GetMinMaxSumCount(
     double dfLocalSum = 0.0;
     double dfVal = 0.0;
 
-    while( TRUE )
+    while( true )
     {
         if( iCurFeatureInPage >= nFeaturesInPage )
         {
@@ -1755,8 +1782,8 @@ int FileGDBIndexIterator::GetMinMaxSumCount(double& dfMin, double& dfMax,
                   eFieldType != FGFT_FLOAT32 && eFieldType != FGFT_FLOAT64 &&
                   eFieldType != FGFT_DATETIME );
 
-    int bSaveAscending = bAscending;
-    bAscending = TRUE;
+    bool bSaveAscending = bAscending;
+    bAscending = true;
     Reset();
 
     switch( eFieldType )
