@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogropenfilegdbdatasource.cpp 28733 2015-03-15 03:06:35Z rouault $
+ * $Id: ogropenfilegdbdatasource.cpp 33712 2016-03-12 10:51:56Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements Open FileGDB OGR driver.
@@ -36,12 +36,11 @@ CPL_CVSID("$Id");
 /************************************************************************/
 /*                      OGROpenFileGDBDataSource()                      */
 /************************************************************************/
-OGROpenFileGDBDataSource::OGROpenFileGDBDataSource()
-{
-    m_pszName = NULL;
-    m_papszFiles = NULL;
-    bLastSQLUsedOptimizedImplementation = FALSE;
-}
+OGROpenFileGDBDataSource::OGROpenFileGDBDataSource() :
+    m_pszName(NULL),
+    m_papszFiles(NULL),
+    bLastSQLUsedOptimizedImplementation(false)
+{}
 
 /************************************************************************/
 /*                     ~OGROpenFileGDBDataSource()                      */
@@ -49,10 +48,9 @@ OGROpenFileGDBDataSource::OGROpenFileGDBDataSource()
 OGROpenFileGDBDataSource::~OGROpenFileGDBDataSource()
 
 {
-    size_t i;
-    for( i = 0; i < m_apoLayers.size(); i++ )
+    for( size_t i = 0; i < m_apoLayers.size(); i++ )
         delete m_apoLayers[i];
-    for( i = 0; i < m_apoHiddenLayers.size(); i++ )
+    for( size_t i = 0; i < m_apoHiddenLayers.size(); i++ )
         delete m_apoHiddenLayers[i];
     CPLFree(m_pszName);
     CSLDestroy(m_papszFiles);
@@ -66,11 +64,9 @@ int OGROpenFileGDBDataSource::FileExists(const char* pszFilename)
 {
     if( m_papszFiles )
         return CSLFindString(m_papszFiles, CPLGetFilename(pszFilename)) >= 0;
-    else
-    {
-        VSIStatBufL sStat;
-        return VSIStatExL(pszFilename, &sStat, VSI_STAT_EXISTS_FLAG) == 0;
-    }
+
+    VSIStatBufL sStat;
+    return VSIStatExL(pszFilename, &sStat, VSI_STAT_EXISTS_FLAG) == 0;
 }
 
 /************************************************************************/
@@ -99,27 +95,28 @@ int OGROpenFileGDBDataSource::Open( const char* pszFilename )
     }
 
     if( EQUAL(CPLGetExtension(m_osDirName), "zip") &&
-        strncmp(m_osDirName, "/vsizip/", strlen("/vsizip/")) != 0 )
+        !STARTS_WITH(m_osDirName, "/vsizip/") )
     {
         m_osDirName = "/vsizip/" + m_osDirName;
     }
     else  if( EQUAL(CPLGetExtension(m_osDirName), "tar") &&
-        strncmp(m_osDirName, "/vsitar/", strlen("/vsitar/")) != 0 )
+        !STARTS_WITH(m_osDirName, "/vsitar/") )
     {
         m_osDirName = "/vsitar/" + m_osDirName;
     }
 
-    if( strncmp(m_osDirName, "/vsizip/", strlen("/vsizip/")) == 0 ||
-        strncmp(m_osDirName, "/vsitar/", strlen("/vsitar/")) == 0)
+    if( STARTS_WITH(m_osDirName, "/vsizip/") ||
+        STARTS_WITH(m_osDirName, "/vsitar/"))
     {
         /* Look for one subdirectory ending with .gdb extension */
-        char** papszDir = CPLReadDir(m_osDirName);
+        char** papszDir = VSIReadDir(m_osDirName);
         int iCandidate = -1;
         for( int i=0; papszDir && papszDir[i] != NULL; i++ )
         {
             VSIStatBufL sStat;
             if( EQUAL(CPLGetExtension(papszDir[i]), "gdb") &&
-                VSIStatL( CPLSPrintf("%s/%s", m_osDirName.c_str(), papszDir[i]), &sStat ) == 0 &&
+                VSIStatL( CPLSPrintf("%s/%s", m_osDirName.c_str(), papszDir[i]),
+                          &sStat ) == 0 &&
                 VSI_ISDIR(sStat.st_mode) )
             {
                 if( iCandidate < 0 )
@@ -142,7 +139,8 @@ int OGROpenFileGDBDataSource::Open( const char* pszFilename )
     m_papszFiles = VSIReadDir(m_osDirName);
 
     /* Explore catalog table */
-    const char* psza00000001 = CPLFormFilename(m_osDirName, "a00000001", "gdbtable");
+    const char* psza00000001 =
+        CPLFormFilename(m_osDirName, "a00000001", "gdbtable");
     if( !FileExists(psza00000001) || !oTable.Open(psza00000001) )
     {
         if( nInterestTable >= 0 && FileExists(m_pszName) )
@@ -151,9 +149,10 @@ int OGROpenFileGDBDataSource::Open( const char* pszFilename )
             OGROpenFileGDBLayer* poLayer = new OGROpenFileGDBLayer(
                                 m_pszName, pszLyrName, "", "");
             const char* pszTablX = CPLResetExtension(m_pszName, "gdbtablx");
-            if( !FileExists(pszTablX) &&
-                poLayer->GetLayerDefn()->GetFieldCount() == 0 &&
-                poLayer->GetFeatureCount() == 0 )
+            if( (!FileExists(pszTablX) &&
+                 poLayer->GetLayerDefn()->GetFieldCount() == 0 &&
+                 poLayer->GetFeatureCount() == 0) ||
+                !poLayer->IsValidLayerDefn() )
             {
                 delete poLayer;
                 return FALSE;
@@ -177,10 +176,9 @@ int OGROpenFileGDBDataSource::Open( const char* pszFilename )
     int iGDBItems = -1; /* V10 */
     int iGDBFeatureClasses = -1; /* V9.X */
     int iGDBObjectClasses = -1; /* V9.X */
-    int i;
 
     std::vector<std::string> aosTableNames;
-    for(i=0;i<oTable.GetTotalRecordCount();i++)
+    for( int i=0;i<oTable.GetTotalRecordCount();i++)
     {
         if( !oTable.SelectRow(i) )
         {
@@ -268,7 +266,7 @@ int OGROpenFileGDBDataSource::Open( const char* pszFilename )
 void OGROpenFileGDBDataSource::AddLayer( const CPLString& osName,
                                          int nInterestTable,
                                          int& nCandidateLayers,
-                                         int& nLayersSDC,
+                                         int& nLayersSDCOrCDF,
                                          const CPLString& osDefinition,
                                          const CPLString& osDocumentation,
                                          const char* pszGeomName,
@@ -290,12 +288,22 @@ void OGROpenFileGDBDataSource::AddLayer( const CPLString& osName,
             if( m_papszFiles != NULL )
             {
                 const char* pszSDC = CPLResetExtension(pszFilename, "gdbtable.sdc");
-                if( FileExists(pszSDC) )
+                const char* pszCDF = CPLResetExtension(pszFilename, "gdbtable.cdf");
+                if( FileExists(pszSDC) || FileExists(pszCDF) )
                 {
-                    nLayersSDC ++;
-                    CPLError(CE_Warning, CPLE_AppDefined,
-                            "%s layer has a %s file whose format is unhandled",
-                            osName.c_str(), pszSDC);
+                    nLayersSDCOrCDF ++;
+                    if( GDALGetDriverByName("FileGDB") == NULL )
+                    {
+                        CPLError(CE_Warning, CPLE_AppDefined,
+                                "%s layer has a %s file whose format is unhandled",
+                                osName.c_str(), FileExists(pszSDC) ? pszSDC : pszCDF);
+                    }
+                    else
+                    {
+                        CPLDebug("OpenFileGDB",
+                                 "%s layer has a %s file whose format is unhandled",
+                                  osName.c_str(), FileExists(pszSDC) ? pszSDC : pszCDF);
+                    }
                     return;
                 }
             }
@@ -317,11 +325,11 @@ void OGROpenFileGDBDataSource::AddLayer( const CPLString& osName,
 int OGROpenFileGDBDataSource::OpenFileGDBv10(int iGDBItems,
                                              int nInterestTable)
 {
-    FileGDBTable oTable;
-    int i;
 
     CPLDebug("OpenFileGDB", "FileGDB v10 or later");
 
+    FileGDBTable oTable;
+
     if( !oTable.Open(CPLFormFilename(m_osDirName,
             CPLSPrintf("a%08x.gdbtable", iGDBItems + 1), NULL)) )
         return FALSE;
@@ -339,8 +347,9 @@ int OGROpenFileGDBDataSource::OpenFileGDBv10(int iGDBItems,
         return FALSE;
     }
 
-    int nCandidateLayers = 0, nLayersSDC = 0;
-    for(i=0;i<oTable.GetTotalRecordCount();i++)
+    int nCandidateLayers = 0;
+    int nLayersSDCOrCDF = 0;
+    for( int i=0;i<oTable.GetTotalRecordCount();i++)
     {
         if( !oTable.SelectRow(i) )
         {
@@ -362,7 +371,7 @@ int OGROpenFileGDBDataSource::OpenFileGDBv10(int iGDBItems,
             psField = oTable.GetFieldValue(iName);
             if( psField != NULL )
             {
-                AddLayer( psField->String, nInterestTable, nCandidateLayers, nLayersSDC,
+                AddLayer( psField->String, nInterestTable, nCandidateLayers, nLayersSDCOrCDF,
                           osDefinition, osDocumentation,
                           NULL, wkbUnknown );
             }
@@ -370,7 +379,7 @@ int OGROpenFileGDBDataSource::OpenFileGDBv10(int iGDBItems,
     }
 
     if( m_apoLayers.size() == 0 && nCandidateLayers > 0 &&
-        nCandidateLayers == nLayersSDC )
+        nCandidateLayers == nLayersSDCOrCDF )
         return FALSE;
 
     return TRUE;
@@ -403,9 +412,9 @@ int OGROpenFileGDBDataSource::OpenFileGDBv9(int iGDBFeatureClasses,
                     "Wrong structure for GDB_ObjectClasses table");
         return FALSE;
     }
-    
+
     std::vector< std::string > aosName;
-    int nCandidateLayers = 0, nLayersSDC = 0;
+    int nCandidateLayers = 0, nLayersSDCOrCDF = 0;
     for(i=0;i<oTable.GetTotalRecordCount();i++)
     {
         if( !oTable.SelectRow(i) )
@@ -427,7 +436,7 @@ int OGROpenFileGDBDataSource::OpenFileGDBv9(int iGDBFeatureClasses,
                 if( strcmp(psField->String, "{7A566981-C114-11D2-8A28-006097AFF44E}") == 0 )
                 {
                     aosName.push_back( "" );
-                    AddLayer( osName, nInterestTable, nCandidateLayers, nLayersSDC,
+                    AddLayer( osName, nInterestTable, nCandidateLayers, nLayersSDCOrCDF,
                               "", "", NULL, wkbNone );
                 }
                 else
@@ -472,7 +481,7 @@ int OGROpenFileGDBDataSource::OpenFileGDBv9(int iGDBFeatureClasses,
         psField = oTable.GetFieldValue(iGeometryType);
         if( psField == NULL )
             continue;
-        int nGeomType = psField->Integer;
+        const int nGeomType = psField->Integer;
         OGRwkbGeometryType eGeomType = wkbUnknown;
         switch( nGeomType )
         {
@@ -494,17 +503,17 @@ int OGROpenFileGDBDataSource::OpenFileGDBv9(int iGDBFeatureClasses,
             continue;
 
         int idx = psField->Integer;
-        if( psField != NULL && idx > 0 && idx <= (int)aosName.size() &&
+        if( idx > 0 && idx <= static_cast<int>(aosName.size()) &&
             aosName[idx-1].size() > 0 )
         {
             const std::string osName(aosName[idx-1]);
-            AddLayer( osName, nInterestTable, nCandidateLayers, nLayersSDC,
+            AddLayer( osName, nInterestTable, nCandidateLayers, nLayersSDCOrCDF,
                       "", "", osGeomFieldName.c_str(), eGeomType);
         }
     }
 
     if( m_apoLayers.size() == 0 && nCandidateLayers > 0 &&
-        nCandidateLayers == nLayersSDC )
+        nCandidateLayers == nLayersSDCOrCDF )
         return FALSE;
 
     return TRUE;
@@ -514,9 +523,8 @@ int OGROpenFileGDBDataSource::OpenFileGDBv9(int iGDBFeatureClasses,
 /*                         TestCapability()                            */
 /***********************************************************************/
 
-int OGROpenFileGDBDataSource::TestCapability( const char * pszCap )
+int OGROpenFileGDBDataSource::TestCapability( const char * /* pszCap */ )
 {
-    (void)pszCap;
     return FALSE;
 }
 
@@ -594,15 +602,16 @@ class OGROpenFileGDBSingleFeatureLayer : public OGRLayer
 /************************************************************************/
 
 OGROpenFileGDBSingleFeatureLayer::OGROpenFileGDBSingleFeatureLayer(const char* pszLayerName,
-                                                                   const char *pszVal )
+                                                                   const char *pszValIn )
 {
     poFeatureDefn = new OGRFeatureDefn( pszLayerName );
+    SetDescription( poFeatureDefn->GetName() );
     poFeatureDefn->Reference();
     OGRFieldDefn oField( "FIELD_1", OFTString );
     poFeatureDefn->AddFieldDefn( &oField );
 
     iNextShapeId = 0;
-    this->pszVal = pszVal ? CPLStrdup(pszVal) : NULL;
+    this->pszVal = pszValIn ? CPLStrdup(pszValIn) : NULL;
 }
 
 /************************************************************************/
@@ -652,13 +661,15 @@ class OGROpenFileGDBSimpleSQLLayer: public OGRLayer
 
        virtual void        ResetReading();
        virtual OGRFeature* GetNextFeature();
-       virtual OGRFeature* GetFeature( long nFeatureId );
+       virtual OGRFeature* GetFeature( GIntBig nFeatureId );
        virtual OGRFeatureDefn* GetLayerDefn() { return poFeatureDefn; }
        virtual int         TestCapability( const char * );
        virtual const char* GetFIDColumn() { return poBaseLayer->GetFIDColumn(); }
        virtual OGRErr      GetExtent( OGREnvelope *psExtent, int bForce )
                             { return poBaseLayer->GetExtent(psExtent, bForce); }
-       virtual int         GetFeatureCount(int bForce);
+       virtual OGRErr      GetExtent(int iGeomField, OGREnvelope *psExtent, int bForce)
+                { return OGRLayer::GetExtent(iGeomField, psExtent, bForce); }
+       virtual GIntBig     GetFeatureCount(int bForce);
 };
 
 /***********************************************************************/
@@ -666,11 +677,11 @@ class OGROpenFileGDBSimpleSQLLayer: public OGRLayer
 /***********************************************************************/
 
 OGROpenFileGDBSimpleSQLLayer::OGROpenFileGDBSimpleSQLLayer(
-                                            OGRLayer* poBaseLayer,
-                                            FileGDBIterator* poIter,
+                                            OGRLayer* poBaseLayerIn,
+                                            FileGDBIterator* poIterIn,
                                             int nColumns,
                                             swq_col_def* pasColDefs) :
-        poBaseLayer(poBaseLayer), poIter(poIter)
+        poBaseLayer(poBaseLayerIn), poIter(poIterIn)
 {
     if( nColumns == 1 && strcmp(pasColDefs[0].field_name, "*") == 0 )
     {
@@ -703,6 +714,7 @@ OGROpenFileGDBSimpleSQLLayer::OGROpenFileGDBSimpleSQLLayer(
             }
         }
     }
+    SetDescription( poFeatureDefn->GetName() );
     ResetReading();
 }
 
@@ -732,7 +744,7 @@ void OGROpenFileGDBSimpleSQLLayer::ResetReading()
 /*                          GetFeature()                               */
 /***********************************************************************/
 
-OGRFeature* OGROpenFileGDBSimpleSQLLayer::GetFeature( long nFeatureId )
+OGRFeature* OGROpenFileGDBSimpleSQLLayer::GetFeature( GIntBig nFeatureId )
 {
     OGRFeature* poSrcFeature = poBaseLayer->GetFeature(nFeatureId);
     if( poSrcFeature == NULL )
@@ -756,7 +768,7 @@ OGRFeature* OGROpenFileGDBSimpleSQLLayer::GetFeature( long nFeatureId )
 
 OGRFeature* OGROpenFileGDBSimpleSQLLayer::GetNextFeature()
 {
-    while(TRUE)
+    while( true )
     {
         int nRow = poIter->GetNextRowSortedByValue();
         if( nRow < 0 )
@@ -781,7 +793,7 @@ OGRFeature* OGROpenFileGDBSimpleSQLLayer::GetNextFeature()
 /*                         GetFeatureCount()                           */
 /***********************************************************************/
 
-int OGROpenFileGDBSimpleSQLLayer::GetFeatureCount( int bForce )
+GIntBig OGROpenFileGDBSimpleSQLLayer::GetFeatureCount( int bForce )
 {
 
     /* No filter */
@@ -832,69 +844,72 @@ OGRLayer* OGROpenFileGDBDataSource::ExecuteSQL( const char *pszSQLCommand,
 /* -------------------------------------------------------------------- */
 /*      Special case GetLayerDefinition                                 */
 /* -------------------------------------------------------------------- */
-    if (EQUALN(pszSQLCommand, "GetLayerDefinition ", strlen("GetLayerDefinition ")))
+    if (STARTS_WITH_CI(pszSQLCommand, "GetLayerDefinition "))
     {
-        OGROpenFileGDBLayer* poLayer = (OGROpenFileGDBLayer*)
-            GetLayerByName(pszSQLCommand + strlen("GetLayerDefinition "));
+        OGROpenFileGDBLayer* poLayer = reinterpret_cast<OGROpenFileGDBLayer *>(
+            GetLayerByName(pszSQLCommand + strlen("GetLayerDefinition ")) );
         if (poLayer)
         {
             OGRLayer* poRet = new OGROpenFileGDBSingleFeatureLayer(
                 "LayerDefinition", poLayer->GetXMLDefinition().c_str() );
             return poRet;
         }
-        else
-            return NULL;
+
+        return NULL;
     }
 
 /* -------------------------------------------------------------------- */
 /*      Special case GetLayerMetadata                                   */
 /* -------------------------------------------------------------------- */
-    if (EQUALN(pszSQLCommand, "GetLayerMetadata ", strlen("GetLayerMetadata ")))
+    if (STARTS_WITH_CI(pszSQLCommand, "GetLayerMetadata "))
     {
-        OGROpenFileGDBLayer* poLayer = (OGROpenFileGDBLayer*)
-            GetLayerByName(pszSQLCommand + strlen("GetLayerMetadata "));
+        OGROpenFileGDBLayer* poLayer = reinterpret_cast<OGROpenFileGDBLayer *>(
+            GetLayerByName(pszSQLCommand + strlen("GetLayerMetadata ")) );
         if (poLayer)
         {
             OGRLayer* poRet = new OGROpenFileGDBSingleFeatureLayer(
                 "LayerMetadata", poLayer->GetXMLDocumentation().c_str() );
             return poRet;
         }
-        else
-            return NULL;
+
+        return NULL;
     }
 
 /* -------------------------------------------------------------------- */
 /*      Special case GetLayerAttrIndexUse (only for debugging purposes) */
 /* -------------------------------------------------------------------- */
-    if (EQUALN(pszSQLCommand, "GetLayerAttrIndexUse ", strlen("GetLayerAttrIndexUse ")))
+    if (STARTS_WITH_CI(pszSQLCommand, "GetLayerAttrIndexUse "))
     {
-        OGROpenFileGDBLayer* poLayer = (OGROpenFileGDBLayer*)
-            GetLayerByName(pszSQLCommand + strlen("GetLayerAttrIndexUse "));
+        OGROpenFileGDBLayer* poLayer = reinterpret_cast<OGROpenFileGDBLayer *>(
+            GetLayerByName(pszSQLCommand + strlen("GetLayerAttrIndexUse ")) );
         if (poLayer)
         {
             OGRLayer* poRet = new OGROpenFileGDBSingleFeatureLayer(
-                "LayerAttrIndexUse", CPLSPrintf("%d", poLayer->GetAttrIndexUse()) );
+                "LayerAttrIndexUse",
+                CPLSPrintf("%d", poLayer->GetAttrIndexUse()) );
             return poRet;
         }
-        else
-            return NULL;
+
+        return NULL;
     }
 
 /* -------------------------------------------------------------------- */
 /*      Special case GetLayerSpatialIndexState (only for debugging purposes) */
 /* -------------------------------------------------------------------- */
-    if (EQUALN(pszSQLCommand, "GetLayerSpatialIndexState ", strlen("GetLayerSpatialIndexState ")))
+    if (STARTS_WITH_CI(pszSQLCommand, "GetLayerSpatialIndexState "))
     {
-        OGROpenFileGDBLayer* poLayer = (OGROpenFileGDBLayer*)
-            GetLayerByName(pszSQLCommand + strlen("GetLayerSpatialIndexState "));
+        OGROpenFileGDBLayer* poLayer = reinterpret_cast<OGROpenFileGDBLayer*>(
+            GetLayerByName(pszSQLCommand +
+                           strlen("GetLayerSpatialIndexState ")) );
         if (poLayer)
         {
             OGRLayer* poRet = new OGROpenFileGDBSingleFeatureLayer(
-                "LayerSpatialIndexState", CPLSPrintf("%d", poLayer->GetSpatialIndexState()) );
+                "LayerSpatialIndexState",
+                CPLSPrintf("%d", poLayer->GetSpatialIndexState()) );
             return poRet;
         }
-        else
-            return NULL;
+
+        return NULL;
     }
 
 /* -------------------------------------------------------------------- */
@@ -904,31 +919,35 @@ OGRLayer* OGROpenFileGDBDataSource::ExecuteSQL( const char *pszSQLCommand,
     {
         OGRLayer* poRet = new OGROpenFileGDBSingleFeatureLayer(
             "GetLastSQLUsedOptimizedImplementation",
-                    CPLSPrintf("%d", bLastSQLUsedOptimizedImplementation) );
+            CPLSPrintf(
+                "%d", static_cast<int>(bLastSQLUsedOptimizedImplementation))) ;
         return poRet;
     }
 
-    bLastSQLUsedOptimizedImplementation = FALSE;
+    bLastSQLUsedOptimizedImplementation = false;
 
 /* -------------------------------------------------------------------- */
 /*      Special cases for SQL optimizations                             */
 /* -------------------------------------------------------------------- */
-    if( EQUALN(pszSQLCommand, "SELECT ", strlen("SELECT ")) &&
-        (pszDialect == NULL || EQUAL(pszDialect, "") || EQUAL(pszDialect, "OGRSQL")) &&
-        CSLTestBoolean(CPLGetConfigOption("OPENFILEGDB_USE_INDEX", "YES")) )
+    if( STARTS_WITH_CI(pszSQLCommand, "SELECT ") &&
+        (pszDialect == NULL || EQUAL(pszDialect, "") ||
+         EQUAL(pszDialect, "OGRSQL")) &&
+        CPLTestBool(CPLGetConfigOption("OPENFILEGDB_USE_INDEX", "YES")) )
     {
         swq_select oSelect;
-        if( oSelect.preparse(pszSQLCommand) != OGRERR_NONE )
+        if( oSelect.preparse(pszSQLCommand) != CE_None )
             return NULL;
 
 /* -------------------------------------------------------------------- */
 /*      MIN/MAX/SUM/AVG/COUNT optimization                              */
 /* -------------------------------------------------------------------- */
         if( oSelect.join_count == 0 && oSelect.poOtherSelect == NULL &&
-            oSelect.table_count == 1 && oSelect.order_specs == 0 )
+            oSelect.table_count == 1 && oSelect.order_specs == 0 &&
+            oSelect.query_mode != SWQM_DISTINCT_LIST )
         {
-            OGROpenFileGDBLayer* poLayer = 
-                (OGROpenFileGDBLayer*)GetLayerByName( oSelect.table_defs[0].table_name);
+            OGROpenFileGDBLayer* poLayer =
+                reinterpret_cast<OGROpenFileGDBLayer *>(
+                    GetLayerByName( oSelect.table_defs[0].table_name));
             if( poLayer )
             {
                 OGRMemLayer* poMemLayer = NULL;
@@ -960,8 +979,6 @@ OGRLayer* OGROpenFileGDBDataSource::ExecuteSQL( const char *pszSQLCommand,
                         break;
 
                     int eOutOGRType = -1;
-                    int nCount = 0;
-                    double dfSum = 0.0;
                     const OGRField* psField = NULL;
                     OGRField sField;
                     if( col_func == SWQCF_MIN || col_func == SWQCF_MAX )
@@ -973,8 +990,13 @@ OGRLayer* OGROpenFileGDBDataSource::ExecuteSQL( const char *pszSQLCommand,
                     }
                     else
                     {
-                        double dfMin = 0.0, dfMax = 0.0;
-                        if( !poLayer->GetMinMaxSumCount(poFieldDefn, dfMin, dfMax,
+                        double dfMin = 0.0;
+                        double dfMax = 0.0;
+                        int nCount = 0;
+                        double dfSum = 0.0;
+
+                        if( !poLayer->GetMinMaxSumCount(poFieldDefn,
+                                                        dfMin, dfMax,
                                                         dfSum, nCount) )
                             break;
                         psField = &sField;
@@ -1015,7 +1037,7 @@ OGRLayer* OGROpenFileGDBDataSource::ExecuteSQL( const char *pszSQLCommand,
                     {
                         poMemLayer = new OGRMemLayer("SELECT", NULL, wkbNone);
                         OGRFeature* poFeature = new OGRFeature(poMemLayer->GetLayerDefn());
-                        poMemLayer->CreateFeature(poFeature);
+                        CPL_IGNORE_RET_VAL(poMemLayer->CreateFeature(poFeature));
                         delete poFeature;
                     }
 
@@ -1033,7 +1055,7 @@ OGRLayer* OGROpenFileGDBDataSource::ExecuteSQL( const char *pszSQLCommand,
                     {
                         OGRFeature* poFeature = poMemLayer->GetFeature(0);
                         poFeature->SetField(oFieldDefn.GetNameRef(), (OGRField*) psField);
-                        poMemLayer->SetFeature(poFeature);
+                        CPL_IGNORE_RET_VAL(poMemLayer->SetFeature(poFeature));
                         delete poFeature;
                     }
                 }
@@ -1045,7 +1067,7 @@ OGRLayer* OGROpenFileGDBDataSource::ExecuteSQL( const char *pszSQLCommand,
                 {
                     CPLDebug("OpenFileGDB",
                         "Using optimized MIN/MAX/SUM/AVG/COUNT implementation");
-                    bLastSQLUsedOptimizedImplementation = TRUE;
+                    bLastSQLUsedOptimizedImplementation = true;
                     return poMemLayer;
                 }
             }
@@ -1055,10 +1077,12 @@ OGRLayer* OGROpenFileGDBDataSource::ExecuteSQL( const char *pszSQLCommand,
 /*      ORDER BY optimization                                           */
 /* -------------------------------------------------------------------- */
         if( oSelect.join_count == 0 && oSelect.poOtherSelect == NULL &&
-            oSelect.table_count == 1 && oSelect.order_specs == 1 )
+            oSelect.table_count == 1 && oSelect.order_specs == 1 &&
+            oSelect.query_mode != SWQM_DISTINCT_LIST )
         {
-            OGROpenFileGDBLayer* poLayer = 
-                (OGROpenFileGDBLayer*)GetLayerByName( oSelect.table_defs[0].table_name);
+            OGROpenFileGDBLayer* poLayer =
+                reinterpret_cast<OGROpenFileGDBLayer *>(
+                    GetLayerByName( oSelect.table_defs[0].table_name) );
             if( poLayer != NULL &&
                 poLayer->HasIndexForField(oSelect.order_defs[0].field_name) )
             {
@@ -1106,11 +1130,11 @@ OGRLayer* OGROpenFileGDBDataSource::ExecuteSQL( const char *pszSQLCommand,
                 }
                 if( eErr == OGRERR_NONE )
                 {
-                    swq_op op = SWQ_UNKNOWN;
+                    int op = -1;
                     swq_expr_node* poValue = NULL;
                     if( oSelect.where_expr != NULL )
                     {
-                        op = (swq_op)oSelect.where_expr->nOperation;
+                        op = oSelect.where_expr->nOperation;
                         poValue = oSelect.where_expr->papoSubExpr[1];
                     }
 
@@ -1130,7 +1154,7 @@ OGRLayer* OGROpenFileGDBDataSource::ExecuteSQL( const char *pszSQLCommand,
                     if( poIter != NULL )
                     {
                         CPLDebug("OpenFileGDB", "Using OGROpenFileGDBSimpleSQLLayer");
-                        bLastSQLUsedOptimizedImplementation = TRUE;
+                        bLastSQLUsedOptimizedImplementation = true;
                         return new OGROpenFileGDBSimpleSQLLayer(poLayer,
                                                                 poIter,
                                                                 oSelect.result_columns,
@@ -1152,3 +1176,36 @@ void OGROpenFileGDBDataSource::ReleaseResultSet( OGRLayer * poResultsSet )
 {
     delete poResultsSet;
 }
+
+/***********************************************************************/
+/*                           GetFileList()                             */
+/***********************************************************************/
+
+char** OGROpenFileGDBDataSource::GetFileList()
+{
+    int nInterestTable = -1;
+    const char* pszFilenameWithoutPath = CPLGetFilename(m_pszName);
+    CPLString osFilenameRadix;
+    if( strlen(pszFilenameWithoutPath) == strlen("a00000000.gdbtable") &&
+        pszFilenameWithoutPath[0] == 'a' &&
+        sscanf(pszFilenameWithoutPath, "a%08x.gdbtable", &nInterestTable) == 1 )
+    {
+        osFilenameRadix = CPLSPrintf("a%08x.", nInterestTable);
+    }
+
+    char** papszFiles = VSIReadDir(m_osDirName);
+    CPLStringList osStringList;
+    char** papszIter = papszFiles;
+    for( ; papszIter != NULL && *papszIter != NULL ; papszIter ++ )
+    {
+        if( strcmp(*papszIter, ".") == 0 || strcmp(*papszIter, "..") == 0 )
+            continue;
+        if( osFilenameRadix.size() == 0 ||
+            strncmp(*papszIter, osFilenameRadix, osFilenameRadix.size()) == 0 )
+        {
+            osStringList.AddString(CPLFormFilename(m_osDirName, *papszIter, NULL));
+        }
+    }
+    CSLDestroy(papszFiles);
+    return osStringList.StealList();
+}
