@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrpdslayer.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogrpdslayer.cpp 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  PDS Translator
  * Purpose:  Implements OGRPDSLayer class.
@@ -32,37 +32,37 @@
 #include "cpl_string.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogrpdslayer.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: ogrpdslayer.cpp 33714 2016-03-13 05:42:13Z goatbar $");
+
+namespace OGRPDS {
 
 /************************************************************************/
 /*                           OGRPDSLayer()                              */
 /************************************************************************/
 
-OGRPDSLayer::OGRPDSLayer(   CPLString osTableID,
+OGRPDSLayer::OGRPDSLayer(   CPLString osTableIDIn,
                             const char* pszLayerName, VSILFILE* fp,
                             CPLString osLabelFilename,
                             CPLString osStructureFilename,
-                            int nRecords,
-                            int nStartBytes, int nRecordSize,
-                            GByte* pabyRecordIn, int bIsASCII)
-
+                            int nRecordsIn,
+                            int nStartBytesIn, int nRecordSizeIn,
+                            GByte* pabyRecordIn, bool bIsASCII) :
+    poFeatureDefn(new OGRFeatureDefn( pszLayerName )),
+    osTableID(osTableIDIn),
+    fpPDS(fp),
+    nRecords(nRecordsIn),
+    nStartBytes(nStartBytesIn),
+    nRecordSize(nRecordSizeIn),
+    pabyRecord(pabyRecordIn),
+    nNextFID(0),
+    nLongitudeIndex(-1),
+    nLatitudeIndex(-1),
+    pasFieldDesc(NULL)
 {
-    fpPDS = fp;
-    this->osTableID = osTableID;
-    this->nRecords = nRecords;
-    this->nStartBytes = nStartBytes;
-    this->nRecordSize = nRecordSize;
-    nLongitudeIndex = -1;
-    nLatitudeIndex = -1;
-
-    poFeatureDefn = new OGRFeatureDefn( pszLayerName );
+    SetDescription( poFeatureDefn->GetName() );
     poFeatureDefn->Reference();
     poFeatureDefn->SetGeomType( wkbNone );
 
-    pasFieldDesc = NULL;
-
-    pabyRecord = pabyRecordIn;
-
     if (osStructureFilename.size() != 0)
     {
         ReadStructure(osStructureFilename);
@@ -80,9 +80,8 @@ OGRPDSLayer::OGRPDSLayer(   CPLString osTableID,
 
         char **papszTokens = CSLTokenizeString2(
                 (const char*)pabyRecord, " ", CSLT_HONOURSTRINGS );
-        int nTokens = CSLCount(papszTokens);
-        int i;
-        for(i=0;i<nTokens;i++)
+        const int nTokens = CSLCount(papszTokens);
+        for( int i=0; i<nTokens; i++)
         {
             const char* pszStr = papszTokens[i];
             char ch;
@@ -104,7 +103,7 @@ OGRPDSLayer::OGRPDSLayer(   CPLString osTableID,
                 pszStr ++;
             }
             char szFieldName[32];
-            sprintf(szFieldName, "field_%d",
+            snprintf(szFieldName, sizeof(szFieldName), "field_%d",
                     poFeatureDefn->GetFieldCount() + 1);
             OGRFieldDefn oFieldDefn(szFieldName, eFieldType);
             poFeatureDefn->AddFieldDefn(&oFieldDefn);
@@ -142,22 +141,26 @@ OGRPDSLayer::~OGRPDSLayer()
 void OGRPDSLayer::ReadStructure(CPLString osStructureFilename)
 
 {
-    int nFields = 0;
     VSILFILE* fpStructure = VSIFOpenL(osStructureFilename, "rb");
     if (fpStructure == NULL)
         return;
 
-    const char* pszLine;
-    int bInObjectColumn = FALSE;
+    int nFields = 0;
+    bool bInObjectColumn = false;
     int nExpectedColumnNumber = 0;
-    CPLString osColumnName, osColumnDataType, osColumnStartByte,
-              osColumnBytes, osColumnFormat, osColumnUnit,
-              osColumnItems, osColumnItemBytes;
+    CPLString osColumnName;
+    CPLString osColumnDataType;
+    CPLString osColumnStartByte;
+    CPLString osColumnBytes;
+    CPLString osColumnFormat;
+    CPLString osColumnUnit;
+    CPLString osColumnItems;
+    CPLString osColumnItemBytes;
     int nRowBytes = nRecordSize;
-    while(TRUE)
+    while( true )
     {
         CPLPushErrorHandler(CPLQuietErrorHandler);
-        pszLine = CPLReadLine2L(fpStructure, 256, NULL);
+        const char* pszLine = CPLReadLine2L(fpStructure, 256, NULL);
         CPLPopErrorHandler();
         CPLErrorReset();
         if (pszLine == NULL)
@@ -165,7 +168,7 @@ void OGRPDSLayer::ReadStructure(CPLString osStructureFilename)
 
         char **papszTokens =
                 CSLTokenizeString2( pszLine, " =", CSLT_HONOURSTRINGS );
-        int nTokens = CSLCount(papszTokens);
+        const int nTokens = CSLCount(papszTokens);
 
         if (bInObjectColumn && nTokens >= 1 &&
             EQUAL(papszTokens[0], "END_OBJECT"))
@@ -176,11 +179,12 @@ void OGRPDSLayer::ReadStructure(CPLString osStructureFilename)
                 osColumnBytes.size() != 0)
             {
                 pasFieldDesc =
-                        (FieldDesc*) CPLRealloc(pasFieldDesc,
-                                (nFields + 1) * sizeof(FieldDesc));
+                    static_cast<FieldDesc*>(
+                        CPLRealloc( pasFieldDesc,
+                                    (nFields + 1) * sizeof(FieldDesc)) );
                 pasFieldDesc[nFields].nStartByte = atoi(osColumnStartByte) - 1;
                 pasFieldDesc[nFields].nByteCount = atoi(osColumnBytes);
-                if (pasFieldDesc[nFields].nStartByte >= 0 && 
+                if (pasFieldDesc[nFields].nStartByte >= 0 &&
                     pasFieldDesc[nFields].nByteCount > 0 &&
                     pasFieldDesc[nFields].nStartByte +
                     pasFieldDesc[nFields].nByteCount <= nRecordSize)
@@ -193,7 +197,8 @@ void OGRPDSLayer::ReadStructure(CPLString osStructureFilename)
                         pasFieldDesc[nFields].nItems = 1;
                     if (pasFieldDesc[nFields].nItemBytes == 0 &&
                         pasFieldDesc[nFields].nItems == 1)
-                        pasFieldDesc[nFields].nItemBytes = pasFieldDesc[nFields].nByteCount;
+                        pasFieldDesc[nFields].nItemBytes =
+                            pasFieldDesc[nFields].nByteCount;
 
                     if (osColumnDataType.compare("ASCII_REAL") == 0)
                     {
@@ -205,7 +210,8 @@ void OGRPDSLayer::ReadStructure(CPLString osStructureFilename)
                         eFieldType = OFTInteger;
                         pasFieldDesc[nFields].eFormat = ASCII_INTEGER;
                     }
-                    else if (osColumnDataType.compare("MSB_UNSIGNED_INTEGER") == 0)
+                    else if (osColumnDataType.compare("MSB_UNSIGNED_INTEGER") ==
+                             0)
                     {
                         if (pasFieldDesc[nFields].nItemBytes == 1 ||
                             pasFieldDesc[nFields].nItemBytes == 2)
@@ -301,7 +307,7 @@ void OGRPDSLayer::ReadStructure(CPLString osStructureFilename)
                 CSLDestroy(papszTokens);
                 break;
             }
-            bInObjectColumn = FALSE;
+            bInObjectColumn = false;
         }
         else if (nTokens == 2)
         {
@@ -309,7 +315,7 @@ void OGRPDSLayer::ReadStructure(CPLString osStructureFilename)
             {
                 CSLDestroy(papszTokens);
                 papszTokens = NULL;
-                while(TRUE)
+                while( true )
                 {
                     CPLPushErrorHandler(CPLQuietErrorHandler);
                     pszLine = CPLReadLine2L(fpStructure, 256, NULL);
@@ -319,8 +325,8 @@ void OGRPDSLayer::ReadStructure(CPLString osStructureFilename)
                         break;
                     papszTokens =
                         CSLTokenizeString2( pszLine, " =", CSLT_HONOURSTRINGS );
-                    int nTokens = CSLCount(papszTokens);
-                    if (nTokens == 2 &&
+                    int nTokens2 = CSLCount(papszTokens);
+                    if (nTokens2 == 2 &&
                         EQUAL(papszTokens[0], "OBJECT") &&
                         EQUAL(papszTokens[1], osTableID.c_str()))
                     {
@@ -356,7 +362,7 @@ void OGRPDSLayer::ReadStructure(CPLString osStructureFilename)
                     nRecordSize = nRowBytes;
 
                 nExpectedColumnNumber ++;
-                bInObjectColumn = TRUE;
+                bInObjectColumn = true;
                 osColumnName = "";
                 osColumnDataType = "";
                 osColumnStartByte = "";
@@ -438,11 +444,9 @@ void OGRPDSLayer::ResetReading()
 
 OGRFeature *OGRPDSLayer::GetNextFeature()
 {
-    OGRFeature  *poFeature;
-
-    while(TRUE)
+    while( true )
     {
-        poFeature = GetNextRawFeature();
+        OGRFeature *poFeature = GetNextRawFeature();
         if (poFeature == NULL)
             return NULL;
 
@@ -453,8 +457,8 @@ OGRFeature *OGRPDSLayer::GetNextFeature()
         {
             return poFeature;
         }
-        else
-            delete poFeature;
+
+        delete poFeature;
     }
 }
 
@@ -475,15 +479,16 @@ OGRFeature *OGRPDSLayer::GetNextRawFeature()
     int nFieldCount = poFeatureDefn->GetFieldCount();
     if (pasFieldDesc != NULL)
     {
-        int i, j;
-        for(i=0;i<nFieldCount;i++)
+        int j;
+        for( int i=0;i<nFieldCount;i++)
         {
             if (pasFieldDesc[i].eFormat == ASCII_REAL ||
                 pasFieldDesc[i].eFormat == ASCII_INTEGER ||
                 pasFieldDesc[i].eFormat == CHARACTER)
             {
-                char* pchEnd = (char*) &pabyRecord[pasFieldDesc[i].nStartByte +
-                                                pasFieldDesc[i].nByteCount];
+                char* pchEnd = reinterpret_cast<char*>(
+                    &pabyRecord[pasFieldDesc[i].nStartByte +
+                                pasFieldDesc[i].nByteCount]);
                 char chSaved = *pchEnd;
                 *pchEnd = 0;
                 poFeature->SetField(i, (const char*)(pabyRecord +
@@ -498,7 +503,8 @@ OGRFeature *OGRPDSLayer::GetNextRawFeature()
                 {
                     if (pasFieldDesc[i].nItems > 1)
                     {
-                        int* panValues = (int*)CPLMalloc(sizeof(int) * pasFieldDesc[i].nItems);
+                        int* panValues = static_cast<int *>(
+                            CPLMalloc(sizeof(int) * pasFieldDesc[i].nItems) );
                         for(j=0;j<pasFieldDesc[i].nItems;j++)
                         {
                             panValues[j] = pabyRecord[pasFieldDesc[i].nStartByte + j];
@@ -688,10 +694,12 @@ int OGRPDSLayer::TestCapability( const char * pszCap )
     if (EQUAL(pszCap,OLCFastFeatureCount) &&
         m_poFilterGeom == NULL && m_poAttrQuery == NULL)
         return TRUE;
-    else if (EQUAL(pszCap,OLCRandomRead))
+
+    if (EQUAL(pszCap,OLCRandomRead))
         return TRUE;
-    else if (EQUAL(pszCap,OLCFastSetNextByIndex) &&
-             m_poFilterGeom == NULL && m_poAttrQuery == NULL)
+
+    if (EQUAL(pszCap,OLCFastSetNextByIndex) &&
+        m_poFilterGeom == NULL && m_poAttrQuery == NULL)
         return TRUE;
 
     return FALSE;
@@ -701,7 +709,7 @@ int OGRPDSLayer::TestCapability( const char * pszCap )
 /*                          GetFeatureCount()                           */
 /************************************************************************/
 
-int OGRPDSLayer::GetFeatureCount(int bForce )
+GIntBig OGRPDSLayer::GetFeatureCount(int bForce )
 {
     if (TestCapability(OLCFastFeatureCount))
         return nRecords;
@@ -713,12 +721,12 @@ int OGRPDSLayer::GetFeatureCount(int bForce )
 /*                             GetFeature()                             */
 /************************************************************************/
 
-OGRFeature *OGRPDSLayer::GetFeature( long nFID )
+OGRFeature *OGRPDSLayer::GetFeature( GIntBig nFID )
 {
     if (nFID < 0 || nFID >= nRecords)
         return NULL;
 
-    nNextFID = nFID;
+    nNextFID = (int)nFID;
     VSIFSeekL( fpPDS, nStartBytes + nNextFID * nRecordSize, SEEK_SET );
     return GetNextRawFeature();
 }
@@ -727,7 +735,7 @@ OGRFeature *OGRPDSLayer::GetFeature( long nFID )
 /*                         SetNextByIndex()                             */
 /************************************************************************/
 
-OGRErr OGRPDSLayer::SetNextByIndex( long nIndex )
+OGRErr OGRPDSLayer::SetNextByIndex( GIntBig nIndex )
 {
     if (!TestCapability(OLCFastSetNextByIndex))
         return OGRLayer::SetNextByIndex( nIndex );
@@ -735,7 +743,9 @@ OGRErr OGRPDSLayer::SetNextByIndex( long nIndex )
     if (nIndex < 0 || nIndex >= nRecords)
         return OGRERR_FAILURE;
 
-    nNextFID = nIndex;
+    nNextFID = (int)nIndex;
     VSIFSeekL( fpPDS, nStartBytes + nNextFID * nRecordSize, SEEK_SET );
     return OGRERR_NONE;
 }
+
+} /* end of OGRPDS namespace */
