@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrpgresultlayer.cpp 28481 2015-02-13 17:11:15Z rouault $
+ * $Id: ogrpgresultlayer.cpp 33713 2016-03-12 17:41:57Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRPGResultLayer class, access the resultset from
@@ -32,7 +32,7 @@
 #include "cpl_conv.h"
 #include "ogr_pg.h"
 
-CPL_CVSID("$Id: ogrpgresultlayer.cpp 28481 2015-02-13 17:11:15Z rouault $");
+CPL_CVSID("$Id: ogrpgresultlayer.cpp 33713 2016-03-12 17:41:57Z goatbar $");
 
 #define PQexec this_is_an_error
 
@@ -40,7 +40,7 @@ CPL_CVSID("$Id: ogrpgresultlayer.cpp 28481 2015-02-13 17:11:15Z rouault $");
 /*                          OGRPGResultLayer()                          */
 /************************************************************************/
 
-OGRPGResultLayer::OGRPGResultLayer( OGRPGDataSource *poDSIn, 
+OGRPGResultLayer::OGRPGResultLayer( OGRPGDataSource *poDSIn,
                                     const char * pszRawQueryIn,
                                     PGresult *hInitialResultIn )
 {
@@ -65,8 +65,7 @@ OGRPGResultLayer::OGRPGResultLayer( OGRPGDataSource *poDSIn,
     CPLString osRequest;
     std::map< std::pair<int,int>, int> oMapAttributeToFieldIndex;
 
-    int iRawField;
-    for( iRawField = 0; iRawField < PQnfields(hInitialResultIn); iRawField++ )
+    for( int iRawField = 0; iRawField < PQnfields(hInitialResultIn); iRawField++ )
     {
         if( poFeatureDefn->GetGeomFieldCount() == 1 &&
             strcmp(PQfname(hInitialResultIn,iRawField),
@@ -82,9 +81,9 @@ OGRPGResultLayer::OGRPGResultLayer( OGRPGDataSource *poDSIn,
             if( osRequest.size() )
                 osRequest += " OR ";
             osRequest += "(attrelid = ";
-            osRequest += CPLSPrintf("%d", tableOID); 
+            osRequest += CPLSPrintf("%d", tableOID);
             osRequest += " AND attnum = ";
-            osRequest += CPLSPrintf("%d)", tableCol); 
+            osRequest += CPLSPrintf("%d)", tableCol);
             oMapAttributeToFieldIndex[std::pair<int,int>(tableOID,tableCol)] = iRawField;
         }
     }
@@ -167,12 +166,13 @@ void OGRPGResultLayer::BuildFullQueryStatement()
         pszQueryStatement = NULL;
     }
 
-    pszQueryStatement = (char*) CPLMalloc(strlen(pszRawStatement) + strlen(osWHERE) + 40);
+    const size_t nLen = strlen(pszRawStatement) + strlen(osWHERE) + 40;
+    pszQueryStatement = (char*) CPLMalloc(nLen);
 
     if (strlen(osWHERE) == 0)
         strcpy(pszQueryStatement, pszRawStatement);
     else
-        sprintf(pszQueryStatement, "SELECT * FROM (%s) AS ogrpgsubquery %s",
+        snprintf(pszQueryStatement, nLen, "SELECT * FROM (%s) AS ogrpgsubquery %s",
                 pszRawStatement, osWHERE.c_str());
 }
 
@@ -224,7 +224,7 @@ int OGRPGResultLayer::TestCapability( const char * pszCap )
 
 {
     GetLayerDefn();
-    
+
     if( EQUAL(pszCap,OLCFastFeatureCount) ||
         EQUAL(pszCap,OLCFastSetNextByIndex) )
     {
@@ -234,7 +234,7 @@ int OGRPGResultLayer::TestCapability( const char * pszCap )
         return (m_poFilterGeom == NULL ||
                 poGeomFieldDefn == NULL ||
                 poGeomFieldDefn->ePostgisType == GEOM_TYPE_GEOMETRY ||
-                poGeomFieldDefn->ePostgisType == GEOM_TYPE_GEOGRAPHY ) 
+                poGeomFieldDefn->ePostgisType == GEOM_TYPE_GEOGRAPHY )
                && m_poAttrQuery == NULL;
     }
     else if( EQUAL(pszCap,OLCFastSpatialFilter) )
@@ -244,7 +244,7 @@ int OGRPGResultLayer::TestCapability( const char * pszCap )
             poGeomFieldDefn = poFeatureDefn->myGetGeomFieldDefn(m_iGeomFieldFilter);
         return (poGeomFieldDefn == NULL ||
                 poGeomFieldDefn->ePostgisType == GEOM_TYPE_GEOMETRY ||
-                poGeomFieldDefn->ePostgisType == GEOM_TYPE_GEOGRAPHY ) 
+                poGeomFieldDefn->ePostgisType == GEOM_TYPE_GEOGRAPHY )
                && m_poAttrQuery == NULL;
     }
 
@@ -254,7 +254,7 @@ int OGRPGResultLayer::TestCapability( const char * pszCap )
         if( poFeatureDefn->GetGeomFieldCount() > 0 )
             poGeomFieldDefn = poFeatureDefn->myGetGeomFieldDefn(0);
         return (poGeomFieldDefn == NULL ||
-                poGeomFieldDefn->ePostgisType == GEOM_TYPE_GEOMETRY ) 
+                poGeomFieldDefn->ePostgisType == GEOM_TYPE_GEOMETRY )
                && m_poAttrQuery == NULL;
     }
     else if( EQUAL(pszCap,OLCStringsAsUTF8) )
@@ -276,7 +276,7 @@ OGRFeature *OGRPGResultLayer::GetNextFeature()
     if( poFeatureDefn->GetGeomFieldCount() != 0 )
         poGeomFieldDefn = poFeatureDefn->myGetGeomFieldDefn(m_iGeomFieldFilter);
 
-    for( ; TRUE; )
+    while( true )
     {
         OGRFeature      *poFeature;
 
@@ -407,6 +407,8 @@ void OGRPGResultLayer::ResolveSRID(OGRPGGeomFieldDefn* poGFldDefn)
             osGetSRID += psGetSRIDFct;
             osGetSRID += "(";
             osGetSRID += OGRPGEscapeColumnName(poGFldDefn->GetNameRef());
+            if (poDS->sPostGISVersion.nMajor > 2 || (poDS->sPostGISVersion.nMajor == 2 && poDS->sPostGISVersion.nMinor >= 2))
+                osGetSRID += "::geometry";
             osGetSRID += ") FROM(";
             osGetSRID += pszRawStatement;
             osGetSRID += ") AS ogrpggetsrid LIMIT 1";
