@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_pgeo.h 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogr_pgeo.h 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Private definitions for Personal Geodatabase driver.
@@ -28,8 +28,8 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _OGR_ODBC_H_INCLUDED
-#define _OGR_ODBC_H_INCLUDED
+#ifndef OGR_ODBC_H_INCLUDED
+#define OGR_ODBC_H_INCLUDED
 
 #include "ogrsf_frmts.h"
 #include "cpl_odbc.h"
@@ -40,7 +40,7 @@
 /************************************************************************/
 
 class OGRPGeoDataSource;
-    
+
 class OGRPGeoLayer : public OGRLayer
 {
   protected:
@@ -52,7 +52,7 @@ class OGRPGeoLayer : public OGRLayer
     OGRSpatialReference *poSRS;
     int                 nSRSId;
 
-    int                 iNextShapeId;
+    GIntBig             iNextShapeId;
 
     OGRPGeoDataSource    *poDS;
 
@@ -76,8 +76,8 @@ class OGRPGeoLayer : public OGRLayer
     virtual OGRFeature *GetNextRawFeature();
     virtual OGRFeature *GetNextFeature();
 
-    virtual OGRFeature *GetFeature( long nFeatureId );
-    
+    virtual OGRFeature *GetFeature( GIntBig nFeatureId );
+
     OGRFeatureDefn *    GetLayerDefn() { return poFeatureDefn; }
 
     virtual int         TestCapability( const char * );
@@ -107,7 +107,7 @@ class OGRPGeoTableLayer : public OGRPGeoLayer
                         OGRPGeoTableLayer( OGRPGeoDataSource * );
                         ~OGRPGeoTableLayer();
 
-    CPLErr              Initialize( const char *pszTableName, 
+    CPLErr              Initialize( const char *pszTableName,
                                     const char *pszGeomCol,
                                     int nShapeType,
                                     double dfExtentLeft,
@@ -118,14 +118,16 @@ class OGRPGeoTableLayer : public OGRPGeoLayer
                                     int bHasZ );
 
     virtual void        ResetReading();
-    virtual int         GetFeatureCount( int );
+    virtual GIntBig     GetFeatureCount( int );
 
     virtual OGRErr      SetAttributeFilter( const char * );
-    virtual OGRFeature *GetFeature( long nFeatureId );
-    
+    virtual OGRFeature *GetFeature( GIntBig nFeatureId );
+
     virtual int         TestCapability( const char * );
 
     virtual OGRErr      GetExtent(OGREnvelope *psExtent, int bForce = TRUE);
+    virtual OGRErr      GetExtent(int iGeomField, OGREnvelope *psExtent, int bForce)
+                { return OGRLayer::GetExtent(iGeomField, psExtent, bForce); }
 };
 
 /************************************************************************/
@@ -142,15 +144,15 @@ class OGRPGeoSelectLayer : public OGRPGeoLayer
     virtual CPLODBCStatement *  GetStatement();
 
   public:
-                        OGRPGeoSelectLayer( OGRPGeoDataSource *, 
+                        OGRPGeoSelectLayer( OGRPGeoDataSource *,
                                            CPLODBCStatement * );
                         ~OGRPGeoSelectLayer();
 
     virtual void        ResetReading();
-    virtual int         GetFeatureCount( int );
+    virtual GIntBig     GetFeatureCount( int );
+
+    virtual OGRFeature *GetFeature( GIntBig nFeatureId );
 
-    virtual OGRFeature *GetFeature( long nFeatureId );
-    
     virtual int         TestCapability( const char * );
 };
 
@@ -162,7 +164,7 @@ class OGRPGeoDataSource : public OGRDataSource
 {
     OGRPGeoLayer        **papoLayers;
     int                 nLayers;
-    
+
     char               *pszName;
 
     int                 bDSUpdate;
@@ -173,7 +175,7 @@ class OGRPGeoDataSource : public OGRDataSource
                         ~OGRPGeoDataSource();
 
     int                 Open( const char *, int bUpdate, int bTestOpen );
-    int                 OpenTable( const char *pszTableName, 
+    int                 OpenTable( const char *pszTableName,
                                    const char *pszGeomCol,
                                    int bUpdate );
 
@@ -219,7 +221,7 @@ class OGRPGeoDriver : public OGRODBCMDBDriver
 {
   public:
                 ~OGRPGeoDriver();
-                
+
     const char  *GetName();
     OGRDataSource *Open( const char *, int );
 
