@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrs57datasource.cpp 34384 2016-06-20 17:09:24Z rouault $
+ * $Id: ogrs57datasource.cpp 34800 2016-07-27 16:52:35Z rouault $
  *
  * Project:  S-57 Translator
  * Purpose:  Implements OGRS57DataSource class
@@ -32,7 +32,7 @@
 #include "cpl_string.h"
 #include "ogr_s57.h"
 
-CPL_CVSID("$Id: ogrs57datasource.cpp 34384 2016-06-20 17:09:24Z rouault $");
+CPL_CVSID("$Id: ogrs57datasource.cpp 34800 2016-07-27 16:52:35Z rouault $");
 
 /************************************************************************/
 /*                          OGRS57DataSource()                          */
@@ -57,18 +57,18 @@ OGRS57DataSource::OGRS57DataSource(char** papszOpenOptionsIn) :
 /* -------------------------------------------------------------------- */
     const char *pszOptString = CPLGetConfigOption( "OGR_S57_OPTIONS", NULL );
 
-    if ( pszOptString == NULL )
-        return;
-
-    papszOptions =
-        CSLTokenizeStringComplex( pszOptString, ",", FALSE, FALSE );
-
-    if ( papszOptions && *papszOptions )
+    if ( pszOptString != NULL )
     {
-        CPLDebug( "S57", "The following S57 options are being set:" );
-        char **papszCurOption = papszOptions;
-        while( *papszCurOption )
-            CPLDebug( "S57", "    %s", *papszCurOption++ );
+        papszOptions =
+            CSLTokenizeStringComplex( pszOptString, ",", FALSE, FALSE );
+
+        if ( papszOptions && *papszOptions )
+        {
+            CPLDebug( "S57", "The following S57 options are being set:" );
+            char **papszCurOption = papszOptions;
+            while( *papszCurOption )
+                CPLDebug( "S57", "    %s", *papszCurOption++ );
+        }
     }
 
 /* -------------------------------------------------------------------- */
