@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: s57.h 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: s57.h 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  S-57 Translator
  * Purpose:  Declarations for S-57 translator not including the
@@ -30,8 +30,8 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _S57_H_INCLUDED
-#define _S57_H_INCLUDED
+#ifndef S57_H_INCLUDED
+#define S57_H_INCLUDED
 
 #include <vector>
 #include "ogr_feature.h"
@@ -98,7 +98,7 @@ char **S57FileCollector( const char * pszDataset );
 
 class S57ClassContentExplorer;
 
-class CPL_DLL S57AttrInfo 
+class CPL_DLL S57AttrInfo
 {
   public:
     CPLString    osName;
@@ -135,7 +135,7 @@ public:
     // attribute table methods.
     //int         GetMaxAttrIndex() { return nAttrMax; }
     const S57AttrInfo *GetAttrInfo(int i);
-    const char *GetAttrName( int i ) 
+    const char *GetAttrName( int i )
     { return GetAttrInfo(i) == NULL ? NULL : aoAttrInfos[i]->osName.c_str(); }
     const char *GetAttrAcronym( int i )
     { return GetAttrInfo(i) == NULL ? NULL : aoAttrInfos[i]->osAcronym.c_str(); }
@@ -182,12 +182,12 @@ class S57ClassContentExplorer
     int         NextClass() { return SelectClassByIndex(iCurrentClass+1); }
 
     int         GetOBJL();
-    const char *GetDescription();
-    const char *GetAcronym();
+    const char *GetDescription() const;
+    const char *GetAcronym() const;
 
     char      **GetAttributeList( const char * = NULL );
 
-    char        GetClassCode();
+    char        GetClassCode() const;
     char      **GetPrimitives();
 };
 
@@ -206,13 +206,13 @@ typedef struct
 
 class CPL_DLL DDFRecordIndex
 {
-    int         bSorted;
+    bool        bSorted;
 
     int         nRecordCount;
     int         nRecordMax;
 
-    int         nLastObjlPos;            /* rjensen. added for FindRecordByObjl() */
-    int         nLastObjl;                  /* rjensen. added for FindRecordByObjl() */
+    int         nLastObjlPos;  // Added for FindRecordByObjl().
+    int         nLastObjl;     // Added for FindRecordByObjl().
 
     DDFIndexedRecord *pasRecords;
 
@@ -227,7 +227,7 @@ public:
 
     DDFRecord  *FindRecord( int nKey );
 
-    DDFRecord  *FindRecordByObjl( int nObjl );    /* rjensen. added for FindRecordByObjl() */
+    DDFRecord  *FindRecordByObjl( int nObjl );  // Added for FindRecordByObjl().
 
     void        Clear();
 
@@ -260,7 +260,7 @@ class CPL_DLL S57Reader
     int                 nCOMF;  /* Coordinate multiplier */
     int                 nSOMF;  /* Vertical (sounding) multiplier */
 
-    int                 bFileIngested;
+    bool                bFileIngested;
     DDFRecordIndex      oVI_Index;
     DDFRecordIndex      oVC_Index;
     DDFRecordIndex      oVE_Index;
@@ -281,11 +281,11 @@ class CPL_DLL S57Reader
 
     char                **papszOptions;
 
-    int                 nOptionFlags; 
+    int                 nOptionFlags;
 
     int                 iPointOffset;
     OGRFeature          *poMultiPoint;
-    
+
     int                 Aall;               // see RecodeByDSSI() function
     int                 Nall;               // see RecodeByDSSI() function
     bool                needAallNallSetup;  // see RecodeByDSSI() function
@@ -313,15 +313,15 @@ class CPL_DLL S57Reader
 
     int                 ApplyRecordUpdate( DDFRecord *, DDFRecord * );
 
-    int                 bMissingWarningIssued;
-    int                 bAttrWarningIssued;
+    bool                bMissingWarningIssued;
+    bool                bAttrWarningIssued;
 
   public:
                         S57Reader( const char * );
                        ~S57Reader();
 
     void                SetClassBased( S57ClassRegistrar *, S57ClassContentExplorer* );
-    void                SetOptions( char ** );
+    int                 SetOptions( char ** );
     int                 GetOptionFlags() { return nOptionFlags; }
 
     int                 Open( int bTestOpen );
@@ -371,12 +371,24 @@ public:
     int                 WriteATTF( DDFRecord *, OGRFeature * );
     int                 WritePrimitive( OGRFeature *poFeature );
     int                 WriteCompleteFeature( OGRFeature *poFeature );
-    int                 WriteDSID( const char *pszDSNM = NULL, 
-                                   const char *pszISDT = NULL, 
+    int                 WriteDSID( int nEXPP = 1,
+                                   int nINTU = 4,
+                                   const char *pszDSNM = NULL,
+                                   const char *pszEDTN = NULL,
+                                   const char *pszUPDN = NULL,
+                                   const char *pszUADT = NULL,
+                                   const char *pszISDT = NULL,
                                    const char *pszSTED = NULL,
                                    int nAGEN = 0,
-                                   const char *pszCOMT = NULL );
-    int                 WriteDSPM( int nScale = 0 );
+                                   const char *pszCOMT = NULL,
+                                   int nNOMR = 0, int nNOGR = 0,
+                                   int nNOLR = 0, int nNOIN = 0,
+                                   int nNOCN = 0, int nNOED = 0
+                                 );
+    int                 WriteDSPM( int nHDAT = 0,
+                                   int nVDAT = 0,
+                                   int nSDAT = 0,
+                                   int nCSCL = 0 );
 
 // semi-private - for sophisticated writers.
     DDFRecord           *MakeRecord();
@@ -396,10 +408,10 @@ private:
 /* -------------------------------------------------------------------- */
 void           CPL_DLL  S57GenerateStandardAttributes( OGRFeatureDefn *, int );
 OGRFeatureDefn CPL_DLL *S57GenerateGeomFeatureDefn( OGRwkbGeometryType, int );
-OGRFeatureDefn CPL_DLL *S57GenerateObjectClassDefn( S57ClassRegistrar *, 
+OGRFeatureDefn CPL_DLL *S57GenerateObjectClassDefn( S57ClassRegistrar *,
                                                     S57ClassContentExplorer* poClassContentExplorer,
                                                     int, int );
 OGRFeatureDefn CPL_DLL  *S57GenerateVectorPrimitiveFeatureDefn( int, int );
 OGRFeatureDefn CPL_DLL  *S57GenerateDSIDFeatureDefn( void );
 
-#endif /* ndef _S57_H_INCLUDED */
+#endif /* ndef S57_H_INCLUDED */
