@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: s57classregistrar.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: s57classregistrar.cpp 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  S-57 Translator
  * Purpose:  Implements S57ClassRegistrar class for keeping track of
@@ -29,11 +29,11 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "s57.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
+#include "s57.h"
 
-CPL_CVSID("$Id: s57classregistrar.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: s57classregistrar.cpp 33714 2016-03-13 05:42:13Z goatbar $");
 
 
 #ifdef S57_BUILTIN_CLASSES
@@ -44,11 +44,11 @@ CPL_CVSID("$Id: s57classregistrar.cpp 27044 2014-03-16 23:41:27Z rouault $");
 /*                         S57ClassRegistrar()                          */
 /************************************************************************/
 
-S57ClassRegistrar::S57ClassRegistrar()
-
-{
-    papszNextLine = NULL;
-}
+S57ClassRegistrar::S57ClassRegistrar() :
+    nClasses(0),
+    nAttrCount(0),
+    papszNextLine(NULL)
+{ }
 
 /************************************************************************/
 /*                         ~S57ClassRegistrar()                         */
@@ -58,16 +58,19 @@ S57ClassRegistrar::~S57ClassRegistrar()
 
 {
     nClasses = 0;
+    for(size_t i=0;i<aoAttrInfos.size();i++)
+        delete aoAttrInfos[i];
+    aoAttrInfos.resize(0);
+    nAttrCount = 0;
 }
 
 /************************************************************************/
 /*                        S57ClassContentExplorer()                     */
 /************************************************************************/
 
-S57ClassContentExplorer::S57ClassContentExplorer(S57ClassRegistrar* poRegistrar):
-    poRegistrar(poRegistrar)
+S57ClassContentExplorer::S57ClassContentExplorer(S57ClassRegistrar* poRegistrarIn):
+    poRegistrar(poRegistrarIn)
 {
-    
     iCurrentClass = -1;
 
     papszCurrentFields = NULL;
@@ -82,7 +85,7 @@ S57ClassContentExplorer::S57ClassContentExplorer(S57ClassRegistrar* poRegistrar)
 S57ClassContentExplorer::~S57ClassContentExplorer()
 {
     CSLDestroy( papszTempResult );
-    
+
     if( papapszClassesFields != NULL )
     {
         for( int i = 0; i < poRegistrar->nClasses; i++ )
@@ -95,14 +98,14 @@ S57ClassContentExplorer::~S57ClassContentExplorer()
 /*                              FindFile()                              */
 /************************************************************************/
 
-int S57ClassRegistrar::FindFile( const char *pszTarget, 
-                                 const char *pszDirectory, 
+int S57ClassRegistrar::FindFile( const char *pszTarget,
+                                 const char *pszDirectory,
                                  int bReportErr,
                                  VSILFILE **pfp )
 
 {
     const char *pszFilename;
-    
+
     if( pszDirectory == NULL )
     {
         pszFilename = CPLFindFile( "s57", pszTarget );
@@ -167,14 +170,11 @@ const char *S57ClassRegistrar::ReadLine( VSILFILE * fp )
 /*                              LoadInfo()                              */
 /************************************************************************/
 
-int S57ClassRegistrar::LoadInfo( const char * pszDirectory, 
+int S57ClassRegistrar::LoadInfo( const char * pszDirectory,
                                  const char * pszProfile,
                                  int bReportErr )
 
 {
-    VSILFILE   *fp;
-    char        szTargetFile[1024];
-
     if( pszDirectory == NULL )
         pszDirectory = CPLGetConfigOption("S57_CSV",NULL);
 
@@ -183,14 +183,15 @@ int S57ClassRegistrar::LoadInfo( const char * pszDirectory,
 /* ==================================================================== */
     if( pszProfile == NULL )
         pszProfile = CPLGetConfigOption( "S57_PROFILE", "" );
-    
+
+    char szTargetFile[1024];  // TODO: Get this off of the stack.
     if( EQUAL(pszProfile, "Additional_Military_Layers") )
     {
-       sprintf( szTargetFile, "s57objectclasses_%s.csv", "aml" );
+       snprintf( szTargetFile, sizeof(szTargetFile), "s57objectclasses_%s.csv", "aml" );
     }
     else if ( EQUAL(pszProfile, "Inland_Waterways") )
     {
-       sprintf( szTargetFile, "s57objectclasses_%s.csv", "iw" );
+       snprintf( szTargetFile, sizeof(szTargetFile), "s57objectclasses_%s.csv", "iw" );
     }
     else if( strlen(pszProfile) > 0 )
     {
@@ -201,6 +202,7 @@ int S57ClassRegistrar::LoadInfo( const char * pszDirectory,
        strcpy( szTargetFile, "s57objectclasses.csv" );
     }
 
+    VSILFILE *fp = NULL;
     if( !FindFile( szTargetFile, pszDirectory, bReportErr, &fp ) )
         return FALSE;
 
@@ -245,11 +247,11 @@ int S57ClassRegistrar::LoadInfo( const char * pszDirectory,
 
     if( EQUAL(pszProfile, "Additional_Military_Layers") )
     {
-        sprintf( szTargetFile, "s57attributes_%s.csv", "aml" );
+      snprintf( szTargetFile, sizeof(szTargetFile), "s57attributes_%s.csv", "aml" );
     }
     else if ( EQUAL(pszProfile, "Inland_Waterways") )
     {
-       sprintf( szTargetFile, "s57attributes_%s.csv", "iw" );
+       snprintf( szTargetFile, sizeof(szTargetFile),"s57attributes_%s.csv", "iw" );
     }
     else if( strlen(pszProfile) > 0 )
     {
@@ -259,7 +261,7 @@ int S57ClassRegistrar::LoadInfo( const char * pszDirectory,
     {
        strcpy( szTargetFile, "s57attributes.csv" );
     }
-       
+
     if( !FindFile( szTargetFile, pszDirectory, bReportErr, &fp ) )
         return FALSE;
 
@@ -277,12 +279,10 @@ int S57ClassRegistrar::LoadInfo( const char * pszDirectory,
             VSIFCloseL( fp );
         return FALSE;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Read and form string list.                                      */
 /* -------------------------------------------------------------------- */
-    int        iAttr;
-
     while( (pszLine = ReadLine(fp)) != NULL )
     {
         char    **papszTokens = CSLTokenizeStringComplex( pszLine, ",",
@@ -293,16 +293,17 @@ int S57ClassRegistrar::LoadInfo( const char * pszDirectory,
             CPLAssert( FALSE );
             continue;
         }
-        
-        iAttr = atoi(papszTokens[0]);
+
+        int iAttr = atoi(papszTokens[0]);
         if( iAttr >= (int) aoAttrInfos.size() )
             aoAttrInfos.resize(iAttr+1);
 
         if( iAttr < 0 || aoAttrInfos[iAttr] != NULL )
         {
-            CPLDebug( "S57", 
-                      "Duplicate/corrupt definition for attribute %d:%s", 
+            CPLDebug( "S57",
+                      "Duplicate/corrupt definition for attribute %d:%s",
                       iAttr, papszTokens[2] );
+            CSLDestroy( papszTokens );
             continue;
         }
 
@@ -318,30 +319,28 @@ int S57ClassRegistrar::LoadInfo( const char * pszDirectory,
     if( fp != NULL )
         VSIFCloseL( fp );
 
-    nAttrCount = anAttrIndex.size();
-    
+    nAttrCount = static_cast<int>(anAttrIndex.size());
+
 /* -------------------------------------------------------------------- */
 /*      Sort index by acronym.                                          */
 /* -------------------------------------------------------------------- */
-    int         bModified;
+    bool bModified = false;
     do
     {
-        bModified = FALSE;
-        for( iAttr = 0; iAttr < nAttrCount-1; iAttr++ )
+        bModified = false;
+        for( int iAttr = 0; iAttr < nAttrCount-1; iAttr++ )
         {
             if( strcmp(aoAttrInfos[anAttrIndex[iAttr]]->osAcronym,
                        aoAttrInfos[anAttrIndex[iAttr+1]]->osAcronym) > 0 )
             {
-                int     nTemp;
-
-                nTemp = anAttrIndex[iAttr];
+                int nTemp = anAttrIndex[iAttr];
                 anAttrIndex[iAttr] = anAttrIndex[iAttr+1];
                 anAttrIndex[iAttr+1] = nTemp;
-                bModified = TRUE;
+                bModified = true;
             }
         }
     } while( bModified );
-    
+
     return TRUE;
 }
 
@@ -367,7 +366,7 @@ int S57ClassContentExplorer::SelectClassByIndex( int nNewIndex )
 /*      Has this info been parsed yet?                                  */
 /* -------------------------------------------------------------------- */
     if( papapszClassesFields[nNewIndex] == NULL )
-        papapszClassesFields[nNewIndex] = 
+        papapszClassesFields[nNewIndex] =
             CSLTokenizeStringComplex( poRegistrar->apszClassesInfo[nNewIndex],
                                       ",", TRUE, TRUE );
 
@@ -422,36 +421,36 @@ int S57ClassContentExplorer::GetOBJL()
 {
     if( iCurrentClass >= 0 )
         return atoi(poRegistrar->apszClassesInfo[iCurrentClass]);
-    else
-        return -1;
+
+    return -1;
 }
 
 /************************************************************************/
 /*                           GetDescription()                           */
 /************************************************************************/
 
-const char * S57ClassContentExplorer::GetDescription()
+const char * S57ClassContentExplorer::GetDescription() const
 
 {
     if( iCurrentClass >= 0 && papszCurrentFields[0] != NULL )
         return papszCurrentFields[1];
-    else
-        return NULL;
+
+    return NULL;
 }
 
 /************************************************************************/
 /*                             GetAcronym()                             */
 /************************************************************************/
 
-const char * S57ClassContentExplorer::GetAcronym()
+const char * S57ClassContentExplorer::GetAcronym() const
 
 {
-    if( iCurrentClass >= 0 
-        && papszCurrentFields[0] != NULL 
+    if( iCurrentClass >= 0
+        && papszCurrentFields[0] != NULL
         && papszCurrentFields[1] != NULL )
         return papszCurrentFields[2];
-    else
-        return NULL;
+
+    return NULL;
 }
 
 /************************************************************************/
@@ -466,7 +465,7 @@ char **S57ClassContentExplorer::GetAttributeList( const char * pszType )
 {
     if( iCurrentClass < 0 )
         return NULL;
-    
+
     CSLDestroy( papszTempResult );
     papszTempResult = NULL;
 
@@ -474,16 +473,14 @@ char **S57ClassContentExplorer::GetAttributeList( const char * pszType )
     {
         if( pszType != NULL && iColumn == 3 && !EQUAL(pszType,"a") )
             continue;
-        
+
         if( pszType != NULL && iColumn == 4 && !EQUAL(pszType,"b") )
             continue;
-        
+
         if( pszType != NULL && iColumn == 5 && !EQUAL(pszType,"c") )
             continue;
 
-        char    **papszTokens;
-
-        papszTokens =
+        char **papszTokens =
             CSLTokenizeStringComplex( papszCurrentFields[iColumn], ";",
                                       TRUE, FALSE );
 
@@ -500,7 +497,7 @@ char **S57ClassContentExplorer::GetAttributeList( const char * pszType )
 /*                            GetClassCode()                            */
 /************************************************************************/
 
-char S57ClassContentExplorer::GetClassCode()
+char S57ClassContentExplorer::GetClassCode() const
 
 {
     if( iCurrentClass >= 0
@@ -509,11 +506,11 @@ char S57ClassContentExplorer::GetClassCode()
         && papszCurrentFields[2] != NULL
         && papszCurrentFields[3] != NULL
         && papszCurrentFields[4] != NULL
-        && papszCurrentFields[5] != NULL 
+        && papszCurrentFields[5] != NULL
         && papszCurrentFields[6] != NULL )
         return papszCurrentFields[6][0];
-    else
-        return '\0';
+
+    return '\0';
 }
 
 /************************************************************************/
@@ -527,13 +524,13 @@ char **S57ClassContentExplorer::GetPrimitives()
         && CSLCount(papszCurrentFields) > 7 )
     {
         CSLDestroy( papszTempResult );
-        papszTempResult = 
+        papszTempResult =
             CSLTokenizeStringComplex( papszCurrentFields[7], ";",
                                       TRUE, FALSE );
         return papszTempResult;
     }
-    else
-        return NULL;
+
+    return NULL;
 }
 
 /************************************************************************/
@@ -544,8 +541,8 @@ const S57AttrInfo *S57ClassRegistrar::GetAttrInfo(int iAttr)
 {
     if( iAttr < 0 || iAttr >= (int) aoAttrInfos.size() )
         return NULL;
-    else 
-        return aoAttrInfos[iAttr];
+
+    return aoAttrInfos[iAttr];
 }
 
 /************************************************************************/
@@ -555,17 +552,13 @@ const S57AttrInfo *S57ClassRegistrar::GetAttrInfo(int iAttr)
 int    S57ClassRegistrar::FindAttrByAcronym( const char * pszName )
 
 {
-    int         iStart, iEnd, iCandidate;
-
-    iStart = 0;
-    iEnd = nAttrCount-1;
+    int iStart = 0;
+    int iEnd = nAttrCount-1;
 
     while( iStart <= iEnd )
     {
-        int     nCompareValue;
-        
-        iCandidate = (iStart + iEnd)/2;
-        nCompareValue =
+        const int iCandidate = (iStart + iEnd)/2;
+        int nCompareValue =
             strcmp(pszName, aoAttrInfos[anAttrIndex[iCandidate]]->osAcronym);
 
         if( nCompareValue < 0 )
