@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: s57featuredefns.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: s57featuredefns.cpp 33271 2016-01-30 16:01:55Z goatbar $
  *
  * Project:  S-57 Translator
  * Purpose:  Implements methods to create OGRFeatureDefns for various
@@ -28,12 +28,12 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "s57.h"
-#include "ogr_api.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
+#include "ogr_api.h"
+#include "s57.h"
 
-CPL_CVSID("$Id: s57featuredefns.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: s57featuredefns.cpp 33271 2016-01-30 16:01:55Z goatbar $");
 
 
 /************************************************************************/
@@ -44,7 +44,6 @@ OGRFeatureDefn *S57GenerateDSIDFeatureDefn()
 
 {
     OGRFeatureDefn      *poFDefn = new OGRFeatureDefn( "DSID" );
-    OGRFieldDefn         oField( "", OFTInteger );
 
     poFDefn->SetGeomType( wkbNone );
     poFDefn->Reference();
@@ -52,122 +51,124 @@ OGRFeatureDefn *S57GenerateDSIDFeatureDefn()
 /* -------------------------------------------------------------------- */
 /*      DSID fields.                                                    */
 /* -------------------------------------------------------------------- */
+    OGRFieldDefn oField( "", OFTInteger );
+
     oField.Set( "DSID_EXPP", OFTInteger, 3, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSID_INTU", OFTInteger, 3, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSID_DSNM", OFTString, 0, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSID_EDTN", OFTString, 0, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSID_UPDN", OFTString, 0, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSID_UADT", OFTString, 8, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSID_ISDT", OFTString, 8, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSID_STED", OFTReal, 11, 6 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSID_PRSP", OFTInteger, 3, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSID_PSDN", OFTString, 0, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSID_PRED", OFTString, 0, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSID_PROF", OFTInteger, 3, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSID_AGEN", OFTInteger, 5, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSID_COMT", OFTString, 0, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
 /* -------------------------------------------------------------------- */
 /*      DSSI fields.                                                    */
 /* -------------------------------------------------------------------- */
 
     oField.Set( "DSSI_DSTR", OFTInteger, 3, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSSI_AALL", OFTInteger, 3, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSSI_NALL", OFTInteger, 3, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSSI_NOMR", OFTInteger, 10, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSSI_NOCR", OFTInteger, 10, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSSI_NOGR", OFTInteger, 10, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSSI_NOLR", OFTInteger, 10, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSSI_NOIN", OFTInteger, 10, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSSI_NOCN", OFTInteger, 10, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSSI_NOED", OFTInteger, 10, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSSI_NOFA", OFTInteger, 10, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
 /* -------------------------------------------------------------------- */
 /*      DSPM fields.                                                    */
 /* -------------------------------------------------------------------- */
 
     oField.Set( "DSPM_HDAT", OFTInteger, 3, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSPM_VDAT", OFTInteger, 3, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSPM_SDAT", OFTInteger, 3, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSPM_CSCL", OFTInteger, 10, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSPM_DUNI", OFTInteger, 3, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSPM_HUNI", OFTInteger, 3, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSPM_PUNI", OFTInteger, 3, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSPM_COUN", OFTInteger, 3, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSPM_COMF", OFTInteger, 10, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSPM_SOMF", OFTInteger, 10, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     oField.Set( "DSPM_COMT", OFTString, 0, 0 );
     poFDefn->AddFieldDefn( &oField );
-    
+
     return poFDefn;
 }
 
@@ -180,8 +181,8 @@ OGRFeatureDefn *S57GenerateGeomFeatureDefn( OGRwkbGeometryType eGType,
 
 {
     OGRFeatureDefn      *poFDefn = NULL;
-    
-    if( eGType == wkbPoint )                                            
+
+    if( eGType == wkbPoint )
     {
         poFDefn = new OGRFeatureDefn( "Point" );
         poFDefn->SetGeomType( eGType );
@@ -220,10 +221,11 @@ OGRFeatureDefn *S57GenerateGeomFeatureDefn( OGRwkbGeometryType eGType,
 /************************************************************************/
 
 OGRFeatureDefn *
-S57GenerateVectorPrimitiveFeatureDefn( int nRCNM, CPL_UNUSED int nOptionFlags )
+S57GenerateVectorPrimitiveFeatureDefn( int nRCNM,
+                                       int /* nOptionFlags */ )
 {
     OGRFeatureDefn      *poFDefn = NULL;
- 
+
     if( nRCNM == RCNM_VI )
     {
         poFDefn = new OGRFeatureDefn( OGRN_VI );
@@ -316,29 +318,28 @@ S57GenerateVectorPrimitiveFeatureDefn( int nRCNM, CPL_UNUSED int nOptionFlags )
 /*                     S57GenerateObjectClassDefn()                     */
 /************************************************************************/
 
-OGRFeatureDefn *S57GenerateObjectClassDefn( S57ClassRegistrar *poCR,
-                                            S57ClassContentExplorer* poClassContentExplorer,
-                                            int nOBJL, int nOptionFlags )
+OGRFeatureDefn *S57GenerateObjectClassDefn(
+    S57ClassRegistrar *poCR,
+    S57ClassContentExplorer* poClassContentExplorer,
+    int nOBJL, int nOptionFlags )
 
 {
-    OGRFeatureDefn      *poFDefn = NULL;
-    char               **papszGeomPrim;
-
     if( !poClassContentExplorer->SelectClass( nOBJL ) )
         return NULL;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Create the feature definition based on the object class         */
 /*      acronym.                                                        */
 /* -------------------------------------------------------------------- */
-    poFDefn = new OGRFeatureDefn( poClassContentExplorer->GetAcronym() );
+    OGRFeatureDefn *poFDefn
+        = new OGRFeatureDefn( poClassContentExplorer->GetAcronym() );
     poFDefn->Reference();
 
 /* -------------------------------------------------------------------- */
 /*      Try and establish the geometry type.  If more than one          */
 /*      geometry type is allowed we just fall back to wkbUnknown.       */
 /* -------------------------------------------------------------------- */
-    papszGeomPrim = poClassContentExplorer->GetPrimitives();
+    char **papszGeomPrim = poClassContentExplorer->GetPrimitives();
     if( CSLCount(papszGeomPrim) == 0 )
     {
         poFDefn->SetGeomType( wkbNone );
@@ -368,7 +369,7 @@ OGRFeatureDefn *S57GenerateObjectClassDefn( S57ClassRegistrar *poCR,
         // unfortunately this could be a multilinestring
         poFDefn->SetGeomType( wkbUnknown );
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Add the standard attributes.                                    */
 /* -------------------------------------------------------------------- */
@@ -377,13 +378,13 @@ OGRFeatureDefn *S57GenerateObjectClassDefn( S57ClassRegistrar *poCR,
 /* -------------------------------------------------------------------- */
 /*      Add the attributes specific to this object class.               */
 /* -------------------------------------------------------------------- */
-    char        **papszAttrList = poClassContentExplorer->GetAttributeList();
+    char **papszAttrList = poClassContentExplorer->GetAttributeList();
 
     for( int iAttr = 0;
          papszAttrList != NULL && papszAttrList[iAttr] != NULL;
          iAttr++ )
     {
-        int     iAttrIndex = poCR->FindAttrByAcronym( papszAttrList[iAttr] );
+        const int iAttrIndex = poCR->FindAttrByAcronym( papszAttrList[iAttr] );
 
         if( iAttrIndex == -1 )
         {
@@ -394,8 +395,8 @@ OGRFeatureDefn *S57GenerateObjectClassDefn( S57ClassRegistrar *poCR,
             continue;
         }
 
-        OGRFieldDefn    oField( papszAttrList[iAttr], OFTInteger );
-        
+        OGRFieldDefn oField( papszAttrList[iAttr], OFTInteger );
+
         switch( poCR->GetAttrType( iAttrIndex ) )
         {
           case SAT_ENUM:
@@ -420,14 +421,13 @@ OGRFeatureDefn *S57GenerateObjectClassDefn( S57ClassRegistrar *poCR,
         poFDefn->AddFieldDefn( &oField );
     }
 
-
 /* -------------------------------------------------------------------- */
 /*      Do we need to add DEPTH attributes to soundings?                */
 /* -------------------------------------------------------------------- */
-    if( EQUAL(poClassContentExplorer->GetAcronym(),"SOUNDG") 
+    if( EQUAL(poClassContentExplorer->GetAcronym(), "SOUNDG")
         && (nOptionFlags & S57M_ADD_SOUNDG_DEPTH) )
     {
-        OGRFieldDefn    oField( "DEPTH", OFTReal );
+        OGRFieldDefn oField( "DEPTH", OFTReal );
         poFDefn->AddFieldDefn( &oField );
     }
 
@@ -500,14 +500,14 @@ void S57GenerateStandardAttributes( OGRFeatureDefn *poFDefn, int nOptionFlags )
     {
         oField.Set( "LNAM", OFTString, 16, 0 );
         poFDefn->AddFieldDefn( &oField );
-        
+
         oField.Set( "LNAM_REFS", OFTStringList, 16, 0 );
         poFDefn->AddFieldDefn( &oField );
-        
+
         oField.Set( "FFPT_RIND", OFTIntegerList, 1, 0 );
         poFDefn->AddFieldDefn( &oField );
 
-        // We should likely include FFPT_COMT here. 
+        // We should likely include FFPT_COMT here.
     }
 
 /* -------------------------------------------------------------------- */
@@ -517,13 +517,13 @@ void S57GenerateStandardAttributes( OGRFeatureDefn *poFDefn, int nOptionFlags )
     {
         oField.Set( "NAME_RCNM", OFTIntegerList, 3, 0 );
         poFDefn->AddFieldDefn( &oField );
-        
+
         oField.Set( "NAME_RCID", OFTIntegerList, 10, 0 );
         poFDefn->AddFieldDefn( &oField );
-        
+
         oField.Set( "ORNT", OFTIntegerList, 1, 0 );
         poFDefn->AddFieldDefn( &oField );
-        
+
         oField.Set( "USAG", OFTIntegerList, 1, 0 );
         poFDefn->AddFieldDefn( &oField );
 
@@ -531,4 +531,3 @@ void S57GenerateStandardAttributes( OGRFeatureDefn *poFDefn, int nOptionFlags )
         poFDefn->AddFieldDefn( &oField );
     }
 }
-
