@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: s57filecollector.cpp 10645 2007-01-18 02:22:39Z warmerdam $
+ * $Id: s57filecollector.cpp 33712 2016-03-12 10:51:56Z goatbar $
  *
  * Project:  S-57 Translator
  * Purpose:  Implements S57FileCollector() function.  This function collects
@@ -29,11 +29,11 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#include "s57.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
+#include "s57.h"
 
-CPL_CVSID("$Id: s57filecollector.cpp 10645 2007-01-18 02:22:39Z warmerdam $");
+CPL_CVSID("$Id: s57filecollector.cpp 33712 2016-03-12 10:51:56Z goatbar $");
 
 /************************************************************************/
 /*                          S57FileCollector()                          */
@@ -42,12 +42,10 @@ CPL_CVSID("$Id: s57filecollector.cpp 10645 2007-01-18 02:22:39Z warmerdam $");
 char **S57FileCollector( const char *pszDataset )
 
 {
-    VSIStatBuf  sStatBuf;
-    char        **papszRetList = NULL;
-
 /* -------------------------------------------------------------------- */
 /*      Stat the dataset, and fail if it isn't a file or directory.     */
 /* -------------------------------------------------------------------- */
+    VSIStatBuf  sStatBuf;
     if( CPLStat( pszDataset, &sStatBuf ) )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
@@ -61,23 +59,22 @@ char **S57FileCollector( const char *pszDataset )
 /*      We handle directories by scanning for all S-57 data files in    */
 /*      them, but not for catalogs.                                     */
 /* -------------------------------------------------------------------- */
+    char **papszRetList = NULL;
+
     if( VSI_ISDIR(sStatBuf.st_mode) )
     {
-        char    **papszDirFiles = CPLReadDir( pszDataset );
-        int     iFile;
+        char    **papszDirFiles = VSIReadDir( pszDataset );
         DDFModule oModule;
 
-        for( iFile = 0;
+        for( int iFile = 0;
              papszDirFiles != NULL && papszDirFiles[iFile] != NULL;
              iFile++ )
         {
-            char        *pszFullFile;
-
-            pszFullFile = CPLStrdup(
+            char *pszFullFile = CPLStrdup(
                 CPLFormFilename( pszDataset, papszDirFiles[iFile], NULL ) );
 
             // Add to list if it is an S-57 _data_ file.
-            if( VSIStat( pszFullFile, &sStatBuf ) == 0 
+            if( VSIStat( pszFullFile, &sStatBuf ) == 0
                 && VSI_ISREG( sStatBuf.st_mode )
                 && oModule.Open( pszFullFile, TRUE ) )
             {
@@ -96,7 +93,6 @@ char **S57FileCollector( const char *pszDataset )
 /*      Note that the caller may still open it and fail.                */
 /* -------------------------------------------------------------------- */
     DDFModule   oModule;
-    DDFRecord   *poRecord;
 
     if( !oModule.Open(pszDataset) )
     {
@@ -107,10 +103,10 @@ char **S57FileCollector( const char *pszDataset )
         return NULL;
     }
 
-    poRecord = oModule.ReadRecord();
+    DDFRecord *poRecord = oModule.ReadRecord();
     if( poRecord == NULL )
         return NULL;
-    
+
     if( poRecord->FindField( "CATD" ) == NULL
         || oModule.FindFieldDefn("CATD")->FindSubfieldDefn( "IMPL" ) == NULL )
     {
@@ -127,25 +123,25 @@ char **S57FileCollector( const char *pszDataset )
 /* -------------------------------------------------------------------- */
     char        *pszCatDir = CPLStrdup( CPLGetPath( pszDataset ) );
     char        *pszRootDir = NULL;
-    
+
     if( CPLStat( CPLFormFilename(pszCatDir,"ENC_ROOT",NULL), &sStatBuf ) == 0
         && VSI_ISDIR(sStatBuf.st_mode) )
     {
         pszRootDir = CPLStrdup(CPLFormFilename( pszCatDir, "ENC_ROOT", NULL ));
     }
-    else if( CPLStat( CPLFormFilename( pszCatDir, "enc_root", NULL ), 
+    else if( CPLStat( CPLFormFilename( pszCatDir, "enc_root", NULL ),
                       &sStatBuf ) == 0 && VSI_ISDIR(sStatBuf.st_mode) )
     {
         pszRootDir = CPLStrdup(CPLFormFilename( pszCatDir, "enc_root", NULL ));
     }
 
     if( pszRootDir )
-        CPLDebug( "S57", "Found root directory to be %s.", 
+        CPLDebug( "S57", "Found root directory to be %s.",
                   pszRootDir );
 
 /* -------------------------------------------------------------------- */
 /*      We have a catalog.  Scan it for data files, those with an       */
-/*      IMPL of BIN.  Shouldn't there be a better way of testing        */
+/*      IMPL of BIN.  Is there be a better way of testing               */
 /*      whether a file is a data file or another catalog file?          */
 /* -------------------------------------------------------------------- */
     for( ; poRecord != NULL; poRecord = oModule.ReadRecord() )
@@ -153,24 +149,24 @@ char **S57FileCollector( const char *pszDataset )
         if( poRecord->FindField( "CATD" ) != NULL
             && EQUAL(poRecord->GetStringSubfield("CATD",0,"IMPL",0),"BIN") )
         {
-            const char  *pszFile, *pszWholePath;
+            const char  *pszFile
+                = poRecord->GetStringSubfield("CATD", 0, "FILE", 0);
 
-            pszFile = poRecord->GetStringSubfield("CATD",0,"FILE",0);
+            // Often there is an extra ENC_ROOT in the path, try finding
+            // this file.
 
-            // Often there is an extra ENC_ROOT in the path, try finding 
-            // this file. 
-
-            pszWholePath = CPLFormFilename( pszCatDir, pszFile, NULL );
+            const char  *pszWholePath
+                = CPLFormFilename( pszCatDir, pszFile, NULL );
             if( CPLStat( pszWholePath, &sStatBuf ) != 0
                 && pszRootDir != NULL )
             {
                 pszWholePath = CPLFormFilename( pszRootDir, pszFile, NULL );
             }
-                
+
             if( CPLStat( pszWholePath, &sStatBuf ) != 0 )
             {
                 CPLError( CE_Warning, CPLE_OpenFailed,
-                          "Can't find file %s from catalog %s.", 
+                          "Can't find file %s from catalog %s.",
                           pszFile, pszDataset );
                 continue;
             }
@@ -185,4 +181,3 @@ char **S57FileCollector( const char *pszDataset )
 
     return papszRetList;
 }
-
