@@ -1,9 +1,11 @@
 /******************************************************************************
- * $Id: ogrsdedriver.cpp 15236 2008-08-27 23:01:17Z tamas $
+ * $Id: ogrsdedriver.cpp 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRSDEDriver class.
  * Author:   Frank Warmerdam, warmerdam@pobox.com
+ * Copyright (c) 2008, Shawn Gervais <project10@project10.net>
+ * Copyright (c) 2008, Howard Butler <hobu.inc@gmail.com>
  *
  ******************************************************************************
  * Copyright (c) 2005, Frank Warmerdam <warmerdam@pobox.com>
@@ -30,7 +32,7 @@
 #include "ogr_sde.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrsdedriver.cpp 15236 2008-08-27 23:01:17Z tamas $");
+CPL_CVSID("$Id: ogrsdedriver.cpp 33714 2016-03-13 05:42:13Z goatbar $");
 
 /************************************************************************/
 /*                            ~OGRSDEDriver()                            */
@@ -57,18 +59,13 @@ const char *OGRSDEDriver::GetName()
 
 OGRDataSource *OGRSDEDriver::Open( const char * pszFilename,
                                    int bUpdate )
-    
+
 {
     OGRSDEDataSource     *poDS;
 
-    if( bUpdate )
-    {
-        return NULL;
-    }
-    
     poDS = new OGRSDEDataSource();
 
-    if( !poDS->Open( pszFilename ) )
+    if( !poDS->Open( pszFilename, bUpdate ) )
     {
         delete poDS;
         return NULL;
@@ -77,6 +74,29 @@ OGRDataSource *OGRSDEDriver::Open( const char * pszFilename,
         return poDS;
 }
 
+/************************************************************************/
+/*                                Open()                                */
+/************************************************************************/
+
+OGRDataSource *OGRSDEDriver::CreateDataSource( const char * pszName,
+                                               char **papszOptions)
+
+{
+    OGRSDEDataSource     *poDS;
+
+    poDS = new OGRSDEDataSource();
+
+    if( !poDS->Open( pszName, TRUE ) )
+    {
+        delete poDS;
+        CPLError( CE_Failure, CPLE_AppDefined,
+          "ArcSDE driver doesn't currently support database or service "
+          "creation.  Please create the service before using.");
+        return NULL;
+    }
+    else
+        return poDS;
+}
 /************************************************************************/
 /*                           TestCapability()                           */
 /************************************************************************/
@@ -84,6 +104,13 @@ OGRDataSource *OGRSDEDriver::Open( const char * pszFilename,
 int OGRSDEDriver::TestCapability( const char * pszCap )
 
 {
+    if( EQUAL(pszCap, ODsCCreateLayer) )
+        return TRUE;
+    if( EQUAL(pszCap, ODsCDeleteLayer) )
+        return true;
+    if( EQUAL(pszCap, ODrCCreateDataSource) )
+        return TRUE;
+
     return FALSE;
 }
 
@@ -94,6 +121,8 @@ int OGRSDEDriver::TestCapability( const char * pszCap )
 void RegisterOGRSDE()
 
 {
+    if( !GDAL_CHECK_VERSION("OGR SDE") )
+        return;
+
     OGRSFDriverRegistrar::GetRegistrar()->RegisterDriver( new OGRSDEDriver );
 }
-
