@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_segukooa.h 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogr_segukooa.h 31777 2015-11-26 14:14:41Z rouault $
  *
  * Project:  SEG-P1 / UKOOA P1-90 Translator
  * Purpose:  Definition of classes for OGR SEG-P1 / UKOOA P1-90 driver.
@@ -27,8 +27,8 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _OGR_SEGUKOOA_H_INCLUDED
-#define _OGR_SEGUKOOA_H_INCLUDED
+#ifndef OGR_SEGUKOOA_H_INCLUDED
+#define OGR_SEGUKOOA_H_INCLUDED
 
 #include "ogrsf_frmts.h"
 
@@ -144,8 +144,7 @@ class OGRSEGUKOOADataSource : public OGRDataSource
                         OGRSEGUKOOADataSource();
                         ~OGRSEGUKOOADataSource();
 
-    int                 Open( const char * pszFilename,
-                              int bUpdate );
+    int                 Open( const char * pszFilename );
 
     virtual const char*         GetName() { return pszName; }
 
@@ -155,19 +154,4 @@ class OGRSEGUKOOADataSource : public OGRDataSource
     virtual int                 TestCapability( const char * );
 };
 
-/************************************************************************/
-/*                          OGRSEGUKOOADriver                           */
-/************************************************************************/
-
-class OGRSEGUKOOADriver : public OGRSFDriver
-{
-  public:
-                ~OGRSEGUKOOADriver();
-
-    virtual const char*         GetName();
-    virtual OGRDataSource*      Open( const char *, int );
-    virtual int                 TestCapability( const char * );
-};
-
-
-#endif /* ndef _OGR_SEGUKOOA_H_INCLUDED */
+#endif /* ndef OGR_SEGUKOOA_H_INCLUDED */
