@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: sbnsearch.c 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: sbnsearch.c 33713 2016-03-12 17:41:57Z goatbar $
  *
  * Project:  Shapelib
  * Purpose:  Implementation of search in ESRI SBN spatial index.
@@ -40,7 +40,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-SHP_CVSID("$Id: sbnsearch.c 27044 2014-03-16 23:41:27Z rouault $")
+SHP_CVSID("$Id: sbnsearch.c 33713 2016-03-12 17:41:57Z goatbar $")
 
 #ifndef TRUE
 #  define TRUE 1
@@ -230,12 +230,13 @@ SBNSearchHandle SBNOpenDiskTree( const char* pszSBNFilename,
     if (nShapeCount < 0 || nShapeCount > 256000000 )
     {
         char szErrorMsg[64];
-        sprintf(szErrorMsg, "Invalid shape count in .sbn : %d", nShapeCount );
+        snprintf(szErrorMsg, sizeof(szErrorMsg),
+                "Invalid shape count in .sbn : %d", nShapeCount );
         hSBN->sHooks.Error( szErrorMsg );
         SBNCloseDiskTree(hSBN);
         return NULL;
     }
-    
+
     /* Empty spatial index */
     if( nShapeCount == 0 )
     {
@@ -279,13 +280,14 @@ SBNSearchHandle SBNOpenDiskTree( const char* pszSBNFilename,
         nNodeDescCount < 0 || nNodeDescCount > nMaxNodes )
     {
         char szErrorMsg[64];
-        sprintf(szErrorMsg,
+        snprintf(szErrorMsg, sizeof(szErrorMsg),
                 "Invalid node descriptor size in .sbn : %d", nNodeDescSize );
         hSBN->sHooks.Error( szErrorMsg );
         SBNCloseDiskTree(hSBN);
         return NULL;
     }
 
+    /* coverity[tainted_data] */
     pabyData = (uchar*) malloc( nNodeDescSize );
     pasNodeDescriptor = (SBNNodeDescriptor*)
                 calloc ( nMaxNodes, sizeof(SBNNodeDescriptor) );
@@ -328,7 +330,7 @@ SBNSearchHandle SBNOpenDiskTree( const char* pszSBNFilename,
         if ((nBinStart > 0 && nNodeShapeCount == 0) ||
             nNodeShapeCount < 0 || nNodeShapeCount > nShapeCount)
         {
-            hSBN->sHooks.Error( "Inconsistant shape count in bin" );
+            hSBN->sHooks.Error( "Inconsistent shape count in bin" );
             SBNCloseDiskTree(hSBN);
             return NULL;
         }
@@ -350,7 +352,7 @@ SBNSearchHandle SBNOpenDiskTree( const char* pszSBNFilename,
     }
 
     pasNodeDescriptor[nCurNode].nBinOffset =
-        hSBN->sHooks.FTell(hSBN->fpSBN);
+        (int) hSBN->sHooks.FTell(hSBN->fpSBN);
 
     /* Compute the index of the next non empty node. */
     nNextNonEmptyNode = nCurNode + 1;
@@ -398,7 +400,7 @@ SBNSearchHandle SBNOpenDiskTree( const char* pszSBNFilename,
         {
             nCurNode = nNextNonEmptyNode;
             pasNodeDescriptor[nCurNode].nBinOffset =
-                hSBN->sHooks.FTell(hSBN->fpSBN) - 8;
+                (int) hSBN->sHooks.FTell(hSBN->fpSBN) - 8;
 
             /* Compute the index of the next non empty node. */
             nNextNonEmptyNode = nCurNode + 1;
@@ -637,7 +639,7 @@ static int SBNSearchDiskInternal( SearchStruct* psSearch,
             {
                 free(psNode->pabyShapeDesc);
                 psNode->pabyShapeDesc = NULL;
-                hSBN->sHooks.Error( "Inconsistant shape count for bin" );
+                hSBN->sHooks.Error( "Inconsistent shape count for bin" );
                 return FALSE;
             }
 
@@ -684,7 +686,7 @@ static int SBNSearchDiskInternal( SearchStruct* psSearch,
 #ifdef sanity_checks
 /* -------------------------------------------------------------------- */
 /*      Those tests only check that the shape bounding box in the bin   */
-/*      are consistant (self-consistant and consistant with the node    */
+/*      are consistent (self-consistent and consistent with the node    */
 /*      they are attached to). They are optional however (as far as     */
 /*      the safety of runtime is concerned at least).                   */
 /* -------------------------------------------------------------------- */
@@ -720,7 +722,7 @@ static int SBNSearchDiskInternal( SearchStruct* psSearch,
                     nShapeId = READ_MSB_INT(pabyBinShape + 4);
 
                     /* Caution : we count shape id starting from 0, and not 1 */
-                    nShapeId --; 
+                    nShapeId --;
 
                     /*printf("shape=%d, minx=%d, miny=%d, maxx=%d, maxy=%d\n",
                         nShapeId, bMinX, bMinY, bMaxX, bMaxY);*/
@@ -737,7 +739,7 @@ static int SBNSearchDiskInternal( SearchStruct* psSearch,
         {
             free(psNode->pabyShapeDesc);
             psNode->pabyShapeDesc = NULL;
-            hSBN->sHooks.Error( "Inconsistant shape count for bin" );
+            hSBN->sHooks.Error( "Inconsistent shape count for bin" );
             return FALSE;
         }
 
@@ -931,7 +933,7 @@ int* SBNSearchDiskTreeInteger( SBNSearchHandle hSBN,
     sSearch.bMaxY = (coord) (bMaxY <= 255 ? bMaxY : 255);
     sSearch.nShapeCount = 0;
     sSearch.nShapeAlloc = 0;
-    sSearch.panShapeId = NULL;
+    sSearch.panShapeId = (int*) calloc(1, sizeof(int));
 #ifdef DEBUG_IO
     sSearch.nBytesRead = 0;
 #endif
@@ -945,8 +947,7 @@ int* SBNSearchDiskTreeInteger( SBNSearchHandle hSBN,
 
     if( !bRet )
     {
-        if( sSearch.panShapeId != NULL )
-            free( sSearch.panShapeId );
+        free( sSearch.panShapeId );
         *pnShapeCount = 0;
         return NULL;
     }
@@ -958,10 +959,6 @@ int* SBNSearchDiskTreeInteger( SBNSearchHandle hSBN,
 /* -------------------------------------------------------------------- */
     qsort(sSearch.panShapeId, *pnShapeCount, sizeof(int), compare_ints);
 
-    /* To distinguish between empty intersection from error case */
-    if( sSearch.panShapeId == NULL )
-        sSearch.panShapeId = (int*) calloc(1, sizeof(int));
-
     return sSearch.panShapeId;
 }
 
