@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrsqliteregexp.h 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogrsqliteregexp.h 31777 2015-11-26 14:14:41Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  SQLite REGEXP function
@@ -27,12 +27,12 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _OGR_SQLITE_REGEXP_INCLUDED
-#define _OGR_SQLITE_REGEXP_INCLUDED
+#ifndef OGR_SQLITE_REGEXP_INCLUDED
+#define OGR_SQLITE_REGEXP_INCLUDED
 
 #include "ogr_sqlite.h"
 
 static void* OGRSQLiteRegisterRegExpFunction(sqlite3* hDB);
 static void OGRSQLiteFreeRegExpCache(void* hRegExpCache);
 
-#endif // _OGR_SQLITE_REGEXP_INCLUDED
+#endif // OGR_SQLITE_REGEXP_INCLUDED
