@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrsqlitesinglefeaturelayer.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogrsqlitesinglefeaturelayer.cpp 32011 2015-12-06 10:19:18Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRSQLiteSingleFeatureLayer class.
@@ -31,7 +31,7 @@
 #include "cpl_string.h"
 #include "ogr_sqlite.h"
 
-CPL_CVSID("$Id: ogrsqlitesinglefeaturelayer.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: ogrsqlitesinglefeaturelayer.cpp 32011 2015-12-06 10:19:18Z rouault $");
 
 /************************************************************************/
 /*                    OGRSQLiteSingleFeatureLayer()                     */
@@ -39,15 +39,16 @@ CPL_CVSID("$Id: ogrsqlitesinglefeaturelayer.cpp 27044 2014-03-16 23:41:27Z rouau
 
 OGRSQLiteSingleFeatureLayer::OGRSQLiteSingleFeatureLayer(
                                                      const char* pszLayerName,
-                                                     int nVal )
+                                                     int nValIn )
 {
     poFeatureDefn = new OGRFeatureDefn( "SELECT" );
+    SetDescription( poFeatureDefn->GetName() );
     poFeatureDefn->Reference();
     OGRFieldDefn oField( pszLayerName, OFTInteger );
     poFeatureDefn->AddFieldDefn( &oField );
 
     iNextShapeId = 0;
-    this->nVal = nVal;
+    this->nVal = nValIn;
     pszVal = NULL;
 }
 
@@ -57,7 +58,7 @@ OGRSQLiteSingleFeatureLayer::OGRSQLiteSingleFeatureLayer(
 
 OGRSQLiteSingleFeatureLayer::OGRSQLiteSingleFeatureLayer(
                                                      const char* pszLayerName,
-                                                     const char *pszVal )
+                                                     const char *pszValIn )
 {
     poFeatureDefn = new OGRFeatureDefn( "SELECT" );
     poFeatureDefn->Reference();
@@ -66,7 +67,7 @@ OGRSQLiteSingleFeatureLayer::OGRSQLiteSingleFeatureLayer(
 
     iNextShapeId = 0;
     nVal = 0;
-    this->pszVal = CPLStrdup(pszVal);
+    this->pszVal = CPLStrdup(pszValIn);
 }
 
 /************************************************************************/
