@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrsqlitesqlfunctions.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: ogrsqlitesqlfunctions.cpp 33714 2016-03-13 05:42:13Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Extension SQL functions
@@ -37,6 +37,7 @@
 #include "ogr_geocoding.h"
 
 #include "ogrsqliteregexp.cpp" /* yes the .cpp file, to make it work on Windows with load_extension('gdalXX.dll') */
+#include "swq.h"
 
 #ifndef HAVE_SPATIALITE
 #define MINIMAL_SPATIAL_FUNCTIONS
@@ -61,7 +62,7 @@ class OGRSQLiteExtensionData
 
     OGRGeocodingSessionH         GetGeocodingSession() { return hGeocodingSession; }
     void                         SetGeocodingSession(OGRGeocodingSessionH hGeocodingSessionIn) { hGeocodingSession = hGeocodingSessionIn; }
-    
+
     void                         SetRegExpCache(void* hRegExpCacheIn) { hRegExpCache = hRegExpCacheIn; }
 };
 
@@ -93,7 +94,7 @@ OGRSQLiteExtensionData::~OGRSQLiteExtensionData()
         delete oIter->second;
 
     OGRSQLiteFreeRegExpCache(hRegExpCache);
-    
+
     OGRGeocodeDestroySession(hGeocodingSession);
 }
 
@@ -191,11 +192,11 @@ void OGR2SQLITE_Transform(sqlite3_context* pContext,
     int nBLOBLen = sqlite3_value_bytes (argv[0]);
     OGRGeometry* poGeom = NULL;
     if( OGRSQLiteLayer::ImportSpatiaLiteGeometry(
-                    pabySLBLOB, nBLOBLen, &poGeom ) == CE_None &&
+                    pabySLBLOB, nBLOBLen, &poGeom ) == OGRERR_NONE &&
         poGeom->transform(poCT) == OGRERR_NONE &&
         OGRSQLiteLayer::ExportSpatiaLiteGeometry(
                     poGeom, nDstSRSId, wkbNDR, FALSE,
-                    FALSE, FALSE, &pabySLBLOB, &nBLOBLen ) == CE_None )
+                    FALSE, &pabySLBLOB, &nBLOBLen ) == OGRERR_NONE )
     {
         sqlite3_result_blob(pContext, pabySLBLOB, nBLOBLen, CPLFree);
     }
@@ -247,13 +248,13 @@ void OGR2SQLITE_ogr_deflate(sqlite3_context* pContext,
     }
     if( pOut != NULL )
     {
-        sqlite3_result_blob (pContext, pOut, nOutBytes, VSIFree);
+        sqlite3_result_blob (pContext, pOut, static_cast<int>(nOutBytes), VSIFree);
     }
     else
     {
         sqlite3_result_null (pContext);
     }
- 
+
     return;
 }
 
@@ -265,7 +266,7 @@ static
 void OGR2SQLITE_ogr_inflate(sqlite3_context* pContext,
                             int argc, sqlite3_value** argv)
 {
-    if( argc != 1 || 
+    if( argc != 1 ||
         sqlite3_value_type (argv[0]) != SQLITE_BLOB )
     {
         sqlite3_result_null (pContext);
@@ -281,7 +282,7 @@ void OGR2SQLITE_ogr_inflate(sqlite3_context* pContext,
 
     if( pOut != NULL )
     {
-        sqlite3_result_blob (pContext, pOut, nOutBytes, VSIFree);
+        sqlite3_result_blob (pContext, pOut, static_cast<int>(nOutBytes), VSIFree);
     }
     else
     {
@@ -316,9 +317,9 @@ void OGR2SQLITE_ogr_geocode_set_result(sqlite3_context* pContext,
             GByte* pabyGeomBLOB = NULL;
             int nGeomBLOBLen = 0;
             if( OGRSQLiteLayer::ExportSpatiaLiteGeometry(
-                    poFeature->GetGeometryRef(), 4326, wkbNDR, FALSE, FALSE, FALSE,
+                    poFeature->GetGeometryRef(), 4326, wkbNDR, FALSE, FALSE,
                     &pabyGeomBLOB,
-                    &nGeomBLOBLen ) != CE_None )
+                    &nGeomBLOBLen ) != OGRERR_NONE )
             {
                 sqlite3_result_null (pContext);
             }
@@ -334,6 +335,9 @@ void OGR2SQLITE_ogr_geocode_set_result(sqlite3_context* pContext,
             if( eType == OFTInteger )
                 sqlite3_result_int(pContext,
                                    poFeature->GetFieldAsInteger(nIdx));
+            else if( eType == OFTInteger64 )
+                sqlite3_result_int64(pContext,
+                                   poFeature->GetFieldAsInteger64(nIdx));
             else if( eType == OFTReal )
                 sqlite3_result_double(pContext,
                                       poFeature->GetFieldAsDouble(nIdx));
@@ -426,7 +430,7 @@ static double OGR2SQLITE_GetValAsDouble(sqlite3_value* val, int* pbGotVal)
 
         case SQLITE_INTEGER:
             if( pbGotVal ) *pbGotVal = TRUE;
-            return sqlite3_value_int64(val);
+            return (double) sqlite3_value_int64(val);
 
         default:
             if( pbGotVal ) *pbGotVal = FALSE;
@@ -439,7 +443,8 @@ static double OGR2SQLITE_GetValAsDouble(sqlite3_value* val, int* pbGotVal)
 /************************************************************************/
 
 static OGRGeometry* OGR2SQLITE_GetGeom(CPL_UNUSED sqlite3_context* pContext,
-                                       CPL_UNUSED int argc, sqlite3_value** argv,
+                                       CPL_UNUSED int argc,
+                                       sqlite3_value** argv,
                                        int* pnSRSId)
 {
     if( sqlite3_value_type (argv[0]) != SQLITE_BLOB )
@@ -451,8 +456,10 @@ static OGRGeometry* OGR2SQLITE_GetGeom(CPL_UNUSED sqlite3_context* pContext,
     int nBLOBLen = sqlite3_value_bytes (argv[0]);
     OGRGeometry* poGeom = NULL;
     if( OGRSQLiteLayer::ImportSpatiaLiteGeometry(
-                        pabySLBLOB, nBLOBLen, &poGeom, pnSRSId) != CE_None )
+                        pabySLBLOB, nBLOBLen, &poGeom, pnSRSId) != OGRERR_NONE )
     {
+        if( poGeom != NULL )
+            delete poGeom;
         return NULL;
     }
 
@@ -485,7 +492,7 @@ void OGR2SQLITE_ogr_geocode_reverse(sqlite3_context* pContext,
     {
         iAfterGeomIdx = 2;
     }
-    else if( argc >= 2 && 
+    else if( argc >= 2 &&
              sqlite3_value_type (argv[0]) == SQLITE_BLOB &&
              sqlite3_value_type (argv[1]) == SQLITE_TEXT )
     {
@@ -647,7 +654,7 @@ void OGR2SQLITE_ogr_GetConfigOption(sqlite3_context* pContext,
         sqlite3_result_null (pContext);
         return;
     }
-    
+
     const char* pszKey = (const char*)sqlite3_value_text(argv[0]);
     const char* pszVal = CPLGetConfigOption(pszKey, NULL);
     if( pszVal == NULL )
@@ -720,7 +727,7 @@ void OGR2SQLITE_ST_AsBinary(sqlite3_context* pContext,
     if( poGeom != NULL )
     {
         int nBLOBLen = poGeom->WkbSize();
-        GByte* pabyGeomBLOB = (GByte*) VSIMalloc(nBLOBLen);
+        GByte* pabyGeomBLOB = (GByte*) VSI_MALLOC_VERBOSE(nBLOBLen);
         if( pabyGeomBLOB != NULL )
         {
             if( poGeom->exportToWkb(wkbNDR, pabyGeomBLOB) == OGRERR_NONE )
@@ -750,8 +757,8 @@ static void OGR2SQLITE_SetGeom_AndDestroy(sqlite3_context* pContext,
     GByte* pabySLBLOB = NULL;
     int nBLOBLen = 0;
     if( poGeom != NULL && OGRSQLiteLayer::ExportSpatiaLiteGeometry(
-                    poGeom, nSRSId, wkbNDR, FALSE,
-                    FALSE, FALSE, &pabySLBLOB, &nBLOBLen ) == CE_None )
+                    poGeom, nSRSId, wkbNDR,
+                    FALSE, FALSE, &pabySLBLOB, &nBLOBLen ) == OGRERR_NONE )
     {
         sqlite3_result_blob(pContext, pabySLBLOB, nBLOBLen, CPLFree);
     }
@@ -1038,6 +1045,32 @@ void OGR2SQLITE_ST_MakePoint(sqlite3_context* pContext,
 
 #endif // #ifdef MINIMAL_SPATIAL_FUNCTIONS
 
+
+/************************************************************************/
+/*                     OGRSQLITE_hstore_get_value()                     */
+/************************************************************************/
+
+static
+void OGRSQLITE_hstore_get_value(sqlite3_context* pContext,
+                                CPL_UNUSED int argc,
+                                sqlite3_value** argv)
+{
+    if( sqlite3_value_type (argv[0]) != SQLITE_TEXT ||
+        sqlite3_value_type (argv[1]) != SQLITE_TEXT )
+    {
+        sqlite3_result_null (pContext);
+        return;
+    }
+
+    const char* pszHStore = (const char*)sqlite3_value_text(argv[0]);
+    const char* pszSearchedKey = (const char*)sqlite3_value_text(argv[1]);
+    char* pszValue = OGRHStoreGetValue(pszHStore, pszSearchedKey);
+    if( pszValue != NULL )
+        sqlite3_result_text( pContext, pszValue, -1, CPLFree );
+    else
+        sqlite3_result_null( pContext );
+}
+
 /************************************************************************/
 /*                   OGRSQLiteRegisterSQLFunctions()                    */
 /************************************************************************/
@@ -1089,15 +1122,19 @@ void* OGRSQLiteRegisterSQLFunctions(sqlite3* hDB)
     sqlite3_create_function(hDB, "Transform3", 3, SQLITE_ANY, pData,
                             OGR2SQLITE_Transform, NULL, NULL);
 
+    // HSTORE functions
+    sqlite3_create_function(hDB, "hstore_get_value", 2, SQLITE_ANY, NULL,
+                            OGRSQLITE_hstore_get_value, NULL, NULL);
+
 #ifdef MINIMAL_SPATIAL_FUNCTIONS
     /* Check if spatialite is available */
     int rc = sqlite3_exec(hDB, "SELECT spatialite_version()", NULL, NULL, NULL);
-    
+
     /* Reset error flag */
     sqlite3_exec(hDB, "SELECT 1", NULL, NULL, NULL);
-    
+
     if( rc != SQLITE_OK &&
-        CSLTestBoolean(CPLGetConfigOption("OGR_SQLITE_SPATIAL_FUNCTIONS", "YES")) )
+        CPLTestBool(CPLGetConfigOption("OGR_SQLITE_SPATIAL_FUNCTIONS", "YES")) )
     {
         CPLDebug("SQLITE",
                  "Spatialite not available. Implementing a few functions");
