@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrsqlitevfs.cpp 27741 2014-09-26 19:20:02Z goatbar $
+ * $Id: ogrsqlitevfs.cpp 33884 2016-04-03 18:50:15Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements SQLite VFS
@@ -30,7 +30,13 @@
 #include "cpl_atomic_ops.h"
 #include "ogr_sqlite.h"
 
-CPL_CVSID("$Id: ogrsqlitevfs.cpp 27741 2014-09-26 19:20:02Z goatbar $");
+CPL_CVSID("$Id: ogrsqlitevfs.cpp 33884 2016-04-03 18:50:15Z rouault $");
+
+#ifdef DEBUG_IO
+# define DEBUG_ONLY
+#else
+# define DEBUG_ONLY CPL_UNUSED
+#endif
 
 //#define DEBUG_IO 1
 
@@ -109,15 +115,7 @@ static int OGRSQLiteIOTruncate(sqlite3_file* pFile, sqlite3_int64 size)
     return (nRet == 0) ? SQLITE_OK : SQLITE_IOERR_TRUNCATE;
 }
 
-static int OGRSQLiteIOSync(
-#ifndef DEBUG_IO
-CPL_UNUSED 
-#endif
-                           sqlite3_file* pFile,
-#ifndef DEBUG_IO
-CPL_UNUSED 
-#endif
-                           int flags)
+static int OGRSQLiteIOSync(DEBUG_ONLY sqlite3_file* pFile, DEBUG_ONLY int flags)
 {
 #ifdef DEBUG_IO
     OGRSQLiteFileStruct* pMyFile = (OGRSQLiteFileStruct*) pFile;
@@ -139,15 +137,7 @@ static int OGRSQLiteIOFileSize(sqlite3_file* pFile, sqlite3_int64 *pSize)
     return SQLITE_OK;
 }
 
-static int OGRSQLiteIOLock(
-#ifndef DEBUG_IO
-CPL_UNUSED 
-#endif
-                           sqlite3_file* pFile,
-#ifndef DEBUG_IO
-CPL_UNUSED 
-#endif
-                           int flags)
+static int OGRSQLiteIOLock(DEBUG_ONLY sqlite3_file* pFile, DEBUG_ONLY int flags)
 {
 #ifdef DEBUG_IO
     OGRSQLiteFileStruct* pMyFile = (OGRSQLiteFileStruct*) pFile;
@@ -156,7 +146,7 @@ CPL_UNUSED
     return SQLITE_OK;
 }
 
-static int OGRSQLiteIOUnlock(CPL_UNUSED sqlite3_file* pFile, CPL_UNUSED int flags)
+static int OGRSQLiteIOUnlock(DEBUG_ONLY sqlite3_file* pFile, DEBUG_ONLY int flags)
 {
 #ifdef DEBUG_IO
     OGRSQLiteFileStruct* pMyFile = (OGRSQLiteFileStruct*) pFile;
@@ -165,15 +155,8 @@ static int OGRSQLiteIOUnlock(CPL_UNUSED sqlite3_file* pFile, CPL_UNUSED int flag
     return SQLITE_OK;
 }
 
-static int OGRSQLiteIOCheckReservedLock(
-#ifndef DEBUG_IO
-CPL_UNUSED 
-#endif
-                                        sqlite3_file* pFile,
-#ifndef DEBUG_IO
-CPL_UNUSED 
-#endif
-                                        int *pResOut)
+static int OGRSQLiteIOCheckReservedLock(DEBUG_ONLY sqlite3_file* pFile,
+                                        DEBUG_ONLY int *pResOut)
 {
 #ifdef DEBUG_IO
     OGRSQLiteFileStruct* pMyFile = (OGRSQLiteFileStruct*) pFile;
@@ -183,19 +166,9 @@ CPL_UNUSED
     return SQLITE_OK;
 }
 
-static int OGRSQLiteIOFileControl(
-#ifndef DEBUG_IO
-CPL_UNUSED 
-#endif
-                                  sqlite3_file* pFile,
-#ifndef DEBUG_IO
-CPL_UNUSED 
-#endif
-                                  int op,
-#ifndef DEBUG_IO
-CPL_UNUSED 
-#endif
-                                  void *pArg)
+static int OGRSQLiteIOFileControl(DEBUG_ONLY sqlite3_file* pFile,
+                           DEBUG_ONLY int op,
+                           DEBUG_ONLY void *pArg)
 {
 #ifdef DEBUG_IO
     OGRSQLiteFileStruct* pMyFile = (OGRSQLiteFileStruct*) pFile;
@@ -204,11 +177,7 @@ CPL_UNUSED
     return SQLITE_NOTFOUND;
 }
 
-static int OGRSQLiteIOSectorSize(
-#ifndef DEBUG_IO
-CPL_UNUSED 
-#endif
-                                 sqlite3_file* pFile)
+static int OGRSQLiteIOSectorSize(DEBUG_ONLY sqlite3_file* pFile)
 {
 #ifdef DEBUG_IO
     OGRSQLiteFileStruct* pMyFile = (OGRSQLiteFileStruct*) pFile;
@@ -217,11 +186,7 @@ CPL_UNUSED
     return 0;
 }
 
-static int OGRSQLiteIODeviceCharacteristics(
-#ifndef DEBUG_IO
-CPL_UNUSED 
-#endif
-                                            sqlite3_file* pFile)
+static int OGRSQLiteIODeviceCharacteristics(DEBUG_ONLY sqlite3_file* pFile)
 {
 #ifdef DEBUG_IO
     OGRSQLiteFileStruct* pMyFile = (OGRSQLiteFileStruct*) pFile;
@@ -244,15 +209,16 @@ static const sqlite3_io_methods OGRSQLiteIOMethods =
     OGRSQLiteIOCheckReservedLock,
     OGRSQLiteIOFileControl,
     OGRSQLiteIOSectorSize,
-    OGRSQLiteIODeviceCharacteristics
-#if 0
-    // TODO: These are in sqlite3.
-    , 0, // xShmMap
-    0, // xShmLock
-    0, // xShmBarrier
-    0, // xShmUnmap
-    0, // xFetch
-    0 // xUnfetch
+    OGRSQLiteIODeviceCharacteristics,
+#if SQLITE_VERSION_NUMBER >= 3007004L /* perhaps older too ? */
+    NULL,  // xShmMap
+    NULL,  // xShmLock
+    NULL,  // xShmBarrier
+    NULL,  // xShmUnmap
+#if SQLITE_VERSION_NUMBER >= 3008002L /* perhaps older too ? */
+    NULL,  // xFetch
+    NULL,  // xUnfetch
+#endif
 #endif
 };
 
@@ -310,9 +276,8 @@ static int OGRSQLiteVFSOpen(sqlite3_vfs* pVFS,
     return SQLITE_OK;
 }
 
-static int OGRSQLiteVFSDelete(CPL_UNUSED sqlite3_vfs* pVFS,
-                              const char *zName,
-                              CPL_UNUSED int syncDir)
+static int OGRSQLiteVFSDelete(DEBUG_ONLY sqlite3_vfs* pVFS,
+                           const char *zName, int DEBUG_ONLY syncDir)
 {
 #ifdef DEBUG_IO
     CPLDebug("SQLITE", "OGRSQLiteVFSDelete(%s)", zName);
@@ -321,7 +286,10 @@ static int OGRSQLiteVFSDelete(CPL_UNUSED sqlite3_vfs* pVFS,
     return SQLITE_OK;
 }
 
-static int OGRSQLiteVFSAccess (CPL_UNUSED sqlite3_vfs* pVFS, const char *zName, int flags, int *pResOut)
+static int OGRSQLiteVFSAccess (DEBUG_ONLY sqlite3_vfs* pVFS,
+                               const char *zName,
+                               int flags,
+                               int *pResOut)
 {
 #ifdef DEBUG_IO
     CPLDebug("SQLITE", "OGRSQLiteVFSAccess(%s, %d)", zName, flags);
@@ -330,11 +298,15 @@ static int OGRSQLiteVFSAccess (CPL_UNUSED sqlite3_vfs* pVFS, const char *zName,
     int nRet;
     if (flags == SQLITE_ACCESS_EXISTS)
     {
-        /* Do not try to check the presence of a journal on /vsicurl ! */
-        if ( strncmp(zName, "/vsicurl/", 9) == 0 &&
-             strlen(zName) > strlen("-journal") &&
-             strcmp(zName + strlen(zName) - strlen("-journal"), "-journal") == 0 )
+        /* Do not try to check the presence of a journal or a wal on /vsicurl ! */
+        if ( STARTS_WITH(zName, "/vsicurl/") &&
+             ((strlen(zName) > strlen("-journal") &&
+               strcmp(zName + strlen(zName) - strlen("-journal"), "-journal") == 0) ||
+              (strlen(zName) > strlen("-wal") &&
+               strcmp(zName + strlen(zName) - strlen("-wal"), "-wal") == 0)) )
+        {
             nRet = -1;
+        }
         else
             nRet = VSIStatExL(zName, &sStatBufL, VSI_STAT_EXISTS_FLAG);
     }
@@ -366,6 +338,16 @@ static int OGRSQLiteVFSFullPathname (sqlite3_vfs* pVFS, const char *zName, int n
 #endif
     if (zName[0] == '/')
     {
+        if( static_cast<int>(strlen( zName )) >= nOut )
+        {
+            // The +8 comes from the fact that sqlite3 does this check as
+            // it needs to be able to append .journal to the filename
+            CPLError(CE_Failure, CPLE_AppDefined,
+                     "Maximum pathname length reserved for SQLite3 VFS "
+                     "isn't large enough. Try raising OGR_SQLITE_VFS_MAXPATHNAME to at least %d",
+                     static_cast<int>(strlen(zName)) + 8);
+            return SQLITE_CANTOPEN;
+        }
         strncpy(zOut, zName, nOut);
         zOut[nOut-1] = '\0';
         return SQLITE_OK;
@@ -422,11 +404,51 @@ static int OGRSQLiteVFSSleep (sqlite3_vfs* pVFS, int microseconds)
     return pUnderlyingVFS->xSleep(pUnderlyingVFS, microseconds);
 }
 
-static int OGRSQLiteVFSCurrentTime (sqlite3_vfs* pVFS, double* p1)
+// Derived for sqlite3.c implementation of unixCurrentTime64 and winCurrentTime64
+#ifdef WIN32
+#include <windows.h>
+static int OGRSQLiteVFSCurrentTimeInt64 (sqlite3_vfs* /*pVFS*/, sqlite3_int64 *piNow)
 {
-    sqlite3_vfs* pUnderlyingVFS = GET_UNDERLYING_VFS(pVFS);
-    //CPLDebug("SQLITE", "OGRSQLiteVFSCurrentTime()");
-    return pUnderlyingVFS->xCurrentTime(pUnderlyingVFS, p1);
+    FILETIME ft;
+    static const sqlite3_int64 winFiletimeEpoch = 23058135*(sqlite3_int64)8640000;
+    static const sqlite3_int64 max32BitValue =
+      (sqlite3_int64)2000000000 + (sqlite3_int64)2000000000 +
+      (sqlite3_int64)294967296;
+
+#if defined(_WIN32_WCE)
+    SYSTEMTIME time;
+    GetSystemTime(&time);
+    /* if SystemTimeToFileTime() fails, it returns zero. */
+    if (!SystemTimeToFileTime(&time,&ft)){
+        return SQLITE_ERROR;
+    }
+#else
+    GetSystemTimeAsFileTime( &ft );
+#endif
+    *piNow = winFiletimeEpoch +
+            ((((sqlite3_int64)ft.dwHighDateTime)*max32BitValue) +
+               (sqlite3_int64)ft.dwLowDateTime)/(sqlite3_int64)10000;
+    return SQLITE_OK;
+}
+#else
+#include <sys/time.h>
+static int OGRSQLiteVFSCurrentTimeInt64 (sqlite3_vfs* /*pVFS*/, sqlite3_int64 *piNow)
+{
+    struct timeval sNow;
+    static const sqlite3_int64 unixEpoch = 24405875*(sqlite3_int64)8640000;
+    (void)gettimeofday(&sNow, NULL);  /* Cannot fail given valid arguments */
+    *piNow = unixEpoch + 1000*(sqlite3_int64)sNow.tv_sec + sNow.tv_usec/1000;
+
+    return SQLITE_OK;
+}
+#endif
+
+static int OGRSQLiteVFSCurrentTime (sqlite3_vfs* /*pVFS*/, double* p1)
+{
+    sqlite3_int64 i = 0;
+    int rc = OGRSQLiteVFSCurrentTimeInt64(NULL, &i);
+    *p1 = i/86400000.0;
+    return rc;
 }
 
 static int OGRSQLiteVFSGetLastError (sqlite3_vfs* pVFS, int p1, char *p2)
@@ -443,15 +465,20 @@ sqlite3_vfs* OGRSQLiteCreateVFS(pfnNotifyFileOpenedType pfn, void* pfnUserData)
 
     OGRSQLiteVFSAppDataStruct* pVFSAppData =
         (OGRSQLiteVFSAppDataStruct*) CPLCalloc(1, sizeof(OGRSQLiteVFSAppDataStruct));
-    sprintf(pVFSAppData->szVFSName, "OGRSQLITEVFS_%p", pVFSAppData);
+    snprintf(pVFSAppData->szVFSName, sizeof(pVFSAppData->szVFSName), "OGRSQLITEVFS_%p", pVFSAppData);
     pVFSAppData->pDefaultVFS = pDefaultVFS;
     pVFSAppData->pfn = pfn;
     pVFSAppData->pfnUserData = pfnUserData;
     pVFSAppData->nCounter = 0;
 
+#if SQLITE_VERSION_NUMBER >= 3008000L /* perhaps not the minimal version that defines xCurrentTimeInt64, but who cares */
+    pMyVFS->iVersion = 2;
+#else
     pMyVFS->iVersion = 1;
+#endif
     pMyVFS->szOsFile = sizeof(OGRSQLiteFileStruct);
-    pMyVFS->mxPathname = pDefaultVFS->mxPathname;
+    // must be large enough to hold potentially very long names like /vsicurl/.... with AWS S3 security tokens
+    pMyVFS->mxPathname = atoi(CPLGetConfigOption("OGR_SQLITE_VFS_MAXPATHNAME", "2048"));
     pMyVFS->zName = pVFSAppData->szVFSName;
     pMyVFS->pAppData = pVFSAppData;
     pMyVFS->xOpen = OGRSQLiteVFSOpen;
@@ -466,6 +493,11 @@ sqlite3_vfs* OGRSQLiteCreateVFS(pfnNotifyFileOpenedType pfn, void* pfnUserData)
     pMyVFS->xSleep = OGRSQLiteVFSSleep;
     pMyVFS->xCurrentTime = OGRSQLiteVFSCurrentTime;
     pMyVFS->xGetLastError = OGRSQLiteVFSGetLastError;
+#if SQLITE_VERSION_NUMBER >= 3008000L /* perhaps not the minimal version that defines xCurrentTimeInt64, but who cares */
+    if( pMyVFS->iVersion >= 2 )
+        pMyVFS->xCurrentTimeInt64 = OGRSQLiteVFSCurrentTimeInt64;
+#endif
+
     return pMyVFS;
 }
 
