@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrsualayer.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: ogrsualayer.cpp 31478 2015-11-14 19:13:16Z goatbar $
  *
  * Project:  SUA Translator
  * Purpose:  Implements OGRSUALayer class.
@@ -34,7 +34,7 @@
 #include "ogr_xplane_geo_utils.h"
 #include "ogr_srs_api.h"
 
-CPL_CVSID("$Id: ogrsualayer.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: ogrsualayer.cpp 31478 2015-11-14 19:13:16Z goatbar $");
 
 /************************************************************************/
 /*                            OGRSUALayer()                             */
@@ -51,6 +51,7 @@ OGRSUALayer::OGRSUALayer( VSILFILE* fp )
     poSRS = new OGRSpatialReference(SRS_WKT_WGS84);
 
     poFeatureDefn = new OGRFeatureDefn( "layer" );
+    SetDescription( poFeatureDefn->GetName() );
     poFeatureDefn->Reference();
     poFeatureDefn->SetGeomType( wkbPolygon );
     poFeatureDefn->GetGeomFieldDefn(0)->SetSpatialRef(poSRS);
@@ -105,7 +106,7 @@ OGRFeature *OGRSUALayer::GetNextFeature()
 {
     OGRFeature  *poFeature;
 
-    while(TRUE)
+    while( true )
     {
         poFeature = GetNextRawFeature();
         if (poFeature == NULL)
@@ -186,7 +187,7 @@ OGRFeature *OGRSUALayer::GetNextRawFeature()
     if (bEOF)
         return NULL;
 
-    while(TRUE)
+    while( true )
     {
         if (bFirst && bHasLastLine)
         {
@@ -210,29 +211,29 @@ OGRFeature *OGRSUALayer::GetNextRawFeature()
         if (pszLine[0] == '#' || pszLine[0] == '\0')
             continue;
 
-        if (EQUALN(pszLine, "TYPE=", 5))
+        if (STARTS_WITH_CI(pszLine, "TYPE="))
         {
             if (osTYPE.size() != 0)
                 break;
             osTYPE = pszLine + 5;
         }
-        else if (EQUALN(pszLine, "CLASS=", 6))
+        else if (STARTS_WITH_CI(pszLine, "CLASS="))
         {
             if (osCLASS.size() != 0)
                 break;
             osCLASS = pszLine + 6;
         }
-        else if (EQUALN(pszLine, "TITLE=", 6))
+        else if (STARTS_WITH_CI(pszLine, "TITLE="))
         {
             if (osTITLE.size() != 0)
                 break;
             osTITLE = pszLine + 6;
         }
-        else if (EQUALN(pszLine, "TOPS=", 5))
+        else if (STARTS_WITH_CI(pszLine, "TOPS="))
             osTOPS = pszLine + 5;
-        else if (EQUALN(pszLine, "BASE=", 5))
+        else if (STARTS_WITH_CI(pszLine, "BASE="))
             osBASE = pszLine + 5;
-        else if (EQUALN(pszLine, "POINT=", 6))
+        else if (STARTS_WITH_CI(pszLine, "POINT="))
         {
             pszLine += 6;
             if (strlen(pszLine) != 16)
@@ -246,17 +247,17 @@ OGRFeature *OGRSUALayer::GetNextRawFeature()
             dfLastLat = dfLat;
             dfLastLon = dfLon;
         }
-        else if (EQUALN(pszLine, "CLOCKWISE", 9) || EQUALN(pszLine, "ANTI-CLOCKWISE", 14))
+        else if (STARTS_WITH_CI(pszLine, "CLOCKWISE") || STARTS_WITH_CI(pszLine, "ANTI-CLOCKWISE"))
         {
             if (oLR.getNumPoints() == 0)
                 continue;
 
-            int bClockWise = EQUALN(pszLine, "CLOCKWISE", 9);
+            int bClockWise = STARTS_WITH_CI(pszLine, "CLOCKWISE");
 
             /*const char* pszRADIUS = strstr(pszLine, "RADIUS=");
             if (pszRADIUS == NULL)
                 continue;
-            double dfRADIUS = atof(pszRADIUS + 7) * 1852;*/
+            double dfRADIUS = CPLAtof(pszRADIUS + 7) * 1852;*/
 
             const char* pszCENTRE = strstr(pszLine, "CENTRE=");
             if (pszCENTRE == NULL)
@@ -302,12 +303,12 @@ OGRFeature *OGRSUALayer::GetNextRawFeature()
             dfLastLat = oLR.getY(oLR.getNumPoints() - 1);
             dfLastLon = oLR.getX(oLR.getNumPoints() - 1);
         }
-        else if (EQUALN(pszLine, "CIRCLE", 6))
+        else if (STARTS_WITH_CI(pszLine, "CIRCLE"))
         {
             const char* pszRADIUS = strstr(pszLine, "RADIUS=");
             if (pszRADIUS == NULL)
                 continue;
-            double dfRADIUS = atof(pszRADIUS + 7) * 1852;
+            double dfRADIUS = CPLAtof(pszRADIUS + 7) * 1852;
 
             const char* pszCENTRE = strstr(pszLine, "CENTRE=");
             if (pszCENTRE == NULL)
@@ -332,7 +333,7 @@ OGRFeature *OGRSUALayer::GetNextRawFeature()
             dfLastLat = oLR.getY(oLR.getNumPoints() - 1);
             dfLastLon = oLR.getX(oLR.getNumPoints() - 1);
         }
-        else if (EQUALN(pszLine, "INCLUDE", 7) || EQUALN(pszLine, "END", 3))
+        else if (STARTS_WITH_CI(pszLine, "INCLUDE") || STARTS_WITH_CI(pszLine, "END"))
         {
         }
         else
@@ -365,4 +366,3 @@ int OGRSUALayer::TestCapability( CPL_UNUSED const char * pszCap )
 {
     return FALSE;
 }
-
