@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_svg.h 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: ogr_svg.h 32177 2015-12-14 07:25:30Z goatbar $
  *
  * Project:  SVG Translator
  * Purpose:  Definition of classes for OGR .svg driver.
@@ -27,8 +27,8 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _OGR_SVG_H_INCLUDED
-#define _OGR_SVG_H_INCLUDED
+#ifndef OGR_SVG_H_INCLUDED
+#define OGR_SVG_H_INCLUDED
 
 #include "ogrsf_frmts.h"
 
@@ -55,7 +55,7 @@ class OGRSVGLayer : public OGRLayer
     OGRSpatialReference *poSRS;
     OGRSVGDataSource*  poDS;
     CPLString          osLayerName;
-    
+
     SVGGeometryType    svgGeomType;
 
     int                nTotalFeatures;
@@ -80,10 +80,12 @@ class OGRSVGLayer : public OGRLayer
     int                inInterestingElement;
 
     int                bStopParsing;
+#ifdef HAVE_EXPAT
     int                nWithoutEventCounter;
     int                nDataHandlerCounter;
 
     OGRSVGLayer       *poCurLayer;
+#endif
 
   private:
     void               LoadSchema();
@@ -101,17 +103,17 @@ class OGRSVGLayer : public OGRLayer
     virtual const char*         GetName() { return osLayerName.c_str(); }
     virtual OGRwkbGeometryType  GetGeomType();
 
-    virtual int                 GetFeatureCount( int bForce = TRUE );
+    virtual GIntBig             GetFeatureCount( int bForce = TRUE );
 
     virtual OGRFeatureDefn *    GetLayerDefn();
-    
+
     virtual int                 TestCapability( const char * );
 
 #ifdef HAVE_EXPAT
     void                startElementCbk(const char *pszName, const char **ppszAttr);
     void                endElementCbk(const char *pszName);
     void                dataHandlerCbk(const char *data, int nLen);
-    
+
     void                startElementLoadSchemaCbk(const char *pszName, const char **ppszAttr);
     void                endElementLoadSchemaCbk(const char *pszName);
     void                dataHandlerLoadSchemaCbk(const char *data, int nLen);
@@ -136,10 +138,9 @@ class OGRSVGDataSource : public OGRDataSource
     OGRSVGLayer**       papoLayers;
     int                 nLayers;
 
+#ifdef HAVE_EXPAT
     OGRSVGValidity      eValidity;
     int                 bIsCloudmade;
-
-#ifdef HAVE_EXPAT
     XML_Parser          oCurrentParser;
     int                 nDataHandlerCounter;
 #endif
@@ -148,8 +149,7 @@ class OGRSVGDataSource : public OGRDataSource
                         OGRSVGDataSource();
                         ~OGRSVGDataSource();
 
-    int                 Open( const char * pszFilename,
-                              int bUpdate );
+    int                 Open( const char * pszFilename );
 
     virtual const char*         GetName() { return pszName; }
 
@@ -158,26 +158,11 @@ class OGRSVGDataSource : public OGRDataSource
 
     virtual int                 TestCapability( const char * );
 
-    
+
 #ifdef HAVE_EXPAT
     void                startElementValidateCbk(const char *pszName, const char **ppszAttr);
     void                dataHandlerValidateCbk(const char *data, int nLen);
 #endif
 };
 
-/************************************************************************/
-/*                             OGRSVGDriver                             */
-/************************************************************************/
-
-class OGRSVGDriver : public OGRSFDriver
-{
-  public:
-                ~OGRSVGDriver();
-
-    const char*         GetName();
-    OGRDataSource*      Open( const char *, int );
-
-    virtual int                 TestCapability( const char * );
-};
-
-#endif /* ndef _OGR_SVG_H_INCLUDED */
+#endif /* ndef OGR_SVG_H_INCLUDED */
