@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrsvglayer.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: ogrsvglayer.cpp 32177 2015-12-14 07:25:30Z goatbar $
  *
  * Project:  SVG Translator
  * Purpose:  Implements OGRSVGLayer class.
@@ -30,7 +30,7 @@
 #include "ogr_svg.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrsvglayer.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: ogrsvglayer.cpp 32177 2015-12-14 07:25:30Z goatbar $");
 
 /************************************************************************/
 /*                            OGRSVGLayer()                             */
@@ -38,26 +38,41 @@ CPL_CVSID("$Id: ogrsvglayer.cpp 27729 2014-09-24 00:40:16Z goatbar $");
 
 OGRSVGLayer::OGRSVGLayer( const char* pszFilename,
                           const char* pszLayerName,
-                          SVGGeometryType svgGeomType,
-                          OGRSVGDataSource* poDS)
+                          SVGGeometryType svgGeomTypeIn,
+                          OGRSVGDataSource* poDSIn) :
+    poFeatureDefn(NULL),
+    poSRS(NULL),
+    poDS(NULL),
+    nTotalFeatures(0),
+    nNextFID(0),
+    fpSVG(NULL),
+#ifdef HAVE_EXPAT
+    oParser(NULL),
+    oSchemaParser(NULL),
+#endif
+    pszSubElementValue(NULL),
+    nSubElementValueLen(0),
+    iCurrentField(0),
+    poFeature(NULL),
+    ppoFeatureTab(NULL),
+    nFeatureTabLength(0),
+    nFeatureTabIndex(0),
+    depthLevel(0),
+    interestingDepthLevel(0),
+    inInterestingElement(FALSE),
+    bStopParsing(FALSE)
+#ifdef HAVE_EXPAT
+        ,
+    nWithoutEventCounter(0),
+    nDataHandlerCounter(0),
+    poCurLayer(NULL)
+#endif
 
 {
-    nNextFID = 0;
-
-    this->poDS = poDS;
-    this->svgGeomType = svgGeomType;
+    this->poDS = poDSIn;
+    this->svgGeomType = svgGeomTypeIn;
     osLayerName = pszLayerName;
-
-    poFeatureDefn = NULL;
-
-    nTotalFeatures = 0;
-
-    ppoFeatureTab = NULL;
-    nFeatureTabIndex = 0;
-    nFeatureTabLength = 0;
-    pszSubElementValue = NULL;
-    nSubElementValueLen = 0;
-    bStopParsing = FALSE;
+    SetDescription( pszLayerName );
 
     poSRS = new OGRSpatialReference("PROJCS[\"WGS 84 / Pseudo-Mercator\","
     "GEOGCS[\"WGS 84\","
@@ -82,12 +97,6 @@ OGRSVGLayer::OGRSVGLayer( const char* pszFilename,
     "AXIS[\"X\",EAST],"
     "AXIS[\"Y\",NORTH]]");
 
-    poFeature = NULL;
-
-#ifdef HAVE_EXPAT
-    oParser = NULL;
-#endif
-
     fpSVG = VSIFOpenL( pszFilename, "r" );
     if( fpSVG == NULL )
     {
@@ -111,7 +120,7 @@ OGRSVGLayer::~OGRSVGLayer()
 #endif
     if (poFeatureDefn)
         poFeatureDefn->Release();
-    
+
     if( poSRS != NULL )
         poSRS->Release();
 
@@ -225,7 +234,7 @@ static void OGRSVGParseD(OGRLineString* poLS, const char* pszD)
     int bRelativeLineto = FALSE;
     double dfX = 0, dfY = 0;
     int nPointCount = 0;
-    while(TRUE)
+    while( true )
     {
         ch = *(pszIter ++);
 
@@ -416,7 +425,7 @@ void OGRSVGLayer::startElementCbk(const char *pszName, const char **ppszAttr)
     }
     else if (inInterestingElement &&
              depthLevel == interestingDepthLevel + 1 &&
-             strncmp(pszName, "cm:", 3) == 0)
+             STARTS_WITH(pszName, "cm:"))
     {
         iCurrentField = poFeatureDefn->GetFieldIndex(pszName + 3);
     }
@@ -497,11 +506,10 @@ void OGRSVGLayer::dataHandlerCbk(const char *data, int nLen)
 
     if (iCurrentField >= 0)
     {
-        char* pszNewSubElementValue = (char*) VSIRealloc(pszSubElementValue,
+        char* pszNewSubElementValue = (char*) VSI_REALLOC_VERBOSE(pszSubElementValue,
                                            nSubElementValueLen + nLen + 1);
         if (pszNewSubElementValue == NULL)
         {
-            CPLError(CE_Failure, CPLE_OutOfMemory, "Out of memory");
             XML_StopParser(oParser, XML_FALSE);
             bStopParsing = TRUE;
             return;
@@ -539,12 +547,12 @@ OGRFeature *OGRSVGLayer::GetNextFeature()
     {
         return ppoFeatureTab[nFeatureTabIndex++];
     }
-    
+
     if (VSIFEofL(fpSVG))
         return NULL;
-    
+
     char aBuf[BUFSIZ];
-    
+
     CPLFree(ppoFeatureTab);
     ppoFeatureTab = NULL;
     nFeatureTabLength = 0;
@@ -733,7 +741,7 @@ void OGRSVGLayer::startElementLoadSchemaCbk(const char *pszName,
     else if (inInterestingElement)
     {
         if (depthLevel == interestingDepthLevel + 1 &&
-            strncmp(pszName, "cm:", 3) == 0)
+            STARTS_WITH(pszName, "cm:"))
         {
             pszName += 3;
             if (poCurLayer->poFeatureDefn->GetFieldIndex(pszName) < 0)
@@ -777,7 +785,8 @@ void OGRSVGLayer::endElementLoadSchemaCbk(CPL_UNUSED const char *pszName)
 /*                   dataHandlerLoadSchemaCbk()                         */
 /************************************************************************/
 
-void OGRSVGLayer::dataHandlerLoadSchemaCbk(CPL_UNUSED const char *data, CPL_UNUSED int nLen)
+void OGRSVGLayer::dataHandlerLoadSchemaCbk(CPL_UNUSED const char *data,
+                                           CPL_UNUSED int nLen)
 {
     if (bStopParsing) return;
 
@@ -831,7 +840,7 @@ OGRwkbGeometryType OGRSVGLayer::GetGeomType()
 /*                           GetGeomType()                              */
 /************************************************************************/
 
-int OGRSVGLayer::GetFeatureCount( int bForce )
+GIntBig OGRSVGLayer::GetFeatureCount( int bForce )
 {
     if (m_poAttrQuery != NULL || m_poFilterGeom != NULL)
         return OGRLayer::GetFeatureCount(bForce);
