@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tigeraltname.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: tigeraltname.cpp 32177 2015-12-14 07:25:30Z goatbar $
  *
  * Project:  TIGER/Line Translator
  * Purpose:  Implements TigerAltName, providing access to RT4 files.
@@ -30,7 +30,7 @@
 #include "ogr_tiger.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: tigeraltname.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: tigeraltname.cpp 32177 2015-12-14 07:25:30Z goatbar $");
 
 #define FILE_CODE "4"
 
@@ -58,7 +58,8 @@ static const TigerRecordInfo rt4_info =
 /************************************************************************/
 
 TigerAltName::TigerAltName( OGRTigerDataSource * poDSIn,
-                            CPL_UNUSED const char * pszPrototypeModule ) : TigerFileBase(&rt4_info, FILE_CODE)
+                            CPL_UNUSED const char * pszPrototypeModule ) :
+    TigerFileBase(&rt4_info, FILE_CODE)
 {
     OGRFieldDefn        oField("",OFTInteger);
 
@@ -160,8 +161,8 @@ OGRErr TigerAltName::CreateFeature( OGRFeature *poFeature )
     for( int i = 0; i < nValueCount; i++ )
     {
         char    szWork[9];
-        
-        sprintf( szWork, "%8d", panValue[i] );
+
+        snprintf( szWork, sizeof(szWork), "%8d", panValue[i] );
         strncpy( szRecord + 18 + 8 * i, szWork, 8 );
     }
 
