@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tigercompletechain.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: tigercompletechain.cpp 33706 2016-03-11 13:33:27Z goatbar $
  *
  * Project:  TIGER/Line Translator
  * Purpose:  Implements TigerCompleteChain, providing access to RT1 and
@@ -31,7 +31,7 @@
 #include "ogr_tiger.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: tigercompletechain.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: tigercompletechain.cpp 33706 2016-03-11 13:33:27Z goatbar $");
 
 static const TigerFieldInfo rt1_2002_fields[] = {
   // fieldname    fmt  type OFTType      beg  end  len  bDefine bSet bWrite
@@ -125,7 +125,7 @@ static const TigerFieldInfo rt1_fields[] = {
   { "CTBNAR",     'L', 'N', OFTInteger,  177, 182,   6,       1,   1,     1 },
   { "BLKL",       'L', 'N', OFTString,   183, 186,   4,       1,   1,     1 },
   { "BLKR",       'L', 'N', OFTString,   187, 190,   4,       1,   1,     1 }
-}; 
+};
 static const TigerRecordInfo rt1_info =
   {
     rt1_fields,
@@ -198,7 +198,7 @@ static const TigerFieldInfo rt3_fields[] = {
   { "BLK90R",     'L', 'A', OFTString,     74,  77,   4,       1,   1,     1 },
   { "AIRL",       'L', 'N', OFTInteger,    78,  81,   4,       1,   1,     1 },
   { "AIRR",       'L', 'N', OFTInteger,    82,  85,   4,       1,   1,     1 },
- 
+
   { "VTDL",       'L', 'A', OFTString,    104, 107,   4,       1,   1,     1 },
   { "VTDR",       'L', 'A', OFTString,    108, 111,   4,       1,   1,     1 }
 
@@ -245,7 +245,7 @@ TigerCompleteChain::TigerCompleteChain( OGRTigerDataSource * poDSIn,
 
     panShapeRecordId = NULL;
     fpShape = NULL;
-    
+
     /* -------------------------------------------------------------------- */
     /*      Fields from type 1 record.                                      */
     /* -------------------------------------------------------------------- */
@@ -281,10 +281,10 @@ TigerCompleteChain::~TigerCompleteChain()
 /*                             SetModule()                              */
 /************************************************************************/
 
-int TigerCompleteChain::SetModule( const char * pszModule )
+int TigerCompleteChain::SetModule( const char * pszModuleIn )
 
 {
-    if( !OpenFile( pszModule, "1" ) )
+    if( !OpenFile( pszModuleIn, "1" ) )
         return FALSE;
 
     EstablishFeatureCount();
@@ -295,20 +295,20 @@ int TigerCompleteChain::SetModule( const char * pszModule )
 /*      first record.                                                   */
 /* -------------------------------------------------------------------- */
     nRT1RecOffset = 0;
-    if( pszModule )
+    if( pszModuleIn )
     {
         char achHeader[10];
-        
+
         VSIFSeekL( fpPrimary, 0, SEEK_SET );
         VSIFReadL( achHeader, sizeof(achHeader), 1, fpPrimary );
-        
-        if( EQUALN(achHeader,"Copyright",8) )
+
+        if( STARTS_WITH_CI(achHeader,"Copyright") )
         {
             nRT1RecOffset = 1;
             nFeatures--;
         }
     }
-        
+
 /* -------------------------------------------------------------------- */
 /*      Open the RT3 file                                               */
 /* -------------------------------------------------------------------- */
@@ -320,18 +320,18 @@ int TigerCompleteChain::SetModule( const char * pszModule )
             fpRT3 = NULL;
         }
 
-        if( pszModule )
+        if( pszModuleIn )
         {
             char        *pszFilename;
-        
-            pszFilename = poDS->BuildFilename( pszModule, "3" );
+
+            pszFilename = poDS->BuildFilename( pszModuleIn, "3" );
 
             fpRT3 = VSIFOpenL( pszFilename, "rb" );
 
             CPLFree( pszFilename );
         }
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Close the shape point file, if open and free the list of        */
 /*      record ids.                                                     */
@@ -341,21 +341,21 @@ int TigerCompleteChain::SetModule( const char * pszModule )
         VSIFCloseL( fpShape );
         fpShape = NULL;
     }
-    
+
     CPLFree( panShapeRecordId );
     panShapeRecordId = NULL;
 
 /* -------------------------------------------------------------------- */
 /*      Try to open the RT2 file corresponding to this RT1 file.        */
 /* -------------------------------------------------------------------- */
-    if( pszModule != NULL )
+    if( pszModuleIn != NULL )
     {
         char    *pszFilename;
 
-        pszFilename = poDS->BuildFilename( pszModule, "2" );
+        pszFilename = poDS->BuildFilename( pszModuleIn, "2" );
 
         fpShape = VSIFOpenL( pszFilename, "rb" );
-        
+
         if( fpShape == NULL )
         {
             if( nRT1RecOffset == 0 )
@@ -364,8 +364,8 @@ int TigerCompleteChain::SetModule( const char * pszModule )
                           pszFilename );
         }
         else
-            panShapeRecordId = (int *)CPLCalloc(sizeof(int),GetFeatureCount());
-        
+            panShapeRecordId = (int *)CPLCalloc(sizeof(int),(size_t)GetFeatureCount());
+
         CPLFree( pszFilename );
     }
 
@@ -395,7 +395,7 @@ OGRFeature *TigerCompleteChain::GetFeature( int nRecordId )
     if( fpPrimary == NULL )
         return NULL;
 
-    if( VSIFSeekL( fpPrimary, (nRecordId+nRT1RecOffset) * nRecordLength, 
+    if( VSIFSeekL( fpPrimary, (nRecordId+nRT1RecOffset) * nRecordLength,
                   SEEK_SET ) != 0 )
     {
         CPLError( CE_Failure, CPLE_FileIO,
@@ -465,7 +465,7 @@ OGRFeature *TigerCompleteChain::GetFeature( int nRecordId )
         delete poFeature;
         return NULL;
     }
-    
+
     poLine->addPoint(atoi(GetField(achRecord, 210, 219)) / 1000000.0,
                      atoi(GetField(achRecord, 220, 228)) / 1000000.0 );
 
@@ -482,13 +482,14 @@ OGRFeature *TigerCompleteChain::GetFeature( int nRecordId )
 /************************************************************************/
 
 int TigerCompleteChain::AddShapePoints( int nTLID, int nRecordId,
-                                        OGRLineString * poLine, CPL_UNUSED int nSeqNum )
+                                        OGRLineString * poLine,
+                                        CPL_UNUSED int nSeqNum )
 {
     int         nShapeRecId;
 
     nShapeRecId = GetShapeRecordId( nRecordId, nTLID );
 
-    // -2 means an error occured.
+    // -2 means an error occurred.
     if( nShapeRecId == -2 )
         return FALSE;
 
@@ -502,7 +503,7 @@ int TigerCompleteChain::AddShapePoints( int nTLID, int nRecordId,
     char        achShapeRec[OGR_TIGER_RECBUF_LEN];
     int         nShapeRecLen = psRT2Info->nRecordLength + nRecordLength - psRT1Info->nRecordLength;
 
-    for( ; TRUE; nShapeRecId++ )
+    for( ; true; nShapeRecId++ )
     {
         int  nBytesRead = 0;
 
@@ -515,15 +516,15 @@ int TigerCompleteChain::AddShapePoints( int nTLID, int nRecordId,
             return FALSE;
         }
 
-        nBytesRead = VSIFReadL( achShapeRec, 1, psRT2Info->nRecordLength, 
-                                fpShape );
+        nBytesRead = static_cast<int>(VSIFReadL( achShapeRec, 1, psRT2Info->nRecordLength,
+                                fpShape ));
 
-        /* 
+        /*
         ** Handle case where the last record in the file is full.  We will
         ** try to read another record but not find it.  We require that we
-        ** have found at least one shape record for this case though. 
+        ** have found at least one shape record for this case though.
         */
-        if( nBytesRead <= 0 && VSIFEofL( fpShape ) 
+        if( nBytesRead <= 0 && VSIFEofL( fpShape )
             && poLine->getNumPoints() > 0 )
             break;
 
@@ -580,19 +581,19 @@ int TigerCompleteChain::GetShapeRecordId( int nChainId, int nTLID )
 
     if( fpShape == NULL || panShapeRecordId == NULL )
         return -1;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Do we already have the answer?                                  */
 /* -------------------------------------------------------------------- */
     if( panShapeRecordId[nChainId] != 0 )
         return panShapeRecordId[nChainId];
-    
+
 /* -------------------------------------------------------------------- */
 /*      If we don't already have this value, then search from the       */
 /*      previous known record.                                          */
 /* -------------------------------------------------------------------- */
     int iTestChain, nWorkingRecId;
-        
+
     for( iTestChain = nChainId-1;
          iTestChain >= 0 && panShapeRecordId[iTestChain] <= 0;
          iTestChain-- ) {}
@@ -672,7 +673,7 @@ int TigerCompleteChain::GetShapeRecordId( int nChainId, int nTLID )
 /************************************************************************/
 /*                           SetWriteModule()                           */
 /************************************************************************/
-int TigerCompleteChain::SetWriteModule( const char *pszFileCode, int nRecLen, 
+int TigerCompleteChain::SetWriteModule( const char *pszFileCode, int nRecLen,
                                         OGRFeature *poFeature )
 
 {
@@ -696,7 +697,7 @@ int TigerCompleteChain::SetWriteModule( const char *pszFileCode, int nRecLen,
         if( pszModule )
         {
             char        *pszFilename;
-        
+
             pszFilename = poDS->BuildFilename( pszModule, "3" );
 
             fpRT3 = VSIFOpenL( pszFilename, "ab" );
@@ -704,7 +705,7 @@ int TigerCompleteChain::SetWriteModule( const char *pszFileCode, int nRecLen,
             CPLFree( pszFilename );
         }
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Close the shape point file, if open and free the list of        */
 /*      record ids.                                                     */
@@ -714,15 +715,15 @@ int TigerCompleteChain::SetWriteModule( const char *pszFileCode, int nRecLen,
         VSIFCloseL( fpShape );
         fpShape = NULL;
     }
-    
+
     if( pszModule )
     {
         char        *pszFilename;
-        
+
         pszFilename = poDS->BuildFilename( pszModule, "2" );
-        
+
         fpShape = VSIFOpenL( pszFilename, "ab" );
-        
+
         CPLFree( pszFilename );
     }
 
@@ -739,7 +740,7 @@ OGRErr TigerCompleteChain::CreateFeature( OGRFeature *poFeature )
     char        szRecord[OGR_TIGER_RECBUF_LEN];
     OGRLineString *poLine = (OGRLineString *) poFeature->GetGeometryRef();
 
-    if( poLine == NULL 
+    if( poLine == NULL
         || (poLine->getGeometryType() != wkbLineString
             && poLine->getGeometryType() != wkbLineString25D) )
         return OGRERR_FAILURE;
@@ -752,8 +753,8 @@ OGRErr TigerCompleteChain::CreateFeature( OGRFeature *poFeature )
     memset( szRecord, ' ', psRT1Info->nRecordLength );
     WriteFields( psRT1Info, poFeature, szRecord );
     WritePoint( szRecord, 191, poLine->getX(0), poLine->getY(0) );
-    WritePoint( szRecord, 210, 
-                poLine->getX(poLine->getNumPoints()-1), 
+    WritePoint( szRecord, 210,
+                poLine->getX(poLine->getNumPoints()-1),
                 poLine->getY(poLine->getNumPoints()-1) );
     WriteRecord( szRecord, psRT1Info->nRecordLength, "1" );
 
@@ -782,21 +783,21 @@ OGRErr TigerCompleteChain::CreateFeature( OGRFeature *poFeature )
             memset( szRecord, ' ', psRT2Info->nRecordLength );
 
             WriteField( poFeature, "TLID", szRecord, 6, 15, 'R', 'N' );
-            
-            sprintf( szTemp, "%3d", nRTSQ );
+
+            snprintf( szTemp, sizeof(szTemp), "%3d", nRTSQ );
             strncpy( ((char *)szRecord) + 15, szTemp, 4 );
 
             for( i = 0; i < 10; i++ )
             {
                 if( iPoint < nPoints-1 )
-                    WritePoint( szRecord, 19+19*i, 
+                    WritePoint( szRecord, 19+19*i,
                                 poLine->getX(iPoint), poLine->getY(iPoint) );
                 else
                     WritePoint( szRecord, 19+19*i, 0.0, 0.0 );
 
                 iPoint++;
             }
-            
+
             WriteRecord( szRecord, psRT2Info->nRecordLength, "2", fpShape );
 
             nRTSQ++;
