@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tigerfilebase.cpp 23871 2012-02-02 03:24:07Z warmerdam $
+ * $Id: tigerfilebase.cpp 33706 2016-03-11 13:33:27Z goatbar $
  *
  * Project:  TIGER/Line Translator
  * Purpose:  Implements TigerBaseFile class, providing common services to all
@@ -8,6 +8,7 @@
  *
  ******************************************************************************
  * Copyright (c) 1999, Frank Warmerdam
+ * Copyright (c) 2009-2013, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -33,27 +34,26 @@
 #include "cpl_error.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: tigerfilebase.cpp 23871 2012-02-02 03:24:07Z warmerdam $");
+CPL_CVSID("$Id: tigerfilebase.cpp 33706 2016-03-11 13:33:27Z goatbar $");
 
 /************************************************************************/
 /*                           TigerFileBase()                            */
 /************************************************************************/
 
 TigerFileBase::TigerFileBase( const TigerRecordInfo *psRTInfoIn,
-                              const char            *m_pszFileCodeIn )
-
-{
-    pszShortModule = NULL;
-    pszModule = NULL;
-    fpPrimary = NULL;
-    poFeatureDefn = NULL;
-    nFeatures = 0;
-    nVersionCode = 0;
-    nVersion = TIGER_Unknown;
-
-    psRTInfo = psRTInfoIn;
-    m_pszFileCode = m_pszFileCodeIn;
-}
+                              const char            *m_pszFileCodeIn ) :
+    poDS(NULL),
+    pszModule(NULL),
+    pszShortModule(NULL),
+    fpPrimary(NULL),
+    poFeatureDefn(NULL),
+    nFeatures(0),
+    nRecordLength(0),
+    nVersionCode(0),
+    nVersion(TIGER_Unknown),
+    psRTInfo(psRTInfoIn),
+    m_pszFileCode(m_pszFileCodeIn)
+{ }
 
 /************************************************************************/
 /*                           ~TigerFileBase()                           */
@@ -92,7 +92,7 @@ int TigerFileBase::OpenFile( const char * pszModuleToOpen,
     pszModule = NULL;
     CPLFree( pszShortModule );
     pszShortModule = NULL;
-    
+
     if( fpPrimary != NULL )
     {
         VSIFCloseL( fpPrimary );
@@ -153,7 +153,7 @@ int TigerFileBase::EstablishRecordLength( VSILFILE * fp )
 {
     char        chCurrent;
     int         nRecLen = 0;
-    
+
     if( fp == NULL || VSIFSeekL( fp, 0, SEEK_SET ) != 0 )
         return -1;
 
@@ -175,7 +175,7 @@ int TigerFileBase::EstablishRecordLength( VSILFILE * fp )
     {
         return -1;
     }
-    
+
     nRecLen++; /* for the 10 or 13 we encountered */
 
 /* -------------------------------------------------------------------- */
@@ -217,12 +217,12 @@ void TigerFileBase::EstablishFeatureCount()
 /*      (including line terminators).  Get the total file size, and     */
 /*      divide by this length to get the presumed number of records.    */
 /* -------------------------------------------------------------------- */
-    long        nFileSize;
-    
+    vsi_l_offset        nFileSize;
+
     VSIFSeekL( fpPrimary, 0, SEEK_END );
     nFileSize = VSIFTellL( fpPrimary );
 
-    if( (nFileSize % nRecordLength) != 0 )
+    if( (nFileSize % (vsi_l_offset)nRecordLength) != 0 )
     {
         CPLError( CE_Warning, CPLE_FileIO,
                   "TigerFileBase::EstablishFeatureCount(): "
@@ -230,7 +230,10 @@ void TigerFileBase::EstablishFeatureCount()
                   (int) nFileSize, (int) nRecordLength );
     }
 
-    nFeatures = nFileSize / nRecordLength;
+    if( nFileSize / (vsi_l_offset)nRecordLength > (vsi_l_offset)INT_MAX )
+        nFeatures = INT_MAX;
+    else
+        nFeatures = static_cast<int>(nFileSize / (vsi_l_offset)nRecordLength);
 }
 
 /************************************************************************/
@@ -243,7 +246,7 @@ const char* TigerFileBase::GetField( const char * pachRawDataRecord,
 {
     char         aszField[128];
     int                 nLength = nEndChar - nStartChar + 1;
-    
+
     CPLAssert( nEndChar - nStartChar + 2 < (int) sizeof(aszField) );
 
     strncpy( aszField, pachRawDataRecord + nStartChar - 1, nLength );
@@ -281,8 +284,8 @@ void TigerFileBase::SetField( OGRFeature *poFeature, const char *pszField,
 /*      formatting, or leave blank if not found.                        */
 /************************************************************************/
 
-int TigerFileBase::WriteField( OGRFeature *poFeature, const char *pszField, 
-                               char *pachRecord, int nStart, int nEnd, 
+int TigerFileBase::WriteField( OGRFeature *poFeature, const char *pszField,
+                               char *pachRecord, int nStart, int nEnd,
                                char chFormat, char chType )
 
 {
@@ -296,27 +299,27 @@ int TigerFileBase::WriteField( OGRFeature *poFeature, const char *pszField,
 
     if( chType == 'N' && chFormat == 'L' )
     {
-        sprintf( szFormat, "%%0%dd", nEnd - nStart + 1 );
-        sprintf( szValue, szFormat, poFeature->GetFieldAsInteger( iField ) );
+        snprintf( szFormat, sizeof(szFormat), "%%0%dd", nEnd - nStart + 1 );
+        snprintf( szValue, sizeof(szValue), szFormat, poFeature->GetFieldAsInteger( iField ) );
     }
     else if( chType == 'N' && chFormat == 'R' )
     {
-        sprintf( szFormat, "%%%dd", nEnd - nStart + 1 );
-        sprintf( szValue, szFormat, poFeature->GetFieldAsInteger( iField ) );
+        snprintf( szFormat, sizeof(szFormat), "%%%dd", nEnd - nStart + 1 );
+        snprintf( szValue, sizeof(szValue), szFormat, poFeature->GetFieldAsInteger( iField ) );
     }
     else if( chType == 'A' && chFormat == 'L' )
     {
-        strncpy( szValue, poFeature->GetFieldAsString( iField ), 
+        strncpy( szValue, poFeature->GetFieldAsString( iField ),
                  sizeof(szValue) - 1 );
         szValue[sizeof(szValue) - 1] = 0;
         if( (int) strlen(szValue) < nEnd - nStart + 1 )
-            memset( szValue + strlen(szValue), ' ', 
+            memset( szValue + strlen(szValue), ' ',
                     nEnd - nStart + 1 - strlen(szValue) );
     }
     else if( chType == 'A' && chFormat == 'R' )
     {
-        sprintf( szFormat, "%%%ds", nEnd - nStart + 1 );
-        sprintf( szValue, szFormat, poFeature->GetFieldAsString( iField ) );
+        snprintf( szFormat, sizeof(szFormat), "%%%ds", nEnd - nStart + 1 );
+        snprintf( szValue, sizeof(szValue), szFormat, poFeature->GetFieldAsString( iField ) );
     }
     else
     {
@@ -333,7 +336,7 @@ int TigerFileBase::WriteField( OGRFeature *poFeature, const char *pszField,
 /*                             WritePoint()                             */
 /************************************************************************/
 
-int TigerFileBase::WritePoint( char *pachRecord, int nStart, 
+int TigerFileBase::WritePoint( char *pachRecord, int nStart,
                                double dfX, double dfY )
 
 {
@@ -341,11 +344,11 @@ int TigerFileBase::WritePoint( char *pachRecord, int nStart,
 
     if( dfX == 0.0 && dfY == 0.0 )
     {
-        strncpy( pachRecord + nStart - 1, "+000000000+00000000", 19 );
+        memcpy( pachRecord + nStart - 1, "+000000000+00000000", 19 );
     }
     else
     {
-        sprintf( szTemp, "%+10d%+9d", 
+        snprintf( szTemp, sizeof(szTemp), "%+10d%+9d",
                  (int) floor(dfX * 1000000 + 0.5),
                  (int) floor(dfY * 1000000 + 0.5) );
         strncpy( pachRecord + nStart - 1, szTemp, 19 );
@@ -358,7 +361,7 @@ int TigerFileBase::WritePoint( char *pachRecord, int nStart,
 /*                            WriteRecord()                             */
 /************************************************************************/
 
-int TigerFileBase::WriteRecord( char *pachRecord, int nRecLen, 
+int TigerFileBase::WriteRecord( char *pachRecord, int nRecLen,
                                 const char *pszType, VSILFILE * fp )
 
 {
@@ -377,7 +380,7 @@ int TigerFileBase::WriteRecord( char *pachRecord, int nRecLen,
          (!EQUAL(pszType, "5")) )
     {
         char    szVersion[5];
-        sprintf( szVersion, "%04d", poDS->GetVersionCode() );
+        snprintf( szVersion, sizeof(szVersion), "%04d", poDS->GetVersionCode() );
         strncpy( pachRecord + 1, szVersion, 4 );
     }
 
@@ -393,9 +396,9 @@ int TigerFileBase::WriteRecord( char *pachRecord, int nRecLen,
 /*      Setup our access to be to the module indicated in the feature.  */
 /************************************************************************/
 
-int TigerFileBase::SetWriteModule( const char *pszExtension, int nRecLen,
+int TigerFileBase::SetWriteModule( const char *pszExtension,
+                                   CPL_UNUSED int nRecLen,
                                    OGRFeature *poFeature )
-
 {
 /* -------------------------------------------------------------------- */
 /*      Work out what module we should be writing to.                   */
@@ -403,13 +406,13 @@ int TigerFileBase::SetWriteModule( const char *pszExtension, int nRecLen,
     const char *pszTargetModule = poFeature->GetFieldAsString( "MODULE" );
     char        szFullModule[30];
 
-    /* TODO/notdef: eventually more logic based on FILE and STATE/COUNTY can 
+    /* TODO/notdef: eventually more logic based on FILE and STATE/COUNTY can
        be inserted here. */
 
     if( pszTargetModule == NULL )
         return FALSE;
 
-    sprintf( szFullModule, "%s.RT", pszTargetModule );
+    snprintf( szFullModule, sizeof(szFullModule), "%s.RT", pszTargetModule );
 
 /* -------------------------------------------------------------------- */
 /*      Is this our current module?                                     */
@@ -441,7 +444,7 @@ int TigerFileBase::SetWriteModule( const char *pszExtension, int nRecLen,
         poDS->DeleteModuleFiles( szFullModule );
         poDS->AddModule( szFullModule );
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Does this file already exist?                                   */
 /* -------------------------------------------------------------------- */
@@ -462,27 +465,27 @@ int TigerFileBase::SetWriteModule( const char *pszExtension, int nRecLen,
 /************************************************************************/
 /*                           AddFieldDefns()                            */
 /************************************************************************/
-void TigerFileBase::AddFieldDefns(const TigerRecordInfo *psRTInfo,
-                                  OGRFeatureDefn  *poFeatureDefn)
+void TigerFileBase::AddFieldDefns(const TigerRecordInfo *psRTInfoIn,
+                                  OGRFeatureDefn  *poFeatureDefnIn)
 {
     OGRFieldDefn        oField("",OFTInteger);
     int i, bLFieldHack;
 
-    bLFieldHack = 
-        CSLTestBoolean( CPLGetConfigOption( "TIGER_LFIELD_AS_STRING", "NO" ) );
-    
-    for (i=0; i<psRTInfo->nFieldCount; ++i) {
-        if (psRTInfo->pasFields[i].bDefine) {
-            OGRFieldType eFT = (OGRFieldType)psRTInfo->pasFields[i].OGRtype;
+    bLFieldHack =
+        CPLTestBool( CPLGetConfigOption( "TIGER_LFIELD_AS_STRING", "NO" ) );
 
-            if( bLFieldHack 
-                && psRTInfo->pasFields[i].cFmt == 'L' 
-                && psRTInfo->pasFields[i].cType == 'N' )
+    for (i=0; i<psRTInfoIn->nFieldCount; ++i) {
+        if (psRTInfoIn->pasFields[i].bDefine) {
+            OGRFieldType eFT = (OGRFieldType)psRTInfoIn->pasFields[i].OGRtype;
+
+            if( bLFieldHack
+                && psRTInfoIn->pasFields[i].cFmt == 'L'
+                && psRTInfoIn->pasFields[i].cType == 'N' )
                 eFT = OFTString;
 
-            oField.Set( psRTInfo->pasFields[i].pszFieldName, eFT, 
-                        psRTInfo->pasFields[i].nLen );
-            poFeatureDefn->AddFieldDefn( &oField );
+            oField.Set( psRTInfoIn->pasFields[i].pszFieldName, eFT,
+                        psRTInfoIn->pasFields[i].nLen );
+            poFeatureDefnIn->AddFieldDefn( &oField );
         }
     }
 }
@@ -491,18 +494,18 @@ void TigerFileBase::AddFieldDefns(const TigerRecordInfo *psRTInfo,
 /*                             SetFields()                              */
 /************************************************************************/
 
-void TigerFileBase::SetFields(const TigerRecordInfo *psRTInfo,
+void TigerFileBase::SetFields(const TigerRecordInfo *psRTInfoIn,
                               OGRFeature      *poFeature,
                               char            *achRecord)
 {
   int i;
-  for (i=0; i<psRTInfo->nFieldCount; ++i) {
-    if (psRTInfo->pasFields[i].bSet) {
+  for (i=0; i<psRTInfoIn->nFieldCount; ++i) {
+    if (psRTInfoIn->pasFields[i].bSet) {
       SetField( poFeature,
-                psRTInfo->pasFields[i].pszFieldName,
-                achRecord, 
-                psRTInfo->pasFields[i].nBeg,
-                psRTInfo->pasFields[i].nEnd );
+                psRTInfoIn->pasFields[i].pszFieldName,
+                achRecord,
+                psRTInfoIn->pasFields[i].nBeg,
+                psRTInfoIn->pasFields[i].nEnd );
     }
   }
 }
@@ -510,20 +513,20 @@ void TigerFileBase::SetFields(const TigerRecordInfo *psRTInfo,
 /************************************************************************/
 /*                             WriteField()                             */
 /************************************************************************/
-void TigerFileBase::WriteFields(const TigerRecordInfo *psRTInfo,
+void TigerFileBase::WriteFields(const TigerRecordInfo *psRTInfoIn,
                                 OGRFeature      *poFeature,
                                 char            *szRecord)
 {
   int i;
-  for (i=0; i<psRTInfo->nFieldCount; ++i) {
-    if (psRTInfo->pasFields[i].bWrite) {
+  for (i=0; i<psRTInfoIn->nFieldCount; ++i) {
+    if (psRTInfoIn->pasFields[i].bWrite) {
       WriteField( poFeature,
-                  psRTInfo->pasFields[i].pszFieldName,
-                  szRecord, 
-                  psRTInfo->pasFields[i].nBeg,
-                  psRTInfo->pasFields[i].nEnd,
-                  psRTInfo->pasFields[i].cFmt,
-                  psRTInfo->pasFields[i].cType );
+                  psRTInfoIn->pasFields[i].pszFieldName,
+                  szRecord,
+                  psRTInfoIn->pasFields[i].nBeg,
+                  psRTInfoIn->pasFields[i].nEnd,
+                  psRTInfoIn->pasFields[i].cFmt,
+                  psRTInfoIn->pasFields[i].cType );
     }
   }
 }
@@ -534,13 +537,13 @@ void TigerFileBase::WriteFields(const TigerRecordInfo *psRTInfo,
 /*                             SetModule()                              */
 /************************************************************************/
 
-int TigerFileBase::SetModule( const char * pszModule )
+int TigerFileBase::SetModule( const char * pszModuleIn )
 
 {
     if (m_pszFileCode == NULL)
         return FALSE;
 
-    if( !OpenFile( pszModule, m_pszFileCode ) )
+    if( !OpenFile( pszModuleIn, m_pszFileCode ) )
         return FALSE;
 
     EstablishFeatureCount();
