@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tigeridhistory.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: tigeridhistory.cpp 27745 2014-09-27 16:38:57Z goatbar $
  *
  * Project:  TIGER/Line Translator
  * Purpose:  Implements TigerIDHistory, providing access to .RTH files.
@@ -30,7 +30,7 @@
 #include "ogr_tiger.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: tigeridhistory.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: tigeridhistory.cpp 27745 2014-09-27 16:38:57Z goatbar $");
 
 #define FILE_CODE "H"
 
@@ -60,7 +60,8 @@ static const TigerRecordInfo rtH_info =
 /************************************************************************/
 
 TigerIDHistory::TigerIDHistory( OGRTigerDataSource * poDSIn,
-                                CPL_UNUSED const char * pszPrototypeModule ) : TigerFileBase(&rtH_info, FILE_CODE)
+                                CPL_UNUSED const char * pszPrototypeModule ) :
+    TigerFileBase(&rtH_info, FILE_CODE)
 {
     poDS = poDSIn;
     poFeatureDefn = new OGRFeatureDefn( "IDHistory" );
