@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tigerpolychainlink.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: tigerpolychainlink.cpp 27745 2014-09-27 16:38:57Z goatbar $
  *
  * Project:  TIGER/Line Translator
  * Purpose:  Implements TigerPolyChainLink, providing access to .RTI files.
@@ -30,7 +30,7 @@
 #include "ogr_tiger.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: tigerpolychainlink.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: tigerpolychainlink.cpp 27745 2014-09-27 16:38:57Z goatbar $");
 
 #define FILE_CODE "I"
 
@@ -84,8 +84,8 @@ static const TigerRecordInfo rtI_info =
 /************************************************************************/
 
 TigerPolyChainLink::TigerPolyChainLink( OGRTigerDataSource * poDSIn,
-                                        CPL_UNUSED const char * pszPrototypeModule )
-  : TigerFileBase(NULL, FILE_CODE)
+                                        CPL_UNUSED const char * pszPrototypeModule ) :
+    TigerFileBase(NULL, FILE_CODE)
 {
     OGRFieldDefn        oField("",OFTInteger);
 
