@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tigerpolygon.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: tigerpolygon.cpp 33706 2016-03-11 13:33:27Z goatbar $
  *
  * Project:  TIGER/Line Translator
  * Purpose:  Implements TigerPolygon, providing access to .RTA files.
@@ -31,7 +31,7 @@
 #include "ogr_tiger.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: tigerpolygon.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: tigerpolygon.cpp 33706 2016-03-11 13:33:27Z goatbar $");
 
 static const TigerFieldInfo rtA_2002_fields[] = {
   // fieldname    fmt  type OFTType      beg  end  len  bDefine bSet bWrite
@@ -382,15 +382,16 @@ static const TigerRecordInfo rtS_info =
 /************************************************************************/
 
 TigerPolygon::TigerPolygon( OGRTigerDataSource * poDSIn,
-                            CPL_UNUSED const char * pszPrototypeModule )
+                            CPL_UNUSED const char * pszPrototypeModule ) :
+    fpRTS(NULL),
+    bUsingRTS(TRUE),
+    nRTSRecLen(0)
 {
     poDS = poDSIn;
     poFeatureDefn = new OGRFeatureDefn( "Polygon" );
     poFeatureDefn->Reference();
     poFeatureDefn->SetGeomType( wkbNone );
 
-    fpRTS = NULL;
-    bUsingRTS = TRUE;
 
     if( poDS->GetVersion() >= TIGER_2004 ) {
         psRTAInfo = &rtA_2004_info;
@@ -440,14 +441,14 @@ TigerPolygon::~TigerPolygon()
 /*                             SetModule()                              */
 /************************************************************************/
 
-int TigerPolygon::SetModule( const char * pszModule )
+int TigerPolygon::SetModule( const char * pszModuleIn )
 
 {
-    if( !OpenFile( pszModule, "A" ) )
+    if( !OpenFile( pszModuleIn, "A" ) )
         return FALSE;
 
     EstablishFeatureCount();
-    
+
 /* -------------------------------------------------------------------- */
 /*      Open the RTS file                                               */
 /* -------------------------------------------------------------------- */
@@ -459,11 +460,11 @@ int TigerPolygon::SetModule( const char * pszModule )
             fpRTS = NULL;
         }
 
-        if( pszModule )
+        if( pszModuleIn )
         {
             char        *pszFilename;
-        
-            pszFilename = poDS->BuildFilename( pszModule, "S" );
+
+            pszFilename = poDS->BuildFilename( pszModuleIn, "S" );
 
             fpRTS = VSIFOpenL( pszFilename, "rb" );
 
@@ -472,7 +473,7 @@ int TigerPolygon::SetModule( const char * pszModule )
             nRTSRecLen = EstablishRecordLength( fpRTS );
         }
     }
-    
+
     return TRUE;
 }
 
@@ -550,7 +551,7 @@ OGRFeature *TigerPolygon::GetFeature( int nRecordId )
         SetFields( psRTSInfo, poFeature, achRTSRec );
 
     }
-    
+
     return poFeature;
 }
 
@@ -558,7 +559,7 @@ OGRFeature *TigerPolygon::GetFeature( int nRecordId )
 /*                           SetWriteModule()                           */
 /************************************************************************/
 
-int TigerPolygon::SetWriteModule( const char *pszFileCode, int nRecLen, 
+int TigerPolygon::SetWriteModule( const char *pszFileCode, int nRecLen,
                                   OGRFeature *poFeature )
 
 {
@@ -582,7 +583,7 @@ int TigerPolygon::SetWriteModule( const char *pszFileCode, int nRecLen,
         if( pszModule )
         {
             char        *pszFilename;
-        
+
             pszFilename = poDS->BuildFilename( pszModule, "S" );
 
             fpRTS = VSIFOpenL( pszFilename, "ab" );
@@ -590,7 +591,7 @@ int TigerPolygon::SetWriteModule( const char *pszFileCode, int nRecLen,
             CPLFree( pszFilename );
         }
     }
-    
+
     return TRUE;
 }
 
@@ -627,4 +628,3 @@ OGRErr TigerPolygon::CreateFeature( OGRFeature *poFeature )
 
     return OGRERR_NONE;
 }
-
