@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tigertlidrange.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: tigertlidrange.cpp 27745 2014-09-27 16:38:57Z goatbar $
  *
  * Project:  TIGER/Line Translator
  * Purpose:  Implements TigerTLIDRange, providing access to .RTR files.
@@ -30,7 +30,7 @@
 #include "ogr_tiger.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: tigertlidrange.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: tigertlidrange.cpp 27745 2014-09-27 16:38:57Z goatbar $");
 
 #define FILE_CODE "R"
 
@@ -78,8 +78,8 @@ static const TigerRecordInfo rtR_info =
 /************************************************************************/
 
 TigerTLIDRange::TigerTLIDRange( OGRTigerDataSource * poDSIn,
-                                CPL_UNUSED const char * pszPrototypeModule )
-  : TigerFileBase(NULL, FILE_CODE)
+                                CPL_UNUSED const char * pszPrototypeModule ) :
+    TigerFileBase(NULL, FILE_CODE)
 {
     poDS = poDSIn;
     poFeatureDefn = new OGRFeatureDefn( "TLIDRange" );
