@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tigerzerocellid.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: tigerzerocellid.cpp 27745 2014-09-27 16:38:57Z goatbar $
  *
  * Project:  TIGER/Line Translator
  * Purpose:  Implements TigerZeroCellID, providing access to .RTT files.
@@ -30,7 +30,7 @@
 #include "ogr_tiger.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: tigerzerocellid.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: tigerzerocellid.cpp 27745 2014-09-27 16:38:57Z goatbar $");
 
 #define FILE_CODE       "T"
 
@@ -54,8 +54,8 @@ static const TigerRecordInfo rtT_info =
 /************************************************************************/
 
 TigerZeroCellID::TigerZeroCellID( OGRTigerDataSource * poDSIn,
-                                  CPL_UNUSED const char * pszPrototypeModule )
-  : TigerFileBase(&rtT_info, FILE_CODE)
+                                  CPL_UNUSED const char * pszPrototypeModule ) :
+    TigerFileBase(&rtT_info, FILE_CODE)
 {
     poDS = poDSIn;
     poFeatureDefn = new OGRFeatureDefn( "ZeroCellID" );
