@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: vfkdatablocksqlite.cpp 27741 2014-09-26 19:20:02Z goatbar $
+ * $Id: vfkdatablocksqlite.cpp 33713 2016-03-12 17:41:57Z goatbar $
  *
  * Project:  VFK Reader - Data block definition (SQLite)
  * Purpose:  Implements VFKDataBlockSQLite
@@ -75,7 +75,7 @@ int VFKDataBlockSQLite::LoadGeometryPoint()
         x = -1.0 * sqlite3_column_double(hStmt, 0); /* S-JTSK coordinate system expected */
         y = -1.0 * sqlite3_column_double(hStmt, 1);
 #ifdef DEBUG
-	const long iFID = sqlite3_column_double(hStmt, 2);
+	const GIntBig iFID = sqlite3_column_int64(hStmt, 2);
 #endif
 	rowId = sqlite3_column_int(hStmt, 3);
 
@@ -94,16 +94,111 @@ int VFKDataBlockSQLite::LoadGeometryPoint()
 	    SaveGeometryToDB(&pt, rowId) != OGRERR_FAILURE)
 	    nGeometries++;
     }
-    
+
     /* update number of geometries in VFK_DB_TABLE table */
     UpdateVfkBlocks(nGeometries);
 
     if (poReader->IsSpatial())
 	poReader->ExecuteSQL("COMMIT");
-    
+
     return bSkipInvalid ? 0 : nInvalid;
 }
 
+/*!
+  \brief Set geometry for linestrings
+
+  \param poLine VFK feature
+  \param oOGRLine line geometry
+  \param[in,out] bValid TRUE when feature's geometry is valid
+  \param[in,out] rowIdFeat list of row ids which forms linestring
+  \param[in,out] nGeometries number of features with valid geometry
+*/
+bool VFKDataBlockSQLite::SetGeometryLineString(VFKFeatureSQLite *poLine, OGRLineString *oOGRLine,
+                                               bool& bValid, const char *ftype,
+                                               std::vector<int>& rowIdFeat, int& nGeometries)
+{
+    int              npoints;
+    VFKReaderSQLite *poReader;
+
+    poReader  = (VFKReaderSQLite*) m_poReader;
+
+    oOGRLine->setCoordinateDimension(2); /* force 2D */
+
+    /* check also VFK validity */
+    if (bValid) {
+        /* Feature types
+
+           - '3'    - line       (2 points)
+           - '4'    - linestring (at least 2 points)
+           - '11'   - curve      (at least 2 points)
+           - '15'   - circle     (3 points)
+           - '15 r' - circle     (center point & radius)
+           - '16'   - arc        (3 points)
+        */
+
+        npoints = oOGRLine->getNumPoints();
+        if (EQUAL(ftype, "3") && npoints > 2) {
+            /* be less pedantic, just inform user about data
+             * inconsistency
+
+               bValid = FALSE;
+            */
+            CPLDebug("OGR-VFK",
+                     "Line (fid=" CPL_FRMT_GIB ") defined by more than two vertices",
+                     poLine->GetFID());
+        }
+        else if (EQUAL(ftype, "11") && npoints < 2) {
+            bValid = FALSE;
+            CPLError(CE_Warning, CPLE_AppDefined,
+                     "Curve (fid=" CPL_FRMT_GIB ") defined by less than two vertices",
+                     poLine->GetFID());
+        }
+        else if (EQUAL(ftype, "15") && npoints != 3) {
+            bValid = FALSE;
+            CPLError(CE_Warning, CPLE_AppDefined,
+                     "Circle (fid=" CPL_FRMT_GIB ") defined by invalid number of vertices (%d)",
+                     poLine->GetFID(), oOGRLine->getNumPoints());
+        }
+        else if (strlen(ftype) > 2 && STARTS_WITH_CI(ftype, "15") && npoints != 1) {
+            bValid = FALSE;
+            CPLError(CE_Warning, CPLE_AppDefined,
+                     "Circle (fid=" CPL_FRMT_GIB ") defined by invalid number of vertices (%d)",
+                     poLine->GetFID(), oOGRLine->getNumPoints());
+        }
+        else if (EQUAL(ftype, "16") && npoints != 3) {
+            bValid = FALSE;
+            CPLError(CE_Warning, CPLE_AppDefined,
+                     "Arc (fid=" CPL_FRMT_GIB ") defined by invalid number of vertices (%d)",
+                     poLine->GetFID(), oOGRLine->getNumPoints());
+        }
+    }
+
+    /* set geometry (NULL for invalid features) */
+    if (bValid) {
+        if (!poLine->SetGeometry(oOGRLine, ftype)) {
+            bValid = FALSE;
+        }
+    }
+    else {
+        poLine->SetGeometry(NULL);
+    }
+
+    /* update fid column */
+    UpdateFID(poLine->GetFID(), rowIdFeat);
+
+    /* store also geometry in DB */
+    CPLAssert(0 != rowIdFeat.size());
+    if (bValid && poReader->IsSpatial() &&
+        SaveGeometryToDB(bValid ? poLine->GetGeometry() : NULL,
+                         rowIdFeat[0]) != OGRERR_FAILURE)
+        nGeometries++;
+
+    rowIdFeat.clear();
+    oOGRLine->empty(); /* restore line */
+
+    return bValid;
+}
+
 /*!
   \brief Load geometry (linestring SBP layer)
 
@@ -111,101 +206,87 @@ int VFKDataBlockSQLite::LoadGeometryPoint()
 */
 int VFKDataBlockSQLite::LoadGeometryLineStringSBP()
 {
-    int      nInvalid, nGeometries, rowId;
-    long int iFID;
+    int       nInvalid, nGeometries, rowId, iIdx;
+    CPLString szFType, szFTypeLine;
+
     GUIntBig id, ipcb;
     bool     bValid;
-    
+
     std::vector<int> rowIdFeat;
     CPLString     osSQL;
     sqlite3_stmt *hStmt;
-    
+
     VFKReaderSQLite    *poReader;
     VFKDataBlockSQLite *poDataBlockPoints;
     VFKFeatureSQLite   *poFeature, *poPoint, *poLine;
-    
+
     OGRLineString oOGRLine;
-    
+
     nInvalid  = nGeometries = 0;
     poReader  = (VFKReaderSQLite*) m_poReader;
     poLine    = NULL;
-    
+
     poDataBlockPoints = (VFKDataBlockSQLite *) m_poReader->GetDataBlock("SOBR");
     if (NULL == poDataBlockPoints) {
-        CPLError(CE_Failure, CPLE_FileIO, 
+        CPLError(CE_Failure, CPLE_FileIO,
                  "Data block %s not found.\n", m_pszName);
         return nInvalid;
     }
-    
+
     poDataBlockPoints->LoadGeometry();
-    
+
     if (LoadGeometryFromDB()) /* try to load geometry from DB */
 	return 0;
 
     osSQL.Printf("UPDATE %s SET %s = -1", m_pszName, FID_COLUMN);
     poReader->ExecuteSQL(osSQL.c_str());
     bValid = TRUE;
-    iFID = 1;
+    iIdx = 0;
     for (int i = 0; i < 2; i++) {
 	/* first collect linestrings related to HP, OB or DPM
 	   then collect rest of linestrings */
         if (i == 0)
-            osSQL.Printf("SELECT BP_ID,PORADOVE_CISLO_BODU,_rowid_ FROM '%s' WHERE "
+            osSQL.Printf("SELECT BP_ID,PORADOVE_CISLO_BODU,PARAMETRY_SPOJENI,_rowid_ FROM '%s' WHERE "
                          "HP_ID IS NOT NULL OR OB_ID IS NOT NULL OR DPM_ID IS NOT NULL "
                          "ORDER BY HP_ID,OB_ID,DPM_ID,PORADOVE_CISLO_BODU", m_pszName);
         else
-            osSQL.Printf("SELECT BP_ID,PORADOVE_CISLO_BODU,_rowid_ FROM '%s' WHERE "
+            osSQL.Printf("SELECT BP_ID,PORADOVE_CISLO_BODU,PARAMETRY_SPOJENI,_rowid_ FROM '%s' WHERE "
                          "OB_ID IS NULL AND HP_ID IS NULL AND DPM_ID IS NULL "
                          "ORDER BY ID,PORADOVE_CISLO_BODU", m_pszName);
-	
+
         hStmt = poReader->PrepareStatement(osSQL.c_str());
-    
+
 	if (poReader->IsSpatial())
 	    poReader->ExecuteSQL("BEGIN");
-	
+
         while(poReader->ExecuteSQL(hStmt) == OGRERR_NONE) {
             // read values
-            id    = sqlite3_column_double(hStmt, 0);
-            ipcb  = sqlite3_column_double(hStmt, 1);
-            rowId = sqlite3_column_int(hStmt, 2);
+            id    = sqlite3_column_int64(hStmt, 0);
+            ipcb  = sqlite3_column_int64(hStmt, 1);
+            szFType = (char *) sqlite3_column_text(hStmt, 2);
+            rowId = sqlite3_column_int(hStmt, 3);
 
             if (ipcb == 1) {
-                /* add feature to the array */
-                poFeature = new VFKFeatureSQLite(this, rowId, iFID);
-                CPLAssert(NULL != poFeature && poFeature->GetFID() == iFID);
-                AddFeature(poFeature);
-                
-                if (poLine) {
-                    oOGRLine.setCoordinateDimension(2); /* force 2D */
-		    if (bValid) {
-			if (!poLine->SetGeometry(&oOGRLine)) {
-                            bValid = FALSE;
-			    nInvalid++;
-                        }
-		    }
-		    else {
-			poLine->SetGeometry(NULL);
-			nInvalid++;
-		    }
-                    
-		    /* update fid column */
-		    UpdateFID(poLine->GetFID(), rowIdFeat);        
-
-		    /* store also geometry in DB */
-		    CPLAssert(0 != rowIdFeat.size());
-		    if (bValid && poReader->IsSpatial() &&
-			SaveGeometryToDB(bValid ? &oOGRLine : NULL,
-					 rowIdFeat[0]) != OGRERR_FAILURE)
-			nGeometries++;
-		    
-		    rowIdFeat.clear();
-                    oOGRLine.empty(); /* restore line */
+                poFeature = (VFKFeatureSQLite *) GetFeatureByIndex(iIdx);
+                if( poFeature == NULL )
+                {
+                    CPLError(CE_Failure, CPLE_AppDefined, "Cannot retrieve feature %d", iIdx);
+                    break;
                 }
+                poFeature->SetRowId(rowId);
+
+                /* set geometry & reset */
+                if (poLine && !SetGeometryLineString(poLine, &oOGRLine,
+                                                     bValid, szFTypeLine, rowIdFeat, nGeometries)) {
+                    nInvalid++;
+                }
+
 		bValid = TRUE;
                 poLine = poFeature;
-                iFID++;
+                szFTypeLine = szFType;
+                iIdx++;
             }
-            
+
             poPoint = (VFKFeatureSQLite *) poDataBlockPoints->GetFeature("ID", id);
 	    if (poPoint) {
 		OGRPoint *pt = (OGRPoint *) poPoint->GetGeometry();
@@ -213,47 +294,28 @@ int VFKDataBlockSQLite::LoadGeometryLineStringSBP()
 		    oOGRLine.addPoint(pt);
 		}
 		else {
-		    CPLDebug("OGR-VFK", 
+		    CPLDebug("OGR-VFK",
 			     "Geometry (point ID = " CPL_FRMT_GUIB ") not valid", id);
 		    bValid = FALSE;
 		}
 	    }
 	    else {
-                CPLDebug("OGR-VFK", 
+                CPLDebug("OGR-VFK",
                          "Point ID = " CPL_FRMT_GUIB " not found (rowid = %d)",
                          id, rowId);
 		bValid = FALSE;
             }
-	    
+
 	    /* add vertex to the linestring */
 	    rowIdFeat.push_back(rowId);
         }
 
         /* add last line */
-        if (poLine) {
-	    oOGRLine.setCoordinateDimension(2); /* force 2D */
-	    if (bValid) {
-		if (!poLine->SetGeometry(&oOGRLine))
-		    nInvalid++;
-	    }
-	    else {
-		poLine->SetGeometry(NULL);
-		nInvalid++;
-	    }
-            
-	    /* update fid column */
-	    UpdateFID(poLine->GetFID(), rowIdFeat);        
-	    
-	    /* store also geometry in DB */
-	    CPLAssert(0 != rowIdFeat.size());
-	    if (poReader->IsSpatial() &&
-		SaveGeometryToDB(bValid ? &oOGRLine : NULL,
-				 rowIdFeat[0]) != OGRERR_FAILURE && bValid)
-		nGeometries++;
+        if (poLine && !SetGeometryLineString(poLine, &oOGRLine,
+                                             bValid, szFType.c_str(), rowIdFeat, nGeometries)) {
+            nInvalid++;
         }
 	poLine = NULL;
-	rowIdFeat.clear();
-        oOGRLine.empty(); /* restore line */
 
 	if (poReader->IsSpatial())
 	    poReader->ExecuteSQL("COMMIT");
@@ -261,7 +323,7 @@ int VFKDataBlockSQLite::LoadGeometryLineStringSBP()
 
     /* update number of geometries in VFK_DB_TABLE table */
     UpdateVfkBlocks(nGeometries);
-    
+
     return nInvalid;
 }
 
@@ -275,51 +337,51 @@ int VFKDataBlockSQLite::LoadGeometryLineStringHP()
     int          nInvalid, nGeometries;
     int          rowId;
     long         iFID;
-    
+
     CPLString    osColumn, osSQL;
     const char  *vrColumn[2];
     GUIntBig     vrValue[2];
-    
+
     sqlite3_stmt *hStmt;
-    
+
     OGRGeometry        *poOgrGeometry;
     VFKReaderSQLite    *poReader;
     VFKDataBlockSQLite *poDataBlockLines;
     VFKFeatureSQLite   *poFeature, *poLine;
-    
+
     nInvalid = nGeometries = 0;
     poReader  = (VFKReaderSQLite*) m_poReader;
-    
+
     poDataBlockLines = (VFKDataBlockSQLite *) m_poReader->GetDataBlock("SBP");
     if (NULL == poDataBlockLines) {
-        CPLError(CE_Failure, CPLE_FileIO, 
+        CPLError(CE_Failure, CPLE_FileIO,
                  "Data block %s not found", m_pszName);
         return nInvalid;
     }
-    
+
     poDataBlockLines->LoadGeometry();
 
     if (LoadGeometryFromDB()) /* try to load geometry from DB */
 	return 0;
-    
+
     osColumn.Printf("%s_ID", m_pszName);
     vrColumn[0] = osColumn.c_str();
     vrColumn[1] = "PORADOVE_CISLO_BODU";
     vrValue[1]  = 1; /* reduce to first segment */
-    
+
     osSQL.Printf("SELECT ID,%s,rowid FROM %s", FID_COLUMN, m_pszName);
     /* TODO: handle points in DPM */
     if (EQUAL(m_pszName, "DPM"))
         osSQL += " WHERE SOURADNICE_X IS NULL";
     hStmt = poReader->PrepareStatement(osSQL.c_str());
-    
+
     if (poReader->IsSpatial())
 	poReader->ExecuteSQL("BEGIN");
 
     while(poReader->ExecuteSQL(hStmt) == OGRERR_NONE) {
         /* read values */
-        vrValue[0] = sqlite3_column_double(hStmt, 0);
-        iFID       = sqlite3_column_double(hStmt, 1);
+        vrValue[0] = sqlite3_column_int64(hStmt, 0);
+        iFID       = static_cast<long>(sqlite3_column_int64(hStmt, 1));
         rowId      = sqlite3_column_int(hStmt, 2);
 
         poFeature = (VFKFeatureSQLite *) GetFeatureByIndex(rowId - 1);
@@ -339,17 +401,17 @@ int VFKDataBlockSQLite::LoadGeometryLineStringHP()
 	    nInvalid++;
             continue;
         }
-	
+
 	/* store also geometry in DB */
 	if (poReader->IsSpatial() &&
 	    SaveGeometryToDB(poOgrGeometry, rowId) != OGRERR_FAILURE &&
 	    poOgrGeometry)
 	    nGeometries++;
     }
-    
+
     /* update number of geometries in VFK_DB_TABLE table */
     UpdateVfkBlocks(nGeometries);
-    
+
     if (poReader->IsSpatial())
 	poReader->ExecuteSQL("COMMIT");
 
@@ -368,14 +430,14 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
     size_t nLines;
     long iFID;
     bool bIsPar, bNewRing, bFound;
-        
+
     CPLString    osSQL;
     const char  *vrColumn[2];
     GUIntBig     vrValue[2];
     GUIntBig     id, idOb;
-    
+
     sqlite3_stmt *hStmt;
-    
+
     VFKReaderSQLite    *poReader;
     VFKDataBlockSQLite *poDataBlockLines1, *poDataBlockLines2;
     VFKFeatureSQLite   *poFeature;
@@ -383,14 +445,14 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
     VFKFeatureSQLiteList  poLineList;
     /* first is to be considered as exterior */
     PointListArray        poRingList;
-    
+
     std::vector<OGRLinearRing *> poLinearRingList;
     OGRPolygon     ogrPolygon;
     OGRLinearRing *poOgrRing;
 
     nInvalidNoLines = nInvalidNoRings = nGeometries = 0;
     poReader  = (VFKReaderSQLite*) m_poReader;
-    
+
     if (EQUAL (m_pszName, "PAR")) {
         poDataBlockLines1 = (VFKDataBlockSQLite *) m_poReader->GetDataBlock("HP");
         poDataBlockLines2 = poDataBlockLines1;
@@ -402,17 +464,17 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
         bIsPar = FALSE;
     }
     if (NULL == poDataBlockLines1 || NULL == poDataBlockLines2) {
-        CPLError(CE_Failure, CPLE_FileIO, 
+        CPLError(CE_Failure, CPLE_FileIO,
                  "Data block %s not found", m_pszName);
         return -1;
     }
-    
+
     poDataBlockLines1->LoadGeometry();
     poDataBlockLines2->LoadGeometry();
-    
+
     if (LoadGeometryFromDB()) /* try to load geometry from DB */
 	return 0;
-    
+
     if (bIsPar) {
         vrColumn[0] = "PAR_ID_1";
         vrColumn[1] = "PAR_ID_2";
@@ -425,18 +487,18 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
 
     osSQL.Printf("SELECT ID,%s,rowid FROM %s", FID_COLUMN, m_pszName);
     hStmt = poReader->PrepareStatement(osSQL.c_str());
-    
+
     if (poReader->IsSpatial())
 	poReader->ExecuteSQL("BEGIN");
 
     while(poReader->ExecuteSQL(hStmt) == OGRERR_NONE) {
         nBridges = 0;
-        
+
         /* read values */
-        id        = sqlite3_column_double(hStmt, 0);
-        iFID      = sqlite3_column_double(hStmt, 1);
+        id        = sqlite3_column_int64(hStmt, 0);
+        iFID      = static_cast<long>(sqlite3_column_int64(hStmt, 1));
         rowId     = sqlite3_column_int(hStmt, 2);
-        
+
         poFeature = (VFKFeatureSQLite *) GetFeatureByIndex(rowId - 1);
         CPLAssert(NULL != poFeature && poFeature->GetFID() == iFID);
 
@@ -448,19 +510,19 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
             VFKFeatureSQLite *poLineSbp;
             std::vector<VFKFeatureSQLite *> poLineListOb;
             sqlite3_stmt *hStmtOb;
-            
+
             osSQL.Printf("SELECT ID FROM %s WHERE BUD_ID = " CPL_FRMT_GUIB,
                          poDataBlockLines1->GetName(), id);
             if (poReader->IsSpatial()) {
                 CPLString osColumn;
-                
+
                 osColumn.Printf(" AND %s IS NULL", GEOM_COLUMN);
                 osSQL += osColumn;
             }
             hStmtOb = poReader->PrepareStatement(osSQL.c_str());
-            
+
             while(poReader->ExecuteSQL(hStmtOb) == OGRERR_NONE) {
-                idOb = sqlite3_column_double(hStmtOb, 0); 
+                idOb = sqlite3_column_int64(hStmtOb, 0);
                 vrValue[0] = idOb;
                 poLineSbp = poDataBlockLines2->GetFeature(vrColumn, vrValue, 2);
                 if (poLineSbp)
@@ -469,21 +531,21 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
         }
         nLines = poLineList.size();
         if (nLines < 1) {
-            CPLDebug("OGR-VFK", 
+            CPLDebug("OGR-VFK",
                      "%s: unable to collect rings for polygon fid = %ld (no lines)",
                      m_pszName, iFID);
             nInvalidNoLines++;
             continue;
         }
-        
+
         /* clear */
         ogrPolygon.empty();
         poRingList.clear();
-        
+
         /* collect rings from lines */
         bFound = FALSE;
         nCount = 0;
-        nCountMax = nLines * 2;
+        nCountMax = static_cast<int>(nLines) * 2;
 	while (poLineList.size() > 0 && nCount < nCountMax) {
             bNewRing = !bFound ? TRUE : FALSE;
             bFound = FALSE;
@@ -503,7 +565,7 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
                  iFID, (int)nLines, (int)poRingList.size());
 
         if (poLineList.size() > 0) {
-            CPLDebug("OGR-VFK", 
+            CPLDebug("OGR-VFK",
                      "%s: unable to collect rings for polygon fid = %ld",
                      m_pszName, iFID);
             nInvalidNoRings++;
@@ -517,11 +579,11 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
              iRing != eRing; ++iRing) {
 	    OGRPoint *poPoint;
             PointList *poList = *iRing;
-	    
+
             poLinearRingList.push_back(new OGRLinearRing());
 	    poOgrRing = poLinearRingList.back();
 	    CPLAssert(NULL != poOgrRing);
-	    
+
             for (PointList::iterator iPoint = poList->begin(), ePoint = poList->end();
                  iPoint != ePoint; ++iPoint) {
 		poPoint = &(*iPoint);
@@ -534,7 +596,7 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
 	if (poLinearRingList.size() > 1) {
 	    double dArea, dMaxArea;
 	    std::vector<OGRLinearRing *>::iterator exteriorRing;
-	    
+
 	    exteriorRing = poLinearRingList.begin();
 	    dMaxArea = -1.;
 	    for (std::vector<OGRLinearRing *>::iterator iRing = poLinearRingList.begin(),
@@ -542,7 +604,7 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
 		poOgrRing = *iRing;
 		if (!IsRingClosed(poOgrRing))
 		    continue; /* skip unclosed rings */
-		
+
 		dArea = poOgrRing->get_Area();
 		if (dArea > dMaxArea) {
 		    dMaxArea = dArea;
@@ -553,12 +615,12 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
 		std::swap(*poLinearRingList.begin(), *exteriorRing);
 	    }
 	}
-	
+
 	/* build polygon from rings */
         for (std::vector<OGRLinearRing *>::iterator iRing = poLinearRingList.begin(),
 		 eRing = poLinearRingList.end(); iRing != eRing; ++iRing) {
 	    poOgrRing = *iRing;
-	    
+
 	    /* check if ring is closed */
 	    if (IsRingClosed(poOgrRing)) {
 		ogrPolygon.addRing(poOgrRing);
@@ -586,23 +648,23 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
             nInvalidNoRings++;
             continue;
         }
-	
+
         /* store also geometry in DB */
-        if (poReader->IsSpatial() && 
+        if (poReader->IsSpatial() &&
 	    SaveGeometryToDB(&ogrPolygon, rowId) != OGRERR_FAILURE)
             nGeometries++;
     }
-    
+
     /* free ring list */
     for (PointListArray::iterator iRing = poRingList.begin(), eRing = poRingList.end();
          iRing != eRing; ++iRing) {
         delete (*iRing);
         *iRing = NULL;
     }
-    
+
     CPLDebug("OGR-VFK", "%s: nolines = %d norings = %d",
              m_pszName, nInvalidNoLines, nInvalidNoRings);
-    
+
     /* update number of geometries in VFK_DB_TABLE table */
     UpdateVfkBlocks(nGeometries);
 
@@ -616,33 +678,33 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
   \brief Get feature by FID
 
   Modifies next feature id.
-  
+
   \param nFID feature id
 
   \return pointer to feature definition or NULL on failure (not found)
 */
-IVFKFeature *VFKDataBlockSQLite::GetFeature(long nFID)
+IVFKFeature *VFKDataBlockSQLite::GetFeature(GIntBig nFID)
 {
     int rowId;
     CPLString osSQL;
     VFKReaderSQLite  *poReader;
-    
+
     sqlite3_stmt *hStmt;
-         
+
     if (m_nFeatureCount < 0) {
         m_poReader->ReadDataRecords(this);
     }
-    
+
     if (nFID < 1 || nFID > m_nFeatureCount)
         return NULL;
 
     if (m_bGeometryPerBlock && !m_bGeometry) {
         LoadGeometry();
     }
-    
+
     poReader = (VFKReaderSQLite*) m_poReader;
-    
-    osSQL.Printf("SELECT rowid FROM %s WHERE %s = %ld",
+
+    osSQL.Printf("SELECT rowid FROM %s WHERE %s = " CPL_FRMT_GIB,
                  m_pszName, FID_COLUMN, nFID);
     if (EQUAL(m_pszName, "SBP")) {
         osSQL += " AND PORADOVE_CISLO_BODU = 1";
@@ -654,17 +716,17 @@ IVFKFeature *VFKDataBlockSQLite::GetFeature(long nFID)
         rowId = sqlite3_column_int(hStmt, 0);
     }
     sqlite3_finalize(hStmt);
-    
+
     return GetFeatureByIndex(rowId - 1);
 }
 
 /*!
   \brief Get first found feature based on it's property
-  
+
   \param column property name
   \param value property value
   \param bGeom True to check also geometry != NULL
-    
+
   \return pointer to feature definition or NULL on failure (not found)
 */
 VFKFeatureSQLite *VFKDataBlockSQLite::GetFeature(const char *column, GUIntBig value,
@@ -675,25 +737,25 @@ VFKFeatureSQLite *VFKDataBlockSQLite::GetFeature(const char *column, GUIntBig va
     VFKReaderSQLite  *poReader;
 
     sqlite3_stmt *hStmt;
-    
+
     poReader = (VFKReaderSQLite*) m_poReader;
-    
+
     osSQL.Printf("SELECT %s from %s WHERE %s = " CPL_FRMT_GUIB,
                  FID_COLUMN, m_pszName, column, value);
     if (bGeom) {
         CPLString osColumn;
-        
+
         osColumn.Printf(" AND %s IS NOT NULL", GEOM_COLUMN);
         osSQL += osColumn;
     }
-    
+
     hStmt = poReader->PrepareStatement(osSQL.c_str());
     if (poReader->ExecuteSQL(hStmt) != OGRERR_NONE)
         return NULL;
-    
+
     idx = sqlite3_column_int(hStmt, 0) - 1;
     sqlite3_finalize(hStmt);
-    
+
     if (idx < 0 || idx >= m_nFeatureCount) // ? assert
         return NULL;
 
@@ -702,7 +764,7 @@ VFKFeatureSQLite *VFKDataBlockSQLite::GetFeature(const char *column, GUIntBig va
 
 /*!
   \brief Get first found feature based on it's properties (AND)
-  
+
   \param column array of property names
   \param value array of property values
   \param num number of array items
@@ -718,9 +780,9 @@ VFKFeatureSQLite *VFKDataBlockSQLite::GetFeature(const char **column, GUIntBig *
     VFKReaderSQLite  *poReader;
 
     sqlite3_stmt *hStmt;
-    
+
     poReader = (VFKReaderSQLite*) m_poReader;
-    
+
     osSQL.Printf("SELECT %s FROM %s WHERE ", FID_COLUMN, m_pszName);
     for (int i = 0; i < num; i++) {
         if (i > 0)
@@ -737,23 +799,23 @@ VFKFeatureSQLite *VFKDataBlockSQLite::GetFeature(const char **column, GUIntBig *
     hStmt = poReader->PrepareStatement(osSQL.c_str());
     if (poReader->ExecuteSQL(hStmt) != OGRERR_NONE)
         return NULL;
-    
+
     idx = sqlite3_column_int(hStmt, 0) - 1; /* rowid starts at 1 */
-    sqlite3_finalize(hStmt);    
+    sqlite3_finalize(hStmt);
 
     if (idx < 0 || idx >= m_nFeatureCount) // ? assert
         return NULL;
-    
+
     return (VFKFeatureSQLite *) GetFeatureByIndex(idx);
 }
 
 /*!
   \brief Get features based on properties
-  
+
   \param column array of property names
   \param value array of property values
   \param num number of array items
-  
+
   \return list of features
 */
 VFKFeatureSQLiteList VFKDataBlockSQLite::GetFeatures(const char **column, GUIntBig *value, int num)
@@ -763,11 +825,11 @@ VFKFeatureSQLiteList VFKDataBlockSQLite::GetFeatures(const char **column, GUIntB
 
     VFKReaderSQLite     *poReader;
     VFKFeatureSQLiteList fList;
-    
+
     sqlite3_stmt *hStmt;
-    
+
     poReader = (VFKReaderSQLite*) m_poReader;
-    
+
     osSQL.Printf("SELECT rowid from %s WHERE ", m_pszName);
     for (int i = 0; i < num; i++) {
         if (i > 0)
@@ -778,13 +840,13 @@ VFKFeatureSQLiteList VFKDataBlockSQLite::GetFeatures(const char **column, GUIntB
     }
     osSQL += " ORDER BY ";
     osSQL += FID_COLUMN;
-    
+
     hStmt = poReader->PrepareStatement(osSQL.c_str());
     while (poReader->ExecuteSQL(hStmt) == OGRERR_NONE) {
         iRowId = sqlite3_column_int(hStmt, 0);
         fList.push_back((VFKFeatureSQLite *)GetFeatureByIndex(iRowId - 1));
     }
-    
+
     return fList;
 }
 
@@ -812,15 +874,15 @@ OGRErr VFKDataBlockSQLite::SaveGeometryToDB(const OGRGeometry *poGeom, int iRowI
 	nWKBLen = poGeom->WkbSize();
 	pabyWKB = (GByte *) CPLMalloc(nWKBLen + 1);
 	poGeom->exportToWkb(wkbNDR, pabyWKB);
-        
+
 	osSQL.Printf("UPDATE %s SET %s = ? WHERE rowid = %d",
 		     m_pszName, GEOM_COLUMN, iRowId);
 	hStmt = poReader->PrepareStatement(osSQL.c_str());
-	
+
 	rc = sqlite3_bind_blob(hStmt, 1, pabyWKB, nWKBLen, CPLFree);
 	if (rc != SQLITE_OK) {
 	    sqlite3_finalize(hStmt);
-	    CPLError(CE_Failure, CPLE_AppDefined, 
+	    CPLError(CE_Failure, CPLE_AppDefined,
 		     "Storing geometry in DB failed");
 	    return OGRERR_FAILURE;
 	}
@@ -842,18 +904,20 @@ OGRErr VFKDataBlockSQLite::SaveGeometryToDB(const OGRGeometry *poGeom, int iRowI
 bool VFKDataBlockSQLite::LoadGeometryFromDB()
 {
     int nInvalid, nGeometries, nGeometriesCount, nBytes, rowId;
-    long iFID;
-    bool bAddFeature, bSkipInvalid;
-    
+#ifdef DEBUG
+    GIntBig iFID;
+#endif
+    bool bSkipInvalid;
+
     CPLString osSQL;
-    
+
     OGRGeometry      *poGeometry;
-    
+
     VFKFeatureSQLite *poFeature;
     VFKReaderSQLite  *poReader;
-    
+
     sqlite3_stmt *hStmt;
-    
+
     poReader = (VFKReaderSQLite*) m_poReader;
 
     if (!poReader->IsSpatial())   /* check if DB is spatial */
@@ -866,13 +930,12 @@ bool VFKDataBlockSQLite::LoadGeometryFromDB()
         return FALSE;
     nGeometries = sqlite3_column_int(hStmt, 0);
     sqlite3_finalize(hStmt);
-    
+
     if (nGeometries < 1)
 	return FALSE;
-    
-    bAddFeature = EQUAL(m_pszName, "SBP");
+
     bSkipInvalid = EQUAL(m_pszName, "OB") || EQUAL(m_pszName, "OP") || EQUAL(m_pszName, "OBBP");
-    
+
     /* load geometry from DB */
     nInvalid = nGeometriesCount = 0;
     osSQL.Printf("SELECT %s,rowid,%s FROM %s ",
@@ -883,20 +946,16 @@ bool VFKDataBlockSQLite::LoadGeometryFromDB()
     osSQL += FID_COLUMN;
     hStmt = poReader->PrepareStatement(osSQL.c_str());
 
+    rowId = 0;
     while(poReader->ExecuteSQL(hStmt) == OGRERR_NONE) {
-        rowId = sqlite3_column_int(hStmt, 1);
-        iFID = sqlite3_column_double(hStmt, 2);
+        rowId++; // =sqlite3_column_int(hStmt, 1);
+#ifdef DEBUG
+        iFID = sqlite3_column_int64(hStmt, 2);
+#endif
 
-        if (bAddFeature) {
-            /* add feature to the array */
-            poFeature = new VFKFeatureSQLite(this, rowId, iFID);
-            AddFeature(poFeature);
-        }
-        else {
-            poFeature = (VFKFeatureSQLite *) GetFeatureByIndex(rowId - 1);
-        }
+        poFeature = (VFKFeatureSQLite *) GetFeatureByIndex(rowId - 1);
         CPLAssert(NULL != poFeature && poFeature->GetFID() == iFID);
-        
+
         // read geometry from DB
 	nBytes = sqlite3_column_bytes(hStmt, 0);
 	if (nBytes > 0 &&
@@ -917,14 +976,14 @@ bool VFKDataBlockSQLite::LoadGeometryFromDB()
 	     m_pszName, nGeometriesCount);
 
     if (nGeometriesCount != nGeometries) {
-	CPLError(CE_Warning, CPLE_AppDefined, 
+	CPLError(CE_Warning, CPLE_AppDefined,
                  "%s: %d geometries loaded (should be %d)",
 		 m_pszName, nGeometriesCount, nGeometries);
     }
 
     if (nInvalid > 0 && !bSkipInvalid) {
-	CPLError(CE_Warning, CPLE_AppDefined, 
-                 "%s: %d features with invalid or empty geometry found",
+	CPLError(CE_Warning, CPLE_AppDefined,
+                 "%s: %d features with invalid or empty geometry",
 		 m_pszName, nInvalid);
     }
 
@@ -937,18 +996,27 @@ bool VFKDataBlockSQLite::LoadGeometryFromDB()
   \param nGeometries number of geometries to update
 */
 void VFKDataBlockSQLite::UpdateVfkBlocks(int nGeometries) {
+    int nFeatCount;
     CPLString osSQL;
-    
+
     VFKReaderSQLite  *poReader;
-    
+
     poReader = (VFKReaderSQLite*) m_poReader;
 
+    /* update number of features in VFK_DB_TABLE table */
+    nFeatCount = (int)GetFeatureCount();
+    if (nFeatCount > 0) {
+        osSQL.Printf("UPDATE %s SET num_features = %d WHERE table_name = '%s'",
+                     VFK_DB_TABLE, nFeatCount, m_pszName);
+        poReader->ExecuteSQL(osSQL.c_str());
+    }
+
+    /* update number of geometries in VFK_DB_TABLE table */
     if (nGeometries > 0) {
-        CPLDebug("OGR-VFK", 
+        CPLDebug("OGR-VFK",
                  "VFKDataBlockSQLite::UpdateVfkBlocks(): name=%s -> "
                  "%d geometries saved to internal DB", m_pszName, nGeometries);
-        
-	/* update number of geometries in VFK_DB_TABLE table */
+
 	osSQL.Printf("UPDATE %s SET num_geometries = %d WHERE table_name = '%s'",
 		     VFK_DB_TABLE, nGeometries, m_pszName);
 	poReader->ExecuteSQL(osSQL.c_str());
@@ -961,15 +1029,15 @@ void VFKDataBlockSQLite::UpdateVfkBlocks(int nGeometries) {
   \param iFID feature id to set up
   \param rowId list of rows to update
 */
-void VFKDataBlockSQLite::UpdateFID(long int iFID, std::vector<int> rowId)
+void VFKDataBlockSQLite::UpdateFID(GIntBig iFID, std::vector<int> rowId)
 {
     CPLString osSQL, osValue;
     VFKReaderSQLite  *poReader;
-    
+
     poReader = (VFKReaderSQLite*) m_poReader;
-    
+
     /* update number of geometries in VFK_DB_TABLE table */
-    osSQL.Printf("UPDATE %s SET %s = %ld WHERE rowid IN (",
+    osSQL.Printf("UPDATE %s SET %s = " CPL_FRMT_GIB " WHERE rowid IN (",
                  m_pszName, FID_COLUMN, iFID);
     for (size_t i = 0; i < rowId.size(); i++) {
 	if (i > 0)
@@ -979,7 +1047,7 @@ void VFKDataBlockSQLite::UpdateFID(long int iFID, std::vector<int> rowId)
 	osSQL += osValue;
     }
     osSQL += ")";
-    
+
     poReader->ExecuteSQL(osSQL.c_str());
 }
 
