@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: vfkfeaturesqlite.cpp 26343 2013-08-20 14:51:07Z martinl $
+ * $Id: vfkfeaturesqlite.cpp 33713 2016-03-12 17:41:57Z goatbar $
  *
  * Project:  VFK Reader - Feature definition (SQLite)
  * Purpose:  Implements VFKFeatureSQLite class.
@@ -45,20 +45,20 @@
 VFKFeatureSQLite::VFKFeatureSQLite(IVFKDataBlock *poDataBlock) : IVFKFeature(poDataBlock)
 {
     m_hStmt  = NULL;
-    m_iRowId = m_poDataBlock->GetFeatureCount() + 1; /* starts at 1 */
+    m_iRowId = (int)m_poDataBlock->GetFeatureCount() + 1; /* starts at 1 */
 
     /* set FID from DB */
     SetFIDFromDB(); /* -> m_nFID */
 }
 
 /*!
-  \brief VFKFeatureSQLite constructor 
+  \brief VFKFeatureSQLite constructor
 
   \param poDataBlock pointer to related IVFKDataBlock
   \param iRowId feature DB rowid (starts at 1)
   \param nFID feature id
 */
-VFKFeatureSQLite::VFKFeatureSQLite(IVFKDataBlock *poDataBlock, int iRowId, long nFID) : IVFKFeature(poDataBlock)
+VFKFeatureSQLite::VFKFeatureSQLite(IVFKDataBlock *poDataBlock, int iRowId, GIntBig nFID) : IVFKFeature(poDataBlock)
 {
     m_hStmt  = NULL;
     m_iRowId = iRowId;
@@ -71,19 +71,29 @@ VFKFeatureSQLite::VFKFeatureSQLite(IVFKDataBlock *poDataBlock, int iRowId, long
 OGRErr VFKFeatureSQLite::SetFIDFromDB()
 {
     CPLString   osSQL;
-    
+
     osSQL.Printf("SELECT %s FROM %s WHERE rowid = %d",
                  FID_COLUMN, m_poDataBlock->GetName(), m_iRowId);
     if (ExecuteSQL(osSQL.c_str()) != OGRERR_NONE)
         return OGRERR_FAILURE;
 
     m_nFID = sqlite3_column_int(m_hStmt, 0);
-  
+
     FinalizeSQL();
-    
+
     return OGRERR_NONE;
 }
 
+/*!
+  \brief Set DB row id
+
+  \param iRowId row id to be set
+*/
+void VFKFeatureSQLite::SetRowId(int iRowId)
+{
+    m_iRowId = iRowId;
+}
+
 /*!
   \brief Finalize SQL statement
 */
@@ -105,17 +115,17 @@ OGRErr VFKFeatureSQLite::ExecuteSQL(const char *pszSQLCommand)
     int rc;
 
     sqlite3  *poDB;
-    
+
     VFKReaderSQLite *poReader = (VFKReaderSQLite *) m_poDataBlock->GetReader();
     poDB = poReader->m_poDB;
-    
-    rc = sqlite3_prepare(poDB, pszSQLCommand, strlen(pszSQLCommand),
+
+    rc = sqlite3_prepare(poDB, pszSQLCommand, -1,
                          &m_hStmt, NULL);
     if (rc != SQLITE_OK) {
-        CPLError(CE_Failure, CPLE_AppDefined, 
+        CPLError(CE_Failure, CPLE_AppDefined,
                  "In ExecuteSQL(): sqlite3_prepare(%s):\n  %s",
                  pszSQLCommand, sqlite3_errmsg(poDB));
-        
+
         if(m_hStmt != NULL) {
             FinalizeSQL();
         }
@@ -123,14 +133,14 @@ OGRErr VFKFeatureSQLite::ExecuteSQL(const char *pszSQLCommand)
     }
     rc = sqlite3_step(m_hStmt);
     if (rc != SQLITE_ROW) {
-        CPLError(CE_Failure, CPLE_AppDefined, 
-                 "In ExecuteSQL(): sqlite3_step(%s):\n  %s", 
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "In ExecuteSQL(): sqlite3_step(%s):\n  %s",
                  pszSQLCommand, sqlite3_errmsg(poDB));
-        
+
         if (m_hStmt) {
             FinalizeSQL();
         }
-        
+
         return OGRERR_FAILURE;
     }
 
@@ -146,14 +156,14 @@ VFKFeatureSQLite::VFKFeatureSQLite(const VFKFeature *poVFKFeature) : IVFKFeature
 {
     m_nFID   = poVFKFeature->m_nFID;
     m_hStmt  = NULL;
-    m_iRowId = m_poDataBlock->GetFeatureCount() + 1; /* starts at 1 */
+    m_iRowId = (int)m_poDataBlock->GetFeatureCount() + 1; /* starts at 1 */
 }
 
 /*!
   \brief Load geometry (point layers)
 
   \todo Implement (really needed?)
-  
+
   \return TRUE on success or FALSE on failure
 */
 bool VFKFeatureSQLite::LoadGeometryPoint()
