@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrvrtdatasource.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: ogrvrtdatasource.cpp 34048 2016-04-21 18:14:02Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRVRTDataSource class.
@@ -34,7 +34,7 @@
 #include "ogrwarpedlayer.h"
 #include "ogrunionlayer.h"
 
-CPL_CVSID("$Id: ogrvrtdatasource.cpp 27729 2014-09-24 00:40:16Z goatbar $");
+CPL_CVSID("$Id: ogrvrtdatasource.cpp 34048 2016-04-21 18:14:02Z rouault $");
 
 /************************************************************************/
 /*                       OGRVRTGetGeometryType()                        */
@@ -55,18 +55,25 @@ static const OGRGeomTypeName asGeomTypeNames[] = { /* 25D versions are implicit
     { wkbMultiLineString, "wkbMultiLineString" },
     { wkbMultiPolygon, "wkbMultiPolygon" },
     { wkbGeometryCollection, "wkbGeometryCollection" },
+    { wkbCircularString, "wkbCircularString" },
+    { wkbCompoundCurve, "wkbCompoundCurve" },
+    { wkbCurvePolygon, "wkbCurvePolygon" },
+    { wkbMultiCurve, "wkbMultiCurve" },
+    { wkbMultiSurface, "wkbMultiSurface" },
+    { wkbCurve, "wkbCurve" },
+    { wkbSurface, "wkbSurface" },
     { wkbNone, "wkbNone" },
     { wkbNone, NULL }
 };
 
 OGRwkbGeometryType OGRVRTGetGeometryType(const char* pszGType, int* pbError)
 {
-    int iType;
-    OGRwkbGeometryType eGeomType = wkbUnknown;
-
     if (pbError)
         *pbError = FALSE;
 
+    OGRwkbGeometryType eGeomType = wkbUnknown;
+    int iType;
+
     for( iType = 0; asGeomTypeNames[iType].pszName != NULL; iType++ )
     {
         if( EQUALN(pszGType, asGeomTypeNames[iType].pszName,
@@ -74,8 +81,10 @@ OGRwkbGeometryType OGRVRTGetGeometryType(const char* pszGType, int* pbError)
         {
             eGeomType = asGeomTypeNames[iType].eType;
 
-            if( strstr(pszGType,"25D") != NULL )
-                eGeomType = (OGRwkbGeometryType) (eGeomType | wkb25DBit);
+            if( strstr(pszGType,"25D") != NULL || strstr(pszGType,"Z") != NULL )
+                eGeomType = wkbSetZ(eGeomType);
+            if( pszGType[strlen(pszGType)-1] == 'M' || pszGType[strlen(pszGType)-2] == 'M' )
+                eGeomType = wkbSetM(eGeomType);
             break;
         }
     }
@@ -93,17 +102,18 @@ OGRwkbGeometryType OGRVRTGetGeometryType(const char* pszGType, int* pbError)
 /*                          OGRVRTDataSource()                          */
 /************************************************************************/
 
-OGRVRTDataSource::OGRVRTDataSource()
-
+OGRVRTDataSource::OGRVRTDataSource(GDALDriver* poDriverIn) :
+    papoLayers(NULL),
+    paeLayerType(NULL),
+    nLayers(0),
+    pszName(NULL),
+    psTree(NULL),
+    nCallLevel(0),
+    poLayerPool(NULL),
+    poParentDS(NULL),
+    bRecursionDetected(FALSE)
 {
-    pszName = NULL;
-    papoLayers = NULL;
-    nLayers = 0;
-    psTree = NULL;
-    nCallLevel = 0;
-    poLayerPool = NULL;
-    poParentDS = NULL;
-    bRecursionDetected = FALSE;
+    this->poDriver = poDriverIn;
 }
 
 /************************************************************************/
@@ -113,14 +123,11 @@ OGRVRTDataSource::OGRVRTDataSource()
 OGRVRTDataSource::~OGRVRTDataSource()
 
 {
-    int         i;
-
     CPLFree( pszName );
 
-    for( i = 0; i < nLayers; i++ )
-        delete papoLayers[i];
-    
-    CPLFree( papoLayers );
+    CloseDependentDatasets();
+
+    CPLFree( paeLayerType );
 
     if( psTree != NULL)
         CPLDestroyXMLNode( psTree );
@@ -129,10 +136,27 @@ OGRVRTDataSource::~OGRVRTDataSource()
 }
 
 /************************************************************************/
-/*                        InstanciateWarpedLayer()                      */
+/*                        CloseDependentDatasets()                      */
 /************************************************************************/
 
-OGRLayer*  OGRVRTDataSource::InstanciateWarpedLayer(
+int OGRVRTDataSource::CloseDependentDatasets()
+{
+    int bHasClosedDependentDatasets = (nLayers > 0);
+    for( int i = 0; i < nLayers; i++ )
+    {
+        delete papoLayers[i];
+    }
+    CPLFree( papoLayers );
+    nLayers = 0;
+    papoLayers = NULL;
+    return bHasClosedDependentDatasets;
+}
+
+/************************************************************************/
+/*                        InstantiateWarpedLayer()                      */
+/************************************************************************/
+
+OGRLayer*  OGRVRTDataSource::InstantiateWarpedLayer(
                                         CPLXMLNode *psLTree,
                                         const char *pszVRTDirectory,
                                         int bUpdate,
@@ -141,17 +165,16 @@ OGRLayer*  OGRVRTDataSource::InstanciateWarpedLayer(
     if( !EQUAL(psLTree->pszValue,"OGRVRTWarpedLayer") )
         return NULL;
 
-    CPLXMLNode *psSubNode;
     OGRLayer* poSrcLayer = NULL;
 
-    for( psSubNode=psLTree->psChild;
+    for( CPLXMLNode *psSubNode=psLTree->psChild;
          psSubNode != NULL;
          psSubNode=psSubNode->psNext )
     {
         if( psSubNode->eType != CXT_Element )
             continue;
 
-        poSrcLayer = InstanciateLayer(psSubNode, pszVRTDirectory,
+        poSrcLayer = InstantiateLayer(psSubNode, pszVRTDirectory,
                                  bUpdate, nRecLevel + 1);
         if( poSrcLayer != NULL )
             break;
@@ -160,7 +183,7 @@ OGRLayer*  OGRVRTDataSource::InstanciateWarpedLayer(
     if( poSrcLayer == NULL )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
-                  "Cannot instanciate source layer" );
+                  "Cannot instantiate source layer" );
         return NULL;
     }
 
@@ -187,15 +210,17 @@ OGRLayer*  OGRVRTDataSource::InstanciateWarpedLayer(
         }
     }
 
-    OGRSpatialReference* poSrcSRS;
-    OGRSpatialReference* poTargetSRS;
+    OGRSpatialReference* poSrcSRS = NULL;
     const char* pszSourceSRS = CPLGetXMLValue(psLTree, "SrcSRS", NULL);
 
     if( pszSourceSRS == NULL )
     {
-        poSrcSRS = poSrcLayer->GetLayerDefn()->GetGeomFieldDefn(iGeomField)->GetSpatialRef();
-        if( poSrcSRS != NULL)
-            poSrcSRS = poSrcSRS->Clone();
+        if( iGeomField < poSrcLayer->GetLayerDefn()->GetGeomFieldCount() )
+        {
+            poSrcSRS = poSrcLayer->GetLayerDefn()->GetGeomFieldDefn(iGeomField)->GetSpatialRef();
+            if( poSrcSRS != NULL)
+                poSrcSRS = poSrcSRS->Clone();
+        }
     }
     else
     {
@@ -215,7 +240,7 @@ OGRLayer*  OGRVRTDataSource::InstanciateWarpedLayer(
         return NULL;
     }
 
-    poTargetSRS = new OGRSpatialReference();
+    OGRSpatialReference* poTargetSRS = new OGRSpatialReference();
     if( poTargetSRS->SetFromUserInput(pszTargetSRS) != OGRERR_NONE )
     {
         delete poTargetSRS;
@@ -279,17 +304,15 @@ OGRLayer*  OGRVRTDataSource::InstanciateWarpedLayer(
 }
 
 /************************************************************************/
-/*                        InstanciateUnionLayer()                       */
+/*                        InstantiateUnionLayer()                       */
 /************************************************************************/
 
-OGRLayer*  OGRVRTDataSource::InstanciateUnionLayer(
+OGRLayer*  OGRVRTDataSource::InstantiateUnionLayer(
                                         CPLXMLNode *psLTree,
                                         const char *pszVRTDirectory,
                                         int bUpdate,
                                         int nRecLevel)
 {
-    CPLXMLNode *psSubNode;
-
     if( !EQUAL(psLTree->pszValue,"OGRVRTUnionLayer") )
         return NULL;
 
@@ -302,7 +325,7 @@ OGRLayer*  OGRVRTDataSource::InstanciateUnionLayer(
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "Missing name attribute on OGRVRTUnionLayer" );
-        return FALSE;
+        return NULL;
     }
 
 /* -------------------------------------------------------------------- */
@@ -343,7 +366,7 @@ OGRLayer*  OGRVRTDataSource::InstanciateUnionLayer(
              {
                  CPLError( CE_Failure, CPLE_AppDefined,
                            "Failed to import LayerSRS `%s'.", pszLayerSRS );
-                 return FALSE;
+                 return NULL;
              }
              poGlobalSRS = oSRS.Clone();
          }
@@ -357,6 +380,7 @@ OGRLayer*  OGRVRTDataSource::InstanciateUnionLayer(
     OGRUnionLayerGeomFieldDefn** papoGeomFields = NULL;
     int nGeomFields = 0;
 
+    CPLXMLNode *psSubNode;
     for( psSubNode=psLTree->psChild;
          psSubNode != NULL;
          psSubNode=psSubNode->psNext )
@@ -369,15 +393,15 @@ OGRLayer*  OGRVRTDataSource::InstanciateUnionLayer(
 /* -------------------------------------------------------------------- */
 /*      Field name.                                                     */
 /* -------------------------------------------------------------------- */
-             const char *pszName = CPLGetXMLValue( psSubNode, "name", NULL );
-             if( pszName == NULL )
+             const char *l_pszName = CPLGetXMLValue( psSubNode, "name", NULL );
+             if( l_pszName == NULL )
              {
                  CPLError( CE_Failure, CPLE_AppDefined,
                            "Unable to identify Field name." );
                  break;
              }
 
-             OGRFieldDefn oFieldDefn( pszName, OFTString );
+             OGRFieldDefn oFieldDefn( l_pszName, OFTString );
 
 /* -------------------------------------------------------------------- */
 /*      Type                                                            */
@@ -415,7 +439,7 @@ OGRLayer*  OGRVRTDataSource::InstanciateUnionLayer(
              {
                 CPLError( CE_Failure, CPLE_IllegalArg,
                           "Invalid width for field %s.",
-                          pszName );
+                          l_pszName );
                 break;
              }
              oFieldDefn.SetWidth(nWidth);
@@ -425,7 +449,7 @@ OGRLayer*  OGRVRTDataSource::InstanciateUnionLayer(
              {
                 CPLError( CE_Failure, CPLE_IllegalArg,
                           "Invalid precision for field %s.",
-                          pszName );
+                          l_pszName );
                 break;
              }
              oFieldDefn.SetPrecision(nPrecision);
@@ -439,8 +463,8 @@ OGRLayer*  OGRVRTDataSource::InstanciateUnionLayer(
          else if( psSubNode->eType == CXT_Element &&
                   EQUAL(psSubNode->pszValue,"GeometryField") )
          {
-             const char *pszName = CPLGetXMLValue( psSubNode, "name", NULL );
-             if( pszName == NULL )
+             const char *l_pszName = CPLGetXMLValue( psSubNode, "name", NULL );
+             if( l_pszName == NULL )
              {
                  CPLError( CE_Failure, CPLE_AppDefined,
                            "Unable to identify GeometryField name." );
@@ -489,7 +513,7 @@ OGRLayer*  OGRVRTDataSource::InstanciateUnionLayer(
              }
 
              OGRUnionLayerGeomFieldDefn* poFieldDefn =
-                    new OGRUnionLayerGeomFieldDefn(pszName, eGeomType);
+                    new OGRUnionLayerGeomFieldDefn(l_pszName, eGeomType);
              if( poSRS != NULL )
              {
                 poFieldDefn->SetSpatialRef(poSRS);
@@ -575,7 +599,7 @@ OGRLayer*  OGRVRTDataSource::InstanciateUnionLayer(
         if( psSubNode->eType != CXT_Element )
             continue;
 
-        OGRLayer* poSrcLayer = InstanciateLayer(psSubNode, pszVRTDirectory,
+        OGRLayer* poSrcLayer = InstantiateLayer(psSubNode, pszVRTDirectory,
                                            bUpdate, nRecLevel + 1);
         if( poSrcLayer != NULL )
         {
@@ -621,7 +645,7 @@ OGRLayer*  OGRVRTDataSource::InstanciateUnionLayer(
     int bPreserveSrcFID = FALSE;
     const char* pszPreserveFID = CPLGetXMLValue( psLTree, "PreserveSrcFID", NULL );
     if( pszPreserveFID != NULL )
-        bPreserveSrcFID = CSLTestBoolean(pszPreserveFID);
+        bPreserveSrcFID = CPLTestBool(pszPreserveFID);
     poLayer->SetPreserveSrcFID(bPreserveSrcFID);
 
 /* -------------------------------------------------------------------- */
@@ -654,11 +678,11 @@ OGRLayer*  OGRVRTDataSource::InstanciateUnionLayer(
     poLayer->SetFields(eFieldStrategy, nFields, papoFields,
                        (nGeomFields == 0 && eGlobalGeomType == wkbNone) ? -1 : nGeomFields,
                        papoGeomFields);
-    int iField;
-    for(iField = 0; iField < nFields; iField++)
+
+    for(int iField = 0; iField < nFields; iField++)
         delete papoFields[iField];
     CPLFree(papoFields);
-    for(iField = 0; iField < nGeomFields; iField++)
+    for(int iField = 0; iField < nGeomFields; iField++)
         delete papoGeomFields[iField];
     CPLFree(papoGeomFields);
 
@@ -675,10 +699,10 @@ OGRLayer*  OGRVRTDataSource::InstanciateUnionLayer(
 }
 
 /************************************************************************/
-/*                     InstanciateLayerInternal()                       */
+/*                     InstantiateLayerInternal()                       */
 /************************************************************************/
 
-OGRLayer* OGRVRTDataSource::InstanciateLayerInternal(CPLXMLNode *psLTree,
+OGRLayer* OGRVRTDataSource::InstantiateLayerInternal(CPLXMLNode *psLTree,
                                                 const char *pszVRTDirectory,
                                                 int bUpdate,
                                                 int nRecLevel)
@@ -700,12 +724,12 @@ OGRLayer* OGRVRTDataSource::InstanciateLayerInternal(CPLXMLNode *psLTree,
     }
     else if( EQUAL(psLTree->pszValue,"OGRVRTWarpedLayer") && nRecLevel < 30 )
     {
-        return InstanciateWarpedLayer( psLTree, pszVRTDirectory,
+        return InstantiateWarpedLayer( psLTree, pszVRTDirectory,
                                        bUpdate, nRecLevel + 1 );
     }
     else if( EQUAL(psLTree->pszValue,"OGRVRTUnionLayer") && nRecLevel < 30 )
     {
-        return InstanciateUnionLayer( psLTree, pszVRTDirectory,
+        return InstantiateUnionLayer( psLTree, pszVRTDirectory,
                                       bUpdate, nRecLevel + 1 );
     }
     else
@@ -727,7 +751,7 @@ typedef struct
 static OGRLayer* OGRVRTOpenProxiedLayer(void* pUserData)
 {
     PooledInitData* pData = (PooledInitData*) pUserData;
-    return pData->poDS->InstanciateLayerInternal(pData->psNode,
+    return pData->poDS->InstantiateLayerInternal(pData->psNode,
                                             pData->pszVRTDirectory,
                                             pData->bUpdate,
                                             0);
@@ -745,10 +769,10 @@ static void OGRVRTFreeProxiedLayerUserData(void* pUserData)
 }
 
 /************************************************************************/
-/*                          InstanciateLayer()                          */
+/*                          InstantiateLayer()                          */
 /************************************************************************/
 
-OGRLayer* OGRVRTDataSource::InstanciateLayer(CPLXMLNode *psLTree,
+OGRLayer* OGRVRTDataSource::InstantiateLayer(CPLXMLNode *psLTree,
                                         const char *pszVRTDirectory,
                                         int bUpdate,
                                         int nRecLevel)
@@ -765,11 +789,9 @@ OGRLayer* OGRVRTDataSource::InstanciateLayer(CPLXMLNode *psLTree,
                                     OGRVRTFreeProxiedLayerUserData,
                                     pData);
     }
-    else
-    {
-        return InstanciateLayerInternal(psLTree, pszVRTDirectory,
+
+    return InstantiateLayerInternal(psLTree, pszVRTDirectory,
                                     bUpdate, nRecLevel);
-    }
 }
 
 /************************************************************************/
@@ -783,10 +805,9 @@ static int CountOGRVRTLayers(CPLXMLNode *psTree)
 
     int nCount = 0;
     if( EQUAL(psTree->pszValue, "OGRVRTLayer") )
-        nCount ++;
+        ++nCount;
 
-    CPLXMLNode* psNode;
-    for( psNode=psTree->psChild; psNode != NULL; psNode=psNode->psNext )
+    for( CPLXMLNode *psNode=psTree->psChild; psNode != NULL; psNode=psNode->psNext )
     {
         nCount += CountOGRVRTLayers(psNode);
     }
@@ -798,13 +819,13 @@ static int CountOGRVRTLayers(CPLXMLNode *psTree)
 /*                             Initialize()                             */
 /************************************************************************/
 
-int OGRVRTDataSource::Initialize( CPLXMLNode *psTree, const char *pszNewName,
+int OGRVRTDataSource::Initialize( CPLXMLNode *psTreeIn, const char *pszNewName,
                                   int bUpdate )
 
 {
     CPLAssert( nLayers == 0 );
 
-    this->psTree = psTree;
+    this->psTree = psTreeIn;
 
 /* -------------------------------------------------------------------- */
 /*      Set name, and capture the directory path so we can use it       */
@@ -821,7 +842,7 @@ int OGRVRTDataSource::Initialize( CPLXMLNode *psTree, const char *pszNewName,
     CPLXMLNode *psVRTDSXML = CPLGetXMLNode( psTree, "=OGRVRTDataSource" );
     if( psVRTDSXML == NULL )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "Did not find the <OGRVRTDataSource> node in the root of the document,\n"
                   "this is not really an OGR VRT." );
         return FALSE;
@@ -838,12 +859,16 @@ int OGRVRTDataSource::Initialize( CPLXMLNode *psTree, const char *pszNewName,
     if( nOGRVRTLayerCount > nMaxSimultaneouslyOpened )
         poLayerPool = new OGRLayerPool(nMaxSimultaneouslyOpened);
 
+/* -------------------------------------------------------------------- */
+/*      Apply any dataset level metadata.                               */
+/* -------------------------------------------------------------------- */
+    oMDMD.XMLInit( psVRTDSXML, TRUE );
+
 /* -------------------------------------------------------------------- */
 /*      Look for layers.                                                */
 /* -------------------------------------------------------------------- */
-    CPLXMLNode *psLTree;
 
-    for( psLTree=psVRTDSXML->psChild; psLTree != NULL; psLTree=psLTree->psNext )
+    for( CPLXMLNode *psLTree=psVRTDSXML->psChild; psLTree != NULL; psLTree=psLTree->psNext )
     {
         if( psLTree->eType != CXT_Element )
             continue;
@@ -851,16 +876,32 @@ int OGRVRTDataSource::Initialize( CPLXMLNode *psTree, const char *pszNewName,
 /* -------------------------------------------------------------------- */
 /*      Create the layer object.                                        */
 /* -------------------------------------------------------------------- */
-        OGRLayer  *poLayer = InstanciateLayer(psLTree, osVRTDirectory, bUpdate);
+        OGRLayer  *poLayer = InstantiateLayer(psLTree, osVRTDirectory, bUpdate);
         if( poLayer == NULL )
             continue;
 
 /* -------------------------------------------------------------------- */
 /*      Add layer to data source layer list.                            */
 /* -------------------------------------------------------------------- */
+        nLayers ++;
         papoLayers = (OGRLayer **)
-            CPLRealloc( papoLayers,  sizeof(OGRLayer *) * (nLayers+1) );
-        papoLayers[nLayers++] = poLayer;
+            CPLRealloc( papoLayers,  sizeof(OGRLayer *) * nLayers );
+        papoLayers[nLayers-1] = poLayer;
+
+        paeLayerType = (OGRLayerType*)
+            CPLRealloc( paeLayerType,  sizeof(int) * nLayers );
+        if( poLayerPool != NULL && EQUAL(psLTree->pszValue,"OGRVRTLayer"))
+        {
+            paeLayerType[nLayers - 1] = OGR_VRT_PROXIED_LAYER;
+        }
+        else if( EQUAL(psLTree->pszValue,"OGRVRTLayer") )
+        {
+            paeLayerType[nLayers - 1] = OGR_VRT_LAYER;
+        }
+        else
+        {
+            paeLayerType[nLayers - 1] = OGR_VRT_OTHER_LAYER;
+        }
     }
 
     return TRUE;
@@ -870,8 +911,10 @@ int OGRVRTDataSource::Initialize( CPLXMLNode *psTree, const char *pszNewName,
 /*                           TestCapability()                           */
 /************************************************************************/
 
-int OGRVRTDataSource::TestCapability( CPL_UNUSED const char * pszCap )
+int OGRVRTDataSource::TestCapability( const char * pszCap )
 {
+    if( EQUAL(pszCap,ODsCCurveGeometries) )
+        return TRUE;
     return FALSE;
 }
 
@@ -884,8 +927,8 @@ OGRLayer *OGRVRTDataSource::GetLayer( int iLayer )
 {
     if( iLayer < 0 || iLayer >= nLayers )
         return NULL;
-    else
-        return papoLayers[iLayer];
+
+    return papoLayers[iLayer];
 }
 
 /************************************************************************/
@@ -905,3 +948,45 @@ int OGRVRTDataSource::IsInForbiddenNames(const char* pszOtherDSName)
 {
     return aosOtherDSNameSet.find(pszOtherDSName) != aosOtherDSNameSet.end();
 }
+
+/************************************************************************/
+/*                             GetFileList()                             */
+/************************************************************************/
+
+char **OGRVRTDataSource::GetFileList()
+{
+    CPLStringList oList;
+    oList.AddString( GetName() );
+    for(int i=0; i<nLayers; i++ )
+    {
+        OGRLayer* poLayer = papoLayers[i];
+        OGRVRTLayer* poVRTLayer = NULL;
+        switch( paeLayerType[nLayers - 1] )
+        {
+            case OGR_VRT_PROXIED_LAYER:
+                poVRTLayer = (OGRVRTLayer*) ((OGRProxiedLayer*)poLayer)->GetUnderlyingLayer();
+                break;
+            case OGR_VRT_LAYER:
+                poVRTLayer = (OGRVRTLayer*) poLayer;
+                break;
+            default:
+                break;
+        }
+        if( poVRTLayer != NULL )
+        {
+            GDALDataset* poSrcDS = poVRTLayer->GetSrcDataset();
+            if( poSrcDS != NULL )
+            {
+                char** papszFileList = poSrcDS->GetFileList();
+                char** papszIter = papszFileList;
+                for(; papszIter != NULL && *papszIter != NULL; papszIter++ )
+                {
+                    if( oList.FindString(*papszIter) < 0 )
+                        oList.AddString(*papszIter);
+                }
+                CSLDestroy(papszFileList);
+            }
+        }
+    }
+    return oList.StealList();
+}
