@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrvrtdriver.cpp 27729 2014-09-24 00:40:16Z goatbar $
+ * $Id: ogrvrtdriver.cpp 32982 2016-01-14 16:53:57Z goatbar $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRVRTDriver class.
@@ -31,56 +31,66 @@
 #include "ogr_vrt.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrvrtdriver.cpp 27729 2014-09-24 00:40:16Z goatbar $");
-
-/************************************************************************/
-/*                            ~OGRVRTDriver()                            */
-/************************************************************************/
-
-OGRVRTDriver::~OGRVRTDriver()
-{
-}
-
-/************************************************************************/
-/*                              GetName()                               */
-/************************************************************************/
-
-const char *OGRVRTDriver::GetName()
-{
-    return "VRT";
-}
+CPL_CVSID("$Id: ogrvrtdriver.cpp 32982 2016-01-14 16:53:57Z goatbar $");
 
 /************************************************************************/
 /*                           OGRVRTErrorHandler()                       */
 /************************************************************************/
 
 static void CPL_STDCALL OGRVRTErrorHandler(CPL_UNUSED CPLErr eErr,
-                                           CPL_UNUSED int nType,
+                                           CPL_UNUSED CPLErrorNum nType,
                                            const char* pszMsg)
 {
     std::vector<CPLString>* paosErrors = (std::vector<CPLString>* )CPLGetErrorHandlerUserData();
     paosErrors->push_back(pszMsg);
 }
 
+/************************************************************************/
+/*                         OGRVRTDriverIdentify()                       */
+/************************************************************************/
+
+static int OGRVRTDriverIdentify( GDALOpenInfo* poOpenInfo )
+{
+    if( !poOpenInfo->bStatOK )
+    {
+/* -------------------------------------------------------------------- */
+/*      Are we being passed the XML definition directly?                */
+/*      Skip any leading spaces/blanks.                                 */
+/* -------------------------------------------------------------------- */
+        const char *pszTestXML = poOpenInfo->pszFilename;
+        while( *pszTestXML != '\0' && isspace( (unsigned char)*pszTestXML ) )
+            pszTestXML++;
+        if( STARTS_WITH_CI(pszTestXML, "<OGRVRTDataSource>") )
+        {
+            return TRUE;
+        }
+        return FALSE;
+    }
+
+    return ( poOpenInfo->fpL != NULL &&
+             strstr((const char*)poOpenInfo->pabyHeader,"<OGRVRTDataSource") != NULL );
+}
+
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
 
-OGRDataSource *OGRVRTDriver::Open( const char * pszFilename,
-                                   int bUpdate )
+static GDALDataset *OGRVRTDriverOpen( GDALOpenInfo* poOpenInfo )
+
 {
-    OGRVRTDataSource     *poDS;
-    char *pszXML = NULL;
+    if( !OGRVRTDriverIdentify(poOpenInfo) )
+        return NULL;
 
 /* -------------------------------------------------------------------- */
 /*      Are we being passed the XML definition directly?                */
 /*      Skip any leading spaces/blanks.                                 */
 /* -------------------------------------------------------------------- */
-    const char *pszTestXML = pszFilename;
+    const char *pszTestXML = poOpenInfo->pszFilename;
     while( *pszTestXML != '\0' && isspace( (unsigned char)*pszTestXML ) )
         pszTestXML++;
 
-    if( EQUALN(pszTestXML,"<OGRVRTDataSource>",18) )
+    char *pszXML = NULL;
+    if( STARTS_WITH_CI(pszTestXML, "<OGRVRTDataSource>") )
     {
         pszXML = CPLStrdup(pszTestXML);
     }
@@ -90,29 +100,11 @@ OGRDataSource *OGRVRTDriver::Open( const char * pszFilename,
 /* -------------------------------------------------------------------- */
     else
     {
-        VSILFILE *fp;
-        char achHeader[512];
-
-        fp = VSIFOpenL( pszFilename, "rb" );
-
-        if( fp == NULL )
-            return NULL;
-
-        memset( achHeader, 0, sizeof(achHeader) );
-        VSIFReadL( achHeader, 1, sizeof(achHeader)-1, fp );
-
-        if( strstr(achHeader,"<OGRVRTDataSource") == NULL )
-        {
-            VSIFCloseL( fp );
-            return NULL;
-        }
-
         VSIStatBufL sStatBuf;
-        if ( VSIStatL( pszFilename, &sStatBuf ) != 0 ||
+        if ( VSIStatL( poOpenInfo->pszFilename, &sStatBuf ) != 0 ||
              sStatBuf.st_size > 1024 * 1024 )
         {
             CPLDebug( "VRT", "Unreasonable long file, not likely really VRT" );
-            VSIFCloseL( fp );
             return NULL;
         }
 
@@ -121,23 +113,19 @@ OGRDataSource *OGRVRTDriver::Open( const char * pszFilename,
 /* -------------------------------------------------------------------- */
         int nLen = (int) sStatBuf.st_size;
 
-        VSIFSeekL( fp, 0, SEEK_SET );
-
-        pszXML = (char *) VSIMalloc(nLen+1);
+        pszXML = (char *) VSI_MALLOC_VERBOSE(nLen+1);
         if (pszXML == NULL)
-        {
-            VSIFCloseL( fp );
             return NULL;
-        }
+
         pszXML[nLen] = '\0';
-        if( ((int) VSIFReadL( pszXML, 1, nLen, fp )) != nLen )
+        VSIFSeekL( poOpenInfo->fpL, 0, SEEK_SET );
+        if( ((int) VSIFReadL( pszXML, 1, nLen, poOpenInfo->fpL )) != nLen )
         {
             CPLFree( pszXML );
-            VSIFCloseL( fp );
-
             return NULL;
         }
-        VSIFCloseL( fp );
+        VSIFCloseL( poOpenInfo->fpL );
+        poOpenInfo->fpL = NULL;
     }
 
 /* -------------------------------------------------------------------- */
@@ -154,7 +142,7 @@ OGRDataSource *OGRVRTDriver::Open( const char * pszFilename,
 /* -------------------------------------------------------------------- */
 /*      XML Validation.                                                 */
 /* -------------------------------------------------------------------- */
-    if( CSLTestBoolean(CPLGetConfigOption("GDAL_XML_VALIDATION", "YES")) )
+    if( CPLTestBool(CPLGetConfigOption("GDAL_XML_VALIDATION", "YES")) )
     {
         const char* pszXSD = CPLFindFile( "gdal", "ogrvrt.xsd" );
         if( pszXSD != NULL )
@@ -182,10 +170,12 @@ OGRDataSource *OGRVRTDriver::Open( const char * pszFilename,
 /* -------------------------------------------------------------------- */
 /*      Create a virtual datasource configured based on this XML input. */
 /* -------------------------------------------------------------------- */
-    poDS = new OGRVRTDataSource();
-    poDS->SetDriver(this);
+    OGRVRTDataSource *poDS
+        = new OGRVRTDataSource((GDALDriver*)GDALGetDriverByName( "OGR_VRT" ));
+
     /* psTree is owned by poDS */
-    if( !poDS->Initialize( psTree, pszFilename, bUpdate ) )
+    if( !poDS->Initialize( psTree, poOpenInfo->pszFilename,
+                           poOpenInfo->eAccess == GA_Update ) )
     {
         delete poDS;
         return NULL;
@@ -194,20 +184,27 @@ OGRDataSource *OGRVRTDriver::Open( const char * pszFilename,
     return poDS;
 }
 
-/************************************************************************/
-/*                           TestCapability()                           */
-/************************************************************************/
-
-int OGRVRTDriver::TestCapability( CPL_UNUSED const char * pszCap )
-{
-    return FALSE;
-}
-
 /************************************************************************/
 /*                           RegisterOGRVRT()                           */
 /************************************************************************/
 
 void RegisterOGRVRT()
+
 {
-    OGRSFDriverRegistrar::GetRegistrar()->RegisterDriver( new OGRVRTDriver );
+    if( GDALGetDriverByName( "OGR_VRT" ) != NULL )
+        return;
+
+    GDALDriver *poDriver = new GDALDriver();
+
+    poDriver->SetDescription( "OGR_VRT" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VECTOR, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "VRT - Virtual Datasource" );
+    poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "vrt" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drv_vrt.html" );
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
+
+    poDriver->pfnOpen = OGRVRTDriverOpen;
+    poDriver->pfnIdentify = OGRVRTDriverIdentify;
+
+    GetGDALDriverManager()->RegisterDriver( poDriver );
 }
