@@ -27,8 +27,8 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-#ifndef _OGR_XPLANE_NAV_READER_H_INCLUDED
-#define _OGR_XPLANE_NAV_READER_H_INCLUDED
+#ifndef OGR_XPLANE_NAV_READER_H_INCLUDED
+#define OGR_XPLANE_NAV_READER_H_INCLUDED
 
 #include "ogr_xplane.h"
 #include "ogr_xplane_reader.h"
@@ -176,10 +176,10 @@ class OGRXPlaneDMELayer : public OGRXPlaneLayer
 
 enum
 {
-    NAVAID_NDB            = 2, 
+    NAVAID_NDB            = 2,
     NAVAID_VOR            = 3, /* VOR, VORTAC or VOR-DME.*/
-    NAVAID_LOC_ILS        = 4, /* Localiser that is part of a full ILS */
-    NAVAID_LOC_STANDALONE = 5, /* Stand-alone localiser (LOC), also including a LDA (Landing Directional Aid) or SDF (Simplified Directional Facility) */
+    NAVAID_LOC_ILS        = 4, /* Localizer that is part of a full ILS */
+    NAVAID_LOC_STANDALONE = 5, /* Stand-alone Localizer (LOC), also including a LDA (Landing Directional Aid) or SDF (Simplified Directional Facility) */
     NAVAID_GS             = 6, /* Glideslope */
     NAVAID_OM             = 7, /* Outer marker */
     NAVAID_MM             = 8, /* Middle marker */
