@@ -3,7 +3,7 @@
  * Component: OGDI Driver Support Library
  * Purpose: Generic SQL WHERE Expression Evaluator Declarations.
  * Author: Frank Warmerdam <warmerdam@pobox.com>
- * 
+ *
  ******************************************************************************
  * Copyright (C) 2001 Information Interoperability Institute (3i)
  * Copyright (c) 2010-2013, Even Rouault <even dot rouault at mines-paris dot org>
@@ -11,24 +11,22 @@
  * its documentation for any purpose and without fee is hereby granted,
  * provided that the above copyright notice appear in all copies, that
  * both the copyright notice and this permission notice appear in
- * supporting documentation, and that the name of 3i not be used 
- * in advertising or publicity pertaining to distribution of the software 
+ * supporting documentation, and that the name of 3i not be used
+ * in advertising or publicity pertaining to distribution of the software
  * without specific, written prior permission.  3i makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.
  ****************************************************************************/
 
-#ifndef _SWQ_H_INCLUDED_
-#define _SWQ_H_INCLUDED_
+#ifndef SWQ_H_INCLUDED_
+#define SWQ_H_INCLUDED_
 
 #include "cpl_conv.h"
 #include "cpl_string.h"
 #include "ogr_core.h"
 
-#if defined(_WIN32) && !defined(_WIN32_WCE)
+#if defined(_WIN32) && !defined(strcasecmp)
 #  define strcasecmp stricmp
-#elif defined(_WIN32_WCE)
-#  define strcasecmp _stricmp
 #endif
 
 typedef enum {
@@ -52,20 +50,22 @@ typedef enum {
     SWQ_MODULUS,
     SWQ_CONCAT,
     SWQ_SUBSTR,
+    SWQ_HSTORE_GET_VALUE,
     SWQ_AVG,
     SWQ_MIN,
     SWQ_MAX,
     SWQ_COUNT,
     SWQ_SUM,
     SWQ_CAST,
-    SWQ_FUNC_DEFINED,
-    SWQ_UNKNOWN
+    SWQ_CUSTOM_FUNC, /* only if parsing done in bAcceptCustomFuncs mode */
+    SWQ_ARGUMENT_LIST /* temporary value only set during parsing and replaced by something else at the end */
 } swq_op;
 
 typedef enum {
     SWQ_INTEGER,
+    SWQ_INTEGER64,
     SWQ_FLOAT,
-    SWQ_STRING, 
+    SWQ_STRING,
     SWQ_BOOLEAN,  // integer
     SWQ_DATE,     // string
     SWQ_TIME,     // string
@@ -76,9 +76,11 @@ typedef enum {
     SWQ_ERROR
 } swq_field_type;
 
+#define SWQ_IS_INTEGER(x) ((x) == SWQ_INTEGER || (x) == SWQ_INTEGER64)
+
 typedef enum {
     SNT_CONSTANT,
-    SNT_COLUMN, 
+    SNT_COLUMN,
     SNT_OPERATION
 } swq_node_type;
 
@@ -92,27 +94,36 @@ typedef swq_expr_node *(*swq_field_fetcher)( swq_expr_node *op,
                                              void *record_handle );
 typedef swq_expr_node *(*swq_op_evaluator)(swq_expr_node *op,
                                            swq_expr_node **sub_field_values );
-typedef swq_field_type (*swq_op_checker)( swq_expr_node *op );
+typedef swq_field_type (*swq_op_checker)( swq_expr_node *op,
+                                          int bAllowMismatchTypeOnFieldComparison );
+
+class swq_custom_func_registrar;
 
 class swq_expr_node {
-    static void   Quote( CPLString &, char chQuote = '\'' );
 public:
     swq_expr_node();
 
-    swq_expr_node( const char * );
-    swq_expr_node( int );
-    swq_expr_node( double );
-    swq_expr_node( OGRGeometry* );
-    swq_expr_node( swq_op );
+    explicit swq_expr_node( const char * );
+    explicit swq_expr_node( int );
+    explicit swq_expr_node( GIntBig );
+    explicit swq_expr_node( double );
+    explicit swq_expr_node( OGRGeometry* );
+    explicit swq_expr_node( swq_op );
 
     ~swq_expr_node();
 
     void           Initialize();
+    CPLString      UnparseOperationFromUnparsedSubExpr(char** apszSubExpr);
     char          *Unparse( swq_field_list *, char chColumnQuote );
     void           Dump( FILE *fp, int depth );
-    swq_field_type Check( swq_field_list *, int bAllowFieldsInSecondaryTables );
-    swq_expr_node* Evaluate( swq_field_fetcher pfnFetcher, 
+    swq_field_type Check( swq_field_list *, int bAllowFieldsInSecondaryTables,
+                          int bAllowMismatchTypeOnFieldComparison,
+                          swq_custom_func_registrar* poCustomFuncRegistrar );
+    swq_expr_node* Evaluate( swq_field_fetcher pfnFetcher,
                              void *record );
+    swq_expr_node* Clone();
+
+    void           ReplaceBetweenByGEAndLERecurse();
 
     swq_node_type eNodeType;
     swq_field_type field_type;
@@ -127,13 +138,20 @@ public:
     /* only for SNT_COLUMN */
     int         field_index;
     int         table_index;
+    char        *table_name;
 
     /* only for SNT_CONSTANT */
     int         is_null;
-    char        *string_value;
-    int         int_value;
+    GIntBig     int_value;
     double      float_value;
     OGRGeometry *geometry_value;
+
+    /* shared by SNT_COLUMN, SNT_CONSTANT and also possibly SNT_OPERATION when */
+    /* nOperation == SWQ_CUSTOM_FUNC */
+    char        *string_value; /* column name when SNT_COLUMN */
+
+    static CPLString   QuoteIfNecessary( const CPLString &, char chQuote = '\'' );
+    static CPLString   Quote( const CPLString &, char chQuote = '\'' );
 };
 
 typedef struct {
@@ -149,6 +167,14 @@ public:
     static const swq_operation *GetOperator( swq_op eOperation );
 };
 
+class swq_custom_func_registrar
+{
+    public:
+        virtual ~swq_custom_func_registrar() {}
+        virtual const swq_operation *GetOperator( const char * ) = 0;
+};
+
+
 typedef struct {
     char       *data_source;
     char       *table_name;
@@ -169,12 +195,15 @@ public:
 
 class swq_parse_context {
 public:
-    swq_parse_context() : nStartToken(0), poRoot(NULL), poCurSelect(NULL) {}
+    swq_parse_context() : nStartToken(0), pszInput(NULL), pszNext(NULL),
+                          pszLastValid(NULL), bAcceptCustomFuncs(FALSE),
+                          poRoot(NULL), poCurSelect(NULL) {}
 
     int        nStartToken;
     const char *pszInput;
     const char *pszNext;
     const char *pszLastValid;
+    int        bAcceptCustomFuncs;
 
     swq_expr_node *poRoot;
 
@@ -182,24 +211,29 @@ public:
 };
 
 /* Compile an SQL WHERE clause into an internal form.  The field_list is
-** the list of fields in the target 'table', used to render where into 
-** field numbers instead of names. 
+** the list of fields in the target 'table', used to render where into
+** field numbers instead of names.
 */
 int swqparse( swq_parse_context *context );
 int swqlex( swq_expr_node **ppNode, swq_parse_context *context );
 void swqerror( swq_parse_context *context, const char *msg );
 
-int swq_identify_field( const char *token, swq_field_list *field_list,
+int swq_identify_field( const char* table_name,
+                        const char *token, swq_field_list *field_list,
                         swq_field_type *this_type, int *table_id );
 
-CPLErr swq_expr_compile( const char *where_clause, 
+CPLErr swq_expr_compile( const char *where_clause,
                          int field_count,
                          char **field_list,
                          swq_field_type *field_types,
+                         int bCheck,
+                         swq_custom_func_registrar* poCustomFuncRegistrar,
                          swq_expr_node **expr_root );
 
-CPLErr swq_expr_compile2( const char *where_clause, 
-                          swq_field_list *field_list, 
+CPLErr swq_expr_compile2( const char *where_clause,
+                          swq_field_list *field_list,
+                          int bCheck,
+                          swq_custom_func_registrar* poCustomFuncRegistrar,
                           swq_expr_node **expr_root );
 
 /*
@@ -208,9 +242,9 @@ CPLErr swq_expr_compile2( const char *where_clause,
 int swq_test_like( const char *input, const char *pattern );
 
 swq_expr_node *SWQGeneralEvaluator( swq_expr_node *, swq_expr_node **);
-swq_field_type SWQGeneralChecker( swq_expr_node *node );
+swq_field_type SWQGeneralChecker( swq_expr_node *node, int bAllowMismatchTypeOnFieldComparison );
 swq_expr_node *SWQCastEvaluator( swq_expr_node *, swq_expr_node **);
-swq_field_type SWQCastChecker( swq_expr_node *node );
+swq_field_type SWQCastChecker( swq_expr_node *node, int bAllowMismatchTypeOnFieldComparison );
 const char*    SWQFieldTypeToString( swq_field_type field_type );
 
 /****************************************************************************/
@@ -233,12 +267,14 @@ typedef enum {
 
 typedef struct {
     swq_col_func col_func;
+    char         *table_name;
     char         *field_name;
     char         *field_alias;
     int          table_index;
     int          field_index;
     swq_field_type field_type;
     swq_field_type target_type;
+    OGRFieldSubType target_subtype;
     int          field_length;
     int          field_precision;
     int          distinct_flag;
@@ -248,8 +284,8 @@ typedef struct {
 } swq_col_def;
 
 typedef struct {
-    int         count;
-    
+    GIntBig     count;
+
     char        **distinct_list; /* items of the list can be NULL */
     double      sum;
     double      min;
@@ -259,6 +295,7 @@ typedef struct {
 } swq_summary;
 
 typedef struct {
+    char *table_name;
     char *field_name;
     int   table_index;
     int   field_index;
@@ -267,18 +304,31 @@ typedef struct {
 
 typedef struct {
     int        secondary_table;
-
-    char      *primary_field_name;
-    int        primary_field;
-
-    swq_op     op;
-
-    char      *secondary_field_name;
-    int        secondary_field;
+    swq_expr_node  *poExpr;
 } swq_join_def;
 
+class swq_select_parse_options
+{
+public:
+    swq_custom_func_registrar* poCustomFuncRegistrar;
+    int                        bAllowFieldsInSecondaryTablesInWhere;
+    int                        bAddSecondaryTablesGeometryFields;
+    int                        bAlwaysPrefixWithTableName;
+    int                        bAllowDistinctOnGeometryField;
+    int                        bAllowDistinctOnMultipleFields;
+
+                    swq_select_parse_options(): poCustomFuncRegistrar(NULL),
+                                                bAllowFieldsInSecondaryTablesInWhere(FALSE),
+                                                bAddSecondaryTablesGeometryFields(FALSE),
+                                                bAlwaysPrefixWithTableName(FALSE),
+                                                bAllowDistinctOnGeometryField(FALSE),
+                                                bAllowDistinctOnMultipleFields(FALSE) {}
+};
+
 class swq_select
 {
+    void        postpreparse();
+
 public:
     swq_select();
     ~swq_select();
@@ -299,26 +349,27 @@ public:
     int         table_count;
     swq_table_def *table_defs;
 
-    void        PushJoin( int iSecondaryTable,
-                          const char *pszPrimaryField,
-                          const char *pszSecondaryField );
+    void        PushJoin( int iSecondaryTable, swq_expr_node* poExpr );
     int         join_count;
     swq_join_def *join_defs;
 
     swq_expr_node *where_expr;
 
-    void        PushOrderBy( const char *pszFieldName, int bAscending );
+    void        PushOrderBy( const char* pszTableName, const char *pszFieldName, int bAscending );
     int         order_specs;
     swq_order_def *order_defs;
 
     swq_select *poOtherSelect;
     void        PushUnionAll( swq_select* poOtherSelectIn );
 
-    CPLErr      preparse( const char *select_statement );
-    void        postpreparse();
-    CPLErr      expand_wildcard( swq_field_list *field_list );
-    CPLErr      parse( swq_field_list *field_list, int parse_flags );
+    CPLErr      preparse( const char *select_statement,
+                          int bAcceptCustomFuncs = FALSE );
+    CPLErr      expand_wildcard( swq_field_list *field_list,
+                                 int bAlwaysPrefixWithTableName );
+    CPLErr      parse( swq_field_list *field_list,
+                       swq_select_parse_options* poParseOptions );
 
+    char       *Unparse();
     void        Dump( FILE * );
 };
 
@@ -327,10 +378,12 @@ CPLErr swq_select_parse( swq_select *select_info,
                          int parse_flags );
 
 const char *swq_select_finish_summarize( swq_select *select_info );
-const char *swq_select_summarize( swq_select *select_info, 
-                                  int dest_column, 
+const char *swq_select_summarize( swq_select *select_info,
+                                  int dest_column,
                                   const char *value );
 
 int swq_is_reserved_keyword(const char* pszStr);
 
-#endif /* def _SWQ_H_INCLUDED_ */
+char* OGRHStoreGetValue(const char* pszHStore, const char* pszSearchedKey);
+
+#endif /* def SWQ_H_INCLUDED_ */
