@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: cpl_getexecpath.cpp 27720 2014-09-21 17:58:47Z goatbar $
+ * $Id: cpl_getexecpath.cpp 33646 2016-03-05 15:54:03Z goatbar $
  *
  * Project:  CPL - Common Portability Library
  * Purpose:  Implement CPLGetExecPath().
@@ -14,33 +14,29 @@
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: cpl_getexecpath.cpp 27720 2014-09-21 17:58:47Z goatbar $");
+CPL_CVSID("$Id: cpl_getexecpath.cpp 33646 2016-03-05 15:54:03Z goatbar $");
 
-#if defined(WIN32) || defined(WIN32CE)
+#if defined(WIN32)
 
 #define HAVE_IMPLEMENTATION 1
 
-#if defined(WIN32CE)
-#  include "cpl_win32ce_api.h"
-#else
-#  include <windows.h>
-#endif
+#include <windows.h>
 
 /************************************************************************/
 /*                           CPLGetExecPath()                           */
@@ -48,7 +44,6 @@ CPL_CVSID("$Id: cpl_getexecpath.cpp 27720 2014-09-21 17:58:47Z goatbar $");
 
 int CPLGetExecPath( char *pszPathBuf, int nMaxLength )
 {
-#ifndef WIN32CE
     if( CSLTestBoolean(
             CPLGetConfigOption( "GDAL_FILENAME_IS_UTF8", "YES" ) ) )
     {
@@ -62,7 +57,7 @@ int CPLGetExecPath( char *pszPathBuf, int nMaxLength )
         }
         else
         {
-            char *pszDecoded = 
+            char *pszDecoded =
                 CPLRecodeFromWChar(pwszPathBuf,CPL_ENC_UCS2,CPL_ENC_UTF8);
 
             strncpy( pszPathBuf, pszDecoded, nMaxLength );
@@ -78,12 +73,6 @@ int CPLGetExecPath( char *pszPathBuf, int nMaxLength )
         else
             return TRUE;
     }
-#else
-    if( CE_GetModuleFileNameA( NULL, pszPathBuf, nMaxLength ) == 0 )
-        return FALSE;
-    else
-        return TRUE;
-#endif
 }
 
 #endif
@@ -102,10 +91,9 @@ int CPLGetExecPath( char *pszPathBuf, int nMaxLength )
 {
     long nPID = getpid();
     CPLString osExeLink;
-    ssize_t nResultLen;
 
     osExeLink.Printf( "/proc/%ld/exe", nPID );
-    nResultLen = readlink( osExeLink, pszPathBuf, nMaxLength );
+    ssize_t nResultLen = readlink( osExeLink, pszPathBuf, nMaxLength );
     if( nResultLen >= 0 )
         pszPathBuf[nResultLen] = '\0';
     else
@@ -121,10 +109,10 @@ int CPLGetExecPath( char *pszPathBuf, int nMaxLength )
 /************************************************************************/
 
 /**
- * Fetch path of executable. 
+ * Fetch path of executable.
  *
  * The path to the executable currently running is returned.  This path
- * includes the name of the executable.   Currently this only works on 
+ * includes the name of the executable.   Currently this only works on
  * win32 and linux platforms.  The returned path is UTF-8 encoded.
  *
  * @param pszPathBuf the buffer into which the path is placed.
@@ -141,4 +129,3 @@ int CPLGetExecPath( CPL_UNUSED char *pszPathBuf, CPL_UNUSED int nMaxLength )
 }
 
 #endif
-
