@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: cpl_hash_set.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: cpl_hash_set.cpp 33724 2016-03-16 17:14:11Z goatbar $
  *
  * Name:     cpl_hash_set.cpp
  * Project:  CPL - Common Portability Library
@@ -15,16 +15,16 @@
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
@@ -41,12 +41,15 @@ struct _CPLHashSet
     int                   nSize;
     int                   nIndiceAllocatedSize;
     int                   nAllocatedSize;
+    CPLList              *psRecyclingList;
+    int                   nRecyclingListSize;
+    int                   bRehash;
 #ifdef HASH_DEBUG
     int                   nCollisions;
 #endif
 };
 
-static const int anPrimes[] = 
+static const int anPrimes[] =
 { 53, 97, 193, 389, 769, 1543, 3079, 6151,
   12289, 24593, 49157, 98317, 196613, 393241,
   786433, 1572869, 3145739, 6291469, 12582917,
@@ -59,7 +62,7 @@ static const int anPrimes[] =
 
 /**
  * Creates a new hash set
- * 
+ *
  * The hash function must return a hash value for the elements to insert.
  * If fnHashFunc is NULL, CPLHashSetHashPointer will be used.
  *
@@ -89,6 +92,9 @@ CPLHashSet* CPLHashSetNew(CPLHashSetHashFunc fnHashFunc,
     set->tabList = (CPLList**) CPLCalloc(sizeof(CPLList*), 53);
     set->nIndiceAllocatedSize = 0;
     set->nAllocatedSize = 53;
+    set->psRecyclingList = NULL;
+    set->nRecyclingListSize = 0;
+    set->bRehash = FALSE;
 #ifdef HASH_DEBUG
     set->nCollisions = 0;
 #endif
@@ -102,7 +108,7 @@ CPLHashSet* CPLHashSetNew(CPLHashSetHashFunc fnHashFunc,
 
 /**
  * Returns the number of elements inserted in the hash set
- * 
+ *
  * Note: this is not the internal size of the hash set
  *
  * @param set the hash set
@@ -116,6 +122,68 @@ int CPLHashSetSize(const CPLHashSet* set)
     return set->nSize;
 }
 
+/************************************************************************/
+/*                       CPLHashSetGetNewListElt()                      */
+/************************************************************************/
+
+static CPLList* CPLHashSetGetNewListElt(CPLHashSet* set)
+{
+    if( set->psRecyclingList )
+    {
+        CPLList* psRet = set->psRecyclingList;
+        psRet->pData = NULL;
+        set->nRecyclingListSize --;
+        set->psRecyclingList = psRet->psNext;
+        return psRet;
+    }
+    else
+        return (CPLList*) CPLMalloc(sizeof(CPLList));
+}
+
+/************************************************************************/
+/*                       CPLHashSetReturnListElt()                      */
+/************************************************************************/
+
+static void CPLHashSetReturnListElt(CPLHashSet* set, CPLList* psList)
+{
+    if( set->nRecyclingListSize < 128 )
+    {
+        psList->psNext = set->psRecyclingList;
+        set->psRecyclingList = psList;
+        set->nRecyclingListSize ++;
+    }
+    else
+    {
+        CPLFree(psList);
+    }
+}
+
+/************************************************************************/
+/*                   CPLHashSetClearInternal()                          */
+/************************************************************************/
+
+static void CPLHashSetClearInternal(CPLHashSet* set, int bFinalize)
+{
+    CPLAssert(set != NULL);
+    for(int i=0;i<set->nAllocatedSize;i++)
+    {
+        CPLList* cur = set->tabList[i];
+        while(cur)
+        {
+            if (set->fnFreeEltFunc)
+                set->fnFreeEltFunc(cur->pData);
+            CPLList* psNext = cur->psNext;
+            if( bFinalize )
+                CPLFree(cur);
+            else
+                CPLHashSetReturnListElt(set, cur);
+            cur = psNext;
+        }
+        set->tabList[i] = NULL;
+    }
+    set->bRehash = FALSE;
+}
+
 /************************************************************************/
 /*                        CPLHashSetDestroy()                           */
 /************************************************************************/
@@ -125,30 +193,44 @@ int CPLHashSetSize(const CPLHashSet* set)
  *
  * This function also frees the elements if a free function was
  * provided at the creation of the hash set.
- * 
+ *
  * @param set the hash set
  */
 
 void CPLHashSetDestroy(CPLHashSet* set)
 {
-    CPLAssert(set != NULL);
-    for(int i=0;i<set->nAllocatedSize;i++)
-    {
-        if (set->fnFreeEltFunc)
-        {
-            CPLList* cur = set->tabList[i];
-            while(cur)
-            {
-                set->fnFreeEltFunc(cur->pData);
-                cur = cur->psNext;
-            }
-        }
-        CPLListDestroy(set->tabList[i]);
-    }
+    CPLHashSetClearInternal(set, TRUE);
     CPLFree(set->tabList);
+    CPLListDestroy(set->psRecyclingList);
     CPLFree(set);
 }
 
+/************************************************************************/
+/*                        CPLHashSetClear()                             */
+/************************************************************************/
+
+/**
+ * Clear all elements from a hash set.
+ *
+ * This function also frees the elements if a free function was
+ * provided at the creation of the hash set.
+ *
+ * @param set the hash set
+ * @since GDAL 2.1
+ */
+
+void CPLHashSetClear(CPLHashSet* set)
+{
+    CPLHashSetClearInternal(set, FALSE);
+    set->tabList = (CPLList**) CPLRealloc(set->tabList, sizeof(CPLList*) * 53);
+    set->nIndiceAllocatedSize = 0;
+    set->nAllocatedSize = 53;
+#ifdef HASH_DEBUG
+    set->nCollisions = 0;
+#endif
+    set->nSize = 0;
+}
+
 /************************************************************************/
 /*                       CPLHashSetForeach()                            */
 /************************************************************************/
@@ -164,7 +246,7 @@ void CPLHashSetDestroy(CPLHashSet* set)
  *
  * Note : the structure of the hash set must *NOT* be modified during the
  * walk.
- * 
+ *
  * @param set the hash set.
  * @param fnIterFunc the function called on each element.
  * @param user_data the user data provided to the function.
@@ -213,14 +295,16 @@ static void CPLHashSetRehash(CPLHashSet* set)
             if (newTabList[nNewHashVal])
                 set->nCollisions ++;
 #endif
-            newTabList[nNewHashVal] = CPLListInsert(newTabList[nNewHashVal], cur->pData, 0);
-            cur = cur->psNext;
+            CPLList* psNext = cur->psNext;
+            cur->psNext = newTabList[nNewHashVal];
+            newTabList[nNewHashVal] = cur;
+            cur = psNext;
         }
-        CPLListDestroy(set->tabList[i]);
     }
     CPLFree(set->tabList);
     set->tabList = newTabList;
     set->nAllocatedSize = nNewAllocatedSize;
+    set->bRehash = FALSE;
 }
 
 
@@ -251,7 +335,7 @@ static void** CPLHashSetFindPtr(CPLHashSet* set, const void* elt)
  * If the element was already inserted in the hash set, the previous
  * element is replaced by the new element. If a free function was provided,
  * it is used to free the previously inserted element
- * 
+ *
  * @param set the hash set
  * @param elt the new element to insert in the hash set
  *
@@ -271,7 +355,8 @@ int CPLHashSetInsert(CPLHashSet* set, void* elt)
         return FALSE;
     }
 
-    if (set->nSize >= 2 * set->nAllocatedSize / 3)
+    if( set->nSize >= 2 * set->nAllocatedSize / 3 ||
+        (set->bRehash && set->nIndiceAllocatedSize > 0 && set->nSize <= set->nAllocatedSize / 2) )
     {
         set->nIndiceAllocatedSize++;
         CPLHashSetRehash(set);
@@ -282,7 +367,11 @@ int CPLHashSetInsert(CPLHashSet* set, void* elt)
     if (set->tabList[nHashVal])
         set->nCollisions ++;
 #endif
-    set->tabList[nHashVal] = CPLListInsert(set->tabList[nHashVal], (void*) elt, 0);
+
+    CPLList* new_elt = CPLHashSetGetNewListElt(set);
+    new_elt->pData = (void*) elt;
+    new_elt->psNext = set->tabList[nHashVal];
+    set->tabList[nHashVal] = new_elt;
     set->nSize++;
 
     return TRUE;
@@ -295,7 +384,7 @@ int CPLHashSetInsert(CPLHashSet* set, void* elt)
 /**
  * Returns the element found in the hash set corresponding to the element to look up
  * The element must not be modified.
- * 
+ *
  * @param set the hash set
  * @param elt the element to look up in the hash set
  *
@@ -313,28 +402,23 @@ void* CPLHashSetLookup(CPLHashSet* set, const void* elt)
 }
 
 /************************************************************************/
-/*                         CPLHashSetRemove()                           */
+/*                     CPLHashSetRemoveInternal()                       */
 /************************************************************************/
 
-/**
- * Removes an element from a hash set
- * 
- * @param set the hash set
- * @param elt the new element to remove from the hash set
- *
- * @return TRUE if the element was in the hash set
- */
-
-int CPLHashSetRemove(CPLHashSet* set, const void* elt)
+static
+int CPLHashSetRemoveInternal(CPLHashSet* set, const void* elt, int bDeferRehash)
 {
     CPLAssert(set != NULL);
     if (set->nIndiceAllocatedSize > 0 && set->nSize <= set->nAllocatedSize / 2)
     {
         set->nIndiceAllocatedSize--;
-        CPLHashSetRehash(set);
+        if( bDeferRehash )
+            set->bRehash = TRUE;
+        else
+            CPLHashSetRehash(set);
     }
 
-    int nHashVal = set->fnHashFunc(elt) % set->nAllocatedSize;
+    int nHashVal = static_cast<int>(set->fnHashFunc(elt) % set->nAllocatedSize);
     CPLList* cur = set->tabList[nHashVal];
     CPLList* prev = NULL;
     while(cur)
@@ -349,7 +433,7 @@ int CPLHashSetRemove(CPLHashSet* set, const void* elt)
             if (set->fnFreeEltFunc)
                 set->fnFreeEltFunc(cur->pData);
 
-            CPLFree(cur);
+            CPLHashSetReturnListElt(set, cur);
 #ifdef HASH_DEBUG
             if (set->tabList[nHashVal])
                 set->nCollisions --;
@@ -364,6 +448,45 @@ int CPLHashSetRemove(CPLHashSet* set, const void* elt)
     return FALSE;
 }
 
+/************************************************************************/
+/*                         CPLHashSetRemove()                           */
+/************************************************************************/
+
+/**
+ * Removes an element from a hash set
+ *
+ * @param set the hash set
+ * @param elt the new element to remove from the hash set
+ *
+ * @return TRUE if the element was in the hash set
+ */
+
+int CPLHashSetRemove(CPLHashSet* set, const void* elt)
+{
+    return CPLHashSetRemoveInternal(set, elt, FALSE);
+}
+
+/************************************************************************/
+/*                     CPLHashSetRemoveDeferRehash()                    */
+/************************************************************************/
+
+/**
+ * Removes an element from a hash set.
+ *
+ * This will defer potential rehashing of the set to later calls to
+ * CPLHashSetInsert() or CPLHashSetRemove().
+ *
+ * @param set the hash set
+ * @param elt the new element to remove from the hash set
+ *
+ * @return TRUE if the element was in the hash set
+ * @since GDAL 2.1
+ */
+
+int CPLHashSetRemoveDeferRehash(CPLHashSet* set, const void* elt)
+{
+    return CPLHashSetRemoveInternal(set, elt, TRUE);
+}
 
 /************************************************************************/
 /*                    CPLHashSetHashPointer()                           */
@@ -371,7 +494,7 @@ int CPLHashSetRemove(CPLHashSet* set, const void* elt)
 
 /**
  * Hash function for an arbitrary pointer
- * 
+ *
  * @param elt the arbitrary pointer to hash
  *
  * @return the hash value of the pointer
@@ -388,7 +511,7 @@ unsigned long CPLHashSetHashPointer(const void* elt)
 
 /**
  * Equality function for arbitrary pointers
- * 
+ *
  * @param elt1 the first arbitrary pointer to compare
  * @param elt2 the second arbitrary pointer to compare
  *
@@ -406,7 +529,7 @@ int CPLHashSetEqualPointer(const void* elt1, const void* elt2)
 
 /**
  * Hash function for a zero-terminated string
- * 
+ *
  * @param elt the string to hash. May be NULL.
  *
  * @return the hash value of the string
@@ -416,11 +539,11 @@ unsigned long CPLHashSetHashStr(const void *elt)
 {
     unsigned char* pszStr = (unsigned char*)elt;
     unsigned long hash = 0;
-    int c;
 
     if (pszStr == NULL)
         return 0;
 
+    int c = 0;
     while ((c = *pszStr++) != '\0')
         hash = c + (hash << 6) + (hash << 16) - hash;
 
@@ -433,7 +556,7 @@ unsigned long CPLHashSetHashStr(const void *elt)
 
 /**
  * Equality function for strings
- * 
+ *
  * @param elt1 the first string to compare. May be NULL.
  * @param elt2 the second string to compare. May be NULL.
  *
