@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cpl_http.cpp 29152 2015-05-04 11:45:44Z rouault $
+ * $Id: cpl_http.cpp 33646 2016-03-05 15:54:03Z goatbar $
  *
  * Project:  libcurl based HTTP client
  * Purpose:  libcurl based HTTP client
@@ -44,9 +44,9 @@ void CPLHTTPSetOptions(CURL *http_handle, char** papszOptions);
 
 #endif
 
-CPL_CVSID("$Id: cpl_http.cpp 29152 2015-05-04 11:45:44Z rouault $");
+CPL_CVSID("$Id: cpl_http.cpp 33646 2016-03-05 15:54:03Z goatbar $");
 
-// list of named persistent http sessions 
+// list of named persistent http sessions
 
 #ifdef HAVE_CURL
 static std::map<CPLString,CURL*>* poSessionMap = NULL;
@@ -61,14 +61,14 @@ static CPLMutex *hSessionMapMutex = NULL;
 /************************************************************************/
 
 #ifdef HAVE_CURL
-static size_t 
+static size_t
 CPLWriteFct(void *buffer, size_t size, size_t nmemb, void *reqInfo)
 
 {
     CPLHTTPResult *psResult = (CPLHTTPResult *) reqInfo;
-    int  nNewSize;
 
-    nNewSize = psResult->nDataLen + nmemb*size + 1;
+    int nBytesToWrite = static_cast<int>(nmemb)*static_cast<int>(size);
+    int nNewSize = psResult->nDataLen + nBytesToWrite + 1;
     if( nNewSize > psResult->nDataAlloc )
     {
         psResult->nDataAlloc = (int) (nNewSize * 1.25 + 100);
@@ -86,10 +86,9 @@ CPLWriteFct(void *buffer, size_t size, size_t nmemb, void *reqInfo)
         psResult->pabyData = pabyNewData;
     }
 
-    memcpy( psResult->pabyData + psResult->nDataLen, buffer,
-            nmemb * size );
+    memcpy( psResult->pabyData + psResult->nDataLen, buffer, nBytesToWrite );
 
-    psResult->nDataLen += nmemb * size;
+    psResult->nDataLen += nBytesToWrite;
     psResult->pabyData[psResult->nDataLen] = 0;
 
     return nmemb;
@@ -103,13 +102,13 @@ static size_t CPLHdrWriteFct(void *buffer, size_t size, size_t nmemb, void *reqI
     CPLHTTPResult *psResult = (CPLHTTPResult *) reqInfo;
     // copy the buffer to a char* and initialize with zeros (zero terminate as well)
     char* pszHdr = (char*)CPLCalloc(nmemb + 1, size);
-    CPLPrintString(pszHdr, (char *)buffer, nmemb * size);
+    CPLPrintString(pszHdr, (char *)buffer, static_cast<int>(nmemb) * static_cast<int>(size));
     char *pszKey = NULL;
     const char *pszValue = CPLParseNameValue(pszHdr, &pszKey );
     psResult->papszHeaders = CSLSetNameValue(psResult->papszHeaders, pszKey, pszValue);
     CPLFree(pszHdr);
     CPLFree(pszKey);
-    return nmemb; 
+    return nmemb;
 }
 
 #endif /* def HAVE_CURL */
@@ -125,7 +124,13 @@ static size_t CPLHdrWriteFct(void *buffer, size_t size, size_t nmemb, void *reqI
  * @param papszOptions option list as a NULL-terminated array of strings. May be NULL.
  *                     The following options are handled :
  * <ul>
- * <li>TIMEOUT=val, where val is in seconds</li>
+ * <li>TIMEOUT=val, where val is in seconds. This is the maximum delay for the whole
+ *     request to complete before being aborted.</li>
+ * <li>LOW_SPEED_TIME=val, where val is in seconds. This is the maximum time where the
+ *      transfer speed should be below the LOW_SPEED_LIMIT (if not specified 1b/s),
+ *      before the transfer to be considered too slow and aborted. (GDAL >= 2.1)</li>
+ * <li>LOW_SPEED_LIMIT=val, where val is in bytes/second. See LOW_SPEED_TIME. Has only
+ *     effect if LOW_SPEED_TIME is specified too. (GDAL >= 2.1)</li>
  * <li>HEADERS=val, where val is an extra header to use when getting a web page.
  *                  For example "Accept: application/x-ogcwkt"
  * <li>HTTPAUTH=[BASIC/NTLM/GSSNEGOTIATE/ANY] to specify an authentication scheme to use.
@@ -145,22 +150,30 @@ static size_t CPLHdrWriteFct(void *buffer, size_t size, size_t nmemb, void *reqI
  *                 Default is 30. (GDAL >= 2.0)
  * </ul>
  *
- * Alternatively, if not defined in the papszOptions arguments, the PROXY,  
- * PROXYUSERPWD, PROXYAUTH, NETRC, MAX_RETRY and RETRY_DELAY values are searched in the configuration 
- * options named GDAL_HTTP_PROXY, GDAL_HTTP_PROXYUSERPWD, GDAL_PROXY_AUTH, 
+ * Alternatively, if not defined in the papszOptions arguments, the TIMEOUT,
+ * LOW_SPEED_TIME, LOW_SPEED_LIMIT, PROXY, PROXYUSERPWD, PROXYAUTH, NETRC,
+ * MAX_RETRY and RETRY_DELAY values are searched in the configuration
+ * options named GDAL_HTTP_TIMEOUT, GDAL_HTTP_LOW_SPEED_TIME, GDAL_HTTP_LOW_SPEED_LIMIT,
+ * GDAL_HTTP_PROXY, GDAL_HTTP_PROXYUSERPWD, GDAL_PROXY_AUTH,
  * GDAL_HTTP_NETRC, GDAL_HTTP_MAX_RETRY and GDAL_HTTP_RETRY_DELAY.
  *
- * @return a CPLHTTPResult* structure that must be freed by 
+ * @return a CPLHTTPResult* structure that must be freed by
  * CPLHTTPDestroyResult(), or NULL if libcurl support is disabled
  */
 CPLHTTPResult *CPLHTTPFetch( const char *pszURL, char **papszOptions )
 
 {
-    if( strncmp(pszURL, "/vsimem/", strlen("/vsimem/")) == 0 &&
+    if( STARTS_WITH(pszURL, "/vsimem/") &&
         /* Disabled by default for potential security issues */
         CSLTestBoolean(CPLGetConfigOption("CPL_CURL_ENABLE_VSIMEM", "FALSE")) )
     {
         CPLString osURL(pszURL);
+        const char* pszCustomRequest = CSLFetchNameValue( papszOptions, "CUSTOMREQUEST" );
+        if( pszCustomRequest != NULL )
+        {
+            osURL += "&CUSTOMREQUEST=";
+            osURL += pszCustomRequest;
+        }
         const char* pszPost = CSLFetchNameValue( papszOptions, "POSTFIELDS" );
         if( pszPost != NULL ) /* Hack: we append post content to filename */
         {
@@ -179,26 +192,24 @@ CPLHTTPResult *CPLHTTPFetch( const char *pszURL, char **papszOptions )
         }
         else if( nLength != 0 )
         {
-            psResult->nDataLen = (size_t)nLength;
+            psResult->nDataLen = static_cast<int>(nLength);
             psResult->pabyData = (GByte*) CPLMalloc((size_t)nLength + 1);
             memcpy(psResult->pabyData, pabyData, (size_t)nLength);
             psResult->pabyData[(size_t)nLength] = 0;
         }
 
         if( psResult->pabyData != NULL &&
-            strncmp((const char*)psResult->pabyData, "Content-Type: ",
-                    strlen("Content-Type: ")) == 0 )
-        {
+            STARTS_WITH((const char*)psResult->pabyData, "Content-Type: ") )        {
             const char* pszContentType = (const char*)psResult->pabyData + strlen("Content-type: ");
             const char* pszEOL = strchr(pszContentType, '\r');
             if( pszEOL )
                 pszEOL = strchr(pszContentType, '\n');
             if( pszEOL )
             {
-                int nLength = pszEOL - pszContentType;
-                psResult->pszContentType = (char*)CPLMalloc(nLength + 1);
-                memcpy(psResult->pszContentType, pszContentType, nLength);
-                psResult->pszContentType[nLength] = 0;
+                size_t nContentLength = pszEOL - pszContentType;
+                psResult->pszContentType = (char*)CPLMalloc(nContentLength + 1);
+                memcpy(psResult->pszContentType, pszContentType, nContentLength);
+                psResult->pszContentType[nContentLength] = 0;
             }
         }
 
@@ -284,7 +295,7 @@ CPLHTTPResult *CPLHTTPFetch( const char *pszURL, char **papszOptions )
 /* -------------------------------------------------------------------- */
     char szCurlErrBuf[CURL_ERROR_SIZE+1];
     CPLHTTPResult *psResult;
-    struct curl_slist *headers=NULL; 
+    struct curl_slist *headers=NULL;
 
     const char* pszArobase = strchr(pszURL, '@');
     const char* pszSlash = strchr(pszURL, '/');
@@ -323,17 +334,17 @@ CPLHTTPResult *CPLHTTPFetch( const char *pszURL, char **papszOptions )
     const char* pszNoBody = NULL;
     if ((pszNoBody = CSLFetchNameValue( papszOptions, "NO_BODY" )) != NULL)
     {
-        if (CSLTestBoolean(pszNoBody)) 
+        if (CSLTestBoolean(pszNoBody))
         {
             CPLDebug ("HTTP", "HEAD Request: %s", pszURL);
-            curl_easy_setopt(http_handle, CURLOPT_NOBODY, 1L);           
+            curl_easy_setopt(http_handle, CURLOPT_NOBODY, 1L);
         }
     }
 
     // capture response headers
     curl_easy_setopt(http_handle, CURLOPT_HEADERDATA, psResult);
     curl_easy_setopt(http_handle, CURLOPT_HEADERFUNCTION, CPLHdrWriteFct);
- 
+
     curl_easy_setopt(http_handle, CURLOPT_WRITEDATA, psResult );
     curl_easy_setopt(http_handle, CURLOPT_WRITEFUNCTION, CPLWriteFct );
 
@@ -341,23 +352,23 @@ CPLHTTPResult *CPLHTTPFetch( const char *pszURL, char **papszOptions )
 
     curl_easy_setopt(http_handle, CURLOPT_ERRORBUFFER, szCurlErrBuf );
 
-    static int bHasCheckVersion = FALSE;
-    static int bSupportGZip = FALSE;
+    static bool bHasCheckVersion = false;
+    static bool bSupportGZip = false;
     if (!bHasCheckVersion)
     {
         bSupportGZip = strstr(curl_version(), "zlib/") != NULL;
-        bHasCheckVersion = TRUE;
+        bHasCheckVersion = true;
     }
-    int bGZipRequested = FALSE;
+    int bGZipRequested = false;
     if (bSupportGZip && CSLTestBoolean(CPLGetConfigOption("CPL_CURL_GZIP", "YES")))
     {
-        bGZipRequested = TRUE;
+        bGZipRequested = true;
         curl_easy_setopt(http_handle, CURLOPT_ENCODING, "gzip");
     }
 
 /* -------------------------------------------------------------------- */
-/*      If 502, 503 or 504 status code retry this HTTP call until max        */
-/*      retry has been rearched                                         */
+/*      If 502, 503 or 504 status code retry this HTTP call until max   */
+/*      retry has been reached                                          */
 /* -------------------------------------------------------------------- */
     const char *pszRetryDelay = CSLFetchNameValue( papszOptions, "RETRY_DELAY" );
     if( pszRetryDelay == NULL )
@@ -372,7 +383,7 @@ CPLHTTPResult *CPLHTTPFetch( const char *pszURL, char **papszOptions )
 
     do
     {
-        bRequestRetry = FALSE;
+        bRequestRetry = false;
 
 /* -------------------------------------------------------------------- */
 /*      Execute the request, waiting for results.                       */
@@ -393,7 +404,7 @@ CPLHTTPResult *CPLHTTPFetch( const char *pszURL, char **papszOptions )
 /* -------------------------------------------------------------------- */
         if( strlen(szCurlErrBuf) > 0 )
         {
-            int bSkipError = FALSE;
+            bool bSkipError = false;
 
             /* Some servers such as http://115.113.193.14/cgi-bin/world/qgis_mapserv.fcgi?VERSION=1.1.1&SERVICE=WMS&REQUEST=GetCapabilities */
             /* invalidly return Content-Length as the uncompressed size, with makes curl to wait for more data */
@@ -416,7 +427,7 @@ CPLHTTPResult *CPLHTTPFetch( const char *pszURL, char **papszOptions )
                                  pszURL);
                     }
                     psResult->nStatus = 0;
-                    bSkipError = TRUE;
+                    bSkipError = true;
                 }
             }
             if (!bSkipError)
@@ -453,7 +464,7 @@ CPLHTTPResult *CPLHTTPFetch( const char *pszURL, char **papszOptions )
                     psResult->nDataLen = 0;
                     psResult->nDataAlloc = 0;
 
-                    bRequestRetry = TRUE;
+                    bRequestRetry = true;
                 }
                 else
                 {
@@ -510,7 +521,7 @@ void CPLHTTPSetOptions(CURL *http_handle, char** papszOptions)
     else
     {
         CPLError( CE_Warning, CPLE_AppDefined,
-                  "Unsupported HTTPAUTH value '%s', ignored.", 
+                  "Unsupported HTTPAUTH value '%s', ignored.",
                   pszHttpAuth );
     }
 #else
@@ -567,7 +578,7 @@ void CPLHTTPSetOptions(CURL *http_handle, char** papszOptions)
     else
     {
         CPLError( CE_Warning, CPLE_AppDefined,
-                  "Unsupported PROXYAUTH value '%s', ignored.", 
+                  "Unsupported PROXYAUTH value '%s', ignored.",
                   pszProxyAuth );
     }
 #else
@@ -581,7 +592,7 @@ void CPLHTTPSetOptions(CURL *http_handle, char** papszOptions)
     /* Enable following redirections.  Requires libcurl 7.10.1 at least */
     curl_easy_setopt(http_handle, CURLOPT_FOLLOWLOCATION, 1 );
     curl_easy_setopt(http_handle, CURLOPT_MAXREDIRS, 10 );
-    
+
     /* Set timeout.*/
     const char *pszTimeout = CSLFetchNameValue( papszOptions, "TIMEOUT" );
     if (pszTimeout == NULL)
@@ -589,6 +600,20 @@ void CPLHTTPSetOptions(CURL *http_handle, char** papszOptions)
     if( pszTimeout != NULL )
         curl_easy_setopt(http_handle, CURLOPT_TIMEOUT, atoi(pszTimeout) );
 
+    /* Set low speed time and limit.*/
+    const char *pszLowSpeedTime = CSLFetchNameValue( papszOptions, "LOW_SPEED_TIME" );
+    if (pszLowSpeedTime == NULL)
+        pszLowSpeedTime = CPLGetConfigOption("GDAL_HTTP_LOW_SPEED_TIME", NULL);
+    if( pszLowSpeedTime != NULL )
+    {
+        curl_easy_setopt(http_handle, CURLOPT_LOW_SPEED_TIME, atoi(pszLowSpeedTime) );
+
+        const char *pszLowSpeedLimit = CSLFetchNameValue( papszOptions, "LOW_SPEED_LIMIT" );
+        if (pszLowSpeedLimit == NULL)
+            pszLowSpeedLimit = CPLGetConfigOption("GDAL_HTTP_LOW_SPEED_LIMIT", "1");
+        curl_easy_setopt(http_handle, CURLOPT_LOW_SPEED_LIMIT, atoi(pszLowSpeedLimit) );
+    }
+
     /* Disable some SSL verification */
     const char *pszUnsafeSSL = CSLFetchNameValue( papszOptions, "UNSAFESSL" );
     if (pszUnsafeSSL == NULL)
@@ -633,7 +658,7 @@ void CPLHTTPSetOptions(CURL *http_handle, char** papszOptions)
     {
         curl_easy_setopt(http_handle, CURLOPT_CUSTOMREQUEST, pszCustomRequest );
     }
-    
+
     const char* pszCookie = CSLFetchNameValue(papszOptions, "COOKIE");
     if (pszCookie == NULL)
         pszCookie = CPLGetConfigOption("GDAL_HTTP_COOKIE", NULL);
@@ -689,7 +714,7 @@ void CPLHTTPCleanup()
         }
     }
 
-    // not quite a safe sequence. 
+    // not quite a safe sequence.
     CPLDestroyMutex( hSessionMapMutex );
     hSessionMapMutex = NULL;
 #endif
@@ -714,13 +739,12 @@ void CPLHTTPDestroyResult( CPLHTTPResult *psResult )
         CPLFree( psResult->pszContentType );
         CSLDestroy( psResult->papszHeaders );
 
-        int i;
-        for(i=0;i<psResult->nMimePartCount;i++)
+        for(int i=0;i<psResult->nMimePartCount;i++)
         {
             CSLDestroy( psResult->pasMimePart[i].papszHeaders );
         }
         CPLFree(psResult->pasMimePart);
-        
+
         CPLFree( psResult );
     }
 }
@@ -730,7 +754,7 @@ void CPLHTTPDestroyResult( CPLHTTPResult *psResult )
 /************************************************************************/
 
 /**
- * \brief Parses a a MIME multipart message
+ * \brief Parses a MIME multipart message.
  *
  * This function will iterate over each part and put it in a separate
  * element of the pasMimePart array of the provided psResult structure.
@@ -763,8 +787,8 @@ int CPLHTTPParseMultipartMime( CPLHTTPResult *psResult )
     }
 
     CPLString osBoundary;
-    char **papszTokens = 
-        CSLTokenizeStringComplex( pszBound + 9, "\n ;", 
+    char **papszTokens =
+        CSLTokenizeStringComplex( pszBound + 9, "\n ;",
                                   TRUE, FALSE );
 
     if( CSLCount(papszTokens) == 0 || strlen(papszTokens[0]) == 0 )
@@ -774,7 +798,7 @@ int CPLHTTPParseMultipartMime( CPLHTTPResult *psResult )
         CSLDestroy( papszTokens );
         return FALSE;
     }
-    
+
     osBoundary = "--";
     osBoundary += papszTokens[0];
     CSLDestroy( papszTokens );
@@ -782,10 +806,9 @@ int CPLHTTPParseMultipartMime( CPLHTTPResult *psResult )
 /* -------------------------------------------------------------------- */
 /*      Find the start of the first chunk.                              */
 /* -------------------------------------------------------------------- */
-    char *pszNext;
-    pszNext = (char *) 
+    char *pszNext = (char *)
         strstr((const char *) psResult->pabyData,osBoundary.c_str());
-    
+
     if( pszNext == NULL )
     {
         CPLError( CE_Failure, CPLE_AppDefined, "No parts found." );
@@ -803,7 +826,7 @@ int CPLHTTPParseMultipartMime( CPLHTTPResult *psResult )
 /* -------------------------------------------------------------------- */
 /*      Loop over parts...                                              */
 /* -------------------------------------------------------------------- */
-    while( TRUE )
+    while( true )
     {
         psResult->nMimePartCount++;
         psResult->pasMimePart = (CPLMimePart *)
@@ -829,18 +852,18 @@ int CPLHTTPParseMultipartMime( CPLHTTPResult *psResult )
             }
 
             *pszEOL = '\0';
-            int bRestoreAntislashR = FALSE;
+            bool bRestoreAntislashR = false;
             if (pszEOL - pszNext > 1 && pszEOL[-1] == '\r')
             {
-                bRestoreAntislashR = TRUE;
+                bRestoreAntislashR = true;
                 pszEOL[-1] = '\0';
             }
-            psPart->papszHeaders = 
+            psPart->papszHeaders =
                 CSLAddString( psPart->papszHeaders, pszNext );
             if (bRestoreAntislashR)
                 pszEOL[-1] = '\r';
             *pszEOL = '\n';
-            
+
             pszNext = pszEOL + 1;
         }
 
@@ -848,23 +871,23 @@ int CPLHTTPParseMultipartMime( CPLHTTPResult *psResult )
             pszNext++;
         if( *pszNext == '\n' )
             pszNext++;
-            
+
 /* -------------------------------------------------------------------- */
 /*      Work out the data block size.                                   */
 /* -------------------------------------------------------------------- */
         psPart->pabyData = (GByte *) pszNext;
 
-        int nBytesAvail = psResult->nDataLen - 
-            (pszNext - (const char *) psResult->pabyData);
+        int nBytesAvail = psResult->nDataLen -
+            static_cast<int>(pszNext - (const char *) psResult->pabyData);
 
         while( nBytesAvail > 0
-               && (*pszNext != '-' 
+               && (*pszNext != '-'
                    || strncmp(pszNext,osBoundary,strlen(osBoundary)) != 0) )
         {
             pszNext++;
             nBytesAvail--;
         }
-        
+
         if( nBytesAvail == 0 )
         {
             CPLError(CE_Failure, CPLE_AppDefined,
@@ -872,10 +895,10 @@ int CPLHTTPParseMultipartMime( CPLHTTPResult *psResult )
             return FALSE;
         }
 
-        psPart->nDataLen = pszNext - (const char *) psPart->pabyData;
+        psPart->nDataLen = static_cast<int>(pszNext - (const char *) psPart->pabyData);
         pszNext += strlen(osBoundary);
 
-        if( strncmp(pszNext,"--",2) == 0 )
+        if( STARTS_WITH(pszNext, "--") )
         {
             break;
         }
