@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cpl_quad_tree.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: cpl_quad_tree.cpp 33199 2016-01-29 15:48:41Z rouault $
  *
  * Project:  CPL - Common Portability Library
  * Purpose:  Implementation of quadtree building and searching functions.
@@ -29,16 +29,15 @@
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  ******************************************************************************
- *
  */
 
 #include "cpl_conv.h"
 #include "cpl_quad_tree.h"
 
-CPL_CVSID("$Id: cpl_quad_tree.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: cpl_quad_tree.cpp 33199 2016-01-29 15:48:41Z rouault $");
 
-#define MAX_DEFAULT_TREE_DEPTH 12
-#define MAX_SUBNODES 4
+static const int MAX_DEFAULT_TREE_DEPTH = 12;
+static const int MAX_SUBNODES = 4;
 
 typedef struct _QuadTreeNode QuadTreeNode;
 
@@ -79,29 +78,31 @@ static void CPLQuadTreeAddFeatureInternal(CPLQuadTree *hQuadTree,
 /*      prevent small objects on a boundary from shifting too high      */
 /*      up the hQuadTree.                                                    */
 /* -------------------------------------------------------------------- */
-#define DEFAULT_SPLIT_RATIO  0.55
+static const double DEFAULT_SPLIT_RATIO = 0.55;
 
 /*
 ** Returns TRUE if rectangle a is contained in rectangle b
 */
-static CPL_INLINE int CPL_RectContained(const CPLRectObj *a, const CPLRectObj *b)
+static CPL_INLINE bool CPL_RectContained( const CPLRectObj *a,
+                                          const CPLRectObj *b )
 {
   if(a->minx >= b->minx && a->maxx <= b->maxx)
     if(a->miny >= b->miny && a->maxy <= b->maxy)
-      return(TRUE);
-  return(FALSE);  
+      return(true);
+  return(false);
 }
 
 /*
 ** Returns TRUE if rectangles a and b overlap
 */
-static CPL_INLINE int CPL_RectOverlap(const CPLRectObj *a, const CPLRectObj *b)
+static CPL_INLINE bool CPL_RectOverlap( const CPLRectObj *a,
+                                        const CPLRectObj *b )
 {
-  if(a->minx > b->maxx) return(FALSE);
-  if(a->maxx < b->minx) return(FALSE);
-  if(a->miny > b->maxy) return(FALSE);
-  if(a->maxy < b->miny) return(FALSE);
-  return(TRUE);
+  if(a->minx > b->maxx) return(false);
+  if(a->maxx < b->minx) return(false);
+  if(a->miny > b->maxy) return(false);
+  if(a->maxy < b->miny) return(false);
+  return(true);
 }
 
 /************************************************************************/
@@ -110,9 +111,8 @@ static CPL_INLINE int CPL_RectOverlap(const CPLRectObj *a, const CPLRectObj *b)
 
 static QuadTreeNode *CPLQuadTreeNodeCreate(const CPLRectObj* pRect)
 {
-    QuadTreeNode	*psNode;
-
-    psNode = (QuadTreeNode *) CPLMalloc(sizeof(QuadTreeNode));
+    QuadTreeNode *psNode
+        = static_cast<QuadTreeNode *>( CPLMalloc( sizeof(QuadTreeNode) ) );
 
     psNode->nFeatures = 0;
     psNode->pahFeatures = NULL;
@@ -131,7 +131,7 @@ static QuadTreeNode *CPLQuadTreeNodeCreate(const CPLRectObj* pRect)
 
 /**
  * Create a new quadtree
- * 
+ *
  * @param pGlobalBounds a pointer to the global extent of all
  *                      the elements that will be inserted
  * @param pfnGetBounds  a user provided function to get the bounding box of
@@ -143,16 +143,16 @@ static QuadTreeNode *CPLQuadTreeNodeCreate(const CPLRectObj* pRect)
  * @return a newly allocated quadtree
  */
 
-CPLQuadTree *CPLQuadTreeCreate(const CPLRectObj* pGlobalBounds, CPLQuadTreeGetBoundsFunc pfnGetBounds)
+CPLQuadTree *CPLQuadTreeCreate( const CPLRectObj* pGlobalBounds,
+                                CPLQuadTreeGetBoundsFunc pfnGetBounds )
 {
-    CPLQuadTree *hQuadTree;
-
     CPLAssert(pGlobalBounds);
 
     /* -------------------------------------------------------------------- */
     /*      Allocate the hQuadTree object                                   */
     /* -------------------------------------------------------------------- */
-    hQuadTree = (CPLQuadTree *) CPLMalloc(sizeof(CPLQuadTree));
+    CPLQuadTree *hQuadTree
+        = static_cast<CPLQuadTree *>( CPLMalloc(sizeof(CPLQuadTree)) );
 
     hQuadTree->nFeatures = 0;
     hQuadTree->pfnGetBounds = pfnGetBounds;
@@ -175,8 +175,9 @@ CPLQuadTree *CPLQuadTreeCreate(const CPLRectObj* pGlobalBounds, CPLQuadTreeGetBo
 
 /**
  * Returns the optimal depth of a quadtree to hold nExpectedFeatures
- * 
- * @param nExpectedFeatures the expected maximum number of elements to be inserted
+ *
+ * @param nExpectedFeatures the expected maximum number of elements to be
+ * inserted.
  *
  * @return the optimal depth of a quadtree to hold nExpectedFeatures
  */
@@ -197,8 +198,8 @@ int CPLQuadTreeGetAdvisedMaxDepth(int nExpectedFeatures)
     }
 
     CPLDebug( "CPLQuadTree",
-                "Estimated spatial index tree depth: %d",
-                nMaxDepth );
+              "Estimated spatial index tree depth: %d",
+              nMaxDepth );
 
     /* NOTE: Due to problems with memory allocation for deep trees,
         * automatically estimated depth is limited up to 12 levels.
@@ -209,8 +210,9 @@ int CPLQuadTreeGetAdvisedMaxDepth(int nExpectedFeatures)
         nMaxDepth = MAX_DEFAULT_TREE_DEPTH;
 
         CPLDebug( "CPLQuadTree",
-                    "Falling back to max number of allowed index tree levels (%d).",
-                    MAX_DEFAULT_TREE_DEPTH );
+                  "Falling back to max number of allowed index tree "
+                  "levels (%d).",
+                  MAX_DEFAULT_TREE_DEPTH );
 
     }
 
@@ -224,7 +226,7 @@ int CPLQuadTreeGetAdvisedMaxDepth(int nExpectedFeatures)
 /**
  * Set the maximum depth of a quadtree. By default, quad trees have
  * no maximum depth, but a maximum bucket capacity.
- * 
+ *
  * @param hQuadTree the quad tree
  * @param nMaxDepth the maximum depth allowed
  */
@@ -242,9 +244,9 @@ void CPLQuadTreeSetMaxDepth(CPLQuadTree *hQuadTree, int nMaxDepth)
  * Set the maximum capacity of a node of a quadtree. The default value is 8.
  * Note that the maximum capacity will only be honoured if the features
  * inserted have a point geometry. Otherwise it may be exceeded.
- * 
+ *
  * @param hQuadTree the quad tree
- * @param nBucketCapacity the maximum capactiy of a node of a quadtree
+ * @param nBucketCapacity the maximum capacity of a node of a quadtree
  */
 
 void CPLQuadTreeSetBucketCapacity(CPLQuadTree *hQuadTree, int nBucketCapacity)
@@ -259,14 +261,13 @@ void CPLQuadTreeSetBucketCapacity(CPLQuadTree *hQuadTree, int nBucketCapacity)
 
 /**
  * Insert a feature into a quadtree
- * 
+ *
  * @param hQuadTree the quad tree
  * @param hFeature the feature to insert
  */
 
 void CPLQuadTreeInsert(CPLQuadTree * hQuadTree, void* hFeature)
 {
-    CPLRectObj bounds;
     if( hQuadTree->pfnGetBounds == NULL )
     {
         CPLError(CE_Failure, CPLE_AppDefined,
@@ -274,6 +275,7 @@ void CPLQuadTreeInsert(CPLQuadTree * hQuadTree, void* hFeature)
         return;
     }
     hQuadTree->nFeatures ++;
+    CPLRectObj bounds;
     hQuadTree->pfnGetBounds(hFeature, &bounds);
     CPLQuadTreeAddFeatureInternal(hQuadTree, hFeature, &bounds);
 }
@@ -284,7 +286,7 @@ void CPLQuadTreeInsert(CPLQuadTree * hQuadTree, void* hFeature)
 
 /**
  * Insert a feature into a quadtree
- * 
+ *
  * @param hQuadTree the quad tree
  * @param hFeature the feature to insert
  * @param psBounds bounds of the feature
@@ -303,11 +305,9 @@ void CPLQuadTreeInsertWithBounds(CPLQuadTree *hQuadTree,
 
 static void CPLQuadTreeNodeDestroy(QuadTreeNode *psNode)
 {
-    int i;
-
-    for(i=0; i<psNode->nNumSubNodes; i++ )
+    for(int i=0; i<psNode->nNumSubNodes; i++ )
     {
-        if(psNode->apSubNode[i]) 
+        if(psNode->apSubNode[i])
             CPLQuadTreeNodeDestroy(psNode->apSubNode[i]);
     }
 
@@ -327,7 +327,7 @@ static void CPLQuadTreeNodeDestroy(QuadTreeNode *psNode)
 
 /**
  * Destroy a quadtree
- * 
+ *
  * @param hQuadTree the quad tree to destroy
  */
 
@@ -349,21 +349,19 @@ static void CPLQuadTreeSplitBounds( double dfSplitRatio,
                                       CPLRectObj *out1,
                                       CPLRectObj *out2)
 {
-  double range;
-
   /* -------------------------------------------------------------------- */
   /*      The output bounds will be very similar to the input bounds,     */
   /*      so just copy over to start.                                     */
   /* -------------------------------------------------------------------- */
   memcpy(out1, in, sizeof(CPLRectObj));
   memcpy(out2, in, sizeof(CPLRectObj));
-  
+
   /* -------------------------------------------------------------------- */
   /*      Split in X direction.                                           */
   /* -------------------------------------------------------------------- */
   if((in->maxx - in->minx) > (in->maxy - in->miny)) {
-    range = in->maxx - in->minx;
-    
+    const double range = in->maxx - in->minx;
+
     out1->maxx = in->minx + range * dfSplitRatio;
     out2->minx = in->maxx - range * dfSplitRatio;
   }
@@ -372,8 +370,8 @@ static void CPLQuadTreeSplitBounds( double dfSplitRatio,
   /*      Otherwise split in Y direction.                                 */
   /* -------------------------------------------------------------------- */
   else {
-    range = in->maxy - in->miny;
-    
+    const double range = in->maxy - in->miny;
+
     out1->maxy = in->miny + range * dfSplitRatio;
     out2->miny = in->maxy - range * dfSplitRatio;
   }
@@ -388,7 +386,6 @@ static void CPLQuadTreeNodeAddFeatureAlg1( CPLQuadTree* hQuadTree,
                                            void* hFeature,
                                            const CPLRectObj* pRect)
 {
-    int i;
     if (psNode->nNumSubNodes == 0)
     {
         /* If we have reached the max bucket capacity, try to insert */
@@ -397,9 +394,12 @@ static void CPLQuadTreeNodeAddFeatureAlg1( CPLQuadTree* hQuadTree,
         {
             CPLRectObj half1, half2, quad1, quad2, quad3, quad4;
 
-            CPLQuadTreeSplitBounds(hQuadTree->dfSplitRatio, &psNode->rect, &half1, &half2);
-            CPLQuadTreeSplitBounds(hQuadTree->dfSplitRatio, &half1, &quad1, &quad2);
-            CPLQuadTreeSplitBounds(hQuadTree->dfSplitRatio, &half2, &quad3, &quad4);
+            CPLQuadTreeSplitBounds( hQuadTree->dfSplitRatio, &psNode->rect,
+                                    &half1, &half2);
+            CPLQuadTreeSplitBounds( hQuadTree->dfSplitRatio, &half1,
+                                    &quad1, &quad2);
+            CPLQuadTreeSplitBounds( hQuadTree->dfSplitRatio, &half2,
+                                    &quad3, &quad4);
 
             if (memcmp(&psNode->rect, &quad1, sizeof(CPLRectObj)) != 0 &&
                 memcmp(&psNode->rect, &quad2, sizeof(CPLRectObj)) != 0 &&
@@ -416,7 +416,7 @@ static void CPLQuadTreeNodeAddFeatureAlg1( CPLQuadTree* hQuadTree,
                 psNode->apSubNode[2] = CPLQuadTreeNodeCreate(&quad3);
                 psNode->apSubNode[3] = CPLQuadTreeNodeCreate(&quad4);
 
-                int oldNumFeatures = psNode->nFeatures;
+                const int oldNumFeatures = psNode->nFeatures;
                 void** oldFeatures = psNode->pahFeatures;
                 CPLRectObj* pasOldBounds = psNode->pasBounds;
                 psNode->nFeatures = 0;
@@ -424,16 +424,19 @@ static void CPLQuadTreeNodeAddFeatureAlg1( CPLQuadTree* hQuadTree,
                 psNode->pasBounds = NULL;
 
                 /* redispatch existing pahFeatures in apSubNodes */
-                int i;
-                for(i=0;i<oldNumFeatures;i++)
+                for(int i=0;i<oldNumFeatures;i++)
                 {
                     if( hQuadTree->pfnGetBounds == NULL )
-                        CPLQuadTreeNodeAddFeatureAlg1(hQuadTree, psNode, oldFeatures[i], &pasOldBounds[i]);
+                        CPLQuadTreeNodeAddFeatureAlg1( hQuadTree, psNode,
+                                                       oldFeatures[i],
+                                                       &pasOldBounds[i]);
                     else
                     {
                         CPLRectObj bounds;
                         hQuadTree->pfnGetBounds(oldFeatures[i], &bounds);
-                        CPLQuadTreeNodeAddFeatureAlg1(hQuadTree, psNode, oldFeatures[i], &bounds);
+                        CPLQuadTreeNodeAddFeatureAlg1( hQuadTree, psNode,
+                                                       oldFeatures[i],
+                                                       &bounds );
                     }
                 }
 
@@ -441,7 +444,8 @@ static void CPLQuadTreeNodeAddFeatureAlg1( CPLQuadTree* hQuadTree,
                 CPLFree(pasOldBounds);
 
                 /* recurse back on this psNode now that it has apSubNodes */
-                CPLQuadTreeNodeAddFeatureAlg1(hQuadTree, psNode, hFeature, pRect);
+                CPLQuadTreeNodeAddFeatureAlg1( hQuadTree, psNode, hFeature,
+                                               pRect );
                 return;
             }
         }
@@ -452,11 +456,12 @@ static void CPLQuadTreeNodeAddFeatureAlg1( CPLQuadTree* hQuadTree,
     /*      If there are apSubNodes, then consider whether this object        */
     /*      will fit in them.                                               */
     /* -------------------------------------------------------------------- */
-        for(i=0; i<psNode->nNumSubNodes; i++ )
+        for(int i=0; i<psNode->nNumSubNodes; i++ )
         {
             if( CPL_RectContained(pRect, &psNode->apSubNode[i]->rect))
             {
-                CPLQuadTreeNodeAddFeatureAlg1( hQuadTree, psNode->apSubNode[i], hFeature, pRect);
+                CPLQuadTreeNodeAddFeatureAlg1( hQuadTree, psNode->apSubNode[i],
+                                               hFeature, pRect );
                 return;
             }
         }
@@ -470,15 +475,21 @@ static void CPLQuadTreeNodeAddFeatureAlg1( CPLQuadTree* hQuadTree,
     if( psNode->nFeatures == 1 )
     {
         CPLAssert( psNode->pahFeatures == NULL );
-        psNode->pahFeatures = (void**) CPLMalloc( hQuadTree->nBucketCapacity * sizeof(void*) );
+        psNode->pahFeatures = static_cast<void **>(
+            CPLMalloc( hQuadTree->nBucketCapacity * sizeof(void*) ) );
         if( hQuadTree->pfnGetBounds == NULL )
-            psNode->pasBounds = (CPLRectObj*) CPLMalloc( hQuadTree->nBucketCapacity * sizeof(CPLRectObj) );
+            psNode->pasBounds = static_cast<CPLRectObj *>(
+                CPLMalloc( hQuadTree->nBucketCapacity * sizeof(CPLRectObj) ) );
     }
     else if( psNode->nFeatures > hQuadTree->nBucketCapacity )
     {
-        psNode->pahFeatures = (void**) CPLRealloc( psNode->pahFeatures, sizeof(void*) * psNode->nFeatures );
+        psNode->pahFeatures = static_cast<void **>(
+            CPLRealloc( psNode->pahFeatures,
+                        sizeof(void*) * psNode->nFeatures ) );
         if( hQuadTree->pfnGetBounds == NULL )
-            psNode->pasBounds = (CPLRectObj*) CPLRealloc( psNode->pasBounds, sizeof(CPLRectObj) * psNode->nFeatures );
+            psNode->pasBounds = static_cast<CPLRectObj *>(
+                CPLRealloc( psNode->pasBounds,
+                            sizeof(CPLRectObj) * psNode->nFeatures ) );
     }
     psNode->pahFeatures[psNode->nFeatures-1] = hFeature;
     if( hQuadTree->pfnGetBounds == NULL )
@@ -498,15 +509,13 @@ static void CPLQuadTreeNodeAddFeatureAlg2( CPLQuadTree *hQuadTree,
                                            const CPLRectObj* pRect,
                                            int nMaxDepth)
 {
-    int i;
-
   /* -------------------------------------------------------------------- */
   /*      If there are apSubNodes, then consider whether this object        */
   /*      will fit in them.                                               */
   /* -------------------------------------------------------------------- */
     if( nMaxDepth > 1 && psNode->nNumSubNodes > 0 )
     {
-        for(i=0; i<psNode->nNumSubNodes; i++ )
+        for(int i=0; i<psNode->nNumSubNodes; i++ )
         {
             if( CPL_RectContained(pRect, &psNode->apSubNode[i]->rect))
             {
@@ -525,7 +534,8 @@ static void CPLQuadTreeNodeAddFeatureAlg2( CPLQuadTree *hQuadTree,
     {
         CPLRectObj half1, half2, quad1, quad2, quad3, quad4;
 
-        CPLQuadTreeSplitBounds(hQuadTree->dfSplitRatio, &psNode->rect, &half1, &half2);
+        CPLQuadTreeSplitBounds(hQuadTree->dfSplitRatio, &psNode->rect,
+                               &half1, &half2);
         CPLQuadTreeSplitBounds(hQuadTree->dfSplitRatio, &half1, &quad1, &quad2);
         CPLQuadTreeSplitBounds(hQuadTree->dfSplitRatio, &half2, &quad3, &quad4);
 
@@ -545,24 +555,27 @@ static void CPLQuadTreeNodeAddFeatureAlg2( CPLQuadTree *hQuadTree,
             psNode->apSubNode[3] = CPLQuadTreeNodeCreate(&quad4);
 
             /* recurse back on this psNode now that it has apSubNodes */
-            CPLQuadTreeNodeAddFeatureAlg2(hQuadTree, psNode, hFeature, pRect, nMaxDepth);
+            CPLQuadTreeNodeAddFeatureAlg2( hQuadTree, psNode, hFeature,
+                                           pRect, nMaxDepth);
             return;
         }
     }
 
+
 /* -------------------------------------------------------------------- */
 /*      If none of that worked, just add it to this psNodes list.         */
 /* -------------------------------------------------------------------- */
     psNode->nFeatures++;
 
     psNode->pahFeatures =
-            (void**) CPLRealloc( psNode->pahFeatures,
-                                 sizeof(void*) * psNode->nFeatures );
+        static_cast<void **>( CPLRealloc( psNode->pahFeatures,
+                                          sizeof(void*) * psNode->nFeatures ) );
     if( hQuadTree->pfnGetBounds == NULL )
     {
         psNode->pasBounds =
-            (CPLRectObj*) CPLRealloc( psNode->pasBounds,
-                                 sizeof(CPLRectObj) * psNode->nFeatures );
+          static_cast<CPLRectObj*>(
+              CPLRealloc( psNode->pasBounds,
+                          sizeof(CPLRectObj) * psNode->nFeatures ) );
     }
     psNode->pahFeatures[psNode->nFeatures-1] = hFeature;
     if( hQuadTree->pfnGetBounds == NULL )
@@ -603,29 +616,27 @@ static void CPLQuadTreeCollectFeatures(const CPLQuadTree *hQuadTree,
                                        int* pnMaxFeatures,
                                        void*** pppFeatureList)
 {
-  int i;
-
   /* -------------------------------------------------------------------- */
-  /*      Does this psNode overlap the area of interest at all?  If not,    */
+  /*      Does this psNode overlap the area of interest at all?  If not,  */
   /*      return without adding to the list at all.                       */
   /* -------------------------------------------------------------------- */
   if(!CPL_RectOverlap(&psNode->rect, pAoi))
      return;
 
-/* -------------------------------------------------------------------- */
-/*      Grow the list to hold the features on this psNode.              */
-/* -------------------------------------------------------------------- */
-    if( *pnFeatureCount + psNode->nFeatures > *pnMaxFeatures )
-    {
-        *pnMaxFeatures = (*pnFeatureCount + psNode->nFeatures) * 2 + 20;
-        *pppFeatureList = (void**)
-            CPLRealloc(*pppFeatureList,sizeof(void*) * *pnMaxFeatures);
-    }
+  /* -------------------------------------------------------------------- */
+  /*      Grow the list to hold the features on this psNode.              */
+  /* -------------------------------------------------------------------- */
+  if( *pnFeatureCount + psNode->nFeatures > *pnMaxFeatures )
+  {
+      *pnMaxFeatures = (*pnFeatureCount + psNode->nFeatures) * 2 + 20;
+      *pppFeatureList = static_cast<void **>(
+          CPLRealloc(*pppFeatureList,sizeof(void*) * *pnMaxFeatures) );
+  }
 
   /* -------------------------------------------------------------------- */
   /*      Add the local features to the list.                             */
   /* -------------------------------------------------------------------- */
-  for(i=0; i<psNode->nFeatures; i++)
+  for(int i=0; i<psNode->nFeatures; i++)
   {
       if( hQuadTree->pfnGetBounds == NULL )
       {
@@ -640,15 +651,16 @@ static void CPLQuadTreeCollectFeatures(const CPLQuadTree *hQuadTree,
             (*pppFeatureList)[(*pnFeatureCount)++] = psNode->pahFeatures[i];
       }
   }
-  
+
   /* -------------------------------------------------------------------- */
   /*      Recurse to subnodes if they exist.                              */
   /* -------------------------------------------------------------------- */
-  for(i=0; i<psNode->nNumSubNodes; i++)
+  for(int i=0; i<psNode->nNumSubNodes; i++)
   {
       if(psNode->apSubNode[i])
-        CPLQuadTreeCollectFeatures(hQuadTree, psNode->apSubNode[i], pAoi,
-                                   pnFeatureCount, pnMaxFeatures, pppFeatureList);
+        CPLQuadTreeCollectFeatures( hQuadTree, psNode->apSubNode[i], pAoi,
+                                    pnFeatureCount, pnMaxFeatures,
+                                    pppFeatureList );
   }
 }
 
@@ -659,7 +671,7 @@ static void CPLQuadTreeCollectFeatures(const CPLQuadTree *hQuadTree,
 /**
  * Returns all the elements inserted whose bounding box intersects the
  * provided area of interest
- * 
+ *
  * @param hQuadTree the quad tree
  * @param pAoi the pointer to the area of interest
  * @param pnFeatureCount the user data provided to the function.
@@ -671,45 +683,44 @@ void** CPLQuadTreeSearch(const CPLQuadTree *hQuadTree,
                          const CPLRectObj* pAoi,
                          int* pnFeatureCount)
 {
-  void** ppFeatureList = NULL;
-  int nMaxFeatures = 0;
-  int nFeatureCount = 0;
-
   CPLAssert(hQuadTree);
   CPLAssert(pAoi);
 
+  int nFeatureCount = 0;
   if (pnFeatureCount == NULL)
       pnFeatureCount = &nFeatureCount;
 
   *pnFeatureCount = 0;
+
+  int nMaxFeatures = 0;
+  void** ppFeatureList = NULL;
   CPLQuadTreeCollectFeatures(hQuadTree, hQuadTree->psRoot, pAoi,
                             pnFeatureCount, &nMaxFeatures, &ppFeatureList);
 
-  return(ppFeatureList);
+  return ppFeatureList;
 }
 
 /************************************************************************/
 /*                    CPLQuadTreeNodeForeach()                          */
 /************************************************************************/
 
-static int CPLQuadTreeNodeForeach(const QuadTreeNode *psNode,
+static bool CPLQuadTreeNodeForeach(const QuadTreeNode *psNode,
                                   CPLQuadTreeForeachFunc pfnForeach,
                                   void* pUserData)
 {
-    int i;
-    for(i=0; i<psNode->nNumSubNodes; i++ )
+    for(int i=0; i<psNode->nNumSubNodes; i++ )
     {
-        if (CPLQuadTreeNodeForeach(psNode->apSubNode[i], pfnForeach, pUserData) == FALSE)
-            return FALSE;
+        if( !CPLQuadTreeNodeForeach(psNode->apSubNode[i], pfnForeach, pUserData) )
+            return false;
     }
 
-    for(i=0; i<psNode->nFeatures; i++)
+    for(int i=0; i<psNode->nFeatures; i++)
     {
         if (pfnForeach(psNode->pahFeatures[i], pUserData) == FALSE)
-            return FALSE;
+            return false;
     }
 
-    return TRUE;
+    return true;
 }
 
 /************************************************************************/
@@ -726,7 +737,7 @@ static int CPLQuadTreeNodeForeach(const QuadTreeNode *psNode,
  *
  * Note : the structure of the quadtree must *NOT* be modified during the
  * walk.
- * 
+ *
  * @param hQuadTree the quad tree
  * @param pfnForeach the function called on each element.
  * @param pUserData the user data provided to the function.
@@ -750,16 +761,14 @@ static void CPLQuadTreeDumpNode(const QuadTreeNode *psNode,
                                 CPLQuadTreeDumpFeatureFunc pfnDumpFeatureFunc,
                                 void* pUserData)
 {
-    int i;
-    int count;
     if (psNode->nNumSubNodes)
     {
-        for(count=nIndentLevel;--count>=0;)
+        for(int count=nIndentLevel;--count>=0;)
             printf("  ");
         printf("SubhQuadTrees :\n");
-        for(i=0; i<psNode->nNumSubNodes; i++ )
+        for(int i=0; i<psNode->nNumSubNodes; i++ )
         {
-            for(count=nIndentLevel+1;--count>=0;)
+            for(int count=nIndentLevel+1;--count>=0;)
                 printf("  ");
             printf("SubhQuadTree %d :\n", i+1);
             CPLQuadTreeDumpNode(psNode->apSubNode[i], nIndentLevel + 2,
@@ -768,17 +777,17 @@ static void CPLQuadTreeDumpNode(const QuadTreeNode *psNode,
     }
     if (psNode->nFeatures)
     {
-        for(count=nIndentLevel;--count>=0;)
+        for(int count=nIndentLevel;--count>=0;)
             printf("  ");
         printf("Leaves (%d):\n", psNode->nFeatures);
-        for(i=0; i<psNode->nFeatures; i++)
+        for(int i=0; i<psNode->nFeatures; i++)
         {
             if (pfnDumpFeatureFunc)
                 pfnDumpFeatureFunc(psNode->pahFeatures[i], nIndentLevel + 2,
                                    pUserData);
             else
             {
-                for(count=nIndentLevel + 1;--count>=0;)
+                for(int count=nIndentLevel + 1;--count>=0;)
                     printf("  ");
                 printf("%p\n", psNode->pahFeatures[i]);
             }
@@ -808,13 +817,13 @@ void CPLQuadTreeGetStatsNode(const QuadTreeNode *psNode,
                              int* pnMaxDepth,
                              int* pnMaxBucketCapacity)
 {
-    int i;
     (*pnNodeCount) ++;
     if (nDepthLevel > *pnMaxDepth)
         *pnMaxDepth = nDepthLevel;
     if (psNode->nFeatures > *pnMaxBucketCapacity)
         *pnMaxBucketCapacity = psNode->nFeatures;
-    for(i=0; i<psNode->nNumSubNodes; i++ )
+
+    for(int i=0; i<psNode->nNumSubNodes; i++ )
     {
         CPLQuadTreeGetStatsNode(psNode->apSubNode[i], nDepthLevel + 1,
                                 pnNodeCount, pnMaxDepth, pnMaxBucketCapacity);
@@ -848,5 +857,6 @@ void CPLQuadTreeGetStats(const CPLQuadTree *hQuadTree,
     *pnMaxDepth = 1;
     *pnMaxBucketCapacity = 0;
 
-    CPLQuadTreeGetStatsNode(hQuadTree->psRoot, 0, pnNodeCount, pnMaxDepth, pnMaxBucketCapacity);
+    CPLQuadTreeGetStatsNode( hQuadTree->psRoot, 0, pnNodeCount, pnMaxDepth,
+                             pnMaxBucketCapacity );
 }
