@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cpl_vsi_mem.cpp 33758 2016-03-21 09:06:22Z rouault $
+ * $Id: cpl_vsi_mem.cpp 35728 2016-10-14 16:17:28Z rouault $
  *
  * Project:  VSI Virtual File System
  * Purpose:  Implementation of Memory Buffer virtual IO functions.
@@ -31,10 +31,11 @@
 #include "cpl_vsi_virtual.h"
 #include "cpl_string.h"
 #include "cpl_multiproc.h"
+#include "cpl_atomic_ops.h"
 #include <time.h>
 #include <map>
 
-CPL_CVSID("$Id: cpl_vsi_mem.cpp 33758 2016-03-21 09:06:22Z rouault $");
+CPL_CVSID("$Id: cpl_vsi_mem.cpp 35728 2016-10-14 16:17:28Z rouault $");
 
 /*
 ** Notes on Multithreading:
@@ -73,7 +74,7 @@ class VSIMemFile
 {
 public:
     CPLString     osFilename;
-    int           nRefCount;
+    volatile int  nRefCount;
 
     int           bIsDirectory;
 
@@ -179,7 +180,8 @@ VSIMemFile::~VSIMemFile()
 
 {
     if( nRefCount != 0 )
-        CPLDebug( "VSIMemFile", "Memory file %s deleted with %d references.",
+        CPLError( CE_Warning, CPLE_AppDefined,
+                  "Memory file %s deleted with %d references.",
                   osFilename.c_str(), nRefCount );
 
     if( bOwnData && pabyData )
@@ -251,7 +253,7 @@ bool VSIMemFile::SetLength( vsi_l_offset nNewLength )
 int VSIMemHandle::Close()
 
 {
-    if( --(poFile->nRefCount) == 0 )
+    if( CPLAtomicDec(&(poFile->nRefCount)) == 0 )
         delete poFile;
 
     poFile = NULL;
@@ -434,7 +436,7 @@ VSIMemFilesystemHandler::~VSIMemFilesystemHandler()
          iter != oFileList.end();
          ++iter )
     {
-        iter->second->nRefCount--;
+        CPLAtomicDec(&(iter->second->nRefCount));
         delete iter->second;
     }
 
@@ -482,7 +484,7 @@ VSIMemFilesystemHandler::Open( const char *pszFilename,
         poFile = new VSIMemFile;
         poFile->osFilename = osFilename;
         oFileList[poFile->osFilename] = poFile;
-        poFile->nRefCount++; // for file list
+        CPLAtomicInc(&(poFile->nRefCount)); // for file list
     }
     // Overwrite
     else if( strstr(pszAccess, "w") )
@@ -510,7 +512,7 @@ VSIMemFilesystemHandler::Open( const char *pszFilename,
     else
         poHandle->bUpdate = FALSE;
 
-    poFile->nRefCount++;
+    CPLAtomicInc(&(poFile->nRefCount));
 
     if( strstr(pszAccess,"a") )
         poHandle->m_nOffset = poFile->nLength;
@@ -596,7 +598,7 @@ int VSIMemFilesystemHandler::Unlink_unlocked( const char * pszFilename )
 
     VSIMemFile *poFile = oFileList[osFilename];
 
-    if( --(poFile->nRefCount) == 0 )
+    if( CPLAtomicDec(&(poFile->nRefCount)) == 0 )
         delete poFile;
 
     oFileList.erase( oFileList.find(osFilename) );
@@ -629,7 +631,7 @@ int VSIMemFilesystemHandler::Mkdir( const char * pszPathname,
     poFile->osFilename = osPathname;
     poFile->bIsDirectory = TRUE;
     oFileList[osPathname] = poFile;
-    poFile->nRefCount++; /* referenced by file list */
+    CPLAtomicInc(&(poFile->nRefCount)); /* referenced by file list */
 
     return 0;
 }
@@ -891,7 +893,7 @@ VSILFILE *VSIFileFromMemBuffer( const char *pszFilename,
         CPLMutexHolder oHolder( &poHandler->hMutex );
         poHandler->Unlink_unlocked(osFilename);
         poHandler->oFileList[poFile->osFilename] = poFile;
-        poFile->nRefCount++;
+        CPLAtomicInc(&(poFile->nRefCount));
     }
 
     return (VSILFILE *) poHandler->Open( osFilename, "r+" );
@@ -949,7 +951,7 @@ GByte *VSIGetMemFileBuffer( const char *pszFilename,
             poFile->bOwnData = FALSE;
 
         poHandler->oFileList.erase( poHandler->oFileList.find(osFilename) );
-        --(poFile->nRefCount);
+        CPLAtomicDec(&(poFile->nRefCount));
         delete poFile;
     }
 
