@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: cpl_vsisimple.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: cpl_vsisimple.cpp 33724 2016-03-16 17:14:11Z goatbar $
  *
- * Project:  Common Portability Library 
+ * Project:  Common Portability Library
  * Purpose:  Simple implementation of POSIX VSI functions.
  * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
@@ -36,10 +36,10 @@
  ****************************************************************************/
 
 #include "cpl_config.h"
-#include "cpl_port.h"
-#include "cpl_vsi.h"
 #include "cpl_error.h"
+#include "cpl_port.h"
 #include "cpl_string.h"
+#include "cpl_vsi.h"
 
 /* Uncomment to check consistent usage of VSIMalloc(), VSIRealloc(), */
 /* VSICalloc(), VSIFree(), VSIStrdup() */
@@ -61,31 +61,32 @@
 /* DEBUG_VSIMALLOC must also be defined */
 //#define DEBUG_VSIMALLOC_VERBOSE
 
-CPL_CVSID("$Id: cpl_vsisimple.cpp 27044 2014-03-16 23:41:27Z rouault $");
+/* Number of bytes of the malloc/calloc/free that triggers a debug trace. Can be 0 for all allocs */
+#define THRESHOLD_PRINT 10000
+
+/* Uncomment to print GDAL block cache use. */
+/* Only used if DEBUG_VSIMALLOC_VERBOSE is enabled */
+//#define DEBUG_BLOCK_CACHE_USE
+
+#ifdef DEBUG_BLOCK_CACHE_USE
+extern "C" GIntBig CPL_DLL CPL_STDCALL GDALGetCacheUsed64(void);
+#endif
+
+CPL_CVSID("$Id: cpl_vsisimple.cpp 33724 2016-03-16 17:14:11Z goatbar $");
 
 /* for stat() */
 
 /* Unix or Windows NT/2000/XP */
-#if !defined(WIN32) && !defined(WIN32CE)
+#if !defined(WIN32)
 #  include <unistd.h>
-#elif !defined(WIN32CE) /* not Win32 platform */
+#else
 #  include <io.h>
 #  include <fcntl.h>
 #  include <direct.h>
 #endif
 
-/* Windows CE or other platforms */
-#if defined(WIN32CE)
-#  include <wce_io.h>
-#  include <wce_stat.h>
-#  include <wce_stdio.h>
-#  include <wce_string.h>
-#  include <wce_time.h>
-# define time wceex_time
-#else
-#  include <sys/stat.h>
-#  include <time.h>
-#endif
+#include <sys/stat.h>
+#include <time.h>
 
 /************************************************************************/
 /*                              VSIFOpen()                              */
@@ -95,15 +96,14 @@ FILE *VSIFOpen( const char * pszFilename, const char * pszAccess )
 
 {
     FILE *fp = NULL;
-    int     nError;
 
-#if defined(WIN32) && !defined(WIN32CE)
+#if defined(WIN32)
     if( CSLTestBoolean(
             CPLGetConfigOption( "GDAL_FILENAME_IS_UTF8", "YES" ) ) )
     {
-        wchar_t *pwszFilename = 
+        wchar_t *pwszFilename =
             CPLRecodeToWChar( pszFilename, CPL_ENC_UTF8, CPL_ENC_UCS2 );
-        wchar_t *pwszAccess = 
+        wchar_t *pwszAccess =
             CPLRecodeToWChar( pszAccess, CPL_ENC_UTF8, CPL_ENC_UCS2 );
 
         fp = _wfopen( pwszFilename, pwszAccess );
@@ -115,11 +115,15 @@ FILE *VSIFOpen( const char * pszFilename, const char * pszAccess )
 #endif
     fp = fopen( (char *) pszFilename, (char *) pszAccess );
 
-    nError = errno;
+#ifdef VSI_DEBUG
+    // Capture the error from fopen to avoid being overwritten by errors
+    // from VSIDebug3.
+    const int nError = errno;
     VSIDebug3( "VSIFOpen(%s,%s) = %p", pszFilename, pszAccess, fp );
     errno = nError;
+#endif
 
-    return( fp );
+    return fp;
 }
 
 /************************************************************************/
@@ -131,7 +135,7 @@ int VSIFClose( FILE * fp )
 {
     VSIDebug1( "VSIClose(%p)", fp );
 
-    return( fclose(fp) );
+    return fclose(fp);
 }
 
 /************************************************************************/
@@ -141,10 +145,19 @@ int VSIFClose( FILE * fp )
 int VSIFSeek( FILE * fp, long nOffset, int nWhence )
 
 {
-    int     nResult = fseek( fp, nOffset, nWhence );
-    int     nError = errno;
+#ifdef DEBUG
+    /* To workaround Coverity strange warning about potential negative seek */
+    /* CID 1340084 when called from dgnwrite.cpp */
+    if( nWhence == SEEK_SET && nOffset < 0 )
+        return -1;
+#endif
+    int nResult = fseek( fp, nOffset, nWhence );
 
 #ifdef VSI_DEBUG
+    // Capture the error from fseek to avoid being overwritten by errors
+    // from VSIDebug.
+    const int nError = errno;
+
     if( nWhence == SEEK_SET )
     {
         VSIDebug3( "VSIFSeek(%p,%ld,SEEK_SET) = %d", fp, nOffset, nResult );
@@ -162,9 +175,10 @@ int VSIFSeek( FILE * fp, long nOffset, int nWhence )
         VSIDebug4( "VSIFSeek(%p,%ld,%d-Unknown) = %d",
                    fp, nOffset, nWhence, nResult );
     }
-#endif 
 
     errno = nError;
+#endif
+
     return nResult;
 }
 
@@ -175,12 +189,16 @@ int VSIFSeek( FILE * fp, long nOffset, int nWhence )
 long VSIFTell( FILE * fp )
 
 {
-    long    nOffset = ftell(fp);
-    int     nError = errno;
+    const long nOffset = ftell(fp);
 
+#ifdef VSI_DEBUG
+    // Capture the error from ftell to avoid being overwritten by errors
+    // from VSIDebug.
+    const int nError = errno;
     VSIDebug2( "VSIFTell(%p) = %ld", fp, nOffset );
-
     errno = nError;
+#endif
+
     return nOffset;
 }
 
@@ -193,6 +211,13 @@ void VSIRewind( FILE * fp )
 {
     VSIDebug1("VSIRewind(%p)", fp );
     rewind( fp );
+#ifdef VSI_DEBUG
+    // Capture the error rewind ftell to avoid being overwritten by errors
+    // from VSIDebug.
+    const int nError = errno;
+    VSIDebug2( "VSIRewind(%p) errno = %d", fp, nError );
+    errno = nError;
+#endif
 }
 
 /************************************************************************/
@@ -202,13 +227,18 @@ void VSIRewind( FILE * fp )
 size_t VSIFRead( void * pBuffer, size_t nSize, size_t nCount, FILE * fp )
 
 {
-    size_t  nResult = fread( pBuffer, nSize, nCount, fp );
-    int     nError = errno;
-
-    VSIDebug4( "VSIFRead(%p,%ld,%ld) = %ld", 
-               fp, (long)nSize, (long)nCount, (long)nResult );
+    const size_t nResult = fread( pBuffer, nSize, nCount, fp );
 
+#ifdef VSI_DEBUG
+    // Capture the error from fread to avoid being overwritten by errors
+    // from VSIDebug.
+    const int nError = errno;
+    VSIDebug4( "VSIFRead(%p,%ld,%ld) = %ld",
+               fp, static_cast<long>(nSize), static_cast<long>(nCount),
+               static_cast<long>(nResult) );
     errno = nError;
+#endif
+
     return nResult;
 }
 
@@ -219,13 +249,18 @@ size_t VSIFRead( void * pBuffer, size_t nSize, size_t nCount, FILE * fp )
 size_t VSIFWrite( const void *pBuffer, size_t nSize, size_t nCount, FILE * fp )
 
 {
-    size_t  nResult = fwrite( pBuffer, nSize, nCount, fp );
-    int     nError = errno;
-
-    VSIDebug4( "VSIFWrite(%p,%ld,%ld) = %ld", 
-               fp, (long)nSize, (long)nCount, (long)nResult );
+    const size_t nResult = fwrite( pBuffer, nSize, nCount, fp );
 
+#ifdef VSI_DEBUG
+    // Capture the error from fwrite to avoid being overwritten by errors
+    // from VSIDebug.
+    const int nError = errno;
+    VSIDebug4( "VSIFWrite(%p,%ld,%ld) = %ld",
+               fp, static_cast<long>(nSize), static_cast<long>(nCount),
+               static_cast<long>(nResult) );
     errno = nError;
+#endif
+
     return nResult;
 }
 
@@ -236,8 +271,24 @@ size_t VSIFWrite( const void *pBuffer, size_t nSize, size_t nCount, FILE * fp )
 void VSIFFlush( FILE * fp )
 
 {
+#ifdef VSI_DEBUG
     VSIDebug1( "VSIFFlush(%p)", fp );
-    fflush( fp );
+    const int result =
+#endif
+        fflush( fp );
+
+#ifdef VSI_DEBUG
+    // Capture the error rewind ftell to avoid being overwritten by errors
+    // from VSIDebug.
+    const int nError = errno;
+    VSIDebug2( "VSIRewind(%p) errno = %d", fp, nError );
+    if ( result != 0 )
+    {
+        CPLError( CE_Failure, CPLE_FileIO, "Flush failed.  errno = %d",
+                  nError);
+    }
+    errno = nError;
+#endif
 }
 
 /************************************************************************/
@@ -247,7 +298,7 @@ void VSIFFlush( FILE * fp )
 char *VSIFGets( char *pszBuffer, int nBufferSize, FILE * fp )
 
 {
-    return( fgets( pszBuffer, nBufferSize, fp ) );
+    return fgets( pszBuffer, nBufferSize, fp );
 }
 
 /************************************************************************/
@@ -257,7 +308,7 @@ char *VSIFGets( char *pszBuffer, int nBufferSize, FILE * fp )
 int VSIFGetc( FILE * fp )
 
 {
-    return( fgetc( fp ) );
+    return fgetc( fp );
 }
 
 /************************************************************************/
@@ -267,7 +318,7 @@ int VSIFGetc( FILE * fp )
 int VSIUngetc( int c, FILE * fp )
 
 {
-    return( ungetc( c, fp ) );
+    return ungetc( c, fp );
 }
 
 /************************************************************************/
@@ -282,13 +333,12 @@ int     VSIFPrintf( FILE * fp, const char * pszFormat, ... )
 
 {
     va_list     args;
-    int         nReturn;
 
     va_start( args, pszFormat );
-    nReturn = vfprintf( fp, pszFormat, args );
+    const int nReturn = vfprintf( fp, pszFormat, args );
     va_end( args );
 
-    return( nReturn );
+    return nReturn;
 }
 
 /************************************************************************/
@@ -298,7 +348,7 @@ int     VSIFPrintf( FILE * fp, const char * pszFormat, ... )
 int VSIFEof( FILE * fp )
 
 {
-    return( feof( fp ) );
+    return feof( fp );
 }
 
 /************************************************************************/
@@ -318,14 +368,14 @@ int VSIFPuts( const char * pszString, FILE * fp )
 int VSIFPutc( int nChar, FILE * fp )
 
 {
-    return( fputc( nChar, fp ) );
+    return fputc( nChar, fp );
 }
 
 
 #ifdef DEBUG_VSIMALLOC_STATS
 #include "cpl_multiproc.h"
 
-static void* hMemStatMutex = 0;
+static CPLMutex* hMemStatMutex = 0;
 static size_t nCurrentTotalAllocs = 0;
 static size_t nMaxTotalAllocs = 0;
 static GUIntBig nVSIMallocs = 0;
@@ -342,6 +392,8 @@ static GUIntBig nVSIFrees = 0;
 /*                         VSIShowMemStats()                            */
 /************************************************************************/
 
+void VSIShowMemStats();
+
 void VSIShowMemStats()
 {
     char* pszShowMemStats = getenv("CPL_SHOW_MEM_STATS");
@@ -424,8 +476,11 @@ void *VSICalloc( size_t nCount, size_t nSize )
     {
         CPLMutexHolderD(&hMemStatMutex);
 #ifdef DEBUG_VSIMALLOC_VERBOSE
-        fprintf(stderr, "Thread[%p] VSICalloc(%d,%d) = %p\n",
-                (void*)CPLGetPID(), (int)nCount, (int)nSize, ptr + 2 * sizeof(void*));
+        if( nMul > THRESHOLD_PRINT )
+        {
+            fprintf(stderr, "Thread[%p] VSICalloc(%d,%d) = %p\n",
+                    (void*)CPLGetPID(), (int)nCount, (int)nSize, ptr + 2 * sizeof(void*));
+        }
 #endif
 #ifdef DEBUG_VSIMALLOC_STATS
         nVSICallocs ++;
@@ -439,7 +494,7 @@ void *VSICalloc( size_t nCount, size_t nSize )
 #endif
     return ptr + 2 * sizeof(void*);
 #else
-    return( calloc( nCount, nSize ) );
+    return calloc( nCount, nSize );
 #endif
 }
 
@@ -447,10 +502,18 @@ void *VSICalloc( size_t nCount, size_t nSize )
 /*                             VSIMalloc()                              */
 /************************************************************************/
 
+#ifndef DEBUG_VSIMALLOC
+void *VSIMalloc( size_t nSize )
+
+{
+    return malloc( nSize );
+}
+
+#else  // DEBUG_VSIMALLOC
+
 void *VSIMalloc( size_t nSize )
 
 {
-#ifdef DEBUG_VSIMALLOC
     if (nMaxPeakAllocSize < 0)
     {
         char* pszMaxPeakAllocSize = getenv("CPL_MAX_PEAK_ALLOC_SIZE");
@@ -463,7 +526,7 @@ void *VSIMalloc( size_t nSize )
 #ifdef DEBUG_VSIMALLOC_STATS
     if (nMaxCumulAllocSize > 0 && (GIntBig)nCurrentTotalAllocs + (GIntBig)nSize > nMaxCumulAllocSize)
         return NULL;
-#endif
+#endif  // DEBUG_VSIMALLOC_STATS
 
 #ifdef DEBUG_VSIMALLOC_MPROTECT
     char* ptr = NULL;
@@ -471,7 +534,7 @@ void *VSIMalloc( size_t nSize )
     posix_memalign((void**)&ptr, nPageSize, (3 * sizeof(void*) + nSize + nPageSize - 1) & ~(nPageSize - 1));
 #else
     char* ptr = (char*) malloc(3 * sizeof(void*) + nSize);
-#endif
+#endif  // DEBUG_VSIMALLOC_MPROTECT
     if (ptr == NULL)
         return NULL;
     ptr[0] = 'V';
@@ -487,9 +550,28 @@ void *VSIMalloc( size_t nSize )
     {
         CPLMutexHolderD(&hMemStatMutex);
 #ifdef DEBUG_VSIMALLOC_VERBOSE
-        fprintf(stderr, "Thread[%p] VSIMalloc(%d) = %p\n",
-                (void*)CPLGetPID(), (int)nSize, ptr + 2 * sizeof(void*));
+        if( nSize > THRESHOLD_PRINT )
+        {
+            fprintf(stderr, "Thread[%p] VSIMalloc(%d) = %p"
+#ifdef DEBUG_VSIMALLOC_STATS
+                         ", current_cumul = " CPL_FRMT_GUIB
+#ifdef DEBUG_BLOCK_CACHE_USE
+                         ", block_cache_used = " CPL_FRMT_GIB
 #endif
+                         ", mal+cal-free = %d"
+#endif
+                         "\n",
+                (void*)CPLGetPID(), (int)nSize, ptr + 2 * sizeof(void*)
+#ifdef DEBUG_VSIMALLOC_STATS
+                , (GUIntBig)(nCurrentTotalAllocs + nSize),
+#ifdef DEBUG_BLOCK_CACHE_USE
+                , GDALGetCacheUsed64()
+#endif
+                ,(int)(nVSIMallocs + nVSICallocs - nVSIFrees)
+#endif
+                );
+        }
+#endif  // DEBUG_VSIMALLOC_VERBOSE
 #ifdef DEBUG_VSIMALLOC_STATS
         nVSIMallocs ++;
         if (nMaxTotalAllocs == 0)
@@ -497,27 +579,23 @@ void *VSIMalloc( size_t nSize )
         nCurrentTotalAllocs += nSize;
         if (nCurrentTotalAllocs > nMaxTotalAllocs)
             nMaxTotalAllocs = nCurrentTotalAllocs;
-#endif
+#endif  // DEBUG_VSIMALLOC_STATS
     }
-#endif
+#endif  // DEBUG_VSIMALLOC_STATS || DEBUG_VSIMALLOC_VERBOSE
     return ptr + 2 * sizeof(void*);
-#else
-    return( malloc( nSize ) );
-#endif
 }
 
-#ifdef DEBUG_VSIMALLOC
-void VSICheckMarkerBegin(char* ptr)
+static void VSICheckMarkerBegin(char* ptr)
 {
     if (memcmp(ptr, "VSIM", 4) != 0)
     {
         CPLError(CE_Fatal, CPLE_AppDefined,
-                 "Inconsistant use of VSI memory allocation primitives for %p : %c%c%c%c",
+                 "Inconsistent use of VSI memory allocation primitives for %p : %c%c%c%c",
                  ptr, ptr[0], ptr[1], ptr[2], ptr[3]);
     }
 }
 
-void VSICheckMarkerEnd(char* ptr, size_t nEnd)
+static void VSICheckMarkerEnd(char* ptr, size_t nEnd)
 {
     if (memcmp(ptr + nEnd, "EVSI", 4) != 0)
     {
@@ -526,11 +604,11 @@ void VSICheckMarkerEnd(char* ptr, size_t nEnd)
     }
 }
 
-#endif
+#endif  // DEBUG_VSIMALLOC
 
 /************************************************************************/
 /*                             VSIRealloc()                             */
-/************************************************************************/      
+/************************************************************************/
 
 void * VSIRealloc( void * pData, size_t nNewSize )
 
@@ -538,17 +616,13 @@ void * VSIRealloc( void * pData, size_t nNewSize )
 #ifdef DEBUG_VSIMALLOC
     if (pData == NULL)
         return VSIMalloc(nNewSize);
-        
+
     char* ptr = ((char*)pData) - 2 * sizeof(void*);
     VSICheckMarkerBegin(ptr);
 
-    size_t nOldSize;
+    size_t nOldSize = 0;
     memcpy(&nOldSize, ptr + sizeof(void*), sizeof(void*));
     VSICheckMarkerEnd(ptr, 2 * sizeof(void*) + nOldSize);
-    ptr[2 * sizeof(void*) + nOldSize + 0] = 'I';
-    ptr[2 * sizeof(void*) + nOldSize + 1] = 'S';
-    ptr[2 * sizeof(void*) + nOldSize + 2] = 'V';
-    ptr[2 * sizeof(void*) + nOldSize + 3] = 'E';
 
     if (nMaxPeakAllocSize < 0)
     {
@@ -562,6 +636,11 @@ void * VSIRealloc( void * pData, size_t nNewSize )
         return NULL;
 #endif
 
+    ptr[2 * sizeof(void*) + nOldSize + 0] = 'I';
+    ptr[2 * sizeof(void*) + nOldSize + 1] = 'S';
+    ptr[2 * sizeof(void*) + nOldSize + 2] = 'V';
+    ptr[2 * sizeof(void*) + nOldSize + 3] = 'E';
+
 #ifdef DEBUG_VSIMALLOC_MPROTECT
     char* newptr = NULL;
     size_t nPageSize = getpagesize();
@@ -606,8 +685,11 @@ void * VSIRealloc( void * pData, size_t nNewSize )
     {
         CPLMutexHolderD(&hMemStatMutex);
 #ifdef DEBUG_VSIMALLOC_VERBOSE
-        fprintf(stderr, "Thread[%p] VSIRealloc(%p, %d) = %p\n",
-                (void*)CPLGetPID(), pData, (int)nNewSize, ptr + 2 * sizeof(void*));
+        if( nNewSize > THRESHOLD_PRINT )
+        {
+            fprintf(stderr, "Thread[%p] VSIRealloc(%p, %d) = %p\n",
+                    (void*)CPLGetPID(), pData, (int)nNewSize, ptr + 2 * sizeof(void*));
+        }
 #endif
 #ifdef DEBUG_VSIMALLOC_STATS
         nVSIReallocs ++;
@@ -620,7 +702,7 @@ void * VSIRealloc( void * pData, size_t nNewSize )
 #endif
     return ptr + 2 * sizeof(void*);
 #else
-    return( realloc( pData, nNewSize ) );
+    return realloc( pData, nNewSize );
 #endif
 }
 
@@ -637,7 +719,7 @@ void VSIFree( void * pData )
 
     char* ptr = ((char*)pData) - 2 * sizeof(void*);
     VSICheckMarkerBegin(ptr);
-    size_t nOldSize;
+    size_t nOldSize = 0;
     memcpy(&nOldSize, ptr + sizeof(void*), sizeof(void*));
     VSICheckMarkerEnd(ptr, 2 * sizeof(void*) + nOldSize);
     ptr[0] = 'M';
@@ -652,8 +734,11 @@ void VSIFree( void * pData )
     {
         CPLMutexHolderD(&hMemStatMutex);
 #ifdef DEBUG_VSIMALLOC_VERBOSE
-        fprintf(stderr, "Thread[%p] VSIFree(%p, (%d bytes))\n",
-                (void*)CPLGetPID(), pData, (int)nOldSize);
+        if( nOldSize > THRESHOLD_PRINT )
+        {
+            fprintf(stderr, "Thread[%p] VSIFree(%p, (%d bytes))\n",
+                    (void*)CPLGetPID(), pData, (int)nOldSize);
+        }
 #endif
 #ifdef DEBUG_VSIMALLOC_STATS
         nVSIFrees ++;
@@ -681,25 +766,22 @@ void VSIFree( void * pData )
 char *VSIStrdup( const char * pszString )
 
 {
-#ifdef DEBUG_VSIMALLOC
-    int nSize = strlen(pszString) + 1;
-    char* ptr = (char*) VSIMalloc(nSize);
+    const size_t nSize = strlen(pszString) + 1;
+    char* ptr = static_cast<char*>( VSIMalloc(nSize) );
     if (ptr == NULL)
         return NULL;
     memcpy(ptr, pszString, nSize);
     return ptr;
-#else
-    return( strdup( pszString ) );
-#endif
 }
 
 /************************************************************************/
 /*                          VSICheckMul2()                              */
 /************************************************************************/
 
-static size_t VSICheckMul2( size_t mul1, size_t mul2, int *pbOverflowFlag)
+static size_t VSICheckMul2( size_t mul1, size_t mul2, int *pbOverflowFlag,
+                            const char* pszFile, int nLine )
 {
-    size_t res = mul1 * mul2;
+    const size_t res = mul1 * mul2;
     if (mul1 != 0)
     {
         if (res / mul1 == mul2)
@@ -713,8 +795,10 @@ static size_t VSICheckMul2( size_t mul1, size_t mul2, int *pbOverflowFlag)
             if (pbOverflowFlag)
                 *pbOverflowFlag = TRUE;
             CPLError(CE_Failure, CPLE_OutOfMemory,
-                     "Multiplication overflow : %lu * %lu",
-                     (unsigned long)mul1, (unsigned long)mul2);
+                     "%s: %d: Multiplication overflow : " CPL_FRMT_GUIB " * " CPL_FRMT_GUIB,
+                     pszFile ? pszFile : "(unknown file)",
+                     nLine,
+                     (GUIntBig)mul1, (GUIntBig)mul2);
         }
     }
     else
@@ -729,14 +813,15 @@ static size_t VSICheckMul2( size_t mul1, size_t mul2, int *pbOverflowFlag)
 /*                          VSICheckMul3()                              */
 /************************************************************************/
 
-static size_t VSICheckMul3( size_t mul1, size_t mul2, size_t mul3, int *pbOverflowFlag)
+static size_t VSICheckMul3( size_t mul1, size_t mul2, size_t mul3, int *pbOverflowFlag,
+                            const char* pszFile, int nLine )
 {
     if (mul1 != 0)
     {
-        size_t res = mul1 * mul2;
+        const size_t res = mul1 * mul2;
         if (res / mul1 == mul2)
         {
-            size_t res2 = res * mul3;
+            const size_t res2 = res * mul3;
             if (mul3 != 0)
             {
                 if (res2 / mul3 == res)
@@ -750,8 +835,10 @@ static size_t VSICheckMul3( size_t mul1, size_t mul2, size_t mul3, int *pbOverfl
                     if (pbOverflowFlag)
                         *pbOverflowFlag = TRUE;
                     CPLError(CE_Failure, CPLE_OutOfMemory,
-                     "Multiplication overflow : %lu * %lu * %lu",
-                     (unsigned long)mul1, (unsigned long)mul2, (unsigned long)mul3);
+                            "%s: %d: Multiplication overflow : " CPL_FRMT_GUIB " * " CPL_FRMT_GUIB " * " CPL_FRMT_GUIB,
+                            pszFile ? pszFile : "(unknown file)",
+                            nLine,
+                            (GUIntBig)mul1, (GUIntBig)mul2, (GUIntBig)mul3);
                 }
             }
             else
@@ -765,8 +852,10 @@ static size_t VSICheckMul3( size_t mul1, size_t mul2, size_t mul3, int *pbOverfl
             if (pbOverflowFlag)
                 *pbOverflowFlag = TRUE;
             CPLError(CE_Failure, CPLE_OutOfMemory,
-                     "Multiplication overflow : %lu * %lu * %lu",
-                     (unsigned long)mul1, (unsigned long)mul2, (unsigned long)mul3);
+                    "%s: %d: Multiplication overflow : " CPL_FRMT_GUIB " * " CPL_FRMT_GUIB " * " CPL_FRMT_GUIB,
+                    pszFile ? pszFile : "(unknown file)",
+                    nLine,
+                    (GUIntBig)mul1, (GUIntBig)mul2, (GUIntBig)mul3);
         }
     }
     else
@@ -788,27 +877,7 @@ static size_t VSICheckMul3( size_t mul1, size_t mul2, size_t mul3, int *pbOverfl
 */
 void CPL_DLL *VSIMalloc2( size_t nSize1, size_t nSize2 )
 {
-    int bOverflowFlag = FALSE;
-    size_t nSizeToAllocate;
-    void* pReturn;
-
-    nSizeToAllocate = VSICheckMul2( nSize1, nSize2, &bOverflowFlag );
-    if (bOverflowFlag)
-        return NULL;
-
-    if (nSizeToAllocate == 0)
-        return NULL;
-
-    pReturn = VSIMalloc(nSizeToAllocate);
-
-    if( pReturn == NULL )
-    {
-        CPLError( CE_Failure, CPLE_OutOfMemory,
-                  "VSIMalloc2(): Out of memory allocating %lu bytes.\n",
-                  (unsigned long)nSizeToAllocate );
-    }
-
-    return pReturn;
+    return VSIMalloc2Verbose( nSize1, nSize2, NULL, 0);
 }
 
 /**
@@ -820,30 +889,121 @@ void CPL_DLL *VSIMalloc2( size_t nSize1, size_t nSize2 )
 */
 void CPL_DLL *VSIMalloc3( size_t nSize1, size_t nSize2, size_t nSize3 )
 {
-    int bOverflowFlag = FALSE;
-
-    size_t nSizeToAllocate;
-    void* pReturn;
-
-    nSizeToAllocate = VSICheckMul3( nSize1, nSize2, nSize3, &bOverflowFlag );
-    if (bOverflowFlag)
-        return NULL;
-
-    if (nSizeToAllocate == 0)
-        return NULL;
-
-    pReturn = VSIMalloc(nSizeToAllocate);
-
-    if( pReturn == NULL )
-    {
-        CPLError( CE_Failure, CPLE_OutOfMemory,
-                  "VSIMalloc3(): Out of memory allocating %lu bytes.\n",
-                  (unsigned long)nSizeToAllocate );
-    }
-
-    return pReturn;
+    return VSIMalloc3Verbose( nSize1, nSize2, nSize3, NULL, 0);
 }
 
+/************************************************************************/
+/*                          VSIMallocVerbose()                          */
+/************************************************************************/
+
+void *VSIMallocVerbose( size_t nSize, const char* pszFile, int nLine )
+{
+    void* pRet = VSIMalloc(nSize);
+    if( pRet == NULL && nSize != 0 )
+    {
+        CPLError(CE_Failure, CPLE_OutOfMemory,
+                 "%s, %d: cannot allocate " CPL_FRMT_GUIB " bytes",
+                 pszFile ? pszFile : "(unknown file)",
+                 nLine, (GUIntBig)nSize);
+    }
+    return pRet;
+}
+
+/************************************************************************/
+/*                          VSIMalloc2Verbose()                         */
+/************************************************************************/
+
+void *VSIMalloc2Verbose( size_t nSize1, size_t nSize2, const char* pszFile, int nLine )
+{
+    int bOverflowFlag = FALSE;
+    size_t nSizeToAllocate = VSICheckMul2( nSize1, nSize2, &bOverflowFlag, pszFile, nLine );
+    if (bOverflowFlag || nSizeToAllocate == 0)
+        return NULL;
+
+    void* pRet = VSIMalloc(nSizeToAllocate);
+    if( pRet == NULL )
+    {
+        CPLError(CE_Failure, CPLE_OutOfMemory,
+                 "%s, %d: cannot allocate " CPL_FRMT_GUIB " bytes",
+                 pszFile ? pszFile : "(unknown file)",
+                 nLine, (GUIntBig)nSize1 * (GUIntBig)nSize2);
+    }
+    return pRet;
+}
+
+/************************************************************************/
+/*                          VSIMalloc3Verbose()                         */
+/************************************************************************/
+
+void *VSIMalloc3Verbose( size_t nSize1, size_t nSize2, size_t nSize3,
+                         const char* pszFile, int nLine )
+{
+    int bOverflowFlag = FALSE;
+    size_t nSizeToAllocate = VSICheckMul3( nSize1, nSize2, nSize3,
+                                           &bOverflowFlag, pszFile, nLine );
+    if (bOverflowFlag || nSizeToAllocate == 0)
+        return NULL;
+
+    void* pRet = VSIMalloc(nSizeToAllocate);
+    if( pRet == NULL )
+    {
+        CPLError(CE_Failure, CPLE_OutOfMemory,
+                 "%s, %d: cannot allocate " CPL_FRMT_GUIB " bytes",
+                 pszFile ? pszFile : "(unknown file)",
+                 nLine, (GUIntBig)nSize1 * (GUIntBig)nSize2 * (GUIntBig)nSize3);
+    }
+    return pRet;
+}
+/************************************************************************/
+/*                          VSICallocVerbose()                          */
+/************************************************************************/
+
+void *VSICallocVerbose(  size_t nCount, size_t nSize, const char* pszFile, int nLine )
+{
+    void* pRet = VSICalloc(nCount, nSize);
+    if( pRet == NULL && nCount != 0 && nSize != 0 )
+    {
+        CPLError(CE_Failure, CPLE_OutOfMemory,
+                 "%s, %d: cannot allocate " CPL_FRMT_GUIB " bytes",
+                 pszFile ? pszFile : "(unknown file)",
+                 nLine, (GUIntBig)nCount * (GUIntBig)nSize);
+    }
+    return pRet;
+}
+
+/************************************************************************/
+/*                          VSIReallocVerbose()                         */
+/************************************************************************/
+
+void *VSIReallocVerbose( void* pOldPtr, size_t nNewSize, const char* pszFile, int nLine )
+{
+    void* pRet = VSIRealloc(pOldPtr, nNewSize);
+    if( pRet == NULL && nNewSize != 0 )
+    {
+        CPLError(CE_Failure, CPLE_OutOfMemory,
+                 "%s, %d: cannot allocate " CPL_FRMT_GUIB " bytes",
+                 pszFile ? pszFile : "(unknown file)",
+                 nLine, (GUIntBig)(nNewSize));
+    }
+    return pRet;
+}
+
+/************************************************************************/
+/*                          VSIStrdupVerbose()                          */
+/************************************************************************/
+
+char *VSIStrdupVerbose(  const char* pszStr, const char* pszFile, int nLine )
+{
+    char* pRet = VSIStrdup(pszStr);
+    if( pRet == NULL )
+    {
+        CPLError(CE_Failure, CPLE_OutOfMemory,
+                 "%s, %d: cannot allocate " CPL_FRMT_GUIB " bytes",
+                 pszFile ? pszFile : "(unknown file)",
+                 nLine, (GUIntBig)(strlen(pszStr)+1));
+    }
+    return pRet;
+}
 
 /************************************************************************/
 /*                              VSIStat()                               */
@@ -852,23 +1012,22 @@ void CPL_DLL *VSIMalloc3( size_t nSize1, size_t nSize2, size_t nSize3 )
 int VSIStat( const char * pszFilename, VSIStatBuf * pStatBuf )
 
 {
-#if defined(WIN32) && !defined(WIN32CE)
+#if defined(WIN32)
     if( CSLTestBoolean(
             CPLGetConfigOption( "GDAL_FILENAME_IS_UTF8", "YES" ) ) )
     {
-        int nResult;
-        wchar_t *pwszFilename = 
+        wchar_t *pwszFilename =
             CPLRecodeToWChar( pszFilename, CPL_ENC_UTF8, CPL_ENC_UCS2 );
 
-        nResult = _wstat( pwszFilename, (struct _stat *) pStatBuf );
+        int nResult = _wstat( pwszFilename, (struct _stat *) pStatBuf );
 
         CPLFree( pwszFilename );
 
         return nResult;
     }
-    else
-#endif 
-        return( stat( pszFilename, pStatBuf ) );
+
+#endif
+    return stat( pszFilename, pStatBuf );
 }
 
 /************************************************************************/
@@ -879,13 +1038,13 @@ unsigned long VSITime( unsigned long * pnTimeToSet )
 
 {
     time_t tTime;
-        
+
     tTime = time( NULL );
 
     if( pnTimeToSet != NULL )
-        *pnTimeToSet = (unsigned long) tTime;
+        *pnTimeToSet = static_cast<unsigned long>( tTime );
 
-    return (unsigned long) tTime;
+    return static_cast<unsigned long>( tTime );
 }
 
 /************************************************************************/
@@ -897,7 +1056,7 @@ const char *VSICTime( unsigned long nTime )
 {
     time_t tTime = (time_t) nTime;
 
-    return (const char *) ctime( &tTime );
+    return reinterpret_cast<const char *>(ctime( &tTime ));
 }
 
 /************************************************************************/
@@ -910,8 +1069,7 @@ struct tm *VSIGMTime( const time_t *pnTime, struct tm *poBrokenTime )
 #if HAVE_GMTIME_R
     gmtime_r( pnTime, poBrokenTime );
 #else
-    struct tm   *poTime;
-    poTime = gmtime( pnTime );
+    struct tm *poTime = gmtime( pnTime );
     memcpy( poBrokenTime, poTime, sizeof(tm) );
 #endif
 
@@ -928,8 +1086,7 @@ struct tm *VSILocalTime( const time_t *pnTime, struct tm *poBrokenTime )
 #if HAVE_LOCALTIME_R
     localtime_r( pnTime, poBrokenTime );
 #else
-    struct tm   *poTime;
-    poTime = localtime( pnTime );
+    struct tm *poTime = localtime( pnTime );
     memcpy( poBrokenTime, poTime, sizeof(tm) );
 #endif
 
@@ -945,3 +1102,92 @@ char *VSIStrerror( int nErrno )
 {
     return strerror( nErrno );
 }
+
+
+/************************************************************************/
+/*                        CPLGetPhysicalRAM()                           */
+/************************************************************************/
+
+#if HAVE_SC_PHYS_PAGES
+
+/** Return the total physical RAM in bytes.
+ *
+ * @return the total physical RAM in bytes (or 0 in case of failure).
+ * @since GDAL 2.0
+ */
+GIntBig CPLGetPhysicalRAM(void)
+{
+    return static_cast<GIntBig>(sysconf(_SC_PHYS_PAGES))
+        * sysconf(_SC_PAGESIZE);
+}
+
+#elif defined(__MACH__) && defined(__APPLE__)
+
+#include <sys/types.h>
+#include <sys/sysctl.h>
+
+GIntBig CPLGetPhysicalRAM(void)
+{
+    GIntBig nPhysMem = 0;
+
+    int mib[2] = { CTL_HW, HW_MEMSIZE };
+    size_t nLengthRes = sizeof(nPhysMem);
+    sysctl(mib, CPL_ARRAYSIZE(mib), &nPhysMem, &nLengthRes, NULL, 0);
+
+    return nPhysMem;
+}
+
+#elif defined(WIN32)
+
+/* GlobalMemoryStatusEx requires _WIN32_WINNT >= 0x0500 */
+#ifndef _WIN32_WINNT
+#define _WIN32_WINNT 0x0500
+#endif
+#include <windows.h>
+
+GIntBig CPLGetPhysicalRAM(void)
+{
+    MEMORYSTATUSEX statex;
+    statex.ullTotalPhys = 0;
+    statex.dwLength = sizeof (statex);
+    GlobalMemoryStatusEx (&statex);
+    return static_cast<GIntBig>( statex.ullTotalPhys );
+}
+
+#else
+
+GIntBig CPLGetPhysicalRAM(void)
+{
+    static bool bOnce = false;
+    if( !bOnce )
+    {
+        bOnce = true;
+        CPLDebug("PORT", "No implementation for CPLGetPhysicalRAM()");
+    }
+    return 0;
+}
+#endif
+
+/************************************************************************/
+/*                       CPLGetUsablePhysicalRAM()                      */
+/************************************************************************/
+
+/** Return the total physical RAM, usable by a process, in bytes.
+ *
+ * This is the same as CPLGetPhysicalRAM() except it will limit to 2 GB
+ * for 32 bit processes.
+ *
+ * Note: This memory may already be partly used by other processes.
+ *
+ * @return the total physical RAM, usable by a process, in bytes (or 0 in case of failure).
+ * @since GDAL 2.0
+ */
+GIntBig  CPLGetUsablePhysicalRAM(void)
+{
+    GIntBig nRAM = CPLGetPhysicalRAM();
+#if SIZEOF_VOIDP == 4
+    if( nRAM > INT_MAX )
+        nRAM = INT_MAX;
+#endif
+    return nRAM;
+}
