@@ -1,12 +1,12 @@
 /******************************************************************************
- * $Id: cpl_xml_validate.cpp 25229 2012-11-16 19:06:58Z rouault $
+ * $Id: cpl_xml_validate.cpp 32883 2016-01-09 18:24:40Z rouault $
  *
  * Project:  CPL - Common Portability Library
  * Purpose:  Implement XML validation against XSD schema
  * Author:   Even Rouault, even.rouault at mines-paris.org
  *
  ******************************************************************************
- * Copyright (c) 2012, Even Rouault
+ * Copyright (c) 2012-2014, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -29,7 +29,7 @@
 
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: cpl_xml_validate.cpp 25229 2012-11-16 19:06:58Z rouault $");
+CPL_CVSID("$Id: cpl_xml_validate.cpp 32883 2016-01-09 18:24:40Z rouault $");
 
 #ifdef HAVE_LIBXML2
 #include <libxml/xmlversion.h>
@@ -78,7 +78,7 @@ static void CPLFixPath(char* pszPath)
             pszPath[i] = '/';
     }
 
-    while(TRUE)
+    while(true)
     {
         char* pszSlashDotDot = strstr(pszPath, "/../");
         if (pszSlashDotDot == NULL || pszSlashDotDot == pszPath)
@@ -101,7 +101,8 @@ static int bHasLibXMLBug = -1;
 /*                  CPLHasLibXMLBugWarningCallback()                    */
 /************************************************************************/
 
-static void CPLHasLibXMLBugWarningCallback (void * ctx, const char * msg, ...)
+static void CPLHasLibXMLBugWarningCallback (void * /*ctx*/,
+                                            const char* /*msg*/, ...)
 {
 }
 
@@ -109,10 +110,10 @@ static void CPLHasLibXMLBugWarningCallback (void * ctx, const char * msg, ...)
 /*                          CPLHasLibXMLBug()                           */
 /************************************************************************/
 
-static int CPLHasLibXMLBug()
+static bool CPLHasLibXMLBug()
 {
     if (bHasLibXMLBug >= 0)
-        return bHasLibXMLBug;
+        return CPL_TO_BOOL(bHasLibXMLBug);
 
     static const char szLibXMLBugTester[] =
     "<schema targetNamespace=\"http://foo\" xmlns:foo=\"http://foo\" xmlns=\"http://www.w3.org/2001/XMLSchema\">"
@@ -158,7 +159,7 @@ static int CPLHasLibXMLBug()
                  "Will try to workaround for GML schemas.");
     }
 
-    return bHasLibXMLBug;
+    return CPL_TO_BOOL(bHasLibXMLBug);
 }
 
 #endif
@@ -197,7 +198,7 @@ static CPLXMLNode* CPLExtractSubSchema(CPLXMLNode* psSubXML, CPLXMLNode* psMainS
         {
             /* Add xmlns: from subschema to main schema if missing */
             if (psNext->eType == CXT_Attribute &&
-                strncmp(psNext->pszValue, "xmlns:", 6) == 0 &&
+                STARTS_WITH(psNext->pszValue, "xmlns:") &&
                 CPLGetXMLValue(psMainSchema, psNext->pszValue, NULL) == NULL)
             {
                 CPLXMLNode* psAttr = CPLCreateXMLNode(NULL, CXT_Attribute, psNext->pszValue);
@@ -228,7 +229,7 @@ static CPLXMLNode* CPLExtractSubSchema(CPLXMLNode* psSubXML, CPLXMLNode* psMainS
 /************************************************************************/
 
 /* Return TRUE if the current node must be destroyed */
-static int CPLWorkaroundLibXMLBug(CPLXMLNode* psIter)
+static bool CPLWorkaroundLibXMLBug(CPLXMLNode* psIter)
 {
     if (psIter->eType == CXT_Element &&
         strcmp(psIter->pszValue, "element") == 0 &&
@@ -276,7 +277,7 @@ static int CPLWorkaroundLibXMLBug(CPLXMLNode* psIter)
               strcmp(CPLGetXMLValue(psIter, "name", ""), "CategoryExtentType") == 0))
     {
         /* Destroy this element */
-        return TRUE;
+        return true;
     }
 
     /* For GML 3.2.1 */
@@ -324,7 +325,7 @@ static int CPLWorkaroundLibXMLBug(CPLXMLNode* psIter)
         psIter->psChild->psNext = psComplexType;
     }
 
-    return FALSE;
+    return false;
 }
 #endif
 
@@ -336,11 +337,6 @@ static
 CPLXMLNode* CPLLoadSchemaStrInternal(CPLHashSet* hSetSchemas,
                                      const char* pszFile)
 {
-    CPLXMLNode* psXML;
-    CPLXMLNode* psSchema;
-    CPLXMLNode* psPrev;
-    CPLXMLNode* psIter;
-
     if (CPLHashSetLookup(hSetSchemas, pszFile))
         return NULL;
 
@@ -348,7 +344,7 @@ CPLXMLNode* CPLLoadSchemaStrInternal(CPLHashSet* hSetSchemas,
 
     CPLDebug("CPL", "Parsing %s", pszFile);
 
-    psXML = CPLParseXMLFile(pszFile);
+    CPLXMLNode* psXML = CPLParseXMLFile(pszFile);
     if (psXML == NULL)
     {
         CPLError(CE_Failure, CPLE_AppDefined,
@@ -356,7 +352,7 @@ CPLXMLNode* CPLLoadSchemaStrInternal(CPLHashSet* hSetSchemas,
         return NULL;
     }
 
-    psSchema = CPLGetXMLNode(psXML, "=schema");
+    CPLXMLNode* psSchema = CPLGetXMLNode(psXML, "=schema");
     if (psSchema == NULL)
         psSchema = CPLGetXMLNode(psXML, "=xs:schema");
     if (psSchema == NULL)
@@ -369,11 +365,11 @@ CPLXMLNode* CPLLoadSchemaStrInternal(CPLHashSet* hSetSchemas,
         return NULL;
     }
 
-    psPrev = NULL;
-    psIter = psSchema->psChild;
+    CPLXMLNode* psPrev = NULL;
+    CPLXMLNode* psIter = psSchema->psChild;
     while(psIter)
     {
-        int bDestroyCurrentNode = FALSE;
+        bool bDestroyCurrentNode = false;
 
 #ifdef HAS_VALIDATION_BUG
         if (bHasLibXMLBug)
@@ -444,7 +440,7 @@ CPLXMLNode* CPLLoadSchemaStrInternal(CPLHashSet* hSetSchemas,
             {
                 /* We have already included that file, */
                 /* so just remove the <include/> node */
-                bDestroyCurrentNode = TRUE;
+                bDestroyCurrentNode = true;
             }
         }
 
@@ -460,8 +456,8 @@ CPLXMLNode* CPLLoadSchemaStrInternal(CPLHashSet* hSetSchemas,
                 if (psIter2->eType == CXT_Attribute &&
                     strcmp(psIter2->pszValue, "schemaLocation") == 0 &&
                     psIter2->psChild != NULL &&
-                    strncmp(psIter2->psChild->pszValue, "http://", 7) != 0 &&
-                    strncmp(psIter2->psChild->pszValue, "ftp://", 6) != 0 &&
+                    !STARTS_WITH(psIter2->psChild->pszValue, "http://") &&
+                    !STARTS_WITH(psIter2->psChild->pszValue, "ftp://") &&
                     /* If the top file is our warping file, don't alter the path of the import */
                     strstr(pszFile, "/vsimem/CPLValidateXML_") == NULL )
                 {
@@ -504,11 +500,7 @@ CPLXMLNode* CPLLoadSchemaStrInternal(CPLHashSet* hSetSchemas,
 static
 void CPLMoveImportAtBeginning(CPLXMLNode* psXML)
 {
-    CPLXMLNode* psIter;
-    CPLXMLNode* psPrev;
-    CPLXMLNode* psSchema;
-
-    psSchema = CPLGetXMLNode(psXML, "=schema");
+    CPLXMLNode* psSchema = CPLGetXMLNode(psXML, "=schema");
     if (psSchema == NULL)
         psSchema = CPLGetXMLNode(psXML, "=xs:schema");
     if (psSchema == NULL)
@@ -516,8 +508,8 @@ void CPLMoveImportAtBeginning(CPLXMLNode* psXML)
     if (psSchema == NULL)
         return;
 
-    psPrev = NULL;
-    psIter = psSchema->psChild;
+    CPLXMLNode* psPrev = NULL;
+    CPLXMLNode* psIter = psSchema->psChild;
     while(psIter)
     {
         if (psPrev != NULL && psIter->eType == CXT_Element &&
@@ -550,8 +542,6 @@ void CPLMoveImportAtBeginning(CPLXMLNode* psXML)
 static
 char* CPLLoadSchemaStr(const char* pszXSDFilename)
 {
-    char* pszStr = NULL;
-
 #ifdef HAS_VALIDATION_BUG
     CPLHasLibXMLBug();
 #endif
@@ -560,6 +550,8 @@ char* CPLLoadSchemaStr(const char* pszXSDFilename)
         CPLHashSetNew(CPLHashSetHashStr, CPLHashSetEqualStr, CPLFree);
     CPLXMLNode* psSchema =
         CPLLoadSchemaStrInternal(hSetSchemas, pszXSDFilename);
+
+    char* pszStr = NULL;
     if (psSchema)
     {
         CPLMoveImportAtBeginning(psSchema);
@@ -585,10 +577,9 @@ static void CPLLibXMLInputStreamCPLFree(xmlChar* pszBuffer)
 
 static CPLString CPLFindLocalXSD(const char* pszXSDFilename)
 {
-    const char *pszSchemasOpenGIS;
     CPLString osTmp;
-
-    pszSchemasOpenGIS = CPLGetConfigOption("GDAL_OPENGIS_SCHEMAS", NULL);
+    const char *pszSchemasOpenGIS
+        = CPLGetConfigOption("GDAL_OPENGIS_SCHEMAS", NULL);
     if (pszSchemasOpenGIS != NULL)
     {
         int nLen = (int)strlen(pszSchemasOpenGIS);
@@ -681,7 +672,7 @@ xmlParserInputPtr CPLExternalEntityLoader (const char * URL,
 {
     //CPLDebug("CPL", "CPLExternalEntityLoader(%s)", URL);
     CPLString osURL;
-    
+
     /* Use libxml2 catalog mechanism to resolve the URL to something else */
     xmlChar* pszResolved = xmlCatalogResolveSystem((const xmlChar*)URL);
     if (pszResolved == NULL)
@@ -695,7 +686,7 @@ xmlParserInputPtr CPLExternalEntityLoader (const char * URL,
         pszResolved = NULL;
     }
 
-    if (strncmp(URL, "http://", 7) == 0)
+    if (STARTS_WITH(URL, "http://"))
     {
         /* Make sure to use http://schemas.opengis.net/ */
         /* when gml/2 or gml/3 is detected */
@@ -736,21 +727,19 @@ xmlParserInputPtr CPLExternalEntityLoader (const char * URL,
                 return xmlNewStringInputStream(context, (const xmlChar*) szXLINK_XSD);
             }
         }
-        else if (strncmp(URL, "http://schemas.opengis.net/",
-                         strlen("http://schemas.opengis.net/")) != 0)
-        {
+        else if (!STARTS_WITH(URL, "http://schemas.opengis.net/"))        {
             CPLDebug("CPL", "Loading %s", URL);
             return pfnLibXMLOldExtranerEntityLoader(URL, ID, context);
         }
     }
-    else if (strncmp(URL, "ftp://", 6) == 0)
+    else if (STARTS_WITH(URL, "ftp://"))
     {
         return pfnLibXMLOldExtranerEntityLoader(URL, ID, context);
     }
-    else if (strncmp(URL, "file://", 7) == 0)
+    else if (STARTS_WITH(URL, "file://"))
     {
         /* Parse file:// URI so as to be able to open them with VSI*L API */
-        if (strncmp(URL, "file://localhost/", 17) == 0)
+        if (STARTS_WITH(URL, "file://localhost/"))
             URL += 16;
         else
             URL += 7;
@@ -763,21 +752,15 @@ xmlParserInputPtr CPLExternalEntityLoader (const char * URL,
     }
 
     CPLString osModURL;
-    if (strncmp(URL, "/vsizip/vsicurl/http%3A//",
-                strlen("/vsizip/vsicurl/http%3A//")) == 0)
-    {
+    if (STARTS_WITH(URL, "/vsizip/vsicurl/http%3A//"))    {
         osModURL = "/vsizip/vsicurl/http://";
         osModURL += URL + strlen("/vsizip/vsicurl/http%3A//");
     }
-    else if (strncmp(URL, "/vsicurl/http%3A//",
-                     strlen("/vsicurl/http%3A//")) == 0)
-    {
+    else if (STARTS_WITH(URL, "/vsicurl/http%3A//"))    {
         osModURL = "vsicurl/http://";
         osModURL += URL + strlen("/vsicurl/http%3A//");
     }
-    else if (strncmp(URL, "http://schemas.opengis.net/",
-                     strlen("http://schemas.opengis.net/")) == 0)
-    {
+    else if (STARTS_WITH(URL, "http://schemas.opengis.net/"))    {
         const char *pszAfterOpenGIS =
                 URL + strlen("http://schemas.opengis.net/");
 
@@ -840,10 +823,9 @@ xmlParserInputPtr CPLExternalEntityLoader (const char * URL,
 static void CPLLibXMLWarningErrorCallback (void * ctx, const char * msg, ...)
 {
     va_list varg;
-    char * pszStr;
-
     va_start(varg, msg);
-    pszStr = (char *)va_arg( varg, char *);
+
+    char *pszStr = (char *)va_arg( varg, char *);
 
     if (strstr(pszStr, "since this namespace was already imported") == NULL)
     {
@@ -853,8 +835,16 @@ static void CPLLibXMLWarningErrorCallback (void * ctx, const char * msg, ...)
         int nLen = (int)strlen(pszStrDup);
         if (nLen > 0 && pszStrDup[nLen-1] == '\n')
             pszStrDup[nLen-1] = '\0';
-        CPLError(CE_Failure, CPLE_AppDefined, "libXML: %s:%d: %s",
-                 pszFilename, pErrorPtr ? pErrorPtr->line : 0, pszStrDup);
+        if( pszFilename != NULL && pszFilename[0] != '<' )
+        {
+            CPLError(CE_Failure, CPLE_AppDefined, "libXML: %s:%d: %s",
+                     pszFilename, pErrorPtr ? pErrorPtr->line : 0, pszStrDup);
+        }
+        else
+        {
+            CPLError(CE_Failure, CPLE_AppDefined, "libXML: %d: %s",
+                     pErrorPtr ? pErrorPtr->line : 0, pszStrDup);
+        }
         CPLFree(pszStrDup);
     }
 
@@ -872,24 +862,37 @@ char* CPLLoadContentFromFile(const char* pszFilename)
     if (fp == NULL)
         return NULL;
     vsi_l_offset nSize;
-    VSIFSeekL(fp, 0, SEEK_END);
+    if( VSIFSeekL(fp, 0, SEEK_END) != 0 )
+    {
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
+        return NULL;
+    }
     nSize = VSIFTellL(fp);
-    VSIFSeekL(fp, 0, SEEK_SET);
+    if( VSIFSeekL(fp, 0, SEEK_SET) != 0 )
+    {
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
+        return NULL;
+    }
     if ((vsi_l_offset)(int)nSize != nSize ||
         nSize > INT_MAX - 1 )
     {
-        VSIFCloseL(fp);
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
         return NULL;
     }
-    char* pszBuffer = (char*)VSIMalloc(nSize + 1);
+    char* pszBuffer = (char*)VSIMalloc((size_t)nSize + 1);
     if (pszBuffer == NULL)
     {
-        VSIFCloseL(fp);
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
+        return NULL;
+    }
+    if( (size_t)VSIFReadL(pszBuffer, 1, (size_t)nSize, fp) != (size_t)nSize )
+    {
+        VSIFree(pszBuffer);
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
         return NULL;
     }
-    VSIFReadL(pszBuffer, 1, nSize, fp);
     pszBuffer[nSize] = '\0';
-    VSIFCloseL(fp);
+    CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
     return pszBuffer;
 }
 
@@ -923,7 +926,7 @@ CPLXMLSchemaPtr CPLLoadXMLSchema(const char* pszXSDFilename)
     xmlSetExternalEntityLoader(CPLExternalEntityLoader);
 
     xmlSchemaParserCtxtPtr pSchemaParserCtxt =
-                            xmlSchemaNewMemParserCtxt(pszStr, strlen(pszStr));
+                            xmlSchemaNewMemParserCtxt(pszStr, static_cast<int>(strlen(pszStr)));
 
     xmlSchemaSetParserErrors(pSchemaParserCtxt,
                              CPLLibXMLWarningErrorCallback,
@@ -976,20 +979,29 @@ void CPLFreeXMLSchema(CPLXMLSchemaPtr pSchema)
 
 int CPLValidateXML(const char* pszXMLFilename,
                    const char* pszXSDFilename,
-                   char** papszOptions)
+                   CPL_UNUSED char** papszOptions)
 {
-    VSILFILE* fpXML = VSIFOpenL(pszXMLFilename, "rb");
-    if (fpXML == NULL)
-    {
-        CPLError(CE_Failure, CPLE_OpenFailed,
-                 "Cannot open %s", pszXMLFilename);
-        return FALSE;
-    }
-    CPLString osTmpXSDFilename;
     char szHeader[2048];
-    int nRead = (int)VSIFReadL(szHeader, 1, sizeof(szHeader)-1, fpXML);
-    szHeader[nRead] = '\0';
-    VSIFCloseL(fpXML);
+    CPLString osTmpXSDFilename;
+
+    if( pszXMLFilename[0] == '<' )
+    {
+        strncpy(szHeader, pszXMLFilename, sizeof(szHeader));
+        szHeader[sizeof(szHeader)-1] = '\0';
+    }
+    else
+    {
+        VSILFILE* fpXML = VSIFOpenL(pszXMLFilename, "rb");
+        if (fpXML == NULL)
+        {
+            CPLError(CE_Failure, CPLE_OpenFailed,
+                    "Cannot open %s", pszXMLFilename);
+            return FALSE;
+        }
+        int nRead = (int)VSIFReadL(szHeader, 1, sizeof(szHeader)-1, fpXML);
+        szHeader[nRead] = '\0';
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fpXML));
+    }
 
     /* Workaround following bug : "element FeatureCollection: Schemas validity error : Element '{http://www.opengis.net/wfs}FeatureCollection': No matching global declaration available for the validation root" */
     /* We create a wrapping XSD that imports the WFS .xsd (and possibly the GML .xsd too) and the application schema */
@@ -1023,7 +1035,7 @@ int CPLValidateXML(const char* pszXMLFilename,
         }
         int nRead = (int)VSIFReadL(szHeader, 1, sizeof(szHeader)-1, fpXSD);
         szHeader[nRead] = '\0';
-        VSIFCloseL(fpXSD);
+        CPL_IGNORE_RET_VAL(VSIFCloseL(fpXSD));
 
         if (strstr(szHeader, "gml/3.1.1") != NULL &&
             strstr(szHeader, "gml/3.1.1/base/gml.xsd") == NULL)
@@ -1036,13 +1048,13 @@ int CPLValidateXML(const char* pszXMLFilename,
             osTmpXSDFilename = CPLSPrintf("/vsimem/CPLValidateXML_%p_%p.xsd", pszXMLFilename, pszXSDFilename);
             char* pszEscapedXSDFilename = CPLEscapeString(pszXSDFilename, -1, CPLES_XML);
             VSILFILE* fpMEM = VSIFOpenL(osTmpXSDFilename, "wb");
-            VSIFPrintfL(fpMEM, "<xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\">\n");
-            VSIFPrintfL(fpMEM, "   <xs:import namespace=\"%s\" schemaLocation=\"%s\"/>\n", pszWFSSchemaNamespace, pszWFSSchemaLocation);
-            VSIFPrintfL(fpMEM, "   <xs:import namespace=\"ignored\" schemaLocation=\"%s\"/>\n", pszEscapedXSDFilename);
+            CPL_IGNORE_RET_VAL(VSIFPrintfL(fpMEM, "<xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\">\n"));
+            CPL_IGNORE_RET_VAL(VSIFPrintfL(fpMEM, "   <xs:import namespace=\"%s\" schemaLocation=\"%s\"/>\n", pszWFSSchemaNamespace, pszWFSSchemaLocation));
+            CPL_IGNORE_RET_VAL(VSIFPrintfL(fpMEM, "   <xs:import namespace=\"ignored\" schemaLocation=\"%s\"/>\n", pszEscapedXSDFilename));
             if (pszGMLSchemaLocation)
-                VSIFPrintfL(fpMEM, "   <xs:import namespace=\"http://www.opengis.net/gml\" schemaLocation=\"%s\"/>\n", pszGMLSchemaLocation);
-            VSIFPrintfL(fpMEM, "</xs:schema>\n");
-            VSIFCloseL(fpMEM);
+                CPL_IGNORE_RET_VAL(VSIFPrintfL(fpMEM, "   <xs:import namespace=\"http://www.opengis.net/gml\" schemaLocation=\"%s\"/>\n", pszGMLSchemaLocation));
+            CPL_IGNORE_RET_VAL(VSIFPrintfL(fpMEM, "</xs:schema>\n"));
+            CPL_IGNORE_RET_VAL(VSIFCloseL(fpMEM));
             CPLFree(pszEscapedXSDFilename);
         }
     }
@@ -1068,8 +1080,17 @@ int CPLValidateXML(const char* pszXMLFilename,
                             CPLLibXMLWarningErrorCallback,
                             (void*) pszXMLFilename);
 
-    int bValid = FALSE;
-    if (strncmp(pszXMLFilename, "/vsi", 4) != 0)
+    bool bValid = false;
+    if( pszXMLFilename[0] == '<' )
+    {
+        xmlDocPtr pDoc = xmlParseDoc((const xmlChar *)pszXMLFilename);
+        if (pDoc != NULL)
+        {
+            bValid = xmlSchemaValidateDoc(pSchemaValidCtxt, pDoc) == 0;
+        }
+        xmlFreeDoc(pDoc);
+    }
+    else if (!STARTS_WITH(pszXMLFilename, "/vsi"))
     {
         bValid =
             xmlSchemaValidateFile(pSchemaValidCtxt, pszXMLFilename, 0) == 0;
@@ -1100,9 +1121,9 @@ int CPLValidateXML(const char* pszXMLFilename,
 /*                          CPLValidateXML()                            */
 /************************************************************************/
 
-int CPLValidateXML(const char* pszXMLFilename,
-                   const char* pszXSDFilename,
-                   char** papszOptions)
+int CPLValidateXML(CPL_UNUSED const char* pszXMLFilename,
+                   CPL_UNUSED const char* pszXSDFilename,
+                   CPL_UNUSED char** papszOptions)
 {
     CPLError(CE_Failure, CPLE_NotSupported,
              "%s not implemented due to missing libxml2 support",
