@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: cplstringlist.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: cplstringlist.cpp 33789 2016-03-26 01:31:36Z goatbar $
  *
- * Project:  GDAL 
+ * Project:  GDAL
  * Purpose:  CPLStringList implementation.
  * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
@@ -31,17 +31,19 @@
 #include "cpl_string.h"
 #include <string>
 
-CPL_CVSID("$Id: cplstringlist.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: cplstringlist.cpp 33789 2016-03-26 01:31:36Z goatbar $");
 
 /************************************************************************/
 /*                           CPLStringList()                            */
 /************************************************************************/
 
-CPLStringList::CPLStringList()
-
-{
-    Initialize();
-}
+CPLStringList::CPLStringList() :
+    papszList(NULL),
+    nCount(0),
+    nAllocation(0),
+    bOwnList(false),
+    bIsSorted(false)
+{}
 
 /************************************************************************/
 /*                           CPLStringList()                            */
@@ -96,7 +98,7 @@ CPLStringList &CPLStringList::operator=(const CPLStringList& oOther)
         MakeOurOwnCopy();
         bIsSorted = oOther.bIsSorted;
     }
-    
+
     return *this;
 }
 
@@ -110,8 +112,8 @@ void CPLStringList::Initialize()
     papszList = NULL;
     nCount = 0;
     nAllocation = 0;
-    bOwnList = FALSE;
-    bIsSorted = FALSE;
+    bOwnList = false;
+    bIsSorted = false;
 }
 
 /************************************************************************/
@@ -129,7 +131,7 @@ CPLStringList::~CPLStringList()
 /************************************************************************/
 
 /**
- * Clear the string list. 
+ * Clear the string list.
  */
 CPLStringList &CPLStringList::Clear()
 
@@ -143,7 +145,7 @@ CPLStringList &CPLStringList::Clear()
         nAllocation = 0;
         nCount = 0;
     }
-    
+
     return *this;
 }
 
@@ -152,9 +154,9 @@ CPLStringList &CPLStringList::Clear()
 /************************************************************************/
 
 /**
- * Assign a list of strings. 
+ * Assign a list of strings.
+ *
  *
- * 
  * @param papszListIn the NULL terminated list of strings to consume.
  * @param bTakeOwnership TRUE if the CPLStringList should take ownership
  * of the list of strings which implies responsibility to free them.
@@ -168,14 +170,14 @@ CPLStringList &CPLStringList::Assign( char **papszListIn, int bTakeOwnership )
     Clear();
 
     papszList = papszListIn;
-    bOwnList = bTakeOwnership;
+    bOwnList = CPL_TO_BOOL(bTakeOwnership);
 
     if( papszList == NULL || *papszList == NULL )
         nCount = 0;
     else
         nCount = -1;      // unknown
 
-    nAllocation = 0;  
+    nAllocation = 0;
     bIsSorted = FALSE;
 
     return *this;
@@ -204,7 +206,7 @@ int CPLStringList::Count() const
             nAllocation = MAX(nCount+1,nAllocation);
         }
     }
-    
+
     return nCount;
 }
 
@@ -225,7 +227,7 @@ void CPLStringList::MakeOurOwnCopy()
         return;
 
     Count();
-    bOwnList = TRUE;
+    bOwnList = true;
     papszList = CSLDuplicate( papszList );
     nAllocation = nCount+1;
 }
@@ -247,14 +249,18 @@ void CPLStringList::EnsureAllocation( int nMaxList )
     if( nAllocation <= nMaxList )
     {
         nAllocation = MAX(nAllocation*2 + 20,nMaxList+1);
-		if( papszList == NULL )
-		{
-			papszList = (char **) CPLCalloc(nAllocation,sizeof(char*));
-			bOwnList = TRUE;
-			nCount = 0;
-		}
-		else
-			papszList = (char **) CPLRealloc(papszList, nAllocation*sizeof(char*));
+        if( papszList == NULL )
+        {
+            papszList = static_cast<char **>(
+                CPLCalloc(nAllocation, sizeof(char*)) );
+            bOwnList = true;
+            nCount = 0;
+        }
+        else
+        {
+            papszList = static_cast<char **>(
+                CPLRealloc(papszList, nAllocation*sizeof(char*)) );
+        }
     }
 }
 
@@ -268,7 +274,7 @@ void CPLStringList::EnsureAllocation( int nMaxList )
  * This method is similar to AddString(), but ownership of the
  * pszNewString is transferred to the CPLStringList class.
  *
- * @param pszNewString the string to add to the list. 
+ * @param pszNewString the string to add to the list.
  */
 
 CPLStringList &CPLStringList::AddStringDirectly( char *pszNewString )
@@ -282,7 +288,7 @@ CPLStringList &CPLStringList::AddStringDirectly( char *pszNewString )
     papszList[nCount++] = pszNewString;
     papszList[nCount] = NULL;
 
-    bIsSorted = FALSE;
+    bIsSorted = false;
 
     return *this;
 }
@@ -296,7 +302,7 @@ CPLStringList &CPLStringList::AddStringDirectly( char *pszNewString )
  *
  * A copy of the passed in string is made and inserted in the list.
  *
- * @param pszNewString the string to add to the list. 
+ * @param pszNewString the string to add to the list.
  */
 
 CPLStringList &CPLStringList::AddString( const char *pszNewString )
@@ -310,7 +316,7 @@ CPLStringList &CPLStringList::AddString( const char *pszNewString )
 /************************************************************************/
 
 /**
- * A a name=value entry to the list.
+ * Add a name=value entry to the list.
  *
  * A key=value string is prepared and appended to the list.  There is no
  * check for other values for the same key in the list.
@@ -319,7 +325,7 @@ CPLStringList &CPLStringList::AddString( const char *pszNewString )
  * @param pszValue the key value to add.
  */
 
-CPLStringList &CPLStringList::AddNameValue( const char  *pszKey, 
+CPLStringList &CPLStringList::AddNameValue( const char  *pszKey,
                                             const char *pszValue )
 
 {
@@ -331,9 +337,9 @@ CPLStringList &CPLStringList::AddNameValue( const char  *pszKey,
 /* -------------------------------------------------------------------- */
 /*      Format the line.                                                */
 /* -------------------------------------------------------------------- */
-    char *pszLine;
-    pszLine = (char *) CPLMalloc(strlen(pszKey)+strlen(pszValue)+2);
-    sprintf( pszLine, "%s=%s", pszKey, pszValue );
+    const size_t nLen = strlen(pszKey)+strlen(pszValue)+2;
+    char *pszLine = static_cast<char *>( CPLMalloc(nLen) );
+    snprintf( pszLine, nLen, "%s=%s", pszKey, pszValue );
 
 /* -------------------------------------------------------------------- */
 /*      If we don't need to keep the sort order things are pretty       */
@@ -346,9 +352,9 @@ CPLStringList &CPLStringList::AddNameValue( const char  *pszKey,
 /*      Find the proper insertion point.                                */
 /* -------------------------------------------------------------------- */
     CPLAssert( IsSorted() );
-    int iKey = FindSortedInsertionPoint( pszLine );
+    const int iKey = FindSortedInsertionPoint( pszLine );
     InsertStringDirectly( iKey, pszLine );
-    bIsSorted = TRUE; // we have actually preserved sort order.
+    bIsSorted = true;  // We have actually preserved sort order.
 
     return *this;
 }
@@ -358,17 +364,17 @@ CPLStringList &CPLStringList::AddNameValue( const char  *pszKey,
 /************************************************************************/
 
 /**
- * Set name=value entry in the list. 
+ * Set name=value entry in the list.
  *
  * Similar to AddNameValue(), except if there is already a value for
  * the key in the list it is replaced instead of adding a new entry to
  * the list.  If pszValue is NULL any existing key entry is removed.
- * 
+ *
  * @param pszKey the key name to add.
  * @param pszValue the key value to add.
  */
 
-CPLStringList &CPLStringList::SetNameValue( const char *pszKey, 
+CPLStringList &CPLStringList::SetNameValue( const char *pszKey,
                                             const char *pszValue )
 
 {
@@ -385,19 +391,19 @@ CPLStringList &CPLStringList::SetNameValue( const char *pszKey,
     {
 
         // shift everything down by one.
-        do 
+        do
         {
             papszList[iKey] = papszList[iKey+1];
-        } 
+        }
         while( papszList[iKey++] != NULL );
 
         nCount--;
     }
     else
     {
-        char *pszLine;
-        pszLine = (char *) CPLMalloc(strlen(pszKey)+strlen(pszValue)+2);
-        sprintf( pszLine, "%s=%s", pszKey, pszValue );
+        const size_t nLen = strlen(pszKey)+strlen(pszValue)+2;
+        char *pszLine = static_cast<char *>( CPLMalloc(nLen) );
+        snprintf( pszLine, nLen, "%s=%s", pszKey, pszValue );
 
         papszList[iKey] = pszLine;
     }
@@ -417,18 +423,18 @@ CPLStringList &CPLStringList::SetNameValue( const char *pszKey,
  * returned.
  *
  * @param i the index of the list item to return.
- * @return selected entry in the list.  
+ * @return selected entry in the list.
  */
 char *CPLStringList::operator[]( int i )
 
 {
     if( nCount == -1 )
         Count();
-    
+
     if( i < 0 || i >= nCount )
         return NULL;
-    else
-        return papszList[i];
+
+    return papszList[i];
 }
 
 const char *CPLStringList::operator[]( int i ) const
@@ -436,11 +442,11 @@ const char *CPLStringList::operator[]( int i ) const
 {
     if( nCount == -1 )
         Count();
-    
+
     if( i < 0 || i >= nCount )
         return NULL;
-    else
-        return papszList[i];
+
+    return papszList[i];
 }
 
 /************************************************************************/
@@ -450,24 +456,57 @@ const char *CPLStringList::operator[]( int i ) const
 /**
  * Seize ownership of underlying string array.
  *
- * This method is simmilar to List(), except that the returned list is
- * now owned by the caller and the CPLStringList is emptied.  
+ * This method is similar to List(), except that the returned list is
+ * now owned by the caller and the CPLStringList is emptied.
  *
- * @return the C style string list. 
+ * @return the C style string list.
  */
 char **CPLStringList::StealList()
 
 {
     char **papszRetList = papszList;
 
-    bOwnList = FALSE;
+    bOwnList = false;
     papszList = NULL;
     nCount = 0;
     nAllocation = 0;
-    
+
     return papszRetList;
 }
 
+
+static int CPLCompareKeyValueString(const char* pszKVa, const char* pszKVb)
+{
+    const char* pszItera = pszKVa;
+    const char* pszIterb = pszKVb;
+    while( true )
+    {
+        char cha = *pszItera;
+        char chb = *pszIterb;
+        if( cha == '=' || cha == '\0' )
+        {
+            if( chb == '=' || chb == '\0' )
+                return 0;
+            else
+                return -1;
+        }
+        if( chb == '=' || chb == '\0' )
+        {
+            return 1;
+        }
+        if( cha >= 'a' && cha <= 'z' )
+            cha -= ('a' - 'A');
+        if( chb >= 'a' && chb <= 'z' )
+            chb -= ('a' - 'A');
+        if( cha < chb )
+            return -1;
+        else if( cha > chb )
+            return 1;
+        pszItera ++;
+        pszIterb ++;
+    }
+}
+
 /************************************************************************/
 /*                            llCompareStr()                            */
 /*                                                                      */
@@ -476,7 +515,9 @@ char **CPLStringList::StealList()
 /************************************************************************/
 static int llCompareStr(const void *a, const void *b)
 {
-	return STRCASECMP((*(const char **)a),(*(const char **)b));
+    return CPLCompareKeyValueString(
+        *static_cast<const char **>(const_cast<void *>(a)),
+        *static_cast<const char **>(const_cast<void *>(b)) );
 }
 
 /************************************************************************/
@@ -487,7 +528,7 @@ static int llCompareStr(const void *a, const void *b)
  * Sort the entries in the list and mark list sorted.
  *
  * Note that once put into "sorted" mode, the CPLStringList will attempt to
- * keep things in sorted order through calls to AddString(), 
+ * keep things in sorted order through calls to AddString(),
  * AddStringDirectly(), AddNameValue(), SetNameValue(). Complete list
  * assignments (via Assign() and operator= will clear the sorting state.
  * When in sorted order FindName(), FetchNameValue() and FetchNameValueDef()
@@ -501,9 +542,10 @@ CPLStringList &CPLStringList::Sort()
     Count();
     MakeOurOwnCopy();
 
-    qsort( papszList, nCount, sizeof(char*), llCompareStr );
-    bIsSorted = TRUE;
-    
+    if( nCount )
+        qsort( papszList, nCount, sizeof(char*), llCompareStr );
+    bIsSorted = true;
+
     return *this;
 }
 
@@ -513,12 +555,12 @@ CPLStringList &CPLStringList::Sort()
 
 /**
  * Get index of given name/value keyword.
- * 
- * Note that this search is for a line in the form name=value or name:value. 
- * Use FindString() or PartialFindString() for searches not based on name=value
- * pairs. 
  *
- * @param pszKey the name to search for.  
+ * Note that this search is for a line in the form name=value or name:value.
+ * Use FindString() or PartialFindString() for searches not based on name=value
+ * pairs.
+ *
+ * @param pszKey the name to search for.
  *
  * @return the string list index of this name, or -1 on failure.
  */
@@ -529,20 +571,21 @@ int CPLStringList::FindName( const char *pszKey ) const
     if( !IsSorted() )
         return CSLFindName( papszList, pszKey );
 
-    // If we are sorted, we can do an optimized binary search. 
-    int iStart=0, iEnd=nCount-1;
-    int nKeyLen = strlen(pszKey);
+    // If we are sorted, we can do an optimized binary search.
+    int iStart = 0;
+    int iEnd = nCount - 1;
+    size_t nKeyLen = strlen(pszKey);
 
     while( iStart <= iEnd )
     {
-        int iMiddle = (iEnd+iStart)/2;
+        const int iMiddle = (iEnd + iStart) / 2;
         const char *pszMiddle = papszList[iMiddle];
 
         if (EQUALN(pszMiddle, pszKey, nKeyLen)
             && (pszMiddle[nKeyLen] == '=' || pszMiddle[nKeyLen] == ':') )
             return iMiddle;
 
-        if( STRCASECMP(pszKey,pszMiddle) < 0 )
+        if( CPLCompareKeyValueString(pszKey,pszMiddle) < 0 )
             iEnd = iMiddle-1;
         else
             iStart = iMiddle+1;
@@ -552,7 +595,7 @@ int CPLStringList::FindName( const char *pszKey ) const
 }
 
 /************************************************************************/
-/*                            FetchBoolean()                            */
+/*                            FetchBool()                               */
 /************************************************************************/
 /**
  *
@@ -560,26 +603,52 @@ int CPLStringList::FindName( const char *pszKey ) const
  *
  * In a CPLStringList of "Name=Value" pairs, look to see if there is a key
  * with the given name, and if it can be interpreted as being TRUE.  If
- * the key appears without any "=Value" portion it will be considered true. 
+ * the key appears without any "=Value" portion it will be considered true.
  * If the value is NO, FALSE or 0 it will be considered FALSE otherwise
  * if the key appears in the list it will be considered TRUE.  If the key
- * doesn't appear at all, the indicated default value will be returned. 
- * 
+ * doesn't appear at all, the indicated default value will be returned.
+ *
  * @param pszKey the key value to look for (case insensitive).
- * @param bDefault the value to return if the key isn't found at all. 
- * 
- * @return TRUE or FALSE 
+ * @param bDefault the value to return if the key isn't found at all.
+ *
+ * @return true or false
  */
 
-int CPLStringList::FetchBoolean( const char *pszKey, int bDefault ) const
+bool CPLStringList::FetchBool( const char *pszKey, bool bDefault ) const
 
 {
     const char *pszValue = FetchNameValue( pszKey );
 
     if( pszValue == NULL )
         return bDefault;
-    else
-        return CSLTestBoolean( pszValue );
+
+    return CPLTestBool( pszValue );
+}
+
+/************************************************************************/
+/*                            FetchBoolean()                            */
+/************************************************************************/
+/**
+ *
+ * DEPRECATED: Check for boolean key value.
+ *
+ * In a CPLStringList of "Name=Value" pairs, look to see if there is a key
+ * with the given name, and if it can be interpreted as being TRUE.  If
+ * the key appears without any "=Value" portion it will be considered true.
+ * If the value is NO, FALSE or 0 it will be considered FALSE otherwise
+ * if the key appears in the list it will be considered TRUE.  If the key
+ * doesn't appear at all, the indicated default value will be returned.
+ *
+ * @param pszKey the key value to look for (case insensitive).
+ * @param bDefault the value to return if the key isn't found at all.
+ *
+ * @return TRUE or FALSE
+ */
+
+int CPLStringList::FetchBoolean( const char *pszKey, int bDefault ) const
+
+{
+    return FetchBool( pszKey, CPL_TO_BOOL(bDefault) ) ? TRUE : FALSE;
 }
 
 /************************************************************************/
@@ -590,19 +659,19 @@ int CPLStringList::FetchBoolean( const char *pszKey, int bDefault ) const
  * Fetch value associated with this key name.
  *
  * If this list sorted, a fast binary search is done, otherwise a linear
- * scan is done.  Name lookup is case insensitive. 
- * 
+ * scan is done.  Name lookup is case insensitive.
+ *
  * @param pszName the key name to search for.
- * 
- * @return the corresponding value or NULL if not found.  The returned string 
- * should not be modified and points into internal object state that may 
- * change on future calls. 
+ *
+ * @return the corresponding value or NULL if not found.  The returned string
+ * should not be modified and points into internal object state that may
+ * change on future calls.
  */
 
 const char *CPLStringList::FetchNameValue( const char *pszName ) const
 
 {
-    int iKey = FindName( pszName );
+    const int iKey = FindName( pszName );
 
     if( iKey == -1 )
         return NULL;
@@ -621,12 +690,12 @@ const char *CPLStringList::FetchNameValue( const char *pszName ) const
  * Fetch value associated with this key name.
  *
  * If this list sorted, a fast binary search is done, otherwise a linear
- * scan is done.  Name lookup is case insensitive. 
- * 
+ * scan is done.  Name lookup is case insensitive.
+ *
  * @param pszName the key name to search for.
  * @param pszDefault the default value returned if the named entry isn't found.
- * 
- * @return the corresponding value or the passed default if not found. 
+ *
+ * @return the corresponding value or the passed default if not found.
  */
 
 const char *CPLStringList::FetchNameValueDef( const char *pszName,
@@ -636,8 +705,8 @@ const char *CPLStringList::FetchNameValueDef( const char *pszName,
     const char *pszValue = FetchNameValue( pszName );
     if( pszValue == NULL )
         return pszDefault;
-    else
-        return pszValue;
+
+    return pszValue;
 }
 
 /************************************************************************/
@@ -676,7 +745,7 @@ const char *CPLStringList::FetchNameValueDef( const char *pszName,
  * heap.
  */
 
-CPLStringList &CPLStringList::InsertStringDirectly( int nInsertAtLineNo, 
+CPLStringList &CPLStringList::InsertStringDirectly( int nInsertAtLineNo,
                                                     char *pszNewLine )
 
 {
@@ -692,7 +761,7 @@ CPLStringList &CPLStringList::InsertStringDirectly( int nInsertAtLineNo,
         return *this;
     }
 
-    bIsSorted = FALSE;
+    bIsSorted = false;
 
     for( int i = nCount; i > nInsertAtLineNo; i-- )
         papszList[i] = papszList[i-1];
@@ -714,15 +783,16 @@ int CPLStringList::FindSortedInsertionPoint( const char *pszLine )
 
 {
     CPLAssert( IsSorted() );
-    
-    int iStart=0, iEnd=nCount-1;
+
+    int iStart=0;
+    int iEnd=nCount-1;
 
     while( iStart <= iEnd )
     {
         int iMiddle = (iEnd+iStart)/2;
         const char *pszMiddle = papszList[iMiddle];
 
-        if( STRCASECMP(pszLine,pszMiddle) < 0 )
+        if( CPLCompareKeyValueString(pszLine,pszMiddle) < 0 )
             iEnd = iMiddle-1;
         else
             iStart = iMiddle+1;
@@ -730,10 +800,10 @@ int CPLStringList::FindSortedInsertionPoint( const char *pszLine )
 
     iEnd++;
     CPLAssert( iEnd >= 0 && iEnd <= nCount );
-    CPLAssert( iEnd == 0 
-               || STRCASECMP(pszLine,papszList[iEnd-1]) >= 0 );
+    CPLAssert( iEnd == 0
+               || CPLCompareKeyValueString(pszLine,papszList[iEnd-1]) >= 0 );
     CPLAssert( iEnd == nCount
-               || STRCASECMP(pszLine,papszList[iEnd]) <= 0 );
-    
+               || CPLCompareKeyValueString(pszLine,papszList[iEnd]) <= 0 );
+
     return iEnd;
 }
