@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: vsipreload.cpp 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: vsipreload.cpp 33724 2016-03-16 17:14:11Z goatbar $
  *
  * Project:  CPL - Common Portability Library
  * Purpose:  Standalone shared library that can be LD_PRELOAD'ed as an overload of
@@ -37,7 +37,7 @@
 
 // Run:
 // LD_PRELOAD=./vsipreload.so ....
-// e.g: 
+// e.g:
 // LD_PRELOAD=./vsipreload.so gdalinfo /vsicurl/http://download.osgeo.org/gdal/data/ecw/spif83.ecw
 // LD_PRELOAD=./vsipreload.so gdalinfo 'HDF4_EOS:EOS_GRID:"/vsicurl/http://download.osgeo.org/gdal/data/hdf4/MOD09Q1G_EVI.A2006233.h07v03.005.2008338190308.hdf":MODIS_NACP_EVI:MODIS_EVI'
 // LD_PRELOAD=./vsipreload.so ogrinfo /vsicurl/http://svn.osgeo.org/gdal/trunk/autotest/ogr/data/testavc -ro
@@ -68,7 +68,7 @@
 #include "cpl_string.h"
 #include "cpl_hash_set.h"
 
-CPL_CVSID("$Id: vsipreload.cpp 27044 2014-03-16 23:41:27Z rouault $");
+CPL_CVSID("$Id: vsipreload.cpp 33724 2016-03-16 17:14:11Z goatbar $");
 
 static int DEBUG_VSIPRELOAD = 0;
 static int DEBUG_VSIPRELOAD_ONLY_VSIL = 1;
@@ -100,6 +100,7 @@ DECLARE_SYMBOL(fgetpos, int, (FILE *stream, fpos_t *pos));
 DECLARE_SYMBOL(fsetpos, int, (FILE *stream, fpos_t *pos));
 DECLARE_SYMBOL(fileno, int, (FILE *stream));
 DECLARE_SYMBOL(ferror, int, (FILE *stream));
+DECLARE_SYMBOL(clearerr, void, (FILE *stream));
 
 DECLARE_SYMBOL(fdopen, FILE*, (int fd, const char *mode));
 DECLARE_SYMBOL(freopen, FILE*, (const char *path, const char *mode, FILE *stream));
@@ -126,11 +127,12 @@ DECLARE_SYMBOL(ftruncate, int, (int fd, off_t length));
 
 DECLARE_SYMBOL(opendir, DIR* , (const char *name));
 DECLARE_SYMBOL(readdir, struct dirent*, (DIR *dirp));
+DECLARE_SYMBOL(readdir64, struct dirent64*, (DIR *dirp));
 DECLARE_SYMBOL(closedir, int, (DIR *dirp));
 DECLARE_SYMBOL(dirfd, int, (DIR *dirp));
 DECLARE_SYMBOL(fchdir, int, (int fd));
 
-static void* hMutex = NULL;
+static CPLLock* hLock = NULL;
 
 typedef struct
 {
@@ -138,6 +140,7 @@ typedef struct
     char** papszDir;
     int    nIter;
     struct dirent ent;
+    struct dirent64 ent64;
     int    fd;
 } VSIDIR;
 
@@ -160,7 +163,7 @@ std::string osCurDir;
 
 static void myinit(void)
 {
-    CPLMutexHolderD(&hMutex);
+    CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX);
 
     if( pfnfopen64 != NULL ) return;
     DEBUG_VSIPRELOAD = getenv("DEBUG_VSIPRELOAD") != NULL;
@@ -182,6 +185,7 @@ static void myinit(void)
     LOAD_SYMBOL(fsetpos);
     LOAD_SYMBOL(fileno);
     LOAD_SYMBOL(ferror);
+    LOAD_SYMBOL(clearerr);
 
     LOAD_SYMBOL(fdopen);
     LOAD_SYMBOL(freopen);
@@ -207,6 +211,7 @@ static void myinit(void)
 
     LOAD_SYMBOL(opendir);
     LOAD_SYMBOL(readdir);
+    LOAD_SYMBOL(readdir64);
     LOAD_SYMBOL(closedir);
     LOAD_SYMBOL(dirfd);
     LOAD_SYMBOL(fchdir);
@@ -219,7 +224,7 @@ static void myinit(void)
 static VSILFILE* getVSILFILE(FILE* stream)
 {
     VSILFILE* ret;
-    CPLMutexHolderD(&hMutex);
+    CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX);
     std::set<VSILFILE*>::iterator oIter = oSetFiles.find((VSILFILE*)stream);
     if( oIter != oSetFiles.end() )
         ret = *oIter;
@@ -235,7 +240,7 @@ static VSILFILE* getVSILFILE(FILE* stream)
 static VSILFILE* getVSILFILE(int fd)
 {
     VSILFILE* ret;
-    CPLMutexHolderD(&hMutex);
+    CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX);
     std::map<int, VSILFILE*>::iterator oIter = oMapfdToVSI.find(fd);
     if( oIter != oMapfdToVSI.end() )
         ret = oIter->second;
@@ -272,7 +277,7 @@ static VSILFILE* VSIFfopenHelper(const char *path, const char *mode)
     VSILFILE* fpVSIL = VSIFOpenL(path, mode);
     if( fpVSIL != NULL )
     {
-        CPLMutexHolderD(&hMutex);
+        CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX);
         oSetFiles.insert(fpVSIL);
         oMapVSIToString[fpVSIL] = path;
     }
@@ -285,9 +290,9 @@ static VSILFILE* VSIFfopenHelper(const char *path, const char *mode)
 
 static int getfdFromVSILFILE(VSILFILE* fpVSIL)
 {
-    CPLMutexHolderD(&hMutex);
+    CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX);
 
-    int fd;
+    int fd = 0;
     std::map<VSILFILE*, int>::iterator oIter = oMapVSITofd.find(fpVSIL);
     if( oIter != oMapVSITofd.end() )
         fd = oIter->second;
@@ -308,9 +313,9 @@ static int getfdFromVSILFILE(VSILFILE* fpVSIL)
 static int VSIFopenHelper(const char *path, int flags)
 {
     const char* pszMode = "rb";
-    if ((flags & 3) == O_RDONLY) 
+    if ((flags & 3) == O_RDONLY)
         pszMode = "rb";
-    else if ((flags & 3) == O_WRONLY) 
+    else if ((flags & 3) == O_WRONLY)
     {
         if( flags & O_APPEND )
             pszMode = "ab";
@@ -325,7 +330,7 @@ static int VSIFopenHelper(const char *path, int flags)
             pszMode = "rb+";
     }
     VSILFILE* fpVSIL = VSIFfopenHelper(path, pszMode );
-    int fd;
+    int fd = 0;
     if( fpVSIL != NULL )
     {
         if( flags & O_TRUNC )
@@ -344,17 +349,17 @@ static int VSIFopenHelper(const char *path, int flags)
 /*                    GET_DEBUG_VSIPRELOAD_COND()                             */
 /************************************************************************/
 
-static int GET_DEBUG_VSIPRELOAD_COND(const char* path)
+static bool GET_DEBUG_VSIPRELOAD_COND(const char* path)
 {
-    return (DEBUG_VSIPRELOAD && (!DEBUG_VSIPRELOAD_ONLY_VSIL || strncmp(path, "/vsi", 4) == 0) );
+    return (DEBUG_VSIPRELOAD && (!DEBUG_VSIPRELOAD_ONLY_VSIL || STARTS_WITH(path, "/vsi")) );
 }
 
-static int GET_DEBUG_VSIPRELOAD_COND(VSILFILE* fpVSIL)
+static bool GET_DEBUG_VSIPRELOAD_COND(VSILFILE* fpVSIL)
 {
     return (DEBUG_VSIPRELOAD && (!DEBUG_VSIPRELOAD_ONLY_VSIL || fpVSIL != NULL));
 }
 
-static int GET_DEBUG_VSIPRELOAD_COND(VSIDIR* dirP)
+static bool GET_DEBUG_VSIPRELOAD_COND(VSIDIR* dirP)
 {
     return (DEBUG_VSIPRELOAD && (!DEBUG_VSIPRELOAD_ONLY_VSIL || oSetVSIDIR.find(dirP) != oSetVSIDIR.end()));
 }
@@ -411,7 +416,7 @@ FILE *fopen(const char *path, const char *mode)
     int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(path);
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "fopen(%s, %s)\n", path, mode);
     FILE* ret;
-    if( strncmp(path, "/vsi", 4) == 0 )
+    if( STARTS_WITH(path, "/vsi") )
         ret = (FILE*) VSIFfopenHelper(path, mode);
     else
         ret = pfnfopen(path, mode);
@@ -429,7 +434,7 @@ FILE *fopen64(const char *path, const char *mode)
     int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(path);
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "fopen64(%s, %s)\n", path, mode);
     FILE* ret;
-    if( strncmp(path, "/vsi", 4) == 0 )
+    if( STARTS_WITH(path, "/vsi") )
         ret = (FILE*) VSIFfopenHelper(path, mode);
     else
         ret = pfnfopen64(path, mode);
@@ -448,7 +453,7 @@ size_t fread(void *ptr, size_t size, size_t nmemb, FILE *stream)
     int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(fpVSIL);
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "fread(stream=%p,size=%d,nmemb=%d)\n",
         stream, (int)size, (int)nmemb);
-    size_t ret;
+    size_t ret = 0;
     if( fpVSIL )
         ret = VSIFReadL(ptr, size, nmemb, fpVSIL);
     else
@@ -469,7 +474,7 @@ size_t fwrite(const void *ptr, size_t size, size_t nmemb, FILE *stream)
     int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(fpVSIL);
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "fwrite(stream=%p,size=%d,nmemb=%d)\n",
         stream, (int)size, (int)nmemb);
-    size_t ret;
+    size_t ret = 0;
     if( fpVSIL != NULL )
         ret = VSIFWriteL(ptr, size, nmemb, fpVSIL);
     else
@@ -491,7 +496,7 @@ int fclose(FILE *stream)
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "fclose(stream=%p)\n", stream);
     if( fpVSIL != NULL )
     {
-        CPLMutexHolderD(&hMutex);
+        CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX);
 
         int ret = VSIFCloseL(fpVSIL);
         oMapVSIToString.erase(fpVSIL);
@@ -523,17 +528,16 @@ int __xstat(int ver, const char *path, struct stat *buf)
     if( DEBUG_VSIPRELOAD && (osCurDir.size() != 0 && path[0] != '/') )
         DEBUG_VSIPRELOAD_COND = 1;
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "__xstat(%s)\n", path);
-    if( (osCurDir.size() != 0 && path[0] != '/') || strncmp(path, "/vsi", 4) == 0 )
+    if( (osCurDir.size() != 0 && path[0] != '/') || STARTS_WITH(path, "/vsi") )
     {
         VSIStatBufL sStatBufL;
-        int ret;
         std::string newpath;
         if( (osCurDir.size() != 0 && path[0] != '/') )
         {
             newpath = CPLFormFilename(osCurDir.c_str(), path, NULL);
             path = newpath.c_str();
         }
-        ret = VSIStatL(path, &sStatBufL);
+        const int ret = VSIStatL(path, &sStatBufL);
         sStatBufL.st_ino = (int)CPLHashSetHashStr(path);
         if( ret == 0 )
         {
@@ -567,17 +571,16 @@ int __lxstat(int ver, const char *path, struct stat *buf)
     if( DEBUG_VSIPRELOAD && (osCurDir.size() != 0 && path[0] != '/') )
         DEBUG_VSIPRELOAD_COND = 1;
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "__lxstat(%s)\n", path);
-    if( (osCurDir.size() != 0 && path[0] != '/') || strncmp(path, "/vsi", 4) == 0 )
+    if( (osCurDir.size() != 0 && path[0] != '/') || STARTS_WITH(path, "/vsi") )
     {
         VSIStatBufL sStatBufL;
-        int ret;
         std::string newpath;
         if( (osCurDir.size() != 0 && path[0] != '/') )
         {
             newpath = CPLFormFilename(osCurDir.c_str(), path, NULL);
             path = newpath.c_str();
         }
-        ret = VSIStatL(path, &sStatBufL);
+        const int ret = VSIStatL(path, &sStatBufL);
         sStatBufL.st_ino = (int)CPLHashSetHashStr(path);
         if( ret == 0 )
         {
@@ -611,17 +614,16 @@ int __xstat64(int ver, const char *path, struct stat64 *buf)
     if( DEBUG_VSIPRELOAD && (osCurDir.size() != 0 && path[0] != '/') )
         DEBUG_VSIPRELOAD_COND = 1;
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "__xstat64(%s)\n", path);
-    if( (osCurDir.size() != 0 && path[0] != '/') || strncmp(path, "/vsi", 4) == 0 )
+    if( (osCurDir.size() != 0 && path[0] != '/') || STARTS_WITH(path, "/vsi") )
     {
         VSIStatBufL sStatBufL;
-        int ret;
         std::string newpath;
         if( (osCurDir.size() != 0 && path[0] != '/') )
         {
             newpath = CPLFormFilename(osCurDir.c_str(), path, NULL);
             path = newpath.c_str();
         }
-        ret = VSIStatL(path, &sStatBufL);
+        const int ret = VSIStatL(path, &sStatBufL);
         sStatBufL.st_ino = (int)CPLHashSetHashStr(path);
         if( ret == 0 )
         {
@@ -824,7 +826,7 @@ int fileno(FILE *stream)
     VSILFILE* fpVSIL = getVSILFILE(stream);
     int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(fpVSIL);
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "fileno(stream=%p)\n", stream);
-    int fd;
+    int fd = 0;
     if( fpVSIL != NULL )
         fd = getfdFromVSILFILE(fpVSIL);
     else
@@ -852,6 +854,24 @@ int ferror(FILE *stream)
         return pfnferror(stream);
 }
 
+/************************************************************************/
+/*                             clearerr()                               */
+/************************************************************************/
+
+void clearerr(FILE *stream)
+{
+    myinit();
+    VSILFILE* fpVSIL = getVSILFILE(stream);
+    int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(fpVSIL);
+    if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "clearerr(stream=%p)\n", stream);
+    if( fpVSIL != NULL )
+    {
+        fprintf(stderr, "clearerr() unimplemented for VSILFILE\n");
+    }
+    else
+        pfnclearerr(stream);
+}
+
 /************************************************************************/
 /*                             fdopen()                                 */
 /************************************************************************/
@@ -911,7 +931,7 @@ int open(const char *path, int flags, ...)
     va_list args;
     va_start(args, flags);
     mode_t mode = va_arg(args, mode_t);
-    int fd;
+    int fd = 0;
     if( osCurDir.size() != 0 && path[0] != '/' && (flags & 3) == O_RDONLY && (flags & O_DIRECTORY) != 0 )
     {
         VSIStatBufL sStatBufL;
@@ -931,13 +951,13 @@ int open(const char *path, int flags, ...)
             S_ISDIR(sStatBufL.st_mode) )
         {
             fd = open("/dev/zero", O_RDONLY);
-            CPLMutexHolderD(&hMutex)
+            CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX)
             oMapDirFdToName[fd] = newname;
         }
         else
             fd = -1;
     }
-    else if( strncmp(path, "/vsi", 4) == 0 )
+    else if( STARTS_WITH(path, "/vsi") )
         fd = VSIFopenHelper(path, flags);
     else
         fd = pfnopen(path, flags, mode);
@@ -967,7 +987,7 @@ int open64(const char *path, int flags, ...)
     va_list args;
     va_start(args, flags);
     mode_t mode = va_arg(args, mode_t);
-    int fd;
+    int fd = 0;
     if( osCurDir.size() != 0 && path[0] != '/' && (flags & 3) == O_RDONLY && (flags & O_DIRECTORY) != 0 )
     {
         VSIStatBufL sStatBufL;
@@ -987,13 +1007,13 @@ int open64(const char *path, int flags, ...)
             S_ISDIR(sStatBufL.st_mode) )
         {
             fd = open("/dev/zero", O_RDONLY);
-            CPLMutexHolderD(&hMutex)
+            CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX)
             oMapDirFdToName[fd] = newname;
         }
         else
             fd = -1;
     }
-    else if( strncmp(path, "/vsi", 4) == 0 )
+    else if( STARTS_WITH(path, "/vsi") )
         fd = VSIFopenHelper(path, flags);
     else
         fd = pfnopen64(path, flags, mode);
@@ -1021,7 +1041,7 @@ int close(int fd)
     VSILFILE* fpVSIL = getVSILFILE(fd);
     int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(fpVSIL);
     {
-        CPLMutexHolderD(&hMutex);
+        CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX);
         assert( oMapfdToVSIDIR.find(fd) == oMapfdToVSIDIR.end() );
 
         if( oMapDirFdToName.find(fd) != oMapDirFdToName.end())
@@ -1035,7 +1055,7 @@ int close(int fd)
     if( fpVSIL != NULL )
     {
         VSIFCloseL(fpVSIL);
-        CPLMutexHolderD(&hMutex);
+        CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX);
         oSetFiles.erase(fpVSIL);
         pfnclose(oMapVSITofd[fpVSIL]);
         oMapVSITofd.erase(fpVSIL);
@@ -1057,7 +1077,7 @@ ssize_t read(int fd, void *buf, size_t count)
     VSILFILE* fpVSIL = getVSILFILE(fd);
     int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(fpVSIL);
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "read(fd=%d, count=%d)\n", fd, (int)count);
-    ssize_t ret;
+    ssize_t ret = 0;
     if( fpVSIL != NULL )
         ret = VSIFReadL(buf, 1, count, fpVSIL);
     else
@@ -1137,7 +1157,7 @@ int __fxstat (int ver, int fd, struct stat *buf)
     std::string name;
     int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(fpVSIL);
     {
-        CPLMutexHolderD(&hMutex)
+        CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX)
         if( oMapDirFdToName.find(fd) != oMapDirFdToName.end())
         {
             name = oMapDirFdToName[fd];
@@ -1165,7 +1185,7 @@ int __fxstat (int ver, int fd, struct stat *buf)
     {
         VSIStatBufL sStatBufL;
         {
-            CPLMutexHolderD(&hMutex);
+            CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX);
             name = oMapVSIToString[fpVSIL];
         }
         int ret = VSIStatL(name.c_str(), &sStatBufL);
@@ -1198,7 +1218,7 @@ int __fxstat64 (int ver, int fd, struct stat64 *buf)
         VSIStatBufL sStatBufL;
         std::string name;
         {
-            CPLMutexHolderD(&hMutex);
+            CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX);
             name = oMapVSIToString[fpVSIL];
         }
         int ret = VSIStatL(name.c_str(), &sStatBufL);
@@ -1230,13 +1250,12 @@ int __fxstatat (int ver, int dirfd, const char *pathname, struct stat *buf,
         DEBUG_VSIPRELOAD_COND = 1;
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "__fxstatat(dirfd=%d,pathname=%s,flags=%d)\n", dirfd, pathname, flags);
 
-    if( osCurDir.size() != 0 || strncmp(pathname, "/vsi", 4) == 0 )
+    if( osCurDir.size() != 0 || STARTS_WITH(pathname, "/vsi") )
     {
         VSIStatBufL sStatBufL;
-        int ret;
         if( osCurDir.size() && dirfd == AT_FDCWD && pathname[0] != '/' )
             pathname = CPLFormFilename(osCurDir.c_str(), pathname, NULL);
-        ret = VSIStatL(pathname, &sStatBufL);
+        const int ret = VSIStatL(pathname, &sStatBufL);
         sStatBufL.st_ino = (int)CPLHashSetHashStr(pathname);
         if( ret == 0 )
         {
@@ -1306,11 +1325,11 @@ off64_t lseek64(int fd, off64_t off, int whence)
 int truncate(const char *path, off_t length)
 {
     myinit();
-    int ret;
     int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(path);
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "truncate(%s)\n", path);
 
-    if( strncmp(path, "/vsi", 4) == 0 )
+    int ret = 0;
+    if( STARTS_WITH(path, "/vsi") )
     {
         VSILFILE* fpVSIL = VSIFOpenL(path, "wb+");
         if( fpVSIL )
@@ -1333,10 +1352,10 @@ int truncate(const char *path, off_t length)
 int ftruncate(int fd, off_t length)
 {
     myinit();
-    int ret;
     VSILFILE* fpVSIL = getVSILFILE(fd);
     int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(fpVSIL);
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "ftruncate(fd=%d)\n", fd);
+    int ret = 0;
     if( fpVSIL != NULL )
     {
         ret = VSIFTruncateL(fpVSIL, length);
@@ -1359,7 +1378,7 @@ DIR *opendir(const char *name)
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "opendir(%s)\n", name);
 
     DIR * ret;
-    if( osCurDir.size() != 0 || strncmp(name, "/vsi", 4) == 0 )
+    if( osCurDir.size() != 0 || STARTS_WITH(name, "/vsi") )
     {
         char** papszDir;
         if( osCurDir.size() != 0 && name[0] != '/' )
@@ -1384,7 +1403,7 @@ DIR *opendir(const char *name)
             mydir->nIter = 0;
             mydir->fd = -1;
             ret = (DIR*)mydir;
-            CPLMutexHolderD(&hMutex);
+            CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX);
             oSetVSIDIR.insert(mydir);
         }
     }
@@ -1394,6 +1413,39 @@ DIR *opendir(const char *name)
     return ret;
 }
 
+/************************************************************************/
+/*                             filldir()                                */
+/************************************************************************/
+
+static bool filldir(VSIDIR* mydir)
+{
+    int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(mydir);
+    char* pszName = mydir->papszDir[mydir->nIter++];
+    if( pszName == NULL )
+        return false;
+    mydir->ent.d_ino = 0;
+    mydir->ent.d_off = 0;
+    mydir->ent.d_reclen = sizeof(mydir->ent);
+    VSIStatBufL sStatBufL;
+    VSIStatL(CPLFormFilename(mydir->pszDirname, pszName, NULL), &sStatBufL);
+    if( DEBUG_VSIPRELOAD_COND && S_ISDIR(sStatBufL.st_mode) )
+        fprintf(stderr, "%s is dir\n", pszName);
+    mydir->ent.d_type = S_ISDIR(sStatBufL.st_mode) ? DT_DIR :
+                        S_ISREG(sStatBufL.st_mode) ? DT_REG :
+                        S_ISLNK(sStatBufL.st_mode) ? DT_LNK :
+                        DT_UNKNOWN;
+    strncpy(mydir->ent.d_name, pszName, 256);
+    mydir->ent.d_name[255] = '\0';
+
+    mydir->ent64.d_ino = 0;
+    mydir->ent64.d_off = 0;
+    mydir->ent64.d_reclen = sizeof(mydir->ent64);
+    mydir->ent64.d_type = mydir->ent.d_type;
+    strcpy(mydir->ent64.d_name, mydir->ent.d_name);
+
+    return true;
+}
+
 /************************************************************************/
 /*                             readdir()                                */
 /************************************************************************/
@@ -1406,28 +1458,36 @@ struct dirent *readdir(DIR *dirp)
     if( oSetVSIDIR.find((VSIDIR*)dirp) != oSetVSIDIR.end() )
     {
         VSIDIR* mydir = (VSIDIR*)dirp;
-        char* pszName = mydir->papszDir[mydir->nIter++];
-        if( pszName == NULL )
-            return NULL;
-        mydir->ent.d_ino = 0;
-        mydir->ent.d_off = 0;
-        mydir->ent.d_reclen = sizeof(mydir->ent);
-        VSIStatBufL sStatBufL;
-        VSIStatL(CPLFormFilename(mydir->pszDirname, pszName, NULL), &sStatBufL);
-        if( DEBUG_VSIPRELOAD_COND && S_ISDIR(sStatBufL.st_mode) )
-            fprintf(stderr, "%s is dir\n", pszName);
-        mydir->ent.d_type = S_ISDIR(sStatBufL.st_mode) ? DT_DIR :
-                            S_ISREG(sStatBufL.st_mode) ? DT_REG :
-                            S_ISLNK(sStatBufL.st_mode) ? DT_LNK :
-                            DT_UNKNOWN;
-        strncpy(mydir->ent.d_name, pszName, 256);
-        mydir->ent.d_name[255] = '\0';
+        if( !filldir(mydir) )
+            return FALSE;
+
         return &(mydir->ent);
     }
     else
         return pfnreaddir(dirp);
 }
 
+/************************************************************************/
+/*                             readdir64()                              */
+/************************************************************************/
+
+struct dirent64 *readdir64(DIR *dirp)
+{
+    myinit();
+    int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND((VSIDIR*)dirp);
+    if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "readdir64(%p)\n", dirp);
+    if( oSetVSIDIR.find((VSIDIR*)dirp) != oSetVSIDIR.end() )
+    {
+        VSIDIR* mydir = (VSIDIR*)dirp;
+        if( !filldir(mydir) )
+            return FALSE;
+
+        return &(mydir->ent64);
+    }
+    else
+        return pfnreaddir64(dirp);
+}
+
 /************************************************************************/
 /*                             closedir()                               */
 /************************************************************************/
@@ -1442,7 +1502,7 @@ int closedir(DIR *dirp)
         VSIDIR* mydir = (VSIDIR*)dirp;
         CPLFree(mydir->pszDirname);
         CSLDestroy(mydir->papszDir);
-        CPLMutexHolderD(&hMutex);
+        CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX);
         if( mydir->fd >= 0 )
         {
             oMapfdToVSIDIR.erase(mydir->fd);
@@ -1465,14 +1525,14 @@ int dirfd(DIR *dirp)
     myinit();
     int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND((VSIDIR*)dirp);
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "dirfd(%p)\n", dirp);
-    int ret;
+    int ret = 0;
     if( oSetVSIDIR.find((VSIDIR*)dirp) != oSetVSIDIR.end() )
     {
         VSIDIR* mydir = (VSIDIR*)dirp;
         if( mydir->fd < 0 )
         {
             mydir->fd = open("/dev/zero", O_RDONLY);
-            CPLMutexHolderD(&hMutex);
+            CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX);
             oMapfdToVSIDIR[mydir->fd] = mydir;
         }
         ret = mydir->fd;
@@ -1491,14 +1551,14 @@ int fchdir(int fd)
 {
     VSIDIR* mydir = NULL;
     {
-        CPLMutexHolderD(&hMutex);
+        CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX);
         if( oMapfdToVSIDIR.find(fd) != oMapfdToVSIDIR.end() )
             mydir = oMapfdToVSIDIR[fd];
     }
     int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(mydir);
     std::string name;
     {
-        CPLMutexHolderD(&hMutex)
+        CPLLockHolderD(&hLock, LOCK_RECURSIVE_MUTEX)
         if( oMapDirFdToName.find(fd) != oMapDirFdToName.end())
         {
             name = oMapDirFdToName[fd];
@@ -1540,8 +1600,8 @@ int acl_extended_file(const char *path)
     myinit();
     int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(path);
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "acl_extended_file(%s)\n", path);
-    int ret;
-    if( strncmp(path, "/vsi", 4) == 0 )
+    int ret = 0;
+    if( STARTS_WITH(path, "/vsi") )
         ret = -1;
     else
     {
@@ -1568,8 +1628,8 @@ int getfilecon(const char *path, /*security_context_t **/ void* con)
     myinit();
     int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(path);
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "getfilecon(%s)\n", path);
-    int ret;
-    if( strncmp(path, "/vsi", 4) == 0 )
+    int ret = 0;
+    if( STARTS_WITH(path, "/vsi") )
     {
         errno = ENOTSUP;
         ret = -1;
@@ -1599,8 +1659,8 @@ int lgetfilecon(const char *path, /*security_context_t **/ void* con)
     myinit();
     int DEBUG_VSIPRELOAD_COND = GET_DEBUG_VSIPRELOAD_COND(path);
     if (DEBUG_VSIPRELOAD_COND) fprintf(stderr, "lgetfilecon(%s)\n", path);
-    int ret;
-    if( strncmp(path, "/vsi", 4) == 0 )
+    int ret = 0;
+    if( STARTS_WITH(path, "/vsi") )
     {
         errno = ENOTSUP;
         ret = -1;
